// Type definitions for Office.js 1.0
// Project: https://github.com/OfficeDev/office-js
// Definitions by: OfficeDev <https://github.com/OfficeDev>, Lance Austin <https://github.com/LanceEA>, Michael Zlatkovsky <https://github.com/Zlatkovsky>, Kim Brandl <https://github.com/kbrandl>, Ricky Kirkham <https://github.com/Rick-Kirkham>, Alex Jerabek <https://github.com/AlexJerabek>, Elizabeth Samuel <https://github.com/ElizabethSamuel-MSFT>
// Definitions: https://github.com/DefinitelyTyped/DefinitelyTyped
// TypeScript Version: 2.4

/*
office-js
Copyright (c) Microsoft Corporation
*/


////////////////////////////////////////////////////////////////
//////////////////// Begin Office namespace ////////////////////
////////////////////////////////////////////////////////////////

declare namespace Office {
    /** A Promise object. Promises can be chained via ".then", and errors can be caught via ".catch". 
     * When a browser-provided native Promise implementation is available, Office.Promise will switch to use the native Promise instead.
     */
    var Promise: IPromiseConstructor;

    // **Note**: this is a copy of the PromiseConstructor object from
    //     https://github.com/Microsoft/TypeScript/blob/master/lib/lib.es2015.promise.d.ts
    // It is necessary so that even with targeting "ES5" and not specifying any libs,
    //     developers will still get IntelliSense for "Office.Promise" just as they would with a regular Promise.
    // (because even though Promise is part of standard lib.d.ts, PromiseConstructor is not)
    interface IPromiseConstructor {
        /**
         * A reference to the prototype.
         */
        readonly prototype: Promise<any>;

        /**
         * Creates a new Promise.
         * @param executor A callback used to initialize the promise. This callback is passed two arguments:
         * a resolve callback used resolve the promise with a value or the result of another promise,
         * and a reject callback used to reject the promise with a provided reason or error.
         */
        new <T>(executor: (resolve: (value?: T | PromiseLike<T>) => void, reject: (reason?: any) => void) => void): Promise<T>;

        /**
         * Creates a Promise that is resolved with an array of results when all of the provided Promises
         * resolve, or rejected when any Promise is rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        all<T1, T2, T3, T4, T5, T6, T7, T8, T9, T10>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>, T8 | PromiseLike<T8>, T9 | PromiseLike<T9>, T10 | PromiseLike<T10>]): Promise<[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10]>;

        /**
         * Creates a Promise that is resolved with an array of results when all of the provided Promises
         * resolve, or rejected when any Promise is rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        all<T1, T2, T3, T4, T5, T6, T7, T8, T9>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>, T8 | PromiseLike<T8>, T9 | PromiseLike<T9>]): Promise<[T1, T2, T3, T4, T5, T6, T7, T8, T9]>;

        /**
         * Creates a Promise that is resolved with an array of results when all of the provided Promises
         * resolve, or rejected when any Promise is rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        all<T1, T2, T3, T4, T5, T6, T7, T8>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>, T8 | PromiseLike<T8>]): Promise<[T1, T2, T3, T4, T5, T6, T7, T8]>;

        /**
         * Creates a Promise that is resolved with an array of results when all of the provided Promises
         * resolve, or rejected when any Promise is rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        all<T1, T2, T3, T4, T5, T6, T7>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>]): Promise<[T1, T2, T3, T4, T5, T6, T7]>;

        /**
         * Creates a Promise that is resolved with an array of results when all of the provided Promises
         * resolve, or rejected when any Promise is rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        all<T1, T2, T3, T4, T5, T6>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>]): Promise<[T1, T2, T3, T4, T5, T6]>;

        /**
         * Creates a Promise that is resolved with an array of results when all of the provided Promises
         * resolve, or rejected when any Promise is rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        all<T1, T2, T3, T4, T5>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>]): Promise<[T1, T2, T3, T4, T5]>;

        /**
         * Creates a Promise that is resolved with an array of results when all of the provided Promises
         * resolve, or rejected when any Promise is rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        all<T1, T2, T3, T4>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>]): Promise<[T1, T2, T3, T4]>;

        /**
         * Creates a Promise that is resolved with an array of results when all of the provided Promises
         * resolve, or rejected when any Promise is rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        all<T1, T2, T3>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>]): Promise<[T1, T2, T3]>;

        /**
         * Creates a Promise that is resolved with an array of results when all of the provided Promises
         * resolve, or rejected when any Promise is rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        all<T1, T2>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>]): Promise<[T1, T2]>;

        /**
         * Creates a Promise that is resolved with an array of results when all of the provided Promises
         * resolve, or rejected when any Promise is rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        all<T>(values: (T | PromiseLike<T>)[]): Promise<T[]>;

        /**
         * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved
         * or rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        race<T1, T2, T3, T4, T5, T6, T7, T8, T9, T10>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>, T8 | PromiseLike<T8>, T9 | PromiseLike<T9>, T10 | PromiseLike<T10>]): Promise<T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8 | T9 | T10>;

        /**
         * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved
         * or rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        race<T1, T2, T3, T4, T5, T6, T7, T8, T9>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>, T8 | PromiseLike<T8>, T9 | PromiseLike<T9>]): Promise<T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8 | T9>;

        /**
         * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved
         * or rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        race<T1, T2, T3, T4, T5, T6, T7, T8>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>, T8 | PromiseLike<T8>]): Promise<T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8>;

        /**
         * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved
         * or rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        race<T1, T2, T3, T4, T5, T6, T7>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>, T7 | PromiseLike<T7>]): Promise<T1 | T2 | T3 | T4 | T5 | T6 | T7>;

        /**
         * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved
         * or rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        race<T1, T2, T3, T4, T5, T6>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>, T6 | PromiseLike<T6>]): Promise<T1 | T2 | T3 | T4 | T5 | T6>;

        /**
         * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved
         * or rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        race<T1, T2, T3, T4, T5>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>, T5 | PromiseLike<T5>]): Promise<T1 | T2 | T3 | T4 | T5>;

        /**
         * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved
         * or rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        race<T1, T2, T3, T4>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>, T4 | PromiseLike<T4>]): Promise<T1 | T2 | T3 | T4>;

        /**
         * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved
         * or rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        race<T1, T2, T3>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>, T3 | PromiseLike<T3>]): Promise<T1 | T2 | T3>;

        /**
         * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved
         * or rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        race<T1, T2>(values: [T1 | PromiseLike<T1>, T2 | PromiseLike<T2>]): Promise<T1 | T2>;

        /**
         * Creates a Promise that is resolved or rejected when any of the provided Promises are resolved
         * or rejected.
         * @param values An array of Promises.
         * @returns A new Promise.
         */
        race<T>(values: (T | PromiseLike<T>)[]): Promise<T>;

        /**
         * Creates a new rejected promise for the provided reason.
         * @param reason The reason the promise was rejected.
         * @returns A new rejected Promise.
         */
        reject(reason: any): Promise<never>;

        /**
         * Creates a new rejected promise for the provided reason.
         * @param reason The reason the promise was rejected.
         * @returns A new rejected Promise.
         */
        reject<T>(reason: any): Promise<T>;

        /**
         * Creates a new resolved promise for the provided value.
         * @param value A promise.
         * @returns A promise whose internal state matches the provided promise.
         */
        resolve<T>(value: T | PromiseLike<T>): Promise<T>;

        /**
         * Creates a new resolved promise.
         * @returns A resolved promise.
         */
        resolve(): Promise<void>;
    }

    /**
     * Gets the Context object that represents the runtime environment of the add-in and provides access to the top-level objects of the API.
     * 
     * @remarks
     * 
     * **Support details**
     * 
     * A capital Y in the following matrix indicates that this property is supported in the corresponding Office host application. 
     * An empty cell indicates that the Office host application doesn't support this enumeration.
     * 
     * For more information about Office host application and server requirements, see 
     * {@link https://docs.microsoft.com/office/dev/add-ins/concepts/requirements-for-running-office-add-ins | Requirements for running Office Add-ins}.
     * 
     * *Supported hosts, by platform*
     *  <table>
     *   <tr><th>                             </th><th> Office on Windows </th><th> Office in web browser </th><th> Office on iPad </th><th> Outlook on mobile devices </th><th> Office on Mac </th></tr>
     *   <tr><td><strong> Access     </strong></td><td>                   </td><td> Y                     </td><td>                </td><td>                           </td><td>               </td></tr>
     *   <tr><td><strong> Excel      </strong></td><td> Y                 </td><td> Y                     </td><td> Y              </td><td>                           </td><td>               </td></tr>
     *   <tr><td><strong> Outlook    </strong></td><td> Y                 </td><td> Y                     </td><td>                </td><td> Y                         </td><td> Y             </td></tr>
     *   <tr><td><strong> PowerPoint </strong></td><td> Y                 </td><td> Y                     </td><td> Y              </td><td>                           </td><td>               </td></tr>
     *   <tr><td><strong> Project    </strong></td><td> Y                 </td><td>                       </td><td>                </td><td>                           </td><td>               </td></tr>
     *   <tr><td><strong> Word       </strong></td><td> Y                 </td><td> Y                     </td><td> Y              </td><td>                           </td><td>               </td></tr>
     *  </table>
     */
    var context: Context;
    /**
     * Occurs when the runtime environment is loaded and the add-in is ready to start interacting with the application and hosted document. 
     * 
     * The reason parameter of the initialize event listener function returns an `InitializationReason` enumeration value that specifies how 
     * initialization occurred. A task pane or content add-in can be initialized in two ways:
     * 
     *  - The user just inserted it from Recently Used Add-ins section of the Add-in drop-down list on the Insert tab of the ribbon in the Office 
     * host application, or from Insert add-in dialog box.
     * 
     *  - The user opened a document that already contains the add-in.
     * 
     * *Note*: The reason parameter of the initialize event listener function only returns an `InitializationReason` enumeration value for task pane 
     * and content add-ins. It does not return a value for Outlook add-ins.
     * 
     * @remarks
     * 
     * **Support details**
     * 
     * A capital Y in the following matrix indicates that this method is supported in the corresponding Office host application. 
     * An empty cell indicates that the Office host application doesn't support this method.
     * 
     * For more information about Office host application and server requirements, see 
     * {@link https://docs.microsoft.com/office/dev/add-ins/concepts/requirements-for-running-office-add-ins | Requirements for running Office Add-ins}.
     * 
     * *Supported hosts, by platform*
     *  <table>
     *   <tr><th>                             </th><th> Office on Windows </th><th> Office in web browser </th><th> Office on iPad </th><th> Outlook on mobile devices </th><th> Office on Mac </th></tr>
     *   <tr><td><strong> Access     </strong></td><td>                   </td><td> Y                     </td><td>                </td><td>                           </td><td>               </td></tr>
     *   <tr><td><strong> Excel      </strong></td><td> Y                 </td><td> Y                     </td><td> Y              </td><td>                           </td><td>               </td></tr>
     *   <tr><td><strong> Outlook    </strong></td><td> Y                 </td><td> Y                     </td><td>                </td><td> Y                         </td><td> Y             </td></tr>
     *   <tr><td><strong> PowerPoint </strong></td><td> Y                 </td><td> Y                     </td><td> Y              </td><td>                           </td><td>               </td></tr>
     *   <tr><td><strong> Project    </strong></td><td> Y                 </td><td>                       </td><td>                </td><td>                           </td><td>               </td></tr>
     *   <tr><td><strong> Word       </strong></td><td> Y                 </td><td> Y                     </td><td> Y              </td><td>                           </td><td>               </td></tr>
     *  </table>
     * 
     * @param reason Indicates how the app was initialized.
     */
    function initialize(reason: InitializationReason): void;
    /**
    * Ensures that the Office JavaScript APIs are ready to be called by the add-in. If the framework hasn't initialized yet, the callback or promise 
    * will wait until the Office host is ready to accept API calls. Note that though this API is intended to be used inside an Office add-in, it can 
    * also be used outside the add-in. In that case, once Office.js determines that it is running outside of an Office host application, it will call 
    * the callback and resolve the promise with "null" for both the host and platform.
    * 
    * @param callback - An optional callback method, that will receive the host and platform info. 
    *                   Alternatively, rather than use a callback, an add-in may simply wait for the Promise returned by the function to resolve.
    * @returns A Promise that contains the host and platform info, once initialization is completed.
    */
    function onReady(callback?: (info: { host: HostType, platform: PlatformType }) => any): Promise<{ host: HostType, platform: PlatformType }>;
    /**
     * Toggles on and off the `Office` alias for the full `Microsoft.Office.WebExtension` namespace.
     * 
     * @remarks
     * 
     * **Support details**
     * 
     * A capital Y in the following matrix indicates that this method is supported in the corresponding Office host application. 
     * An empty cell indicates that the Office host application doesn't support this method.
     * 
     * For more information about Office host application and server requirements, see 
     * {@link https://docs.microsoft.com/office/dev/add-ins/concepts/requirements-for-running-office-add-ins | Requirements for running Office Add-ins}.
     * 
     * *Supported hosts, by platform*
     *  <table>
     *   <tr><th>                             </th><th> Office on Windows </th><th> Office in web browser </th><th> Office on iPad </th><th> Outlook on mobile devices </th><th> Office on Mac </th></tr>
     *   <tr><td><strong> Access     </strong></td><td>                   </td><td> Y                          </td><td>           </td><td>                           </td><td>               </td></tr>
     *   <tr><td><strong> Excel      </strong></td><td> Y                 </td><td> Y                          </td><td> Y         </td><td>                           </td><td>               </td></tr>
     *   <tr><td><strong> Outlook    </strong></td><td> Y                 </td><td> Y                          </td><td>           </td><td> Y                         </td><td> Y             </td></tr>
     *   <tr><td><strong> PowerPoint </strong></td><td> Y                 </td><td> Y                          </td><td> Y         </td><td>                           </td><td>               </td></tr>
     *   <tr><td><strong> Project    </strong></td><td> Y                 </td><td>                            </td><td>           </td><td>                           </td><td>               </td></tr>
     *   <tr><td><strong> Word       </strong></td><td> Y                 </td><td> Y                          </td><td> Y         </td><td>                           </td><td>               </td></tr>
     *  </table>
     * 
     * @param useShortNamespace True to use the shortcut alias; otherwise false to disable it. The default is true.
     */
    function useShortNamespace(useShortNamespace: boolean): void;
    // Enumerations
    /**
     * Specifies the result of an asynchronous call.
     * 
     * @remarks
     * 
     * Returned by the `status` property of the {@link Office.AsyncResult | AsyncResult} object.
     */
    enum AsyncResultStatus {
        /**
         * The call succeeded.
         */
        Succeeded,
        /**
         * The call failed, check the error object.
         */
        Failed
    }
    /**
     * Specifies whether the add-in was just inserted or was already contained in the document.
     */
    enum InitializationReason {
        /**
         * The add-in was just inserted into the document.
         */
        Inserted,
        /**
         * The add-in is already part of the document that was opened.
         */
        DocumentOpened
    }
    /**
     * Specifies the host Office application in which the add-in is running.
     */
    enum HostType {
        /**
         * The Office host is Microsoft Word.
         */
        Word,
        /**
         * The Office host is Microsoft Excel.
         */
        Excel,
        /**
         * The Office host is Microsoft PowerPoint.
         */
        PowerPoint,
        /**
         * The Office host is Microsoft Outlook.
         */
        Outlook,
        /**
         * The Office host is Microsoft OneNote.
         */
        OneNote,
        /**
         * The Office host is Microsoft Project.
         */
        Project,
        /**
         * The Office host is Microsoft Access.
         */
        Access
    }
    /**
     * Specifies the OS or other platform on which the Office host application is running.
     */
    enum PlatformType {
        /**
         * The platform is PC (Windows).
         */
        PC,
        /**
         * The platform is Office on the web (in a browser).
         */
        OfficeOnline,
        /**
         * The platform is Mac.
         */
        Mac,
        /**
         * The platform an iOS device.
         */
        iOS,
        /**
         * The platform is an Android device.
         */
        Android,
        /**
         * The platform is WinRT.
         */
        Universal
    }
    // Objects
        /**
        * An object which encapsulates the result of an asynchronous request, including status and error information if the request failed.
        *
        * When the function you pass to the `callback` parameter of an "Async" method executes, it receives an AsyncResult object that you can access 
        * from the `callback` function's only parameter.
        */
        interface AsyncResult<T> {
        /**
        * Gets the user-defined item passed to the optional `asyncContext` parameter of the invoked method in the same state as it was passed in. 
        * This returns the user-defined item (which can be of any JavaScript type: String, Number, Boolean, Object, Array, Null, or Undefined) passed 
        * to the optional `asyncContext` parameter of the invoked method. Returns Undefined, if you didn't pass anything to the asyncContext parameter.
        */
        asyncContext: any;
        /**
        * Gets an object that may provide additional information if an error occurred.
        *
        * @remarks
        * 
        * *Supported APIs*
        * 
        * `Office.context.mailbox.item.getCallbackTokenAsync`, `Office.context.mailbox.item.getUserIdentityTokenAsync`
        * 
        * *Supported errors*
        * <table>
        *   <tr><th>`AsyncResult.error.name`            </th><th>`AsyncResult.error.message`                                                                       </th><th>Description of diagnostics object returned                                                                                          </th></tr>
        *   <tr><td><strong>HTTPRequestFailure</strong> </td><td>The request has failed. Please look at the diagnostics object for the HTTP error code.            </td><td>The HTTP error code in a JSON object e.g., `{"HTTPCode":"401"}`                                                                     </td></tr>
        *   <tr><td><strong>InternalServerError</strong></td><td>The Exchange server returned an error. Please look at the diagnostics object for more information.</td><td>The error message from the Exchange server in a JSON object e.g., `{"ErrorText": "The mailbox database is temporarily unavailable"}`</td></tr>
        * </table>
        */
       diagnostics: any;
       /**
        * Gets an {@link Office.Error} object that provides a description of the error, if any error occurred.
        */
        error: Office.Error;
        /**
        * Gets the {@link Office.AsyncResultStatus} of the asynchronous operation.
        */
        status: AsyncResultStatus;
        /**
        * Gets the payload or content of this asynchronous operation, if any.
        * 
        * @remarks
        * You access the AsyncResult object in the function passed as the argument to the callback parameter of an "Async" method, such as the 
        * `getSelectedDataAsync` and `setSelectedDataAsync` methods of the {@link Office.Document | Document} object.
        *
        * **Note**: What the value property returns for a particular "Async" method varies depending on the purpose and context of that method. 
        * To determine what is returned by the value property for an "Async" method, refer to the "Callback value" section of the method's topic.
        */
        value: T;
    }
    /**
     * Represents the runtime environment of the add-in and provides access to key objects of the API. 
     * The current context exists as a property of Office. It is accessed using `Office.context`.
     *
     * @remarks 
     * **Hosts**: Access, Excel, Outlook, PowerPoint, Project, Word
     */     
    interface Context {
        /**
        * True, if the current platform allows the add-in to display a UI for selling or upgrading; otherwise returns False.
        * 
        * @remarks
        * **Hosts**: Excel, Word
        * 
        * `commerceAllowed` is only supported in Office on iPad.
        * 
        * The iOS App Store doesn't support apps with add-ins that provide links to additional payment systems. However, Office Add-ins running in 
        * Office on the Windows desktop or in the browser do allow such links. If you want the UI of your add-in to provide a link to an 
        * external payment system on platforms other than iOS, you can use the commerceAllowed property to control when that link is displayed.
        */
        commerceAllowed: boolean;
        /**
        * Gets the locale (language) specified by the user for editing the document or item.
        * 
        * @remarks
        * The `contentLanguage` value reflects the **Editing Language** setting specified with **File \> Options \> Language** in the Office host 
        * application.
        * 
        * In content add-ins for Access web apps, the `contentLanguage` property gets the add-in culture (e.g., "en-GB").
        * 
        * **Support details**
        * 
        * A capital Y in the following matrix indicates that this property is supported in the corresponding Office host application. 
        * An empty cell indicates that the Office host application doesn't support this enumeration.
        * 
        * For more information about Office host application and server requirements, see 
        * {@link https://docs.microsoft.com/office/dev/add-ins/concepts/requirements-for-running-office-add-ins | Requirements for running Office Add-ins}.
        *  
        * *Supported hosts, by platform*
        *  <table>
        *   <tr><th>                             </th><th> Office on Windows </th><th> Office in web browser </th><th> Office on iPad </th><th> Outlook on mobile devices </th><th> Office on Mac </th></tr>
        *   <tr><td><strong> Access     </strong></td><td>                   </td><td> Y                     </td><td>                </td><td>                           </td><td>               </td></tr>
        *   <tr><td><strong> Excel      </strong></td><td> Y                 </td><td> Y                     </td><td> Y              </td><td>                           </td><td>               </td></tr>
        *   <tr><td><strong> Outlook    </strong></td><td> Y                 </td><td> Y                     </td><td>                </td><td> Y                         </td><td> Y             </td></tr>
        *   <tr><td><strong> PowerPoint </strong></td><td> Y                 </td><td> Y                     </td><td> Y              </td><td>                           </td><td>               </td></tr>
        *   <tr><td><strong> Project    </strong></td><td> Y                 </td><td>                       </td><td>                </td><td>                           </td><td>               </td></tr>
        *   <tr><td><strong> Word       </strong></td><td> Y                 </td><td> Y                     </td><td> Y              </td><td>                           </td><td>               </td></tr>
        *  </table>
        */
        contentLanguage: string;
        /**
        * Gets information about the environment in which the add-in is running.
        */
        diagnostics: ContextInformation;
        /**
        * Gets the locale (language) specified by the user for the UI of the Office host application.
        * 
        * @remarks
        * 
        * The returned value is a string in the RFC 1766 Language tag format, such as en-US.
        * 
        * The `displayLanguage` value reflects the current **Display Language** setting specified with **File \> Options \> Language** in the Office 
        * host application.
        * 
        * In content add-ins for Access web apps, the `displayLanguage property` gets the add-in language (e.g., "en-US").
        * 
        * When using in Outlook, the applicable modes are Compose or Read.
        * 
        * **Support details**
        * 
        * A capital Y in the following matrix indicates that this property is supported in the corresponding Office host application. 
        * An empty cell indicates that the Office host application doesn't support this enumeration.
        * 
        * For more information about Office host application and server requirements, see 
        * {@link https://docs.microsoft.com/office/dev/add-ins/concepts/requirements-for-running-office-add-ins | Requirements for running Office Add-ins}.
        * 
        * *Supported hosts, by platform*
        *  <table>
        *   <tr><th>                             </th><th> Office on Windows </th><th> Office in web browser </th><th> Office on iPad </th><th> Outlook on mobile devices </th><th> Office on Mac </th></tr>
        *   <tr><td><strong> Access     </strong></td><td> Y                 </td><td>                       </td><td>                </td><td>                           </td><td>               </td></tr>
        *   <tr><td><strong> Excel      </strong></td><td> Y                 </td><td> Y                     </td><td> Y              </td><td>                           </td><td>               </td></tr>
        *   <tr><td><strong> Outlook    </strong></td><td> Y                 </td><td> Y                     </td><td>                </td><td> Y                         </td><td> Y             </td></tr>
        *   <tr><td><strong> PowerPoint </strong></td><td> Y                 </td><td> Y                     </td><td> Y              </td><td>                           </td><td>               </td></tr>
        *   <tr><td><strong> Project    </strong></td><td> Y                 </td><td>                       </td><td>                </td><td>                           </td><td>               </td></tr>
        *   <tr><td><strong> Word       </strong></td><td> Y                 </td><td>                       </td><td> Y              </td><td>                           </td><td>               </td></tr>
        *  </table>
        */
        displayLanguage: string;
        /**
        * Gets an object that represents the document the content or task pane add-in is interacting with.
        */
        document: Office.Document;
        /**
        * Contains the Office application host in which the add-in is running.
        */
        host: HostType;
        /**
        * Gets the license information for the user's Office installation.
        */
        license: string;
        /**
         * Provides access to the Outlook Add-in object model for Microsoft Outlook and Microsoft Outlook on the web.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * **Namespaces**:
         *
         * - `diagnostics`: Provides diagnostic information to an Outlook add-in.
         *
         * - `item`: Provides methods and properties for accessing a message or appointment in an Outlook add-in.
         *
         * - `userProfile`: Provides information about the user in an Outlook add-in.
         */
        mailbox: Office.Mailbox;
        /**
        * Provides access to the properties for Office theme colors.
        */
        officeTheme: OfficeTheme;
        /**
        * Provides the platform on which the add-in is running.
        */
        platform: PlatformType;
        /**
        * Provides a method for determining what requirement sets are supported on the current host and platform.
        */
        requirements: RequirementSetSupport;
        /**
         * Gets an object that represents the custom settings or state of a mail add-in saved to a user's mailbox.
         *
         * The RoamingSettings object lets you store and access data for a mail add-in that is stored in a user's mailbox, so that is available to 
         * that add-in when it is running from any host client application used to access that mailbox.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        roamingSettings: Office.RoamingSettings;
        /**
        * Specifies whether the platform and device allows touch interaction. 
        * True if the add-in is running on a touch device, such as an iPad; false otherwise.
        * 
        * @remarks
        * **Hosts**: Excel, PowerPoint, Word
        * 
        * `touchEnabled` is only supported in Office on iPad.
        * 
        * Use the touchEnabled property to determine when your add-in is running on a touch device and if necessary, adjust the kind of controls, and 
        * size and spacing of elements in your add-in's UI to accommodate touch interactions.
        */
        touchEnabled: boolean;
        /**
        * Provides objects and methods that you can use to create and manipulate UI components, such as dialog boxes.
        */
        ui: UI;
    }
    /**
     * Provides specific information about an error that occurred during an asynchronous data operation.
     *
     * @remarks
     * The Error object is accessed from the AsyncResult object that is returned in the function passed as the callback argument of an asynchronous 
     * data operation, such as the `setSelectedDataAsync` method of the Document object.
     */
    interface Error {
        /**
         * Gets the numeric code of the error. For a list of error codes, see {@link https://docs.microsoft.com/office/dev/add-ins/reference/javascript-api-for-office-error-codes | JavaScript API for Office error codes}.
         */
        code: number;
        /**
         * Gets the name of the error.
         */
        message: string;
        /**
         * Gets a detailed description of the error.
         */
        name: string;
    }
    namespace AddinCommands {
        /**
         * The event object is passed as a parameter to add-in functions invoked by UI-less command buttons. The object allows the add-in to identify 
         * which button was clicked and to signal the host that it has completed its processing.
         * 
         * @remarks
         * See {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/add-in-commands-requirement-sets | Add-in commands requirement sets} for more support information.
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        interface Event {
            
            /**
             * Information about the control that triggered calling this function.
             * 
             * @remarks
             * 
             * This property is only supported in Outlook in {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/outlook-api-requirement-sets | requirement set} Mailbox 1.3 and later.
             */
            source:Source;
            /**
             * Indicates that the add-in has completed processing that was triggered by an add-in command button or event handler.
             * 
             * This method must be called at the end of a function which was invoked by an add-in command defined with an Action element with an 
             * xsi:type attribute set to ExecuteFunction. Calling this method signals the host client that the function is complete and that it can 
             * clean up any state involved with invoking the function. For example, if the user closes Outlook before this method is called, Outlook 
             * will warn that a function is still executing.
             * 
             * This method must be called in an event handler added via Office.context.mailbox.addHandlerAsync after completing processing of the event.
             * 
             * [Api set: Mailbox 1.3]
             *
             * @remarks
             * 
             * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
             * 
             * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
             * 
             * Mailbox 1.3 does not have the `options` parameter while Mailbox Preview does have support for `options` parameter.
             * 
             * @param options Optional. An object that specifies behavior options for when the event is completed.
             */
            completed(options?: EventCompletedOptions): void;
        }

        /**
         * Specifies the behavior for when the event is completed.
         */
        interface EventCompletedOptions {
            /**
             * A boolean value. When the completed method is used to signal completion of an event handler, 
             * this value indicates of the handled event should continue execution or be canceled. 
             * For example, an add-in that handles the `ItemSend` event can set `allowEvent = false` to cancel sending of the message.
             */
            allowEvent: boolean;
        }

        /**
         * Encapsulates source data for add-in events.
         */
        interface Source {

            /**
             * The id of the control that triggered calling this function. The id comes from the manifest and is the unique ID of your Office Add-in 
             * as a GUID.
             */
            id: string;
        }
    }
    /**
     * Provides objects and methods that you can use to create and manipulate UI components, such as dialog boxes, in your Office Add-ins.
     * 
     * Visit "{@link https://docs.microsoft.com/office/dev/add-ins/develop/dialog-api-in-office-add-ins | Use the Dialog API in your Office Add-ins}" 
     * for more information.
     */
    interface UI {
        /**
        * Displays a dialog to show or collect information from the user or to facilitate Web navigation.
        *
        * @remarks
        * 
        * **Hosts**: Word, Excel, Outlook, PowerPoint
        * 
        * **Requirement sets**: 
        * 
        * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/dialog-api-requirement-sets | DialogApi}
        * 
        * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/outlook-api-requirement-sets | Mailbox 1.4}
        * 
        * This method is available in the DialogApi requirement set for Word, Excel, or PowerPoint add-ins, and in the Mailbox requirement set 1.4 
        * for Outlook. For more on how to specify a requirement set in your manifest, see 
        * {@link https://docs.microsoft.com/office/dev/add-ins/develop/specify-office-hosts-and-api-requirements | Specify Office hosts and API requirements}.
        *
        * The initial page must be on the same domain as the parent page (the startAddress parameter). After the initial page loads, you can go to 
        * other domains.
        *
        * Any page calling `office.context.ui.messageParent` must also be on the same domain as the parent page.
        * 
        * **Design considerations**:
        *
        * The following design considerations apply to dialog boxes:
        *
        * - An Office Add-in task pane can have only one dialog box open at any time. Multiple dialogs can be open at the same time from Add-in 
        * Commands (custom ribbon buttons or menu items).
        *
        * - Every dialog box can be moved and resized by the user.
        *
        * - Every dialog box is centered on the screen when opened.
        *
        * - Dialog boxes appear on top of the host application and in the order in which they were created.
        *
        * Use a dialog box to:
        *
        * - Display authentication pages to collect user credentials.
        *
        * - Display an error/progress/input screen from a ShowTaskpane or ExecuteAction command.
        *
        * - Temporarily increase the surface area that a user has available to complete a task.
        *
        * Do not use a dialog box to interact with a document. Use a task pane instead.
        * 
        * For a design pattern that you can use to create a dialog box, see 
        * {@link https://github.com/OfficeDev/Office-Add-in-UX-Design-Patterns/blob/master/Patterns/Client_Dialog.md | Client Dialog}  in the Office 
        * Add-in UX Design Patterns repository on GitHub.
        * 
        * **displayDialogAsync Errors**:
        * 
        * <table>
        *   <tr>
        *     <th>Code number</th>
        *     <th>Meaning</th>
        *   </tr>
        *   <tr>
        *     <td>12004</td>
        *     <td>The domain of the URL passed to displayDialogAsync is not trusted. The domain must be either the same domain as the host page (including protocol and port number), or it must be registered in the <AppDomains> section of the add-in manifest.</td>
        *   </tr>
        *   <tr>
        *     <td>12005</td>
        *     <td>The URL passed to displayDialogAsync uses the HTTP protocol. HTTPS is required. (In some versions of Office, the error message returned with 12005 is the same one returned for 12004.)</td>
        *   </tr>
        *   <tr>
        *     <td>12007</td>
        *     <td>A dialog box is already opened from the task pane. A task pane add-in can only have one dialog box open at a time.</td>
        *   </tr>
        *   <tr>
        *     <td>12009</td>
        *     <td>The user chose to ignore the dialog box. This error can occur in online versions of Office, where users may choose not to allow an add-in to present a dialog.</td>
        *   </tr>
        * </table>
        * 
        * In the callback function passed to the displayDialogAsync method, you can use the properties of the AsyncResult object to return the 
        * following information.
        * 
        * <table>
        *   <tr>
        *     <th>Property</th>
        *     <th>Use to</th>
        *   </tr>
        *   <tr>
        *     <td>AsyncResult.value</td>
        *     <td>Access the Dialog object.</td>
        *   </tr>
        *   <tr>
        *     <td>AsyncResult.status</td>
        *     <td>Determine the success or failure of the operation.</td>
        *   </tr>
        *   <tr>
        *     <td>AsyncResult.error</td>
        *     <td>Access an Error object that provides error information if the operation failed.</td>
        *   </tr>
        *   <tr>
        *     <td>AsyncResult.asyncContext</td>
        *     <td>Access your user-defined object or value, if you passed one as the asyncContext parameter.</td>
        *   </tr>
        * </table>
        *
        * @param startAddress - Accepts the initial HTTPS URL that opens in the dialog.
        * @param options - Optional. Accepts an {@link Office.DialogOptions} object to define dialog display.
        * @param callback - Optional. Accepts a callback method to handle the dialog creation attempt. If successful, the AsyncResult.value is a Dialog object.
        */
        displayDialogAsync(startAddress: string, options?: DialogOptions, callback?: (result: AsyncResult<Dialog>) => void): void;
        /**
        * Displays a dialog to show or collect information from the user or to facilitate Web navigation.
        *
        * @remarks
        * 
        * **Hosts**: Excel, Outlook, PowerPoint, Word
        * 
        * **Requirement sets**: 
        * 
        * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/dialog-api-requirement-sets | DialogApi}
        * 
        * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/outlook-api-requirement-sets | Mailbox 1.4}
        * 
        * This method is available in the DialogApi requirement set for Word, Excel, or PowerPoint add-ins, and in the Mailbox requirement set 1.4 
        * for Outlook. For more on how to specify a requirement set in your manifest, see 
        * {@link https://docs.microsoft.com/office/dev/add-ins/develop/specify-office-hosts-and-api-requirements | Specify Office hosts and API requirements}.
        *
        * The initial page must be on the same domain as the parent page (the startAddress parameter). After the initial page loads, you can go to 
        * other domains.
        *
        * Any page calling `office.context.ui.messageParent` must also be on the same domain as the parent page.
        * 
        * **Design considerations**:
        *
        * The following design considerations apply to dialog boxes:
        *
        * - An Office Add-in task pane can have only one dialog box open at any time. Multiple dialogs can be open at the same time from Add-in 
        * Commands (custom ribbon buttons or menu items).
        *
        * - Every dialog box can be moved and resized by the user.
        *
        * - Every dialog box is centered on the screen when opened.
        *
        * - Dialog boxes appear on top of the host application and in the order in which they were created.
        *
        * Use a dialog box to:
        *
        * - Display authentication pages to collect user credentials.
        *
        * - Display an error/progress/input screen from a ShowTaskpane or ExecuteAction command.
        *
        * - Temporarily increase the surface area that a user has available to complete a task.
        *
        * Do not use a dialog box to interact with a document. Use a task pane instead.
        * 
        * For a design pattern that you can use to create a dialog box, see 
        * {@link https://github.com/OfficeDev/Office-Add-in-UX-Design-Patterns/blob/master/Patterns/Client_Dialog.md | Client Dialog}  in the Office 
        * Add-in UX Design Patterns repository on GitHub.
        * 
        * **displayDialogAsync Errors**:
        * 
        * <table>
        *   <tr>
        *     <th>Code number</th>
        *     <th>Meaning</th>
        *   </tr>
        *   <tr>
        *     <td>12004</td>
        *     <td>The domain of the URL passed to displayDialogAsync is not trusted. The domain must be either the same domain as the host page (including protocol and port number), or it must be registered in the <AppDomains> section of the add-in manifest.</td>
        *   </tr>
        *   <tr>
        *     <td>12005</td>
        *     <td>The URL passed to displayDialogAsync uses the HTTP protocol. HTTPS is required. (In some versions of Office, the error message returned with 12005 is the same one returned for 12004.)</td>
        *   </tr>
        *   <tr>
        *     <td>12007</td>
        *     <td>A dialog box is already opened from the task pane. A task pane add-in can only have one dialog box open at a time.</td>
        *   </tr>
        *   <tr>
        *     <td>12009</td>
        *     <td>The user chose to ignore the dialog box. This error can occur in online versions of Office, where users may choose not to allow an add-in to present a dialog.</td>
        *   </tr>
        * </table>
        * 
        * In the callback function passed to the displayDialogAsync method, you can use the properties of the AsyncResult object to return the 
        * following information.
        * 
        * <table>
        *   <tr>
        *     <th>Property</th>
        *     <th>Use to</th>
        *   </tr>
        *   <tr>
        *     <td>AsyncResult.value</td>
        *     <td>Access the Dialog object.</td>
        *   </tr>
        *   <tr>
        *     <td>AsyncResult.status</td>
        *     <td>Determine the success or failure of the operation.</td>
        *   </tr>
        *   <tr>
        *     <td>AsyncResult.error</td>
        *     <td>Access an Error object that provides error information if the operation failed.</td>
        *   </tr>
        *   <tr>
        *     <td>AsyncResult.asyncContext</td>
        *     <td>Access your user-defined object or value, if you passed one as the asyncContext parameter.</td>
        *   </tr>
        * </table>
        *
        * @param startAddress - Accepts the initial HTTPS URL that opens in the dialog.
        * @param callback - Optional. Accepts a callback method to handle the dialog creation attempt. If successful, the AsyncResult.value is a Dialog object.
        */
        displayDialogAsync(startAddress: string, callback?: (result: AsyncResult<Dialog>) => void): void;
        /**
         * Delivers a message from the dialog box to its parent/opener page. The page calling this API must be on the same domain as the parent. 
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/dialog-api-requirement-sets | DialogApi}
         * 
         * @param message Accepts a message from the dialog to deliver to the add-in. In addition to a boolean, anything that can serialized to a string including JSON and XML can be sent. 
         */
        messageParent(message: boolean | string): void;
        /**
         * Closes the UI container where the JavaScript is executing.
         *
         * @remarks
         * 
         * **Hosts**: Excel, Word, PowerPoint, Outlook (Minimum requirement set: Mailbox 1.5)
         * 
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/dialog-api-requirement-sets | DialogApi}
         * 
         * The behavior of this method is specified by the following:
         *
         * - Called from a UI-less command button: No effect. Any dialog opened by displayDialogAsync will remain open.
         *
         * - Called from a task pane: The task pane will close. Any dialog opened by displayDialogAsync will also close. 
         * If the task pane supports pinning and was pinned by the user, it will be un-pinned.
         *
         * - Called from a module extension: No effect.
         */
        closeContainer(): void;
    }

    /**
     * Provides information about what Requirement Sets are supported in current environment.
     */
    interface RequirementSetSupport {
        /**
        * Check if the specified requirement set is supported by the host Office application.
        * @param name - Set name; e.g., "MatrixBindings".
        * @param minVersion - The minimum required version; e.g., "1.4".
        */
       isSetSupported(name: string, minVersion?: number): boolean;
    }

    /**
     * Provides options for how a dialog is displayed.
     */
    interface DialogOptions {
        /**
         * Defines the width of the dialog as a percentage of the current display. Defaults to 80%. 250px minimum.
         */
        height?: number,
        /**
         * Defines the height of the dialog as a percentage of the current display. Defaults to 80%. 150px minimum.
         */
        width?: number,
        /**
         * Determines whether the dialog box should be displayed within an IFrame. This setting is only applicable in Office on the web, and is 
         * ignored by other platforms. If false (default), the dialog will be displayed as a new browser window (pop-up). Recommended for 
         * authentication pages that cannot be displayed in an IFrame. If true, the dialog will be displayed as a floating overlay with an IFrame. 
         * This is best for user experience and performance.
         */
        displayInIframe?: boolean
        /**
         * Determines if the pop-up blocker dialog will be shown to the user. Defaults to true.
         * 
         * `true` - The framework displays a pop-up to trigger the navigation and avoid the browser's pop-up blocker.
         * `false` - The dialog will not be shown and the developer must handle pop-ups (by providing a user interface artifact to trigger the navigation).
         */
        promptBeforeOpen?: boolean;
        /**
         * A user-defined item of any type that is returned, unchanged, in the asyncContext property of the AsyncResult object that is passed to a callback.
         */
        asyncContext?: any
    }
    
    /**
     * Provides an option for preserving context data of any type, unchanged, for use in a callback.
     */
    interface AsyncContextOptions {
        /**
         * A user-defined item of any type that is returned, unchanged, in the asyncContext property of the AsyncResult object that is passed to a callback.
         */
        asyncContext?: any
    }
    /**
     * Provides information about the environment in which the add-in is running.
     */
    interface ContextInformation {
        /**
        * Gets the Office application host in which the add-in is running.
        */
        host: Office.HostType;
        /**
        * Gets the platform on which the add-in is running.
        */
        platform: Office.PlatformType;
        /**
        * Gets the version of Office on which the add-in is running.
        */
        version: string;
    }
    /**
     * Provides options for how to get the data in a binding.
     *
     * @remarks
     * If the rows option is used, the value must be "thisRow".
     */
    interface GetBindingDataOptions {
        /**
         * The expected shape of the selection. Use {@link Office.CoercionType} or text value. Default: The original, uncoerced type of the binding.
         */
        coercionType?: Office.CoercionType | string
        /**
         * Specifies whether values, such as numbers and dates, are returned with their formatting applied. Use Office.ValueFormat or text value. 
         * Default: Unformatted data.
         */
        valueFormat?: Office.ValueFormat | string
        /**
        * For table or matrix bindings, specifies the zero-based starting row for a subset of the data in the binding. Default is first row.
        */
        startRow?: number
        /**
        * For table or matrix bindings, specifies the zero-based starting column for a subset of the data in the binding. Default is first column.
        */
        startColumn?: number
        /**
        * For table or matrix bindings, specifies the number of rows offset from the startRow. Default is all subsequent rows.
        */
        rowCount?: number
        /**
        * For table or matrix bindings, specifies the number of columns offset from the startColumn. Default is all subsequent columns.
        */
        columnCount?: number
        /**
        * Specify whether to get only the visible (filtered in) data or all the data (default is all). Useful when filtering data. 
        * Use Office.FilterType or text value.
        */
        filterType?: Office.FilterType | string
        /**
        * Only for table bindings in content add-ins for Access. Specifies the pre-defined string "thisRow" to get data in the currently selected row.
        */
        rows?: string
        /**
         * A user-defined item of any type that is returned, unchanged, in the asyncContext property of the AsyncResult object that is passed to a callback.
         */
        asyncContext?: any
    }
    /**
     * Provides options for how to set the data in a binding.
     *
     * @remarks
     * If the rows option is used, the value must be "thisRow".
     */
    interface SetBindingDataOptions {
        /**
         * Use only with binding type table and when a TableData object is passed for the data parameter. An array of objects that specify a range of 
         * columns, rows, or cells and specify, as key-value pairs, the cell formatting to apply to that range. 
         * 
         * Example: `[{cells: Office.Table.Data, format: {fontColor: "yellow"}}, {cells: {row: 3, column: 4}, format: {borderColor: "white", fontStyle: "bold"}}]`
         */
        cellFormat?: RangeFormatConfiguration[]
        /**
         * Explicitly sets the shape of the data object. If not supplied is inferred from the data type.
         */
        coercionType?: Office.CoercionType | string
        /**
        * Only for table bindings in content add-ins for Access. Array of strings. Specifies the column names.
        */
        columns?: string[]
        /**
        * Only for table bindings in content add-ins for Access. Specifies the pre-defined string "thisRow" to get data in the currently selected row.
        */
        rows?: string
        /**
        * Specifies the zero-based starting row for a subset of the data in the binding. Only for table or matrix bindings. If omitted, data is set 
        * starting in the first row.
        */
        startRow?: number
        /**
        * Specifies the zero-based starting column for a subset of the data. Only for table or matrix bindings. If omitted, data is set starting in 
        * the first column.
        */
        startColumn?: number
        /**
        * For an inserted table, a list of key-value pairs that specify table formatting options, such as header row, total row, and banded rows. 
        * Example: `{bandedRows: true,  filterButton: false}`
        */
        tableOptions?: object
        /**
         * A user-defined item of any type that is returned, unchanged, in the asyncContext property of the AsyncResult object that is passed to a callback.
         */
        asyncContext?: any
    }
    /**
     * Specifies a range and its formatting.
     */
    interface RangeFormatConfiguration {
        /**
         * Specifies the range. Example of using Office.Table enum: Office.Table.All. Example of using RangeCoordinates: `{row: 3, column: 4}` specifies 
         * the cell in the 3rd (zero-based) row in the 4th (zero-based) column.
         */
         cells: Office.Table | RangeCoordinates
        /**
         * Specifies the formatting as key-value pairs. Example: `{borderColor: "white", fontStyle: "bold"}`
         */
         format: object
    }
    /**
     * Specifies a cell, or row, or column, by its zero-based row and/or column number. Example: `{row: 3, column: 4}` specifies the cell in the 3rd 
     * (zero-based) row in the 4th (zero-based) column.
     */
    interface RangeCoordinates {
        /**
         * The zero-based row of the range. If not specified, all cells, in the column specified by `column` are included.
         */
         row?: number
        /**
         * The zero-based column of the range. If not specified, all cells, in the row specified by `row` are included.
         */
         column?: number
    }
    /**
     * Provides options to determine which event handler or handlers are removed.
     */
    interface RemoveHandlerOptions {
        /**
         * The handler to be removed. If not specified all handlers for the specified event type are removed.
         */
        handler?: string
        /**
         * A user-defined item of any type that is returned, unchanged, in the asyncContext property of the AsyncResult object that is passed to a callback.
         */
        asyncContext?: any
    }
    /**
     * Provides options for configuring the binding that is created.
     */
    interface AddBindingFromNamedItemOptions {
        /**
         * The unique ID of the binding. Autogenerated if not supplied.
         */
        id?: string
        /**
         * A user-defined item of any type that is returned, unchanged, in the asyncContext property of the AsyncResult object that is passed to a callback.
         */
        asyncContext?: any
    }
    /**
     * Provides options for configuring the prompt and identifying the binding that is created.
     */
    interface AddBindingFromPromptOptions {
        /**
         * The unique ID of the binding. Autogenerated if not supplied.
         */
        id?: string
        /**
         * Specifies the string to display in the prompt UI that tells the user what to select. Limited to 200 characters. 
         * If no promptText argument is passed, "Please make a selection" is displayed.
         */
        promptText?: string
        /**
         * Specifies a table of sample data displayed in the prompt UI as an example of the kinds of fields (columns) that can be bound by your add-in. 
         * The headers provided in the TableData object specify the labels used in the field selection UI. 
         * **Note**: This parameter is used only in add-ins for Access. It is ignored if provided when calling the method in an add-in for Excel.
         */
        sampleData?: Office.TableData
        /**
         * A user-defined item of any type that is returned, unchanged, in the asyncContext property of the AsyncResult object that is passed to a callback.
         */
        asyncContext?: any
    }
    /**
     * Provides options for configuring the prompt and identifying the binding that is created.
     */
    interface AddBindingFromSelectionOptions {
        /**
         * The unique ID of the binding. Autogenerated if not supplied.
         */
        id?: string
        /**
         * The names of the columns involved in the binding.
         */
        columns?: string[]
        /**
         * A user-defined item of any type that is returned, unchanged, in the asyncContext property of the AsyncResult object that is passed to a callback.
         */
        asyncContext?: any
    }
    /**
     * Provides options for setting the size of slices that the document will be divided into.
     */
    interface GetFileOptions {
        /**
         * The the size of the slices in bytes. The maximum (and the default) is 4194304 (4MB).
         */
        sliceSize?: number
        /**
         * A user-defined item of any type that is returned, unchanged, in the asyncContext property of the AsyncResult object that is passed to a callback.
         */
        asyncContext?: any
    }
    /**
     * Provides options for customizing what data is returned and how it is formatted.
     */
    interface GetSelectedDataOptions {
        /**
         * Specify whether the data is formatted. Use Office.ValueFormat or string equivalent.
         */
        valueFormat?: Office.ValueFormat | string
        /**
         * Specify whether to get only the visible (that is, filtered-in) data or all the data. Useful when filtering data. 
         * Use {@link Office.FilterType} or string equivalent. This parameter is ignored in Word documents.
         */
        filterType?: Office.FilterType | string
        /**
         * A user-defined item of any type that is returned, unchanged, in the asyncContext property of the AsyncResult object that is passed to a callback.
         */
        asyncContext?: any
    }
    /**
     * Provides options for whether to select the location that is navigated to.
     *
     * @remarks
     * The behavior caused by the {@link Office.SelectionMode | options.selectionMode} option varies by host:
     *
     * In Excel: `Office.SelectionMode.Selected` selects all content in the binding, or named item. `Office.SelectionMode.None` for text bindings, 
     * selects the cell; for matrix bindings, table bindings, and named items, selects the first data cell (not first cell in header row for tables).
     *
     * In PowerPoint: `Office.SelectionMode.Selected` selects the slide title or first textbox on the slide. 
     * `Office.SelectionMode.None` doesn't select anything.
     *
     * In Word: `Office.SelectionMode.Selected` selects all content in the binding. `Office.SelectionMode.None` for text bindings, moves the cursor to 
     * the beginning of the text; for matrix bindings and table bindings, selects the first data cell (not first cell in header row for tables).
     */
    interface GoToByIdOptions {
        /**
         * Specifies whether the location specified by the id parameter is selected (highlighted). 
         * Use {@link Office.SelectionMode} or string equivalent. See the Remarks for more information.
         */
        selectionMode?: Office.SelectionMode | string
        /**
         * A user-defined item of any type that is returned, unchanged, in the asyncContext property of the AsyncResult object that is passed to a callback.
         */
        asyncContext?: any
    }
    /**
     * Provides options for how to insert data to the selection.
     */
    interface SetSelectedDataOptions {
        /**
         * Use only with binding type table and when a TableData object is passed for the data parameter. An array of objects that specify a range of 
         * columns, rows, or cells and specify, as key-value pairs, the cell formatting to apply to that range. 
         * 
         * Example: `[{cells: Office.Table.Data, format: {fontColor: "yellow"}}, {cells: {row: 3, column: 4}, format: {borderColor: "white", fontStyle: "bold"}}]`
         */
        cellFormat?: RangeFormatConfiguration[]
        /**
         * Explicitly sets the shape of the data object. If not supplied is inferred from the data type.
         */
        coercionType?: Office.CoercionType | string
        /**
         * For an inserted table, a list of key-value pairs that specify table formatting options, such as header row, total row, and banded rows. 
         * Example: `{bandedRows: true,  filterButton: false}`
         */
        tableOptions?: object
        /**
        * This option is applicable for inserting images. Indicates the insert location in relation to the top of the slide for PowerPoint, and its 
        * relation to the currently selected cell in Excel. This value is ignored for Word. This value is in points.
        */
        imageTop?: number
        /**
        * This option is applicable for inserting images. Indicates the image width. If this option is provided without the imageHeight, the image 
        * will scale to match the value of the image width. If both image width and image height are provided, the image will be resized accordingly. 
        * If neither the image height or width is provided, the default image size and aspect ratio will be used. This value is in points.
        */
        imageWidth?: number
        /**
        * This option is applicable for inserting images. Indicates the insert location in relation to the left side of the slide for PowerPoint, and 
        * its relation to the currently selected cell in Excel. This value is ignored for Word. This value is in points.
        */
        imageLeft?: number
        /**
        * This option is applicable for inserting images. Indicates the image height. If this option is provided without the imageWidth, the image 
        * will scale to match the value of the image height. If both image width and image height are provided, the image will be resized accordingly. 
        * If neither the image height or width is provided, the default image size and aspect ratio will be used. This value is in points.
        */
        imageHeight?: number
        /**
         * A user-defined item of any type that is returned, unchanged, in the asyncContext property of the AsyncResult object that is passed to a callback.
         */
        asyncContext?: any
    }
    /**
     * Provides options for saving settings.
     */
    interface SaveSettingsOptions {
        /**
         * Indicates whether the setting will be replaced if stale.
         */
        overwriteIfStale?: boolean
        /**
         * A user-defined item of any type that is returned, unchanged, in the asyncContext property of the AsyncResult object that is passed to a callback.
         */
        asyncContext?: any
    }
    /**
     * Provides access to the properties for Office theme colors.
     *
     * Using Office theme colors lets you coordinate the color scheme of your add-in with the current Office theme selected by the user with File \> 
     * Office Account \> Office Theme UI, which is applied across all Office host applications. Using Office theme colors is appropriate for mail and 
     * task pane add-ins.
     * 
     * @remarks
     * **Hosts**: Excel, Outlook (in preview), PowerPoint, Word
     * 
     * `OfficeTheme` is only supported in Office on Windows.
     */
    interface OfficeTheme {
        /**
         * Gets the Office theme body background color as a hexadecimal color triplet (e.g. "FFA500").
         */
        bodyBackgroundColor: string;
        /**
         * Gets the Office theme body foreground color as a hexadecimal color triplet (e.g. "FFA500").
         */
        bodyForegroundColor: string;
        /**
         * Gets the Office theme control background color as a hexadecimal color triplet (e.g. "FFA500").
         */
        controlBackgroundColor: string;
        /**
         * Gets the Office theme control foreground color as a hexadecimal color triplet (e.g. "FFA500").
         */
        controlForegroundColor: string;
    }
    /**
     * The object that is returned when `UI.displayDialogAsync` is called. It exposes methods for registering event handlers and closing the dialog.
     * 
     * @remarks
     * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/dialog-api-requirement-sets | DialogApi}
     */
    interface Dialog {
        /**
         * Called from a parent page to close the corresponding dialog box. 
         */
        close(): void;
        /**
         * Registers an event handler. The two supported events are:
         *
         * - DialogMessageReceived. Triggered when the dialog box sends a message to its parent.
         *
         * - DialogEventReceived. Triggered when the dialog box has been closed or otherwise unloaded.
         */
        addEventHandler(eventType: Office.EventType, handler: Function): void;
        /**
         * FOR INTERNAL USE ONLY. DO NOT CALL IN YOUR CODE.
         */
        sendMessage(name: string): void;
    }

    /**
     * Returns a promise of an object described in the expression. Callback is invoked only if method fails.
     * 
     * @param expression The object to be retrieved. Example "bindings#BindingName", retrieves a binding promise for a binding named 'BindingName'
     * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
     * 
     * @remarks
     * 
     * **Support details**
     * 
     * A capital Y in the following matrix indicates that this method is supported in the corresponding Office host application. 
     * An empty cell indicates that the Office host application doesn't support this method.
     * 
     * For more information about Office host application and server requirements, see 
     * {@link https://docs.microsoft.com/office/dev/add-ins/concepts/requirements-for-running-office-add-ins | Requirements for running Office Add-ins}.
     * 
     * *Supported hosts, by platform*
     *  <table>
     *   <tr><th>                         </th><th> Office on Windows </th><th> Office in web browser </th><th> Office on iPad </th></tr>
     *   <tr><td><strong> Access </strong></td><td>                   </td><td> Y                     </td><td>                </td></tr>
     *   <tr><td><strong> Excel  </strong></td><td> Y                 </td><td> Y                     </td><td> Y              </td></tr>
     *   <tr><td><strong> Word   </strong></td><td> Y                 </td><td>                       </td><td> Y              </td></tr>
     *  </table>
     */
    function select(expression: string, callback?: (result: AsyncResult<any>) => void): Binding;

    // Enumerations

    /**
     * Specifies the state of the active view of the document, for example, whether the user can edit the document.
     */
    enum ActiveView {
        /**
         * The active view of the host application only lets the user read the content in the document.
         */
        Read,
        /**
         * The active view of the host application lets the user edit the content in the document.
         */
        Edit
    }
    /**
     * Specifies the type of the binding object that should be returned.
     */
    enum BindingType {
        /**
         * Plain text. Data is returned as a run of characters.
         */
        Text,
        /**
         * Tabular data without a header row. Data is returned as an array of arrays, for example in this form: 
         * [[row1column1, row1column2],[row2column1, row2column2]]
         */
        Matrix,
        /**
         * Tabular data with a header row. Data is returned as a {@link Office.TableData | TableData} object.
         */
        Table
    }
    /**
     * Specifies how to coerce data returned or set by the invoked method.
     *
     * @remarks
     * 
     * PowerPoint supports only `Office.CoercionType.Text`, `Office.CoercionType.Image`, `Office.CoercionType.SlideRange`, and `Office.CoercionType.XmlSvg`.
     * 
     * Project supports only `Office.CoercionType.Text`.
     */
    enum CoercionType {
        /**
         * Return or set data as text (string). Data is returned or set as a one-dimensional run of characters.
         */
        Text,
        /**
         * Return or set data as tabular data with no headers. Data is returned or set as an array of arrays containing one-dimensional runs of 
         * characters. For example, three rows of  string values in two columns would be: [["R1C1", "R1C2"], ["R2C1", "R2C2"], ["R3C1", "R3C2"]].
         *
         * **Note**: Only applies to data in Excel and Word.
         */
        Matrix,
        /**
         * Return or set data as tabular data with optional headers. Data is returned or set as an array of arrays with optional headers.
         * 
         * **Note**: Only applies to data in Access, Excel, and Word.
         */
        Table,
        /**
         * Return or set data as HTML.
         * 
         * **Note**: Only applies to data in add-ins for Word and Outlook add-ins for Outlook (compose mode).
         */
        Html,
        /**
         * Return or set data as Office Open XML.
         * 
         * **Note**: Only applies to data in Word.
         */
        Ooxml,
        /**
         * Return a JSON object that contains an array of the ids, titles, and indexes of the selected slides. For example, 
         * `{"slides":[{"id":257,"title":"Slide 2","index":2},{"id":256,"title":"Slide 1","index":1}]}` for a selection of two slides.
         * 
         * **Note**: Only applies to data in PowerPoint when calling the {@link Office.Document | Document}.getSelectedData method to get the current 
         * slide or selected range of slides.
         */
        SlideRange,
        /**
        * Data is returned or set as an image stream.
        * **Note**: Only applies to data in Excel, Word, and PowerPoint.
        */
        Image,
        /**
         * Data is returned or set as XML data containing an SVG image.
         * **Note**: Only applies to data in Excel, Word, and PowerPoint.
         */
        XmlSvg
    }
    /**
     * Specifies whether the document in the associated application is read-only or read-write.
     * 
     * @remarks
     *  
     * Returned by the mode property of the {@link Office.Document | Document} object.
     */
    enum DocumentMode {
        /**
         * The document is read-only.
         */
        ReadOnly,
        /**
         * The document can be read and written to.
         */
        ReadWrite
    }
    /**
     * Specifies the type of the XML node.
     */
    enum CustomXMLNodeType {
        /**
         * The node is an attribute.
         */
        Attribute,
        /**
         * The node is CData.
         */
        CData,
        /**
         * The node is a comment.
         */
        NodeComment,
        /**
         * The node is an element.
         */
        Element,
        /**
         * The node is a Document element.
         */
        NodeDocument,
        /**
         * The node is a processing instruction.
         */
        ProcessingInstruction,
        /**
         * The node is text.
         */
        Text,
    }
    /**
     * Specifies the kind of event that was raised. Returned by the `type` property of an *EventArgs object.
     * 
     * Add-ins for Project support the `Office.EventType.ResourceSelectionChanged`, `Office.EventType.TaskSelectionChanged`, and 
     * `Office.EventType.ViewSelectionChanged` event types.
     * 
     * @remarks
     * 
     * **`BindingDataChanged` and `BindingSelectionChanged` hosts**: Access, Excel, Word.
     * 
     */
    enum EventType {
        /**
         * A Document.ActiveViewChanged event was raised.
         * 
         * @remarks 
         * **Hosts**: PowerPoint
         */
        ActiveViewChanged,
        /**
         * Triggers when any date or time of the selected appointment or series is changed in Outlook.
         * 
         * [Api set: Mailbox 1.7]
         */
        AppointmentTimeChanged,
        /**
         * Triggers when an attachment is added to or removed from an item.
         * 
         * [Api set: Mailbox Preview]
         * 
         * @beta
         */
        AttachmentsChanged,
        /**
         * Occurs when data within the binding is changed. 
         * To add an event handler for the BindingDataChanged event of a binding, use the addHandlerAsync method of the Binding object. 
         * The event handler receives an argument of type {@link Office.BindingDataChangedEventArgs}.
         * 
         * @remarks 
         * **Hosts**: Access, Excel, Word
         */
        BindingDataChanged,
        /**
         * Occurs when the selection is changed within the binding. To add an event handler for the BindingSelectionChanged event of a binding, use 
         * the addHandlerAsync method of the Binding object. The event handler receives an argument of type {@link Office.BindingSelectionChangedEventArgs}.
         * 
         * @remarks 
         * **Hosts**: Access, Excel, Word 
         */
        BindingSelectionChanged,
        /**
         * Triggers when Dialog sends a message via MessageParent.
         */
        DialogMessageReceived,
        /**
         * Triggers when Dialog has an event, such as dialog closed or dialog navigation failed.
         */
        DialogEventReceived,
        /**
         * Triggers when a document-level selection happens.
         * 
         * @remarks 
         * **Hosts**: Excel, Word
         */
        DocumentSelectionChanged,
        /**
         * Triggers when a different Outlook item is selected for viewing while the task pane is pinned.
         * 
         * [Api set: Mailbox 1.5]
         */
        ItemChanged,
        /**
         * Triggers when the appointment location is changed in Outlook.
         * 
         * [Api set: Mailbox Preview]
         */
        EnhancedLocationsChanged,
        /**
         * Triggers when a customXmlPart node is deleted.
         */
        NodeDeleted,
        /**
         * Triggers when a customXmlPart node is inserted.
         */
        NodeInserted,
        /**
         * Triggers when a customXmlPart node is replaced.
         */
        NodeReplaced,
        /**
         * Triggers when the OfficeTheme is changed in Outlook.
         * 
         * [Api set: Mailbox Preview]
         * 
         * @beta
         */
        OfficeThemeChanged,
        /**
         * Triggers when the recipient list of the selected item or the appointment location is changed in Outlook.
         * 
         * [Api set: Mailbox 1.7]
         */
        RecipientsChanged,
        /**
         * Triggers when the recurrence pattern of the selected series is changed in Outlook.
         * 
         * [Api set: Mailbox 1.7]
         */
        RecurrenceChanged,
        /**
         * Triggers when a Resource selection happens in Project.
         */
        ResourceSelectionChanged,
        /**
         * A Settings.settingsChanged event was raised.
         * 
         * @remarks 
         * **Hosts**: Access, Excel, PowerPoint, Word
         */
        SettingsChanged,
        /**
         * Triggers when a Task selection happens in Project.
         */
        TaskSelectionChanged,
        /**
         * Triggers when a View selection happens in Project.
         */
        ViewSelectionChanged
    }
    /**
     * Specifies the format in which to return the document.
     *
     * @remarks
     * `FileType.Text` is only supported in Word, `FileType.Pdf` is only supported in Word on the web, Windows, and Mac, and PowerPoint.
     */
    enum FileType {
        /**
         * Returns only the text of the document as a string. (Word only)
         */
        Text,
        /**
         * Returns the entire document (.pptx, .docx, or .xlsx) in Office Open XML (OOXML) format as a byte array.
         */
        Compressed,
        /**
         * Returns the entire document in PDF format as a byte array.
         */
        Pdf
    }
    /**
     * Specifies whether filtering from the host application is applied when the data is retrieved.
     */
    enum FilterType {
        /**
         * Return all data (not filtered by the host application).
         */
        All,
        /**
         * Return only the visible data (as filtered by the host application).
         */
        OnlyVisible
    }
    /**
     * Specifies the type of place or object to navigate to.
     */
    enum GoToType {
        /**
         * Goes to a binding object using the specified binding id.
         * 
         * Supported hosts: Excel, Word
         */
        Binding,
        /**
         * Goes to a named item using that item's name.
         * In Excel, you can use any structured reference for a named range or table: "Worksheet2!Table1"
         * 
         * Supported hosts: Excel
         */
        NamedItem,
        /**
         * Goes to a slide using the specified id.
         * 
         * Supported hosts: PowerPoint
         */
        Slide,
        /**
         * Goes to the specified index by slide number or {@link Office.Index}.
         * 
         * Supported hosts: PowerPoint
         */
        Index
    }
    /**
     * Specifies the relative PowerPoint slide.
     */
    enum Index {
        /**
         * Represents the first PowerPoint slide
         */
        First,
        /**
         * Represents the last PowerPoint slide
         */
        Last,
        /**
         * Represents the next PowerPoint slide
         */
        Next,
        /**
         * Represents the previous PowerPoint slide
         */
        Previous
    }
    /**
     * Specifies whether to select (highlight) the location to navigate to (when using the {@link Office.Document | Document}.goToByIdAsync method).
     */
    enum SelectionMode {
        Default,
        /**
         * The location will be selected (highlighted).
         */
        Selected,
        /**
         * The cursor is moved to the beginning of the location.
         */
        None
    }
    /**
     * Specifies whether values, such as numbers and dates, returned by the invoked method are returned with their formatting applied.
     *
     * @remarks
     * For example, if the valueFormat parameter is specified as "formatted", a number formatted as currency, or a date formatted as mm/dd/yy in the 
     * host application will have its formatting preserved. If the valueFormat parameter is specified as "unformatted", a date will be returned in its 
     * underlying sequential serial number form.
     */
    enum ValueFormat {
        /**
         * Return unformatted data.
         */
        Unformatted,
        /**
         * Return formatted data.
         */
        Formatted
    }
    // Objects
    /**
    * Represents a binding to a section of the document.
    *
    * The Binding object exposes the functionality possessed by all bindings regardless of type.
    *
    * The Binding object is never called directly. It is the abstract parent class of the objects that represent each type of binding: 
    * {@link Office.MatrixBinding}, {@link Office.TableBinding}, or {@link Office.TextBinding}. All three of these objects inherit the getDataAsync 
    * and setDataAsync methods from the Binding object that enable to you interact with the data in the binding. They also inherit the id and type 
    * properties for querying those property values. Additionally, the MatrixBinding and TableBinding objects expose additional methods for matrix- 
    * and table-specific features, such as counting the number of rows and columns.
    *
    * @remarks
    * 
    * **Hosts**: Access, Excel, Word
    * 
    * **Requirement sets**: 
    * 
    * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixbindings | MatrixBindings}
    * 
    * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablebindings | TableBindings}
    * 
    * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textbindings | TextBindings}
    */
    interface Binding {
        /**
        * Get the Document object associated with the binding.
        */
        document: Office.Document;
        /**
         * A string that uniquely identifies this binding among the bindings in the same {@link Office.Document} object.
         */
        id: string;
        /**
        * Gets the type of the binding.
        */
        type: Office.BindingType;
        /**
         * Adds an event handler to the object for the specified {@link Office.EventType}. Supported EventTypes are 
         * `Office.EventType.BindingDataChanged` and `Office.EventType.BindingSelectionChanged`.
         *
         * @remarks
         * 
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#bindingevents | BindingEvents}
         *  
         * You can add multiple event handlers for the specified eventType as long as the name of each event handler function is unique.
         *
         * @param eventType The event type. For bindings, it can be `Office.EventType.BindingDataChanged` or `Office.EventType.BindingSelectionChanged`.
         * @param handler The event handler function to add, whose only parameter is of type {@link Office.BindingDataChangedEventArgs} or {@link Office.BindingSelectionChangedEventArgs}.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        addHandlerAsync(eventType: Office.EventType, handler: any, options?: Office.AsyncContextOptions, callback?: (result: Office.AsyncResult<void>) => void): void;
        /**
         * Adds an event handler to the object for the specified {@link Office.EventType}. Supported EventTypes are 
         * `Office.EventType.BindingDataChanged` and `Office.EventType.BindingSelectionChanged`.
         *
         * @remarks
         * 
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#bindingevents | BindingEvents}
         * 
         * You can add multiple event handlers for the specified eventType as long as the name of each event handler function is unique.
         *
         * @param eventType The event type. For bindings, it can be `Office.EventType.BindingDataChanged` or `Office.EventType.BindingSelectionChanged`.
         * @param handler The event handler function to add, whose only parameter is of type {@link Office.BindingDataChangedEventArgs} or {@link Office.BindingSelectionChangedEventArgs}.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        addHandlerAsync(eventType: Office.EventType, handler: any, callback?: (result: Office.AsyncResult<void>) => void): void;
        /**
         * Returns the data contained within the binding.
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#htmlcoercion | HtmlCoercion} (when using `Office.CoercionType.Html`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixbindings | MatrixBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixcoercion | MatrixCoercion} (when using `Office.CoercionType.Matrix`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#ooxmlcoercion | OoxmlCoercion} (when using `Office.CoercionType.Ooxml`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablebindings | TableBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablecoercion | TableCoercion} (when using `Office.CoercionType.Table`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textbindings | TextBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textcoercion | TextCoercion} (when using `Office.CoercionType.Text`)
         *
         * When called from a MatrixBinding or TableBinding, the getDataAsync method will return a subset of the bound values if the optional startRow, 
         * startColumn, rowCount, and columnCount parameters are specified (and they specify a contiguous and valid range).
         *
         * @param options Provides options for how to get the data in a binding.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the values in the specified binding. 
         *                  If the `coercionType` parameter is specified (and the call is successful), the data is returned in the format described in the CoercionType enumeration topic.
         */
        getDataAsync<T>(options?: GetBindingDataOptions, callback?: (result: AsyncResult<T>) => void): void;
        /**
         * Returns the data contained within the binding.
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#htmlcoercion | HtmlCoercion} (when using `Office.CoercionType.Html`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixbindings | MatrixBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixcoercion | MatrixCoercion} (when using `Office.CoercionType.Matrix`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#ooxmlcoercion | OoxmlCoercion} (when using `Office.CoercionType.Ooxml`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablebindings | TableBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablecoercion | TableCoercion} (when using `Office.CoercionType.Table`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textbindings | TextBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textcoercion | TextCoercion} (when using `Office.CoercionType.Text`)
         * 
         * When called from a MatrixBinding or TableBinding, the getDataAsync method will return a subset of the bound values if the optional startRow, 
         * startColumn, rowCount, and columnCount parameters are specified (and they specify a contiguous and valid range).
         *
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the values in the specified binding. 
         *                  If the `coercionType` parameter is specified (and the call is successful), the data is returned in the format described in the CoercionType enumeration topic.
         */
        getDataAsync<T>(callback?: (result: AsyncResult<T>) => void): void;
        /**
         * Removes the specified handler from the binding for the specified event type.
         *
         * @remarks
         * 
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#bindingevents | BindingEvents}
         * 
         * @param eventType The event type. For bindings, it can be `Office.EventType.BindingDataChanged` or `Office.EventType.BindingSelectionChanged`.
         * @param options Provides options to determine which event handler or handlers are removed.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        removeHandlerAsync(eventType: Office.EventType, options?: RemoveHandlerOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Removes the specified handler from the binding for the specified event type.
         *
         * @remarks
         * 
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#bindingevents | BindingEvents}
         * 
         * @param eventType The event type. For bindings, it can be `Office.EventType.BindingDataChanged` or `Office.EventType.BindingSelectionChanged`.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        removeHandlerAsync(eventType: Office.EventType, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Writes data to the bound section of the document represented by the specified binding object.
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#htmlcoercion | HtmlCoercion} (when using `Office.CoercionType.Html`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixbindings | MatrixBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixcoercion | MatrixCoercion} (when using `Office.CoercionType.Matrix`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#ooxmlcoercion | OoxmlCoercion} (when using `Office.CoercionType.Ooxml`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablebindings | TableBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablecoercion | TableCoercion} (when using `Office.CoercionType.Table`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textbindings | TextBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textcoercion | TextCoercion} (when using `Office.CoercionType.Text`)
         *
         * The value passed for data contains the data to be written in the binding. The kind of value passed determines what will be written as 
         * described in the following table.
         * 
         * <table>
         *   <tr>
         *     <th>`data` value</th>
         *     <th>Data written</th>
         *   </tr>
         *   <tr>
         *     <td>A string</td>
         *     <td>Plain text or anything that can be coerced to a string will be written.</td>
         *   </tr>
         *   <tr>
         *     <td>An array of arrays ("matrix")</td>
         *     <td>Tabular data without headers will be written. For example, to write data to three rows in two columns, you can pass an array like this: `[["R1C1", "R1C2"], ["R2C1", "R2C2"], ["R3C1", "R3C2"]]`. To write a single column of three rows, pass an array like this: `[["R1C1"], ["R2C1"], ["R3C1"]]`.</td>
         *   </tr>
         *    <tr>
         *     <td>A `TableData` object</td>
         *     <td>A table with headers will be written.</td>
         *   </tr>
         * </table>
         * 
         * Additionally, these application-specific actions apply when writing data to a binding. For Word, the specified data is written to the 
         * binding as follows:
         * 
         * <table>
         *   <tr>
         *     <th>`data` value</th>
         *     <th>Data written</th>
         *   </tr>
         *   <tr>
         *     <td>A string</td>
         *     <td>The specified text is written.</td>
         *   </tr>
         *   <tr>
         *     <td>An array of arrays ("matrix") or a `TableData` object</td>
         *     <td>A Word table is written.</td>
         *   </tr>
         *   <tr>
         *     <td>HTML</td>
         *     <td>The specified HTML is written. If any of the HTML you write is invalid, Word will not raise an error. Word will write as much of the HTML as it can and will omit any invalid data.</td>
         *   </tr>
         *   <tr>
         *     <td>Office Open XML ("Open XML")</td>
         *     <td>The specified the XML is written.</td>
         *   </tr>
         * </table>
         * 
         * For Excel, the specified data is written to the binding as follows:
         * 
         * <table>
         *   <tr>
         *     <th>`data` value</th>
         *     <th>Data written</th>
         *   </tr>
         *   <tr>
         *     <td>A string</td>
         *     <td>The specified text is inserted as the value of the first bound cell.You can also specify a valid formula to add that formula to the bound cell. For example, setting  data to `"=SUM(A1:A5)"` will total the values in the specified range. However, when you set a formula on the bound cell, after doing so, you can't read the added formula (or any pre-existing formula) from the bound cell. If you call the Binding.getDataAsync method on the bound cell to read its data, the method can return only the data displayed in the cell (the formula's result).</td>
         *   </tr>
         *   <tr>
         *     <td>An array of arrays ("matrix"), and the shape exactly matches the shape of the binding specified</td>
         *     <td>The set of rows and columns are written.You can also specify an array of arrays that contain valid formulas to add them to the bound cells. For example, setting  data to `[["=SUM(A1:A5)","=AVERAGE(A1:A5)"]]` will add those two formulas to a binding that contains two cells. Just as when setting a formula on a single bound cell, you can't read the added formulas (or any pre-existing formulas) from the binding with the `Binding.getDataAsync` method - it returns only the data displayed in the bound cells.</td>
         *   </tr>
         *   <tr>
         *     <td>A `TableData` object, and the shape of the table matches the bound table.</td>
         *     <td>The specified set of rows and/or headers are written, if no other data in surrounding cells will be overwritten. **Note**: If you specify formulas in the TableData object you pass for the *data* parameter, you might not get the results you expect due to the "calculated columns" feature of Excel, which automatically duplicates formulas within a column. To work around this when you want to write *data* that contains formulas to a bound table, try specifying the data as an array of arrays (instead of a TableData object), and specify the *coercionType* as Microsoft.Office.Matrix or "matrix".</td>
         *   </tr>
         * </table>
         * 
         * For Excel on the web:
         * 
         *  - The total number of cells in the value passed to the data parameter can't exceed 20,000 in a single call to this method.
         * 
         *  - The number of formatting groups passed to the cellFormat parameter can't exceed 100. 
         * A single formatting group consists of a set of formatting applied to a specified range of cells.
         * 
         * In all other cases, an error is returned.
         * 
         * The setDataAsync method will write data in a subset of a table or matrix binding if the optional startRow and startColumn parameters are 
         * specified, and they specify a valid range.
         * 
         * In the callback function passed to the setDataAsync method, you can use the properties of the AsyncResult object to return the following 
         * information.
         * 
         * <table>
         *   <tr>
         *     <th>Property</th>
         *     <th>Use to...</th>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.value</td>
         *     <td>Always returns undefined because there is no object or data to retrieve.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.status</td>
         *     <td>Determine the success or failure of the operation.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.error</td>
         *     <td>Access an Error object that provides error information if the operation failed.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.asyncContext</td>
         *     <td>A user-defined item of any type that is returned in the AsyncResult object without being altered.</td>
         *   </tr>
         * </table>
         *
         * @param data The data to be set in the current selection. Possible data types by host:
         *
         *        string: Excel on the web and Windows, and Word on the web and Windows only
         *
         *        array of arrays: Excel and Word only
         *
         *        `TableData`: Access, Excel, and Word only
         *
         *        HTML: Word on the web and Windows only
         *
         *        Office Open XML: Word only
         *
         * @param options Provides options for how to set the data in a binding.
         *
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        setDataAsync(data: TableData | any, options?: SetBindingDataOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Writes data to the bound section of the document represented by the specified binding object.
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#htmlcoercion | HtmlCoercion} (when using `Office.CoercionType.Html`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixbindings | MatrixBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixcoercion | MatrixCoercion} (when using `Office.CoercionType.Matrix`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#ooxmlcoercion | OoxmlCoercion} (when using `Office.CoercionType.Ooxml`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablebindings | TableBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablecoercion | TableCoercion} (when using `Office.CoercionType.Table`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textbindings | TextBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textcoercion | TextCoercion} (when using `Office.CoercionType.Text`)
         * 
         * The value passed for data contains the data to be written in the binding. The kind of value passed determines what will be written as 
         * described in the following table.
         * 
         * <table>
         *   <tr>
         *     <th>`data` value</th>
         *     <th>Data written</th>
         *   </tr>
         *   <tr>
         *     <td>A string</td>
         *     <td>Plain text or anything that can be coerced to a string will be written.</td>
         *   </tr>
         *   <tr>
         *     <td>An array of arrays ("matrix")</td>
         *     <td>Tabular data without headers will be written. For example, to write data to three rows in two columns, you can pass an array like this: `[["R1C1", "R1C2"], ["R2C1", "R2C2"], ["R3C1", "R3C2"]]`. To write a single column of three rows, pass an array like this: `[["R1C1"], ["R2C1"], ["R3C1"]]`.</td>
         *   </tr>
         *    <tr>
         *     <td>A `TableData` object</td>
         *     <td>A table with headers will be written.</td>
         *   </tr>
         * </table>
         * 
         * Additionally, these application-specific actions apply when writing data to a binding. For Word, the specified data is written to the 
         * binding as follows:
         * 
         * <table>
         *   <tr>
         *     <th>`data` value</th>
         *     <th>Data written</th>
         *   </tr>
         *   <tr>
         *     <td>A string</td>
         *     <td>The specified text is written.</td>
         *   </tr>
         *   <tr>
         *     <td>An array of arrays ("matrix") or an `TableData` object</td>
         *     <td>A Word table is written.</td>
         *   </tr>
         *   <tr>
         *     <td>HTML</td>
         *     <td>The specified HTML is written. If any of the HTML you write is invalid, Word will not raise an error. Word will write as much of the HTML as it can and will omit any invalid data.</td>
         *   </tr>
         *   <tr>
         *     <td>Office Open XML ("Open XML")</td>
         *     <td>The specified the XML is written.</td>
         *   </tr>
         * </table>
         * 
         * For Excel, the specified data is written to the binding as follows:
         * 
         * <table>
         *   <tr>
         *     <th>`data` value</th>
         *     <th>Data written</th>
         *   </tr>
         *   <tr>
         *     <td>A string</td>
         *     <td>The specified text is inserted as the value of the first bound cell.You can also specify a valid formula to add that formula to the bound cell. For example, setting  data to `"=SUM(A1:A5)"` will total the values in the specified range. However, when you set a formula on the bound cell, after doing so, you can't read the added formula (or any pre-existing formula) from the bound cell. If you call the Binding.getDataAsync method on the bound cell to read its data, the method can return only the data displayed in the cell (the formula's result).</td>
         *   </tr>
         *   <tr>
         *     <td>An array of arrays ("matrix"), and the shape exactly matches the shape of the binding specified</td>
         *     <td>The set of rows and columns are written.You can also specify an array of arrays that contain valid formulas to add them to the bound cells. For example, setting  data to `[["=SUM(A1:A5)","=AVERAGE(A1:A5)"]]` will add those two formulas to a binding that contains two cells. Just as when setting a formula on a single bound cell, you can't read the added formulas (or any pre-existing formulas) from the binding with the `Binding.getDataAsync` method - it returns only the data displayed in the bound cells.</td>
         *   </tr>
         *   <tr>
         *     <td>A `TableData` object, and the shape of the table matches the bound table.</td>
         *     <td>The specified set of rows and/or headers are written, if no other data in surrounding cells will be overwritten. **Note**: If you specify formulas in the TableData object you pass for the *data* parameter, you might not get the results you expect due to the "calculated columns" feature of Excel, which automatically duplicates formulas within a column. To work around this when you want to write *data* that contains formulas to a bound table, try specifying the data as an array of arrays (instead of a TableData object), and specify the *coercionType* as Microsoft.Office.Matrix or "matrix".</td>
         *   </tr>
         * </table>
         * 
         * For Excel on the web:
         * 
         *  - The total number of cells in the value passed to the data parameter can't exceed 20,000 in a single call to this method.
         * 
         *  - The number of formatting groups passed to the cellFormat parameter can't exceed 100. 
         * A single formatting group consists of a set of formatting applied to a specified range of cells.
         * 
         * In all other cases, an error is returned.
         * 
         * The setDataAsync method will write data in a subset of a table or matrix binding if the optional startRow and startColumn parameters are 
         * specified, and they specify a valid range.
         * 
         * In the callback function passed to the setDataAsync method, you can use the properties of the AsyncResult object to return the following 
         * information.
         * 
         * <table>
         *   <tr>
         *     <th>Property</th>
         *     <th>Use to...</th>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.value</td>
         *     <td>Always returns undefined because there is no object or data to retrieve.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.status</td>
         *     <td>Determine the success or failure of the operation.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.error</td>
         *     <td>Access an Error object that provides error information if the operation failed.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.asyncContext</td>
         *     <td>A user-defined item of any type that is returned in the AsyncResult object without being altered.</td>
         *   </tr>
         * </table>
         *
         * @param data The data to be set in the current selection. Possible data types by host:
         *
         *        string: Excel on the web and Windows, and Word on the web and Windows only
         *
         *        array of arrays: Excel and Word only
         *
         *        `TableData`: Access, Excel, and Word only
         *
         *        HTML: Word on the web and Windows only
         *
         *        Office Open XML: Word only
         *
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        setDataAsync(data: TableData | any, callback?: (result: AsyncResult<void>) => void): void;
    }

    /**
     * Provides information about the binding that raised the DataChanged event.
     */
    interface BindingDataChangedEventArgs {
        /**
         * Gets an {@link Office.Binding} object that represents the binding that raised the DataChanged event.
         */
        binding: Binding;

        /**
         * Gets an {@link Office.EventType} enumeration value that identifies the kind of event that was raised.
         */
        type: EventType;
    }

    /**
     * Provides information about the binding that raised the SelectionChanged event.
     */
    interface BindingSelectionChangedEventArgs {
        /**
         * Gets an {@link Office.Binding} object that represents the binding that raised the SelectionChanged event.
         */
        binding: Binding;
        /**
         * Gets the number of columns selected. If a single cell is selected returns 1. 
         * 
         * If the user makes a non-contiguous selection, the count for the last contiguous selection within the binding is returned. 
         * 
         * For Word, this property will work only for bindings of {@link Office.BindingType} "table". If the binding is of type "matrix", null is 
         * returned. Also, the call will fail if the table contains merged cells, because the structure of the table must be uniform for this property 
         * to work correctly.
         */
        columnCount: number;
        /**
         * Gets the number of rows selected. If a single cell is selected returns 1. 
         * 
         * If the user makes a non-contiguous selection, the count for the last contiguous selection within the binding is returned. 
         * 
         * For Word, this property will work only for bindings of {@link Office.BindingType} "table". If the binding is of type "matrix", null is 
         * returned. Also, the call will fail if the table contains merged cells, because the structure of the table must be uniform for this property 
         * to work correctly.
         */
        rowCount: number;
        /**
         * The zero-based index of the first column of the selection counting from the leftmost column in the binding. 
         * 
         * If the user makes a non-contiguous selection, the coordinates for the last contiguous selection within the binding are returned. 
         * 
         * For Word, this property will work only for bindings of {@link Office.BindingType} "table". If the binding is of type "matrix", null is 
         * returned. Also, the call will fail if the table contains merged cells, because the structure of the table must be uniform for this property 
         * to work correctly.
         */
        startColumn: number;
        /**
         * The zero-based index of the first row of the selection counting from the first row in the binding. 
         * 
         * If the user makes a non-contiguous selection, the coordinates for the last contiguous selection within the binding are returned. 
         * 
         * For Word, this property will work only for bindings of {@link Office.BindingType} "table". If the binding is of type "matrix", null is 
         * returned. Also, the call will fail if the table contains merged cells, because the structure of the table must be uniform for this property 
         * to work correctly.
         */
        startRow: number;
        /**
         * Gets an {@link Office.EventType} enumeration value that identifies the kind of event that was raised.
         */
        type: EventType;
    }

    /**
    * Represents the bindings the add-in has within the document.
    */
    interface Bindings {
        /**
         * Gets an {@link Office.Document} object that represents the document associated with this set of bindings.
         */
        document: Document;
        /**
         * Creates a binding against a named object in the document.
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixbindings | MatrixBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablebindings | TableBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textbindings | TextBindings}
         *
         * For Excel, the itemName parameter can refer to a named range or a table.
         *
         * By default, adding a table in Excel assigns the name "Table1" for the first table you add, "Table2" for the second table you add, and so on. 
         * To assign a meaningful name for a table in the Excel UI, use the Table Name property on the Table Tools | Design tab of the ribbon.
         *
         *     **Note**: In Excel, when specifying a table as a named item, you must fully qualify the name to include the worksheet name in the name of 
         * the table in this format: "Sheet1!Table1"
         *
         * For Word, the itemName parameter refers to the Title property of a Rich Text content control. (You can't bind to content controls other 
         * than the Rich Text content control).
         *
         * By default, a content control has no Title value assigned. To assign a meaningful name in the Word UI, after inserting a Rich Text content 
         * control from the Controls group on the Developer tab of the ribbon, use the Properties command in the Controls group to display the Content 
         * Control Properties dialog box. Then set the Title property of the content control to the name you want to reference from your code.
         *
         *     **Note**: In Word, if there are multiple Rich Text content controls with the same Title property value (name), and you try to bind to one 
         * these content controls with this method (by specifying its name as the itemName parameter), the operation will fail.
         *
         * @param itemName Name of the bindable object in the document. For Example 'MyExpenses' table in Excel."
         * @param bindingType The {@link Office.BindingType} for the data. The method returns null if the selected object cannot be coerced into the specified type.
         * @param options Provides options for configuring the binding that is created.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the Binding object that represents the specified named item.
         */
        addFromNamedItemAsync(itemName: string, bindingType: BindingType, options?: AddBindingFromNamedItemOptions, callback?: (result: AsyncResult<Binding>) => void): void;
        /**
         * Creates a binding against a named object in the document.
         *
         * @remarks
         * {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixbindings | MatrixBindings}, 
         * {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablebindings | TableBindings}, 
         * {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textbindings | TextBindings}
         *
         * For Excel, the itemName parameter can refer to a named range or a table.
         *
         * By default, adding a table in Excel assigns the name "Table1" for the first table you add, "Table2" for the second table you add, and so on. 
         * To assign a meaningful name for a table in the Excel UI, use the Table Name property on the Table Tools | Design tab of the ribbon.
         *
         *     **Note**: In Excel, when specifying a table as a named item, you must fully qualify the name to include the worksheet name in the name of 
         * the table in this format: "Sheet1!Table1"
         *
         * For Word, the itemName parameter refers to the Title property of a Rich Text content control. (You can't bind to content controls other 
         * than the Rich Text content control).
         *
         * By default, a content control has no Title value assigned. To assign a meaningful name in the Word UI, after inserting a Rich Text content 
         * control from the Controls group on the Developer tab of the ribbon, use the Properties command in the Controls group to display the Content 
         * Control Properties dialog box. Then set the Title property of the content control to the name you want to reference from your code.
         *
         *     **Note**: In Word, if there are multiple Rich Text content controls with the same Title property value (name), and you try to bind to one 
         * these content controls with this method (by specifying its name as the itemName parameter), the operation will fail.
         *
         * @param itemName Name of the bindable object in the document. For Example 'MyExpenses' table in Excel."
         * @param bindingType The {@link Office.BindingType} for the data. The method returns null if the selected object cannot be coerced into the specified type.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the Binding object that represents the specified named item.
         */
        addFromNamedItemAsync(itemName: string, bindingType: BindingType, callback?: (result: AsyncResult<Binding>) => void): void;
        /**
         * Create a binding by prompting the user to make a selection on the document.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#methods-that-arent-part-of-a-requirement-set | Not in a set}
         *
         * Adds a binding object of the specified type to the Bindings collection, which will be identified with the supplied id. 
         * The method fails if the specified selection cannot be bound.
         *
         * @param bindingType Specifies the type of the binding object to create. Required. 
         *                    Returns null if the selected object cannot be coerced into the specified type.
         * @param options Provides options for configuring the prompt and identifying the binding that is created.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the Binding object that represents the selection specified by the user.
         */
        addFromPromptAsync(bindingType: BindingType, options?: AddBindingFromPromptOptions, callback?: (result: AsyncResult<Binding>) => void): void;
        /**
         * Create a binding by prompting the user to make a selection on the document.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#methods-that-arent-part-of-a-requirement-set | Not in a set}
         *
         * Adds a binding object of the specified type to the Bindings collection, which will be identified with the supplied id. 
         * The method fails if the specified selection cannot be bound.
         *
         * @param bindingType Specifies the type of the binding object to create. Required. 
         *                    Returns null if the selected object cannot be coerced into the specified type.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the Binding object that represents the selection specified by the user.
         */
        addFromPromptAsync(bindingType: BindingType, callback?: (result: AsyncResult<Binding>) => void): void;
        /**
         * Create a binding based on the user's current selection.
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixbindings | MatrixBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablebindings | TableBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textbindings | TextBindings}
         *
         * Adds the specified type of binding object to the Bindings collection, which will be identified with the supplied id.
         *
         * Note In Excel, if you call the addFromSelectionAsync method passing in the Binding.id of an existing binding, the Binding.type of that 
         * binding is used, and its type cannot be changed by specifying a different value for the bindingType parameter. 
         * If you need to use an existing id and change the bindingType, call the Bindings.releaseByIdAsync method first to release the binding, and 
         * then call the addFromSelectionAsync method to reestablish the binding with a new type.
         *
         * @param bindingType Specifies the type of the binding object to create. Required. 
         *                    Returns null if the selected object cannot be coerced into the specified type.
         * @param options Provides options for configuring the prompt and identifying the binding that is created.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the Binding object that represents the selection specified by the user.
         */
        addFromSelectionAsync(bindingType: BindingType, options?: AddBindingFromSelectionOptions, callback?: (result: AsyncResult<Binding>) => void): void;
        /**
         * Create a binding based on the user's current selection.
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixbindings | MatrixBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablebindings | TableBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textbindings | TextBindings}
         *
         * Adds the specified type of binding object to the Bindings collection, which will be identified with the supplied id.
         *
         * Note In Excel, if you call the addFromSelectionAsync method passing in the Binding.id of an existing binding, the Binding.type of that 
         * binding is used, and its type cannot be changed by specifying a different value for the bindingType parameter. 
         * If you need to use an existing id and change the bindingType, call the Bindings.releaseByIdAsync method first to release the binding, and 
         * then call the addFromSelectionAsync method to reestablish the binding with a new type.
         *
         * @param bindingType Specifies the type of the binding object to create. Required. 
         *                    Returns null if the selected object cannot be coerced into the specified type.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the Binding object that represents the selection specified by the user.
         */
        addFromSelectionAsync(bindingType: BindingType, callback?: (result: AsyncResult<Binding>) => void): void;
        /**
         * Gets all bindings that were previously created.
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixbindings | MatrixBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablebindings | TableBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textbindings | TextBindings}
         *
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is an array that contains each binding created for the referenced Bindings object.
         */
        getAllAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<Binding[]>) => void): void;
        /**
         * Gets all bindings that were previously created.
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixbindings | MatrixBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablebindings | TableBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textbindings | TextBindings}
         *
         * @param callback A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is an array that contains each binding created for the referenced Bindings object.
         */
        getAllAsync(callback?: (result: AsyncResult<Binding[]>) => void): void;
        /**
         * Retrieves a binding based on its Name
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixbindings | MatrixBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablebindings | TableBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textbindings | TextBindings}
         *
         * Fails if the specified id does not exist.
         *
         * @param id Specifies the unique name of the binding object. Required.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the Binding object specified by the id in the call.
         */
        getByIdAsync(id: string, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<Binding>) => void): void;
        /**
         * Retrieves a binding based on its Name
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixbindings | MatrixBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablebindings | TableBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textbindings | TextBindings}
         *
         * Fails if the specified id does not exist.
         *
         * @param id Specifies the unique name of the binding object. Required.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the Binding object specified by the id in the call.
         */
        getByIdAsync(id: string, callback?: (result: AsyncResult<Binding>) => void): void;
        /**
         * Removes the binding from the document
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixbindings | MatrixBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablebindings | TableBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textbindings | TextBindings}
         *
         * Fails if the specified id does not exist.
         *
         * @param id Specifies the unique name to be used to identify the binding object. Required.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        releaseByIdAsync(id: string, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Removes the binding from the document
         *
         * @remarks
         * 
         * **Requirement sets**:
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixbindings | MatrixBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablebindings | TableBindings}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textbindings | TextBindings}
         *
         * Fails if the specified id does not exist.
         *
         * @param id Specifies the unique name to be used to identify the binding object. Required.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        releaseByIdAsync(id: string, callback?: (result: AsyncResult<void>) => void): void;
    }
    /**
     * Represents an XML node in a tree in a document.
     * 
     * @remarks
     * **Hosts**: Word
     */
    interface CustomXmlNode {
        /**
         * Gets the base name of the node without the namespace prefix, if one exists.
         */
        baseName: string;
        /**
         * Retrieves the string GUID of the CustomXMLPart.
         */
        namespaceUri: string;
        /**
         * Gets the type of the CustomXMLNode.
         */
        nodeType: string;
        /**
         * Gets the nodes associated with the XPath expression.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param xPath The XPath expression that specifies the nodes to get. Required.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is an array of CustomXmlNode objects that represent the nodes specified by the XPath expression passed to the `xPath` parameter.
         */
        getNodesAsync(xPath: string, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<CustomXmlNode[]>) => void): void;
        /**
         * Gets the nodes associated with the XPath expression.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param xPath The XPath expression that specifies the nodes to get. Required.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is an array of CustomXmlNode objects that represent the nodes specified by the XPath expression passed to the `xPath` parameter.
         */
        getNodesAsync(xPath: string, callback?: (result: AsyncResult<CustomXmlNode[]>) => void): void;
        /**
         * Gets the node value.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is a string that contains the value of the referenced node.
         */
        getNodeValueAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Gets the node value.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is a string that contains the value of the referenced node.
         */
        getNodeValueAsync(callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Gets the text of an XML node in a custom XML part.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is a string that contains the inner text of the referenced nodes.
         */
        getTextAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Gets the text of an XML node in a custom XML part.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is a string that contains the inner text of the referenced nodes.
         */
        getTextAsync(callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Gets the node's XML.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is a string that contains the XML of the referenced node.
         */
        getXmlAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Gets the node's XML.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is a string that contains the XML of the referenced node.
         */
        getXmlAsync(callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Sets the node value.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param value The value to be set on the node
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        setNodeValueAsync(value: string, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Sets the node value.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param value The value to be set on the node
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        setNodeValueAsync(value: string, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Asynchronously sets the text of an XML node in a custom XML part.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param text Required. The text value of the XML node.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        setTextAsync(text: string, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Asynchronously sets the text of an XML node in a custom XML part.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param text Required. The text value of the XML node.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        setTextAsync(text: string, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Sets the node XML.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param xml The XML to be set on the node
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        setXmlAsync(xml: string, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Sets the node XML.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param xml The XML to be set on the node
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        setXmlAsync(xml: string, callback?: (result: AsyncResult<void>) => void): void;
    }
    /**
     * Represents a single CustomXMLPart in an {@link Office.CustomXmlParts} collection.
     * 
     * @remarks
     * **Hosts**: Word
     */
    interface CustomXmlPart {
        /**
         * True, if the custom XML part is built in; otherwise false.
         */
        builtIn: boolean;
        /**
         * Gets the GUID of the CustomXMLPart.
         */
        id: string;
        /**
         * Gets the set of namespace prefix mappings ({@link Office.CustomXmlPrefixMappings}) used against the current CustomXmlPart.
         */
        namespaceManager: CustomXmlPrefixMappings;
        /**
         * Adds an event handler to the object using the specified event type.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * You can add multiple event handlers for the specified eventType as long as the name of each event handler function is unique.
         *
         * @param eventType Specifies the type of event to add. For a CustomXmlPart object, the eventType parameter can be specified as 
         *                  `Office.EventType.NodeDeleted`, `Office.EventType.NodeInserted`, and `Office.EventType.NodeReplaced`.
         * @param handler The event handler function to add, whose only parameter is of type {@link Office.NodeDeletedEventArgs}, 
         *                {@link Office.NodeInsertedEventArgs}, or {@link Office.NodeReplacedEventArgs}
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        addHandlerAsync(eventType: Office.EventType, handler: (result: any) => void, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Adds an event handler to the object using the specified event type.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * You can add multiple event handlers for the specified eventType as long as the name of each event handler function is unique.
         *
         * @param eventType Specifies the type of event to add. For a CustomXmlPart object, the eventType parameter can be specified as 
         *                  `Office.EventType.NodeDeleted`, `Office.EventType.NodeInserted`, and `Office.EventType.NodeReplaced`.
         * @param handler The event handler function to add, whose only parameter is of type {@link Office.NodeDeletedEventArgs}, 
         *                {@link Office.NodeInsertedEventArgs}, or {@link Office.NodeReplacedEventArgs}
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        addHandlerAsync(eventType: Office.EventType, handler: (result: any) => void, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Deletes the Custom XML Part.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        deleteAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Deletes the Custom XML Part.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        deleteAsync(callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Asynchronously gets any CustomXmlNodes in this custom XML part which match the specified XPath.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param xPath An XPath expression that specifies the nodes you want returned. Required.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is an array of CustomXmlNode objects that represent the nodes specified by the XPath expression passed to the xPath parameter.
         */
        getNodesAsync(xPath: string, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<CustomXmlNode[]>) => void): void;
        /**
         * Asynchronously gets any CustomXmlNodes in this custom XML part which match the specified XPath.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param xPath An XPath expression that specifies the nodes you want returned. Required.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is an array of CustomXmlNode objects that represent the nodes specified by the XPath expression passed to the xPath parameter.
         */
        getNodesAsync(xPath: string, callback?: (result: AsyncResult<CustomXmlNode[]>) => void): void;
        /**
         * Asynchronously gets the XML inside this custom XML part.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is a string that contains the XML of the referenced CustomXmlPart object.
         */
        getXmlAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Asynchronously gets the XML inside this custom XML part.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is a string that contains the XML of the referenced CustomXmlPart object.
         */
        getXmlAsync(callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Removes an event handler for the specified event type.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param eventType Specifies the type of event to remove. For a CustomXmlPart object, the eventType parameter can be specified as 
         *                  `Office.EventType.NodeDeleted`, `Office.EventType.NodeInserted`, and `Office.EventType.NodeReplaced`.
         * @param handler The name of the handler to remove.
         * @param options Provides options to determine which event handler or handlers are removed.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        removeHandlerAsync(eventType: Office.EventType, handler?: (result: any) => void, options?: RemoveHandlerOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Removes an event handler for the specified event type.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param eventType Specifies the type of event to remove. For a CustomXmlPart object, the eventType parameter can be specified as 
         *                  `Office.EventType.NodeDeleted`, `Office.EventType.NodeInserted`, and `Office.EventType.NodeReplaced`.
         * @param handler The name of the handler to remove.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        removeHandlerAsync(eventType: Office.EventType, handler?: (result: any) => void, callback?: (result: AsyncResult<void>) => void): void;
    }

    /**
     * Provides information about the deleted node that raised the nodeDeleted event.
     */
    interface NodeDeletedEventArgs {
        /**
         * Gets whether the node was deleted as part of an Undo/Redo action by the user.
         */
        isUndoRedo: boolean;
        /**
         * Gets the former next sibling of the node that was just deleted from the {@link Office.CustomXmlPart} object.
         */
        oldNextSibling: CustomXmlNode;
        /**
         * Gets the node which was just deleted from the {@link Office.CustomXmlPart} object.
         * 
         * Note that this node may have children, if a subtree is being removed from the document. Also, this node will be a "disconnected" node in 
         * that you can query down from the node, but you cannot query up the tree - the node appears to exist alone.
         */
        oldNode: CustomXmlNode;
    }

    /**
     * Provides information about the inserted node that raised the nodeInserted event.
     */
    interface NodeInsertedEventArgs  {
        /**
         * Gets whether the node was inserted as part of an Undo/Redo action by the user.
         */
        isUndoRedo: boolean;
        /**
         * Gets the node that was just added to the CustomXMLPart object.
         * 
         * Note that this node may have children, if a subtree was just added to the document.
         */
        newNode: CustomXmlNode;
    }
    
    /**
     * Provides information about the replaced node that raised the nodeReplaced event.
     */
    interface NodeReplacedEventArgs  {
        /**
         * Gets whether the replaced node was inserted as part of an undo or redo operation by the user.
         */
        isUndoRedo: boolean;
        /**
         * Gets the node that was just added to the CustomXMLPart object.
         * 
         * Note that this node may have children, if a subtree was just added to the document.
         */
        newNode: CustomXmlNode;
        /**
         * Gets the node which was just deleted (replaced) from the CustomXmlPart object.
         * 
         * Note that this node may have children, if a subtree is being removed from the document. Also, this node will be a "disconnected" node in 
         * that you can query down from the node, but you cannot query up the tree - the node appears to exist alone.
         */
        oldNode: CustomXmlNode;
    }

    /**
     * Represents a collection of CustomXmlPart objects.
     * 
     * @remarks
     * **Hosts**: Word
     */
    interface CustomXmlParts {
        /**
         * Asynchronously adds a new custom XML part to a file.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param xml The XML to add to the newly created custom XML part.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the newly created CustomXmlPart object.
         */
        addAsync(xml: string, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<CustomXmlPart>) => void): void;
        /**
         * Asynchronously adds a new custom XML part to a file.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param xml The XML to add to the newly created custom XML part.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the newly created CustomXmlPart object.
         */
        addAsync(xml: string, callback?: (result: AsyncResult<CustomXmlPart>) => void): void;
        /**
         * Asynchronously gets the specified custom XML part by its id.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param id The GUID of the custom XML part, including opening and closing braces.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is a CustomXmlPart object that represents the specified custom XML part.
         *                  If there is no custom XML part with the specified id, the method returns null.
         */
        getByIdAsync(id: string, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<CustomXmlPart>) => void): void;
        /**
         * Asynchronously gets the specified custom XML part by its id.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param id The GUID of the custom XML part, including opening and closing braces.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is a CustomXmlPart object that represents the specified custom XML part.
         *                  If there is no custom XML part with the specified id, the method returns null.
         */
        getByIdAsync(id: string, callback?: (result: AsyncResult<CustomXmlPart>) => void): void;
        /**
         * Asynchronously gets the specified custom XML part(s) by its namespace.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param ns  The namespace URI.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is an array of CustomXmlPart objects that match the specified namespace.
        */
        getByNamespaceAsync(ns: string, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<CustomXmlPart[]>) => void): void;
        /**
         * Asynchronously gets the specified custom XML part(s) by its namespace.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         *
         * @param ns  The namespace URI.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is an array of CustomXmlPart objects that match the specified namespace.
        */
        getByNamespaceAsync(ns: string, callback?: (result: AsyncResult<CustomXmlPart[]>) => void): void;
    }
    /**
     * Represents a collection of CustomXmlPart objects.
     * 
     * @remarks
     * **Hosts**: Word
     */
    interface CustomXmlPrefixMappings {
        /**
         * Asynchronously adds a prefix to namespace mapping to use when querying an item.
         *
         * @remarks
         * 
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         * 
         * If no namespace is assigned to the requested prefix, the method returns an empty string ("").
         *
         * @param prefix Specifies the prefix to add to the prefix mapping list. Required.
         * @param ns Specifies the namespace URI to assign to the newly added prefix. Required.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        addNamespaceAsync(prefix: string, ns: string, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Asynchronously adds a prefix to namespace mapping to use when querying an item.
         *
         * @remarks
         * 
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         * 
         * If no namespace is assigned to the requested prefix, the method returns an empty string ("").
         *
         * @param prefix Specifies the prefix to add to the prefix mapping list. Required.
         * @param ns Specifies the namespace URI to assign to the newly added prefix. Required.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        addNamespaceAsync(prefix: string, ns: string, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Asynchronously gets the namespace mapped to the specified prefix.
         *
         * @remarks
         * 
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         * 
         * If the prefix already exists in the namespace manager, this method will overwrite the mapping of that prefix except when the prefix is one 
         * added or used by the data store internally, in which case it will return an error.
         *
         * @param prefix TSpecifies the prefix to get the namespace for. Required.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is a string that contains the namespace mapped to the specified prefix.
         */
        getNamespaceAsync(prefix: string, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Asynchronously gets the namespace mapped to the specified prefix.
         *
         * @remarks
         * 
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         * 
         * If the prefix already exists in the namespace manager, this method will overwrite the mapping of that prefix except when the prefix is one 
         * added or used by the data store internally, in which case it will return an error.
         *
         * @param prefix TSpecifies the prefix to get the namespace for. Required.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is a string that contains the namespace mapped to the specified prefix.
         */
        getNamespaceAsync(prefix: string, callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Asynchronously gets the prefix for the specified namespace.
         *
         * @remarks
         * 
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         * 
         * If no prefix is assigned to the requested namespace, the method returns an empty string (""). If there are multiple prefixes specified in 
         * the namespace manager, the method returns the first prefix that matches the supplied namespace.
         *
         * @param ns Specifies the namespace to get the prefix for. Required.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is a string that contains the prefix of the specified namespace.
         */
        getPrefixAsync(ns: string, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Asynchronously gets the prefix for the specified namespace.
         *
         * @remarks
         * 
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#customxmlparts | CustomXmlParts}
         * 
         * If no prefix is assigned to the requested namespace, the method returns an empty string (""). If there are multiple prefixes specified in 
         * the namespace manager, the method returns the first prefix that matches the supplied namespace.
         *
         * @param ns Specifies the namespace to get the prefix for. Required.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is a string that contains the prefix of the specified namespace.
         */
        getPrefixAsync(ns: string, callback?: (result: AsyncResult<string>) => void): void;
    }
    /**
     * An abstract class that represents the document the add-in is interacting with.
     *
     * @remarks
     * **Hosts**: Access, Excel, PowerPoint, Project, Word
     */
    interface Document {
        /**
         * Gets an object that provides access to the bindings defined in the document.
         *
         * @remarks
         * You don't instantiate the Document object directly in your script. To call members of the Document object to interact with the current 
         * document or worksheet, use `Office.context.document` in your script.
         */
        bindings: Bindings;
        /**
         * Gets an object that represents the custom XML parts in the document.
         */
        customXmlParts: CustomXmlParts;
        /**
         * Gets the mode the document is in.
         */
        mode: DocumentMode;
        /**
         * Gets an object that represents the saved custom settings of the content or task pane add-in for the current document.
         */
        settings: Settings;
        /**
         * Gets the URL of the document that the host application currently has open. Returns null if the URL is unavailable.
         */
        url: string;
        /**
         * Adds an event handler for a Document object event.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#documentevents | DocumentEvents}
         *
         * You can add multiple event handlers for the specified eventType as long as the name of each event handler function is unique.
         *
         * @param eventType For a Document object event, the eventType parameter can be specified as `Office.EventType.Document.SelectionChanged` or 
         *                  `Office.EventType.Document.ActiveViewChanged`, or the corresponding text value of this enumeration.
         * @param handler The event handler function to add, whose only parameter is of type {@link Office.DocumentSelectionChangedEventArgs}. Required.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        addHandlerAsync(eventType: Office.EventType, handler: any, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Adds an event handler for a Document object event.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#documentevents | DocumentEvents}
         *
         * You can add multiple event handlers for the specified eventType as long as the name of each event handler function is unique.
         *
         * @param eventType For a Document object event, the eventType parameter can be specified as `Office.EventType.Document.SelectionChanged` or 
         *                  `Office.EventType.Document.ActiveViewChanged`, or the corresponding text value of this enumeration.
         * @param handler The event handler function to add, whose only parameter is of type {@link Office.DocumentSelectionChangedEventArgs}. Required.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        addHandlerAsync(eventType: Office.EventType, handler: any, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Returns the state of the current view of the presentation (edit or read).
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#activeview | ActiveView}
         *
         * Can trigger an event when the view changes.
         *
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the state of the presentation's current view. 
         *                  The value returned can be either "edit" or "read". "edit" corresponds to any of the views in which you can edit slides, 
         *                  such as Normal or Outline View. "read" corresponds to either Slide Show or Reading View.
         */
        getActiveViewAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<"edit" | "read">) => void): void;
        /**
         * Returns the state of the current view of the presentation (edit or read).
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#activeview | ActiveView}
         *
         * Can trigger an event when the view changes.
         *
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the state of the presentation's current view. 
         *                  The value returned can be either "edit" or "read". "edit" corresponds to any of the views in which you can edit slides, 
         *                  such as Normal or Outline View. "read" corresponds to either Slide Show or Reading View.
         */
        getActiveViewAsync(callback?: (result: AsyncResult<"edit" | "read">) => void): void;
        /**
         * Returns the entire document file in slices of up to 4194304 bytes (4 MB). For add-ins on iOS, file slice is supported up to 65536 (64 KB). 
         * Note that specifying file slice size of above permitted limit will result in an "Internal Error" failure.
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#compressedfile | CompressedFile} (when using `Office.FileType.Compressed`)
         *  
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#file | File}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textfile | TextFile} (when using `Office.FileType.Text`)
         *
         * For add-ins running in Office host applications other than Office on iOS, the getFileAsync method supports getting files in slices of up 
         * to 4194304 bytes (4 MB). For add-ins running in Office apps on iOS, the getFileAsync method supports getting files in slices of up to 
         * 65536 (64 KB).
         *
         * The fileType parameter can be specified by using the {@link Office.FileType} enumeration or text values. But the possible values vary with 
         * the host:
         *
         * Excel on the web and Windows desktop: `Office.FileType.Compressed`
         * 
         * Excel on Mac: `Office.FileType.Compressed`, `Office.FileType.Pdf`
         *
         * PowerPoint on the web, Windows desktop, Mac, and iPad: `Office.FileType.Compressed`, `Office.FileType.Pdf`
         *
         * Word on the web, Windows desktop, Mac, and iPad: `Office.FileType.Compressed`, `Office.FileType.Pdf`, `Office.FileType.Text`
         *
         * @param fileType The format in which the file will be returned
         * @param options Provides options for setting the size of slices that the document will be divided into.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the File object.
         */
        getFileAsync(fileType: FileType, options?: GetFileOptions, callback?: (result: AsyncResult<Office.File>) => void): void;
        /**
         * Returns the entire document file in slices of up to 4194304 bytes (4 MB). For add-ins on iOS, file slice is supported up to 65536 (64 KB). 
         * Note that specifying file slice size of above permitted limit will result in an "Internal Error" failure.
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#compressedfile | CompressedFile} (when using `Office.FileType.Compressed`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#file | File}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textfile | TextFile} (when using `Office.FileType.Text`)
         * 
         * For add-ins running in Office host applications other than Office on iOS, the getFileAsync method supports getting files in slices of up 
         * to 4194304 bytes (4 MB). For add-ins running in Office on iOS apps, the getFileAsync method supports getting files in slices of up to 
         * 65536 (64 KB).
         *
         * The fileType parameter can be specified by using the {@link Office.FileType} enumeration or text values. But the possible values vary with 
         * the host:
         *
         * Excel on the web and Windows desktop: `Office.FileType.Compressed`
         * 
         * Excel on Mac: `Office.FileType.Compressed`, `Office.FileType.Pdf`
         *
         * PowerPoint on the web, Windows desktop, Mac, and iPad: `Office.FileType.Compressed`, `Office.FileType.Pdf`
         *
         * Word on the web, Windows desktop, Mac, and iPad: `Office.FileType.Compressed`, `Office.FileType.Pdf`, `Office.FileType.Text`
         *
         * @param fileType The format in which the file will be returned
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the File object.
         */
        getFileAsync(fileType: FileType, callback?: (result: AsyncResult<Office.File>) => void): void;
        /**
         * Gets file properties of the current document.
         *
         * @remarks
         * **Requirement sets**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#methods-that-arent-part-of-a-requirement-set | Not in a set}
         *
         * You get the file's URL with the url property `asyncResult.value.url`.
         *
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the file's properties (with the URL found at `asyncResult.value.url`).
         */
        getFilePropertiesAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<Office.FileProperties>) => void): void;
        /**
         * Gets file properties of the current document.
         *
         * @remarks
         * **Requirement sets**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#methods-that-arent-part-of-a-requirement-set | Not in a set}
         *
         * You get the file's URL with the url property `asyncResult.value.url`.
         *
         * @param callback A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the file's properties (with the URL found at `asyncResult.value.url`).
         */
        getFilePropertiesAsync(callback?: (result: AsyncResult<Office.FileProperties>) => void): void;
        /**
         * Reads the data contained in the current selection in the document.
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#htmlcoercion | HtmlCoercion} (when using `Office.CoercionType.Html`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixcoercion | MatrixCoercion} (when using `Office.CoercionType.Matrix`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#ooxmlcoercion | OoxmlCoercion} (when using `Office.CoercionType.Ooxml`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#selection | Selection}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablecoercion | TableCoercion} (when using `Office.CoercionType.Table`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textcoercion | TextCoercion} (when using `Office.CoercionType.Text`)
         * 
         * In the callback function that is passed to the getSelectedDataAsync method, you can use the properties of the AsyncResult object to return 
         * the following information.
         * 
         * <table>
         *   <tr>
         *     <th>Property</th>
         *     <th>Use to...</th>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.value</td>
         *     <td>Always returns undefined because there is no object or data to retrieve.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.status</td>
         *     <td>Determine the success or failure of the operation.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.error</td>
         *     <td>Access an Error object that provides error information if the operation failed.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.asyncContext</td>
         *     <td>A user-defined item of any type that is returned in the AsyncResult object without being altered.</td>
         *   </tr>
         * </table>
         * 
         * The possible values for the {@link Office.CoercionType} parameter vary by the host. 
         * 
         * <table>
         *   <tr>
         *     <th>Host</th>
         *     <th>Supported coercionType</th>
         *   </tr>
         *   <tr>
         *     <td>Excel, PowerPoint, Project, and Word</td>
         *     <td>`Office.CoercionType.Text` (string)</td>
         *   </tr>
         *   <tr>
         *     <td>Excel and Word</td>
         *     <td>`Office.CoercionType.Matrix` (array of arrays)</td>
         *   </tr>
         *   <tr>
         *     <td>Access, Excel, and Word</td>
         *     <td>`Office.CoercionType.Table` (TableData object)</td>
         *   </tr>
         *   <tr>
         *     <td>Word</td>
         *     <td>`Office.CoercionType.Html`</td>
         *   </tr>
         *   <tr>
         *     <td>Word</td>
         *     <td>`Office.CoercionType.Ooxml` (Office Open XML)</td>
         *   </tr>
         *   <tr>
         *     <td>PowerPoint on the web and Windows</td>
         *     <td>`Office.CoercionType.SlideRange`</td>
         *   </tr>
         *   <tr>
         *     <td>Excel, PowerPoint, and Word</td>
         *     <td>`Office.CoercionType.XmlSvg`</td>
         *   </tr>
         * </table>
         * 
         * @param coercionType The type of data structure to return. See the remarks section for each host's supported coercion types.
         * 
         * @param options Provides options for customizing what data is returned and how it is formatted.
         * 
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the data in the current selection. 
         *                  This is returned in the data structure or format you specified with the coercionType parameter. 
         *                  (See Remarks for more information about data coercion.)
         */
        getSelectedDataAsync<T>(coercionType: Office.CoercionType, options?: GetSelectedDataOptions, callback?: (result: AsyncResult<T>) => void): void;
        /**
         * Reads the data contained in the current selection in the document.
         *
         * @remarks
         * 
         * **Requirement sets**:
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#htmlcoercion | HtmlCoercion} (when using `Office.CoercionType.Html`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixcoercion | MatrixCoercion} (when using `Office.CoercionType.Matrix`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#ooxmlcoercion | OoxmlCoercion} (when using `Office.CoercionType.Ooxml`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#selection | Selection}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablecoercion | TableCoercion} (when using `Office.CoercionType.Table`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textcoercion | TextCoercion} (when using `Office.CoercionType.Text`)
         * 
         * In the callback function that is passed to the getSelectedDataAsync method, you can use the properties of the AsyncResult object to return 
         * the following information.
         * 
         * <table>
         *   <tr>
         *     <th>Property</th>
         *     <th>Use to...</th>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.value</td>
         *     <td>Always returns undefined because there is no object or data to retrieve.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.status</td>
         *     <td>Determine the success or failure of the operation.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.error</td>
         *     <td>Access an Error object that provides error information if the operation failed.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.asyncContext</td>
         *     <td>A user-defined item of any type that is returned in the AsyncResult object without being altered.</td>
         *   </tr>
         * </table>
         * 
         * The possible values for the {@link Office.CoercionType} parameter vary by the host. 
         * 
         * <table>
         *   <tr>
         *     <th>Host</th>
         *     <th>Supported coercionType</th>
         *   </tr>
         *   <tr>
         *     <td>Excel, PowerPoint, Project, and Word</td>
         *     <td>`Office.CoercionType.Text` (string)</td>
         *   </tr>
         *   <tr>
         *     <td>Excel and Word</td>
         *     <td>`Office.CoercionType.Matrix` (array of arrays)</td>
         *   </tr>
         *   <tr>
         *     <td>Access, Excel, and Word</td>
         *     <td>`Office.CoercionType.Table` (TableData object)</td>
         *   </tr>
         *   <tr>
         *     <td>Word</td>
         *     <td>`Office.CoercionType.Html`</td>
         *   </tr>
         *   <tr>
         *     <td>Word</td>
         *     <td>`Office.CoercionType.Ooxml` (Office Open XML)</td>
         *   </tr>
         *   <tr>
         *     <td>PowerPoint on the web and Windows</td>
         *     <td>`Office.CoercionType.SlideRange`</td>
         *   </tr>
         *   <tr>
         *     <td>Excel, PowerPoint, and Word</td>
         *     <td>`Office.CoercionType.XmlSvg`</td>
         *   </tr>
         * </table>
         * 
         * @param coercionType The type of data structure to return. See the remarks section for each host's supported coercion types.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the data in the current selection. 
         *                  This is returned in the data structure or format you specified with the coercionType parameter. 
         *                  (See Remarks for more information about data coercion.)
         */
        getSelectedDataAsync<T>(coercionType: Office.CoercionType, callback?: (result: AsyncResult<T>) => void): void;
        /**
         * Goes to the specified object or location in the document.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#methods-that-arent-part-of-a-requirement-set | Not in a set}
         *
         * PowerPoint doesn't support the goToByIdAsync method in Master Views.
         *
         * The behavior caused by the selectionMode option varies by host:
         *
         * In Excel: `Office.SelectionMode.Selected` selects all content in the binding, or named item. Office.SelectionMode.None for text bindings, 
         * selects the cell; for matrix bindings, table bindings, and named items, selects the first data cell (not first cell in header row for tables).
         *
         * In PowerPoint: `Office.SelectionMode.Selected` selects the slide title or first textbox on the slide. 
         * `Office.SelectionMode.None` doesn't select anything.
         *
         * In Word: `Office.SelectionMode.Selected` selects all content in the binding. Office.SelectionMode.None for text bindings, moves the cursor 
         * to the beginning of the text; for matrix bindings and table bindings, selects the first data cell (not first cell in header row for tables).
         *
         * @param id The identifier of the object or location to go to.
         * @param goToType The type of the location to go to.
         * @param options Provides options for whether to select the location that is navigated to.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the current view.
         */
        goToByIdAsync(id: string | number, goToType: GoToType, options?: GoToByIdOptions, callback?: (result: AsyncResult<any>) => void): void;
        /**
         * Goes to the specified object or location in the document.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#methods-that-arent-part-of-a-requirement-set | Not in a set}
         *
         * PowerPoint doesn't support the goToByIdAsync method in Master Views.
         *
         * The behavior caused by the selectionMode option varies by host:
         *
         * In Excel: `Office.SelectionMode.Selected` selects all content in the binding, or named item. Office.SelectionMode.None for text bindings, 
         * selects the cell; for matrix bindings, table bindings, and named items, selects the first data cell (not first cell in header row for tables).
         *
         * In PowerPoint: `Office.SelectionMode.Selected` selects the slide title or first textbox on the slide. 
         * `Office.SelectionMode.None` doesn't select anything.
         *
         * In Word: `Office.SelectionMode.Selected` selects all content in the binding. Office.SelectionMode.None for text bindings, moves the cursor 
         * to the beginning of the text; for matrix bindings and table bindings, selects the first data cell (not first cell in header row for tables).
         *
         * @param id The identifier of the object or location to go to.
         * @param goToType The type of the location to go to.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the current view.
         */
        goToByIdAsync(id: string | number, goToType: GoToType, callback?: (result: AsyncResult<any>) => void): void;
        /**
         * Removes an event handler for the specified event type.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#documentevents | DocumentEvents}
         *
         * @param eventType The event type. For document can be 'Document.SelectionChanged' or 'Document.ActiveViewChanged'.
         * @param options Provides options to determine which event handler or handlers are removed.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        removeHandlerAsync(eventType: Office.EventType, options?: RemoveHandlerOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Removes an event handler for the specified event type.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#documentevents | DocumentEvents}
         *
         * @param eventType The event type. For document can be 'Document.SelectionChanged' or 'Document.ActiveViewChanged'.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        removeHandlerAsync(eventType: Office.EventType, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Writes the specified data into the current selection.
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#htmlcoercion | HtmlCoercion}, (when using `Office.CoercionType.Html`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#imagecoercion | ImageCoercion} (when using `Office.CoercionType.Image`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixcoercion | MatrixCoercion} (when using `Office.CoercionType.Matrix`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#ooxmlcoercion | OoxmlCoercion} (when using `Office.CoercionType.Ooxml`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#selection | Selection}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablecoercion | TableCoercion} (when using `Office.CoercionType.Table`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textcoercion | TextCoercion} (when using `Office.CoercionType.Text`)
         * 
         * **Application-specific behaviors**
         * 
         * The following application-specific actions apply when writing data to a selection.
         * 
         * <table>
         *   <tr>
         *     <td>Word</td>
         *     <td>If there is no selection and the insertion point is at a valid location, the specified `data` is inserted at the insertion point</td>
         *     <td>If `data` is a string, the specified text is inserted.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>If `data` is an array of arrays ("matrix") or a TableData object, a new Word table is inserted.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>If `data` is HTML, the specified HTML is inserted. (**Important**: If any of the HTML you insert is invalid, Word won't raise an error. Word will insert as much of the HTML as it can and omits any invalid data).</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>If `data` is Office Open XML, the specified XML is inserted.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>If `data` is a base64 encoded image stream, the specified image is inserted.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td>If there is a selection</td>
         *     <td>It will be replaced with the specified `data` following the same rules as above.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td>Insert images</td>
         *     <td>Inserted images are placed inline. The imageLeft and imageTop parameters are ignored. The image aspect ratio is always locked. If only one of the imageWidth and imageHeight parameter is given, the other value will be automatically scaled to keep the original aspect ratio.</td>
         * </tr>
         * 
         *   <tr>
         *     <td>Excel</td>
         *     <td>If a single cell is selected</td>
         *     <td>If `data` is a string, the specified text is inserted as the value of the current cell.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>If `data` is an array of arrays ("matrix"), the specified set of rows and columns are inserted, if no other data in surrounding cells will be overwritten.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>If `data` is a TableData object, a new Excel table with the specified set of rows and headers is inserted, if no other data in surrounding cells will be overwritten.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td>If multiple cells are selected</td>
         *     <td>If the shape does not match the shape of `data`, an error is returned.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>If the shape of the selection exactly matches the shape of `data`, the values of the selected cells are updated based on the values in `data`.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td>Insert images</td>
         *     <td>Inserted images are floating. The position imageLeft and imageTop parameters are relative to currently selected cell(s). Negative imageLeft and imageTop values are allowed and possibly readjusted by Excel to position the image inside a worksheet. Image aspect ratio is locked unless both imageWidth and imageHeight parameters are provided. If only one of the imageWidth and imageHeight parameter is given, the other value will be automatically scaled to keep the original aspect ratio.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td>All other cases</td>
         *     <td>An error is returned.</td>
         *   </tr>
         * 
         *   <tr>
         *     <td>Excel on the web</td>
         *     <td>In addition to the behaviors described for Excel above, these limits apply when writing data in Excel on the web</td>
         *     <td>The total number of cells you can write to a worksheet with the `data` parameter can't exceed 20,000 in a single call to this method.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>The number of formatting groups passed to the `cellFormat` parameter can't exceed 100. A single formatting group consists of a set of formatting applied to a specified range of cells.</td>
         *   </tr>
         * 
         *   <tr>
         *     <td>PowerPoint</td>
         *     <td>Insert image</td>
         *     <td>Inserted images are floating. The position imageLeft and imageTop parameters are optional but if provided, both should be present. If a single value is provided, it will be ignored. Negative imageLeft and imageTop values are allowed and can position an image outside of a slide. If no optional parameter is given and slide has a placeholder, the image will replace the placeholder in the slide. Image aspect ratio will be locked unless both imageWidth and imageHeight parameters are provided. If only one of the imageWidth and imageHeight parameter is given, the other value will be automatically scaled to keep the original aspect ratio.</td>
         *   </tr>
         * </table>
         * 
         * The possible values for the {@link Office.CoercionType} parameter vary by the host. 
         * 
         * <table>
         *   <tr>
         *     <th>Host</th>
         *     <th>Supported coercionType</th>
         *   </tr>
         *   <tr>
         *     <td>Excel, PowerPoint, Project, and Word</td>
         *     <td>`Office.CoercionType.Text` (string)</td>
         *   </tr>
         *   <tr>
         *     <td>Excel and Word</td>
         *     <td>`Office.CoercionType.Matrix` (array of arrays)</td>
         *   </tr>
         *   <tr>
         *     <td>Access, Excel, and Word</td>
         *     <td>`Office.CoercionType.Table` (TableData object)</td>
         *   </tr>
         *   <tr>
         *     <td>Word</td>
         *     <td>`Office.CoercionType.Html`</td>
         *   </tr>
         *   <tr>
         *     <td>Word</td>
         *     <td>`Office.CoercionType.Ooxml` (Office Open XML)</td>
         *   </tr>
         *   <tr>
         *     <td>PowerPoint on the web and Windows</td>
         *     <td>`Office.CoercionType.SlideRange`</td>
         *   </tr>
         *   <tr>
         *     <td>Excel, PowerPoint, and Word</td>
         *     <td>`Office.CoercionType.XmlSvg`</td>
         *   </tr>
         * </table>
         * 
         * 
         * @param data The data to be set. Either a string or  {@link Office.CoercionType} value, 2d array or TableData object.
         * 
         * If the value passed for `data` is:
         * 
         * - A string: Plain text or anything that can be coerced to a string will be inserted. 
         * In Excel, you can also specify data as a valid formula to add that formula to the selected cell. For example, setting data to "=SUM(A1:A5)" 
         * will total the values in the specified range. However, when you set a formula on the bound cell, after doing so, you can't read the added 
         * formula (or any pre-existing formula) from the bound cell. If you call the Document.getSelectedDataAsync method on the selected cell to 
         * read its data, the method can return only the data displayed in the cell (the formula's result).
         * 
         * - An array of arrays ("matrix"): Tabular data without headers will be inserted. For example, to write data to three rows in two columns, 
         * you can pass an array like this: [["R1C1", "R1C2"], ["R2C1", "R2C2"], ["R3C1", "R3C2"]]. To write a single column of three rows, pass an 
         * array like this: [["R1C1"], ["R2C1"], ["R3C1"]]
         * 
         * In Excel, you can also specify data as an array of arrays that contains valid formulas to add them to the selected cells. For example if no 
         * other data will be overwritten, setting data to [["=SUM(A1:A5)","=AVERAGE(A1:A5)"]] will add those two formulas to the selection. Just as 
         * when setting a formula on a single cell as "text", you can't read the added formulas (or any pre-existing formulas) after they have been 
         * set - you can only read the formulas' results.
         * 
         * - A TableData object: A table with headers will be inserted.
         * In Excel, if you specify formulas in the TableData object you pass for the data parameter, you might not get the results you expect due to 
         * the "calculated columns" feature of Excel, which automatically duplicates formulas within a column. To work around this when you want to 
         * write `data` that contains formulas to a selected table, try specifying the data as an array of arrays (instead of a TableData object), and 
         * specify the coercionType as Microsoft.Office.Matrix or "matrix". However, this technique will block the "calculated columns" feature only 
         * when one of the following conditions is met: (1) you are writing to all the cells of the column, or (2) there are already at least two 
         * different formulas in the column.
         * 
         * @param options Provides options for how to insert data to the selection.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The AsyncResult.value property always returns undefined because there is no object or data to retrieve.
         */
        setSelectedDataAsync(data: string | TableData | any[][], options?: SetSelectedDataOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Writes the specified data into the current selection.
         *
         * @remarks
         * 
         * **Requirement sets**: 
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#htmlcoercion | HtmlCoercion}, (when using `Office.CoercionType.Html`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#imagecoercion | ImageCoercion} (when using `Office.CoercionType.Image`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#matrixcoercion | MatrixCoercion} (when using `Office.CoercionType.Matrix`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#ooxmlcoercion | OoxmlCoercion} (when using `Office.CoercionType.Ooxml`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#selection | Selection}
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#tablecoercion | TableCoercion} (when using `Office.CoercionType.Table`)
         * 
         * - {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#textcoercion | TextCoercion} (when using `Office.CoercionType.Text`)
         * 
         * **Application-specific behaviors**
         * 
         * The following application-specific actions apply when writing data to a selection.
         * 
         * <table>
         *   <tr>
         *     <td>Word</td>
         *     <td>If there is no selection and the insertion point is at a valid location, the specified `data` is inserted at the insertion point</td>
         *     <td>If `data` is a string, the specified text is inserted.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>If `data` is an array of arrays ("matrix") or a TableData object, a new Word table is inserted.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>If `data` is HTML, the specified HTML is inserted. (**Important**: If any of the HTML you insert is invalid, Word won't raise an error. Word will insert as much of the HTML as it can and omits any invalid data).</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>If `data` is Office Open XML, the specified XML is inserted.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>If `data` is a base64 encoded image stream, the specified image is inserted.</td></tr></td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td>If there is a selection</td>
         *     <td>It will be replaced with the specified `data` following the same rules as above.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td>Insert images</td>
         *     <td>Inserted images are placed inline. The imageLeft and imageTop parameters are ignored. The image aspect ratio is always locked. If only one of the imageWidth and imageHeight parameter is given, the other value will be automatically scaled to keep the original aspect ratio.</td>
         *   </tr>
         * 
         *   <tr>
         *     <td>Excel</td>
         *     <td>If a single cell is selected</td>
         *     <td>If `data` is a string, the specified text is inserted as the value of the current cell.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>If `data` is an array of arrays ("matrix"), the specified set of rows and columns are inserted, if no other data in surrounding cells will be overwritten.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>If `data` is a TableData object, a new Excel table with the specified set of rows and headers is inserted, if no other data in surrounding cells will be overwritten.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td>If multiple cells are selected</td><td>If the shape does not match the shape of `data`, an error is returned.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>If the shape of the selection exactly matches the shape of `data`, the values of the selected cells are updated based on the values in `data`.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td>Insert images</td>
         *     <td>Inserted images are floating. The position imageLeft and imageTop parameters are relative to currently selected cell(s). Negative imageLeft and imageTop values are allowed and possibly readjusted by Excel to position the image inside a worksheet. Image aspect ratio is locked unless both imageWidth and imageHeight parameters are provided. If only one of the imageWidth and imageHeight parameter is given, the other value will be automatically scaled to keep the original aspect ratio.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td>All other cases</td>
         *     <td>An error is returned.</td>
         *   </tr>
         * 
         *   <tr>
         *     <td>Excel on the web</td>
         *     <td>In addition to the behaviors described for Excel above, these limits apply when writing data in Excel on the web</td>
         *     <td>The total number of cells you can write to a worksheet with the `data` parameter can't exceed 20,000 in a single call to this method.</td>
         *   </tr>
         *   <tr>
         *     <td></td>
         *     <td></td>
         *     <td>The number of formatting groups passed to the `cellFormat` parameter can't exceed 100. A single formatting group consists of a set of formatting applied to a specified range of cells.</td>
         *   </tr>
         * 
         *   <tr>
         *     <td>PowerPoint</td>
         *     <td>Insert image</td>
         *     <td>Inserted images are floating. The position imageLeft and imageTop parameters are optional but if provided, both should be present. If a single value is provided, it will be ignored. Negative imageLeft and imageTop values are allowed and can position an image outside of a slide. If no optional parameter is given and slide has a placeholder, the image will replace the placeholder in the slide. Image aspect ratio will be locked unless both imageWidth and imageHeight parameters are provided. If only one of the imageWidth and imageHeight parameter is given, the other value will be automatically scaled to keep the original aspect ratio.</td>
         *   </tr>
         * </table>
         * 
         * The possible values for the {@link Office.CoercionType} parameter vary by the host. 
         * 
         * <table>
         *   <tr>
         *     <th>Host</th>
         *     <th>Supported coercionType</th>
         *   </tr>
         *   <tr>
         *     <td>Excel, PowerPoint, Project, and Word</td>
         *     <td>`Office.CoercionType.Text` (string)</td>
         *   </tr>
         *   <tr>
         *     <td>Excel and Word</td>
         *     <td>`Office.CoercionType.Matrix` (array of arrays)</td>
         *   </tr>
         *   <tr>
         *     <td>Access, Excel, and Word</td>
         *     <td>`Office.CoercionType.Table` (TableData object)</td>
         *   </tr>
         *   <tr>
         *     <td>Word</td>
         *     <td>`Office.CoercionType.Html`</td>
         *   </tr>
         *   <tr>
         *     <td>Word</td>
         *     <td>`Office.CoercionType.Ooxml` (Office Open XML)</td>
         *   </tr>
         *   <tr>
         *     <td>PowerPoint on the web and Windows</td>
         *     <td>`Office.CoercionType.SlideRange`</td>
         *   </tr>
         *   <tr>
         *     <td>Excel, PowerPoint, and Word</td>
         *     <td>`Office.CoercionType.XmlSvg`</td>
         *   </tr>
         * </table>
         * 
         * 
         * @param data The data to be set. Either a string or  {@link Office.CoercionType} value, 2d array or TableData object.
         * 
         * If the value passed for `data` is:
         * 
         * - A string: Plain text or anything that can be coerced to a string will be inserted. 
         * In Excel, you can also specify data as a valid formula to add that formula to the selected cell. For example, setting data to "=SUM(A1:A5)" 
         * will total the values in the specified range. However, when you set a formula on the bound cell, after doing so, you can't read the added 
         * formula (or any pre-existing formula) from the bound cell. If you call the Document.getSelectedDataAsync method on the selected cell to 
         * read its data, the method can return only the data displayed in the cell (the formula's result).
         * 
         * - An array of arrays ("matrix"): Tabular data without headers will be inserted. For example, to write data to three rows in two columns, 
         * you can pass an array like this: [["R1C1", "R1C2"], ["R2C1", "R2C2"], ["R3C1", "R3C2"]]. To write a single column of three rows, pass an 
         * array like this: [["R1C1"], ["R2C1"], ["R3C1"]]
         * 
         * In Excel, you can also specify data as an array of arrays that contains valid formulas to add them to the selected cells. For example if no 
         * other data will be overwritten, setting data to [["=SUM(A1:A5)","=AVERAGE(A1:A5)"]] will add those two formulas to the selection. Just as 
         * when setting a formula on a single cell as "text", you can't read the added formulas (or any pre-existing formulas) after they have been 
         * set - you can only read the formulas' results.
         * 
         * - A TableData object: A table with headers will be inserted.
         * In Excel, if you specify formulas in the TableData object you pass for the data parameter, you might not get the results you expect due to 
         * the "calculated columns" feature of Excel, which automatically duplicates formulas within a column. To work around this when you want to 
         * write `data` that contains formulas to a selected table, try specifying the data as an array of arrays (instead of a TableData object), and 
         * specify the coercionType as Microsoft.Office.Matrix or "matrix". However, this technique will block the "calculated columns" feature only 
         * when one of the following conditions is met: (1) you are writing to all the cells of the column, or (2) there are already at least two 
         * different formulas in the column.
         * 
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The AsyncResult.value property always returns undefined because there is no object or data to retrieve.
         */
        setSelectedDataAsync(data: string | TableData | any[][], callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Project documents only. Get Project field (Ex. ProjectWebAccessURL).
         * @param fieldId Project level fields.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result contains the `fieldValue` property, which represents the value of the specified field.
         */
        getProjectFieldAsync(fieldId: number, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<any>) => void): void;
        /**
         * Project documents only. Get Project field (Ex. ProjectWebAccessURL).
         * @param fieldId Project level fields.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result contains the `fieldValue` property, which represents the value of the specified field.
         * 
         */
        getProjectFieldAsync(fieldId: number, callback?: (result: AsyncResult<any>) => void): void;
        /**
         * Project documents only. Get resource field for provided resource Id. (Ex.ResourceName)
         * @param resourceId Either a string or value of the Resource Id.
         * @param fieldId Resource Fields.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the GUID of the resource as a string.
         * 
         */
        getResourceFieldAsync(resourceId: string, fieldId: number, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Project documents only. Get resource field for provided resource Id. (Ex.ResourceName)
         * @param resourceId Either a string or value of the Resource Id.
         * @param fieldId Resource Fields.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the GUID of the resource as a string.
         * 
         */
        getResourceFieldAsync(resourceId: string, fieldId: number, callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Project documents only. Get the current selected Resource's Id.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the GUID of the resource as a string.
         * 
         */
        getSelectedResourceAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Project documents only. Get the current selected Resource's Id.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the GUID of the resource as a string.
         * 
         */
        getSelectedResourceAsync(callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Project documents only. Get the current selected Task's Id.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the GUID of the resource as a string.
         * 
         */
        getSelectedTaskAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Project documents only. Get the current selected Task's Id.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the GUID of the resource as a string.
         * 
         */
        getSelectedTaskAsync(callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Project documents only. Get the current selected View Type (Ex. Gantt) and View Name.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result contains the following properties:
         *                  `viewName` - The name of the view, as a ProjectViewTypes constant.
         *                  `viewType` - The type of view, as the integer value of a ProjectViewTypes constant.
         * 
         */
        getSelectedViewAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<any>) => void): void;
        /**
         * Project documents only. Get the current selected View Type (Ex. Gantt) and View Name.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result contains the following properties:
         *                  `viewName` - The name of the view, as a ProjectViewTypes constant.
         *                  `viewType` - The type of view, as the integer value of a ProjectViewTypes constant.
         * 
         */
        getSelectedViewAsync(callback?: (result: AsyncResult<any>) => void): void;
        /**
         * Project documents only. Get the Task Name, WSS Task Id, and ResourceNames for given taskId.
         * @param taskId Either a string or value of the Task Id.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result contains the following properties:
         *                  `taskName` - The name of the task.
         *                  `wssTaskId` - The ID of the task in the synchronized SharePoint task list. If the project is not synchronized with a SharePoint task list, the value is 0.
         *                  `resourceNames` - The comma-separated list of the names of resources that are assigned to the task.
         * 
         */
        getTaskAsync(taskId: string, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<any>) => void): void;
        /**
         * Project documents only. Get the Task Name, WSS Task Id, and ResourceNames for given taskId.
         * @param taskId Either a string or value of the Task Id.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result contains the following properties:
         *                  `taskName` - The name of the task.
         *                  `wssTaskId` - The ID of the task in the synchronized SharePoint task list. If the project is not synchronized with a SharePoint task list, the value is 0.
         *                  `resourceNames` - The comma-separated list of the names of resources that are assigned to the task.
         * 
         */
        getTaskAsync(taskId: string, callback?: (result: AsyncResult<any>) => void): void;
        /**
         * Project documents only. Get task field for provided task Id. (Ex. StartDate).
         * @param taskId Either a string or value of the Task Id.
         * @param fieldId Task Fields.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result contains the `fieldValue` property, which represents the value of the specified field.
         * 
         */
        getTaskFieldAsync(taskId: string, fieldId: number, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<any>) => void): void;
        /**
         * Project documents only. Get task field for provided task Id. (Ex. StartDate).
         * @param taskId Either a string or value of the Task Id.
         * @param fieldId Task Fields.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result contains the `fieldValue` property, which represents the value of the specified field.
         * 
         */
        getTaskFieldAsync(taskId: string, fieldId: number, callback?: (result: AsyncResult<any>) => void): void;
        /**
         * Project documents only. Get the WSS Url and list name for the Tasks List, the MPP is synced too.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result contains the following properties:
         *                  `listName` - the name of the synchronized SharePoint task list.
         *                  `serverUrl` - the URL of the synchronized SharePoint task list.
         * 
         */
        getWSSUrlAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<any>) => void): void;
        /**
         * Project documents only. Get the WSS Url and list name for the Tasks List, the MPP is synced too.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result contains the following properties:
         *                  `listName` - the name of the synchronized SharePoint task list.
         *                  `serverUrl` - the URL of the synchronized SharePoint task list.
         * 
         */
        getWSSUrlAsync(callback?: (result: AsyncResult<any>) => void): void;
        /**
         * Project documents only. Get the maximum index of the collection of resources in the current project.
         * 
         * **Important**: This API works only in Project 2016 on Windows desktop.
         * 
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the highest index number in the current project's resource collection.
         * 
         */
        getMaxResourceIndexAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<number>) => void): void;
        /**
         * Project documents only. Get the maximum index of the collection of resources in the current project.
         * 
         * **Important**: This API works only in Project 2016 on Windows desktop.
         * 
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the highest index number in the current project's resource collection.
         * 
         */
        getMaxResourceIndexAsync(callback?: (result: AsyncResult<number>) => void): void;
        /**
         * Project documents only. Get the maximum index of the collection of tasks in the current project.
         * 
         * **Important**: This API works only in Project 2016 on Windows desktop.
         * 
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the highest index number in the current project's task collection.
         * 
         */
        getMaxTaskIndexAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<number>) => void): void;
        /**
         * Project documents only. Get the maximum index of the collection of tasks in the current project.
         * 
         * **Important**: This API works only in Project 2016 on Windows desktop.
         * 
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the highest index number in the current project's task collection.
         * 
         */
        getMaxTaskIndexAsync(callback?: (result: AsyncResult<number>) => void): void;
        /**
         * Project documents only. Get the GUID of the resource that has the specified index in the resource collection.
         * 
         * **Important**: This API works only in Project 2016 on Windows desktop.
         * 
         * @param resourceIndex The index of the resource in the collection of resources for the project.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the GUID of the resource as a string.
         * 
         */
        getResourceByIndexAsync(resourceIndex: number, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Project documents only. Get the GUID of the resource that has the specified index in the resource collection.
         * 
         * **Important**: This API works only in Project 2016 on Windows desktop.
         * 
         * @param resourceIndex The index of the resource in the collection of resources for the project.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the GUID of the resource as a string.
         * 
         */
        getResourceByIndexAsync(resourceIndex: number, callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Project documents only. Get the GUID of the task that has the specified index in the task collection.
         * 
         * **Important**: This API works only in Project 2016 on Windows desktop.
         * 
         * @param taskIndex The index of the task in the collection of tasks for the project.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the GUID of the task as a string.
         * 
         */
        getTaskByIndexAsync(taskIndex: number, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Project documents only. Get the GUID of the task that has the specified index in the task collection.
         * 
         * **Important**: This API works only in Project 2016 on Windows desktop.
         * 
         * @param taskIndex The index of the task in the collection of tasks for the project.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the GUID of the task as a string.
         * 
         */
        getTaskByIndexAsync(taskIndex: number, callback?: (result: AsyncResult<string>) => void): void;
        /**
         * Project documents only. Set resource field for specified resource Id.
         * 
         * **Important**: This API works only in Project 2016 on Windows desktop.
         * 
         * @param resourceId Either a string or value of the Resource Id.
         * @param fieldId Resource Fields.
         * @param fieldValue Value of the target field.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         * 
         */
        setResourceFieldAsync(resourceId: string, fieldId: number, fieldValue: string | number | boolean | object, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Project documents only. Set resource field for specified resource Id.
         * 
         * **Important**: This API works only in Project 2016 on Windows desktop.
         * 
         * @param resourceId Either a string or value of the Resource Id.
         * @param fieldId Resource Fields.
         * @param fieldValue Value of the target field.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         * 
         */
        setResourceFieldAsync(resourceId: string, fieldId: number, fieldValue: string | number | boolean | object, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Project documents only. Set task field for specified task Id.
         * 
         * **Important**: This API works only in Project 2016 on Windows desktop.
         * 
         * @param taskId Either a string or value of the Task Id.
         * @param fieldId Task Fields.
         * @param fieldValue Value of the target field.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         * 
         */
        setTaskFieldAsync(taskId: string, fieldId: number, fieldValue: string | number | boolean | object, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Project documents only. Set task field for specified task Id.
         * 
         * **Important**: This API works only in Project 2016 on Windows desktop.
         * 
         * @param taskId Either a string or value of the Task Id.
         * @param fieldId Task Fields.
         * @param fieldValue Value of the target field.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         * 
         */
        setTaskFieldAsync(taskId: string, fieldId: number, fieldValue: string | number | boolean | object, callback?: (result: AsyncResult<void>) => void): void;
    }
    /**
     * Provides information about the document that raised the SelectionChanged event.
     */
    interface DocumentSelectionChangedEventArgs {
        /**
         * Gets an {@link Office.Document} object that represents the document that raised the SelectionChanged event.
         */
        document: Document;
        /**
         * Get an {@link Office.EventType} enumeration value that identifies the kind of event that was raised.
         */
        type: EventType;
    }
    /**
     * Represents the document file associated with an Office Add-in.
     *
     * @remarks
     * Access the File object with the AsyncResult.value property in the callback function passed to the Document.getFileAsync method.
     * 
     */
    interface File {
        /**
         * Gets the document file size in bytes.
         */
        size: number;
        /**
         * Gets the number of slices into which the file is divided.
         */
        sliceCount: number;
        /**
         * Closes the document file.
         * 
         * @remarks
         * 
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#file | File}
         * 
         * No more than two documents are allowed to be in memory; otherwise the Document.getFileAsync operation will fail. Use the File.closeAsync 
         * method to close the file when you are finished working with it.
         * 
         * In the callback function passed to the closeAsync method, you can use the properties of the AsyncResult object to return the following 
         * information.
         * 
         * <table>
         *   <tr>
         *     <th>Property</th>
         *     <th>Use to...</th>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.value</td>
         *     <td>Always returns undefined because there is no object or data to retrieve.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.status</td>
         *     <td>Determine the success or failure of the operation.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.error</td>
         *     <td>Access an Error object that provides error information if the operation failed.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.asyncContext</td>
         *     <td>A user-defined item of any type that is returned in the AsyncResult object without being altered.</td>
         *   </tr>
         * </table>
         *
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        closeAsync(callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Returns the specified slice.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#file | File}
         * 
         * In the callback function passed to the getSliceAsync method, you can use the properties of the AsyncResult object to return the following 
         * information.
         * 
         * <table>
         *   <tr>
         *     <th>Property</th>
         *     <th>Use to...</th>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.value</td>
         *     <td>Access the Slice object.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.status</td>
         *     <td>Determine the success or failure of the operation.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.error</td>
         *     <td>Access an Error object that provides error information if the operation failed.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.asyncContext</td>
         *     <td>A user-defined item of any type that is returned in the AsyncResult object without being altered.</td>
         *   </tr>
         * </table>
         * 
         * @param sliceIndex Specifies the zero-based index of the slice to be retrieved. Required.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is the {@link Office.Slice} object.
         */
        getSliceAsync(sliceIndex: number, callback?: (result: AsyncResult<Office.Slice>) => void): void;
    }
    interface FileProperties {
        /**
         * File's URL
         */
        url: string
    }
    /**
     * Represents a binding in two dimensions of rows and columns.
     *
     * @remarks
     * The MatrixBinding object inherits the id property, type property, getDataAsync method, and setDataAsync method from the Binding object.
     */
    interface MatrixBinding extends Binding {
        /**
        * Gets the number of columns in the matrix data structure, as an integer value.
        */
        columnCount: number;
        /**
        * Gets the number of rows in the matrix data structure, as an integer value.
        */
        rowCount: number;
    }
    /**
     * Represents custom settings for a task pane or content add-in that are stored in the host document as name/value pairs.
     *
     * @remarks
     * **Hosts**: Access, Excel, PowerPoint, Word
     * 
     * The settings created by using the methods of the Settings object are saved per add-in and per document. 
     * That is, they are available only to the add-in that created them, and only from the document in which they are saved.
     *
     * The name of a setting is a string, while the value can be a string, number, boolean, null, object, or array.
     *
     * The Settings object is automatically loaded as part of the Document object, and is available by calling the settings property of that object 
     * when the add-in is activated. 
     * 
     * The developer is responsible for calling the saveAsync method after adding or deleting settings to save the settings in the document.
     */
    interface Settings {
        /**
         * Adds an event handler for the settingsChanged event.
         *
         * **Important**: Your add-in's code can register a handler for the settingsChanged event when the add-in is running with any Excel client, but 
         * the event will fire only when the add-in is loaded with a spreadsheet that is opened in Excel on the web, and more than one user is editing the 
         * spreadsheet (coauthoring). Therefore, effectively the settingsChanged event is supported only in Excel on the web in coauthoring scenarios.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#methods-that-arent-part-of-a-requirement-set | Not in a set}
         * 
         * You can add multiple event handlers for the specified eventType as long as the name of each event handler function is unique.
         *
         * @param eventType Specifies the type of event to add. Required.
         * @param handler The event handler function to add, whose only parameter is of type {@link Office.SettingsChangedEventArgs}. Required.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         * 
         * <table>
         *   <tr>
         *     <th>Property</th>
         *     <th>Use to...</th>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.value</td>
         *     <td>Always returns undefined because there is no data or object to retrieve when adding an event handler.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.status</td>
         *     <td>Determine the success or failure of the operation.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.error</td>
         *     <td>Access an Error object that provides error information if the operation failed.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.asyncContext</td>
         *     <td>A user-defined item of any type that is returned in the AsyncResult object without being altered.</td>
         *   </tr>
         * </table>
         */
        addHandlerAsync(eventType: Office.EventType, handler: any, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Adds an event handler for the settingsChanged event.
         *
         * **Important**: Your add-in's code can register a handler for the settingsChanged event when the add-in is running with any Excel client, but 
         * the event will fire only when the add-in is loaded with a spreadsheet that is opened in Excel on the web, and more than one user is editing the 
         * spreadsheet (coauthoring). Therefore, effectively the settingsChanged event is supported only in Excel on the web in coauthoring scenarios.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#methods-that-arent-part-of-a-requirement-set | Not in a set}
         * 
         * You can add multiple event handlers for the specified eventType as long as the name of each event handler function is unique.
         *
         * @param eventType Specifies the type of event to add. Required.
         * @param handler The event handler function to add, whose only parameter is of type {@link Office.SettingsChangedEventArgs}. Required.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         * 
         * <table>
         *   <tr>
         *     <th>Property</th>
         *     <th>Use to...</th>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.value</td>
         *     <td>Always returns undefined because there is no data or object to retrieve when adding an event handler.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.status</td>
         *     <td>Determine the success or failure of the operation.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.error</td>
         *     <td>Access an Error object that provides error information if the operation failed.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.asyncContext</td>
         *     <td>A user-defined item of any type that is returned in the AsyncResult object without being altered.</td>
         *   </tr>
         * </table>
         */
        addHandlerAsync(eventType: Office.EventType, handler: any, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Retrieves the specified setting.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#settings | Settings}
         *
         * @param settingName The case-sensitive name of the setting to retrieve.
         * @returns An object that has property names mapped to JSON serialized values.
         */
        get(name: string): any;
        /**
         * Reads all settings persisted in the document and refreshes the content or task pane add-in's copy of those settings held in memory.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#methods-that-arent-part-of-a-requirement-set | Not in a set}
         * 
         * This method is useful in Excel, Word, and PowerPoint coauthoring scenarios when multiple instances of the same add-in are working against 
         * the same document. Because each add-in is working against an in-memory copy of the settings loaded from the document at the time the user 
         * opened it, the settings values used by each user can get out of sync. This can happen whenever an instance of the add-in calls the 
         * Settings.saveAsync method to persist all of that user's settings to the document. Calling the refreshAsync method from the event handler 
         * for the settingsChanged event of the add-in will refresh the settings values for all users.
         *
         * In the callback function passed to the refreshAsync method, you can use the properties of the AsyncResult object to return the following 
         * information.
         * 
         * <table>
         *   <tr>
         *     <th>Property</th>
         *     <th>Use to...</th>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.value</td>
         *     <td>Access a Settings object with the refreshed values.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.status</td>
         *     <td>Determine the success or failure of the operation.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.error</td>
         *     <td>Access an Error object that provides error information if the operation failed.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.asyncContext</td>
         *     <td>A user-defined item of any type that is returned in the AsyncResult object without being altered.</td>
         *   </tr>
         * </table>
         *
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is an {@link Office.Settings} object with the refreshed values.
         */
        refreshAsync(callback?: (result: AsyncResult<Office.Settings>) => void): void;
        /**
         * Removes the specified setting.
         *
         * **Important**: Be aware that the Settings.remove method affects only the in-memory copy of the settings property bag. To persist the removal of 
         * the specified setting in the document, at some point after calling the Settings.remove method and before the add-in is closed, you must 
         * call the Settings.saveAsync method.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#settings | Settings}
         * 
         * null is a valid value for a setting. Therefore, assigning null to the setting will not remove it from the settings property bag.
         * 
         * @param settingName The case-sensitive name of the setting to remove.
         */
        remove(name: string): void;
        /**
         * Removes an event handler for the settingsChanged event.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#methods-that-arent-part-of-a-requirement-set | Not in a set}
         * 
         * If the optional handler parameter is omitted when calling the removeHandlerAsync method, all event handlers for the specified eventType 
         * will be removed.
         * 
         * When the function you passed to the callback parameter executes, it receives an AsyncResult object that you can access from the callback 
         * function's only parameter.
         * 
         * In the callback function passed to the removeHandlerAsync method, you can use the properties of the AsyncResult object to return the 
         * following information.
         *
         * @param eventType Specifies the type of event to remove. Required.
         * @param options Provides options to determine which event handler or handlers are removed.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        removeHandlerAsync(eventType: Office.EventType, options?: RemoveHandlerOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Removes an event handler for the settingsChanged event.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#methods-that-arent-part-of-a-requirement-set | Not in a set}
         * 
         * If the optional handler parameter is omitted when calling the removeHandlerAsync method, all event handlers for the specified eventType 
         * will be removed.
         * 
         * When the function you passed to the callback parameter executes, it receives an AsyncResult object that you can access from the callback 
         * function's only parameter.
         * 
         * In the callback function passed to the removeHandlerAsync method, you can use the properties of the AsyncResult object to return the 
         * following information.
         *
         * @param eventType Specifies the type of event to remove. Required.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        removeHandlerAsync(eventType: Office.EventType, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Persists the in-memory copy of the settings property bag in the document.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#settings | Settings}
         * 
         * Any settings previously saved by an add-in are loaded when it is initialized, so during the lifetime of the session you can just use the 
         * set and get methods to work with the in-memory copy of the settings property bag. When you want to persist the settings so that they are 
         * available the next time the add-in is used, use the saveAsync method.
         *
         * **Note**: The saveAsync method persists the in-memory settings property bag into the document file. However, the changes to the document file 
         * itself are saved only when the user (or AutoRecover setting) saves the document to the file system. The refreshAsync method is only useful 
         * in coauthoring scenarios when other instances of the same add-in might change the settings and those changes should be made available to 
         * all instances.
         * 
         * <table>
         *   <tr>
         *     <th>Property</th>
         *     <th>Use to...</th>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.value</td>
         *     <td>Always returns undefined because there is no object or data to retrieve.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.status</td>
         *     <td>Determine the success or failure of the operation.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.error</td>
         *     <td>Access an Error object that provides error information if the operation failed.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.asyncContext</td>
         *     <td>A user-defined item of any type that is returned in the AsyncResult object without being altered.</td>
         *   </tr>
         * </table>
         * 
         * @param options Provides options for saving settings.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        saveAsync(options?: SaveSettingsOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Persists the in-memory copy of the settings property bag in the document.
         * 
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#settings | Settings}
         * 
         * Any settings previously saved by an add-in are loaded when it is initialized, so during the lifetime of the session you can just use the 
         * set and get methods to work with the in-memory copy of the settings property bag. When you want to persist the settings so that they are 
         * available the next time the add-in is used, use the saveAsync method.
         *
         * **Note**: The saveAsync method persists the in-memory settings property bag into the document file. However, the changes to the document file 
         * itself are saved only when the user (or AutoRecover setting) saves the document to the file system. The refreshAsync method is only useful 
         * in coauthoring scenarios when other instances of the same add-in might change the settings and those changes should be made available to 
         * all instances.
         * 
         * <table>
         *   <tr>
         *     <th>Property</th>
         *     <th>Use to...</th>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.value</td>
         *     <td>Always returns undefined because there is no object or data to retrieve.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.status</td>
         *     <td>Determine the success or failure of the operation.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.error</td>
         *     <td>Access an Error object that provides error information if the operation failed.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.asyncContext</td>
         *     <td>A user-defined item of any type that is returned in the AsyncResult object without being altered.</td>
         *   </tr>
         * </table>
         * 
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        saveAsync(callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Sets or creates the specified setting.
         *
         * **Important**: Be aware that the Settings.set method affects only the in-memory copy of the settings property bag. 
         * To make sure that additions or changes to settings will be available to your add-in the next time the document is opened, at some point 
         * after calling the Settings.set method and before the add-in is closed, you must call the Settings.saveAsync method to persist settings in 
         * the document.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#settings | Settings}
         * 
         * The set method creates a new setting of the specified name if it does not already exist, or sets an existing setting of the specified name 
         * in the in-memory copy of the settings property bag. After you call the Settings.saveAsync method, the value is stored in the document as 
         * the serialized JSON representation of its data type. A maximum of 2MB is available for the settings of each add-in.
         * 
         * @param settingName The case-sensitive name of the setting to set or create.
         * @param value Specifies the value to be stored.
         */
        set(name: string, value: any): void;
    }
    /**
     * Provides information about the settings that raised the settingsChanged event.
     * 
     * To add an event handler for the settingsChanged event, use the addHandlerAsync method of the 
     * {@link Office.Settings} object.
     * 
     * The settingsChanged event fires only when your add-in's script calls the Settings.saveAsync method to persist 
     * the in-memory copy of the settings into the document file. The settingsChanged event is not triggered when the 
     * Settings.set or Settings.remove methods are called.
     * 
     * The settingsChanged event was designed to let you to handle potential conflicts when two or more users are 
     * attempting to save settings at the same time when your add-in is used in a shared (coauthored) document.
     * 
     * **Important**: Your add-in's code can register a handler for the settingsChanged event when the add-in 
     * is running with any Excel client, but the event will fire only when the add-in is loaded with a spreadsheet 
     * that is opened in Excel on the web, and more than one user is editing the spreadsheet (coauthoring). 
     * Therefore, effectively the settingsChanged event is supported only in Excel on the web in coauthoring scenarios.
     */
    interface SettingsChangedEventArgs {
        /**
         * Gets an {@link Office.Settings} object that represents the settings that raised the settingsChanged event.
         */
        settings: Settings;
        /**
         * Get an {@link Office.EventType} enumeration value that identifies the kind of event that was raised.
         */
        type: EventType;
    }
    /**
     * Represents a slice of a document file. The Slice object is accessed with the `File.getSliceAsync` method.
     */
    interface Slice {
        /**
         * Gets the raw data of the file slice in `Office.FileType.Text` ("text") or `Office.FileType.Compressed` ("compressed") format as specified 
         * by the fileType parameter of the call to the Document.getFileAsync method.
         *
         * @remarks
         * 
         * Files in the "compressed" format will return a byte array that can be transformed to a base64-encoded string if required.
         */
        data: any;
        /**
         * Gets the zero-based index of the file slice.
         */
        index: number;
        /**
         * Gets the size of the slice in bytes.
         */
        size: number;
    }
    /**
     * Represents a binding in two dimensions of rows and columns, optionally with headers.
     *
     * @remarks
     *
     * The TableBinding object inherits the `id` property, `type` property, `getDataAsync` method, and `setDataAsync` method from the 
     * {@link Office.Binding} object.
     *
     * For Excel, note that after you establish a table binding, each new row a user adds to the table is automatically included in the binding and 
     * rowCount increases.
     */
    interface TableBinding extends Binding {
        /**
        * Gets the number of columns in the TableBinding, as an integer value.
        */
        columnCount: number;
        /**
        * True, if the table has headers; otherwise false.
        */
        hasHeaders: boolean;
         /**
        * Gets the number of rows in the TableBinding, as an integer value.
        *
        * @remarks
        * When you insert an empty table by selecting a single row in Excel 2013 and Excel on the web (using Table on the Insert tab), both Office host 
        * applications create a single row of headers followed by a single blank row. However, if your add-in's script creates a binding for this 
        * newly inserted table (for example, by using the {@link Office.Bindings}.addFromSelectionAsync method), and then checks the value of the 
        * rowCount property, the value returned will differ depending whether the spreadsheet is open in Excel 2013 or Excel on the web.
        * 
        * - In Excel on the desktop, rowCount will return 0 (the blank row following the headers is not counted).
        *
        * - In Excel on the web, rowCount will return 1 (the blank row following the headers is counted).
        *
        * You can work around this difference in your script by checking if rowCount == 1, and if so, then checking if the row contains all empty 
        * strings.
        *
        * In content add-ins for Access, for performance reasons the rowCount property always returns -1.
        */
        rowCount: number;
        /**
         * Adds the specified data to the table as additional columns.
         *
         * @remarks
         *
         * To add one or more columns specifying the values of the data and headers, pass a TableData object as the data parameter. To add one or more 
         * columns specifying only the data, pass an array of arrays ("matrix") as the data parameter.
         *
         * The success or failure of an addColumnsAsync operation is atomic. That is, the entire add columns operation must succeed, or it will be 
         * completely rolled back (and the AsyncResult.status property returned to the callback will report failure):
         *
         *  - Each row in the array you pass as the data argument must have the same number of rows as the table being updated. If not, the entire 
         * operation will fail.
         *
         *  - Each row and cell in the array must successfully add that row or cell to the table in the newly added column(s). If any row or cell 
         * fails to be set for any reason, the entire operation will fail.
         *
         *  - If you pass a TableData object as the data argument, the number of header rows must match that of the table being updated.
         *
         * Additional remark for Excel on the web: The total number of cells in the TableData object passed to the data parameter can't exceed 20,000 in 
         * a single call to this method.
         *
         * @param tableData An array of arrays ("matrix") or a TableData object that contains one or more columns of data to add to the table. Required.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        addColumnsAsync(tableData: TableData | any[][], options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Adds the specified data to the table as additional columns.
         *
         * @remarks
         *
         * To add one or more columns specifying the values of the data and headers, pass a TableData object as the data parameter. To add one or more 
         * columns specifying only the data, pass an array of arrays ("matrix") as the data parameter.
         *
         * The success or failure of an addColumnsAsync operation is atomic. That is, the entire add columns operation must succeed, or it will be 
         * completely rolled back (and the AsyncResult.status property returned to the callback will report failure):
         *
         *  - Each row in the array you pass as the data argument must have the same number of rows as the table being updated. If not, the entire 
         * operation will fail.
         *
         *  - Each row and cell in the array must successfully add that row or cell to the table in the newly added column(s). If any row or cell 
         * fails to be set for any reason, the entire operation will fail.
         *
         *  - If you pass a TableData object as the data argument, the number of header rows must match that of the table being updated.
         *
         * Additional remark for Excel on the web: The total number of cells in the TableData object passed to the data parameter can't exceed 20,000 in 
         * a single call to this method.
         *
         * @param tableData An array of arrays ("matrix") or a TableData object that contains one or more columns of data to add to the table. Required.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        addColumnsAsync(tableData: TableData | any[][], callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Adds the specified data to the table as additional rows.
         *
         * @remarks
         *
         * The success or failure of an addRowsAsync operation is atomic. That is, the entire add columns operation must succeed, or it will be 
         * completely rolled back (and the AsyncResult.status property returned to the callback will report failure):
         *
         *  - Each row in the array you pass as the data argument must have the same number of columns as the table being updated. If not, the entire 
         * operation will fail.
         *
         *  - Each column and cell in the array must successfully add that column or cell to the table in the newly added rows(s). If any column or 
         * cell fails to be set for any reason, the entire operation will fail.
         *
         *  - If you pass a TableData object as the data argument, the number of header rows must match that of the table being updated.
         *
         * Additional remark for Excel on the web: The total number of cells in the TableData object passed to the data parameter can't exceed 20,000 in 
         * a single call to this method.
         *
         * @param rows An array of arrays ("matrix") or a TableData object that contains one or more rows of data to add to the table. Required.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        addRowsAsync(rows: TableData | any[][], options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Adds the specified data to the table as additional rows.
         *
         * @remarks
         *
         * The success or failure of an addRowsAsync operation is atomic. That is, the entire add columns operation must succeed, or it will be 
         * completely rolled back (and the AsyncResult.status property returned to the callback will report failure):
         *
         *  - Each row in the array you pass as the data argument must have the same number of columns as the table being updated. If not, the entire 
         * operation will fail.
         *
         *  - Each column and cell in the array must successfully add that column or cell to the table in the newly added rows(s). If any column or 
         * cell fails to be set for any reason, the entire operation will fail.
         *
         *  - If you pass a TableData object as the data argument, the number of header rows must match that of the table being updated.
         *
         * Additional remark for Excel on the web: The total number of cells in the TableData object passed to the data parameter can't exceed 20,000 in 
         * a single call to this method.
         *
         * @param rows An array of arrays ("matrix") or a TableData object that contains one or more rows of data to add to the table. Required.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        addRowsAsync(rows: TableData | any[][], callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Deletes all non-header rows and their values in the table, shifting appropriately for the host application.
         *
         * @remarks
         *
         * In Excel, if the table has no header row, this method will delete the table itself.
         *
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        deleteAllDataValuesAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Deletes all non-header rows and their values in the table, shifting appropriately for the host application.
         *
         * @remarks
         *
         * In Excel, if the table has no header row, this method will delete the table itself.
         *
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        deleteAllDataValuesAsync(callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Clears formatting on the bound table.
         *
         * @remarks
         * See {@link https://docs.microsoft.com/office/dev/add-ins/excel/excel-add-ins-tables#format-a-table | Format tables in add-ins for Excel} for more information.
         *
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        clearFormatsAsync(options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Clears formatting on the bound table.
         *
         * @remarks
         * See {@link https://docs.microsoft.com/office/dev/add-ins/excel/excel-add-ins-tables#format-a-table | Format tables in add-ins for Excel} for more information.
         *
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        clearFormatsAsync(callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Gets the formatting on specified items in the table.
         * 
         * @remarks
         * 
         * **Returned format structure**
         * 
         * Each JavaScript object in the return value array has this form: `{cells:{ cell_range }, format:{ format_definition }}`
         * 
         * The `cells:` property specifies the range you want format using one of the following values:
         * 
         * **Supported ranges in cells property**
         * 
         * <table>
         *   <tr>
         *     <th>cells range settings</th>
         *     <th>Description</th>
         *   </tr>
         *   <tr>
         *     <td>`{row: n}`</td>
         *     <td>Specifies the range that is the zero-based nth row of data in the table.</td>
         *   </tr>
         *   <tr>
         *     <td>`{column: n}`</td>
         *     <td>Specifies the range that is the zero-based nth column of data in the table.</td>
         *   </tr>
         *   <tr>
         *     <td>`{row: i, column: j}`</td>
         *     <td>Specifies the single cell that is the ith row and jth column of the table.</td>
         *   </tr>
         *   <tr>
         *     <td>`Office.Table.All`</td>
         *     <td>Specifies the entire table, including column headers, data, and totals (if any).</td>
         *   </tr>
         *   <tr>
         *     <td>`Office.Table.Data`</td>
         *     <td>Specifies only the data in the table (no headers and totals).</td>
         *   </tr>
         *   <tr>
         *     <td>`Office.Table.Headers`</td>
         *     <td>Specifies only the header row.</td>
         *   </tr>
         * </table>
         * 
         * The `format:` property specifies values that correspond to a subset of the settings available in the Format Cells dialog box in Excel 
         * (Right-click \> Format Cells or Home \> Format \> Format Cells).
         * 
         * @param cellReference An object literal containing name-value pairs that specify the range of cells to get formatting from.
         * @param formats An array specifying the format properties to get.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is an array containing one or more JavaScript objects specifying the formatting of their corresponding cells. 
         */
        getFormatsAsync(cellReference?: any, formats?: any[], options?: Office.AsyncContextOptions, callback?: (result: AsyncResult< ({ cells: any, format: any})[]>) => void): void;
        /**
         * Gets the formatting on specified items in the table.
         * 
         * @remarks
         * 
         * **Returned format structure**
         * 
         * Each JavaScript object in the return value array has this form: `{cells:{ cell_range }, format:{ format_definition }}`
         * 
         * The `cells:` property specifies the range you want format using one of the following values:
         * 
         * **Supported ranges in cells property**
         * 
         * <table>
         *   <tr>
         *     <th>cells range settings</th>
         *     <th>Description</th>
         *   </tr>
         *   <tr>
         *     <td>`{row: n}`</td>
         *     <td>Specifies the range that is the zero-based nth row of data in the table.</td>
         *   </tr>
         *   <tr>
         *     <td>`{column: n}`</td>
         *     <td>Specifies the range that is the zero-based nth column of data in the table.</td>
         *   </tr>
         *   <tr>
         *     <td>`{row: i, column: j}`</td>
         *     <td>Specifies the single cell that is the ith row and jth column of the table.</td>
         *   </tr>
         *   <tr>
         *     <td>`Office.Table.All`</td>
         *     <td>Specifies the entire table, including column headers, data, and totals (if any).</td>
         *   </tr>
         *   <tr>
         *     <td>`Office.Table.Data`</td>
         *     <td>Specifies only the data in the table (no headers and totals).</td>
         *   </tr>
         *   <tr>
         *     <td>`Office.Table.Headers`</td>
         *     <td>Specifies only the header row.</td>
         *   </tr>
         * </table>
         * 
         * The `format:` property specifies values that correspond to a subset of the settings available in the Format Cells dialog box in Excel 
         * (Right-click \> Format Cells or Home \> Format \> Format Cells).
         * 
         * @param cellReference An object literal containing name-value pairs that specify the range of cells to get formatting from.
         * @param formats An array specifying the format properties to get.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         *                  The `value` property of the result is an array containing one or more JavaScript objects specifying the formatting of their corresponding cells. 
         */
        getFormatsAsync(cellReference?: any, formats?: any[], callback?: (result: AsyncResult< ({ cells: any, format: any})[]>) => void): void;
        /**
         * Sets formatting on specified items and data in the table.
         *
         * @remarks
         * 
         * **Specifying the cellFormat parameter**
         * 
         * Use the cellFormat parameter to set or change cell formatting values, such as width, height, font, background, alignment, and so on. 
         * The value you pass as the cellFormat parameter is an array that contains a list of one or more JavaScript objects that specify which cells 
         * to target (`cells:`) and the formats (`format:`) to apply to them.
         * 
         * Each JavaScript object in the cellFormat array has this form: `{cells:{ cell_range }, format:{ format_definition }}`
         * 
         * The `cells:` property specifies the range you want format using one of the following values:
         * 
         * **Supported ranges in cells property**
         * 
         * <table>
         *   <tr>
         *     <th>cells range settings</th>
         *     <th>Description</th>
         *   </tr>
         *   <tr>
         *     <td>`{row: n}`</td>
         *     <td>Specifies the range that is the zero-based nth row of data in the table.</td>
         *   </tr>
         *   <tr>
         *     <td>`{column: n}`</td>
         *     <td>Specifies the range that is the zero-based nth column of data in the table.</td>
         *   </tr>
         *   <tr>
         *     <td>`{row: i, column: j}`</td>
         *     <td>Specifies the single cell that is the ith row and jth column of the table.</td>
         *   </tr>
         *   <tr>
         *     <td>`Office.Table.All`</td>
         *     <td>Specifies the entire table, including column headers, data, and totals (if any).</td>
         *   </tr>
         *   <tr>
         *     <td>`Office.Table.Data`</td>
         *     <td>Specifies only the data in the table (no headers and totals).</td>
         *   </tr>
         *   <tr>
         *     <td>`Office.Table.Headers`</td>
         *     <td>Specifies only the header row.</td>
         *   </tr>
         * </table>
         * 
         * The `format:` property specifies values that correspond to a subset of the settings available in the Format Cells dialog box in Excel 
         * (Right-click \> Format Cells or Home \> Format \> Format Cells).
         * 
         * You specify the value of the `format:` property as a list of one or more property name - value pairs in a JavaScript object literal. The 
         * property name specifies the name of the formatting property to set, and value specifies the property value. 
         * You can specify multiple values for a given format, such as both a font's color and size. 
         * 
         * Here's three `format:` property value examples:
         * 
         * `//Set cells: font color to green and size to 15 points.`
         * 
         * `format: {fontColor : "green", fontSize : 15}`
         * 
         * `//Set cells: border to dotted blue.`
         * 
         * `format: {borderStyle: "dotted", borderColor: "blue"}`
         * 
         * `//Set cells: background to red and alignment to centered.`
         * 
         * `format: {backgroundColor: "red", alignHorizontal: "center"}`
         * 
         * 
         * You can specify number formats by specifying the number formatting "code" string in the `numberFormat:` property. 
         * The number format strings you can specify correspond to those you can set in Excel using the Custom category on the Number tab of the Format Cells dialog box. 
         * This example shows how to format a number as a percentage with two decimal places:
         * 
         * `format: {numberFormat:"0.00%"}`
         * 
         * For more detail, see how to {@link https://support.office.com/article/create-or-delete-a-custom-number-format-78f2a361-936b-4c03-8772-09fab54be7f4 | Create a custom number format}.
         * 
         * To set formatting on tables when writing data, use the tableOptions and cellFormat optional parameters of the 
         * `Document.setSelectedDataAsync` or `TableBinding.setDataAsync` methods.
         * 
         * Setting formatting with the optional parameters of the `Document.setSelectedDataAsync` and `TableBinding.setDataAsync` methods only works 
         * to set formatting when writing data the first time. 
         * To make formatting changes after writing data, use the following methods:
         * 
         *  - To update cell formatting, such as font color and style, use the `TableBinding.setFormatsAsync` method (this method).
         * 
         *  - To update table options, such as banded rows and filter buttons, use the `TableBinding.setTableOptions` method.
         * 
         *  - To clear formatting, use the `TableBinding.clearFormats` method.
         * 
         * For more details and examples, see 
         * {@link https://docs.microsoft.com/office/dev/add-ins/excel/excel-add-ins-tables#format-a-table | How to format tables in add-ins for Excel}.
         *
         * @param cellFormat An array that contains one or more JavaScript objects that specify which cells to target and the formatting to apply to them.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        setFormatsAsync(cellFormat: any[], options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Sets formatting on specified items and data in the table.
         *
         * @remarks
         * 
         * **Specifying the cellFormat parameter**
         * 
         * Use the cellFormat parameter to set or change cell formatting values, such as width, height, font, background, alignment, and so on. 
         * The value you pass as the cellFormat parameter is an array that contains a list of one or more JavaScript objects that specify which cells 
         * to target (`cells:`) and the formats (`format:`) to apply to them.
         * 
         * Each JavaScript object in the cellFormat array has this form: `{cells:{ cell_range }, format:{ format_definition }}`
         * 
         * The `cells:` property specifies the range you want format using one of the following values:
         * 
         * **Supported ranges in cells property**
         * 
         * <table>
         *   <tr>
         *     <th>cells range settings</th>
         *     <th>Description</th>
         *   </tr>
         *   <tr>
         *     <td>`{row: n}`</td>
         *     <td>Specifies the range that is the zero-based nth row of data in the table.</td>
         *   </tr>
         *   <tr>
         *     <td>`{column: n}`</td>
         *     <td>Specifies the range that is the zero-based nth column of data in the table.</td>
         *   </tr>
         *   <tr>
         *     <td>`{row: i, column: j}`</td>
         *     <td>Specifies the single cell that is the ith row and jth column of the table.</td>
         *   </tr>
         *   <tr>
         *     <td>`Office.Table.All`</td>
         *     <td>Specifies the entire table, including column headers, data, and totals (if any).</td>
         *   </tr>
         *   <tr>
         *     <td>`Office.Table.Data`</td>
         *     <td>Specifies only the data in the table (no headers and totals).</td>
         *   </tr>
         *   <tr>
         *     <td>`Office.Table.Headers`</td>
         *     <td>Specifies only the header row.</td>
         *   </tr>
         * </table>
         * 
         * The `format:` property specifies values that correspond to a subset of the settings available in the Format Cells dialog box in Excel 
         * (Right-click \> Format Cells or Home \> Format \> Format Cells).
         * 
         * You specify the value of the `format:` property as a list of one or more property name - value pairs in a JavaScript object literal. The 
         * property name specifies the name of the formatting property to set, and value specifies the property value. 
         * You can specify multiple values for a given format, such as both a font's color and size. 
         * 
         * Here's three `format:` property value examples:
         * 
         * `//Set cells: font color to green and size to 15 points.`
         * 
         * `format: {fontColor : "green", fontSize : 15}`
         * 
         * `//Set cells: border to dotted blue.`
         * 
         * `format: {borderStyle: "dotted", borderColor: "blue"}`
         * 
         * `//Set cells: background to red and alignment to centered.`
         * 
         * `format: {backgroundColor: "red", alignHorizontal: "center"}`
         * 
         * 
         * You can specify number formats by specifying the number formatting "code" string in the `numberFormat:` property. 
         * The number format strings you can specify correspond to those you can set in Excel using the Custom category on the Number tab of the Format Cells dialog box. 
         * This example shows how to format a number as a percentage with two decimal places:
         * 
         * `format: {numberFormat:"0.00%"}`
         * 
         * For more detail, see how to {@link https://support.office.com/article/create-or-delete-a-custom-number-format-78f2a361-936b-4c03-8772-09fab54be7f4 | Create a custom number format}.
         * 
         * To set formatting on tables when writing data, use the tableOptions and cellFormat optional parameters of the 
         * `Document.setSelectedDataAsync` or `TableBinding.setDataAsync` methods.
         * 
         * Setting formatting with the optional parameters of the `Document.setSelectedDataAsync` and `TableBinding.setDataAsync` methods only works 
         * to set formatting when writing data the first time. 
         * To make formatting changes after writing data, use the following methods:
         * 
         *  - To update cell formatting, such as font color and style, use the `TableBinding.setFormatsAsync` method (this method).
         * 
         *  - To update table options, such as banded rows and filter buttons, use the `TableBinding.setTableOptions` method.
         * 
         *  - To clear formatting, use the `TableBinding.clearFormats` method.
         * 
         * For more details and examples, see 
         * {@link https://docs.microsoft.com/office/dev/add-ins/excel/excel-add-ins-tables#format-a-table | How to format tables in add-ins for Excel}.
         *
         * @param cellFormat An array that contains one or more JavaScript objects that specify which cells to target and the formatting to apply to them.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         */
        setFormatsAsync(cellFormat: any[], callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Updates table formatting options on the bound table.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#methods-that-arent-part-of-a-requirement-set | Not in a set}
         * 
         * In the callback function passed to the goToByIdAsync method, you can use the properties of the AsyncResult object to return the following information.
         * 
         * <table>
         *   <tr>
         *     <th>Property</th>
         *     <th>Use to...</th>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.value</td>
         *     <td>Always returns undefined because there is no data or object to retrieve when setting formats.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.status</td>
         *     <td>Determine the success or failure of the operation.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.error</td>
         *     <td>Access an Error object that provides error information if the operation failed.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.asyncContext</td>
         *     <td>A user-defined item of any type that is returned in the AsyncResult object without being altered.</td>
         *   </tr>
         * </table>
         *
         * @param tableOptions An object literal containing a list of property name-value pairs that define the table options to apply.
         * @param options Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         * 
         */
        setTableOptionsAsync(tableOptions: any, options?: Office.AsyncContextOptions, callback?: (result: AsyncResult<void>) => void): void;
        /**
         * Updates table formatting options on the bound table.
         *
         * @remarks
         * **Requirement set**: {@link https://docs.microsoft.com/office/dev/add-ins/reference/requirement-sets/office-add-in-requirement-sets#methods-that-arent-part-of-a-requirement-set | Not in a set}
         * 
         * In the callback function passed to the goToByIdAsync method, you can use the properties of the AsyncResult object to return the following information.
         * 
         * <table>
         *   <tr>
         *     <th>Property</th>
         *     <th>Use to...</th>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.value</td>
         *     <td>Always returns undefined because there is no data or object to retrieve when setting formats.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.status</td>
         *     <td>Determine the success or failure of the operation.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.error</td>
         *     <td>Access an Error object that provides error information if the operation failed.</td>
         *   </tr>
         *   <tr>
         *     <td>AsyncResult.asyncContext</td>
         *     <td>A user-defined item of any type that is returned in the AsyncResult object without being altered.</td>
         *   </tr>
         * </table>
         *
         * @param tableOptions An object literal containing a list of property name-value pairs that define the table options to apply.
         * @param callback Optional. A function that is invoked when the callback returns, whose only parameter is of type {@link Office.AsyncResult}.
         * 
         */
        setTableOptionsAsync(tableOptions: any, callback?: (result: AsyncResult<void>) => void): void;
    }
    /**
     * Represents the data in a table or an {@link Office.TableBinding}.
     */
    class TableData {
        constructor(rows: any[][], headers: any[]);
        constructor();
        /**
         * Gets or sets the headers of the table.
         * 
         * @remarks
         *
         * To specify headers, you must specify an array of arrays that corresponds to the structure of the table. For example, to specify headers 
         * for a two-column table you would set the header property to [['header1', 'header2']].
         *
         * If you specify null for the headers property (or leaving the property empty when you construct a TableData object), the following results 
         * occur when your code executes:
         *
         * - If you insert a new table, the default column headers for the table are created.
         *
         * - If you overwrite or update an existing table, the existing headers are not altered.
         */
        headers: any[];
        /**
         * Gets or sets the rows in the table. Returns an array of arrays that contains the data in the table. 
         * Returns an empty array if there are no rows.
         * 
         * @remarks
         *
         * To specify rows, you must specify an array of arrays that corresponds to the structure of the table. For example, to specify two rows of 
         * string values in a two-column table you would set the rows property to [['a', 'b'], ['c', 'd']].
         *
         * If you specify null for the rows property (or leave the property empty when you construct a TableData object), the following results occur 
         * when your code executes:
         *
         * - If you insert a new table, a blank row will be inserted.
         *
         * - If you overwrite or update an existing table, the existing rows are not altered.
         */
        rows: any[][];
    }
    /**
     * Specifies enumerated values for the `cells` property in the cellFormat parameter of 
     * {@link https://docs.microsoft.com/office/dev/add-ins/excel/excel-add-ins-tables#format-a-table | table formatting methods}.
     */
    enum Table {
        /**
         * The entire table, including column headers, data, and totals (if any).
         */
        All,
        /**
         * Only the data (no headers and totals).
         */
        Data,
        /**
         * Only the header row.
         */
        Headers
    }
    /**
     * Represents a bound text selection in the document.
     * 
     * The TextBinding object inherits the id property, type property, getDataAsync method, and setDataAsync method from the {@link Office.Binding} 
     * object. It does not implement any additional properties or methods of its own.
     */
    interface TextBinding extends Binding { }
    /**
     * Specifies the project fields that are available as a parameter for the {@link Office.Document | Document}.getProjectFieldAsync method.
     *
     * @remarks
     * 
     * A ProjectProjectFields constant can be used as a parameter of the {@link Office.Document | Document}.getProjectFieldAsync method.
     */
    enum ProjectProjectFields {
        /**
         * The number of digits after the decimal for the currency.
         */
        CurrencyDigits,
        /**
         * The currency symbol.
         */
        CurrencySymbol,
        /**
         * The placement of the currency symbol: Not specified = -1; Before the value with no space ($0) = 0; After the value with no space (0$) = 1; 
         * Before the value with a space ($ 0) = 2; After the value with a space (0 $) = 3.
         */
        CurrencySymbolPosition,
        DurationUnits,
        /**
         * The GUID of the project.
         */
        GUID,
        /**
         * The project finish date.
         */
        Finish,
        /**
         * The project start date.
         */
        Start,
        /**
         * Specifies whether the project is read-only.
         */
        ReadOnly,
        /**
         * The project version.
         */
        VERSION,
        /**
         * The work units of the project, such as days or hours.
         */
        WorkUnits,
        /**
         * The Project Web App URL, for projects that are stored in Project Server.
         */
        ProjectServerUrl,
        /**
         * The SharePoint URL, for projects that are synchronized with a SharePoint list.
         */
        WSSUrl,
        /**
         * The name of the SharePoint list, for projects that are synchronized with a tasks list.
         */
        WSSList
    }
    /**
     * Specifies the resource fields that are available as a parameter for the {@link Office.Document | Document}.getResourceFieldAsync method.
     *
     * @remarks
     * A ProjectResourceFields constant can be used as a parameter of the {@link Office.Document | Document}.getResourceFieldAsync method.
     *
     * For more information about working with fields in Project, see 
     * {@link https://support.office.com/article/Available-fields-reference-615a4563-1cc3-40f4-b66f-1b17e793a460 | Available fields} reference. In 
     * Project Help, search for Available fields.
     */
    enum ProjectResourceFields {
        /**
         * The accrual method that defines how a task accrues the cost of the resource: Accrues when the task starts = 1, 
         * accrues when the task ends = 2, accrues as the task progresses (prorated) = 3.
         */
        Accrual,
        /**
         * The calculated actual cost of the resource for assignments in the project.
         */
        ActualCost,
        /**
         * The actual overtime cost for a resource.
         */
        ActualOvertimeCost,
        /**
         * The actual overtime work for a resource, in minutes.
         */
        ActualOvertimeWork,
        /**
         * The actual overtime work for the resource that has been protected (made read-only).
         */
        ActualOvertimeWorkProtected,
        /**
         * The actual work that the resource has done on assignments in the project.
         */
        ActualWork,
        /**
         * The actual work for the resource that has been protected (made read-only).
         */
        ActualWorkProtected,
        /**
         * The name of the base calendar for the resource.
         */
        BaseCalendar,
        /**
         * The budget cost for the baseline resource.
         */
        Baseline10BudgetCost,
        /**
         * The budget work for the baseline resource, in hours.
         */
        Baseline10BudgetWork,
        /**
         * The cost for the baseline resource.
         */
        Baseline10Cost,
        /**
         * The work for the baseline resource, in minutes.
         */
        Baseline10Work,
        /**
         * The budget cost for the baseline resource.
         */
        Baseline1BudgetCost,
        /**
         * The budget work for the baseline resource, in hours.
         */
        Baseline1BudgetWork,
        /**
         * The cost for the baseline resource.
         */
        Baseline1Cost,
        /**
         * The work for the baseline resource, in minutes.
         */
        Baseline1Work,
        /**
         * The budget cost for the baseline resource.
         */
        Baseline2BudgetCost,
        /**
         * The budget work for the baseline resource, in hours.
         */
        Baseline2BudgetWork,
        /**
         * The cost for the baseline resource.
         */
        Baseline2Cost,
        /**
         * The work for the baseline resource, in minutes.
         */
        Baseline2Work,
        /**
         * The budget cost for the baseline resource.
         */
        Baseline3BudgetCost,
        /**
         * The budget work for the baseline resource, in hours.
         */
        Baseline3BudgetWork,
        /**
         * The cost for the baseline resource.
         */
        Baseline3Cost,
        /**
         * The work for the baseline resource, in minutes.
         */
        Baseline3Work,
        /**
         * The budget cost for the baseline resource.
         */
        Baseline4BudgetCost,
        /**
         * The budget work for the baseline resource, in hours.
         */
        Baseline4BudgetWork,
        /**
         * The cost for the baseline resource.
         */
        Baseline4Cost,
        /**
         * The work for the baseline resource, in minutes.
         */
        Baseline4Work,
        /**
         * The budget cost for the baseline resource.
         */
        Baseline5BudgetCost,
        /**
         * The budget work for the baseline resource, in hours.
         */
        Baseline5BudgetWork,
        /**
         * The cost for the baseline resource.
         */
        Baseline5Cost,
        /**
         * The work for the baseline resource, in minutes.
         */
        Baseline5Work,
        /**
         * The budget cost for the baseline resource.
         */
        Baseline6BudgetCost,
        /**
         * The budget work for the baseline resource, in hours.
         */
        Baseline6BudgetWork,
        /**
         * The cost for the baseline resource.
         */
        Baseline6Cost,
        /**
         * The work for the baseline resource, in minutes.
         */
        Baseline6Work,
        /**
         * The budget cost for the baseline resource.
         */
        Baseline7BudgetCost,
        /**
         * The budget work for the baseline resource, in hours.
         */
        Baseline7BudgetWork,
        /**
         * The cost for the baseline resource.
         */
        Baseline7Cost,
        /**
         * The work for the baseline resource, in minutes.
         */
        Baseline7Work,
        /**
         * The budget cost for the baseline resource.
         */
        Baseline8BudgetCost,
        /**
         * The budget work for the baseline resource, in hours.
         */
        Baseline8BudgetWork,
        /**
         * The cost for the baseline resource.
         */
        Baseline8Cost,
        /**
         * The work for the baseline resource, in minutes.
         */
        Baseline8Work,
        /**
         * The budget cost for the baseline resource.
         */
        Baseline9BudgetCost,
        /**
         * The budget work for the baseline resource, in hours.
         */
        Baseline9BudgetWork,
        /**
         * The cost for the baseline resource.
         */
        Baseline9Cost,
        /**
         * The work for the baseline resource, in minutes.
         */
        Baseline9Work,
        /**
         * The budget cost for the baseline resource.
         */
        BaselineBudgetCost,
        /**
         * The budget work for the baseline resource, in hours.
         */
        BaselineBudgetWork,
        /**
         * The baseline cost for the resource for assignments in the project.
         */
        BaselineCost,
        /**
         * The baseline work for the resource for assignments in the project, in minutes.
         */
        BaselineWork,
        /**
         * The budget cost for the resource.
         */
        BudgetCost,
        /**
         * The budget work for the resource.
         */
        BudgetWork,
        /**
         * The GUID of the resource calendar.
         */
        ResourceCalendarGUID,
        /**
         * The code value of the resource.
         */
        Code,
        /**
         * A cost field for the resource.
         */
        Cost1,
        /**
         * A cost field for the resource.
         */
        Cost10,
        /**
         * A cost field for the resource.
         */
        Cost2,
        /**
         * A cost field for the resource.
         */
        Cost3,
        /**
         * A cost field for the resource.
         */
        Cost4,
        /**
         * A cost field for the resource.
         */
        Cost5,
        /**
         * A cost field for the resource.
         */
        Cost6,
        /**
         * A cost field for the resource.
         */
        Cost7,
        /**
         * A cost field for the resource.
         */
        Cost8,
        /**
         * A cost field for the resource.
         */
        Cost9,
        /**
         * The date the resource was created.
         */
        ResourceCreationDate,
        /**
         * A date field for the resource.
         */
        Date1,
        /**
         * A date field for the resource.
         */
        Date10,
        /**
         * A date field for the resource.
         */
        Date2,
        /**
         * A date field for the resource.
         */
        Date3,
        /**
         * A date field for the resource.
         */
        Date4,
        /**
         * A date field for the resource.
         */
        Date5,
        /**
         * A date field for the resource.
         */
        Date6,
        /**
         * A date field for the resource.
         */
        Date7,
        /**
         * A date field for the resource.
         */
        Date8,
        /**
         * A date field for the resource.
         */
        Date9,
        /**
         * A duration field for the resource.
         */
        Duration1,
        /**
         * A duration field for the resource.
         */
        Duration10,
        /**
         * A duration field for the resource.
         */
        Duration2,
        /**
         * A duration field for the resource.
         */
        Duration3,
        /**
         * A duration field for the resource.
         */
        Duration4,
        /**
         * A duration field for the resource.
         */
        Duration5,
        /**
         * A duration field for the resource.
         */
        Duration6,
        /**
         * A duration field for the resource.
         */
        Duration7,
        /**
         * A duration field for the resource.
         */
        Duration8,
        /**
         * A duration field for the resource.
         */
        Duration9,
        /**
         * The email address of the resource.
         */
        Email,
        /**
         * The end date of the resource availability.
         */
        End,
        /**
         * A finish field for the task.
         */
        Finish1,
        /**
         * A finish field for the task.
         */
        Finish10,
        /**
         * A finish field for the task.
         */
        Finish2,
        /**
         * A finish field for the task.
         */
        Finish3,
        /**
         * A finish field for the task.
         */
        Finish4,
        /**
         * A finish field for the task.
         */
        Finish5,
        /**
         * A finish field for the task.
         */
        Finish6,
        /**
         * A finish field for the task.
         */
        Finish7,
        /**
         * A finish field for the task.
         */
        Finish8,
        /**
         * A finish field for the task.
         */
        Finish9,
        /**
         * A Boolean flag field for the resource.
         */
        Flag10,
        /**
         * A Boolean flag field for the resource.
         */
        Flag1,
        /**
         * A Boolean flag field for the resource.
         */
        Flag11,
        /**
         * A Boolean flag field for the resource.
         */
        Flag12,
        /**
         * A Boolean flag field for the resource.
         */
        Flag13,
        /**
         * A Boolean flag field for the resource.
         */
        Flag14,
        /**
         * A Boolean flag field for the resource.
         */
        Flag15,
        /**
         * A Boolean flag field for the resource.
         */
        Flag16,
        /**
         * A Boolean flag field for the resource.
         */
        Flag17,
        /**
         * A Boolean flag field for the resource.
         */
        Flag18,
        /**
         * A Boolean flag field for the resource.
         */
        Flag19,
        /**
         * A Boolean flag field for the resource.
         */
        Flag2,
        /**
         * A Boolean flag field for the resource.
         */
        Flag20,
        /**
         * A Boolean flag field for the resource.
         */
        Flag3,
        /**
         * A Boolean flag field for the resource.
         */
        Flag4,
        /**
         * A Boolean flag field for the resource.
         */
        Flag5,
        /**
         * A Boolean flag field for the resource.
         */
        Flag6,
        /**
         * A Boolean flag field for the resource.
         */
        Flag7,
        /**
         * A Boolean flag field for the resource.
         */
        Flag8,
        /**
         * A Boolean flag field for the resource.
         */
        Flag9,
        /**
         * The group the resource belongs to.
         */
        Group,
        /**
         * The percentage of work units that the resource has assigned in the project. If the resource is working full-time on the project, Units = 100.
         */
        Units,
        /**
         * The name of the resource.
         */
        Name,
        /**
         * The text value of the notes regarding the resource.
         */
        Notes,
        /**
         * A number field for the resource.
         */
        Number1,
        /**
         * A number field for the resource.
         */
        Number10,
        /**
         * A number field for the resource.
         */
        Number11,
        /**
         * A number field for the resource.
         */
        Number12,
        /**
         * A number field for the resource.
         */
        Number13,
        /**
         * A number field for the resource.
         */
        Number14,
        /**
         * A number field for the resource.
         */
        Number15,
        /**
         * A number field for the resource.
         */
        Number16,
        /**
         * A number field for the resource.
         */
        Number17,
        /**
         * A number field for the resource.
         */
        Number18,
        /**
         * A number field for the resource.
         */
        Number19,
        /**
         * A number field for the resource.
         */
        Number2,
        /**
         * A number field for the resource.
         */
        Number20,
        /**
         * A number field for the resource.
         */
        Number3,
        /**
         * A number field for the resource.
         */
        Number4,
        /**
         * A number field for the resource.
         */
        Number5,
        /**
         * A number field for the resource.
         */
        Number6,
        /**
         * A number field for the resource.
         */
        Number7,
        /**
         * A number field for the resource.
         */
        Number8,
        /**
         * A number field for the resource.
         */
        Number9,
        /**
         * The overtime cost for a resource.
         */
        OvertimeCost,
        /**
         * The overtime rate for a resource.
         */
        OvertimeRate,
        /**
         * The overtime work for a resource.
         */
        OvertimeWork,
        /**
         * The percentage of work complete for a resource.
         */
        PercentWorkComplete,
        /**
         * The cost per use of the resource.
         */
        CostPerUse,
        /**
         * Indicates whether the resource is a generic resource (identified by skill rather than by name).
         */
        Generic,
        /**
         * Indicates whether the resource is overallocated.
         */
        OverAllocated,
        /**
         * The amount of regular work for the resource.
         */
        RegularWork,
        /**
         * The remaining cost for the resource.
         */
        RemainingCost,
        /**
         * The remaining overtime cost for the resource.
         */
        RemainingOvertimeCost,
        /**
         * The remaining overtime work for the resource, in minutes.
         */
        RemainingOvertimeWork,
        /**
         * The remaining work for the resource, in minutes.
         */
        RemainingWork,
        /**
         * The ID of the resource.
         */
        ResourceGUID,
        /**
         * The total cost of the resource.
         */
        Cost,
        /**
         * The total work for the resource, in minutes.
         */
        Work,
        /**
         * The start date for the resource.
         */
        Start,
        /**
         * A start field for the resource.
         */
        Start1,
        /**
         * A start field for the resource.
         */
        Start10,
        /**
         * A start field for the resource.
         */
        Start2,
        /**
         * A start field for the resource.
         */
        Start3,
        /**
         * A start field for the resource.
         */
        Start4,
        /**
         * A start field for the resource.
         */
        Start5,
        /**
         * A start field for the resource.
         */
        Start6,
        /**
         * A start field for the resource.
         */
        Start7,
        /**
         * A start field for the resource.
         */
        Start8,
        /**
         * A start field for the resource.
         */
        Start9,
        /**
         * The standard rate of pay for the resource, in cost per hour.
         */
        StandardRate,
        /**
         * A text field for the resource.
         */
        Text1,
        /**
         * A text field for the resource.
         */
        Text10,
        /**
         * A text field for the resource.
         */
        Text11,
        /**
         * A text field for the resource.
         */
        Text12,
        /**
         * A text field for the resource.
         */
        Text13,
        /**
         * A text field for the resource.
         */
        Text14,
        /**
         * A text field for the resource.
         */
        Text15,
        /**
         * A text field for the resource.
         */
        Text16,
        /**
         * A text field for the resource.
         */
        Text17,
        /**
         * A text field for the resource.
         */
        Text18,
        /**
         * A text field for the resource.
         */
        Text19,
        /**
         * A text field for the resource.
         */
        Text2,
        /**
         * A text field for the resource.
         */
        Text20,
        /**
         * A text field for the resource.
         */
        Text21,
        /**
         * A text field for the resource.
         */
        Text22,
        /**
         * A text field for the resource.
         */
        Text23,
        /**
         * A text field for the resource.
         */
        Text24,
        /**
         * A text field for the resource.
         */
        Text25,
        /**
         * A text field for the resource.
         */
        Text26,
        /**
         * A text field for the resource.
         */
        Text27,
        /**
         * A text field for the resource.
         */
        Text28,
        /**
         * A text field for the resource.
         */
        Text29,
        /**
         * A text field for the resource.
         */
        Text3,
        /**
         * A text field for the resource.
         */
        Text30,
        /**
         * A text field for the resource.
         */
        Text4,
        /**
         * A text field for the resource.
         */
        Text5,
        /**
         * A text field for the resource.
         */
        Text6,
        /**
         * A text field for the resource.
         */
        Text7,
        /**
         * A text field for the resource.
         */
        Text8,
        /**
         * A text field for the resource.
         */
        Text9
    }
    /**
     * Specifies the task fields that are available as a parameter for the {@link Office.Document | Document}.getTaskFieldAsync method.
     *
     * @remarks
     * A ProjectTaskFields constant can be used as a parameter of the {@link Office.Document | Document}.getTaskFieldAsync method.
     *
     * For more information about working with fields in Project, see the 
     * {@link https://support.office.com/article/Available-fields-reference-615a4563-1cc3-40f4-b66f-1b17e793a460 | Available fields} reference. 
     * In Project Help, search for Available fields.
     */
    enum ProjectTaskFields {
        /**
         * The current actual cost for the task.
         */
        ActualCost,
        /**
         * The actual duration of the task, in minutes.
         */
        ActualDuration,
        /**
         * The actual finish date of the task.
         */
        ActualFinish,
        /**
         * The actual overtime cost for the task.
         */
        ActualOvertimeCost,
        /**
         * The actual overtime work for the task, in minutes.
         */
        ActualOvertimeWork,
        /**
         * The actual start date of the task.
         */
        ActualStart,
        /**
         * The actual work for the task, in minutes.
         */
        ActualWork,
        /**
         * A text field for the task.
         */
        Text1,
        /**
         * A text field for the task.
         */
        Text10,
        /**
         * A finish field for the task.
         */
        Finish10,
        /**
         * A start field for the task.
         */
        Start10,
        /**
         * A text field for the task.
         */
        Text11,
        /**
         * A text field for the task.
         */
        Text12,
        /**
         * A text field for the task.
         */
        Text13,
        /**
         * A text field for the task.
         */
        Text14,
        /**
         * A text field for the task.
         */
        Text15,
        /**
         * A text field for the task.
         */
        Text16,
        /**
         * A text field for the task.
         */
        Text17,
        /**
         * A text field for the task.
         */
        Text18,
        /**
         * A text field for the task.
         */
        Text19,
        /**
         * A finish field for the task.
         */
        Finish1,
        /**
         * A start field for the task.
         */
        Start1,
        /**
         * A text field for the task.
         */
        Text2,
        /**
         * A text field for the task.
         */
        Text20,
        /**
         * A text field for the task.
         */
        Text21,
        /**
         * A text field for the task.
         */
        Text22,
        /**
         * A text field for the task.
         */
        Text23,
        /**
         * A text field for the task.
         */
        Text24,
        /**
         * A text field for the task.
         */
        Text25,
        /**
         * A text field for the task.
         */
        Text26,
        /**
         * A text field for the task.
         */
        Text27,
        /**
         * A text field for the task.
         */
        Text28,
        /**
         * A text field for the task.
         */
        Text29,
        /**
         * A finish field for the task.
         */
        Finish2,
        /**
         * A start field for the task.
         */
        Start2,
        /**
         * A text field for the task.
         */
        Text3,
        /**
         * A text field for the task.
         */
        Text30,
        /**
         * A finish field for the task.
         */
        Finish3,
        /**
         * A start field for the task.
         */
        Start3,
        /**
         * A text field for the task.
         */
        Text4,
        /**
         * A finish field for the task.
         */
        Finish4,
        /**
         * A start field for the task.
         */
        Start4,
        /**
         * A text field for the task.
         */
        Text5,
        /**
         * A finish field for the task.
         */
        Finish5,
        /**
         * A start field for the task.
         */
        Start5,
        /**
         * A text field for the task.
         */
        Text6,
        /**
         * A finish field for the task.
         */
        Finish6,
        /**
         * A start field for the task.
         */
        Start6,
        /**
         * A text field for the task.
         */
        Text7,
        /**
         * A finish field for the task.
         */
        Finish7,
        /**
         * A start field for the task.
         */
        Start7,
        /**
         * A text field for the task.
         */
        Text8,
        /**
         * A finish field for the task.
         */
        Finish8,
        /**
         * A start field for the task.
         */
        Start8,
        /**
         * A text field for the task.
         */
        Text9,
        /**
         * A finish field for the task.
         */
        Finish9,
        /**
         * A start field for the task.
         */
        Start9,
        /**
         * The budget cost for the baseline task.
         */
        Baseline10BudgetCost,
        /**
         * The budget work for the baseline task, in hours.
         */
        Baseline10BudgetWork,
        /**
         * The cost for the baseline task.
         */
        Baseline10Cost,
        /**
         * The duration for the baseline task, in minutes.
         */
        Baseline10Duration,
        /**
         * The finish date for the baseline task.
         */
        Baseline10Finish,
        /**
         * The fixed cost of any non-resource expense for the baseline task.
         */
        Baseline10FixedCost,
        /**
         * The accrual method that defines how the baseline task accrues fixed costs: Accrues when the task starts = 1, 
         * accrues when the task ends = 2, accrues as the task progresses (prorated) = 3.
         */
        Baseline10FixedCostAccrual,
        /**
         * The start date for the baseline task.
         */
        Baseline10Start,
        /**
         * The total person-hours scheduled for the baseline task, in minutes.
         */
        Baseline10Work,
        /**
         * The budget cost for the baseline task.
         */
        Baseline1BudgetCost,
        /**
         * The budget work for the baseline task, in hours.
         */
        Baseline1BudgetWork,
        /**
         * The cost for the baseline task.
         */
        Baseline1Cost,
        /**
         * The duration for the baseline task, in minutes.
         */
        Baseline1Duration,
        /**
         * The finish date for the baseline task.
         */
        Baseline1Finish,
        /**
         * The fixed cost of any non-resource expense for the baseline task.
         */
        Baseline1FixedCost,
        /**
         * The accrual method that defines how the baseline task accrues fixed costs: Accrues when the task starts = 1, 
         * accrues when the task ends = 2, accrues as the task progresses (prorated) = 3.
         */
        Baseline1FixedCostAccrual,
        /**
         * The start date for the baseline task.
         */
        Baseline1Start,
        /**
         * The total person-hours scheduled for the baseline task, in minutes.
         */
        Baseline1Work,
        /**
         * The budget cost for the baseline task.
         */
        Baseline2BudgetCost,
        /**
         * The budget work for the baseline task, in hours.
         */
        Baseline2BudgetWork,
        /**
         * The cost for the baseline task.
         */
        Baseline2Cost,
        /**
         * The duration for the baseline task, in minutes.
         */
        Baseline2Duration,
        /**
         * The finish date for the baseline task.
         */
        Baseline2Finish,
        /**
         * The fixed cost of any non-resource expense for the baseline task.
         */
        Baseline2FixedCost,
        /**
         * The accrual method that defines how the baseline task accrues fixed costs: Accrues when the task starts = 1, 
         * accrues when the task ends = 2, accrues as the task progresses (prorated) = 3.
         */
        Baseline2FixedCostAccrual,
        /**
         * The start date for the baseline task.
         */
        Baseline2Start,
        /**
         * The total person-hours scheduled for the baseline task, in minutes.
         */
        Baseline2Work,
        /**
         * The budget cost for the baseline task.
         */
        Baseline3BudgetCost,
        /**
         * The budget work for the baseline task, in hours.
         */
        Baseline3BudgetWork,
        /**
         * The cost for the baseline task.
         */
        Baseline3Cost,
        /**
         * The duration for the baseline task, in minutes.
         */
        Baseline3Duration,
        /**
         * The finish date for the baseline task.
         */
        Baseline3Finish,
        /**
         * The fixed cost of any non-resource expense for the baseline task.
         */
        Baseline3FixedCost,
        /**
         * The accrual method that defines how the baseline task accrues fixed costs: Accrues when the task starts = 1, 
         * accrues when the task ends = 2, accrues as the task progresses (prorated) = 3.
         */
        Baseline3FixedCostAccrual,
        /**
         * The start date for the baseline task.
         */
        Baseline3Start,
        /**
         * The total person-hours scheduled for the baseline task, in minutes.
         */
        Baseline3Work,
        /**
         * The budget cost for the baseline task.
         */
        Baseline4BudgetCost,
        /**
         * The budget work for the baseline task, in hours.
         */
        Baseline4BudgetWork,
        /**
         * The cost for the baseline task.
         */
        Baseline4Cost,
        /**
         * The duration for the baseline task, in minutes.
         */
        Baseline4Duration,
        /**
         * The finish date for the baseline task.
         */
        Baseline4Finish,
        /**
         * The fixed cost of any non-resource expense for the baseline task.
         */
        Baseline4FixedCost,
        /**
         * The accrual method that defines how the baseline task accrues fixed costs: Accrues when the task starts = 1, 
         * accrues when the task ends = 2, accrues as the task progresses (prorated) = 3.
         */
        Baseline4FixedCostAccrual,
        /**
         * The start date for the baseline task.
         */
        Baseline4Start,
        /**
         * The total person-hours scheduled for the baseline task, in minutes.
         */
        Baseline4Work,
        /**
         * The budget cost for the baseline task.
         */
        Baseline5BudgetCost,
        /**
         * The budget work for the baseline task, in hours.
         */
        Baseline5BudgetWork,
        /**
         * The cost for the baseline task.
         */
        Baseline5Cost,
        /**
         * The duration for the baseline task, in minutes.
         */
        Baseline5Duration,
        /**
         * The finish date for the baseline task.
         */
        Baseline5Finish,
        /**
         * The fixed cost of any non-resource expense for the baseline task.
         */
        Baseline5FixedCost,
        /**
         * The accrual method that defines how the baseline task accrues fixed costs: Accrues when the task starts = 1, 
         * accrues when the task ends = 2, accrues as the task progresses (prorated) = 3.
         */
        Baseline5FixedCostAccrual,
        /**
         * The start date for the baseline task.
         */
        Baseline5Start,
        /**
         * The total person-hours scheduled for the baseline task, in minutes.
         */
        Baseline5Work,
        /**
         * The budget cost for the baseline task.
         */
        Baseline6BudgetCost,
        /**
         * The budget work for the baseline task, in hours.
         */
        Baseline6BudgetWork,
        /**
         * The cost for the baseline task.
         */
        Baseline6Cost,
        /**
         * The duration for the baseline task, in minutes.
         */
        Baseline6Duration,
        /**
         * The finish date for the baseline task.
         */
        Baseline6Finish,
        /**
         * The fixed cost of any non-resource expense for the baseline task.
         */
        Baseline6FixedCost,
        /**
         * The accrual method that defines how the baseline task accrues fixed costs: Accrues when the task starts = 1, 
         * accrues when the task ends = 2, accrues as the task progresses (prorated) = 3.
         */
        Baseline6FixedCostAccrual,
        /**
         * The start date for the baseline task.
         */
        Baseline6Start,
        /**
         * The total person-hours scheduled for the baseline task, in minutes.
         */
        Baseline6Work,
        /**
         * The budget cost for the baseline task.
         */
        Baseline7BudgetCost,
        /**
         * The budget work for the baseline task, in hours.
         */
        Baseline7BudgetWork,
        /**
         * The cost for the baseline task.
         */
        Baseline7Cost,
        /**
         * The duration for the baseline task, in minutes.
         */
        Baseline7Duration,
        /**
         * The finish date for the baseline task.
         */
        Baseline7Finish,
        /**
         * The fixed cost of any non-resource expense for the baseline task.
         */
        Baseline7FixedCost,
        /**
         * The accrual method that defines how the baseline task accrues fixed costs: Accrues when the task starts = 1, 
         * accrues when the task ends = 2, accrues as the task progresses (prorated) = 3.
         */
        Baseline7FixedCostAccrual,
        /**
         * The start date for the baseline task.
         */
        Baseline7Start,
        /**
         * The total person-hours scheduled for the baseline task, in minutes.
         */
        Baseline7Work,
        /**
         * The budget cost for the baseline task.
         */
        Baseline8BudgetCost,
        /**
         * The budget work for the baseline task, in hours.
         */
        Baseline8BudgetWork,
        /**
         * The cost for the baseline task.
         */
        Baseline8Cost,
        /**
         * The duration for the baseline task, in minutes.
         */
        Baseline8Duration,
        /**
         * The finish date for the baseline task.
         */
        Baseline8Finish,
        /**
         * The fixed cost of any non-resource expense for the baseline task.
         */
        Baseline8FixedCost,
        /**
         * The accrual method that defines how the baseline task accrues fixed costs: Accrues when the task starts = 1, 
         * accrues when the task ends = 2, accrues as the task progresses (prorated) = 3.
         */
        Baseline8FixedCostAccrual,
        /**
         * The start date for the baseline task.
         */
        Baseline8Start,
        /**
         * The total person-hours scheduled for the baseline task, in minutes.
         */
        Baseline8Work,
        /**
         * The budget cost for the baseline task.
         */
        Baseline9BudgetCost,
        /**
         * The budget work for the baseline task, in hours.
         */
        Baseline9BudgetWork,
        /**
         * The cost for the baseline task.
         */
        Baseline9Cost,
        /**
         * The duration for the baseline task, in minutes.
         */
        Baseline9Duration,
        /**
         * The finish date for the baseline task.
         */
        Baseline9Finish,
        /**
         * The fixed cost of any non-resource expense for the baseline task.
         */
        Baseline9FixedCost,
        /**
         * The accrual method that defines how the baseline task accrues fixed costs: Accrues when the task starts = 1, 
         * accrues when the task ends = 2, accrues as the task progresses (prorated) = 3.
         */
        Baseline9FixedCostAccrual,
        /**
         * The start date for the baseline task.
         */
        Baseline9Start,
        /**
         * The total person-hours scheduled for the baseline task, in minutes.
         */
        Baseline9Work,
        /**
         * The budget cost for the baseline task.
         */
        BaselineBudgetCost,
        /**
         * The budget work for the baseline task, in hours.
         */
        BaselineBudgetWork,
        /**
         * The cost for the baseline task.
         */
        BaselineCost,
        /**
         * The duration for the baseline task, in minutes.
         */
        BaselineDuration,
        /**
         * The finish date for the baseline task.
         */
        BaselineFinish,
        /**
         * The fixed cost of any non-resource expense for the baseline task.
         */
        BaselineFixedCost,
        /**
         * The accrual method that defines how the baseline task accrues fixed costs: Accrues when the task starts = 1, 
         * accrues when the task ends = 2, accrues as the task progresses (prorated) = 3.
         */
        BaselineFixedCostAccrual,
        /**
         * The start date for the baseline task.
         */
        BaselineStart,
        /**
         * The total person-hours scheduled for the baseline task, in minutes.
         */
        BaselineWork,
        /**
         * The budget cost for the task.
         */
        BudgetCost,
        BudgetFixedCost,
        BudgetFixedWork,
        /**
        * The budget work for the task, in hours.
        */
        BudgetWork,
        /**
         * The GUID of the task calendar.
         */
        TaskCalendarGUID,
        /**
         * A constraint date for the task.
         */
        ConstraintDate,
        /**
         * A constraint type for the task: As Soon As Possible = 0, As Late As Possible = 1, Must Start On = 2, Must Finish On = 3, 
         * Start No Earlier Than = 4, Start No Later Than = 5, Finish No Earlier Than = 6, Finish No Later Than = 7.
         */
        ConstraintType,
        /**
         * A cost field of the task.
         */
        Cost1,
        /**
         * A cost field of the task.
         */
        Cost10,
        /**
         * A cost field of the task.
         */
        Cost2,
        /**
         * A cost field of the task.
         */
        Cost3,
        /**
         * A cost field of the task.
         */
        Cost4,
        /**
         * A cost field of the task.
         */
        Cost5,
        /**
         * A cost field of the task.
         */
        Cost6,
        /**
         * A cost field of the task.
         */
        Cost7,
        /**
         * A cost field of the task.
         */
        Cost8,
        /**
         * A cost field of the task.
         */
        Cost9,
        /**
         * A date field of the task.
         */
        Date1,
        /**
         * A date field of the task.
         */
        Date10,
        /**
         * A date field of the task.
         */
        Date2,
        /**
         * A date field of the task.
         */
        Date3,
        /**
         * A date field of the task.
         */
        Date4,
        /**
         * A date field of the task.
         */
        Date5,
        /**
         * A date field of the task.
         */
        Date6,
        /**
         * A date field of the task.
         */
        Date7,
        /**
         * A date field of the task.
         */
        Date8,
        /**
         * A date field of the task.
         */
        Date9,
        /**
         * The deadline for a task.
         */
        Deadline,
        /**
         * A duration field of the task.
         */
        Duration1,
        /**
         * A duration field of the task.
         */
        Duration10,
        /**
         * A duration field of the task.
         */
        Duration2,
        /**
         * A duration field of the task.
         */
        Duration3,
        /**
         * A duration field of the task.
         */
        Duration4,
        /**
         * A duration field of the task.
         */
        Duration5,
        /**
         * A duration field of the task.
         */
        Duration6,
        /**
         * A duration field of the task.
         */
        Duration7,
        /**
         * A duration field of the task.
         */
        Duration8,
        /**
         * A duration field of the task.
         */
        Duration9,
        /**
         * A duration field of the task.
         */
        Duration,
        /**
         * The method for calculating earned value for the task.
         */
        EarnedValueMethod,
        /**
         * The duration between the Early Finish and Late Finish dates for the task, in minutes.
         */
        FinishSlack,
        /**
         * The fixed cost for the task.
         */
        FixedCost,
        /**
         * The accrual method that defines how the baseline task accrues fixed costs: Accrues when the task starts = 1, 
         * accrues when the task ends = 2, accrues as the task progresses (prorated) = 3.
         */
        FixedCostAccrual,
        /**
         * A Boolean flag field for the task.
         */
        Flag10,
        /**
         * A Boolean flag field for the task.
         */
        Flag1,
        /**
         * A Boolean flag field for the task.
         */
        Flag11,
        /**
         * A Boolean flag field for the task.
         */
        Flag12,
        /**
         * A Boolean flag field for the task.
         */
        Flag13,
        /**
         * A Boolean flag field for the task.
         */
        Flag14,
        /**
         * A Boolean flag field for the task.
         */
        Flag15,
        /**
         * A Boolean flag field for the task.
         */
        Flag16,
        /**
         * A Boolean flag field for the task.
         */
        Flag17,
        /**
         * A Boolean flag field for the task.
         */
        Flag18,
        /**
         * A Boolean flag field for the task.
         */
        Flag19,
        /**
         * A Boolean flag field for the task.
         */
        Flag2,
        /**
         * A Boolean flag field for the task.
         */
        Flag20,
        /**
         * A Boolean flag field for the task.
         */
        Flag3,
        /**
         * A Boolean flag field for the task.
         */
        Flag4,
        /**
         * A Boolean flag field for the task.
         */
        Flag5,
        /**
         * A Boolean flag field for the task.
         */
        Flag6,
        /**
         * A Boolean flag field for the task.
         */
        Flag7,
        /**
         * A Boolean flag field for the task.
         */
        Flag8,
        /**
         * A Boolean flag field for the task.
         */
        Flag9,
        /**
         * The amount of time that the task can be delayed without delaying its successor tasks.
         */
        FreeSlack,
        /**
         * Indicates whether the task has rollup subtasks.
         */
        HasRollupSubTasks,
        /**
         * The index of the selected task. After the project summary task, the index of the first task in a project is 1.
         */
        ID,
        /**
         * The name of the task.
         */
        Name,
        /**
         * The text value of the notes regarding the task.
         */
        Notes,
        /**
         * A number field for the task.
         */
        Number1,
        /**
         * A number field for the task.
         */
        Number10,
        /**
         * A number field for the task.
         */
        Number11,
        /**
         * A number field for the task.
         */
        Number12,
        /**
         * A number field for the task.
         */
        Number13,
        /**
         * A number field for the task.
         */
        Number14,
        /**
         * A number field for the task.
         */
        Number15,
        /**
         * A number field for the task.
         */
        Number16,
        /**
         * A number field for the task.
         */
        Number17,
        /**
         * A number field for the task.
         */
        Number18,
        /**
         * A number field for the task.
         */
        Number19,
        /**
         * A number field for the task.
         */
        Number2,
        /**
         * A number field for the task.
         */
        Number20,
        /**
         * A number field for the task.
         */
        Number3,
        /**
         * A number field for the task.
         */
        Number4,
        /**
         * A number field for the task.
         */
        Number5,
        /**
         * A number field for the task.
         */
        Number6,
        /**
         * A number field for the task.
         */
        Number7,
        /**
         * A number field for the task.
         */
        Number8,
        /**
         * A number field for the task.
         */
        Number9,
        /**
         * The scheduled (as opposed to actual) duration of the task.
         */
        ScheduledDuration,
        /**
         * The scheduled (as opposed to actual) finish date of the task.
         */
        ScheduledFinish,
        /**
         * The scheduled (as opposed to actual) start date of the task.
         */
        ScheduledStart,
        /**
         * The level of the task in the outline hierarchy.
         */
        OutlineLevel,
        /**
         * The overtime cost for the task.
         */
        OvertimeCost,
        /**
         * The overtime work for the task.
         */
        OvertimeWork,
        /**
         * The percent complete status of the task.
         */
        PercentComplete,
        /**
         * The percentage of work completed for the task.
         */
        PercentWorkComplete,
        /**
         * The IDs of the task's predecessors.
         */
        Predecessors,
        /**
         * The finish date of a task before leveling occurred.
         */
        PreleveledFinish,
        /**
         * The start date of a task before leveling occurred.
         */
        PreleveledStart,
        /**
         * The priority of the task, with values from 0 (low) to 1000 (high). The default priority value is 500.
         */
        Priority,
        /**
         * Indicates whether the task is active.
         */
        Active,
        /**
         * Indicates whether the task is on the critical path.
         */
        Critical,
        /**
         * Indicates whether the task is a milestone.
         */
        Milestone,
        /**
         * Indicates whether any assignments for a task are overallocated.
         */
        Overallocated,
        /**
         * Indicates whether subtask information is rolled up to the summary task bar.
         */
        IsRollup,
        /**
         * Indicates whether the task is a summary task.
         */
        Summary,
        /**
         * The amount of regular work for the task.
         */
        RegularWork,
        /**
         * The remaining cost for the task.
         */
        RemainingCost,
        /**
         * The remaining duration for the task, in minutes.
         */
        RemainingDuration,
        /**
         * The remaining overtime cost for the task.
         */
        RemainingOvertimeCost,
        /**
         * The remaining work for the task, in minutes.
         */
        RemainingWork,
        /**
         * The names of the resources assigned to a task.
         */
        ResourceNames,
        /**
         * The total cost of the task.
         */
        Cost,
        /**
         * The finish date of the task.
         */
        Finish,
        /**
         * The start date of the task.
         */
        Start,
        /**
         * The total person-hours scheduled for the task, in minutes.
         */
        Work,
        /**
         * The duration between the Early Start and Late Start dates for the task.
         */
        StartSlack,
        /**
         * The status of the task: Complete = 0, on schedule = 1, late = 2, future task = 3, status not available = 4.
         */
        Status,
        /**
         * The IDs of the task's successors.
         */
        Successors,
        /**
         * The enterprise resource responsible for accepting or rejecting assignment progress updates for the task.
         */
        StatusManager,
        /**
         * The total slack time for the task, in minutes.
         */
        TotalSlack,
        /**
         * The GUID of the task.
         */
        TaskGUID,
        /**
         * The way the task is calculated: Fixed units = 0, fixed duration = 1, fixed work = 2.
         */
        Type,
        /**
         * The work breakdown structure code of the task.
         */
        WBS,
        /**
         * The work breakdown structure codes of the task predecessors, separated by the list separator.
         */
        WBSPREDECESSORS,
        /**
         * The work breakdown structure codes of the task successors, separated by the list separator.
         */
        WBSSUCCESSORS,
        /**
         * The ID of the task in a SharePoint list, for a project that is synchronized with a SharePoint tasks list.
         */
        WSSID
    }
    /**
     * Specifies the types of views that the {@link Office.Document | Document}.getSelectedViewAsync method can recognize.
     *
     * @remarks
     * The {@link Office.Document | Document}.getSelectedViewAsync method returns the ProjectViewTypes constant value and name that corresponds to the 
     * active view.
     */
    enum ProjectViewTypes {
        /**
         * The Gantt chart view.
         */
        Gantt,
        /**
         * The Network Diagram view.
         */
        NetworkDiagram,
        /**
         * The Task Diagram view.
         */
        TaskDiagram,
        /**
         * The Task form view.
         */
        TaskForm,
        /**
         * The Task Sheet view.
         */
        TaskSheet,
        /**
         * The Resource Form view.
         */
        ResourceForm,
        /**
         * The Resource Sheet view.
         */
        ResourceSheet,
        /**
         * The Resource Graph view.
         */
        ResourceGraph,
        /**
         * The Team Planner view.
         */
        TeamPlanner,
        /**
         * The Task Details view.
         */
        TaskDetails,
        /**
         * The Task Name Form view.
         */
        TaskNameForm,
        /**
         * The Resource Names view.
         */
        ResourceNames,
        /**
         * The Calendar view.
         */
        Calendar,
        /**
         * The Task Usage view.
         */
        TaskUsage,
        /**
         * The Resource Usage view.
         */
        ResourceUsage,
        /**
         * The Timeline view.
         */
        Timeline
    }
}


////////////////////////////////////////////////////////////////
///////////////////// End Office namespace /////////////////////
////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////
////////////////////// Begin Exchange APIs /////////////////////
////////////////////////////////////////////////////////////////

declare namespace Office {
    namespace MailboxEnums {
        /**
         * Specifies the formatting that applies to an attachment's content.
         * 
         * [Api set: Mailbox Preview]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @beta
         */
        enum AttachmentContentFormat {
            /**
             * The content of the attachment is returned as a base64-encoded string.
             */
            Base64 = "base64",

            /**
             * The content of the attachment is returned as a string representing a URL.
             */
            Url = "url",

            /**
             * The content of the attachment is returned as a string representing an .eml formatted file.
             */
            Eml = "eml",

            /**
             * The content of the attachment is returned as a string representing an .icalendar formatted file.
             */
            ICalendar = "iCalendar"
        }
        /**
         * Specifies whether an attachment was added to or removed from an item.
         * 
         * [Api set: Mailbox Preview]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @beta
         */
        enum AttachmentStatus {
            /**
             * An attachment was added to the item.
             */
            Added = "added",

            /**
             * An attachment was removed from the item.
             */
            Removed = "removed"
        }
        /**
         * Specifies an attachment's type.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        enum AttachmentType {
            /**
             * The attachment is a file
             */
            File = "file",
            /**
             * The attachment is an Exchange item
             */
            Item = "item",
            /**
             * The attachment is stored in a cloud location, such as OneDrive. The id property of the attachment contains a URL to the file.
             */
            Cloud = "cloud"
        }
        /**
         * Specifies the day of week or type of day.
         *
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        enum Days {
            /**
             * Monday
             */
            Mon = "mon",
            /**
             * Tuesday
             */
            Tue = "tue",
            /**
             * Wednesday
             */
            Wed = "wed",
            /**
             * Thursday
             */
            Thu = "thu",
            /**
             * Friday
             */
            Fri = "fri",
            /**
             * Saturday
             */
            Sat = "sat",
            /**
             * Sunday
             */
            Sun = "sun",
            /**
             * Week day (excludes weekend days): 'Mon', 'Tue', 'Wed', 'Thu', and 'Fri'.
             */
            Weekday = "weekday",
            /**
             * Weekend day: 'Sat' and 'Sun'.
             */
            WeekendDay = "weekendDay",
            /**
             * Day of week.
             */
            Day = "day"
        }
        /**
         * This bit mask represents a delegate's permissions on a shared folder.
         *
         * [Api set: Mailbox Preview]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @beta
         */
        enum DelegatePermissions {
            /**
             * Delegate has permission to read items.
             */
            Read = 1,
            /**
             * Delegate has permission to create and write items.
             */
            Write = 2,
            /**
             * Delegate has permission to delete only the items they created.
             */
            DeleteOwn = 4,
            /**
             * Delegate has permission to delete any items.
             */
            DeleteAll = 8,
            /**
             * Delegate has permission to edit only they items they created.
             */
            EditOwn = 16,
            /**
             * Delegate has permission to edit any items.
             */
            EditAll = 32
        }
        /**
         * Specifies an entity's type.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        enum EntityType {
            /**
             * Specifies that the entity is a meeting suggestion.
             */
            MeetingSuggestion = "meetingSuggestion",
            /**
             * Specifies that the entity is a task suggestion.
             */
            TaskSuggestion = "taskSuggestion",
            /**
             * Specifies that the entity is a postal address.
             */
            Address = "address",
            /**
             * Specifies that the entity is an SMTP email address.
             */
            EmailAddress = "emailAddress",
            /**
             * Specifies that the entity is an Internet URL.
             */
            Url = "url",
            /**
             * Specifies that the entity is a US phone number.
             */
            PhoneNumber = "phoneNumber",
            /**
             * Specifies that the entity is a contact.
             */
            Contact = "contact"
        }
        /**
         * Specifies the notification message type for an appointment or message.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        enum ItemNotificationMessageType {
            /**
             * The notificationMessage is a progress indicator.
             */
            ProgressIndicator = "progressIndicator",
            /**
             * The notificationMessage is an informational message.
             */
            InformationalMessage = "informationalMessage",
            /**
             * The notificationMessage is an error message.
             */
            ErrorMessage = "errorMessage"
        }
        /**
         * Specifies an item's type.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or 
         */
        enum ItemType {
            /**
             * An email, meeting request, meeting response, or meeting cancellation.
             */
            Message = "message",
            /**
             * An appointment item.
             */
            Appointment = "appointment"
        }
        /**
         * Specifies an appointment location's type.
         * 
         * [Api set: Mailbox Preview]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @beta
         */
        enum LocationType {
            /**
             * A custom location.
             */
            Custom = "custom",
            /**
             * A conference room or similar resource.
             */
            Room = "room"
        }
        /**
         * Specifies the month.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        enum Month {
            /**
             * January
             */
            Jan = "jan",
            /**
             * February
             */
            Feb = "feb",
            /**
             * March
             */
            Mar = "mar",
            /**
             * April
             */
            Apr = "apr",
            /**
             * May
             */
            May = "may",
            /**
             * June
             */
            Jun = "jun",
            /**
             * July
             */
            Jul = "jul",
            /**
             * August
             */
            Aug = "aug",
            /**
             * September
             */
            Sep = "sep",
            /**
             * October
             */
            Oct = "oct",
            /**
             * November
             */
            Nov = "nov",
            /**
             * December
             */
            Dec = "dec"
        }
        /**
         * Represents the current view of Outlook on the web.
         */
        enum OWAView {
            /**
             * One column view. Displayed when the screen is narrow. Outlook on the web uses this single-column layout on the entire screen of a smartphone.
             */
            OneColumn = "OneColumn",
            /**
             * Two column view. Displayed when the screen is wider. Outlook on the web uses this view on most tablets.
             */
            TwoColumns = "TwoColumns",
            /**
             Three column view. Displayed when the screen is wide. For example, Outlook on the web uses this view in a full screen window on a desktop 
             computer.
             */
            ThreeColumns = "ThreeColumns"
        }
        /**
         * Specifies the type of recipient for an appointment.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        enum RecipientType {
            /**
             * Specifies that the recipient is a distribution list containing a list of email addresses.
             */
            DistributionList = "distributionList",
            /**
             * Specifies that the recipient is an SMTP email address that is on the Exchange server.
             */
            User = "user",
            /**
             * Specifies that the recipient is an SMTP email address that is not on the Exchange server.
             */
            ExternalUser = "externalUser",
            /**
             * Specifies that the recipient is not one of the other recipient types.
             */
            Other = "other"
        }
        /**
         * Specifies the time zone applied to the recurrence.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        enum RecurrenceTimeZone {
            /**
             * Afghanistan Standard Time
             */
            AfghanistanStandardTime = "Afghanistan Standard Time",
            /**
             * Alaskan Standard Time
             */
            AlaskanStandardTime = "Alaskan Standard Time",
            /**
             * Aleutian Standard Time
             */
            AleutianStandardTime = "Aleutian Standard Time",
            /**
             * Altai Standard Time
             */
            AltaiStandardTime = "Altai Standard Time",
            /**
             * Arab Standard Time
             */
            ArabStandardTime = "Arab Standard Time",
            /**
             * Arabian Standard Time
             */
            ArabianStandardTime = "Arabian Standard Time",
            /**
             * Arabic Standard Time
             */
            ArabicStandardTime = "Arabic Standard Time",
            /**
             * Argentina Standard Time
             */
            ArgentinaStandardTime = "Argentina Standard Time",
            /**
             * Astrakhan Standard Time
             */
            AstrakhanStandardTime = "Astrakhan Standard Time",
            /**
             * Atlantic Standard Time
             */
            AtlanticStandardTime = "Atlantic Standard Time",
            /**
             * Australia Central Standard Time
             */
            AUSCentralStandardTime = "AUS Central Standard Time",
            /**
             * Australia Central West Standard Time
             */
            AusCentralW_StandardTime = "Aus Central W. Standard Time",
            /**
             * AUS Eastern Standard Time
             */
            AUSEasternStandardTime = "AUS Eastern Standard Time",
            /**
             * Azerbaijan Standard Time
             */
            AzerbaijanStandardTime = "Azerbaijan Standard Time",
            /**
             * Azores Standard Time
             */
            AzoresStandardTime = "Azores Standard Time",
            /**
             * Bahia Standard Time
             */
            BahiaStandardTime = "Bahia Standard Time",
            /**
             * Bangladesh Standard Time
             */
            BangladeshStandardTime = "Bangladesh Standard Time",
            /**
             * Belarus Standard Time
             */
            BelarusStandardTime = "Belarus Standard Time",
            /**
             * Bougainville Standard Time
             */
            BougainvilleStandardTime = "Bougainville Standard Time",
            /**
             * Canada Central Standard Time
             */
            CanadaCentralStandardTime = "Canada Central Standard Time",
            /**
             * Cape Verde Standard Time
             */
            CapeVerdeStandardTime = "Cape Verde Standard Time",
            /**
             * Caucasus Standard Time
             */
            CaucasusStandardTime = "Caucasus Standard Time",
            /**
             * Central Australia Standard Time
             */
            CenAustraliaStandardTime = "Cen. Australia Standard Time",
            /**
             * Central America Standard Time
             */
            CentralAmericaStandardTime = "Central America Standard Time",
            /**
             * Central Asia Standard Time
             */
            CentralAsiaStandardTime = "Central Asia Standard Time",
            /**
             * Central Brazilian Standard Time
             */
            CentralBrazilianStandardTime = "Central Brazilian Standard Time",
            /**
             * Central Europe Standard Time
             */
            CentralEuropeStandardTime = "Central Europe Standard Time",
            /**
             * Central European Standard Time
             */
            CentralEuropeanStandardTime = "Central European Standard Time",
            /**
             * Central Pacific Standard Time
             */
            CentralPacificStandardTime = "Central Pacific Standard Time",
            /**
             * Central Standard Time
             */
            CentralStandardTime = "Central Standard Time",
            /**
             * Central Standard Time (Mexico)
             */
            CentralStandardTime_Mexico = "Central Standard Time (Mexico)",
            /**
             * Chatham Islands Standard Time
             */
            ChathamIslandsStandardTime = "Chatham Islands Standard Time",
            /**
             * China Standard Time
             */
            ChinaStandardTime = "China Standard Time",
            /**
             * Cuba Standard Time
             */
            CubaStandardTime = "Cuba Standard Time",
            /**
             * Dateline Standard Time
             */
            DatelineStandardTime = "Dateline Standard Time",
            /**
             * East Africa Standard Time
             */
            E_AfricaStandardTime = "E. Africa Standard Time",
            /**
             * East Australia Standard Time
             */
            E_AustraliaStandardTime = "E. Australia Standard Time",
            /**
             * East Europe Standard Time
             */
            E_EuropeStandardTime = "E. Europe Standard Time",
            /**
             * East South America Standard Time
             */
            E_SouthAmericaStandardTime = "E. South America Standard Time",
            /**
             * Easter Island Standard Time
             */
            EasterIslandStandardTime = "Easter Island Standard Time",
            /**
             * Eastern Standard Time
             */
            EasternStandardTime = "Eastern Standard Time",
            /**
             * Eastern Standard Time (Mexico)
             */
            EasternStandardTime_Mexico = "Eastern Standard Time (Mexico)",
            /**
             * Egypt Standard Time
             */
            EgyptStandardTime = "Egypt Standard Time",
            /**
             * Ekaterinburg Standard Time
             */
            EkaterinburgStandardTime = "Ekaterinburg Standard Time",
            /**
             * Fiji Standard Time
             */
            FijiStandardTime = "Fiji Standard Time",
            /**
             * FLE Standard Time
             */
            FLEStandardTime = "FLE Standard Time",
            /**
             * Georgian Standard Time
             */
            GeorgianStandardTime = "Georgian Standard Time",
            /**
             * GMT Standard Time
             */
            GMTStandardTime = "GMT Standard Time",
            /**
             * Greenland Standard Time
             */
            GreenlandStandardTime = "Greenland Standard Time",
            /**
             * Greenwich Standard Time
             */
            GreenwichStandardTime = "Greenwich Standard Time",
            /**
             * GTB Standard Time
             */
            GTBStandardTime = "GTB Standard Time",
            /**
             * Haiti Standard Time
             */
            HaitiStandardTime = "Haiti Standard Time",
            /**
             * Hawaiian Standard Time
             */
            HawaiianStandardTime = "Hawaiian Standard Time",
            /**
             * India Standard Time
             */
            IndiaStandardTime = "India Standard Time",
            /**
             * Iran Standard Time
             */
            IranStandardTime = "Iran Standard Time",
            /**
             * Israel Standard Time
             */
            IsraelStandardTime = "Israel Standard Time",
            /**
             * Jordan Standard Time
             */
            JordanStandardTime = "Jordan Standard Time",
            /**
             * Kaliningrad Standard Time
             */
            KaliningradStandardTime = "Kaliningrad Standard Time",
            /**
             * Kamchatka Standard Time
             */
            KamchatkaStandardTime = "Kamchatka Standard Time",
            /**
             * Korea Standard Time
             */
            KoreaStandardTime = "Korea Standard Time",
            /**
             * Libya Standard Time
             */
            LibyaStandardTime = "Libya Standard Time",
            /**
             * Line Islands Standard Time
             */
            LineIslandsStandardTime = "Line Islands Standard Time",
            /**
             * Lord Howe Standard Time
             */
            LordHoweStandardTime = "Lord Howe Standard Time",
            /**
             * Magadan Standard Time
             */
            MagadanStandardTime = "Magadan Standard Time",
            /**
             * Magallanes Standard Time
             */
            MagallanesStandardTime = "Magallanes Standard Time",
            /**
             * Marquesas Standard Time
             */
            MarquesasStandardTime = "Marquesas Standard Time",
            /**
             * Mauritius Standard Time
             */
            MauritiusStandardTime = "Mauritius Standard Time",
            /**
             * Mid-Atlantic Standard Time
             */
            MidAtlanticStandardTime = "Mid-Atlantic Standard Time",
            /**
             * Middle East Standard Time
             */
            MiddleEastStandardTime = "Middle East Standard Time",
            /**
             * Montevideo Standard Time
             */
            MontevideoStandardTime = "Montevideo Standard Time",
            /**
             * Morocco Standard Time
             */
            MoroccoStandardTime = "Morocco Standard Time",
            /**
             * Mountain Standard Time
             */
            MountainStandardTime = "Mountain Standard Time",
            /**
             * Mountain Standard Time (Mexico)
             */
            MountainStandardTime_Mexico = "Mountain Standard Time (Mexico)",
            /**
             * Myanmar Standard Time
             */
            MyanmarStandardTime = "Myanmar Standard Time",
            /**
             * North Central Asia Standard Time
             */
            N_CentralAsiaStandardTime = "N. Central Asia Standard Time",
            /**
             * Namibia Standard Time
             */
            NamibiaStandardTime = "Namibia Standard Time",
            /**
             * Nepal Standard Time
             */
            NepalStandardTime = "Nepal Standard Time",
            /**
             * New Zealand Standard Time
             */
            NewZealandStandardTime = "New Zealand Standard Time",
            /**
             * Newfoundland Standard Time
             */
            NewfoundlandStandardTime = "Newfoundland Standard Time",
            /**
             * Norfolk Standard Time
             */
            NorfolkStandardTime = "Norfolk Standard Time",
            /**
             * North Asia East Standard Time
             */
            NorthAsiaEastStandardTime = "North Asia East Standard Time",
            /**
             * North Asia Standard Time
             */
            NorthAsiaStandardTime = "North Asia Standard Time",
            /**
             * North Korea Standard Time
             */
            NorthKoreaStandardTime = "North Korea Standard Time",
            /**
             * Omsk Standard Time
             */
            OmskStandardTime = "Omsk Standard Time",
            /**
             * Pacific SA Standard Time
             */
            PacificSAStandardTime = "Pacific SA Standard Time",
            /**
             * Pacific Standard Time
             */
            PacificStandardTime = "Pacific Standard Time",
            /**
             * Pacific Standard Time (Mexico)
             */
            PacificStandardTimeMexico = "Pacific Standard Time (Mexico)",
            /**
             * Pakistan Standard Time
             */
            PakistanStandardTime = "Pakistan Standard Time",
            /**
             * Paraguay Standard Time
             */
            ParaguayStandardTime = "Paraguay Standard Time",
            /**
             * Romance Standard Time
             */
            RomanceStandardTime = "Romance Standard Time",
            /**
             * Russia Time Zone 10
             */
            RussiaTimeZone10 = "Russia Time Zone 10",
            /**
             * Russia Time Zone 11
             */
            RussiaTimeZone11 = "Russia Time Zone 11",
            /**
             * Russia Time Zone 3
             */
            RussiaTimeZone3 = "Russia Time Zone 3",
            /**
             * Russian Standard Time
             */
            RussianStandardTime = "Russian Standard Time",
            /**
             * SA Eastern Standard Time
             */
            SAEasternStandardTime = "SA Eastern Standard Time",
            /**
             * SA Pacific Standard Time
             */
            SAPacificStandardTime = "SA Pacific Standard Time",
            /**
             * SA Western Standard Time
             */
            SAWesternStandardTime = "SA Western Standard Time",
            /**
             * Saint Pierre Standard Time
             */
            SaintPierreStandardTime = "Saint Pierre Standard Time",
            /**
             * Sakhalin Standard Time
             */
            SakhalinStandardTime = "Sakhalin Standard Time",
            /**
             * Samoa Standard Time
             */
            SamoaStandardTime = "Samoa Standard Time",
            /**
             * Saratov Standard Time
             */
            SaratovStandardTime = "Saratov Standard Time",
            /**
             * Southeast Asia Standard Time
             */
            SEAsiaStandardTime = "SE Asia Standard Time",
            /**
             * Singapore Standard Time
             */
            SingaporeStandardTime = "Singapore Standard Time",
            /**
             * South Africa Standard Time
             */
            SouthAfricaStandardTime = "South Africa Standard Time",
            /**
             * Sri Lanka Standard Time
             */
            SriLankaStandardTime = "Sri Lanka Standard Time",
            /**
             * Sudan Standard Time
             */
            SudanStandardTime = "Sudan Standard Time",
            /**
             * Syria Standard Time
             */
            SyriaStandardTime = "Syria Standard Time",
            /**
             * Taipei Standard Time
             */
            TaipeiStandardTime = "Taipei Standard Time",
            /**
             * Tasmania Standard Time
             */
            TasmaniaStandardTime = "Tasmania Standard Time",
            /**
             * Tocantins Standard Time
             */
            TocantinsStandardTime = "Tocantins Standard Time",
            /**
             * Tokyo Standard Time
             */
            TokyoStandardTime = "Tokyo Standard Time",
            /**
             * Tomsk Standard Time
             */
            TomskStandardTime = "Tomsk Standard Time",
            /**
             * Tonga Standard Time
             */
            TongaStandardTime = "Tonga Standard Time",
            /**
             * Transbaikal Standard Time
             */
            TransbaikalStandardTime = "Transbaikal Standard Time",
            /**
             * Turkey Standard Time
             */
            TurkeyStandardTime = "Turkey Standard Time",
            /**
             * Turks And Caicos Standard Time
             */
            TurksAndCaicosStandardTime = "Turks And Caicos Standard Time",
            /**
             * Ulaanbaatar Standard Time
             */
            UlaanbaatarStandardTime = "Ulaanbaatar Standard Time",
            /**
             * United States Eastern Standard Time
             */
            USEasternStandardTime = "US Eastern Standard Time",
            /**
             * United States Mountain Standard Time
             */
            USMountainStandardTime = "US Mountain Standard Time",
            /**
             * Coordinated Universal Time (UTC)
             */
            UTC = "UTC",
            /**
             * Coordinated Universal Time (UTC) + 12 hours
             */
            UTCPLUS12 = "UTC+12",
            /**
             * Coordinated Universal Time (UTC) + 13 hours
             */
            UTCPLUS13 = "UTC+13",
            /**
             * Coordinated Universal Time (UTC) - 2 hours
             */
            UTCMINUS02 = "UTC-02",
            /**
             * Coordinated Universal Time (UTC) - 8 hours
             */
            UTCMINUS08 = "UTC-08",
            /**
             * Coordinated Universal Time (UTC) - 9 hours
             */
            UTCMINUS09 = "UTC-09",
            /**
             * Coordinated Universal Time (UTC) - 11 hours
             */
            UTCMINUS11 = "UTC-11",
            /**
             * Venezuela Standard Time
             */
            VenezuelaStandardTime = "Venezuela Standard Time",
            /**
             * Vladivostok Standard Time
             */
            VladivostokStandardTime = "Vladivostok Standard Time",
            /**
             * West Australia Standard Time
             */
            W_AustraliaStandardTime = "W. Australia Standard Time",
            /**
             * West Central Africa Standard Time
             */
            W_CentralAfricaStandardTime = "W. Central Africa Standard Time",
            /**
             * West Europe Standard Time
             */
            W_EuropeStandardTime = "W. Europe Standard Time",
            /**
             * West Mongolia Standard Time
             */
            W_MongoliaStandardTime = "W. Mongolia Standard Time",
            /**
             * West Asia Standard Time
             */
            WestAsiaStandardTime = "West Asia Standard Time",
            /**
             * West Bank Standard Time
             */
            WestBankStandardTime = "West Bank Standard Time",
            /**
             * West Pacific Standard Time
             */
            WestPacificStandardTime = "West Pacific Standard Time",
            /**
             * Yakutsk Standard Time
             */
            YakutskStandardTime = "Yakutsk Standard Time"
            
        }

        /**
         * Specifies the type of recurrence.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        enum RecurrenceType {
            /**
             * Daily.
             */
            Daily = "daily",
            /**
             * Weekday.
             */
            Weekday = "weekday",
            /**
             * Weekly.
             */
            Weekly = "weekly",
            /**
             * Monthly.
             */
            Monthly = "monthly",
            /**
             * Yearly.
             */
            Yearly = "yearly"
        }
        /**  
         * Specifies the type of response to a meeting invitation.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        enum ResponseType {
            /**
             * There has been no response from the attendee.
             */
            None = "none",
            /**
             * The attendee is the meeting organizer.
             */
            Organizer = "organizer",
            /**
             * The meeting request was tentatively accepted by the attendee.
             */
            Tentative = "tentative",
            /**
             * The meeting request was accepted by the attendee.
             */
            Accepted = "accepted",
            /**
             * The meeting request was declined by the attendee.
             */
            Declined = "declined"
        }
        /**
         * Specifies the version of the REST API that corresponds to a REST-formatted item ID.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        enum RestVersion {
            /**
             * Version 1.0.
             */
            v1_0 = "v1.0",
            /**
             * Version 2.0.
             */
            v2_0 = "v2.0",
            /**
             * Beta.
             */
            Beta = "beta"
        }
        /**
         * Specifies the week of the month.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        enum WeekNumber {
            /**
             * First week of the month.
             */
            First = "first",
            /**
             * Second week of the month.
             */
            Second = "second",
            /**
             * Third week of the month.
             */
            Third = "third",
            /**
             * Fourth week of the month.
             */
            Fourth = "fourth",
            /**
             * Last week of the month.
             */
            Last = "last"
        }
    }
    interface CoercionTypeOptions {
        coercionType?: Office.CoercionType | string;
    }
    enum SourceProperty {
        /**
         * The source of the data is from the body of the message.
         */
        Body,
        /**
         * The source of the data is from the subject of the message.
         */
        Subject
    }
    /**
     * The AppointmentForm namespace is used to access the currently selected appointment.
     *
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     */
    interface AppointmentForm {
        /**
         * Gets an object that provides methods for manipulating the body of an item.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        body: string;
        /**
         * Gets or sets the date and time that the appointment is to end.
         *
         * The end property is expressed as a Coordinated Universal Time (UTC) date and time value. You can use the convertToLocalClientTime method to 
         * convert the end property value to the client's local date and time.
         *
         * *Read mode*
         *
         * The end property returns a Date object.
         *
         * *Compose mode*
         *
         * The end property returns a Time object.
         *
         * When you use the Time.setAsync method to set the end time, you should use the convertToUtcClientTime method to convert the local time on 
         * the client to UTC for the server.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        end: Date;
        /**
        * Gets or sets the location of an appointment.
        *
        * *Read mode*
        *
        * The location property returns a string that contains the location of the appointment.
        *
        * *Compose mode*
        *
        * The location property returns a Location object that provides methods that are used to get and set the location of the appointment.
        *
        * [Api set: Mailbox 1.0]
        *
        * @remarks
        *
        * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
        * 
        * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
        */
        location: string;
        /**
        * Provides access to the optional attendees of an event. The type of object and level of access depends on the mode of the current item.
        *
        * *Read mode*
        *
        * The optionalAttendees property returns an array that contains an EmailAddressDetails object for each optional attendee to the meeting.
        *
        * *Compose mode*
        *
        * The optionalAttendees property returns a Recipients object that provides methods to get or update the optional attendees for a meeting.
        *
        * [Api set: Mailbox 1.0]
        *
        * @remarks
        *
        * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
        * 
        * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
        */
       optionalAttendees: string[] | EmailAddressDetails[];
        /**
        * Provides access to the resources of an event. Returns an array of strings containing the resources required for the appointment.
        *
        * [Api set: Mailbox 1.0]
        *
        * @remarks
        *
        * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
        * 
        * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
        */
       resources: string[];
        /**
         * Provides access to the required attendees of an event. The type of object and level of access depends on the mode of the current item.
         *
         * *Read mode*
         *
         * The requiredAttendees property returns an array that contains an EmailAddressDetails object for each required attendee to the meeting.
         *
         * *Compose mode*
         *
         * The requiredAttendees property returns a Recipients object that provides methods to get or update the required attendees for a meeting.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        requiredAttendees: string[] | EmailAddressDetails[];
        /**
         * Gets or sets the date and time that the appointment is to begin.
         *
         * The start property is expressed as a Coordinated Universal Time (UTC) date and time value. You can use the convertToLocalClientTime method 
         * to convert the value to the client's local date and time.
         *
         * *Read mode*
         *
         * The start property returns a Date object.
         *
         * *Compose mode*
         *
         * The start property returns a Time object.
         *
         * When you use the Time.setAsync method to set the start time, you should use the convertToUtcClientTime method to convert the local time on 
         * the client to UTC for the server.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        start: Date;
        /**
         * Gets or sets the description that appears in the subject field of an item.
         *
         * The subject property gets or sets the entire subject of the item, as sent by the email server.
         *
         * *Read mode*
         *
         * The subject property returns a string. Use the normalizedSubject property to get the subject minus any leading prefixes such as RE: and FW:.
         *
         * *Compose mode*
         *
         * The subject property returns a Subject object that provides methods to get and set the subject.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        subject: string;
    }
    /**
     * Represents the content of an attachment on a message or appointment item.
     *
     * [Api set: Mailbox Preview]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     * 
     * @beta
     */
    interface AttachmentContent {
        /**
         * The content of an attachment as a string.
         */
        content: string;
        /**
         * The string format to use for an attachment's content.
         * 
         * For file attachments, the formatting is a base64-encoded string.
         * 
         * For item attachments that represent messages, the formatting is a string representing an .eml formatted file.
         * 
         * For item attachments that represent calendar items, the formatting is a string representing an .icalendar file.
         * 
         * For cloud attachments, the formatting is a URL string.
         */
        format: MailboxEnums.AttachmentContentFormat | string;
    }
    /**
     * Represents an attachment on an item from the server. Read mode only.
     *
     * An array of **AttachmentDetails** objects is returned as the attachments property of an appointment or message item.
     *
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
     */
    interface AttachmentDetails {
        /**
         * Gets a value that indicates the type of an attachment.
         */
        attachmentType: MailboxEnums.AttachmentType | string;
        /**
         * Gets the MIME content type of the attachment.
         */
        contentType: string;
        /**
         * Gets the Exchange attachment ID of the attachment.
         */
        id: string;
        /**
         * Gets a value that indicates whether the attachment should be displayed in the body of the item.
         */
        isInline: boolean;
        /**
         * Gets the name of the attachment.
         */
        name: string;
        /**
         * Gets the size of the attachment in bytes.
         */
        size: number;
        /**
         * Gets the url of the attachment if its type is `MailboxEnums.AttachmentType.Cloud`.
         * 
         * [Api set: Mailbox Preview]
         */
        url: string;
    }
    /**
     * The body object provides methods for adding and updating the content of the message or appointment. 
     * It is returned in the body property of the selected item.
     *
     * [Api set: Mailbox 1.1]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     *    **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     */
    interface Body {
        /**
         * Returns the current body in a specified format.
         *
         * This method returns the entire current body in the format specified by coercionType.
         *
         * When working with HTML-formatted bodies, it is important to note that the Body.getAsync and Body.setAsync methods are not idempotent. 
         * The value returned from the getAsync method will not necessarily be exactly the same as the value that was passed in the setAsync method previously. 
         * The client may modify the value passed to setAsync in order to make it render efficiently with its rendering engine.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @param coercionType - The format for the returned body.
         * @param options - Optional. An object literal that contains one or more of the following properties:
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult. 
         *                  The body is provided in the requested format in the asyncResult.value property.
         */
        getAsync(coercionType: Office.CoercionType | string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Returns the current body in a specified format.
         *
         * This method returns the entire current body in the format specified by coercionType.
         *
         * When working with HTML-formatted bodies, it is important to note that the Body.getAsync and Body.setAsync methods are not idempotent. 
         * The value returned from the getAsync method will not necessarily be exactly the same as the value that was passed in the setAsync method previously. 
         * The client may modify the value passed to setAsync in order to make it render efficiently with its rendering engine.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param coercionType - The format for the returned body.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                  The body is provided in the requested format in the asyncResult.value property.
         */
        getAsync(coercionType: Office.CoercionType | string, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Gets a value that indicates whether the content is in HTML or text format.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         *
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                  The content type is returned as one of the CoercionType values in the asyncResult.value property.
         */
        getTypeAsync(options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<Office.CoercionType>) => void): void;
        /**
         * Gets a value that indicates whether the content is in HTML or text format.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         *
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                  The content type is returned as one of the CoercionType values in the asyncResult.value property.
         */
        getTypeAsync(callback?: (asyncResult: Office.AsyncResult<Office.CoercionType>) => void): void;
        /**
         * Adds the specified content to the beginning of the item body.
         *
         * The prependAsync method inserts the specified string at the beginning of the item body. 
         * After insertion, the cursor is returned to its original place, relative to the inserted content.
         *
         * When including links in HTML markup, you can disable online link preview by setting the id attribute on the anchor (\<a\>) to "LPNoLP" 
         * (please see the Examples section for a sample).
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**:
         * 
         * - DataExceedsMaximumSize: The data parameter is longer than 1,000,000 characters.
         * 
         * @param data - The string to be inserted at the beginning of the body. The string is limited to 1,000,000 characters.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         *        coercionType: The desired format for the body. The string in the data parameter will be converted to this format.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                  Any errors encountered will be provided in the asyncResult.error property.
         */
        prependAsync(data: string, options?: Office.AsyncContextOptions & CoercionTypeOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Adds the specified content to the beginning of the item body.
         *
         * The prependAsync method inserts the specified string at the beginning of the item body. 
         * After insertion, the cursor is returned to its original place, relative to the inserted content.
         *
         * When including links in HTML markup, you can disable online link preview by setting the id attribute on the anchor (\<a\>) to "LPNoLP" 
         * (please see the Examples section for a sample).
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - DataExceedsMaximumSize: The data parameter is longer than 1,000,000 characters.
         * 
         * @param data - The string to be inserted at the beginning of the body. The string is limited to 1,000,000 characters.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                  Any errors encountered will be provided in the asyncResult.error property.
         */
        prependAsync(data: string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Replaces the entire body with the specified text.
         *
         * When working with HTML-formatted bodies, it is important to note that the Body.getAsync and Body.setAsync methods are not idempotent. 
         * The value returned from the getAsync method will not necessarily be exactly the same as the value that was passed in the setAsync method 
         * previously. The client may modify the value passed to setAsync in order to make it render efficiently with its rendering engine.
         *
         * When including links in HTML markup, you can disable online link preview by setting the id attribute on the anchor (\<a\>) to "LPNoLP" 
         * (please see the Examples section for a sample).
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - DataExceedsMaximumSize: The data parameter is longer than 1,000,000 characters.
         * 
         * - InvalidFormatError: The options.coercionType parameter is set to Office.CoercionType.Html and the message body is in plain text.
         * 
         * @param data - The string that will replace the existing body. The string is limited to 1,000,000 characters.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         *        coercionType: The desired format for the body. The string in the data parameter will be converted to this format.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                  Any errors encountered will be provided in the asyncResult.error property.
         */
        setAsync(data: string, options?: Office.AsyncContextOptions & CoercionTypeOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Replaces the entire body with the specified text.
         *
         * When working with HTML-formatted bodies, it is important to note that the Body.getAsync and Body.setAsync methods are not idempotent. 
         * The value returned from the getAsync method will not necessarily be exactly the same as the value that was passed in the setAsync method 
         * previously. The client may modify the value passed to setAsync in order to make it render efficiently with its rendering engine.
         *
         * When including links in HTML markup, you can disable online link preview by setting the id attribute on the anchor (\<a\>) to "LPNoLP" 
         * (please see the Examples section for a sample).
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - DataExceedsMaximumSize: The data parameter is longer than 1,000,000 characters.
         * 
         * - InvalidFormatError: The options.coercionType parameter is set to Office.CoercionType.Html and the message body is in plain text.
         *
         * @param data - The string that will replace the existing body. The string is limited to 1,000,000 characters.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                  Any errors encountered will be provided in the asyncResult.error property.
         */
        setAsync(data: string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;

        /**
         * Replaces the selection in the body with the specified text.
         *
         * The setSelectedDataAsync method inserts the specified string at the cursor location in the body of the item, or, if text is selected in 
         * the editor, it replaces the selected text. If the cursor was never in the body of the item, or if the body of the item lost focus in the 
         * UI, the string will be inserted at the top of the body content. After insertion, the cursor is placed at the end of the inserted content.
         *
         * When including links in HTML markup, you can disable online link preview by setting the id attribute on the anchor (\<a\>) to "LPNoLP" 
         * (please see the Examples section for a sample).
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - DataExceedsMaximumSize: The data parameter is longer than 1,000,000 characters.
         * 
         * - InvalidFormatError: The options.coercionType parameter is set to Office.CoercionType.Html and the message body is in plain text.
         * 
         * @param data - The string that will replace the existing body. The string is limited to 1,000,000 characters.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         *        coercionType: The desired format for the body. The string in the data parameter will be converted to this format.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                  Any errors encountered will be provided in the asyncResult.error property.
         */
        setSelectedDataAsync(data: string, options?: Office.AsyncContextOptions & CoercionTypeOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Replaces the selection in the body with the specified text.
         *
         * The setSelectedDataAsync method inserts the specified string at the cursor location in the body of the item, or, if text is selected in 
         * the editor, it replaces the selected text. If the cursor was never in the body of the item, or if the body of the item lost focus in the 
         * UI, the string will be inserted at the top of the body content. After insertion, the cursor is placed at the end of the inserted content.
         *
         * When including links in HTML markup, you can disable online link preview by setting the id attribute on the anchor (\<a\>) to "LPNoLP" 
         * (please see the Examples section for a sample).
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - DataExceedsMaximumSize: The data parameter is longer than 1,000,000 characters.
         * 
         * - InvalidFormatError: The options.coercionType parameter is set to Office.CoercionType.Html and the message body is in plain text.
         *
         * @param data - The string that will replace the existing body. The string is limited to 1,000,000 characters.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                  Any errors encountered will be provided in the asyncResult.error property.
         */
        setSelectedDataAsync(data: string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
    }
    /**
     * Represents a contact stored on the server. Read mode only.
     *
     * The list of contacts associated with an email message or appointment is returned in the contacts property of the {@link Office.Entities} object 
     * that is returned by the getEntities or getEntitiesByType method of the active item.
     *
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
     */
    interface Contact {
        /**
         * An array of strings containing the mailing and street addresses associated with the contact. Nullable.
         */
        addresses: string[];
        /**
         * A string containing the name of the business associated with the contact. Nullable.
         */
        businessName: string;
        /**
         * An array of strings containing the SMTP email addresses associated with the contact. Nullable.
         */
        emailAddresses: string[];
        /**
         * A string containing the name of the person associated with the contact. Nullable.
         */
        personName: string;
        /**
         * An array containing a PhoneNumber object for each phone number associated with the contact. Nullable.
         */
        phoneNumbers: PhoneNumber[];
        /**
         * An array of strings containing the Internet URLs associated with the contact. Nullable.
         */
        urls: string[];
    }
    /**
     * The CustomProperties object represents custom properties that are specific to a particular item and specific to a mail add-in for Outlook. 
     * For example, there might be a need for a mail add-in to save some data that is specific to the current email message that activated the add-in. 
     * If the user revisits the same message in the future and activates the mail add-in again, the add-in will be able to retrieve the data that had 
     * been saved as custom properties.
     *
     * Because Outlook on Mac doesn't cache custom properties, if the user's network goes down, mail add-ins cannot access their custom properties.
     *
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     */
    interface CustomProperties {
        /**
         * Returns the value of the specified custom property.
         * @param name - The name of the custom property to be returned.
         * @returns The value of the specified custom property.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        get(name: string): any;
        /**
         * Sets the specified property to the specified value.
         *
         * The set method sets the specified property to the specified value. You must use the saveAsync method to save the property to the server.
         *
         * The set method creates a new property if the specified property does not already exist; 
         * otherwise, the existing value is replaced with the new value. 
         * The value parameter can be of any type; however, it is always passed to the server as a string.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param name - The name of the property to be set.
         * @param value - The value of the property to be set.
         */
        set(name: string, value: string): void;
        /**
         * Removes the specified property from the custom property collection.
         *
         * To make the removal of the property permanent, you must call the saveAsync method of the CustomProperties object.
         * @param name - The name of the property to be removed.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        remove(name: string): void;
        /**
         * Saves item-specific custom properties to the server.
         *
         * You must call the saveAsync method to persist any changes made with the set method or the remove method of the CustomProperties object. 
         * The saving action is asynchronous.
         *
         * It's a good practice to have your callback function check for and handle errors from saveAsync. 
         * In particular, a read add-in can be activated while the user is in a connected state in a read form, and subsequently the user becomes 
         * disconnected. 
         * If the add-in calls saveAsync while in the disconnected state, saveAsync would return an error. 
         * Your callback method should handle this error accordingly.
         *
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         * @param asyncContext - Optional. Any state data that is passed to the callback method.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        saveAsync(callback?: (asyncResult: Office.AsyncResult<void>) => void, asyncContext?: any): void;
    }
    /**
     * Provides diagnostic information to an Outlook add-in.
     *
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     */
    interface Diagnostics {
        /**
         * Gets a string that represents the name of the host application.
         *
         * A string that can be one of the following values: Outlook, Mac Outlook, OutlookIOS, or OutlookWebApp.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        hostName: string;
        /**
         * Gets a string that represents the version of either the host application or the Exchange Server.
         *
         * If the mail add-in is running in Outlook client on the desktop or iOS, the hostVersion property returns the version of the host 
         * application, Outlook. In Outlook on the web, the property returns the version of the Exchange Server. An example is the string 15.0.468.0.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        hostVersion: string;
        /**
         * Gets a string that represents the current view of Outlook on the web.
         *
         * The returned string can be one of the following values: OneColumn, TwoColumns, or ThreeColumns.
         *
         * If the host application is not Outlook on the web, then accessing this property results in undefined.
         *
         * Outlook on the web has three views that correspond to the width of the screen and the window, and the number of columns that can be displayed:
         *
         * - OneColumn, which is displayed when the screen is narrow. Outlook on the web uses this single-column layout on the entire screen of a 
         * smartphone.
         *
         * - TwoColumns, which is displayed when the screen is wider. Outlook on the web uses this view on most tablets.
         *
         * - ThreeColumns, which is displayed when the screen is wide. For example, Outlook on the web uses this view in a full screen window on a 
         * desktop computer.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        OWAView: MailboxEnums.OWAView | "OneColumn" | "TwoColumns" | "ThreeColumns";
    }
    /**
     * Provides the email properties of the sender or specified recipients of an email message or appointment.
     *
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     */
    interface EmailAddressDetails {
        /**
         * Gets the SMTP email address.
         */
        emailAddress: string;
        /**
         * Gets the display name associated with an email address.
         */
        displayName: string;
        /**
         * Gets the response that an attendee returned for an appointment. 
         * This property applies to only an attendee of an appointment, as represented by the optionalAttendees or requiredAttendees property. 
         * This property returns undefined in other scenarios.
         */
        appointmentResponse: MailboxEnums.ResponseType | string;
        /**
         * Gets the email address type of a recipient.
         */
        recipientType: MailboxEnums.RecipientType | string;
    }
    /**
     * Represents an email account on an Exchange Server.
     *
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
     */
    interface EmailUser {
        /**
         * Gets the display name associated with an email address.
         */
        displayName: string;
        /**
         * Gets the SMTP email address.
         */
        emailAddress: string;
    }
    /**
     * Represents the set of locations on an appointment.
     * 
     * [Api set: Mailbox Preview]
     * 
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     * 
     * @beta
     */
    export interface EnhancedLocation {
        /**
         * Adds to the set of locations associated with the appointment.
         * 
         * [Api set: Mailbox Preview]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - InvalidFormatError: The format of the specified data object is not valid.
         * 
         * @param locationIdentifiers The locations to be added to the current list of locations.
         * @param options Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object. Check the `status` property of asyncResult to determine if the call succeeded.
         * 
         * @beta
         */
        addAsync(locationIdentifiers: LocationIdentifier[], options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResultStatus) => void): void;
        /**
         * Adds to the set of locations associated with the appointment.
         * 
         * [Api set: Mailbox Preview]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - InvalidFormatError: The format of the specified data object is not valid.
         * 
         * @param locationIdentifiers The locations to be added to the current list of locations.
         * @param callback Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object. Check the `status` property of asyncResult to determine if the call succeeded.
         * 
         * @beta
         */
        addAsync(locationIdentifiers: LocationIdentifier[], callback?: (asyncResult: Office.AsyncResultStatus) => void): void;
        /**
         * Gets the set of locations associated with the appointment.
         * 
         * [Api set: Mailbox Preview]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @param options Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         * 
         * @beta
         */
        getAsync(options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<LocationDetails[]>) => void): void;
        /**
         * Gets the set of locations associated with the appointment.
         * 
         * [Api set: Mailbox Preview]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @param callback Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         * 
         * @beta
         */
        getAsync(callback?: (asyncResult: Office.AsyncResult<LocationDetails[]>) => void): void;
        /**
         * Removes the set of locations associated with the appointment.
         * 
         * If there are multiple locations with the same name, all matching locations will be removed even if only one was specified in locationIdentifiers.
         * 
         * [Api set: Mailbox Preview]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * @param locationIdentifiers The locations to be removed from the current list of locations.
         * @param options Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object. Check the `status` property of asyncResult to determine if the call succeeded.
         * 
         * @beta
         */
        removeAsync(locationIdentifiers: LocationIdentifier[], options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResultStatus) => void): void;
        /**
         * Removes the set of locations associated with the appointment.
         * 
         * If there are multiple locations with the same name, all matching locations will be removed even if only one was specified in locationIdentifiers.
         * 
         * [Api set: Mailbox Preview]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * @param locationIdentifiers The locations to be removed from the current list of locations.
         * @param callback Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object. Check the `status` property of asyncResult to determine if the call succeeded.
         * 
         * @beta
         */
        removeAsync(locationIdentifiers: LocationIdentifier[], callback?: (asyncResult: Office.AsyncResultStatus) => void): void;
    }
    /**
     * Represents a collection of entities found in an email message or appointment. Read mode only.
     *
     * The Entities object is a container for the entity arrays returned by the getEntities and getEntitiesByType methods when the item 
     * (either an email message or an appointment) contains one or more entities that have been found by the server. 
     * You can use these entities in your code to provide additional context information to the viewer, such as a map to an address found in the item, 
     * or to open a dialer for a phone number found in the item.
     *
     * If no entities of the type specified in the property are present in the item, the property associated with that entity is null. 
     * For example, if a message contains a street address and a phone number, the addresses property and phoneNumbers property would contain 
     * information, and the other properties would be null.
     *
     * To be recognized as an address, the string must contain a United States postal address that has at least a subset of the elements of a street 
     * number, street name, city, state, and zip code.
     *
     * To be recognized as a phone number, the string must contain a North American phone number format.
     *
     * Entity recognition relies on natural language recognition that is based on machine learning of large amounts of data. 
     * The recognition of an entity is non-deterministic and success sometimes relies on the particular context in the item.
     *
     * When the property arrays are returned by the getEntitiesByType method, only the property for the specified entity contains data; 
     * all other properties are null.
     *
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
     */
    interface Entities {
        /**
         * Gets the physical addresses (street or mailing addresses) found in an email message or appointment.
         */
        addresses: string[];
        /**
         * Gets the contacts found in an email address or appointment.
         */
        contacts: Contact[];
        /**
         * Gets the email addresses found in an email message or appointment.
         */
        emailAddresses: string[];
        /**
         * Gets the meeting suggestions found in an email message.
         */
        meetingSuggestions: MeetingSuggestion[];
        /**
         * Gets the phone numbers found in an email message or appointment.
         */
        phoneNumbers: PhoneNumber[];
        /**
         * Gets the task suggestions found in an email message or appointment.
         */
        taskSuggestions: string[];
        /**
         * Gets the Internet URLs present in an email message or appointment.
         */
        urls: string[];
    }

    /**
     * Provides a method to get the from value of a message in an Outlook add-in.
     * 
     * [Api set: Mailbox 1.7]
     * 
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
     */
    interface From {
        /**
         * Gets the from value of a message.
         * 
         * The getAsync method starts an asynchronous call to the Exchange server to get the from value of a message.
         * 
         * The from value of the item is provided as an {@link Office.EmailAddressDetails} in the asyncResult.value property.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter,
         *                             asyncResult, which is an Office.AsyncResult object.
         *                  The `value` property of the result is the item's from value, as an EmailAddressDetails object.
         */
        getAsync(options: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<EmailAddressDetails>) => void): void;
        /**
         * Gets the from value of a message.
         * 
         * The getAsync method starts an asynchronous call to the Exchange server to get the from value of a message.
         * 
         * The from value of the item is provided as an {@link Office.EmailAddressDetails} in the asyncResult.value property.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter,
         *                             asyncResult, which is an Office.AsyncResult object.
         *                  The `value` property of the result is the item's from value, as an EmailAddressDetails object.
         */
        getAsync(callback?: (asyncResult: Office.AsyncResult<EmailAddressDetails>) => void): void;
    }

    /**
     * The InternetHeaders object represents properties that are preserved after the message item leaves Exchange and is converted to a MIME message. 
     * These properties are stored as x-headers in the MIME message.
     * 
     * InternetHeaders are stored as key/value pairs on a per-item basis.
     *
     * [Api set: Mailbox Preview]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     * 
     * @beta
     */
    interface InternetHeaders {
        /**
         * Given an array of internet header names, this method returns a dictionary containing those internet headers and their values. 
         * If the add-in requests an x-header that is not available, that x-header will not be returned in the results. 
         *
         * [Api set: Mailbox Preview]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @param names - The names of the internet headers to be returned.
         * @param options - Optional. An object literal that contains one or more of the following properties:
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         * 
         * @beta
         */
        getAsync(names: string[], options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<InternetHeaders>) => void): void;
        /**
         * Given an array of internet header names, this method returns a dictionary containing those internet headers and their values. 
         * If the add-in requests an x-header that is not available, that x-header will not be returned in the results. 
         *
         * [Api set: Mailbox Preview]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @param names - The names of the internet headers to be returned.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         * 
         * @beta
         */
        getAsync(names: string[], callback?: (asyncResult: Office.AsyncResult<InternetHeaders>) => void): void;
        /**
         * Given an array of internet header names, this method removes the specified headers from the internet header collection.
         *
         * [Api set: Mailbox Preview]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * @param names - The names of the internet headers to be removed.
         * @param options - Optional. An object literal that contains one or more of the following properties:
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         * 
         * @beta
         */
        removeAsync(names: string[], options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<InternetHeaders>) => void): void;
        /**
         * Given an array of internet header names, this method removes the specified headers from the internet header collection.
         *
         * [Api set: Mailbox Preview]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * @param names - The names of the internet headers to be removed.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         * 
         * @beta
         */
        removeAsync(names: string[], callback?: (asyncResult: Office.AsyncResult<InternetHeaders>) => void): void;
        /**
         * Sets the specified internet headers to the specified values.
         * 
         * The setAsync method creates a new header if the specified header does not already exist; otherwise, the existing value is replaced with 
         * the new value.
         *
         * [Api set: Mailbox Preview]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         *
         * @param headers - The names and corresponding values of the headers to be set. Should be a dictionary object with keys being the names of the 
         *                internet headers and values being the values of the internet headers.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                  Any errors encountered will be provided in the asyncResult.error property.
         * 
         * @beta
         */
        setAsync(headers: Object, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Sets the specified internet headers to the specified values.
         * 
         * The setAsync method creates a new header if the specified header does not already exist; otherwise, the existing value is replaced with 
         * the new value.
         *
         * [Api set: Mailbox Preview]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         *
         * @param headers - The names and corresponding values of the headers to be set. Should be a dictionary object with keys being the names of the 
         *                internet headers and values being the values of the internet headers.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                  Any errors encountered will be provided in the asyncResult.error property.
         * 
         * @beta
         */
        setAsync(headers: Object, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
    }

    /**
     * Represents a location. Read only.
     * 
     * [Api set: Mailbox Preview]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     * 
     * @beta
     */
    export interface LocationDetails {
        /**
         * The LocationIdentifier of the location.
         */
        locationIdentifier: LocationIdentifier;
        /**
         * The location's display name.
         */
        displayName: string;
        /**
         * The email address associated with the location.
         */
        emailAddress: string;
    }

    /**
     * Represents the id of a location.
     * 
     * [Api set: Mailbox Preview]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     * 
     * @beta
     */
    interface LocationIdentifier {
        /**
         * The location's unique id.
         * 
         * For **Room** type, it's the room's email address.
         * 
         * For **Custom** type, it's the displayName.
         */
        id: string;
        /**
         * The location's type.
         */
        type: MailboxEnums.LocationType | string;
    }

    /**
     * Represents the appointment organizer, even if an alias or a delegate was used to create the appointment. 
     * This object provides a method to get the organizer value of an appointment in an Outlook add-in.
     * 
     * [Api set: Mailbox 1.7]
     * 
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
     */
    interface Organizer {
        /**
         * Gets the organizer value of an appointment as an {@link Office.EmailAddressDetails} in the asyncResult.value property.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * @param options - An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter, asyncResult, which is an AsyncResult object.
         *                  The `value` property of the result is the appointment's organizer value, as an EmailAddressDetails object.
         */
        getAsync(options: Office.AsyncContextOptions, callback: (asyncResult: Office.AsyncResult<EmailAddressDetails>) => void): void;
        /**
         * Gets the organizer value of an appointment as an {@link Office.EmailAddressDetails} in the asyncResult.value property.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter, asyncResult, which is an AsyncResult object.
         *                  The `value` property of the result is the appointment's organizer value, as an EmailAddressDetails object.
         */
        getAsync(callback?: (asyncResult: Office.AsyncResult<EmailAddressDetails>) => void): void;
    }

    /**
     * The subclass of {@link Office.Item} dealing with appointments.
     * 
     * **Important**: This is an internal Outlook object, not directly exposed through existing interfaces. 
     * You should treat this as a mode of Office.context.mailbox.item. Refer to the
     * {@link https://docs.microsoft.com/office/dev/add-ins/reference/objectmodel/preview-requirement-set/office.context.mailbox.item | Object Model} page for more information.
     */
    interface Appointment extends Item {
    }
    /**
     * The appointment organizer mode of {@link Office.Item | Office.context.mailbox.item}.
     * 
     * **Important**: This is an internal Outlook object, not directly exposed through existing interfaces. 
     * You should treat this as a mode of Office.context.mailbox.item. Refer to the
     * {@link https://docs.microsoft.com/office/dev/add-ins/reference/objectmodel/preview-requirement-set/office.context.mailbox.item | Object Model} page for more information.
     */
    interface AppointmentCompose extends Appointment, ItemCompose {
         /**
         * Gets an object that provides methods for manipulating the body of an item.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         */
        body: Body;
        /**
         * Gets or sets the date and time that the appointment is to end.
         *
         * The end property is an {@link Office.Time} object expressed as a Coordinated Universal Time (UTC) date and time value. 
         * You can use the convertToLocalClientTime method to convert the end property value to the client's local date and time.
         *
         * When you use the Time.setAsync method to set the end time, you should use the convertToUtcClientTime method to convert the local time on 
         * the client to UTC for the server.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         */
        end: Time;
        /**
         * Gets or sets the locations of the appointment. The `enhancedLocation` property returns an {@link Office.EnhancedLocation} object that 
         * provides methods to get, remove, or add locations on an item.
         * 
         * [Api set: Mailbox Preview]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * @beta
         */
        enhancedLocation: EnhancedLocation;
        /**
         * Gets the type of item that an instance represents.
         *
         * The itemType property returns one of the ItemType enumeration values, indicating whether the item object instance is a message or an appointment.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         */
        itemType: MailboxEnums.ItemType | string;
        /**
         * Gets or sets the {@link Office.Location} of an appointment. The location property returns a Location object that provides methods that are 
         * used to get and set the location of the appointment.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         */
        location: Location;
        /**
         * Gets the notification messages for an item.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         */
        notificationMessages: NotificationMessages;
        /**
         * Provides access to the optional attendees of an event. The type of object and level of access depends on the mode of the current item. 
         * The optionalAttendees property returns an {@link Office.Recipients} object that provides methods to get or update the optional attendees 
         * for a meeting.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         */
        optionalAttendees: Recipients;
        /**
         * Gets the organizer for the specified meeting. 
         * 
         * The organizer property returns an {@link Office.Organizer | Organizer} object that provides a method to get the organizer value.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         */
        organizer: Organizer;
        /**
         * Gets or sets the recurrence pattern of an appointment.
         * 
         * The recurrence property returns a recurrence object for recurring appointments or meetings requests if an item is a series or an instance 
         * in a series. `null` is returned for single appointments and meeting requests of single appointments.
         * 
         * **Note**: Meeting requests have an itemClass value of IPM.Schedule.Meeting.Request.
         * 
         * **Note**: If the recurrence object is null, this indicates that the object is a single appointment or a meeting request of a single 
         * appointment and NOT a part of a series.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         */
        recurrence: Recurrence;
        /**
         * Provides access to the required attendees of an event. The type of object and level of access depends on the mode of the current item. 
         * The requiredAttendees property returns an {@link Office.Recipients} object that provides methods to get or update the required attendees 
         * for a meeting.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         */
        requiredAttendees: Recipients;
        /**
         * Gets the id of the series that an instance belongs to.
         * 
         * In Outlook on the web and desktop clients, the seriesId returns the Exchange Web Services (EWS) ID of the parent (series) item that this item belongs to. 
         * However, on iOS and Android, the seriesId returns the REST ID of the parent item.
         * 
         * **Note**: The identifier returned by the seriesId property is the same as the Exchange Web Services item identifier. 
         * The seriesId property is not identical to the Outlook IDs used by the Outlook REST API. 
         * Before making REST API calls using this value, it should be converted using Office.context.mailbox.convertToRestId. 
         * For more details, see {@link https://docs.microsoft.com/outlook/add-ins/use-rest-api | Use the Outlook REST APIs from an Outlook add-in}.
         * 
         * The seriesId property returns null for items that do not have parent items such as single appointments, series items, or meeting requests 
         * and returns undefined for any other items that are not meeting requests.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         */
        seriesId: string;
        /**
         * Gets or sets the date and time that the appointment is to begin.
         *
         * The start property is an {@link Office.Time} object expressed as a Coordinated Universal Time (UTC) date and time value. 
         * You can use the convertToLocalClientTime method to convert the value to the client's local date and time.
         *
         * When you use the Time.setAsync method to set the start time, you should use the convertToUtcClientTime method to convert the local time on 
         * the client to UTC for the server.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         */
        start: Time;
        /**
         * Gets or sets the description that appears in the subject field of an item.
         *
         * The subject property gets or sets the entire subject of the item, as sent by the email server.
         *
         * The subject property returns a Subject object that provides methods to get and set the subject.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         */
        subject: Subject;
        /**
         * Adds a file to a message or appointment as an attachment.
         *
         * The addFileAttachmentAsync method uploads the file at the specified URI and attaches it to the item in the compose form.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * **Errors**: 
         * 
         * - AttachmentSizeExceeded: The attachment is larger than allowed.
         * 
         * - FileTypeNotSupported: The attachment has an extension that is not allowed.
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         * 
         * @param uri - The URI that provides the location of the file to attach to the message or appointment. The maximum length is 2048 characters.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         *        isInline: If true, indicates that the attachment will be shown inline in the message body, and should not be displayed in the attachment list.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult. 
         *                 On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                 If uploading the attachment fails, the asyncResult object will contain an Error object that provides a description of the error.
         */
        addFileAttachmentAsync(uri: string, attachmentName: string, options?: Office.AsyncContextOptions & { isInline: boolean }, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Adds a file to a message or appointment as an attachment.
         *
         * The addFileAttachmentAsync method uploads the file at the specified URI and attaches it to the item in the compose form.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * **Errors**: 
         * 
         * - AttachmentSizeExceeded: The attachment is larger than allowed.
         * 
         * - FileTypeNotSupported: The attachment has an extension that is not allowed.
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         *
         * @param uri - The URI that provides the location of the file to attach to the message or appointment. The maximum length is 2048 characters.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult. 
         *                 On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                 If uploading the attachment fails, the asyncResult object will contain an Error object that provides a description of the error.
         */
        addFileAttachmentAsync(uri: string, attachmentName: string, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Adds a file to a message or appointment as an attachment.
         *
         * The addFileAttachmentFromBase64Async method uploads the file from the base64 encoding and attaches it to the item in the compose form. This method returns the attachment identifier in the asyncResult.value object.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * [Api set: Mailbox Preview]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * **Errors**: 
         * 
         * - AttachmentSizeExceeded: The attachment is larger than allowed.
         * 
         * - FileTypeNotSupported: The attachment has an extension that is not allowed.
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         * 
         * @param base64File - The base64 encoded content of an image or file to be added to an email or event.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         *        isInline: If true, indicates that the attachment will be shown inline in the message body and should not be displayed in the attachment list.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult. 
         *                  On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                  If uploading the attachment fails, the asyncResult object will contain an Error object that provides a description of the error.
         * 
         * @beta
         */
        addFileAttachmentFromBase64Async(base64File: string, attachmentName: string, options?: Office.AsyncContextOptions &  { isInline: boolean }, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Adds a file to a message or appointment as an attachment.
         *
         * The addFileAttachmentFromBase64Async method uploads the file from the base64 encoding and attaches it to the item in the compose form. This method returns the attachment identifier in the asyncResult.value object.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * [Api set: Mailbox Preview]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * **Errors**: 
         * 
         * - AttachmentSizeExceeded: The attachment is larger than allowed.
         * 
         * - FileTypeNotSupported: The attachment has an extension that is not allowed.
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         * 
         * @param base64File - The base64 encoded content of an image or file to be added to an email or event.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult. 
         *                  On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                  If uploading the attachment fails, the asyncResult object will contain an Error object that provides a description of the error.
         * 
         * @beta
         */
        addFileAttachmentFromBase64Async(base64File: string, attachmentName: string, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Adds an event handler for a supported event.
         * 
         * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and
         * `Office.EventType.RecurrenceChanged`.
         * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * @param eventType - The event that should invoke the handler.
         * @param handler - The function to handle the event. The function must accept a single parameter, which is an object literal. 
         *                The type property on the parameter will match the eventType parameter passed to addHandlerAsync.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        addHandlerAsync(eventType: Office.EventType | string, handler: any, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Adds an event handler for a supported event.
         * 
         * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
         * `Office.EventType.RecurrenceChanged`.
         * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * @param eventType - The event that should invoke the handler.
         * @param handler - The function to handle the event. The function must accept a single parameter, which is an object literal. 
         *                The type property on the parameter will match the eventType parameter passed to addHandlerAsync.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        addHandlerAsync(eventType: Office.EventType | string, handler: any, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Adds an Exchange item, such as a message, as an attachment to the message or appointment.
         *
         * The addItemAttachmentAsync method attaches the item with the specified Exchange identifier to the item in the compose form. 
         * If you specify a callback method, the method is called with one parameter, asyncResult, which contains either the attachment identifier or 
         * a code that indicates any error that occurred while attaching the item. 
         * You can use the options parameter to pass state information to the callback method, if needed.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * If your Office add-in is running in Outlook on the web, the addItemAttachmentAsync method can attach items to items other than the item that 
         * you are editing; however, this is not supported and is not recommended.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * **Errors**: 
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         * 
         * @param itemId - The Exchange identifier of the item to attach. The maximum length is 100 characters.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult. 
         *                 On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                 If adding the attachment fails, the asyncResult object will contain an Error object that provides a description of the error.
         */
        addItemAttachmentAsync(itemId: any, attachmentName: string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Adds an Exchange item, such as a message, as an attachment to the message or appointment.
         *
         * The addItemAttachmentAsync method attaches the item with the specified Exchange identifier to the item in the compose form. 
         * If you specify a callback method, the method is called with one parameter, asyncResult, which contains either the attachment identifier or 
         * a code that indicates any error that occurred while attaching the item. 
         * You can use the options parameter to pass state information to the callback method, if needed.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * If your Office add-in is running in Outlook on the web, the addItemAttachmentAsync method can attach items to items other than the item that 
         * you are editing; however, this is not supported and is not recommended.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * **Errors**: 
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         *
         * @param itemId - The Exchange identifier of the item to attach. The maximum length is 100 characters.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult. 
         *                 On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                 If adding the attachment fails, the asyncResult object will contain an Error object that provides a description of the error.
         */
        addItemAttachmentAsync(itemId: any, attachmentName: string, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Closes the current item that is being composed
         *
         * The behaviors of the close method depends on the current state of the item being composed. 
         * If the item has unsaved changes, the client prompts the user to save, discard, or close the action.
         *
         * In the Outlook desktop client, if the message is an inline reply, the close method has no effect.
         *
         * **Note**: In Outlook on the web, if the item is an appointment and it has previously been saved using saveAsync, the user is prompted to save, 
         * discard, or cancel even if no changes have occurred since the item was last saved.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         */
        close(): void;
        /**
         * Gets the item's attachments as an array.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. If the call fails, the asyncResult.error property will contain and error code with the reason for 
         *                 the failure.
         * 
         * @beta
         */
        getAttachmentsAsync(options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<AttachmentDetails[]>) => void): void;
        /**
         * Gets the item's attachments as an array.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. If the call fails, the asyncResult.error property will contain and error code with the reason for 
         *                 the failure.
         * 
         * @beta
         */
        getAttachmentsAsync(callback?: (asyncResult: Office.AsyncResult<AttachmentDetails[]>) => void): void;
        /**
         * Gets initialization data passed when the add-in is activated by an actionable message.
         *
         * **Note**: This method is only supported by Outlook 2016 or later on Windows (Click-to-Run versions greater than 16.0.8413.1000) and Outlook on the web for Office 365.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * More information on {@link https://docs.microsoft.com/outlook/actionable-messages/invoke-add-in-from-actionable-message | actionable messages}.
         *
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult. 
         *                 On success, the initialization data is provided in the asyncResult.value property as a string. 
         *                 If there is no initialization context, the asyncResult object will contain an Error object with its code property set to 9020 and its name property set to GenericResponseError.
         *
         * @beta
         */
        getInitializationContextAsync(options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Gets initialization data passed when the add-in is activated by an actionable message.
         *
         * **Note**: This method is only supported by Outlook 2016 or later on Windows (Click-to-Run versions greater than 16.0.8413.1000) and Outlook on the web for Office 365.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * More information on {@link https://docs.microsoft.com/outlook/actionable-messages/invoke-add-in-from-actionable-message | actionable messages}.
         *
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult. 
         *                 On success, the initialization data is provided in the asyncResult.value property as a string. 
         *                 If there is no initialization context, the asyncResult object will contain an Error object with its code property set to 9020 and its name property set to GenericResponseError.
         *
         * @beta
         */
        getInitializationContextAsync(callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Asynchronously returns selected data from the subject or body of a message.
         *
         * If there is no selection but the cursor is in the body or subject, the method returns null for the selected data. 
         * If a field other than the body or subject is selected, the method returns the InvalidSelection error.
         *
         * To access the selected data from the callback method, call asyncResult.value.data. 
         * To access the source property that the selection comes from, call asyncResult.value.sourceProperty, which will be either body or subject.
         *
         * [Api set: Mailbox 1.2]
         *
         * @returns
         * The selected data as a string with format determined by coercionType.
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         *
         * @param coercionType - Requests a format for the data. If Text, the method returns the plain text as a string , removing any HTML tags present. 
         *                     If HTML, the method returns the selected text, whether it is plaintext or HTML.
         * @param options - An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         */
        getSelectedDataAsync(coercionType: Office.CoercionType | string, options: Office.AsyncContextOptions, callback: (asyncResult: Office.AsyncResult<any>) => void): void;
         /**
         * Asynchronously returns selected data from the subject or body of a message.
         *
         * If there is no selection but the cursor is in the body or subject, the method returns null for the selected data. 
         * If a field other than the body or subject is selected, the method returns the InvalidSelection error.
         *
         * To access the selected data from the callback method, call asyncResult.value.data. 
         * To access the source property that the selection comes from, call asyncResult.value.sourceProperty, which will be either body or subject.
         *
         * [Api set: Mailbox 1.2]
         *
         * @returns
         * The selected data as a string with format determined by coercionType.
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         *
         * @param coercionType - Requests a format for the data. If Text, the method returns the plain text as a string , removing any HTML tags present. 
         *                     If HTML, the method returns the selected text, whether it is plaintext or HTML.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        getSelectedDataAsync(coercionType: Office.CoercionType | string, callback: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Asynchronously loads custom properties for this add-in on the selected item.
         *
         * Custom properties are stored as key/value pairs on a per-app, per-item basis. 
         * This method returns a CustomProperties object in the callback, which provides methods to access the custom properties specific to the 
         * current item and the current add-in. Custom properties are not encrypted on the item, so this should not be used as secure storage.
         *
         * The custom properties are provided as a CustomProperties object in the asyncResult.value property. 
         * This object can be used to get, set, and remove custom properties from the item and save changes to the custom property set back to 
         * the server.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         *
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         * @param userContext - Optional. Developers can provide any object they wish to access in the callback function. 
         *                    This object can be accessed by the asyncResult.asyncContext property in the callback function.
         */
        loadCustomPropertiesAsync(callback: (asyncResult: Office.AsyncResult<CustomProperties>) => void, userContext?: any): void;
        /**
         * Removes an attachment from a message or appointment.
         *
         * The removeAttachmentAsync method removes the attachment with the specified identifier from the item. 
         * As a best practice, you should use the attachment identifier to remove an attachment only if the same mail app has added that attachment 
         * in the same session. In Outlook on the web and mobile devices, the attachment identifier is valid only within the same session. 
         * A session is over when the user closes the app, or if the user starts composing an inline form then subsequently pops out the form to 
         * continue in a separate window.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * **Errors**: 
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         *
         * @param attachmentId - The identifier of the attachment to remove.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult. 
         *                 If removing the attachment fails, the asyncResult.error property will contain an error code with the reason for the failure.
         */
        removeAttachmentAsync(attachmentId: string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Removes an attachment from a message or appointment.
         *
         * The removeAttachmentAsync method removes the attachment with the specified identifier from the item. 
         * As a best practice, you should use the attachment identifier to remove an attachment only if the same mail app has added that attachment 
         * in the same session. 
         * In Outlook on the web and mobile devices, the attachment identifier is valid only within the same session. 
         * A session is over when the user closes the app, or if the user starts composing an inline form then subsequently pops out the form to 
         * continue in a separate window.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * **Errors**: 
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         *
         * @param attachmentId - The identifier of the attachment to remove.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         *                 If removing the attachment fails, the asyncResult.error property will contain an error code with the reason for the failure.
         */
        removeAttachmentAsync(attachmentId: string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
       /**
        * Removes the event handlers for a supported event type.
        * 
        * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
        * `Office.EventType.RecurrenceChanged`.
        * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
        * 
        * [Api set: Mailbox 1.7]
        *
        * @remarks
        *
        * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
        * 
        * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
        * 
        * @param eventType - The event that should revoke the handler.
        * @param options - Optional. An object literal that contains one or more of the following properties.
        *        asyncContext: Developers can provide any object they wish to access in the callback method.
        * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
        *                asyncResult, which is an Office.AsyncResult object.
        */
       removeHandlerAsync(eventType: Office.EventType | string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
       /**
        * Removes the event handlers for a supported event type.
        * 
        * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
        * `Office.EventType.RecurrenceChanged`.
        * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
        * 
        * [Api set: Mailbox 1.7]
        *
        * @remarks
        *
        * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
        * 
        * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
        * 
        * @param eventType - The event that should revoke the handler.
        * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
        *                asyncResult, which is an Office.AsyncResult object.
        */
       removeHandlerAsync(eventType: Office.EventType | string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Asynchronously saves an item.
         *
         * When invoked, this method saves the current message as a draft and returns the item id via the callback method. 
         * In Outlook on the web or Outlook in online mode, the item is saved to the server. 
         * In Outlook in cached mode, the item is saved to the local cache.
         *
         * Since appointments have no draft state, if saveAsync is called on an appointment in compose mode, the item will be saved as a normal 
         * appointment on the user's calendar. For new appointments that have not been saved before, no invitation will be sent. 
         * Saving an existing appointment will send an update to added or removed attendees.
         *
         * **Note**: If your add-in calls saveAsync on an item in compose mode in order to get an itemId to use with EWS or the REST API, be aware that 
         * when Outlook is in cached mode, it may take some time before the item is actually synced to the server. 
         * Until the item is synced, using the itemId will return an error.
         *
         * **Note**: The following clients have different behavior for saveAsync on appointments in compose mode:
         *
         * - Outlook on Mac does not support saving a meeting. The saveAsync method fails when called from a meeting in compose mode.
         * See {@link https://support.microsoft.com/help/4505745 | Cannot save a meeting as a draft in Outlook for Mac by using Office JS API} for a workaround.
         *
         * - Outlook on the web always sends an invitation or update when saveAsync is called on an appointment in compose mode.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * **Errors**: 
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         * 
         * @param options - An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult. 
         */
        saveAsync(options: Office.AsyncContextOptions, callback: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Asynchronously saves an item.
         *
         * When invoked, this method saves the current message as a draft and returns the item id via the callback method. 
         * In Outlook on the web or Outlook in online mode, the item is saved to the server. In Outlook in cached mode, the item is saved to the local cache.
         *
         * Since appointments have no draft state, if saveAsync is called on an appointment in compose mode, the item will be saved as a normal 
         * appointment on the user's calendar. For new appointments that have not been saved before, no invitation will be sent. 
         * Saving an existing appointment will send an update to added or removed attendees.
         *
         * **Note**: If your add-in calls saveAsync on an item in compose mode in order to get an itemId to use with EWS or the REST API, be aware that 
         * when Outlook is in cached mode, it may take some time before the item is actually synced to the server. 
         * Until the item is synced, using the itemId will return an error.
         *
         * **Note**: The following clients have different behavior for saveAsync on appointments in compose mode:
         *
         * - Outlook on Mac does not support saving a meeting. The saveAsync method fails when called from a meeting in compose mode.
         * See {@link https://support.microsoft.com/help/4505745 | Cannot save a meeting as a draft in Outlook for Mac by using Office JS API} for a workaround.
         *
         * - Outlook on the web always sends an invitation or update when saveAsync is called on an appointment in compose mode.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * **Errors**: 
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         *
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         */
        saveAsync(callback: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Asynchronously inserts data into the body or subject of a message.
         *
         * The setSelectedDataAsync method inserts the specified string at the cursor location in the subject or body of the item, or, if text is 
         * selected in the editor, it replaces the selected text. If the cursor is not in the body or subject field, an error is returned. 
         * After insertion, the cursor is placed at the end of the inserted content.
         *
         * [Api set: Mailbox 1.2]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * **Errors**: 
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         * 
         * @param data - The data to be inserted. Data is not to exceed 1,000,000 characters. 
         *             If more than 1,000,000 characters are passed in, an ArgumentOutOfRange exception is thrown.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         *        coercionType: If text, the current style is applied in Outlook on the web and Windows. 
         *                      If the field is an HTML editor, only the text data is inserted, even if the data is HTML. 
         *                      If html and the field supports HTML (the subject doesn't), the current style is applied in Outlook on the web and the 
         *                      default style is applied in Outlook on desktop clients. 
         *                      If the field is a text field, an InvalidDataFormat error is returned. 
         *                      If coercionType is not set, the result depends on the field: if the field is HTML then HTML is used; 
         *                      if the field is text, then plain text is used.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        setSelectedDataAsync(data: string, options?: Office.AsyncContextOptions & CoercionTypeOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Asynchronously inserts data into the body or subject of a message.
         *
         * The setSelectedDataAsync method inserts the specified string at the cursor location in the subject or body of the item, or, if text is 
         * selected in the editor, it replaces the selected text. If the cursor is not in the body or subject field, an error is returned. 
         * After insertion, the cursor is placed at the end of the inserted content.
         *
         * [Api set: Mailbox 1.2]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Organizer
         * 
         * **Errors**: 
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.S
         *
         * @param data - The data to be inserted. Data is not to exceed 1,000,000 characters. 
         *             If more than 1,000,000 characters are passed in, an ArgumentOutOfRange exception is thrown.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        setSelectedDataAsync(data: string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
    }

    /**
     * The appointment attendee mode of {@link Office.Item | Office.context.mailbox.item}.
     * 
     * **Important**: This is an internal Outlook object, not directly exposed through existing interfaces. 
     * You should treat this as a mode of Office.context.mailbox.item. Refer to the
     * {@link https://docs.microsoft.com/office/dev/add-ins/reference/objectmodel/preview-requirement-set/office.context.mailbox.item | Object Model} page for more information.
     */
    interface AppointmentRead extends Appointment, ItemRead {
        /**
         * Gets the item's attachments as an array.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         *
         * **Note**: Certain types of files are blocked by Outlook due to potential security issues and are therefore not returned. For more information, see 
         * {@link https://support.office.com/article/Blocked-attachments-in-Outlook-434752E1-02D3-4E90-9124-8B81E49A8519 | Blocked attachments in Outlook}.
         *
         */
        attachments: AttachmentDetails[];
        /**
         * Gets an object that provides methods for manipulating the body of an item.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        body: Body;
        /**
         * Gets the date and time that an item was created.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        dateTimeCreated: Date;
        /**
         * Gets the date and time that an item was last modified.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         *
         * **Note**: This member is not supported in Outlook on iOS or Android.
         */
        dateTimeModified: Date;
        /**
         * Gets the date and time that the appointment is to end.
         *
         * The end property is a Date object expressed as a Coordinated Universal Time (UTC) date and time value. 
         * You can use the convertToLocalClientTime method to convert the end property value to the client's local date and time.
         *
         * When you use the Time.setAsync method to set the end time, you should use the convertToUtcClientTime method to convert the local time on 
         * the client to UTC for the server.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        end: Date;
        /**
         * Gets the locations of an appointment.
         *
         * The enhancedLocation property returns an {@link Office.EnhancedLocation} object that allows you to get the set of locations (each represented by 
         * an {@link Office.LocationDetails} object) associated with the appointment.
         * 
         * [Api set: Mailbox Preview]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         * 
         * @beta
         */
        enhancedLocation: EnhancedLocation;
        /**
         * Gets the Exchange Web Services item class of the selected item.
         *
         *
         * You can create custom message classes that extends a default message class, for example, a custom appointment message class IPM.Appointment.Contoso.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         * 
         * The itemClass property specifies the message class of the selected item. The following are the default message classes for the message or appointment item.
         * 
         * <table>
         *   <tr>
         *     <th>Type</th>
         *     <th>Description</th>
         *     <th>Item Class</th>
         *   </tr>
         *   <tr>
         *     <td>Appointment items</td>
         *     <td>These are calendar items of the item class IPM.Appointment or IPM.Appointment.Occurrence.</td>
         *     <td>IPM.Appointment,IPM.Appointment.Occurrence</td>
         *   </tr>
         *   <tr>
         *     <td>Message items</td>
         *     <td>These include email messages that have the default message class IPM.Note, and meeting requests, responses, and cancellations, that use IPM.Schedule.Meeting as the base message class.</td>
         *     <td>IPM.Note,IPM.Schedule.Meeting.Request,IPM.Schedule.Meeting.Neg,IPM.Schedule.Meeting.Pos,IPM.Schedule.Meeting.Tent,IPM.Schedule.Meeting.Canceled</td>
         *   </tr>
         * </table>
         * 
         */
        itemClass: string;
        /**
         * Gets the Exchange Web Services item identifier for the current item.
         *
         * The itemId property is not available in compose mode. 
         * If an item identifier is required, the saveAsync method can be used to save the item to the store, which will return the item identifier 
         * in the asyncResult.value parameter in the callback function.
         *
         * **Note**: The identifier returned by the itemId property is the same as the Exchange Web Services item identifier. 
         * The itemId property is not identical to the Outlook Entry ID or the ID used by the Outlook REST API. 
         * Before making REST API calls using this value, it should be converted using Office.context.mailbox.convertToRestId. 
         * For more details, see {@link https://docs.microsoft.com/outlook/add-ins/use-rest-api#get-the-item-id | Use the Outlook REST APIs from an Outlook add-in}.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        itemId: string;
        /**
         * Gets the type of item that an instance represents.
         *
         * The itemType property returns one of the ItemType enumeration values, indicating whether the item object instance is a message or an appointment.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        itemType: MailboxEnums.ItemType | string;
        /**
         * Gets the location of an appointment.
         *
         * The location property returns a string that contains the location of the appointment.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        location: string;
        /**
         * Gets the subject of an item, with all prefixes removed (including RE: and FWD:).
         *
         * The normalizedSubject property gets the subject of the item, with any standard prefixes (such as RE: and FW:) that are added by email programs. 
         * To get the subject of the item with the prefixes intact, use the subject property.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        normalizedSubject: string;
        /**
         * Gets the notification messages for an item.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        notificationMessages: NotificationMessages;
        /**
         * Provides access to the optional attendees of an event. The type of object and level of access depends on the mode of the current item.
         *
         * The optionalAttendees property returns an array that contains an {@link Office.EmailAddressDetails} object for each optional attendee to 
         * the meeting.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        optionalAttendees: EmailAddressDetails[];
        /**
         * Gets the email address of the meeting organizer for a specified meeting.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        organizer: EmailAddressDetails;
        /**
         * Gets the recurrence pattern of an appointment. Gets the recurrence pattern of a meeting request.
         * 
         * The recurrence property returns a recurrence object for recurring appointments or meetings requests if an item is a series or an instance 
         * in a series. `null` is returned for single appointments and meeting requests of single appointments.
         * 
         * **Note**: Meeting requests have an itemClass value of IPM.Schedule.Meeting.Request.
         * 
         * **Note**: If the recurrence object is null, this indicates that the object is a single appointment or a meeting request of a single 
         * appointment and NOT a part of a series.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        recurrence: Recurrence;
        /**
         * Provides access to the required attendees of an event. The type of object and level of access depends on the mode of the current item.
         *
         * The requiredAttendees property returns an array that contains an {@link Office.EmailAddressDetails} object for each required attendee to 
         * the meeting.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        requiredAttendees: EmailAddressDetails[];
        /**
         * Gets the date and time that the appointment is to begin.
         *
         * The start property is a Date object expressed as a Coordinated Universal Time (UTC) date and time value. 
         * You can use the convertToLocalClientTime method to convert the value to the client's local date and time.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        start: Date;
        /**
         * Gets the id of the series that an instance belongs to.
         * 
         * In Outlook on the web and desktop clients, the seriesId returns the Exchange Web Services (EWS) ID of the parent (series) item that this item belongs to. 
         * However, on iOS and Android, the seriesId returns the REST ID of the parent item.
         * 
         * **Note**: The identifier returned by the seriesId property is the same as the Exchange Web Services item identifier. 
         * The seriesId property is not identical to the Outlook IDs used by the Outlook REST API. Before making REST API calls using this value, it 
         * should be converted using Office.context.mailbox.convertToRestId. 
         * For more details, see {@link https://docs.microsoft.com/outlook/add-ins/use-rest-api | Use the Outlook REST APIs from an Outlook add-in}.
         * 
         * The seriesId property returns null for items that do not have parent items such as single appointments, series items, or meeting requests 
         * and returns undefined for any other items that are not meeting requests.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        seriesId: string;
        /**
         * Gets the description that appears in the subject field of an item.
         *
         * The subject property gets or sets the entire subject of the item, as sent by the email server.
         *
         * The subject property returns a string. Use the normalizedSubject property to get the subject minus any leading prefixes such as RE: and FW:.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        subject: string;

        /**
         * Adds an event handler for a supported event.
         * 
         * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
         * `Office.EventType.RecurrenceChanged`.
         * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         * 
         * @param eventType - The event that should invoke the handler.
         * @param handler - The function to handle the event. The function must accept a single parameter, which is an object literal. 
         *                The type property on the parameter will match the eventType parameter passed to addHandlerAsync.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        addHandlerAsync(eventType: Office.EventType | string, handler: any, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Adds an event handler for a supported event.
         * 
         * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
         * `Office.EventType.RecurrenceChanged`.
         * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         * 
         * @param eventType - The event that should invoke the handler.
         * @param handler - The function to handle the event. The function must accept a single parameter, which is an object literal. 
         *                The type property on the parameter will match the eventType parameter passed to addHandlerAsync.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        addHandlerAsync(eventType: Office.EventType | string, handler: any, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Displays a reply form that includes the sender and all recipients of the selected message or the organizer and all attendees of the 
         * selected appointment.
         *
         * In Outlook on the web, the reply form is displayed as a pop-out form in the 3-column view and a pop-up form in the 2- or 1-column view.
         *
         * If any of the string parameters exceed their limits, displayReplyAllForm throws an exception.
         *
         * When attachments are specified in the formData.attachments parameter, Outlook on the web and desktop clients attempt to download all attachments and 
         * attach them to the reply form. If any attachments fail to be added, an error is shown in the form UI. 
         * If this isn't possible, then no error message is thrown.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         *
         * @param formData - A string that contains text and HTML and that represents the body of the reply form. The string is limited to 32 KB
         *                   OR an {@link Office.ReplyFormData} object that contains body or attachment data and a callback function.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        displayReplyAllForm(formData: string | ReplyFormData, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Displays a reply form that includes only the sender of the selected message or the organizer of the selected appointment.
         *
         * In Outlook on the web, the reply form is displayed as a pop-out form in the 3-column view and a pop-up form in the 2- or 1-column view.
         *
         * If any of the string parameters exceed their limits, displayReplyForm throws an exception.
         *
         * When attachments are specified in the formData.attachments parameter, Outlook on the web and desktop clients attempt to download all attachments and 
         * attach them to the reply form. If any attachments fail to be added, an error is shown in the form UI. 
         * If this isn't possible, then no error message is thrown.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         *
         * @param formData - A string that contains text and HTML and that represents the body of the reply form. The string is limited to 32 KB
         *                   OR an {@link Office.ReplyFormData} object that contains body or attachment data and a callback function.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        displayReplyForm(formData: string | ReplyFormData, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Gets initialization data passed when the add-in is {@link https://docs.microsoft.com/outlook/actionable-messages/invoke-add-in-from-actionable-message | activated by an actionable message}.
         * 
         * **Note**: This method is only supported by Outlook 2016 or later on Windows (Click-to-Run versions greater than 16.0.8413.1000) and Outlook on the web 
         * for Office 365.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         * 
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object. 
         *                 On success, the initialization data is provided in the asyncResult.value property as a string. 
         *                 If there is no initialization context, the asyncResult object will contain an Error object with its code property 
         *                 set to 9020 and its name property set to GenericResponseError.
         *
         * @beta
         */
        getInitializationContextAsync(options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Gets initialization data passed when the add-in is {@link https://docs.microsoft.com/outlook/actionable-messages/invoke-add-in-from-actionable-message | activated by an actionable message}.
         * 
         * **Note**: This method is only supported by Outlook 2016 or later on Windows (Click-to-Run versions greater than 16.0.8413.1000) and Outlook on the web for Office 365.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         * 
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object. 
         *                 On success, the initialization data is provided in the asyncResult.value property as a string. 
         *                 If there is no initialization context, the asyncResult object will contain an Error object with its code property 
         *                 set to 9020 and its name property set to GenericResponseError.
         *
         * @beta
         */
        getInitializationContextAsync(callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Gets the entities found in the selected item's body.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        getEntities(): Entities;
        /**
         * Gets an array of all the entities of the specified entity type found in the selected item's body.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         * 
         * @param entityType - One of the EntityType enumeration values.
         *
         * @returns
         * If the value passed in entityType is not a valid member of the EntityType enumeration, the method returns null. 
         * If no entities of the specified type are present in the item's body, the method returns an empty array. 
         * Otherwise, the type of the objects in the returned array depends on the type of entity requested in the entityType parameter.
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         * 
         * While the minimum permission level to use this method is Restricted, some entity types require ReadItem to access, as specified in the following table.
         * 
         * <table>
         *   <tr>
         *     <th>Value of entityType</th>
         *     <th>Type of objects in returned array</th>
         *     <th>Required Permission Level</th>
         *   </tr>
         *   <tr>
         *     <td>Address</td>
         *     <td>String</td>
         *     <td>Restricted</td>
         *   </tr>
         *   <tr>
         *     <td>Contact</td>
         *     <td>Contact</td>
         *     <td>ReadItem</td>
         *   </tr>
         *   <tr>
         *     <td>EmailAddress</td>
         *     <td>String</td>
         *     <td>ReadItem</td>
         *   </tr>
         *   <tr>
         *     <td>MeetingSuggestion</td>
         *     <td>MeetingSuggestion</td>
         *     <td>ReadItem</td>
         *   </tr>
         *   <tr>
         *     <td>PhoneNumber</td>
         *     <td>PhoneNumber</td>
         *     <td>Restricted</td>
         *   </tr>
         *   <tr>
         *     <td>TaskSuggestion</td>
         *     <td>TaskSuggestion</td>
         *     <td>ReadItem</td>
         *   </tr>
         *   <tr>
         *     <td>URL</td>
         *     <td>String</td>
         *     <td>Restricted</td>
         *   </tr>
         * </table>
         */
        getEntitiesByType(entityType: MailboxEnums.EntityType | string): (string | Contact | MeetingSuggestion | PhoneNumber | TaskSuggestion)[];
        /**
         * Returns well-known entities in the selected item that pass the named filter defined in the manifest XML file.
         *
         * The getFilteredEntitiesByName method returns the entities that match the regular expression defined in the ItemHasKnownEntity rule element 
         * in the manifest XML file with the specified FilterName element value.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         *
         * @param name - The name of the ItemHasKnownEntity rule element that defines the filter to match.
         * @returns If there is no ItemHasKnownEntity element in the manifest with a FilterName element value that matches the name parameter, 
         * the method returns null. 
         * If the name parameter does match an ItemHasKnownEntity element in the manifest, but there are no entities in the current item that match, 
         * the method return an empty array.
         */
        getFilteredEntitiesByName(name: string): (string | Contact | MeetingSuggestion | PhoneNumber | TaskSuggestion)[];
        /**
         * Returns string values in the selected item that match the regular expressions defined in the manifest XML file.
         *
         * The getRegExMatches method returns the strings that match the regular expression defined in each ItemHasRegularExpressionMatch or 
         * ItemHasKnownEntity rule element in the manifest XML file. 
         * For an ItemHasRegularExpressionMatch rule, a matching string has to occur in the property of the item that is specified by that rule. 
         * The PropertyName simple type defines the supported properties.
         *
         * If you specify an ItemHasRegularExpressionMatch rule on the body property of an item, the regular expression should further filter the body 
         * and should not attempt to return the entire body of the item. 
         * Using a regular expression such as .* to obtain the entire body of an item does not always return the expected results. 
         * Instead, use the Body.getAsync method to retrieve the entire body.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @returns
         * An object that contains arrays of strings that match the regular expressions defined in the manifest XML file. 
         * The name of each array is equal to the corresponding value of the RegExName attribute of the matching ItemHasRegularExpressionMatch rule 
         * or the FilterName attribute of the matching ItemHasKnownEntity rule.
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        getRegExMatches(): any;
        /**
         * Returns string values in the selected item that match the named regular expression defined in the manifest XML file.
         *
         * The getRegExMatchesByName method returns the strings that match the regular expression defined in the ItemHasRegularExpressionMatch rule 
         * element in the manifest XML file with the specified RegExName element value.
         *
         * If you specify an ItemHasRegularExpressionMatch rule on the body property of an item, the regular expression should further filter the body 
         * and should not attempt to return the entire body of the item. 
         * Using a regular expression such as .* to obtain the entire body of an item does not always return the expected results.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @returns
         * An array that contains the strings that match the regular expression defined in the manifest XML file.
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         *
         * @param name - The name of the ItemHasRegularExpressionMatch rule element that defines the filter to match.
         */
        getRegExMatchesByName(name: string): string[];
        /**
         * Gets the entities found in a highlighted match a user has selected. Highlighted matches apply to contextual add-ins.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.6]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         *
         * @param name - The name of the ItemHasRegularExpressionMatch rule element that defines the filter to match.
         */
        getSelectedEntities(): Entities;
        /**
         * Returns string values in a highlighted match that match the regular expressions defined in the manifest XML file. 
         * Highlighted matches apply to contextual add-ins.
         *
         * The getSelectedRegExMatches method returns the strings that match the regular expression defined in each ItemHasRegularExpressionMatch or 
         * ItemHasKnownEntity rule element in the manifest XML file. 
         * For an ItemHasRegularExpressionMatch rule, a matching string has to occur in the property of the item that is specified by that rule. 
         * The PropertyName simple type defines the supported properties.
         *
         * If you specify an ItemHasRegularExpressionMatch rule on the body property of an item, the regular expression should further filter the body 
         * and should not attempt to return the entire body of the item. 
         * Using a regular expression such as .* to obtain the entire body of an item does not always return the expected results. 
         * Instead, use the Body.getAsync method to retrieve the entire body.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.6]
         *
         * @returns
         * An object that contains arrays of strings that match the regular expressions defined in the manifest XML file. 
         * The name of each array is equal to the corresponding value of the RegExName attribute of the matching ItemHasRegularExpressionMatch rule 
         * or the FilterName attribute of the matching ItemHasKnownEntity rule.
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
         */
        getSelectedRegExMatches(): any;
       /**
        * Asynchronously loads custom properties for this add-in on the selected item.
        *
        * Custom properties are stored as key/value pairs on a per-app, per-item basis. 
        * This method returns a CustomProperties object in the callback, which provides methods to access the custom properties specific to the 
        * current item and the current add-in. Custom properties are not encrypted on the item, so this should not be used as secure storage.
        *
        * The custom properties are provided as a CustomProperties object in the asyncResult.value property. 
        * This object can be used to get, set, and remove custom properties from the item and save changes to the custom property set back to 
        * the server.
        *
        * [Api set: Mailbox 1.0]
        *
        * @remarks
        *
        * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
        * 
        * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
        *
        * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
        *                 type Office.AsyncResult.
        * @param userContext - Optional. Developers can provide any object they wish to access in the callback function. 
        *                    This object can be accessed by the asyncResult.asyncContext property in the callback function.
        */
       loadCustomPropertiesAsync(callback: (asyncResult: Office.AsyncResult<CustomProperties>) => void, userContext?: any): void;

       /**
        * Removes the event handlers for a supported event type.
        * 
        * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
        * `Office.EventType.RecurrenceChanged`.
        * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
        * 
        * [Api set: Mailbox 1.7]
        *
        * @remarks
        *
        * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
        * 
        * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
        * 
        * @param eventType - The event that should revoke the handler.
        * @param options - Optional. An object literal that contains one or more of the following properties.
        *        asyncContext: Developers can provide any object they wish to access in the callback method.
        * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
        *                asyncResult, which is an Office.AsyncResult object.
        */
       removeHandlerAsync(eventType: Office.EventType | string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
       /**
        * Removes the event handlers for a supported event type.
        * 
        * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
        * `Office.EventType.RecurrenceChanged`.
        * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
        * 
        * [Api set: Mailbox 1.7]
        *
        * @remarks
        *
        * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
        * 
        * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Appointment Attendee
        * 
        * @param eventType - The event that should revoke the handler.
        * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
        *                asyncResult, which is an Office.AsyncResult object.
        */
       removeHandlerAsync(eventType: Office.EventType | string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void; 
    }

    /**
     * The item namespace is used to access the currently selected message, meeting request, or appointment. 
     * You can determine the type of the item by using the `itemType` property.
     *
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     */
    interface Item {
        /**
         * Gets an object that provides methods for manipulating the body of an item.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        body: Body;
        /**
         * Gets the type of item that an instance represents.
         *
         * The itemType property returns one of the ItemType enumeration values, indicating whether the item object instance is a message or 
         * an appointment.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        itemType: MailboxEnums.ItemType | string;
        /**
         * Gets the notification messages for an item.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        notificationMessages: NotificationMessages;

        /**
         * Gets the id of the series that an instance belongs to.
         * 
         * In Outlook on the web and desktop clients, the seriesId returns the Exchange Web Services (EWS) ID of the parent (series) item that this item belongs to. 
         * However, on iOS and Android, the seriesId returns the REST ID of the parent item.
         * 
         * **Note**: The identifier returned by the seriesId property is the same as the Exchange Web Services item identifier. 
         * The seriesId property is not identical to the Outlook IDs used by the Outlook REST API. 
         * Before making REST API calls using this value, it should be converted using Office.context.mailbox.convertToRestId. 
         * For more details, see {@link https://docs.microsoft.com/outlook/add-ins/use-rest-api | Use the Outlook REST APIs from an Outlook add-in}.
         * 
         * The seriesId property returns null for items that do not have parent items such as single appointments, series items, or meeting requests 
         * and returns undefined for any other items that are not meeting requests.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        seriesId: string;

        /**
         * Adds an event handler for a supported event.
         * 
         * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
         * `Office.EventType.RecurrenceChanged`.
         * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @param eventType - The event that should invoke the handler.
         * @param handler - The function to handle the event. The function must accept a single parameter, which is an object literal. 
         *                The type property on the parameter will match the eventType parameter passed to addHandlerAsync.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        addHandlerAsync(eventType: Office.EventType | string, handler: any, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;

        /**
         * Adds an event handler for a supported event.
         * 
         * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
         * `Office.EventType.RecurrenceChanged`.
         * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @param eventType - The event that should invoke the handler.
         * @param handler - The function to handle the event. The function must accept a single parameter, which is an object literal. 
         *                The type property on the parameter will match the eventType parameter passed to addHandlerAsync.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        addHandlerAsync(eventType: Office.EventType | string, handler: any, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;

        /**
         * Gets an attachment from a message or appointment and returns it as an **AttachmentContent** object.
         * 
         * The `getAttachmentContentAsync` method gets the attachment with the specified identifier from the item. As a best practice, you should use 
         * the identifier to retrieve an attachment in the same session that the attachmentIds were retrieved with the `getAttachmentsAsync` or 
         * `item.attachments` call. In Outlook on the web and mobile devices, the attachment identifier is valid only within the same session. 
         * A session is over when the user closes the app, or if the user starts composing an inline form then subsequently pops out the form to 
         * continue in a separate window.
         * 
         * [Api set: Mailbox Preview]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * **Errors**: 
         * 
         *  - InvalidAttachmentId: The attachment identifier does not exist.
         * 
         * @param attachmentId - The identifier of the attachment you want to get. 
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object. If the call fails, the asyncResult.error property will contain and error code 
         *                 with the reason for the failure.
         * 
         * @beta
         */
        getAttachmentContentAsync(attachmentId: string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<AttachmentContent>) => void): void;

        /**
         * Gets an attachment from a message or appointment and returns it as an **AttachmentContent** object.
         * 
         * The `getAttachmentContentAsync` method gets the attachment with the specified identifier from the item. As a best practice, you should use 
         * the identifier to retrieve an attachment in the same session that the attachmentIds were retrieved with the `getAttachmentsAsync` or 
         * `item.attachments` call. In Outlook on the web and mobile devices, the attachment identifier is valid only within the same session. 
         * A session is over when the user closes the app, or if the user starts composing an inline form then subsequently pops out the form to 
         * continue in a separate window.
         * 
         * [Api set: Mailbox Preview]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * **Errors**: 
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         * 
         * @param attachmentId - The identifier of the attachment you want to get. 
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object. If the call fails, the asyncResult.error property will contain and error code 
         *                 with the reason for the failure.
         * 
         * @beta
         */
        getAttachmentContentAsync(attachmentId: string, callback?: (asyncResult: Office.AsyncResult<AttachmentContent>) => void): void;
		
        /**
         * Gets initialization data passed when the add-in is {@link https://docs.microsoft.com/outlook/actionable-messages/invoke-add-in-from-actionable-message | activated by an actionable message}.
         * 
         * **Note**: This method is only supported by Outlook 2016 or later on Windows (Click-to-Run versions greater than 16.0.8413.1000) and Outlook on the web 
         * for Office 365.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object. 
         *                 On success, the initialization data is provided in the asyncResult.value property as a string. 
         *                 If there is no initialization context, the asyncResult object will contain an Error object with its code property 
         *                 set to 9020 and its name property set to GenericResponseError.
         *
         * @beta
         */
        getInitializationContextAsync(options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
		
        /**
         * Gets initialization data passed when the add-in is {@link https://docs.microsoft.com/outlook/actionable-messages/invoke-add-in-from-actionable-message | activated by an actionable message}.
         * 
         * **Note**: This method is only supported by Outlook 2016 or later on Windows (Click-to-Run versions greater than 16.0.8413.1000) and Outlook on the web 
         * for Office 365.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object. 
         *                 On success, the initialization data is provided in the asyncResult.value property as a string. 
         *                 If there is no initialization context, the asyncResult object will contain an Error object with its code property 
         *                 set to 9020 and its name property set to GenericResponseError.
         *
         * @beta
         */
        getInitializationContextAsync(callback?: (asyncResult: Office.AsyncResult<string>) => void): void;

       /**
        * Gets the properties of an appointment or message in a shared folder, calendar, or mailbox.
        *
        * [Api set: Mailbox Preview]
        *
        * @remarks
        * 
        * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
        * 
        * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
        * 
        * @param options - An object literal that contains one or more of the following properties.
        *        asyncContext: Developers can provide any object they wish to access in the callback method.
        * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
        *                 type Office.AsyncResult.
        *                 The `value` property of the result is the properties of the shared item.
        * 
        * @beta
        */
        getSharedPropertiesAsync(options: Office.AsyncContextOptions, callback: (asyncResult: Office.AsyncResult<SharedProperties>) => void): void;

       /**
        * Gets the properties of an appointment or message in a shared folder, calendar, or mailbox.
        *
        * [Api set: Mailbox Preview]
        *
        * @remarks
        * 
        * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
        * 
        * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
        * 
        * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
        *                 type Office.AsyncResult.
        *                 The `value` property of the result is the properties of the shared item.
        * 
        * @beta
        */
       getSharedPropertiesAsync(callback: (asyncResult: Office.AsyncResult<SharedProperties>) => void): void;

       /**
        * Asynchronously loads custom properties for this add-in on the selected item.
        *
        * Custom properties are stored as key/value pairs on a per-app, per-item basis. 
        * This method returns a CustomProperties object in the callback, which provides methods to access the custom properties specific to the 
        * current item and the current add-in. Custom properties are not encrypted on the item, so this should not be used as secure storage.
        *
        * The custom properties are provided as a CustomProperties object in the asyncResult.value property. 
        * This object can be used to get, set, and remove custom properties from the item and save changes to the custom property set back to 
        * the server.
        *
        * [Api set: Mailbox 1.0]
        *
        * @remarks
        *
        * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
        * 
        * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
        * </table>        *
        * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
        *                 type Office.AsyncResult.
        * @param userContext - Optional. Developers can provide any object they wish to access in the callback function. 
        *                    This object can be accessed by the asyncResult.asyncContext property in the callback function.
        */
       loadCustomPropertiesAsync(callback: (asyncResult: Office.AsyncResult<CustomProperties>) => void, userContext?: any): void;


       /**
        * Removes the event handlers for a supported event type.
        * 
        * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
        * `Office.EventType.RecurrenceChanged`.
        * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
        * 
        * [Api set: Mailbox 1.7]
        *
        * @remarks
        *
        * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
        * 
        * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
        * 
        * @param eventType - The event that should revoke the handler.
        * @param options - Optional. An object literal that contains one or more of the following properties.
        *        asyncContext: Developers can provide any object they wish to access in the callback method.
        * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
        *                asyncResult, which is an Office.AsyncResult object.
        */
       removeHandlerAsync(eventType: Office.EventType | string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;

       /**
        * Removes the event handlers for a supported event type.
        * 
        * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
        * `Office.EventType.RecurrenceChanged`.
        * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
        * 
        * [Api set: Mailbox 1.7]
        *
        * @remarks
        *
        * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
        * 
        * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
        * 
        * @param eventType - The event that should revoke the handler.
        * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
        *                asyncResult, which is an Office.AsyncResult object.
        */
       removeHandlerAsync(eventType: Office.EventType | string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
    }
    /**
     * The compose mode of {@link Office.Item | Office.context.mailbox.item}.
     * 
     * **Important**: This is an internal Outlook object, not directly exposed through existing interfaces. 
     * You should treat this as a mode of Office.context.mailbox.item. Refer to the
     * {@link https://docs.microsoft.com/office/dev/add-ins/reference/objectmodel/preview-requirement-set/office.context.mailbox.item | Object Model} page for more information.
     */
    interface ItemCompose extends Item {
        /**
         * Gets or sets the description that appears in the subject field of an item.
         *
         * The subject property gets or sets the entire subject of the item, as sent by the email server.
         *
         * The subject property returns a Subject object that provides methods to get and set the subject.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         */
        subject: Subject;

        /**
         * Adds a file to a message or appointment as an attachment.
         *
         * The addFileAttachmentAsync method uploads the file at the specified URI and attaches it to the item in the compose form.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - AttachmentSizeExceeded: The attachment is larger than allowed.
         * 
         * - FileTypeNotSupported: The attachment has an extension that is not allowed.
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         * 
         * @param uri - The URI that provides the location of the file to attach to the message or appointment. The maximum length is 2048 characters.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         *        isInline: If true, indicates that the attachment will be shown inline in the message body, and should not be displayed in the 
         *        attachment list.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                 If uploading the attachment fails, the asyncResult object will contain an Error object that provides a description of 
         *                 the error.
         */
        addFileAttachmentAsync(uri: string, attachmentName: string, options?: Office.AsyncContextOptions & { isInline: boolean }, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Adds a file to a message or appointment as an attachment.
         *
         * The addFileAttachmentAsync method uploads the file at the specified URI and attaches it to the item in the compose form.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - AttachmentSizeExceeded: The attachment is larger than allowed.
         * 
         * - FileTypeNotSupported: The attachment has an extension that is not allowed.
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         *
         * @param uri - The URI that provides the location of the file to attach to the message or appointment. The maximum length is 2048 characters.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                 If uploading the attachment fails, the asyncResult object will contain an Error object that provides a description of 
         *                 the error.
         */
        addFileAttachmentAsync(uri: string, attachmentName: string, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;

        /**
         * Adds a file to a message or appointment as an attachment.
         *
         * The addFileAttachmentFromBase64Async method uploads the file from the base64 encoding and attaches it to the item in the compose form. This method returns the attachment identifier in the asyncResult.value object.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * [Api set: Mailbox Preview]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - AttachmentSizeExceeded: The attachment is larger than allowed.
         * 
         * - FileTypeNotSupported: The attachment has an extension that is not allowed.
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         * 
         * @param base64File - The base64 encoded content of an image or file to be added to an email or event.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         *        isInline: If true, indicates that the attachment will be shown inline in the message body and should not be displayed in the attachment list.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult. 
         *                  On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                  If uploading the attachment fails, the asyncResult object will contain an Error object that provides a description of the error.
         * 
         * @beta
         */
        addFileAttachmentFromBase64Async(base64File: string, attachmentName: string, options?: Office.AsyncContextOptions & { isInline: boolean }, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Adds a file to a message or appointment as an attachment.
         *
         * The addFileAttachmentFromBase64Async method uploads the file from the base64 encoding and attaches it to the item in the compose form. This method returns the attachment identifier in the asyncResult.value object.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * [Api set: Mailbox Preview]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - AttachmentSizeExceeded: The attachment is larger than allowed.
         * 
         * - FileTypeNotSupported: The attachment has an extension that is not allowed.
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         * 
         * @param base64File - The base64 encoded content of an image or file to be added to an email or event.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult. 
         *                  On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                  If uploading the attachment fails, the asyncResult object will contain an Error object that provides a description of the error.
         * 
         * @beta
         */
        addFileAttachmentFromBase64Async(base64File: string, attachmentName: string, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;

        /**
         * Adds an Exchange item, such as a message, as an attachment to the message or appointment.
         *
         * The addItemAttachmentAsync method attaches the item with the specified Exchange identifier to the item in the compose form. 
         * If you specify a callback method, the method is called with one parameter, asyncResult, which contains either the attachment identifier or 
         * a code that indicates any error that occurred while attaching the item. You can use the options parameter to pass state information to the 
         * callback method, if needed.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * If your Office add-in is running in Outlook on the web, the addItemAttachmentAsync method can attach items to items other than the item that 
         * you are editing; however, this is not supported and is not recommended.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         * 
         * @param itemId - The Exchange identifier of the item to attach. The maximum length is 100 characters.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                 If adding the attachment fails, the asyncResult object will contain an Error object that provides a description of 
         *                 the error.
         */
        addItemAttachmentAsync(itemId: any, attachmentName: string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Adds an Exchange item, such as a message, as an attachment to the message or appointment.
         *
         * The addItemAttachmentAsync method attaches the item with the specified Exchange identifier to the item in the compose form. 
         * If you specify a callback method, the method is called with one parameter, asyncResult, which contains either the attachment identifier or 
         * a code that indicates any error that occurred while attaching the item. You can use the options parameter to pass state information to the 
         * callback method, if needed.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * If your Office add-in is running in Outlook on the web, the addItemAttachmentAsync method can attach items to items other than the item that 
         * you are editing; however, this is not supported and is not recommended.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         *
         * @param itemId - The Exchange identifier of the item to attach. The maximum length is 100 characters.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                 If adding the attachment fails, the asyncResult object will contain an Error object that provides a description of 
         *                 the error.
         */
        addItemAttachmentAsync(itemId: any, attachmentName: string, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;

        /**
         * Closes the current item that is being composed
         *
         * The behaviors of the close method depends on the current state of the item being composed. 
         * If the item has unsaved changes, the client prompts the user to save, discard, or close the action.
         *
         * In the Outlook desktop client, if the message is an inline reply, the close method has no effect.
         *
         * **Note**: In Outlook on the web, if the item is an appointment and it has previously been saved using saveAsync, the user is prompted to save, 
         * discard, or cancel even if no changes have occurred since the item was last saved.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         */
        close(): void;
        /**
         * Gets the item's attachments as an array.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. If the call fails, the asyncResult.error property will contain and error code with the reason for 
         *                 the failure.
         * 
         * @beta
         */
        getAttachmentsAsync(options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<AttachmentDetails[]>) => void): void;
        /**
         * Gets the item's attachments as an array.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. If the call fails, the asyncResult.error property will contain and error code with the reason for 
         *                 the failure.
         * 
         * @beta
         */
        getAttachmentsAsync(callback?: (asyncResult: Office.AsyncResult<AttachmentDetails[]>) => void): void;
        /**
         * Gets initialization data passed when the add-in is activated by an actionable message.
         *
         * **Note**: This method is only supported by Outlook 2016 or later on Windows (Click-to-Run versions greater than 16.0.8413.1000) and Outlook on the web for Office 365.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         *
         * More information on {@link https://docs.microsoft.com/outlook/actionable-messages/invoke-add-in-from-actionable-message | actionable messages}.
         *
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         *                 On success, the initialization data is provided in the asyncResult.value property as a string. 
         *                 If there is no initialization context, the asyncResult object will contain an Error object with its code property 
         *                 set to 9020 and its name property set to GenericResponseError.
         *
         * @beta
         */
        getInitializationContextAsync(options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Gets initialization data passed when the add-in is activated by an actionable message.
         *
         * **Note**: This method is only supported by Outlook 2016 or later on Windows (Click-to-Run versions greater than 16.0.8413.1000) and Outlook on the web for Office 365.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         *
         * More information on {@link https://docs.microsoft.com/outlook/actionable-messages/invoke-add-in-from-actionable-message | actionable messages}.
         *
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         *                 On success, the initialization data is provided in the asyncResult.value property as a string. 
         *                 If there is no initialization context, the asyncResult object will contain an Error object with its code property 
         *                 set to 9020 and its name property set to GenericResponseError.
         *
         * @beta
         */
        getInitializationContextAsync(callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Asynchronously returns selected data from the subject or body of a message.
         *
         * If there is no selection but the cursor is in the body or subject, the method returns null for the selected data. 
         * If a field other than the body or subject is selected, the method returns the InvalidSelection error.
         *
         * To access the selected data from the callback method, call asyncResult.value.data.
         * To access the source property that the selection comes from, call asyncResult.value.sourceProperty, which will be either body or subject.
         *
         * [Api set: Mailbox 1.2]
         *
         * @returns
         * The selected data as a string with format determined by coercionType.
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         *
         * @param coercionType - Requests a format for the data. If Text, the method returns the plain text as a string, removing any HTML tags present. 
         *                     If HTML, the method returns the selected text, whether it is plaintext or HTML.
         * @param options - An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        getSelectedDataAsync(coercionType: Office.CoercionType | string, options: Office.AsyncContextOptions, callback: (asyncResult: Office.AsyncResult<any>) => void): void;
        /**
         * Asynchronously returns selected data from the subject or body of a message.
         *
         * If there is no selection but the cursor is in the body or subject, the method returns null for the selected data. 
         * If a field other than the body or subject is selected, the method returns the InvalidSelection error.
         *
         * To access the selected data from the callback method, call asyncResult.value.data.
         * To access the source property that the selection comes from, call asyncResult.value.sourceProperty, which will be either body or subject.
         *
         * [Api set: Mailbox 1.2]
         *
         * @returns
         * The selected data as a string with format determined by coercionType.
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         *
         * @param coercionType - Requests a format for the data. If Text, the method returns the plain text as a string , removing any HTML tags present. 
         *                     If HTML, the method returns the selected text, whether it is plaintext or HTML.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        getSelectedDataAsync(coercionType: Office.CoercionType | string, callback: (asyncResult: Office.AsyncResult<any>) => void): void;
        /**
         * Removes an attachment from a message or appointment.
         *
         * The removeAttachmentAsync method removes the attachment with the specified identifier from the item. 
         * As a best practice, you should use the attachment identifier to remove an attachment only if the same mail app has added that attachment 
         * in the same session. In Outlook on the web and mobile devices, the attachment identifier is valid only within the same session. 
         * A session is over when the user closes the app, or if the user starts composing an inline form then subsequently pops out the form to 
         * continue in a separate window.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         * 
         * @param attachmentId - The identifier of the attachment to remove.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         *                 If removing the attachment fails, the asyncResult.error property will contain an error code with the reason for the failure.
         */
        removeAttachmentAsync(attachmentId: string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Removes an attachment from a message or appointment.
         *
         * The removeAttachmentAsync method removes the attachment with the specified identifier from the item. 
         * As a best practice, you should use the attachment identifier to remove an attachment only if the same mail app has added that attachment 
         * in the same session. In Outlook on the web and mobile devices, the attachment identifier is valid only within the same session. 
         * A session is over when the user closes the app, or if the user starts composing an inline form then subsequently pops out the form to 
         * continue in a separate window.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**:
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         *
         * @param attachmentId - The identifier of the attachment to remove.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         *                 If removing the attachment fails, the asyncResult.error property will contain an error code with the reason for the failure.
         */
        removeAttachmentAsync(attachmentId: string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;

        /**
         * Asynchronously saves an item.
         *
         * When invoked, this method saves the current message as a draft and returns the item id via the callback method. 
         * In Outlook on the web or Outlook in online mode, the item is saved to the server. 
         * In Outlook in cached mode, the item is saved to the local cache.
         *
         * Since appointments have no draft state, if saveAsync is called on an appointment in compose mode, the item will be saved as a normal 
         * appointment on the user's calendar. For new appointments that have not been saved before, no invitation will be sent. 
         * Saving an existing appointment will send an update to added or removed attendees.
         *
         * **Note**: If your add-in calls saveAsync on an item in compose mode in order to get an itemId to use with EWS or the REST API, be aware that 
         * when Outlook is in cached mode, it may take some time before the item is actually synced to the server. 
         * Until the item is synced, using the itemId will return an error.
         *
         * **Note**: The following clients have different behavior for saveAsync on appointments in compose mode:
         *
         * - Outlook on Mac does not support saving a meeting. The saveAsync method fails when called from a meeting in compose mode.
         * See {@link https://support.microsoft.com/help/4505745 | Cannot save a meeting as a draft in Outlook for Mac by using Office JS API} for a workaround.
         *
         * - Outlook on the web always sends an invitation or update when saveAsync is called on an appointment in compose mode.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         * 
         * @param options - An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         *                 If removing the attachment fails, the asyncResult.error property will contain an error code with the reason for the failure.
         */
        saveAsync(options: Office.AsyncContextOptions, callback: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Asynchronously saves an item.
         *
         * When invoked, this method saves the current message as a draft and returns the item id via the callback method. 
         * In Outlook on the web or Outlook in online mode, the item is saved to the server. 
         * In Outlook in cached mode, the item is saved to the local cache.
         *
         * Since appointments have no draft state, if saveAsync is called on an appointment in compose mode, the item will be saved as a normal 
         * appointment on the user's calendar. For new appointments that have not been saved before, no invitation will be sent. 
         * Saving an existing appointment will send an update to added or removed attendees.
         *
         * **Note**: If your add-in calls saveAsync on an item in compose mode in order to get an itemId to use with EWS or the REST API, be aware that 
         * when Outlook is in cached mode, it may take some time before the item is actually synced to the server. 
         * Until the item is synced, using the itemId will return an error.
         *
         * **Note**: The following clients have different behavior for saveAsync on appointments in compose mode:
         *
         * - Outlook on Mac does not support saving a meeting. The saveAsync method fails when called from a meeting in compose mode.
         * See {@link https://support.microsoft.com/help/4505745 | Cannot save a meeting as a draft in Outlook for Mac by using Office JS API} for a workaround.
         *
         * - Outlook on the web always sends an invitation or update when saveAsync is called on an appointment in compose mode.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         *
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         *                 If removing the attachment fails, the asyncResult.error property will contain an error code with the reason for the failure.
         */
        saveAsync(callback: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Asynchronously inserts data into the body or subject of a message.
         *
         * The setSelectedDataAsync method inserts the specified string at the cursor location in the subject or body of the item, or, if text is 
         * selected in the editor, it replaces the selected text. If the cursor is not in the body or subject field, an error is returned. 
         * After insertion, the cursor is placed at the end of the inserted content.
         *
         * [Api set: Mailbox 1.2]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**:
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         * 
         * @param data - The data to be inserted. Data is not to exceed 1,000,000 characters. 
         *             If more than 1,000,000 characters are passed in, an ArgumentOutOfRange exception is thrown.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         *        coercionType: If text, the current style is applied in Outlook on the web and desktop clients. 
         *        If the field is an HTML editor, only the text data is inserted, even if the data is HTML. 
         *        If html and the field supports HTML (the subject doesn't), the current style is applied in Outlook on the web and the default style is 
         *        applied in Outlook on desktop clients.
         *        If the field is a text field, an InvalidDataFormat error is returned. 
         *        If coercionType is not set, the result depends on the field: if the field is HTML then HTML is used; 
         *        if the field is text, then plain text is used.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         */
        setSelectedDataAsync(data: string, options?: Office.AsyncContextOptions & CoercionTypeOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Asynchronously inserts data into the body or subject of a message.
         *
         * The setSelectedDataAsync method inserts the specified string at the cursor location in the subject or body of the item, or, if text is 
         * selected in the editor, it replaces the selected text. If the cursor is not in the body or subject field, an error is returned. 
         * After insertion, the cursor is placed at the end of the inserted content.
         *
         * [Api set: Mailbox 1.2]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         *
         * @param data - The data to be inserted. Data is not to exceed 1,000,000 characters. 
         *             If more than 1,000,000 characters are passed in, an ArgumentOutOfRange exception is thrown.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         */
        setSelectedDataAsync(data: string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
    }
    /**
     * The read mode of {@link Office.Item | Office.context.mailbox.item}.
     * 
     * **Important**: This is an internal Outlook object, not directly exposed through existing interfaces. 
     * You should treat this as a mode of Office.context.mailbox.item. Refer to the
     * {@link https://docs.microsoft.com/office/dev/add-ins/reference/objectmodel/preview-requirement-set/office.context.mailbox.item | Object Model} page for more information.
     */
    interface ItemRead extends Item {
        /**
         * Gets the item's attachments as an array.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         *
         * **Note**: Certain types of files are blocked by Outlook due to potential security issues and are therefore not returned. 
         * For more information, see 
         * {@link https://support.office.com/article/Blocked-attachments-in-Outlook-434752E1-02D3-4E90-9124-8B81E49A8519 | Blocked attachments in Outlook}.
         *
         */
        attachments: AttachmentDetails[];
        /**
         * Gets the Exchange Web Services item class of the selected item.
         *
         *
         * You can create custom message classes that extends a default message class, for example, a custom appointment message class 
         * IPM.Appointment.Contoso.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         * 
         * The itemClass property specifies the message class of the selected item. The following are the default message classes for the message or 
         * appointment item.
         * 
         * <table>
         *   <tr>
         *     <th>Type</th>
         *     <th>Description</th>
         *     <th>Item Class</th>
         *   </tr>
         *   <tr>
         *     <td>Appointment items</td>
         *     <td>These are calendar items of the item class IPM.Appointment or IPM.Appointment.Occurrence.</td>
         *     <td>IPM.Appointment,IPM.Appointment.Occurrence</td>
         *   </tr>
         *   <tr>
         *     <td>Message items</td>
         *     <td>These include email messages that have the default message class IPM.Note, and meeting requests, responses, and cancellations, that use IPM.Schedule.Meeting as the base message class.</td>
         *     <td>IPM.Note,IPM.Schedule.Meeting.Request,IPM.Schedule.Meeting.Neg,IPM.Schedule.Meeting.Pos,IPM.Schedule.Meeting.Tent,IPM.Schedule.Meeting.Canceled</td>
         *   </tr>
         * </table>
         */
        itemClass: string;
        /**
         * Gets the Exchange Web Services item identifier for the current item.
         *
         * The itemId property is not available in compose mode. 
         * If an item identifier is required, the saveAsync method can be used to save the item to the store, which will return the item identifier 
         * in the asyncResult.value parameter in the callback function.
         *
         * **Note**: The identifier returned by the itemId property is the same as the Exchange Web Services item identifier. 
         * The itemId property is not identical to the Outlook Entry ID or the ID used by the Outlook REST API. 
         * Before making REST API calls using this value, it should be converted using Office.context.mailbox.convertToRestId. 
         * For more details, see {@link https://docs.microsoft.com/outlook/add-ins/use-rest-api#get-the-item-id | Use the Outlook REST APIs from an Outlook add-in}.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         */
        itemId: string;
        /**
         * Gets the subject of an item, with all prefixes removed (including RE: and FWD:).
         *
         * The normalizedSubject property gets the subject of the item, with any standard prefixes (such as RE: and FW:) that are added by 
         * email programs. To get the subject of the item with the prefixes intact, use the subject property.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         */
        normalizedSubject: string;
        /**
         * Gets the description that appears in the subject field of an item.
         *
         * The subject property gets or sets the entire subject of the item, as sent by the email server.
         *
         * The subject property returns a string. Use the normalizedSubject property to get the subject minus any leading prefixes such as RE: and FW:.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         */
        subject: string;
        /**
         * Displays a reply form that includes the sender and all recipients of the selected message or the organizer and all attendees of the 
         * selected appointment.
         *
         * In Outlook on the web, the reply form is displayed as a pop-out form in the 3-column view and a pop-up form in the 2- or 1-column view.
         *
         * If any of the string parameters exceed their limits, displayReplyAllForm throws an exception.
         *
         * When attachments are specified in the formData.attachments parameter, Outlook on the web and desktop clients attempt to download all attachments and 
         * attach them to the reply form. If any attachments fail to be added, an error is shown in the form UI. 
         * If this isn't possible, then no error message is thrown.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         *
         * @param formData - A string that contains text and HTML and that represents the body of the reply form. The string is limited to 32 KB
         *                   OR an {@link Office.ReplyFormData} object that contains body or attachment data and a callback function.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        displayReplyAllForm(formData: string | ReplyFormData, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Displays a reply form that includes only the sender of the selected message or the organizer of the selected appointment.
         *
         * In Outlook on the web, the reply form is displayed as a pop-out form in the 3-column view and a pop-up form in the 2- or 1-column view.
         *
         * If any of the string parameters exceed their limits, displayReplyForm throws an exception.
         *
         * When attachments are specified in the formData.attachments parameter, Outlook on the web and desktop clients attempt to download all attachments and 
         * attach them to the reply form. If any attachments fail to be added, an error is shown in the form UI. 
         * If this isn't possible, then no error message is thrown.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         *
         * @param formData - A string that contains text and HTML and that represents the body of the reply form. The string is limited to 32 KB
         *                   OR an {@link Office.ReplyFormData} object that contains body or attachment data and a callback function.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        displayReplyForm(formData: string | ReplyFormData, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Gets initialization data passed when the add-in is {@link https://docs.microsoft.com/outlook/actionable-messages/invoke-add-in-from-actionable-message | activated by an actionable message}.
         * 
         * **Note**: This method is only supported by Outlook 2016 or later on Windows (Click-to-Run versions greater than 16.0.8413.1000) and Outlook on the web 
         * for Office 365.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         * 
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object. 
         *                 On success, the initialization data is provided in the asyncResult.value property as a string. 
         *                 If there is no initialization context, the asyncResult object will contain an Error object with its code property 
         *                 set to 9020 and its name property set to GenericResponseError.
         *
         * @beta
         */
        getInitializationContextAsync(options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Gets initialization data passed when the add-in is {@link https://docs.microsoft.com/outlook/actionable-messages/invoke-add-in-from-actionable-message | activated by an actionable message}.
         * 
         * **Note**: This method is only supported by Outlook 2016 or later on Windows (Click-to-Run versions greater than 16.0.8413.1000) and Outlook on the web 
         * for Office 365.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         * 
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object. 
         *                 On success, the initialization data is provided in the asyncResult.value property as a string. 
         *                 If there is no initialization context, the asyncResult object will contain an Error object with its code property 
         *                 set to 9020 and its name property set to GenericResponseError.
         *
         * @beta
         */
        getInitializationContextAsync(callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Gets the entities found in the selected item's body.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         */
        getEntities(): Entities;
        /**
         * Gets an array of all the entities of the specified entity type found in the selected item's body.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         * 
         * @param entityType - One of the EntityType enumeration values.
         *
         * @returns
         * If the value passed in entityType is not a valid member of the EntityType enumeration, the method returns null. 
         * If no entities of the specified type are present in the item's body, the method returns an empty array. 
         * Otherwise, the type of the objects in the returned array depends on the type of entity requested in the entityType parameter.
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         * 
         * While the minimum permission level to use this method is Restricted, some entity types require ReadItem to access, as specified in the 
         * following table.
         * 
         * <table>
         *   <tr>
         *     <th>Value of entityType</th>
         *     <th>Type of objects in returned array</th>
         *     <th>Required Permission Level</th>
         *   </tr>
         *   <tr>
         *     <td>Address</td>
         *     <td>String</td>
         *     <td>Restricted</td>
         *   </tr>
         *   <tr>
         *     <td>Contact</td>
         *     <td>Contact</td>
         *     <td>ReadItem</td>
         *   </tr>
         *   <tr>
         *     <td>EmailAddress</td>
         *     <td>String</td>
         *     <td>ReadItem</td>
         *   </tr>
         *   <tr>
         *     <td>MeetingSuggestion</td>
         *     <td>MeetingSuggestion</td>
         *     <td>ReadItem</td>
         *   </tr>
         *   <tr>
         *     <td>PhoneNumber</td>
         *     <td>PhoneNumber</td>
         *     <td>Restricted</td>
         *   </tr>
         *   <tr>
         *     <td>TaskSuggestion</td>
         *     <td>TaskSuggestion</td>
         *     <td>ReadItem</td>
         *   </tr>
         *   <tr>
         *     <td>URL</td>
         *     <td>String</td>
         *     <td>Restricted</td>
         *   </tr>
         * </table>
         */
        getEntitiesByType(entityType: MailboxEnums.EntityType | string): (string | Contact | MeetingSuggestion | PhoneNumber | TaskSuggestion)[];
        /**
         * Returns well-known entities in the selected item that pass the named filter defined in the manifest XML file.
         *
         * The getFilteredEntitiesByName method returns the entities that match the regular expression defined in the ItemHasKnownEntity rule element 
         * in the manifest XML file with the specified FilterName element value.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         *
         * @param name - The name of the ItemHasKnownEntity rule element that defines the filter to match.
         * @returns If there is no ItemHasKnownEntity element in the manifest with a FilterName element value that matches the name parameter, 
         * the method returns null. 
         * If the name parameter does match an ItemHasKnownEntity element in the manifest, but there are no entities in the current item that match, 
         * the method return an empty array.
         */
        getFilteredEntitiesByName(name: string): (string | Contact | MeetingSuggestion | PhoneNumber | TaskSuggestion)[];
        /**
         * Returns string values in the selected item that match the regular expressions defined in the manifest XML file.
         *
         * The getRegExMatches method returns the strings that match the regular expression defined in each ItemHasRegularExpressionMatch or 
         * ItemHasKnownEntity rule element in the manifest XML file. 
         * For an ItemHasRegularExpressionMatch rule, a matching string has to occur in the property of the item that is specified by that rule. 
         * The PropertyName simple type defines the supported properties.
         *
         * If you specify an ItemHasRegularExpressionMatch rule on the body property of an item, the regular expression should further filter the body 
         * and should not attempt to return the entire body of the item. 
         * Using a regular expression such as .* to obtain the entire body of an item does not always return the expected results. 
         * Instead, use the Body.getAsync method to retrieve the entire body.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @returns
         * An object that contains arrays of strings that match the regular expressions defined in the manifest XML file. 
         * The name of each array is equal to the corresponding value of the RegExName attribute of the matching ItemHasRegularExpressionMatch rule 
         * or the FilterName attribute of the matching ItemHasKnownEntity rule.
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         */
        getRegExMatches(): any;
        /**
         * Returns string values in the selected item that match the named regular expression defined in the manifest XML file.
         *
         * The getRegExMatchesByName method returns the strings that match the regular expression defined in the ItemHasRegularExpressionMatch rule 
         * element in the manifest XML file with the specified RegExName element value.
         *
         * If you specify an ItemHasRegularExpressionMatch rule on the body property of an item, the regular expression should further filter the body 
         * and should not attempt to return the entire body of the item. 
         * Using a regular expression such as .* to obtain the entire body of an item does not always return the expected results.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @returns
         * An array that contains the strings that match the regular expression defined in the manifest XML file.
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         *
         * @param name - The name of the ItemHasRegularExpressionMatch rule element that defines the filter to match.
         */
        getRegExMatchesByName(name: string): string[];
        /**
         * Gets the entities found in a highlighted match a user has selected. Highlighted matches apply to contextual add-ins.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.6]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         *
         * @param name - The name of the ItemHasRegularExpressionMatch rule element that defines the filter to match.
         */
        getSelectedEntities(): Entities;
        /**
         * Returns string values in a highlighted match that match the regular expressions defined in the manifest XML file. 
         * Highlighted matches apply to contextual add-ins.
         *
         * The getSelectedRegExMatches method returns the strings that match the regular expression defined in each ItemHasRegularExpressionMatch or 
         * ItemHasKnownEntity rule element in the manifest XML file. For an ItemHasRegularExpressionMatch rule, a matching string has to occur in the property of the item that is specified by that rule. The PropertyName simple type defines the supported properties.
         *
         * If you specify an ItemHasRegularExpressionMatch rule on the body property of an item, the regular expression should further filter the body 
         * and should not attempt to return the entire body of the item. 
         * Using a regular expression such as .* to obtain the entire body of an item does not always return the expected results. 
         * Instead, use the Body.getAsync method to retrieve the entire body.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.6]
         *
         * @returns
         * An object that contains arrays of strings that match the regular expressions defined in the manifest XML file. 
         * The name of each array is equal to the corresponding value of the RegExName attribute of the matching ItemHasRegularExpressionMatch rule 
         * or the FilterName attribute of the matching ItemHasKnownEntity rule.
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         */
        getSelectedRegExMatches(): any;
    }
    /**
     * A subclass of {@link Office.Item} for messages.
     * 
     * **Important**: This is an internal Outlook object, not directly exposed through existing interfaces. 
     * You should treat this as a mode of Office.context.mailbox.item. Refer to the
     * {@link https://docs.microsoft.com/office/dev/add-ins/reference/objectmodel/preview-requirement-set/office.context.mailbox.item | Object Model} page for more information.
     */
    interface Message extends Item {
        /**
         * Gets an identifier for the email conversation that contains a particular message.
         *
         * You can get an integer for this property if your mail app is activated in read forms or responses in compose forms. 
         * If subsequently the user changes the subject of the reply message, upon sending the reply, the conversation ID for that message will 
         * change and that value you obtained earlier will no longer apply.
         *
         * You get null for this property for a new item in a compose form. 
         * If the user sets a subject and saves the item, the conversationId property will return a value.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        conversationId: string;
    }

     /**
     * The message compose mode of {@link Office.Item | Office.context.mailbox.item}.
     * 
     * **Important**: This is an internal Outlook object, not directly exposed through existing interfaces. 
     * You should treat this as a mode of Office.context.mailbox.item. Refer to the
     * {@link https://docs.microsoft.com/office/dev/add-ins/reference/objectmodel/preview-requirement-set/office.context.mailbox.item | Object Model} page for more information.
     */
    interface MessageCompose extends Message, ItemCompose {
        /**
         * Gets an object that provides methods to get or update the recipients on the Bcc (blind carbon copy) line of a message.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         */
        bcc: Recipients;
        /**
         * Gets an object that provides methods for manipulating the body of an item.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         */
        body: Body;
        /**
         * Provides access to the Cc (carbon copy) recipients of a message. The type of object and level of access depends on the mode of the 
         * current item.
         *
         * The cc property returns an {@link Office.Recipients} object that provides methods to get or update the recipients on the Cc line of 
         * the message.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         */
        cc: Recipients;
        /**
         * Gets an identifier for the email conversation that contains a particular message.
         *
         * You can get an integer for this property if your mail app is activated in read forms or responses in compose forms. 
         * If subsequently the user changes the subject of the reply message, upon sending the reply, the conversation ID for that message will change 
         * and that value you obtained earlier will no longer apply.
         *
         * You get null for this property for a new item in a compose form. 
         * If the user sets a subject and saves the item, the conversationId property will return a value.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         */
        conversationId: string;
        /**
         * Gets the email address of the sender of a message.
         *
         * The from and sender properties represent the same person unless the message is sent by a delegate. 
         * In that case, the from property represents the owner, and the sender property represents the delegate.
         *
         * The from property returns a From object that provides a method to get the from value.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         */
        from: From;
        /**
         * Sets the internet headers of a message.
         * 
         * The internetHeaders property returns an InternetHeaders object that provides methods to manage the internet headers on the message.
         *
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * @beta
         */
        internetHeaders: InternetHeaders;
        /**
         * Gets the type of item that an instance represents.
         *
         * The itemType property returns one of the ItemType enumeration values, indicating whether the item object instance is a message or 
         * an appointment.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         */
        itemType: MailboxEnums.ItemType | string;
        /**
         * Gets the notification messages for an item.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         */
        notificationMessages: NotificationMessages;
        /**
         * Gets the id of the series that an instance belongs to.
         * 
         * In Outlook on the web and desktop clients, the seriesId returns the Exchange Web Services (EWS) ID of the parent (series) item that this item belongs to. 
         * However, on iOS and Android, the seriesId returns the REST ID of the parent item.
         * 
         * **Note**: The identifier returned by the seriesId property is the same as the Exchange Web Services item identifier. 
         * The seriesId property is not identical to the Outlook IDs used by the Outlook REST API. 
         * Before making REST API calls using this value, it should be converted using Office.context.mailbox.convertToRestId. 
         * For more details, see {@link https://docs.microsoft.com/outlook/add-ins/use-rest-api | Use the Outlook REST APIs from an Outlook add-in}.
         * 
         * The seriesId property returns null for items that do not have parent items such as single appointments, series items, or meeting requests 
         * and returns undefined for any other items that are not meeting requests.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         */
        seriesId: string;
        /**
         * Gets or sets the description that appears in the subject field of an item.
         *
         * The subject property gets or sets the entire subject of the item, as sent by the email server.
         *
         * The subject property returns a Subject object that provides methods to get and set the subject.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         */
        subject: Subject;
        /**
         * Provides access to the recipients on the To line of a message. The type of object and level of access depends on the mode of the 
         * current item.
         *
         * The to property returns a Recipients object that provides methods to get or update the recipients on the To line of the message.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         */
        to: Recipients;

        /**
         * Adds a file to a message or appointment as an attachment.
         *
         * The addFileAttachmentAsync method uploads the file at the specified URI and attaches it to the item in the compose form.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * **Errors**: 
         * 
         * - AttachmentSizeExceeded: The attachment is larger than allowed.
         * 
         * - FileTypeNotSupported: The attachment has an extension that is not allowed.
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         * 
         * @param uri - The URI that provides the location of the file to attach to the message or appointment. The maximum length is 2048 characters.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         *        isInline: If true, indicates that the attachment will be shown inline in the message body, and should not be displayed in the 
         *        attachment list.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                 If uploading the attachment fails, the asyncResult object will contain an Error object that provides a description of 
         *                 the error.
         */
        addFileAttachmentAsync(uri: string, attachmentName: string, options?: Office.AsyncContextOptions & { isInline: boolean }, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Adds a file to a message or appointment as an attachment.
         *
         * The addFileAttachmentAsync method uploads the file at the specified URI and attaches it to the item in the compose form.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * **Errors**:
         * 
         * - AttachmentSizeExceeded: The attachment is larger than allowed.
         * 
         * - FileTypeNotSupported: The attachment has an extension that is not allowed.
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         *
         * @param uri - The URI that provides the location of the file to attach to the message or appointment. The maximum length is 2048 characters.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                 If uploading the attachment fails, the asyncResult object will contain an Error object that provides a description of 
         *                 the error.
         */
        addFileAttachmentAsync(uri: string, attachmentName: string, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Adds a file to a message or appointment as an attachment.
         *
         * The addFileAttachmentFromBase64Async method uploads the file from the base64 encoding and attaches it to the item in the compose form. This method returns the attachment identifier in the asyncResult.value object.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * [Api set: Mailbox Preview]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * **Errors**: 
         * 
         * - AttachmentSizeExceeded: The attachment is larger than allowed.
         * 
         * - FileTypeNotSupported: The attachment has an extension that is not allowed.
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         * 
         * @param base64File - The base64 encoded content of an image or file to be added to an email or event.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         *        isInline: If true, indicates that the attachment will be shown inline in the message body and should not be displayed in the attachment list.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult. 
         *                  On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                  If uploading the attachment fails, the asyncResult object will contain an Error object that provides a description of the error.
         * 
         * @beta
         */
        addFileAttachmentFromBase64Async(base64File: string, attachmentName: string, options?: Office.AsyncContextOptions & { isInline: boolean }, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Adds a file to a message or appointment as an attachment.
         *
         * The addFileAttachmentFromBase64Async method uploads the file from the base64 encoding and attaches it to the item in the compose form. This method returns the attachment identifier in the asyncResult.value object.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * [Api set: Mailbox Preview]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * **Errors**:
         * 
         * - AttachmentSizeExceeded: The attachment is larger than allowed.
         * 
         * - FileTypeNotSupported: The attachment has an extension that is not allowed.
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         * 
         * @param base64File - The base64 encoded content of an image or file to be added to an email or event.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult. 
         *                  On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                  If uploading the attachment fails, the asyncResult object will contain an Error object that provides a description of the error.
         * 
         * @beta
         */
        addFileAttachmentFromBase64Async(base64File: string, attachmentName: string, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Adds an event handler for a supported event.
         * 
         * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
         * `Office.EventType.RecurrenceChanged`.
         * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * @param eventType - The event that should invoke the handler.
         * @param handler - The function to handle the event. The function must accept a single parameter, which is an object literal. 
         *                The type property on the parameter will match the eventType parameter passed to addHandlerAsync.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        addHandlerAsync(eventType: Office.EventType | string, handler: any, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Adds an event handler for a supported event.
         * 
         * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
         * `Office.EventType.RecurrenceChanged`.
         * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * @param eventType - The event that should invoke the handler.
         * @param handler - The function to handle the event. The function must accept a single parameter, which is an object literal. 
         *                The type property on the parameter will match the eventType parameter passed to addHandlerAsync.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        addHandlerAsync(eventType: Office.EventType | string, handler: any, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Adds an Exchange item, such as a message, as an attachment to the message or appointment.
         *
         * The addItemAttachmentAsync method attaches the item with the specified Exchange identifier to the item in the compose form. 
         * If you specify a callback method, the method is called with one parameter, asyncResult, which contains either the attachment identifier or 
         * a code that indicates any error that occurred while attaching the item. You can use the options parameter to pass state information to the 
         * callback method, if needed.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * If your Office add-in is running in Outlook on the web, the addItemAttachmentAsync method can attach items to items other than the item that 
         * you are editing; however, this is not supported and is not recommended.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * **Errors**:
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         * 
         * @param itemId - The Exchange identifier of the item to attach. The maximum length is 100 characters.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                 If adding the attachment fails, the asyncResult object will contain an Error object that provides a description of 
         *                 the error.
         */
        addItemAttachmentAsync(itemId: any, attachmentName: string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Adds an Exchange item, such as a message, as an attachment to the message or appointment.
         *
         * The addItemAttachmentAsync method attaches the item with the specified Exchange identifier to the item in the compose form. 
         * If you specify a callback method, the method is called with one parameter, asyncResult, which contains either the attachment identifier or 
         * a code that indicates any error that occurred while attaching the item. 
         * You can use the options parameter to pass state information to the callback method, if needed.
         *
         * You can subsequently use the identifier with the removeAttachmentAsync method to remove the attachment in the same session.
         *
         * If your Office add-in is running in Outlook on the web, the addItemAttachmentAsync method can attach items to items other than the item that 
         * you are editing; however, this is not supported and is not recommended.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * **Errors**:
         * 
         * - NumberOfAttachmentsExceeded: The message or appointment has too many attachments.
         *
         * @param itemId - The Exchange identifier of the item to attach. The maximum length is 100 characters.
         * @param attachmentName - The name of the attachment that is shown while the attachment is uploading. The maximum length is 255 characters.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. On success, the attachment identifier will be provided in the asyncResult.value property. 
         *                 If adding the attachment fails, the asyncResult object will contain an Error object that provides a description of 
         *                 the error.
         */
        addItemAttachmentAsync(itemId: any, attachmentName: string, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Closes the current item that is being composed
         *
         * The behaviors of the close method depends on the current state of the item being composed. 
         * If the item has unsaved changes, the client prompts the user to save, discard, or close the action.
         *
         * In the Outlook desktop client, if the message is an inline reply, the close method has no effect.
         *
         * **Note**: In Outlook on the web, if the item is an appointment and it has previously been saved using saveAsync, the user is prompted to save, 
         * discard, or cancel even if no changes have occurred since the item was last saved.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         */
        close(): void;
        /**
         * Gets the item's attachments as an array.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. If the call fails, the asyncResult.error property will contain and error code with the reason for 
         *                 the failure.
         * 
         * @beta
         */
        getAttachmentsAsync(options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<AttachmentDetails[]>) => void): void;
        /**
         * Gets the item's attachments as an array.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. If the call fails, the asyncResult.error property will contain and error code with the reason for 
         *                 the failure.
         * 
         * @beta
         */
        getAttachmentsAsync(callback?: (asyncResult: Office.AsyncResult<AttachmentDetails[]>) => void): void;
        /**
         * Gets initialization data passed when the add-in is activated by an actionable message.
         *
         * **Note**: This method is only supported by Outlook 2016 or later on Windows (Click-to-Run versions greater than 16.0.8413.1000) and Outlook on the web 
         * for Office 365.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         *
         * More information on {@link https://docs.microsoft.com/outlook/actionable-messages/invoke-add-in-from-actionable-message | actionable messages}.
         *
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         *                 On success, the initialization data is provided in the asyncResult.value property as a string. 
         *                 If there is no initialization context, the asyncResult object will contain an Error object with its code property 
         *                 set to 9020 and its name property set to GenericResponseError.
         *
         * @beta
         */
        getInitializationContextAsync(options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Gets initialization data passed when the add-in is activated by an actionable message.
         *
         * **Note**: This method is only supported by Outlook 2016 or later on Windows (Click-to-Run versions greater than 16.0.8413.1000) and Outlook on the web 
         * for Office 365.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         *
         * More information on {@link https://docs.microsoft.com/outlook/actionable-messages/invoke-add-in-from-actionable-message | actionable messages}.
         *
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         *                 On success, the initialization data is provided in the asyncResult.value property as a string. 
         *                 If there is no initialization context, the asyncResult object will contain an Error object with its code property 
         *                 set to 9020 and its name property set to GenericResponseError.
         *
         * @beta
         */
        getInitializationContextAsync(callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Asynchronously returns selected data from the subject or body of a message.
         *
         * If there is no selection but the cursor is in the body or subject, the method returns null for the selected data. 
         * If a field other than the body or subject is selected, the method returns the InvalidSelection error.
         *
         * To access the selected data from the callback method, call asyncResult.value.data. 
         * To access the source property that the selection comes from, call asyncResult.value.sourceProperty, which will be either body or subject.
         *
         * [Api set: Mailbox 1.2]
         *
         * @returns
         * The selected data as a string with format determined by coercionType.
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         *
         * @param coercionType - Requests a format for the data. If Text, the method returns the plain text as a string, removing any HTML tags present. 
         *                     If HTML, the method returns the selected text, whether it is plaintext or HTML.
         * @param options - An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        getSelectedDataAsync(coercionType: Office.CoercionType | string, options: Office.AsyncContextOptions, callback: (asyncResult: Office.AsyncResult<any>) => void): void;
        /**
         * Asynchronously returns selected data from the subject or body of a message.
         *
         * If there is no selection but the cursor is in the body or subject, the method returns null for the selected data. 
         * If a field other than the body or subject is selected, the method returns the InvalidSelection error.
         *
         * To access the selected data from the callback method, call asyncResult.value.data. 
         * To access the source property that the selection comes from, call asyncResult.value.sourceProperty, which will be either body or subject.
         *
         * [Api set: Mailbox 1.2]
         *
         * @returns
         * The selected data as a string with format determined by coercionType.
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         *
         * @param coercionType - Requests a format for the data. If Text, the method returns the plain text as a string, removing any HTML tags present. 
         *                     If HTML, the method returns the selected text, whether it is plaintext or HTML.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        getSelectedDataAsync(coercionType: Office.CoercionType | string, callback: (asyncResult: Office.AsyncResult<any>) => void): void;
        /**
         * Asynchronously loads custom properties for this add-in on the selected item.
         *
         * Custom properties are stored as key/value pairs on a per-app, per-item basis. 
         * This method returns a CustomProperties object in the callback, which provides methods to access the custom properties specific to the 
         * current item and the current add-in. Custom properties are not encrypted on the item, so this should not be used as secure storage.
         *
         * The custom properties are provided as a CustomProperties object in the asyncResult.value property. 
         * This object can be used to get, set, and remove custom properties from the item and save changes to the custom property set back to 
         * the server.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         *
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         * @param userContext - Optional. Developers can provide any object they wish to access in the callback function. 
         *                    This object can be accessed by the asyncResult.asyncContext property in the callback function.
         */
        loadCustomPropertiesAsync(callback: (asyncResult: Office.AsyncResult<CustomProperties>) => void, userContext?: any): void;
        /**
         * Removes an attachment from a message or appointment.
         *
         * The removeAttachmentAsync method removes the attachment with the specified identifier from the item. 
         * As a best practice, you should use the attachment identifier to remove an attachment only if the same mail app has added that attachment 
         * in the same session. In Outlook on the web and mobile devices, the attachment identifier is valid only within the same session. 
         * A session is over when the user closes the app, or if the user starts composing an inline form then subsequently pops out the form to 
         * continue in a separate window.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * **Errors**:
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         * 
         * @param attachmentId - The identifier of the attachment to remove.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         *                 If removing the attachment fails, the asyncResult.error property will contain an error code with the reason for the failure.
         */
        removeAttachmentAsync(attachmentId: string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Removes an attachment from a message or appointment.
         *
         * The removeAttachmentAsync method removes the attachment with the specified identifier from the item. 
         * As a best practice, you should use the attachment identifier to remove an attachment only if the same mail app has added that attachment 
         * in the same session. In Outlook on the web and mobile devices, the attachment identifier is valid only within the same session. 
         * A session is over when the user closes the app, or if the user starts composing an inline form then subsequently pops out the form to 
         * continue in a separate window.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * **Errors**: 
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         *
         * @param attachmentId - The identifier of the attachment to remove.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         *                 If removing the attachment fails, the asyncResult.error property will contain an error code with the reason for the failure.
         */
        removeAttachmentAsync(attachmentId: string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Removes the event handlers for a supported event type.
         * 
         * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
         * `Office.EventType.RecurrenceChanged`.
         * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * @param eventType - The event that should revoke the handler.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        removeHandlerAsync(eventType: Office.EventType | string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Removes the event handlers for a supported event type.
         * 
         * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
         * `Office.EventType.RecurrenceChanged`.
         * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * @param eventType - The event that should revoke the handler.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        removeHandlerAsync(eventType: Office.EventType | string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Asynchronously saves an item.
         *
         * When invoked, this method saves the current message as a draft and returns the item id via the callback method. 
         * In Outlook on the web or Outlook in online mode, the item is saved to the server. 
         * In Outlook in cached mode, the item is saved to the local cache.
         *
         * Since appointments have no draft state, if saveAsync is called on an appointment in compose mode, the item will be saved as a normal 
         * appointment on the user's calendar. For new appointments that have not been saved before, no invitation will be sent. 
         * Saving an existing appointment will send an update to added or removed attendees.
         *
         * **Note**: If your add-in calls saveAsync on an item in compose mode in order to get an itemId to use with EWS or the REST API, be aware that 
         * when Outlook is in cached mode, it may take some time before the item is actually synced to the server. 
         * Until the item is synced, using the itemId will return an error.
         *
         * **Note**: The following clients have different behavior for saveAsync on appointments in compose mode:
         *
         * - Outlook on Mac does not support saving a meeting. The saveAsync method fails when called from a meeting in compose mode.
         * See {@link https://support.microsoft.com/help/4505745 | Cannot save a meeting as a draft in Outlook for Mac by using Office JS API} for a workaround.
         *
         * - Outlook on the web always sends an invitation or update when saveAsync is called on an appointment in compose mode.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * **Errors**:
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         * 
         * @param options - An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         */
        saveAsync(options: Office.AsyncContextOptions, callback: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Asynchronously saves an item.
         *
         * When invoked, this method saves the current message as a draft and returns the item id via the callback method. 
         * In Outlook on the web or Outlook in online mode, the item is saved to the server. 
         * In Outlook in cached mode, the item is saved to the local cache.
         *
         * Since appointments have no draft state, if saveAsync is called on an appointment in compose mode, the item will be saved as a normal 
         * appointment on the user's calendar. For new appointments that have not been saved before, no invitation will be sent. 
         * Saving an existing appointment will send an update to added or removed attendees.
         *
         * **Note**: If your add-in calls saveAsync on an item in compose mode in order to get an itemId to use with EWS or the REST API, be aware that 
         * when Outlook is in cached mode, it may take some time before the item is actually synced to the server. 
         * Until the item is synced, using the itemId will return an error.
         *
         * **Note**: The following clients have different behavior for saveAsync on appointments in compose mode:
         *
         * - Outlook on Mac does not support saving a meeting. The saveAsync method fails when called from a meeting in compose mode.
         * See {@link https://support.microsoft.com/help/4505745 | Cannot save a meeting as a draft in Outlook for Mac by using Office JS API} for a workaround.
         *
         * - Outlook on the web always sends an invitation or update when saveAsync is called on an appointment in compose mode.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * **Errors**:
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         *
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         */
        saveAsync(callback: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Asynchronously inserts data into the body or subject of a message.
         *
         * The setSelectedDataAsync method inserts the specified string at the cursor location in the subject or body of the item, or, if text is 
         * selected in the editor, it replaces the selected text. If the cursor is not in the body or subject field, an error is returned. 
         * After insertion, the cursor is placed at the end of the inserted content.
         *
         * [Api set: Mailbox 1.2]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * **Errors**: 
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         * 
         * @param data - The data to be inserted. Data is not to exceed 1,000,000 characters. 
         *             If more than 1,000,000 characters are passed in, an ArgumentOutOfRange exception is thrown.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         *        coercionType: If text, the current style is applied in Outlook on the web and desktop clients. 
         *        If the field is an HTML editor, only the text data is inserted, even if the data is HTML. 
         *        If html and the field supports HTML (the subject doesn't), the current style is applied in Outlook on the web and the default style is 
         *        applied in Outlook on desktop clients. If the field is a text field, an InvalidDataFormat error is returned. 
         *        If coercionType is not set, the result depends on the field: if the field is HTML then HTML is used; 
         *        if the field is text, then plain text is used.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        setSelectedDataAsync(data: string, options?: Office.AsyncContextOptions & CoercionTypeOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Asynchronously inserts data into the body or subject of a message.
         *
         * The setSelectedDataAsync method inserts the specified string at the cursor location in the subject or body of the item, or, if text is 
         * selected in the editor, it replaces the selected text. If the cursor is not in the body or subject field, an error is returned. 
         * After insertion, the cursor is placed at the end of the inserted content.
         *
         * [Api set: Mailbox 1.2]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Compose
         * 
         * **Errors**: 
         * 
         * - InvalidAttachmentId: The attachment identifier does not exist.
         *
         * @param data - The data to be inserted. Data is not to exceed 1,000,000 characters. 
         *             If more than 1,000,000 characters are passed in, an ArgumentOutOfRange exception is thrown.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        setSelectedDataAsync(data: string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
    }
    /**
     * The message read mode of {@link Office.Item | Office.context.mailbox.item}.
     * 
     * **Important**: This is an internal Outlook object, not directly exposed through existing interfaces. 
     * You should treat this as a mode of Office.context.mailbox.item. Refer to the
     * {@link https://docs.microsoft.com/office/dev/add-ins/reference/objectmodel/preview-requirement-set/office.context.mailbox.item | Object Model} page for more information.
     */
    interface MessageRead extends Message, ItemRead {
        /**
         * Gets the item's attachments as an array.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         * 
         * **Note**: Certain types of files are blocked by Outlook due to potential security issues and are therefore not returned. 
         * For more information, see 
         * {@link https://support.office.com/article/Blocked-attachments-in-Outlook-434752E1-02D3-4E90-9124-8B81E49A8519 | Blocked attachments in Outlook}.
         *
         */
        attachments: AttachmentDetails[];
        /**
         * Gets an object that provides methods for manipulating the body of an item.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        body: Body;
        /**
         * Provides access to the Cc (carbon copy) recipients of a message. The type of object and level of access depends on the mode of the 
         * current item.
         *
         * The cc property returns an array that contains an EmailAddressDetails object for each recipient listed on the Cc line of the message. 
         * The collection is limited to a maximum of 100 members.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        cc: EmailAddressDetails[];
        /**
         * Gets an identifier for the email conversation that contains a particular message.
         *
         * You can get an integer for this property if your mail app is activated in read forms or responses in compose forms. 
         * If subsequently the user changes the subject of the reply message, upon sending the reply, the conversation ID for that message will change 
         * and that value you obtained earlier will no longer apply.
         *
         * You get null for this property for a new item in a compose form. 
         * If the user sets a subject and saves the item, the conversationId property will return a value.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        conversationId: string;
        /**
         * Gets the date and time that an item was created.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        dateTimeCreated: Date;
        /**
         * Gets the date and time that an item was last modified.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         *
         * **Note**: This member is not supported in Outlook on iOS or Android.
         */
        dateTimeModified: Date;
        /**
         * Gets the email address of the sender of a message.
         *
         * The from and sender properties represent the same person unless the message is sent by a delegate. 
         * In that case, the from property represents the delegator, and the sender property represents the delegate.
         *
         * **Note**: The recipientType property of the EmailAddressDetails object in the from property is undefined.
         * 
         * The from property returns an EmailAddressDetails object.
         * 
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        from: EmailAddressDetails;
        /**
         * Gets the internet headers of a message.
         * 
         * The internetHeaders property returns an InternetHeaders object that provides methods to manage the internet headers on the message.
         *
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         * 
         * @beta
         */
        internetHeaders: InternetHeaders;
        /**
         * Gets the Internet message identifier for an email message.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        internetMessageId: string;
        /**
         * Gets the Exchange Web Services item class of the selected item.
         * 
         * You can create custom message classes that extends a default message class, for example, a custom appointment message class 
         * IPM.Appointment.Contoso.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
		 
         * The itemClass property specifies the message class of the selected item. 
         * The following are the default message classes for the message or appointment item.
         * 
         * <table>
         *   <tr>
         *     <th>Type</th>
         *     <th>Description</th>
         *     <th>Item Class</th>
         *   </tr>
         *   <tr>
         *     <td>Appointment items</td>
         *     <td>These are calendar items of the item class IPM.Appointment or IPM.Appointment.Occurrence.</td>
         *     <td>IPM.Appointment,IPM.Appointment.Occurrence</td>
         *   </tr>
         *   <tr>
         *     <td>Message items</td>
         *     <td>These include email messages that have the default message class IPM.Note, and meeting requests, responses, and cancellations, that use IPM.Schedule.Meeting as the base message class.</td>
         *     <td>IPM.Note,IPM.Schedule.Meeting.Request,IPM.Schedule.Meeting.Neg,IPM.Schedule.Meeting.Pos,IPM.Schedule.Meeting.Tent,IPM.Schedule.Meeting.Canceled</td>
         *   </tr>
         * </table>
         * 
         */
        itemClass: string;
        /**
         * Gets the Exchange Web Services item identifier for the current item.
         *
         * The itemId property is not available in compose mode. 
         * If an item identifier is required, the saveAsync method can be used to save the item to the store, which will return the item identifier 
         * in the asyncResult.value parameter in the callback function.
         *
         * **Note**: The identifier returned by the itemId property is the same as the Exchange Web Services item identifier. 
         * The itemId property is not identical to the Outlook Entry ID or the ID used by the Outlook REST API. 
         * Before making REST API calls using this value, it should be converted using Office.context.mailbox.convertToRestId. 
         * For more details, see {@link https://docs.microsoft.com/outlook/add-ins/use-rest-api#get-the-item-id | Use the Outlook REST APIs from an Outlook add-in}.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        itemId: string;
        /**
         * Gets the type of item that an instance represents.
         *
         * The itemType property returns one of the ItemType enumeration values, indicating whether the item object instance is a message or 
         * an appointment.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        itemType: MailboxEnums.ItemType | string;
        /**
         * Gets the subject of an item, with all prefixes removed (including RE: and FWD:).
         *
         * The normalizedSubject property gets the subject of the item, with any standard prefixes (such as RE: and FW:) that are added by 
         * email programs. 
         * To get the subject of the item with the prefixes intact, use the subject property.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        normalizedSubject: string;
        /**
         * Gets the notification messages for an item.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        notificationMessages: NotificationMessages;
        /**
         * Gets the recurrence pattern of an appointment. Gets the recurrence pattern of a meeting request. 
         * Read and compose modes for appointment items. Read mode for meeting request items.
         * 
         * The recurrence property returns a recurrence object for recurring appointments or meetings requests if an item is a series or an instance 
         * in a series. `null` is returned for single appointments and meeting requests of single appointments. 
         * `undefined` is returned for messages that are not meeting requests.
         * 
         * **Note**: Meeting requests have an itemClass value of IPM.Schedule.Meeting.Request.
         * 
         * **Note**: If the recurrence object is null, this indicates that the object is a single appointment or a meeting request of a single appointment 
         * and NOT a part of a series.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        recurrence: Recurrence;
        /**
         * Gets the id of the series that an instance belongs to.
         * 
         * In Outlook on the web and desktop clients, the seriesId returns the Exchange Web Services (EWS) ID of the parent (series) item that this item belongs to. 
         * However, on iOS and Android, the seriesId returns the REST ID of the parent item.
         * 
         * **Note**: The identifier returned by the seriesId property is the same as the Exchange Web Services item identifier. 
         * The seriesId property is not identical to the Outlook IDs used by the Outlook REST API. 
         * Before making REST API calls using this value, it should be converted using Office.context.mailbox.convertToRestId. 
         * For more details, see {@link https://docs.microsoft.com/outlook/add-ins/use-rest-api | Use the Outlook REST APIs from an Outlook add-in}.
         * 
         * The seriesId property returns null for items that do not have parent items such as single appointments, series items, or meeting requests 
         * and returns undefined for any other items that are not meeting requests.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        seriesId: string;
        /**
         * Gets the email address of the sender of an email message.
         *
         * The from and sender properties represent the same person unless the message is sent by a delegate. 
         * In that case, the from property represents the delegator, and the sender property represents the delegate.
         *
         * **Note**: The recipientType property of the EmailAddressDetails object in the sender property is undefined.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        sender: EmailAddressDetails;
        /**
         * Gets the description that appears in the subject field of an item.
         *
         * The subject property gets or sets the entire subject of the item, as sent by the email server.
         *
         * The subject property returns a string. Use the normalizedSubject property to get the subject minus any leading prefixes such as RE: and FW:.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        subject: string;
        /**
         * Provides access to the recipients on the To line of a message. The type of object and level of access depends on the mode of the 
         * current item.
         *
         * The to property returns an array that contains an EmailAddressDetails object for each recipient listed on the To line of the message. 
         * The collection is limited to a maximum of 100 members.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        to: EmailAddressDetails[];

        /**
         * Adds an event handler for a supported event.
         * 
         * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
         * `Office.EventType.RecurrenceChanged`.
         * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         * 
         * @param eventType - The event that should invoke the handler.
         * @param handler - The function to handle the event. The function must accept a single parameter, which is an object literal. 
         *                The type property on the parameter will match the eventType parameter passed to addHandlerAsync.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        addHandlerAsync(eventType: Office.EventType | string, handler: any, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Adds an event handler for a supported event.
         * 
         * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
         * `Office.EventType.RecurrenceChanged`.
         * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         * 
         * @param eventType - The event that should invoke the handler.
         * @param handler - The function to handle the event. The function must accept a single parameter, which is an object literal. 
         *                The type property on the parameter will match the eventType parameter passed to addHandlerAsync.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        addHandlerAsync(eventType: Office.EventType | string, handler: any, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Displays a reply form that includes the sender and all recipients of the selected message or the organizer and all attendees of the 
         * selected appointment.
         *
         * In Outlook on the web, the reply form is displayed as a pop-out form in the 3-column view and a pop-up form in the 2- or 1-column view.
         *
         * If any of the string parameters exceed their limits, displayReplyAllForm throws an exception.
         *
         * When attachments are specified in the formData.attachments parameter, Outlook on the web and desktop clients attempt to download all attachments and 
         * attach them to the reply form. If any attachments fail to be added, an error is shown in the form UI. 
         * If this isn't possible, then no error message is thrown.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         *
         * @param formData - A string that contains text and HTML and that represents the body of the reply form. The string is limited to 32 KB
         *                   OR an {@link Office.ReplyFormData} object that contains body or attachment data and a callback function.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        displayReplyAllForm(formData: string | ReplyFormData, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Displays a reply form that includes only the sender of the selected message or the organizer of the selected appointment.
         *
         * In Outlook on the web, the reply form is displayed as a pop-out form in the 3-column view and a pop-up form in the 2- or 1-column view.
         *
         * If any of the string parameters exceed their limits, displayReplyForm throws an exception.
         *
         * When attachments are specified in the formData.attachments parameter, Outlook on the web and desktop clients attempt to download all attachments and 
         * attach them to the reply form. If any attachments fail to be added, an error is shown in the form UI. 
         * If this isn't possible, then no error message is thrown.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         *
         * @param formData - A string that contains text and HTML and that represents the body of the reply form. The string is limited to 32 KB
         *                   OR an {@link Office.ReplyFormData} object that contains body or attachment data and a callback function.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        displayReplyForm(formData: string | ReplyFormData, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Gets initialization data passed when the add-in is 
         * {@link https://docs.microsoft.com/outlook/actionable-messages/invoke-add-in-from-actionable-message | activated by an actionable message}.
         * 
         * **Note**: This method is only supported by Outlook 2016 or later on Windows (Click-to-Run versions greater than 16.0.8413.1000) and Outlook on the web 
         * for Office 365.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         * 
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object. 
         *                 On success, the initialization data is provided in the asyncResult.value property as a string. 
         *                 If there is no initialization context, the asyncResult object will contain an Error object with its code property 
         *                 set to 9020 and its name property set to GenericResponseError.
         *
         * @beta
         */
        getInitializationContextAsync(options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Gets initialization data passed when the add-in is 
         * {@link https://docs.microsoft.com/outlook/actionable-messages/invoke-add-in-from-actionable-message | activated by an actionable message}.
         * 
         * **Note**: This method is only supported by Outlook 2016 or later on Windows (Click-to-Run versions greater than 16.0.8413.1000) and Outlook on the 
         * web for Office 365.
         * 
         * [Api set: Mailbox Preview]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         * 
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object. 
         *                 On success, the initialization data is provided in the asyncResult.value property as a string. 
         *                 If there is no initialization context, the asyncResult object will contain an Error object with its code property 
         *                 set to 9020 and its name property set to GenericResponseError.
         *
         * @beta
         */
        getInitializationContextAsync(callback?: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Gets the entities found in the selected item's body.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        getEntities(): Entities;
        /**
         * Gets an array of all the entities of the specified entity type found in the selected item's body.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @param entityType - One of the EntityType enumeration values.
         * 
         * @returns
         * If the value passed in entityType is not a valid member of the EntityType enumeration, the method returns null. 
         * If no entities of the specified type are present in the item's body, the method returns an empty array. 
         * Otherwise, the type of the objects in the returned array depends on the type of entity requested in the entityType parameter.
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         * 
         * While the minimum permission level to use this method is Restricted, some entity types require ReadItem to access, as specified in the 
         * following table.
         * 
         * <table>
         *   <tr>
         *     <th>Value of entityType</th>
         *     <th>Type of objects in returned array</th>
         *     <th>Required Permission Level</th>
         *   </tr>
         *   <tr>
         *     <td>Address</td>
         *     <td>String</td>
         *     <td>Restricted</td>
         *   </tr>
         *   <tr>
         *     <td>Contact</td>
         *     <td>Contact</td>
         *     <td>ReadItem</td>
         *   </tr>
         *   <tr>
         *     <td>EmailAddress</td>
         *     <td>String</td>
         *     <td>ReadItem</td>
         *   </tr>
         *   <tr>
         *     <td>MeetingSuggestion</td>
         *     <td>MeetingSuggestion</td>
         *     <td>ReadItem</td>
         *   </tr>
         *   <tr>
         *     <td>PhoneNumber</td>
         *     <td>PhoneNumber</td>
         *     <td>Restricted</td>
         *   </tr>
         *   <tr>
         *     <td>TaskSuggestion</td>
         *     <td>TaskSuggestion</td>
         *     <td>ReadItem</td>
         *   </tr>
         *   <tr>
         *     <td>URL</td>
         *     <td>String</td>
         *     <td>Restricted</td>
         *   </tr>
         * </table>
         */
        getEntitiesByType(entityType: MailboxEnums.EntityType | string): (string | Contact | MeetingSuggestion | PhoneNumber | TaskSuggestion)[];
        /**
         * Returns well-known entities in the selected item that pass the named filter defined in the manifest XML file.
         *
         * The getFilteredEntitiesByName method returns the entities that match the regular expression defined in the ItemHasKnownEntity rule element 
         * in the manifest XML file with the specified FilterName element value.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         *
         * @param name - The name of the ItemHasKnownEntity rule element that defines the filter to match.
         * @returns If there is no ItemHasKnownEntity element in the manifest with a FilterName element value that matches the name parameter, 
         * the method returns null. 
         * If the name parameter does match an ItemHasKnownEntity element in the manifest, but there are no entities in the current item that match, 
         * the method return an empty array.
         */
        getFilteredEntitiesByName(name: string): (string | Contact | MeetingSuggestion | PhoneNumber | TaskSuggestion)[];
        /**
         * Returns string values in the selected item that match the regular expressions defined in the manifest XML file.
         *
         * The getRegExMatches method returns the strings that match the regular expression defined in each ItemHasRegularExpressionMatch or 
         * ItemHasKnownEntity rule element in the manifest XML file. 
         * For an ItemHasRegularExpressionMatch rule, a matching string has to occur in the property of the item that is specified by that rule. 
         * The PropertyName simple type defines the supported properties.
         *
         * If you specify an ItemHasRegularExpressionMatch rule on the body property of an item, the regular expression should further filter the body 
         * and should not attempt to return the entire body of the item. 
         * Using a regular expression such as .* to obtain the entire body of an item does not always return the expected results. 
         * Instead, use the Body.getAsync method to retrieve the entire body.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @returns
         * An object that contains arrays of strings that match the regular expressions defined in the manifest XML file. 
         * The name of each array is equal to the corresponding value of the RegExName attribute of the matching ItemHasRegularExpressionMatch rule 
         * or the FilterName attribute of the matching ItemHasKnownEntity rule.
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        getRegExMatches(): any;
        /**
         * Returns string values in the selected item that match the named regular expression defined in the manifest XML file.
         *
         * The getRegExMatchesByName method returns the strings that match the regular expression defined in the ItemHasRegularExpressionMatch rule 
         * element in the manifest XML file with the specified RegExName element value.
         *
         * If you specify an ItemHasRegularExpressionMatch rule on the body property of an item, the regular expression should further filter the body 
         * and should not attempt to return the entire body of the item. 
         * Using a regular expression such as .* to obtain the entire body of an item does not always return the expected results.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @returns
         * An array that contains the strings that match the regular expression defined in the manifest XML file.
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         *
         * @param name - The name of the ItemHasRegularExpressionMatch rule element that defines the filter to match.
         */
        getRegExMatchesByName(name: string): string[];
        /**
         * Gets the entities found in a highlighted match a user has selected. Highlighted matches apply to contextual add-ins.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.6]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         *
         * @param name - The name of the ItemHasRegularExpressionMatch rule element that defines the filter to match.
         */
        getSelectedEntities(): Entities;
        /**
         * Returns string values in a highlighted match that match the regular expressions defined in the manifest XML file. 
         * Highlighted matches apply to contextual add-ins.
         *
         * The getSelectedRegExMatches method returns the strings that match the regular expression defined in each ItemHasRegularExpressionMatch or 
         * ItemHasKnownEntity rule element in the manifest XML file. 
         * For an ItemHasRegularExpressionMatch rule, a matching string has to occur in the property of the item that is specified by that rule. 
         * The PropertyName simple type defines the supported properties.
         *
         * If you specify an ItemHasRegularExpressionMatch rule on the body property of an item, the regular expression should further filter the body 
         * and should not attempt to return the entire body of the item. 
         * Using a regular expression such as .* to obtain the entire body of an item does not always return the expected results. 
         * Instead, use the Body.getAsync method to retrieve the entire body.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.6]
         *
         * @returns
         * An object that contains arrays of strings that match the regular expressions defined in the manifest XML file. 
         * The name of each array is equal to the corresponding value of the RegExName attribute of the matching ItemHasRegularExpressionMatch rule or 
         * the FilterName attribute of the matching ItemHasKnownEntity rule.
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         */
        getSelectedRegExMatches(): any;
        /**
         * Asynchronously loads custom properties for this add-in on the selected item.
         *
         * Custom properties are stored as key/value pairs on a per-app, per-item basis. 
         * This method returns a CustomProperties object in the callback, which provides methods to access the custom properties specific to the 
         * current item and the current add-in. Custom properties are not encrypted on the item, so this should not be used as secure storage.
         *
         * The custom properties are provided as a CustomProperties object in the asyncResult.value property. 
         * This object can be used to get, set, and remove custom properties from the item and save changes to the custom property set back to 
         * the server.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         *
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         * @param userContext - Optional. Developers can provide any object they wish to access in the callback function. 
         *                    This object can be accessed by the asyncResult.asyncContext property in the callback function.
         */
        loadCustomPropertiesAsync(callback: (asyncResult: Office.AsyncResult<CustomProperties>) => void, userContext?: any): void;
        /**
         * Removes the event handlers for a supported event type.
         * 
         * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
         * `Office.EventType.RecurrenceChanged`.
         * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         * 
         * @param eventType - The event that should revoke the handler.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        removeHandlerAsync(eventType: Office.EventType | string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Removes the event handlers for a supported event type.
         * 
         * Currently the supported event types are `Office.EventType.AppointmentTimeChanged`, `Office.EventType.RecipientsChanged`, and 
         * `Office.EventType.RecurrenceChanged`.
         * In Preview, `Office.EventType.AttachmentsChanged` and `Office.EventType.EnhancedLocationsChanged` are also supported.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Message Read
         * 
         * @param eventType - The event that should revoke the handler.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        removeHandlerAsync(eventType: Office.EventType | string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
    }

    /**
     * Represents a date and time in the local client's time zone. Read mode only.
     *
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     *
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
     */
    interface LocalClientTime {
        /**
         * Integer value representing the month, beginning with 0 for January to 11 for December.
         */
        month: number;
        /**
         * Integer value representing the day of the month.
         */
        date: number;
        /**
         * Integer value representing the year.
         */
        year: number;
        /**
         * Integer value representing the hour on a 24-hour clock.
         */
        hours: number;
        /**
         * Integer value representing the minutes.
         */
        minutes: number;
        /**
         * Integer value representing the seconds.
         */
        seconds: number;
        /**
         * Integer value representing the milliseconds.
         */
        milliseconds: number;
        /**
         * Integer value representing the number of minutes difference between the local time zone and UTC.
         */
        timezoneOffset: number;
    }
    /**
     * Provides methods to get and set the location of a meeting in an Outlook add-in.
     *
     * [Api set: Mailbox 1.1]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
     */
    interface Location {
        /**
         * Gets the location of an appointment.
         *
         * The getAsync method starts an asynchronous call to the Exchange server to get the location of an appointment. 
         * The location of the appointment is provided as a string in the asyncResult.value property.
         *
         * @param options - An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         */
        getAsync(options: Office.AsyncContextOptions, callback: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Gets the location of an appointment.
         *
         * The getAsync method starts an asynchronous call to the Exchange server to get the location of an appointment. 
         * The location of the appointment is provided as a string in the asyncResult.value property.
         *
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose
         */
        getAsync(callback: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Sets the location of an appointment.
         *
         * The setAsync method starts an asynchronous call to the Exchange server to set the location of an appointment. 
         * Setting the location of an appointment overwrites the current location.
         *
         * @param location - The location of the appointment. The string is limited to 255 characters.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. If setting the location fails, the asyncResult.error property will contain an error code.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**:
         * 
         * - DataExceedsMaximumSize: The location parameter is longer than 255 characters.
         */
        setAsync(location: string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Sets the location of an appointment.
         *
         * The setAsync method starts an asynchronous call to the Exchange server to set the location of an appointment. 
         * Setting the location of an appointment overwrites the current location.
         *
         * @param location - The location of the appointment. The string is limited to 255 characters.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. If setting the location fails, the asyncResult.error property will contain an error code.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - DataExceedsMaximumSize: The location parameter is longer than 255 characters.
         */
        setAsync(location: string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
    }
    /**
     * Provides access to the Outlook Add-in object model for Microsoft Outlook and Microsoft Outlook on the web.
     *
     * Namespaces:
     *
     * - diagnostics: Provides diagnostic information to an Outlook add-in.
     *
     * - item: Provides methods and properties for accessing a message or appointment in an Outlook add-in.
     *
     * - userProfile: Provides information about the user in an Outlook add-in.
     *
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     */
    interface Mailbox {
        /**
         * Provides diagnostic information to an Outlook add-in.
         * 
         * Contains the following members:
         * 
         *  - hostName (string): A string that represents the name of the host application. 
         * It be one of the following values: Outlook, Mac Outlook, OutlookIOS, or OutlookWebApp.
         * 
         *  - hostVersion (string): A string that represents the version of either the host application or the Exchange Server. 
         * If the mail add-in is running in Outlook on desktop clients or iOS, the hostVersion property returns the version of the 
         * host application, Outlook. In Outlook on the web, the property returns the version of the Exchange Server. An example is the string 15.0.468.0.
         * 
         *  - OWAView (MailboxEnums.OWAView or string): An enum (or string literal) that represents the current view of Outlook on the web. 
         * If the host application is not Outlook on the web, then accessing this property results in undefined. 
         * Outlook on the web has three views (OneColumn - displayed when the screen is narrow, TwoColumns - displayed when the screen is wider, 
         * and ThreeColumns - displayed when the screen is wide) that correspond to the width of the screen and the window, and the number of columns 
         * that can be displayed.
         *
         *  More information is under {@link Office.Diagnostics}. 
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        diagnostics: Diagnostics;
        /**
         * Gets the URL of the Exchange Web Services (EWS) endpoint for this email account. Read mode only.
         *
         * Your app must have the ReadItem permission specified in its manifest to call the ewsUrl member in read mode.
         *
         * In compose mode you must call the saveAsync method before you can use the ewsUrl member. 
         * Your app must have ReadWriteItem permissions to call the saveAsync method.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * The ewsUrl value can be used by a remote service to make EWS calls to the user's mailbox. For example, you can create a remote service to {@link https://docs.microsoft.com/outlook/add-ins/get-attachments-of-an-outlook-item | get attachments from the selected item}.
         *
         * **Note**: This member is not supported in Outlook on iOS or Android.
         */
        ewsUrl: string;
        /**
         * The mailbox item.  Depending on the context in which the add-in opened, the item may be of any number of types.
         * If you want to see IntelliSense for only a specific type, you should cast this item to one of the following:
         * `ItemCompose`, `ItemRead`, `MessageCompose`, `MessageRead`, `AppointmentCompose`, `AppointmentRead`
         */
        item: Item & ItemCompose & ItemRead & MessageRead & MessageCompose & AppointmentRead & AppointmentCompose;
        /**
         * Gets the URL of the REST endpoint for this email account.
         *
         * Your app must have the ReadItem permission specified in its manifest to call the restUrl member in read mode.
         *
         * In compose mode you must call the saveAsync method before you can use the restUrl member. 
         * Your app must have ReadWriteItem permissions to call the saveAsync method.
         *
         * [Api set: Mailbox 1.5]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * The restUrl value can be used to make {@link https://docs.microsoft.com/outlook/rest/ | REST API} calls to the user's mailbox.
         */
        restUrl: string;
        /**
         * Information about the user associated with the mailbox. This includes their account type, display name, email address, and time zone.
         * 
         * More information is under {@link Office.UserProfile}
         */
        userProfile: UserProfile;
        /**
         * Adds an event handler for a supported event.
         *
         * Currently, the only supported event type is `Office.EventType.ItemChanged`.
         * In Preview, `Office.EventType.OfficeThemeChanged` is also supported.
         *
         * [Api set: Mailbox 1.5]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param eventType - The event that should invoke the handler.
         * @param handler - The function to handle the event. The function must accept a single parameter, which is an object literal. 
         *                The type property on the parameter will match the eventType parameter passed to addHandlerAsync.
         * @param options - Optional. Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        addHandlerAsync(eventType: Office.EventType | string, handler: (type: Office.EventType) => void, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Adds an event handler for a supported event.
         *
         * Currently, the only supported event type is `Office.EventType.ItemChanged`.
         * In Preview, `Office.EventType.OfficeThemeChanged` is also supported.
         *
         * [Api set: Mailbox 1.5]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param eventType - The event that should invoke the handler.
         * @param handler - The function to handle the event. The function must accept a single parameter, which is an object literal. 
         *                The type property on the parameter will match the eventType parameter passed to addHandlerAsync.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        addHandlerAsync(eventType: Office.EventType | string, handler: (type: Office.EventType) => void, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Converts an item ID formatted for REST into EWS format.
         *
         * Item IDs retrieved via a REST API (such as the Outlook Mail API or the Microsoft Graph) use a different format than the format used by 
         * Exchange Web Services (EWS). The convertToEwsId method converts a REST-formatted ID into the proper format for EWS.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param itemId - An item ID formatted for the Outlook REST APIs.
         * @param restVersion - A value indicating the version of the Outlook REST API used to retrieve the item ID.
         */
        convertToEwsId(itemId: string, restVersion: MailboxEnums.RestVersion | string): string;
        /**
         * Gets a dictionary containing time information in local client time.
         *
         * The dates and times used by a mail app for Outlook on the web or desktop clients can use different time zones. 
         * Outlook uses the client computer time zone; Outlook on the web uses the time zone set on the Exchange Admin Center (EAC). 
         * You should handle date and time values so that the values you display on the user interface are always consistent with the time zone that 
         * the user expects.
         *
         * If the mail app is running in Outlook on desktop clients, the convertToLocalClientTime method will return a dictionary object with the values set to the 
         * client computer time zone. 
         * If the mail app is running in Outlook on the web, the convertToLocalClientTime method will return a dictionary object with the values set to 
         * the time zone specified in the EAC.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param timeValue - A Date object.
         */
        convertToLocalClientTime(timeValue: Date): LocalClientTime;
        /**
         * Converts an item ID formatted for EWS into REST format.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * Item IDs retrieved via EWS or via the itemId property use a different format than the format used by REST APIs (such as the 
         * {@link https://docs.microsoft.com/previous-versions/office/office-365-api/api/version-2.0/mail-rest-operations | Outlook Mail API} or the {@link https://graph.microsoft.io/ | Microsoft Graph}. 
         * The convertToRestId method converts an EWS-formatted ID into the proper format for REST.
         *
         * @param itemId - An item ID formatted for Exchange Web Services (EWS)
         * @param restVersion - A value indicating the version of the Outlook REST API that the converted ID will be used with.
         */
        convertToRestId(itemId: string, restVersion: MailboxEnums.RestVersion | string): string;
        /**
         * Gets a Date object from a dictionary containing time information.
         *
         * The convertToUtcClientTime method converts a dictionary containing a local date and time to a Date object with the correct values for the 
         * local date and time.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param input - The local time value to convert.
         * @returns A Date object with the time expressed in UTC.
         */
        convertToUtcClientTime(input: LocalClientTime): Date;
        /**
         * Displays an existing calendar appointment.
         *
         * The displayAppointmentForm method opens an existing calendar appointment in a new window on the desktop or in a dialog box on 
         * mobile devices.
         *
         * In Outlook on Mac, you can use this method to display a single appointment that is not part of a recurring series, or the 
         * master appointment of a recurring series, but you cannot display an instance of the series. 
         * This is because in Outlook on Mac, you cannot access the properties (including the item ID) of instances of a recurring series.
         *
         * In Outlook on the web, this method opens the specified form only if the body of the form is less than or equal to 32KB number of characters.
         *
         * If the specified item identifier does not identify an existing appointment, a blank pane opens on the client computer or device, and 
         * no error message will be returned.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param itemId - The Exchange Web Services (EWS) identifier for an existing calendar appointment.
         */
        displayAppointmentForm(itemId: string): void;
        /**
         * Displays an existing message.
         *
         * The displayMessageForm method opens an existing message in a new window on the desktop or in a dialog box on mobile devices.
         *
         * In Outlook on the web, this method opens the specified form only if the body of the form is less than or equal to 32 KB number of characters.
         *
         * If the specified item identifier does not identify an existing message, no message will be displayed on the client computer, and 
         * no error message will be returned.
         *
         * Do not use the displayMessageForm with an itemId that represents an appointment. Use the displayAppointmentForm method to display 
         * an existing appointment, and displayNewAppointmentForm to display a form to create a new appointment.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param itemId - The Exchange Web Services (EWS) identifier for an existing message.
         */
        displayMessageForm(itemId: string): void;
        /**
         * Displays a form for creating a new calendar appointment.
         *
         * The displayNewAppointmentForm method opens a form that enables the user to create a new appointment or meeting. 
         * If parameters are specified, the appointment form fields are automatically populated with the contents of the parameters.
         *
         * In Outlook on the web, this method always displays a form with an attendees field. 
         * If you do not specify any attendees as input arguments, the method displays a form with a Save button. 
         * If you have specified attendees, the form would include the attendees and a Send button.
         *
         * In the Outlook rich client and Outlook RT, if you specify any attendees or resources in the requiredAttendees, optionalAttendees, or 
         * resources parameter, this method displays a meeting form with a Send button. 
         * If you don't specify any recipients, this method displays an appointment form with a Save & Close button.
         *
         * If any of the parameters exceed the specified size limits, or if an unknown parameter name is specified, an exception is thrown.
         *
         * **Note**: This method is not supported in Outlook on iOS or Android.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         *
         * @param parameters - An AppointmentForm describing the new appointment. All properties are optional.
         */
        displayNewAppointmentForm(parameters: AppointmentForm): void;
        /**
         * Displays a form for creating a new message.
         *
         * The displayNewMessageForm method opens a form that enables the user to create a new message. If parameters are specified, the message form 
         * fields are automatically populated with the contents of the parameters.
         *
         * If any of the parameters exceed the specified size limits, or if an unknown parameter name is specified, an exception is thrown.
         *
         * [Api set: Mailbox 1.6]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
         *
         * @param parameters - A dictionary containing all values to be filled in for the user in the new form. All parameters are optional.
         * 
         *        toRecipients: An array of strings containing the email addresses or an array containing an {@link Office.EmailAddressDetails} object 
         *        for each of the recipients on the To line. The array is limited to a maximum of 100 entries.
         * 
         *        ccRecipients: An array of strings containing the email addresses or an array containing an {@link Office.EmailAddressDetails} object 
         *        for each of the recipients on the Cc line. The array is limited to a maximum of 100 entries.
         * 
         *        bccRecipients: An array of strings containing the email addresses or an array containing an {@link Office.EmailAddressDetails} object 
         *        for each of the recipients on the Bcc line. The array is limited to a maximum of 100 entries.
         * 
         *        subject: A string containing the subject of the message. The string is limited to a maximum of 255 characters.
         * 
         *        htmlBody: The HTML body of the message. The body content is limited to a maximum size of 32 KB.
         * 
         *        attachments: An array of JSON objects that are either file or item attachments.
         * 
         *        attachments.type: Indicates the type of attachment. Must be file for a file attachment or item for an item attachment.
         * 
         *        attachments.name: A string that contains the name of the attachment, up to 255 characters in length.
         * 
         *        attachments.url: Only used if type is set to file. The URI of the location for the file.
         * 
         *        attachments.isInline: Only used if type is set to file. If true, indicates that the attachment will be shown inline in the 
         *        message body, and should not be displayed in the attachment list.
         * 
         *        attachments.itemId: Only used if type is set to item. The EWS item id of the existing e-mail you want to attach to the new message. 
         *        This is a string up to 100 characters.
         */
        displayNewMessageForm(parameters: any): void;
        /**
         * Gets a string that contains a token used to call REST APIs or Exchange Web Services.
         *
         * The getCallbackTokenAsync method makes an asynchronous call to get an opaque token from the Exchange Server that hosts the user's mailbox. 
         * The lifetime of the callback token is 5 minutes.
         *
         * *REST Tokens*
         *
         * When a REST token is requested (options.isRest = true), the resulting token will not work to authenticate Exchange Web Services calls. 
         * The token will be limited in scope to read-only access to the current item and its attachments, unless the add-in has specified the 
         * ReadWriteMailbox permission in its manifest. 
         * If the ReadWriteMailbox permission is specified, the resulting token will grant read/write access to mail, calendar, and contacts, 
         * including the ability to send mail.
         *
         * The add-in should use the restUrl property to determine the correct URL to use when making REST API calls.
         *
         * *EWS Tokens*
         *
         * When an EWS token is requested (options.isRest = false), the resulting token will not work to authenticate REST API calls. 
         * The token will be limited in scope to accessing the current item.
         *
         * The add-in should use the ewsUrl property to determine the correct URL to use when making EWS calls.
         *
         * **Note**: It is recommended that add-ins use the REST APIs instead of Exchange Web Services whenever possible.
         *
         * [Api set: Mailbox 1.5]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @param options - An object literal that contains one or more of the following properties.
         *        isRest: Determines if the token provided will be used for the Outlook REST APIs or Exchange Web Services. Default value is false.
         *        asyncContext: Any state data that is passed to the asynchronous method.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. The token is provided as a string in the `asyncResult.value` property.
         *                 If there was an error, then the `asyncResult.error` and `asyncResult.diagnostics` properties may provide additional information.
         */
        getCallbackTokenAsync(options: Office.AsyncContextOptions & { isRest?: boolean }, callback: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Gets a string that contains a token used to get an attachment or item from an Exchange Server.
         *
         * The getCallbackTokenAsync method makes an asynchronous call to get an opaque token from the Exchange Server that hosts the user's mailbox. 
         * The lifetime of the callback token is 5 minutes.
         *
         * You can pass the token and an attachment identifier or item identifier to a third-party system. 
         * The third-party system uses the token as a bearer authorization token to call the Exchange Web Services (EWS) GetAttachment or 
         * GetItem operation to return an attachment or item. For example, you can create a remote service to get attachments from the selected item.
         *
         * Your app must have the ReadItem permission specified in its manifest to call the getCallbackTokenAsync method in read mode.
         *
         * In compose mode you must call the saveAsync method to get an item identifier to pass to the getCallbackTokenAsync method. 
         * Your app must have ReadWriteItem permissions to call the saveAsync method.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. The token is provided as a string in the `asyncResult.value` property.
         *                 If there was an error, then the `asyncResult.error` and `asyncResult.diagnostics` properties may provide additional information.
         * @param userContext - Optional. Any state data that is passed to the asynchronous method.
         */
        getCallbackTokenAsync(callback: (asyncResult: Office.AsyncResult<string>) => void, userContext?: any): void;
        /**
         * Gets a token identifying the user and the Office Add-in.
         *
         * The token is provided as a string in the asyncResult.value property.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * The getUserIdentityTokenAsync method returns a token that you can use to identify and 
         * {@link https://docs.microsoft.com/outlook/add-ins/authentication | authenticate the add-in and user with a third-party system}.
         *
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         *                 The token is provided as a string in the `asyncResult.value` property.
         *                 If there was an error, then the `asyncResult.error` and `asyncResult.diagnostics` properties may provide additional information.
         * @param userContext - Optional. Any state data that is passed to the asynchronous method.
         */
        getUserIdentityTokenAsync(callback: (asyncResult: Office.AsyncResult<string>) => void, userContext?: any): void;
        /**
         * Makes an asynchronous request to an Exchange Web Services (EWS) service on the Exchange server that hosts the user's mailbox.
         *
         * In these cases, add-ins should use REST APIs to access the user's mailbox instead.
         *
         * The makeEwsRequestAsync method sends an EWS request on behalf of the add-in to Exchange.
         *
         * You cannot request Folder Associated Items with the makeEwsRequestAsync method.
         *
         * The XML request must specify UTF-8 encoding. \<?xml version="1.0" encoding="utf-8"?\>
         *
         * Your add-in must have the ReadWriteMailbox permission to use the makeEwsRequestAsync method. 
         * For information about using the ReadWriteMailbox permission and the EWS operations that you can call with the makeEwsRequestAsync method, 
         * see {@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Specify permissions for mail add-in access to the user's mailbox}.
         *
         * The XML result of the EWS call is provided as a string in the asyncResult.value property. 
         * If the result exceeds 1 MB in size, an error message is returned instead.
         *
         * **Note**: This method is not supported in the following scenarios:
         * 
         * - In Outlook on iOS or Android.
         * 
         * - When the add-in is loaded in a Gmail mailbox.
         *
         * **Note**: The server administrator must set OAuthAuthentication to true on the Client Access Server EWS directory to enable the 
         * makeEwsRequestAsync method to make EWS requests.
         *
         * *Version differences*
         *
         * When you use the makeEwsRequestAsync method in mail apps running in Outlook versions earlier than version 15.0.4535.1004, you should set 
         * the encoding value to ISO-8859-1.
         *
         * `<?xml version="1.0" encoding="iso-8859-1"?>`
         *
         * You do not need to set the encoding value when your mail app is running in Outlook on the web. 
         * You can determine whether your mail app is running in Outlook or Outlook on the web by using the mailbox.diagnostics.hostName property. 
         * You can determine what version of Outlook is running by using the mailbox.diagnostics.hostVersion property.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteMailbox
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param data - The EWS request.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                 The `value` property of the result is the XML of the EWS request provided as a string. 
         *                 If the result exceeds 1 MB in size, an error message is returned instead.
         * @param userContext - Optional. Any state data that is passed to the asynchronous method.
         */
        makeEwsRequestAsync(data: any, callback: (asyncResult: Office.AsyncResult<string>) => void, userContext?: any): void;
        /**
         * Removes the event handlers for a supported event type.
         *
         * Currently, the only supported event type is `Office.EventType.ItemChanged`.
         * In Preview, `Office.EventType.OfficeThemeChanged` is also supported.
         *
         * [Api set: Mailbox 1.5]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param eventType - The event that should revoke the handler.
         * @param options - Optional. Provides an option for preserving context data of any type, unchanged, for use in a callback.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        removeHandlerAsync(eventType: Office.EventType | string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Removes the event handlers for a supported event type.
         *
         * Currently, the only supported event type is `Office.EventType.ItemChanged`.
         * In Preview, `Office.EventType.OfficeThemeChanged` is also supported.
         *
         * [Api set: Mailbox 1.5]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param eventType - The event that should revoke the handler.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        removeHandlerAsync(eventType: Office.EventType | string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
    }

    /**
     * Represents a suggested meeting found in an item. Read mode only.
     *
     * The list of meetings suggested in an email message is returned in the meetingSuggestions property of the Entities object that is returned when 
     * the getEntities or getEntitiesByType method is called on the active item.
     *
     * The start and end values are string representations of a Date object that contains the date and time at which the suggested meeting is to 
     * begin and end. 
     * The values are in the default time zone specified for the current user.
     *
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
     */
    interface MeetingSuggestion {
        /**
         * Gets the attendees for a suggested meeting.
         */
        attendees: EmailUser[];
        /**
         * Gets the date and time that a suggested meeting is to end.
         */
        end: string;
        /**
         * Gets the location of a suggested meeting.
         */
        location: string;
        /**
         * Gets a string that was identified as a meeting suggestion.
         */
        meetingString: string;
        /**
         * Gets the date and time that a suggested meeting is to begin.
         */
        start: string;
        /**
         * Gets the subject of a suggested meeting.
         */
        subject: string;
    }
    /**
     * An array of NotificationMessageDetails objects are returned by the NotificationMessages.getAllAsync method.
     *
     * [Api set: Mailbox 1.3]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     */
    interface NotificationMessageDetails {
        /**
         * The identifier for the notification message.
         */
        key?: string;
        /**
         * Specifies the ItemNotificationMessageType of message. If type is ProgressIndicator or ErrorMessage, an icon is automatically supplied and 
         * the message is not persistent. Therefore the icon and persistent properties are not valid for these types of messages. 
         * Including them will result in an ArgumentException. 
         * If type is ProgressIndicator, the developer should remove or replace the progress indicator when the action is complete.
         */
        type: MailboxEnums.ItemNotificationMessageType | string;
        /**
         * A reference to an icon that is defined in the manifest in the Resources section. It appears in the infobar area. 
         * It is only applicable if the type is InformationalMessage. Specifying this parameter for an unsupported type results in an exception.
         */
        icon?: string;
        /**
         * The text of the notification message. Maximum length is 150 characters. 
         * If the developer passes in a longer string, an ArgumentOutOfRange exception is thrown.
         */
        message: string;
        /**
         * Only applicable when type is InformationalMessage. If true, the message remains until removed by this add-in or dismissed by the user. 
         * If false, it is removed when the user navigates to a different item. 
         * For error notifications, the message persists until the user sees it once. 
         * Specifying this parameter for an unsupported type throws an exception.
         */
        persistent?: Boolean;
    }
    /**
     * The NotificationMessages object is returned as the notificationMessages property of an item.
     *
     * [Api set: Mailbox 1.3]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     */
    interface NotificationMessages {
        /**
         * Adds a notification to an item.
         *
         * There are a maximum of 5 notifications per message. Setting more will return a NumberOfNotificationMessagesExceeded error.
         *
         * @param key - A developer-specified key used to reference this notification message. 
         *             Developers can use it to modify this message later. It can't be longer than 32 characters.
         * @param JSONmessage - A JSON object that contains the notification message to be added to the item. 
         *                    It contains a NotificationMessageDetails object.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        addAsync(key: string, JSONmessage: NotificationMessageDetails, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Adds a notification to an item.
         *
         * There are a maximum of 5 notifications per message. Setting more will return a NumberOfNotificationMessagesExceeded error.
         *
         * @param key - A developer-specified key used to reference this notification message. Developers can use it to modify this message later. 
         *             It can't be longer than 32 characters.
         * @param JSONmessage - A JSON object that contains the notification message to be added to the item. 
         *                    It contains a NotificationMessageDetails object.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        addAsync(key: string, JSONmessage: NotificationMessageDetails, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Returns all keys and messages for an item.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                 The `value` property of the result is an array of NotificationMessageDetails objects.
         */
        getAllAsync(options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<NotificationMessageDetails[]>) => void): void;
        /**
         * Returns all keys and messages for an item.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                 The `value` property of the result is an array of NotificationMessageDetails objects.
         */
        getAllAsync(callback?: (asyncResult: Office.AsyncResult<NotificationMessageDetails[]>) => void): void;
        /**
         * Removes a notification message for an item.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @param key - The key for the notification message to remove.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        removeAsync(key: string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Removes a notification message for an item.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param key - The key for the notification message to remove.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        removeAsync(key: string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Replaces a notification message that has a given key with another message.
         *
         * If a notification message with the specified key doesn't exist, replaceAsync will add the notification.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @param key - The key for the notification message to replace. It can't be longer than 32 characters.
         * @param JSONmessage - A JSON object that contains the new notification message to replace the existing message. 
         *                    It contains a NotificationMessageDetails object.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        replaceAsync(key: string, JSONmessage: NotificationMessageDetails, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Replaces a notification message that has a given key with another message.
         *
         * If a notification message with the specified key doesn't exist, replaceAsync will add the notification.
         *
         * [Api set: Mailbox 1.3]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param key - The key for the notification message to replace. It can't be longer than 32 characters.
         * @param JSONmessage - A JSON object that contains the new notification message to replace the existing message. 
         *                    It contains a NotificationMessageDetails object.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        replaceAsync(key: string, JSONmessage: NotificationMessageDetails, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
    }
    /**
     * Represents a phone number identified in an item. Read mode only.
     *
     * An array of PhoneNumber objects containing the phone numbers found in an email message is returned in the phoneNumbers property of the 
     * Entities object that is returned when you call the getEntities method on the selected item.
     *
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
     */
    interface PhoneNumber {
        /**
         * Gets a string containing a phone number. This string contains only the digits of the telephone number and excludes characters like parentheses and hyphens, if they exist in the original item.
         */
        phoneString: string;
        /**
         * Gets the text that was identified in an item as a phone number.
         */
        originalPhoneString: string;
        /**
         * Gets a string that identifies the type of phone number: Home, Work, Mobile, Unspecified.
         */
        type: string;
    }
    /**
     * [Api set: Mailbox 1.1]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
     */
    interface Recipients {
        /**
         * Adds a recipient list to the existing recipients for an appointment or message.
         *
         * The recipients parameter can be an array of one of the following:
         *
         * - Strings containing SMTP email addresses
         *
         * - EmailUser objects
         *
         * - EmailAddressDetails objects
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - NumberOfRecipientsExceeded: The number of recipients exceeded 100 entries.
         * 
         * @param recipients - The recipients to add to the recipients list.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. If adding the recipients fails, the asyncResult.error property will contain an error code.
         */
        addAsync(recipients: (string | EmailUser | EmailAddressDetails)[], options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Adds a recipient list to the existing recipients for an appointment or message.
         *
         * The recipients parameter can be an array of one of the following:
         *
         * - Strings containing SMTP email addresses
         *
         * - {@link Office.EmailUser} objects
         *
         * - {@link Office.EmailAddressDetails} objects
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**:
         * 
         * - NumberOfRecipientsExceeded: The number of recipients exceeded 100 entries.
         *
         * @param recipients - The recipients to add to the recipients list.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. If adding the recipients fails, the asyncResult.error property will contain an error code.
         */
        addAsync(recipients: (string | EmailUser | EmailAddressDetails)[], callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Gets a recipient list for an appointment or message.
         *
         * When the call completes, the asyncResult.value property will contain an array of {@link Office.EmailAddressDetails} objects.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * @param options - An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         *                 The `value` property of the result is an array of EmailAddressDetails objects.
         */
        getAsync(options: Office.AsyncContextOptions, callback: (asyncResult: Office.AsyncResult<EmailAddressDetails[]>) => void): void;
        /**
         * Gets a recipient list for an appointment or message.
         *
         * When the call completes, the asyncResult.value property will contain an array of {@link Office.EmailAddressDetails} objects.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         *
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         *                 The `value` property of the result is an array of EmailAddressDetails objects.
         */
        getAsync(callback: (asyncResult: Office.AsyncResult<EmailAddressDetails[]>) => void): void;
        /**
         * Sets a recipient list for an appointment or message.
         *
         * The setAsync method overwrites the current recipient list.
         *
         * The recipients parameter can be an array of one of the following:
         *
         * - Strings containing SMTP email addresses
         *
         * - {@link Office.EmailUser} objects
         *
         * - {@link Office.EmailAddressDetails} objects
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**:
         * 
         * - NumberOfRecipientsExceeded: The number of recipients exceeded 100 entries.
         * 
         * @param recipients - The recipients to add to the recipients list.
         * @param options - An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         *                 If setting the recipients fails the asyncResult.error property will contain a code that indicates any error that occurred 
         *                 while adding the data.
         */
        setAsync(recipients: (string | EmailUser | EmailAddressDetails)[], options: Office.AsyncContextOptions, callback: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Sets a recipient list for an appointment or message.
         *
         * The setAsync method overwrites the current recipient list.
         *
         * The recipients parameter can be an array of one of the following:
         *
         * - Strings containing SMTP email addresses
         *
         * - {@link Office.EmailUser} objects
         *
         * - {@link Office.EmailAddressDetails} objects
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**:
         * 
         * - NumberOfRecipientsExceeded: The number of recipients exceeded 100 entries.
         *
         * @param recipients - The recipients to add to the recipients list.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         *                 If setting the recipients fails the asyncResult.error property will contain a code that indicates any error that occurred 
         *                 while adding the data.
         */
        setAsync(recipients: (string | EmailUser | EmailAddressDetails)[], callback: (asyncResult: Office.AsyncResult<void>) => void): void;
    }

    /**
     * The recurrence object provides methods to get and set the recurrence pattern of appointments but only get the recurrence pattern of 
     * meeting requests. 
     * It will have a dictionary with the following keys: seriesTime, recurrenceType, recurrenceProperties, and recurrenceTimeZone (optional).
     * 
     * [Api set: Mailbox 1.7]
     * 
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     * 
     * **States**
     * 
     * <table>
     *   <tr>
     *     <th>State</th>
     *     <th>Editable?</th>
     *     <th>Viewable?</th>
     *   </tr>
     *   <tr>
     *     <td>Appointment Organizer - Compose Series</td>
     *     <td>Yes (setAsync)</td>
     *     <td>Yes (getAsync)</td>
     *   </tr>
     *   <tr>
     *     <td>Appointment Organizer - Compose Instance</td>
     *     <td>No (setAsync returns error)</td>
     *     <td>Yes (getAsync)</td>
     *   </tr>
     *   <tr>
     *     <td>Appointment Attendee - Read Series</td>
     *     <td>No (setAsync not available)</td>
     *     <td>Yes (item.recurrence)</td>
     *   </tr>
     *   <tr>
     *     <td>Appointment Attendee - Read Instance</td>
     *     <td>No (setAsync not available)</td>
     *     <td>Yes (item.recurrence)</td>
     *   </tr>
     *   <tr>
     *     <td>Meeting Request - Read Series</td>
     *     <td>No (setAsync not available)</td>
     *     <td>Yes (item.recurrence)</td>
     *   </tr>
     *   <tr>
     *     <td>Meeting Request - Read Instance</td>
     *     <td>No (setAsync not available)</td>
     *     <td>Yes (item.recurrence)</td>
     *   </tr>
     * </table>
     */
    interface Recurrence {
        /**
         * Gets or sets the properties of the recurring appointment series.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        recurrenceProperties: RecurrenceProperties;
        /**
         * Gets or sets the properties of the recurring appointment series.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        recurrenceTimeZone: RecurrenceTimeZone;

        /**
         * Gets or sets the type of the recurring appointment series.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        recurrenceType: MailboxEnums.RecurrenceType | string;

        /**
         * The {@link Office.SeriesTime} object enables you to manage the start and end dates of the recurring appointment series and the usual start 
         * and end times of instances. **This object is not in UTC time.** 
         * Instead, it is set in the time zone specified by the recurrenceTimeZone value or defaulted to the item's time zone.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        seriesTime: SeriesTime;

        /**
         * Returns the current recurrence object of an appointment series.
         * 
         * This method returns the entire recurrence object for the appointment series.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         *                 The `value` property of the result is a Recurrence object.
         */
        getAsync(options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<Recurrence>) => void): void;

        /**
         * Returns the current recurrence object of an appointment series.
         * 
         * This method returns the entire recurrence object for the appointment series.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         * 
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         *                 The `value` property of the result is a Recurrence object.
         */
        getAsync(callback?: (asyncResult: Office.AsyncResult<Recurrence>) => void): void;

        /**
         * Sets the recurrence pattern of an appointment series.
         * 
         * **Note**: setAsync should only be available for series items and not instance items.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - InvalidEndTime: The appointment end time is before its start time.
         * 
         * @param recurrencePattern - A recurrence object.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        setAsync(recurrencePattern: Recurrence, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;

        /**
         * Sets the recurrence pattern of an appointment series.
         * 
         * **Note**: setAsync should only be available for series items and not instance items.
         * 
         * [Api set: Mailbox 1.7]
         * 
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**:
         * 
         * - InvalidEndTime: The appointment end time is before its start time.
         * 
         * @param recurrencePattern - A recurrence object.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter, 
         *                asyncResult, which is an Office.AsyncResult object.
         */
        setAsync(recurrencePattern: Recurrence, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
    }

    /**
     * Represents the properties of the recurrence.
     * 
     * [Api set: Mailbox 1.7]
     * 
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     */
    interface RecurrenceProperties {
        /**
         * Represents the period between instances of the same recurring series.
         */
        interval: number;
        /**
         * Represents the day of the month.
         */
        dayOfMonth: number;
        /**
         * Represents the day of the week or type of day, for example, weekend day vs weekday.
         */
        dayOfWeek: MailboxEnums.Days | string;
        /**
         * Represents the set of days for this recurrence. Valid values are: 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', and 'Sun'.
         */
        days: MailboxEnums.Days[] | string[];
        /**
         * Represents the number of the week in the selected month e.g. 'first' for first week of the month.
         */
        weekNumber: MailboxEnums.WeekNumber | string;
        /**
         * Represents the month.
         */
        month: MailboxEnums.Month | string;
        /**
         * Represents your chosen first day of the week otherwise the default is the value in the current user's settings. 
         * Valid values are: 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', and 'Sun'.
         */
        firstDayOfWeek: MailboxEnums.Days | string;
    }

    /**
     * Represents the time zone of the recurrence.
     * 
     * [Api set: Mailbox 1.7]
     * 
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     */
    interface RecurrenceTimeZone {
        /**
         * Represents the name of the recurrence time zone.
         */
        name: MailboxEnums.RecurrenceTimeZone | string;

        /**
         * Integer value representing the difference in minutes between the local time zone and UTC at the date that the meeting series began.
         */
        offset: number;
    }

    /**
     * A file or item attachment. Used when displaying a reply form.
     */
    interface ReplyFormAttachment {
        /**
         * Indicates the type of attachment. Must be file for a file attachment or item for an item attachment.
         */
        type: string;
        /**
         * A string that contains the name of the attachment, up to 255 characters in length.
         */
        name: string;
        /**
         * Only used if type is set to file. The URI of the location for the file.
         */
        url?: string;
        /**
         * Only used if type is set to file. If true, indicates that the attachment will be shown inline in the message body, and should not be 
         * displayed in the attachment list.
         */
        inLine?: boolean;
        /**
         * Only used if type is set to item. The EWS item id of the attachment. This is a string up to 100 characters.
         */
        itemId?: string;
    }

    /**
     * A ReplyFormData object that contains body or attachment data and a callback function. Used when displaying a reply form.
     */
    interface ReplyFormData {
        /**
         * A string that contains text and HTML and that represents the body of the reply form. The string is limited to 32 KB.
         */
        htmlBody?: string;
        /**
         * An array of {@link Office.ReplyFormAttachment} that are either file or item attachments.
         */
        attachments?: ReplyFormAttachment[];
        /**
         * When the reply display call completes, the function passed in the callback parameter is called with a single parameter, 
         * asyncResult, which is an Office.AsyncResult object.
         */
        callback?: (asyncResult: Office.AsyncResult<any>) => void;
    }
    /**
     * The settings created by using the methods of the RoamingSettings object are saved per add-in and per user. 
     * That is, they are available only to the add-in that created them, and only from the user's mail box in which they are saved.
     *
     * While the Outlook Add-in API limits access to these settings to only the add-in that created them, these settings should not be considered 
     * secure storage. They can be accessed by Exchange Web Services or Extended MAPI. 
     * They should not be used to store sensitive information such as user credentials or security tokens.
     *
     * The name of a setting is a String, while the value can be a String, Number, Boolean, null, Object, or Array.
     *
     * The RoamingSettings object is accessible via the roamingSettings property in the Office.context namespace.
     *
     * **Important**: The RoamingSettings object is initialized from the persisted storage only when the add-in is first loaded. 
     * For task panes, this means that it is only initialized when the task pane first opens. 
     * If the task pane navigates to another page or reloads the current page, the in-memory object is reset to its initial values, even if 
     * your add-in has persisted changes.
     * The persisted changes will not be available until the task pane (or item in the case of UI-less add-ins) is closed and reopened.
     *
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     */
    interface RoamingSettings {
        /**
         * Retrieves the specified setting.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param name - The case-sensitive name of the setting to retrieve.
         * @returns Type: String | Number | Boolean | Object | Array
         */
        get(name: string): any;
        /**
         * Removes the specified setting
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param name - The case-sensitive name of the setting to remove.
         */
        remove(name: string): void;
        /**
         * Saves the settings.
         *
         * Any settings previously saved by an add-in are loaded when it is initialized, so during the lifetime of the session you can just use 
         * the set and get methods to work with the in-memory copy of the settings property bag. 
         * When you want to persist the settings so that they are available the next time the add-in is used, use the saveAsync method.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         */
        saveAsync(callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Sets or creates the specified setting.
         *
         * The set method creates a new setting of the specified name if it does not already exist, or sets an existing setting of the specified name. 
         * The value is stored in the document as the serialized JSON representation of its data type.
         *
         * A maximum of 32KB is available for the settings of each add-in.
         *
         * Any changes made to settings using the set function will not be saved to the server until the saveAsync function is called.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: Restricted
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * @param name - The case-sensitive name of the setting to set or create.
         * @param value - Specifies the value to be stored.
         */
        set(name: string, value: any): void;
    }

    /**
     * The SeriesTime object provides methods to get and set the dates and times of appointments in a recurring series and get the dates and times of 
     * meeting requests in a recurring series.
     * 
     * [Api set: Mailbox 1.7]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     */
    interface SeriesTime {
        /**
         * Gets the duration in minutes of a usual instance in a recurring appointment series.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        getDuration(): number;

        /**
         * Gets the end date of a recurrence pattern in the following {@link https://www.iso.org/iso-8601-date-and-time-format.html | ISO 8601} date format: "YYYY-MM-DD"
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        getEndDate(): string;

        /**
         * Gets the end time of a usual appointment or meeting request instance of a recurrence pattern in whichever time zone that the user or 
         * add-in set the recurrence pattern using the following {@link https://www.iso.org/iso-8601-date-and-time-format.html | ISO 8601} format: 
         * "THH:mm:ss:mmm"
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        getEndTime(): string;

        /**
         * Gets the start date of a recurrence pattern in the following {@link https://www.iso.org/iso-8601-date-and-time-format.html | ISO 8601} date format: "YYYY-MM-DD"
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        getStartDate(): string;

        /**
         * Gets the start time of a usual appointment instance of a recurrence pattern in whichever time zone that the user/add-in set the 
         * recurrence pattern using the following {@link https://www.iso.org/iso-8601-date-and-time-format.html | ISO 8601} format: "THH:mm:ss:mmm"
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        getStartTime(): string;

        /**
         * Sets the duration of all appointments in a recurrence pattern. This will also change the end time of the recurrence pattern.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * @param minutes - The length of the appointment in minutes.
         */
        setDuration(minutes: number): void;

        /**
         * Sets the end date of a recurring appointment series.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * @param year - The year value of the end date.
         * @param month - The month value of the end date. Valid range is 0-11 where 0 represents the 1st month and 11 represents the 12th month.
         * @param day - The day value of the end date.
         */
        setEndDate(year: number, month: number, day: number): void;
        /**
         * Sets the end date of a recurring appointment series.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose
         * 
         * @param date - End date of the recurring appointment series represented in the {@link https://www.iso.org/iso-8601-date-and-time-format.html | ISO 8601} date format: "YYYY-MM-DD".
         */
        setEndDate(date: string): void;
        /**
         * Sets the start date of a recurring appointment series.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose
         * 
         * @param year - The year value of the start date.
         * @param month - The month value of the start date. Valid range is 0-11 where 0 represents the 1st month and 11 represents the 12th month.
         * @param day - The day value of the start date.
         */
        setStartDate(year:number, month:number, day:number): void;

        /**
         * Sets the start date of a recurring appointment series.
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose
         * 
         * @param date - Start date of the recurring appointment series represented in the {@link https://www.iso.org/iso-8601-date-and-time-format.html | ISO 8601} date format: "YYYY-MM-DD".
         */
        setStartDate(date:string): void;

        /**
         * Sets the start time of all instances of a recurring appointment series in whichever time zone the recurrence pattern is set 
         * (the item's time zone is used by default).
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose
         * 
         * @param hours - The hour value of the start time. Valid range: 0-24.
         * @param minutes - The minute value of the start time. Valid range: 0-59.
         */
        setStartTime(hours: number, minutes: number): void;

        /**
         * Sets the start time of all instances of a recurring appointment series in whichever time zone the recurrence pattern is set 
         * (the item's time zone is used by default).
         * 
         * [Api set: Mailbox 1.7]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose
         * 
         * @param time - Start time of all instances represented by standard datetime string format: "THH:mm:ss:mmm".
         */
        setStartTime(time: string): void;
    }

    /**
     * Represents the properties of an appointment or message in a shared folder, mailbox, or calendar.
     *
     * [Api set: Mailbox Preview]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     * 
     * @beta
     */
    interface SharedProperties {
        /**
         * The email address of the owner of a shared item.
         */
        owner: String;
        /**
         * The remote REST URL related to the owner’s mailbox.
         */
        restUrl: String;
        /**
         * The permissions that the delegate has on a shared folder.
         */
        delegatePermissions: MailboxEnums.DelegatePermissions;
    }

    /**
     * Provides methods to get and set the subject of an appointment or message in an Outlook add-in.
     *
     * [Api set: Mailbox 1.1]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
     */
    interface Subject {
        /**
         * Gets the subject of an appointment or message.
         *
         * The getAsync method starts an asynchronous call to the Exchange server to get the subject of an appointment or message.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * @param options - An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult.
         *                 The `value` property of the result is the subject of the item.
         */
        getAsync(options: Office.AsyncContextOptions, callback: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Gets the subject of an appointment or message.
         * 
         * The getAsync method starts an asynchronous call to the Exchange server to get the subject of an appointment or message.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         *
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                  The `value` property of the result is the subject of the item.
         */
        getAsync(callback: (asyncResult: Office.AsyncResult<string>) => void): void;
        /**
         * Sets the subject of an appointment or message.
         *
         * The setAsync method starts an asynchronous call to the Exchange server to set the subject of an appointment or message. 
         * Setting the subject overwrites the current subject, but leaves any prefixes, such as "Fwd:" or "Re:" in place.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**:
         * 
         * - DataExceedsMaximumSize: The subject parameter is longer than 255 characters.
         * 
         * @param subject - The subject of the appointment or message. The string is limited to 255 characters.
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. If setting the subject fails, the asyncResult.error property will contain an error code.
         */
        setAsync(subject: string, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Sets the subject of an appointment or message.
         *
         * The setAsync method starts an asynchronous call to the Exchange server to set the subject of an appointment or message. 
         * Setting the subject overwrites the current subject, but leaves any prefixes, such as "Fwd:" or "Re:" in place.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**: 
         * 
         * - DataExceedsMaximumSize: The subject parameter is longer than 255 characters.
         *
         * @param subject - The subject of the appointment or message. The string is limited to 255 characters.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. If setting the subject fails, the asyncResult.error property will contain an error code.
         */
        setAsync(data: string, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;

    }
    /**
     * Represents a suggested task identified in an item. Read mode only.
     *
     * The list of tasks suggested in an email message is returned in the taskSuggestions property of the {@link Office.Entities | Entities} object 
     * that is returned when the getEntities or getEntitiesByType method is called on the active item.
     *
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Read
     */
    interface TaskSuggestion {
        /**
         * Gets the users that should be assigned a suggested task.
         */
        assignees: EmailUser[];
        /**
         * Gets the text of an item that was identified as a task suggestion.
         */
        taskString: string;
    }
    /**
     * The Time object is returned as the start or end property of an appointment in compose mode.
     *
     * [Api set: Mailbox 1.1]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose
     */
    interface Time {
        /**
         * Gets the start or end time of an appointment.
         *
         * The date and time is provided as a Date object in the asyncResult.value property. The value is in Coordinated Universal Time (UTC). 
         * You can convert the UTC time to the local client time by using the convertToLocalClientTime method.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * @param options - An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                  The `value` property of the result is a Date object.
         */
        getAsync(options: Office.AsyncContextOptions, callback: (asyncResult: Office.AsyncResult<Date>) => void): void;
        /**
         * Gets the start or end time of an appointment.
         *
         * The date and time is provided as a Date object in the asyncResult.value property. The value is in Coordinated Universal Time (UTC). 
         * You can convert the UTC time to the local client time by using the convertToLocalClientTime method.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         *
         * @param callback - When the method completes, the function passed in the callback parameter is called with a single parameter of type Office.AsyncResult.
         *                  The `value` property of the result is a Date object.
         */
        getAsync(callback: (asyncResult: Office.AsyncResult<Date>) => void): void;
        /**
         * Sets the start or end time of an appointment.
         *
         * If the setAsync method is called on the start property, the end property will be adjusted to maintain the duration of the appointment as 
         * previously set. If the setAsync method is called on the end property, the duration of the appointment will be extended to the new end time.
         *
         * The time must be in UTC; you can get the correct UTC time by using the convertToUtcClientTime method.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**:
         * 
         * - InvalidEndTime: The appointment end time is before the appointment start time.
         * 
         * @param dateTime - A date-time object in Coordinated Universal Time (UTC).
         * @param options - Optional. An object literal that contains one or more of the following properties.
         *        asyncContext: Developers can provide any object they wish to access in the callback method.
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         *                 If setting the date and time fails, the asyncResult.error property will contain an error code.
         */
        setAsync(dateTime: Date, options?: Office.AsyncContextOptions, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;
        /**
         * Sets the start or end time of an appointment.
         *
         * If the setAsync method is called on the start property, the end property will be adjusted to maintain the duration of the appointment as 
         * previously set. If the setAsync method is called on the end property, the duration of the appointment will be extended to the new end time.
         *
         * The time must be in UTC; you can get the correct UTC time by using the convertToUtcClientTime method.
         *
         * [Api set: Mailbox 1.1]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadWriteItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose 
         * 
         * **Errors**:
         * 
         * - InvalidEndTime: The appointment end time is before the appointment start time.
         *
         * @param dateTime - A date-time object in Coordinated Universal Time (UTC).
         * @param callback - Optional. When the method completes, the function passed in the callback parameter is called with a single parameter of 
         *                 type Office.AsyncResult. 
         *                 If setting the date and time fails, the asyncResult.error property will contain an error code.
         */
        setAsync(dateTime: Date, callback?: (asyncResult: Office.AsyncResult<void>) => void): void;

    }
    /**
     * Information about the user associated with the mailbox. This includes their account type, display name, email address, and time zone.
     * 
     * [Api set: Mailbox 1.0]
     *
     * @remarks
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
     * 
     * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
     */
    interface UserProfile {
        /**
         * Gets the account type of the user associated with the mailbox. 
         *
         * **Note**: This member is currently only supported in Outlook 2016 or later on Mac, build 16.9.1212 and greater.
         *
         * [Api set: Mailbox 1.6]
         *
         * @remarks
         *
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         *
         * The possible account types are listed in the following table.
         * 
         * <table>
         *   <tr>
         *     <th>Value</th>
         *     <th>Description?</th>
         *   </tr>
         *   <tr>
         *     <td>enterprise</td>
         *     <td>The mailbox is on an on-premises Exchange server.</td>
         *   </tr>
         *   <tr>
         *     <td>gmail</td>
         *     <td>The mailbox is associated with a Gmail account.</td>
         *   </tr>
         *   <tr>
         *     <td>office365</td>
         *     <td>The mailbox is associated with an Office 365 work or school account.</td>
         *   </tr>
         *   <tr>
         *     <td>outlookCom</td>
         *     <td>The mailbox is associated with a personal Outlook.com account.</td>
         *   </tr>
         * </table>
         */
        accountType: string;
        /**
         * Gets the user's display name.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        displayName: string;
        /**
         * Gets the user's display name.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        emailAddress: string;
        /**
         * Gets the user's SMTP email address.
         *
         * [Api set: Mailbox 1.0]
         *
         * @remarks
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/understanding-outlook-add-in-permissions | Minimum permission level}**: ReadItem
         * 
         * **{@link https://docs.microsoft.com/outlook/add-ins/#extension-points | Applicable Outlook mode}**: Compose or Read
         */
        timeZone: string;
    }
}


////////////////////////////////////////////////////////////////
/////////////////////// End Exchange APIs //////////////////////
////////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////
//////////////// Begin OfficeExtension runtime /////////////////
////////////////////////////////////////////////////////////////

declare namespace OfficeExtension {
    /**
     * An abstract proxy object that represents an object in an Office document. 
     * You create proxy objects from the context (or from other proxy objects), add commands to a queue to act on the object, and then synchronize the 
     * proxy object state with the document by calling `context.sync()`.
     */
    class ClientObject {
        /** The request context associated with the object */
        context: ClientRequestContext;
        /**
         *  Returns a boolean value for whether the corresponding object is a null object. You must call `context.sync()` before reading the 
         * isNullObject property.
         */
        isNullObject: boolean;
    }

    /**
     * Specifies which properties of an object should be loaded. This load happens when the sync() method is executed.
     * This synchronizes the states between Office objects and corresponding JavaScript proxy objects.
     * 
     * @remarks
     * 
     * For Word, the preferred method for specifying the properties and paging information is by using a string literal. 
     * The first two examples show the preferred way to request the text and font size properties for paragraphs in a paragraph collection:
     * 
     * `context.load(paragraphs, 'text, font/size');`
     * 
     * `paragraphs.load('text, font/size');`
     * 
     * Here is a similar example using object notation (includes paging):
     * 
     * `context.load(paragraphs, {select: 'text, font/size', expand: 'font', top: 50, skip: 0});`
     * 
     * `paragraphs.load({select: 'text, font/size', expand: 'font', top: 50, skip: 0});`
     * 
     * Note that if we don't specify the specific properties on the font object in the select statement, the expand statement by itself would 
     * indicate that all of the font properties are loaded.
     */
    interface LoadOption {
        /**
         * A comma-delimited string, or array of strings, that specifies the properties to load.
         */
        select?: string | string[];
        /**
         * A comma-delimited string, or array of strings, that specifies the navigation properties to load.
         */
        expand?: string | string[];
        /**
         * Only usable on collection types. Specifies the maximum number of collection items that can be included in the result.
         */
        top?: number;
        /**
         * Only usable on collection types. Specifies the number of items in the collection that are to be skipped and not included in the result. 
         * If top is specified, the result set will start after skipping the specified number of items.
         */
        skip?: number;
    }
    /**
     * Provides an option for suppressing an error when the object that is used to set multiple properties tries to set read-only properties.
     */
    interface UpdateOptions {
        /**
         * Throw an error if the passed-in property list includes read-only properties (default = true).
         */
        throwOnReadOnly?: boolean
    }

    /**
     * Additional options passed into `{Host}.run(...)`.
     */
    export interface RunOptions<T> {
        /**
         * The URL of the remote workbook and the request headers to be sent.
         */
        session?: RequestUrlAndHeaderInfo | T;
    
        /**
         *  A previously-created context, or API object, or array of objects. 
         * The batch will use the same RequestContext as the passed-in object, which means that any changes applied to the object will be picked up 
         * by `context.sync()`.
         */
        previousObjects?: ClientObject | ClientObject[] | ClientRequestContext;
    }

    /** Contains debug information about the request context. */
    interface RequestContextDebugInfo {
        /**
         * The statements to be executed in the host.
         *
         * These statements may not match the code exactly as written, but will be a close approximation.
         */
        pendingStatements: string[];
    }

    /**
     * An abstract RequestContext object that facilitates requests to the host Office application. 
     * The `Excel.run` and `Word.run` methods provide a request context.
     */
    class ClientRequestContext {
        constructor(url?: string);

        /** Collection of objects that are tracked for automatic adjustments based on surrounding changes in the document. */
        trackedObjects: TrackedObjects;

        /** Request headers */
        requestHeaders: { [name: string]: string };

        /**
         * Queues up a command to load the specified properties of the object. You must call `context.sync()` before reading the properties. 
         * 
         * @param object The object whose properties are loaded.
         * @param option A comma-delimited string, or array of strings, that specifies the properties to load, or an 
         * {@link OfficeExtension.LoadOption} object.
         */
        load(object: ClientObject, option?: string | string[] | LoadOption): void;

        /**
        * Queues up a command to recursively load the specified properties of the object and its navigation properties.
        * 
        * You must call `context.sync()` before reading the properties.
        *
        * @param object The object to be loaded.
        * @param options The key-value pairing of load options for the types, such as 
        *                `{ "Workbook": "worksheets,tables",  "Worksheet": "tables",  "Tables": "name" }`
        * @param maxDepth The maximum recursive depth.
        */
        loadRecursive(object: ClientObject, options: { [typeName: string]: string | string[] | LoadOption }, maxDepth?: number): void;

        /**
         * Adds a trace message to the queue. If the promise returned by `context.sync()` is rejected due to an error, this adds a ".traceMessages" 
         * array to the OfficeExtension.Error object, containing all trace messages that were executed. 
         * These messages can help you monitor the program execution sequence and detect the cause of the error. 
         */
        trace(message: string): void;

        /**
         * Synchronizes the state between JavaScript proxy objects and the Office document, by executing instructions queued on the request context 
         * and retrieving properties of loaded Office objects for use in your code. 
         * This method returns a promise, which is resolved when the synchronization is complete.
         */
        sync<T>(passThroughValue?: T): Promise<T>;

        /** Debug information */
        readonly debugInfo: RequestContextDebugInfo;
    }

    interface EmbeddedOptions {
        sessionKey?: string,
        container?: HTMLElement,
        id?: string;
        timeoutInMilliseconds?: number;
        height?: string;
        width?: string;
    }

    class EmbeddedSession {
        constructor(url: string, options?: EmbeddedOptions);
        public init(): Promise<any>;
    }

    /** Contains the result for methods that return primitive types. The object's value property is retrieved from the document after `context.sync()` is invoked. */
    class ClientResult<T> {
        /** The value of the result that is retrieved from the document after `context.sync()` is invoked. */
        value: T;
    }

    /** Configuration */
    var config: {
        /**
         * Determines whether to log additional error information upon failure.
         *
         * When this property is set to true, the error object will include a "debugInfo.fullStatements" property that lists all statements in the 
         * batch request, including all statements that precede and follow the point of failure.
         *
         * Setting this property to true will negatively impact performance and will log all statements in the batch request, including any statements 
         * that may contain potentially-sensitive data.
         * It is recommended that you only set this property to true during debugging and that you never log the value of 
         * error.debugInfo.fullStatements to an external database or analytics service.
         */
        extendedErrorLogging: boolean;
    };
    /**
     * Provides information about an error.
     */
    interface DebugInfo {
        /** Error code string, such as "InvalidArgument". */
        code: string;
        /** The error message passed through from the host Office application. */
        message: string;
        /** Inner error, if applicable. */
        innerError?: DebugInfo | string;
        /** The object type and property or method name (or similar information), if available. */
        errorLocation?: string;
        /**
         * The statement that caused the error, if available.
         *
         * This statement will never contain any potentially-sensitive data and may not match the code exactly as written, 
         * but will be a close approximation.
         */
        statements?: string;
        /**
         * The statements that closely precede and follow the statement that caused the error, if available.
         *
         * These statements will never contain any potentially-sensitive data and may not match the code exactly as written, 
         * but will be a close approximation.
         */
        surroundingStatements?: string[];
        /**
         * All statements in the batch request (including any potentially-sensitive information that was specified in the request), if available.
         *
         * These statements may not match the code exactly as written, but will be a close approximation.
         */
        fullStatements?: string[];
    }

    /** The error object returned by `context.sync()`, if a promise is rejected due to an error while processing the request. */
    class Error {
        /** Error name: "OfficeExtension.Error".*/
        name: string;
        /** The error message passed through from the host Office application. */
        message: string;
        /** Stack trace, if applicable. */
        stack: string;
        /** Error code string, such as "InvalidArgument". */
        code: string;
        /**
         * Trace messages (if any) that were added via a `context.trace()` invocation before calling `context.sync()`. 
         * If there was an error, this contains all trace messages that were executed before the error occurred. 
         * These messages can help you monitor the program execution sequence and detect the case of the error.
         */
        traceMessages: string[];
        /** Debug info (useful for detailed logging of the error, i.e., via `JSON.stringify(...)`). */
        debugInfo: DebugInfo;
        /** Inner error, if applicable. */
        innerError: Error;
    }

    class ErrorCodes {
        public static accessDenied: string;
        public static generalException: string;
        public static activityLimitReached: string;
        public static invalidObjectPath: string;
        public static propertyNotLoaded: string;
        public static valueNotLoaded: string;
        public static invalidRequestContext: string;
        public static invalidArgument: string;
        public static runMustReturnPromise: string;
        public static cannotRegisterEvent: string;
        public static apiNotFound: string;
        public static connectionFailure: string;
    }

    /**
     * A Promise object that represents a deferred interaction with the host Office application. 
     * The publicly-consumable OfficeExtension.Promise is available starting in ExcelApi 1.2 and WordApi 1.2. 
     * Promises can be chained via ".then", and errors can be caught via ".catch". 
     * Remember to always use a ".catch" on the outer promise, and to return intermediary promises so as not to break the promise chain. 
     * When a browser-provided native Promise implementation is available, OfficeExtension.Promise will switch to use the native Promise instead.
     */
    const Promise: Office.IPromiseConstructor;
    type IPromise<T> = Promise<T>;

    /** Collection of tracked objects, contained within a request context. See "context.trackedObjects" for more information. */
    class TrackedObjects {
        /** 
         * Track a new object for automatic adjustment based on surrounding changes in the document. Only some object types require this. 
         * If you are using an object across ".sync" calls and outside the sequential execution of a ".run" batch, 
         * and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object 
         * to the tracked object collection when the object was first created. 
         * 
         * This method also has the following signature: 
         * 
         * `add(objects: ClientObject[]): void;` Where objects is an array of objects to be tracked.
         */
        add(object: ClientObject): void;
        /**
         * Track a set of objects  for automatic adjustment based on surrounding changes in the document. Only some object types require this. 
         * If you are using an object across ".sync" calls and outside the sequential execution of a ".run" batch, 
         * and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object 
         * to the tracked object collection when the object was first created.
         */
        add(objects: ClientObject[]): void;
        /** 
         * Release the memory associated with an object that was previously added to this collection. 
         * Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. 
         * You will need to call `context.sync()` before the memory release takes effect.
         * 
         * This method also has the following signature: 
         * 
         * `remove(objects: ClientObject[]): void;` Where objects is an array of objects to be removed.
         */
        remove(object: ClientObject): void;
        /**
         * Release the memory associated with an object that was previously added to this collection. 
         * Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. 
         * You will need to call `context.sync()` before the memory release takes effect.
         */
        remove(objects: ClientObject[]): void;
    }

    class EventHandlers<T> {
        constructor(context: ClientRequestContext, parentObject: ClientObject, name: string, eventInfo: EventInfo<T>);
        add(handler: (args: T) => Promise<any>): EventHandlerResult<T>;
        remove(handler: (args: T) => Promise<any>): void;
    }

    class EventHandlerResult<T> {
        constructor(context: ClientRequestContext, handlers: EventHandlers<T>, handler: (args: T) => Promise<any>);
        /** The request context associated with the object */
        context: ClientRequestContext;
        remove(): void;
    }

    interface EventInfo<T> {
        registerFunc: (callback: (args: any) => void) => Promise<any>;
        unregisterFunc: (callback: (args: any) => void) => Promise<any>;
        eventArgsTransformFunc: (args: any) => Promise<T>;
    }

    /**
    * Request URL and headers
    */
    interface RequestUrlAndHeaderInfo {
        /** Request URL */
        url: string;
        /** Request headers */
        headers?: {
            [name: string]: string;
        };
    }
}


////////////////////////////////////////////////////////////////
///////////////// End OfficeExtension runtime //////////////////
////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////
/////////////////////// Begin OfficeCore ///////////////////////
////////////////////////////////////////////////////////////////


declare namespace OfficeCore {
    class RequestContext extends OfficeExtension.ClientRequestContext {
        constructor(url?: string | OfficeExtension.RequestUrlAndHeaderInfo | any);
    }
}


////////////////////////////////////////////////////////////////
//////////////////////// End OfficeCore ////////////////////////
////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////
/////////////////////// Begin Excel APIs ///////////////////////
////////////////////////////////////////////////////////////////

declare namespace Excel {
    /**
     *
     * Represents which cell properties to load, when used as part of a "range.getCellProperties" method.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface CellPropertiesLoadOptions {
        /**
        *
        * Specifies whether to load on the `address` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        address?: boolean;
        /**
        *
        * Specifies whether to load on the `addressLocal` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        addressLocal?: boolean;
        /**
        *
        * Specifies whether to load on the `format` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        format?: Excel.CellPropertiesFormatLoadOptions;
        /**
        *
        * Specifies whether to load on the `hidden` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        hidden?: boolean;
        /**
        *
        * Specifies whether to load on the `hyperlink` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        hyperlink?: boolean;
        /**
        *
        * Specifies whether to load on the `style` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        style?: boolean;
    }
    /**
     *
     * Represents which row properties to load, when used as part of a "range.getRowProperties" method.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface RowPropertiesLoadOptions extends CellPropertiesLoadOptions {
        /**
        *
        * Specifies whether to load on the `rowHidden` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        rowHidden?: boolean;
        /**
        *
        * Specifies whether to load on the `rowIndex` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        rowIndex?: boolean;
        /**
        *
        * Specifies whether to load on the `format` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        format?: Excel.CellPropertiesFormatLoadOptions & {
            rowHeight?: boolean;
        };
    }
    /**
     *
     * Represents which column properties to load, when used as part of a "range.getColumnProperties" method.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface ColumnPropertiesLoadOptions extends CellPropertiesLoadOptions {
        /**
        *
        * Specifies whether to load on the `columnIndex` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        columnIndex?: boolean;
        /**
        *
        * Specifies whether to load on the `columnHidden` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        columnHidden?: boolean;
        /**
        *
        * Specifies whether to load on the `format` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        format?: Excel.CellPropertiesFormatLoadOptions & {
            columnWidth?: boolean;
        };
    }
    /**
     *
     * Represents which properties to load on the format object.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface CellPropertiesFormatLoadOptions {
        /**
        *
        * Specifies whether to load on the `autoIndent` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        autoIndent?: boolean;
        /**
        *
        * Specifies whether to load on the `borders` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        borders?: Excel.CellPropertiesBorderLoadOptions;
        /**
        *
        * Specifies whether to load on the `fill` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        fill?: Excel.CellPropertiesFillLoadOptions;
        /**
        *
        * Specifies whether to load on the `font` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        font?: Excel.CellPropertiesFontLoadOptions;
        /**
        *
        * Specifies whether to load on the `horizontalAlignment` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        horizontalAlignment?: boolean;
        /**
        *
        * Specifies whether to load on the `indentLevel` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        indentLevel?: boolean;
        /**
        *
        * Specifies whether to load on the `protection` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        protection?: boolean;
        /**
        *
        * Specifies whether to load on the `readingOrder` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        readingOrder?: boolean;
        /**
        *
        * Specifies whether to load on the `shrinkToFit` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        shrinkToFit?: boolean;
        /**
        *
        * Specifies whether to load on the `textOrientation` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        textOrientation?: boolean;
        /**
        *
        * Specifies whether to load on the `useStandardHeight` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        useStandardHeight?: boolean;
        /**
        *
        * Specifies whether to load on the `useStandardWidth` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        useStandardWidth?: boolean;
        /**
        *
        * Specifies whether to load on the `verticalAlignment` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        verticalAlignment?: boolean;
        /**
        *
        * Specifies whether to load on the `wrapText` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        wrapText?: boolean;
    }
    /**
     *
     * Represents the input parameter of setCellProperties.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface SettableCellProperties {
        /**
        *
        * Represents the `format` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        format?: Excel.CellPropertiesFormat;
        /**
        *
        * Represents the `hyperlink` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        hyperlink?: Excel.RangeHyperlink;
        /**
        *
        * Represents the `style` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        style?: string;
    }
    /**
     *
     * Represents the returned properties of getCellProperties.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface CellProperties extends SettableCellProperties {
        /**
        *
        * Represents the `address` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        address?: string;
        /**
        *
        * Represents the `addressLocal` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        addressLocal?: string;
        /**
        *
        * Represents the `hidden` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        hidden?: boolean;
    }
    /**
     *
     * Represents the input parameter of setRowProperties.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface SettableRowProperties extends SettableCellProperties {
        /**
        *
        * Represents the `rowHidden` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        rowHidden?: boolean;
        /**
        *
        * Represents the `format` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        format?: Excel.CellPropertiesFormat & {
            rowHeight?: number;
        };
    }
    /**
     *
     * Represents the returned properties of getRowProperties.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface RowProperties extends SettableRowProperties {
        /**
        *
        * Represents the `rowIndex` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        rowIndex?: number;
        /**
        *
        * Represents the `address` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        address?: string;
        /**
        *
        * Represents the `addressLocal` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        addressLocal?: string;
    }
    /**
     *
     * Represents the input parameter of setColumnProperties.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface SettableColumnProperties extends SettableCellProperties {
        /**
        *
        * Represents the `columnHidden` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        columnHidden?: boolean;
        /**
        *
        * Represents the `format` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        format?: Excel.CellPropertiesFormat & {
            columnWidth?: number;
        };
    }
    /**
     *
     * Represents the returned properties of getColumnProperties.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface ColumnProperties extends SettableColumnProperties {
        /**
        *
        * Represents the `columnIndex` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        columnIndex?: number;
        /**
        *
        * Represents the `address` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        address?: string;
        /**
        *
        * Represents the `addressLocal` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        addressLocal?: string;
    }
    /**
     *
     * Represents the returned format properties of getCellProperties or format input parameter of setCellProperties.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface CellPropertiesFormat {
        /**
            *
            * Represents the `autoIndent` property.
            *
            * [Api set: ExcelApi 1.9]
            */
        autoIndent?: boolean;
        /**
        *
        * Represents the `borders` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        borders?: Excel.CellBorderCollection;
        /**
        *
        * Represents the `fill` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        fill?: Excel.CellPropertiesFill;
        /**
        *
        * Represents the `font` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        font?: Excel.CellPropertiesFont;
        /**
        *
        * Represents the `horizontalAlignment` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        horizontalAlignment?: Excel.HorizontalAlignment | "General" | "Left" | "Center" | "Right" | "Fill" | "Justify" | "CenterAcrossSelection" | "Distributed";
        /**
        *
        * Represents the `indentLevel` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        indentLevel?: number;
        /**
        *
        * Represents the `protection` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        protection?: Excel.CellPropertiesProtection;
        /**
        *
        * Represents the `readingOrder` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        readingOrder?: Excel.ReadingOrder | "Context" | "LeftToRight" | "RightToLeft";
        /**
        *
        * Represents the `shrinkToFit` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        shrinkToFit?: boolean;
        /**
        *
        * Represents the `textOrientation` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        textOrientation?: number;
        /**
        *
        * Represents the `useStandardHeight` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        useStandardHeight?: boolean;
        /**
        *
        * Represents the `useStandardWidth` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        useStandardWidth?: boolean;
        /**
        *
        * Represents the `verticalAlignment` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        verticalAlignment?: Excel.VerticalAlignment | "Top" | "Center" | "Bottom" | "Justify" | "Distributed";
        /**
        *
        * Represents the `wrapText` property.
        *
        * [Api set: ExcelApi 1.9]
        */
        wrapText?: boolean;
    }
    /**
     *
     * Creates and opens a new workbook.  Optionally, the workbook can be pre-populated with a base64-encoded .xlsx file.
     *
     * [Api set: ExcelApi 1.8]
     *
     * @param base64File Optional. The base64 encoded .xlsx file. The default value is null.
     */
    function createWorkbook(base64?: string): Promise<void>;
    interface ThreeArrowsSet {
        [index: number]: Icon;
        redDownArrow: Icon;
        yellowSideArrow: Icon;
        greenUpArrow: Icon;
    }
    interface ThreeArrowsGraySet {
        [index: number]: Icon;
        grayDownArrow: Icon;
        graySideArrow: Icon;
        grayUpArrow: Icon;
    }
    interface ThreeFlagsSet {
        [index: number]: Icon;
        redFlag: Icon;
        yellowFlag: Icon;
        greenFlag: Icon;
    }
    interface ThreeTrafficLights1Set {
        [index: number]: Icon;
        redCircleWithBorder: Icon;
        yellowCircle: Icon;
        greenCircle: Icon;
    }
    interface ThreeTrafficLights2Set {
        [index: number]: Icon;
        redTrafficLight: Icon;
        yellowTrafficLight: Icon;
        greenTrafficLight: Icon;
    }
    interface ThreeSignsSet {
        [index: number]: Icon;
        redDiamond: Icon;
        yellowTriangle: Icon;
        greenCircle: Icon;
    }
    interface ThreeSymbolsSet {
        [index: number]: Icon;
        redCrossSymbol: Icon;
        yellowExclamationSymbol: Icon;
        greenCheckSymbol: Icon;
    }
    interface ThreeSymbols2Set {
        [index: number]: Icon;
        redCross: Icon;
        yellowExclamation: Icon;
        greenCheck: Icon;
    }
    interface FourArrowsSet {
        [index: number]: Icon;
        redDownArrow: Icon;
        yellowDownInclineArrow: Icon;
        yellowUpInclineArrow: Icon;
        greenUpArrow: Icon;
    }
    interface FourArrowsGraySet {
        [index: number]: Icon;
        grayDownArrow: Icon;
        grayDownInclineArrow: Icon;
        grayUpInclineArrow: Icon;
        grayUpArrow: Icon;
    }
    interface FourRedToBlackSet {
        [index: number]: Icon;
        blackCircle: Icon;
        grayCircle: Icon;
        pinkCircle: Icon;
        redCircle: Icon;
    }
    interface FourRatingSet {
        [index: number]: Icon;
        oneBar: Icon;
        twoBars: Icon;
        threeBars: Icon;
        fourBars: Icon;
    }
    interface FourTrafficLightsSet {
        [index: number]: Icon;
        blackCircleWithBorder: Icon;
        redCircleWithBorder: Icon;
        yellowCircle: Icon;
        greenCircle: Icon;
    }
    interface FiveArrowsSet {
        [index: number]: Icon;
        redDownArrow: Icon;
        yellowDownInclineArrow: Icon;
        yellowSideArrow: Icon;
        yellowUpInclineArrow: Icon;
        greenUpArrow: Icon;
    }
    interface FiveArrowsGraySet {
        [index: number]: Icon;
        grayDownArrow: Icon;
        grayDownInclineArrow: Icon;
        graySideArrow: Icon;
        grayUpInclineArrow: Icon;
        grayUpArrow: Icon;
    }
    interface FiveRatingSet {
        [index: number]: Icon;
        noBars: Icon;
        oneBar: Icon;
        twoBars: Icon;
        threeBars: Icon;
        fourBars: Icon;
    }
    interface FiveQuartersSet {
        [index: number]: Icon;
        whiteCircleAllWhiteQuarters: Icon;
        circleWithThreeWhiteQuarters: Icon;
        circleWithTwoWhiteQuarters: Icon;
        circleWithOneWhiteQuarter: Icon;
        blackCircle: Icon;
    }
    interface ThreeStarsSet {
        [index: number]: Icon;
        silverStar: Icon;
        halfGoldStar: Icon;
        goldStar: Icon;
    }
    interface ThreeTrianglesSet {
        [index: number]: Icon;
        redDownTriangle: Icon;
        yellowDash: Icon;
        greenUpTriangle: Icon;
    }
    interface FiveBoxesSet {
        [index: number]: Icon;
        noFilledBoxes: Icon;
        oneFilledBox: Icon;
        twoFilledBoxes: Icon;
        threeFilledBoxes: Icon;
        fourFilledBoxes: Icon;
    }
    interface IconCollections {
        threeArrows: ThreeArrowsSet;
        threeArrowsGray: ThreeArrowsGraySet;
        threeFlags: ThreeFlagsSet;
        threeTrafficLights1: ThreeTrafficLights1Set;
        threeTrafficLights2: ThreeTrafficLights2Set;
        threeSigns: ThreeSignsSet;
        threeSymbols: ThreeSymbolsSet;
        threeSymbols2: ThreeSymbols2Set;
        fourArrows: FourArrowsSet;
        fourArrowsGray: FourArrowsGraySet;
        fourRedToBlack: FourRedToBlackSet;
        fourRating: FourRatingSet;
        fourTrafficLights: FourTrafficLightsSet;
        fiveArrows: FiveArrowsSet;
        fiveArrowsGray: FiveArrowsGraySet;
        fiveRating: FiveRatingSet;
        fiveQuarters: FiveQuartersSet;
        threeStars: ThreeStarsSet;
        threeTriangles: ThreeTrianglesSet;
        fiveBoxes: FiveBoxesSet;
    }
    var icons: IconCollections;
    /**
     * Provides connection session for a remote workbook.
     */
    class Session {
        private static WorkbookSessionIdHeaderName;
        private static WorkbookSessionIdHeaderNameLower;
        constructor(workbookUrl?: string, requestHeaders?: {
            [name: string]: string;
        }, persisted?: boolean);
        /**
         * Close the session.
         */
        close(): Promise<void>;
    }
    /**
     * The RequestContext object facilitates requests to the Excel application. Since the Office add-in and the Excel application run in two different processes, the request context is required to get access to the Excel object model from the add-in.
     */
    class RequestContext extends OfficeCore.RequestContext {
        constructor(url?: string | Session);
        readonly workbook: Workbook;
        readonly application: Application;
        readonly runtime: Runtime;
    }
    interface RunOptions extends OfficeExtension.RunOptions<Session> {
        /**
         * Determines whether Excel will delay the batch request until the user exits cell edit mode.
         *
         * When false, if the user is in cell edit when the batch request is processed by the host, the batch will automatically fail.
         * When true, the batch request will be executed immediately if the user is not in cell edit mode, but if the user is in cell edit mode the batch request will be delayed until the user exits cell edit mode.
         */
        delayForCellEdit?: boolean;
    }
    /**
     * Executes a batch script that performs actions on the Excel object model, using a new RequestContext. When the promise is resolved, any tracked objects that were automatically allocated during execution will be released.
     * @param batch - A function that takes in a RequestContext and returns a promise (typically, just the result of "context.sync()"). The context parameter facilitates requests to the Excel application. Since the Office add-in and the Excel application run in two different processes, the RequestContext is required to get access to the Excel object model from the add-in.
     */
    function run<T>(batch: (context: Excel.RequestContext) => Promise<T>): Promise<T>;
    /**
     * Executes a batch script that performs actions on the Excel object model, using the RequestContext of a previously-created API object. When the promise is resolved, any tracked objects that were automatically allocated during execution will be released.
     * @param object - A previously-created API object. The batch will use the same RequestContext as the passed-in object, which means that any changes applied to the object will be picked up by "context.sync()".
     * @param batch - A function that takes in a RequestContext and returns a promise (typically, just the result of "context.sync()"). The context parameter facilitates requests to the Excel application. Since the Office add-in and the Excel application run in two different processes, the RequestContext is required to get access to the Excel object model from the add-in.
     */
    function run<T>(object: OfficeExtension.ClientObject, batch: (context: Excel.RequestContext) => Promise<T>): Promise<T>;
    /**
     * Executes a batch script that performs actions on the Excel object model, using the RequestContext of previously-created API objects.
     * @param objects - An array of previously-created API objects. The array will be validated to make sure that all of the objects share the same context. The batch will use this shared RequestContext, which means that any changes applied to these objects will be picked up by "context.sync()".
     * @param batch - A function that takes in a RequestContext and returns a promise (typically, just the result of "context.sync()"). The context parameter facilitates requests to the Excel application. Since the Office add-in and the Excel application run in two different processes, the RequestContext is required to get access to the Excel object model from the add-in.
     */
    function run<T>(objects: OfficeExtension.ClientObject[], batch: (context: Excel.RequestContext) => Promise<T>): Promise<T>;
    /**
    * Executes a batch script that performs actions on the Excel object model, using the RequestContext of a previously-created API object. When the promise is resolved, any tracked objects that were automatically allocated during execution will be released.
    * @param options - The additional options for this Excel.run which specify previous objects, whether to delay the request for cell edit, session info, etc.
    * @param batch - A function that takes in a RequestContext and returns a promise (typically, just the result of "context.sync()"). The context parameter facilitates requests to the Excel application. Since the Office add-in and the Excel application run in two different processes, the RequestContext is required to get access to the Excel object model from the add-in.
    */
    function run<T>(options: Excel.RunOptions, batch: (context: Excel.RequestContext) => Promise<T>): Promise<T>;
    /**
     * Executes a batch script that performs actions on the Excel object model, using the RequestContext of a previously-created object. When the promise is resolved, any tracked objects that were automatically allocated during execution will be released.
     *
     * @remarks
     *
     * In addition to this signature, the method also has the following signatures:
     *
     * `run<T>(object: OfficeExtension.ClientObject, batch: (context: Excel.RequestContext) => Promise<T>): Promise<T>;`
     *
     * `run<T>(objects: OfficeExtension.ClientObject[], batch: (context: Excel.RequestContext) => Promise<T>): Promise<T>;`
     *
     * `run<T>(options: Excel.RunOptions, batch: (context: Excel.RequestContext) => Promise<T>): Promise<T>;`
     *
     * `run<T>(batch: (context: Excel.RequestContext) => Promise<T>): Promise<T>;`
     *
     * @param context - A previously-created object. The batch will use the same RequestContext as the passed-in object, which means that any changes applied to the object will be picked up by "context.sync()".
     * @param batch - A function that takes in a RequestContext and returns a promise (typically, just the result of "context.sync()"). The context parameter facilitates requests to the Excel application. Since the Office add-in and the Excel application run in two different processes, the RequestContext is required to get access to the Excel object model from the add-in.
     */
    function run<T>(context: OfficeExtension.ClientRequestContext, batch: (context: Excel.RequestContext) => Promise<T>): Promise<T>;
    function postprocessBindingDescriptor(response: any): any;
    function getDataCommonPostprocess(response: any, callArgs: any): any;
    /**
     *
     * Provides information about the binding that raised the SelectionChanged event.
     *
     * [Api set: ExcelApi 1.2]
     */
    interface BindingSelectionChangedEventArgs {
        /**
         *
         * Gets the Binding object that represents the binding that raised the SelectionChanged event.
         *
         * [Api set: ExcelApi 1.2]
         */
        binding: Excel.Binding;
        /**
         *
         * Gets the number of columns selected.
         *
         * [Api set: ExcelApi 1.2]
         */
        columnCount: number;
        /**
         *
         * Gets the number of rows selected.
         *
         * [Api set: ExcelApi 1.2]
         */
        rowCount: number;
        /**
         *
         * Gets the index of the first column of the selection (zero-based).
         *
         * [Api set: ExcelApi 1.2]
         */
        startColumn: number;
        /**
         *
         * Gets the index of the first row of the selection (zero-based).
         *
         * [Api set: ExcelApi 1.2]
         */
        startRow: number;
    }
    /**
     *
     * Provides information about the binding that raised the DataChanged event.
     *
     * [Api set: ExcelApi 1.2]
     */
    interface BindingDataChangedEventArgs {
        /**
         *
         * Gets the Binding object that represents the binding that raised the DataChanged event.
         *
         * [Api set: ExcelApi 1.2]
         */
        binding: Excel.Binding;
    }
    /**
     *
     * Provides information about the document that raised the SelectionChanged event.
     *
     * [Api set: ExcelApi 1.2]
     */
    interface SelectionChangedEventArgs {
        /**
         *
         * Gets the workbook object that raised the SelectionChanged event.
         *
         * [Api set: ExcelApi 1.2]
         */
        workbook: Excel.Workbook;
    }
    /**
     *
     * Provides information about the setting that raised the SettingsChanged event
     *
     * [Api set: ExcelApi 1.4]
     */
    interface SettingsChangedEventArgs {
        /**
         *
         * Gets the Setting object that represents the binding that raised the SettingsChanged event
         *
         * [Api set: ExcelApi 1.4]
         */
        settings: Excel.SettingCollection;
    }
    /**
     *
     * Provides information about the workbook's onAutoSaveSettingChanged event.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface WorkbookAutoSaveSettingChangedEventArgs {
        /**
         *
         * Represents the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        type: "WorkbookAutoSaveSettingChanged";
    }
    /**
     *
     * Provide information about the detail of WorksheetChangedEvent/TableChangedEvent
     *
     * [Api set: ExcelApi 1.9]
     */
    interface ChangedEventDetail {
        /**
         *
         * Represents the value after changed. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
         *
         * [Api set: ExcelApi 1.9]
         */
        valueAfter: any;
        /**
         *
         * Represents the value before changed. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
         *
         * [Api set: ExcelApi 1.9]
         */
        valueBefore: any;
        /**
         *
         * Represents the type of value after changed
         *
         * [Api set: ExcelApi 1.9]
         */
        valueTypeAfter: Excel.RangeValueType | "Unknown" | "Empty" | "String" | "Integer" | "Double" | "Boolean" | "Error" | "RichValue";
        /**
         *
         * Represents the type of value before changed
         *
         * [Api set: ExcelApi 1.9]
         */
        valueTypeBefore: Excel.RangeValueType | "Unknown" | "Empty" | "String" | "Integer" | "Double" | "Boolean" | "Error" | "RichValue";
    }
    /**
     *
     * Provides information about the worksheet that raised the Changed event.
     *
     * [Api set: ExcelApi 1.7]
     */
    interface WorksheetChangedEventArgs {
        /**
         *
         * Gets the range address that represents the changed area of a specific worksheet.
         *
         * [Api set: ExcelApi 1.7]
         */
        address: string;
        /**
         *
         * Gets the change type that represents how the Changed event is triggered. See Excel.DataChangeType for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        changeType: Excel.DataChangeType | "Unknown" | "RangeEdited" | "RowInserted" | "RowDeleted" | "ColumnInserted" | "ColumnDeleted" | "CellInserted" | "CellDeleted";
        /**
         *
         * Represents the information about the change detail. This property can be retrieved when the Changed event is triggered on single cell. If the Changed event is triggered on multiple cells, this property can not be retrieved.
         *
         * [Api set: ExcelApi 1.9]
         */
        details: Excel.ChangedEventDetail;
        /**
         *
         * Gets the source of the event. See Excel.EventSource for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        source: Excel.EventSource | "Local" | "Remote";
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        type: "WorksheetChanged";
        /**
         *
         * Gets the id of the worksheet in which the data changed.
         *
         * [Api set: ExcelApi 1.7]
         */
        worksheetId: string;
        /**
         *
         * Gets the range that represents the changed area of a specific worksheet.
         *
         * [Api set: ExcelApi 1.8]
         */
        getRange(ctx: Excel.RequestContext): Excel.Range;
        /**
         *
         * Gets the range that represents the changed area of a specific worksheet. It might return null object.
         *
         * [Api set: ExcelApi 1.8]
         */
        getRangeOrNullObject(ctx: Excel.RequestContext): Excel.Range;
    }
    /**
     *
     * Provides information about the worksheet format change event.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface WorksheetFormatChangedEventArgs {
        /**
         *
         * Gets the range address that represents the changed area of a specific worksheet.
         *
         * [Api set: ExcelApi 1.9]
         */
        address: string;
        /**
         *
         * Gets the source of the event. See Excel.EventSource for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        source: Excel.EventSource | "Local" | "Remote";
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        type: "WorksheetFormatChanged";
        /**
         *
         * Gets the id of the worksheet in which the data changed.
         *
         * [Api set: ExcelApi 1.9]
         */
        worksheetId: string;
        /**
         *
         * Gets the range that represents the changed area of a specific worksheet.
         *
         * [Api set: ExcelApi 1.9]
         */
        getRange(ctx: Excel.RequestContext): Excel.Range;
        /**
         *
         * Gets the range that represents the changed area of a specific worksheet. It might return null object.
         *
         * [Api set: ExcelApi 1.9]
         */
        getRangeOrNullObject(ctx: Excel.RequestContext): Excel.Range;
    }
    /**
     *
     * Provides information about the table that raised the Changed event.
     *
     * [Api set: ExcelApi 1.7]
     */
    interface TableChangedEventArgs {
        /**
         *
         * Gets the address that represents the changed area of a table on a specific worksheet.
         *
         * [Api set: ExcelApi 1.7]
         */
        address: string;
        /**
         *
         * Gets the change type that represents how the Changed event is triggered. See Excel.DataChangeType for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        changeType: Excel.DataChangeType | "Unknown" | "RangeEdited" | "RowInserted" | "RowDeleted" | "ColumnInserted" | "ColumnDeleted" | "CellInserted" | "CellDeleted";
        /**
         *
         * Gets the source of the event. See Excel.EventSource for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        source: Excel.EventSource | "Local" | "Remote";
        /**
         *
         * Gets the id of the table in which the data changed.
         *
         * [Api set: ExcelApi 1.7]
         */
        tableId: string;
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        type: "TableChanged";
        /**
         *
         * Gets the id of the worksheet in which the data changed.
         *
         * [Api set: ExcelApi 1.7]
         */
        worksheetId: string;
        /**
         *
         * Represents the information about the change detail. This property can be retrieved when the Changed event is triggered on single cell. If the Changed event is triggered on multiple cells, this property can not be retrieved.
         *
         * [Api set: ExcelApi 1.9]
         */
        details: Excel.ChangedEventDetail;
        /**
         *
         * Gets the range that represents the changed area of a table on a specific worksheet.
         *
         * [Api set: ExcelApi 1.8]
         */
        getRange(ctx: Excel.RequestContext): Excel.Range;
        /**
         *
         * Gets the range that represents the changed area of a table on a specific worksheet. It might return null object.
         *
         * [Api set: ExcelApi 1.8]
         */
        getRangeOrNullObject(ctx: Excel.RequestContext): Excel.Range;
    }
    /**
     *
     * Provides information about the worksheet that raised the Activated event.
     *
     * [Api set: ExcelApi 1.7]
     */
    interface WorksheetActivatedEventArgs {
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        type: "WorksheetActivated";
        /**
         *
         * Gets the id of the worksheet that is activated.
         *
         * [Api set: ExcelApi 1.7]
         */
        worksheetId: string;
    }
    /**
     *
     * Provides information about the worksheet that raised the Deactivated event.
     *
     * [Api set: ExcelApi 1.7]
     */
    interface WorksheetDeactivatedEventArgs {
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        type: "WorksheetDeactivated";
        /**
         *
         * Gets the id of the worksheet that is deactivated.
         *
         * [Api set: ExcelApi 1.7]
         */
        worksheetId: string;
    }
    /**
     *
     * Provides information about the worksheet that raised the SelectionChanged event.
     *
     * [Api set: ExcelApi 1.7]
     */
    interface WorksheetSelectionChangedEventArgs {
        /**
         *
         * Gets the range address that represents the selected area of a specific worksheet.
         *
         * [Api set: ExcelApi 1.7]
         */
        address: string;
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        type: "WorksheetSelectionChanged";
        /**
         *
         * Gets the id of the worksheet in which the selection changed.
         *
         * [Api set: ExcelApi 1.7]
         */
        worksheetId: string;
    }
    /**
     *
     * Provides information about the table that raised the SelectionChanged event.
     *
     * [Api set: ExcelApi 1.7]
     */
    interface TableSelectionChangedEventArgs {
        /**
         *
         * Gets the range address that represents the selected area of the table on a specific worksheet.
         *
         * [Api set: ExcelApi 1.7]
         */
        address: string;
        /**
         *
         * Indicates if the selection is inside a table, address will be useless if IsInsideTable is false.
         *
         * [Api set: ExcelApi 1.7]
         */
        isInsideTable: boolean;
        /**
         *
         * Gets the id of the table in which the selection changed.
         *
         * [Api set: ExcelApi 1.7]
         */
        tableId: string;
        /**
         *
         * Gets the type of the event. See Excel.EventType for details. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        type: "TableSelectionChanged";
        /**
         *
         * Gets the id of the worksheet in which the selection changed.
         *
         * [Api set: ExcelApi 1.7]
         */
        worksheetId: string;
    }
    /**
     *
     * Provides information about the worksheet that raised the Added event.
     *
     * [Api set: ExcelApi 1.7]
     */
    interface WorksheetAddedEventArgs {
        /**
         *
         * Gets the source of the event. See Excel.EventSource for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        source: Excel.EventSource | "Local" | "Remote";
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        type: "WorksheetAdded";
        /**
         *
         * Gets the id of the worksheet that is added to the workbook.
         *
         * [Api set: ExcelApi 1.7]
         */
        worksheetId: string;
    }
    /**
     *
     * Provides information about the worksheet that raised the Deleted event.
     *
     * [Api set: ExcelApi 1.7]
     */
    interface WorksheetDeletedEventArgs {
        /**
         *
         * Gets the source of the event. See Excel.EventSource for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        source: Excel.EventSource | "Local" | "Remote";
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        type: "WorksheetDeleted";
        /**
         *
         * Gets the id of the worksheet that is deleted from the workbook.
         *
         * [Api set: ExcelApi 1.7]
         */
        worksheetId: string;
    }
    /**
     *
     * Provides information about the chart that raised the Added event.
     *
     * [Api set: ExcelApi 1.8]
     */
    interface ChartAddedEventArgs {
        /**
         *
         * Gets the id of the chart that is added to the worksheet.
         *
         * [Api set: ExcelApi 1.8]
         */
        chartId: string;
        /**
         *
         * Gets the source of the event. See Excel.EventSource for details.
         *
         * [Api set: ExcelApi 1.8]
         */
        source: Excel.EventSource | "Local" | "Remote";
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.8]
         */
        type: "ChartAdded";
        /**
         *
         * Gets the id of the worksheet in which the chart is added.
         *
         * [Api set: ExcelApi 1.8]
         */
        worksheetId: string;
    }
    /**
     *
     * Provides information about the chart that raised the Activated event.
     *
     * [Api set: ExcelApi 1.8]
     */
    interface ChartActivatedEventArgs {
        /**
         *
         * Gets the id of the chart that is activated.
         *
         * [Api set: ExcelApi 1.8]
         */
        chartId: string;
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.8]
         */
        type: "ChartActivated";
        /**
         *
         * Gets the id of the worksheet in which the chart is activated.
         *
         * [Api set: ExcelApi 1.8]
         */
        worksheetId: string;
    }
    /**
     *
     * Provides information about the chart that raised the Deactivated event.
     *
     * [Api set: ExcelApi 1.8]
     */
    interface ChartDeactivatedEventArgs {
        /**
         *
         * Gets the id of the chart that is deactivated.
         *
         * [Api set: ExcelApi 1.8]
         */
        chartId: string;
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.8]
         */
        type: "ChartDeactivated";
        /**
         *
         * Gets the id of the worksheet in which the chart is deactivated.
         *
         * [Api set: ExcelApi 1.8]
         */
        worksheetId: string;
    }
    /**
     *
     * Provides information about the chart that raised the Deleted event.
     *
     * [Api set: ExcelApi 1.8]
     */
    interface ChartDeletedEventArgs {
        /**
         *
         * Gets the id of the chart that is deleted from the worksheet.
         *
         * [Api set: ExcelApi 1.8]
         */
        chartId: string;
        /**
         *
         * Gets the source of the event. See Excel.EventSource for details.
         *
         * [Api set: ExcelApi 1.8]
         */
        source: Excel.EventSource | "Local" | "Remote";
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.8]
         */
        type: "ChartDeleted";
        /**
         *
         * Gets the id of the worksheet in which the chart is deleted.
         *
         * [Api set: ExcelApi 1.8]
         */
        worksheetId: string;
    }
    /**
     *
     * Provides information about the worksheet that raised the Calculated event.
     *
     * [Api set: ExcelApi 1.8]
     */
    interface WorksheetCalculatedEventArgs {
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.8]
         */
        type: "WorksheetCalculated";
        /**
         *
         * Gets the id of the worksheet that is calculated.
         *
         * [Api set: ExcelApi 1.8]
         */
        worksheetId: string;
    }
    /**
     *
     * Provides information about the table that raised the OnAdded event.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface TableAddedEventArgs {
        /**
         *
         * Gets the source of the event. See Excel.EventSource for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        source: Excel.EventSource | "Local" | "Remote";
        /**
         *
         * Gets the id of the table that is added.
         *
         * [Api set: ExcelApi 1.9]
         */
        tableId: string;
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        type: "TableAdded";
        /**
         *
         * Gets the id of the worksheet in which the table is added.
         *
         * [Api set: ExcelApi 1.9]
         */
        worksheetId: string;
    }
    /**
     *
     * Provides information about the table that raised the OnDeleted event.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface TableDeletedEventArgs {
        /**
         *
         * Specifies the source of the event. See Excel.EventSource for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        source: Excel.EventSource | "Local" | "Remote";
        /**
         *
         * Specifies the id of the table that is deleted.
         *
         * [Api set: ExcelApi 1.9]
         */
        tableId: string;
        /**
         *
         * Specifies the name of the table that is deleted.
         *
         * [Api set: ExcelApi 1.9]
         */
        tableName: string;
        /**
         *
         * Specifies the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        type: "TableDeleted";
        /**
         *
         * Specifies the id of the worksheet in which the table is deleted.
         *
         * [Api set: ExcelApi 1.9]
         */
        worksheetId: string;
    }
    /**
     *
     * Provides information about the shape that raised the Activated event.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface ShapeActivatedEventArgs {
        /**
         *
         * Gets the id of the activated shape.
         *
         * [Api set: ExcelApi 1.9]
         */
        shapeId: string;
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        type: "ShapeActivated";
        /**
         *
         * Gets the id of the worksheet in which the shape is activated.
         *
         * [Api set: ExcelApi 1.9]
         */
        worksheetId: string;
    }
    /**
     *
     * Provides information about the shape that raised the Deactivated event.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface ShapeDeactivatedEventArgs {
        /**
         *
         * Gets the id of the shape deactivated shape.
         *
         * [Api set: ExcelApi 1.9]
         */
        shapeId: string;
        /**
         *
         * Gets the type of the event. See Excel.EventType for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        type: "ShapeDeactivated";
        /**
         *
         * Gets the id of the worksheet in which the shape is deactivated.
         *
         * [Api set: ExcelApi 1.9]
         */
        worksheetId: string;
    }
    /**
     *
     * Represents the Excel Runtime class.
     *
     * [Api set: ExcelApi 1.5]
     */
    class Runtime extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Toggle JavaScript events in the current task pane or content add-in.
         *
         * [Api set: ExcelApi 1.8]
         */
        enableEvents: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.Runtime): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.RuntimeUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.Runtime): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.Runtime` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.Runtime` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.Runtime` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.RuntimeLoadOptions): Excel.Runtime;
        load(option?: string | string[]): Excel.Runtime;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.Runtime;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.Runtime object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.RuntimeData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.RuntimeData;
    }
    /**
     *
     * Represents the Excel application that manages the workbook.
     *
     * [Api set: ExcelApi 1.1]
     */
    class Application extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns the Iterative Calculation settings.
            On Excel for Windows and Excel for Mac, the settings will apply to the Excel Application.
            On Excel Online and Excel for other platforms, the settings will apply to the active workbook.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly iterativeCalculation: Excel.IterativeCalculation;
        /**
         *
         * Returns the Excel calculation engine version used for the last full recalculation. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly calculationEngineVersion: number;
        /**
         *
         * Returns the calculation mode used in the workbook, as defined by the constants in Excel.CalculationMode. Possible values are: `Automatic`, where Excel controls recalculation; `AutomaticExceptTables`, where Excel controls recalculation but ignores changes in tables; `Manual`, where calculation is done when the user requests it.
         *
         * [Api set: ExcelApi 1.1 for get, 1.8 for set]
         */
        calculationMode: Excel.CalculationMode | "Automatic" | "AutomaticExceptTables" | "Manual";
        /**
         *
         * Returns the calculation state of the application. See Excel.CalculationState for details. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly calculationState: Excel.CalculationState | "Done" | "Calculating" | "Pending";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.Application): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ApplicationUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.Application): void;
        /**
         *
         * Recalculate all currently opened workbooks in Excel.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param calculationType Specifies the calculation type to use. See Excel.CalculationType for details.
         */
        calculate(calculationType: Excel.CalculationType): void;
        /**
         *
         * Recalculate all currently opened workbooks in Excel.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param calculationType Specifies the calculation type to use. See Excel.CalculationType for details.
         */
        calculate(calculationType: "Recalculate" | "Full" | "FullRebuild"): void;
        /**
         *
         * Suspends calculation until the next "context.sync()" is called. Once set, it is the developer's responsibility to re-calc the workbook, to ensure that any dependencies are propagated.
         *
         * [Api set: ExcelApi 1.6]
         */
        suspendApiCalculationUntilNextSync(): void;
        /**
         *
         * Suspends sceen updating until the next "context.sync()" is called.
         *
         * [Api set: ExcelApi 1.9]
         */
        suspendScreenUpdatingUntilNextSync(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.Application` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.Application` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.Application` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ApplicationLoadOptions): Excel.Application;
        load(option?: string | string[]): Excel.Application;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.Application;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.Application object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ApplicationData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ApplicationData;
    }
    /**
     *
     * Represents the Iterative Calculation settings.
     *
     * [Api set: ExcelApi 1.9]
     */
    class IterativeCalculation extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * True if Excel will use iteration to resolve circular references.
         *
         * [Api set: ExcelApi 1.9]
         */
        enabled: boolean;
        /**
         *
         * Returns or sets the maximum amount of change between each iteration as Excel resolves circular references.
         *
         * [Api set: ExcelApi 1.9]
         */
        maxChange: number;
        /**
         *
         * Returns or sets the maximum number of iterations that Excel can use to resolve a circular reference.
         *
         * [Api set: ExcelApi 1.9]
         */
        maxIteration: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.IterativeCalculation): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.IterativeCalculationUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.IterativeCalculation): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.IterativeCalculation` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.IterativeCalculation` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.IterativeCalculation` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.IterativeCalculationLoadOptions): Excel.IterativeCalculation;
        load(option?: string | string[]): Excel.IterativeCalculation;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.IterativeCalculation;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.IterativeCalculation object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.IterativeCalculationData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.IterativeCalculationData;
    }
    /**
     *
     * Workbook is the top level object which contains related workbook objects such as worksheets, tables, ranges, etc.
     *
     * [Api set: ExcelApi 1.1]
     */
    class Workbook extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the Excel application instance that contains this workbook. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly application: Excel.Application;
        /**
         *
         * Represents a collection of bindings that are part of the workbook. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly bindings: Excel.BindingCollection;
        /**
         *
         * Represents the collection of custom XML parts contained by this workbook. Read-only.
         *
         * [Api set: ExcelApi 1.5]
         */
        readonly customXmlParts: Excel.CustomXmlPartCollection;
        /**
         *
         * Represents all data connections in the workbook. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly dataConnections: Excel.DataConnectionCollection;
        /**
         *
         * Represents a collection of worksheet functions that can be used for computation. Read-only.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly functions: Excel.Functions;
        /**
         *
         * Represents a collection of workbook scoped named items (named ranges and constants). Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly names: Excel.NamedItemCollection;
        /**
         *
         * Represents a collection of PivotTables associated with the workbook. Read-only.
         *
         * [Api set: ExcelApi 1.3]
         */
        readonly pivotTables: Excel.PivotTableCollection;
        /**
         *
         * Gets the workbook properties. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly properties: Excel.DocumentProperties;
        /**
         *
         * Returns workbook protection object for a workbook. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly protection: Excel.WorkbookProtection;
        /**
         *
         * Represents a collection of Settings associated with the workbook. Read-only.
         *
         * [Api set: ExcelApi 1.4]
         */
        readonly settings: Excel.SettingCollection;
        /**
         *
         * Represents a collection of styles associated with the workbook. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly styles: Excel.StyleCollection;
        /**
         *
         * Represents a collection of tables associated with the workbook. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly tables: Excel.TableCollection;
        /**
         *
         * Represents a collection of worksheets associated with the workbook. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly worksheets: Excel.WorksheetCollection;
        /**
         *
         * Specifies whether or not the workbook is in autosave mode. Read-Only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly autoSave: boolean;
        /**
         *
         * Returns a number about the version of Excel Calculation Engine. Read-Only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly calculationEngineVersion: number;
        /**
         *
         * True if all charts in the workbook are tracking the actual data points to which they are attached.
            False if the charts track the index of the data points.
         *
         * [Api set: ExcelApi 1.9]
         */
        chartDataPointTrack: boolean;
        /**
         *
         * Specifies whether or not changes have been made since the workbook was last saved.
            You can set this property to true if you want to close a modified workbook without either saving it or being prompted to save it.
         *
         * [Api set: ExcelApi 1.9]
         */
        isDirty: boolean;
        /**
         *
         * Gets the workbook name. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly name: string;
        /**
         *
         * Specifies whether or not the workbook has ever been saved locally or online. Read-Only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly previouslySaved: boolean;
        /**
         *
         * True if the workbook is open in Read-only mode. Read-only.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly readOnly: boolean;
        /**
         *
         * True if calculations in this workbook will be done using only the precision of the numbers as they're displayed.
            Data will permanently lose accuracy when switching this property from false to true.
         *
         * [Api set: ExcelApi 1.9]
         */
        usePrecisionAsDisplayed: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.Workbook): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.WorkbookUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.Workbook): void;
        /**
         *
         * Gets the currently active cell from the workbook.
         *
         * [Api set: ExcelApi 1.7]
         */
        getActiveCell(): Excel.Range;
        /**
         *
         * Gets the currently active chart in the workbook. If there is no active chart, an `ItemNotFound` exception is thrown.
         *
         * [Api set: ExcelApi 1.9]
         */
        getActiveChart(): Excel.Chart;
        /**
         *
         * Gets the currently active chart in the workbook. If there is no active chart, a null object is returned.
         *
         * [Api set: ExcelApi 1.9]
         */
        getActiveChartOrNullObject(): Excel.Chart;
        /**
         *
         * True if the workbook is being edited by multiple users (co-authoring).
            Please be aware there might be some delay between when the workbook status changes and when the changes are reflected on the result of the method.
         *
         * [Api set: ExcelApi 1.9]
         */
        getIsActiveCollabSession(): OfficeExtension.ClientResult<boolean>;
        /**
         *
         * Gets the currently selected single range from the workbook. If there are multiple ranges selected, this method will throw an error.
         *
         * [Api set: ExcelApi 1.1]
         */
        getSelectedRange(): Excel.Range;
        /**
         *
         * Gets the currently selected one or more ranges from the workbook. Unlike getSelectedRange(), this method returns a RangeAreas object that represents all the selected ranges.
         *
         * [Api set: ExcelApi 1.9]
         */
        getSelectedRanges(): Excel.RangeAreas;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.Workbook` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.Workbook` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.Workbook` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.WorkbookLoadOptions): Excel.Workbook;
        load(option?: string | string[]): Excel.Workbook;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.Workbook;
        /**
         *
         * Occurs when the autoSave setting is changed on the workbook.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @eventproperty
         */
        readonly onAutoSaveSettingChanged: OfficeExtension.EventHandlers<Excel.WorkbookAutoSaveSettingChangedEventArgs>;
        /**
         *
         * Occurs when the selection in the document is changed.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @eventproperty
         */
        readonly onSelectionChanged: OfficeExtension.EventHandlers<Excel.SelectionChangedEventArgs>;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.Workbook object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.WorkbookData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.WorkbookData;
    }
    /**
     *
     * Represents the protection of a workbook object.
     *
     * [Api set: ExcelApi 1.7]
     */
    class WorkbookProtection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Indicates if the workbook is protected. Read-Only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly protected: boolean;
        /**
         *
         * Protects a workbook. Fails if the workbook has been protected.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param password workbook protection password.
         */
        protect(password?: string): void;
        /**
         *
         * Unprotects a workbook.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param password workbook protection password.
         */
        unprotect(password?: string): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.WorkbookProtection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.WorkbookProtection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.WorkbookProtection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.WorkbookProtectionLoadOptions): Excel.WorkbookProtection;
        load(option?: string | string[]): Excel.WorkbookProtection;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.WorkbookProtection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.WorkbookProtection object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.WorkbookProtectionData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.WorkbookProtectionData;
    }
    /**
     *
     * The WorkbookCreated object is the top level object created by Application.CreateWorkbook. A WorkbookCreated object is a special Workbook object.
     *
     * [Api set: ExcelApi 1.8]
     */
    class WorkbookCreated extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.WorkbookCreated` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.WorkbookCreated` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.WorkbookCreated` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: string | string[]): Excel.WorkbookCreated;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.WorkbookCreated;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.WorkbookCreated object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.WorkbookCreatedData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.WorkbookCreatedData;
    }
    /**
     *
     * An Excel worksheet is a grid of cells. It can contain data, tables, charts, etc.
     *
     * [Api set: ExcelApi 1.1]
     */
    class Worksheet extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the AutoFilter object of the worksheet. Read-Only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly autoFilter: Excel.AutoFilter;
        /**
         *
         * Returns collection of charts that are part of the worksheet. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly charts: Excel.ChartCollection;
        /**
         *
         * Gets an object that can be used to manipulate frozen panes on the worksheet. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly freezePanes: Excel.WorksheetFreezePanes;
        /**
         *
         * Gets the horizontal page break collection for the worksheet. This collection only contains manual page breaks.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly horizontalPageBreaks: Excel.PageBreakCollection;
        /**
         *
         * Collection of names scoped to the current worksheet. Read-only.
         *
         * [Api set: ExcelApi 1.4]
         */
        readonly names: Excel.NamedItemCollection;
        /**
         *
         * Gets the PageLayout object of the worksheet.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly pageLayout: Excel.PageLayout;
        /**
         *
         * Collection of PivotTables that are part of the worksheet. Read-only.
         *
         * [Api set: ExcelApi 1.3]
         */
        readonly pivotTables: Excel.PivotTableCollection;
        /**
         *
         * Returns sheet protection object for a worksheet. Read-only.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly protection: Excel.WorksheetProtection;
        /**
         *
         * Returns the collection of all the Shape objects on the worksheet. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly shapes: Excel.ShapeCollection;
        /**
         *
         * Collection of tables that are part of the worksheet. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly tables: Excel.TableCollection;
        /**
         *
         * Gets the vertical page break collection for the worksheet. This collection only contains manual page breaks.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly verticalPageBreaks: Excel.PageBreakCollection;
        /**
         *
         * Gets or sets the enableCalculation property of the worksheet.
            True if Excel recalculates the worksheet when necessary. False if Excel doesn't recalculate the sheet.
         *
         * [Api set: ExcelApi 1.9]
         */
        enableCalculation: boolean;
        /**
         *
         * Returns a value that uniquely identifies the worksheet in a given workbook. The value of the identifier remains the same even when the worksheet is renamed or moved. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly id: string;
        /**
         *
         * The display name of the worksheet.
         *
         * [Api set: ExcelApi 1.1]
         */
        name: string;
        /**
         *
         * The zero-based position of the worksheet within the workbook.
         *
         * [Api set: ExcelApi 1.1]
         */
        position: number;
        /**
         *
         * Gets or sets the worksheet's gridlines flag.
            This flag determines whether gridlines are visible to the user.
         *
         * [Api set: ExcelApi 1.8]
         */
        showGridlines: boolean;
        /**
         *
         * Gets or sets the worksheet's headings flag.
            This flag determines whether headings are visible to the user.
         *
         * [Api set: ExcelApi 1.8]
         */
        showHeadings: boolean;
        /**
         *
         * Returns the standard (default) height of all the rows in the worksheet, in points. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly standardHeight: number;
        /**
         *
         * Returns or sets the standard (default) width of all the columns in the worksheet.
            One unit of column width is equal to the width of one character in the Normal style. For proportional fonts, the width of the character 0 (zero) is used.
         *
         * [Api set: ExcelApi 1.7]
         */
        standardWidth: number;
        /**
         *
         * Gets or sets the worksheet tab color.
            When retrieving the tab color, if the worksheet is invisible, the value will be null. If the worksheet is visible but the tab color is set to auto, an empty string will be returned. Otherwise, the property will be set to a color, in the form "#123456"
            When setting the color, use an empty-string to set an "auto" color, or a real color otherwise.
         *
         * [Api set: ExcelApi 1.7]
         */
        tabColor: string;
        /**
         *
         * The Visibility of the worksheet.
         *
         * [Api set: ExcelApi 1.1 for reading visibility; 1.2 for setting it.]
         */
        visibility: Excel.SheetVisibility | "Visible" | "Hidden" | "VeryHidden";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.Worksheet): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.WorksheetUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.Worksheet): void;
        /**
         *
         * Activate the worksheet in the Excel UI.
         *
         * [Api set: ExcelApi 1.1]
         */
        activate(): void;
        /**
         *
         * Calculates all cells on a worksheet.
         *
         * [Api set: ExcelApi 1.6]
         *
         * @param markAllDirty True, to mark all as dirty.
         */
        calculate(markAllDirty: boolean): void;
        /**
         *
         * Copy a worksheet and place it at the specified position. Return the copied worksheet.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param positionType Optional.
         * @param relativeTo Optional.
         */
        copy(positionType?: Excel.WorksheetPositionType, relativeTo?: Excel.Worksheet): Excel.Worksheet;
        /**
         *
         * Copy a worksheet and place it at the specified position. Return the copied worksheet.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param positionType Optional.
         * @param relativeTo Optional.
         */
        copy(positionType?: "None" | "Before" | "After" | "Beginning" | "End", relativeTo?: Excel.Worksheet): Excel.Worksheet;
        /**
         *
         * Deletes the worksheet from the workbook. Note that if the worksheet's visibility is set to "VeryHidden", the delete operation will fail with an `InvalidOperation` exception. You should first change its visibility to hidden or visible before deleting it.
         *
         * [Api set: ExcelApi 1.1]
         */
        delete(): void;
        /**
         *
         * Finds all occurrences of the given string based on the criteria specified and returns them as a RangeAreas object, comprising one or more rectangular ranges.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param text The string to find.
         * @param criteria Additional search criteria, including whether the search needs to match the entire cell or be case sensitive.
         * @returns A RangeArea object, comprising one or more rectangular ranges, that matches the search criteria. If no cells meet this criteria, an ItemNotFound error will be thrown.
         */
        findAll(text: string, criteria: Excel.WorksheetSearchCriteria): Excel.RangeAreas;
        /**
         *
         * Finds all occurrences of the given string based on the criteria specified and returns them as a RangeAreas object, comprising one or more rectangular ranges.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param text The string to find.
         * @param criteria Additional search criteria, including whether the search needs to match the entire cell or be case sensitive.
         * @returns A RangeArea object, comprising one or more rectangular ranges, that matches the search criteria. If there are no matches, this function will return a null object.
         */
        findAllOrNullObject(text: string, criteria: Excel.WorksheetSearchCriteria): Excel.RangeAreas;
        /**
         *
         * Gets the range object containing the single cell based on row and column numbers. The cell can be outside the bounds of its parent range, so long as it stays within the worksheet grid.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param row The row number of the cell to be retrieved. Zero-indexed.
         * @param column the column number of the cell to be retrieved. Zero-indexed.
         */
        getCell(row: number, column: number): Excel.Range;
        /**
         *
         * Gets the worksheet that follows this one. If there are no worksheets following this one, this method will throw an error.
         *
         * [Api set: ExcelApi 1.5]
         *
         * @param visibleOnly Optional. If true, considers only visible worksheets, skipping over any hidden ones.
         */
        getNext(visibleOnly?: boolean): Excel.Worksheet;
        /**
         *
         * Gets the worksheet that follows this one. If there are no worksheets following this one, this method will return a null object.
         *
         * [Api set: ExcelApi 1.5]
         *
         * @param visibleOnly Optional. If true, considers only visible worksheets, skipping over any hidden ones.
         */
        getNextOrNullObject(visibleOnly?: boolean): Excel.Worksheet;
        /**
         *
         * Gets the worksheet that precedes this one. If there are no previous worksheets, this method will throw an error.
         *
         * [Api set: ExcelApi 1.5]
         *
         * @param visibleOnly Optional. If true, considers only visible worksheets, skipping over any hidden ones.
         */
        getPrevious(visibleOnly?: boolean): Excel.Worksheet;
        /**
         *
         * Gets the worksheet that precedes this one. If there are no previous worksheets, this method will return a null objet.
         *
         * [Api set: ExcelApi 1.5]
         *
         * @param visibleOnly Optional. If true, considers only visible worksheets, skipping over any hidden ones.
         */
        getPreviousOrNullObject(visibleOnly?: boolean): Excel.Worksheet;
        /**
         *
         * Gets the range object, representing a single rectangular block of cells, specified by the address or name.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param address Optional. The string representing the address or name of the range. For example, "A1:B2". If not specified, the entire worksheet range is returned.
         */
        getRange(address?: string): Excel.Range;
        /**
         *
         * Gets the range object beginning at a particular row index and column index, and spanning a certain number of rows and columns.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param startRow Start row (zero-indexed).
         * @param startColumn Start column (zero-indexed).
         * @param rowCount Number of rows to include in the range.
         * @param columnCount Number of columns to include in the range.
         */
        getRangeByIndexes(startRow: number, startColumn: number, rowCount: number, columnCount: number): Excel.Range;
        /**
         *
         * Gets the RangeAreas object, representing one or more blocks of rectangular ranges, specified by the address or name.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param address Optional. A string containing the comma-separated addresses or names of the individual ranges. For example, "A1:B2, A5:B5". If not specified, an RangeArea object for the entire worksheet is returned.
         */
        getRanges(address?: string): Excel.RangeAreas;
        /**
         *
         * The used range is the smallest range that encompasses any cells that have a value or formatting assigned to them. If the entire worksheet is blank, this function will return the top left cell (i.e. it will *not* throw an error).
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param valuesOnly Optional. If true, considers only cells with values as used cells (ignoring formatting). [Api set: ExcelApi 1.2]
         */
        getUsedRange(valuesOnly?: boolean): Excel.Range;
        /**
         *
         * The used range is the smallest range that encompasses any cells that have a value or formatting assigned to them. If the entire worksheet is blank, this function will return a null object.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param valuesOnly Optional. Considers only cells with values as used cells.
         */
        getUsedRangeOrNullObject(valuesOnly?: boolean): Excel.Range;
        /**
         *
         * Finds and replaces the given string based on the criteria specified within the current worksheet.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param text String to find.
         * @param replacement String to replace the original with.
         * @param criteria Additional Replace Criteria.
         * @returns The number of replacements performed.
         */
        replaceAll(text: string, replacement: string, criteria: Excel.ReplaceCriteria): OfficeExtension.ClientResult<number>;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.Worksheet` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.Worksheet` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.Worksheet` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.WorksheetLoadOptions): Excel.Worksheet;
        load(option?: string | string[]): Excel.Worksheet;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.Worksheet;
        /**
         *
         * Occurs when the worksheet is activated.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @eventproperty
         */
        readonly onActivated: OfficeExtension.EventHandlers<Excel.WorksheetActivatedEventArgs>;
        /**
         *
         * Occurs when the worksheet is calculated.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @eventproperty
         */
        readonly onCalculated: OfficeExtension.EventHandlers<Excel.WorksheetCalculatedEventArgs>;
        /**
         *
         * Occurs when data changed on a specific worksheet.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @eventproperty
         */
        readonly onChanged: OfficeExtension.EventHandlers<Excel.WorksheetChangedEventArgs>;
        /**
         *
         * Occurs when the worksheet is deactivated.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @eventproperty
         */
        readonly onDeactivated: OfficeExtension.EventHandlers<Excel.WorksheetDeactivatedEventArgs>;
        /**
         *
         * Occurs when format changed on a specific worksheet.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @eventproperty
         */
        readonly onFormatChanged: OfficeExtension.EventHandlers<Excel.WorksheetFormatChangedEventArgs>;
        /**
         *
         * Occurs when the selection changes on a specific worksheet.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @eventproperty
         */
        readonly onSelectionChanged: OfficeExtension.EventHandlers<Excel.WorksheetSelectionChangedEventArgs>;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.Worksheet object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.WorksheetData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.WorksheetData;
    }
    /**
     *
     * Represents a collection of worksheet objects that are part of the workbook.
     *
     * [Api set: ExcelApi 1.1]
     */
    class WorksheetCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.Worksheet[];
        /**
         *
         * Adds a new worksheet to the workbook. The worksheet will be added at the end of existing worksheets. If you wish to activate the newly added worksheet, call ".activate() on it.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param name Optional. The name of the worksheet to be added. If specified, name should be unqiue. If not specified, Excel determines the name of the new worksheet.
         */
        add(name?: string): Excel.Worksheet;
        /**
         *
         * Gets the currently active worksheet in the workbook.
         *
         * [Api set: ExcelApi 1.1]
         */
        getActiveWorksheet(): Excel.Worksheet;
        /**
         *
         * Gets the number of worksheets in the collection.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param visibleOnly Optional. If true, considers only visible worksheets, skipping over any hidden ones.
         */
        getCount(visibleOnly?: boolean): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets the first worksheet in the collection.
         *
         * [Api set: ExcelApi 1.5]
         *
         * @param visibleOnly Optional. If true, considers only visible worksheets, skipping over any hidden ones.
         */
        getFirst(visibleOnly?: boolean): Excel.Worksheet;
        /**
         *
         * Gets a worksheet object using its Name or ID.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param key The Name or ID of the worksheet.
         */
        getItem(key: string): Excel.Worksheet;
        /**
         *
         * Gets a worksheet object using its Name or ID. If the worksheet does not exist, will return a null object.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param key The Name or ID of the worksheet.
         */
        getItemOrNullObject(key: string): Excel.Worksheet;
        /**
         *
         * Gets the last worksheet in the collection.
         *
         * [Api set: ExcelApi 1.5]
         *
         * @param visibleOnly Optional. If true, considers only visible worksheets, skipping over any hidden ones.
         */
        getLast(visibleOnly?: boolean): Excel.Worksheet;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.WorksheetCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.WorksheetCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.WorksheetCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.WorksheetCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.WorksheetCollection;
        load(option?: string | string[]): Excel.WorksheetCollection;
        load(option?: OfficeExtension.LoadOption): Excel.WorksheetCollection;
        /**
         *
         * Occurs when any worksheet in the workbook is activated.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @eventproperty
         */
        readonly onActivated: OfficeExtension.EventHandlers<Excel.WorksheetActivatedEventArgs>;
        /**
         *
         * Occurs when a new worksheet is added to the workbook.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @eventproperty
         */
        readonly onAdded: OfficeExtension.EventHandlers<Excel.WorksheetAddedEventArgs>;
        /**
         *
         * Occurs when any worksheet in the workbook is calculated.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @eventproperty
         */
        readonly onCalculated: OfficeExtension.EventHandlers<Excel.WorksheetCalculatedEventArgs>;
        /**
         *
         * Occurs when any worksheet in the workbook is changed.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @eventproperty
         */
        readonly onChanged: OfficeExtension.EventHandlers<Excel.WorksheetChangedEventArgs>;
        /**
         *
         * Occurs when any worksheet in the workbook is deactivated.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @eventproperty
         */
        readonly onDeactivated: OfficeExtension.EventHandlers<Excel.WorksheetDeactivatedEventArgs>;
        /**
         *
         * Occurs when a worksheet is deleted from the workbook.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @eventproperty
         */
        readonly onDeleted: OfficeExtension.EventHandlers<Excel.WorksheetDeletedEventArgs>;
        /**
         *
         * Occurs when any worksheet in the workbook has format changed.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @eventproperty
         */
        readonly onFormatChanged: OfficeExtension.EventHandlers<Excel.WorksheetFormatChangedEventArgs>;
        /**
         *
         * Occurs when the selection changes on any worksheet.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @eventproperty
         */
        readonly onSelectionChanged: OfficeExtension.EventHandlers<Excel.WorksheetSelectionChangedEventArgs>;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.WorksheetCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.WorksheetCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.WorksheetCollectionData;
    }
    /**
     *
     * Represents the protection of a sheet object.
     *
     * [Api set: ExcelApi 1.2]
     */
    class WorksheetProtection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Sheet protection options. Read-only.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly options: Excel.WorksheetProtectionOptions;
        /**
         *
         * Indicates if the worksheet is protected. Read-only.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly protected: boolean;
        /**
         *
         * Protects a worksheet. Fails if the worksheet has already been protected.
         *
         * [Api set: ExcelApi 1.2 for options; 1.7 for password]
         *
         * @param options Optional. Sheet protection options.
         * @param password Optional. Sheet protection password.
         */
        protect(options?: Excel.WorksheetProtectionOptions, password?: string): void;
        /**
         *
         * Unprotects a worksheet.
         *
         * [Api set: ExcelApi 1.7 for password]
         *
         * @param password sheet protection password.
         */
        unprotect(password?: string): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.WorksheetProtection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.WorksheetProtection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.WorksheetProtection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.WorksheetProtectionLoadOptions): Excel.WorksheetProtection;
        load(option?: string | string[]): Excel.WorksheetProtection;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.WorksheetProtection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.WorksheetProtection object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.WorksheetProtectionData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.WorksheetProtectionData;
    }
    /**
     *
     * Represents the options in sheet protection.
     *
     * [Api set: ExcelApi 1.2]
     */
    interface WorksheetProtectionOptions {
        /**
         *
         * Represents the worksheet protection option of allowing using auto filter feature.
         *
         * [Api set: ExcelApi 1.2]
         */
        allowAutoFilter?: boolean;
        /**
         *
         * Represents the worksheet protection option of allowing deleting columns.
         *
         * [Api set: ExcelApi 1.2]
         */
        allowDeleteColumns?: boolean;
        /**
         *
         * Represents the worksheet protection option of allowing deleting rows.
         *
         * [Api set: ExcelApi 1.2]
         */
        allowDeleteRows?: boolean;
        /**
         *
         * Represents the worksheet protection option of allowing editing objects.
         *
         * [Api set: ExcelApi 1.7]
         */
        allowEditObjects?: boolean;
        /**
         *
         * Represents the worksheet protection option of allowing editing scenarios.
         *
         * [Api set: ExcelApi 1.7]
         */
        allowEditScenarios?: boolean;
        /**
         *
         * Represents the worksheet protection option of allowing formatting cells.
         *
         * [Api set: ExcelApi 1.2]
         */
        allowFormatCells?: boolean;
        /**
         *
         * Represents the worksheet protection option of allowing formatting columns.
         *
         * [Api set: ExcelApi 1.2]
         */
        allowFormatColumns?: boolean;
        /**
         *
         * Represents the worksheet protection option of allowing formatting rows.
         *
         * [Api set: ExcelApi 1.2]
         */
        allowFormatRows?: boolean;
        /**
         *
         * Represents the worksheet protection option of allowing inserting columns.
         *
         * [Api set: ExcelApi 1.2]
         */
        allowInsertColumns?: boolean;
        /**
         *
         * Represents the worksheet protection option of allowing inserting hyperlinks.
         *
         * [Api set: ExcelApi 1.2]
         */
        allowInsertHyperlinks?: boolean;
        /**
         *
         * Represents the worksheet protection option of allowing inserting rows.
         *
         * [Api set: ExcelApi 1.2]
         */
        allowInsertRows?: boolean;
        /**
         *
         * Represents the worksheet protection option of allowing using PivotTable feature.
         *
         * [Api set: ExcelApi 1.2]
         */
        allowPivotTables?: boolean;
        /**
         *
         * Represents the worksheet protection option of allowing using sort feature.
         *
         * [Api set: ExcelApi 1.2]
         */
        allowSort?: boolean;
        /**
         *
         * Represents the worksheet protection option of selection mode.
         *
         * [Api set: ExcelApi 1.7]
         */
        selectionMode?: Excel.ProtectionSelectionMode | "Normal" | "Unlocked" | "None";
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    class WorksheetFreezePanes extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Sets the frozen cells in the active worksheet view.
            The range provided corresponds to cells that will be frozen in the top- and left-most pane.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param frozenRange A range that represents the cells to be frozen, or null to remove all frozen panes.
         */
        freezeAt(frozenRange: Range | string): void;
        /**
         *
         * Freeze the first column(s) of the worksheet in place.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param count Optional number of columns to freeze, or zero to unfreeze all columns
         */
        freezeColumns(count?: number): void;
        /**
         *
         * Freeze the top row(s) of the worksheet in place.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param count Optional number of rows to freeze, or zero to unfreeze all rows
         */
        freezeRows(count?: number): void;
        /**
         *
         * Gets a range that describes the frozen cells in the active worksheet view.
            The frozen range is corresponds to cells that are frozen in the top- and left-most pane.
         *
         * [Api set: ExcelApi 1.7]
         */
        getLocation(): Excel.Range;
        /**
         *
         * Gets a range that describes the frozen cells in the active worksheet view.
            The frozen range is corresponds to cells that are frozen in the top- and left-most pane.
            If there is no frozen pane, returns a null object.
         *
         * [Api set: ExcelApi 1.7]
         */
        getLocationOrNullObject(): Excel.Range;
        /**
         *
         * Removes all frozen panes in the worksheet.
         *
         * [Api set: ExcelApi 1.7]
         */
        unfreeze(): void;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.WorksheetFreezePanes object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.WorksheetFreezePanesData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): {
            [key: string]: string;
        };
    }
    /**
     *
     * Range represents a set of one or more contiguous cells such as a cell, a row, a column, block of cells, etc.
     *
     * [Api set: ExcelApi 1.1]
     */
    class Range extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Collection of ConditionalFormats that intersect the range. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly conditionalFormats: Excel.ConditionalFormatCollection;
        /**
         *
         * Returns a data validation object.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly dataValidation: Excel.DataValidation;
        /**
         *
         * Returns a format object, encapsulating the range's font, fill, borders, alignment, and other properties. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly format: Excel.RangeFormat;
        /**
         *
         * Represents the range sort of the current range. Read-only.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly sort: Excel.RangeSort;
        /**
         *
         * The worksheet containing the current range. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly worksheet: Excel.Worksheet;
        /**
         *
         * Represents the range reference in A1-style. Address value will contain the Sheet reference (e.g. "Sheet1!A1:B4"). Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly address: string;
        /**
         *
         * Represents range reference for the specified range in the language of the user. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly addressLocal: string;
        /**
         *
         * Number of cells in the range. This API will return -1 if the cell count exceeds 2^31-1 (2,147,483,647). Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly cellCount: number;
        /**
         *
         * Represents the total number of columns in the range. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly columnCount: number;
        /**
         *
         * Represents if all columns of the current range are hidden.
         *
         * [Api set: ExcelApi 1.2]
         */
        columnHidden: boolean;
        /**
         *
         * Represents the column number of the first cell in the range. Zero-indexed. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly columnIndex: number;
        /**
         *
         * Represents the formula in A1-style notation.
            When setting formulas to a range, the value argument can be either a single value (a string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
         *
         * [Api set: ExcelApi 1.1]
         */
        formulas: any[][];
        /**
         *
         * Represents the formula in A1-style notation, in the user's language and number-formatting locale.  For example, the English "=SUM(A1, 1.5)" formula would become "=SUMME(A1; 1,5)" in German.
            When setting formulas to a range, the value argument can be either a single value (a string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
         *
         * [Api set: ExcelApi 1.1]
         */
        formulasLocal: any[][];
        /**
         *
         * Represents the formula in R1C1-style notation.
            When setting formulas to a range, the value argument can be either a single value (a string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
         *
         * [Api set: ExcelApi 1.2]
         */
        formulasR1C1: any[][];
        /**
         *
         * Represents if all cells of the current range are hidden. Read-only.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly hidden: boolean;
        /**
         *
         * Represents the hyperlink for the current range.
         *
         * [Api set: ExcelApi 1.7]
         */
        hyperlink: Excel.RangeHyperlink;
        /**
         *
         * Represents if the current range is an entire column. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly isEntireColumn: boolean;
        /**
         *
         * Represents if the current range is an entire row. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly isEntireRow: boolean;
        /**
         *
         * Represents the data type state of each cell. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly linkedDataTypeState: Excel.LinkedDataTypeState[][];
        /**
         *
         * Represents Excel's number format code for the given range.
            When setting number format to a range, the value argument can be either a single value (string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
         *
         * [Api set: ExcelApi 1.1]
         */
        numberFormat: any[][];
        /**
         *
         * Represents Excel's number format code for the given range as a string in the language of the user.
            When setting number format local to a range, the value argument can be either a single value (string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
         *
         * [Api set: ExcelApi 1.7]
         */
        numberFormatLocal: any[][];
        /**
         *
         * Returns the total number of rows in the range. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly rowCount: number;
        /**
         *
         * Represents if all rows of the current range are hidden.
         *
         * [Api set: ExcelApi 1.2]
         */
        rowHidden: boolean;
        /**
         *
         * Returns the row number of the first cell in the range. Zero-indexed. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly rowIndex: number;
        /**
         *
         * Represents the style of the current range.
            If the styles of the cells are inconsistent, null will be returned.
            For custom styles, the style name will be returned. For built-in styles, a string representing a value in the BuiltInStyle enum will be returned.
         *
         * [Api set: ExcelApi 1.7]
         */
        style: string;
        /**
         *
         * Text values of the specified range. The Text value will not depend on the cell width. The # sign substitution that happens in Excel UI will not affect the text value returned by the API. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly text: string[][];
        /**
         *
         * Represents the type of data of each cell. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly valueTypes: Excel.RangeValueType[][];
        /**
         *
         * Represents the raw values of the specified range. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
            When setting values to a range, the value argument can be either a single value (string, number or boolean) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
         *
         * [Api set: ExcelApi 1.1]
         */
        values: any[][];
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.Range): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.RangeUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.Range): void;
        /**
         *
         * Fills range from the current range to the destination range.
            The destination range must extend the source either horizontally or vertically. Discontiguous ranges are not supported.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param destinationRange The destination range to autofill.
         * @param autoFillType The type of autofill. Specifies how the destination range is to be filled, based on the contents of the current range. Default is "FillDefault".
         */
        autoFill(destinationRange: Range | string, autoFillType?: Excel.AutoFillType): void;
        /**
         *
         * Fills range from the current range to the destination range.
            The destination range must extend the source either horizontally or vertically. Discontiguous ranges are not supported.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param destinationRange The destination range to autofill.
         * @param autoFillType The type of autofill. Specifies how the destination range is to be filled, based on the contents of the current range. Default is "FillDefault".
         */
        autoFill(destinationRange: Range | string, autoFillType?: "FillDefault" | "FillCopy" | "FillSeries" | "FillFormats" | "FillValues" | "FillDays" | "FillWeekdays" | "FillMonths" | "FillYears" | "LinearTrend" | "GrowthTrend" | "FlashFill"): void;
        /**
         *
         * Calculates a range of cells on a worksheet.
         *
         * [Api set: ExcelApi 1.6]
         */
        calculate(): void;
        /**
         *
         * Clear range values, format, fill, border, etc.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param applyTo Optional. Determines the type of clear action. See Excel.ClearApplyTo for details.
         */
        clear(applyTo?: Excel.ClearApplyTo): void;
        /**
         *
         * Clear range values, format, fill, border, etc.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param applyTo Optional. Determines the type of clear action. See Excel.ClearApplyTo for details.
         */
        clear(applyTo?: "All" | "Formats" | "Contents" | "Hyperlinks" | "RemoveHyperlinks"): void;
        /**
         *
         * Converts the range cells with datatypes into text.
         *
         * [Api set: ExcelApi 1.9]
         */
        convertDataTypeToText(): void;
        /**
         *
         * Converts the range cells into linked datatype in the worksheet.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param serviceID The Service ID which will be used to query the data.
         * @param languageCulture Language Culture to query the service for.
         */
        convertToLinkedDataType(serviceID: number, languageCulture: string): void;
        /**
         *
         * Copies cell data or formatting from the source range or RangeAreas to the current range.
            The destination range can be of different size than the source range or RangeAreas. The destination will be expanded automatically if it is smaller than the source.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param sourceRange The source range or RangeAreas to copy from. When the source RangeAreas has multiple ranges, it must in the outline form which can be created by removing full rows or columns from a rectangular range.
         * @param copyType The type of cell data or formatting to copy over. Default is "All".
         * @param skipBlanks True if to skip blank cells in the source range. Default is false.
         * @param transpose True if to transpose the cells in the destination range. Default is false.
         */
        copyFrom(sourceRange: Range | RangeAreas | string, copyType?: Excel.RangeCopyType, skipBlanks?: boolean, transpose?: boolean): void;
        /**
         *
         * Copies cell data or formatting from the source range or RangeAreas to the current range.
            The destination range can be of different size than the source range or RangeAreas. The destination will be expanded automatically if it is smaller than the source.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param sourceRange The source range or RangeAreas to copy from. When the source RangeAreas has multiple ranges, it must in the outline form which can be created by removing full rows or columns from a rectangular range.
         * @param copyType The type of cell data or formatting to copy over. Default is "All".
         * @param skipBlanks True if to skip blank cells in the source range. Default is false.
         * @param transpose True if to transpose the cells in the destination range. Default is false.
         */
        copyFrom(sourceRange: Range | RangeAreas | string, copyType?: "All" | "Formulas" | "Values" | "Formats", skipBlanks?: boolean, transpose?: boolean): void;
        /**
         *
         * Deletes the cells associated with the range.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param shift Specifies which way to shift the cells. See Excel.DeleteShiftDirection for details.
         */
        delete(shift: Excel.DeleteShiftDirection): void;
        /**
         *
         * Deletes the cells associated with the range.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param shift Specifies which way to shift the cells. See Excel.DeleteShiftDirection for details.
         */
        delete(shift: "Up" | "Left"): void;
        /**
         *
         * Finds the given string based on the criteria specified.
            If the current range is larger than a single cell, then the search will be limited to that range, else the search will cover the entire sheet starting after that cell.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param text The string to find.
         * @param criteria Additional search criteria, including the search direction and whether the search needs to match the entire cell or be case sensitive.
         * @returns The Range which matched the search criteria.
         */
        find(text: string, criteria: Excel.SearchCriteria): Excel.Range;
        /**
         *
         * Finds the given string based on the criteria specified.
            If the current range is larger than a single cell, then the search will be limited to that range, else the search will cover the entire sheet starting after that cell.
            If there are no matches, this function will return a null object.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param text The string to find.
         * @param criteria Additional search criteria, including the search direction and whether the search needs to match the entire cell or be case sensitive.
         * @returns The Range which matched the search criteria.
         */
        findOrNullObject(text: string, criteria: Excel.SearchCriteria): Excel.Range;
        /**
         *
         * Does FlashFill to current range.Flash Fill will automatically fills data when it senses a pattern, so the range must be single column range and have data around in order to find pattern.
         *
         * [Api set: ExcelApi 1.9]
         */
        flashFill(): void;
        /**
         *
         * Gets a Range object with the same top-left cell as the current Range object, but with the specified numbers of rows and columns.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param numRows The number of rows of the new range size.
         * @param numColumns The number of columns of the new range size.
         */
        getAbsoluteResizedRange(numRows: number, numColumns: number): Excel.Range;
        /**
         *
         * Gets the smallest range object that encompasses the given ranges. For example, the GetBoundingRect of "B2:C5" and "D10:E15" is "B2:E15".
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param anotherRange The range object or address or range name.
         */
        getBoundingRect(anotherRange: Range | string): Excel.Range;
        /**
         *
         * Gets the range object containing the single cell based on row and column numbers. The cell can be outside the bounds of its parent range, so long as it stays within the worksheet grid. The returned cell is located relative to the top left cell of the range.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param row Row number of the cell to be retrieved. Zero-indexed.
         * @param column Column number of the cell to be retrieved. Zero-indexed.
         */
        getCell(row: number, column: number): Excel.Range;
        /**
         *
         * Returns a 2D array, encapsulating the data for each cell's font, fill, borders, alignment, and other properties.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param cellPropertiesLoadOptions An object that represents which cell properties to load.
         * @returns A 2D array where each item represents the requested properties of the corresponding cell.
         */
        getCellProperties(cellPropertiesLoadOptions: CellPropertiesLoadOptions): OfficeExtension.ClientResult<CellProperties[][]>;
        /**
         *
         * Gets a column contained in the range.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param column Column number of the range to be retrieved. Zero-indexed.
         */
        getColumn(column: number): Excel.Range;
        /**
         *
         * Returns a single-dimensional array, encapsulating the data for each column's font, fill, borders, alignment, and other properties.  For properties that are not consistent across each cell within a given column, null will be returned.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param columnPropertiesLoadOptions An object that represents which column properties to load.
         * @returns An array where each item represents the requested properties of the corresponding column.
         */
        getColumnProperties(columnPropertiesLoadOptions: ColumnPropertiesLoadOptions): OfficeExtension.ClientResult<ColumnProperties[]>;
        /**
         *
         * Gets a certain number of columns to the right of the current Range object.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param count Optional. The number of columns to include in the resulting range. In general, use a positive number to create a range outside the current range. You can also use a negative number to create a range within the current range. The default value is 1.
         */
        getColumnsAfter(count?: number): Excel.Range;
        /**
         *
         * Gets a certain number of columns to the left of the current Range object.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param count Optional. The number of columns to include in the resulting range. In general, use a positive number to create a range outside the current range. You can also use a negative number to create a range within the current range. The default value is 1.
         */
        getColumnsBefore(count?: number): Excel.Range;
        /**
         *
         * Gets an object that represents the entire column of the range (for example, if the current range represents cells "B4:E11", its `getEntireColumn` is a range that represents columns "B:E").
         *
         * [Api set: ExcelApi 1.1]
         */
        getEntireColumn(): Excel.Range;
        /**
         *
         * Gets an object that represents the entire row of the range (for example, if the current range represents cells "B4:E11", its `GetEntireRow` is a range that represents rows "4:11").
         *
         * [Api set: ExcelApi 1.1]
         */
        getEntireRow(): Excel.Range;
        /**
         *
         * Renders the range as a base64-encoded png image.
         *
         * [Api set: ExcelApi 1.7]
         */
        getImage(): OfficeExtension.ClientResult<string>;
        /**
         *
         * Gets the range object that represents the rectangular intersection of the given ranges.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param anotherRange The range object or range address that will be used to determine the intersection of ranges.
         */
        getIntersection(anotherRange: Range | string): Excel.Range;
        /**
         *
         * Gets the range object that represents the rectangular intersection of the given ranges. If no intersection is found, will return a null object.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param anotherRange The range object or range address that will be used to determine the intersection of ranges.
         */
        getIntersectionOrNullObject(anotherRange: Range | string): Excel.Range;
        /**
         *
         * Gets the last cell within the range. For example, the last cell of "B2:D5" is "D5".
         *
         * [Api set: ExcelApi 1.1]
         */
        getLastCell(): Excel.Range;
        /**
         *
         * Gets the last column within the range. For example, the last column of "B2:D5" is "D2:D5".
         *
         * [Api set: ExcelApi 1.1]
         */
        getLastColumn(): Excel.Range;
        /**
         *
         * Gets the last row within the range. For example, the last row of "B2:D5" is "B5:D5".
         *
         * [Api set: ExcelApi 1.1]
         */
        getLastRow(): Excel.Range;
        /**
         *
         * Gets an object which represents a range that's offset from the specified range. The dimension of the returned range will match this range. If the resulting range is forced outside the bounds of the worksheet grid, an error will be thrown.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param rowOffset The number of rows (positive, negative, or 0) by which the range is to be offset. Positive values are offset downward, and negative values are offset upward.
         * @param columnOffset The number of columns (positive, negative, or 0) by which the range is to be offset. Positive values are offset to the right, and negative values are offset to the left.
         */
        getOffsetRange(rowOffset: number, columnOffset: number): Excel.Range;
        /**
         *
         * Gets a Range object similar to the current Range object, but with its bottom-right corner expanded (or contracted) by some number of rows and columns.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param deltaRows The number of rows by which to expand the bottom-right corner, relative to the current range. Use a positive number to expand the range, or a negative number to decrease it.
         * @param deltaColumns The number of columns by which to expand the bottom-right corner, relative to the current range. Use a positive number to expand the range, or a negative number to decrease it.
         */
        getResizedRange(deltaRows: number, deltaColumns: number): Excel.Range;
        /**
         *
         * Gets a row contained in the range.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param row Row number of the range to be retrieved. Zero-indexed.
         */
        getRow(row: number): Excel.Range;
        /**
         *
         * Returns a single-dimensional array, encapsulating the data for each row's font, fill, borders, alignment, and other properties.  For properties that are not consistent across each cell within a given row, null will be returned.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param rowPropertiesLoadOptions An object that represents which row properties to load.
         * @returns An array where each item represents the requested properties of the corresponding row.
         */
        getRowProperties(rowPropertiesLoadOptions: RowPropertiesLoadOptions): OfficeExtension.ClientResult<RowProperties[]>;
        /**
         *
         * Gets a certain number of rows above the current Range object.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param count Optional. The number of rows to include in the resulting range. In general, use a positive number to create a range outside the current range. You can also use a negative number to create a range within the current range. The default value is 1.
         */
        getRowsAbove(count?: number): Excel.Range;
        /**
         *
         * Gets a certain number of rows below the current Range object.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param count Optional. The number of rows to include in the resulting range. In general, use a positive number to create a range outside the current range. You can also use a negative number to create a range within the current range. The default value is 1.
         */
        getRowsBelow(count?: number): Excel.Range;
        /**
         *
         * Gets the RangeAreas object, comprising one or more rectangular ranges, that represents all the cells that match the specified type and value.
            If no special cells are found, an ItemNotFound error will be thrown.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param cellType The type of cells to include.
         * @param cellValueType If cellType is either Constants or Formulas, this argument is used to determine which types of cells to include in the result. These values can be combined together to return more than one type. The default is to select all constants or formulas, no matter what the type.
         */
        getSpecialCells(cellType: Excel.SpecialCellType, cellValueType?: Excel.SpecialCellValueType): Excel.RangeAreas;
        /**
         *
         * Gets the RangeAreas object, comprising one or more rectangular ranges, that represents all the cells that match the specified type and value.
            If no special cells are found, an ItemNotFound error will be thrown.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param cellType The type of cells to include.
         * @param cellValueType If cellType is either Constants or Formulas, this argument is used to determine which types of cells to include in the result. These values can be combined together to return more than one type. The default is to select all constants or formulas, no matter what the type.
         */
        getSpecialCells(cellType: "ConditionalFormats" | "DataValidations" | "Blanks" | "Constants" | "Formulas" | "SameConditionalFormat" | "SameDataValidation" | "Visible", cellValueType?: "All" | "Errors" | "ErrorsLogical" | "ErrorsNumbers" | "ErrorsText" | "ErrorsLogicalNumber" | "ErrorsLogicalText" | "ErrorsNumberText" | "Logical" | "LogicalNumbers" | "LogicalText" | "LogicalNumbersText" | "Numbers" | "NumbersText" | "Text"): Excel.RangeAreas;
        /**
         *
         * Gets the RangeAreas object, comprising one or more ranges, that represents all the cells that match the specified type and value.
            If no special cells are found, a null object will be returned.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param cellType The type of cells to include.
         * @param cellValueType If cellType is either Constants or Formulas, this argument is used to determine which types of cells to include in the result. These values can be combined together to return more than one type. The default is to select all constants or formulas, no matter what the type.
         */
        getSpecialCellsOrNullObject(cellType: Excel.SpecialCellType, cellValueType?: Excel.SpecialCellValueType): Excel.RangeAreas;
        /**
         *
         * Gets the RangeAreas object, comprising one or more ranges, that represents all the cells that match the specified type and value.
            If no special cells are found, a null object will be returned.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param cellType The type of cells to include.
         * @param cellValueType If cellType is either Constants or Formulas, this argument is used to determine which types of cells to include in the result. These values can be combined together to return more than one type. The default is to select all constants or formulas, no matter what the type.
         */
        getSpecialCellsOrNullObject(cellType: "ConditionalFormats" | "DataValidations" | "Blanks" | "Constants" | "Formulas" | "SameConditionalFormat" | "SameDataValidation" | "Visible", cellValueType?: "All" | "Errors" | "ErrorsLogical" | "ErrorsNumbers" | "ErrorsText" | "ErrorsLogicalNumber" | "ErrorsLogicalText" | "ErrorsNumberText" | "Logical" | "LogicalNumbers" | "LogicalText" | "LogicalNumbersText" | "Numbers" | "NumbersText" | "Text"): Excel.RangeAreas;
        /**
         *
         * Returns a Range object that represents the surrounding region for the top-left cell in this range. A surrounding region is a range bounded by any combination of blank rows and blank columns relative to this range.
         *
         * [Api set: ExcelApi 1.7]
         */
        getSurroundingRegion(): Excel.Range;
        /**
         *
         * Gets a scoped collection of tables that overlap with the range.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param fullyContained If true, returns only tables that are fully contained within the range bounds. The default value is false.
         */
        getTables(fullyContained?: boolean): Excel.TableScopedCollection;
        /**
         *
         * Returns the used range of the given range object. If there are no used cells within the range, this function will throw an ItemNotFound error.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param valuesOnly Considers only cells with values as used cells. [Api set: ExcelApi 1.2]
         */
        getUsedRange(valuesOnly?: boolean): Excel.Range;
        /**
         *
         * Returns the used range of the given range object. If there are no used cells within the range, this function will return a null object.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param valuesOnly Considers only cells with values as used cells.
         */
        getUsedRangeOrNullObject(valuesOnly?: boolean): Excel.Range;
        /**
         *
         * Represents the visible rows of the current range.
         *
         * [Api set: ExcelApi 1.3]
         */
        getVisibleView(): Excel.RangeView;
        /**
         *
         * Inserts a cell or a range of cells into the worksheet in place of this range, and shifts the other cells to make space. Returns a new Range object at the now blank space.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param shift Specifies which way to shift the cells. See Excel.InsertShiftDirection for details.
         */
        insert(shift: Excel.InsertShiftDirection): Excel.Range;
        /**
         *
         * Inserts a cell or a range of cells into the worksheet in place of this range, and shifts the other cells to make space. Returns a new Range object at the now blank space.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param shift Specifies which way to shift the cells. See Excel.InsertShiftDirection for details.
         */
        insert(shift: "Down" | "Right"): Excel.Range;
        /**
         *
         * Merge the range cells into one region in the worksheet.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param across Optional. Set true to merge cells in each row of the specified range as separate merged cells. The default value is false.
         */
        merge(across?: boolean): void;
        /**
         *
         * Removes duplicate values from the range specified by the columns.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param columns The columns inside the range that may contain duplicates. At least one column needs to be specified. Zero-indexed.
         * @param includesHeader True if the input data contains header. Default is false.
         * @returns The resulting object that contains the number of rows removed and the number of remaining unique rows.
         */
        removeDuplicates(columns: number[], includesHeader: boolean): Excel.RemoveDuplicatesResult;
        /**
         *
         * Finds and replaces the given string based on the criteria specified within the current range.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param text String to find.
         * @param replacement String to replace the original with.
         * @param criteria Additional Replace Criteria.
         * @returns The number of replacements performed.
         */
        replaceAll(text: string, replacement: string, criteria: Excel.ReplaceCriteria): OfficeExtension.ClientResult<number>;
        /**
         *
         * Selects the specified range in the Excel UI.
         *
         * [Api set: ExcelApi 1.1]
         */
        select(): void;
        /**
         *
         * Updates the range based on a 2D array of cell properties , encapsulating things like font, fill, borders, alignment, and so forth.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param cellPropertiesData A 2D array that represents which properties to set in each cell.
         */
        setCellProperties(cellPropertiesData: SettableCellProperties[][]): void;
        /**
         *
         * Updates the range based on a single-dimensional array of column properties, encapsulating things like font, fill, borders, alignment, and so forth.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param columnPropertiesData An array that represents which properties to set in each column.
         */
        setColumnProperties(columnPropertiesData: SettableColumnProperties[]): void;
        /**
         *
         * Set a range to be recalculated when the next recalculation occurs.
         *
         * [Api set: ExcelApi 1.9]
         */
        setDirty(): void;
        /**
         *
         * Updates the range based on a single-dimensional array of row properties, encapsulating things like font, fill, borders, alignment, and so forth.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param rowPropertiesData An array that represents which properties to set in each row.
         */
        setRowProperties(rowPropertiesData: SettableRowProperties[]): void;
        /**
         *
         * Displays the card for an active cell if it has rich value content.
         *
         * [Api set: ExcelApi 1.7]
         */
        showCard(): void;
        /**
         *
         * Unmerge the range cells into separate cells.
         *
         * [Api set: ExcelApi 1.2]
         */
        unmerge(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.Range` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.Range` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.Range` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.RangeLoadOptions): Excel.Range;
        load(option?: string | string[]): Excel.Range;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.Range;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Excel.Range;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Excel.Range;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.Range object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.RangeData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.RangeData;
    }
    /**
     *
     * Represents a string reference of the form SheetName!A1:B5, or a global or local named range.
     *
     * [Api set: ExcelApi 1.2]
     */
    interface RangeReference {
        /**
         *
         * Gets or sets the address of the range; for example 'SheetName!A1:B5'.
         *
         * [Api set: ExcelApi 1.2]
         */
        address: string;
    }
    /**
     *
     * Represents the necessary strings to get/set a hyperlink (XHL) object.
     *
     * [Api set: ExcelApi 1.7]
     */
    interface RangeHyperlink {
        /**
         *
         * Represents the url target for the hyperlink.
         *
         * [Api set: ExcelApi 1.7]
         */
        address?: string;
        /**
         *
         * Represents the document reference target for the hyperlink.
         *
         * [Api set: ExcelApi 1.7]
         */
        documentReference?: string;
        /**
         *
         * Represents the string displayed when hovering over the hyperlink.
         *
         * [Api set: ExcelApi 1.7]
         */
        screenTip?: string;
        /**
         *
         * Represents the string that is displayed in the top left most cell in the range.
         *
         * [Api set: ExcelApi 1.7]
         */
        textToDisplay?: string;
    }
    /**
     *
     * RangeAreas represents a collection of one or more rectangular ranges in the same worksheet.
     *
     * [Api set: ExcelApi 1.9]
     */
    class RangeAreas extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns a collection of rectangular ranges that comprise this RangeAreas object.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly areas: Excel.RangeCollection;
        /**
         *
         * Returns a collection of ConditionalFormats that intersect with any cells in this RangeAreas object. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly conditionalFormats: Excel.ConditionalFormatCollection;
        /**
         *
         * Returns a dataValidation object for all ranges in the RangeAreas.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly dataValidation: Excel.DataValidation;
        /**
         *
         * Returns a rangeFormat object, encapsulating the the font, fill, borders, alignment, and other properties for all ranges in the RangeAreas object. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly format: Excel.RangeFormat;
        /**
         *
         * Returns the worksheet for the current RangeAreas. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly worksheet: Excel.Worksheet;
        /**
         *
         * Returns the RageAreas reference in A1-style. Address value will contain the worksheet name for each rectangular block of cells (e.g. "Sheet1!A1:B4, Sheet1!D1:D4"). Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly address: string;
        /**
         *
         * Returns the RageAreas reference in the user locale. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly addressLocal: string;
        /**
         *
         * Returns the number of rectangular ranges that comprise this RangeAreas object.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly areaCount: number;
        /**
         *
         * Returns the number of cells in the RangeAreas object, summing up the cell counts of all of the individual rectangular ranges. Returns -1 if the cell count exceeds 2^31-1 (2,147,483,647). Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly cellCount: number;
        /**
         *
         * Indicates whether all the ranges on this RangeAreas object represent entire columns (e.g., "A:C, Q:Z"). Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly isEntireColumn: boolean;
        /**
         *
         * Indicates whether all the ranges on this RangeAreas object represent entire rows (e.g., "1:3, 5:7"). Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly isEntireRow: boolean;
        /**
         *
         * Represents the style for all ranges in this RangeAreas object.
            If the styles of the cells are inconsistent, null will be returned.
            For custom styles, the style name will be returned. For built-in styles, a string representing a value in the BuiltInStyle enum will be returned.
         *
         * [Api set: ExcelApi 1.9]
         */
        style: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.RangeAreas): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.RangeAreasUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.RangeAreas): void;
        /**
         *
         * Calculates all cells in the RangeAreas.
         *
         * [Api set: ExcelApi 1.9]
         */
        calculate(): void;
        /**
         *
         * Clears values, format, fill, border, etc on each of the areas that comprise this RangeAreas object.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param applyTo Optional. Determines the type of clear action. See Excel.ClearApplyTo for details. Default is "All".
         */
        clear(applyTo?: Excel.ClearApplyTo): void;
        /**
         *
         * Clears values, format, fill, border, etc on each of the areas that comprise this RangeAreas object.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param applyTo Optional. Determines the type of clear action. See Excel.ClearApplyTo for details. Default is "All".
         */
        clear(applyTo?: "All" | "Formats" | "Contents" | "Hyperlinks" | "RemoveHyperlinks"): void;
        /**
         *
         * Converts all cells in the RangeAreas with datatypes into text.
         *
         * [Api set: ExcelApi 1.9]
         */
        convertDataTypeToText(): void;
        /**
         *
         * Converts all cells in the RangeAreas into linked datatype.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param serviceID The Service ID which will be used to query the data.
         * @param languageCulture Language Culture to query the service for.
         */
        convertToLinkedDataType(serviceID: number, languageCulture: string): void;
        /**
         *
         * Copies cell data or formatting from the source range or RangeAreas to the current RangeAreas.
            The destination rangeAreas can be of different size than the source range or RangeAreas. The destination will be expanded automatically if it is smaller than the source.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param sourceRange The source range or RangeAreas to copy from. When the source RangeAreas has multiple ranges, it must be in the outline form which can be created by removing full rows or columns from a rectangular range.
         * @param copyType The type of cell data or formatting to copy over. Default is "All".
         * @param skipBlanks True if to skip blank cells in the source range or RangeAreas. Default is false.
         * @param transpose True if to transpose the cells in the destination RangeAreas. Default is false.
         */
        copyFrom(sourceRange: Range | RangeAreas | string, copyType?: Excel.RangeCopyType, skipBlanks?: boolean, transpose?: boolean): void;
        /**
         *
         * Copies cell data or formatting from the source range or RangeAreas to the current RangeAreas.
            The destination rangeAreas can be of different size than the source range or RangeAreas. The destination will be expanded automatically if it is smaller than the source.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param sourceRange The source range or RangeAreas to copy from. When the source RangeAreas has multiple ranges, it must be in the outline form which can be created by removing full rows or columns from a rectangular range.
         * @param copyType The type of cell data or formatting to copy over. Default is "All".
         * @param skipBlanks True if to skip blank cells in the source range or RangeAreas. Default is false.
         * @param transpose True if to transpose the cells in the destination RangeAreas. Default is false.
         */
        copyFrom(sourceRange: Range | RangeAreas | string, copyType?: "All" | "Formulas" | "Values" | "Formats", skipBlanks?: boolean, transpose?: boolean): void;
        /**
         *
         * Returns a RangeAreas object that represents the entire columns of the RangeAreas (for example, if the current RangeAreas represents cells "B4:E11, H2", it returns a RangeAreas that represents columns "B:E, H:H").
         *
         * [Api set: ExcelApi 1.9]
         */
        getEntireColumn(): Excel.RangeAreas;
        /**
         *
         * Returns a RangeAreas object that represents the entire rows of the RangeAreas (for example, if the current RangeAreas represents cells "B4:E11", it returns a RangeAreas that represents rows "4:11").
         *
         * [Api set: ExcelApi 1.9]
         */
        getEntireRow(): Excel.RangeAreas;
        /**
         *
         * Returns the RangeAreas object that represents the intersection of the given ranges or RangeAreas. If no intersection is found, an ItemNotFound error will be thrown.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param anotherRange The range, RangeAreas object or range address that will be used to determine the intersection.
         */
        getIntersection(anotherRange: Range | RangeAreas | string): Excel.RangeAreas;
        /**
         *
         * Returns the RangeAreas object that represents the intersection of the given ranges or RangeAreas. If no intersection is found, a null object is returned.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param anotherRange The range, RangeAreas, or address that will be used to determine the intersection.
         */
        getIntersectionOrNullObject(anotherRange: Range | RangeAreas | string): Excel.RangeAreas;
        /**
         *
         * Returns an RangeAreas object that is shifted by the specific row and column offset. The dimension of the returned RangeAreas will match the original object. If the resulting RangeAreas is forced outside the bounds of the worksheet grid, an error will be thrown.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param rowOffset The number of rows (positive, negative, or 0) by which the RangeAreas is to be offset. Positive values are offset downward, and negative values are offset upward.
         * @param columnOffset The number of columns (positive, negative, or 0) by which the RangeAreas is to be offset. Positive values are offset to the right, and negative values are offset to the left.
         */
        getOffsetRangeAreas(rowOffset: number, columnOffset: number): Excel.RangeAreas;
        /**
         *
         * Returns a RangeAreas object that represents all the cells that match the specified type and value. Throws an error if no special cells are found that match the criteria.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param cellType The type of cells to include.
         * @param cellValueType If cellType is either Constants or Formulas, this argument is used to determine which types of cells to include in the result. These values can be combined together to return more than one type. The default is to select all constants or formulas, no matter what the type.
         */
        getSpecialCells(cellType: Excel.SpecialCellType, cellValueType?: Excel.SpecialCellValueType): Excel.RangeAreas;
        /**
         *
         * Returns a RangeAreas object that represents all the cells that match the specified type and value. Throws an error if no special cells are found that match the criteria.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param cellType The type of cells to include.
         * @param cellValueType If cellType is either Constants or Formulas, this argument is used to determine which types of cells to include in the result. These values can be combined together to return more than one type. The default is to select all constants or formulas, no matter what the type.
         */
        getSpecialCells(cellType: "ConditionalFormats" | "DataValidations" | "Blanks" | "Constants" | "Formulas" | "SameConditionalFormat" | "SameDataValidation" | "Visible", cellValueType?: "All" | "Errors" | "ErrorsLogical" | "ErrorsNumbers" | "ErrorsText" | "ErrorsLogicalNumber" | "ErrorsLogicalText" | "ErrorsNumberText" | "Logical" | "LogicalNumbers" | "LogicalText" | "LogicalNumbersText" | "Numbers" | "NumbersText" | "Text"): Excel.RangeAreas;
        /**
         *
         * Returns a RangeAreas object that represents all the cells that match the specified type and value. Returns a null object if no special cells are found that match the criteria.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param cellType The type of cells to include.
         * @param cellValueType If cellType is either Constants or Formulas, this argument is used to determine which types of cells to include in the result. These values can be combined together to return more than one type. The default is to select all constants or formulas, no matter what the type.
         */
        getSpecialCellsOrNullObject(cellType: Excel.SpecialCellType, cellValueType?: Excel.SpecialCellValueType): Excel.RangeAreas;
        /**
         *
         * Returns a RangeAreas object that represents all the cells that match the specified type and value. Returns a null object if no special cells are found that match the criteria.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param cellType The type of cells to include.
         * @param cellValueType If cellType is either Constants or Formulas, this argument is used to determine which types of cells to include in the result. These values can be combined together to return more than one type. The default is to select all constants or formulas, no matter what the type.
         */
        getSpecialCellsOrNullObject(cellType: "ConditionalFormats" | "DataValidations" | "Blanks" | "Constants" | "Formulas" | "SameConditionalFormat" | "SameDataValidation" | "Visible", cellValueType?: "All" | "Errors" | "ErrorsLogical" | "ErrorsNumbers" | "ErrorsText" | "ErrorsLogicalNumber" | "ErrorsLogicalText" | "ErrorsNumberText" | "Logical" | "LogicalNumbers" | "LogicalText" | "LogicalNumbersText" | "Numbers" | "NumbersText" | "Text"): Excel.RangeAreas;
        /**
         *
         * Returns a scoped collection of tables that overlap with any range in this RangeAreas object.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param fullyContained If true, returns only tables that are fully contained within the range bounds. Default is false.
         */
        getTables(fullyContained?: boolean): Excel.TableScopedCollection;
        /**
         *
         * Returns the used RangeAreas that comprises all the used areas of individual rectangular ranges in the RangeAreas object.
            If there are no used cells within the RangeAreas, the ItemNotFound error will be thrown.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param valuesOnly Whether to only consider cells with values as used cells. Default is false.
         */
        getUsedRangeAreas(valuesOnly?: boolean): Excel.RangeAreas;
        /**
         *
         * Returns the used RangeAreas that comprises all the used areas of individual rectangular ranges in the RangeAreas object.
            If there are no used cells within the RangeAreas, a null object will be returned.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param valuesOnly Whether to only consider cells with values as used cells.
         */
        getUsedRangeAreasOrNullObject(valuesOnly?: boolean): Excel.RangeAreas;
        /**
         *
         * Sets the RangeAreas to be recalculated when the next recalculation occurs.
         *
         * [Api set: ExcelApi 1.9]
         */
        setDirty(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.RangeAreas` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.RangeAreas` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.RangeAreas` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.RangeAreasLoadOptions): Excel.RangeAreas;
        load(option?: string | string[]): Excel.RangeAreas;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.RangeAreas;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Excel.RangeAreas;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Excel.RangeAreas;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.RangeAreas object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.RangeAreasData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.RangeAreasData;
    }
    /**
     *
     * Represents the search criteria to be used.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface SearchCriteria {
        /**
         *
         * Specifies whether the match needs to be complete or partial. A complete match matches the entire contents of the cell. Default is false (partial).
         *
         * [Api set: ExcelApi 1.9]
         */
        completeMatch?: boolean;
        /**
         *
         * Specifies whether the match is case sensitive. Default is false (insensitive).
         *
         * [Api set: ExcelApi 1.9]
         */
        matchCase?: boolean;
        /**
         *
         * Specifies the search direction. Default is forward. See Excel.SearchDirection.
         *
         * [Api set: ExcelApi 1.9]
         */
        searchDirection?: Excel.SearchDirection | "Forward" | "Backwards";
    }
    /**
     *
     * Represents the worksheet search criteria to be used.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface WorksheetSearchCriteria {
        /**
         *
         * Specifies whether the match needs to be complete or partial. A complete match matches the entire contents of the cell. Default is false (partial).
         *
         * [Api set: ExcelApi 1.9]
         */
        completeMatch?: boolean;
        /**
         *
         * Specifies whether the match is case sensitive. Default is false (insensitive).
         *
         * [Api set: ExcelApi 1.9]
         */
        matchCase?: boolean;
    }
    /**
     *
     * Represents the replace criteria to be used.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface ReplaceCriteria {
        /**
         *
         * Specifies whether the match needs to be complete or partial. Default is false (partial).
         *
         * [Api set: ExcelApi 1.9]
         */
        completeMatch?: boolean;
        /**
         *
         * Specifies whether the match is case sensitive. Default is false (insensitive).
         *
         * [Api set: ExcelApi 1.9]
         */
        matchCase?: boolean;
    }
    /**
     *
     * Specifies which properties to load on the `format.fill` object.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface CellPropertiesFillLoadOptions {
        /**
         *
         * Specifies whether to load on the `color` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        color?: boolean;
        /**
         *
         * Specifies whether to load on the `pattern` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        pattern?: boolean;
        /**
         *
         * Specifies whether to load on the `patternColor` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        patternColor?: boolean;
        /**
         *
         * Specifies whether to load on the `patternTintAndShade` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        patternTintAndShade?: boolean;
        /**
         *
         * Specifies whether to load on the `tintAndShade` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        tintAndShade?: boolean;
    }
    /**
     *
     * Specifies which properties to load on the `format.font` object.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface CellPropertiesFontLoadOptions {
        /**
         *
         * Specifies whether to load on the `bold` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        bold?: boolean;
        /**
         *
         * Specifies whether to load on the `color` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        color?: boolean;
        /**
         *
         * Specifies whether to load on the `italic` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        italic?: boolean;
        /**
         *
         * Specifies whether to load on the `name` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        name?: boolean;
        /**
         *
         * Specifies whether to load on the `size` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        size?: boolean;
        /**
         *
         * Specifies whether to load on the `strikethrough` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        strikethrough?: boolean;
        /**
         *
         * Specifies whether to load on the `subscript` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        subscript?: boolean;
        /**
         *
         * Specifies whether to load on the `superscript` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        superscript?: boolean;
        /**
         *
         * Specifies whether to load on the `tintAndShade` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        tintAndShade?: boolean;
        /**
         *
         * Specifies whether to load on the `underline` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        underline?: boolean;
    }
    /**
     *
     * Specifies which properties to load on the `format.borders` object.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface CellPropertiesBorderLoadOptions {
        /**
         *
         * Specifies whether to load on the `color` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        color?: boolean;
        /**
         *
         * Specifies whether to load on the `style` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        style?: boolean;
        /**
         *
         * Specifies whether to load on the `tintAndShade` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        tintAndShade?: boolean;
        /**
         *
         * Specifies whether to load on the `weight` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        weight?: boolean;
    }
    /**
     *
     * Represents the `format.protection` properties of `getCellProperties`, `getRowProperties`, and `getColumnProperties` or the `format.protection` input parameter of `setCellProperties`, `setRowProperties`, and `setColumnProperties`.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface CellPropertiesProtection {
        /**
         *
         * Represents the `format.protection.formulaHidden` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        formulaHidden?: boolean;
        /**
         *
         * Represents the `format.protection.locked` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        locked?: boolean;
    }
    /**
     *
     * Represents the `format.fill` properties of `getCellProperties`, `getRowProperties`, and `getColumnProperties` or the `format.fill` input parameter of `setCellProperties`, `setRowProperties`, and `setColumnProperties`.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface CellPropertiesFill {
        /**
         *
         * Represents the `format.fill.color` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        color?: string;
        /**
         *
         * Represents the `format.fill.pattern` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        pattern?: Excel.FillPattern | "None" | "Solid" | "Gray50" | "Gray75" | "Gray25" | "Horizontal" | "Vertical" | "Down" | "Up" | "Checker" | "SemiGray75" | "LightHorizontal" | "LightVertical" | "LightDown" | "LightUp" | "Grid" | "CrissCross" | "Gray16" | "Gray8" | "LinearGradient" | "RectangularGradient";
        /**
         *
         * Represents the `format.fill.patternColor` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        patternColor?: string;
        /**
         *
         * Represents the `format.fill.patternTintAndShade` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        patternTintAndShade?: number;
        /**
         *
         * Represents the `format.fill.tintAndShade` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        tintAndShade?: number;
    }
    /**
     *
     * Represents the `format.font` properties of `getCellProperties`, `getRowProperties`, and `getColumnProperties` or the `format.font` input parameter of `setCellProperties`, `setRowProperties`, and `setColumnProperties`.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface CellPropertiesFont {
        /**
         *
         * Represents the `format.font.bold` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        bold?: boolean;
        /**
         *
         * Represents the `format.font.color` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        color?: string;
        /**
         *
         * Represents the `format.font.italic` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        italic?: boolean;
        /**
         *
         * Represents the `format.font.name` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        name?: string;
        /**
         *
         * Represents the `format.font.size` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        size?: number;
        /**
         *
         * Represents the `format.font.strikethrough` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        strikethrough?: boolean;
        /**
         *
         * Represents the `format.font.subscript` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        subscript?: boolean;
        /**
         *
         * Represents the `format.font.superscript` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        superscript?: boolean;
        /**
         *
         * Represents the `format.font.tintAndShade` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        tintAndShade?: number;
        /**
         *
         * Represents the `format.font.underline` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        underline?: Excel.RangeUnderlineStyle | "None" | "Single" | "Double" | "SingleAccountant" | "DoubleAccountant";
    }
    /**
     *
     * Represents the `format.borders` properties of `getCellProperties`, `getRowProperties`, and `getColumnProperties` or the `format.borders` input parameter of `setCellProperties`, `setRowProperties`, and `setColumnProperties`.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface CellBorderCollection {
        /**
         *
         * Represents the `format.borders.bottom` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        bottom?: Excel.CellBorder;
        /**
         *
         * Represents the `format.borders.diagonalDown` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        diagonalDown?: Excel.CellBorder;
        /**
         *
         * Represents the `format.borders.diagonalUp` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        diagonalUp?: Excel.CellBorder;
        /**
         *
         * Represents the `format.borders.horizontal` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        horizontal?: Excel.CellBorder;
        /**
         *
         * Represents the `format.borders.left` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        left?: Excel.CellBorder;
        /**
         *
         * Represents the `format.borders.right` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        right?: Excel.CellBorder;
        /**
         *
         * Represents the `format.borders.top` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        top?: Excel.CellBorder;
        /**
         *
         * Represents the `format.borders.vertical` property.
         *
         * [Api set: ExcelApi 1.9]
         */
        vertical?: Excel.CellBorder;
    }
    /**
     *
     * Represents the properties of a single border returned by `getCellProperties`, `getRowProperties`, and `getColumnProperties` or the border property input parameter of `setCellProperties`, `setRowProperties`, and `setColumnProperties`.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface CellBorder {
        /**
         *
         * Represents the `color` property of a single border.
         *
         * [Api set: ExcelApi 1.9]
         */
        color?: string;
        /**
         *
         * Represents the `style` property of a single border.
         *
         * [Api set: ExcelApi 1.9]
         */
        style?: Excel.BorderLineStyle | "None" | "Continuous" | "Dash" | "DashDot" | "DashDotDot" | "Dot" | "Double" | "SlantDashDot";
        /**
         *
         * Represents the `tintAndShade` property of a single border.
         *
         * [Api set: ExcelApi 1.9]
         */
        tintAndShade?: number;
        /**
         *
         * Represents the `weight` property of a single border.
         *
         * [Api set: ExcelApi 1.9]
         */
        weight?: Excel.BorderWeight | "Hairline" | "Thin" | "Medium" | "Thick";
    }
    /**
     *
     * RangeView represents a set of visible cells of the parent range.
     *
     * [Api set: ExcelApi 1.3]
     */
    class RangeView extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents a collection of range views associated with the range. Read-only.
         *
         * [Api set: ExcelApi 1.3]
         */
        readonly rows: Excel.RangeViewCollection;
        /**
         *
         * Represents the cell addresses of the RangeView. Read-only.
         *
         * [Api set: ExcelApi 1.3]
         */
        readonly cellAddresses: any[][];
        /**
         *
         * Returns the number of visible columns. Read-only.
         *
         * [Api set: ExcelApi 1.3]
         */
        readonly columnCount: number;
        /**
         *
         * Represents the formula in A1-style notation.
         *
         * [Api set: ExcelApi 1.3]
         */
        formulas: any[][];
        /**
         *
         * Represents the formula in A1-style notation, in the user's language and number-formatting locale.  For example, the English "=SUM(A1, 1.5)" formula would become "=SUMME(A1; 1,5)" in German.
         *
         * [Api set: ExcelApi 1.3]
         */
        formulasLocal: any[][];
        /**
         *
         * Represents the formula in R1C1-style notation.
         *
         * [Api set: ExcelApi 1.3]
         */
        formulasR1C1: any[][];
        /**
         *
         * Returns a value that represents the index of the RangeView. Read-only.
         *
         * [Api set: ExcelApi 1.3]
         */
        readonly index: number;
        /**
         *
         * Represents Excel's number format code for the given cell.
         *
         * [Api set: ExcelApi 1.3]
         */
        numberFormat: any[][];
        /**
         *
         * Returns the number of visible rows. Read-only.
         *
         * [Api set: ExcelApi 1.3]
         */
        readonly rowCount: number;
        /**
         *
         * Text values of the specified range. The Text value will not depend on the cell width. The # sign substitution that happens in Excel UI will not affect the text value returned by the API. Read-only.
         *
         * [Api set: ExcelApi 1.3]
         */
        readonly text: string[][];
        /**
         *
         * Represents the type of data of each cell. Read-only.
         *
         * [Api set: ExcelApi 1.3]
         */
        readonly valueTypes: Excel.RangeValueType[][];
        /**
         *
         * Represents the raw values of the specified range view. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
         *
         * [Api set: ExcelApi 1.3]
         */
        values: any[][];
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.RangeView): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.RangeViewUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.RangeView): void;
        /**
         *
         * Gets the parent range associated with the current RangeView.
         *
         * [Api set: ExcelApi 1.3]
         */
        getRange(): Excel.Range;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.RangeView` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.RangeView` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.RangeView` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.RangeViewLoadOptions): Excel.RangeView;
        load(option?: string | string[]): Excel.RangeView;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.RangeView;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.RangeView object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.RangeViewData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.RangeViewData;
    }
    /**
     *
     * Represents a collection of RangeView objects.
     *
     * [Api set: ExcelApi 1.3]
     */
    class RangeViewCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.RangeView[];
        /**
         *
         * Gets the number of RangeView objects in the collection.
         *
         * [Api set: ExcelApi 1.4]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a RangeView Row via its index. Zero-Indexed.
         *
         * [Api set: ExcelApi 1.3]
         *
         * @param index Index of the visible row.
         */
        getItemAt(index: number): Excel.RangeView;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.RangeViewCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.RangeViewCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.RangeViewCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.RangeViewCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.RangeViewCollection;
        load(option?: string | string[]): Excel.RangeViewCollection;
        load(option?: OfficeExtension.LoadOption): Excel.RangeViewCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.RangeViewCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.RangeViewCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.RangeViewCollectionData;
    }
    /**
     *
     * Represents a collection of key-value pair setting objects that are part of the workbook. The scope is limited to per file and add-in (task-pane or content) combination.
     *
     * [Api set: ExcelApi 1.4]
     */
    class SettingCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.Setting[];
        /**
         *
         * Sets or adds the specified setting to the workbook.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param key The Key of the new setting.
         * @param value The Value for the new setting.
         */
        add(key: string, value: string | number | boolean | Date | Array<any> | any): Excel.Setting;
        /**
         *
         * Gets the number of Settings in the collection.
         *
         * [Api set: ExcelApi 1.4]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a Setting entry via the key.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param key Key of the setting.
         */
        getItem(key: string): Excel.Setting;
        /**
         *
         * Gets a Setting entry via the key. If the Setting does not exist, will return a null object.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param key The key of the setting.
         */
        getItemOrNullObject(key: string): Excel.Setting;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.SettingCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.SettingCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.SettingCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.SettingCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.SettingCollection;
        load(option?: string | string[]): Excel.SettingCollection;
        load(option?: OfficeExtension.LoadOption): Excel.SettingCollection;
        /**
         *
         * Occurs when the Settings in the document are changed.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @eventproperty
         */
        readonly onSettingsChanged: OfficeExtension.EventHandlers<Excel.SettingsChangedEventArgs>;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.SettingCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.SettingCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.SettingCollectionData;
    }
    /**
     *
     * Setting represents a key-value pair of a setting persisted to the document (per file per add-in). These custom key-value pair can be used to store state or lifecycle information needed by the content or task-pane add-in. Note that settings are persisted in the document and hence it is not a place to store any sensitive or protected information such as user information and password.
     *
     * [Api set: ExcelApi 1.4]
     */
    class Setting extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        private static DateJSONPrefix;
        private static DateJSONSuffix;
        private static replaceStringDateWithDate;
        /**
         *
         * Returns the key that represents the id of the Setting. Read-only.
         *
         * [Api set: ExcelApi 1.4]
         */
        readonly key: string;
        /**
         *
         * Represents the value stored for this setting.
         *
         * [Api set: ExcelApi 1.4]
         */
        value: any;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.Setting): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.SettingUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.Setting): void;
        /**
         *
         * Deletes the setting.
         *
         * [Api set: ExcelApi 1.4]
         */
        delete(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.Setting` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.Setting` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.Setting` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.SettingLoadOptions): Excel.Setting;
        load(option?: string | string[]): Excel.Setting;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.Setting;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.Setting object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.SettingData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.SettingData;
    }
    /**
     *
     * A collection of all the NamedItem objects that are part of the workbook or worksheet, depending on how it was reached.
     *
     * [Api set: ExcelApi 1.1]
     */
    class NamedItemCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.NamedItem[];
        /**
         *
         * Adds a new name to the collection of the given scope.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param name The name of the named item.
         * @param reference The formula or the range that the name will refer to.
         * @param comment Optional. The comment associated with the named item.
         * @returns
         */
        add(name: string, reference: Range | string, comment?: string): Excel.NamedItem;
        /**
         *
         * Adds a new name to the collection of the given scope using the user's locale for the formula.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param name The "name" of the named item.
         * @param formula The formula in the user's locale that the name will refer to.
         * @param comment Optional. The comment associated with the named item.
         * @returns
         */
        addFormulaLocal(name: string, formula: string, comment?: string): Excel.NamedItem;
        /**
         *
         * Gets the number of named items in the collection.
         *
         * [Api set: ExcelApi 1.4]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a NamedItem object using its name.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param name Nameditem name.
         */
        getItem(name: string): Excel.NamedItem;
        /**
         *
         * Gets a NamedItem object using its name. If the nameditem object does not exist, will return a null object.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param name Nameditem name.
         */
        getItemOrNullObject(name: string): Excel.NamedItem;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.NamedItemCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.NamedItemCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.NamedItemCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.NamedItemCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.NamedItemCollection;
        load(option?: string | string[]): Excel.NamedItemCollection;
        load(option?: OfficeExtension.LoadOption): Excel.NamedItemCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.NamedItemCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.NamedItemCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.NamedItemCollectionData;
    }
    /**
     *
     * Represents a defined name for a range of cells or value. Names can be primitive named objects (as seen in the type below), range object, or a reference to a range. This object can be used to obtain range object associated with names.
     *
     * [Api set: ExcelApi 1.1]
     */
    class NamedItem extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns an object containing values and types of the named item. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly arrayValues: Excel.NamedItemArrayValues;
        /**
         *
         * Returns the worksheet on which the named item is scoped to. Throws an error if the item is scoped to the workbook instead.
         *
         * [Api set: ExcelApi 1.4]
         */
        readonly worksheet: Excel.Worksheet;
        /**
         *
         * Returns the worksheet on which the named item is scoped to. Returns a null object if the item is scoped to the workbook instead.
         *
         * [Api set: ExcelApi 1.4]
         */
        readonly worksheetOrNullObject: Excel.Worksheet;
        /**
         *
         * Represents the comment associated with this name.
         *
         * [Api set: ExcelApi 1.4]
         */
        comment: string;
        /**
         *
         * Gets or sets the formula of the named item.  Formula always starts with a '=' sign.
         *
         * [Api set: ExcelApi 1.7]
         */
        formula: any;
        /**
         *
         * The name of the object. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly name: string;
        /**
         *
         * Indicates whether the name is scoped to the workbook or to a specific worksheet. Possible values are: Worksheet, Workbook. Read-only.
         *
         * [Api set: ExcelApi 1.4]
         */
        readonly scope: Excel.NamedItemScope | "Worksheet" | "Workbook";
        /**
         *
         * Indicates the type of the value returned by the name's formula. See Excel.NamedItemType for details. Read-only.
         *
         * [Api set: ExcelApi 1.1 for String,Integer,Double,Boolean,Range,Error; 1.7 for Array]
         */
        readonly type: Excel.NamedItemType | "String" | "Integer" | "Double" | "Boolean" | "Range" | "Error" | "Array";
        /**
         *
         * Represents the value computed by the name's formula. For a named range, will return the range address. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly value: any;
        /**
         *
         * Specifies whether the object is visible or not.
         *
         * [Api set: ExcelApi 1.1]
         */
        visible: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.NamedItem): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.NamedItemUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.NamedItem): void;
        /**
         *
         * Deletes the given name.
         *
         * [Api set: ExcelApi 1.4]
         */
        delete(): void;
        /**
         *
         * Returns the range object that is associated with the name. Throws an error if the named item's type is not a range.
         *
         * [Api set: ExcelApi 1.1]
         */
        getRange(): Excel.Range;
        /**
         *
         * Returns the range object that is associated with the name. Returns a null object if the named item's type is not a range.
         *
         * [Api set: ExcelApi 1.4]
         */
        getRangeOrNullObject(): Excel.Range;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.NamedItem` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.NamedItem` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.NamedItem` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.NamedItemLoadOptions): Excel.NamedItem;
        load(option?: string | string[]): Excel.NamedItem;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.NamedItem;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.NamedItem object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.NamedItemData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.NamedItemData;
    }
    /**
     *
     * Represents an object containing values and types of a named item.
     *
     * [Api set: ExcelApi 1.7]
     */
    class NamedItemArrayValues extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the types for each item in the named item array
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly types: Excel.RangeValueType[][];
        /**
         *
         * Represents the values of each item in the named item array.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly values: any[][];
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.NamedItemArrayValues` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.NamedItemArrayValues` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.NamedItemArrayValues` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.NamedItemArrayValuesLoadOptions): Excel.NamedItemArrayValues;
        load(option?: string | string[]): Excel.NamedItemArrayValues;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.NamedItemArrayValues;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.NamedItemArrayValues object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.NamedItemArrayValuesData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.NamedItemArrayValuesData;
    }
    /**
     *
     * Represents an Office.js binding that is defined in the workbook.
     *
     * [Api set: ExcelApi 1.1]
     */
    class Binding extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents binding identifier. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly id: string;
        /**
         *
         * Returns the type of the binding. See Excel.BindingType for details. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly type: Excel.BindingType | "Range" | "Table" | "Text";
        /**
         *
         * Deletes the binding.
         *
         * [Api set: ExcelApi 1.3]
         */
        delete(): void;
        /**
         *
         * Returns the range represented by the binding. Will throw an error if binding is not of the correct type.
         *
         * [Api set: ExcelApi 1.1]
         */
        getRange(): Excel.Range;
        /**
         *
         * Returns the table represented by the binding. Will throw an error if binding is not of the correct type.
         *
         * [Api set: ExcelApi 1.1]
         */
        getTable(): Excel.Table;
        /**
         *
         * Returns the text represented by the binding. Will throw an error if binding is not of the correct type.
         *
         * [Api set: ExcelApi 1.1]
         */
        getText(): OfficeExtension.ClientResult<string>;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.Binding` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.Binding` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.Binding` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.BindingLoadOptions): Excel.Binding;
        load(option?: string | string[]): Excel.Binding;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.Binding;
        /**
         *
         * Occurs when data or formatting within the binding is changed.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @eventproperty
         */
        readonly onDataChanged: OfficeExtension.EventHandlers<Excel.BindingDataChangedEventArgs>;
        /**
         *
         * Occurs when the selected content in the binding is changed.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @eventproperty
         */
        readonly onSelectionChanged: OfficeExtension.EventHandlers<Excel.BindingSelectionChangedEventArgs>;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.Binding object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.BindingData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.BindingData;
    }
    /**
     *
     * Represents the collection of all the binding objects that are part of the workbook.
     *
     * [Api set: ExcelApi 1.1]
     */
    class BindingCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.Binding[];
        /**
         *
         * Returns the number of bindings in the collection. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Add a new binding to a particular Range.
         *
         * [Api set: ExcelApi 1.3]
         *
         * @param range Range to bind the binding to. May be an Excel Range object, or a string. If string, must contain the full address, including the sheet name
         * @param bindingType Type of binding. See Excel.BindingType.
         * @param id Name of binding.
         */
        add(range: Range | string, bindingType: Excel.BindingType, id: string): Excel.Binding;
        /**
         *
         * Add a new binding to a particular Range.
         *
         * [Api set: ExcelApi 1.3]
         *
         * @param range Range to bind the binding to. May be an Excel Range object, or a string. If string, must contain the full address, including the sheet name
         * @param bindingType Type of binding. See Excel.BindingType.
         * @param id Name of binding.
         */
        add(range: Range | string, bindingType: "Range" | "Table" | "Text", id: string): Excel.Binding;
        /**
         *
         * Add a new binding based on a named item in the workbook.
            If the named item references to multiple areas, the "InvalidReference" error will be returned.
         *
         * [Api set: ExcelApi 1.3]
         *
         * @param name Name from which to create binding.
         * @param bindingType Type of binding. See Excel.BindingType.
         * @param id Name of binding.
         */
        addFromNamedItem(name: string, bindingType: Excel.BindingType, id: string): Excel.Binding;
        /**
         *
         * Add a new binding based on a named item in the workbook.
            If the named item references to multiple areas, the "InvalidReference" error will be returned.
         *
         * [Api set: ExcelApi 1.3]
         *
         * @param name Name from which to create binding.
         * @param bindingType Type of binding. See Excel.BindingType.
         * @param id Name of binding.
         */
        addFromNamedItem(name: string, bindingType: "Range" | "Table" | "Text", id: string): Excel.Binding;
        /**
         *
         * Add a new binding based on the current selection.
            If the selection has multiple areas, the "InvalidReference" error will be returned.
         *
         * [Api set: ExcelApi 1.3]
         *
         * @param bindingType Type of binding. See Excel.BindingType.
         * @param id Name of binding.
         */
        addFromSelection(bindingType: Excel.BindingType, id: string): Excel.Binding;
        /**
         *
         * Add a new binding based on the current selection.
            If the selection has multiple areas, the "InvalidReference" error will be returned.
         *
         * [Api set: ExcelApi 1.3]
         *
         * @param bindingType Type of binding. See Excel.BindingType.
         * @param id Name of binding.
         */
        addFromSelection(bindingType: "Range" | "Table" | "Text", id: string): Excel.Binding;
        /**
         *
         * Gets the number of bindings in the collection.
         *
         * [Api set: ExcelApi 1.4]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a binding object by ID.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param id Id of the binding object to be retrieved.
         */
        getItem(id: string): Excel.Binding;
        /**
         *
         * Gets a binding object based on its position in the items array.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): Excel.Binding;
        /**
         *
         * Gets a binding object by ID. If the binding object does not exist, will return a null object.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param id Id of the binding object to be retrieved.
         */
        getItemOrNullObject(id: string): Excel.Binding;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.BindingCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.BindingCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.BindingCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.BindingCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.BindingCollection;
        load(option?: string | string[]): Excel.BindingCollection;
        load(option?: OfficeExtension.LoadOption): Excel.BindingCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.BindingCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.BindingCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.BindingCollectionData;
    }
    /**
     *
     * Represents a collection of all the tables that are part of the workbook or worksheet, depending on how it was reached.
     *
     * [Api set: ExcelApi 1.1]
     */
    class TableCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.Table[];
        /**
         *
         * Returns the number of tables in the workbook. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Create a new table. The range object or source address determines the worksheet under which the table will be added. If the table cannot be added (e.g., because the address is invalid, or the table would overlap with another table), an error will be thrown.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param address A Range object, or a string address or name of the range representing the data source. If the address does not contain a sheet name, the currently-active sheet is used. [Api set: ExcelApi 1.1 / 1.3.  Prior to ExcelApi 1.3, this parameter must be a string. Starting with Excel Api 1.3, this parameter may be a Range object or a string.]
         * @param hasHeaders Boolean value that indicates whether the data being imported has column labels. If the source does not contain headers (i.e,. when this property set to false), Excel will automatically generate header shifting the data down by one row.
         */
        add(address: Range | string, hasHeaders: boolean): Excel.Table;
        /**
         *
         * Gets the number of tables in the collection.
         *
         * [Api set: ExcelApi 1.4]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a table by Name or ID.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param key Name or ID of the table to be retrieved.
         */
        getItem(key: string): Excel.Table;
        /**
         *
         * Gets a table based on its position in the collection.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): Excel.Table;
        /**
         *
         * Gets a table by Name or ID. If the table does not exist, will return a null object.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param key Name or ID of the table to be retrieved.
         */
        getItemOrNullObject(key: string): Excel.Table;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.TableCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.TableCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.TableCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.TableCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.TableCollection;
        load(option?: string | string[]): Excel.TableCollection;
        load(option?: OfficeExtension.LoadOption): Excel.TableCollection;
        /**
         *
         * Occurs when new table is added in a workbook.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @eventproperty
         */
        readonly onAdded: OfficeExtension.EventHandlers<Excel.TableAddedEventArgs>;
        /**
         *
         * Occurs when data changes on any table in a workbook, or a worksheet.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @eventproperty
         */
        readonly onChanged: OfficeExtension.EventHandlers<Excel.TableChangedEventArgs>;
        /**
         *
         * Occurs when the specified table is deleted in a workbook.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @eventproperty
         */
        readonly onDeleted: OfficeExtension.EventHandlers<Excel.TableDeletedEventArgs>;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.TableCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.TableCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.TableCollectionData;
    }
    /**
     *
     * Represents a scoped collection of tables. For each table its top-left corner is considered its anchor location and the tables are sorted top to bottom and then left to right.
     *
     * [Api set: ExcelApi 1.9]
     */
    class TableScopedCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.Table[];
        /**
         *
         * Gets the number of tables in the collection.
         *
         * [Api set: ExcelApi 1.9]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets the first table in the collection. The tables in the collection are sorted top to bottom and left to right, such that top left table is the first table in the collection.
         *
         * [Api set: ExcelApi 1.9]
         */
        getFirst(): Excel.Table;
        /**
         *
         * Gets a table by Name or ID.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param key Name or ID of the table to be retrieved.
         */
        getItem(key: string): Excel.Table;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.TableScopedCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.TableScopedCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.TableScopedCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.TableScopedCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.TableScopedCollection;
        load(option?: string | string[]): Excel.TableScopedCollection;
        load(option?: OfficeExtension.LoadOption): Excel.TableScopedCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.TableScopedCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.TableScopedCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.TableScopedCollectionData;
    }
    /**
     *
     * Represents an Excel table.
     *
     * [Api set: ExcelApi 1.1]
     */
    class Table extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the AutoFilter object of the table. Read-Only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly autoFilter: Excel.AutoFilter;
        /**
         *
         * Represents a collection of all the columns in the table. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly columns: Excel.TableColumnCollection;
        /**
         *
         * Represents a collection of all the rows in the table. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly rows: Excel.TableRowCollection;
        /**
         *
         * Represents the sorting for the table. Read-only.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly sort: Excel.TableSort;
        /**
         *
         * The worksheet containing the current table. Read-only.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly worksheet: Excel.Worksheet;
        /**
         *
         * Indicates whether the first column contains special formatting.
         *
         * [Api set: ExcelApi 1.3]
         */
        highlightFirstColumn: boolean;
        /**
         *
         * Indicates whether the last column contains special formatting.
         *
         * [Api set: ExcelApi 1.3]
         */
        highlightLastColumn: boolean;
        /**
         *
         * Returns a value that uniquely identifies the table in a given workbook. The value of the identifier remains the same even when the table is renamed. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly id: string;
        /**
         *
         * Returns a numeric id.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly legacyId: string;
        /**
         *
         * Name of the table.
         *
         * [Api set: ExcelApi 1.1]
         */
        name: string;
        /**
         *
         * Indicates whether the columns show banded formatting in which odd columns are highlighted differently from even ones to make reading the table easier.
         *
         * [Api set: ExcelApi 1.3]
         */
        showBandedColumns: boolean;
        /**
         *
         * Indicates whether the rows show banded formatting in which odd rows are highlighted differently from even ones to make reading the table easier.
         *
         * [Api set: ExcelApi 1.3]
         */
        showBandedRows: boolean;
        /**
         *
         * Indicates whether the filter buttons are visible at the top of each column header. Setting this is only allowed if the table contains a header row.
         *
         * [Api set: ExcelApi 1.3]
         */
        showFilterButton: boolean;
        /**
         *
         * Indicates whether the header row is visible or not. This value can be set to show or remove the header row.
         *
         * [Api set: ExcelApi 1.1]
         */
        showHeaders: boolean;
        /**
         *
         * Indicates whether the total row is visible or not. This value can be set to show or remove the total row.
         *
         * [Api set: ExcelApi 1.1]
         */
        showTotals: boolean;
        /**
         *
         * Constant value that represents the Table style. Possible values are: "TableStyleLight1" through "TableStyleLight21", "TableStyleMedium1" through "TableStyleMedium28", "TableStyleStyleDark1" through "TableStyleStyleDark11". A custom user-defined style present in the workbook can also be specified.
         *
         * [Api set: ExcelApi 1.1]
         */
        style: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.Table): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.TableUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.Table): void;
        /**
         *
         * Clears all the filters currently applied on the table.
         *
         * [Api set: ExcelApi 1.2]
         */
        clearFilters(): void;
        /**
         *
         * Converts the table into a normal range of cells. All data is preserved.
         *
         * [Api set: ExcelApi 1.2]
         */
        convertToRange(): Excel.Range;
        /**
         *
         * Deletes the table.
         *
         * [Api set: ExcelApi 1.1]
         */
        delete(): void;
        /**
         *
         * Gets the range object associated with the data body of the table.
         *
         * [Api set: ExcelApi 1.1]
         */
        getDataBodyRange(): Excel.Range;
        /**
         *
         * Gets the range object associated with header row of the table.
         *
         * [Api set: ExcelApi 1.1]
         */
        getHeaderRowRange(): Excel.Range;
        /**
         *
         * Gets the range object associated with the entire table.
         *
         * [Api set: ExcelApi 1.1]
         */
        getRange(): Excel.Range;
        /**
         *
         * Gets the range object associated with totals row of the table.
         *
         * [Api set: ExcelApi 1.1]
         */
        getTotalRowRange(): Excel.Range;
        /**
         *
         * Reapplies all the filters currently on the table.
         *
         * [Api set: ExcelApi 1.2]
         */
        reapplyFilters(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.Table` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.Table` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.Table` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.TableLoadOptions): Excel.Table;
        load(option?: string | string[]): Excel.Table;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.Table;
        /**
         *
         * Occurs when data in cells changes on a specific table.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @eventproperty
         */
        readonly onChanged: OfficeExtension.EventHandlers<Excel.TableChangedEventArgs>;
        /**
         *
         * Occurs when the selection changes on a specific table.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @eventproperty
         */
        readonly onSelectionChanged: OfficeExtension.EventHandlers<Excel.TableSelectionChangedEventArgs>;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.Table object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.TableData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.TableData;
    }
    /**
     *
     * Represents a collection of all the columns that are part of the table.
     *
     * [Api set: ExcelApi 1.1]
     */
    class TableColumnCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.TableColumn[];
        /**
         *
         * Returns the number of columns in the table. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Adds a new column to the table.
         *
         * [Api set: ExcelApi 1.1 requires an index smaller than the total column count; 1.4 allows index to be optional (null or -1) and will append a column at the end; 1.4 allows name parameter at creation time.]
         *
         * @param index Optional. Specifies the relative position of the new column. If null or -1, the addition happens at the end. Columns with a higher index will be shifted to the side. Zero-indexed.
         * @param values Optional. A 2-dimensional array of unformatted values of the table column.
         * @param name Optional. Specifies the name of the new column. If null, the default name will be used.
         */
        add(index?: number, values?: Array<Array<boolean | string | number>> | boolean | string | number, name?: string): Excel.TableColumn;
        /**
         *
         * Gets the number of columns in the table.
         *
         * [Api set: ExcelApi 1.4]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a column object by Name or ID.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param key Column Name or ID.
         */
        getItem(key: number | string): Excel.TableColumn;
        /**
         *
         * Gets a column based on its position in the collection.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): Excel.TableColumn;
        /**
         *
         * Gets a column object by Name or ID. If the column does not exist, will return a null object.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param key Column Name or ID.
         */
        getItemOrNullObject(key: number | string): Excel.TableColumn;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.TableColumnCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.TableColumnCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.TableColumnCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.TableColumnCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.TableColumnCollection;
        load(option?: string | string[]): Excel.TableColumnCollection;
        load(option?: OfficeExtension.LoadOption): Excel.TableColumnCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.TableColumnCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.TableColumnCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.TableColumnCollectionData;
    }
    /**
     *
     * Represents a column in a table.
     *
     * [Api set: ExcelApi 1.1]
     */
    class TableColumn extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Retrieve the filter applied to the column. Read-only.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly filter: Excel.Filter;
        /**
         *
         * Returns a unique key that identifies the column within the table. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly id: number;
        /**
         *
         * Returns the index number of the column within the columns collection of the table. Zero-indexed. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly index: number;
        /**
         *
         * Represents the name of the table column.
         *
         * [Api set: ExcelApi 1.1 for getting the name; 1.4 for setting it.]
         */
        name: string;
        /**
         *
         * Represents the raw values of the specified range. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
         *
         * [Api set: ExcelApi 1.1]
         */
        values: any[][];
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.TableColumn): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.TableColumnUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.TableColumn): void;
        /**
         *
         * Deletes the column from the table.
         *
         * [Api set: ExcelApi 1.1]
         */
        delete(): void;
        /**
         *
         * Gets the range object associated with the data body of the column.
         *
         * [Api set: ExcelApi 1.1]
         */
        getDataBodyRange(): Excel.Range;
        /**
         *
         * Gets the range object associated with the header row of the column.
         *
         * [Api set: ExcelApi 1.1]
         */
        getHeaderRowRange(): Excel.Range;
        /**
         *
         * Gets the range object associated with the entire column.
         *
         * [Api set: ExcelApi 1.1]
         */
        getRange(): Excel.Range;
        /**
         *
         * Gets the range object associated with the totals row of the column.
         *
         * [Api set: ExcelApi 1.1]
         */
        getTotalRowRange(): Excel.Range;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.TableColumn` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.TableColumn` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.TableColumn` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.TableColumnLoadOptions): Excel.TableColumn;
        load(option?: string | string[]): Excel.TableColumn;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.TableColumn;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.TableColumn object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.TableColumnData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.TableColumnData;
    }
    /**
     *
     * Represents a collection of all the rows that are part of the table.
            
             Note that unlike Ranges or Columns, which will adjust if new rows/columns are added before them,
             a TableRow object represent the physical location of the table row, but not the data.
             That is, if the data is sorted or if new rows are added, a table row will continue
             to point at the index for which it was created.
     *
     * [Api set: ExcelApi 1.1]
     */
    class TableRowCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.TableRow[];
        /**
         *
         * Returns the number of rows in the table. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Adds one or more rows to the table. The return object will be the top of the newly added row(s).
            
             Note that unlike Ranges or Columns, which will adjust if new rows/columns are added before them,
             a TableRow object represent the physical location of the table row, but not the data.
             That is, if the data is sorted or if new rows are added, a table row will continue
             to point at the index for which it was created.
         *
         * [Api set: ExcelApi 1.1 for adding a single row; 1.4 allows adding of multiple rows.]
         *
         * @param index Optional. Specifies the relative position of the new row. If null or -1, the addition happens at the end. Any rows below the inserted row are shifted downwards. Zero-indexed.
         * @param values Optional. A 2-dimensional array of unformatted values of the table row.
         */
        add(index?: number, values?: Array<Array<boolean | string | number>> | boolean | string | number): Excel.TableRow;
        /**
         *
         * Gets the number of rows in the table.
         *
         * [Api set: ExcelApi 1.4]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a row based on its position in the collection.
            
             Note that unlike Ranges or Columns, which will adjust if new rows/columns are added before them,
             a TableRow object represent the physical location of the table row, but not the data.
             That is, if the data is sorted or if new rows are added, a table row will continue
             to point at the index for which it was created.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): Excel.TableRow;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.TableRowCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.TableRowCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.TableRowCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.TableRowCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.TableRowCollection;
        load(option?: string | string[]): Excel.TableRowCollection;
        load(option?: OfficeExtension.LoadOption): Excel.TableRowCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.TableRowCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.TableRowCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.TableRowCollectionData;
    }
    /**
     *
     * Represents a row in a table.
            
             Note that unlike Ranges or Columns, which will adjust if new rows/columns are added before them,
             a TableRow object represent the physical location of the table row, but not the data.
             That is, if the data is sorted or if new rows are added, a table row will continue
             to point at the index for which it was created.
     *
     * [Api set: ExcelApi 1.1]
     */
    class TableRow extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns the index number of the row within the rows collection of the table. Zero-indexed. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly index: number;
        /**
         *
         * Represents the raw values of the specified range. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
         *
         * [Api set: ExcelApi 1.1]
         */
        values: any[][];
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.TableRow): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.TableRowUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.TableRow): void;
        /**
         *
         * Deletes the row from the table.
         *
         * [Api set: ExcelApi 1.1]
         */
        delete(): void;
        /**
         *
         * Returns the range object associated with the entire row.
         *
         * [Api set: ExcelApi 1.1]
         */
        getRange(): Excel.Range;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.TableRow` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.TableRow` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.TableRow` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.TableRowLoadOptions): Excel.TableRow;
        load(option?: string | string[]): Excel.TableRow;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.TableRow;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.TableRow object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.TableRowData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.TableRowData;
    }
    /**
     *
     * Represents the data validation applied to the current range.
     *
     * [Api set: ExcelApi 1.8]
     */
    class DataValidation extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Error alert when user enters invalid data.
         *
         * [Api set: ExcelApi 1.8]
         */
        errorAlert: Excel.DataValidationErrorAlert;
        /**
         *
         * Ignore blanks: no data validation will be performed on blank cells, it defaults to true.
         *
         * [Api set: ExcelApi 1.8]
         */
        ignoreBlanks: boolean;
        /**
         *
         * Prompt when users select a cell.
         *
         * [Api set: ExcelApi 1.8]
         */
        prompt: Excel.DataValidationPrompt;
        /**
         *
         * Data validation rule that contains different type of data validation criteria.
         *
         * [Api set: ExcelApi 1.8]
         */
        rule: Excel.DataValidationRule;
        /**
         *
         * Type of the data validation, see Excel.DataValidationType for details.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly type: Excel.DataValidationType | "None" | "WholeNumber" | "Decimal" | "List" | "Date" | "Time" | "TextLength" | "Custom" | "Inconsistent" | "MixedCriteria";
        /**
         *
         * Represents if all cell values are valid according to the data validation rules.
            Returns true if all cell values are valid, or false if all cell values are invalid.
            Returns null if there are both valid and invalid cell values within the range.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly valid: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.DataValidation): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.DataValidationUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.DataValidation): void;
        /**
         *
         * Clears the data validation from the current range.
         *
         * [Api set: ExcelApi 1.8]
         */
        clear(): void;
        /**
         *
         * Returns a RangeAreas, comprising one or more rectangular ranges, with invalid cell values. If all cell values are valid, this function will throw an ItemNotFound error.
         *
         * [Api set: ExcelApi 1.9]
         */
        getInvalidCells(): Excel.RangeAreas;
        /**
         *
         * Returns a RangeAreas, comprising one or more rectangular ranges, with invalid cell values. If all cell values are valid, this function will return null.
         *
         * [Api set: ExcelApi 1.9]
         */
        getInvalidCellsOrNullObject(): Excel.RangeAreas;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.DataValidation` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.DataValidation` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.DataValidation` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.DataValidationLoadOptions): Excel.DataValidation;
        load(option?: string | string[]): Excel.DataValidation;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.DataValidation;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.DataValidation object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.DataValidationData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.DataValidationData;
    }
    /**
     *
     * Data validation rule contains different types of data validation. You can only use one of them at a time according the Excel.DataValidationType.
     *
     * [Api set: ExcelApi 1.8]
     */
    interface DataValidationRule {
        /**
         *
         * Custom data validation criteria.
         *
         * [Api set: ExcelApi 1.8]
         */
        custom?: Excel.CustomDataValidation;
        /**
         *
         * Date data validation criteria.
         *
         * [Api set: ExcelApi 1.8]
         */
        date?: Excel.DateTimeDataValidation;
        /**
         *
         * Decimal data validation criteria.
         *
         * [Api set: ExcelApi 1.8]
         */
        decimal?: Excel.BasicDataValidation;
        /**
         *
         * List data validation criteria.
         *
         * [Api set: ExcelApi 1.8]
         */
        list?: Excel.ListDataValidation;
        /**
         *
         * TextLength data validation criteria.
         *
         * [Api set: ExcelApi 1.8]
         */
        textLength?: Excel.BasicDataValidation;
        /**
         *
         * Time data validation criteria.
         *
         * [Api set: ExcelApi 1.8]
         */
        time?: Excel.DateTimeDataValidation;
        /**
         *
         * WholeNumber data validation criteria.
         *
         * [Api set: ExcelApi 1.8]
         */
        wholeNumber?: Excel.BasicDataValidation;
    }
    /**
     *
     * Represents the results from the removeDuplicates method on range
     *
     * [Api set: ExcelApi 1.9]
     */
    class RemoveDuplicatesResult extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Number of duplicated rows removed by the operation.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly removed: number;
        /**
         *
         * Number of remaining unique rows present in the resulting range.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly uniqueRemaining: number;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.RemoveDuplicatesResult` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.RemoveDuplicatesResult` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.RemoveDuplicatesResult` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.RemoveDuplicatesResultLoadOptions): Excel.RemoveDuplicatesResult;
        load(option?: string | string[]): Excel.RemoveDuplicatesResult;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.RemoveDuplicatesResult;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.RemoveDuplicatesResult object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.RemoveDuplicatesResultData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.RemoveDuplicatesResultData;
    }
    /**
     *
     * Represents the Basic Type data validation criteria.
     *
     * [Api set: ExcelApi 1.8]
     */
    interface BasicDataValidation {
        /**
         *
         * Specifies the right-hand operand when the operator property is set to a binary operator such as GreaterThan (the left-hand operand is the value the user tries to enter in the cell). With the ternary operators Between and NotBetween, specifies the lower bound operand.
            For example, setting formula1 to 10 and operator to GreaterThan means that valid data for the range must be greater than 10.
            When setting the value, it can be passed in as a number, a range object, or a string formula (where the string is either a stringified number, a cell reference like "=A1", or a formula like "=MIN(A1, B1)").
            When retrieving the value, it will always be returned as a string formula, for example: "=10", "=A1", "=SUM(A1:B5)", etc.
         *
         * [Api set: ExcelApi 1.8]
         */
        formula1: string | number | Range;
        /**
         *
         * With the ternary operators Between and NotBetween, specifies the upper bound operand. Is not used with the binary operators, such as GreaterThan.
            When setting the value, it can be passed in as a number, a range object, or a string formula (where the string is either a stringified number, a cell reference like "=A1", or a formula like "=MIN(A1, B1)").
            When retrieving the value, it will always be returned as a string formula, for example: "=10", "=A1", "=SUM(A1:B5)", etc.
         *
         * [Api set: ExcelApi 1.8]
         */
        formula2?: string | number | Range;
        /**
         *
         * The operator to use for validating the data.
         *
         * [Api set: ExcelApi 1.8]
         */
        operator: Excel.DataValidationOperator | "Between" | "NotBetween" | "EqualTo" | "NotEqualTo" | "GreaterThan" | "LessThan" | "GreaterThanOrEqualTo" | "LessThanOrEqualTo";
    }
    /**
     *
     * Represents the Date data validation criteria.
     *
     * [Api set: ExcelApi 1.8]
     */
    interface DateTimeDataValidation {
        /**
         *
         * Specifies the right-hand operand when the operator property is set to a binary operator such as GreaterThan (the left-hand operand is the value the user tries to enter in the cell). With the ternary operators Between and NotBetween, specifies the lower bound operand.
            When setting the value, it can be passed in as a Date, a Range object, or a string formula (where the string is either a stringified date/time in ISO8601 format, a cell reference like "=A1", or a formula like "=MIN(A1, B1)").
            When retrieving the value, it will always be returned as a string formula, for example: "=10", "=A1", "=SUM(A1:B5)", etc.
         *
         * [Api set: ExcelApi 1.8]
         */
        formula1: string | Date | Range;
        /**
         *
         * With the ternary operators Between and NotBetween, specifies the upper bound operand. Is not used with the binary operators, such as GreaterThan.
            When setting the value, it can be passed in as a Date, a Range object, or a string (where the string is either a stringified date/time in ISO8601 format, a cell reference like "=A1", or a formula like "=MIN(A1, B1)").
            When retrieving the value, it will always be returned as a string formula, for example: "=10", "=A1", "=SUM(A1:B5)", etc.
         *
         * [Api set: ExcelApi 1.8]
         */
        formula2?: string | Date | Range;
        /**
         *
         * The operator to use for validating the data.
         *
         * [Api set: ExcelApi 1.8]
         */
        operator: Excel.DataValidationOperator | "Between" | "NotBetween" | "EqualTo" | "NotEqualTo" | "GreaterThan" | "LessThan" | "GreaterThanOrEqualTo" | "LessThanOrEqualTo";
    }
    /**
     *
     * Represents the List data validation criteria.
     *
     * [Api set: ExcelApi 1.8]
     */
    interface ListDataValidation {
        /**
         *
         * Displays the list in cell drop down or not, it defaults to true.
         *
         * [Api set: ExcelApi 1.8]
         */
        inCellDropDown: boolean;
        /**
         *
         * Source of the list for data validation
            When setting the value, it can be passed in as a Excel Range object, or a string that contains comma separated number, boolean or date.
         *
         * [Api set: ExcelApi 1.8]
         */
        source: string | Range;
    }
    /**
     *
     * Represents the Custom data validation criteria.
     *
     * [Api set: ExcelApi 1.8]
     */
    interface CustomDataValidation {
        /**
         *
         * A custom data validation formula. This creates special input rules, such as preventing duplicates, or limiting the total in a range of cells.
         *
         * [Api set: ExcelApi 1.8]
         */
        formula: string;
    }
    /**
     *
     * Represents the error alert properties for the data validation.
     *
     * [Api set: ExcelApi 1.8]
     */
    interface DataValidationErrorAlert {
        /**
         *
         * Represents error alert message.
         *
         * [Api set: ExcelApi 1.8]
         */
        message: string;
        /**
         *
         * Determines whether to show an error alert dialog or not when a user enters invalid data. The default is true.
         *
         * [Api set: ExcelApi 1.8]
         */
        showAlert: boolean;
        /**
         *
         * Represents data validation alert type, please see Excel.DataValidationAlertStyle for details.
         *
         * [Api set: ExcelApi 1.8]
         */
        style: Excel.DataValidationAlertStyle | "Stop" | "Warning" | "Information";
        /**
         *
         * Represents error alert dialog title.
         *
         * [Api set: ExcelApi 1.8]
         */
        title: string;
    }
    /**
     *
     * Represents the user prompt properties for the data validation.
     *
     * [Api set: ExcelApi 1.8]
     */
    interface DataValidationPrompt {
        /**
         *
         * Represents the message of the prompt.
         *
         * [Api set: ExcelApi 1.8]
         */
        message: string;
        /**
         *
         * Determines whether or not to show the prompt when user selects a cell with data validation.
         *
         * [Api set: ExcelApi 1.8]
         */
        showPrompt: boolean;
        /**
         *
         * Represents the title for the prompt.
         *
         * [Api set: ExcelApi 1.8]
         */
        title: string;
    }
    /**
     *
     * A format object encapsulating the range's font, fill, borders, alignment, and other properties.
     *
     * [Api set: ExcelApi 1.1]
     */
    class RangeFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Collection of border objects that apply to the overall range. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly borders: Excel.RangeBorderCollection;
        /**
         *
         * Returns the fill object defined on the overall range. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly fill: Excel.RangeFill;
        /**
         *
         * Returns the font object defined on the overall range. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly font: Excel.RangeFont;
        /**
         *
         * Returns the format protection object for a range. Read-only.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly protection: Excel.FormatProtection;
        /**
         *
         * Indicates if text is automatically indented when text alignment is set to equal distribution.
         *
         * [Api set: ExcelApi 1.9]
         */
        autoIndent: boolean;
        /**
         *
         * Gets or sets the width of all colums within the range. If the column widths are not uniform, null will be returned.
         *
         * [Api set: ExcelApi 1.2]
         */
        columnWidth: number;
        /**
         *
         * Represents the horizontal alignment for the specified object. See Excel.HorizontalAlignment for details.
         *
         * [Api set: ExcelApi 1.1]
         */
        horizontalAlignment: Excel.HorizontalAlignment | "General" | "Left" | "Center" | "Right" | "Fill" | "Justify" | "CenterAcrossSelection" | "Distributed";
        /**
         *
         * An integer from 0 to 250 that indicates the indent level.
         *
         * [Api set: ExcelApi 1.9]
         */
        indentLevel: number;
        /**
         *
         * The reading order for the range.
         *
         * [Api set: ExcelApi 1.9]
         */
        readingOrder: Excel.ReadingOrder | "Context" | "LeftToRight" | "RightToLeft";
        /**
         *
         * Gets or sets the height of all rows in the range. If the row heights are not uniform, null will be returned.
         *
         * [Api set: ExcelApi 1.2]
         */
        rowHeight: number;
        /**
         *
         * Indicates if text automatically shrinks to fit in the available column width.
         *
         * [Api set: ExcelApi 1.9]
         */
        shrinkToFit: boolean;
        /**
         *
         * Gets or sets the text orientation of all the cells within the range.
            The text orientation should be an integer either from -90 to 90, or 180 for vertically-oriented text.
            If the orientation within a range are not uniform, then null will be returned.
         *
         * [Api set: ExcelApi 1.7]
         */
        textOrientation: number;
        /**
         *
         * Determines if the row height of the Range object equals the standard height of the sheet.
            Returns True if the row height of the Range object equals the standard height of the sheet.
            Returns Null if the range contains more than one row and the rows aren't all the same height.
            Returns False otherwise.
         *
         * [Api set: ExcelApi 1.7]
         */
        useStandardHeight: boolean;
        /**
         *
         * Indicates whether the column width of the Range object equals the standard width of the sheet.
            Returns True if the column width of the Range object equals the standard width of the sheet.
            Returns Null if the range contains more than one column and the columns aren't all the same height.
            Returns False otherwise.
         *
         * [Api set: ExcelApi 1.7]
         */
        useStandardWidth: boolean;
        /**
         *
         * Represents the vertical alignment for the specified object. See Excel.VerticalAlignment for details.
         *
         * [Api set: ExcelApi 1.1]
         */
        verticalAlignment: Excel.VerticalAlignment | "Top" | "Center" | "Bottom" | "Justify" | "Distributed";
        /**
         *
         * Indicates if Excel wraps the text in the object. A null value indicates that the entire range doesn't have uniform wrap setting
         *
         * [Api set: ExcelApi 1.1]
         */
        wrapText: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.RangeFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.RangeFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.RangeFormat): void;
        /**
         *
         * Changes the width of the columns of the current range to achieve the best fit, based on the current data in the columns.
         *
         * [Api set: ExcelApi 1.2]
         */
        autofitColumns(): void;
        /**
         *
         * Changes the height of the rows of the current range to achieve the best fit, based on the current data in the columns.
         *
         * [Api set: ExcelApi 1.2]
         */
        autofitRows(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.RangeFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.RangeFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.RangeFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.RangeFormatLoadOptions): Excel.RangeFormat;
        load(option?: string | string[]): Excel.RangeFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.RangeFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.RangeFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.RangeFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.RangeFormatData;
    }
    /**
     *
     * Represents the format protection of a range object.
     *
     * [Api set: ExcelApi 1.2]
     */
    class FormatProtection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Indicates if Excel hides the formula for the cells in the range. A null value indicates that the entire range doesn't have uniform formula hidden setting.
         *
         * [Api set: ExcelApi 1.2]
         */
        formulaHidden: boolean;
        /**
         *
         * Indicates if Excel locks the cells in the object. A null value indicates that the entire range doesn't have uniform lock setting.
         *
         * [Api set: ExcelApi 1.2]
         */
        locked: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.FormatProtection): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.FormatProtectionUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.FormatProtection): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.FormatProtection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.FormatProtection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.FormatProtection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.FormatProtectionLoadOptions): Excel.FormatProtection;
        load(option?: string | string[]): Excel.FormatProtection;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.FormatProtection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.FormatProtection object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.FormatProtectionData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.FormatProtectionData;
    }
    /**
     *
     * Represents the background of a range object.
     *
     * [Api set: ExcelApi 1.1]
     */
    class RangeFill extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange")
         *
         * [Api set: ExcelApi 1.1]
         */
        color: string;
        /**
         *
         * Gets or sets the pattern of a Range. See Excel.FillPattern for details. LinearGradient and RectangularGradient are not supported.
            A null value indicates that the entire range doesn't have uniform pattern setting.
         *
         * [Api set: ExcelApi 1.9]
         */
        pattern: Excel.FillPattern | "None" | "Solid" | "Gray50" | "Gray75" | "Gray25" | "Horizontal" | "Vertical" | "Down" | "Up" | "Checker" | "SemiGray75" | "LightHorizontal" | "LightVertical" | "LightDown" | "LightUp" | "Grid" | "CrissCross" | "Gray16" | "Gray8" | "LinearGradient" | "RectangularGradient";
        /**
         *
         * Sets HTML color code representing the color of the Range pattern, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            Gets HTML color code representing the color of the Range pattern, of the form #RRGGBB (e.g. "FFA500").
         *
         * [Api set: ExcelApi 1.9]
         */
        patternColor: string;
        /**
         *
         * Returns or sets a double that lightens or darkens a pattern color for Range Fill, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            If the pattern tintAndShades are not uniform, null will be returned.
         *
         * [Api set: ExcelApi 1.9]
         */
        patternTintAndShade: number;
        /**
         *
         * Returns or sets a double that lightens or darkens a color for Range Fill, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            If the tintAndShades are not uniform, null will be returned.
         *
         * [Api set: ExcelApi 1.9]
         */
        tintAndShade: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.RangeFill): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.RangeFillUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.RangeFill): void;
        /**
         *
         * Resets the range background.
         *
         * [Api set: ExcelApi 1.1]
         */
        clear(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.RangeFill` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.RangeFill` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.RangeFill` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.RangeFillLoadOptions): Excel.RangeFill;
        load(option?: string | string[]): Excel.RangeFill;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.RangeFill;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.RangeFill object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.RangeFillData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.RangeFillData;
    }
    /**
     *
     * Represents the border of an object.
     *
     * [Api set: ExcelApi 1.1]
     */
    class RangeBorder extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
         *
         * [Api set: ExcelApi 1.1]
         */
        color: string;
        /**
         *
         * Constant value that indicates the specific side of the border. See Excel.BorderIndex for details. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly sideIndex: Excel.BorderIndex | "EdgeTop" | "EdgeBottom" | "EdgeLeft" | "EdgeRight" | "InsideVertical" | "InsideHorizontal" | "DiagonalDown" | "DiagonalUp";
        /**
         *
         * One of the constants of line style specifying the line style for the border. See Excel.BorderLineStyle for details.
         *
         * [Api set: ExcelApi 1.1]
         */
        style: Excel.BorderLineStyle | "None" | "Continuous" | "Dash" | "DashDot" | "DashDotDot" | "Dot" | "Double" | "SlantDashDot";
        /**
         *
         * Returns or sets a double that lightens or darkens a color for Range Border, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            A null value indicates that the border doesn't have uniform tintAndShade setting.
         *
         * [Api set: ExcelApi 1.9]
         */
        tintAndShade: number;
        /**
         *
         * Specifies the weight of the border around a range. See Excel.BorderWeight for details.
         *
         * [Api set: ExcelApi 1.1]
         */
        weight: Excel.BorderWeight | "Hairline" | "Thin" | "Medium" | "Thick";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.RangeBorder): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.RangeBorderUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.RangeBorder): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.RangeBorder` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.RangeBorder` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.RangeBorder` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.RangeBorderLoadOptions): Excel.RangeBorder;
        load(option?: string | string[]): Excel.RangeBorder;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.RangeBorder;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.RangeBorder object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.RangeBorderData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.RangeBorderData;
    }
    /**
     *
     * Represents the border objects that make up the range border.
     *
     * [Api set: ExcelApi 1.1]
     */
    class RangeBorderCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.RangeBorder[];
        /**
         *
         * Number of border objects in the collection. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Returns or sets a double that lightens or darkens a color for Range Borders, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            A null value indicates that the entire border collections don't have uniform tintAndShade setting.
         *
         * [Api set: ExcelApi 1.9]
         */
        tintAndShade: number;
        /**
         *
         * Gets a border object using its name.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param index Index value of the border object to be retrieved. See Excel.BorderIndex for details.
         */
        getItem(index: Excel.BorderIndex): Excel.RangeBorder;
        /**
         *
         * Gets a border object using its name.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param index Index value of the border object to be retrieved. See Excel.BorderIndex for details.
         */
        getItem(index: "EdgeTop" | "EdgeBottom" | "EdgeLeft" | "EdgeRight" | "InsideVertical" | "InsideHorizontal" | "DiagonalDown" | "DiagonalUp"): Excel.RangeBorder;
        /**
         *
         * Gets a border object using its index.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): Excel.RangeBorder;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.RangeBorderCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.RangeBorderCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.RangeBorderCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.RangeBorderCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.RangeBorderCollection;
        load(option?: string | string[]): Excel.RangeBorderCollection;
        load(option?: OfficeExtension.LoadOption): Excel.RangeBorderCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.RangeBorderCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.RangeBorderCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.RangeBorderCollectionData;
    }
    /**
     *
     * This object represents the font attributes (font name, font size, color, etc.) for an object.
     *
     * [Api set: ExcelApi 1.1]
     */
    class RangeFont extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the bold status of font.
         *
         * [Api set: ExcelApi 1.1]
         */
        bold: boolean;
        /**
         *
         * HTML color code representation of the text color. E.g. #FF0000 represents Red.
         *
         * [Api set: ExcelApi 1.1]
         */
        color: string;
        /**
         *
         * Represents the italic status of the font.
         *
         * [Api set: ExcelApi 1.1]
         */
        italic: boolean;
        /**
         *
         * Font name (e.g. "Calibri")
         *
         * [Api set: ExcelApi 1.1]
         */
        name: string;
        /**
         *
         * Font size.
         *
         * [Api set: ExcelApi 1.1]
         */
        size: number;
        /**
         *
         * Represents the strikethrough status of font. A null value indicates that the entire range doesn't have uniform Strikethrough setting.
         *
         * [Api set: ExcelApi 1.9]
         */
        strikethrough: boolean;
        /**
         *
         * Represents the Subscript status of font.
            Returns True if all the fonts of the range are Subscript.
            Returns False if all the fonts of the range are Superscript or normal (neither Superscript, nor Subscript).
            Returns Null otherwise.
         *
         * [Api set: ExcelApi 1.9]
         */
        subscript: boolean;
        /**
         *
         * Represents the Superscript status of font.
            Returns True if all the fonts of the range are Superscript.
            Returns False if all the fonts of the range are Subscript or normal (neither Superscript, nor Subscript).
            Returns Null otherwise.
         *
         * [Api set: ExcelApi 1.9]
         */
        superscript: boolean;
        /**
         *
         * Returns or sets a double that lightens or darkens a color for Range Font, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            A null value indicates that the entire range doesn't have uniform font tintAndShade setting.
         *
         * [Api set: ExcelApi 1.9]
         */
        tintAndShade: number;
        /**
         *
         * Type of underline applied to the font. See Excel.RangeUnderlineStyle for details.
         *
         * [Api set: ExcelApi 1.1]
         */
        underline: Excel.RangeUnderlineStyle | "None" | "Single" | "Double" | "SingleAccountant" | "DoubleAccountant";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.RangeFont): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.RangeFontUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.RangeFont): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.RangeFont` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.RangeFont` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.RangeFont` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.RangeFontLoadOptions): Excel.RangeFont;
        load(option?: string | string[]): Excel.RangeFont;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.RangeFont;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.RangeFont object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.RangeFontData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.RangeFontData;
    }
    /**
     *
     * A collection of all the chart objects on a worksheet.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.Chart[];
        /**
         *
         * Returns the number of charts in the worksheet. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Creates a new chart.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param type Represents the type of a chart. See Excel.ChartType for details.
         * @param sourceData The Range object corresponding to the source data.
         * @param seriesBy Optional. Specifies the way columns or rows are used as data series on the chart. See Excel.ChartSeriesBy for details.
         */
        add(type: Excel.ChartType, sourceData: Range, seriesBy?: Excel.ChartSeriesBy): Excel.Chart;
        /**
         *
         * Creates a new chart.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param type Represents the type of a chart. See Excel.ChartType for details.
         * @param sourceData The Range object corresponding to the source data.
         * @param seriesBy Optional. Specifies the way columns or rows are used as data series on the chart. See Excel.ChartSeriesBy for details.
         */
        add(type: "Invalid" | "ColumnClustered" | "ColumnStacked" | "ColumnStacked100" | "3DColumnClustered" | "3DColumnStacked" | "3DColumnStacked100" | "BarClustered" | "BarStacked" | "BarStacked100" | "3DBarClustered" | "3DBarStacked" | "3DBarStacked100" | "LineStacked" | "LineStacked100" | "LineMarkers" | "LineMarkersStacked" | "LineMarkersStacked100" | "PieOfPie" | "PieExploded" | "3DPieExploded" | "BarOfPie" | "XYScatterSmooth" | "XYScatterSmoothNoMarkers" | "XYScatterLines" | "XYScatterLinesNoMarkers" | "AreaStacked" | "AreaStacked100" | "3DAreaStacked" | "3DAreaStacked100" | "DoughnutExploded" | "RadarMarkers" | "RadarFilled" | "Surface" | "SurfaceWireframe" | "SurfaceTopView" | "SurfaceTopViewWireframe" | "Bubble" | "Bubble3DEffect" | "StockHLC" | "StockOHLC" | "StockVHLC" | "StockVOHLC" | "CylinderColClustered" | "CylinderColStacked" | "CylinderColStacked100" | "CylinderBarClustered" | "CylinderBarStacked" | "CylinderBarStacked100" | "CylinderCol" | "ConeColClustered" | "ConeColStacked" | "ConeColStacked100" | "ConeBarClustered" | "ConeBarStacked" | "ConeBarStacked100" | "ConeCol" | "PyramidColClustered" | "PyramidColStacked" | "PyramidColStacked100" | "PyramidBarClustered" | "PyramidBarStacked" | "PyramidBarStacked100" | "PyramidCol" | "3DColumn" | "Line" | "3DLine" | "3DPie" | "Pie" | "XYScatter" | "3DArea" | "Area" | "Doughnut" | "Radar" | "Histogram" | "Boxwhisker" | "Pareto" | "RegionMap" | "Treemap" | "Waterfall" | "Sunburst" | "Funnel", sourceData: Range, seriesBy?: "Auto" | "Columns" | "Rows"): Excel.Chart;
        /**
         *
         * Returns the number of charts in the worksheet.
         *
         * [Api set: ExcelApi 1.4]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a chart using its name. If there are multiple charts with the same name, the first one will be returned.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param name Name of the chart to be retrieved.
         */
        getItem(name: string): Excel.Chart;
        /**
         *
         * Gets a chart based on its position in the collection.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): Excel.Chart;
        /**
         *
         * Gets a chart using its name. If there are multiple charts with the same name, the first one will be returned.
            If the chart does not exist, will return a null object.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param name Name of the chart to be retrieved.
         */
        getItemOrNullObject(name: string): Excel.Chart;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.ChartCollection;
        load(option?: string | string[]): Excel.ChartCollection;
        load(option?: OfficeExtension.LoadOption): Excel.ChartCollection;
        /**
         *
         * Occurs when a chart is activated.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @eventproperty
         */
        readonly onActivated: OfficeExtension.EventHandlers<Excel.ChartActivatedEventArgs>;
        /**
         *
         * Occurs when a new chart is added to the worksheet.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @eventproperty
         */
        readonly onAdded: OfficeExtension.EventHandlers<Excel.ChartAddedEventArgs>;
        /**
         *
         * Occurs when a chart is deactivated.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @eventproperty
         */
        readonly onDeactivated: OfficeExtension.EventHandlers<Excel.ChartDeactivatedEventArgs>;
        /**
         *
         * Occurs when a chart is deleted.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @eventproperty
         */
        readonly onDeleted: OfficeExtension.EventHandlers<Excel.ChartDeletedEventArgs>;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.ChartCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.ChartCollectionData;
    }
    /**
     *
     * Represents a chart object in a workbook.
     *
     * [Api set: ExcelApi 1.1]
     */
    class Chart extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents chart axes. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly axes: Excel.ChartAxes;
        /**
         *
         * Represents the datalabels on the chart. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly dataLabels: Excel.ChartDataLabels;
        /**
         *
         * Encapsulates the format properties for the chart area. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly format: Excel.ChartAreaFormat;
        /**
         *
         * Represents the legend for the chart. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly legend: Excel.ChartLegend;
        /**
         *
         * Encapsulates the options for a pivot chart. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly pivotOptions: Excel.ChartPivotOptions;
        /**
         *
         * Represents the plotArea for the chart.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly plotArea: Excel.ChartPlotArea;
        /**
         *
         * Represents either a single series or collection of series in the chart. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly series: Excel.ChartSeriesCollection;
        /**
         *
         * Represents the title of the specified chart, including the text, visibility, position, and formatting of the title. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly title: Excel.ChartTitle;
        /**
         *
         * The worksheet containing the current chart. Read-only.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly worksheet: Excel.Worksheet;
        /**
         *
         * Returns or sets a ChartCategoryLabelLevel enumeration constant referring to
            the level of where the category labels are being sourced from. Read/Write.
         *
         * [Api set: ExcelApi 1.8]
         */
        categoryLabelLevel: number;
        /**
         *
         * Represents the type of the chart. See Excel.ChartType for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        chartType: Excel.ChartType | "Invalid" | "ColumnClustered" | "ColumnStacked" | "ColumnStacked100" | "3DColumnClustered" | "3DColumnStacked" | "3DColumnStacked100" | "BarClustered" | "BarStacked" | "BarStacked100" | "3DBarClustered" | "3DBarStacked" | "3DBarStacked100" | "LineStacked" | "LineStacked100" | "LineMarkers" | "LineMarkersStacked" | "LineMarkersStacked100" | "PieOfPie" | "PieExploded" | "3DPieExploded" | "BarOfPie" | "XYScatterSmooth" | "XYScatterSmoothNoMarkers" | "XYScatterLines" | "XYScatterLinesNoMarkers" | "AreaStacked" | "AreaStacked100" | "3DAreaStacked" | "3DAreaStacked100" | "DoughnutExploded" | "RadarMarkers" | "RadarFilled" | "Surface" | "SurfaceWireframe" | "SurfaceTopView" | "SurfaceTopViewWireframe" | "Bubble" | "Bubble3DEffect" | "StockHLC" | "StockOHLC" | "StockVHLC" | "StockVOHLC" | "CylinderColClustered" | "CylinderColStacked" | "CylinderColStacked100" | "CylinderBarClustered" | "CylinderBarStacked" | "CylinderBarStacked100" | "CylinderCol" | "ConeColClustered" | "ConeColStacked" | "ConeColStacked100" | "ConeBarClustered" | "ConeBarStacked" | "ConeBarStacked100" | "ConeCol" | "PyramidColClustered" | "PyramidColStacked" | "PyramidColStacked100" | "PyramidBarClustered" | "PyramidBarStacked" | "PyramidBarStacked100" | "PyramidCol" | "3DColumn" | "Line" | "3DLine" | "3DPie" | "Pie" | "XYScatter" | "3DArea" | "Area" | "Doughnut" | "Radar" | "Histogram" | "Boxwhisker" | "Pareto" | "RegionMap" | "Treemap" | "Waterfall" | "Sunburst" | "Funnel";
        /**
         *
         * Returns or sets the way that blank cells are plotted on a chart. Read/Write.
         *
         * [Api set: ExcelApi 1.8]
         */
        displayBlanksAs: Excel.ChartDisplayBlanksAs | "NotPlotted" | "Zero" | "Interplotted";
        /**
         *
         * Represents the height, in points, of the chart object.
         *
         * [Api set: ExcelApi 1.1]
         */
        height: number;
        /**
         *
         * The unique id of chart. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly id: string;
        /**
         *
         * The distance, in points, from the left side of the chart to the worksheet origin.
         *
         * [Api set: ExcelApi 1.1]
         */
        left: number;
        /**
         *
         * Represents the name of a chart object.
         *
         * [Api set: ExcelApi 1.1]
         */
        name: string;
        /**
         *
         * Returns or sets the way columns or rows are used as data series on the chart. Read/Write.
         *
         * [Api set: ExcelApi 1.8]
         */
        plotBy: Excel.ChartPlotBy | "Rows" | "Columns";
        /**
         *
         * True if only visible cells are plotted. False if both visible and hidden cells are plotted. Read/Write.
         *
         * [Api set: ExcelApi 1.8]
         */
        plotVisibleOnly: boolean;
        /**
         *
         * Returns or sets a ChartSeriesNameLevel enumeration constant referring to
            the level of where the series names are being sourced from. Read/Write.
         *
         * [Api set: ExcelApi 1.8]
         */
        seriesNameLevel: number;
        /**
         *
         * Represents whether to display all field buttons on a PivotChart.
         *
         * [Api set: ExcelApi 1.7]
         */
        showAllFieldButtons: boolean;
        /**
         *
         * Represents whether to show the data labels when the value is greater than the maximum value on the value axis.
            If value axis became smaller than the size of data points, you can use this property to set whether to show the data labels.
            This property applies to 2-D charts only.
         *
         * [Api set: ExcelApi 1.8]
         */
        showDataLabelsOverMaximum: boolean;
        /**
         *
         * Returns or sets the chart style for the chart. Read/Write.
         *
         * [Api set: ExcelApi 1.8]
         */
        style: number;
        /**
         *
         * Represents the distance, in points, from the top edge of the object to the top of row 1 (on a worksheet) or the top of the chart area (on a chart).
         *
         * [Api set: ExcelApi 1.1]
         */
        top: number;
        /**
         *
         * Represents the width, in points, of the chart object.
         *
         * [Api set: ExcelApi 1.1]
         */
        width: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.Chart): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.Chart): void;
        /**
         *
         * Activates the chart in the Excel UI.
         *
         * [Api set: ExcelApi 1.9]
         */
        activate(): void;
        /**
         *
         * Deletes the chart object.
         *
         * [Api set: ExcelApi 1.1]
         */
        delete(): void;
        /**
         *
         * Renders the chart as a base64-encoded image by scaling the chart to fit the specified dimensions.
            The aspect ratio is preserved as part of the resizing.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param height (Optional) The desired height of the resulting image.
         * @param width (Optional) The desired width of the resulting image.
         * @param fittingMode (Optional) The method used to scale the chart to the specified to the specified dimensions (if both height and width are set).
         */
        getImage(width?: number, height?: number, fittingMode?: Excel.ImageFittingMode): OfficeExtension.ClientResult<string>;
        /**
         *
         * Renders the chart as a base64-encoded image by scaling the chart to fit the specified dimensions.
            The aspect ratio is preserved as part of the resizing.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param height (Optional) The desired height of the resulting image.
         * @param width (Optional) The desired width of the resulting image.
         * @param fittingMode (Optional) The method used to scale the chart to the specified to the specified dimensions (if both height and width are set).
         */
        getImage(width?: number, height?: number, fittingMode?: "Fit" | "FitAndCenter" | "Fill"): OfficeExtension.ClientResult<string>;
        /**
         *
         * Resets the source data for the chart.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param sourceData The range object corresponding to the source data.
         * @param seriesBy Specifies the way columns or rows are used as data series on the chart. Can be one of the following: Auto (default), Rows, and Columns. See Excel.ChartSeriesBy for details.
         */
        setData(sourceData: Range, seriesBy?: Excel.ChartSeriesBy): void;
        /**
         *
         * Resets the source data for the chart.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param sourceData The range object corresponding to the source data.
         * @param seriesBy Specifies the way columns or rows are used as data series on the chart. Can be one of the following: Auto (default), Rows, and Columns. See Excel.ChartSeriesBy for details.
         */
        setData(sourceData: Range, seriesBy?: "Auto" | "Columns" | "Rows"): void;
        /**
         *
         * Positions the chart relative to cells on the worksheet.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param startCell The start cell. This is where the chart will be moved to. The start cell is the top-left or top-right cell, depending on the user's right-to-left display settings.
         * @param endCell (Optional) The end cell. If specified, the chart's width and height will be set to fully cover up this cell/range.
         */
        setPosition(startCell: Range | string, endCell?: Range | string): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.Chart` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.Chart` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.Chart` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartLoadOptions): Excel.Chart;
        load(option?: string | string[]): Excel.Chart;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.Chart;
        /**
         *
         * Occurs when the chart is activated.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @eventproperty
         */
        readonly onActivated: OfficeExtension.EventHandlers<Excel.ChartActivatedEventArgs>;
        /**
         *
         * Occurs when the chart is deactivated.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @eventproperty
         */
        readonly onDeactivated: OfficeExtension.EventHandlers<Excel.ChartDeactivatedEventArgs>;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.Chart object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartData;
    }
    /**
     *
     * Encapsulates the options for the pivot chart.
     *
     * [Api set: ExcelApi 1.9]
     */
    class ChartPivotOptions extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Specifies whether or not to display the axis field buttons on a PivotChart. The ShowAxisFieldButtons property corresponds to the "Show Axis Field Buttons" command on the "Field Buttons" drop-down list of the "Analyze" tab, which is available when a PivotChart is selected.
         *
         * [Api set: ExcelApi 1.9]
         */
        showAxisFieldButtons: boolean;
        /**
         *
         * Specifies whether or not to display the legend field buttons on a PivotChart
         *
         * [Api set: ExcelApi 1.9]
         */
        showLegendFieldButtons: boolean;
        /**
         *
         * Specifies whether or not to display the report filter field buttons on a PivotChart.
         *
         * [Api set: ExcelApi 1.9]
         */
        showReportFilterFieldButtons: boolean;
        /**
         *
         * Specifies whether or not to display the show value field buttons on a PivotChart
         *
         * [Api set: ExcelApi 1.9]
         */
        showValueFieldButtons: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartPivotOptions): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartPivotOptionsUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartPivotOptions): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartPivotOptions` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartPivotOptions` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartPivotOptions` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartPivotOptionsLoadOptions): Excel.ChartPivotOptions;
        load(option?: string | string[]): Excel.ChartPivotOptions;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartPivotOptions;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartPivotOptions object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartPivotOptionsData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartPivotOptionsData;
    }
    /**
     *
     * Encapsulates the format properties for the overall chart area.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartAreaFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the border format of chart area, which includes color, linestyle, and weight. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly border: Excel.ChartBorder;
        /**
         *
         * Represents the fill format of an object, which includes background formatting information. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly fill: Excel.ChartFill;
        /**
         *
         * Represents the font attributes (font name, font size, color, etc.) for the current object. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly font: Excel.ChartFont;
        /**
         *
         * Returns or sets color scheme of the chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        colorScheme: Excel.ChartColorScheme | "ColorfulPalette1" | "ColorfulPalette2" | "ColorfulPalette3" | "ColorfulPalette4" | "MonochromaticPalette1" | "MonochromaticPalette2" | "MonochromaticPalette3" | "MonochromaticPalette4" | "MonochromaticPalette5" | "MonochromaticPalette6" | "MonochromaticPalette7" | "MonochromaticPalette8" | "MonochromaticPalette9" | "MonochromaticPalette10" | "MonochromaticPalette11" | "MonochromaticPalette12" | "MonochromaticPalette13";
        /**
         *
         * Specifies whether or not chart area of the chart has rounded corners. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        roundedCorners: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartAreaFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartAreaFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartAreaFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartAreaFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartAreaFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartAreaFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartAreaFormatLoadOptions): Excel.ChartAreaFormat;
        load(option?: string | string[]): Excel.ChartAreaFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartAreaFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartAreaFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartAreaFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartAreaFormatData;
    }
    /**
     *
     * Represents a collection of chart series.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartSeriesCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.ChartSeries[];
        /**
         *
         * Returns the number of series in the collection. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Add a new series to the collection. The new added series is not visible until set values/x axis values/bubble sizes for it (depending on chart type).
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param name Optional. Name of the series.
         * @param index Optional. Index value of the series to be added. Zero-indexed.
         */
        add(name?: string, index?: number): Excel.ChartSeries;
        /**
         *
         * Returns the number of series in the collection.
         *
         * [Api set: ExcelApi 1.4]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Retrieves a series based on its position in the collection.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): Excel.ChartSeries;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartSeriesCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartSeriesCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartSeriesCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartSeriesCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.ChartSeriesCollection;
        load(option?: string | string[]): Excel.ChartSeriesCollection;
        load(option?: OfficeExtension.LoadOption): Excel.ChartSeriesCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.ChartSeriesCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartSeriesCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.ChartSeriesCollectionData;
    }
    /**
     *
     * Represents a series in a chart.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartSeries extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Encapsulates the bin options for histogram charts and pareto charts. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly binOptions: Excel.ChartBinOptions;
        /**
         *
         * Encapsulates the options for the box and whisker charts. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly boxwhiskerOptions: Excel.ChartBoxwhiskerOptions;
        /**
         *
         * Represents a collection of all dataLabels in the series.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly dataLabels: Excel.ChartDataLabels;
        /**
         *
         * Represents the formatting of a chart series, which includes fill and line formatting. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly format: Excel.ChartSeriesFormat;
        /**
         *
         * Encapsulates the options for a region map chart. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly mapOptions: Excel.ChartMapOptions;
        /**
         *
         * Represents a collection of all points in the series. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly points: Excel.ChartPointsCollection;
        /**
         *
         * Represents a collection of trendlines in the series. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly trendlines: Excel.ChartTrendlineCollection;
        /**
         *
         * Represents the error bar object of a chart series.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly xErrorBars: Excel.ChartErrorBars;
        /**
         *
         * Represents the error bar object of a chart series.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly yErrorBars: Excel.ChartErrorBars;
        /**
         *
         * Returns or sets the group for the specified series. Read/Write
         *
         * [Api set: ExcelApi 1.8]
         */
        axisGroup: Excel.ChartAxisGroup | "Primary" | "Secondary";
        /**
         *
         * This can be an integer value from 0 (zero) to 300, representing the percentage of the default size. This property only applies to bubble charts. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        bubbleScale: number;
        /**
         *
         * Represents the chart type of a series. See Excel.ChartType for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        chartType: Excel.ChartType | "Invalid" | "ColumnClustered" | "ColumnStacked" | "ColumnStacked100" | "3DColumnClustered" | "3DColumnStacked" | "3DColumnStacked100" | "BarClustered" | "BarStacked" | "BarStacked100" | "3DBarClustered" | "3DBarStacked" | "3DBarStacked100" | "LineStacked" | "LineStacked100" | "LineMarkers" | "LineMarkersStacked" | "LineMarkersStacked100" | "PieOfPie" | "PieExploded" | "3DPieExploded" | "BarOfPie" | "XYScatterSmooth" | "XYScatterSmoothNoMarkers" | "XYScatterLines" | "XYScatterLinesNoMarkers" | "AreaStacked" | "AreaStacked100" | "3DAreaStacked" | "3DAreaStacked100" | "DoughnutExploded" | "RadarMarkers" | "RadarFilled" | "Surface" | "SurfaceWireframe" | "SurfaceTopView" | "SurfaceTopViewWireframe" | "Bubble" | "Bubble3DEffect" | "StockHLC" | "StockOHLC" | "StockVHLC" | "StockVOHLC" | "CylinderColClustered" | "CylinderColStacked" | "CylinderColStacked100" | "CylinderBarClustered" | "CylinderBarStacked" | "CylinderBarStacked100" | "CylinderCol" | "ConeColClustered" | "ConeColStacked" | "ConeColStacked100" | "ConeBarClustered" | "ConeBarStacked" | "ConeBarStacked100" | "ConeCol" | "PyramidColClustered" | "PyramidColStacked" | "PyramidColStacked100" | "PyramidBarClustered" | "PyramidBarStacked" | "PyramidBarStacked100" | "PyramidCol" | "3DColumn" | "Line" | "3DLine" | "3DPie" | "Pie" | "XYScatter" | "3DArea" | "Area" | "Doughnut" | "Radar" | "Histogram" | "Boxwhisker" | "Pareto" | "RegionMap" | "Treemap" | "Waterfall" | "Sunburst" | "Funnel";
        /**
         *
         * Represents the doughnut hole size of a chart series.  Only valid on doughnut and doughnutExploded charts.
            Throws an invalid argument exception on invalid charts.
         *
         * [Api set: ExcelApi 1.7]
         */
        doughnutHoleSize: number;
        /**
         *
         * Returns or sets the explosion value for a pie-chart or doughnut-chart slice. Returns 0 (zero) if there's no explosion (the tip of the slice is in the center of the pie). Read/Write.
         *
         * [Api set: ExcelApi 1.8]
         */
        explosion: number;
        /**
         *
         * Boolean value representing if the series is filtered or not. Not applicable for surface charts.
         *
         * [Api set: ExcelApi 1.7]
         */
        filtered: boolean;
        /**
         *
         * Returns or sets the angle of the first pie-chart or doughnut-chart slice, in degrees (clockwise from vertical). Applies only to pie, 3-D pie, and doughnut charts. Can be a value from 0 through 360. Read/Write
         *
         * [Api set: ExcelApi 1.8]
         */
        firstSliceAngle: number;
        /**
         *
         * Represents the gap width of a chart series.  Only valid on bar and column charts, as well as
            specific classes of line and pie charts.  Throws an invalid argument exception on invalid charts.
         *
         * [Api set: ExcelApi 1.7]
         */
        gapWidth: number;
        /**
         *
         * Returns or sets the color for maximum value of a region map chart series. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        gradientMaximumColor: string;
        /**
         *
         * Returns or sets the type for maximum value of a region map chart series. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        gradientMaximumType: Excel.ChartGradientStyleType | "ExtremeValue" | "Number" | "Percent";
        /**
         *
         * Returns or sets the maximum value of a region map chart series. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        gradientMaximumValue: number;
        /**
         *
         * Returns or sets the color for midpoint value of a region map chart series. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        gradientMidpointColor: string;
        /**
         *
         * Returns or sets the type for midpoint value of a region map chart series. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        gradientMidpointType: Excel.ChartGradientStyleType | "ExtremeValue" | "Number" | "Percent";
        /**
         *
         * Returns or sets the midpoint value of a region map chart series. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        gradientMidpointValue: number;
        /**
         *
         * Returns or sets the color for minimum value of a region map chart series. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        gradientMinimumColor: string;
        /**
         *
         * Returns or sets the type for minimum value of a region map chart series. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        gradientMinimumType: Excel.ChartGradientStyleType | "ExtremeValue" | "Number" | "Percent";
        /**
         *
         * Returns or sets the minimum value of a region map chart series. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        gradientMinimumValue: number;
        /**
         *
         * Returns or sets series gradient style of a region map chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        gradientStyle: Excel.ChartGradientStyle | "TwoPhaseColor" | "ThreePhaseColor";
        /**
         *
         * Boolean value representing if the series has data labels or not.
         *
         * [Api set: ExcelApi 1.7]
         */
        hasDataLabels: boolean;
        /**
         *
         * Returns or sets the fill color for negative data points in a series. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        invertColor: string;
        /**
         *
         * True if Microsoft Excel inverts the pattern in the item when it corresponds to a negative number. Read/Write.
         *
         * [Api set: ExcelApi 1.8]
         */
        invertIfNegative: boolean;
        /**
         *
         * Represents markers background color of a chart series.
         *
         * [Api set: ExcelApi 1.7]
         */
        markerBackgroundColor: string;
        /**
         *
         * Represents markers foreground color of a chart series.
         *
         * [Api set: ExcelApi 1.7]
         */
        markerForegroundColor: string;
        /**
         *
         * Represents marker size of a chart series.
         *
         * [Api set: ExcelApi 1.7]
         */
        markerSize: number;
        /**
         *
         * Represents marker style of a chart series. See Excel.ChartMarkerStyle for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        markerStyle: Excel.ChartMarkerStyle | "Invalid" | "Automatic" | "None" | "Square" | "Diamond" | "Triangle" | "X" | "Star" | "Dot" | "Dash" | "Circle" | "Plus" | "Picture";
        /**
         *
         * Represents the name of a series in a chart.
         *
         * [Api set: ExcelApi 1.1]
         */
        name: string;
        /**
         *
         * Specifies how bars and columns are positioned. Can be a value between –100 and 100. Applies only to 2-D bar and 2-D column charts. Read/Write.
         *
         * [Api set: ExcelApi 1.8]
         */
        overlap: number;
        /**
         *
         * Returns or sets the series parent label strategy area for a treemap chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        parentLabelStrategy: Excel.ChartParentLabelStrategy | "None" | "Banner" | "Overlapping";
        /**
         *
         * Represents the plot order of a chart series within the chart group.
         *
         * [Api set: ExcelApi 1.7]
         */
        plotOrder: number;
        /**
         *
         * Returns or sets the size of the secondary section of either a pie of pie chart or a bar of pie chart, as a percentage of the size of the primary pie. Can be a value from 5 to 200. Read/Write.
         *
         * [Api set: ExcelApi 1.8]
         */
        secondPlotSize: number;
        /**
         *
         * Specifies whether or not connector lines are shown in waterfall charts. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        showConnectorLines: boolean;
        /**
         *
         * Specifies whether or not leader lines are displayed for each data label in the series. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        showLeaderLines: boolean;
        /**
         *
         * Boolean value representing if the series has a shadow or not.
         *
         * [Api set: ExcelApi 1.7]
         */
        showShadow: boolean;
        /**
         *
         * Boolean value representing if the series is smooth or not. Only applicable to line and scatter charts.
         *
         * [Api set: ExcelApi 1.7]
         */
        smooth: boolean;
        /**
         *
         * Returns or sets the way the two sections of either a pie of pie chart or a bar of pie chart are split. Read/Write.
         *
         * [Api set: ExcelApi 1.8]
         */
        splitType: Excel.ChartSplitType | "SplitByPosition" | "SplitByValue" | "SplitByPercentValue" | "SplitByCustomSplit";
        /**
         *
         * Returns or sets the threshold value that separates two sections of either a pie-of-pie chart or a bar-of-pie chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        splitValue: number;
        /**
         *
         * True if Microsoft Excel assigns a different color or pattern to each data marker. The chart must contain only one series. Read/Write.
         *
         * [Api set: ExcelApi 1.8]
         */
        varyByCategories: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartSeries): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartSeriesUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartSeries): void;
        /**
         *
         * Deletes the chart series.
         *
         * [Api set: ExcelApi 1.7]
         */
        delete(): void;
        /**
         *
         * Set bubble sizes for a chart series. Only works for bubble charts.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param sourceData The Range object corresponding to the source data.
         */
        setBubbleSizes(sourceData: Range): void;
        /**
         *
         * Set values for a chart series. For scatter chart, it means Y axis values.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param sourceData The Range object corresponding to the source data.
         */
        setValues(sourceData: Range): void;
        /**
         *
         * Set values of X axis for a chart series. Only works for scatter charts.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param sourceData The Range object corresponding to the source data.
         */
        setXAxisValues(sourceData: Range): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartSeries` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartSeries` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartSeries` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartSeriesLoadOptions): Excel.ChartSeries;
        load(option?: string | string[]): Excel.ChartSeries;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartSeries;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartSeries object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartSeriesData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartSeriesData;
    }
    /**
     *
     * Encapsulates the format properties for the chart series
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartSeriesFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the fill format of a chart series, which includes background formatting information. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly fill: Excel.ChartFill;
        /**
         *
         * Represents line formatting. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly line: Excel.ChartLineFormat;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartSeriesFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartSeriesFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartSeriesFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartSeriesFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartSeriesFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartSeriesFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartSeriesFormatLoadOptions): Excel.ChartSeriesFormat;
        load(option?: string | string[]): Excel.ChartSeriesFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartSeriesFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartSeriesFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartSeriesFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartSeriesFormatData;
    }
    /**
     *
     * A collection of all the chart points within a series inside a chart.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartPointsCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.ChartPoint[];
        /**
         *
         * Returns the number of chart points in the series. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Returns the number of chart points in the series.
         *
         * [Api set: ExcelApi 1.4]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Retrieve a point based on its position within the series.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): Excel.ChartPoint;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartPointsCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartPointsCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartPointsCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartPointsCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.ChartPointsCollection;
        load(option?: string | string[]): Excel.ChartPointsCollection;
        load(option?: OfficeExtension.LoadOption): Excel.ChartPointsCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.ChartPointsCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartPointsCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.ChartPointsCollectionData;
    }
    /**
     *
     * Represents a point of a series in a chart.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartPoint extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns the data label of a chart point. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly dataLabel: Excel.ChartDataLabel;
        /**
         *
         * Encapsulates the format properties chart point. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly format: Excel.ChartPointFormat;
        /**
         *
         * Represents whether a data point has a data label. Not applicable for surface charts.
         *
         * [Api set: ExcelApi 1.7]
         */
        hasDataLabel: boolean;
        /**
         *
         * HTML color code representation of the marker background color of data point. E.g. #FF0000 represents Red.
         *
         * [Api set: ExcelApi 1.7]
         */
        markerBackgroundColor: string;
        /**
         *
         * HTML color code representation of the marker foreground color of data point. E.g. #FF0000 represents Red.
         *
         * [Api set: ExcelApi 1.7]
         */
        markerForegroundColor: string;
        /**
         *
         * Represents marker size of data point.
         *
         * [Api set: ExcelApi 1.7]
         */
        markerSize: number;
        /**
         *
         * Represents marker style of a chart data point. See Excel.ChartMarkerStyle for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        markerStyle: Excel.ChartMarkerStyle | "Invalid" | "Automatic" | "None" | "Square" | "Diamond" | "Triangle" | "X" | "Star" | "Dot" | "Dash" | "Circle" | "Plus" | "Picture";
        /**
         *
         * Returns the value of a chart point. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly value: any;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartPoint): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartPointUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartPoint): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartPoint` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartPoint` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartPoint` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartPointLoadOptions): Excel.ChartPoint;
        load(option?: string | string[]): Excel.ChartPoint;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartPoint;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartPoint object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartPointData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartPointData;
    }
    /**
     *
     * Represents formatting object for chart points.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartPointFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the border format of a chart data point, which includes color, style, and weight information. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly border: Excel.ChartBorder;
        /**
         *
         * Represents the fill format of a chart, which includes background formatting information. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly fill: Excel.ChartFill;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartPointFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartPointFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartPointFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartPointFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartPointFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartPointFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartPointFormatLoadOptions): Excel.ChartPointFormat;
        load(option?: string | string[]): Excel.ChartPointFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartPointFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartPointFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartPointFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartPointFormatData;
    }
    /**
     *
     * Represents the chart axes.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartAxes extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the category axis in a chart. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly categoryAxis: Excel.ChartAxis;
        /**
         *
         * Represents the series axis of a 3-dimensional chart. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly seriesAxis: Excel.ChartAxis;
        /**
         *
         * Represents the value axis in an axis. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly valueAxis: Excel.ChartAxis;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartAxes): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartAxesUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartAxes): void;
        /**
         *
         * Returns the specific axis identified by type and group.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param type Specifies the axis type. See Excel.ChartAxisType for details.
         * @param group Optional. Specifies the axis group. See Excel.ChartAxisGroup for details.
         */
        getItem(type: Excel.ChartAxisType, group?: Excel.ChartAxisGroup): Excel.ChartAxis;
        /**
         *
         * Returns the specific axis identified by type and group.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param type Specifies the axis type. See Excel.ChartAxisType for details.
         * @param group Optional. Specifies the axis group. See Excel.ChartAxisGroup for details.
         */
        getItem(type: "Invalid" | "Category" | "Value" | "Series", group?: "Primary" | "Secondary"): Excel.ChartAxis;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartAxes` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartAxes` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartAxes` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartAxesLoadOptions): Excel.ChartAxes;
        load(option?: string | string[]): Excel.ChartAxes;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartAxes;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartAxes object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartAxesData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartAxesData;
    }
    /**
     *
     * Represents a single axis in a chart.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartAxis extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the formatting of a chart object, which includes line and font formatting. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly format: Excel.ChartAxisFormat;
        /**
         *
         * Returns a Gridlines object that represents the major gridlines for the specified axis. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly majorGridlines: Excel.ChartGridlines;
        /**
         *
         * Returns a Gridlines object that represents the minor gridlines for the specified axis. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly minorGridlines: Excel.ChartGridlines;
        /**
         *
         * Represents the axis title. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly title: Excel.ChartAxisTitle;
        /**
         *
         * Represents the alignment for the specified axis tick label. See Excel.ChartTextHorizontalAlignment for detail.
         *
         * [Api set: ExcelApi 1.8]
         */
        alignment: Excel.ChartTickLabelAlignment | "Center" | "Left" | "Right";
        /**
         *
         * Represents the group for the specified axis. See Excel.ChartAxisGroup for details. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly axisGroup: Excel.ChartAxisGroup | "Primary" | "Secondary";
        /**
         *
         * Returns or sets the base unit for the specified category axis.
         *
         * [Api set: ExcelApi 1.7]
         */
        baseTimeUnit: Excel.ChartAxisTimeUnit | "Days" | "Months" | "Years";
        /**
         *
         * Returns or sets the category axis type.
         *
         * [Api set: ExcelApi 1.7]
         */
        categoryType: Excel.ChartAxisCategoryType | "Automatic" | "TextAxis" | "DateAxis";
        /**
         *
         * Represents the custom axis display unit value. Read-only. To set this property, please use the SetCustomDisplayUnit(double) method.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly customDisplayUnit: number;
        /**
         *
         * Represents the axis display unit. See Excel.ChartAxisDisplayUnit for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        displayUnit: Excel.ChartAxisDisplayUnit | "None" | "Hundreds" | "Thousands" | "TenThousands" | "HundredThousands" | "Millions" | "TenMillions" | "HundredMillions" | "Billions" | "Trillions" | "Custom";
        /**
         *
         * Represents the height, in points, of the chart axis. Null if the axis is not visible. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly height: number;
        /**
         *
         * Represents whether value axis crosses the category axis between categories.
         *
         * [Api set: ExcelApi 1.8]
         */
        isBetweenCategories: boolean;
        /**
         *
         * Represents the distance, in points, from the left edge of the axis to the left of chart area. Null if the axis is not visible. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly left: number;
        /**
         *
         * Represents whether or not the number format is linked to the cells. If true, the number format will change in the labels when it changes in the cells.
         *
         * [Api set: ExcelApi 1.9]
         */
        linkNumberFormat: boolean;
        /**
         *
         * Represents the base of the logarithm when using logarithmic scales.
         *
         * [Api set: ExcelApi 1.7]
         */
        logBase: number;
        /**
         *
         * Represents the type of major tick mark for the specified axis. See Excel.ChartAxisTickMark for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        majorTickMark: Excel.ChartAxisTickMark | "None" | "Cross" | "Inside" | "Outside";
        /**
         *
         * Returns or sets the major unit scale value for the category axis when the CategoryType property is set to TimeScale.
         *
         * [Api set: ExcelApi 1.7]
         */
        majorTimeUnitScale: Excel.ChartAxisTimeUnit | "Days" | "Months" | "Years";
        /**
         *
         * Represents the interval between two major tick marks. Can be set to a numeric value or an empty string.  The returned value is always a number.
         *
         * [Api set: ExcelApi 1.1]
         */
        majorUnit: any;
        /**
         *
         * Represents the maximum value on the value axis.  Can be set to a numeric value or an empty string (for automatic axis values).  The returned value is always a number.
         *
         * [Api set: ExcelApi 1.1]
         */
        maximum: any;
        /**
         *
         * Represents the minimum value on the value axis. Can be set to a numeric value or an empty string (for automatic axis values).  The returned value is always a number.
         *
         * [Api set: ExcelApi 1.1]
         */
        minimum: any;
        /**
         *
         * Represents the type of minor tick mark for the specified axis. See Excel.ChartAxisTickMark for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        minorTickMark: Excel.ChartAxisTickMark | "None" | "Cross" | "Inside" | "Outside";
        /**
         *
         * Returns or sets the minor unit scale value for the category axis when the CategoryType property is set to TimeScale.
         *
         * [Api set: ExcelApi 1.7]
         */
        minorTimeUnitScale: Excel.ChartAxisTimeUnit | "Days" | "Months" | "Years";
        /**
         *
         * Represents the interval between two minor tick marks. Can be set to a numeric value or an empty string (for automatic axis values). The returned value is always a number.
         *
         * [Api set: ExcelApi 1.1]
         */
        minorUnit: any;
        /**
         *
         * Represents whether an axis is multilevel or not.
         *
         * [Api set: ExcelApi 1.8]
         */
        multiLevel: boolean;
        /**
         *
         * Represents the format code for the axis tick label.
         *
         * [Api set: ExcelApi 1.8]
         */
        numberFormat: string;
        /**
         *
         * Represents the distance between the levels of labels, and the distance between the first level and the axis line. The value should be an integer from 0 to 1000.
         *
         * [Api set: ExcelApi 1.8]
         */
        offset: number;
        /**
         *
         * Represents the specified axis position where the other axis crosses. See Excel.ChartAxisPosition for details.
         *
         * [Api set: ExcelApi 1.8]
         */
        position: Excel.ChartAxisPosition | "Automatic" | "Maximum" | "Minimum" | "Custom";
        /**
         *
         * Represents the specified axis position where the other axis crosses at. You should use the SetPositionAt(double) method to set this property.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly positionAt: number;
        /**
         *
         * Represents whether Microsoft Excel plots data points from last to first.
         *
         * [Api set: ExcelApi 1.7]
         */
        reversePlotOrder: boolean;
        /**
         *
         * Represents the value axis scale type. See Excel.ChartAxisScaleType for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        scaleType: Excel.ChartAxisScaleType | "Linear" | "Logarithmic";
        /**
         *
         * Represents whether the axis display unit label is visible.
         *
         * [Api set: ExcelApi 1.7]
         */
        showDisplayUnitLabel: boolean;
        /**
         *
         * Represents the text orientation of the axis tick label. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
         *
         * [Api set: ExcelApi 1.8]
         */
        textOrientation: any;
        /**
         *
         * Represents the position of tick-mark labels on the specified axis. See Excel.ChartAxisTickLabelPosition for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        tickLabelPosition: Excel.ChartAxisTickLabelPosition | "NextToAxis" | "High" | "Low" | "None";
        /**
         *
         * Represents the number of categories or series between tick-mark labels. Can be a value from 1 through 31999 or an empty string for automatic setting. The returned value is always a number.
         *
         * [Api set: ExcelApi 1.7]
         */
        tickLabelSpacing: any;
        /**
         *
         * Represents the number of categories or series between tick marks.
         *
         * [Api set: ExcelApi 1.7]
         */
        tickMarkSpacing: number;
        /**
         *
         * Represents the distance, in points, from the top edge of the axis to the top of chart area. Null if the axis is not visible. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly top: number;
        /**
         *
         * Represents the axis type. See Excel.ChartAxisType for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly type: Excel.ChartAxisType | "Invalid" | "Category" | "Value" | "Series";
        /**
         *
         * A boolean value represents the visibility of the axis.
         *
         * [Api set: ExcelApi 1.7]
         */
        visible: boolean;
        /**
         *
         * Represents the width, in points, of the chart axis. Null if the axis is not visible. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly width: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartAxis): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartAxisUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartAxis): void;
        /**
         *
         * Sets all the category names for the specified axis.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param sourceData The Range object corresponding to the source data.
         */
        setCategoryNames(sourceData: Range): void;
        /**
         *
         * Sets the axis display unit to a custom value.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param value Custom value of the display unit.
         */
        setCustomDisplayUnit(value: number): void;
        /**
         *
         * Set the specified axis position where the other axis crosses at.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param value Custom value of the crosses at
         */
        setPositionAt(value: number): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartAxis` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartAxis` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartAxis` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartAxisLoadOptions): Excel.ChartAxis;
        load(option?: string | string[]): Excel.ChartAxis;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartAxis;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartAxis object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartAxisData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartAxisData;
    }
    /**
     *
     * Encapsulates the format properties for the chart axis.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartAxisFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents chart fill formatting. Read-only.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly fill: Excel.ChartFill;
        /**
         *
         * Represents the font attributes (font name, font size, color, etc.) for a chart axis element. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly font: Excel.ChartFont;
        /**
         *
         * Represents chart line formatting. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly line: Excel.ChartLineFormat;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartAxisFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartAxisFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartAxisFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartAxisFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartAxisFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartAxisFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartAxisFormatLoadOptions): Excel.ChartAxisFormat;
        load(option?: string | string[]): Excel.ChartAxisFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartAxisFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartAxisFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartAxisFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartAxisFormatData;
    }
    /**
     *
     * Represents the title of a chart axis.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartAxisTitle extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the formatting of chart axis title. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly format: Excel.ChartAxisTitleFormat;
        /**
         *
         * Represents the axis title.
         *
         * [Api set: ExcelApi 1.1]
         */
        text: string;
        /**
         *
         * A boolean that specifies the visibility of an axis title.
         *
         * [Api set: ExcelApi 1.1]
         */
        visible: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartAxisTitle): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartAxisTitleUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartAxisTitle): void;
        /**
         *
         * A string value that represents the formula of chart axis title using A1-style notation.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param formula a string that present the formula to set
         */
        setFormula(formula: string): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartAxisTitle` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartAxisTitle` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartAxisTitle` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartAxisTitleLoadOptions): Excel.ChartAxisTitle;
        load(option?: string | string[]): Excel.ChartAxisTitle;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartAxisTitle;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartAxisTitle object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartAxisTitleData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartAxisTitleData;
    }
    /**
     *
     * Represents the chart axis title formatting.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartAxisTitleFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the border format, which includes color, linestyle, and weight.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly border: Excel.ChartBorder;
        /**
         *
         * Represents chart fill formatting.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly fill: Excel.ChartFill;
        /**
         *
         * Represents the font attributes, such as font name, font size, color, etc. of chart axis title object. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly font: Excel.ChartFont;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartAxisTitleFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartAxisTitleFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartAxisTitleFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartAxisTitleFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartAxisTitleFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartAxisTitleFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartAxisTitleFormatLoadOptions): Excel.ChartAxisTitleFormat;
        load(option?: string | string[]): Excel.ChartAxisTitleFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartAxisTitleFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartAxisTitleFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartAxisTitleFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartAxisTitleFormatData;
    }
    /**
     *
     * Represents a collection of all the data labels on a chart point.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartDataLabels extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the format of chart data labels, which includes fill and font formatting. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly format: Excel.ChartDataLabelFormat;
        /**
         *
         * Represents whether data labels automatically generate appropriate text based on context.
         *
         * [Api set: ExcelApi 1.8]
         */
        autoText: boolean;
        /**
         *
         * Represents the horizontal alignment for chart data label. See Excel.ChartTextHorizontalAlignment for details.
            This property is valid only when TextOrientation of data label is 0.
         *
         * [Api set: ExcelApi 1.8]
         */
        horizontalAlignment: Excel.ChartTextHorizontalAlignment | "Center" | "Left" | "Right" | "Justify" | "Distributed";
        /**
         *
         * Represents whether or not the number format is linked to the cells. If true, the number format will change in the labels when it changes in the cells
         *
         * [Api set: ExcelApi 1.9]
         */
        linkNumberFormat: boolean;
        /**
         *
         * Represents the format code for data labels.
         *
         * [Api set: ExcelApi 1.8]
         */
        numberFormat: string;
        /**
         *
         * DataLabelPosition value that represents the position of the data label. See Excel.ChartDataLabelPosition for details.
         *
         * [Api set: ExcelApi 1.1]
         */
        position: Excel.ChartDataLabelPosition | "Invalid" | "None" | "Center" | "InsideEnd" | "InsideBase" | "OutsideEnd" | "Left" | "Right" | "Top" | "Bottom" | "BestFit" | "Callout";
        /**
         *
         * String representing the separator used for the data labels on a chart.
         *
         * [Api set: ExcelApi 1.1]
         */
        separator: string;
        /**
         *
         * Boolean value representing if the data label bubble size is visible or not.
         *
         * [Api set: ExcelApi 1.1]
         */
        showBubbleSize: boolean;
        /**
         *
         * Boolean value representing if the data label category name is visible or not.
         *
         * [Api set: ExcelApi 1.1]
         */
        showCategoryName: boolean;
        /**
         *
         * Boolean value representing if the data label legend key is visible or not.
         *
         * [Api set: ExcelApi 1.1]
         */
        showLegendKey: boolean;
        /**
         *
         * Boolean value representing if the data label percentage is visible or not.
         *
         * [Api set: ExcelApi 1.1]
         */
        showPercentage: boolean;
        /**
         *
         * Boolean value representing if the data label series name is visible or not.
         *
         * [Api set: ExcelApi 1.1]
         */
        showSeriesName: boolean;
        /**
         *
         * Boolean value representing if the data label value is visible or not.
         *
         * [Api set: ExcelApi 1.1]
         */
        showValue: boolean;
        /**
         *
         * Represents the text orientation of data labels. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
         *
         * [Api set: ExcelApi 1.8]
         */
        textOrientation: number;
        /**
         *
         * Represents the vertical alignment of chart data label. See Excel.ChartTextVerticalAlignment for details.
            This property is valid only when TextOrientation of data label is 90, -90 or 180.
         *
         * [Api set: ExcelApi 1.8]
         */
        verticalAlignment: Excel.ChartTextVerticalAlignment | "Center" | "Bottom" | "Top" | "Justify" | "Distributed";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartDataLabels): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartDataLabelsUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartDataLabels): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartDataLabels` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartDataLabels` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartDataLabels` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartDataLabelsLoadOptions): Excel.ChartDataLabels;
        load(option?: string | string[]): Excel.ChartDataLabels;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartDataLabels;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartDataLabels object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartDataLabelsData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartDataLabelsData;
    }
    /**
     *
     * Represents the data label of a chart point.
     *
     * [Api set: ExcelApi 1.7]
     */
    class ChartDataLabel extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the format of chart data label.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly format: Excel.ChartDataLabelFormat;
        /**
         *
         * Boolean value representing if data label automatically generates appropriate text based on context.
         *
         * [Api set: ExcelApi 1.8]
         */
        autoText: boolean;
        /**
         *
         * String value that represents the formula of chart data label using A1-style notation.
         *
         * [Api set: ExcelApi 1.8]
         */
        formula: string;
        /**
         *
         * Returns the height, in points, of the chart data label. Read-only. Null if chart data label is not visible.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly height: number;
        /**
         *
         * Represents the horizontal alignment for chart data label. See Excel.ChartTextHorizontalAlignment for details.
            This property is valid only when TextOrientation of data label is 90, -90 or 180.
         *
         * [Api set: ExcelApi 1.8]
         */
        horizontalAlignment: Excel.ChartTextHorizontalAlignment | "Center" | "Left" | "Right" | "Justify" | "Distributed";
        /**
         *
         * Represents the distance, in points, from the left edge of chart data label to the left edge of chart area. Null if chart data label is not visible.
         *
         * [Api set: ExcelApi 1.8]
         */
        left: number;
        /**
         *
         * Boolean value representing if the number format is linked to the cells (so that the number format changes in the labels when it changes in the cells).
         *
         * [Api set: ExcelApi 1.9]
         */
        linkNumberFormat: boolean;
        /**
         *
         * String value that represents the format code for data label.
         *
         * [Api set: ExcelApi 1.8]
         */
        numberFormat: string;
        /**
         *
         * DataLabelPosition value that represents the position of the data label. See Excel.ChartDataLabelPosition for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        position: Excel.ChartDataLabelPosition | "Invalid" | "None" | "Center" | "InsideEnd" | "InsideBase" | "OutsideEnd" | "Left" | "Right" | "Top" | "Bottom" | "BestFit" | "Callout";
        /**
         *
         * String representing the separator used for the data label on a chart.
         *
         * [Api set: ExcelApi 1.7]
         */
        separator: string;
        /**
         *
         * Boolean value representing if the data label bubble size is visible or not.
         *
         * [Api set: ExcelApi 1.7]
         */
        showBubbleSize: boolean;
        /**
         *
         * Boolean value representing if the data label category name is visible or not.
         *
         * [Api set: ExcelApi 1.7]
         */
        showCategoryName: boolean;
        /**
         *
         * Boolean value representing if the data label legend key is visible or not.
         *
         * [Api set: ExcelApi 1.7]
         */
        showLegendKey: boolean;
        /**
         *
         * Boolean value representing if the data label percentage is visible or not.
         *
         * [Api set: ExcelApi 1.7]
         */
        showPercentage: boolean;
        /**
         *
         * Boolean value representing if the data label series name is visible or not.
         *
         * [Api set: ExcelApi 1.7]
         */
        showSeriesName: boolean;
        /**
         *
         * Boolean value representing if the data label value is visible or not.
         *
         * [Api set: ExcelApi 1.7]
         */
        showValue: boolean;
        /**
         *
         * String representing the text of the data label on a chart.
         *
         * [Api set: ExcelApi 1.8]
         */
        text: string;
        /**
         *
         * Represents the text orientation of chart data label. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
         *
         * [Api set: ExcelApi 1.8]
         */
        textOrientation: number;
        /**
         *
         * Represents the distance, in points, from the top edge of chart data label to the top of chart area. Null if chart data label is not visible.
         *
         * [Api set: ExcelApi 1.8]
         */
        top: number;
        /**
         *
         * Represents the vertical alignment of chart data label. See Excel.ChartTextVerticalAlignment for details.
            This property is valid only when TextOrientation of data label is 0.
         *
         * [Api set: ExcelApi 1.8]
         */
        verticalAlignment: Excel.ChartTextVerticalAlignment | "Center" | "Bottom" | "Top" | "Justify" | "Distributed";
        /**
         *
         * Returns the width, in points, of the chart data label. Read-only. Null if chart data label is not visible.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly width: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartDataLabel): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartDataLabelUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartDataLabel): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartDataLabel` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartDataLabel` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartDataLabel` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartDataLabelLoadOptions): Excel.ChartDataLabel;
        load(option?: string | string[]): Excel.ChartDataLabel;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartDataLabel;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartDataLabel object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartDataLabelData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartDataLabelData;
    }
    /**
     *
     * Encapsulates the format properties for the chart data labels.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartDataLabelFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the border format, which includes color, linestyle, and weight. Read-only.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly border: Excel.ChartBorder;
        /**
         *
         * Represents the fill format of the current chart data label. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly fill: Excel.ChartFill;
        /**
         *
         * Represents the font attributes (font name, font size, color, etc.) for a chart data label. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly font: Excel.ChartFont;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartDataLabelFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartDataLabelFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartDataLabelFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartDataLabelFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartDataLabelFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartDataLabelFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartDataLabelFormatLoadOptions): Excel.ChartDataLabelFormat;
        load(option?: string | string[]): Excel.ChartDataLabelFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartDataLabelFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartDataLabelFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartDataLabelFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartDataLabelFormatData;
    }
    /**
     *
     * This object represents the attributes for a chart's error bars.
     *
     * [Api set: ExcelApi 1.9]
     */
    class ChartErrorBars extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Specifies the formatting type of the error bars.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly format: Excel.ChartErrorBarsFormat;
        /**
         *
         * Specifies whether or not the error bars have an end style cap.
         *
         * [Api set: ExcelApi 1.9]
         */
        endStyleCap: boolean;
        /**
         *
         * Specifies which parts of the error bars to include.
         *
         * [Api set: ExcelApi 1.9]
         */
        include: Excel.ChartErrorBarsInclude | "Both" | "MinusValues" | "PlusValues";
        /**
         *
         * The type of range marked by the error bars.
         *
         * [Api set: ExcelApi 1.9]
         */
        type: Excel.ChartErrorBarsType | "FixedValue" | "Percent" | "StDev" | "StError" | "Custom";
        /**
         *
         * Specifies whether or not the error bars are displayed.
         *
         * [Api set: ExcelApi 1.9]
         */
        visible: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartErrorBars): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartErrorBarsUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartErrorBars): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartErrorBars` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartErrorBars` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartErrorBars` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartErrorBarsLoadOptions): Excel.ChartErrorBars;
        load(option?: string | string[]): Excel.ChartErrorBars;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartErrorBars;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartErrorBars object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartErrorBarsData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartErrorBarsData;
    }
    /**
     *
     * Encapsulates the format properties for chart error bars.
     *
     * [Api set: ExcelApi 1.9]
     */
    class ChartErrorBarsFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the chart line formatting.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly line: Excel.ChartLineFormat;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartErrorBarsFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartErrorBarsFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartErrorBarsFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartErrorBarsFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartErrorBarsFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartErrorBarsFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartErrorBarsFormatLoadOptions): Excel.ChartErrorBarsFormat;
        load(option?: string | string[]): Excel.ChartErrorBarsFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartErrorBarsFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartErrorBarsFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartErrorBarsFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartErrorBarsFormatData;
    }
    /**
     *
     * Represents major or minor gridlines on a chart axis.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartGridlines extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the formatting of chart gridlines. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly format: Excel.ChartGridlinesFormat;
        /**
         *
         * Boolean value representing if the axis gridlines are visible or not.
         *
         * [Api set: ExcelApi 1.1]
         */
        visible: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartGridlines): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartGridlinesUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartGridlines): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartGridlines` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartGridlines` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartGridlines` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartGridlinesLoadOptions): Excel.ChartGridlines;
        load(option?: string | string[]): Excel.ChartGridlines;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartGridlines;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartGridlines object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartGridlinesData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartGridlinesData;
    }
    /**
     *
     * Encapsulates the format properties for chart gridlines.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartGridlinesFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents chart line formatting. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly line: Excel.ChartLineFormat;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartGridlinesFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartGridlinesFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartGridlinesFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartGridlinesFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartGridlinesFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartGridlinesFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartGridlinesFormatLoadOptions): Excel.ChartGridlinesFormat;
        load(option?: string | string[]): Excel.ChartGridlinesFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartGridlinesFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartGridlinesFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartGridlinesFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartGridlinesFormatData;
    }
    /**
     *
     * Represents the legend in a chart.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartLegend extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the formatting of a chart legend, which includes fill and font formatting. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly format: Excel.ChartLegendFormat;
        /**
         *
         * Represents a collection of legendEntries in the legend. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly legendEntries: Excel.ChartLegendEntryCollection;
        /**
         *
         * Represents the height, in points, of the legend on the chart. Null if legend is not visible.
         *
         * [Api set: ExcelApi 1.7]
         */
        height: number;
        /**
         *
         * Represents the left, in points, of a chart legend. Null if legend is not visible.
         *
         * [Api set: ExcelApi 1.7]
         */
        left: number;
        /**
         *
         * Boolean value for whether the chart legend should overlap with the main body of the chart.
         *
         * [Api set: ExcelApi 1.1]
         */
        overlay: boolean;
        /**
         *
         * Represents the position of the legend on the chart. See Excel.ChartLegendPosition for details.
         *
         * [Api set: ExcelApi 1.1]
         */
        position: Excel.ChartLegendPosition | "Invalid" | "Top" | "Bottom" | "Left" | "Right" | "Corner" | "Custom";
        /**
         *
         * Represents if the legend has a shadow on the chart.
         *
         * [Api set: ExcelApi 1.7]
         */
        showShadow: boolean;
        /**
         *
         * Represents the top of a chart legend.
         *
         * [Api set: ExcelApi 1.7]
         */
        top: number;
        /**
         *
         * A boolean value the represents the visibility of a ChartLegend object.
         *
         * [Api set: ExcelApi 1.1]
         */
        visible: boolean;
        /**
         *
         * Represents the width, in points, of the legend on the chart. Null if legend is not visible.
         *
         * [Api set: ExcelApi 1.7]
         */
        width: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartLegend): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartLegendUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartLegend): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartLegend` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartLegend` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartLegend` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartLegendLoadOptions): Excel.ChartLegend;
        load(option?: string | string[]): Excel.ChartLegend;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartLegend;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartLegend object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartLegendData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartLegendData;
    }
    /**
     *
     * Represents the legendEntry in legendEntryCollection.
     *
     * [Api set: ExcelApi 1.7]
     */
    class ChartLegendEntry extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the height of the legendEntry on the chart legend.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly height: number;
        /**
         *
         * Represents the index of the legendEntry in the chart legend.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly index: number;
        /**
         *
         * Represents the left of a chart legendEntry.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly left: number;
        /**
         *
         * Represents the top of a chart legendEntry.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly top: number;
        /**
         *
         * Represents the visible of a chart legend entry.
         *
         * [Api set: ExcelApi 1.7]
         */
        visible: boolean;
        /**
         *
         * Represents the width of the legendEntry on the chart Legend.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly width: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartLegendEntry): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartLegendEntryUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartLegendEntry): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartLegendEntry` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartLegendEntry` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartLegendEntry` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartLegendEntryLoadOptions): Excel.ChartLegendEntry;
        load(option?: string | string[]): Excel.ChartLegendEntry;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartLegendEntry;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartLegendEntry object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartLegendEntryData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartLegendEntryData;
    }
    /**
     *
     * Represents a collection of legendEntries.
     *
     * [Api set: ExcelApi 1.7]
     */
    class ChartLegendEntryCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.ChartLegendEntry[];
        /**
         *
         * Returns the number of legendEntry in the collection.
         *
         * [Api set: ExcelApi 1.7]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Returns a legendEntry at the given index.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param index Index of the legendEntry to be retrieved.
         */
        getItemAt(index: number): Excel.ChartLegendEntry;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartLegendEntryCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartLegendEntryCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartLegendEntryCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartLegendEntryCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.ChartLegendEntryCollection;
        load(option?: string | string[]): Excel.ChartLegendEntryCollection;
        load(option?: OfficeExtension.LoadOption): Excel.ChartLegendEntryCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.ChartLegendEntryCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartLegendEntryCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.ChartLegendEntryCollectionData;
    }
    /**
     *
     * Encapsulates the format properties of a chart legend.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartLegendFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the border format, which includes color, linestyle, and weight. Read-only.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly border: Excel.ChartBorder;
        /**
         *
         * Represents the fill format of an object, which includes background formatting information. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly fill: Excel.ChartFill;
        /**
         *
         * Represents the font attributes such as font name, font size, color, etc. of a chart legend. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly font: Excel.ChartFont;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartLegendFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartLegendFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartLegendFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartLegendFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartLegendFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartLegendFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartLegendFormatLoadOptions): Excel.ChartLegendFormat;
        load(option?: string | string[]): Excel.ChartLegendFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartLegendFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartLegendFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartLegendFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartLegendFormatData;
    }
    /**
     *
     * Encapsulates the properties for a region map chart.
     *
     * [Api set: ExcelApi 1.9]
     */
    class ChartMapOptions extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns or sets the series map labels strategy of a region map chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        labelStrategy: Excel.ChartMapLabelStrategy | "None" | "BestFit" | "ShowAll";
        /**
         *
         * Returns or sets the series mapping level of a region map chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        level: Excel.ChartMapAreaLevel | "Automatic" | "DataOnly" | "City" | "County" | "State" | "Country" | "Continent" | "World";
        /**
         *
         * Returns or sets the series projection type of a region map chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        projectionType: Excel.ChartMapProjectionType | "Automatic" | "Mercator" | "Miller" | "Robinson" | "Albers";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartMapOptions): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartMapOptionsUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartMapOptions): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartMapOptions` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartMapOptions` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartMapOptions` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartMapOptionsLoadOptions): Excel.ChartMapOptions;
        load(option?: string | string[]): Excel.ChartMapOptions;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartMapOptions;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartMapOptions object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartMapOptionsData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartMapOptionsData;
    }
    /**
     *
     * Represents a chart title object of a chart.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartTitle extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the formatting of a chart title, which includes fill and font formatting. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly format: Excel.ChartTitleFormat;
        /**
         *
         * Returns the height, in points, of the chart title. Null if chart title is not visible. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly height: number;
        /**
         *
         * Represents the horizontal alignment for chart title.
         *
         * [Api set: ExcelApi 1.7]
         */
        horizontalAlignment: Excel.ChartTextHorizontalAlignment | "Center" | "Left" | "Right" | "Justify" | "Distributed";
        /**
         *
         * Represents the distance, in points, from the left edge of chart title to the left edge of chart area. Null if chart title is not visible.
         *
         * [Api set: ExcelApi 1.7]
         */
        left: number;
        /**
         *
         * Boolean value representing if the chart title will overlay the chart or not.
         *
         * [Api set: ExcelApi 1.1]
         */
        overlay: boolean;
        /**
         *
         * Represents the position of chart title. See Excel.ChartTitlePosition for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        position: Excel.ChartTitlePosition | "Automatic" | "Top" | "Bottom" | "Left" | "Right";
        /**
         *
         * Represents a boolean value that determines if the chart title has a shadow.
         *
         * [Api set: ExcelApi 1.7]
         */
        showShadow: boolean;
        /**
         *
         * Represents the title text of a chart.
         *
         * [Api set: ExcelApi 1.1]
         */
        text: string;
        /**
         *
         * Represents the text orientation of chart title. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
         *
         * [Api set: ExcelApi 1.7]
         */
        textOrientation: number;
        /**
         *
         * Represents the distance, in points, from the top edge of chart title to the top of chart area. Null if chart title is not visible.
         *
         * [Api set: ExcelApi 1.7]
         */
        top: number;
        /**
         *
         * Represents the vertical alignment of chart title. See Excel.ChartTextVerticalAlignment for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        verticalAlignment: Excel.ChartTextVerticalAlignment | "Center" | "Bottom" | "Top" | "Justify" | "Distributed";
        /**
         *
         * A boolean value the represents the visibility of a chart title object.
         *
         * [Api set: ExcelApi 1.1]
         */
        visible: boolean;
        /**
         *
         * Returns the width, in points, of the chart title. Null if chart title is not visible. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly width: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartTitle): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartTitleUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartTitle): void;
        /**
         *
         * Get the substring of a chart title. Line break '\n' also counts one character.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param start Start position of substring to be retrieved. Position start with 0.
         * @param length Length of substring to be retrieved.
         */
        getSubstring(start: number, length: number): Excel.ChartFormatString;
        /**
         *
         * Sets a string value that represents the formula of chart title using A1-style notation.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param formula A string that represents the formula to set.
         */
        setFormula(formula: string): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartTitle` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartTitle` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartTitle` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartTitleLoadOptions): Excel.ChartTitle;
        load(option?: string | string[]): Excel.ChartTitle;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartTitle;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartTitle object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartTitleData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartTitleData;
    }
    /**
     *
     * Represents the substring in chart related objects that contains text, like ChartTitle object, ChartAxisTitle object, etc.
     *
     * [Api set: ExcelApi 1.7]
     */
    class ChartFormatString extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the font attributes, such as font name, font size, color, etc. of chart characters object.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly font: Excel.ChartFont;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartFormatString): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartFormatStringUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartFormatString): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartFormatString` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartFormatString` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartFormatString` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartFormatStringLoadOptions): Excel.ChartFormatString;
        load(option?: string | string[]): Excel.ChartFormatString;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartFormatString;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartFormatString object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartFormatStringData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartFormatStringData;
    }
    /**
     *
     * Provides access to the office art formatting for chart title.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartTitleFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the border format of chart title, which includes color, linestyle, and weight. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly border: Excel.ChartBorder;
        /**
         *
         * Represents the fill format of an object, which includes background formatting information. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly fill: Excel.ChartFill;
        /**
         *
         * Represents the font attributes (font name, font size, color, etc.) for an object. Read-only.
         *
         * [Api set: ExcelApi 1.1]
         */
        readonly font: Excel.ChartFont;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartTitleFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartTitleFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartTitleFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartTitleFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartTitleFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartTitleFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartTitleFormatLoadOptions): Excel.ChartTitleFormat;
        load(option?: string | string[]): Excel.ChartTitleFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartTitleFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartTitleFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartTitleFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartTitleFormatData;
    }
    /**
     *
     * Represents the fill formatting for a chart element.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartFill extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         */
        load(option?: string | string[] | OfficeExtension.LoadOption): Excel.ChartFill;
        /**
         *
         * Clear the fill color of a chart element.
         *
         * [Api set: ExcelApi 1.1]
         */
        clear(): void;
        /**
         *
         * Sets the fill formatting of a chart element to a uniform color.
         *
         * [Api set: ExcelApi 1.1]
         *
         * @param color HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
         */
        setSolidColor(color: string): void;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartFill object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartFillData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): {
            [key: string]: string;
        };
    }
    /**
     *
     * Represents the border formatting of a chart element.
     *
     * [Api set: ExcelApi 1.7]
     */
    class ChartBorder extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * HTML color code representing the color of borders in the chart.
         *
         * [Api set: ExcelApi 1.7]
         */
        color: string;
        /**
         *
         * Represents the line style of the border. See Excel.ChartLineStyle for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        lineStyle: Excel.ChartLineStyle | "None" | "Continuous" | "Dash" | "DashDot" | "DashDotDot" | "Dot" | "Grey25" | "Grey50" | "Grey75" | "Automatic" | "RoundDot";
        /**
         *
         * Represents weight of the border, in points.
         *
         * [Api set: ExcelApi 1.7]
         */
        weight: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartBorder): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartBorderUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartBorder): void;
        /**
         *
         * Clear the border format of a chart element.
         *
         * [Api set: ExcelApi 1.8]
         */
        clear(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartBorder` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartBorder` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartBorder` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartBorderLoadOptions): Excel.ChartBorder;
        load(option?: string | string[]): Excel.ChartBorder;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartBorder;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartBorder object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartBorderData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartBorderData;
    }
    /**
     *
     * Encapsulates the bin options for histogram charts and pareto charts.
     *
     * [Api set: ExcelApi 1.9]
     */
    class ChartBinOptions extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Specifies whether or not the bin overflow is enabled in a histogram chart or pareto chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        allowOverflow: boolean;
        /**
         *
         * Specifies whether or not the bin underflow is enabled in a histogram chart or pareto chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        allowUnderflow: boolean;
        /**
         *
         * Returns or sets the bin count of a histogram chart or pareto chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        count: number;
        /**
         *
         * Returns or sets the bin overflow value of a histogram chart or pareto chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        overflowValue: number;
        /**
         *
         * Returns or sets the bin's type for a histogram chart or pareto chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        type: Excel.ChartBinType | "Category" | "Auto" | "BinWidth" | "BinCount";
        /**
         *
         * Returns or sets the bin underflow value of a histogram chart or pareto chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        underflowValue: number;
        /**
         *
         * Returns or sets the bin width value of a histogram chart or pareto chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        width: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartBinOptions): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartBinOptionsUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartBinOptions): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartBinOptions` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartBinOptions` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartBinOptions` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartBinOptionsLoadOptions): Excel.ChartBinOptions;
        load(option?: string | string[]): Excel.ChartBinOptions;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartBinOptions;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartBinOptions object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartBinOptionsData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartBinOptionsData;
    }
    /**
     *
     * Represents the properties of a box and whisker chart.
     *
     * [Api set: ExcelApi 1.9]
     */
    class ChartBoxwhiskerOptions extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns or sets the quartile calculation type of a box and whisker chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        quartileCalculation: Excel.ChartBoxQuartileCalculation | "Inclusive" | "Exclusive";
        /**
         *
         * Specifies whether or not the inner points are shown in a box and whisker chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        showInnerPoints: boolean;
        /**
         *
         * Specifies whether or not the mean line is shown in a box and whisker chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        showMeanLine: boolean;
        /**
         *
         * Specifies whether or not the mean marker is shown in a box and whisker chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        showMeanMarker: boolean;
        /**
         *
         * Specifies whether or not outlier points are shown in a box and whisker chart. Read/Write.
         *
         * [Api set: ExcelApi 1.9]
         */
        showOutlierPoints: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartBoxwhiskerOptions): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartBoxwhiskerOptionsUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartBoxwhiskerOptions): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartBoxwhiskerOptions` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartBoxwhiskerOptions` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartBoxwhiskerOptions` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartBoxwhiskerOptionsLoadOptions): Excel.ChartBoxwhiskerOptions;
        load(option?: string | string[]): Excel.ChartBoxwhiskerOptions;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartBoxwhiskerOptions;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartBoxwhiskerOptions object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartBoxwhiskerOptionsData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartBoxwhiskerOptionsData;
    }
    /**
     *
     * Encapsulates the formatting options for line elements.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartLineFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * HTML color code representing the color of lines in the chart.
         *
         * [Api set: ExcelApi 1.1]
         */
        color: string;
        /**
         *
         * Represents the line style. See Excel.ChartLineStyle for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        lineStyle: Excel.ChartLineStyle | "None" | "Continuous" | "Dash" | "DashDot" | "DashDotDot" | "Dot" | "Grey25" | "Grey50" | "Grey75" | "Automatic" | "RoundDot";
        /**
         *
         * Represents weight of the line, in points.
         *
         * [Api set: ExcelApi 1.7]
         */
        weight: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartLineFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartLineFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartLineFormat): void;
        /**
         *
         * Clear the line format of a chart element.
         *
         * [Api set: ExcelApi 1.1]
         */
        clear(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartLineFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartLineFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartLineFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartLineFormatLoadOptions): Excel.ChartLineFormat;
        load(option?: string | string[]): Excel.ChartLineFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartLineFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartLineFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartLineFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartLineFormatData;
    }
    /**
     *
     * This object represents the font attributes (font name, font size, color, etc.) for a chart object.
     *
     * [Api set: ExcelApi 1.1]
     */
    class ChartFont extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the bold status of font.
         *
         * [Api set: ExcelApi 1.1]
         */
        bold: boolean;
        /**
         *
         * HTML color code representation of the text color. E.g. #FF0000 represents Red.
         *
         * [Api set: ExcelApi 1.1]
         */
        color: string;
        /**
         *
         * Represents the italic status of the font.
         *
         * [Api set: ExcelApi 1.1]
         */
        italic: boolean;
        /**
         *
         * Font name (e.g. "Calibri")
         *
         * [Api set: ExcelApi 1.1]
         */
        name: string;
        /**
         *
         * Size of the font (e.g. 11)
         *
         * [Api set: ExcelApi 1.1]
         */
        size: number;
        /**
         *
         * Type of underline applied to the font. See Excel.ChartUnderlineStyle for details.
         *
         * [Api set: ExcelApi 1.1]
         */
        underline: Excel.ChartUnderlineStyle | "None" | "Single";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartFont): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartFontUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartFont): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartFont` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartFont` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartFont` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartFontLoadOptions): Excel.ChartFont;
        load(option?: string | string[]): Excel.ChartFont;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartFont;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartFont object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartFontData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartFontData;
    }
    /**
     *
     * This object represents the attributes for a chart trendline object.
     *
     * [Api set: ExcelApi 1.7]
     */
    class ChartTrendline extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the formatting of a chart trendline.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly format: Excel.ChartTrendlineFormat;
        /**
         *
         * Represents the label of a chart trendline.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly label: Excel.ChartTrendlineLabel;
        /**
         *
         * Represents the number of periods that the trendline extends backward.
         *
         * [Api set: ExcelApi 1.8]
         */
        backwardPeriod: number;
        /**
         *
         * Represents the number of periods that the trendline extends forward.
         *
         * [Api set: ExcelApi 1.8]
         */
        forwardPeriod: number;
        /**
         *
         * Represents the intercept value of the trendline. Can be set to a numeric value or an empty string (for automatic values). The returned value is always a number.
         *
         * [Api set: ExcelApi 1.7]
         */
        intercept: any;
        /**
         *
         * Represents the period of a chart trendline. Only applicable for trendline with MovingAverage type.
         *
         * [Api set: ExcelApi 1.7]
         */
        movingAveragePeriod: number;
        /**
         *
         * Represents the name of the trendline. Can be set to a string value, or can be set to null value represents automatic values. The returned value is always a string
         *
         * [Api set: ExcelApi 1.7]
         */
        name: string;
        /**
         *
         * Represents the order of a chart trendline. Only applicable for trendline with Polynomial type.
         *
         * [Api set: ExcelApi 1.7]
         */
        polynomialOrder: number;
        /**
         *
         * True if the equation for the trendline is displayed on the chart.
         *
         * [Api set: ExcelApi 1.8]
         */
        showEquation: boolean;
        /**
         *
         * True if the R-squared for the trendline is displayed on the chart.
         *
         * [Api set: ExcelApi 1.8]
         */
        showRSquared: boolean;
        /**
         *
         * Represents the type of a chart trendline.
         *
         * [Api set: ExcelApi 1.7]
         */
        type: Excel.ChartTrendlineType | "Linear" | "Exponential" | "Logarithmic" | "MovingAverage" | "Polynomial" | "Power";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartTrendline): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartTrendlineUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartTrendline): void;
        /**
         *
         * Delete the trendline object.
         *
         * [Api set: ExcelApi 1.7]
         */
        delete(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartTrendline` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartTrendline` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartTrendline` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartTrendlineLoadOptions): Excel.ChartTrendline;
        load(option?: string | string[]): Excel.ChartTrendline;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartTrendline;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartTrendline object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartTrendlineData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartTrendlineData;
    }
    /**
     *
     * Represents a collection of Chart Trendlines.
     *
     * [Api set: ExcelApi 1.7]
     */
    class ChartTrendlineCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.ChartTrendline[];
        /**
         *
         * Adds a new trendline to trendline collection.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param type Specifies the trendline type. The default value is "Linear". See Excel.ChartTrendline for details.
         */
        add(type?: Excel.ChartTrendlineType): Excel.ChartTrendline;
        /**
         *
         * Adds a new trendline to trendline collection.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param type Specifies the trendline type. The default value is "Linear". See Excel.ChartTrendline for details.
         */
        add(type?: "Linear" | "Exponential" | "Logarithmic" | "MovingAverage" | "Polynomial" | "Power"): Excel.ChartTrendline;
        /**
         *
         * Returns the number of trendlines in the collection.
         *
         * [Api set: ExcelApi 1.7]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Get trendline object by index, which is the insertion order in items array.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param index Represents the insertion order in items array.
         */
        getItem(index: number): Excel.ChartTrendline;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartTrendlineCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartTrendlineCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartTrendlineCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartTrendlineCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.ChartTrendlineCollection;
        load(option?: string | string[]): Excel.ChartTrendlineCollection;
        load(option?: OfficeExtension.LoadOption): Excel.ChartTrendlineCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.ChartTrendlineCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartTrendlineCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.ChartTrendlineCollectionData;
    }
    /**
     *
     * Represents the format properties for chart trendline.
     *
     * [Api set: ExcelApi 1.7]
     */
    class ChartTrendlineFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents chart line formatting. Read-only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly line: Excel.ChartLineFormat;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartTrendlineFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartTrendlineFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartTrendlineFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartTrendlineFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartTrendlineFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartTrendlineFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartTrendlineFormatLoadOptions): Excel.ChartTrendlineFormat;
        load(option?: string | string[]): Excel.ChartTrendlineFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartTrendlineFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartTrendlineFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartTrendlineFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartTrendlineFormatData;
    }
    /**
     *
     * This object represents the attributes for a chart trendline lable object.
     *
     * [Api set: ExcelApi 1.8]
     */
    class ChartTrendlineLabel extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the format of chart trendline label.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly format: Excel.ChartTrendlineLabelFormat;
        /**
         *
         * Boolean value representing if trendline label automatically generates appropriate text based on context.
         *
         * [Api set: ExcelApi 1.8]
         */
        autoText: boolean;
        /**
         *
         * String value that represents the formula of chart trendline label using A1-style notation.
         *
         * [Api set: ExcelApi 1.8]
         */
        formula: string;
        /**
         *
         * Returns the height, in points, of the chart trendline label. Read-only. Null if chart trendline label is not visible.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly height: number;
        /**
         *
         * Represents the horizontal alignment for chart trendline label. See Excel.ChartTextHorizontalAlignment for details.
            This property is valid only when TextOrientation of trendline label is 90, -90 or 180.
         *
         * [Api set: ExcelApi 1.8]
         */
        horizontalAlignment: Excel.ChartTextHorizontalAlignment | "Center" | "Left" | "Right" | "Justify" | "Distributed";
        /**
         *
         * Represents the distance, in points, from the left edge of chart trendline label to the left edge of chart area. Null if chart trendline label is not visible.
         *
         * [Api set: ExcelApi 1.8]
         */
        left: number;
        /**
         *
         * Boolean value representing if the number format is linked to the cells (so that the number format changes in the labels when it changes in the cells).
         *
         * [Api set: ExcelApi 1.9]
         */
        linkNumberFormat: boolean;
        /**
         *
         * String value that represents the format code for trendline label.
         *
         * [Api set: ExcelApi 1.8]
         */
        numberFormat: string;
        /**
         *
         * String representing the text of the trendline label on a chart.
         *
         * [Api set: ExcelApi 1.8]
         */
        text: string;
        /**
         *
         * Represents the text orientation of chart trendline label. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
         *
         * [Api set: ExcelApi 1.8]
         */
        textOrientation: number;
        /**
         *
         * Represents the distance, in points, from the top edge of chart trendline label to the top of chart area. Null if chart trendline label is not visible.
         *
         * [Api set: ExcelApi 1.8]
         */
        top: number;
        /**
         *
         * Represents the vertical alignment of chart trendline label. See Excel.ChartTextVerticalAlignment for details.
            This property is valid only when TextOrientation of trendline label is 0.
         *
         * [Api set: ExcelApi 1.8]
         */
        verticalAlignment: Excel.ChartTextVerticalAlignment | "Center" | "Bottom" | "Top" | "Justify" | "Distributed";
        /**
         *
         * Returns the width, in points, of the chart trendline label. Read-only. Null if chart trendline label is not visible.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly width: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartTrendlineLabel): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartTrendlineLabelUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartTrendlineLabel): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartTrendlineLabel` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartTrendlineLabel` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartTrendlineLabel` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartTrendlineLabelLoadOptions): Excel.ChartTrendlineLabel;
        load(option?: string | string[]): Excel.ChartTrendlineLabel;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartTrendlineLabel;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartTrendlineLabel object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartTrendlineLabelData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartTrendlineLabelData;
    }
    /**
     *
     * Encapsulates the format properties for the chart trendline label.
     *
     * [Api set: ExcelApi 1.8]
     */
    class ChartTrendlineLabelFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the border format, which includes color, linestyle, and weight.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly border: Excel.ChartBorder;
        /**
         *
         * Represents the fill format of the current chart trendline label.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly fill: Excel.ChartFill;
        /**
         *
         * Represents the font attributes (font name, font size, color, etc.) for a chart trendline label.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly font: Excel.ChartFont;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartTrendlineLabelFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartTrendlineLabelFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartTrendlineLabelFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartTrendlineLabelFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartTrendlineLabelFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartTrendlineLabelFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartTrendlineLabelFormatLoadOptions): Excel.ChartTrendlineLabelFormat;
        load(option?: string | string[]): Excel.ChartTrendlineLabelFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartTrendlineLabelFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartTrendlineLabelFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartTrendlineLabelFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartTrendlineLabelFormatData;
    }
    /**
     *
     * This object represents the attributes for a chart plotArea object.
     *
     * [Api set: ExcelApi 1.8]
     */
    class ChartPlotArea extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the formatting of a chart plotArea.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly format: Excel.ChartPlotAreaFormat;
        /**
         *
         * Represents the height value of plotArea.
         *
         * [Api set: ExcelApi 1.8]
         */
        height: number;
        /**
         *
         * Represents the insideHeight value of plotArea.
         *
         * [Api set: ExcelApi 1.8]
         */
        insideHeight: number;
        /**
         *
         * Represents the insideLeft value of plotArea.
         *
         * [Api set: ExcelApi 1.8]
         */
        insideLeft: number;
        /**
         *
         * Represents the insideTop value of plotArea.
         *
         * [Api set: ExcelApi 1.8]
         */
        insideTop: number;
        /**
         *
         * Represents the insideWidth value of plotArea.
         *
         * [Api set: ExcelApi 1.8]
         */
        insideWidth: number;
        /**
         *
         * Represents the left value of plotArea.
         *
         * [Api set: ExcelApi 1.8]
         */
        left: number;
        /**
         *
         * Represents the position of plotArea.
         *
         * [Api set: ExcelApi 1.8]
         */
        position: Excel.ChartPlotAreaPosition | "Automatic" | "Custom";
        /**
         *
         * Represents the top value of plotArea.
         *
         * [Api set: ExcelApi 1.8]
         */
        top: number;
        /**
         *
         * Represents the width value of plotArea.
         *
         * [Api set: ExcelApi 1.8]
         */
        width: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartPlotArea): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartPlotAreaUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartPlotArea): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartPlotArea` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartPlotArea` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartPlotArea` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartPlotAreaLoadOptions): Excel.ChartPlotArea;
        load(option?: string | string[]): Excel.ChartPlotArea;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartPlotArea;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartPlotArea object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartPlotAreaData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartPlotAreaData;
    }
    /**
     *
     * Represents the format properties for chart plotArea.
     *
     * [Api set: ExcelApi 1.8]
     */
    class ChartPlotAreaFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the border attributes of a chart plotArea.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly border: Excel.ChartBorder;
        /**
         *
         * Represents the fill format of an object, which includes background formatting information.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly fill: Excel.ChartFill;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ChartPlotAreaFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ChartPlotAreaFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ChartPlotAreaFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ChartPlotAreaFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ChartPlotAreaFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ChartPlotAreaFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ChartPlotAreaFormatLoadOptions): Excel.ChartPlotAreaFormat;
        load(option?: string | string[]): Excel.ChartPlotAreaFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ChartPlotAreaFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ChartPlotAreaFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ChartPlotAreaFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ChartPlotAreaFormatData;
    }
    /**
     *
     * Manages sorting operations on Range objects.
     *
     * [Api set: ExcelApi 1.2]
     */
    class RangeSort extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Perform a sort operation.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param fields The list of conditions to sort on.
         * @param matchCase Optional. Whether to have the casing impact string ordering.
         * @param hasHeaders Optional. Whether the range has a header.
         * @param orientation Optional. Whether the operation is sorting rows or columns.
         * @param method Optional. The ordering method used for Chinese characters.
         */
        apply(fields: Excel.SortField[], matchCase?: boolean, hasHeaders?: boolean, orientation?: Excel.SortOrientation, method?: Excel.SortMethod): void;
        /**
         *
         * Perform a sort operation.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param fields The list of conditions to sort on.
         * @param matchCase Optional. Whether to have the casing impact string ordering.
         * @param hasHeaders Optional. Whether the range has a header.
         * @param orientation Optional. Whether the operation is sorting rows or columns.
         * @param method Optional. The ordering method used for Chinese characters.
         */
        apply(fields: Excel.SortField[], matchCase?: boolean, hasHeaders?: boolean, orientation?: "Rows" | "Columns", method?: "PinYin" | "StrokeCount"): void;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.RangeSort object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.RangeSortData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): {
            [key: string]: string;
        };
    }
    /**
     *
     * Manages sorting operations on Table objects.
     *
     * [Api set: ExcelApi 1.2]
     */
    class TableSort extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the current conditions used to last sort the table. Read-only.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly fields: Excel.SortField[];
        /**
         *
         * Represents whether the casing impacted the last sort of the table. Read-only.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly matchCase: boolean;
        /**
         *
         * Represents Chinese character ordering method last used to sort the table. Read-only.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly method: Excel.SortMethod | "PinYin" | "StrokeCount";
        /**
         *
         * Perform a sort operation.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param fields The list of conditions to sort on.
         * @param matchCase Optional. Whether to have the casing impact string ordering.
         * @param method Optional. The ordering method used for Chinese characters.
         */
        apply(fields: Excel.SortField[], matchCase?: boolean, method?: Excel.SortMethod): void;
        /**
         *
         * Perform a sort operation.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param fields The list of conditions to sort on.
         * @param matchCase Optional. Whether to have the casing impact string ordering.
         * @param method Optional. The ordering method used for Chinese characters.
         */
        apply(fields: Excel.SortField[], matchCase?: boolean, method?: "PinYin" | "StrokeCount"): void;
        /**
         *
         * Clears the sorting that is currently on the table. While this doesn't modify the table's ordering, it clears the state of the header buttons.
         *
         * [Api set: ExcelApi 1.2]
         */
        clear(): void;
        /**
         *
         * Reapplies the current sorting parameters to the table.
         *
         * [Api set: ExcelApi 1.2]
         */
        reapply(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.TableSort` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.TableSort` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.TableSort` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.TableSortLoadOptions): Excel.TableSort;
        load(option?: string | string[]): Excel.TableSort;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.TableSort;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.TableSort object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.TableSortData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.TableSortData;
    }
    /**
     *
     * Represents a condition in a sorting operation.
     *
     * [Api set: ExcelApi 1.2]
     */
    interface SortField {
        /**
         *
         * Represents whether the sorting is done in an ascending fashion.
         *
         * [Api set: ExcelApi 1.2]
         */
        ascending?: boolean;
        /**
         *
         * Represents the color that is the target of the condition if the sorting is on font or cell color.
         *
         * [Api set: ExcelApi 1.2]
         */
        color?: string;
        /**
         *
         * Represents additional sorting options for this field.
         *
         * [Api set: ExcelApi 1.2]
         */
        dataOption?: Excel.SortDataOption | "Normal" | "TextAsNumber";
        /**
         *
         * Represents the icon that is the target of the condition if the sorting is on the cell's icon.
         *
         * [Api set: ExcelApi 1.2]
         */
        icon?: Excel.Icon;
        /**
         *
         * Represents the column (or row, depending on the sort orientation) that the condition is on. Represented as an offset from the first column (or row).
         *
         * [Api set: ExcelApi 1.2]
         */
        key: number;
        /**
         *
         * Represents the type of sorting of this condition.
         *
         * [Api set: ExcelApi 1.2]
         */
        sortOn?: Excel.SortOn | "Value" | "CellColor" | "FontColor" | "Icon";
        /**
         *
         * Represents the subfield that is the target property name of a rich value to sort on.
         *
         * [Api set: ExcelApi 1.9]
         */
        subField?: string;
    }
    /**
     *
     * Manages the filtering of a table's column.
     *
     * [Api set: ExcelApi 1.2]
     */
    class Filter extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * The currently applied filter on the given column. Read-only.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly criteria: Excel.FilterCriteria;
        /**
         *
         * Apply the given filter criteria on the given column.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param criteria The criteria to apply.
         */
        apply(criteria: Excel.FilterCriteria): void;
        /**
         *
         * Apply a "Bottom Item" filter to the column for the given number of elements.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param count The number of elements from the bottom to show.
         */
        applyBottomItemsFilter(count: number): void;
        /**
         *
         * Apply a "Bottom Percent" filter to the column for the given percentage of elements.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param percent The percentage of elements from the bottom to show.
         */
        applyBottomPercentFilter(percent: number): void;
        /**
         *
         * Apply a "Cell Color" filter to the column for the given color.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param color The background color of the cells to show.
         */
        applyCellColorFilter(color: string): void;
        /**
         *
         * Apply an "Icon" filter to the column for the given criteria strings.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param criteria1 The first criteria string.
         * @param criteria2 Optional. The second criteria string.
         * @param oper Optional. The operator that describes how the two criteria are joined.
         */
        applyCustomFilter(criteria1: string, criteria2?: string, oper?: Excel.FilterOperator): void;
        /**
         *
         * Apply an "Icon" filter to the column for the given criteria strings.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param criteria1 The first criteria string.
         * @param criteria2 Optional. The second criteria string.
         * @param oper Optional. The operator that describes how the two criteria are joined.
         */
        applyCustomFilter(criteria1: string, criteria2?: string, oper?: "And" | "Or"): void;
        /**
         *
         * Apply a "Dynamic" filter to the column.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param criteria The dynamic criteria to apply.
         */
        applyDynamicFilter(criteria: Excel.DynamicFilterCriteria): void;
        /**
         *
         * Apply a "Dynamic" filter to the column.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param criteria The dynamic criteria to apply.
         */
        applyDynamicFilter(criteria: "Unknown" | "AboveAverage" | "AllDatesInPeriodApril" | "AllDatesInPeriodAugust" | "AllDatesInPeriodDecember" | "AllDatesInPeriodFebruray" | "AllDatesInPeriodJanuary" | "AllDatesInPeriodJuly" | "AllDatesInPeriodJune" | "AllDatesInPeriodMarch" | "AllDatesInPeriodMay" | "AllDatesInPeriodNovember" | "AllDatesInPeriodOctober" | "AllDatesInPeriodQuarter1" | "AllDatesInPeriodQuarter2" | "AllDatesInPeriodQuarter3" | "AllDatesInPeriodQuarter4" | "AllDatesInPeriodSeptember" | "BelowAverage" | "LastMonth" | "LastQuarter" | "LastWeek" | "LastYear" | "NextMonth" | "NextQuarter" | "NextWeek" | "NextYear" | "ThisMonth" | "ThisQuarter" | "ThisWeek" | "ThisYear" | "Today" | "Tomorrow" | "YearToDate" | "Yesterday"): void;
        /**
         *
         * Apply a "Font Color" filter to the column for the given color.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param color The font color of the cells to show.
         */
        applyFontColorFilter(color: string): void;
        /**
         *
         * Apply an "Icon" filter to the column for the given icon.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param icon The icons of the cells to show.
         */
        applyIconFilter(icon: Excel.Icon): void;
        /**
         *
         * Apply a "Top Item" filter to the column for the given number of elements.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param count The number of elements from the top to show.
         */
        applyTopItemsFilter(count: number): void;
        /**
         *
         * Apply a "Top Percent" filter to the column for the given percentage of elements.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param percent The percentage of elements from the top to show.
         */
        applyTopPercentFilter(percent: number): void;
        /**
         *
         * Apply a "Values" filter to the column for the given values.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values The list of values to show. This must be an array of strings or an array of Excel.FilterDateTime objects.
         */
        applyValuesFilter(values: Array<string | FilterDatetime>): void;
        /**
         *
         * Clear the filter on the given column.
         *
         * [Api set: ExcelApi 1.2]
         */
        clear(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.Filter` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.Filter` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.Filter` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.FilterLoadOptions): Excel.Filter;
        load(option?: string | string[]): Excel.Filter;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.Filter;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.Filter object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.FilterData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.FilterData;
    }
    /**
     *
     * Represents the filtering criteria applied to a column.
     *
     * [Api set: ExcelApi 1.2]
     */
    interface FilterCriteria {
        /**
         *
         * The HTML color string used to filter cells. Used with "cellColor" and "fontColor" filtering.
         *
         * [Api set: ExcelApi 1.2]
         */
        color?: string;
        /**
         *
         * The first criterion used to filter data. Used as an operator in the case of "custom" filtering.
             For example ">50" for number greater than 50 or "=*s" for values ending in "s".
            
             Used as a number in the case of top/bottom items/percents. E.g. "5" for the top 5 items if filterOn is set to "topItems"
         *
         * [Api set: ExcelApi 1.2]
         */
        criterion1?: string;
        /**
         *
         * The second criterion used to filter data. Only used as an operator in the case of "custom" filtering.
         *
         * [Api set: ExcelApi 1.2]
         */
        criterion2?: string;
        /**
         *
         * The dynamic criteria from the Excel.DynamicFilterCriteria set to apply on this column. Used with "dynamic" filtering.
         *
         * [Api set: ExcelApi 1.2]
         */
        dynamicCriteria?: Excel.DynamicFilterCriteria | "Unknown" | "AboveAverage" | "AllDatesInPeriodApril" | "AllDatesInPeriodAugust" | "AllDatesInPeriodDecember" | "AllDatesInPeriodFebruray" | "AllDatesInPeriodJanuary" | "AllDatesInPeriodJuly" | "AllDatesInPeriodJune" | "AllDatesInPeriodMarch" | "AllDatesInPeriodMay" | "AllDatesInPeriodNovember" | "AllDatesInPeriodOctober" | "AllDatesInPeriodQuarter1" | "AllDatesInPeriodQuarter2" | "AllDatesInPeriodQuarter3" | "AllDatesInPeriodQuarter4" | "AllDatesInPeriodSeptember" | "BelowAverage" | "LastMonth" | "LastQuarter" | "LastWeek" | "LastYear" | "NextMonth" | "NextQuarter" | "NextWeek" | "NextYear" | "ThisMonth" | "ThisQuarter" | "ThisWeek" | "ThisYear" | "Today" | "Tomorrow" | "YearToDate" | "Yesterday";
        /**
         *
         * The property used by the filter to determine whether the values should stay visible.
         *
         * [Api set: ExcelApi 1.2]
         */
        filterOn: Excel.FilterOn | "BottomItems" | "BottomPercent" | "CellColor" | "Dynamic" | "FontColor" | "Values" | "TopItems" | "TopPercent" | "Icon" | "Custom";
        /**
         *
         * The icon used to filter cells. Used with "icon" filtering.
         *
         * [Api set: ExcelApi 1.2]
         */
        icon?: Excel.Icon;
        /**
         *
         * The operator used to combine criterion 1 and 2 when using "custom" filtering.
         *
         * [Api set: ExcelApi 1.2]
         */
        operator?: Excel.FilterOperator | "And" | "Or";
        /**
         *
         * The property used by the filter to do rich filter on richvalues.
         *
         * [Api set: ExcelApi 1.9]
         */
        subField?: string;
        /**
         *
         * The set of values to be used as part of "values" filtering.
         *
         * [Api set: ExcelApi 1.2]
         */
        values?: Array<string | FilterDatetime>;
    }
    /**
     *
     * Represents how to filter a date when filtering on values.
     *
     * [Api set: ExcelApi 1.2]
     */
    interface FilterDatetime {
        /**
         *
         * The date in ISO8601 format used to filter data.
         *
         * [Api set: ExcelApi 1.2]
         */
        date: string;
        /**
         *
         * How specific the date should be used to keep data. For example, if the date is 2005-04-02 and the specifity is set to "month", the filter operation will keep all rows with a date in the month of april 2009.
         *
         * [Api set: ExcelApi 1.2]
         */
        specificity: Excel.FilterDatetimeSpecificity | "Year" | "Month" | "Day" | "Hour" | "Minute" | "Second";
    }
    /**
     *
     * Represents the AutoFilter object.
            AutoFilter turns the values in Excel column into specific filters based on the cell contents.
     *
     * [Api set: ExcelApi 1.9]
     */
    class AutoFilter extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * An array that holds all the filter criteria in the autofiltered range. Read-Only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly criteria: Excel.FilterCriteria[];
        /**
         *
         * Indicates if the AutoFilter is enabled or not. Read-Only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly enabled: boolean;
        /**
         *
         * Indicates if the AutoFilter has filter criteria. Read-Only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly isDataFiltered: boolean;
        /**
         *
         * Applies the AutoFilter to a range. This filters the column if column index and filter criteria are specified.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param range The range over which the AutoFilter will apply on.
         * @param columnIndex The zero-based column index to which the AutoFilter is applied.
         * @param criteria The filter criteria.
         */
        apply(range: Range | string, columnIndex?: number, criteria?: Excel.FilterCriteria): void;
        /**
         *
         * Clears the filter criteria of the AutoFilter.
         *
         * [Api set: ExcelApi 1.9]
         */
        clearCriteria(): void;
        /**
         *
         * Returns the Range object that represents the range to which the AutoFilter applies.
         *
         * [Api set: ExcelApi 1.9]
         */
        getRange(): Excel.Range;
        /**
         *
         * Returns the Range object that represents the range to which the AutoFilter applies.
            If there is no Range object associated with the AutoFilter, this method returns a null object.
         *
         * [Api set: ExcelApi 1.9]
         */
        getRangeOrNullObject(): Excel.Range;
        /**
         *
         * Applies the specified Autofilter object currently on the range.
         *
         * [Api set: ExcelApi 1.9]
         */
        reapply(): void;
        /**
         *
         * Removes the AutoFilter for the range.
         *
         * [Api set: ExcelApi 1.9]
         */
        remove(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.AutoFilter` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.AutoFilter` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.AutoFilter` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.AutoFilterLoadOptions): Excel.AutoFilter;
        load(option?: string | string[]): Excel.AutoFilter;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.AutoFilter;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.AutoFilter object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.AutoFilterData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.AutoFilterData;
    }
    /**
     *
     * Represents a cell icon.
     *
     * [Api set: ExcelApi 1.2]
     */
    interface Icon {
        /**
         *
         * Represents the index of the icon in the given set.
         *
         * [Api set: ExcelApi 1.2]
         */
        index: number;
        /**
         *
         * Represents the set that the icon is part of.
         *
         * [Api set: ExcelApi 1.2]
         */
        set: Excel.IconSet | "Invalid" | "ThreeArrows" | "ThreeArrowsGray" | "ThreeFlags" | "ThreeTrafficLights1" | "ThreeTrafficLights2" | "ThreeSigns" | "ThreeSymbols" | "ThreeSymbols2" | "FourArrows" | "FourArrowsGray" | "FourRedToBlack" | "FourRating" | "FourTrafficLights" | "FiveArrows" | "FiveArrowsGray" | "FiveRating" | "FiveQuarters" | "ThreeStars" | "ThreeTriangles" | "FiveBoxes" | "LinkedEntityFinanceIcon" | "LinkedEntityMapIcon";
    }
    /**
     *
     * A scoped collection of custom XML parts.
            A scoped collection is the result of some operation, e.g. filtering by namespace.
            A scoped collection cannot be scoped any further.
     *
     * [Api set: ExcelApi 1.5]
     */
    class CustomXmlPartScopedCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.CustomXmlPart[];
        /**
         *
         * Gets the number of CustomXML parts in this collection.
         *
         * [Api set: ExcelApi 1.5]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a custom XML part based on its ID.
         *
         * [Api set: ExcelApi 1.5]
         *
         * @param id ID of the object to be retrieved.
         */
        getItem(id: string): Excel.CustomXmlPart;
        /**
         *
         * Gets a custom XML part based on its ID.
            If the CustomXmlPart does not exist, the return object's isNull property will be true.
         *
         * [Api set: ExcelApi 1.5]
         *
         * @param id ID of the object to be retrieved.
         */
        getItemOrNullObject(id: string): Excel.CustomXmlPart;
        /**
         *
         * If the collection contains exactly one item, this method returns it.
            Otherwise, this method produces an error.
         *
         * [Api set: ExcelApi 1.5]
         */
        getOnlyItem(): Excel.CustomXmlPart;
        /**
         *
         * If the collection contains exactly one item, this method returns it.
            Otherwise, this method returns Null.
         *
         * [Api set: ExcelApi 1.5]
         */
        getOnlyItemOrNullObject(): Excel.CustomXmlPart;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.CustomXmlPartScopedCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.CustomXmlPartScopedCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.CustomXmlPartScopedCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.CustomXmlPartScopedCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.CustomXmlPartScopedCollection;
        load(option?: string | string[]): Excel.CustomXmlPartScopedCollection;
        load(option?: OfficeExtension.LoadOption): Excel.CustomXmlPartScopedCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.CustomXmlPartScopedCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.CustomXmlPartScopedCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.CustomXmlPartScopedCollectionData;
    }
    /**
     *
     * A collection of custom XML parts.
     *
     * [Api set: ExcelApi 1.5]
     */
    class CustomXmlPartCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.CustomXmlPart[];
        /**
         *
         * Adds a new custom XML part to the workbook.
         *
         * [Api set: ExcelApi 1.5]
         *
         * @param xml XML content. Must be a valid XML fragment.
         */
        add(xml: string): Excel.CustomXmlPart;
        /**
         *
         * Gets a new scoped collection of custom XML parts whose namespaces match the given namespace.
         *
         * [Api set: ExcelApi 1.5]
         *
         * @param namespaceUri This must be a fully qualified schema URI; for example, "http://schemas.contoso.com/review/1.0".
         */
        getByNamespace(namespaceUri: string): Excel.CustomXmlPartScopedCollection;
        /**
         *
         * Gets the number of CustomXml parts in the collection.
         *
         * [Api set: ExcelApi 1.5]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a custom XML part based on its ID.
         *
         * [Api set: ExcelApi 1.5]
         *
         * @param id ID of the object to be retrieved.
         */
        getItem(id: string): Excel.CustomXmlPart;
        /**
         *
         * Gets a custom XML part based on its ID.
            If the CustomXmlPart does not exist, the return object's isNull property will be true.
         *
         * [Api set: ExcelApi 1.5]
         *
         * @param id ID of the object to be retrieved.
         */
        getItemOrNullObject(id: string): Excel.CustomXmlPart;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.CustomXmlPartCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.CustomXmlPartCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.CustomXmlPartCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.CustomXmlPartCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.CustomXmlPartCollection;
        load(option?: string | string[]): Excel.CustomXmlPartCollection;
        load(option?: OfficeExtension.LoadOption): Excel.CustomXmlPartCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.CustomXmlPartCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.CustomXmlPartCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.CustomXmlPartCollectionData;
    }
    /**
     *
     * Represents a custom XML part object in a workbook.
     *
     * [Api set: ExcelApi 1.5]
     */
    class CustomXmlPart extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * The custom XML part's ID. Read-only.
         *
         * [Api set: ExcelApi 1.5]
         */
        readonly id: string;
        /**
         *
         * The custom XML part's namespace URI. Read-only.
         *
         * [Api set: ExcelApi 1.5]
         */
        readonly namespaceUri: string;
        /**
         *
         * Deletes the custom XML part.
         *
         * [Api set: ExcelApi 1.5]
         */
        delete(): void;
        /**
         *
         * Gets the custom XML part's full XML content.
         *
         * [Api set: ExcelApi 1.5]
         */
        getXml(): OfficeExtension.ClientResult<string>;
        /**
         *
         * Sets the custom XML part's full XML content.
         *
         * [Api set: ExcelApi 1.5]
         *
         * @param xml XML content for the part.
         */
        setXml(xml: string): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.CustomXmlPart` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.CustomXmlPart` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.CustomXmlPart` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.CustomXmlPartLoadOptions): Excel.CustomXmlPart;
        load(option?: string | string[]): Excel.CustomXmlPart;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.CustomXmlPart;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.CustomXmlPart object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.CustomXmlPartData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.CustomXmlPartData;
    }
    /**
     *
     * Represents a collection of all the PivotTables that are part of the workbook or worksheet.
     *
     * [Api set: ExcelApi 1.3]
     */
    class PivotTableCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.PivotTable[];
        /**
         *
         * Add a Pivottable based on the specified source data and insert it at the top left cell of the destination range.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param name The name of the new PivotTable.
         * @param source The source data for the new PivotTable, this can either be a range (or string address including the worksheet name) or a table.
         * @param destination The cell in the upper-left corner of the PivotTable report's destination range (the range on the worksheet where the resulting report will be placed).
         * @returns Returns the newly inserted PivotTable.
         */
        add(name: string, source: Range | string | Table, destination: Range | string): Excel.PivotTable;
        /**
         *
         * Gets the number of pivot tables in the collection.
         *
         * [Api set: ExcelApi 1.4]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a PivotTable by name.
         *
         * [Api set: ExcelApi 1.3]
         *
         * @param name Name of the PivotTable to be retrieved.
         */
        getItem(name: string): Excel.PivotTable;
        /**
         *
         * Gets a PivotTable by name. If the PivotTable does not exist, will return a null object.
         *
         * [Api set: ExcelApi 1.4]
         *
         * @param name Name of the PivotTable to be retrieved.
         */
        getItemOrNullObject(name: string): Excel.PivotTable;
        /**
         *
         * Refreshes all the pivot tables in the collection.
         *
         * [Api set: ExcelApi 1.3]
         */
        refreshAll(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.PivotTableCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.PivotTableCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.PivotTableCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.PivotTableCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.PivotTableCollection;
        load(option?: string | string[]): Excel.PivotTableCollection;
        load(option?: OfficeExtension.LoadOption): Excel.PivotTableCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.PivotTableCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.PivotTableCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.PivotTableCollectionData;
    }
    /**
     *
     * Represents an Excel PivotTable.
     *
     * [Api set: ExcelApi 1.3]
     */
    class PivotTable extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * The Column Pivot Hierarchies of the PivotTable.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly columnHierarchies: Excel.RowColumnPivotHierarchyCollection;
        /**
         *
         * The Data Pivot Hierarchies of the PivotTable.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly dataHierarchies: Excel.DataPivotHierarchyCollection;
        /**
         *
         * The Filter Pivot Hierarchies of the PivotTable.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly filterHierarchies: Excel.FilterPivotHierarchyCollection;
        /**
         *
         * The Pivot Hierarchies of the PivotTable.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly hierarchies: Excel.PivotHierarchyCollection;
        /**
         *
         * The PivotLayout describing the layout and visual structure of the PivotTable.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly layout: Excel.PivotLayout;
        /**
         *
         * The Row Pivot Hierarchies of the PivotTable.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly rowHierarchies: Excel.RowColumnPivotHierarchyCollection;
        /**
         *
         * The worksheet containing the current PivotTable.
         *
         * [Api set: ExcelApi 1.3]
         */
        readonly worksheet: Excel.Worksheet;
        /**
         *
         * Specifies whether the PivotTable allows values in the data body to be edited by the user.
         *
         * [Api set: ExcelApi 1.9]
         */
        enableDataValueEditing: boolean;
        /**
         *
         * Id of the PivotTable. Read-only.
         *
         * [Api set: ExcelApi 1.5]
         */
        readonly id: string;
        /**
         *
         * Name of the PivotTable.
         *
         * [Api set: ExcelApi 1.3]
         */
        name: string;
        /**
         *
         * Specifies whether the PivotTable uses custom lists when sorting.
         *
         * [Api set: ExcelApi 1.9]
         */
        useCustomSortLists: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.PivotTable): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.PivotTableUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.PivotTable): void;
        /**
         *
         * Deletes the PivotTable.
         *
         * [Api set: ExcelApi 1.8]
         */
        delete(): void;
        /**
         *
         * Refreshes the PivotTable.
         *
         * [Api set: ExcelApi 1.3]
         */
        refresh(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.PivotTable` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.PivotTable` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.PivotTable` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.PivotTableLoadOptions): Excel.PivotTable;
        load(option?: string | string[]): Excel.PivotTable;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.PivotTable;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.PivotTable object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.PivotTableData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.PivotTableData;
    }
    /**
     *
     * Represents the visual layout of the PivotTable.
     *
     * [Api set: ExcelApi 1.8]
     */
    class PivotLayout extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Specifies whether formatting will be automatically formatted when it’s refreshed or when fields are moved
         *
         * [Api set: ExcelApi 1.9]
         */
        autoFormat: boolean;
        /**
         *
         * This property indicates the PivotLayoutType of all fields on the PivotTable. If fields have different states, this will be null.
         *
         * [Api set: ExcelApi 1.8]
         */
        layoutType: Excel.PivotLayoutType | "Compact" | "Tabular" | "Outline";
        /**
         *
         * Specifies whether formatting is preserved when the report is refreshed or recalculated by operations such as pivoting, sorting, or changing page field items.
         *
         * [Api set: ExcelApi 1.9]
         */
        preserveFormatting: boolean;
        /**
         *
         * Specifies whether the PivotTable report shows grand totals for columns.
         *
         * [Api set: ExcelApi 1.8]
         */
        showColumnGrandTotals: boolean;
        /**
         *
         * Specifies whether the PivotTable report shows grand totals for rows.
         *
         * [Api set: ExcelApi 1.8]
         */
        showRowGrandTotals: boolean;
        /**
         *
         * This property indicates the SubtotalLocationType of all fields on the PivotTable. If fields have different states, this will be null.
         *
         * [Api set: ExcelApi 1.8]
         */
        subtotalLocation: Excel.SubtotalLocationType | "AtTop" | "AtBottom" | "Off";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.PivotLayout): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.PivotLayoutUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.PivotLayout): void;
        /**
         *
         * Returns the range where the PivotTable's column labels reside.
         *
         * [Api set: ExcelApi 1.8]
         */
        getColumnLabelRange(): Excel.Range;
        /**
         *
         * Returns the range where the PivotTable's data values reside.
         *
         * [Api set: ExcelApi 1.8]
         */
        getDataBodyRange(): Excel.Range;
        /**
         *
         * Gets the DataHierarchy that is used to calculate the value in a specified range within the PivotTable.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param cell A single cell within the PivotTable data body.
         * @returns The DataPivotHierarchy object used to calculate the value in the specified cell.
         */
        getDataHierarchy(cell: Range | string): Excel.DataPivotHierarchy;
        /**
         *
         * Returns the range of the PivotTable's filter area.
         *
         * [Api set: ExcelApi 1.8]
         */
        getFilterAxisRange(): Excel.Range;
        /**
         *
         * Gets the PivotItems from an axis that make up the value in a specified range within the PivotTable.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param axis The axis from which to get the PivotItems. Must be either "row" or "column."
         * @param cell A single cell within the PivotTable's data body.
         * @returns A collection of PivotItems that are used to calculate the values in the specified row.
         */
        getPivotItems(axis: Excel.PivotAxis, cell: Range | string): OfficeExtension.ClientResult<Excel.PivotItem[]>;
        /**
         *
         * Gets the PivotItems from an axis that make up the value in a specified range within the PivotTable.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param axis The axis from which to get the PivotItems. Must be either "row" or "column."
         * @param cell A single cell within the PivotTable's data body.
         * @returns A collection of PivotItems that are used to calculate the values in the specified row.
         */
        getPivotItems(axis: "Unknown" | "Row" | "Column" | "Data" | "Filter", cell: Range | string): OfficeExtension.ClientResult<Excel.PivotItem[]>;
        /**
         *
         * Returns the range the PivotTable exists on, excluding the filter area.
         *
         * [Api set: ExcelApi 1.8]
         */
        getRange(): Excel.Range;
        /**
         *
         * Returns the range where the PivotTable's row labels reside.
         *
         * [Api set: ExcelApi 1.8]
         */
        getRowLabelRange(): Excel.Range;
        /**
         *
         * Sets the PivotTable to automatically sort using the specified cell to automatically select all necessary criteria and context. This behaves identically to applying an autosort from the UI.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param cell A single cell to use get the criteria from for applying the autosort.
         * @param sortBy The direction of the sort.
         */
        setAutoSortOnCell(cell: Range | string, sortBy: Excel.SortBy): void;
        /**
         *
         * Sets the PivotTable to automatically sort using the specified cell to automatically select all necessary criteria and context. This behaves identically to applying an autosort from the UI.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param cell A single cell to use get the criteria from for applying the autosort.
         * @param sortBy The direction of the sort.
         */
        setAutoSortOnCell(cell: Range | string, sortBy: "Ascending" | "Descending"): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.PivotLayout` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.PivotLayout` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.PivotLayout` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.PivotLayoutLoadOptions): Excel.PivotLayout;
        load(option?: string | string[]): Excel.PivotLayout;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.PivotLayout;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.PivotLayout object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.PivotLayoutData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.PivotLayoutData;
    }
    /**
     *
     * Represents a collection of all the PivotTables that are part of the workbook or worksheet.
     *
     * [Api set: ExcelApi 1.8]
     */
    class PivotHierarchyCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.PivotHierarchy[];
        /**
         *
         * Gets the number of pivot hierarchies in the collection.
         *
         * [Api set: ExcelApi 1.8]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a PivotHierarchy by its name or id.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param name Name of the PivotTable to be retrieved.
         */
        getItem(name: string): Excel.PivotHierarchy;
        /**
         *
         * Gets a PivotHierarchy by name. If the PivotHierarchy does not exist, will return a null object.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param name Name of the PivotHierarchy to be retrieved.
         */
        getItemOrNullObject(name: string): Excel.PivotHierarchy;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.PivotHierarchyCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.PivotHierarchyCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.PivotHierarchyCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.PivotHierarchyCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.PivotHierarchyCollection;
        load(option?: string | string[]): Excel.PivotHierarchyCollection;
        load(option?: OfficeExtension.LoadOption): Excel.PivotHierarchyCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.PivotHierarchyCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.PivotHierarchyCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.PivotHierarchyCollectionData;
    }
    /**
     *
     * Represents the Excel PivotHierarchy.
     *
     * [Api set: ExcelApi 1.8]
     */
    class PivotHierarchy extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns the PivotFields associated with the PivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly fields: Excel.PivotFieldCollection;
        /**
         *
         * Id of the PivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly id: string;
        /**
         *
         * Name of the PivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        name: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.PivotHierarchy): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.PivotHierarchyUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.PivotHierarchy): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.PivotHierarchy` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.PivotHierarchy` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.PivotHierarchy` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.PivotHierarchyLoadOptions): Excel.PivotHierarchy;
        load(option?: string | string[]): Excel.PivotHierarchy;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.PivotHierarchy;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.PivotHierarchy object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.PivotHierarchyData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.PivotHierarchyData;
    }
    /**
     *
     * Represents a collection of RowColumnPivotHierarchy items associated with the PivotTable.
     *
     * [Api set: ExcelApi 1.8]
     */
    class RowColumnPivotHierarchyCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.RowColumnPivotHierarchy[];
        /**
         *
         * Adds the PivotHierarchy to the current axis. If the hierarchy is present elsewhere on the row, column,
            or filter axis, it will be removed from that location.
         *
         * [Api set: ExcelApi 1.8]
         */
        add(pivotHierarchy: Excel.PivotHierarchy): Excel.RowColumnPivotHierarchy;
        /**
         *
         * Gets the number of pivot hierarchies in the collection.
         *
         * [Api set: ExcelApi 1.8]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a RowColumnPivotHierarchy by its name or id.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param name Name of the PivotTable to be retrieved.
         */
        getItem(name: string): Excel.RowColumnPivotHierarchy;
        /**
         *
         * Gets a RowColumnPivotHierarchy by name. If the RowColumnPivotHierarchy does not exist, will return a null object.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param name Name of the RowColumnPivotHierarchy to be retrieved.
         */
        getItemOrNullObject(name: string): Excel.RowColumnPivotHierarchy;
        /**
         *
         * Removes the PivotHierarchy from the current axis.
         *
         * [Api set: ExcelApi 1.8]
         */
        remove(rowColumnPivotHierarchy: Excel.RowColumnPivotHierarchy): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.RowColumnPivotHierarchyCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.RowColumnPivotHierarchyCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.RowColumnPivotHierarchyCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.RowColumnPivotHierarchyCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.RowColumnPivotHierarchyCollection;
        load(option?: string | string[]): Excel.RowColumnPivotHierarchyCollection;
        load(option?: OfficeExtension.LoadOption): Excel.RowColumnPivotHierarchyCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.RowColumnPivotHierarchyCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.RowColumnPivotHierarchyCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.RowColumnPivotHierarchyCollectionData;
    }
    /**
     *
     * Represents the Excel RowColumnPivotHierarchy.
     *
     * [Api set: ExcelApi 1.8]
     */
    class RowColumnPivotHierarchy extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns the PivotFields associated with the RowColumnPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly fields: Excel.PivotFieldCollection;
        /**
         *
         * Id of the RowColumnPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly id: string;
        /**
         *
         * Name of the RowColumnPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        name: string;
        /**
         *
         * Position of the RowColumnPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        position: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.RowColumnPivotHierarchy): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.RowColumnPivotHierarchyUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.RowColumnPivotHierarchy): void;
        /**
         *
         * Reset the RowColumnPivotHierarchy back to its default values.
         *
         * [Api set: ExcelApi 1.8]
         */
        setToDefault(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.RowColumnPivotHierarchy` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.RowColumnPivotHierarchy` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.RowColumnPivotHierarchy` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.RowColumnPivotHierarchyLoadOptions): Excel.RowColumnPivotHierarchy;
        load(option?: string | string[]): Excel.RowColumnPivotHierarchy;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.RowColumnPivotHierarchy;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.RowColumnPivotHierarchy object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.RowColumnPivotHierarchyData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.RowColumnPivotHierarchyData;
    }
    /**
     *
     * Represents a collection of FilterPivotHierarchy items associated with the PivotTable.
     *
     * [Api set: ExcelApi 1.8]
     */
    class FilterPivotHierarchyCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.FilterPivotHierarchy[];
        /**
         *
         * Adds the PivotHierarchy to the current axis. If the hierarchy is present elsewhere on the row, column,
            or filter axis, it will be removed from that location.
         *
         * [Api set: ExcelApi 1.8]
         */
        add(pivotHierarchy: Excel.PivotHierarchy): Excel.FilterPivotHierarchy;
        /**
         *
         * Gets the number of pivot hierarchies in the collection.
         *
         * [Api set: ExcelApi 1.8]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a FilterPivotHierarchy by its name or id.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param name Name of the PivotTable to be retrieved.
         */
        getItem(name: string): Excel.FilterPivotHierarchy;
        /**
         *
         * Gets a FilterPivotHierarchy by name. If the FilterPivotHierarchy does not exist, will return a null object.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param name Name of the FilterPivotHierarchy to be retrieved.
         */
        getItemOrNullObject(name: string): Excel.FilterPivotHierarchy;
        /**
         *
         * Removes the PivotHierarchy from the current axis.
         *
         * [Api set: ExcelApi 1.8]
         */
        remove(filterPivotHierarchy: Excel.FilterPivotHierarchy): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.FilterPivotHierarchyCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.FilterPivotHierarchyCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.FilterPivotHierarchyCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.FilterPivotHierarchyCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.FilterPivotHierarchyCollection;
        load(option?: string | string[]): Excel.FilterPivotHierarchyCollection;
        load(option?: OfficeExtension.LoadOption): Excel.FilterPivotHierarchyCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.FilterPivotHierarchyCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.FilterPivotHierarchyCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.FilterPivotHierarchyCollectionData;
    }
    /**
     *
     * Represents the Excel FilterPivotHierarchy.
     *
     * [Api set: ExcelApi 1.8]
     */
    class FilterPivotHierarchy extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns the PivotFields associated with the FilterPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly fields: Excel.PivotFieldCollection;
        /**
         *
         * Determines whether to allow multiple filter items.
         *
         * [Api set: ExcelApi 1.8]
         */
        enableMultipleFilterItems: boolean;
        /**
         *
         * Id of the FilterPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly id: string;
        /**
         *
         * Name of the FilterPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        name: string;
        /**
         *
         * Position of the FilterPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        position: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.FilterPivotHierarchy): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.FilterPivotHierarchyUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.FilterPivotHierarchy): void;
        /**
         *
         * Reset the FilterPivotHierarchy back to its default values.
         *
         * [Api set: ExcelApi 1.8]
         */
        setToDefault(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.FilterPivotHierarchy` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.FilterPivotHierarchy` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.FilterPivotHierarchy` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.FilterPivotHierarchyLoadOptions): Excel.FilterPivotHierarchy;
        load(option?: string | string[]): Excel.FilterPivotHierarchy;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.FilterPivotHierarchy;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.FilterPivotHierarchy object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.FilterPivotHierarchyData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.FilterPivotHierarchyData;
    }
    /**
     *
     * Represents a collection of DataPivotHierarchy items associated with the PivotTable.
     *
     * [Api set: ExcelApi 1.8]
     */
    class DataPivotHierarchyCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.DataPivotHierarchy[];
        /**
         *
         * Adds the PivotHierarchy to the current axis.
         *
         * [Api set: ExcelApi 1.8]
         */
        add(pivotHierarchy: Excel.PivotHierarchy): Excel.DataPivotHierarchy;
        /**
         *
         * Gets the number of pivot hierarchies in the collection.
         *
         * [Api set: ExcelApi 1.8]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a DataPivotHierarchy by its name or id.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param name Name of the PivotTable to be retrieved.
         */
        getItem(name: string): Excel.DataPivotHierarchy;
        /**
         *
         * Gets a DataPivotHierarchy by name. If the DataPivotHierarchy does not exist, will return a null object.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param name Name of the DataPivotHierarchy to be retrieved.
         */
        getItemOrNullObject(name: string): Excel.DataPivotHierarchy;
        /**
         *
         * Removes the PivotHierarchy from the current axis.
         *
         * [Api set: ExcelApi 1.8]
         */
        remove(DataPivotHierarchy: Excel.DataPivotHierarchy): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.DataPivotHierarchyCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.DataPivotHierarchyCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.DataPivotHierarchyCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.DataPivotHierarchyCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.DataPivotHierarchyCollection;
        load(option?: string | string[]): Excel.DataPivotHierarchyCollection;
        load(option?: OfficeExtension.LoadOption): Excel.DataPivotHierarchyCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.DataPivotHierarchyCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.DataPivotHierarchyCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.DataPivotHierarchyCollectionData;
    }
    /**
     *
     * Represents the Excel DataPivotHierarchy.
     *
     * [Api set: ExcelApi 1.8]
     */
    class DataPivotHierarchy extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns the PivotFields associated with the DataPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly field: Excel.PivotField;
        /**
         *
         * Id of the DataPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly id: string;
        /**
         *
         * Name of the DataPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        name: string;
        /**
         *
         * Number format of the DataPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        numberFormat: string;
        /**
         *
         * Position of the DataPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        position: number;
        /**
         *
         * Determines whether the data should be shown as a specific summary calculation or not.
         *
         * [Api set: ExcelApi 1.8]
         */
        showAs: Excel.ShowAsRule;
        /**
         *
         * Determines whether to show all items of the DataPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        summarizeBy: Excel.AggregationFunction | "Unknown" | "Automatic" | "Sum" | "Count" | "Average" | "Max" | "Min" | "Product" | "CountNumbers" | "StandardDeviation" | "StandardDeviationP" | "Variance" | "VarianceP";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.DataPivotHierarchy): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.DataPivotHierarchyUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.DataPivotHierarchy): void;
        /**
         *
         * Reset the DataPivotHierarchy back to its default values.
         *
         * [Api set: ExcelApi 1.8]
         */
        setToDefault(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.DataPivotHierarchy` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.DataPivotHierarchy` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.DataPivotHierarchy` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.DataPivotHierarchyLoadOptions): Excel.DataPivotHierarchy;
        load(option?: string | string[]): Excel.DataPivotHierarchy;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.DataPivotHierarchy;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.DataPivotHierarchy object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.DataPivotHierarchyData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.DataPivotHierarchyData;
    }
    /**
     * [Api set: ExcelApi 1.8]
     */
    interface ShowAsRule {
        /**
         *
         * The base PivotField to base the ShowAs calculation, if applicable based on the ShowAsCalculation type, else null.
         *
         * [Api set: ExcelApi 1.8]
         */
        baseField?: Excel.PivotField;
        /**
         *
         * The base Item to base the ShowAs calculation on, if applicable based on the ShowAsCalculation type, else null.
         *
         * [Api set: ExcelApi 1.8]
         */
        baseItem?: Excel.PivotItem;
        /**
         *
         * The ShowAs Calculation to use for the Data PivotField. See Excel.ShowAsCalculation for Details.
         *
         * [Api set: ExcelApi 1.8]
         */
        calculation: Excel.ShowAsCalculation | "Unknown" | "None" | "PercentOfGrandTotal" | "PercentOfRowTotal" | "PercentOfColumnTotal" | "PercentOfParentRowTotal" | "PercentOfParentColumnTotal" | "PercentOfParentTotal" | "PercentOf" | "RunningTotal" | "PercentRunningTotal" | "DifferenceFrom" | "PercentDifferenceFrom" | "RankAscending" | "RankDecending" | "Index";
    }
    /**
     *
     * Represents a collection of all the PivotTables that are part of the workbook or worksheet.
     *
     * [Api set: ExcelApi 1.8]
     */
    class PivotFieldCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.PivotField[];
        /**
         *
         * Gets the number of pivot hierarchies in the collection.
         *
         * [Api set: ExcelApi 1.8]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a PivotHierarchy by its name or id.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param name Name of the PivotTable to be retrieved.
         */
        getItem(name: string): Excel.PivotField;
        /**
         *
         * Gets a PivotHierarchy by name. If the PivotHierarchy does not exist, will return a null object.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param name Name of the PivotHierarchy to be retrieved.
         */
        getItemOrNullObject(name: string): Excel.PivotField;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.PivotFieldCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.PivotFieldCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.PivotFieldCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.PivotFieldCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.PivotFieldCollection;
        load(option?: string | string[]): Excel.PivotFieldCollection;
        load(option?: OfficeExtension.LoadOption): Excel.PivotFieldCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.PivotFieldCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.PivotFieldCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.PivotFieldCollectionData;
    }
    /**
     *
     * Represents the Excel PivotField.
     *
     * [Api set: ExcelApi 1.8]
     */
    class PivotField extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns the PivotFields associated with the PivotField.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly items: Excel.PivotItemCollection;
        /**
         *
         * Id of the PivotField.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly id: string;
        /**
         *
         * Name of the PivotField.
         *
         * [Api set: ExcelApi 1.8]
         */
        name: string;
        /**
         *
         * Determines whether to show all items of the PivotField.
         *
         * [Api set: ExcelApi 1.8]
         */
        showAllItems: boolean;
        /**
         *
         * Subtotals of the PivotField.
         *
         * [Api set: ExcelApi 1.8]
         */
        subtotals: Excel.Subtotals;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.PivotField): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.PivotFieldUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.PivotField): void;
        /**
         *
         * Sorts the PivotField. If a DataPivotHierarchy is specified, then sort will be applied based on it, if not sort will be based on the PivotField itself.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param sortBy Represents whether the sorting is done in an ascending or descending order.
         */
        sortByLabels(sortBy: SortBy): void;
        /**
         *
         * Sorts the PivotField by specified values in a given scope. The scope defines which specific values will be used to sort when
            there are multiple values from the same DataPivotHierarchy.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param sortBy Represents whether the sorting is done in an ascending or descending order.
         * @param valuesHierarchy Specifies the values hierarchy on the data axis to be used for sorting.
         * @param pivotItemScope The items that should be used for the scope of the sorting. These will be the
            items that make up the row or column that you want to sort on. If a string is used instead of a PivotItem,
            the string represents the ID of the PivotItem. If there are no items other than data hierarchy on the axis
            you want to sort on, this can be empty.
         */
        sortByValues(sortBy: Excel.SortBy, valuesHierarchy: Excel.DataPivotHierarchy, pivotItemScope?: Array<PivotItem | string>): void;
        /**
         *
         * Sorts the PivotField by specified values in a given scope. The scope defines which specific values will be used to sort when
            there are multiple values from the same DataPivotHierarchy.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param sortBy Represents whether the sorting is done in an ascending or descending order.
         * @param valuesHierarchy Specifies the values hierarchy on the data axis to be used for sorting.
         * @param pivotItemScope The items that should be used for the scope of the sorting. These will be the
            items that make up the row or column that you want to sort on. If a string is used instead of a PivotItem,
            the string represents the ID of the PivotItem. If there are no items other than data hierarchy on the axis
            you want to sort on, this can be empty.
         */
        sortByValues(sortBy: "Ascending" | "Descending", valuesHierarchy: Excel.DataPivotHierarchy, pivotItemScope?: Array<PivotItem | string>): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.PivotField` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.PivotField` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.PivotField` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.PivotFieldLoadOptions): Excel.PivotField;
        load(option?: string | string[]): Excel.PivotField;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.PivotField;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.PivotField object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.PivotFieldData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.PivotFieldData;
    }
    /**
     *
     * Represents a collection of all the Pivot Items related to their parent PivotField.
     *
     * [Api set: ExcelApi 1.8]
     */
    class PivotItemCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.PivotItem[];
        /**
         *
         * Gets the number of pivot hierarchies in the collection.
         *
         * [Api set: ExcelApi 1.8]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a PivotHierarchy by its name or id.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param name Name of the PivotTable to be retrieved.
         */
        getItem(name: string): Excel.PivotItem;
        /**
         *
         * Gets a PivotHierarchy by name. If the PivotHierarchy does not exist, will return a null object.
         *
         * [Api set: ExcelApi 1.8]
         *
         * @param name Name of the PivotHierarchy to be retrieved.
         */
        getItemOrNullObject(name: string): Excel.PivotItem;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.PivotItemCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.PivotItemCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.PivotItemCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.PivotItemCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.PivotItemCollection;
        load(option?: string | string[]): Excel.PivotItemCollection;
        load(option?: OfficeExtension.LoadOption): Excel.PivotItemCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.PivotItemCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.PivotItemCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.PivotItemCollectionData;
    }
    /**
     *
     * Represents the Excel PivotItem.
     *
     * [Api set: ExcelApi 1.8]
     */
    class PivotItem extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Id of the PivotItem.
         *
         * [Api set: ExcelApi 1.8]
         */
        readonly id: string;
        /**
         *
         * Determines whether the item is expanded to show child items or if it's collapsed and child items are hidden.
         *
         * [Api set: ExcelApi 1.8]
         */
        isExpanded: boolean;
        /**
         *
         * Name of the PivotItem.
         *
         * [Api set: ExcelApi 1.8]
         */
        name: string;
        /**
         *
         * Determines whether the PivotItem is visible or not.
         *
         * [Api set: ExcelApi 1.8]
         */
        visible: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.PivotItem): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.PivotItemUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.PivotItem): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.PivotItem` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.PivotItem` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.PivotItem` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.PivotItemLoadOptions): Excel.PivotItem;
        load(option?: string | string[]): Excel.PivotItem;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.PivotItem;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.PivotItem object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.PivotItemData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.PivotItemData;
    }
    /**
     *
     * Subtotals for the Pivot Field.
     *
     * [Api set: ExcelApi 1.8]
     */
    interface Subtotals {
        /**
         *
         * If Automatic is set to true, then all other values will be ignored when setting the Subtotals.
         *
         * [Api set: ExcelApi 1.8]
         */
        automatic?: boolean;
        average?: boolean;
        count?: boolean;
        countNumbers?: boolean;
        max?: boolean;
        min?: boolean;
        product?: boolean;
        standardDeviation?: boolean;
        standardDeviationP?: boolean;
        sum?: boolean;
        variance?: boolean;
        varianceP?: boolean;
    }
    /**
     *
     * Represents the criteria for the top/bottom values filter.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum PivotFilterTopBottomCriterion {
        invalid = "Invalid",
        topItems = "TopItems",
        topPercent = "TopPercent",
        topSum = "TopSum",
        bottomItems = "BottomItems",
        bottomPercent = "BottomPercent",
        bottomSum = "BottomSum"
    }
    /**
     *
     * Represents the sort direction.
     *
     * [Api set: ExcelApi 1.8]
     */
    enum SortBy {
        /**
         *
         * Ascending sort. Smallest to largest or A to Z.
         *
         */
        ascending = "Ascending",
        /**
         *
         * Descending sort. Largest to smallest or Z to A.
         *
         */
        descending = "Descending"
    }
    /**
     *
     * Aggregation Function for the Data Pivot Field.
     *
     * [Api set: ExcelApi 1.8]
     */
    enum AggregationFunction {
        /**
         *
         * Aggregation function is unknown or unsupported.
         *
         */
        unknown = "Unknown",
        /**
         *
         * Excel will automatically select the aggregation based on the data items.
         *
         */
        automatic = "Automatic",
        /**
         *
         * Aggregate using the sum of the data, equivalent to the SUM function.
         *
         */
        sum = "Sum",
        /**
         *
         * Aggregate using the count of items in the data, equivalent to the COUNTA function.
         *
         */
        count = "Count",
        /**
         *
         * Aggregate using the average of the data, equivalent to the AVERAGE function.
         *
         */
        average = "Average",
        /**
         *
         * Aggregate using the maximum value of the data, equivalent to the MAX function.
         *
         */
        max = "Max",
        /**
         *
         * Aggregate using the minimum value of the data, equivalent to the MIN function.
         *
         */
        min = "Min",
        /**
         *
         * Aggregate using the product of the data, equivalent to the PRODUCT function.
         *
         */
        product = "Product",
        /**
         *
         * Aggregate using the count of numbers in the data, equivalent to the COUNT function.
         *
         */
        countNumbers = "CountNumbers",
        /**
         *
         * Aggregate using the standard deviation of the data, equivalent to the STDEV function.
         *
         */
        standardDeviation = "StandardDeviation",
        /**
         *
         * Aggregate using the standard deviation of the data, equivalent to the STDEVP function.
         *
         */
        standardDeviationP = "StandardDeviationP",
        /**
         *
         * Aggregate using the variance of the data, equivalent to the VAR function.
         *
         */
        variance = "Variance",
        /**
         *
         * Aggregate using the variance of the data, equivalent to the VARP function.
         *
         */
        varianceP = "VarianceP"
    }
    /**
     *
     * The ShowAs Calculation function for the Data Pivot Field.
     *
     * [Api set: ExcelApi 1.8]
     */
    enum ShowAsCalculation {
        /**
         *
         * Calculation is unknown or unsupported.
         *
         */
        unknown = "Unknown",
        /**
         *
         * No calculation is applied.
         *
         */
        none = "None",
        /**
         *
         * Percent of the grand total.
         *
         */
        percentOfGrandTotal = "PercentOfGrandTotal",
        /**
         *
         * Percent of the row total.
         *
         */
        percentOfRowTotal = "PercentOfRowTotal",
        /**
         *
         * Percent of the column total.
         *
         */
        percentOfColumnTotal = "PercentOfColumnTotal",
        /**
         *
         * Percent of the row total for the specified Base Field.
         *
         */
        percentOfParentRowTotal = "PercentOfParentRowTotal",
        /**
         *
         * Percent of the column total for the specified Base Field.
         *
         */
        percentOfParentColumnTotal = "PercentOfParentColumnTotal",
        /**
         *
         * Percent of the grand total for the specified Base Field.
         *
         */
        percentOfParentTotal = "PercentOfParentTotal",
        /**
         *
         * Percent of the specified Base Field and Base Item.
         *
         */
        percentOf = "PercentOf",
        /**
         *
         * Running Total of the specified Base Field.
         *
         */
        runningTotal = "RunningTotal",
        /**
         *
         * Percent Running Total of the specified Base Field.
         *
         */
        percentRunningTotal = "PercentRunningTotal",
        /**
         *
         * Difference from the specified Base Field and Base Item.
         *
         */
        differenceFrom = "DifferenceFrom",
        /**
         *
         * Difference from the specified Base Field and Base Item.
         *
         */
        percentDifferenceFrom = "PercentDifferenceFrom",
        /**
         *
         * Ascending Rank of the specified Base Field.
         *
         */
        rankAscending = "RankAscending",
        /**
         *
         * Descending Rank of the specified Base Field.
         *
         */
        rankDecending = "RankDecending",
        /**
         *
         * Calculates the values as follows:
            ((value in cell) x (Grand Total of Grand Totals)) / ((Grand Row Total) x (Grand Column Total))
         *
         */
        index = "Index"
    }
    /**
     *
     * The ShowAs Calculation function for the Data Pivot Field.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum PivotAxis {
        /**
         *
         * The axis or region is unknown or unsupported.
         *
         */
        unknown = "Unknown",
        /**
         *
         * The row axis.
         *
         */
        row = "Row",
        /**
         *
         * The column axis.
         *
         */
        column = "Column",
        /**
         *
         * The data axis.
         *
         */
        data = "Data",
        /**
         *
         * The filter axis.
         *
         */
        filter = "Filter"
    }
    /**
     *
     * Represents workbook properties.
     *
     * [Api set: ExcelApi 1.7]
     */
    class DocumentProperties extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the collection of custom properties of the workbook. Read only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly custom: Excel.CustomPropertyCollection;
        /**
         *
         * Gets or sets the author of the workbook.
         *
         * [Api set: ExcelApi 1.7]
         */
        author: string;
        /**
         *
         * Gets or sets the category of the workbook.
         *
         * [Api set: ExcelApi 1.7]
         */
        category: string;
        /**
         *
         * Gets or sets the comments of the workbook.
         *
         * [Api set: ExcelApi 1.7]
         */
        comments: string;
        /**
         *
         * Gets or sets the company of the workbook.
         *
         * [Api set: ExcelApi 1.7]
         */
        company: string;
        /**
         *
         * Gets the creation date of the workbook. Read only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly creationDate: Date;
        /**
         *
         * Gets or sets the keywords of the workbook.
         *
         * [Api set: ExcelApi 1.7]
         */
        keywords: string;
        /**
         *
         * Gets the last author of the workbook. Read only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly lastAuthor: string;
        /**
         *
         * Gets or sets the manager of the workbook.
         *
         * [Api set: ExcelApi 1.7]
         */
        manager: string;
        /**
         *
         * Gets the revision number of the workbook. Read only.
         *
         * [Api set: ExcelApi 1.7]
         */
        revisionNumber: number;
        /**
         *
         * Gets or sets the subject of the workbook.
         *
         * [Api set: ExcelApi 1.7]
         */
        subject: string;
        /**
         *
         * Gets or sets the title of the workbook.
         *
         * [Api set: ExcelApi 1.7]
         */
        title: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.DocumentProperties): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.DocumentPropertiesUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.DocumentProperties): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.DocumentProperties` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.DocumentProperties` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.DocumentProperties` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.DocumentPropertiesLoadOptions): Excel.DocumentProperties;
        load(option?: string | string[]): Excel.DocumentProperties;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.DocumentProperties;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.DocumentProperties object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.DocumentPropertiesData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.DocumentPropertiesData;
    }
    /**
     *
     * Represents a custom property.
     *
     * [Api set: ExcelApi 1.7]
     */
    class CustomProperty extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the key of the custom property. Read only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly key: string;
        /**
         *
         * Gets the value type of the custom property. Read only.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly type: Excel.DocumentPropertyType | "Number" | "Boolean" | "Date" | "String" | "Float";
        /**
         *
         * Gets or sets the value of the custom property.
         *
         * [Api set: ExcelApi 1.7]
         */
        value: any;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.CustomProperty): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.CustomPropertyUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.CustomProperty): void;
        /**
         *
         * Deletes the custom property.
         *
         * [Api set: ExcelApi 1.7]
         */
        delete(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.CustomProperty` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.CustomProperty` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.CustomProperty` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.CustomPropertyLoadOptions): Excel.CustomProperty;
        load(option?: string | string[]): Excel.CustomProperty;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.CustomProperty;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.CustomProperty object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.CustomPropertyData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.CustomPropertyData;
    }
    /**
     *
     * Contains the collection of customProperty objects.
     *
     * [Api set: ExcelApi 1.7]
     */
    class CustomPropertyCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.CustomProperty[];
        /**
         *
         * Creates a new or sets an existing custom property.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param key Required. The custom property's key, which is case-insensitive.
         * @param value Required. The custom property's value.
         */
        add(key: string, value: any): Excel.CustomProperty;
        /**
         *
         * Deletes all custom properties in this collection.
         *
         * [Api set: ExcelApi 1.7]
         */
        deleteAll(): void;
        /**
         *
         * Gets the count of custom properties.
         *
         * [Api set: ExcelApi 1.7]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a custom property object by its key, which is case-insensitive. Throws if the custom property does not exist.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param key The key that identifies the custom property object.
         */
        getItem(key: string): Excel.CustomProperty;
        /**
         *
         * Gets a custom property object by its key, which is case-insensitive. Returns a null object if the custom property does not exist.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param key Required. The key that identifies the custom property object.
         */
        getItemOrNullObject(key: string): Excel.CustomProperty;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.CustomPropertyCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.CustomPropertyCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.CustomPropertyCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.CustomPropertyCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.CustomPropertyCollection;
        load(option?: string | string[]): Excel.CustomPropertyCollection;
        load(option?: OfficeExtension.LoadOption): Excel.CustomPropertyCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.CustomPropertyCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.CustomPropertyCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.CustomPropertyCollectionData;
    }
    /**
     *
     * Represents a collection of all the conditional formats that are overlap the range.
     *
     * [Api set: ExcelApi 1.6]
     */
    class ConditionalFormatCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.ConditionalFormat[];
        /**
         *
         * Adds a new conditional format to the collection at the first/top priority.
         *
         * [Api set: ExcelApi 1.6]
         *
         * @param type The type of conditional format being added. See Excel.ConditionalFormatType for details.
         */
        add(type: Excel.ConditionalFormatType): Excel.ConditionalFormat;
        /**
         *
         * Adds a new conditional format to the collection at the first/top priority.
         *
         * [Api set: ExcelApi 1.6]
         *
         * @param type The type of conditional format being added. See Excel.ConditionalFormatType for details.
         */
        add(type: "Custom" | "DataBar" | "ColorScale" | "IconSet" | "TopBottom" | "PresetCriteria" | "ContainsText" | "CellValue"): Excel.ConditionalFormat;
        /**
         *
         * Clears all conditional formats active on the current specified range.
         *
         * [Api set: ExcelApi 1.6]
         */
        clearAll(): void;
        /**
         *
         * Returns the number of conditional formats in the workbook. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Returns a conditional format for the given ID.
         *
         * [Api set: ExcelApi 1.6]
         *
         * @param id The id of the conditional format.
         * @returns Conditional Format object.
         */
        getItem(id: string): Excel.ConditionalFormat;
        /**
         *
         * Returns a conditional format at the given index.
         *
         * [Api set: ExcelApi 1.6]
         *
         * @param index Index of the conditional formats to be retrieved.
         */
        getItemAt(index: number): Excel.ConditionalFormat;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ConditionalFormatCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ConditionalFormatCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ConditionalFormatCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ConditionalFormatCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.ConditionalFormatCollection;
        load(option?: string | string[]): Excel.ConditionalFormatCollection;
        load(option?: OfficeExtension.LoadOption): Excel.ConditionalFormatCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.ConditionalFormatCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ConditionalFormatCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.ConditionalFormatCollectionData;
    }
    /**
     *
     * An object encapsulating a conditional format's range, format, rule, and other properties.
     *
     * [Api set: ExcelApi 1.6]
     */
    class ConditionalFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns the cell value conditional format properties if the current conditional format is a CellValue type.
            For example to format all cells between 5 and 10. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly cellValue: Excel.CellValueConditionalFormat;
        /**
         *
         * Returns the cell value conditional format properties if the current conditional format is a CellValue type.
            For example to format all cells between 5 and 10. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly cellValueOrNullObject: Excel.CellValueConditionalFormat;
        /**
         *
         * Returns the ColorScale conditional format properties if the current conditional format is an ColorScale type. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly colorScale: Excel.ColorScaleConditionalFormat;
        /**
         *
         * Returns the ColorScale conditional format properties if the current conditional format is an ColorScale type. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly colorScaleOrNullObject: Excel.ColorScaleConditionalFormat;
        /**
         *
         * Returns the custom conditional format properties if the current conditional format is a custom type. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly custom: Excel.CustomConditionalFormat;
        /**
         *
         * Returns the custom conditional format properties if the current conditional format is a custom type. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly customOrNullObject: Excel.CustomConditionalFormat;
        /**
         *
         * Returns the data bar properties if the current conditional format is a data bar. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly dataBar: Excel.DataBarConditionalFormat;
        /**
         *
         * Returns the data bar properties if the current conditional format is a data bar. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly dataBarOrNullObject: Excel.DataBarConditionalFormat;
        /**
         *
         * Returns the IconSet conditional format properties if the current conditional format is an IconSet type. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly iconSet: Excel.IconSetConditionalFormat;
        /**
         *
         * Returns the IconSet conditional format properties if the current conditional format is an IconSet type. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly iconSetOrNullObject: Excel.IconSetConditionalFormat;
        /**
         *
         * Returns the preset criteria conditional format. See Excel.PresetCriteriaConditionalFormat for more details.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly preset: Excel.PresetCriteriaConditionalFormat;
        /**
         *
         * Returns the preset criteria conditional format. See Excel.PresetCriteriaConditionalFormat for more details.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly presetOrNullObject: Excel.PresetCriteriaConditionalFormat;
        /**
         *
         * Returns the specific text conditional format properties if the current conditional format is a text type.
            For example to format cells matching the word "Text". Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly textComparison: Excel.TextConditionalFormat;
        /**
         *
         * Returns the specific text conditional format properties if the current conditional format is a text type.
            For example to format cells matching the word "Text". Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly textComparisonOrNullObject: Excel.TextConditionalFormat;
        /**
         *
         * Returns the Top/Bottom conditional format properties if the current conditional format is an TopBottom type.
            For example to format the top 10% or bottom 10 items. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly topBottom: Excel.TopBottomConditionalFormat;
        /**
         *
         * Returns the Top/Bottom conditional format properties if the current conditional format is an TopBottom type.
            For example to format the top 10% or bottom 10 items. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly topBottomOrNullObject: Excel.TopBottomConditionalFormat;
        /**
         *
         * The Priority of the Conditional Format within the current ConditionalFormatCollection. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly id: string;
        /**
         *
         * The priority (or index) within the conditional format collection that this conditional format currently exists in. Changing this also
            changes other conditional formats' priorities, to allow for a contiguous priority order.
            Use a negative priority to begin from the back.
            Priorities greater than than bounds will get and set to the maximum (or minimum if negative) priority.
            Also note that if you change the priority, you have to re-fetch a new copy of the object at that new priority location if you want to make further changes to it. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        priority: number;
        /**
         *
         * If the conditions of this conditional format are met, no lower-priority formats shall take effect on that cell.
            Null on databars, icon sets, and colorscales as there's no concept of StopIfTrue for these
         *
         * [Api set: ExcelApi 1.6]
         */
        stopIfTrue: boolean;
        /**
         *
         * A type of conditional format. Only one can be set at a time. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly type: Excel.ConditionalFormatType | "Custom" | "DataBar" | "ColorScale" | "IconSet" | "TopBottom" | "PresetCriteria" | "ContainsText" | "CellValue";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ConditionalFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ConditionalFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ConditionalFormat): void;
        /**
         *
         * Deletes this conditional format.
         *
         * [Api set: ExcelApi 1.6]
         */
        delete(): void;
        /**
         *
         * Returns the range the conditonal format is applied to. Throws an error if the conditional format is applied to multiple ranges. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        getRange(): Excel.Range;
        /**
         *
         * Returns the range the conditonal format is applied to, or a null object if the conditional format is applied to multiple ranges. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        getRangeOrNullObject(): Excel.Range;
        /**
         *
         * Returns the RangeAreas, comprising one or more rectangular ranges, the conditonal format is applied to. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        getRanges(): Excel.RangeAreas;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ConditionalFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ConditionalFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ConditionalFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ConditionalFormatLoadOptions): Excel.ConditionalFormat;
        load(option?: string | string[]): Excel.ConditionalFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ConditionalFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ConditionalFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ConditionalFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ConditionalFormatData;
    }
    /**
     *
     * Represents an Excel Conditional Data Bar Type.
     *
     * [Api set: ExcelApi 1.6]
     */
    class DataBarConditionalFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Representation of all values to the left of the axis in an Excel data bar. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly negativeFormat: Excel.ConditionalDataBarNegativeFormat;
        /**
         *
         * Representation of all values to the right of the axis in an Excel data bar. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly positiveFormat: Excel.ConditionalDataBarPositiveFormat;
        /**
         *
         * HTML color code representing the color of the Axis line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            "" (empty string) if no axis is present or set.
         *
         * [Api set: ExcelApi 1.6]
         */
        axisColor: string;
        /**
         *
         * Representation of how the axis is determined for an Excel data bar.
         *
         * [Api set: ExcelApi 1.6]
         */
        axisFormat: Excel.ConditionalDataBarAxisFormat | "Automatic" | "None" | "CellMidPoint";
        /**
         *
         * Represents the direction that the data bar graphic should be based on.
         *
         * [Api set: ExcelApi 1.6]
         */
        barDirection: Excel.ConditionalDataBarDirection | "Context" | "LeftToRight" | "RightToLeft";
        /**
         *
         * The rule for what consistutes the lower bound (and how to calculate it, if applicable) for a data bar.
         *
         * [Api set: ExcelApi 1.6]
         */
        lowerBoundRule: Excel.ConditionalDataBarRule;
        /**
         *
         * If true, hides the values from the cells where the data bar is applied.
         *
         * [Api set: ExcelApi 1.6]
         */
        showDataBarOnly: boolean;
        /**
         *
         * The rule for what constitutes the upper bound (and how to calculate it, if applicable) for a data bar.
         *
         * [Api set: ExcelApi 1.6]
         */
        upperBoundRule: Excel.ConditionalDataBarRule;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.DataBarConditionalFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.DataBarConditionalFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.DataBarConditionalFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.DataBarConditionalFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.DataBarConditionalFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.DataBarConditionalFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.DataBarConditionalFormatLoadOptions): Excel.DataBarConditionalFormat;
        load(option?: string | string[]): Excel.DataBarConditionalFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.DataBarConditionalFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.DataBarConditionalFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.DataBarConditionalFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.DataBarConditionalFormatData;
    }
    /**
     *
     * Represents a conditional format DataBar Format for the positive side of the data bar.
     *
     * [Api set: ExcelApi 1.6]
     */
    class ConditionalDataBarPositiveFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            "" (empty string) if no border is present or set.
         *
         * [Api set: ExcelApi 1.6]
         */
        borderColor: string;
        /**
         *
         * HTML color code representing the fill color, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
         *
         * [Api set: ExcelApi 1.6]
         */
        fillColor: string;
        /**
         *
         * Boolean representation of whether or not the DataBar has a gradient.
         *
         * [Api set: ExcelApi 1.6]
         */
        gradientFill: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ConditionalDataBarPositiveFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ConditionalDataBarPositiveFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ConditionalDataBarPositiveFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ConditionalDataBarPositiveFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ConditionalDataBarPositiveFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ConditionalDataBarPositiveFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ConditionalDataBarPositiveFormatLoadOptions): Excel.ConditionalDataBarPositiveFormat;
        load(option?: string | string[]): Excel.ConditionalDataBarPositiveFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ConditionalDataBarPositiveFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ConditionalDataBarPositiveFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ConditionalDataBarPositiveFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ConditionalDataBarPositiveFormatData;
    }
    /**
     *
     * Represents a conditional format DataBar Format for the negative side of the data bar.
     *
     * [Api set: ExcelApi 1.6]
     */
    class ConditionalDataBarNegativeFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            "Empty String" if no border is present or set.
         *
         * [Api set: ExcelApi 1.6]
         */
        borderColor: string;
        /**
         *
         * HTML color code representing the fill color, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
         *
         * [Api set: ExcelApi 1.6]
         */
        fillColor: string;
        /**
         *
         * Boolean representation of whether or not the negative DataBar has the same border color as the positive DataBar.
         *
         * [Api set: ExcelApi 1.6]
         */
        matchPositiveBorderColor: boolean;
        /**
         *
         * Boolean representation of whether or not the negative DataBar has the same fill color as the positive DataBar.
         *
         * [Api set: ExcelApi 1.6]
         */
        matchPositiveFillColor: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ConditionalDataBarNegativeFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ConditionalDataBarNegativeFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ConditionalDataBarNegativeFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ConditionalDataBarNegativeFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ConditionalDataBarNegativeFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ConditionalDataBarNegativeFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ConditionalDataBarNegativeFormatLoadOptions): Excel.ConditionalDataBarNegativeFormat;
        load(option?: string | string[]): Excel.ConditionalDataBarNegativeFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ConditionalDataBarNegativeFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ConditionalDataBarNegativeFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ConditionalDataBarNegativeFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ConditionalDataBarNegativeFormatData;
    }
    /**
     *
     * Represents a rule-type for a Data Bar.
     *
     * [Api set: ExcelApi 1.6]
     */
    interface ConditionalDataBarRule {
        /**
         *
         * The formula, if required, to evaluate the databar rule on.
         *
         * [Api set: ExcelApi 1.6]
         */
        formula?: string;
        /**
         *
         * The type of rule for the databar.
         *
         * [Api set: ExcelApi 1.6]
         */
        type: Excel.ConditionalFormatRuleType | "Invalid" | "Automatic" | "LowestValue" | "HighestValue" | "Number" | "Percent" | "Formula" | "Percentile";
    }
    /**
     *
     * Represents a custom conditional format type.
     *
     * [Api set: ExcelApi 1.6]
     */
    class CustomConditionalFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly format: Excel.ConditionalRangeFormat;
        /**
         *
         * Represents the Rule object on this conditional format. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly rule: Excel.ConditionalFormatRule;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.CustomConditionalFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.CustomConditionalFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.CustomConditionalFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.CustomConditionalFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.CustomConditionalFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.CustomConditionalFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.CustomConditionalFormatLoadOptions): Excel.CustomConditionalFormat;
        load(option?: string | string[]): Excel.CustomConditionalFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.CustomConditionalFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.CustomConditionalFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.CustomConditionalFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.CustomConditionalFormatData;
    }
    /**
     *
     * Represents a rule, for all traditional rule/format pairings.
     *
     * [Api set: ExcelApi 1.6]
     */
    class ConditionalFormatRule extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * The formula, if required, to evaluate the conditional format rule on.
         *
         * [Api set: ExcelApi 1.6]
         */
        formula: string;
        /**
         *
         * The formula, if required, to evaluate the conditional format rule on in the user's language.
         *
         * [Api set: ExcelApi 1.6]
         */
        formulaLocal: string;
        /**
         *
         * The formula, if required, to evaluate the conditional format rule on in R1C1-style notation.
         *
         * [Api set: ExcelApi 1.6]
         */
        formulaR1C1: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ConditionalFormatRule): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ConditionalFormatRuleUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ConditionalFormatRule): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ConditionalFormatRule` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ConditionalFormatRule` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ConditionalFormatRule` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ConditionalFormatRuleLoadOptions): Excel.ConditionalFormatRule;
        load(option?: string | string[]): Excel.ConditionalFormatRule;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ConditionalFormatRule;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ConditionalFormatRule object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ConditionalFormatRuleData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ConditionalFormatRuleData;
    }
    /**
     *
     * Represents an IconSet criteria for conditional formatting.
     *
     * [Api set: ExcelApi 1.6]
     */
    class IconSetConditionalFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * An array of Criteria and IconSets for the rules and potential custom icons for conditional icons. Note that for the first criterion only the custom icon can be modified, while type, formula, and operator will be ignored when set.
         *
         * [Api set: ExcelApi 1.6]
         */
        criteria: Excel.ConditionalIconCriterion[];
        /**
         *
         * If true, reverses the icon orders for the IconSet. Note that this cannot be set if custom icons are used.
         *
         * [Api set: ExcelApi 1.6]
         */
        reverseIconOrder: boolean;
        /**
         *
         * If true, hides the values and only shows icons.
         *
         * [Api set: ExcelApi 1.6]
         */
        showIconOnly: boolean;
        /**
         *
         * If set, displays the IconSet option for the conditional format.
         *
         * [Api set: ExcelApi 1.6]
         */
        style: Excel.IconSet | "Invalid" | "ThreeArrows" | "ThreeArrowsGray" | "ThreeFlags" | "ThreeTrafficLights1" | "ThreeTrafficLights2" | "ThreeSigns" | "ThreeSymbols" | "ThreeSymbols2" | "FourArrows" | "FourArrowsGray" | "FourRedToBlack" | "FourRating" | "FourTrafficLights" | "FiveArrows" | "FiveArrowsGray" | "FiveRating" | "FiveQuarters" | "ThreeStars" | "ThreeTriangles" | "FiveBoxes" | "LinkedEntityFinanceIcon" | "LinkedEntityMapIcon";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.IconSetConditionalFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.IconSetConditionalFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.IconSetConditionalFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.IconSetConditionalFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.IconSetConditionalFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.IconSetConditionalFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.IconSetConditionalFormatLoadOptions): Excel.IconSetConditionalFormat;
        load(option?: string | string[]): Excel.IconSetConditionalFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.IconSetConditionalFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.IconSetConditionalFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.IconSetConditionalFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.IconSetConditionalFormatData;
    }
    /**
     *
     * Represents an Icon Criterion which contains a type, value, an Operator, and an optional custom icon, if not using an iconset.
     *
     * [Api set: ExcelApi 1.6]
     */
    interface ConditionalIconCriterion {
        /**
         *
         * The custom icon for the current criterion if different from the default IconSet, else null will be returned.
         *
         * [Api set: ExcelApi 1.6]
         */
        customIcon?: Excel.Icon;
        /**
         *
         * A number or a formula depending on the type.
         *
         * [Api set: ExcelApi 1.6]
         */
        formula: string;
        /**
         *
         * GreaterThan or GreaterThanOrEqual for each of the rule type for the Icon conditional format.
         *
         * [Api set: ExcelApi 1.6]
         */
        operator: Excel.ConditionalIconCriterionOperator | "Invalid" | "GreaterThan" | "GreaterThanOrEqual";
        /**
         *
         * What the icon conditional formula should be based on.
         *
         * [Api set: ExcelApi 1.6]
         */
        type: Excel.ConditionalFormatIconRuleType | "Invalid" | "Number" | "Percent" | "Formula" | "Percentile";
    }
    /**
     *
     * Represents an IconSet criteria for conditional formatting.
     *
     * [Api set: ExcelApi 1.6]
     */
    class ColorScaleConditionalFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * The criteria of the color scale. Midpoint is optional when using a two point color scale.
         *
         * [Api set: ExcelApi 1.6]
         */
        criteria: Excel.ConditionalColorScaleCriteria;
        /**
         *
         * If true the color scale will have three points (minimum, midpoint, maximum), otherwise it will have two (minimum, maximum).
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly threeColorScale: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ColorScaleConditionalFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ColorScaleConditionalFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ColorScaleConditionalFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ColorScaleConditionalFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ColorScaleConditionalFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ColorScaleConditionalFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ColorScaleConditionalFormatLoadOptions): Excel.ColorScaleConditionalFormat;
        load(option?: string | string[]): Excel.ColorScaleConditionalFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ColorScaleConditionalFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ColorScaleConditionalFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ColorScaleConditionalFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ColorScaleConditionalFormatData;
    }
    /**
     *
     * Represents the criteria of the color scale.
     *
     * [Api set: ExcelApi 1.6]
     */
    interface ConditionalColorScaleCriteria {
        /**
         *
         * The maximum point Color Scale Criterion.
         *
         * [Api set: ExcelApi 1.6]
         */
        maximum: Excel.ConditionalColorScaleCriterion;
        /**
         *
         * The midpoint Color Scale Criterion if the color scale is a 3-color scale.
         *
         * [Api set: ExcelApi 1.6]
         */
        midpoint?: Excel.ConditionalColorScaleCriterion;
        /**
         *
         * The minimum point Color Scale Criterion.
         *
         * [Api set: ExcelApi 1.6]
         */
        minimum: Excel.ConditionalColorScaleCriterion;
    }
    /**
     *
     * Represents a Color Scale Criterion which contains a type, value, and a color.
     *
     * [Api set: ExcelApi 1.6]
     */
    interface ConditionalColorScaleCriterion {
        /**
         *
         * HTML color code representation of the color scale color. E.g. #FF0000 represents Red.
         *
         * [Api set: ExcelApi 1.6]
         */
        color?: string;
        /**
         *
         * A number, a formula, or null (if Type is LowestValue).
         *
         * [Api set: ExcelApi 1.6]
         */
        formula?: string;
        /**
         *
         * What the criterion conditional formula should be based on.
         *
         * [Api set: ExcelApi 1.6]
         */
        type: Excel.ConditionalFormatColorCriterionType | "Invalid" | "LowestValue" | "HighestValue" | "Number" | "Percent" | "Formula" | "Percentile";
    }
    /**
     *
     * Represents a Top/Bottom conditional format.
     *
     * [Api set: ExcelApi 1.6]
     */
    class TopBottomConditionalFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly format: Excel.ConditionalRangeFormat;
        /**
         *
         * The criteria of the Top/Bottom conditional format.
         *
         * [Api set: ExcelApi 1.6]
         */
        rule: Excel.ConditionalTopBottomRule;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.TopBottomConditionalFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.TopBottomConditionalFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.TopBottomConditionalFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.TopBottomConditionalFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.TopBottomConditionalFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.TopBottomConditionalFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.TopBottomConditionalFormatLoadOptions): Excel.TopBottomConditionalFormat;
        load(option?: string | string[]): Excel.TopBottomConditionalFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.TopBottomConditionalFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.TopBottomConditionalFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.TopBottomConditionalFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.TopBottomConditionalFormatData;
    }
    /**
     *
     * Represents the rule of the top/bottom conditional format.
     *
     * [Api set: ExcelApi 1.6]
     */
    interface ConditionalTopBottomRule {
        /**
         *
         * The rank between 1 and 1000 for numeric ranks or 1 and 100 for percent ranks.
         *
         * [Api set: ExcelApi 1.6]
         */
        rank: number;
        /**
         *
         * Format values based on the top or bottom rank.
         *
         * [Api set: ExcelApi 1.6]
         */
        type: Excel.ConditionalTopBottomCriterionType | "Invalid" | "TopItems" | "TopPercent" | "BottomItems" | "BottomPercent";
    }
    /**
     *
     * Represents the the preset criteria conditional format such as above average, below average, unique values, contains blank, nonblank, error, and noerror.
     *
     * [Api set: ExcelApi 1.6]
     */
    class PresetCriteriaConditionalFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly format: Excel.ConditionalRangeFormat;
        /**
         *
         * The rule of the conditional format.
         *
         * [Api set: ExcelApi 1.6]
         */
        rule: Excel.ConditionalPresetCriteriaRule;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.PresetCriteriaConditionalFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.PresetCriteriaConditionalFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.PresetCriteriaConditionalFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.PresetCriteriaConditionalFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.PresetCriteriaConditionalFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.PresetCriteriaConditionalFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.PresetCriteriaConditionalFormatLoadOptions): Excel.PresetCriteriaConditionalFormat;
        load(option?: string | string[]): Excel.PresetCriteriaConditionalFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.PresetCriteriaConditionalFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.PresetCriteriaConditionalFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.PresetCriteriaConditionalFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.PresetCriteriaConditionalFormatData;
    }
    /**
     *
     * Represents the Preset Criteria Conditional Format Rule
     *
     * [Api set: ExcelApi 1.6]
     */
    interface ConditionalPresetCriteriaRule {
        /**
         *
         * The criterion of the conditional format.
         *
         * [Api set: ExcelApi 1.6]
         */
        criterion: Excel.ConditionalFormatPresetCriterion | "Invalid" | "Blanks" | "NonBlanks" | "Errors" | "NonErrors" | "Yesterday" | "Today" | "Tomorrow" | "LastSevenDays" | "LastWeek" | "ThisWeek" | "NextWeek" | "LastMonth" | "ThisMonth" | "NextMonth" | "AboveAverage" | "BelowAverage" | "EqualOrAboveAverage" | "EqualOrBelowAverage" | "OneStdDevAboveAverage" | "OneStdDevBelowAverage" | "TwoStdDevAboveAverage" | "TwoStdDevBelowAverage" | "ThreeStdDevAboveAverage" | "ThreeStdDevBelowAverage" | "UniqueValues" | "DuplicateValues";
    }
    /**
     *
     * Represents a specific text conditional format.
     *
     * [Api set: ExcelApi 1.6]
     */
    class TextConditionalFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly format: Excel.ConditionalRangeFormat;
        /**
         *
         * The rule of the conditional format.
         *
         * [Api set: ExcelApi 1.6]
         */
        rule: Excel.ConditionalTextComparisonRule;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.TextConditionalFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.TextConditionalFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.TextConditionalFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.TextConditionalFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.TextConditionalFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.TextConditionalFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.TextConditionalFormatLoadOptions): Excel.TextConditionalFormat;
        load(option?: string | string[]): Excel.TextConditionalFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.TextConditionalFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.TextConditionalFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.TextConditionalFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.TextConditionalFormatData;
    }
    /**
     *
     * Represents a Cell Value Conditional Format Rule
     *
     * [Api set: ExcelApi 1.6]
     */
    interface ConditionalTextComparisonRule {
        /**
         *
         * The operator of the text conditional format.
         *
         * [Api set: ExcelApi 1.6]
         */
        operator: Excel.ConditionalTextOperator | "Invalid" | "Contains" | "NotContains" | "BeginsWith" | "EndsWith";
        /**
         *
         * The Text value of conditional format.
         *
         * [Api set: ExcelApi 1.6]
         */
        text: string;
    }
    /**
     *
     * Represents a cell value conditional format.
     *
     * [Api set: ExcelApi 1.6]
     */
    class CellValueConditionalFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly format: Excel.ConditionalRangeFormat;
        /**
         *
         * Represents the Rule object on this conditional format.
         *
         * [Api set: ExcelApi 1.6]
         */
        rule: Excel.ConditionalCellValueRule;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.CellValueConditionalFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.CellValueConditionalFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.CellValueConditionalFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.CellValueConditionalFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.CellValueConditionalFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.CellValueConditionalFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.CellValueConditionalFormatLoadOptions): Excel.CellValueConditionalFormat;
        load(option?: string | string[]): Excel.CellValueConditionalFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.CellValueConditionalFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.CellValueConditionalFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.CellValueConditionalFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.CellValueConditionalFormatData;
    }
    /**
     *
     * Represents a cell value conditional format rule.
     *
     * [Api set: ExcelApi 1.6]
     */
    interface ConditionalCellValueRule {
        /**
         *
         * The formula, if required, to evaluate the conditional format rule on.
         *
         * [Api set: ExcelApi 1.6]
         */
        formula1: string;
        /**
         *
         * The formula, if required, to evaluate the conditional format rule on.
         *
         * [Api set: ExcelApi 1.6]
         */
        formula2?: string;
        /**
         *
         * The operator of the text conditional format.
         *
         * [Api set: ExcelApi 1.6]
         */
        operator: Excel.ConditionalCellValueOperator | "Invalid" | "Between" | "NotBetween" | "EqualTo" | "NotEqualTo" | "GreaterThan" | "LessThan" | "GreaterThanOrEqual" | "LessThanOrEqual";
    }
    /**
     *
     * A format object encapsulating the conditional formats range's font, fill, borders, and other properties.
     *
     * [Api set: ExcelApi 1.6]
     */
    class ConditionalRangeFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Collection of border objects that apply to the overall conditional format range. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly borders: Excel.ConditionalRangeBorderCollection;
        /**
         *
         * Returns the fill object defined on the overall conditional format range. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly fill: Excel.ConditionalRangeFill;
        /**
         *
         * Returns the font object defined on the overall conditional format range. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly font: Excel.ConditionalRangeFont;
        /**
         *
         * Represents Excel's number format code for the given range. Cleared if null is passed in.
         *
         * [Api set: ExcelApi 1.6]
         */
        numberFormat: any;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ConditionalRangeFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ConditionalRangeFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ConditionalRangeFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ConditionalRangeFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ConditionalRangeFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ConditionalRangeFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ConditionalRangeFormatLoadOptions): Excel.ConditionalRangeFormat;
        load(option?: string | string[]): Excel.ConditionalRangeFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ConditionalRangeFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ConditionalRangeFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ConditionalRangeFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ConditionalRangeFormatData;
    }
    /**
     *
     * This object represents the font attributes (font style, color, etc.) for an object.
     *
     * [Api set: ExcelApi 1.6]
     */
    class ConditionalRangeFont extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the bold status of font.
         *
         * [Api set: ExcelApi 1.6]
         */
        bold: boolean;
        /**
         *
         * HTML color code representation of the text color. E.g. #FF0000 represents Red.
         *
         * [Api set: ExcelApi 1.6]
         */
        color: string;
        /**
         *
         * Represents the italic status of the font.
         *
         * [Api set: ExcelApi 1.6]
         */
        italic: boolean;
        /**
         *
         * Represents the strikethrough status of the font.
         *
         * [Api set: ExcelApi 1.6]
         */
        strikethrough: boolean;
        /**
         *
         * Type of underline applied to the font. See Excel.ConditionalRangeFontUnderlineStyle for details.
         *
         * [Api set: ExcelApi 1.6]
         */
        underline: Excel.ConditionalRangeFontUnderlineStyle | "None" | "Single" | "Double";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ConditionalRangeFont): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ConditionalRangeFontUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ConditionalRangeFont): void;
        /**
         *
         * Resets the font formats.
         *
         * [Api set: ExcelApi 1.6]
         */
        clear(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ConditionalRangeFont` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ConditionalRangeFont` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ConditionalRangeFont` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ConditionalRangeFontLoadOptions): Excel.ConditionalRangeFont;
        load(option?: string | string[]): Excel.ConditionalRangeFont;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ConditionalRangeFont;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ConditionalRangeFont object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ConditionalRangeFontData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ConditionalRangeFontData;
    }
    /**
     *
     * Represents the background of a conditional range object.
     *
     * [Api set: ExcelApi 1.6]
     */
    class ConditionalRangeFill extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * HTML color code representing the color of the fill, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
         *
         * [Api set: ExcelApi 1.6]
         */
        color: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ConditionalRangeFill): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ConditionalRangeFillUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ConditionalRangeFill): void;
        /**
         *
         * Resets the fill.
         *
         * [Api set: ExcelApi 1.6]
         */
        clear(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ConditionalRangeFill` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ConditionalRangeFill` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ConditionalRangeFill` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ConditionalRangeFillLoadOptions): Excel.ConditionalRangeFill;
        load(option?: string | string[]): Excel.ConditionalRangeFill;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ConditionalRangeFill;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ConditionalRangeFill object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ConditionalRangeFillData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ConditionalRangeFillData;
    }
    /**
     *
     * Represents the border of an object.
     *
     * [Api set: ExcelApi 1.6]
     */
    class ConditionalRangeBorder extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
         *
         * [Api set: ExcelApi 1.6]
         */
        color: string;
        /**
         *
         * Constant value that indicates the specific side of the border. See Excel.ConditionalRangeBorderIndex for details. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly sideIndex: Excel.ConditionalRangeBorderIndex | "EdgeTop" | "EdgeBottom" | "EdgeLeft" | "EdgeRight";
        /**
         *
         * One of the constants of line style specifying the line style for the border. See Excel.BorderLineStyle for details.
         *
         * [Api set: ExcelApi 1.6]
         */
        style: Excel.ConditionalRangeBorderLineStyle | "None" | "Continuous" | "Dash" | "DashDot" | "DashDotDot" | "Dot";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ConditionalRangeBorder): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ConditionalRangeBorderUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ConditionalRangeBorder): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ConditionalRangeBorder` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ConditionalRangeBorder` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ConditionalRangeBorder` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ConditionalRangeBorderLoadOptions): Excel.ConditionalRangeBorder;
        load(option?: string | string[]): Excel.ConditionalRangeBorder;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ConditionalRangeBorder;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ConditionalRangeBorder object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ConditionalRangeBorderData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ConditionalRangeBorderData;
    }
    /**
     *
     * Represents the border objects that make up range border.
     *
     * [Api set: ExcelApi 1.6]
     */
    class ConditionalRangeBorderCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the bottom border. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly bottom: Excel.ConditionalRangeBorder;
        /**
         *
         * Gets the left border. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly left: Excel.ConditionalRangeBorder;
        /**
         *
         * Gets the right border. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly right: Excel.ConditionalRangeBorder;
        /**
         *
         * Gets the top border. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly top: Excel.ConditionalRangeBorder;
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.ConditionalRangeBorder[];
        /**
         *
         * Number of border objects in the collection. Read-only.
         *
         * [Api set: ExcelApi 1.6]
         */
        readonly count: number;
        /**
         *
         * Gets a border object using its name.
         *
         * [Api set: ExcelApi 1.6]
         *
         * @param index Index value of the border object to be retrieved. See Excel.ConditionalRangeBorderIndex for details.
         */
        getItem(index: Excel.ConditionalRangeBorderIndex): Excel.ConditionalRangeBorder;
        /**
         *
         * Gets a border object using its name.
         *
         * [Api set: ExcelApi 1.6]
         *
         * @param index Index value of the border object to be retrieved. See Excel.ConditionalRangeBorderIndex for details.
         */
        getItem(index: "EdgeTop" | "EdgeBottom" | "EdgeLeft" | "EdgeRight"): Excel.ConditionalRangeBorder;
        /**
         *
         * Gets a border object using its index.
         *
         * [Api set: ExcelApi 1.6]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): Excel.ConditionalRangeBorder;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ConditionalRangeBorderCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ConditionalRangeBorderCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ConditionalRangeBorderCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ConditionalRangeBorderCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.ConditionalRangeBorderCollection;
        load(option?: string | string[]): Excel.ConditionalRangeBorderCollection;
        load(option?: OfficeExtension.LoadOption): Excel.ConditionalRangeBorderCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.ConditionalRangeBorderCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ConditionalRangeBorderCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.ConditionalRangeBorderCollectionData;
    }
    /**
     *
     * An object encapsulating a style's format and other properties.
     *
     * [Api set: ExcelApi 1.7]
     */
    class Style extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * A Border collection of four Border objects that represent the style of the four borders.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly borders: Excel.RangeBorderCollection;
        /**
         *
         * The Fill of the style.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly fill: Excel.RangeFill;
        /**
         *
         * A Font object that represents the font of the style.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly font: Excel.RangeFont;
        /**
         *
         * Indicates if text is automatically indented when the text alignment in a cell is set to equal distribution.
         *
         * [Api set: ExcelApi 1.8]
         */
        autoIndent: boolean;
        /**
         *
         * Indicates if the style is a built-in style.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly builtIn: boolean;
        /**
         *
         * Indicates if the formula will be hidden when the worksheet is protected.
         *
         * [Api set: ExcelApi 1.7]
         */
        formulaHidden: boolean;
        /**
         *
         * Represents the horizontal alignment for the style. See Excel.HorizontalAlignment for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        horizontalAlignment: Excel.HorizontalAlignment | "General" | "Left" | "Center" | "Right" | "Fill" | "Justify" | "CenterAcrossSelection" | "Distributed";
        /**
         *
         * Indicates if the style includes the AutoIndent, HorizontalAlignment, VerticalAlignment, WrapText, IndentLevel, and TextOrientation properties.
         *
         * [Api set: ExcelApi 1.7]
         */
        includeAlignment: boolean;
        /**
         *
         * Indicates if the style includes the Color, ColorIndex, LineStyle, and Weight border properties.
         *
         * [Api set: ExcelApi 1.7]
         */
        includeBorder: boolean;
        /**
         *
         * Indicates if the style includes the Background, Bold, Color, ColorIndex, FontStyle, Italic, Name, Size, Strikethrough, Subscript, Superscript, and Underline font properties.
         *
         * [Api set: ExcelApi 1.7]
         */
        includeFont: boolean;
        /**
         *
         * Indicates if the style includes the NumberFormat property.
         *
         * [Api set: ExcelApi 1.7]
         */
        includeNumber: boolean;
        /**
         *
         * Indicates if the style includes the Color, ColorIndex, InvertIfNegative, Pattern, PatternColor, and PatternColorIndex interior properties.
         *
         * [Api set: ExcelApi 1.7]
         */
        includePatterns: boolean;
        /**
         *
         * Indicates if the style includes the FormulaHidden and Locked protection properties.
         *
         * [Api set: ExcelApi 1.7]
         */
        includeProtection: boolean;
        /**
         *
         * An integer from 0 to 250 that indicates the indent level for the style.
         *
         * [Api set: ExcelApi 1.7]
         */
        indentLevel: number;
        /**
         *
         * Indicates if the object is locked when the worksheet is protected.
         *
         * [Api set: ExcelApi 1.7]
         */
        locked: boolean;
        /**
         *
         * The name of the style.
         *
         * [Api set: ExcelApi 1.7]
         */
        readonly name: string;
        /**
         *
         * The format code of the number format for the style.
         *
         * [Api set: ExcelApi 1.7]
         */
        numberFormat: string;
        /**
         *
         * The localized format code of the number format for the style.
         *
         * [Api set: ExcelApi 1.7]
         */
        numberFormatLocal: string;
        /**
         *
         * The reading order for the style.
         *
         * [Api set: ExcelApi 1.7]
         */
        readingOrder: Excel.ReadingOrder | "Context" | "LeftToRight" | "RightToLeft";
        /**
         *
         * Indicates if text automatically shrinks to fit in the available column width.
         *
         * [Api set: ExcelApi 1.7]
         */
        shrinkToFit: boolean;
        /**
         *
         * The text orientation for the style.
         *
         * [Api set: ExcelApi 1.8]
         */
        textOrientation: number;
        /**
         *
         * Represents the vertical alignment for the style. See Excel.VerticalAlignment for details.
         *
         * [Api set: ExcelApi 1.7]
         */
        verticalAlignment: Excel.VerticalAlignment | "Top" | "Center" | "Bottom" | "Justify" | "Distributed";
        /**
         *
         * Indicates if Microsoft Excel wraps the text in the object.
         *
         * [Api set: ExcelApi 1.7]
         */
        wrapText: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.Style): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.StyleUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.Style): void;
        /**
         *
         * Deletes this style.
         *
         * [Api set: ExcelApi 1.7]
         */
        delete(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.Style` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.Style` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.Style` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.StyleLoadOptions): Excel.Style;
        load(option?: string | string[]): Excel.Style;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.Style;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.Style object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.StyleData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.StyleData;
    }
    /**
     *
     * Represents a collection of all the styles.
     *
     * [Api set: ExcelApi 1.7]
     */
    class StyleCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.Style[];
        /**
         *
         * Adds a new style to the collection.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param name Name of the style to be added.
         */
        add(name: string): void;
        /**
         *
         * Gets the number of styles in the collection.
         *
         * [Api set: ExcelApi 1.9]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a style by name.
         *
         * [Api set: ExcelApi 1.7]
         *
         * @param name Name of the style to be retrieved.
         */
        getItem(name: string): Excel.Style;
        /**
         *
         * Gets a style based on its position in the collection.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param index Index value of the style object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): Excel.Style;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.StyleCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.StyleCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.StyleCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.StyleCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.StyleCollection;
        load(option?: string | string[]): Excel.StyleCollection;
        load(option?: OfficeExtension.LoadOption): Excel.StyleCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.StyleCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.StyleCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.StyleCollectionData;
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    class PageLayout extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Header and footer configuration for the worksheet.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly headersFooters: Excel.HeaderFooterGroup;
        /**
         *
         * Gets or sets the worksheet's black and white print option.
         *
         * [Api set: ExcelApi 1.9]
         */
        blackAndWhite: boolean;
        /**
         *
         * Gets or sets the worksheet's bottom page margin to use for printing in points.
         *
         * [Api set: ExcelApi 1.9]
         */
        bottomMargin: number;
        /**
         *
         * Gets or sets the worksheet's center horizontally flag. This flag determines whether the worksheet will be centered horizontally when it's printed.
         *
         * [Api set: ExcelApi 1.9]
         */
        centerHorizontally: boolean;
        /**
         *
         * Gets or sets the worksheet's center vertically flag. This flag determines whether the worksheet will be centered vertically when it's printed.
         *
         * [Api set: ExcelApi 1.9]
         */
        centerVertically: boolean;
        /**
         *
         * Gets or sets the worksheet's draft mode option. If true the sheet will be printed without graphics.
         *
         * [Api set: ExcelApi 1.9]
         */
        draftMode: boolean;
        /**
         *
         * Gets or sets the worksheet's first page number to print. Null value represents "auto" page numbering.
         *
         * [Api set: ExcelApi 1.9]
         */
        firstPageNumber: number | "";
        /**
         *
         * Gets or sets the worksheet's footer margin, in points, for use when printing.
         *
         * [Api set: ExcelApi 1.9]
         */
        footerMargin: number;
        /**
         *
         * Gets or sets the worksheet's header margin, in points, for use when printing.
         *
         * [Api set: ExcelApi 1.9]
         */
        headerMargin: number;
        /**
         *
         * Gets or sets the worksheet's left margin, in points, for use when printing.
         *
         * [Api set: ExcelApi 1.9]
         */
        leftMargin: number;
        /**
         *
         * Gets or sets the worksheet's orientation of the page.
         *
         * [Api set: ExcelApi 1.9]
         */
        orientation: Excel.PageOrientation | "Portrait" | "Landscape";
        /**
         *
         * Gets or sets the worksheet's paper size of the page.
         *
         * [Api set: ExcelApi 1.9]
         */
        paperSize: Excel.PaperType | "Letter" | "LetterSmall" | "Tabloid" | "Ledger" | "Legal" | "Statement" | "Executive" | "A3" | "A4" | "A4Small" | "A5" | "B4" | "B5" | "Folio" | "Quatro" | "Paper10x14" | "Paper11x17" | "Note" | "Envelope9" | "Envelope10" | "Envelope11" | "Envelope12" | "Envelope14" | "Csheet" | "Dsheet" | "Esheet" | "EnvelopeDL" | "EnvelopeC5" | "EnvelopeC3" | "EnvelopeC4" | "EnvelopeC6" | "EnvelopeC65" | "EnvelopeB4" | "EnvelopeB5" | "EnvelopeB6" | "EnvelopeItaly" | "EnvelopeMonarch" | "EnvelopePersonal" | "FanfoldUS" | "FanfoldStdGerman" | "FanfoldLegalGerman";
        /**
         *
         * Gets or sets whether the worksheet's comments should be displayed when printing.
         *
         * [Api set: ExcelApi 1.9]
         */
        printComments: Excel.PrintComments | "NoComments" | "EndSheet" | "InPlace";
        /**
         *
         * Gets or sets the worksheet's print errors option.
         *
         * [Api set: ExcelApi 1.9]
         */
        printErrors: Excel.PrintErrorType | "AsDisplayed" | "Blank" | "Dash" | "NotAvailable";
        /**
         *
         * Gets or sets the worksheet's print gridlines flag. This flag determines whether gridlines will be printed or not.
         *
         * [Api set: ExcelApi 1.9]
         */
        printGridlines: boolean;
        /**
         *
         * Gets or sets the worksheet's print headings flag. This flag determines whether headings will be printed or not.
         *
         * [Api set: ExcelApi 1.9]
         */
        printHeadings: boolean;
        /**
         *
         * Gets or sets the worksheet's page print order option. This specifies the order to use for processing the page number printed.
         *
         * [Api set: ExcelApi 1.9]
         */
        printOrder: Excel.PrintOrder | "DownThenOver" | "OverThenDown";
        /**
         *
         * Gets or sets the worksheet's right margin, in points, for use when printing.
         *
         * [Api set: ExcelApi 1.9]
         */
        rightMargin: number;
        /**
         *
         * Gets or sets the worksheet's top margin, in points, for use when printing.
         *
         * [Api set: ExcelApi 1.9]
         */
        topMargin: number;
        /**
         *
         * Gets or sets the worksheet's print zoom options.
         *
         * [Api set: ExcelApi 1.9]
         */
        zoom: Excel.PageLayoutZoomOptions;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.PageLayout): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.PageLayoutUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.PageLayout): void;
        /**
         *
         * Gets the RangeAreas object, comprising one or more rectangular ranges, that represents the print area for the worksheet. If there is no print area, an ItemNotFound error will be thrown.
         *
         * [Api set: ExcelApi 1.9]
         */
        getPrintArea(): Excel.RangeAreas;
        /**
         *
         * Gets the RangeAreas object, comprising one or more rectangular ranges, that represents the print area for the worksheet. If there is no print area, a null object will be returned.
         *
         * [Api set: ExcelApi 1.9]
         */
        getPrintAreaOrNullObject(): Excel.RangeAreas;
        /**
         *
         * Gets the range object representing the title columns.
         *
         * [Api set: ExcelApi 1.9]
         */
        getPrintTitleColumns(): Excel.Range;
        /**
         *
         * Gets the range object representing the title columns. If not set, this will return a null object.
         *
         * [Api set: ExcelApi 1.9]
         */
        getPrintTitleColumnsOrNullObject(): Excel.Range;
        /**
         *
         * Gets the range object representing the title rows.
         *
         * [Api set: ExcelApi 1.9]
         */
        getPrintTitleRows(): Excel.Range;
        /**
         *
         * Gets the range object representing the title rows. If not set, this will return a null object.
         *
         * [Api set: ExcelApi 1.9]
         */
        getPrintTitleRowsOrNullObject(): Excel.Range;
        /**
         *
         * Sets the worksheet's print area.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param printArea The range, or RangeAreas of the content to print.
         */
        setPrintArea(printArea: Range | RangeAreas | string): void;
        /**
         *
         * Sets the worksheet's page margins with units.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param unit Measurement unit for the margins provided.
         * @param marginOptions Margin values to set, margins not provided will remain unchanged.
         */
        setPrintMargins(unit: Excel.PrintMarginUnit, marginOptions: Excel.PageLayoutMarginOptions): void;
        /**
         *
         * Sets the worksheet's page margins with units.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param unit Measurement unit for the margins provided.
         * @param marginOptions Margin values to set, margins not provided will remain unchanged.
         */
        setPrintMargins(unit: "Points" | "Inches" | "Centimeters", marginOptions: Excel.PageLayoutMarginOptions): void;
        /**
         *
         * Sets the columns that contain the cells to be repeated at the left of each page of the worksheet for printing.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param printTitleColumns The columns to be repeated to the left of each page, range must span the entire column to be valid.
         */
        setPrintTitleColumns(printTitleColumns: Range | string): void;
        /**
         *
         * Sets the rows that contain the cells to be repeated at the top of each page of the worksheet for printing.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param printTitleRows The rows to be repeated at the top of each page, range must span the entire row to be valid.
         */
        setPrintTitleRows(printTitleRows: Range | string): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.PageLayout` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.PageLayout` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.PageLayout` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.PageLayoutLoadOptions): Excel.PageLayout;
        load(option?: string | string[]): Excel.PageLayout;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.PageLayout;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.PageLayout object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.PageLayoutData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.PageLayoutData;
    }
    /**
     *
     * Represents page zoom properties.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface PageLayoutZoomOptions {
        /**
         *
         * Number of pages to fit horizontally. This value can be null if percentage scale is used.
         *
         * [Api set: ExcelApi 1.9]
         */
        horizontalFitToPages?: number;
        /**
         *
         * Print page scale value can be between 10 and 400. This value can be null if fit to page tall or wide is specified.
         *
         * [Api set: ExcelApi 1.9]
         */
        scale?: number;
        /**
         *
         * Number of pages to fit vertically. This value can be null if percentage scale is used.
         *
         * [Api set: ExcelApi 1.9]
         */
        verticalFitToPages?: number;
    }
    /**
     *
     * Represents the options in page layout margins.
     *
     * [Api set: ExcelApi 1.9]
     */
    interface PageLayoutMarginOptions {
        /**
         *
         * Represents the page layout bottom margin in the unit specified to use for printing.
         *
         * [Api set: ExcelApi 1.9]
         */
        bottom?: number;
        /**
         *
         * Represents the page layout footer margin in the unit specified to use for printing.
         *
         * [Api set: ExcelApi 1.9]
         */
        footer?: number;
        /**
         *
         * Represents the page layout header margin in the unit specified to use for printing.
         *
         * [Api set: ExcelApi 1.9]
         */
        header?: number;
        /**
         *
         * Represents the page layout left margin in the unit specified to use for printing.
         *
         * [Api set: ExcelApi 1.9]
         */
        left?: number;
        /**
         *
         * Represents the page layout right margin in the unit specified to use for printing.
         *
         * [Api set: ExcelApi 1.9]
         */
        right?: number;
        /**
         *
         * Represents the page layout top margin in the unit specified to use for printing.
         *
         * [Api set: ExcelApi 1.9]
         */
        top?: number;
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    class HeaderFooter extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets or sets the center footer of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
         *
         * [Api set: ExcelApi 1.9]
         */
        centerFooter: string;
        /**
         *
         * Gets or sets the center header of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
         *
         * [Api set: ExcelApi 1.9]
         */
        centerHeader: string;
        /**
         *
         * Gets or sets the left footer of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
         *
         * [Api set: ExcelApi 1.9]
         */
        leftFooter: string;
        /**
         *
         * Gets or sets the left header of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
         *
         * [Api set: ExcelApi 1.9]
         */
        leftHeader: string;
        /**
         *
         * Gets or sets the right footer of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
         *
         * [Api set: ExcelApi 1.9]
         */
        rightFooter: string;
        /**
         *
         * Gets or sets the right header of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
         *
         * [Api set: ExcelApi 1.9]
         */
        rightHeader: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.HeaderFooter): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.HeaderFooterUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.HeaderFooter): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.HeaderFooter` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.HeaderFooter` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.HeaderFooter` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.HeaderFooterLoadOptions): Excel.HeaderFooter;
        load(option?: string | string[]): Excel.HeaderFooter;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.HeaderFooter;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.HeaderFooter object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.HeaderFooterData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.HeaderFooterData;
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    class HeaderFooterGroup extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * The general header/footer, used for all pages unless even/odd or first page is specified.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly defaultForAllPages: Excel.HeaderFooter;
        /**
         *
         * The header/footer to use for even pages, odd header/footer needs to be specified for odd pages.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly evenPages: Excel.HeaderFooter;
        /**
         *
         * The first page header/footer, for all other pages general or even/odd is used.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly firstPage: Excel.HeaderFooter;
        /**
         *
         * The header/footer to use for odd pages, even header/footer needs to be specified for even pages.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly oddPages: Excel.HeaderFooter;
        /**
         *
         * Gets or sets the state of which headers/footers are set. See Excel.HeaderFooterState for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        state: Excel.HeaderFooterState | "Default" | "FirstAndDefault" | "OddAndEven" | "FirstOddAndEven";
        /**
         *
         * Gets or sets a flag indicating if headers/footers are aligned with the page margins set in the page layout options for the worksheet.
         *
         * [Api set: ExcelApi 1.9]
         */
        useSheetMargins: boolean;
        /**
         *
         * Gets or sets a flag indicating if headers/footers should be scaled by the page percentage scale set in the page layout options for the worksheet.
         *
         * [Api set: ExcelApi 1.9]
         */
        useSheetScale: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.HeaderFooterGroup): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.HeaderFooterGroupUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.HeaderFooterGroup): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.HeaderFooterGroup` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.HeaderFooterGroup` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.HeaderFooterGroup` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.HeaderFooterGroupLoadOptions): Excel.HeaderFooterGroup;
        load(option?: string | string[]): Excel.HeaderFooterGroup;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.HeaderFooterGroup;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.HeaderFooterGroup object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.HeaderFooterGroupData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.HeaderFooterGroupData;
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    class PageBreak extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the column index for the page break
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly columnIndex: number;
        /**
         *
         * Represents the row index for the page break
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly rowIndex: number;
        /**
         *
         * Deletes a page break object.
         *
         * [Api set: ExcelApi 1.9]
         */
        delete(): void;
        /**
         *
         * Gets the first cell after the page break.
         *
         * [Api set: ExcelApi 1.9]
         */
        getCellAfterBreak(): Excel.Range;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.PageBreak` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.PageBreak` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.PageBreak` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.PageBreakLoadOptions): Excel.PageBreak;
        load(option?: string | string[]): Excel.PageBreak;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.PageBreak;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.PageBreak object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.PageBreakData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.PageBreakData;
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    class PageBreakCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.PageBreak[];
        /**
         *
         * Adds a page break before the top-left cell of the range specified.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param pageBreakRange The range immediately after the page break to be added.
         */
        add(pageBreakRange: Range | string): Excel.PageBreak;
        /**
         *
         * Gets the number of page breaks in the collection.
         *
         * [Api set: ExcelApi 1.9]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a page break object via the index.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param index Index of the page break.
         */
        getItem(index: number): Excel.PageBreak;
        /**
         *
         * Resets all manual page breaks in the collection.
         *
         * [Api set: ExcelApi 1.9]
         */
        removePageBreaks(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.PageBreakCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.PageBreakCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.PageBreakCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.PageBreakCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.PageBreakCollection;
        load(option?: string | string[]): Excel.PageBreakCollection;
        load(option?: OfficeExtension.LoadOption): Excel.PageBreakCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.PageBreakCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.PageBreakCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.PageBreakCollectionData;
    }
    /**
     *
     * Represents a collection of all the Data Connections that are part of the workbook or worksheet.
     *
     * [Api set: ExcelApi 1.7]
     */
    class DataConnectionCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Refreshes all the Data Connections in the collection.
         *
         * [Api set: ExcelApi 1.7]
         */
        refreshAll(): void;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.DataConnectionCollection object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.DataConnectionCollectionData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): {
            [key: string]: string;
        };
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    class RangeCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.Range[];
        /**
         *
         * Returns the number of ranges in the RangeCollection.
         *
         * [Api set: ExcelApi 1.9]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Returns the range object based on its position in the RangeCollection.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param index Index value of the range object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): Excel.Range;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.RangeCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.RangeCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.RangeCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.RangeCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.RangeCollection;
        load(option?: string | string[]): Excel.RangeCollection;
        load(option?: OfficeExtension.LoadOption): Excel.RangeCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.RangeCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.RangeCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.RangeCollectionData;
    }
    /**
     *
     * Represents a collection of all the shapes in the worksheet.
     *
     * [Api set: ExcelApi 1.9]
     */
    class ShapeCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.Shape[];
        /**
         *
         * Adds a geometric shape to the worksheet. Returns a Shape object that represents the new shape.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param geometricShapeType Represents the type of the geometric shape. See Excel.GeometricShapeType for details.
         */
        addGeometricShape(geometricShapeType: Excel.GeometricShapeType): Excel.Shape;
        /**
         *
         * Adds a geometric shape to the worksheet. Returns a Shape object that represents the new shape.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param geometricShapeType Represents the type of the geometric shape. See Excel.GeometricShapeType for details.
         */
        addGeometricShape(geometricShapeType: "LineInverse" | "Triangle" | "RightTriangle" | "Rectangle" | "Diamond" | "Parallelogram" | "Trapezoid" | "NonIsoscelesTrapezoid" | "Pentagon" | "Hexagon" | "Heptagon" | "Octagon" | "Decagon" | "Dodecagon" | "Star4" | "Star5" | "Star6" | "Star7" | "Star8" | "Star10" | "Star12" | "Star16" | "Star24" | "Star32" | "RoundRectangle" | "Round1Rectangle" | "Round2SameRectangle" | "Round2DiagonalRectangle" | "SnipRoundRectangle" | "Snip1Rectangle" | "Snip2SameRectangle" | "Snip2DiagonalRectangle" | "Plaque" | "Ellipse" | "Teardrop" | "HomePlate" | "Chevron" | "PieWedge" | "Pie" | "BlockArc" | "Donut" | "NoSmoking" | "RightArrow" | "LeftArrow" | "UpArrow" | "DownArrow" | "StripedRightArrow" | "NotchedRightArrow" | "BentUpArrow" | "LeftRightArrow" | "UpDownArrow" | "LeftUpArrow" | "LeftRightUpArrow" | "QuadArrow" | "LeftArrowCallout" | "RightArrowCallout" | "UpArrowCallout" | "DownArrowCallout" | "LeftRightArrowCallout" | "UpDownArrowCallout" | "QuadArrowCallout" | "BentArrow" | "UturnArrow" | "CircularArrow" | "LeftCircularArrow" | "LeftRightCircularArrow" | "CurvedRightArrow" | "CurvedLeftArrow" | "CurvedUpArrow" | "CurvedDownArrow" | "SwooshArrow" | "Cube" | "Can" | "LightningBolt" | "Heart" | "Sun" | "Moon" | "SmileyFace" | "IrregularSeal1" | "IrregularSeal2" | "FoldedCorner" | "Bevel" | "Frame" | "HalfFrame" | "Corner" | "DiagonalStripe" | "Chord" | "Arc" | "LeftBracket" | "RightBracket" | "LeftBrace" | "RightBrace" | "BracketPair" | "BracePair" | "Callout1" | "Callout2" | "Callout3" | "AccentCallout1" | "AccentCallout2" | "AccentCallout3" | "BorderCallout1" | "BorderCallout2" | "BorderCallout3" | "AccentBorderCallout1" | "AccentBorderCallout2" | "AccentBorderCallout3" | "WedgeRectCallout" | "WedgeRRectCallout" | "WedgeEllipseCallout" | "CloudCallout" | "Cloud" | "Ribbon" | "Ribbon2" | "EllipseRibbon" | "EllipseRibbon2" | "LeftRightRibbon" | "VerticalScroll" | "HorizontalScroll" | "Wave" | "DoubleWave" | "Plus" | "FlowChartProcess" | "FlowChartDecision" | "FlowChartInputOutput" | "FlowChartPredefinedProcess" | "FlowChartInternalStorage" | "FlowChartDocument" | "FlowChartMultidocument" | "FlowChartTerminator" | "FlowChartPreparation" | "FlowChartManualInput" | "FlowChartManualOperation" | "FlowChartConnector" | "FlowChartPunchedCard" | "FlowChartPunchedTape" | "FlowChartSummingJunction" | "FlowChartOr" | "FlowChartCollate" | "FlowChartSort" | "FlowChartExtract" | "FlowChartMerge" | "FlowChartOfflineStorage" | "FlowChartOnlineStorage" | "FlowChartMagneticTape" | "FlowChartMagneticDisk" | "FlowChartMagneticDrum" | "FlowChartDisplay" | "FlowChartDelay" | "FlowChartAlternateProcess" | "FlowChartOffpageConnector" | "ActionButtonBlank" | "ActionButtonHome" | "ActionButtonHelp" | "ActionButtonInformation" | "ActionButtonForwardNext" | "ActionButtonBackPrevious" | "ActionButtonEnd" | "ActionButtonBeginning" | "ActionButtonReturn" | "ActionButtonDocument" | "ActionButtonSound" | "ActionButtonMovie" | "Gear6" | "Gear9" | "Funnel" | "MathPlus" | "MathMinus" | "MathMultiply" | "MathDivide" | "MathEqual" | "MathNotEqual" | "CornerTabs" | "SquareTabs" | "PlaqueTabs" | "ChartX" | "ChartStar" | "ChartPlus"): Excel.Shape;
        /**
         *
         * Groups a subset of shapes in this collection's worksheet. Returns a Shape object that represents the new group of shapes.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param values An array of shape ID or shape objects.
         */
        addGroup(values: Array<string | Shape>): Excel.Shape;
        /**
         *
         * Creates an image from a base64-encoded string and adds it to the worksheet. Returns the Shape object that represents the new image.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param base64ImageString A base64-encoded string representing an image in either JPEG or PNG format.
         */
        addImage(base64ImageString: string): Excel.Shape;
        /**
         *
         * Adds a line to worksheet. Returns a Shape object that represents the new line.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param startLeft The distance, in points, from the start of the line to the left side of the worksheet.
         * @param startTop The distance, in points, from the start of the line to the top of the worksheet.
         * @param endLeft The distance, in points, from the end of the line to the left of the worksheet.
         * @param endTop The distance, in points, from the end of the line to the top of the worksheet.
         * @param connectorType Represents the connector type. See Excel.ConnectorType for details.
         */
        addLine(startLeft: number, startTop: number, endLeft: number, endTop: number, connectorType?: Excel.ConnectorType): Excel.Shape;
        /**
         *
         * Adds a line to worksheet. Returns a Shape object that represents the new line.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param startLeft The distance, in points, from the start of the line to the left side of the worksheet.
         * @param startTop The distance, in points, from the start of the line to the top of the worksheet.
         * @param endLeft The distance, in points, from the end of the line to the left of the worksheet.
         * @param endTop The distance, in points, from the end of the line to the top of the worksheet.
         * @param connectorType Represents the connector type. See Excel.ConnectorType for details.
         */
        addLine(startLeft: number, startTop: number, endLeft: number, endTop: number, connectorType?: "Straight" | "Elbow" | "Curve"): Excel.Shape;
        /**
         *
         * Adds a text box to the worksheet with the provided text as the content. Returns a Shape object that represents the new text box.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param text Represents the text that will be shown in the created text box.
         */
        addTextBox(text?: string): Excel.Shape;
        /**
         *
         * Returns the number of shapes in the worksheet. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a shape using its Name or ID.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param key Name or ID of the shape to be retrieved.
         */
        getItem(key: string): Excel.Shape;
        /**
         *
         * Gets a shape using its position in the collection.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param index The zero-based index of the shape to be retrieved.
         */
        getItemAt(index: number): Excel.Shape;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ShapeCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ShapeCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ShapeCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ShapeCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.ShapeCollection;
        load(option?: string | string[]): Excel.ShapeCollection;
        load(option?: OfficeExtension.LoadOption): Excel.ShapeCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.ShapeCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ShapeCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.ShapeCollectionData;
    }
    /**
     *
     * Represents a generic shape object in the worksheet. A shape could be a geometric shape, a line, a group of shapes, etc.
     *
     * [Api set: ExcelApi 1.9]
     */
    class Shape extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns the fill formatting of this shape. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly fill: Excel.ShapeFill;
        /**
         *
         * Returns the geometric shape associated with the shape. An error will be thrown if the shape type is not "GeometricShape".
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly geometricShape: Excel.GeometricShape;
        /**
         *
         * Returns the shape group associated with the shape. An error will be thrown if the shape type is not "GroupShape".
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly group: Excel.ShapeGroup;
        /**
         *
         * Returns the image associated with the shape. An error will be thrown if the shape type is not "Image".
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly image: Excel.Image;
        /**
         *
         * Returns the line associated with the shape. An error will be thrown if the shape type is not "Line".
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly line: Excel.Line;
        /**
         *
         * Returns the line formatting of this shape. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly lineFormat: Excel.ShapeLineFormat;
        /**
         *
         * Represents the parent group of this shape.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly parentGroup: Excel.Shape;
        /**
         *
         * Returns the text frame object of this shape. Read only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly textFrame: Excel.TextFrame;
        /**
         *
         * Returns or sets the alternative description text for a Shape object.
         *
         * [Api set: ExcelApi 1.9]
         */
        altTextDescription: string;
        /**
         *
         * Returns or sets the alternative title text for a Shape object.
         *
         * [Api set: ExcelApi 1.9]
         */
        altTextTitle: string;
        /**
         *
         * Returns the number of connection sites on this shape. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly connectionSiteCount: number;
        /**
         *
         * Represents the geometric shape type of this geometric shape. See Excel.GeometricShapeType for details. Returns null if the shape type is not "GeometricShape".
         *
         * [Api set: ExcelApi 1.9]
         */
        geometricShapeType: Excel.GeometricShapeType | "LineInverse" | "Triangle" | "RightTriangle" | "Rectangle" | "Diamond" | "Parallelogram" | "Trapezoid" | "NonIsoscelesTrapezoid" | "Pentagon" | "Hexagon" | "Heptagon" | "Octagon" | "Decagon" | "Dodecagon" | "Star4" | "Star5" | "Star6" | "Star7" | "Star8" | "Star10" | "Star12" | "Star16" | "Star24" | "Star32" | "RoundRectangle" | "Round1Rectangle" | "Round2SameRectangle" | "Round2DiagonalRectangle" | "SnipRoundRectangle" | "Snip1Rectangle" | "Snip2SameRectangle" | "Snip2DiagonalRectangle" | "Plaque" | "Ellipse" | "Teardrop" | "HomePlate" | "Chevron" | "PieWedge" | "Pie" | "BlockArc" | "Donut" | "NoSmoking" | "RightArrow" | "LeftArrow" | "UpArrow" | "DownArrow" | "StripedRightArrow" | "NotchedRightArrow" | "BentUpArrow" | "LeftRightArrow" | "UpDownArrow" | "LeftUpArrow" | "LeftRightUpArrow" | "QuadArrow" | "LeftArrowCallout" | "RightArrowCallout" | "UpArrowCallout" | "DownArrowCallout" | "LeftRightArrowCallout" | "UpDownArrowCallout" | "QuadArrowCallout" | "BentArrow" | "UturnArrow" | "CircularArrow" | "LeftCircularArrow" | "LeftRightCircularArrow" | "CurvedRightArrow" | "CurvedLeftArrow" | "CurvedUpArrow" | "CurvedDownArrow" | "SwooshArrow" | "Cube" | "Can" | "LightningBolt" | "Heart" | "Sun" | "Moon" | "SmileyFace" | "IrregularSeal1" | "IrregularSeal2" | "FoldedCorner" | "Bevel" | "Frame" | "HalfFrame" | "Corner" | "DiagonalStripe" | "Chord" | "Arc" | "LeftBracket" | "RightBracket" | "LeftBrace" | "RightBrace" | "BracketPair" | "BracePair" | "Callout1" | "Callout2" | "Callout3" | "AccentCallout1" | "AccentCallout2" | "AccentCallout3" | "BorderCallout1" | "BorderCallout2" | "BorderCallout3" | "AccentBorderCallout1" | "AccentBorderCallout2" | "AccentBorderCallout3" | "WedgeRectCallout" | "WedgeRRectCallout" | "WedgeEllipseCallout" | "CloudCallout" | "Cloud" | "Ribbon" | "Ribbon2" | "EllipseRibbon" | "EllipseRibbon2" | "LeftRightRibbon" | "VerticalScroll" | "HorizontalScroll" | "Wave" | "DoubleWave" | "Plus" | "FlowChartProcess" | "FlowChartDecision" | "FlowChartInputOutput" | "FlowChartPredefinedProcess" | "FlowChartInternalStorage" | "FlowChartDocument" | "FlowChartMultidocument" | "FlowChartTerminator" | "FlowChartPreparation" | "FlowChartManualInput" | "FlowChartManualOperation" | "FlowChartConnector" | "FlowChartPunchedCard" | "FlowChartPunchedTape" | "FlowChartSummingJunction" | "FlowChartOr" | "FlowChartCollate" | "FlowChartSort" | "FlowChartExtract" | "FlowChartMerge" | "FlowChartOfflineStorage" | "FlowChartOnlineStorage" | "FlowChartMagneticTape" | "FlowChartMagneticDisk" | "FlowChartMagneticDrum" | "FlowChartDisplay" | "FlowChartDelay" | "FlowChartAlternateProcess" | "FlowChartOffpageConnector" | "ActionButtonBlank" | "ActionButtonHome" | "ActionButtonHelp" | "ActionButtonInformation" | "ActionButtonForwardNext" | "ActionButtonBackPrevious" | "ActionButtonEnd" | "ActionButtonBeginning" | "ActionButtonReturn" | "ActionButtonDocument" | "ActionButtonSound" | "ActionButtonMovie" | "Gear6" | "Gear9" | "Funnel" | "MathPlus" | "MathMinus" | "MathMultiply" | "MathDivide" | "MathEqual" | "MathNotEqual" | "CornerTabs" | "SquareTabs" | "PlaqueTabs" | "ChartX" | "ChartStar" | "ChartPlus";
        /**
         *
         * Represents the height, in points, of the shape.
            Throws an invalid argument exception when set with a negative value or zero as input.
         *
         * [Api set: ExcelApi 1.9]
         */
        height: number;
        /**
         *
         * Represents the shape identifier. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly id: string;
        /**
         *
         * The distance, in points, from the left side of the shape to the left side of the worksheet.
            Throws an invalid argument exception when set with a negative value as input.
         *
         * [Api set: ExcelApi 1.9]
         */
        left: number;
        /**
         *
         * Represents the level of the specified shape. For example, a level of 0 means that the shape is not part of any groups, a level of 1 means the shape is part of a top-level group, and a level of 2 means the shape is part of a sub-group of the top level.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly level: number;
        /**
         *
         * Specifies whether or not the aspect ratio of this shape is locked.
         *
         * [Api set: ExcelApi 1.9]
         */
        lockAspectRatio: boolean;
        /**
         *
         * Represents the name of the shape.
         *
         * [Api set: ExcelApi 1.9]
         */
        name: string;
        /**
         *
         * Represents the rotation, in degrees, of the shape.
         *
         * [Api set: ExcelApi 1.9]
         */
        rotation: number;
        /**
         *
         * The distance, in points, from the top edge of the shape to the top edge of the worksheet.
            Throws an invalid argument exception when set with a negative value as input.
         *
         * [Api set: ExcelApi 1.9]
         */
        top: number;
        /**
         *
         * Returns the type of this shape. See Excel.ShapeType for details. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly type: Excel.ShapeType | "Unsupported" | "Image" | "GeometricShape" | "Group" | "Line";
        /**
         *
         * Represents the visibility of this shape.
         *
         * [Api set: ExcelApi 1.9]
         */
        visible: boolean;
        /**
         *
         * Represents the width, in points, of the shape.
            Throws an invalid argument exception when set with a negative value or zero as input.
         *
         * [Api set: ExcelApi 1.9]
         */
        width: number;
        /**
         *
         * Returns the position of the specified shape in the z-order, with 0 representing the bottom of the order stack. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly zOrderPosition: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.Shape): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ShapeUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.Shape): void;
        /**
         *
         * Removes the shape from the worksheet.
         *
         * [Api set: ExcelApi 1.9]
         */
        delete(): void;
        /**
         *
         * Converts the shape to an image and returns the image as a base64-encoded string. The DPI is 96. The only supported formats are `Excel.PictureFormat.BMP`, `Excel.PictureFormat.PNG`, `Excel.PictureFormat.JPEG`, and `Excel.PictureFormat.GIF`.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param format Specifies the format of the image.
         */
        getAsImage(format: Excel.PictureFormat): OfficeExtension.ClientResult<string>;
        /**
         *
         * Converts the shape to an image and returns the image as a base64-encoded string. The DPI is 96. The only supported formats are `Excel.PictureFormat.BMP`, `Excel.PictureFormat.PNG`, `Excel.PictureFormat.JPEG`, and `Excel.PictureFormat.GIF`.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param format Specifies the format of the image.
         */
        getAsImage(format: "UNKNOWN" | "BMP" | "JPEG" | "GIF" | "PNG" | "SVG"): OfficeExtension.ClientResult<string>;
        /**
         *
         * Moves the shape horizontally by the specified number of points.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param increment The increment, in points, the shape will be horizontally moved. A positive value moves the shape to the right and a negative value moves it to the left. If the sheet is right-to-left oriented, this is reversed: positive values will move the shape to the left and negative values will move it to the right.
         */
        incrementLeft(increment: number): void;
        /**
         *
         * Rotates the shape clockwise around the z-axis by the specified number of degrees.
            Use the `rotation` property to set the absolute rotation of the shape.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param increment How many degrees the shape will be rotated. A positive value rotates the shape clockwise; a negative value rotates it counterclockwise.
         */
        incrementRotation(increment: number): void;
        /**
         *
         * Moves the shape vertically by the specified number of points.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param increment The increment, in points, the shape will be vertically moved. in points. A positive value moves the shape down and a negative value moves it up.
         */
        incrementTop(increment: number): void;
        /**
         *
         * Scales the height of the shape by a specified factor. For images, you can indicate whether you want to scale the shape relative to the original or the current size. Shapes other than pictures are always scaled relative to their current height.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param scaleFactor Specifies the ratio between the height of the shape after you resize it and the current or original height.
         * @param scaleType Specifies whether the shape is scaled relative to its original or current size. The original size scaling option only works for images.
         * @param scaleFrom Optional. Specifies which part of the shape retains its position when the shape is scaled. If omitted, it represents the shape's upper left corner retains its position.
         */
        scaleHeight(scaleFactor: number, scaleType: Excel.ShapeScaleType, scaleFrom?: Excel.ShapeScaleFrom): void;
        /**
         *
         * Scales the height of the shape by a specified factor. For images, you can indicate whether you want to scale the shape relative to the original or the current size. Shapes other than pictures are always scaled relative to their current height.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param scaleFactor Specifies the ratio between the height of the shape after you resize it and the current or original height.
         * @param scaleType Specifies whether the shape is scaled relative to its original or current size. The original size scaling option only works for images.
         * @param scaleFrom Optional. Specifies which part of the shape retains its position when the shape is scaled. If omitted, it represents the shape's upper left corner retains its position.
         */
        scaleHeight(scaleFactor: number, scaleType: "CurrentSize" | "OriginalSize", scaleFrom?: "ScaleFromTopLeft" | "ScaleFromMiddle" | "ScaleFromBottomRight"): void;
        /**
         *
         * Scales the width of the shape by a specified factor. For images, you can indicate whether you want to scale the shape relative to the original or the current size. Shapes other than pictures are always scaled relative to their current width.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param scaleFactor Specifies the ratio between the width of the shape after you resize it and the current or original width.
         * @param scaleType Specifies whether the shape is scaled relative to its original or current size. The original size scaling option only works for images.
         * @param scaleFrom Optional. Specifies which part of the shape retains its position when the shape is scaled. If omitted, it represents the shape's upper left corner retains its position.
         */
        scaleWidth(scaleFactor: number, scaleType: Excel.ShapeScaleType, scaleFrom?: Excel.ShapeScaleFrom): void;
        /**
         *
         * Scales the width of the shape by a specified factor. For images, you can indicate whether you want to scale the shape relative to the original or the current size. Shapes other than pictures are always scaled relative to their current width.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param scaleFactor Specifies the ratio between the width of the shape after you resize it and the current or original width.
         * @param scaleType Specifies whether the shape is scaled relative to its original or current size. The original size scaling option only works for images.
         * @param scaleFrom Optional. Specifies which part of the shape retains its position when the shape is scaled. If omitted, it represents the shape's upper left corner retains its position.
         */
        scaleWidth(scaleFactor: number, scaleType: "CurrentSize" | "OriginalSize", scaleFrom?: "ScaleFromTopLeft" | "ScaleFromMiddle" | "ScaleFromBottomRight"): void;
        /**
         *
         * Moves the specified shape up or down the collection's z-order, which shifts it in front of or behind other shapes.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param position Where to move the shape in the z-order stack relative to the other shapes. See Excel.ShapeZOrder for details.
         */
        setZOrder(position: Excel.ShapeZOrder): void;
        /**
         *
         * Moves the specified shape up or down the collection's z-order, which shifts it in front of or behind other shapes.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param position Where to move the shape in the z-order stack relative to the other shapes. See Excel.ShapeZOrder for details.
         */
        setZOrder(position: "BringToFront" | "BringForward" | "SendToBack" | "SendBackward"): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.Shape` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.Shape` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.Shape` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ShapeLoadOptions): Excel.Shape;
        load(option?: string | string[]): Excel.Shape;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.Shape;
        /**
         *
         * Occurs when the shape is activated.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @eventproperty
         */
        readonly onActivated: OfficeExtension.EventHandlers<Excel.ShapeActivatedEventArgs>;
        /**
         *
         * Occurs when the shape is deactivated.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @eventproperty
         */
        readonly onDeactivated: OfficeExtension.EventHandlers<Excel.ShapeDeactivatedEventArgs>;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.Shape object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ShapeData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ShapeData;
    }
    /**
     *
     * Represents a geometric shape inside a worksheet. A geometric shape can be a rectangle, block arrow, equation symbol, flowchart item, star, banner, callout, or any other basic shape in Excel.
     *
     * [Api set: ExcelApi 1.9]
     */
    class GeometricShape extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns the Shape object for the geometric shape. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly shape: Excel.Shape;
        /**
         *
         * Returns the shape identifier. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly id: string;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.GeometricShape` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.GeometricShape` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.GeometricShape` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.GeometricShapeLoadOptions): Excel.GeometricShape;
        load(option?: string | string[]): Excel.GeometricShape;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.GeometricShape;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.GeometricShape object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.GeometricShapeData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.GeometricShapeData;
    }
    /**
     *
     * Represents an image in the worksheet. To get the corresponding Shape object, use Image.shape.
     *
     * [Api set: ExcelApi 1.9]
     */
    class Image extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns the Shape object associated with the image. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly shape: Excel.Shape;
        /**
         *
         * Represents the shape identifier for the image object. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly id: string;
        /**
         *
         * Returns the format of the image. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly format: Excel.PictureFormat | "UNKNOWN" | "BMP" | "JPEG" | "GIF" | "PNG" | "SVG";
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.Image` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.Image` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.Image` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ImageLoadOptions): Excel.Image;
        load(option?: string | string[]): Excel.Image;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.Image;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.Image object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ImageData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ImageData;
    }
    /**
     *
     * Represents a shape group inside a worksheet. To get the corresponding Shape object, use `ShapeGroup.shape`.
     *
     * [Api set: ExcelApi 1.9]
     */
    class ShapeGroup extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns the Shape object associated with the group. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly shape: Excel.Shape;
        /**
         *
         * Returns the collection of Shape objects. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly shapes: Excel.GroupShapeCollection;
        /**
         *
         * Represents the shape identifier. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly id: string;
        /**
         *
         * Ungroups any grouped shapes in the specified shape group.
         *
         * [Api set: ExcelApi 1.9]
         */
        ungroup(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ShapeGroup` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ShapeGroup` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ShapeGroup` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ShapeGroupLoadOptions): Excel.ShapeGroup;
        load(option?: string | string[]): Excel.ShapeGroup;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ShapeGroup;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ShapeGroup object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ShapeGroupData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ShapeGroupData;
    }
    /**
     *
     * Represents the shape collection inside a shape group.
     *
     * [Api set: ExcelApi 1.9]
     */
    class GroupShapeCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Excel.Shape[];
        /**
         *
         * Returns the number of shapes in the shape group. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a shape using its Name or ID.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param key The Name or ID of the shape to be retrieved.
         */
        getItem(key: string): Excel.Shape;
        /**
         *
         * Gets a shape based on its position in the collection.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param index The zero-based index value of the object to be retrieved.
         */
        getItemAt(index: number): Excel.Shape;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.GroupShapeCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.GroupShapeCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.GroupShapeCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.GroupShapeCollectionLoadOptions & Excel.Interfaces.CollectionLoadOptions): Excel.GroupShapeCollection;
        load(option?: string | string[]): Excel.GroupShapeCollection;
        load(option?: OfficeExtension.LoadOption): Excel.GroupShapeCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Excel.GroupShapeCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.GroupShapeCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Excel.Interfaces.GroupShapeCollectionData;
    }
    /**
     *
     * Represents a line inside a worksheet. To get the corresponding Shape object, use `Line.shape`.
     *
     * [Api set: ExcelApi 1.9]
     */
    class Line extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the shape to which the beginning of the specified line is attached. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly beginConnectedShape: Excel.Shape;
        /**
         *
         * Represents the shape to which the end of the specified line is attached. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly endConnectedShape: Excel.Shape;
        /**
         *
         * Returns the Shape object associated with the line. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly shape: Excel.Shape;
        /**
         *
         * Represents the length of the arrowhead at the beginning of the specified line.
         *
         * [Api set: ExcelApi 1.9]
         */
        beginArrowheadLength: Excel.ArrowheadLength | "Short" | "Medium" | "Long";
        /**
         *
         * Represents the style of the arrowhead at the beginning of the specified line.
         *
         * [Api set: ExcelApi 1.9]
         */
        beginArrowheadStyle: Excel.ArrowheadStyle | "None" | "Triangle" | "Stealth" | "Diamond" | "Oval" | "Open";
        /**
         *
         * Represents the width of the arrowhead at the beginning of the specified line.
         *
         * [Api set: ExcelApi 1.9]
         */
        beginArrowheadWidth: Excel.ArrowheadWidth | "Narrow" | "Medium" | "Wide";
        /**
         *
         * Represents the connection site to which the beginning of a connector is connected. Read-only. Returns null when the beginning of the line is not attached to any shape.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly beginConnectedSite: number;
        /**
         *
         * Represents the length of the arrowhead at the end of the specified line.
         *
         * [Api set: ExcelApi 1.9]
         */
        endArrowheadLength: Excel.ArrowheadLength | "Short" | "Medium" | "Long";
        /**
         *
         * Represents the style of the arrowhead at the end of the specified line.
         *
         * [Api set: ExcelApi 1.9]
         */
        endArrowheadStyle: Excel.ArrowheadStyle | "None" | "Triangle" | "Stealth" | "Diamond" | "Oval" | "Open";
        /**
         *
         * Represents the width of the arrowhead at the end of the specified line.
         *
         * [Api set: ExcelApi 1.9]
         */
        endArrowheadWidth: Excel.ArrowheadWidth | "Narrow" | "Medium" | "Wide";
        /**
         *
         * Represents the connection site to which the end of a connector is connected. Read-only. Returns null when the end of the line is not attached to any shape.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly endConnectedSite: number;
        /**
         *
         * Represents the shape identifier. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly id: string;
        /**
         *
         * Specifies whether or not the beginning of the specified line is connected to a shape. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly isBeginConnected: boolean;
        /**
         *
         * Specifies whether or not the end of the specified line is connected to a shape. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly isEndConnected: boolean;
        /**
         *
         * Represents the connector type for the line.
         *
         * [Api set: ExcelApi 1.9]
         */
        connectorType: Excel.ConnectorType | "Straight" | "Elbow" | "Curve";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.Line): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.LineUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.Line): void;
        /**
         *
         * Attaches the beginning of the specified connector to a specified shape.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param shape The shape to connect.
         * @param connectionSite The connection site on the shape to which the beginning of the connector is attached. Must be an integer between 0 (inclusive) and the connection-site count of the specified shape (exclusive).
         */
        connectBeginShape(shape: Excel.Shape, connectionSite: number): void;
        /**
         *
         * Attaches the end of the specified connector to a specified shape.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param shape The shape to connect.
         * @param connectionSite The connection site on the shape to which the end of the connector is attached. Must be an integer between 0 (inclusive) and the connection-site count of the specified shape (exclusive).
         */
        connectEndShape(shape: Excel.Shape, connectionSite: number): void;
        /**
         *
         * Detaches the beginning of the specified connector from a shape.
         *
         * [Api set: ExcelApi 1.9]
         */
        disconnectBeginShape(): void;
        /**
         *
         * Detaches the end of the specified connector from a shape.
         *
         * [Api set: ExcelApi 1.9]
         */
        disconnectEndShape(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.Line` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.Line` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.Line` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.LineLoadOptions): Excel.Line;
        load(option?: string | string[]): Excel.Line;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.Line;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.Line object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.LineData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.LineData;
    }
    /**
     *
     * Represents the fill formatting of a shape object.
     *
     * [Api set: ExcelApi 1.9]
     */
    class ShapeFill extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the shape fill foreground color in HTML color format, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange")
         *
         * [Api set: ExcelApi 1.9]
         */
        foregroundColor: string;
        /**
         *
         * Returns or sets the transparency percentage of the fill as a value from 0.0 (opaque) through 1.0 (clear). Returns null if the shape type does not support transparency or the shape fill has inconsistent transparency, such as with a gradient fill type.
         *
         * [Api set: ExcelApi 1.9]
         */
        transparency: number;
        /**
         *
         * Returns the fill type of the shape. Read-only. See Excel.ShapeFillType for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly type: Excel.ShapeFillType | "NoFill" | "Solid" | "Gradient" | "Pattern" | "PictureAndTexture" | "Mixed";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ShapeFill): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ShapeFillUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ShapeFill): void;
        /**
         *
         * Clears the fill formatting of this shape.
         *
         * [Api set: ExcelApi 1.9]
         */
        clear(): void;
        /**
         *
         * Sets the fill formatting of the shape to a uniform color. This changes the fill type to "Solid".
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param color A string that represents the fill color in HTML color format, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
         */
        setSolidColor(color: string): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ShapeFill` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ShapeFill` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ShapeFill` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ShapeFillLoadOptions): Excel.ShapeFill;
        load(option?: string | string[]): Excel.ShapeFill;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ShapeFill;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ShapeFill object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ShapeFillData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ShapeFillData;
    }
    /**
     *
     * Represents the line formatting for the shape object. For images and geometric shapes, line formatting represents the border of the shape.
     *
     * [Api set: ExcelApi 1.9]
     */
    class ShapeLineFormat extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the line color in HTML color format, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
         *
         * [Api set: ExcelApi 1.9]
         */
        color: string;
        /**
         *
         * Represents the line style of the shape. Returns null when the line is not visible or there are inconsistent dash styles. See Excel.ShapeLineStyle for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        dashStyle: Excel.ShapeLineDashStyle | "Dash" | "DashDot" | "DashDotDot" | "LongDash" | "LongDashDot" | "RoundDot" | "Solid" | "SquareDot" | "LongDashDotDot" | "SystemDash" | "SystemDot" | "SystemDashDot";
        /**
         *
         * Represents the line style of the shape. Returns null when the line is not visible or there are inconsistent styles. See Excel.ShapeLineStyle for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        style: Excel.ShapeLineStyle | "Single" | "ThickBetweenThin" | "ThickThin" | "ThinThick" | "ThinThin";
        /**
         *
         * Represents the degree of transparency of the specified line as a value from 0.0 (opaque) through 1.0 (clear). Returns null when the shape has inconsistent transparencies.
         *
         * [Api set: ExcelApi 1.9]
         */
        transparency: number;
        /**
         *
         * Represents whether or not the line formatting of a shape element is visible. Returns null when the shape has inconsistent visibilities.
         *
         * [Api set: ExcelApi 1.9]
         */
        visible: boolean;
        /**
         *
         * Represents the weight of the line, in points. Returns null when the line is not visible or there are inconsistent line weights.
         *
         * [Api set: ExcelApi 1.9]
         */
        weight: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ShapeLineFormat): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ShapeLineFormatUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ShapeLineFormat): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ShapeLineFormat` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ShapeLineFormat` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ShapeLineFormat` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ShapeLineFormatLoadOptions): Excel.ShapeLineFormat;
        load(option?: string | string[]): Excel.ShapeLineFormat;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ShapeLineFormat;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ShapeLineFormat object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ShapeLineFormatData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ShapeLineFormatData;
    }
    /**
     *
     * Represents the text frame of a shape object.
     *
     * [Api set: ExcelApi 1.9]
     */
    class TextFrame extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the text that is attached to a shape in the text frame, and properties and methods for manipulating the text. See Excel.TextRange for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly textRange: Excel.TextRange;
        /**
         *
         * Gets or sets the automatic sizing settings for the text frame. A text frame can be set to automatically fit the text to the text frame, to automatically fit the text frame to the text, or not perform any automatic sizing.
         *
         * [Api set: ExcelApi 1.9]
         */
        autoSizeSetting: Excel.ShapeAutoSize | "AutoSizeNone" | "AutoSizeTextToFitShape" | "AutoSizeShapeToFitText" | "AutoSizeMixed";
        /**
         *
         * Represents the bottom margin, in points, of the text frame.
         *
         * [Api set: ExcelApi 1.9]
         */
        bottomMargin: number;
        /**
         *
         * Specifies whether the text frame contains text.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly hasText: boolean;
        /**
         *
         * Represents the horizontal alignment of the text frame. See Excel.ShapeTextHorizontalAlignment for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        horizontalAlignment: Excel.ShapeTextHorizontalAlignment | "Left" | "Center" | "Right" | "Justify" | "JustifyLow" | "Distributed" | "ThaiDistributed";
        /**
         *
         * Represents the horizontal overflow behavior of the text frame. See Excel.ShapeTextHorizontalOverflow for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        horizontalOverflow: Excel.ShapeTextHorizontalOverflow | "Overflow" | "Clip";
        /**
         *
         * Represents the left margin, in points, of the text frame.
         *
         * [Api set: ExcelApi 1.9]
         */
        leftMargin: number;
        /**
         *
         * Represents the text orientation of the text frame. See Excel.ShapeTextOrientation for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        orientation: Excel.ShapeTextOrientation | "Horizontal" | "Vertical" | "Vertical270" | "WordArtVertical" | "EastAsianVertical" | "MongolianVertical" | "WordArtVerticalRTL";
        /**
         *
         * Represents the reading order of the text frame, either left-to-right or right-to-left. See Excel.ShapeTextReadingOrder for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        readingOrder: Excel.ShapeTextReadingOrder | "LeftToRight" | "RightToLeft";
        /**
         *
         * Represents the right margin, in points, of the text frame.
         *
         * [Api set: ExcelApi 1.9]
         */
        rightMargin: number;
        /**
         *
         * Represents the top margin, in points, of the text frame.
         *
         * [Api set: ExcelApi 1.9]
         */
        topMargin: number;
        /**
         *
         * Represents the vertical alignment of the text frame. See Excel.ShapeTextVerticalAlignment for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        verticalAlignment: Excel.ShapeTextVerticalAlignment | "Top" | "Middle" | "Bottom" | "Justified" | "Distributed";
        /**
         *
         * Represents the vertical overflow behavior of the text frame. See Excel.ShapeTextVerticalOverflow for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        verticalOverflow: Excel.ShapeTextVerticalOverflow | "Overflow" | "Ellipsis" | "Clip";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.TextFrame): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.TextFrameUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.TextFrame): void;
        /**
         *
         * Deletes all the text in the text frame.
         *
         * [Api set: ExcelApi 1.9]
         */
        deleteText(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.TextFrame` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.TextFrame` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.TextFrame` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.TextFrameLoadOptions): Excel.TextFrame;
        load(option?: string | string[]): Excel.TextFrame;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.TextFrame;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.TextFrame object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.TextFrameData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.TextFrameData;
    }
    /**
     *
     * Contains the text that is attached to a shape, in addition to properties and methods for manipulating the text.
     *
     * [Api set: ExcelApi 1.9]
     */
    class TextRange extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns a ShapeFont object that represents the font attributes for the text range. Read-only.
         *
         * [Api set: ExcelApi 1.9]
         */
        readonly font: Excel.ShapeFont;
        /**
         *
         * Represents the plain text content of the text range.
         *
         * [Api set: ExcelApi 1.9]
         */
        text: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.TextRange): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.TextRangeUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.TextRange): void;
        /**
         *
         * Returns a TextRange object for the substring in the given range.
         *
         * [Api set: ExcelApi 1.9]
         *
         * @param start The zero-based index of the first character to get from the text range.
         * @param length Optional. The number of characters to be returned in the new text range. If length is omitted, all the characters from start to the end of the text range's last paragraph will be returned.
         */
        getSubstring(start: number, length?: number): Excel.TextRange;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.TextRange` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.TextRange` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.TextRange` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.TextRangeLoadOptions): Excel.TextRange;
        load(option?: string | string[]): Excel.TextRange;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.TextRange;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.TextRange object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.TextRangeData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.TextRangeData;
    }
    /**
     *
     * Represents the font attributes, such as font name, font size, and color, for a shape's TextRange object.
     *
     * [Api set: ExcelApi 1.9]
     */
    class ShapeFont extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the bold status of font. Returns null the TextRange includes both bold and non-bold text fragments.
         *
         * [Api set: ExcelApi 1.9]
         */
        bold: boolean;
        /**
         *
         * The HTML color code representation of the text color (e.g. "#FF0000" represents red). Returns null if the TextRange includes text fragments with different colors.
         *
         * [Api set: ExcelApi 1.9]
         */
        color: string;
        /**
         *
         * Represents the italic status of font. Returns null if the TextRange includes both italic and non-italic text fragments.
         *
         * [Api set: ExcelApi 1.9]
         */
        italic: boolean;
        /**
         *
         * Represents font name (e.g. "Calibri"). If the text is Complex Script or East Asian language, this is the corresponding font name; otherwise it is the Latin font name.
         *
         * [Api set: ExcelApi 1.9]
         */
        name: string;
        /**
         *
         * Represents font size in points (e.g. 11). Returns null if the TextRange includes text fragments with different font sizes.
         *
         * [Api set: ExcelApi 1.9]
         */
        size: number;
        /**
         *
         * Type of underline applied to the font. Returns null if the TextRange includes text fragments with different underline styles. See Excel.ShapeFontUnderlineStyle for details.
         *
         * [Api set: ExcelApi 1.9]
         */
        underline: Excel.ShapeFontUnderlineStyle | "None" | "Single" | "Double" | "Heavy" | "Dotted" | "DottedHeavy" | "Dash" | "DashHeavy" | "DashLong" | "DashLongHeavy" | "DotDash" | "DotDashHeavy" | "DotDotDash" | "DotDotDashHeavy" | "Wavy" | "WavyHeavy" | "WavyDouble";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Excel.ShapeFont): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ShapeFontUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Excel.ShapeFont): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Excel.ShapeFont` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Excel.ShapeFont` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Excel.ShapeFont` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.ShapeFontLoadOptions): Excel.ShapeFont;
        load(option?: string | string[]): Excel.ShapeFont;
        load(option?: {
            select?: string;
            expand?: string;
        }): Excel.ShapeFont;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.ShapeFont object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.ShapeFontData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Excel.Interfaces.ShapeFontData;
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum ChartAxisType {
        invalid = "Invalid",
        /**
         *
         * Axis displays categories.
         *
         */
        category = "Category",
        /**
         *
         * Axis displays values.
         *
         */
        value = "Value",
        /**
         *
         * Axis displays data series.
         *
         */
        series = "Series"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum ChartAxisGroup {
        primary = "Primary",
        secondary = "Secondary"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum ChartAxisScaleType {
        linear = "Linear",
        logarithmic = "Logarithmic"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum ChartAxisPosition {
        automatic = "Automatic",
        maximum = "Maximum",
        minimum = "Minimum",
        custom = "Custom"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum ChartAxisTickMark {
        none = "None",
        cross = "Cross",
        inside = "Inside",
        outside = "Outside"
    }
    /**
     *
     * Represents the state of calculation across the entire Excel application.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum CalculationState {
        /**
         *
         * Calculations complete.
         *
         */
        done = "Done",
        /**
         *
         * Calculations in progress.
         *
         */
        calculating = "Calculating",
        /**
         *
         * Changes that trigger calculation have been made, but a recalculation has not yet been performed.
         *
         */
        pending = "Pending"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum ChartAxisTickLabelPosition {
        nextToAxis = "NextToAxis",
        high = "High",
        low = "Low",
        none = "None"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum ChartAxisDisplayUnit {
        /**
         *
         * Default option. This will reset display unit to the axis, and set unit label invisible.
         *
         */
        none = "None",
        /**
         *
         * This will set the axis in units of hundreds.
         *
         */
        hundreds = "Hundreds",
        /**
         *
         * This will set the axis in units of thousands.
         *
         */
        thousands = "Thousands",
        /**
         *
         * This will set the axis in units of tens of thousands.
         *
         */
        tenThousands = "TenThousands",
        /**
         *
         * This will set the axis in units of hundreds of thousands.
         *
         */
        hundredThousands = "HundredThousands",
        /**
         *
         * This will set the axis in units of millions.
         *
         */
        millions = "Millions",
        /**
         *
         * This will set the axis in units of tens of millions.
         *
         */
        tenMillions = "TenMillions",
        /**
         *
         * This will set the axis in units of hundreds of millions.
         *
         */
        hundredMillions = "HundredMillions",
        /**
         *
         * This will set the axis in units of billions.
         *
         */
        billions = "Billions",
        /**
         *
         * This will set the axis in units of trillions.
         *
         */
        trillions = "Trillions",
        /**
         *
         * This will set the axis in units of custom value.
         *
         */
        custom = "Custom"
    }
    /**
     *
     * Specifies the unit of time for chart axes and data series.
     *
     * [Api set: ExcelApi 1.7]
     */
    enum ChartAxisTimeUnit {
        days = "Days",
        months = "Months",
        years = "Years"
    }
    /**
     *
     * Represents the quartile calculation type of chart series layout. Only applies to a box and whisker chart.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ChartBoxQuartileCalculation {
        inclusive = "Inclusive",
        exclusive = "Exclusive"
    }
    /**
     *
     * Specifies the type of the category axis.
     *
     * [Api set: ExcelApi 1.7]
     */
    enum ChartAxisCategoryType {
        /**
         *
         * Excel controls the axis type.
         *
         */
        automatic = "Automatic",
        /**
         *
         * Axis groups data by an arbitrary set of categories.
         *
         */
        textAxis = "TextAxis",
        /**
         *
         * Axis groups data on a time scale.
         *
         */
        dateAxis = "DateAxis"
    }
    /**
     *
     * Specifies the bin's type of a histogram chart or pareto chart series.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ChartBinType {
        category = "Category",
        auto = "Auto",
        binWidth = "BinWidth",
        binCount = "BinCount"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum ChartLineStyle {
        none = "None",
        continuous = "Continuous",
        dash = "Dash",
        dashDot = "DashDot",
        dashDotDot = "DashDotDot",
        dot = "Dot",
        grey25 = "Grey25",
        grey50 = "Grey50",
        grey75 = "Grey75",
        automatic = "Automatic",
        roundDot = "RoundDot"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum ChartDataLabelPosition {
        invalid = "Invalid",
        none = "None",
        center = "Center",
        insideEnd = "InsideEnd",
        insideBase = "InsideBase",
        outsideEnd = "OutsideEnd",
        left = "Left",
        right = "Right",
        top = "Top",
        bottom = "Bottom",
        bestFit = "BestFit",
        callout = "Callout"
    }
    /**
     *
     * Represents which parts of the error bar to include.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ChartErrorBarsInclude {
        both = "Both",
        minusValues = "MinusValues",
        plusValues = "PlusValues"
    }
    /**
     *
     * Represents the range type for error bars.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ChartErrorBarsType {
        fixedValue = "FixedValue",
        percent = "Percent",
        stDev = "StDev",
        stError = "StError",
        custom = "Custom"
    }
    /**
     *
     * Represents the mapping level of a chart series. This only applies to region map charts.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ChartMapAreaLevel {
        automatic = "Automatic",
        dataOnly = "DataOnly",
        city = "City",
        county = "County",
        state = "State",
        country = "Country",
        continent = "Continent",
        world = "World"
    }
    /**
     *
     * Represents the gradient style of a chart series. This is only applicable for region map charts.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ChartGradientStyle {
        twoPhaseColor = "TwoPhaseColor",
        threePhaseColor = "ThreePhaseColor"
    }
    /**
     *
     * Represents the gradient style type of a chart series. This is only applicable for region map charts.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ChartGradientStyleType {
        extremeValue = "ExtremeValue",
        number = "Number",
        percent = "Percent"
    }
    /**
     *
     * Represents the position of chart title.
     *
     * [Api set: ExcelApi 1.7]
     */
    enum ChartTitlePosition {
        automatic = "Automatic",
        top = "Top",
        bottom = "Bottom",
        left = "Left",
        right = "Right"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum ChartLegendPosition {
        invalid = "Invalid",
        top = "Top",
        bottom = "Bottom",
        left = "Left",
        right = "Right",
        corner = "Corner",
        custom = "Custom"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum ChartMarkerStyle {
        invalid = "Invalid",
        automatic = "Automatic",
        none = "None",
        square = "Square",
        diamond = "Diamond",
        triangle = "Triangle",
        x = "X",
        star = "Star",
        dot = "Dot",
        dash = "Dash",
        circle = "Circle",
        plus = "Plus",
        picture = "Picture"
    }
    /**
     * [Api set: ExcelApi 1.8]
     */
    enum ChartPlotAreaPosition {
        automatic = "Automatic",
        custom = "Custom"
    }
    /**
     *
     * Represents the region level of a chart series layout. This only applies to region map charts.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ChartMapLabelStrategy {
        none = "None",
        bestFit = "BestFit",
        showAll = "ShowAll"
    }
    /**
     *
     * Represents the region projection type of a chart series layout. This only applies to region map charts.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ChartMapProjectionType {
        automatic = "Automatic",
        mercator = "Mercator",
        miller = "Miller",
        robinson = "Robinson",
        albers = "Albers"
    }
    /**
     *
     * Represents the parent label strategy of the chart series layout. This only applies to treemap charts
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ChartParentLabelStrategy {
        none = "None",
        banner = "Banner",
        overlapping = "Overlapping"
    }
    /**
     *
     * Specifies whether the series are by rows or by columns. On Desktop, the "auto" option will inspect the source data shape to automatically guess whether the data is by rows or columns; on Excel Online, "auto" will simply default to "columns".
     *
     * [Api set: ExcelApi 1.1]
     */
    enum ChartSeriesBy {
        /**
         *
         * On Desktop, the "auto" option will inspect the source data shape to automatically guess whether the data is by rows or columns; on Excel Online, "auto" will simply default to "columns".
         *
         */
        auto = "Auto",
        columns = "Columns",
        rows = "Rows"
    }
    /**
     *
     * Represents the horizontal alignment for the specified object.
     *
     * [Api set: ExcelApi 1.7]
     */
    enum ChartTextHorizontalAlignment {
        center = "Center",
        left = "Left",
        right = "Right",
        justify = "Justify",
        distributed = "Distributed"
    }
    /**
     *
     * Represents the vertical alignment for the specified object.
     *
     * [Api set: ExcelApi 1.7]
     */
    enum ChartTextVerticalAlignment {
        center = "Center",
        bottom = "Bottom",
        top = "Top",
        justify = "Justify",
        distributed = "Distributed"
    }
    /**
     * [Api set: ExcelApi 1.8]
     */
    enum ChartTickLabelAlignment {
        center = "Center",
        left = "Left",
        right = "Right"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum ChartType {
        invalid = "Invalid",
        columnClustered = "ColumnClustered",
        columnStacked = "ColumnStacked",
        columnStacked100 = "ColumnStacked100",
        _3DColumnClustered = "3DColumnClustered",
        _3DColumnStacked = "3DColumnStacked",
        _3DColumnStacked100 = "3DColumnStacked100",
        barClustered = "BarClustered",
        barStacked = "BarStacked",
        barStacked100 = "BarStacked100",
        _3DBarClustered = "3DBarClustered",
        _3DBarStacked = "3DBarStacked",
        _3DBarStacked100 = "3DBarStacked100",
        lineStacked = "LineStacked",
        lineStacked100 = "LineStacked100",
        lineMarkers = "LineMarkers",
        lineMarkersStacked = "LineMarkersStacked",
        lineMarkersStacked100 = "LineMarkersStacked100",
        pieOfPie = "PieOfPie",
        pieExploded = "PieExploded",
        _3DPieExploded = "3DPieExploded",
        barOfPie = "BarOfPie",
        xyscatterSmooth = "XYScatterSmooth",
        xyscatterSmoothNoMarkers = "XYScatterSmoothNoMarkers",
        xyscatterLines = "XYScatterLines",
        xyscatterLinesNoMarkers = "XYScatterLinesNoMarkers",
        areaStacked = "AreaStacked",
        areaStacked100 = "AreaStacked100",
        _3DAreaStacked = "3DAreaStacked",
        _3DAreaStacked100 = "3DAreaStacked100",
        doughnutExploded = "DoughnutExploded",
        radarMarkers = "RadarMarkers",
        radarFilled = "RadarFilled",
        surface = "Surface",
        surfaceWireframe = "SurfaceWireframe",
        surfaceTopView = "SurfaceTopView",
        surfaceTopViewWireframe = "SurfaceTopViewWireframe",
        bubble = "Bubble",
        bubble3DEffect = "Bubble3DEffect",
        stockHLC = "StockHLC",
        stockOHLC = "StockOHLC",
        stockVHLC = "StockVHLC",
        stockVOHLC = "StockVOHLC",
        cylinderColClustered = "CylinderColClustered",
        cylinderColStacked = "CylinderColStacked",
        cylinderColStacked100 = "CylinderColStacked100",
        cylinderBarClustered = "CylinderBarClustered",
        cylinderBarStacked = "CylinderBarStacked",
        cylinderBarStacked100 = "CylinderBarStacked100",
        cylinderCol = "CylinderCol",
        coneColClustered = "ConeColClustered",
        coneColStacked = "ConeColStacked",
        coneColStacked100 = "ConeColStacked100",
        coneBarClustered = "ConeBarClustered",
        coneBarStacked = "ConeBarStacked",
        coneBarStacked100 = "ConeBarStacked100",
        coneCol = "ConeCol",
        pyramidColClustered = "PyramidColClustered",
        pyramidColStacked = "PyramidColStacked",
        pyramidColStacked100 = "PyramidColStacked100",
        pyramidBarClustered = "PyramidBarClustered",
        pyramidBarStacked = "PyramidBarStacked",
        pyramidBarStacked100 = "PyramidBarStacked100",
        pyramidCol = "PyramidCol",
        _3DColumn = "3DColumn",
        line = "Line",
        _3DLine = "3DLine",
        _3DPie = "3DPie",
        pie = "Pie",
        xyscatter = "XYScatter",
        _3DArea = "3DArea",
        area = "Area",
        doughnut = "Doughnut",
        radar = "Radar",
        histogram = "Histogram",
        boxwhisker = "Boxwhisker",
        pareto = "Pareto",
        regionMap = "RegionMap",
        treemap = "Treemap",
        waterfall = "Waterfall",
        sunburst = "Sunburst",
        funnel = "Funnel"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum ChartUnderlineStyle {
        none = "None",
        single = "Single"
    }
    /**
     * [Api set: ExcelApi 1.8]
     */
    enum ChartDisplayBlanksAs {
        notPlotted = "NotPlotted",
        zero = "Zero",
        interplotted = "Interplotted"
    }
    /**
     * [Api set: ExcelApi 1.8]
     */
    enum ChartPlotBy {
        rows = "Rows",
        columns = "Columns"
    }
    /**
     * [Api set: ExcelApi 1.8]
     */
    enum ChartSplitType {
        splitByPosition = "SplitByPosition",
        splitByValue = "SplitByValue",
        splitByPercentValue = "SplitByPercentValue",
        splitByCustomSplit = "SplitByCustomSplit"
    }
    /**
     * [Api set: ExcelApi 1.8]
     */
    enum ChartColorScheme {
        colorfulPalette1 = "ColorfulPalette1",
        colorfulPalette2 = "ColorfulPalette2",
        colorfulPalette3 = "ColorfulPalette3",
        colorfulPalette4 = "ColorfulPalette4",
        monochromaticPalette1 = "MonochromaticPalette1",
        monochromaticPalette2 = "MonochromaticPalette2",
        monochromaticPalette3 = "MonochromaticPalette3",
        monochromaticPalette4 = "MonochromaticPalette4",
        monochromaticPalette5 = "MonochromaticPalette5",
        monochromaticPalette6 = "MonochromaticPalette6",
        monochromaticPalette7 = "MonochromaticPalette7",
        monochromaticPalette8 = "MonochromaticPalette8",
        monochromaticPalette9 = "MonochromaticPalette9",
        monochromaticPalette10 = "MonochromaticPalette10",
        monochromaticPalette11 = "MonochromaticPalette11",
        monochromaticPalette12 = "MonochromaticPalette12",
        monochromaticPalette13 = "MonochromaticPalette13"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum ChartTrendlineType {
        linear = "Linear",
        exponential = "Exponential",
        logarithmic = "Logarithmic",
        movingAverage = "MovingAverage",
        polynomial = "Polynomial",
        power = "Power"
    }
    /**
     *
     * Specifies where in the z-order a shape should be moved relative to other shapes.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ShapeZOrder {
        bringToFront = "BringToFront",
        bringForward = "BringForward",
        sendToBack = "SendToBack",
        sendBackward = "SendBackward"
    }
    /**
     *
     * Specifies the type of a shape.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ShapeType {
        unsupported = "Unsupported",
        image = "Image",
        geometricShape = "GeometricShape",
        group = "Group",
        line = "Line"
    }
    /**
     *
     * Specifies whether the shape is scaled relative to its original or current size.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ShapeScaleType {
        currentSize = "CurrentSize",
        originalSize = "OriginalSize"
    }
    /**
     *
     * Specifies which part of the shape retains its position when the shape is scaled.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ShapeScaleFrom {
        scaleFromTopLeft = "ScaleFromTopLeft",
        scaleFromMiddle = "ScaleFromMiddle",
        scaleFromBottomRight = "ScaleFromBottomRight"
    }
    /**
     *
     * Specifies a shape's fill type.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ShapeFillType {
        /**
         *
         * No fill.
         *
         */
        noFill = "NoFill",
        /**
         *
         * Solid fill.
         *
         */
        solid = "Solid",
        /**
         *
         * Gradient fill.
         *
         */
        gradient = "Gradient",
        /**
         *
         * Pattern fill.
         *
         */
        pattern = "Pattern",
        /**
         *
         * Picture and texture fill.
         *
         */
        pictureAndTexture = "PictureAndTexture",
        /**
         *
         * Mixed fill.
         *
         */
        mixed = "Mixed"
    }
    /**
     *
     * The type of underline applied to a font.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ShapeFontUnderlineStyle {
        none = "None",
        single = "Single",
        double = "Double",
        heavy = "Heavy",
        dotted = "Dotted",
        dottedHeavy = "DottedHeavy",
        dash = "Dash",
        dashHeavy = "DashHeavy",
        dashLong = "DashLong",
        dashLongHeavy = "DashLongHeavy",
        dotDash = "DotDash",
        dotDashHeavy = "DotDashHeavy",
        dotDotDash = "DotDotDash",
        dotDotDashHeavy = "DotDotDashHeavy",
        wavy = "Wavy",
        wavyHeavy = "WavyHeavy",
        wavyDouble = "WavyDouble"
    }
    /**
     *
     * The format of the image.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum PictureFormat {
        unknown = "UNKNOWN",
        /**
         *
         * Bitmap image.
         *
         */
        bmp = "BMP",
        /**
         *
         * Joint Photographic Experts Group.
         *
         */
        jpeg = "JPEG",
        /**
         *
         * Graphics Interchange Format.
         *
         */
        gif = "GIF",
        /**
         *
         * Portable Network Graphics.
         *
         */
        png = "PNG",
        /**
         *
         * Scalable Vector Graphic.
         *
         */
        svg = "SVG"
    }
    /**
     *
     * The style for a line.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ShapeLineStyle {
        /**
         *
         * Single line.
         *
         */
        single = "Single",
        /**
         *
         * Thick line with a thin line on each side.
         *
         */
        thickBetweenThin = "ThickBetweenThin",
        /**
         *
         * Thick line next to thin line. For horizontal lines, the thick line is above the thin line. For vertical lines, the thick line is to the left of the thin line.
         *
         */
        thickThin = "ThickThin",
        /**
         *
         * Thick line next to thin line. For horizontal lines, the thick line is below the thin line. For vertical lines, the thick line is to the right of the thin line.
         *
         */
        thinThick = "ThinThick",
        /**
         *
         * Two thin lines.
         *
         */
        thinThin = "ThinThin"
    }
    /**
     *
     * The dash style for a line.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ShapeLineDashStyle {
        dash = "Dash",
        dashDot = "DashDot",
        dashDotDot = "DashDotDot",
        longDash = "LongDash",
        longDashDot = "LongDashDot",
        roundDot = "RoundDot",
        solid = "Solid",
        squareDot = "SquareDot",
        longDashDotDot = "LongDashDotDot",
        systemDash = "SystemDash",
        systemDot = "SystemDot",
        systemDashDot = "SystemDashDot"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum ArrowheadLength {
        short = "Short",
        medium = "Medium",
        long = "Long"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum ArrowheadStyle {
        none = "None",
        triangle = "Triangle",
        stealth = "Stealth",
        diamond = "Diamond",
        oval = "Oval",
        open = "Open"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum ArrowheadWidth {
        narrow = "Narrow",
        medium = "Medium",
        wide = "Wide"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum BindingType {
        range = "Range",
        table = "Table",
        text = "Text"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum BorderIndex {
        edgeTop = "EdgeTop",
        edgeBottom = "EdgeBottom",
        edgeLeft = "EdgeLeft",
        edgeRight = "EdgeRight",
        insideVertical = "InsideVertical",
        insideHorizontal = "InsideHorizontal",
        diagonalDown = "DiagonalDown",
        diagonalUp = "DiagonalUp"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum BorderLineStyle {
        none = "None",
        continuous = "Continuous",
        dash = "Dash",
        dashDot = "DashDot",
        dashDotDot = "DashDotDot",
        dot = "Dot",
        double = "Double",
        slantDashDot = "SlantDashDot"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum BorderWeight {
        hairline = "Hairline",
        thin = "Thin",
        medium = "Medium",
        thick = "Thick"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum CalculationMode {
        /**
         *
         * The default recalculation behavior where Excel calculates new formula results every time the relevant data is changed.
         *
         */
        automatic = "Automatic",
        /**
         *
         * Calculates new formula results every time the relevant data is changed, unless the formula is in a data table.
         *
         */
        automaticExceptTables = "AutomaticExceptTables",
        /**
         *
         * Calculations only occur when the user or add-in requests them.
         *
         */
        manual = "Manual"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum CalculationType {
        /**
         *
         * Recalculates all cells that Excel has marked as dirty, that is, dependents of volatile or changed data, and cells programmatically marked as dirty.
         *
         */
        recalculate = "Recalculate",
        /**
         *
         * This will mark all cells as dirty and then recalculate them.
         *
         */
        full = "Full",
        /**
         *
         * This will rebuild the full dependency chain, mark all cells as dirty and then recalculate them.
         *
         */
        fullRebuild = "FullRebuild"
    }
    /**
     * [Api set: ExcelApi 1.1 for All/Formats/Contents, 1.7 for Hyperlinks & HyperlinksAndFormats.]
     */
    enum ClearApplyTo {
        all = "All",
        /**
         *
         * Clears all formatting for the range.
         *
         */
        formats = "Formats",
        /**
         *
         * Clears the contents of the range.
         *
         */
        contents = "Contents",
        /**
         *
         * Clears all hyperlinks, but leaves all content and formatting intact.
         *
         */
        hyperlinks = "Hyperlinks",
        /**
         *
         * Removes hyperlinks and formatting for the cell but leaves content, conditional formats, and data validation intact.
         *
         */
        removeHyperlinks = "RemoveHyperlinks"
    }
    /**
     *
     * Represents the format options for a Data Bar Axis.
     *
     * [Api set: ExcelApi 1.6]
     */
    enum ConditionalDataBarAxisFormat {
        automatic = "Automatic",
        none = "None",
        cellMidPoint = "CellMidPoint"
    }
    /**
     *
     * Represents the Data Bar direction within a cell.
     *
     * [Api set: ExcelApi 1.6]
     */
    enum ConditionalDataBarDirection {
        context = "Context",
        leftToRight = "LeftToRight",
        rightToLeft = "RightToLeft"
    }
    /**
     *
     * Represents the direction for a selection.
     *
     * [Api set: ExcelApi 1.6]
     */
    enum ConditionalFormatDirection {
        top = "Top",
        bottom = "Bottom"
    }
    /**
     * [Api set: ExcelApi 1.6]
     */
    enum ConditionalFormatType {
        custom = "Custom",
        dataBar = "DataBar",
        colorScale = "ColorScale",
        iconSet = "IconSet",
        topBottom = "TopBottom",
        presetCriteria = "PresetCriteria",
        containsText = "ContainsText",
        cellValue = "CellValue"
    }
    /**
     *
     * Represents the types of conditional format values.
     *
     * [Api set: ExcelApi 1.6]
     */
    enum ConditionalFormatRuleType {
        invalid = "Invalid",
        automatic = "Automatic",
        lowestValue = "LowestValue",
        highestValue = "HighestValue",
        number = "Number",
        percent = "Percent",
        formula = "Formula",
        percentile = "Percentile"
    }
    /**
     *
     * Represents the types of conditional format values.
     *
     * [Api set: ExcelApi 1.6]
     */
    enum ConditionalFormatIconRuleType {
        invalid = "Invalid",
        number = "Number",
        percent = "Percent",
        formula = "Formula",
        percentile = "Percentile"
    }
    /**
     *
     * Represents the types of conditional format values.
     *
     * [Api set: ExcelApi 1.6]
     */
    enum ConditionalFormatColorCriterionType {
        invalid = "Invalid",
        lowestValue = "LowestValue",
        highestValue = "HighestValue",
        number = "Number",
        percent = "Percent",
        formula = "Formula",
        percentile = "Percentile"
    }
    /**
     *
     * Represents the criteria for the above/below average conditional format type.
     *
     * [Api set: ExcelApi 1.6]
     */
    enum ConditionalTopBottomCriterionType {
        invalid = "Invalid",
        topItems = "TopItems",
        topPercent = "TopPercent",
        bottomItems = "BottomItems",
        bottomPercent = "BottomPercent"
    }
    /**
     *
     * Represents the criteria for the Preset Criteria conditional format type.
     *
     * [Api set: ExcelApi 1.6]
     */
    enum ConditionalFormatPresetCriterion {
        invalid = "Invalid",
        blanks = "Blanks",
        nonBlanks = "NonBlanks",
        errors = "Errors",
        nonErrors = "NonErrors",
        yesterday = "Yesterday",
        today = "Today",
        tomorrow = "Tomorrow",
        lastSevenDays = "LastSevenDays",
        lastWeek = "LastWeek",
        thisWeek = "ThisWeek",
        nextWeek = "NextWeek",
        lastMonth = "LastMonth",
        thisMonth = "ThisMonth",
        nextMonth = "NextMonth",
        aboveAverage = "AboveAverage",
        belowAverage = "BelowAverage",
        equalOrAboveAverage = "EqualOrAboveAverage",
        equalOrBelowAverage = "EqualOrBelowAverage",
        oneStdDevAboveAverage = "OneStdDevAboveAverage",
        oneStdDevBelowAverage = "OneStdDevBelowAverage",
        twoStdDevAboveAverage = "TwoStdDevAboveAverage",
        twoStdDevBelowAverage = "TwoStdDevBelowAverage",
        threeStdDevAboveAverage = "ThreeStdDevAboveAverage",
        threeStdDevBelowAverage = "ThreeStdDevBelowAverage",
        uniqueValues = "UniqueValues",
        duplicateValues = "DuplicateValues"
    }
    /**
     *
     * Represents the operator of the text conditional format type.
     *
     * [Api set: ExcelApi 1.6]
     */
    enum ConditionalTextOperator {
        invalid = "Invalid",
        contains = "Contains",
        notContains = "NotContains",
        beginsWith = "BeginsWith",
        endsWith = "EndsWith"
    }
    /**
     *
     * Represents the operator of the text conditional format type.
     *
     * [Api set: ExcelApi 1.6]
     */
    enum ConditionalCellValueOperator {
        invalid = "Invalid",
        between = "Between",
        notBetween = "NotBetween",
        equalTo = "EqualTo",
        notEqualTo = "NotEqualTo",
        greaterThan = "GreaterThan",
        lessThan = "LessThan",
        greaterThanOrEqual = "GreaterThanOrEqual",
        lessThanOrEqual = "LessThanOrEqual"
    }
    /**
     *
     * Represents the operator for each icon criteria.
     *
     * [Api set: ExcelApi 1.6]
     */
    enum ConditionalIconCriterionOperator {
        invalid = "Invalid",
        greaterThan = "GreaterThan",
        greaterThanOrEqual = "GreaterThanOrEqual"
    }
    /**
     * [Api set: ExcelApi 1.6]
     */
    enum ConditionalRangeBorderIndex {
        edgeTop = "EdgeTop",
        edgeBottom = "EdgeBottom",
        edgeLeft = "EdgeLeft",
        edgeRight = "EdgeRight"
    }
    /**
     * [Api set: ExcelApi 1.6]
     */
    enum ConditionalRangeBorderLineStyle {
        none = "None",
        continuous = "Continuous",
        dash = "Dash",
        dashDot = "DashDot",
        dashDotDot = "DashDotDot",
        dot = "Dot"
    }
    /**
     * [Api set: ExcelApi 1.6]
     */
    enum ConditionalRangeFontUnderlineStyle {
        none = "None",
        single = "Single",
        double = "Double"
    }
    /**
     *
     * Represents Data validation type enum.
     *
     * [Api set: ExcelApi 1.8]
     */
    enum DataValidationType {
        /**
         *
         * None means allow any value and so there is no data validation in the range.
         *
         */
        none = "None",
        /**
         *
         * Whole number data validation type
         *
         */
        wholeNumber = "WholeNumber",
        /**
         *
         * Decimal data validation type
         *
         */
        decimal = "Decimal",
        /**
         *
         * List data validation type
         *
         */
        list = "List",
        /**
         *
         * Date data validation type
         *
         */
        date = "Date",
        /**
         *
         * Time data validation type
         *
         */
        time = "Time",
        /**
         *
         * Text length data validation type
         *
         */
        textLength = "TextLength",
        /**
         *
         * Custom data validation type
         *
         */
        custom = "Custom",
        /**
         *
         * Inconsistent means that the range has inconsistent data validation (there are different rules on different cells)
         *
         */
        inconsistent = "Inconsistent",
        /**
         *
         * MixedCriteria means that the range has data validation present on some but not all cells
         *
         */
        mixedCriteria = "MixedCriteria"
    }
    /**
     *
     * Represents Data validation operator enum.
     *
     * [Api set: ExcelApi 1.8]
     */
    enum DataValidationOperator {
        between = "Between",
        notBetween = "NotBetween",
        equalTo = "EqualTo",
        notEqualTo = "NotEqualTo",
        greaterThan = "GreaterThan",
        lessThan = "LessThan",
        greaterThanOrEqualTo = "GreaterThanOrEqualTo",
        lessThanOrEqualTo = "LessThanOrEqualTo"
    }
    /**
     *
     * Represents Data validation error alert style. The default is "Stop".
     *
     * [Api set: ExcelApi 1.8]
     */
    enum DataValidationAlertStyle {
        stop = "Stop",
        warning = "Warning",
        information = "Information"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum DeleteShiftDirection {
        up = "Up",
        left = "Left"
    }
    /**
     * [Api set: ExcelApi 1.2]
     */
    enum DynamicFilterCriteria {
        unknown = "Unknown",
        aboveAverage = "AboveAverage",
        allDatesInPeriodApril = "AllDatesInPeriodApril",
        allDatesInPeriodAugust = "AllDatesInPeriodAugust",
        allDatesInPeriodDecember = "AllDatesInPeriodDecember",
        allDatesInPeriodFebruray = "AllDatesInPeriodFebruray",
        allDatesInPeriodJanuary = "AllDatesInPeriodJanuary",
        allDatesInPeriodJuly = "AllDatesInPeriodJuly",
        allDatesInPeriodJune = "AllDatesInPeriodJune",
        allDatesInPeriodMarch = "AllDatesInPeriodMarch",
        allDatesInPeriodMay = "AllDatesInPeriodMay",
        allDatesInPeriodNovember = "AllDatesInPeriodNovember",
        allDatesInPeriodOctober = "AllDatesInPeriodOctober",
        allDatesInPeriodQuarter1 = "AllDatesInPeriodQuarter1",
        allDatesInPeriodQuarter2 = "AllDatesInPeriodQuarter2",
        allDatesInPeriodQuarter3 = "AllDatesInPeriodQuarter3",
        allDatesInPeriodQuarter4 = "AllDatesInPeriodQuarter4",
        allDatesInPeriodSeptember = "AllDatesInPeriodSeptember",
        belowAverage = "BelowAverage",
        lastMonth = "LastMonth",
        lastQuarter = "LastQuarter",
        lastWeek = "LastWeek",
        lastYear = "LastYear",
        nextMonth = "NextMonth",
        nextQuarter = "NextQuarter",
        nextWeek = "NextWeek",
        nextYear = "NextYear",
        thisMonth = "ThisMonth",
        thisQuarter = "ThisQuarter",
        thisWeek = "ThisWeek",
        thisYear = "ThisYear",
        today = "Today",
        tomorrow = "Tomorrow",
        yearToDate = "YearToDate",
        yesterday = "Yesterday"
    }
    /**
     * [Api set: ExcelApi 1.2]
     */
    enum FilterDatetimeSpecificity {
        year = "Year",
        month = "Month",
        day = "Day",
        hour = "Hour",
        minute = "Minute",
        second = "Second"
    }
    /**
     * [Api set: ExcelApi 1.2]
     */
    enum FilterOn {
        bottomItems = "BottomItems",
        bottomPercent = "BottomPercent",
        cellColor = "CellColor",
        dynamic = "Dynamic",
        fontColor = "FontColor",
        values = "Values",
        topItems = "TopItems",
        topPercent = "TopPercent",
        icon = "Icon",
        custom = "Custom"
    }
    /**
     * [Api set: ExcelApi 1.2]
     */
    enum FilterOperator {
        and = "And",
        or = "Or"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum HorizontalAlignment {
        general = "General",
        left = "Left",
        center = "Center",
        right = "Right",
        fill = "Fill",
        justify = "Justify",
        centerAcrossSelection = "CenterAcrossSelection",
        distributed = "Distributed"
    }
    /**
     * [Api set: ExcelApi 1.2]
     */
    enum IconSet {
        invalid = "Invalid",
        threeArrows = "ThreeArrows",
        threeArrowsGray = "ThreeArrowsGray",
        threeFlags = "ThreeFlags",
        threeTrafficLights1 = "ThreeTrafficLights1",
        threeTrafficLights2 = "ThreeTrafficLights2",
        threeSigns = "ThreeSigns",
        threeSymbols = "ThreeSymbols",
        threeSymbols2 = "ThreeSymbols2",
        fourArrows = "FourArrows",
        fourArrowsGray = "FourArrowsGray",
        fourRedToBlack = "FourRedToBlack",
        fourRating = "FourRating",
        fourTrafficLights = "FourTrafficLights",
        fiveArrows = "FiveArrows",
        fiveArrowsGray = "FiveArrowsGray",
        fiveRating = "FiveRating",
        fiveQuarters = "FiveQuarters",
        threeStars = "ThreeStars",
        threeTriangles = "ThreeTriangles",
        fiveBoxes = "FiveBoxes",
        linkedEntityFinanceIcon = "LinkedEntityFinanceIcon",
        linkedEntityMapIcon = "LinkedEntityMapIcon"
    }
    /**
     * [Api set: ExcelApi 1.2]
     */
    enum ImageFittingMode {
        fit = "Fit",
        fitAndCenter = "FitAndCenter",
        fill = "Fill"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum InsertShiftDirection {
        down = "Down",
        right = "Right"
    }
    /**
     * [Api set: ExcelApi 1.4]
     */
    enum NamedItemScope {
        worksheet = "Worksheet",
        workbook = "Workbook"
    }
    /**
     * [Api set: ExcelApi 1.1 for String,Integer,Double,Boolean,Range,Error; 1.7 for Array]
     */
    enum NamedItemType {
        string = "String",
        integer = "Integer",
        double = "Double",
        boolean = "Boolean",
        range = "Range",
        error = "Error",
        array = "Array"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum RangeUnderlineStyle {
        none = "None",
        single = "Single",
        double = "Double",
        singleAccountant = "SingleAccountant",
        doubleAccountant = "DoubleAccountant"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum SheetVisibility {
        visible = "Visible",
        hidden = "Hidden",
        veryHidden = "VeryHidden"
    }
    /**
     * [Api set: ExcelApi 1.1 for Unknown, Empty, String, Integer, Double, Boolean, Error. 1.7 for RichValue]
     */
    enum RangeValueType {
        unknown = "Unknown",
        empty = "Empty",
        string = "String",
        integer = "Integer",
        double = "Double",
        boolean = "Boolean",
        error = "Error",
        richValue = "RichValue"
    }
    /**
     *
     * Specifies the search direction.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum SearchDirection {
        /**
         *
         * Search in forward order.
         *
         */
        forward = "Forward",
        /**
         *
         * Search in reverse order.
         *
         */
        backwards = "Backwards"
    }
    /**
     * [Api set: ExcelApi 1.2]
     */
    enum SortOrientation {
        rows = "Rows",
        columns = "Columns"
    }
    /**
     * [Api set: ExcelApi 1.2]
     */
    enum SortOn {
        value = "Value",
        cellColor = "CellColor",
        fontColor = "FontColor",
        icon = "Icon"
    }
    /**
     * [Api set: ExcelApi 1.2]
     */
    enum SortDataOption {
        normal = "Normal",
        textAsNumber = "TextAsNumber"
    }
    /**
     * [Api set: ExcelApi 1.2]
     */
    enum SortMethod {
        pinYin = "PinYin",
        strokeCount = "StrokeCount"
    }
    /**
     * [Api set: ExcelApi 1.1]
     */
    enum VerticalAlignment {
        top = "Top",
        center = "Center",
        bottom = "Bottom",
        justify = "Justify",
        distributed = "Distributed"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum DocumentPropertyType {
        number = "Number",
        boolean = "Boolean",
        date = "Date",
        string = "String",
        float = "Float"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum EventSource {
        /**
         *
         * Local means event comes from local user session.
         *
         */
        local = "Local",
        /**
         *
         * Remote means event comes from remote user session.
         *
         */
        remote = "Remote"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum DataChangeType {
        /**
         *
         * Unknown represents the type of data change is not the listed types.
         *
         */
        unknown = "Unknown",
        /**
         *
         * RangeEdited represents the data change event is triggered by range being edited.
         *
         */
        rangeEdited = "RangeEdited",
        /**
         *
         * RowInserted represents the data change event is triggered by inserting new rows.
         *
         */
        rowInserted = "RowInserted",
        /**
         *
         * RowDeleted represents the data change event is triggered by deleting rows.
         *
         */
        rowDeleted = "RowDeleted",
        /**
         *
         * ColumnInserted represents the data change event is triggered by inserting new columns.
         *
         */
        columnInserted = "ColumnInserted",
        /**
         *
         * ColumnDeleted represents the data change event is triggered by deleting columns.
         *
         */
        columnDeleted = "ColumnDeleted",
        /**
         *
         * CellInserted represents the data change event is triggered by inserting new cells.
         *
         */
        cellInserted = "CellInserted",
        /**
         *
         * CellDeleted represents the data change event is triggered by deleting cells.
         *
         */
        cellDeleted = "CellDeleted"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum EventType {
        /**
         *
         * WorksheetChanged represents the type of event registered on Worksheet or WorksheetCollection, and occurs when data changes.
         *
         */
        worksheetChanged = "WorksheetChanged",
        /**
         *
         * WorksheetSelectionChanged represents the type of event registered on Worksheet, and occurs when selection changes.
         *
         */
        worksheetSelectionChanged = "WorksheetSelectionChanged",
        /**
         *
         * WorksheetAdded represents the type of event registered on WorksheetCollection, and occurs when a new worksheet is added to the workbook.
         *
         */
        worksheetAdded = "WorksheetAdded",
        /**
         *
         * WorksheetActivated represents the type of event registered on Worksheet or WorksheetCollection, and occurs when worksheet activates.
         *
         */
        worksheetActivated = "WorksheetActivated",
        /**
         *
         * WorksheetDeactivated represents the type of event registered on Worksheet or WorksheetCollection, and occurs when worksheet deactivates.
         *
         */
        worksheetDeactivated = "WorksheetDeactivated",
        /**
         *
         * TableChanged represents the type of event registered on Table, and occurs when data changes.
         *
         */
        tableChanged = "TableChanged",
        /**
         *
         * TableSelectionChanged represents the type of event registered on Table, and occurs when selection changes.
         *
         */
        tableSelectionChanged = "TableSelectionChanged",
        /**
         *
         * WorksheetDeleted represents the type of event registered on WorksheetCollection, and occurs when a worksheet is deleted from the workbook.
         *
         */
        worksheetDeleted = "WorksheetDeleted",
        /**
         *
         * ChartAdded represents the type of event registered on ChartCollection, and occurs when a new chart is added to the worksheet.
         *
         */
        chartAdded = "ChartAdded",
        /**
         *
         * ChartActivated represents the type of event registered on Chart or ChartCollection, and occurs when chart activates.
         *
         */
        chartActivated = "ChartActivated",
        /**
         *
         * ChartDeactivated represents the type of event registered on Chart or ChartCollection, and occurs when chart deactivates.
         *
         */
        chartDeactivated = "ChartDeactivated",
        /**
         *
         * ChartDeleted represents the type of event registered on ChartCollection, and occurs when a chart is deleted from the worksheet.
         *
         */
        chartDeleted = "ChartDeleted",
        /**
         *
         * WorksheetCalculated represents the type of event registered on Worksheet or WorksheetCollection, and occurs when a worksheet is calculated.
         *
         */
        worksheetCalculated = "WorksheetCalculated",
        /**
         *
         * VisualSelectionChanged represents the type of event registered on VisualCollection, and occurs when visual selection changes.
         *
         */
        visualSelectionChanged = "VisualSelectionChanged",
        /**
         *
         * AgaveVisualUpdate represents the type of an event that is associated with an agave visual, and carries a new data view following a data change
         *
         */
        agaveVisualUpdate = "AgaveVisualUpdate",
        /**
         *
         * TableAdded represents the type of event registered on TableCollection, and occurs when a table is added.
         *
         */
        tableAdded = "TableAdded",
        /**
         *
         * TableDeleted represents the type of event that is registered on TableCollection, and occurs when a table is deleted.
         *
         */
        tableDeleted = "TableDeleted",
        /**
         *
         * TableFiltered represents the type of event registered on table or TableCollection, and occurs when data of table is filtered.
         *
         */
        tableFiltered = "TableFiltered",
        /**
         *
         * WorksheetFiltered represents the type of event registered on worksheet or WorksheetCollection, and occurs when data of autofilter in the worksheet is filtered.
         *
         */
        worksheetFiltered = "WorksheetFiltered",
        /**
         *
         * ShapeActivated represents the type of event that is registered on Shape, and occurs when shape activates.
         *
         */
        shapeActivated = "ShapeActivated",
        /**
         *
         * ShapeDeactivated represents the type of event that is registered on Shape, and occurs when shape deactivates.
         *
         */
        shapeDeactivated = "ShapeDeactivated",
        /**
         *
         * VisualChange represents the type of event registered on Visual, and occurs when there is a visual change.
         *
         */
        visualChange = "VisualChange",
        /**
         *
         * WorkbookAutoSaveSettingChanged represents the type of event registered on workbook, and occurs when there is an auto save setting change.
         *
         */
        workbookAutoSaveSettingChanged = "WorkbookAutoSaveSettingChanged",
        /**
         *
         * WorksheetFormatChanged represents the type of event registered on worksheet, and occurs when there is a format changed.
         *
         */
        worksheetFormatChanged = "WorksheetFormatChanged",
        wacoperationEvent = "WACOperationEvent",
        /**
         *
         * RibbonCommandExecuted represents the type of event registered on ribbon, and occurs when user click on ribbon
         *
         */
        ribbonCommandExecuted = "RibbonCommandExecuted",
        /**
         *
         * WorksheetRowSorted represents the type of event registered on worksheet, and occurs when there is a sorting on rows happened.
         *
         */
        worksheetRowSorted = "WorksheetRowSorted",
        /**
         *
         * WorksheetColumnSorted represents the type of event registered on worksheet, and occurs when there is a sorting on columns happened.
         *
         */
        worksheetColumnSorted = "WorksheetColumnSorted",
        /**
         *
         * WorksheetSingleClicked represents the type of event registered on worksheet, and occurs when cell is left-clicked/tapped.
         *
         */
        worksheetSingleClicked = "WorksheetSingleClicked",
        /**
         *
         * WorksheetRowHiddenChanged represents the type of event registered on worksheet, and occurs when there is a row hidden changed.
         *
         */
        worksheetRowHiddenChanged = "WorksheetRowHiddenChanged"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum DocumentPropertyItem {
        title = "Title",
        subject = "Subject",
        author = "Author",
        keywords = "Keywords",
        comments = "Comments",
        template = "Template",
        lastAuth = "LastAuth",
        revision = "Revision",
        appName = "AppName",
        lastPrint = "LastPrint",
        creation = "Creation",
        lastSave = "LastSave",
        category = "Category",
        format = "Format",
        manager = "Manager",
        company = "Company"
    }
    /**
     * [Api set: ExcelApi 1.8]
     */
    enum SubtotalLocationType {
        /**
         *
         * Subtotals are at the top.
         *
         */
        atTop = "AtTop",
        /**
         *
         * Subtotals are at the bottom.
         *
         */
        atBottom = "AtBottom",
        /**
         *
         * Subtotals are off.
         *
         */
        off = "Off"
    }
    /**
     * [Api set: ExcelApi 1.8]
     */
    enum PivotLayoutType {
        /**
         *
         * A horizontally compressed form with labels from the next field in the same column.
         *
         */
        compact = "Compact",
        /**
         *
         * Inner fields' items are always on a new line relative to the outer fields' items.
         *
         */
        tabular = "Tabular",
        /**
         *
         * Inner fields' items are on same row as outer fields' items and subtotals are always on the bottom.
         *
         */
        outline = "Outline"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum ProtectionSelectionMode {
        /**
         *
         * Selection is allowed for all cells.
         *
         */
        normal = "Normal",
        /**
         *
         * Selection is allowed only for cells that are not locked.
         *
         */
        unlocked = "Unlocked",
        /**
         *
         * Selection is not allowed for all cells.
         *
         */
        none = "None"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum PageOrientation {
        portrait = "Portrait",
        landscape = "Landscape"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum PaperType {
        letter = "Letter",
        letterSmall = "LetterSmall",
        tabloid = "Tabloid",
        ledger = "Ledger",
        legal = "Legal",
        statement = "Statement",
        executive = "Executive",
        a3 = "A3",
        a4 = "A4",
        a4Small = "A4Small",
        a5 = "A5",
        b4 = "B4",
        b5 = "B5",
        folio = "Folio",
        quatro = "Quatro",
        paper10x14 = "Paper10x14",
        paper11x17 = "Paper11x17",
        note = "Note",
        envelope9 = "Envelope9",
        envelope10 = "Envelope10",
        envelope11 = "Envelope11",
        envelope12 = "Envelope12",
        envelope14 = "Envelope14",
        csheet = "Csheet",
        dsheet = "Dsheet",
        esheet = "Esheet",
        envelopeDL = "EnvelopeDL",
        envelopeC5 = "EnvelopeC5",
        envelopeC3 = "EnvelopeC3",
        envelopeC4 = "EnvelopeC4",
        envelopeC6 = "EnvelopeC6",
        envelopeC65 = "EnvelopeC65",
        envelopeB4 = "EnvelopeB4",
        envelopeB5 = "EnvelopeB5",
        envelopeB6 = "EnvelopeB6",
        envelopeItaly = "EnvelopeItaly",
        envelopeMonarch = "EnvelopeMonarch",
        envelopePersonal = "EnvelopePersonal",
        fanfoldUS = "FanfoldUS",
        fanfoldStdGerman = "FanfoldStdGerman",
        fanfoldLegalGerman = "FanfoldLegalGerman"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum ReadingOrder {
        /**
         *
         * Reading order is determined by the language of the first character entered.
            If a right-to-left language character is entered first, reading order is right to left.
            If a left-to-right language character is entered first, reading order is left to right.
         *
         */
        context = "Context",
        /**
         *
         * Left to right reading order
         *
         */
        leftToRight = "LeftToRight",
        /**
         *
         * Right to left reading order
         *
         */
        rightToLeft = "RightToLeft"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum BuiltInStyle {
        normal = "Normal",
        comma = "Comma",
        currency = "Currency",
        percent = "Percent",
        wholeComma = "WholeComma",
        wholeDollar = "WholeDollar",
        hlink = "Hlink",
        hlinkTrav = "HlinkTrav",
        note = "Note",
        warningText = "WarningText",
        emphasis1 = "Emphasis1",
        emphasis2 = "Emphasis2",
        emphasis3 = "Emphasis3",
        sheetTitle = "SheetTitle",
        heading1 = "Heading1",
        heading2 = "Heading2",
        heading3 = "Heading3",
        heading4 = "Heading4",
        input = "Input",
        output = "Output",
        calculation = "Calculation",
        checkCell = "CheckCell",
        linkedCell = "LinkedCell",
        total = "Total",
        good = "Good",
        bad = "Bad",
        neutral = "Neutral",
        accent1 = "Accent1",
        accent1_20 = "Accent1_20",
        accent1_40 = "Accent1_40",
        accent1_60 = "Accent1_60",
        accent2 = "Accent2",
        accent2_20 = "Accent2_20",
        accent2_40 = "Accent2_40",
        accent2_60 = "Accent2_60",
        accent3 = "Accent3",
        accent3_20 = "Accent3_20",
        accent3_40 = "Accent3_40",
        accent3_60 = "Accent3_60",
        accent4 = "Accent4",
        accent4_20 = "Accent4_20",
        accent4_40 = "Accent4_40",
        accent4_60 = "Accent4_60",
        accent5 = "Accent5",
        accent5_20 = "Accent5_20",
        accent5_40 = "Accent5_40",
        accent5_60 = "Accent5_60",
        accent6 = "Accent6",
        accent6_20 = "Accent6_20",
        accent6_40 = "Accent6_40",
        accent6_60 = "Accent6_60",
        explanatoryText = "ExplanatoryText"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum PrintErrorType {
        asDisplayed = "AsDisplayed",
        blank = "Blank",
        dash = "Dash",
        notAvailable = "NotAvailable"
    }
    /**
     * [Api set: ExcelApi 1.7]
     */
    enum WorksheetPositionType {
        none = "None",
        before = "Before",
        after = "After",
        beginning = "Beginning",
        end = "End"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum PrintComments {
        /**
         *
         * Comments will not be printed.
         *
         */
        noComments = "NoComments",
        /**
         *
         * Comments will be printed as end notes at the end of the worksheet.
         *
         */
        endSheet = "EndSheet",
        /**
         *
         * Comments will be printed where they were inserted in the worksheet.
         *
         */
        inPlace = "InPlace"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum PrintOrder {
        /**
         *
         * Process down the rows before processing across pages or page fields to the right.
         *
         */
        downThenOver = "DownThenOver",
        /**
         *
         * Process across pages or page fields to the right before moving down the rows.
         *
         */
        overThenDown = "OverThenDown"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum PrintMarginUnit {
        /**
         *
         * Assign the page margins in points. A point is 1/72 of an inch.
         *
         */
        points = "Points",
        /**
         *
         * Assign the page margins in inches.
         *
         */
        inches = "Inches",
        /**
         *
         * Assign the page margins in centimeters.
         *
         */
        centimeters = "Centimeters"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum HeaderFooterState {
        /**
         *
         * Only one general header/footer is used for all pages printed.
         *
         */
        default = "Default",
        /**
         *
         * There is a separate first page header/footer, and a general header/footer used for all other pages.
         *
         */
        firstAndDefault = "FirstAndDefault",
        /**
         *
         * There is a different header/footer for odd and even pages.
         *
         */
        oddAndEven = "OddAndEven",
        /**
         *
         * There is a separate first page header/footer, then there is a separate header/footer for odd and even pages.
         *
         */
        firstOddAndEven = "FirstOddAndEven"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum AutoFillType {
        fillDefault = "FillDefault",
        fillCopy = "FillCopy",
        fillSeries = "FillSeries",
        fillFormats = "FillFormats",
        fillValues = "FillValues",
        fillDays = "FillDays",
        fillWeekdays = "FillWeekdays",
        fillMonths = "FillMonths",
        fillYears = "FillYears",
        linearTrend = "LinearTrend",
        growthTrend = "GrowthTrend",
        flashFill = "FlashFill"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum RangeCopyType {
        all = "All",
        formulas = "Formulas",
        values = "Values",
        formats = "Formats"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum LinkedDataTypeState {
        none = "None",
        validLinkedData = "ValidLinkedData",
        disambiguationNeeded = "DisambiguationNeeded",
        brokenLinkedData = "BrokenLinkedData",
        fetchingData = "FetchingData"
    }
    /**
     *
     * Specifies the shape type for a GeometricShape object.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum GeometricShapeType {
        lineInverse = "LineInverse",
        triangle = "Triangle",
        rightTriangle = "RightTriangle",
        rectangle = "Rectangle",
        diamond = "Diamond",
        parallelogram = "Parallelogram",
        trapezoid = "Trapezoid",
        nonIsoscelesTrapezoid = "NonIsoscelesTrapezoid",
        pentagon = "Pentagon",
        hexagon = "Hexagon",
        heptagon = "Heptagon",
        octagon = "Octagon",
        decagon = "Decagon",
        dodecagon = "Dodecagon",
        star4 = "Star4",
        star5 = "Star5",
        star6 = "Star6",
        star7 = "Star7",
        star8 = "Star8",
        star10 = "Star10",
        star12 = "Star12",
        star16 = "Star16",
        star24 = "Star24",
        star32 = "Star32",
        roundRectangle = "RoundRectangle",
        round1Rectangle = "Round1Rectangle",
        round2SameRectangle = "Round2SameRectangle",
        round2DiagonalRectangle = "Round2DiagonalRectangle",
        snipRoundRectangle = "SnipRoundRectangle",
        snip1Rectangle = "Snip1Rectangle",
        snip2SameRectangle = "Snip2SameRectangle",
        snip2DiagonalRectangle = "Snip2DiagonalRectangle",
        plaque = "Plaque",
        ellipse = "Ellipse",
        teardrop = "Teardrop",
        homePlate = "HomePlate",
        chevron = "Chevron",
        pieWedge = "PieWedge",
        pie = "Pie",
        blockArc = "BlockArc",
        donut = "Donut",
        noSmoking = "NoSmoking",
        rightArrow = "RightArrow",
        leftArrow = "LeftArrow",
        upArrow = "UpArrow",
        downArrow = "DownArrow",
        stripedRightArrow = "StripedRightArrow",
        notchedRightArrow = "NotchedRightArrow",
        bentUpArrow = "BentUpArrow",
        leftRightArrow = "LeftRightArrow",
        upDownArrow = "UpDownArrow",
        leftUpArrow = "LeftUpArrow",
        leftRightUpArrow = "LeftRightUpArrow",
        quadArrow = "QuadArrow",
        leftArrowCallout = "LeftArrowCallout",
        rightArrowCallout = "RightArrowCallout",
        upArrowCallout = "UpArrowCallout",
        downArrowCallout = "DownArrowCallout",
        leftRightArrowCallout = "LeftRightArrowCallout",
        upDownArrowCallout = "UpDownArrowCallout",
        quadArrowCallout = "QuadArrowCallout",
        bentArrow = "BentArrow",
        uturnArrow = "UturnArrow",
        circularArrow = "CircularArrow",
        leftCircularArrow = "LeftCircularArrow",
        leftRightCircularArrow = "LeftRightCircularArrow",
        curvedRightArrow = "CurvedRightArrow",
        curvedLeftArrow = "CurvedLeftArrow",
        curvedUpArrow = "CurvedUpArrow",
        curvedDownArrow = "CurvedDownArrow",
        swooshArrow = "SwooshArrow",
        cube = "Cube",
        can = "Can",
        lightningBolt = "LightningBolt",
        heart = "Heart",
        sun = "Sun",
        moon = "Moon",
        smileyFace = "SmileyFace",
        irregularSeal1 = "IrregularSeal1",
        irregularSeal2 = "IrregularSeal2",
        foldedCorner = "FoldedCorner",
        bevel = "Bevel",
        frame = "Frame",
        halfFrame = "HalfFrame",
        corner = "Corner",
        diagonalStripe = "DiagonalStripe",
        chord = "Chord",
        arc = "Arc",
        leftBracket = "LeftBracket",
        rightBracket = "RightBracket",
        leftBrace = "LeftBrace",
        rightBrace = "RightBrace",
        bracketPair = "BracketPair",
        bracePair = "BracePair",
        callout1 = "Callout1",
        callout2 = "Callout2",
        callout3 = "Callout3",
        accentCallout1 = "AccentCallout1",
        accentCallout2 = "AccentCallout2",
        accentCallout3 = "AccentCallout3",
        borderCallout1 = "BorderCallout1",
        borderCallout2 = "BorderCallout2",
        borderCallout3 = "BorderCallout3",
        accentBorderCallout1 = "AccentBorderCallout1",
        accentBorderCallout2 = "AccentBorderCallout2",
        accentBorderCallout3 = "AccentBorderCallout3",
        wedgeRectCallout = "WedgeRectCallout",
        wedgeRRectCallout = "WedgeRRectCallout",
        wedgeEllipseCallout = "WedgeEllipseCallout",
        cloudCallout = "CloudCallout",
        cloud = "Cloud",
        ribbon = "Ribbon",
        ribbon2 = "Ribbon2",
        ellipseRibbon = "EllipseRibbon",
        ellipseRibbon2 = "EllipseRibbon2",
        leftRightRibbon = "LeftRightRibbon",
        verticalScroll = "VerticalScroll",
        horizontalScroll = "HorizontalScroll",
        wave = "Wave",
        doubleWave = "DoubleWave",
        plus = "Plus",
        flowChartProcess = "FlowChartProcess",
        flowChartDecision = "FlowChartDecision",
        flowChartInputOutput = "FlowChartInputOutput",
        flowChartPredefinedProcess = "FlowChartPredefinedProcess",
        flowChartInternalStorage = "FlowChartInternalStorage",
        flowChartDocument = "FlowChartDocument",
        flowChartMultidocument = "FlowChartMultidocument",
        flowChartTerminator = "FlowChartTerminator",
        flowChartPreparation = "FlowChartPreparation",
        flowChartManualInput = "FlowChartManualInput",
        flowChartManualOperation = "FlowChartManualOperation",
        flowChartConnector = "FlowChartConnector",
        flowChartPunchedCard = "FlowChartPunchedCard",
        flowChartPunchedTape = "FlowChartPunchedTape",
        flowChartSummingJunction = "FlowChartSummingJunction",
        flowChartOr = "FlowChartOr",
        flowChartCollate = "FlowChartCollate",
        flowChartSort = "FlowChartSort",
        flowChartExtract = "FlowChartExtract",
        flowChartMerge = "FlowChartMerge",
        flowChartOfflineStorage = "FlowChartOfflineStorage",
        flowChartOnlineStorage = "FlowChartOnlineStorage",
        flowChartMagneticTape = "FlowChartMagneticTape",
        flowChartMagneticDisk = "FlowChartMagneticDisk",
        flowChartMagneticDrum = "FlowChartMagneticDrum",
        flowChartDisplay = "FlowChartDisplay",
        flowChartDelay = "FlowChartDelay",
        flowChartAlternateProcess = "FlowChartAlternateProcess",
        flowChartOffpageConnector = "FlowChartOffpageConnector",
        actionButtonBlank = "ActionButtonBlank",
        actionButtonHome = "ActionButtonHome",
        actionButtonHelp = "ActionButtonHelp",
        actionButtonInformation = "ActionButtonInformation",
        actionButtonForwardNext = "ActionButtonForwardNext",
        actionButtonBackPrevious = "ActionButtonBackPrevious",
        actionButtonEnd = "ActionButtonEnd",
        actionButtonBeginning = "ActionButtonBeginning",
        actionButtonReturn = "ActionButtonReturn",
        actionButtonDocument = "ActionButtonDocument",
        actionButtonSound = "ActionButtonSound",
        actionButtonMovie = "ActionButtonMovie",
        gear6 = "Gear6",
        gear9 = "Gear9",
        funnel = "Funnel",
        mathPlus = "MathPlus",
        mathMinus = "MathMinus",
        mathMultiply = "MathMultiply",
        mathDivide = "MathDivide",
        mathEqual = "MathEqual",
        mathNotEqual = "MathNotEqual",
        cornerTabs = "CornerTabs",
        squareTabs = "SquareTabs",
        plaqueTabs = "PlaqueTabs",
        chartX = "ChartX",
        chartStar = "ChartStar",
        chartPlus = "ChartPlus"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum ConnectorType {
        straight = "Straight",
        elbow = "Elbow",
        curve = "Curve"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum SpecialCellType {
        /**
         *
         * All cells with conditional formats
         *
         */
        conditionalFormats = "ConditionalFormats",
        /**
         *
         * Cells having validation criteria.
         *
         */
        dataValidations = "DataValidations",
        /**
         *
         * Cells with no content.
         *
         */
        blanks = "Blanks",
        /**
         *
         * Cells containing constants.
         *
         */
        constants = "Constants",
        /**
         *
         * Cells containing formulas.
         *
         */
        formulas = "Formulas",
        /**
         *
         * Cells having the same conditional format as the first cell in the range.
         *
         */
        sameConditionalFormat = "SameConditionalFormat",
        /**
         *
         * Cells having the same data validation criteria as the first cell in the range.
         *
         */
        sameDataValidation = "SameDataValidation",
        /**
         *
         * Cells that are visible.
         *
         */
        visible = "Visible"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum SpecialCellValueType {
        /**
         *
         * Cells that have errors, true/false, numeric, or a string value.
         *
         */
        all = "All",
        /**
         *
         * Cells that have errors.
         *
         */
        errors = "Errors",
        /**
         *
         * Cells that have errors, or a true/false value.
         *
         */
        errorsLogical = "ErrorsLogical",
        /**
         *
         * Cells that have errors, or a numeric value.
         *
         */
        errorsNumbers = "ErrorsNumbers",
        /**
         *
         * Cells that have errors, or a string value.
         *
         */
        errorsText = "ErrorsText",
        /**
         *
         * Cells that have errors, true/false, or a numeric value.
         *
         */
        errorsLogicalNumber = "ErrorsLogicalNumber",
        /**
         *
         * Cells that have errors, true/false, or a string value.
         *
         */
        errorsLogicalText = "ErrorsLogicalText",
        /**
         *
         * Cells that have errors, numeric, or a string value.
         *
         */
        errorsNumberText = "ErrorsNumberText",
        /**
         *
         * Cells that have a true/false value.
         *
         */
        logical = "Logical",
        /**
         *
         * Cells that have a true/false, or a numeric value.
         *
         */
        logicalNumbers = "LogicalNumbers",
        /**
         *
         * Cells that have a true/false, or a string value.
         *
         */
        logicalText = "LogicalText",
        /**
         *
         * Cells that have a true/false, numeric, or a string value.
         *
         */
        logicalNumbersText = "LogicalNumbersText",
        /**
         *
         * Cells that have a numeric value.
         *
         */
        numbers = "Numbers",
        /**
         *
         * Cells that have a numeric, or a string value.
         *
         */
        numbersText = "NumbersText",
        /**
         *
         * Cells that have a string value.
         *
         */
        text = "Text"
    }
    /**
     *
     * Specifies the way that an object is attached to its underlying cells.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum Placement {
        /**
         *
         * The object is moved with the cells.
         *
         */
        twoCell = "TwoCell",
        /**
         *
         * The object is moved and sized with the cells.
         *
         */
        oneCell = "OneCell",
        /**
         *
         * The object is free floating.
         *
         */
        absolute = "Absolute"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum FillPattern {
        none = "None",
        solid = "Solid",
        gray50 = "Gray50",
        gray75 = "Gray75",
        gray25 = "Gray25",
        horizontal = "Horizontal",
        vertical = "Vertical",
        down = "Down",
        up = "Up",
        checker = "Checker",
        semiGray75 = "SemiGray75",
        lightHorizontal = "LightHorizontal",
        lightVertical = "LightVertical",
        lightDown = "LightDown",
        lightUp = "LightUp",
        grid = "Grid",
        crissCross = "CrissCross",
        gray16 = "Gray16",
        gray8 = "Gray8",
        linearGradient = "LinearGradient",
        rectangularGradient = "RectangularGradient"
    }
    /**
     *
     * Specifies the horizontal alignment for the text frame in a shape.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ShapeTextHorizontalAlignment {
        left = "Left",
        center = "Center",
        right = "Right",
        justify = "Justify",
        justifyLow = "JustifyLow",
        distributed = "Distributed",
        thaiDistributed = "ThaiDistributed"
    }
    /**
     *
     * Specifies the vertical alignment for the text frame in a shape.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ShapeTextVerticalAlignment {
        top = "Top",
        middle = "Middle",
        bottom = "Bottom",
        justified = "Justified",
        distributed = "Distributed"
    }
    /**
     *
     * Specifies the vertical overflow for the text frame in a shape.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ShapeTextVerticalOverflow {
        /**
         *
         * Allow text to overflow the text frame vertically (can be from the top, bottom, or both depending on the text alignment).
         *
         */
        overflow = "Overflow",
        /**
         *
         * Hide text that does not fit vertically within the text frame, and add an ellipsis (...) at the end of the visible text.
         *
         */
        ellipsis = "Ellipsis",
        /**
         *
         * Hide text that does not fit vertically within the text frame.
         *
         */
        clip = "Clip"
    }
    /**
     *
     * Specifies the horizontal overflow for the text frame in a shape.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ShapeTextHorizontalOverflow {
        overflow = "Overflow",
        clip = "Clip"
    }
    /**
     *
     * Specifies the reading order for the text frame in a shape.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ShapeTextReadingOrder {
        leftToRight = "LeftToRight",
        rightToLeft = "RightToLeft"
    }
    /**
     *
     * Specifies the orientation for the text frame in a shape.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ShapeTextOrientation {
        horizontal = "Horizontal",
        vertical = "Vertical",
        vertical270 = "Vertical270",
        wordArtVertical = "WordArtVertical",
        eastAsianVertical = "EastAsianVertical",
        mongolianVertical = "MongolianVertical",
        wordArtVerticalRTL = "WordArtVerticalRTL"
    }
    /**
     *
     * Determines the type of automatic sizing allowed.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum ShapeAutoSize {
        /**
         *
         * No autosizing.
         *
         */
        autoSizeNone = "AutoSizeNone",
        /**
         *
         * The text is adjusted to fit the shape.
         *
         */
        autoSizeTextToFitShape = "AutoSizeTextToFitShape",
        /**
         *
         * The shape is adjusted to fit the text.
         *
         */
        autoSizeShapeToFitText = "AutoSizeShapeToFitText",
        /**
         *
         * A combination of automatic sizing schemes are used.
         *
         */
        autoSizeMixed = "AutoSizeMixed"
    }
    /**
     *
     * Specifies the close behavior for Workbook.close API.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum CloseBehavior {
        /**
         *
         * Save the possible changes before closing the workbook.
         *
         */
        save = "Save",
        /**
         *
         * Discard the possible changes when closing the workbook.
         *
         */
        skipSave = "SkipSave"
    }
    /**
     *
     * Specifies the save behavior for Workbook.save API.
     *
     * [Api set: ExcelApi 1.9]
     */
    enum SaveBehavior {
        /**
         *
         * Save the workbook without prompting the user. If file is not saved,
            it will save it with default name under default location.
         *
         */
        save = "Save",
        /**
         *
         * Prompts the saveas experience to the user if the workbook has not been saved,
            Ignored if the workbook was previously saved.
         *
         */
        prompt = "Prompt"
    }
    /**
     * [Api set: ExcelApi 1.9]
     */
    enum RibbonTab {
        others = "Others",
        home = "Home",
        insert = "Insert",
        draw = "Draw",
        pageLayout = "PageLayout",
        formulas = "Formulas",
        data = "Data",
        review = "Review",
        view = "View",
        developer = "Developer",
        addIns = "AddIns",
        help = "Help"
    }
    /**
     *
     * An object containing the result of a function-evaluation operation
     *
     * [Api set: ExcelApi 1.2]
     */
    class FunctionResult<T> extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Error value (such as "#DIV/0") representing the error. If the error string is not set, then the function succeeded, and its result is written to the Value field. The error is always in the English locale.
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly error: string;
        /**
         *
         * The value of function evaluation. The value field will be populated only if no error has occurred (i.e., the Error property is not set).
         *
         * [Api set: ExcelApi 1.2]
         */
        readonly value: T;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): FunctionResult<T>` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): FunctionResult<T>` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): FunctionResult<T>` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Excel.Interfaces.FunctionResultLoadOptions): FunctionResult<T>;
        load(option?: string | string[]): FunctionResult<T>;
        load(option?: {
            select?: string;
            expand?: string;
        }): FunctionResult<T>;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original FunctionResult<T> object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Interfaces.FunctionResultData<T>`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Interfaces.FunctionResultData<T>;
    }
    /**
     *
     * An object for evaluating Excel functions.
     *
     * [Api set: ExcelApi 1.2]
     */
    class Functions extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns the absolute value of a number, a number without its sign.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the real number for which you want the absolute value.
         */
        abs(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the accrued interest for a security that pays periodic interest.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param issue Is the security's issue date, expressed as a serial date number.
         * @param firstInterest Is the security's first interest date, expressed as a serial date number.
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param rate Is the security's annual coupon rate.
         * @param par Is the security's par value.
         * @param frequency Is the number of coupon payments per year.
         * @param basis Is the type of day count basis to use.
         * @param calcMethod Is a logical value: to accrued interest from issue date = TRUE or omitted; to calculate from last coupon payment date = FALSE.
         */
        accrInt(issue: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, firstInterest: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, rate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, par: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, frequency: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, calcMethod?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the accrued interest for a security that pays interest at maturity.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param issue Is the security's issue date, expressed as a serial date number.
         * @param settlement Is the security's maturity date, expressed as a serial date number.
         * @param rate Is the security's annual coupon rate.
         * @param par Is the security's par value.
         * @param basis Is the type of day count basis to use.
         */
        accrIntM(issue: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, rate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, par: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the arccosine of a number, in radians in the range 0 to Pi. The arccosine is the angle whose cosine is Number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the cosine of the angle you want and must be from -1 to 1.
         */
        acos(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the inverse hyperbolic cosine of a number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is any real number equal to or greater than 1.
         */
        acosh(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the arccotangent of a number, in radians in the range 0 to Pi.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the cotangent of the angle you want.
         */
        acot(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the inverse hyperbolic cotangent of a number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the hyperbolic cotangent of the angle that you want.
         */
        acoth(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the prorated linear depreciation of an asset for each accounting period.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param cost Is the cost of the asset.
         * @param datePurchased Is the date the asset is purchased.
         * @param firstPeriod Is the date of the end of the first period.
         * @param salvage Is the salvage value at the end of life of the asset.
         * @param period Is the period.
         * @param rate Is the rate of depreciation.
         * @param basis Year_basis : 0 for year of 360 days, 1 for actual, 3 for year of 365 days.
         */
        amorDegrc(cost: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, datePurchased: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, firstPeriod: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, salvage: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, period: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, rate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the prorated linear depreciation of an asset for each accounting period.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param cost Is the cost of the asset.
         * @param datePurchased Is the date the asset is purchased.
         * @param firstPeriod Is the date of the end of the first period.
         * @param salvage Is the salvage value at the end of life of the asset.
         * @param period Is the period.
         * @param rate Is the rate of depreciation.
         * @param basis Year_basis : 0 for year of 360 days, 1 for actual, 3 for year of 365 days.
         */
        amorLinc(cost: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, datePurchased: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, firstPeriod: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, salvage: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, period: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, rate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Checks whether all arguments are TRUE, and returns TRUE if all arguments are TRUE.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 conditions you want to test that can be either TRUE or FALSE and can be logical values, arrays, or references.
         */
        and(...values: Array<boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<boolean>;
        /**
         *
         * Converts a Roman numeral to Arabic.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the Roman numeral you want to convert.
         */
        arabic(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of areas in a reference. An area is a range of contiguous cells or a single cell.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param reference Is a reference to a cell or range of cells and can refer to multiple areas.
         */
        areas(reference: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Changes full-width (double-byte) characters to half-width (single-byte) characters. Use with double-byte character sets (DBCS).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is a text, or a reference to a cell containing a text.
         */
        asc(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns the arcsine of a number in radians, in the range -Pi/2 to Pi/2.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the sine of the angle you want and must be from -1 to 1.
         */
        asin(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the inverse hyperbolic sine of a number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is any real number equal to or greater than 1.
         */
        asinh(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the arctangent of a number in radians, in the range -Pi/2 to Pi/2.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the tangent of the angle you want.
         */
        atan(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the arctangent of the specified x- and y- coordinates, in radians between -Pi and Pi, excluding -Pi.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param xNum Is the x-coordinate of the point.
         * @param yNum Is the y-coordinate of the point.
         */
        atan2(xNum: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, yNum: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the inverse hyperbolic tangent of a number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is any real number between -1 and 1 excluding -1 and 1.
         */
        atanh(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the average of the absolute deviations of data points from their mean. Arguments can be numbers or names, arrays, or references that contain numbers.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 arguments for which you want the average of the absolute deviations.
         */
        aveDev(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the average (arithmetic mean) of its arguments, which can be numbers or names, arrays, or references that contain numbers.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numeric arguments for which you want the average.
         */
        average(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the average (arithmetic mean) of its arguments, evaluating text and FALSE in arguments as 0; TRUE evaluates as 1. Arguments can be numbers, names, arrays, or references.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 arguments for which you want the average.
         */
        averageA(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Finds average(arithmetic mean) for the cells specified by a given condition or criteria.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param range Is the range of cells you want evaluated.
         * @param criteria Is the condition or criteria in the form of a number, expression, or text that defines which cells will be used to find the average.
         * @param averageRange Are the actual cells to be used to find the average. If omitted, the cells in range are used.
         */
        averageIf(range: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, criteria: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, averageRange?: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Finds average(arithmetic mean) for the cells specified by a given set of conditions or criteria.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param averageRange Are the actual cells to be used to find the average.
         * @param values List of parameters, where the first element of each pair is the Is the range of cells you want evaluated for the particular condition , and the second element is is the condition or criteria in the form of a number, expression, or text that defines which cells will be used to find the average.
         */
        averageIfs(averageRange: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, ...values: Array<Excel.Range | Excel.RangeReference | Excel.FunctionResult<any> | number | string | boolean>): FunctionResult<number>;
        /**
         *
         * Converts a number to text (baht).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is a number that you want to convert.
         */
        bahtText(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Converts a number into a text representation with the given radix (base).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the number that you want to convert.
         * @param radix Is the base Radix that you want to convert the number into.
         * @param minLength Is the minimum length of the returned string.  If omitted leading zeros are not added.
         */
        base(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, radix: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, minLength?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns the modified Bessel function In(x).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value at which to evaluate the function.
         * @param n Is the order of the Bessel function.
         */
        besselI(x: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, n: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the Bessel function Jn(x).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value at which to evaluate the function.
         * @param n Is the order of the Bessel function.
         */
        besselJ(x: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, n: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the modified Bessel function Kn(x).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value at which to evaluate the function.
         * @param n Is the order of the function.
         */
        besselK(x: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, n: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the Bessel function Yn(x).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value at which to evaluate the function.
         * @param n Is the order of the function.
         */
        besselY(x: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, n: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the beta probability distribution function.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value between A and B at which to evaluate the function.
         * @param alpha Is a parameter to the distribution and must be greater than 0.
         * @param beta Is a parameter to the distribution and must be greater than 0.
         * @param cumulative Is a logical value: for the cumulative distribution function, use TRUE; for the probability density function, use FALSE.
         * @param A Is an optional lower bound to the interval of x. If omitted, A = 0.
         * @param B Is an optional upper bound to the interval of x. If omitted, B = 1.
         */
        beta_Dist(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, alpha: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, beta: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, cumulative: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, A?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, B?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the inverse of the cumulative beta probability density function (BETA.DIST).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param probability Is a probability associated with the beta distribution.
         * @param alpha Is a parameter to the distribution and must be greater than 0.
         * @param beta Is a parameter to the distribution and must be greater than 0.
         * @param A Is an optional lower bound to the interval of x. If omitted, A = 0.
         * @param B Is an optional upper bound to the interval of x. If omitted, B = 1.
         */
        beta_Inv(probability: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, alpha: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, beta: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, A?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, B?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a binary number to decimal.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the binary number you want to convert.
         */
        bin2Dec(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a binary number to hexadecimal.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the binary number you want to convert.
         * @param places Is the number of characters to use.
         */
        bin2Hex(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, places?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a binary number to octal.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the binary number you want to convert.
         * @param places Is the number of characters to use.
         */
        bin2Oct(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, places?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the individual term binomial distribution probability.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param numberS Is the number of successes in trials.
         * @param trials Is the number of independent trials.
         * @param probabilityS Is the probability of success on each trial.
         * @param cumulative Is a logical value: for the cumulative distribution function, use TRUE; for the probability mass function, use FALSE.
         */
        binom_Dist(numberS: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, trials: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, probabilityS: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, cumulative: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the probability of a trial result using a binomial distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param trials Is the number of independent trials.
         * @param probabilityS Is the probability of success on each trial.
         * @param numberS Is the number of successes in trials.
         * @param numberS2 If provided this function returns the probability that the number of successful trials shall lie between numberS and numberS2.
         */
        binom_Dist_Range(trials: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, probabilityS: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numberS: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numberS2?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the smallest value for which the cumulative binomial distribution is greater than or equal to a criterion value.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param trials Is the number of Bernoulli trials.
         * @param probabilityS Is the probability of success on each trial, a number between 0 and 1 inclusive.
         * @param alpha Is the criterion value, a number between 0 and 1 inclusive.
         */
        binom_Inv(trials: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, probabilityS: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, alpha: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns a bitwise 'And' of two numbers.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number1 Is the decimal representation of the binary number you want to evaluate.
         * @param number2 Is the decimal representation of the binary number you want to evaluate.
         */
        bitand(number1: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, number2: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns a number shifted left by shift_amount bits.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the decimal representation of the binary number you want to evaluate.
         * @param shiftAmount Is the number of bits that you want to shift Number left by.
         */
        bitlshift(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, shiftAmount: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns a bitwise 'Or' of two numbers.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number1 Is the decimal representation of the binary number you want to evaluate.
         * @param number2 Is the decimal representation of the binary number you want to evaluate.
         */
        bitor(number1: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, number2: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns a number shifted right by shift_amount bits.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the decimal representation of the binary number you want to evaluate.
         * @param shiftAmount Is the number of bits that you want to shift Number right by.
         */
        bitrshift(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, shiftAmount: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns a bitwise 'Exclusive Or' of two numbers.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number1 Is the decimal representation of the binary number you want to evaluate.
         * @param number2 Is the decimal representation of the binary number you want to evaluate.
         */
        bitxor(number1: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, number2: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Rounds a number up, to the nearest integer or to the nearest multiple of significance.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the value you want to round.
         * @param significance Is the multiple to which you want to round.
         * @param mode When given and nonzero this function will round away from zero.
         */
        ceiling_Math(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, significance?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, mode?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Rounds a number up, to the nearest integer or to the nearest multiple of significance.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the value you want to round.
         * @param significance Is the multiple to which you want to round.
         */
        ceiling_Precise(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, significance?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the character specified by the code number from the character set for your computer.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is a number between 1 and 255 specifying which character you want.
         */
        char(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns the left-tailed probability of the chi-squared distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value at which you want to evaluate the distribution, a nonnegative number.
         * @param degFreedom Is the number of degrees of freedom, a number between 1 and 10^10, excluding 10^10.
         * @param cumulative Is a logical value for the function to return: the cumulative distribution function = TRUE; the probability density function = FALSE.
         */
        chiSq_Dist(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, cumulative: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the right-tailed probability of the chi-squared distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value at which you want to evaluate the distribution, a nonnegative number.
         * @param degFreedom Is the number of degrees of freedom, a number between 1 and 10^10, excluding 10^10.
         */
        chiSq_Dist_RT(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the inverse of the left-tailed probability of the chi-squared distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param probability Is a probability associated with the chi-squared distribution, a value between 0 and 1 inclusive.
         * @param degFreedom Is the number of degrees of freedom, a number between 1 and 10^10, excluding 10^10.
         */
        chiSq_Inv(probability: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the inverse of the right-tailed probability of the chi-squared distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param probability Is a probability associated with the chi-squared distribution, a value between 0 and 1 inclusive.
         * @param degFreedom Is the number of degrees of freedom, a number between 1 and 10^10, excluding 10^10.
         */
        chiSq_Inv_RT(probability: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Chooses a value or action to perform from a list of values, based on an index number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param indexNum Specifies which value argument is selected. indexNum must be between 1 and 254, or a formula or a reference to a number between 1 and 254.
         * @param values List of parameters, whose elements are 1 to 254 numbers, cell references, defined names, formulas, functions, or text arguments from which CHOOSE selects.
         */
        choose(indexNum: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, ...values: Array<Excel.Range | number | string | boolean | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number | string | boolean>;
        /**
         *
         * Removes all nonprintable characters from text.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is any worksheet information from which you want to remove nonprintable characters.
         */
        clean(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns a numeric code for the first character in a text string, in the character set used by your computer.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the text for which you want the code of the first character.
         */
        code(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of columns in an array or reference.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param array Is an array or array formula, or a reference to a range of cells for which you want the number of columns.
         */
        columns(array: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of combinations for a given number of items.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the total number of items.
         * @param numberChosen Is the number of items in each combination.
         */
        combin(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numberChosen: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of combinations with repetitions for a given number of items.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the total number of items.
         * @param numberChosen Is the number of items in each combination.
         */
        combina(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numberChosen: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts real and imaginary coefficients into a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param realNum Is the real coefficient of the complex number.
         * @param iNum Is the imaginary coefficient of the complex number.
         * @param suffix Is the suffix for the imaginary component of the complex number.
         */
        complex(realNum: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, iNum: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, suffix?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Joins several text strings into one text string.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 text strings to be joined into a single text string and can be text strings, numbers, or single-cell references.
         */
        concatenate(...values: Array<string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<string>;
        /**
         *
         * Returns the confidence interval for a population mean, using a normal distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param alpha Is the significance level used to compute the confidence level, a number greater than 0 and less than 1.
         * @param standardDev Is the population standard deviation for the data range and is assumed to be known. standardDev must be greater than 0.
         * @param size Is the sample size.
         */
        confidence_Norm(alpha: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, standardDev: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, size: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the confidence interval for a population mean, using a Student's T distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param alpha Is the significance level used to compute the confidence level, a number greater than 0 and less than 1.
         * @param standardDev Is the population standard deviation for the data range and is assumed to be known. standardDev must be greater than 0.
         * @param size Is the sample size.
         */
        confidence_T(alpha: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, standardDev: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, size: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a number from one measurement system to another.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the value in from_units to convert.
         * @param fromUnit Is the units for number.
         * @param toUnit Is the units for the result.
         */
        convert(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, fromUnit: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, toUnit: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the cosine of an angle.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the angle in radians for which you want the cosine.
         */
        cos(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the hyperbolic cosine of a number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is any real number.
         */
        cosh(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the cotangent of an angle.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the angle in radians for which you want the cotangent.
         */
        cot(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the hyperbolic cotangent of a number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the angle in radians for which you want the hyperbolic cotangent.
         */
        coth(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Counts the number of cells in a range that contain numbers.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 arguments that can contain or refer to a variety of different types of data, but only numbers are counted.
         */
        count(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Counts the number of cells in a range that are not empty.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 arguments representing the values and cells you want to count. Values can be any type of information.
         */
        countA(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Counts the number of empty cells in a specified range of cells.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param range Is the range from which you want to count the empty cells.
         */
        countBlank(range: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Counts the number of cells within a range that meet the given condition.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param range Is the range of cells from which you want to count nonblank cells.
         * @param criteria Is the condition in the form of a number, expression, or text that defines which cells will be counted.
         */
        countIf(range: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, criteria: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Counts the number of cells specified by a given set of conditions or criteria.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, where the first element of each pair is the Is the range of cells you want evaluated for the particular condition , and the second element is is the condition in the form of a number, expression, or text that defines which cells will be counted.
         */
        countIfs(...values: Array<Excel.Range | Excel.RangeReference | Excel.FunctionResult<any> | number | string | boolean>): FunctionResult<number>;
        /**
         *
         * Returns the number of days from the beginning of the coupon period to the settlement date.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param frequency Is the number of coupon payments per year.
         * @param basis Is the type of day count basis to use.
         */
        coupDayBs(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, frequency: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of days in the coupon period that contains the settlement date.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param frequency Is the number of coupon payments per year.
         * @param basis Is the type of day count basis to use.
         */
        coupDays(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, frequency: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of days from the settlement date to the next coupon date.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param frequency Is the number of coupon payments per year.
         * @param basis Is the type of day count basis to use.
         */
        coupDaysNc(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, frequency: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the next coupon date after the settlement date.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param frequency Is the number of coupon payments per year.
         * @param basis Is the type of day count basis to use.
         */
        coupNcd(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, frequency: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of coupons payable between the settlement date and maturity date.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param frequency Is the number of coupon payments per year.
         * @param basis Is the type of day count basis to use.
         */
        coupNum(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, frequency: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the previous coupon date before the settlement date.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param frequency Is the number of coupon payments per year.
         * @param basis Is the type of day count basis to use.
         */
        coupPcd(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, frequency: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the cosecant of an angle.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the angle in radians for which you want the cosecant.
         */
        csc(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the hyperbolic cosecant of an angle.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the angle in radians for which you want the hyperbolic cosecant.
         */
        csch(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the cumulative interest paid between two periods.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param rate Is the interest rate.
         * @param nper Is the total number of payment periods.
         * @param pv Is the present value.
         * @param startPeriod Is the first period in the calculation.
         * @param endPeriod Is the last period in the calculation.
         * @param type Is the timing of the payment.
         */
        cumIPmt(rate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, nper: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pv: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, startPeriod: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, endPeriod: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, type: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the cumulative principal paid on a loan between two periods.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param rate Is the interest rate.
         * @param nper Is the total number of payment periods.
         * @param pv Is the present value.
         * @param startPeriod Is the first period in the calculation.
         * @param endPeriod Is the last period in the calculation.
         * @param type Is the timing of the payment.
         */
        cumPrinc(rate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, nper: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pv: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, startPeriod: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, endPeriod: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, type: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Averages the values in a column in a list or database that match conditions you specify.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param database Is the range of cells that makes up the list or database. A database is a list of related data.
         * @param field Is either the label of the column in double quotation marks or a number that represents the column's position in the list.
         * @param criteria Is the range of cells that contains the conditions you specify. The range includes a column label and one cell below the label for a condition.
         */
        daverage(database: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, field: number | string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, criteria: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Counts the cells containing numbers in the field (column) of records in the database that match the conditions you specify.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param database Is the range of cells that makes up the list or database. A database is a list of related data.
         * @param field Is either the label of the column in double quotation marks or a number that represents the column's position in the list.
         * @param criteria Is the range of cells that contains the conditions you specify. The range includes a column label and one cell below the label for a condition.
         */
        dcount(database: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, field: number | string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, criteria: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Counts nonblank cells in the field (column) of records in the database that match the conditions you specify.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param database Is the range of cells that makes up the list or database. A database is a list of related data.
         * @param field Is either the label of the column in double quotation marks or a number that represents the column's position in the list.
         * @param criteria Is the range of cells that contains the conditions you specify. The range includes a column label and one cell below the label for a condition.
         */
        dcountA(database: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, field: number | string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, criteria: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Extracts from a database a single record that matches the conditions you specify.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param database Is the range of cells that makes up the list or database. A database is a list of related data.
         * @param field Is either the label of the column in double quotation marks or a number that represents the column's position in the list.
         * @param criteria Is the range of cells that contains the conditions you specify. The range includes a column label and one cell below the label for a condition.
         */
        dget(database: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, field: number | string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, criteria: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number | boolean | string>;
        /**
         *
         * Returns the largest number in the field (column) of records in the database that match the conditions you specify.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param database Is the range of cells that makes up the list or database. A database is a list of related data.
         * @param field Is either the label of the column in double quotation marks or a number that represents the column's position in the list.
         * @param criteria Is the range of cells that contains the conditions you specify. The range includes a column label and one cell below the label for a condition.
         */
        dmax(database: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, field: number | string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, criteria: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the smallest number in the field (column) of records in the database that match the conditions you specify.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param database Is the range of cells that makes up the list or database. A database is a list of related data.
         * @param field Is either the label of the column in double quotation marks or a number that represents the column's position in the list.
         * @param criteria Is the range of cells that contains the conditions you specify. The range includes a column label and one cell below the label for a condition.
         */
        dmin(database: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, field: number | string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, criteria: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Multiplies the values in the field (column) of records in the database that match the conditions you specify.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param database Is the range of cells that makes up the list or database. A database is a list of related data.
         * @param field Is either the label of the column in double quotation marks or a number that represents the column's position in the list.
         * @param criteria Is the range of cells that contains the conditions you specify. The range includes a column label and one cell below the label for a condition.
         */
        dproduct(database: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, field: number | string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, criteria: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Estimates the standard deviation based on a sample from selected database entries.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param database Is the range of cells that makes up the list or database. A database is a list of related data.
         * @param field Is either the label of the column in double quotation marks or a number that represents the column's position in the list.
         * @param criteria Is the range of cells that contains the conditions you specify. The range includes a column label and one cell below the label for a condition.
         */
        dstDev(database: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, field: number | string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, criteria: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Calculates the standard deviation based on the entire population of selected database entries.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param database Is the range of cells that makes up the list or database. A database is a list of related data.
         * @param field Is either the label of the column in double quotation marks or a number that represents the column's position in the list.
         * @param criteria Is the range of cells that contains the conditions you specify. The range includes a column label and one cell below the label for a condition.
         */
        dstDevP(database: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, field: number | string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, criteria: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Adds the numbers in the field (column) of records in the database that match the conditions you specify.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param database Is the range of cells that makes up the list or database. A database is a list of related data.
         * @param field Is either the label of the column in double quotation marks or a number that represents the column's position in the list.
         * @param criteria Is the range of cells that contains the conditions you specify. The range includes a column label and one cell below the label for a condition.
         */
        dsum(database: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, field: number | string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, criteria: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Estimates variance based on a sample from selected database entries.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param database Is the range of cells that makes up the list or database. A database is a list of related data.
         * @param field Is either the label of the column in double quotation marks or a number that represents the column's position in the list.
         * @param criteria Is the range of cells that contains the conditions you specify. The range includes a column label and one cell below the label for a condition.
         */
        dvar(database: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, field: number | string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, criteria: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Calculates variance based on the entire population of selected database entries.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param database Is the range of cells that makes up the list or database. A database is a list of related data.
         * @param field Is either the label of the column in double quotation marks or a number that represents the column's position in the list.
         * @param criteria Is the range of cells that contains the conditions you specify. The range includes a column label and one cell below the label for a condition.
         */
        dvarP(database: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, field: number | string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, criteria: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number that represents the date in Microsoft Excel date-time code.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param year Is a number from 1900 or 1904 (depending on the workbook's date system) to 9999.
         * @param month Is a number from 1 to 12 representing the month of the year.
         * @param day Is a number from 1 to 31 representing the day of the month.
         */
        date(year: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, month: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, day: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a date in the form of text to a number that represents the date in Microsoft Excel date-time code.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param dateText Is text that represents a date in a Microsoft Excel date format, between 1/1/1900 or 1/1/1904 (depending on the workbook's date system) and 12/31/9999.
         */
        datevalue(dateText: string | number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the day of the month, a number from 1 to 31.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param serialNumber Is a number in the date-time code used by Microsoft Excel.
         */
        day(serialNumber: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of days between the two dates.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param endDate startDate and endDate are the two dates between which you want to know the number of days.
         * @param startDate startDate and endDate are the two dates between which you want to know the number of days.
         */
        days(endDate: string | number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, startDate: string | number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of days between two dates based on a 360-day year (twelve 30-day months).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param startDate startDate and endDate are the two dates between which you want to know the number of days.
         * @param endDate startDate and endDate are the two dates between which you want to know the number of days.
         * @param method Is a logical value specifying the calculation method: U.S. (NASD) = FALSE or omitted; European = TRUE.
         */
        days360(startDate: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, endDate: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, method?: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the depreciation of an asset for a specified period using the fixed-declining balance method.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param cost Is the initial cost of the asset.
         * @param salvage Is the salvage value at the end of the life of the asset.
         * @param life Is the number of periods over which the asset is being depreciated (sometimes called the useful life of the asset).
         * @param period Is the period for which you want to calculate the depreciation. Period must use the same units as Life.
         * @param month Is the number of months in the first year. If month is omitted, it is assumed to be 12.
         */
        db(cost: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, salvage: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, life: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, period: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, month?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Changes half-width (single-byte) characters within a character string to full-width (double-byte) characters. Use with double-byte character sets (DBCS).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is a text, or a reference to a cell containing a text.
         */
        dbcs(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns the depreciation of an asset for a specified period using the double-declining balance method or some other method you specify.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param cost Is the initial cost of the asset.
         * @param salvage Is the salvage value at the end of the life of the asset.
         * @param life Is the number of periods over which the asset is being depreciated (sometimes called the useful life of the asset).
         * @param period Is the period for which you want to calculate the depreciation. Period must use the same units as Life.
         * @param factor Is the rate at which the balance declines. If Factor is omitted, it is assumed to be 2 (the double-declining balance method).
         */
        ddb(cost: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, salvage: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, life: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, period: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, factor?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a decimal number to binary.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the decimal integer you want to convert.
         * @param places Is the number of characters to use.
         */
        dec2Bin(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, places?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a decimal number to hexadecimal.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the decimal integer you want to convert.
         * @param places Is the number of characters to use.
         */
        dec2Hex(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, places?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a decimal number to octal.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the decimal integer you want to convert.
         * @param places Is the number of characters to use.
         */
        dec2Oct(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, places?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a text representation of a number in a given base into a decimal number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the number that you want to convert.
         * @param radix Is the base Radix of the number you are converting.
         */
        decimal(number: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, radix: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts radians to degrees.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param angle Is the angle in radians that you want to convert.
         */
        degrees(angle: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Tests whether two numbers are equal.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number1 Is the first number.
         * @param number2 Is the second number.
         */
        delta(number1: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, number2?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the sum of squares of deviations of data points from their sample mean.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 arguments, or an array or array reference, on which you want DEVSQ to calculate.
         */
        devSq(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the discount rate for a security.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param pr Is the security's price per $100 face value.
         * @param redemption Is the security's redemption value per $100 face value.
         * @param basis Is the type of day count basis to use.
         */
        disc(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pr: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, redemption: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a number to text, using currency format.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is a number, a reference to a cell containing a number, or a formula that evaluates to a number.
         * @param decimals Is the number of digits to the right of the decimal point. The number is rounded as necessary; if omitted, Decimals = 2.
         */
        dollar(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, decimals?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Converts a dollar price, expressed as a fraction, into a dollar price, expressed as a decimal number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param fractionalDollar Is a number expressed as a fraction.
         * @param fraction Is the integer to use in the denominator of the fraction.
         */
        dollarDe(fractionalDollar: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, fraction: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a dollar price, expressed as a decimal number, into a dollar price, expressed as a fraction.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param decimalDollar Is a decimal number.
         * @param fraction Is the integer to use in the denominator of a fraction.
         */
        dollarFr(decimalDollar: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, fraction: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the annual duration of a security with periodic interest payments.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param coupon Is the security's annual coupon rate.
         * @param yld Is the security's annual yield.
         * @param frequency Is the number of coupon payments per year.
         * @param basis Is the type of day count basis to use.
         */
        duration(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, coupon: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, yld: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, frequency: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Rounds a number up, to the nearest integer or to the nearest multiple of significance.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the value you want to round.
         * @param significance Is the multiple to which you want to round.
         */
        ecma_Ceiling(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, significance: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the serial number of the date that is the indicated number of months before or after the start date.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param startDate Is a serial date number that represents the start date.
         * @param months Is the number of months before or after startDate.
         */
        edate(startDate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, months: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the effective annual interest rate.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param nominalRate Is the nominal interest rate.
         * @param npery Is the number of compounding periods per year.
         */
        effect(nominalRate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, npery: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the serial number of the last day of the month before or after a specified number of months.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param startDate Is a serial date number that represents the start date.
         * @param months Is the number of months before or after the startDate.
         */
        eoMonth(startDate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, months: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the error function.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param lowerLimit Is the lower bound for integrating ERF.
         * @param upperLimit Is the upper bound for integrating ERF.
         */
        erf(lowerLimit: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, upperLimit?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the complementary error function.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the lower bound for integrating ERF.
         */
        erfC(x: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the complementary error function.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param X Is the lower bound for integrating ERFC.PRECISE.
         */
        erfC_Precise(X: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the error function.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param X Is the lower bound for integrating ERF.PRECISE.
         */
        erf_Precise(X: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns a number matching an error value.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param errorVal Is the error value for which you want the identifying number, and can be an actual error value or a reference to a cell containing an error value.
         */
        error_Type(errorVal: string | number | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Rounds a positive number up and negative number down to the nearest even integer.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the value to round.
         */
        even(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Checks whether two text strings are exactly the same, and returns TRUE or FALSE. EXACT is case-sensitive.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text1 Is the first text string.
         * @param text2 Is the second text string.
         */
        exact(text1: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, text2: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<boolean>;
        /**
         *
         * Returns e raised to the power of a given number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the exponent applied to the base e. The constant e equals 2.71828182845904, the base of the natural logarithm.
         */
        exp(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the exponential distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value of the function, a nonnegative number.
         * @param lambda Is the parameter value, a positive number.
         * @param cumulative Is a logical value for the function to return: the cumulative distribution function = TRUE; the probability density function = FALSE.
         */
        expon_Dist(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, lambda: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, cumulative: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the future value of an initial principal after applying a series of compound interest rates.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param principal Is the present value.
         * @param schedule Is an array of interest rates to apply.
         */
        fvschedule(principal: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, schedule: number | string | Excel.Range | boolean | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the (left-tailed) F probability distribution (degree of diversity) for two data sets.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value at which to evaluate the function, a nonnegative number.
         * @param degFreedom1 Is the numerator degrees of freedom, a number between 1 and 10^10, excluding 10^10.
         * @param degFreedom2 Is the denominator degrees of freedom, a number between 1 and 10^10, excluding 10^10.
         * @param cumulative Is a logical value for the function to return: the cumulative distribution function = TRUE; the probability density function = FALSE.
         */
        f_Dist(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom1: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom2: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, cumulative: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the (right-tailed) F probability distribution (degree of diversity) for two data sets.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value at which to evaluate the function, a nonnegative number.
         * @param degFreedom1 Is the numerator degrees of freedom, a number between 1 and 10^10, excluding 10^10.
         * @param degFreedom2 Is the denominator degrees of freedom, a number between 1 and 10^10, excluding 10^10.
         */
        f_Dist_RT(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom1: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom2: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the inverse of the (left-tailed) F probability distribution: if p = F.DIST(x,...), then F.INV(p,...) = x.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param probability Is a probability associated with the F cumulative distribution, a number between 0 and 1 inclusive.
         * @param degFreedom1 Is the numerator degrees of freedom, a number between 1 and 10^10, excluding 10^10.
         * @param degFreedom2 Is the denominator degrees of freedom, a number between 1 and 10^10, excluding 10^10.
         */
        f_Inv(probability: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom1: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom2: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the inverse of the (right-tailed) F probability distribution: if p = F.DIST.RT(x,...), then F.INV.RT(p,...) = x.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param probability Is a probability associated with the F cumulative distribution, a number between 0 and 1 inclusive.
         * @param degFreedom1 Is the numerator degrees of freedom, a number between 1 and 10^10, excluding 10^10.
         * @param degFreedom2 Is the denominator degrees of freedom, a number between 1 and 10^10, excluding 10^10.
         */
        f_Inv_RT(probability: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom1: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom2: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the factorial of a number, equal to 1*2*3*...* Number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the nonnegative number you want the factorial of.
         */
        fact(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the double factorial of a number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the value for which to return the double factorial.
         */
        factDouble(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the logical value FALSE.
         *
         * [Api set: ExcelApi 1.2]
         */
        false(): FunctionResult<boolean>;
        /**
         *
         * Returns the starting position of one text string within another text string. FIND is case-sensitive.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param findText Is the text you want to find. Use double quotes (empty text) to match the first character in withinText; wildcard characters not allowed.
         * @param withinText Is the text containing the text you want to find.
         * @param startNum Specifies the character at which to start the search. The first character in withinText is character number 1. If omitted, startNum = 1.
         */
        find(findText: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, withinText: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, startNum?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Finds the starting position of one text string within another text string. FINDB is case-sensitive. Use with double-byte character sets (DBCS).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param findText Is the text you want to find.
         * @param withinText Is the text containing the text you want to find.
         * @param startNum Specifies the character at which to start the search.
         */
        findB(findText: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, withinText: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, startNum?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the Fisher transformation.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value for which you want the transformation, a number between -1 and 1, excluding -1 and 1.
         */
        fisher(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the inverse of the Fisher transformation: if y = FISHER(x), then FISHERINV(y) = x.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param y Is the value for which you want to perform the inverse of the transformation.
         */
        fisherInv(y: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Rounds a number to the specified number of decimals and returns the result as text with or without commas.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the number you want to round and convert to text.
         * @param decimals Is the number of digits to the right of the decimal point. If omitted, Decimals = 2.
         * @param noCommas Is a logical value: do not display commas in the returned text = TRUE; do display commas in the returned text = FALSE or omitted.
         */
        fixed(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, decimals?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, noCommas?: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Rounds a number down, to the nearest integer or to the nearest multiple of significance.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the value you want to round.
         * @param significance Is the multiple to which you want to round.
         * @param mode When given and nonzero this function will round towards zero.
         */
        floor_Math(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, significance?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, mode?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Rounds a number down, to the nearest integer or to the nearest multiple of significance.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the numeric value you want to round.
         * @param significance Is the multiple to which you want to round.
         */
        floor_Precise(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, significance?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the future value of an investment based on periodic, constant payments and a constant interest rate.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param rate Is the interest rate per period. For example, use 6%/4 for quarterly payments at 6% APR.
         * @param nper Is the total number of payment periods in the investment.
         * @param pmt Is the payment made each period; it cannot change over the life of the investment.
         * @param pv Is the present value, or the lump-sum amount that a series of future payments is worth now. If omitted, Pv = 0.
         * @param type Is a value representing the timing of payment: payment at the beginning of the period = 1; payment at the end of the period = 0 or omitted.
         */
        fv(rate: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, nper: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pmt: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pv?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, type?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the Gamma function value.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value for which you want to calculate Gamma.
         */
        gamma(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the natural logarithm of the gamma function.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value for which you want to calculate GAMMALN, a positive number.
         */
        gammaLn(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the natural logarithm of the gamma function.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value for which you want to calculate GAMMALN.PRECISE, a positive number.
         */
        gammaLn_Precise(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the gamma distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value at which you want to evaluate the distribution, a nonnegative number.
         * @param alpha Is a parameter to the distribution, a positive number.
         * @param beta Is a parameter to the distribution, a positive number. If beta = 1, GAMMA.DIST returns the standard gamma distribution.
         * @param cumulative Is a logical value: return the cumulative distribution function = TRUE; return the probability mass function = FALSE or omitted.
         */
        gamma_Dist(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, alpha: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, beta: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, cumulative: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the inverse of the gamma cumulative distribution: if p = GAMMA.DIST(x,...), then GAMMA.INV(p,...) = x.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param probability Is the probability associated with the gamma distribution, a number between 0 and 1, inclusive.
         * @param alpha Is a parameter to the distribution, a positive number.
         * @param beta Is a parameter to the distribution, a positive number. If beta = 1, GAMMA.INV returns the inverse of the standard gamma distribution.
         */
        gamma_Inv(probability: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, alpha: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, beta: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns 0.5 less than the standard normal cumulative distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value for which you want the distribution.
         */
        gauss(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the greatest common divisor.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 values.
         */
        gcd(...values: Array<number | string | Excel.Range | boolean | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Tests whether a number is greater than a threshold value.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the value to test against step.
         * @param step Is the threshold value.
         */
        geStep(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, step?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the geometric mean of an array or range of positive numeric data.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numbers or names, arrays, or references that contain numbers for which you want the mean.
         */
        geoMean(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Looks for a value in the top row of a table or array of values and returns the value in the same column from a row you specify.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param lookupValue Is the value to be found in the first row of the table and can be a value, a reference, or a text string.
         * @param tableArray Is a table of text, numbers, or logical values in which data is looked up. tableArray can be a reference to a range or a range name.
         * @param rowIndexNum Is the row number in tableArray from which the matching value should be returned. The first row of values in the table is row 1.
         * @param rangeLookup Is a logical value: to find the closest match in the top row (sorted in ascending order) = TRUE or omitted; find an exact match = FALSE.
         */
        hlookup(lookupValue: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, tableArray: Excel.Range | number | Excel.RangeReference | Excel.FunctionResult<any>, rowIndexNum: Excel.Range | number | Excel.RangeReference | Excel.FunctionResult<any>, rangeLookup?: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number | string | boolean>;
        /**
         *
         * Returns the harmonic mean of a data set of positive numbers: the reciprocal of the arithmetic mean of reciprocals.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numbers or names, arrays, or references that contain numbers for which you want the harmonic mean.
         */
        harMean(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Converts a Hexadecimal number to binary.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the hexadecimal number you want to convert.
         * @param places Is the number of characters to use.
         */
        hex2Bin(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, places?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a hexadecimal number to decimal.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the hexadecimal number you want to convert.
         */
        hex2Dec(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a hexadecimal number to octal.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the hexadecimal number you want to convert.
         * @param places Is the number of characters to use.
         */
        hex2Oct(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, places?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the hour as a number from 0 (12:00 A.M.) to 23 (11:00 P.M.).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param serialNumber Is a number in the date-time code used by Microsoft Excel, or text in time format, such as 16:48:00 or 4:48:00 PM.
         */
        hour(serialNumber: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the hypergeometric distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param sampleS Is the number of successes in the sample.
         * @param numberSample Is the size of the sample.
         * @param populationS Is the number of successes in the population.
         * @param numberPop Is the population size.
         * @param cumulative Is a logical value: for the cumulative distribution function, use TRUE; for the probability density function, use FALSE.
         */
        hypGeom_Dist(sampleS: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numberSample: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, populationS: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numberPop: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, cumulative: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Creates a shortcut or jump that opens a document stored on your hard drive, a network server, or on the Internet.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param linkLocation Is the text giving the path and file name to the document to be opened, a hard drive location, UNC address, or URL path.
         * @param friendlyName Is text or a number that is displayed in the cell. If omitted, the cell displays the linkLocation text.
         */
        hyperlink(linkLocation: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, friendlyName?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number | string | boolean>;
        /**
         *
         * Rounds a number up, to the nearest integer or to the nearest multiple of significance.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the value you want to round.
         * @param significance Is the optional multiple to which you want to round.
         */
        iso_Ceiling(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, significance?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Checks whether a condition is met, and returns one value if TRUE, and another value if FALSE.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param logicalTest Is any value or expression that can be evaluated to TRUE or FALSE.
         * @param valueIfTrue Is the value that is returned if logicalTest is TRUE. If omitted, TRUE is returned. You can nest up to seven IF functions.
         * @param valueIfFalse Is the value that is returned if logicalTest is FALSE. If omitted, FALSE is returned.
         */
        if(logicalTest: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, valueIfTrue?: Excel.Range | number | string | boolean | Excel.RangeReference | Excel.FunctionResult<any>, valueIfFalse?: Excel.Range | number | string | boolean | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number | string | boolean>;
        /**
         *
         * Returns the absolute value (modulus) of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the absolute value.
         */
        imAbs(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the argument q, an angle expressed in radians.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the argument.
         */
        imArgument(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the complex conjugate of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the conjugate.
         */
        imConjugate(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the cosine of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the cosine.
         */
        imCos(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the hyperbolic cosine of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the hyperbolic cosine.
         */
        imCosh(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the cotangent of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the cotangent.
         */
        imCot(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the cosecant of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the cosecant.
         */
        imCsc(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the hyperbolic cosecant of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the hyperbolic cosecant.
         */
        imCsch(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the quotient of two complex numbers.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber1 Is the complex numerator or dividend.
         * @param inumber2 Is the complex denominator or divisor.
         */
        imDiv(inumber1: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, inumber2: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the exponential of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the exponential.
         */
        imExp(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the natural logarithm of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the natural logarithm.
         */
        imLn(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the base-10 logarithm of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the common logarithm.
         */
        imLog10(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the base-2 logarithm of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the base-2 logarithm.
         */
        imLog2(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns a complex number raised to an integer power.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number you want to raise to a power.
         * @param number Is the power to which you want to raise the complex number.
         */
        imPower(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the product of 1 to 255 complex numbers.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values Inumber1, Inumber2,... are from 1 to 255 complex numbers to multiply.
         */
        imProduct(...values: Array<Excel.Range | number | string | boolean | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the real coefficient of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the real coefficient.
         */
        imReal(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the secant of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the secant.
         */
        imSec(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the hyperbolic secant of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the hyperbolic secant.
         */
        imSech(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the sine of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the sine.
         */
        imSin(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the hyperbolic sine of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the hyperbolic sine.
         */
        imSinh(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the square root of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the square root.
         */
        imSqrt(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the difference of two complex numbers.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber1 Is the complex number from which to subtract inumber2.
         * @param inumber2 Is the complex number to subtract from inumber1.
         */
        imSub(inumber1: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, inumber2: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the sum of complex numbers.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are from 1 to 255 complex numbers to add.
         */
        imSum(...values: Array<Excel.Range | number | string | boolean | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the tangent of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the tangent.
         */
        imTan(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the imaginary coefficient of a complex number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param inumber Is a complex number for which you want the imaginary coefficient.
         */
        imaginary(inumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Rounds a number down to the nearest integer.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the real number you want to round down to an integer.
         */
        int(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the interest rate for a fully invested security.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param investment Is the amount invested in the security.
         * @param redemption Is the amount to be received at maturity.
         * @param basis Is the type of day count basis to use.
         */
        intRate(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, investment: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, redemption: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the interest payment for a given period for an investment, based on periodic, constant payments and a constant interest rate.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param rate Is the interest rate per period. For example, use 6%/4 for quarterly payments at 6% APR.
         * @param per Is the period for which you want to find the interest and must be in the range 1 to Nper.
         * @param nper Is the total number of payment periods in an investment.
         * @param pv Is the present value, or the lump-sum amount that a series of future payments is worth now.
         * @param fv Is the future value, or a cash balance you want to attain after the last payment is made. If omitted, Fv = 0.
         * @param type Is a logical value representing the timing of payment: at the end of the period = 0 or omitted, at the beginning of the period = 1.
         */
        ipmt(rate: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, per: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, nper: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pv: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, fv?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, type?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the internal rate of return for a series of cash flows.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values Is an array or a reference to cells that contain numbers for which you want to calculate the internal rate of return.
         * @param guess Is a number that you guess is close to the result of IRR; 0.1 (10 percent) if omitted.
         */
        irr(values: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, guess?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Checks whether a value is an error (#VALUE!, #REF!, #DIV/0!, #NUM!, #NAME?, or #NULL!) excluding #N/A, and returns TRUE or FALSE.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param value Is the value you want to test. Value can refer to a cell, a formula, or a name that refers to a cell, formula, or value.
         */
        isErr(value: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<boolean>;
        /**
         *
         * Checks whether a value is an error (#N/A, #VALUE!, #REF!, #DIV/0!, #NUM!, #NAME?, or #NULL!), and returns TRUE or FALSE.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param value Is the value you want to test. Value can refer to a cell, a formula, or a name that refers to a cell, formula, or value.
         */
        isError(value: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<boolean>;
        /**
         *
         * Returns TRUE if the number is even.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the value to test.
         */
        isEven(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Checks whether a reference is to a cell containing a formula, and returns TRUE or FALSE.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param reference Is a reference to the cell you want to test.  Reference can be a cell reference, a formula, or name that refers to a cell.
         */
        isFormula(reference: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<boolean>;
        /**
         *
         * Checks whether a value is a logical value (TRUE or FALSE), and returns TRUE or FALSE.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param value Is the value you want to test. Value can refer to a cell, a formula, or a name that refers to a cell, formula, or value.
         */
        isLogical(value: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<boolean>;
        /**
         *
         * Checks whether a value is #N/A, and returns TRUE or FALSE.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param value Is the value you want to test. Value can refer to a cell, a formula, or a name that refers to a cell, formula, or value.
         */
        isNA(value: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<boolean>;
        /**
         *
         * Checks whether a value is not text (blank cells are not text), and returns TRUE or FALSE.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param value Is the value you want tested: a cell; a formula; or a name referring to a cell, formula, or value.
         */
        isNonText(value: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<boolean>;
        /**
         *
         * Checks whether a value is a number, and returns TRUE or FALSE.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param value Is the value you want to test. Value can refer to a cell, a formula, or a name that refers to a cell, formula, or value.
         */
        isNumber(value: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<boolean>;
        /**
         *
         * Returns TRUE if the number is odd.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the value to test.
         */
        isOdd(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Checks whether a value is text, and returns TRUE or FALSE.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param value Is the value you want to test. Value can refer to a cell, a formula, or a name that refers to a cell, formula, or value.
         */
        isText(value: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<boolean>;
        /**
         *
         * Returns the ISO week number in the year for a given date.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param date Is the date-time code used by Microsoft Excel for date and time calculation.
         */
        isoWeekNum(date: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the interest paid during a specific period of an investment.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param rate Interest rate per period. For example, use 6%/4 for quarterly payments at 6% APR.
         * @param per Period for which you want to find the interest.
         * @param nper Number of payment periods in an investment.
         * @param pv Lump sum amount that a series of future payments is right now.
         */
        ispmt(rate: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, per: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, nper: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pv: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Checks whether a value is a reference, and returns TRUE or FALSE.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param value Is the value you want to test. Value can refer to a cell, a formula, or a name that refers to a cell, formula, or value.
         */
        isref(value: Excel.Range | number | string | boolean | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<boolean>;
        /**
         *
         * Returns the kurtosis of a data set.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numbers or names, arrays, or references that contain numbers for which you want the kurtosis.
         */
        kurt(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the k-th largest value in a data set. For example, the fifth largest number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param array Is the array or range of data for which you want to determine the k-th largest value.
         * @param k Is the position (from the largest) in the array or cell range of the value to return.
         */
        large(array: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, k: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the least common multiple.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 values for which you want the least common multiple.
         */
        lcm(...values: Array<number | string | Excel.Range | boolean | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the specified number of characters from the start of a text string.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the text string containing the characters you want to extract.
         * @param numChars Specifies how many characters you want LEFT to extract; 1 if omitted.
         */
        left(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numChars?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns the specified number of characters from the start of a text string. Use with double-byte character sets (DBCS).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the text string containing the characters you want to extract.
         * @param numBytes Specifies how many characters you want LEFT to return.
         */
        leftb(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numBytes?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns the number of characters in a text string.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the text whose length you want to find. Spaces count as characters.
         */
        len(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of characters in a text string. Use with double-byte character sets (DBCS).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the text whose length you want to find.
         */
        lenb(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the natural logarithm of a number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the positive real number for which you want the natural logarithm.
         */
        ln(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the logarithm of a number to the base you specify.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the positive real number for which you want the logarithm.
         * @param base Is the base of the logarithm; 10 if omitted.
         */
        log(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, base?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the base-10 logarithm of a number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the positive real number for which you want the base-10 logarithm.
         */
        log10(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the lognormal distribution of x, where ln(x) is normally distributed with parameters Mean and Standard_dev.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value at which to evaluate the function, a positive number.
         * @param mean Is the mean of ln(x).
         * @param standardDev Is the standard deviation of ln(x), a positive number.
         * @param cumulative Is a logical value: for the cumulative distribution function, use TRUE; for the probability density function, use FALSE.
         */
        logNorm_Dist(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, mean: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, standardDev: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, cumulative: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the inverse of the lognormal cumulative distribution function of x, where ln(x) is normally distributed with parameters Mean and Standard_dev.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param probability Is a probability associated with the lognormal distribution, a number between 0 and 1, inclusive.
         * @param mean Is the mean of ln(x).
         * @param standardDev Is the standard deviation of ln(x), a positive number.
         */
        logNorm_Inv(probability: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, mean: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, standardDev: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Looks up a value either from a one-row or one-column range or from an array. Provided for backward compatibility.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param lookupValue Is a value that LOOKUP searches for in lookupVector and can be a number, text, a logical value, or a name or reference to a value.
         * @param lookupVector Is a range that contains only one row or one column of text, numbers, or logical values, placed in ascending order.
         * @param resultVector Is a range that contains only one row or column, the same size as lookupVector.
         */
        lookup(lookupValue: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, lookupVector: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, resultVector?: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number | string | boolean>;
        /**
         *
         * Converts all letters in a text string to lowercase.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the text you want to convert to lowercase. Characters in Text that are not letters are not changed.
         */
        lower(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns the Macauley modified duration for a security with an assumed par value of $100.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param coupon Is the security's annual coupon rate.
         * @param yld Is the security's annual yield.
         * @param frequency Is the number of coupon payments per year.
         * @param basis Is the type of day count basis to use.
         */
        mduration(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, coupon: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, yld: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, frequency: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the internal rate of return for a series of periodic cash flows, considering both cost of investment and interest on reinvestment of cash.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values Is an array or a reference to cells that contain numbers that represent a series of payments (negative) and income (positive) at regular periods.
         * @param financeRate Is the interest rate you pay on the money used in the cash flows.
         * @param reinvestRate Is the interest rate you receive on the cash flows as you reinvest them.
         */
        mirr(values: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, financeRate: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, reinvestRate: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns a number rounded to the desired multiple.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the value to round.
         * @param multiple Is the multiple to which you want to round number.
         */
        mround(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, multiple: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the relative position of an item in an array that matches a specified value in a specified order.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param lookupValue Is the value you use to find the value you want in the array, a number, text, or logical value, or a reference to one of these.
         * @param lookupArray Is a contiguous range of cells containing possible lookup values, an array of values, or a reference to an array.
         * @param matchType Is a number 1, 0, or -1 indicating which value to return.
         */
        match(lookupValue: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, lookupArray: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, matchType?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the largest value in a set of values. Ignores logical values and text.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numbers, empty cells, logical values, or text numbers for which you want the maximum.
         */
        max(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the largest value in a set of values. Does not ignore logical values and text.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numbers, empty cells, logical values, or text numbers for which you want the maximum.
         */
        maxA(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the median, or the number in the middle of the set of given numbers.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numbers or names, arrays, or references that contain numbers for which you want the median.
         */
        median(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the characters from the middle of a text string, given a starting position and length.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the text string from which you want to extract the characters.
         * @param startNum Is the position of the first character you want to extract. The first character in Text is 1.
         * @param numChars Specifies how many characters to return from Text.
         */
        mid(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, startNum: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numChars: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns characters from the middle of a text string, given a starting position and length. Use with double-byte character sets (DBCS).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the text string containing the characters you want to extract.
         * @param startNum Is the position of the first character you want to extract in text.
         * @param numBytes Specifies how many characters to return from text.
         */
        midb(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, startNum: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numBytes: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns the smallest number in a set of values. Ignores logical values and text.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numbers, empty cells, logical values, or text numbers for which you want the minimum.
         */
        min(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the smallest value in a set of values. Does not ignore logical values and text.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numbers, empty cells, logical values, or text numbers for which you want the minimum.
         */
        minA(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the minute, a number from 0 to 59.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param serialNumber Is a number in the date-time code used by Microsoft Excel or text in time format, such as 16:48:00 or 4:48:00 PM.
         */
        minute(serialNumber: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the remainder after a number is divided by a divisor.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the number for which you want to find the remainder after the division is performed.
         * @param divisor Is the number by which you want to divide Number.
         */
        mod(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, divisor: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the month, a number from 1 (January) to 12 (December).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param serialNumber Is a number in the date-time code used by Microsoft Excel.
         */
        month(serialNumber: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the multinomial of a set of numbers.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 values for which you want the multinomial.
         */
        multiNomial(...values: Array<number | string | Excel.Range | boolean | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Converts non-number value to a number, dates to serial numbers, TRUE to 1, anything else to 0 (zero).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param value Is the value you want converted.
         */
        n(value: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of periods for an investment based on periodic, constant payments and a constant interest rate.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param rate Is the interest rate per period. For example, use 6%/4 for quarterly payments at 6% APR.
         * @param pmt Is the payment made each period; it cannot change over the life of the investment.
         * @param pv Is the present value, or the lump-sum amount that a series of future payments is worth now.
         * @param fv Is the future value, or a cash balance you want to attain after the last payment is made. If omitted, zero is used.
         * @param type Is a logical value: payment at the beginning of the period = 1; payment at the end of the period = 0 or omitted.
         */
        nper(rate: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pmt: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pv: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, fv?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, type?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the error value #N/A (value not available).
         *
         * [Api set: ExcelApi 1.2]
         */
        na(): FunctionResult<number | string>;
        /**
         *
         * Returns the negative binomial distribution, the probability that there will be Number_f failures before the Number_s-th success, with Probability_s probability of a success.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param numberF Is the number of failures.
         * @param numberS Is the threshold number of successes.
         * @param probabilityS Is the probability of a success; a number between 0 and 1.
         * @param cumulative Is a logical value: for the cumulative distribution function, use TRUE; for the probability mass function, use FALSE.
         */
        negBinom_Dist(numberF: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numberS: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, probabilityS: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, cumulative: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of whole workdays between two dates.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param startDate Is a serial date number that represents the start date.
         * @param endDate Is a serial date number that represents the end date.
         * @param holidays Is an optional set of one or more serial date numbers to exclude from the working calendar, such as state and federal holidays and floating holidays.
         */
        networkDays(startDate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, endDate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, holidays?: number | string | Excel.Range | boolean | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of whole workdays between two dates with custom weekend parameters.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param startDate Is a serial date number that represents the start date.
         * @param endDate Is a serial date number that represents the end date.
         * @param weekend Is a number or string specifying when weekends occur.
         * @param holidays Is an optional set of one or more serial date numbers to exclude from the working calendar, such as state and federal holidays and floating holidays.
         */
        networkDays_Intl(startDate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, endDate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, weekend?: number | string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, holidays?: number | string | Excel.Range | boolean | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the annual nominal interest rate.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param effectRate Is the effective interest rate.
         * @param npery Is the number of compounding periods per year.
         */
        nominal(effectRate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, npery: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the normal distribution for the specified mean and standard deviation.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value for which you want the distribution.
         * @param mean Is the arithmetic mean of the distribution.
         * @param standardDev Is the standard deviation of the distribution, a positive number.
         * @param cumulative Is a logical value: for the cumulative distribution function, use TRUE; for the probability density function, use FALSE.
         */
        norm_Dist(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, mean: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, standardDev: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, cumulative: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the inverse of the normal cumulative distribution for the specified mean and standard deviation.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param probability Is a probability corresponding to the normal distribution, a number between 0 and 1 inclusive.
         * @param mean Is the arithmetic mean of the distribution.
         * @param standardDev Is the standard deviation of the distribution, a positive number.
         */
        norm_Inv(probability: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, mean: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, standardDev: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the standard normal distribution (has a mean of zero and a standard deviation of one).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param z Is the value for which you want the distribution.
         * @param cumulative Is a logical value for the function to return: the cumulative distribution function = TRUE; the probability density function = FALSE.
         */
        norm_S_Dist(z: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, cumulative: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the inverse of the standard normal cumulative distribution (has a mean of zero and a standard deviation of one).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param probability Is a probability corresponding to the normal distribution, a number between 0 and 1 inclusive.
         */
        norm_S_Inv(probability: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Changes FALSE to TRUE, or TRUE to FALSE.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param logical Is a value or expression that can be evaluated to TRUE or FALSE.
         */
        not(logical: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<boolean>;
        /**
         *
         * Returns the current date and time formatted as a date and time.
         *
         * [Api set: ExcelApi 1.2]
         */
        now(): FunctionResult<number>;
        /**
         *
         * Returns the net present value of an investment based on a discount rate and a series of future payments (negative values) and income (positive values).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param rate Is the rate of discount over the length of one period.
         * @param values List of parameters, whose elements are 1 to 254 payments and income, equally spaced in time and occurring at the end of each period.
         */
        npv(rate: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, ...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Converts text to number in a locale-independent manner.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the string representing the number you want to convert.
         * @param decimalSeparator Is the character used as the decimal separator in the string.
         * @param groupSeparator Is the character used as the group separator in the string.
         */
        numberValue(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, decimalSeparator?: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, groupSeparator?: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts an octal number to binary.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the octal number you want to convert.
         * @param places Is the number of characters to use.
         */
        oct2Bin(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, places?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts an octal number to decimal.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the octal number you want to convert.
         */
        oct2Dec(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts an octal number to hexadecimal.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the octal number you want to convert.
         * @param places Is the number of characters to use.
         */
        oct2Hex(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, places?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Rounds a positive number up and negative number down to the nearest odd integer.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the value to round.
         */
        odd(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the price per $100 face value of a security with an odd first period.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param issue Is the security's issue date, expressed as a serial date number.
         * @param firstCoupon Is the security's first coupon date, expressed as a serial date number.
         * @param rate Is the security's interest rate.
         * @param yld Is the security's annual yield.
         * @param redemption Is the security's redemption value per $100 face value.
         * @param frequency Is the number of coupon payments per year.
         * @param basis Is the type of day count basis to use.
         */
        oddFPrice(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, issue: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, firstCoupon: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, rate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, yld: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, redemption: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, frequency: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the yield of a security with an odd first period.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param issue Is the security's issue date, expressed as a serial date number.
         * @param firstCoupon Is the security's first coupon date, expressed as a serial date number.
         * @param rate Is the security's interest rate.
         * @param pr Is the security's price.
         * @param redemption Is the security's redemption value per $100 face value.
         * @param frequency Is the number of coupon payments per year.
         * @param basis Is the type of day count basis to use.
         */
        oddFYield(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, issue: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, firstCoupon: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, rate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pr: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, redemption: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, frequency: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the price per $100 face value of a security with an odd last period.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param lastInterest Is the security's last coupon date, expressed as a serial date number.
         * @param rate Is the security's interest rate.
         * @param yld Is the security's annual yield.
         * @param redemption Is the security's redemption value per $100 face value.
         * @param frequency Is the number of coupon payments per year.
         * @param basis Is the type of day count basis to use.
         */
        oddLPrice(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, lastInterest: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, rate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, yld: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, redemption: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, frequency: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the yield of a security with an odd last period.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param lastInterest Is the security's last coupon date, expressed as a serial date number.
         * @param rate Is the security's interest rate.
         * @param pr Is the security's price.
         * @param redemption Is the security's redemption value per $100 face value.
         * @param frequency Is the number of coupon payments per year.
         * @param basis Is the type of day count basis to use.
         */
        oddLYield(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, lastInterest: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, rate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pr: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, redemption: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, frequency: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Checks whether any of the arguments are TRUE, and returns TRUE or FALSE. Returns FALSE only if all arguments are FALSE.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 conditions that you want to test that can be either TRUE or FALSE.
         */
        or(...values: Array<boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<boolean>;
        /**
         *
         * Returns the number of periods required by an investment to reach a specified value.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param rate Is the interest rate per period.
         * @param pv Is the present value of the investment.
         * @param fv Is the desired future value of the investment.
         */
        pduration(rate: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pv: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, fv: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the rank of a value in a data set as a percentage of the data set as a percentage (0..1, exclusive) of the data set.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param array Is the array or range of data with numeric values that defines relative standing.
         * @param x Is the value for which you want to know the rank.
         * @param significance Is an optional value that identifies the number of significant digits for the returned percentage, three digits if omitted (0.xxx%).
         */
        percentRank_Exc(array: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, significance?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the rank of a value in a data set as a percentage of the data set as a percentage (0..1, inclusive) of the data set.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param array Is the array or range of data with numeric values that defines relative standing.
         * @param x Is the value for which you want to know the rank.
         * @param significance Is an optional value that identifies the number of significant digits for the returned percentage, three digits if omitted (0.xxx%).
         */
        percentRank_Inc(array: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, significance?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the k-th percentile of values in a range, where k is in the range 0..1, exclusive.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param array Is the array or range of data that defines relative standing.
         * @param k Is the percentile value that is between 0 through 1, inclusive.
         */
        percentile_Exc(array: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, k: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the k-th percentile of values in a range, where k is in the range 0..1, inclusive.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param array Is the array or range of data that defines relative standing.
         * @param k Is the percentile value that is between 0 through 1, inclusive.
         */
        percentile_Inc(array: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, k: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of permutations for a given number of objects that can be selected from the total objects.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the total number of objects.
         * @param numberChosen Is the number of objects in each permutation.
         */
        permut(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numberChosen: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of permutations for a given number of objects (with repetitions) that can be selected from the total objects.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the total number of objects.
         * @param numberChosen Is the number of objects in each permutation.
         */
        permutationa(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numberChosen: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the value of the density function for a standard normal distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the number for which you want the density of the standard normal distribution.
         */
        phi(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the value of Pi, 3.14159265358979, accurate to 15 digits.
         *
         * [Api set: ExcelApi 1.2]
         */
        pi(): FunctionResult<number>;
        /**
         *
         * Calculates the payment for a loan based on constant payments and a constant interest rate.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param rate Is the interest rate per period for the loan. For example, use 6%/4 for quarterly payments at 6% APR.
         * @param nper Is the total number of payments for the loan.
         * @param pv Is the present value: the total amount that a series of future payments is worth now.
         * @param fv Is the future value, or a cash balance you want to attain after the last payment is made, 0 (zero) if omitted.
         * @param type Is a logical value: payment at the beginning of the period = 1; payment at the end of the period = 0 or omitted.
         */
        pmt(rate: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, nper: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pv: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, fv?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, type?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the Poisson distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the number of events.
         * @param mean Is the expected numeric value, a positive number.
         * @param cumulative Is a logical value: for the cumulative Poisson probability, use TRUE; for the Poisson probability mass function, use FALSE.
         */
        poisson_Dist(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, mean: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, cumulative: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the result of a number raised to a power.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the base number, any real number.
         * @param power Is the exponent, to which the base number is raised.
         */
        power(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, power: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the payment on the principal for a given investment based on periodic, constant payments and a constant interest rate.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param rate Is the interest rate per period. For example, use 6%/4 for quarterly payments at 6% APR.
         * @param per Specifies the period and must be in the range 1 to nper.
         * @param nper Is the total number of payment periods in an investment.
         * @param pv Is the present value: the total amount that a series of future payments is worth now.
         * @param fv Is the future value, or cash balance you want to attain after the last payment is made.
         * @param type Is a logical value: payment at the beginning of the period = 1; payment at the end of the period = 0 or omitted.
         */
        ppmt(rate: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, per: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, nper: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pv: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, fv?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, type?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the price per $100 face value of a security that pays periodic interest.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param rate Is the security's annual coupon rate.
         * @param yld Is the security's annual yield.
         * @param redemption Is the security's redemption value per $100 face value.
         * @param frequency Is the number of coupon payments per year.
         * @param basis Is the type of day count basis to use.
         */
        price(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, rate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, yld: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, redemption: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, frequency: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the price per $100 face value of a discounted security.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param discount Is the security's discount rate.
         * @param redemption Is the security's redemption value per $100 face value.
         * @param basis Is the type of day count basis to use.
         */
        priceDisc(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, discount: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, redemption: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the price per $100 face value of a security that pays interest at maturity.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param issue Is the security's issue date, expressed as a serial date number.
         * @param rate Is the security's interest rate at date of issue.
         * @param yld Is the security's annual yield.
         * @param basis Is the type of day count basis to use.
         */
        priceMat(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, issue: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, rate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, yld: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Multiplies all the numbers given as arguments.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numbers, logical values, or text representations of numbers that you want to multiply.
         */
        product(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Converts a text string to proper case; the first letter in each word to uppercase, and all other letters to lowercase.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is text enclosed in quotation marks, a formula that returns text, or a reference to a cell containing text to partially capitalize.
         */
        proper(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns the present value of an investment: the total amount that a series of future payments is worth now.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param rate Is the interest rate per period. For example, use 6%/4 for quarterly payments at 6% APR.
         * @param nper Is the total number of payment periods in an investment.
         * @param pmt Is the payment made each period and cannot change over the life of the investment.
         * @param fv Is the future value, or a cash balance you want to attain after the last payment is made.
         * @param type Is a logical value: payment at the beginning of the period = 1; payment at the end of the period = 0 or omitted.
         */
        pv(rate: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, nper: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pmt: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, fv?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, type?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the quartile of a data set, based on percentile values from 0..1, exclusive.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param array Is the array or cell range of numeric values for which you want the quartile value.
         * @param quart Is a number: minimum value = 0; 1st quartile = 1; median value = 2; 3rd quartile = 3; maximum value = 4.
         */
        quartile_Exc(array: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, quart: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the quartile of a data set, based on percentile values from 0..1, inclusive.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param array Is the array or cell range of numeric values for which you want the quartile value.
         * @param quart Is a number: minimum value = 0; 1st quartile = 1; median value = 2; 3rd quartile = 3; maximum value = 4.
         */
        quartile_Inc(array: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, quart: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the integer portion of a division.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param numerator Is the dividend.
         * @param denominator Is the divisor.
         */
        quotient(numerator: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, denominator: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts degrees to radians.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param angle Is an angle in degrees that you want to convert.
         */
        radians(angle: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns a random number greater than or equal to 0 and less than 1, evenly distributed (changes on recalculation).
         *
         * [Api set: ExcelApi 1.2]
         */
        rand(): FunctionResult<number>;
        /**
         *
         * Returns a random number between the numbers you specify.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param bottom Is the smallest integer RANDBETWEEN will return.
         * @param top Is the largest integer RANDBETWEEN will return.
         */
        randBetween(bottom: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, top: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the rank of a number in a list of numbers: its size relative to other values in the list; if more than one value has the same rank, the average rank is returned.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the number for which you want to find the rank.
         * @param ref Is an array of, or a reference to, a list of numbers. Nonnumeric values are ignored.
         * @param order Is a number: rank in the list sorted descending = 0 or omitted; rank in the list sorted ascending = any nonzero value.
         */
        rank_Avg(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, ref: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, order?: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the rank of a number in a list of numbers: its size relative to other values in the list; if more than one value has the same rank, the top rank of that set of values is returned.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the number for which you want to find the rank.
         * @param ref Is an array of, or a reference to, a list of numbers. Nonnumeric values are ignored.
         * @param order Is a number: rank in the list sorted descending = 0 or omitted; rank in the list sorted ascending = any nonzero value.
         */
        rank_Eq(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, ref: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, order?: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the interest rate per period of a loan or an investment. For example, use 6%/4 for quarterly payments at 6% APR.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param nper Is the total number of payment periods for the loan or investment.
         * @param pmt Is the payment made each period and cannot change over the life of the loan or investment.
         * @param pv Is the present value: the total amount that a series of future payments is worth now.
         * @param fv Is the future value, or a cash balance you want to attain after the last payment is made. If omitted, uses Fv = 0.
         * @param type Is a logical value: payment at the beginning of the period = 1; payment at the end of the period = 0 or omitted.
         * @param guess Is your guess for what the rate will be; if omitted, Guess = 0.1 (10 percent).
         */
        rate(nper: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pmt: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pv: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, fv?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, type?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, guess?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the amount received at maturity for a fully invested security.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param investment Is the amount invested in the security.
         * @param discount Is the security's discount rate.
         * @param basis Is the type of day count basis to use.
         */
        received(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, investment: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, discount: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Replaces part of a text string with a different text string.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param oldText Is text in which you want to replace some characters.
         * @param startNum Is the position of the character in oldText that you want to replace with newText.
         * @param numChars Is the number of characters in oldText that you want to replace.
         * @param newText Is the text that will replace characters in oldText.
         */
        replace(oldText: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, startNum: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numChars: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, newText: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Replaces part of a text string with a different text string. Use with double-byte character sets (DBCS).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param oldText Is text in which you want to replace some characters.
         * @param startNum Is the position of the character in oldText that you want to replace with newText.
         * @param numBytes Is the number of characters in oldText that you want to replace with newText.
         * @param newText Is the text that will replace characters in oldText.
         */
        replaceB(oldText: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, startNum: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numBytes: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, newText: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Repeats text a given number of times. Use REPT to fill a cell with a number of instances of a text string.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the text you want to repeat.
         * @param numberTimes Is a positive number specifying the number of times to repeat text.
         */
        rept(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numberTimes: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns the specified number of characters from the end of a text string.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the text string that contains the characters you want to extract.
         * @param numChars Specifies how many characters you want to extract, 1 if omitted.
         */
        right(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numChars?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns the specified number of characters from the end of a text string. Use with double-byte character sets (DBCS).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the text string containing the characters you want to extract.
         * @param numBytes Specifies how many characters you want to extract.
         */
        rightb(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numBytes?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Converts an Arabic numeral to Roman, as text.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the Arabic numeral you want to convert.
         * @param form Is the number specifying the type of Roman numeral you want.
         */
        roman(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, form?: boolean | number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Rounds a number to a specified number of digits.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the number you want to round.
         * @param numDigits Is the number of digits to which you want to round. Negative rounds to the left of the decimal point; zero to the nearest integer.
         */
        round(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numDigits: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Rounds a number down, toward zero.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is any real number that you want rounded down.
         * @param numDigits Is the number of digits to which you want to round. Negative rounds to the left of the decimal point; zero or omitted, to the nearest integer.
         */
        roundDown(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numDigits: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Rounds a number up, away from zero.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is any real number that you want rounded up.
         * @param numDigits Is the number of digits to which you want to round. Negative rounds to the left of the decimal point; zero or omitted, to the nearest integer.
         */
        roundUp(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numDigits: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of rows in a reference or array.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param array Is an array, an array formula, or a reference to a range of cells for which you want the number of rows.
         */
        rows(array: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns an equivalent interest rate for the growth of an investment.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param nper Is the number of periods for the investment.
         * @param pv Is the present value of the investment.
         * @param fv Is the future value of the investment.
         */
        rri(nper: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pv: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, fv: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the secant of an angle.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the angle in radians for which you want the secant.
         */
        sec(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the hyperbolic secant of an angle.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the angle in radians for which you want the hyperbolic secant.
         */
        sech(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the second, a number from 0 to 59.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param serialNumber Is a number in the date-time code used by Microsoft Excel or text in time format, such as 16:48:23 or 4:48:47 PM.
         */
        second(serialNumber: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the sum of a power series based on the formula.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the input value to the power series.
         * @param n Is the initial power to which you want to raise x.
         * @param m Is the step by which to increase n for each term in the series.
         * @param coefficients Is a set of coefficients by which each successive power of x is multiplied.
         */
        seriesSum(x: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, n: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, m: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, coefficients: Excel.Range | string | number | boolean | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the sheet number of the referenced sheet.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param value Is the name of a sheet or a reference that you want the sheet number of.  If omitted the number of the sheet containing the function is returned.
         */
        sheet(value?: Excel.Range | string | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the number of sheets in a reference.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param reference Is a reference for which you want to know the number of sheets it contains.  If omitted the number of sheets in the workbook containing the function is returned.
         */
        sheets(reference?: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the sign of a number: 1 if the number is positive, zero if the number is zero, or -1 if the number is negative.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is any real number.
         */
        sign(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the sine of an angle.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the angle in radians for which you want the sine. Degrees * PI()/180 = radians.
         */
        sin(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the hyperbolic sine of a number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is any real number.
         */
        sinh(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the skewness of a distribution: a characterization of the degree of asymmetry of a distribution around its mean.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numbers or names, arrays, or references that contain numbers for which you want the skewness.
         */
        skew(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the skewness of a distribution based on a population: a characterization of the degree of asymmetry of a distribution around its mean.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 254 numbers or names, arrays, or references that contain numbers for which you want the population skewness.
         */
        skew_p(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the straight-line depreciation of an asset for one period.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param cost Is the initial cost of the asset.
         * @param salvage Is the salvage value at the end of the life of the asset.
         * @param life Is the number of periods over which the asset is being depreciated (sometimes called the useful life of the asset).
         */
        sln(cost: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, salvage: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, life: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the k-th smallest value in a data set. For example, the fifth smallest number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param array Is an array or range of numerical data for which you want to determine the k-th smallest value.
         * @param k Is the position (from the smallest) in the array or range of the value to return.
         */
        small(array: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, k: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the square root of a number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the number for which you want the square root.
         */
        sqrt(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the square root of (number * Pi).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the number by which p is multiplied.
         */
        sqrtPi(number: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Estimates standard deviation based on a sample, including logical values and text. Text and the logical value FALSE have the value 0; the logical value TRUE has the value 1.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 values corresponding to a sample of a population and can be values or names or references to values.
         */
        stDevA(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Calculates standard deviation based on an entire population, including logical values and text. Text and the logical value FALSE have the value 0; the logical value TRUE has the value 1.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 values corresponding to a population and can be values, names, arrays, or references that contain values.
         */
        stDevPA(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Calculates standard deviation based on the entire population given as arguments (ignores logical values and text).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numbers corresponding to a population and can be numbers or references that contain numbers.
         */
        stDev_P(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Estimates standard deviation based on a sample (ignores logical values and text in the sample).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numbers corresponding to a sample of a population and can be numbers or references that contain numbers.
         */
        stDev_S(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns a normalized value from a distribution characterized by a mean and standard deviation.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value you want to normalize.
         * @param mean Is the arithmetic mean of the distribution.
         * @param standardDev Is the standard deviation of the distribution, a positive number.
         */
        standardize(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, mean: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, standardDev: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Replaces existing text with new text in a text string.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the text or the reference to a cell containing text in which you want to substitute characters.
         * @param oldText Is the existing text you want to replace. If the case of oldText does not match the case of text, SUBSTITUTE will not replace the text.
         * @param newText Is the text you want to replace oldText with.
         * @param instanceNum Specifies which occurrence of oldText you want to replace. If omitted, every instance of oldText is replaced.
         */
        substitute(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, oldText: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, newText: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, instanceNum?: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns a subtotal in a list or database.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param functionNum Is the number 1 to 11 that specifies the summary function for the subtotal.
         * @param values List of parameters, whose elements are 1 to 254 ranges or references for which you want the subtotal.
         */
        subtotal(functionNum: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, ...values: Array<Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Adds all the numbers in a range of cells.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numbers to sum. Logical values and text are ignored in cells, included if typed as arguments.
         */
        sum(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Adds the cells specified by a given condition or criteria.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param range Is the range of cells you want evaluated.
         * @param criteria Is the condition or criteria in the form of a number, expression, or text that defines which cells will be added.
         * @param sumRange Are the actual cells to sum. If omitted, the cells in range are used.
         */
        sumIf(range: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, criteria: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, sumRange?: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Adds the cells specified by a given set of conditions or criteria.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param sumRange Are the actual cells to sum.
         * @param values List of parameters, where the first element of each pair is the Is the range of cells you want evaluated for the particular condition , and the second element is is the condition or criteria in the form of a number, expression, or text that defines which cells will be added.
         */
        sumIfs(sumRange: Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, ...values: Array<Excel.Range | Excel.RangeReference | Excel.FunctionResult<any> | number | string | boolean>): FunctionResult<number>;
        /**
         *
         * Returns the sum of the squares of the arguments. The arguments can be numbers, arrays, names, or references to cells that contain numbers.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numbers, arrays, names, or references to arrays for which you want the sum of the squares.
         */
        sumSq(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the sum-of-years' digits depreciation of an asset for a specified period.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param cost Is the initial cost of the asset.
         * @param salvage Is the salvage value at the end of the life of the asset.
         * @param life Is the number of periods over which the asset is being depreciated (sometimes called the useful life of the asset).
         * @param per Is the period and must use the same units as Life.
         */
        syd(cost: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, salvage: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, life: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, per: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Checks whether a value is text, and returns the text if it is, or returns double quotes (empty text) if it is not.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param value Is the value to test.
         */
        t(value: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns the bond-equivalent yield for a treasury bill.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the Treasury bill's settlement date, expressed as a serial date number.
         * @param maturity Is the Treasury bill's maturity date, expressed as a serial date number.
         * @param discount Is the Treasury bill's discount rate.
         */
        tbillEq(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, discount: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the price per $100 face value for a treasury bill.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the Treasury bill's settlement date, expressed as a serial date number.
         * @param maturity Is the Treasury bill's maturity date, expressed as a serial date number.
         * @param discount Is the Treasury bill's discount rate.
         */
        tbillPrice(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, discount: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the yield for a treasury bill.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the Treasury bill's settlement date, expressed as a serial date number.
         * @param maturity Is the Treasury bill's maturity date, expressed as a serial date number.
         * @param pr Is the Treasury Bill's price per $100 face value.
         */
        tbillYield(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pr: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the left-tailed Student's t-distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the numeric value at which to evaluate the distribution.
         * @param degFreedom Is an integer indicating the number of degrees of freedom that characterize the distribution.
         * @param cumulative Is a logical value: for the cumulative distribution function, use TRUE; for the probability density function, use FALSE.
         */
        t_Dist(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, cumulative: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the two-tailed Student's t-distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the numeric value at which to evaluate the distribution.
         * @param degFreedom Is an integer indicating the number of degrees of freedom that characterize the distribution.
         */
        t_Dist_2T(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the right-tailed Student's t-distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the numeric value at which to evaluate the distribution.
         * @param degFreedom Is an integer indicating the number of degrees of freedom that characterize the distribution.
         */
        t_Dist_RT(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the left-tailed inverse of the Student's t-distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param probability Is the probability associated with the two-tailed Student's t-distribution, a number between 0 and 1 inclusive.
         * @param degFreedom Is a positive integer indicating the number of degrees of freedom to characterize the distribution.
         */
        t_Inv(probability: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the two-tailed inverse of the Student's t-distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param probability Is the probability associated with the two-tailed Student's t-distribution, a number between 0 and 1 inclusive.
         * @param degFreedom Is a positive integer indicating the number of degrees of freedom to characterize the distribution.
         */
        t_Inv_2T(probability: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, degFreedom: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the tangent of an angle.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the angle in radians for which you want the tangent. Degrees * PI()/180 = radians.
         */
        tan(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the hyperbolic tangent of a number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is any real number.
         */
        tanh(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a value to text in a specific number format.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param value Is a number, a formula that evaluates to a numeric value, or a reference to a cell containing a numeric value.
         * @param formatText Is a number format in text form from the Category box on the Number tab in the Format Cells dialog box (not General).
         */
        text(value: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, formatText: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Converts hours, minutes, and seconds given as numbers to an Excel serial number, formatted with a time format.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param hour Is a number from 0 to 23 representing the hour.
         * @param minute Is a number from 0 to 59 representing the minute.
         * @param second Is a number from 0 to 59 representing the second.
         */
        time(hour: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, minute: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, second: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a text time to an Excel serial number for a time, a number from 0 (12:00:00 AM) to 0.999988426 (11:59:59 PM). Format the number with a time format after entering the formula.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param timeText Is a text string that gives a time in any one of the Microsoft Excel time formats (date information in the string is ignored).
         */
        timevalue(timeText: string | number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the current date formatted as a date.
         *
         * [Api set: ExcelApi 1.2]
         */
        today(): FunctionResult<number>;
        /**
         *
         * Removes all spaces from a text string except for single spaces between words.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the text from which you want spaces removed.
         */
        trim(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns the mean of the interior portion of a set of data values.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param array Is the range or array of values to trim and average.
         * @param percent Is the fractional number of data points to exclude from the top and bottom of the data set.
         */
        trimMean(array: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, percent: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the logical value TRUE.
         *
         * [Api set: ExcelApi 1.2]
         */
        true(): FunctionResult<boolean>;
        /**
         *
         * Truncates a number to an integer by removing the decimal, or fractional, part of the number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the number you want to truncate.
         * @param numDigits Is a number specifying the precision of the truncation, 0 (zero) if omitted.
         */
        trunc(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, numDigits?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns an integer representing the data type of a value: number = 1; text = 2; logical value = 4; error value = 16; array = 64.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param value Can be any value.
         */
        type(value: boolean | string | number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a number to text, using currency format.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is a number, a reference to a cell containing a number, or a formula that evaluates to a number.
         * @param decimals Is the number of digits to the right of the decimal point.
         */
        usdollar(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, decimals?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns the Unicode character referenced by the given numeric value.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param number Is the Unicode number representing a character.
         */
        unichar(number: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Returns the number (code point) corresponding to the first character of the text.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the character that you want the Unicode value of.
         */
        unicode(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Converts a text string to all uppercase letters.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the text you want converted to uppercase, a reference or a text string.
         */
        upper(text: string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<string>;
        /**
         *
         * Looks for a value in the leftmost column of a table, and then returns a value in the same row from a column you specify. By default, the table must be sorted in an ascending order.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param lookupValue Is the value to be found in the first column of the table, and can be a value, a reference, or a text string.
         * @param tableArray Is a table of text, numbers, or logical values, in which data is retrieved. tableArray can be a reference to a range or a range name.
         * @param colIndexNum Is the column number in tableArray from which the matching value should be returned. The first column of values in the table is column 1.
         * @param rangeLookup Is a logical value: to find the closest match in the first column (sorted in ascending order) = TRUE or omitted; find an exact match = FALSE.
         */
        vlookup(lookupValue: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, tableArray: Excel.Range | number | Excel.RangeReference | Excel.FunctionResult<any>, colIndexNum: Excel.Range | number | Excel.RangeReference | Excel.FunctionResult<any>, rangeLookup?: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number | string | boolean>;
        /**
         *
         * Converts a text string that represents a number to a number.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param text Is the text enclosed in quotation marks or a reference to a cell containing the text you want to convert.
         */
        value(text: string | boolean | number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Estimates variance based on a sample, including logical values and text. Text and the logical value FALSE have the value 0; the logical value TRUE has the value 1.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 value arguments corresponding to a sample of a population.
         */
        varA(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Calculates variance based on the entire population, including logical values and text. Text and the logical value FALSE have the value 0; the logical value TRUE has the value 1.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 value arguments corresponding to a population.
         */
        varPA(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Calculates variance based on the entire population (ignores logical values and text in the population).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numeric arguments corresponding to a population.
         */
        var_P(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Estimates variance based on a sample (ignores logical values and text in the sample).
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 255 numeric arguments corresponding to a sample of a population.
         */
        var_S(...values: Array<number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<number>;
        /**
         *
         * Returns the depreciation of an asset for any period you specify, including partial periods, using the double-declining balance method or some other method you specify.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param cost Is the initial cost of the asset.
         * @param salvage Is the salvage value at the end of the life of the asset.
         * @param life Is the number of periods over which the asset is being depreciated (sometimes called the useful life of the asset).
         * @param startPeriod Is the starting period for which you want to calculate the depreciation, in the same units as Life.
         * @param endPeriod Is the ending period for which you want to calculate the depreciation, in the same units as Life.
         * @param factor Is the rate at which the balance declines, 2 (double-declining balance) if omitted.
         * @param noSwitch Switch to straight-line depreciation when depreciation is greater than the declining balance = FALSE or omitted; do not switch = TRUE.
         */
        vdb(cost: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, salvage: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, life: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, startPeriod: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, endPeriod: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, factor?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, noSwitch?: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the week number in the year.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param serialNumber Is the date-time code used by Microsoft Excel for date and time calculation.
         * @param returnType Is a number (1 or 2) that determines the type of the return value.
         */
        weekNum(serialNumber: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, returnType?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns a number from 1 to 7 identifying the day of the week of a date.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param serialNumber Is a number that represents a date.
         * @param returnType Is a number: for Sunday=1 through Saturday=7, use 1; for Monday=1 through Sunday=7, use 2; for Monday=0 through Sunday=6, use 3.
         */
        weekday(serialNumber: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, returnType?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the Weibull distribution.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param x Is the value at which to evaluate the function, a nonnegative number.
         * @param alpha Is a parameter to the distribution, a positive number.
         * @param beta Is a parameter to the distribution, a positive number.
         * @param cumulative Is a logical value: for the cumulative distribution function, use TRUE; for the probability mass function, use FALSE.
         */
        weibull_Dist(x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, alpha: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, beta: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, cumulative: boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the serial number of the date before or after a specified number of workdays.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param startDate Is a serial date number that represents the start date.
         * @param days Is the number of nonweekend and non-holiday days before or after startDate.
         * @param holidays Is an optional array of one or more serial date numbers to exclude from the working calendar, such as state and federal holidays and floating holidays.
         */
        workDay(startDate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, days: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, holidays?: number | string | Excel.Range | boolean | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the serial number of the date before or after a specified number of workdays with custom weekend parameters.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param startDate Is a serial date number that represents the start date.
         * @param days Is the number of nonweekend and non-holiday days before or after startDate.
         * @param weekend Is a number or string specifying when weekends occur.
         * @param holidays Is an optional array of one or more serial date numbers to exclude from the working calendar, such as state and federal holidays and floating holidays.
         */
        workDay_Intl(startDate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, days: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, weekend?: number | string | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, holidays?: number | string | Excel.Range | boolean | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the internal rate of return for a schedule of cash flows.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values Is a series of cash flows that correspond to a schedule of payments in dates.
         * @param dates Is a schedule of payment dates that corresponds to the cash flow payments.
         * @param guess Is a number that you guess is close to the result of XIRR.
         */
        xirr(values: number | string | Excel.Range | boolean | Excel.RangeReference | Excel.FunctionResult<any>, dates: number | string | Excel.Range | boolean | Excel.RangeReference | Excel.FunctionResult<any>, guess?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the net present value for a schedule of cash flows.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param rate Is the discount rate to apply to the cash flows.
         * @param values Is a series of cash flows that correspond to a schedule of payments in dates.
         * @param dates Is a schedule of payment dates that corresponds to the cash flow payments.
         */
        xnpv(rate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, values: number | string | Excel.Range | boolean | Excel.RangeReference | Excel.FunctionResult<any>, dates: number | string | Excel.Range | boolean | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns a logical 'Exclusive Or' of all arguments.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param values List of parameters, whose elements are 1 to 254 conditions you want to test that can be either TRUE or FALSE and can be logical values, arrays, or references.
         */
        xor(...values: Array<boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>>): FunctionResult<boolean>;
        /**
         *
         * Returns the year of a date, an integer in the range 1900 - 9999.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param serialNumber Is a number in the date-time code used by Microsoft Excel.
         */
        year(serialNumber: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the year fraction representing the number of whole days between start_date and end_date.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param startDate Is a serial date number that represents the start date.
         * @param endDate Is a serial date number that represents the end date.
         * @param basis Is the type of day count basis to use.
         */
        yearFrac(startDate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, endDate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the yield on a security that pays periodic interest.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param rate Is the security's annual coupon rate.
         * @param pr Is the security's price per $100 face value.
         * @param redemption Is the security's redemption value per $100 face value.
         * @param frequency Is the number of coupon payments per year.
         * @param basis Is the type of day count basis to use.
         */
        yield(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, rate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pr: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, redemption: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, frequency: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the annual yield for a discounted security. For example, a treasury bill.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param pr Is the security's price per $100 face value.
         * @param redemption Is the security's redemption value per $100 face value.
         * @param basis Is the type of day count basis to use.
         */
        yieldDisc(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pr: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, redemption: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the annual yield of a security that pays interest at maturity.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param settlement Is the security's settlement date, expressed as a serial date number.
         * @param maturity Is the security's maturity date, expressed as a serial date number.
         * @param issue Is the security's issue date, expressed as a serial date number.
         * @param rate Is the security's interest rate at date of issue.
         * @param pr Is the security's price per $100 face value.
         * @param basis Is the type of day count basis to use.
         */
        yieldMat(settlement: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, maturity: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, issue: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, rate: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, pr: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, basis?: number | string | boolean | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
         *
         * Returns the one-tailed P-value of a z-test.
         *
         * [Api set: ExcelApi 1.2]
         *
         * @param array Is the array or range of data against which to test X.
         * @param x Is the value to test.
         * @param sigma Is the population (known) standard deviation. If omitted, the sample standard deviation is used.
         */
        z_Test(array: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, x: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>, sigma?: number | Excel.Range | Excel.RangeReference | Excel.FunctionResult<any>): FunctionResult<number>;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Excel.Functions object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Excel.Interfaces.FunctionsData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): {
            [key: string]: string;
        };
    }
    enum ErrorCodes {
        accessDenied = "AccessDenied",
        apiNotFound = "ApiNotFound",
        conflict = "Conflict",
        generalException = "GeneralException",
        insertDeleteConflict = "InsertDeleteConflict",
        invalidArgument = "InvalidArgument",
        invalidBinding = "InvalidBinding",
        invalidOperation = "InvalidOperation",
        invalidReference = "InvalidReference",
        invalidSelection = "InvalidSelection",
        itemAlreadyExists = "ItemAlreadyExists",
        itemNotFound = "ItemNotFound",
        nonBlankCellOffSheet = "NonBlankCellOffSheet",
        notImplemented = "NotImplemented",
        unsupportedOperation = "UnsupportedOperation",
        invalidOperationInCellEditMode = "InvalidOperationInCellEditMode"
    }
    module Interfaces {
        /**
        * Provides ways to load properties of only a subset of members of a collection.
        */
        interface CollectionLoadOptions {
            /**
            * Specify the number of items in the queried collection to be included in the result.
            */
            $top?: number;
            /**
            * Specify the number of items in the collection that are to be skipped and not included in the result. If top is specified, the selection of result will start after skipping the specified number of items.
            */
            $skip?: number;
        }
        /** An interface for updating data on the Runtime object, for use in "runtime.set({ ... })". */
        interface RuntimeUpdateData {
            /**
             *
             * Toggle JavaScript events in the current task pane or content add-in.
             *
             * [Api set: ExcelApi 1.8]
             */
            enableEvents?: boolean;
        }
        /** An interface for updating data on the Application object, for use in "application.set({ ... })". */
        interface ApplicationUpdateData {
            /**
            *
            * Returns the Iterative Calculation settings.
            On Excel for Windows and Excel for Mac, the settings will apply to the Excel Application.
            On Excel Online and Excel for other platforms, the settings will apply to the active workbook.
            *
            * [Api set: ExcelApi 1.9]
            */
            iterativeCalculation?: Excel.Interfaces.IterativeCalculationUpdateData;
            /**
             *
             * Returns the calculation mode used in the workbook, as defined by the constants in Excel.CalculationMode. Possible values are: `Automatic`, where Excel controls recalculation; `AutomaticExceptTables`, where Excel controls recalculation but ignores changes in tables; `Manual`, where calculation is done when the user requests it.
             *
             * [Api set: ExcelApi 1.1 for get, 1.8 for set]
             */
            calculationMode?: Excel.CalculationMode | "Automatic" | "AutomaticExceptTables" | "Manual";
        }
        /** An interface for updating data on the IterativeCalculation object, for use in "iterativeCalculation.set({ ... })". */
        interface IterativeCalculationUpdateData {
            /**
             *
             * True if Excel will use iteration to resolve circular references.
             *
             * [Api set: ExcelApi 1.9]
             */
            enabled?: boolean;
            /**
             *
             * Returns or sets the maximum amount of change between each iteration as Excel resolves circular references.
             *
             * [Api set: ExcelApi 1.9]
             */
            maxChange?: number;
            /**
             *
             * Returns or sets the maximum number of iterations that Excel can use to resolve a circular reference.
             *
             * [Api set: ExcelApi 1.9]
             */
            maxIteration?: number;
        }
        /** An interface for updating data on the Workbook object, for use in "workbook.set({ ... })". */
        interface WorkbookUpdateData {
            /**
            *
            * Gets the workbook properties.
            *
            * [Api set: ExcelApi 1.7]
            */
            properties?: Excel.Interfaces.DocumentPropertiesUpdateData;
            /**
             *
             * True if all charts in the workbook are tracking the actual data points to which they are attached.
            False if the charts track the index of the data points.
             *
             * [Api set: ExcelApi 1.9]
             */
            chartDataPointTrack?: boolean;
            /**
             *
             * Specifies whether or not changes have been made since the workbook was last saved.
            You can set this property to true if you want to close a modified workbook without either saving it or being prompted to save it.
             *
             * [Api set: ExcelApi 1.9]
             */
            isDirty?: boolean;
            /**
             *
             * True if calculations in this workbook will be done using only the precision of the numbers as they're displayed.
            Data will permanently lose accuracy when switching this property from false to true.
             *
             * [Api set: ExcelApi 1.9]
             */
            usePrecisionAsDisplayed?: boolean;
        }
        /** An interface for updating data on the Worksheet object, for use in "worksheet.set({ ... })". */
        interface WorksheetUpdateData {
            /**
            *
            * Gets the PageLayout object of the worksheet.
            *
            * [Api set: ExcelApi 1.9]
            */
            pageLayout?: Excel.Interfaces.PageLayoutUpdateData;
            /**
             *
             * Gets or sets the enableCalculation property of the worksheet.
            True if Excel recalculates the worksheet when necessary. False if Excel doesn't recalculate the sheet.
             *
             * [Api set: ExcelApi 1.9]
             */
            enableCalculation?: boolean;
            /**
             *
             * The display name of the worksheet.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: string;
            /**
             *
             * The zero-based position of the worksheet within the workbook.
             *
             * [Api set: ExcelApi 1.1]
             */
            position?: number;
            /**
             *
             * Gets or sets the worksheet's gridlines flag.
            This flag determines whether gridlines are visible to the user.
             *
             * [Api set: ExcelApi 1.8]
             */
            showGridlines?: boolean;
            /**
             *
             * Gets or sets the worksheet's headings flag.
            This flag determines whether headings are visible to the user.
             *
             * [Api set: ExcelApi 1.8]
             */
            showHeadings?: boolean;
            /**
             *
             * Returns or sets the standard (default) width of all the columns in the worksheet.
            One unit of column width is equal to the width of one character in the Normal style. For proportional fonts, the width of the character 0 (zero) is used.
             *
             * [Api set: ExcelApi 1.7]
             */
            standardWidth?: number;
            /**
             *
             * Gets or sets the worksheet tab color.
            When retrieving the tab color, if the worksheet is invisible, the value will be null. If the worksheet is visible but the tab color is set to auto, an empty string will be returned. Otherwise, the property will be set to a color, in the form "#123456"
            When setting the color, use an empty-string to set an "auto" color, or a real color otherwise.
             *
             * [Api set: ExcelApi 1.7]
             */
            tabColor?: string;
            /**
             *
             * The Visibility of the worksheet.
             *
             * [Api set: ExcelApi 1.1 for reading visibility; 1.2 for setting it.]
             */
            visibility?: Excel.SheetVisibility | "Visible" | "Hidden" | "VeryHidden";
        }
        /** An interface for updating data on the WorksheetCollection object, for use in "worksheetCollection.set({ ... })". */
        interface WorksheetCollectionUpdateData {
            items?: Excel.Interfaces.WorksheetData[];
        }
        /** An interface for updating data on the Range object, for use in "range.set({ ... })". */
        interface RangeUpdateData {
            /**
            *
            * Returns a data validation object.
            *
            * [Api set: ExcelApi 1.8]
            */
            dataValidation?: Excel.Interfaces.DataValidationUpdateData;
            /**
            *
            * Returns a format object, encapsulating the range's font, fill, borders, alignment, and other properties.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.RangeFormatUpdateData;
            /**
             *
             * Represents if all columns of the current range are hidden.
             *
             * [Api set: ExcelApi 1.2]
             */
            columnHidden?: boolean;
            /**
             *
             * Represents the formula in A1-style notation.
            When setting formulas to a range, the value argument can be either a single value (a string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            formulas?: any[][];
            /**
             *
             * Represents the formula in A1-style notation, in the user's language and number-formatting locale.  For example, the English "=SUM(A1, 1.5)" formula would become "=SUMME(A1; 1,5)" in German.
            When setting formulas to a range, the value argument can be either a single value (a string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            formulasLocal?: any[][];
            /**
             *
             * Represents the formula in R1C1-style notation.
            When setting formulas to a range, the value argument can be either a single value (a string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.2]
             */
            formulasR1C1?: any[][];
            /**
             *
             * Represents the hyperlink for the current range.
             *
             * [Api set: ExcelApi 1.7]
             */
            hyperlink?: Excel.RangeHyperlink;
            /**
             *
             * Represents Excel's number format code for the given range.
            When setting number format to a range, the value argument can be either a single value (string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            numberFormat?: any[][];
            /**
             *
             * Represents Excel's number format code for the given range as a string in the language of the user.
            When setting number format local to a range, the value argument can be either a single value (string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.7]
             */
            numberFormatLocal?: any[][];
            /**
             *
             * Represents if all rows of the current range are hidden.
             *
             * [Api set: ExcelApi 1.2]
             */
            rowHidden?: boolean;
            /**
             *
             * Represents the style of the current range.
            If the styles of the cells are inconsistent, null will be returned.
            For custom styles, the style name will be returned. For built-in styles, a string representing a value in the BuiltInStyle enum will be returned.
             *
             * [Api set: ExcelApi 1.7]
             */
            style?: string;
            /**
             *
             * Represents the raw values of the specified range. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
            When setting values to a range, the value argument can be either a single value (string, number or boolean) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            values?: any[][];
        }
        /** An interface for updating data on the RangeAreas object, for use in "rangeAreas.set({ ... })". */
        interface RangeAreasUpdateData {
            /**
            *
            * Returns a dataValidation object for all ranges in the RangeAreas.
            *
            * [Api set: ExcelApi 1.9]
            */
            dataValidation?: Excel.Interfaces.DataValidationUpdateData;
            /**
            *
            * Returns a rangeFormat object, encapsulating the the font, fill, borders, alignment, and other properties for all ranges in the RangeAreas object.
            *
            * [Api set: ExcelApi 1.9]
            */
            format?: Excel.Interfaces.RangeFormatUpdateData;
            /**
             *
             * Represents the style for all ranges in this RangeAreas object.
            If the styles of the cells are inconsistent, null will be returned.
            For custom styles, the style name will be returned. For built-in styles, a string representing a value in the BuiltInStyle enum will be returned.
             *
             * [Api set: ExcelApi 1.9]
             */
            style?: string;
        }
        /** An interface for updating data on the RangeView object, for use in "rangeView.set({ ... })". */
        interface RangeViewUpdateData {
            /**
             *
             * Represents the formula in A1-style notation.
             *
             * [Api set: ExcelApi 1.3]
             */
            formulas?: any[][];
            /**
             *
             * Represents the formula in A1-style notation, in the user's language and number-formatting locale.  For example, the English "=SUM(A1, 1.5)" formula would become "=SUMME(A1; 1,5)" in German.
             *
             * [Api set: ExcelApi 1.3]
             */
            formulasLocal?: any[][];
            /**
             *
             * Represents the formula in R1C1-style notation.
             *
             * [Api set: ExcelApi 1.3]
             */
            formulasR1C1?: any[][];
            /**
             *
             * Represents Excel's number format code for the given cell.
             *
             * [Api set: ExcelApi 1.3]
             */
            numberFormat?: any[][];
            /**
             *
             * Represents the raw values of the specified range view. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
             *
             * [Api set: ExcelApi 1.3]
             */
            values?: any[][];
        }
        /** An interface for updating data on the RangeViewCollection object, for use in "rangeViewCollection.set({ ... })". */
        interface RangeViewCollectionUpdateData {
            items?: Excel.Interfaces.RangeViewData[];
        }
        /** An interface for updating data on the SettingCollection object, for use in "settingCollection.set({ ... })". */
        interface SettingCollectionUpdateData {
            items?: Excel.Interfaces.SettingData[];
        }
        /** An interface for updating data on the Setting object, for use in "setting.set({ ... })". */
        interface SettingUpdateData {
            /**
             *
             * Represents the value stored for this setting.
             *
             * [Api set: ExcelApi 1.4]
             */
            value?: any;
        }
        /** An interface for updating data on the NamedItemCollection object, for use in "namedItemCollection.set({ ... })". */
        interface NamedItemCollectionUpdateData {
            items?: Excel.Interfaces.NamedItemData[];
        }
        /** An interface for updating data on the NamedItem object, for use in "namedItem.set({ ... })". */
        interface NamedItemUpdateData {
            /**
             *
             * Represents the comment associated with this name.
             *
             * [Api set: ExcelApi 1.4]
             */
            comment?: string;
            /**
             *
             * Gets or sets the formula of the named item.  Formula always starts with a '=' sign.
             *
             * [Api set: ExcelApi 1.7]
             */
            formula?: any;
            /**
             *
             * Specifies whether the object is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
        }
        /** An interface for updating data on the BindingCollection object, for use in "bindingCollection.set({ ... })". */
        interface BindingCollectionUpdateData {
            items?: Excel.Interfaces.BindingData[];
        }
        /** An interface for updating data on the TableCollection object, for use in "tableCollection.set({ ... })". */
        interface TableCollectionUpdateData {
            items?: Excel.Interfaces.TableData[];
        }
        /** An interface for updating data on the TableScopedCollection object, for use in "tableScopedCollection.set({ ... })". */
        interface TableScopedCollectionUpdateData {
            items?: Excel.Interfaces.TableData[];
        }
        /** An interface for updating data on the Table object, for use in "table.set({ ... })". */
        interface TableUpdateData {
            /**
             *
             * Indicates whether the first column contains special formatting.
             *
             * [Api set: ExcelApi 1.3]
             */
            highlightFirstColumn?: boolean;
            /**
             *
             * Indicates whether the last column contains special formatting.
             *
             * [Api set: ExcelApi 1.3]
             */
            highlightLastColumn?: boolean;
            /**
             *
             * Name of the table.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: string;
            /**
             *
             * Indicates whether the columns show banded formatting in which odd columns are highlighted differently from even ones to make reading the table easier.
             *
             * [Api set: ExcelApi 1.3]
             */
            showBandedColumns?: boolean;
            /**
             *
             * Indicates whether the rows show banded formatting in which odd rows are highlighted differently from even ones to make reading the table easier.
             *
             * [Api set: ExcelApi 1.3]
             */
            showBandedRows?: boolean;
            /**
             *
             * Indicates whether the filter buttons are visible at the top of each column header. Setting this is only allowed if the table contains a header row.
             *
             * [Api set: ExcelApi 1.3]
             */
            showFilterButton?: boolean;
            /**
             *
             * Indicates whether the header row is visible or not. This value can be set to show or remove the header row.
             *
             * [Api set: ExcelApi 1.1]
             */
            showHeaders?: boolean;
            /**
             *
             * Indicates whether the total row is visible or not. This value can be set to show or remove the total row.
             *
             * [Api set: ExcelApi 1.1]
             */
            showTotals?: boolean;
            /**
             *
             * Constant value that represents the Table style. Possible values are: "TableStyleLight1" through "TableStyleLight21", "TableStyleMedium1" through "TableStyleMedium28", "TableStyleStyleDark1" through "TableStyleStyleDark11". A custom user-defined style present in the workbook can also be specified.
             *
             * [Api set: ExcelApi 1.1]
             */
            style?: string;
        }
        /** An interface for updating data on the TableColumnCollection object, for use in "tableColumnCollection.set({ ... })". */
        interface TableColumnCollectionUpdateData {
            items?: Excel.Interfaces.TableColumnData[];
        }
        /** An interface for updating data on the TableColumn object, for use in "tableColumn.set({ ... })". */
        interface TableColumnUpdateData {
            /**
             *
             * Represents the name of the table column.
             *
             * [Api set: ExcelApi 1.1 for getting the name; 1.4 for setting it.]
             */
            name?: string;
            /**
             *
             * Represents the raw values of the specified range. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
             *
             * [Api set: ExcelApi 1.1]
             */
            values?: any[][];
        }
        /** An interface for updating data on the TableRowCollection object, for use in "tableRowCollection.set({ ... })". */
        interface TableRowCollectionUpdateData {
            items?: Excel.Interfaces.TableRowData[];
        }
        /** An interface for updating data on the TableRow object, for use in "tableRow.set({ ... })". */
        interface TableRowUpdateData {
            /**
             *
             * Represents the raw values of the specified range. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
             *
             * [Api set: ExcelApi 1.1]
             */
            values?: any[][];
        }
        /** An interface for updating data on the DataValidation object, for use in "dataValidation.set({ ... })". */
        interface DataValidationUpdateData {
            /**
             *
             * Error alert when user enters invalid data.
             *
             * [Api set: ExcelApi 1.8]
             */
            errorAlert?: Excel.DataValidationErrorAlert;
            /**
             *
             * Ignore blanks: no data validation will be performed on blank cells, it defaults to true.
             *
             * [Api set: ExcelApi 1.8]
             */
            ignoreBlanks?: boolean;
            /**
             *
             * Prompt when users select a cell.
             *
             * [Api set: ExcelApi 1.8]
             */
            prompt?: Excel.DataValidationPrompt;
            /**
             *
             * Data validation rule that contains different type of data validation criteria.
             *
             * [Api set: ExcelApi 1.8]
             */
            rule?: Excel.DataValidationRule;
        }
        /** An interface for updating data on the RangeFormat object, for use in "rangeFormat.set({ ... })". */
        interface RangeFormatUpdateData {
            /**
            *
            * Collection of border objects that apply to the overall range.
            *
            * [Api set: ExcelApi 1.1]
            */
            borders?: Excel.Interfaces.RangeBorderCollectionUpdateData;
            /**
            *
            * Returns the fill object defined on the overall range.
            *
            * [Api set: ExcelApi 1.1]
            */
            fill?: Excel.Interfaces.RangeFillUpdateData;
            /**
            *
            * Returns the font object defined on the overall range.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.RangeFontUpdateData;
            /**
            *
            * Returns the format protection object for a range.
            *
            * [Api set: ExcelApi 1.2]
            */
            protection?: Excel.Interfaces.FormatProtectionUpdateData;
            /**
             *
             * Indicates if text is automatically indented when text alignment is set to equal distribution.
             *
             * [Api set: ExcelApi 1.9]
             */
            autoIndent?: boolean;
            /**
             *
             * Gets or sets the width of all colums within the range. If the column widths are not uniform, null will be returned.
             *
             * [Api set: ExcelApi 1.2]
             */
            columnWidth?: number;
            /**
             *
             * Represents the horizontal alignment for the specified object. See Excel.HorizontalAlignment for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            horizontalAlignment?: Excel.HorizontalAlignment | "General" | "Left" | "Center" | "Right" | "Fill" | "Justify" | "CenterAcrossSelection" | "Distributed";
            /**
             *
             * An integer from 0 to 250 that indicates the indent level.
             *
             * [Api set: ExcelApi 1.9]
             */
            indentLevel?: number;
            /**
             *
             * The reading order for the range.
             *
             * [Api set: ExcelApi 1.9]
             */
            readingOrder?: Excel.ReadingOrder | "Context" | "LeftToRight" | "RightToLeft";
            /**
             *
             * Gets or sets the height of all rows in the range. If the row heights are not uniform, null will be returned.
             *
             * [Api set: ExcelApi 1.2]
             */
            rowHeight?: number;
            /**
             *
             * Indicates if text automatically shrinks to fit in the available column width.
             *
             * [Api set: ExcelApi 1.9]
             */
            shrinkToFit?: boolean;
            /**
             *
             * Gets or sets the text orientation of all the cells within the range.
            The text orientation should be an integer either from -90 to 90, or 180 for vertically-oriented text.
            If the orientation within a range are not uniform, then null will be returned.
             *
             * [Api set: ExcelApi 1.7]
             */
            textOrientation?: number;
            /**
             *
             * Determines if the row height of the Range object equals the standard height of the sheet.
            Returns True if the row height of the Range object equals the standard height of the sheet.
            Returns Null if the range contains more than one row and the rows aren't all the same height.
            Returns False otherwise.
             *
             * [Api set: ExcelApi 1.7]
             */
            useStandardHeight?: boolean;
            /**
             *
             * Indicates whether the column width of the Range object equals the standard width of the sheet.
            Returns True if the column width of the Range object equals the standard width of the sheet.
            Returns Null if the range contains more than one column and the columns aren't all the same height.
            Returns False otherwise.
             *
             * [Api set: ExcelApi 1.7]
             */
            useStandardWidth?: boolean;
            /**
             *
             * Represents the vertical alignment for the specified object. See Excel.VerticalAlignment for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            verticalAlignment?: Excel.VerticalAlignment | "Top" | "Center" | "Bottom" | "Justify" | "Distributed";
            /**
             *
             * Indicates if Excel wraps the text in the object. A null value indicates that the entire range doesn't have uniform wrap setting
             *
             * [Api set: ExcelApi 1.1]
             */
            wrapText?: boolean;
        }
        /** An interface for updating data on the FormatProtection object, for use in "formatProtection.set({ ... })". */
        interface FormatProtectionUpdateData {
            /**
             *
             * Indicates if Excel hides the formula for the cells in the range. A null value indicates that the entire range doesn't have uniform formula hidden setting.
             *
             * [Api set: ExcelApi 1.2]
             */
            formulaHidden?: boolean;
            /**
             *
             * Indicates if Excel locks the cells in the object. A null value indicates that the entire range doesn't have uniform lock setting.
             *
             * [Api set: ExcelApi 1.2]
             */
            locked?: boolean;
        }
        /** An interface for updating data on the RangeFill object, for use in "rangeFill.set({ ... })". */
        interface RangeFillUpdateData {
            /**
             *
             * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange")
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: string;
            /**
             *
             * Gets or sets the pattern of a Range. See Excel.FillPattern for details. LinearGradient and RectangularGradient are not supported.
            A null value indicates that the entire range doesn't have uniform pattern setting.
             *
             * [Api set: ExcelApi 1.9]
             */
            pattern?: Excel.FillPattern | "None" | "Solid" | "Gray50" | "Gray75" | "Gray25" | "Horizontal" | "Vertical" | "Down" | "Up" | "Checker" | "SemiGray75" | "LightHorizontal" | "LightVertical" | "LightDown" | "LightUp" | "Grid" | "CrissCross" | "Gray16" | "Gray8" | "LinearGradient" | "RectangularGradient";
            /**
             *
             * Sets HTML color code representing the color of the Range pattern, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            Gets HTML color code representing the color of the Range pattern, of the form #RRGGBB (e.g. "FFA500").
             *
             * [Api set: ExcelApi 1.9]
             */
            patternColor?: string;
            /**
             *
             * Returns or sets a double that lightens or darkens a pattern color for Range Fill, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            If the pattern tintAndShades are not uniform, null will be returned.
             *
             * [Api set: ExcelApi 1.9]
             */
            patternTintAndShade?: number;
            /**
             *
             * Returns or sets a double that lightens or darkens a color for Range Fill, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            If the tintAndShades are not uniform, null will be returned.
             *
             * [Api set: ExcelApi 1.9]
             */
            tintAndShade?: number;
        }
        /** An interface for updating data on the RangeBorder object, for use in "rangeBorder.set({ ... })". */
        interface RangeBorderUpdateData {
            /**
             *
             * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: string;
            /**
             *
             * One of the constants of line style specifying the line style for the border. See Excel.BorderLineStyle for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            style?: Excel.BorderLineStyle | "None" | "Continuous" | "Dash" | "DashDot" | "DashDotDot" | "Dot" | "Double" | "SlantDashDot";
            /**
             *
             * Returns or sets a double that lightens or darkens a color for Range Border, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            A null value indicates that the border doesn't have uniform tintAndShade setting.
             *
             * [Api set: ExcelApi 1.9]
             */
            tintAndShade?: number;
            /**
             *
             * Specifies the weight of the border around a range. See Excel.BorderWeight for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            weight?: Excel.BorderWeight | "Hairline" | "Thin" | "Medium" | "Thick";
        }
        /** An interface for updating data on the RangeBorderCollection object, for use in "rangeBorderCollection.set({ ... })". */
        interface RangeBorderCollectionUpdateData {
            /**
             *
             * Returns or sets a double that lightens or darkens a color for Range Borders, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            A null value indicates that the entire border collections don't have uniform tintAndShade setting.
             *
             * [Api set: ExcelApi 1.9]
             */
            tintAndShade?: number;
            items?: Excel.Interfaces.RangeBorderData[];
        }
        /** An interface for updating data on the RangeFont object, for use in "rangeFont.set({ ... })". */
        interface RangeFontUpdateData {
            /**
             *
             * Represents the bold status of font.
             *
             * [Api set: ExcelApi 1.1]
             */
            bold?: boolean;
            /**
             *
             * HTML color code representation of the text color. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: string;
            /**
             *
             * Represents the italic status of the font.
             *
             * [Api set: ExcelApi 1.1]
             */
            italic?: boolean;
            /**
             *
             * Font name (e.g. "Calibri")
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: string;
            /**
             *
             * Font size.
             *
             * [Api set: ExcelApi 1.1]
             */
            size?: number;
            /**
             *
             * Represents the strikethrough status of font. A null value indicates that the entire range doesn't have uniform Strikethrough setting.
             *
             * [Api set: ExcelApi 1.9]
             */
            strikethrough?: boolean;
            /**
             *
             * Represents the Subscript status of font.
            Returns True if all the fonts of the range are Subscript.
            Returns False if all the fonts of the range are Superscript or normal (neither Superscript, nor Subscript).
            Returns Null otherwise.
             *
             * [Api set: ExcelApi 1.9]
             */
            subscript?: boolean;
            /**
             *
             * Represents the Superscript status of font.
            Returns True if all the fonts of the range are Superscript.
            Returns False if all the fonts of the range are Subscript or normal (neither Superscript, nor Subscript).
            Returns Null otherwise.
             *
             * [Api set: ExcelApi 1.9]
             */
            superscript?: boolean;
            /**
             *
             * Returns or sets a double that lightens or darkens a color for Range Font, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            A null value indicates that the entire range doesn't have uniform font tintAndShade setting.
             *
             * [Api set: ExcelApi 1.9]
             */
            tintAndShade?: number;
            /**
             *
             * Type of underline applied to the font. See Excel.RangeUnderlineStyle for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            underline?: Excel.RangeUnderlineStyle | "None" | "Single" | "Double" | "SingleAccountant" | "DoubleAccountant";
        }
        /** An interface for updating data on the ChartCollection object, for use in "chartCollection.set({ ... })". */
        interface ChartCollectionUpdateData {
            items?: Excel.Interfaces.ChartData[];
        }
        /** An interface for updating data on the Chart object, for use in "chart.set({ ... })". */
        interface ChartUpdateData {
            /**
            *
            * Represents chart axes.
            *
            * [Api set: ExcelApi 1.1]
            */
            axes?: Excel.Interfaces.ChartAxesUpdateData;
            /**
            *
            * Represents the datalabels on the chart.
            *
            * [Api set: ExcelApi 1.1]
            */
            dataLabels?: Excel.Interfaces.ChartDataLabelsUpdateData;
            /**
            *
            * Encapsulates the format properties for the chart area.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartAreaFormatUpdateData;
            /**
            *
            * Represents the legend for the chart.
            *
            * [Api set: ExcelApi 1.1]
            */
            legend?: Excel.Interfaces.ChartLegendUpdateData;
            /**
            *
            * Encapsulates the options for a pivot chart.
            *
            * [Api set: ExcelApi 1.9]
            */
            pivotOptions?: Excel.Interfaces.ChartPivotOptionsUpdateData;
            /**
            *
            * Represents the plotArea for the chart.
            *
            * [Api set: ExcelApi 1.8]
            */
            plotArea?: Excel.Interfaces.ChartPlotAreaUpdateData;
            /**
            *
            * Represents the title of the specified chart, including the text, visibility, position, and formatting of the title.
            *
            * [Api set: ExcelApi 1.1]
            */
            title?: Excel.Interfaces.ChartTitleUpdateData;
            /**
             *
             * Returns or sets a ChartCategoryLabelLevel enumeration constant referring to
            the level of where the category labels are being sourced from. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            categoryLabelLevel?: number;
            /**
             *
             * Represents the type of the chart. See Excel.ChartType for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            chartType?: Excel.ChartType | "Invalid" | "ColumnClustered" | "ColumnStacked" | "ColumnStacked100" | "3DColumnClustered" | "3DColumnStacked" | "3DColumnStacked100" | "BarClustered" | "BarStacked" | "BarStacked100" | "3DBarClustered" | "3DBarStacked" | "3DBarStacked100" | "LineStacked" | "LineStacked100" | "LineMarkers" | "LineMarkersStacked" | "LineMarkersStacked100" | "PieOfPie" | "PieExploded" | "3DPieExploded" | "BarOfPie" | "XYScatterSmooth" | "XYScatterSmoothNoMarkers" | "XYScatterLines" | "XYScatterLinesNoMarkers" | "AreaStacked" | "AreaStacked100" | "3DAreaStacked" | "3DAreaStacked100" | "DoughnutExploded" | "RadarMarkers" | "RadarFilled" | "Surface" | "SurfaceWireframe" | "SurfaceTopView" | "SurfaceTopViewWireframe" | "Bubble" | "Bubble3DEffect" | "StockHLC" | "StockOHLC" | "StockVHLC" | "StockVOHLC" | "CylinderColClustered" | "CylinderColStacked" | "CylinderColStacked100" | "CylinderBarClustered" | "CylinderBarStacked" | "CylinderBarStacked100" | "CylinderCol" | "ConeColClustered" | "ConeColStacked" | "ConeColStacked100" | "ConeBarClustered" | "ConeBarStacked" | "ConeBarStacked100" | "ConeCol" | "PyramidColClustered" | "PyramidColStacked" | "PyramidColStacked100" | "PyramidBarClustered" | "PyramidBarStacked" | "PyramidBarStacked100" | "PyramidCol" | "3DColumn" | "Line" | "3DLine" | "3DPie" | "Pie" | "XYScatter" | "3DArea" | "Area" | "Doughnut" | "Radar" | "Histogram" | "Boxwhisker" | "Pareto" | "RegionMap" | "Treemap" | "Waterfall" | "Sunburst" | "Funnel";
            /**
             *
             * Returns or sets the way that blank cells are plotted on a chart. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            displayBlanksAs?: Excel.ChartDisplayBlanksAs | "NotPlotted" | "Zero" | "Interplotted";
            /**
             *
             * Represents the height, in points, of the chart object.
             *
             * [Api set: ExcelApi 1.1]
             */
            height?: number;
            /**
             *
             * The distance, in points, from the left side of the chart to the worksheet origin.
             *
             * [Api set: ExcelApi 1.1]
             */
            left?: number;
            /**
             *
             * Represents the name of a chart object.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: string;
            /**
             *
             * Returns or sets the way columns or rows are used as data series on the chart. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            plotBy?: Excel.ChartPlotBy | "Rows" | "Columns";
            /**
             *
             * True if only visible cells are plotted. False if both visible and hidden cells are plotted. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            plotVisibleOnly?: boolean;
            /**
             *
             * Returns or sets a ChartSeriesNameLevel enumeration constant referring to
            the level of where the series names are being sourced from. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            seriesNameLevel?: number;
            /**
             *
             * Represents whether to display all field buttons on a PivotChart.
             *
             * [Api set: ExcelApi 1.7]
             */
            showAllFieldButtons?: boolean;
            /**
             *
             * Represents whether to show the data labels when the value is greater than the maximum value on the value axis.
            If value axis became smaller than the size of data points, you can use this property to set whether to show the data labels.
            This property applies to 2-D charts only.
             *
             * [Api set: ExcelApi 1.8]
             */
            showDataLabelsOverMaximum?: boolean;
            /**
             *
             * Returns or sets the chart style for the chart. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            style?: number;
            /**
             *
             * Represents the distance, in points, from the top edge of the object to the top of row 1 (on a worksheet) or the top of the chart area (on a chart).
             *
             * [Api set: ExcelApi 1.1]
             */
            top?: number;
            /**
             *
             * Represents the width, in points, of the chart object.
             *
             * [Api set: ExcelApi 1.1]
             */
            width?: number;
        }
        /** An interface for updating data on the ChartPivotOptions object, for use in "chartPivotOptions.set({ ... })". */
        interface ChartPivotOptionsUpdateData {
            /**
             *
             * Specifies whether or not to display the axis field buttons on a PivotChart. The ShowAxisFieldButtons property corresponds to the "Show Axis Field Buttons" command on the "Field Buttons" drop-down list of the "Analyze" tab, which is available when a PivotChart is selected.
             *
             * [Api set: ExcelApi 1.9]
             */
            showAxisFieldButtons?: boolean;
            /**
             *
             * Specifies whether or not to display the legend field buttons on a PivotChart
             *
             * [Api set: ExcelApi 1.9]
             */
            showLegendFieldButtons?: boolean;
            /**
             *
             * Specifies whether or not to display the report filter field buttons on a PivotChart.
             *
             * [Api set: ExcelApi 1.9]
             */
            showReportFilterFieldButtons?: boolean;
            /**
             *
             * Specifies whether or not to display the show value field buttons on a PivotChart
             *
             * [Api set: ExcelApi 1.9]
             */
            showValueFieldButtons?: boolean;
        }
        /** An interface for updating data on the ChartAreaFormat object, for use in "chartAreaFormat.set({ ... })". */
        interface ChartAreaFormatUpdateData {
            /**
            *
            * Represents the border format of chart area, which includes color, linestyle, and weight.
            *
            * [Api set: ExcelApi 1.7]
            */
            border?: Excel.Interfaces.ChartBorderUpdateData;
            /**
            *
            * Represents the font attributes (font name, font size, color, etc.) for the current object.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontUpdateData;
            /**
             *
             * Returns or sets color scheme of the chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            colorScheme?: Excel.ChartColorScheme | "ColorfulPalette1" | "ColorfulPalette2" | "ColorfulPalette3" | "ColorfulPalette4" | "MonochromaticPalette1" | "MonochromaticPalette2" | "MonochromaticPalette3" | "MonochromaticPalette4" | "MonochromaticPalette5" | "MonochromaticPalette6" | "MonochromaticPalette7" | "MonochromaticPalette8" | "MonochromaticPalette9" | "MonochromaticPalette10" | "MonochromaticPalette11" | "MonochromaticPalette12" | "MonochromaticPalette13";
            /**
             *
             * Specifies whether or not chart area of the chart has rounded corners. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            roundedCorners?: boolean;
        }
        /** An interface for updating data on the ChartSeriesCollection object, for use in "chartSeriesCollection.set({ ... })". */
        interface ChartSeriesCollectionUpdateData {
            items?: Excel.Interfaces.ChartSeriesData[];
        }
        /** An interface for updating data on the ChartSeries object, for use in "chartSeries.set({ ... })". */
        interface ChartSeriesUpdateData {
            /**
            *
            * Encapsulates the bin options for histogram charts and pareto charts.
            *
            * [Api set: ExcelApi 1.9]
            */
            binOptions?: Excel.Interfaces.ChartBinOptionsUpdateData;
            /**
            *
            * Encapsulates the options for the box and whisker charts.
            *
            * [Api set: ExcelApi 1.9]
            */
            boxwhiskerOptions?: Excel.Interfaces.ChartBoxwhiskerOptionsUpdateData;
            /**
            *
            * Represents a collection of all dataLabels in the series.
            *
            * [Api set: ExcelApi 1.8]
            */
            dataLabels?: Excel.Interfaces.ChartDataLabelsUpdateData;
            /**
            *
            * Represents the formatting of a chart series, which includes fill and line formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartSeriesFormatUpdateData;
            /**
            *
            * Encapsulates the options for a region map chart.
            *
            * [Api set: ExcelApi 1.9]
            */
            mapOptions?: Excel.Interfaces.ChartMapOptionsUpdateData;
            /**
            *
            * Represents the error bar object of a chart series.
            *
            * [Api set: ExcelApi 1.9]
            */
            xErrorBars?: Excel.Interfaces.ChartErrorBarsUpdateData;
            /**
            *
            * Represents the error bar object of a chart series.
            *
            * [Api set: ExcelApi 1.9]
            */
            yErrorBars?: Excel.Interfaces.ChartErrorBarsUpdateData;
            /**
             *
             * Returns or sets the group for the specified series. Read/Write
             *
             * [Api set: ExcelApi 1.8]
             */
            axisGroup?: Excel.ChartAxisGroup | "Primary" | "Secondary";
            /**
             *
             * This can be an integer value from 0 (zero) to 300, representing the percentage of the default size. This property only applies to bubble charts. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            bubbleScale?: number;
            /**
             *
             * Represents the chart type of a series. See Excel.ChartType for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            chartType?: Excel.ChartType | "Invalid" | "ColumnClustered" | "ColumnStacked" | "ColumnStacked100" | "3DColumnClustered" | "3DColumnStacked" | "3DColumnStacked100" | "BarClustered" | "BarStacked" | "BarStacked100" | "3DBarClustered" | "3DBarStacked" | "3DBarStacked100" | "LineStacked" | "LineStacked100" | "LineMarkers" | "LineMarkersStacked" | "LineMarkersStacked100" | "PieOfPie" | "PieExploded" | "3DPieExploded" | "BarOfPie" | "XYScatterSmooth" | "XYScatterSmoothNoMarkers" | "XYScatterLines" | "XYScatterLinesNoMarkers" | "AreaStacked" | "AreaStacked100" | "3DAreaStacked" | "3DAreaStacked100" | "DoughnutExploded" | "RadarMarkers" | "RadarFilled" | "Surface" | "SurfaceWireframe" | "SurfaceTopView" | "SurfaceTopViewWireframe" | "Bubble" | "Bubble3DEffect" | "StockHLC" | "StockOHLC" | "StockVHLC" | "StockVOHLC" | "CylinderColClustered" | "CylinderColStacked" | "CylinderColStacked100" | "CylinderBarClustered" | "CylinderBarStacked" | "CylinderBarStacked100" | "CylinderCol" | "ConeColClustered" | "ConeColStacked" | "ConeColStacked100" | "ConeBarClustered" | "ConeBarStacked" | "ConeBarStacked100" | "ConeCol" | "PyramidColClustered" | "PyramidColStacked" | "PyramidColStacked100" | "PyramidBarClustered" | "PyramidBarStacked" | "PyramidBarStacked100" | "PyramidCol" | "3DColumn" | "Line" | "3DLine" | "3DPie" | "Pie" | "XYScatter" | "3DArea" | "Area" | "Doughnut" | "Radar" | "Histogram" | "Boxwhisker" | "Pareto" | "RegionMap" | "Treemap" | "Waterfall" | "Sunburst" | "Funnel";
            /**
             *
             * Represents the doughnut hole size of a chart series.  Only valid on doughnut and doughnutExploded charts.
            Throws an invalid argument exception on invalid charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            doughnutHoleSize?: number;
            /**
             *
             * Returns or sets the explosion value for a pie-chart or doughnut-chart slice. Returns 0 (zero) if there's no explosion (the tip of the slice is in the center of the pie). Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            explosion?: number;
            /**
             *
             * Boolean value representing if the series is filtered or not. Not applicable for surface charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            filtered?: boolean;
            /**
             *
             * Returns or sets the angle of the first pie-chart or doughnut-chart slice, in degrees (clockwise from vertical). Applies only to pie, 3-D pie, and doughnut charts. Can be a value from 0 through 360. Read/Write
             *
             * [Api set: ExcelApi 1.8]
             */
            firstSliceAngle?: number;
            /**
             *
             * Represents the gap width of a chart series.  Only valid on bar and column charts, as well as
            specific classes of line and pie charts.  Throws an invalid argument exception on invalid charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            gapWidth?: number;
            /**
             *
             * Returns or sets the color for maximum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMaximumColor?: string;
            /**
             *
             * Returns or sets the type for maximum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMaximumType?: Excel.ChartGradientStyleType | "ExtremeValue" | "Number" | "Percent";
            /**
             *
             * Returns or sets the maximum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMaximumValue?: number;
            /**
             *
             * Returns or sets the color for midpoint value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMidpointColor?: string;
            /**
             *
             * Returns or sets the type for midpoint value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMidpointType?: Excel.ChartGradientStyleType | "ExtremeValue" | "Number" | "Percent";
            /**
             *
             * Returns or sets the midpoint value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMidpointValue?: number;
            /**
             *
             * Returns or sets the color for minimum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMinimumColor?: string;
            /**
             *
             * Returns or sets the type for minimum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMinimumType?: Excel.ChartGradientStyleType | "ExtremeValue" | "Number" | "Percent";
            /**
             *
             * Returns or sets the minimum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMinimumValue?: number;
            /**
             *
             * Returns or sets series gradient style of a region map chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientStyle?: Excel.ChartGradientStyle | "TwoPhaseColor" | "ThreePhaseColor";
            /**
             *
             * Boolean value representing if the series has data labels or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            hasDataLabels?: boolean;
            /**
             *
             * Returns or sets the fill color for negative data points in a series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            invertColor?: string;
            /**
             *
             * True if Microsoft Excel inverts the pattern in the item when it corresponds to a negative number. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            invertIfNegative?: boolean;
            /**
             *
             * Represents markers background color of a chart series.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerBackgroundColor?: string;
            /**
             *
             * Represents markers foreground color of a chart series.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerForegroundColor?: string;
            /**
             *
             * Represents marker size of a chart series.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerSize?: number;
            /**
             *
             * Represents marker style of a chart series. See Excel.ChartMarkerStyle for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerStyle?: Excel.ChartMarkerStyle | "Invalid" | "Automatic" | "None" | "Square" | "Diamond" | "Triangle" | "X" | "Star" | "Dot" | "Dash" | "Circle" | "Plus" | "Picture";
            /**
             *
             * Represents the name of a series in a chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: string;
            /**
             *
             * Specifies how bars and columns are positioned. Can be a value between –100 and 100. Applies only to 2-D bar and 2-D column charts. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            overlap?: number;
            /**
             *
             * Returns or sets the series parent label strategy area for a treemap chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            parentLabelStrategy?: Excel.ChartParentLabelStrategy | "None" | "Banner" | "Overlapping";
            /**
             *
             * Represents the plot order of a chart series within the chart group.
             *
             * [Api set: ExcelApi 1.7]
             */
            plotOrder?: number;
            /**
             *
             * Returns or sets the size of the secondary section of either a pie of pie chart or a bar of pie chart, as a percentage of the size of the primary pie. Can be a value from 5 to 200. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            secondPlotSize?: number;
            /**
             *
             * Specifies whether or not connector lines are shown in waterfall charts. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showConnectorLines?: boolean;
            /**
             *
             * Specifies whether or not leader lines are displayed for each data label in the series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showLeaderLines?: boolean;
            /**
             *
             * Boolean value representing if the series has a shadow or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showShadow?: boolean;
            /**
             *
             * Boolean value representing if the series is smooth or not. Only applicable to line and scatter charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            smooth?: boolean;
            /**
             *
             * Returns or sets the way the two sections of either a pie of pie chart or a bar of pie chart are split. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            splitType?: Excel.ChartSplitType | "SplitByPosition" | "SplitByValue" | "SplitByPercentValue" | "SplitByCustomSplit";
            /**
             *
             * Returns or sets the threshold value that separates two sections of either a pie-of-pie chart or a bar-of-pie chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            splitValue?: number;
            /**
             *
             * True if Microsoft Excel assigns a different color or pattern to each data marker. The chart must contain only one series. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            varyByCategories?: boolean;
        }
        /** An interface for updating data on the ChartSeriesFormat object, for use in "chartSeriesFormat.set({ ... })". */
        interface ChartSeriesFormatUpdateData {
            /**
            *
            * Represents line formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            line?: Excel.Interfaces.ChartLineFormatUpdateData;
        }
        /** An interface for updating data on the ChartPointsCollection object, for use in "chartPointsCollection.set({ ... })". */
        interface ChartPointsCollectionUpdateData {
            items?: Excel.Interfaces.ChartPointData[];
        }
        /** An interface for updating data on the ChartPoint object, for use in "chartPoint.set({ ... })". */
        interface ChartPointUpdateData {
            /**
            *
            * Returns the data label of a chart point.
            *
            * [Api set: ExcelApi 1.7]
            */
            dataLabel?: Excel.Interfaces.ChartDataLabelUpdateData;
            /**
            *
            * Encapsulates the format properties chart point.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartPointFormatUpdateData;
            /**
             *
             * Represents whether a data point has a data label. Not applicable for surface charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            hasDataLabel?: boolean;
            /**
             *
             * HTML color code representation of the marker background color of data point. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerBackgroundColor?: string;
            /**
             *
             * HTML color code representation of the marker foreground color of data point. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerForegroundColor?: string;
            /**
             *
             * Represents marker size of data point.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerSize?: number;
            /**
             *
             * Represents marker style of a chart data point. See Excel.ChartMarkerStyle for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerStyle?: Excel.ChartMarkerStyle | "Invalid" | "Automatic" | "None" | "Square" | "Diamond" | "Triangle" | "X" | "Star" | "Dot" | "Dash" | "Circle" | "Plus" | "Picture";
        }
        /** An interface for updating data on the ChartPointFormat object, for use in "chartPointFormat.set({ ... })". */
        interface ChartPointFormatUpdateData {
            /**
            *
            * Represents the border format of a chart data point, which includes color, style, and weight information.
            *
            * [Api set: ExcelApi 1.7]
            */
            border?: Excel.Interfaces.ChartBorderUpdateData;
        }
        /** An interface for updating data on the ChartAxes object, for use in "chartAxes.set({ ... })". */
        interface ChartAxesUpdateData {
            /**
            *
            * Represents the category axis in a chart.
            *
            * [Api set: ExcelApi 1.1]
            */
            categoryAxis?: Excel.Interfaces.ChartAxisUpdateData;
            /**
            *
            * Represents the series axis of a 3-dimensional chart.
            *
            * [Api set: ExcelApi 1.1]
            */
            seriesAxis?: Excel.Interfaces.ChartAxisUpdateData;
            /**
            *
            * Represents the value axis in an axis.
            *
            * [Api set: ExcelApi 1.1]
            */
            valueAxis?: Excel.Interfaces.ChartAxisUpdateData;
        }
        /** An interface for updating data on the ChartAxis object, for use in "chartAxis.set({ ... })". */
        interface ChartAxisUpdateData {
            /**
            *
            * Represents the formatting of a chart object, which includes line and font formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartAxisFormatUpdateData;
            /**
            *
            * Returns a Gridlines object that represents the major gridlines for the specified axis.
            *
            * [Api set: ExcelApi 1.1]
            */
            majorGridlines?: Excel.Interfaces.ChartGridlinesUpdateData;
            /**
            *
            * Returns a Gridlines object that represents the minor gridlines for the specified axis.
            *
            * [Api set: ExcelApi 1.1]
            */
            minorGridlines?: Excel.Interfaces.ChartGridlinesUpdateData;
            /**
            *
            * Represents the axis title.
            *
            * [Api set: ExcelApi 1.1]
            */
            title?: Excel.Interfaces.ChartAxisTitleUpdateData;
            /**
             *
             * Represents the alignment for the specified axis tick label. See Excel.ChartTextHorizontalAlignment for detail.
             *
             * [Api set: ExcelApi 1.8]
             */
            alignment?: Excel.ChartTickLabelAlignment | "Center" | "Left" | "Right";
            /**
             *
             * Returns or sets the base unit for the specified category axis.
             *
             * [Api set: ExcelApi 1.7]
             */
            baseTimeUnit?: Excel.ChartAxisTimeUnit | "Days" | "Months" | "Years";
            /**
             *
             * Returns or sets the category axis type.
             *
             * [Api set: ExcelApi 1.7]
             */
            categoryType?: Excel.ChartAxisCategoryType | "Automatic" | "TextAxis" | "DateAxis";
            /**
             *
             * Represents the axis display unit. See Excel.ChartAxisDisplayUnit for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            displayUnit?: Excel.ChartAxisDisplayUnit | "None" | "Hundreds" | "Thousands" | "TenThousands" | "HundredThousands" | "Millions" | "TenMillions" | "HundredMillions" | "Billions" | "Trillions" | "Custom";
            /**
             *
             * Represents whether value axis crosses the category axis between categories.
             *
             * [Api set: ExcelApi 1.8]
             */
            isBetweenCategories?: boolean;
            /**
             *
             * Represents whether or not the number format is linked to the cells. If true, the number format will change in the labels when it changes in the cells.
             *
             * [Api set: ExcelApi 1.9]
             */
            linkNumberFormat?: boolean;
            /**
             *
             * Represents the base of the logarithm when using logarithmic scales.
             *
             * [Api set: ExcelApi 1.7]
             */
            logBase?: number;
            /**
             *
             * Represents the type of major tick mark for the specified axis. See Excel.ChartAxisTickMark for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            majorTickMark?: Excel.ChartAxisTickMark | "None" | "Cross" | "Inside" | "Outside";
            /**
             *
             * Returns or sets the major unit scale value for the category axis when the CategoryType property is set to TimeScale.
             *
             * [Api set: ExcelApi 1.7]
             */
            majorTimeUnitScale?: Excel.ChartAxisTimeUnit | "Days" | "Months" | "Years";
            /**
             *
             * Represents the interval between two major tick marks. Can be set to a numeric value or an empty string.  The returned value is always a number.
             *
             * [Api set: ExcelApi 1.1]
             */
            majorUnit?: any;
            /**
             *
             * Represents the maximum value on the value axis.  Can be set to a numeric value or an empty string (for automatic axis values).  The returned value is always a number.
             *
             * [Api set: ExcelApi 1.1]
             */
            maximum?: any;
            /**
             *
             * Represents the minimum value on the value axis. Can be set to a numeric value or an empty string (for automatic axis values).  The returned value is always a number.
             *
             * [Api set: ExcelApi 1.1]
             */
            minimum?: any;
            /**
             *
             * Represents the type of minor tick mark for the specified axis. See Excel.ChartAxisTickMark for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            minorTickMark?: Excel.ChartAxisTickMark | "None" | "Cross" | "Inside" | "Outside";
            /**
             *
             * Returns or sets the minor unit scale value for the category axis when the CategoryType property is set to TimeScale.
             *
             * [Api set: ExcelApi 1.7]
             */
            minorTimeUnitScale?: Excel.ChartAxisTimeUnit | "Days" | "Months" | "Years";
            /**
             *
             * Represents the interval between two minor tick marks. Can be set to a numeric value or an empty string (for automatic axis values). The returned value is always a number.
             *
             * [Api set: ExcelApi 1.1]
             */
            minorUnit?: any;
            /**
             *
             * Represents whether an axis is multilevel or not.
             *
             * [Api set: ExcelApi 1.8]
             */
            multiLevel?: boolean;
            /**
             *
             * Represents the format code for the axis tick label.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: string;
            /**
             *
             * Represents the distance between the levels of labels, and the distance between the first level and the axis line. The value should be an integer from 0 to 1000.
             *
             * [Api set: ExcelApi 1.8]
             */
            offset?: number;
            /**
             *
             * Represents the specified axis position where the other axis crosses. See Excel.ChartAxisPosition for details.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: Excel.ChartAxisPosition | "Automatic" | "Maximum" | "Minimum" | "Custom";
            /**
             *
             * Represents whether Microsoft Excel plots data points from last to first.
             *
             * [Api set: ExcelApi 1.7]
             */
            reversePlotOrder?: boolean;
            /**
             *
             * Represents the value axis scale type. See Excel.ChartAxisScaleType for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            scaleType?: Excel.ChartAxisScaleType | "Linear" | "Logarithmic";
            /**
             *
             * Represents whether the axis display unit label is visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            showDisplayUnitLabel?: boolean;
            /**
             *
             * Represents the text orientation of the axis tick label. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: any;
            /**
             *
             * Represents the position of tick-mark labels on the specified axis. See Excel.ChartAxisTickLabelPosition for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            tickLabelPosition?: Excel.ChartAxisTickLabelPosition | "NextToAxis" | "High" | "Low" | "None";
            /**
             *
             * Represents the number of categories or series between tick-mark labels. Can be a value from 1 through 31999 or an empty string for automatic setting. The returned value is always a number.
             *
             * [Api set: ExcelApi 1.7]
             */
            tickLabelSpacing?: any;
            /**
             *
             * Represents the number of categories or series between tick marks.
             *
             * [Api set: ExcelApi 1.7]
             */
            tickMarkSpacing?: number;
            /**
             *
             * A boolean value represents the visibility of the axis.
             *
             * [Api set: ExcelApi 1.7]
             */
            visible?: boolean;
        }
        /** An interface for updating data on the ChartAxisFormat object, for use in "chartAxisFormat.set({ ... })". */
        interface ChartAxisFormatUpdateData {
            /**
            *
            * Represents the font attributes (font name, font size, color, etc.) for a chart axis element.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontUpdateData;
            /**
            *
            * Represents chart line formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            line?: Excel.Interfaces.ChartLineFormatUpdateData;
        }
        /** An interface for updating data on the ChartAxisTitle object, for use in "chartAxisTitle.set({ ... })". */
        interface ChartAxisTitleUpdateData {
            /**
            *
            * Represents the formatting of chart axis title.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartAxisTitleFormatUpdateData;
            /**
             *
             * Represents the axis title.
             *
             * [Api set: ExcelApi 1.1]
             */
            text?: string;
            /**
             *
             * A boolean that specifies the visibility of an axis title.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
        }
        /** An interface for updating data on the ChartAxisTitleFormat object, for use in "chartAxisTitleFormat.set({ ... })". */
        interface ChartAxisTitleFormatUpdateData {
            /**
            *
            * Represents the border format, which includes color, linestyle, and weight.
            *
            * [Api set: ExcelApi 1.8]
            */
            border?: Excel.Interfaces.ChartBorderUpdateData;
            /**
            *
            * Represents the font attributes, such as font name, font size, color, etc. of chart axis title object.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontUpdateData;
        }
        /** An interface for updating data on the ChartDataLabels object, for use in "chartDataLabels.set({ ... })". */
        interface ChartDataLabelsUpdateData {
            /**
            *
            * Represents the format of chart data labels, which includes fill and font formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartDataLabelFormatUpdateData;
            /**
             *
             * Represents whether data labels automatically generate appropriate text based on context.
             *
             * [Api set: ExcelApi 1.8]
             */
            autoText?: boolean;
            /**
             *
             * Represents the horizontal alignment for chart data label. See Excel.ChartTextHorizontalAlignment for details.
            This property is valid only when TextOrientation of data label is 0.
             *
             * [Api set: ExcelApi 1.8]
             */
            horizontalAlignment?: Excel.ChartTextHorizontalAlignment | "Center" | "Left" | "Right" | "Justify" | "Distributed";
            /**
             *
             * Represents whether or not the number format is linked to the cells. If true, the number format will change in the labels when it changes in the cells
             *
             * [Api set: ExcelApi 1.9]
             */
            linkNumberFormat?: boolean;
            /**
             *
             * Represents the format code for data labels.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: string;
            /**
             *
             * DataLabelPosition value that represents the position of the data label. See Excel.ChartDataLabelPosition for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            position?: Excel.ChartDataLabelPosition | "Invalid" | "None" | "Center" | "InsideEnd" | "InsideBase" | "OutsideEnd" | "Left" | "Right" | "Top" | "Bottom" | "BestFit" | "Callout";
            /**
             *
             * String representing the separator used for the data labels on a chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            separator?: string;
            /**
             *
             * Boolean value representing if the data label bubble size is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showBubbleSize?: boolean;
            /**
             *
             * Boolean value representing if the data label category name is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showCategoryName?: boolean;
            /**
             *
             * Boolean value representing if the data label legend key is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showLegendKey?: boolean;
            /**
             *
             * Boolean value representing if the data label percentage is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showPercentage?: boolean;
            /**
             *
             * Boolean value representing if the data label series name is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showSeriesName?: boolean;
            /**
             *
             * Boolean value representing if the data label value is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showValue?: boolean;
            /**
             *
             * Represents the text orientation of data labels. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: number;
            /**
             *
             * Represents the vertical alignment of chart data label. See Excel.ChartTextVerticalAlignment for details.
            This property is valid only when TextOrientation of data label is 90, -90 or 180.
             *
             * [Api set: ExcelApi 1.8]
             */
            verticalAlignment?: Excel.ChartTextVerticalAlignment | "Center" | "Bottom" | "Top" | "Justify" | "Distributed";
        }
        /** An interface for updating data on the ChartDataLabel object, for use in "chartDataLabel.set({ ... })". */
        interface ChartDataLabelUpdateData {
            /**
            *
            * Represents the format of chart data label.
            *
            * [Api set: ExcelApi 1.8]
            */
            format?: Excel.Interfaces.ChartDataLabelFormatUpdateData;
            /**
             *
             * Boolean value representing if data label automatically generates appropriate text based on context.
             *
             * [Api set: ExcelApi 1.8]
             */
            autoText?: boolean;
            /**
             *
             * String value that represents the formula of chart data label using A1-style notation.
             *
             * [Api set: ExcelApi 1.8]
             */
            formula?: string;
            /**
             *
             * Represents the horizontal alignment for chart data label. See Excel.ChartTextHorizontalAlignment for details.
            This property is valid only when TextOrientation of data label is 90, -90 or 180.
             *
             * [Api set: ExcelApi 1.8]
             */
            horizontalAlignment?: Excel.ChartTextHorizontalAlignment | "Center" | "Left" | "Right" | "Justify" | "Distributed";
            /**
             *
             * Represents the distance, in points, from the left edge of chart data label to the left edge of chart area. Null if chart data label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            left?: number;
            /**
             *
             * Boolean value representing if the number format is linked to the cells (so that the number format changes in the labels when it changes in the cells).
             *
             * [Api set: ExcelApi 1.9]
             */
            linkNumberFormat?: boolean;
            /**
             *
             * String value that represents the format code for data label.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: string;
            /**
             *
             * DataLabelPosition value that represents the position of the data label. See Excel.ChartDataLabelPosition for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            position?: Excel.ChartDataLabelPosition | "Invalid" | "None" | "Center" | "InsideEnd" | "InsideBase" | "OutsideEnd" | "Left" | "Right" | "Top" | "Bottom" | "BestFit" | "Callout";
            /**
             *
             * String representing the separator used for the data label on a chart.
             *
             * [Api set: ExcelApi 1.7]
             */
            separator?: string;
            /**
             *
             * Boolean value representing if the data label bubble size is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showBubbleSize?: boolean;
            /**
             *
             * Boolean value representing if the data label category name is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showCategoryName?: boolean;
            /**
             *
             * Boolean value representing if the data label legend key is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showLegendKey?: boolean;
            /**
             *
             * Boolean value representing if the data label percentage is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showPercentage?: boolean;
            /**
             *
             * Boolean value representing if the data label series name is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showSeriesName?: boolean;
            /**
             *
             * Boolean value representing if the data label value is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showValue?: boolean;
            /**
             *
             * String representing the text of the data label on a chart.
             *
             * [Api set: ExcelApi 1.8]
             */
            text?: string;
            /**
             *
             * Represents the text orientation of chart data label. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: number;
            /**
             *
             * Represents the distance, in points, from the top edge of chart data label to the top of chart area. Null if chart data label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            top?: number;
            /**
             *
             * Represents the vertical alignment of chart data label. See Excel.ChartTextVerticalAlignment for details.
            This property is valid only when TextOrientation of data label is 0.
             *
             * [Api set: ExcelApi 1.8]
             */
            verticalAlignment?: Excel.ChartTextVerticalAlignment | "Center" | "Bottom" | "Top" | "Justify" | "Distributed";
        }
        /** An interface for updating data on the ChartDataLabelFormat object, for use in "chartDataLabelFormat.set({ ... })". */
        interface ChartDataLabelFormatUpdateData {
            /**
            *
            * Represents the border format, which includes color, linestyle, and weight.
            *
            * [Api set: ExcelApi 1.8]
            */
            border?: Excel.Interfaces.ChartBorderUpdateData;
            /**
            *
            * Represents the font attributes (font name, font size, color, etc.) for a chart data label.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontUpdateData;
        }
        /** An interface for updating data on the ChartErrorBars object, for use in "chartErrorBars.set({ ... })". */
        interface ChartErrorBarsUpdateData {
            /**
            *
            * Specifies the formatting type of the error bars.
            *
            * [Api set: ExcelApi 1.9]
            */
            format?: Excel.Interfaces.ChartErrorBarsFormatUpdateData;
            /**
             *
             * Specifies whether or not the error bars have an end style cap.
             *
             * [Api set: ExcelApi 1.9]
             */
            endStyleCap?: boolean;
            /**
             *
             * Specifies which parts of the error bars to include.
             *
             * [Api set: ExcelApi 1.9]
             */
            include?: Excel.ChartErrorBarsInclude | "Both" | "MinusValues" | "PlusValues";
            /**
             *
             * The type of range marked by the error bars.
             *
             * [Api set: ExcelApi 1.9]
             */
            type?: Excel.ChartErrorBarsType | "FixedValue" | "Percent" | "StDev" | "StError" | "Custom";
            /**
             *
             * Specifies whether or not the error bars are displayed.
             *
             * [Api set: ExcelApi 1.9]
             */
            visible?: boolean;
        }
        /** An interface for updating data on the ChartErrorBarsFormat object, for use in "chartErrorBarsFormat.set({ ... })". */
        interface ChartErrorBarsFormatUpdateData {
            /**
            *
            * Represents the chart line formatting.
            *
            * [Api set: ExcelApi 1.9]
            */
            line?: Excel.Interfaces.ChartLineFormatUpdateData;
        }
        /** An interface for updating data on the ChartGridlines object, for use in "chartGridlines.set({ ... })". */
        interface ChartGridlinesUpdateData {
            /**
            *
            * Represents the formatting of chart gridlines.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartGridlinesFormatUpdateData;
            /**
             *
             * Boolean value representing if the axis gridlines are visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
        }
        /** An interface for updating data on the ChartGridlinesFormat object, for use in "chartGridlinesFormat.set({ ... })". */
        interface ChartGridlinesFormatUpdateData {
            /**
            *
            * Represents chart line formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            line?: Excel.Interfaces.ChartLineFormatUpdateData;
        }
        /** An interface for updating data on the ChartLegend object, for use in "chartLegend.set({ ... })". */
        interface ChartLegendUpdateData {
            /**
            *
            * Represents the formatting of a chart legend, which includes fill and font formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartLegendFormatUpdateData;
            /**
             *
             * Represents the height, in points, of the legend on the chart. Null if legend is not visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            height?: number;
            /**
             *
             * Represents the left, in points, of a chart legend. Null if legend is not visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            left?: number;
            /**
             *
             * Boolean value for whether the chart legend should overlap with the main body of the chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            overlay?: boolean;
            /**
             *
             * Represents the position of the legend on the chart. See Excel.ChartLegendPosition for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            position?: Excel.ChartLegendPosition | "Invalid" | "Top" | "Bottom" | "Left" | "Right" | "Corner" | "Custom";
            /**
             *
             * Represents if the legend has a shadow on the chart.
             *
             * [Api set: ExcelApi 1.7]
             */
            showShadow?: boolean;
            /**
             *
             * Represents the top of a chart legend.
             *
             * [Api set: ExcelApi 1.7]
             */
            top?: number;
            /**
             *
             * A boolean value the represents the visibility of a ChartLegend object.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
            /**
             *
             * Represents the width, in points, of the legend on the chart. Null if legend is not visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            width?: number;
        }
        /** An interface for updating data on the ChartLegendEntry object, for use in "chartLegendEntry.set({ ... })". */
        interface ChartLegendEntryUpdateData {
            /**
             *
             * Represents the visible of a chart legend entry.
             *
             * [Api set: ExcelApi 1.7]
             */
            visible?: boolean;
        }
        /** An interface for updating data on the ChartLegendEntryCollection object, for use in "chartLegendEntryCollection.set({ ... })". */
        interface ChartLegendEntryCollectionUpdateData {
            items?: Excel.Interfaces.ChartLegendEntryData[];
        }
        /** An interface for updating data on the ChartLegendFormat object, for use in "chartLegendFormat.set({ ... })". */
        interface ChartLegendFormatUpdateData {
            /**
            *
            * Represents the border format, which includes color, linestyle, and weight.
            *
            * [Api set: ExcelApi 1.8]
            */
            border?: Excel.Interfaces.ChartBorderUpdateData;
            /**
            *
            * Represents the font attributes such as font name, font size, color, etc. of a chart legend.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontUpdateData;
        }
        /** An interface for updating data on the ChartMapOptions object, for use in "chartMapOptions.set({ ... })". */
        interface ChartMapOptionsUpdateData {
            /**
             *
             * Returns or sets the series map labels strategy of a region map chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            labelStrategy?: Excel.ChartMapLabelStrategy | "None" | "BestFit" | "ShowAll";
            /**
             *
             * Returns or sets the series mapping level of a region map chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            level?: Excel.ChartMapAreaLevel | "Automatic" | "DataOnly" | "City" | "County" | "State" | "Country" | "Continent" | "World";
            /**
             *
             * Returns or sets the series projection type of a region map chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            projectionType?: Excel.ChartMapProjectionType | "Automatic" | "Mercator" | "Miller" | "Robinson" | "Albers";
        }
        /** An interface for updating data on the ChartTitle object, for use in "chartTitle.set({ ... })". */
        interface ChartTitleUpdateData {
            /**
            *
            * Represents the formatting of a chart title, which includes fill and font formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartTitleFormatUpdateData;
            /**
             *
             * Represents the horizontal alignment for chart title.
             *
             * [Api set: ExcelApi 1.7]
             */
            horizontalAlignment?: Excel.ChartTextHorizontalAlignment | "Center" | "Left" | "Right" | "Justify" | "Distributed";
            /**
             *
             * Represents the distance, in points, from the left edge of chart title to the left edge of chart area. Null if chart title is not visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            left?: number;
            /**
             *
             * Boolean value representing if the chart title will overlay the chart or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            overlay?: boolean;
            /**
             *
             * Represents the position of chart title. See Excel.ChartTitlePosition for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            position?: Excel.ChartTitlePosition | "Automatic" | "Top" | "Bottom" | "Left" | "Right";
            /**
             *
             * Represents a boolean value that determines if the chart title has a shadow.
             *
             * [Api set: ExcelApi 1.7]
             */
            showShadow?: boolean;
            /**
             *
             * Represents the title text of a chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            text?: string;
            /**
             *
             * Represents the text orientation of chart title. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
             *
             * [Api set: ExcelApi 1.7]
             */
            textOrientation?: number;
            /**
             *
             * Represents the distance, in points, from the top edge of chart title to the top of chart area. Null if chart title is not visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            top?: number;
            /**
             *
             * Represents the vertical alignment of chart title. See Excel.ChartTextVerticalAlignment for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            verticalAlignment?: Excel.ChartTextVerticalAlignment | "Center" | "Bottom" | "Top" | "Justify" | "Distributed";
            /**
             *
             * A boolean value the represents the visibility of a chart title object.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
        }
        /** An interface for updating data on the ChartFormatString object, for use in "chartFormatString.set({ ... })". */
        interface ChartFormatStringUpdateData {
            /**
            *
            * Represents the font attributes, such as font name, font size, color, etc. of chart characters object.
            *
            * [Api set: ExcelApi 1.7]
            */
            font?: Excel.Interfaces.ChartFontUpdateData;
        }
        /** An interface for updating data on the ChartTitleFormat object, for use in "chartTitleFormat.set({ ... })". */
        interface ChartTitleFormatUpdateData {
            /**
            *
            * Represents the border format of chart title, which includes color, linestyle, and weight.
            *
            * [Api set: ExcelApi 1.7]
            */
            border?: Excel.Interfaces.ChartBorderUpdateData;
            /**
            *
            * Represents the font attributes (font name, font size, color, etc.) for an object.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontUpdateData;
        }
        /** An interface for updating data on the ChartBorder object, for use in "chartBorder.set({ ... })". */
        interface ChartBorderUpdateData {
            /**
             *
             * HTML color code representing the color of borders in the chart.
             *
             * [Api set: ExcelApi 1.7]
             */
            color?: string;
            /**
             *
             * Represents the line style of the border. See Excel.ChartLineStyle for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            lineStyle?: Excel.ChartLineStyle | "None" | "Continuous" | "Dash" | "DashDot" | "DashDotDot" | "Dot" | "Grey25" | "Grey50" | "Grey75" | "Automatic" | "RoundDot";
            /**
             *
             * Represents weight of the border, in points.
             *
             * [Api set: ExcelApi 1.7]
             */
            weight?: number;
        }
        /** An interface for updating data on the ChartBinOptions object, for use in "chartBinOptions.set({ ... })". */
        interface ChartBinOptionsUpdateData {
            /**
             *
             * Specifies whether or not the bin overflow is enabled in a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            allowOverflow?: boolean;
            /**
             *
             * Specifies whether or not the bin underflow is enabled in a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            allowUnderflow?: boolean;
            /**
             *
             * Returns or sets the bin count of a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            count?: number;
            /**
             *
             * Returns or sets the bin overflow value of a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            overflowValue?: number;
            /**
             *
             * Returns or sets the bin's type for a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            type?: Excel.ChartBinType | "Category" | "Auto" | "BinWidth" | "BinCount";
            /**
             *
             * Returns or sets the bin underflow value of a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            underflowValue?: number;
            /**
             *
             * Returns or sets the bin width value of a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            width?: number;
        }
        /** An interface for updating data on the ChartBoxwhiskerOptions object, for use in "chartBoxwhiskerOptions.set({ ... })". */
        interface ChartBoxwhiskerOptionsUpdateData {
            /**
             *
             * Returns or sets the quartile calculation type of a box and whisker chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            quartileCalculation?: Excel.ChartBoxQuartileCalculation | "Inclusive" | "Exclusive";
            /**
             *
             * Specifies whether or not the inner points are shown in a box and whisker chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showInnerPoints?: boolean;
            /**
             *
             * Specifies whether or not the mean line is shown in a box and whisker chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showMeanLine?: boolean;
            /**
             *
             * Specifies whether or not the mean marker is shown in a box and whisker chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showMeanMarker?: boolean;
            /**
             *
             * Specifies whether or not outlier points are shown in a box and whisker chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showOutlierPoints?: boolean;
        }
        /** An interface for updating data on the ChartLineFormat object, for use in "chartLineFormat.set({ ... })". */
        interface ChartLineFormatUpdateData {
            /**
             *
             * HTML color code representing the color of lines in the chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: string;
            /**
             *
             * Represents the line style. See Excel.ChartLineStyle for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            lineStyle?: Excel.ChartLineStyle | "None" | "Continuous" | "Dash" | "DashDot" | "DashDotDot" | "Dot" | "Grey25" | "Grey50" | "Grey75" | "Automatic" | "RoundDot";
            /**
             *
             * Represents weight of the line, in points.
             *
             * [Api set: ExcelApi 1.7]
             */
            weight?: number;
        }
        /** An interface for updating data on the ChartFont object, for use in "chartFont.set({ ... })". */
        interface ChartFontUpdateData {
            /**
             *
             * Represents the bold status of font.
             *
             * [Api set: ExcelApi 1.1]
             */
            bold?: boolean;
            /**
             *
             * HTML color code representation of the text color. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: string;
            /**
             *
             * Represents the italic status of the font.
             *
             * [Api set: ExcelApi 1.1]
             */
            italic?: boolean;
            /**
             *
             * Font name (e.g. "Calibri")
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: string;
            /**
             *
             * Size of the font (e.g. 11)
             *
             * [Api set: ExcelApi 1.1]
             */
            size?: number;
            /**
             *
             * Type of underline applied to the font. See Excel.ChartUnderlineStyle for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            underline?: Excel.ChartUnderlineStyle | "None" | "Single";
        }
        /** An interface for updating data on the ChartTrendline object, for use in "chartTrendline.set({ ... })". */
        interface ChartTrendlineUpdateData {
            /**
            *
            * Represents the formatting of a chart trendline.
            *
            * [Api set: ExcelApi 1.7]
            */
            format?: Excel.Interfaces.ChartTrendlineFormatUpdateData;
            /**
            *
            * Represents the label of a chart trendline.
            *
            * [Api set: ExcelApi 1.8]
            */
            label?: Excel.Interfaces.ChartTrendlineLabelUpdateData;
            /**
             *
             * Represents the number of periods that the trendline extends backward.
             *
             * [Api set: ExcelApi 1.8]
             */
            backwardPeriod?: number;
            /**
             *
             * Represents the number of periods that the trendline extends forward.
             *
             * [Api set: ExcelApi 1.8]
             */
            forwardPeriod?: number;
            /**
             *
             * Represents the intercept value of the trendline. Can be set to a numeric value or an empty string (for automatic values). The returned value is always a number.
             *
             * [Api set: ExcelApi 1.7]
             */
            intercept?: any;
            /**
             *
             * Represents the period of a chart trendline. Only applicable for trendline with MovingAverage type.
             *
             * [Api set: ExcelApi 1.7]
             */
            movingAveragePeriod?: number;
            /**
             *
             * Represents the name of the trendline. Can be set to a string value, or can be set to null value represents automatic values. The returned value is always a string
             *
             * [Api set: ExcelApi 1.7]
             */
            name?: string;
            /**
             *
             * Represents the order of a chart trendline. Only applicable for trendline with Polynomial type.
             *
             * [Api set: ExcelApi 1.7]
             */
            polynomialOrder?: number;
            /**
             *
             * True if the equation for the trendline is displayed on the chart.
             *
             * [Api set: ExcelApi 1.8]
             */
            showEquation?: boolean;
            /**
             *
             * True if the R-squared for the trendline is displayed on the chart.
             *
             * [Api set: ExcelApi 1.8]
             */
            showRSquared?: boolean;
            /**
             *
             * Represents the type of a chart trendline.
             *
             * [Api set: ExcelApi 1.7]
             */
            type?: Excel.ChartTrendlineType | "Linear" | "Exponential" | "Logarithmic" | "MovingAverage" | "Polynomial" | "Power";
        }
        /** An interface for updating data on the ChartTrendlineCollection object, for use in "chartTrendlineCollection.set({ ... })". */
        interface ChartTrendlineCollectionUpdateData {
            items?: Excel.Interfaces.ChartTrendlineData[];
        }
        /** An interface for updating data on the ChartTrendlineFormat object, for use in "chartTrendlineFormat.set({ ... })". */
        interface ChartTrendlineFormatUpdateData {
            /**
            *
            * Represents chart line formatting.
            *
            * [Api set: ExcelApi 1.7]
            */
            line?: Excel.Interfaces.ChartLineFormatUpdateData;
        }
        /** An interface for updating data on the ChartTrendlineLabel object, for use in "chartTrendlineLabel.set({ ... })". */
        interface ChartTrendlineLabelUpdateData {
            /**
            *
            * Represents the format of chart trendline label.
            *
            * [Api set: ExcelApi 1.8]
            */
            format?: Excel.Interfaces.ChartTrendlineLabelFormatUpdateData;
            /**
             *
             * Boolean value representing if trendline label automatically generates appropriate text based on context.
             *
             * [Api set: ExcelApi 1.8]
             */
            autoText?: boolean;
            /**
             *
             * String value that represents the formula of chart trendline label using A1-style notation.
             *
             * [Api set: ExcelApi 1.8]
             */
            formula?: string;
            /**
             *
             * Represents the horizontal alignment for chart trendline label. See Excel.ChartTextHorizontalAlignment for details.
            This property is valid only when TextOrientation of trendline label is 90, -90 or 180.
             *
             * [Api set: ExcelApi 1.8]
             */
            horizontalAlignment?: Excel.ChartTextHorizontalAlignment | "Center" | "Left" | "Right" | "Justify" | "Distributed";
            /**
             *
             * Represents the distance, in points, from the left edge of chart trendline label to the left edge of chart area. Null if chart trendline label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            left?: number;
            /**
             *
             * Boolean value representing if the number format is linked to the cells (so that the number format changes in the labels when it changes in the cells).
             *
             * [Api set: ExcelApi 1.9]
             */
            linkNumberFormat?: boolean;
            /**
             *
             * String value that represents the format code for trendline label.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: string;
            /**
             *
             * String representing the text of the trendline label on a chart.
             *
             * [Api set: ExcelApi 1.8]
             */
            text?: string;
            /**
             *
             * Represents the text orientation of chart trendline label. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: number;
            /**
             *
             * Represents the distance, in points, from the top edge of chart trendline label to the top of chart area. Null if chart trendline label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            top?: number;
            /**
             *
             * Represents the vertical alignment of chart trendline label. See Excel.ChartTextVerticalAlignment for details.
            This property is valid only when TextOrientation of trendline label is 0.
             *
             * [Api set: ExcelApi 1.8]
             */
            verticalAlignment?: Excel.ChartTextVerticalAlignment | "Center" | "Bottom" | "Top" | "Justify" | "Distributed";
        }
        /** An interface for updating data on the ChartTrendlineLabelFormat object, for use in "chartTrendlineLabelFormat.set({ ... })". */
        interface ChartTrendlineLabelFormatUpdateData {
            /**
            *
            * Represents the border format, which includes color, linestyle, and weight.
            *
            * [Api set: ExcelApi 1.8]
            */
            border?: Excel.Interfaces.ChartBorderUpdateData;
            /**
            *
            * Represents the font attributes (font name, font size, color, etc.) for a chart trendline label.
            *
            * [Api set: ExcelApi 1.8]
            */
            font?: Excel.Interfaces.ChartFontUpdateData;
        }
        /** An interface for updating data on the ChartPlotArea object, for use in "chartPlotArea.set({ ... })". */
        interface ChartPlotAreaUpdateData {
            /**
            *
            * Represents the formatting of a chart plotArea.
            *
            * [Api set: ExcelApi 1.8]
            */
            format?: Excel.Interfaces.ChartPlotAreaFormatUpdateData;
            /**
             *
             * Represents the height value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            height?: number;
            /**
             *
             * Represents the insideHeight value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            insideHeight?: number;
            /**
             *
             * Represents the insideLeft value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            insideLeft?: number;
            /**
             *
             * Represents the insideTop value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            insideTop?: number;
            /**
             *
             * Represents the insideWidth value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            insideWidth?: number;
            /**
             *
             * Represents the left value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            left?: number;
            /**
             *
             * Represents the position of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: Excel.ChartPlotAreaPosition | "Automatic" | "Custom";
            /**
             *
             * Represents the top value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            top?: number;
            /**
             *
             * Represents the width value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            width?: number;
        }
        /** An interface for updating data on the ChartPlotAreaFormat object, for use in "chartPlotAreaFormat.set({ ... })". */
        interface ChartPlotAreaFormatUpdateData {
            /**
            *
            * Represents the border attributes of a chart plotArea.
            *
            * [Api set: ExcelApi 1.8]
            */
            border?: Excel.Interfaces.ChartBorderUpdateData;
        }
        /** An interface for updating data on the CustomXmlPartScopedCollection object, for use in "customXmlPartScopedCollection.set({ ... })". */
        interface CustomXmlPartScopedCollectionUpdateData {
            items?: Excel.Interfaces.CustomXmlPartData[];
        }
        /** An interface for updating data on the CustomXmlPartCollection object, for use in "customXmlPartCollection.set({ ... })". */
        interface CustomXmlPartCollectionUpdateData {
            items?: Excel.Interfaces.CustomXmlPartData[];
        }
        /** An interface for updating data on the PivotTableCollection object, for use in "pivotTableCollection.set({ ... })". */
        interface PivotTableCollectionUpdateData {
            items?: Excel.Interfaces.PivotTableData[];
        }
        /** An interface for updating data on the PivotTable object, for use in "pivotTable.set({ ... })". */
        interface PivotTableUpdateData {
            /**
             *
             * Specifies whether the PivotTable allows values in the data body to be edited by the user.
             *
             * [Api set: ExcelApi 1.9]
             */
            enableDataValueEditing?: boolean;
            /**
             *
             * Name of the PivotTable.
             *
             * [Api set: ExcelApi 1.3]
             */
            name?: string;
            /**
             *
             * Specifies whether the PivotTable uses custom lists when sorting.
             *
             * [Api set: ExcelApi 1.9]
             */
            useCustomSortLists?: boolean;
        }
        /** An interface for updating data on the PivotLayout object, for use in "pivotLayout.set({ ... })". */
        interface PivotLayoutUpdateData {
            /**
             *
             * Specifies whether formatting will be automatically formatted when it’s refreshed or when fields are moved
             *
             * [Api set: ExcelApi 1.9]
             */
            autoFormat?: boolean;
            /**
             *
             * This property indicates the PivotLayoutType of all fields on the PivotTable. If fields have different states, this will be null.
             *
             * [Api set: ExcelApi 1.8]
             */
            layoutType?: Excel.PivotLayoutType | "Compact" | "Tabular" | "Outline";
            /**
             *
             * Specifies whether formatting is preserved when the report is refreshed or recalculated by operations such as pivoting, sorting, or changing page field items.
             *
             * [Api set: ExcelApi 1.9]
             */
            preserveFormatting?: boolean;
            /**
             *
             * Specifies whether the PivotTable report shows grand totals for columns.
             *
             * [Api set: ExcelApi 1.8]
             */
            showColumnGrandTotals?: boolean;
            /**
             *
             * Specifies whether the PivotTable report shows grand totals for rows.
             *
             * [Api set: ExcelApi 1.8]
             */
            showRowGrandTotals?: boolean;
            /**
             *
             * This property indicates the SubtotalLocationType of all fields on the PivotTable. If fields have different states, this will be null.
             *
             * [Api set: ExcelApi 1.8]
             */
            subtotalLocation?: Excel.SubtotalLocationType | "AtTop" | "AtBottom" | "Off";
        }
        /** An interface for updating data on the PivotHierarchyCollection object, for use in "pivotHierarchyCollection.set({ ... })". */
        interface PivotHierarchyCollectionUpdateData {
            items?: Excel.Interfaces.PivotHierarchyData[];
        }
        /** An interface for updating data on the PivotHierarchy object, for use in "pivotHierarchy.set({ ... })". */
        interface PivotHierarchyUpdateData {
            /**
             *
             * Name of the PivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: string;
        }
        /** An interface for updating data on the RowColumnPivotHierarchyCollection object, for use in "rowColumnPivotHierarchyCollection.set({ ... })". */
        interface RowColumnPivotHierarchyCollectionUpdateData {
            items?: Excel.Interfaces.RowColumnPivotHierarchyData[];
        }
        /** An interface for updating data on the RowColumnPivotHierarchy object, for use in "rowColumnPivotHierarchy.set({ ... })". */
        interface RowColumnPivotHierarchyUpdateData {
            /**
             *
             * Name of the RowColumnPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: string;
            /**
             *
             * Position of the RowColumnPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: number;
        }
        /** An interface for updating data on the FilterPivotHierarchyCollection object, for use in "filterPivotHierarchyCollection.set({ ... })". */
        interface FilterPivotHierarchyCollectionUpdateData {
            items?: Excel.Interfaces.FilterPivotHierarchyData[];
        }
        /** An interface for updating data on the FilterPivotHierarchy object, for use in "filterPivotHierarchy.set({ ... })". */
        interface FilterPivotHierarchyUpdateData {
            /**
             *
             * Determines whether to allow multiple filter items.
             *
             * [Api set: ExcelApi 1.8]
             */
            enableMultipleFilterItems?: boolean;
            /**
             *
             * Name of the FilterPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: string;
            /**
             *
             * Position of the FilterPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: number;
        }
        /** An interface for updating data on the DataPivotHierarchyCollection object, for use in "dataPivotHierarchyCollection.set({ ... })". */
        interface DataPivotHierarchyCollectionUpdateData {
            items?: Excel.Interfaces.DataPivotHierarchyData[];
        }
        /** An interface for updating data on the DataPivotHierarchy object, for use in "dataPivotHierarchy.set({ ... })". */
        interface DataPivotHierarchyUpdateData {
            /**
            *
            * Returns the PivotFields associated with the DataPivotHierarchy.
            *
            * [Api set: ExcelApi 1.8]
            */
            field?: Excel.Interfaces.PivotFieldUpdateData;
            /**
             *
             * Name of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: string;
            /**
             *
             * Number format of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: string;
            /**
             *
             * Position of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: number;
            /**
             *
             * Determines whether the data should be shown as a specific summary calculation or not.
             *
             * [Api set: ExcelApi 1.8]
             */
            showAs?: Excel.ShowAsRule;
            /**
             *
             * Determines whether to show all items of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            summarizeBy?: Excel.AggregationFunction | "Unknown" | "Automatic" | "Sum" | "Count" | "Average" | "Max" | "Min" | "Product" | "CountNumbers" | "StandardDeviation" | "StandardDeviationP" | "Variance" | "VarianceP";
        }
        /** An interface for updating data on the PivotFieldCollection object, for use in "pivotFieldCollection.set({ ... })". */
        interface PivotFieldCollectionUpdateData {
            items?: Excel.Interfaces.PivotFieldData[];
        }
        /** An interface for updating data on the PivotField object, for use in "pivotField.set({ ... })". */
        interface PivotFieldUpdateData {
            /**
             *
             * Name of the PivotField.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: string;
            /**
             *
             * Determines whether to show all items of the PivotField.
             *
             * [Api set: ExcelApi 1.8]
             */
            showAllItems?: boolean;
            /**
             *
             * Subtotals of the PivotField.
             *
             * [Api set: ExcelApi 1.8]
             */
            subtotals?: Excel.Subtotals;
        }
        /** An interface for updating data on the PivotItemCollection object, for use in "pivotItemCollection.set({ ... })". */
        interface PivotItemCollectionUpdateData {
            items?: Excel.Interfaces.PivotItemData[];
        }
        /** An interface for updating data on the PivotItem object, for use in "pivotItem.set({ ... })". */
        interface PivotItemUpdateData {
            /**
             *
             * Determines whether the item is expanded to show child items or if it's collapsed and child items are hidden.
             *
             * [Api set: ExcelApi 1.8]
             */
            isExpanded?: boolean;
            /**
             *
             * Name of the PivotItem.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: string;
            /**
             *
             * Determines whether the PivotItem is visible or not.
             *
             * [Api set: ExcelApi 1.8]
             */
            visible?: boolean;
        }
        /** An interface for updating data on the DocumentProperties object, for use in "documentProperties.set({ ... })". */
        interface DocumentPropertiesUpdateData {
            /**
             *
             * Gets or sets the author of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            author?: string;
            /**
             *
             * Gets or sets the category of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            category?: string;
            /**
             *
             * Gets or sets the comments of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            comments?: string;
            /**
             *
             * Gets or sets the company of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            company?: string;
            /**
             *
             * Gets or sets the keywords of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            keywords?: string;
            /**
             *
             * Gets or sets the manager of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            manager?: string;
            /**
             *
             * Gets the revision number of the workbook. Read only.
             *
             * [Api set: ExcelApi 1.7]
             */
            revisionNumber?: number;
            /**
             *
             * Gets or sets the subject of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            subject?: string;
            /**
             *
             * Gets or sets the title of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            title?: string;
        }
        /** An interface for updating data on the CustomProperty object, for use in "customProperty.set({ ... })". */
        interface CustomPropertyUpdateData {
            /**
             *
             * Gets or sets the value of the custom property.
             *
             * [Api set: ExcelApi 1.7]
             */
            value?: any;
        }
        /** An interface for updating data on the CustomPropertyCollection object, for use in "customPropertyCollection.set({ ... })". */
        interface CustomPropertyCollectionUpdateData {
            items?: Excel.Interfaces.CustomPropertyData[];
        }
        /** An interface for updating data on the ConditionalFormatCollection object, for use in "conditionalFormatCollection.set({ ... })". */
        interface ConditionalFormatCollectionUpdateData {
            items?: Excel.Interfaces.ConditionalFormatData[];
        }
        /** An interface for updating data on the ConditionalFormat object, for use in "conditionalFormat.set({ ... })". */
        interface ConditionalFormatUpdateData {
            /**
            *
            * Returns the cell value conditional format properties if the current conditional format is a CellValue type.
            For example to format all cells between 5 and 10.
            *
            * [Api set: ExcelApi 1.6]
            */
            cellValue?: Excel.Interfaces.CellValueConditionalFormatUpdateData;
            /**
            *
            * Returns the cell value conditional format properties if the current conditional format is a CellValue type.
            For example to format all cells between 5 and 10.
            *
            * [Api set: ExcelApi 1.6]
            */
            cellValueOrNullObject?: Excel.Interfaces.CellValueConditionalFormatUpdateData;
            /**
            *
            * Returns the ColorScale conditional format properties if the current conditional format is an ColorScale type.
            *
            * [Api set: ExcelApi 1.6]
            */
            colorScale?: Excel.Interfaces.ColorScaleConditionalFormatUpdateData;
            /**
            *
            * Returns the ColorScale conditional format properties if the current conditional format is an ColorScale type.
            *
            * [Api set: ExcelApi 1.6]
            */
            colorScaleOrNullObject?: Excel.Interfaces.ColorScaleConditionalFormatUpdateData;
            /**
            *
            * Returns the custom conditional format properties if the current conditional format is a custom type.
            *
            * [Api set: ExcelApi 1.6]
            */
            custom?: Excel.Interfaces.CustomConditionalFormatUpdateData;
            /**
            *
            * Returns the custom conditional format properties if the current conditional format is a custom type.
            *
            * [Api set: ExcelApi 1.6]
            */
            customOrNullObject?: Excel.Interfaces.CustomConditionalFormatUpdateData;
            /**
            *
            * Returns the data bar properties if the current conditional format is a data bar.
            *
            * [Api set: ExcelApi 1.6]
            */
            dataBar?: Excel.Interfaces.DataBarConditionalFormatUpdateData;
            /**
            *
            * Returns the data bar properties if the current conditional format is a data bar.
            *
            * [Api set: ExcelApi 1.6]
            */
            dataBarOrNullObject?: Excel.Interfaces.DataBarConditionalFormatUpdateData;
            /**
            *
            * Returns the IconSet conditional format properties if the current conditional format is an IconSet type.
            *
            * [Api set: ExcelApi 1.6]
            */
            iconSet?: Excel.Interfaces.IconSetConditionalFormatUpdateData;
            /**
            *
            * Returns the IconSet conditional format properties if the current conditional format is an IconSet type.
            *
            * [Api set: ExcelApi 1.6]
            */
            iconSetOrNullObject?: Excel.Interfaces.IconSetConditionalFormatUpdateData;
            /**
            *
            * Returns the preset criteria conditional format. See Excel.PresetCriteriaConditionalFormat for more details.
            *
            * [Api set: ExcelApi 1.6]
            */
            preset?: Excel.Interfaces.PresetCriteriaConditionalFormatUpdateData;
            /**
            *
            * Returns the preset criteria conditional format. See Excel.PresetCriteriaConditionalFormat for more details.
            *
            * [Api set: ExcelApi 1.6]
            */
            presetOrNullObject?: Excel.Interfaces.PresetCriteriaConditionalFormatUpdateData;
            /**
            *
            * Returns the specific text conditional format properties if the current conditional format is a text type.
            For example to format cells matching the word "Text".
            *
            * [Api set: ExcelApi 1.6]
            */
            textComparison?: Excel.Interfaces.TextConditionalFormatUpdateData;
            /**
            *
            * Returns the specific text conditional format properties if the current conditional format is a text type.
            For example to format cells matching the word "Text".
            *
            * [Api set: ExcelApi 1.6]
            */
            textComparisonOrNullObject?: Excel.Interfaces.TextConditionalFormatUpdateData;
            /**
            *
            * Returns the Top/Bottom conditional format properties if the current conditional format is an TopBottom type.
            For example to format the top 10% or bottom 10 items.
            *
            * [Api set: ExcelApi 1.6]
            */
            topBottom?: Excel.Interfaces.TopBottomConditionalFormatUpdateData;
            /**
            *
            * Returns the Top/Bottom conditional format properties if the current conditional format is an TopBottom type.
            For example to format the top 10% or bottom 10 items.
            *
            * [Api set: ExcelApi 1.6]
            */
            topBottomOrNullObject?: Excel.Interfaces.TopBottomConditionalFormatUpdateData;
            /**
             *
             * The priority (or index) within the conditional format collection that this conditional format currently exists in. Changing this also
            changes other conditional formats' priorities, to allow for a contiguous priority order.
            Use a negative priority to begin from the back.
            Priorities greater than than bounds will get and set to the maximum (or minimum if negative) priority.
            Also note that if you change the priority, you have to re-fetch a new copy of the object at that new priority location if you want to make further changes to it. Read-only.
             *
             * [Api set: ExcelApi 1.6]
             */
            priority?: number;
            /**
             *
             * If the conditions of this conditional format are met, no lower-priority formats shall take effect on that cell.
            Null on databars, icon sets, and colorscales as there's no concept of StopIfTrue for these
             *
             * [Api set: ExcelApi 1.6]
             */
            stopIfTrue?: boolean;
        }
        /** An interface for updating data on the DataBarConditionalFormat object, for use in "dataBarConditionalFormat.set({ ... })". */
        interface DataBarConditionalFormatUpdateData {
            /**
            *
            * Representation of all values to the left of the axis in an Excel data bar.
            *
            * [Api set: ExcelApi 1.6]
            */
            negativeFormat?: Excel.Interfaces.ConditionalDataBarNegativeFormatUpdateData;
            /**
            *
            * Representation of all values to the right of the axis in an Excel data bar.
            *
            * [Api set: ExcelApi 1.6]
            */
            positiveFormat?: Excel.Interfaces.ConditionalDataBarPositiveFormatUpdateData;
            /**
             *
             * HTML color code representing the color of the Axis line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            "" (empty string) if no axis is present or set.
             *
             * [Api set: ExcelApi 1.6]
             */
            axisColor?: string;
            /**
             *
             * Representation of how the axis is determined for an Excel data bar.
             *
             * [Api set: ExcelApi 1.6]
             */
            axisFormat?: Excel.ConditionalDataBarAxisFormat | "Automatic" | "None" | "CellMidPoint";
            /**
             *
             * Represents the direction that the data bar graphic should be based on.
             *
             * [Api set: ExcelApi 1.6]
             */
            barDirection?: Excel.ConditionalDataBarDirection | "Context" | "LeftToRight" | "RightToLeft";
            /**
             *
             * The rule for what consistutes the lower bound (and how to calculate it, if applicable) for a data bar.
             *
             * [Api set: ExcelApi 1.6]
             */
            lowerBoundRule?: Excel.ConditionalDataBarRule;
            /**
             *
             * If true, hides the values from the cells where the data bar is applied.
             *
             * [Api set: ExcelApi 1.6]
             */
            showDataBarOnly?: boolean;
            /**
             *
             * The rule for what constitutes the upper bound (and how to calculate it, if applicable) for a data bar.
             *
             * [Api set: ExcelApi 1.6]
             */
            upperBoundRule?: Excel.ConditionalDataBarRule;
        }
        /** An interface for updating data on the ConditionalDataBarPositiveFormat object, for use in "conditionalDataBarPositiveFormat.set({ ... })". */
        interface ConditionalDataBarPositiveFormatUpdateData {
            /**
             *
             * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            "" (empty string) if no border is present or set.
             *
             * [Api set: ExcelApi 1.6]
             */
            borderColor?: string;
            /**
             *
             * HTML color code representing the fill color, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.6]
             */
            fillColor?: string;
            /**
             *
             * Boolean representation of whether or not the DataBar has a gradient.
             *
             * [Api set: ExcelApi 1.6]
             */
            gradientFill?: boolean;
        }
        /** An interface for updating data on the ConditionalDataBarNegativeFormat object, for use in "conditionalDataBarNegativeFormat.set({ ... })". */
        interface ConditionalDataBarNegativeFormatUpdateData {
            /**
             *
             * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            "Empty String" if no border is present or set.
             *
             * [Api set: ExcelApi 1.6]
             */
            borderColor?: string;
            /**
             *
             * HTML color code representing the fill color, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.6]
             */
            fillColor?: string;
            /**
             *
             * Boolean representation of whether or not the negative DataBar has the same border color as the positive DataBar.
             *
             * [Api set: ExcelApi 1.6]
             */
            matchPositiveBorderColor?: boolean;
            /**
             *
             * Boolean representation of whether or not the negative DataBar has the same fill color as the positive DataBar.
             *
             * [Api set: ExcelApi 1.6]
             */
            matchPositiveFillColor?: boolean;
        }
        /** An interface for updating data on the CustomConditionalFormat object, for use in "customConditionalFormat.set({ ... })". */
        interface CustomConditionalFormatUpdateData {
            /**
            *
            * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties.
            *
            * [Api set: ExcelApi 1.6]
            */
            format?: Excel.Interfaces.ConditionalRangeFormatUpdateData;
            /**
            *
            * Represents the Rule object on this conditional format.
            *
            * [Api set: ExcelApi 1.6]
            */
            rule?: Excel.Interfaces.ConditionalFormatRuleUpdateData;
        }
        /** An interface for updating data on the ConditionalFormatRule object, for use in "conditionalFormatRule.set({ ... })". */
        interface ConditionalFormatRuleUpdateData {
            /**
             *
             * The formula, if required, to evaluate the conditional format rule on.
             *
             * [Api set: ExcelApi 1.6]
             */
            formula?: string;
            /**
             *
             * The formula, if required, to evaluate the conditional format rule on in the user's language.
             *
             * [Api set: ExcelApi 1.6]
             */
            formulaLocal?: string;
            /**
             *
             * The formula, if required, to evaluate the conditional format rule on in R1C1-style notation.
             *
             * [Api set: ExcelApi 1.6]
             */
            formulaR1C1?: string;
        }
        /** An interface for updating data on the IconSetConditionalFormat object, for use in "iconSetConditionalFormat.set({ ... })". */
        interface IconSetConditionalFormatUpdateData {
            /**
             *
             * An array of Criteria and IconSets for the rules and potential custom icons for conditional icons. Note that for the first criterion only the custom icon can be modified, while type, formula, and operator will be ignored when set.
             *
             * [Api set: ExcelApi 1.6]
             */
            criteria?: Excel.ConditionalIconCriterion[];
            /**
             *
             * If true, reverses the icon orders for the IconSet. Note that this cannot be set if custom icons are used.
             *
             * [Api set: ExcelApi 1.6]
             */
            reverseIconOrder?: boolean;
            /**
             *
             * If true, hides the values and only shows icons.
             *
             * [Api set: ExcelApi 1.6]
             */
            showIconOnly?: boolean;
            /**
             *
             * If set, displays the IconSet option for the conditional format.
             *
             * [Api set: ExcelApi 1.6]
             */
            style?: Excel.IconSet | "Invalid" | "ThreeArrows" | "ThreeArrowsGray" | "ThreeFlags" | "ThreeTrafficLights1" | "ThreeTrafficLights2" | "ThreeSigns" | "ThreeSymbols" | "ThreeSymbols2" | "FourArrows" | "FourArrowsGray" | "FourRedToBlack" | "FourRating" | "FourTrafficLights" | "FiveArrows" | "FiveArrowsGray" | "FiveRating" | "FiveQuarters" | "ThreeStars" | "ThreeTriangles" | "FiveBoxes" | "LinkedEntityFinanceIcon" | "LinkedEntityMapIcon";
        }
        /** An interface for updating data on the ColorScaleConditionalFormat object, for use in "colorScaleConditionalFormat.set({ ... })". */
        interface ColorScaleConditionalFormatUpdateData {
            /**
             *
             * The criteria of the color scale. Midpoint is optional when using a two point color scale.
             *
             * [Api set: ExcelApi 1.6]
             */
            criteria?: Excel.ConditionalColorScaleCriteria;
        }
        /** An interface for updating data on the TopBottomConditionalFormat object, for use in "topBottomConditionalFormat.set({ ... })". */
        interface TopBottomConditionalFormatUpdateData {
            /**
            *
            * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties.
            *
            * [Api set: ExcelApi 1.6]
            */
            format?: Excel.Interfaces.ConditionalRangeFormatUpdateData;
            /**
             *
             * The criteria of the Top/Bottom conditional format.
             *
             * [Api set: ExcelApi 1.6]
             */
            rule?: Excel.ConditionalTopBottomRule;
        }
        /** An interface for updating data on the PresetCriteriaConditionalFormat object, for use in "presetCriteriaConditionalFormat.set({ ... })". */
        interface PresetCriteriaConditionalFormatUpdateData {
            /**
            *
            * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties.
            *
            * [Api set: ExcelApi 1.6]
            */
            format?: Excel.Interfaces.ConditionalRangeFormatUpdateData;
            /**
             *
             * The rule of the conditional format.
             *
             * [Api set: ExcelApi 1.6]
             */
            rule?: Excel.ConditionalPresetCriteriaRule;
        }
        /** An interface for updating data on the TextConditionalFormat object, for use in "textConditionalFormat.set({ ... })". */
        interface TextConditionalFormatUpdateData {
            /**
            *
            * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties.
            *
            * [Api set: ExcelApi 1.6]
            */
            format?: Excel.Interfaces.ConditionalRangeFormatUpdateData;
            /**
             *
             * The rule of the conditional format.
             *
             * [Api set: ExcelApi 1.6]
             */
            rule?: Excel.ConditionalTextComparisonRule;
        }
        /** An interface for updating data on the CellValueConditionalFormat object, for use in "cellValueConditionalFormat.set({ ... })". */
        interface CellValueConditionalFormatUpdateData {
            /**
            *
            * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties.
            *
            * [Api set: ExcelApi 1.6]
            */
            format?: Excel.Interfaces.ConditionalRangeFormatUpdateData;
            /**
             *
             * Represents the Rule object on this conditional format.
             *
             * [Api set: ExcelApi 1.6]
             */
            rule?: Excel.ConditionalCellValueRule;
        }
        /** An interface for updating data on the ConditionalRangeFormat object, for use in "conditionalRangeFormat.set({ ... })". */
        interface ConditionalRangeFormatUpdateData {
            /**
            *
            * Collection of border objects that apply to the overall conditional format range.
            *
            * [Api set: ExcelApi 1.6]
            */
            borders?: Excel.Interfaces.ConditionalRangeBorderCollectionUpdateData;
            /**
            *
            * Returns the fill object defined on the overall conditional format range.
            *
            * [Api set: ExcelApi 1.6]
            */
            fill?: Excel.Interfaces.ConditionalRangeFillUpdateData;
            /**
            *
            * Returns the font object defined on the overall conditional format range.
            *
            * [Api set: ExcelApi 1.6]
            */
            font?: Excel.Interfaces.ConditionalRangeFontUpdateData;
            /**
             *
             * Represents Excel's number format code for the given range. Cleared if null is passed in.
             *
             * [Api set: ExcelApi 1.6]
             */
            numberFormat?: any;
        }
        /** An interface for updating data on the ConditionalRangeFont object, for use in "conditionalRangeFont.set({ ... })". */
        interface ConditionalRangeFontUpdateData {
            /**
             *
             * Represents the bold status of font.
             *
             * [Api set: ExcelApi 1.6]
             */
            bold?: boolean;
            /**
             *
             * HTML color code representation of the text color. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.6]
             */
            color?: string;
            /**
             *
             * Represents the italic status of the font.
             *
             * [Api set: ExcelApi 1.6]
             */
            italic?: boolean;
            /**
             *
             * Represents the strikethrough status of the font.
             *
             * [Api set: ExcelApi 1.6]
             */
            strikethrough?: boolean;
            /**
             *
             * Type of underline applied to the font. See Excel.ConditionalRangeFontUnderlineStyle for details.
             *
             * [Api set: ExcelApi 1.6]
             */
            underline?: Excel.ConditionalRangeFontUnderlineStyle | "None" | "Single" | "Double";
        }
        /** An interface for updating data on the ConditionalRangeFill object, for use in "conditionalRangeFill.set({ ... })". */
        interface ConditionalRangeFillUpdateData {
            /**
             *
             * HTML color code representing the color of the fill, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.6]
             */
            color?: string;
        }
        /** An interface for updating data on the ConditionalRangeBorder object, for use in "conditionalRangeBorder.set({ ... })". */
        interface ConditionalRangeBorderUpdateData {
            /**
             *
             * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.6]
             */
            color?: string;
            /**
             *
             * One of the constants of line style specifying the line style for the border. See Excel.BorderLineStyle for details.
             *
             * [Api set: ExcelApi 1.6]
             */
            style?: Excel.ConditionalRangeBorderLineStyle | "None" | "Continuous" | "Dash" | "DashDot" | "DashDotDot" | "Dot";
        }
        /** An interface for updating data on the ConditionalRangeBorderCollection object, for use in "conditionalRangeBorderCollection.set({ ... })". */
        interface ConditionalRangeBorderCollectionUpdateData {
            /**
            *
            * Gets the bottom border.
            *
            * [Api set: ExcelApi 1.6]
            */
            bottom?: Excel.Interfaces.ConditionalRangeBorderUpdateData;
            /**
            *
            * Gets the left border.
            *
            * [Api set: ExcelApi 1.6]
            */
            left?: Excel.Interfaces.ConditionalRangeBorderUpdateData;
            /**
            *
            * Gets the right border.
            *
            * [Api set: ExcelApi 1.6]
            */
            right?: Excel.Interfaces.ConditionalRangeBorderUpdateData;
            /**
            *
            * Gets the top border.
            *
            * [Api set: ExcelApi 1.6]
            */
            top?: Excel.Interfaces.ConditionalRangeBorderUpdateData;
            items?: Excel.Interfaces.ConditionalRangeBorderData[];
        }
        /** An interface for updating data on the Style object, for use in "style.set({ ... })". */
        interface StyleUpdateData {
            /**
            *
            * A Border collection of four Border objects that represent the style of the four borders.
            *
            * [Api set: ExcelApi 1.7]
            */
            borders?: Excel.Interfaces.RangeBorderCollectionUpdateData;
            /**
            *
            * The Fill of the style.
            *
            * [Api set: ExcelApi 1.7]
            */
            fill?: Excel.Interfaces.RangeFillUpdateData;
            /**
            *
            * A Font object that represents the font of the style.
            *
            * [Api set: ExcelApi 1.7]
            */
            font?: Excel.Interfaces.RangeFontUpdateData;
            /**
             *
             * Indicates if text is automatically indented when the text alignment in a cell is set to equal distribution.
             *
             * [Api set: ExcelApi 1.8]
             */
            autoIndent?: boolean;
            /**
             *
             * Indicates if the formula will be hidden when the worksheet is protected.
             *
             * [Api set: ExcelApi 1.7]
             */
            formulaHidden?: boolean;
            /**
             *
             * Represents the horizontal alignment for the style. See Excel.HorizontalAlignment for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            horizontalAlignment?: Excel.HorizontalAlignment | "General" | "Left" | "Center" | "Right" | "Fill" | "Justify" | "CenterAcrossSelection" | "Distributed";
            /**
             *
             * Indicates if the style includes the AutoIndent, HorizontalAlignment, VerticalAlignment, WrapText, IndentLevel, and TextOrientation properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeAlignment?: boolean;
            /**
             *
             * Indicates if the style includes the Color, ColorIndex, LineStyle, and Weight border properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeBorder?: boolean;
            /**
             *
             * Indicates if the style includes the Background, Bold, Color, ColorIndex, FontStyle, Italic, Name, Size, Strikethrough, Subscript, Superscript, and Underline font properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeFont?: boolean;
            /**
             *
             * Indicates if the style includes the NumberFormat property.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeNumber?: boolean;
            /**
             *
             * Indicates if the style includes the Color, ColorIndex, InvertIfNegative, Pattern, PatternColor, and PatternColorIndex interior properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includePatterns?: boolean;
            /**
             *
             * Indicates if the style includes the FormulaHidden and Locked protection properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeProtection?: boolean;
            /**
             *
             * An integer from 0 to 250 that indicates the indent level for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            indentLevel?: number;
            /**
             *
             * Indicates if the object is locked when the worksheet is protected.
             *
             * [Api set: ExcelApi 1.7]
             */
            locked?: boolean;
            /**
             *
             * The format code of the number format for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            numberFormat?: string;
            /**
             *
             * The localized format code of the number format for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            numberFormatLocal?: string;
            /**
             *
             * The reading order for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            readingOrder?: Excel.ReadingOrder | "Context" | "LeftToRight" | "RightToLeft";
            /**
             *
             * Indicates if text automatically shrinks to fit in the available column width.
             *
             * [Api set: ExcelApi 1.7]
             */
            shrinkToFit?: boolean;
            /**
             *
             * The text orientation for the style.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: number;
            /**
             *
             * Represents the vertical alignment for the style. See Excel.VerticalAlignment for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            verticalAlignment?: Excel.VerticalAlignment | "Top" | "Center" | "Bottom" | "Justify" | "Distributed";
            /**
             *
             * Indicates if Microsoft Excel wraps the text in the object.
             *
             * [Api set: ExcelApi 1.7]
             */
            wrapText?: boolean;
        }
        /** An interface for updating data on the StyleCollection object, for use in "styleCollection.set({ ... })". */
        interface StyleCollectionUpdateData {
            items?: Excel.Interfaces.StyleData[];
        }
        /** An interface for updating data on the PageLayout object, for use in "pageLayout.set({ ... })". */
        interface PageLayoutUpdateData {
            /**
            *
            * Header and footer configuration for the worksheet.
            *
            * [Api set: ExcelApi 1.9]
            */
            headersFooters?: Excel.Interfaces.HeaderFooterGroupUpdateData;
            /**
             *
             * Gets or sets the worksheet's black and white print option.
             *
             * [Api set: ExcelApi 1.9]
             */
            blackAndWhite?: boolean;
            /**
             *
             * Gets or sets the worksheet's bottom page margin to use for printing in points.
             *
             * [Api set: ExcelApi 1.9]
             */
            bottomMargin?: number;
            /**
             *
             * Gets or sets the worksheet's center horizontally flag. This flag determines whether the worksheet will be centered horizontally when it's printed.
             *
             * [Api set: ExcelApi 1.9]
             */
            centerHorizontally?: boolean;
            /**
             *
             * Gets or sets the worksheet's center vertically flag. This flag determines whether the worksheet will be centered vertically when it's printed.
             *
             * [Api set: ExcelApi 1.9]
             */
            centerVertically?: boolean;
            /**
             *
             * Gets or sets the worksheet's draft mode option. If true the sheet will be printed without graphics.
             *
             * [Api set: ExcelApi 1.9]
             */
            draftMode?: boolean;
            /**
             *
             * Gets or sets the worksheet's first page number to print. Null value represents "auto" page numbering.
             *
             * [Api set: ExcelApi 1.9]
             */
            firstPageNumber?: number | "";
            /**
             *
             * Gets or sets the worksheet's footer margin, in points, for use when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            footerMargin?: number;
            /**
             *
             * Gets or sets the worksheet's header margin, in points, for use when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            headerMargin?: number;
            /**
             *
             * Gets or sets the worksheet's left margin, in points, for use when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            leftMargin?: number;
            /**
             *
             * Gets or sets the worksheet's orientation of the page.
             *
             * [Api set: ExcelApi 1.9]
             */
            orientation?: Excel.PageOrientation | "Portrait" | "Landscape";
            /**
             *
             * Gets or sets the worksheet's paper size of the page.
             *
             * [Api set: ExcelApi 1.9]
             */
            paperSize?: Excel.PaperType | "Letter" | "LetterSmall" | "Tabloid" | "Ledger" | "Legal" | "Statement" | "Executive" | "A3" | "A4" | "A4Small" | "A5" | "B4" | "B5" | "Folio" | "Quatro" | "Paper10x14" | "Paper11x17" | "Note" | "Envelope9" | "Envelope10" | "Envelope11" | "Envelope12" | "Envelope14" | "Csheet" | "Dsheet" | "Esheet" | "EnvelopeDL" | "EnvelopeC5" | "EnvelopeC3" | "EnvelopeC4" | "EnvelopeC6" | "EnvelopeC65" | "EnvelopeB4" | "EnvelopeB5" | "EnvelopeB6" | "EnvelopeItaly" | "EnvelopeMonarch" | "EnvelopePersonal" | "FanfoldUS" | "FanfoldStdGerman" | "FanfoldLegalGerman";
            /**
             *
             * Gets or sets whether the worksheet's comments should be displayed when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            printComments?: Excel.PrintComments | "NoComments" | "EndSheet" | "InPlace";
            /**
             *
             * Gets or sets the worksheet's print errors option.
             *
             * [Api set: ExcelApi 1.9]
             */
            printErrors?: Excel.PrintErrorType | "AsDisplayed" | "Blank" | "Dash" | "NotAvailable";
            /**
             *
             * Gets or sets the worksheet's print gridlines flag. This flag determines whether gridlines will be printed or not.
             *
             * [Api set: ExcelApi 1.9]
             */
            printGridlines?: boolean;
            /**
             *
             * Gets or sets the worksheet's print headings flag. This flag determines whether headings will be printed or not.
             *
             * [Api set: ExcelApi 1.9]
             */
            printHeadings?: boolean;
            /**
             *
             * Gets or sets the worksheet's page print order option. This specifies the order to use for processing the page number printed.
             *
             * [Api set: ExcelApi 1.9]
             */
            printOrder?: Excel.PrintOrder | "DownThenOver" | "OverThenDown";
            /**
             *
             * Gets or sets the worksheet's right margin, in points, for use when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            rightMargin?: number;
            /**
             *
             * Gets or sets the worksheet's top margin, in points, for use when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            topMargin?: number;
            /**
             *
             * Gets or sets the worksheet's print zoom options.
             *
             * [Api set: ExcelApi 1.9]
             */
            zoom?: Excel.PageLayoutZoomOptions;
        }
        /** An interface for updating data on the HeaderFooter object, for use in "headerFooter.set({ ... })". */
        interface HeaderFooterUpdateData {
            /**
             *
             * Gets or sets the center footer of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            centerFooter?: string;
            /**
             *
             * Gets or sets the center header of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            centerHeader?: string;
            /**
             *
             * Gets or sets the left footer of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            leftFooter?: string;
            /**
             *
             * Gets or sets the left header of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            leftHeader?: string;
            /**
             *
             * Gets or sets the right footer of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            rightFooter?: string;
            /**
             *
             * Gets or sets the right header of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            rightHeader?: string;
        }
        /** An interface for updating data on the HeaderFooterGroup object, for use in "headerFooterGroup.set({ ... })". */
        interface HeaderFooterGroupUpdateData {
            /**
            *
            * The general header/footer, used for all pages unless even/odd or first page is specified.
            *
            * [Api set: ExcelApi 1.9]
            */
            defaultForAllPages?: Excel.Interfaces.HeaderFooterUpdateData;
            /**
            *
            * The header/footer to use for even pages, odd header/footer needs to be specified for odd pages.
            *
            * [Api set: ExcelApi 1.9]
            */
            evenPages?: Excel.Interfaces.HeaderFooterUpdateData;
            /**
            *
            * The first page header/footer, for all other pages general or even/odd is used.
            *
            * [Api set: ExcelApi 1.9]
            */
            firstPage?: Excel.Interfaces.HeaderFooterUpdateData;
            /**
            *
            * The header/footer to use for odd pages, even header/footer needs to be specified for even pages.
            *
            * [Api set: ExcelApi 1.9]
            */
            oddPages?: Excel.Interfaces.HeaderFooterUpdateData;
            /**
             *
             * Gets or sets the state of which headers/footers are set. See Excel.HeaderFooterState for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            state?: Excel.HeaderFooterState | "Default" | "FirstAndDefault" | "OddAndEven" | "FirstOddAndEven";
            /**
             *
             * Gets or sets a flag indicating if headers/footers are aligned with the page margins set in the page layout options for the worksheet.
             *
             * [Api set: ExcelApi 1.9]
             */
            useSheetMargins?: boolean;
            /**
             *
             * Gets or sets a flag indicating if headers/footers should be scaled by the page percentage scale set in the page layout options for the worksheet.
             *
             * [Api set: ExcelApi 1.9]
             */
            useSheetScale?: boolean;
        }
        /** An interface for updating data on the PageBreakCollection object, for use in "pageBreakCollection.set({ ... })". */
        interface PageBreakCollectionUpdateData {
            items?: Excel.Interfaces.PageBreakData[];
        }
        /** An interface for updating data on the RangeCollection object, for use in "rangeCollection.set({ ... })". */
        interface RangeCollectionUpdateData {
            items?: Excel.Interfaces.RangeData[];
        }
        /** An interface for updating data on the ShapeCollection object, for use in "shapeCollection.set({ ... })". */
        interface ShapeCollectionUpdateData {
            items?: Excel.Interfaces.ShapeData[];
        }
        /** An interface for updating data on the Shape object, for use in "shape.set({ ... })". */
        interface ShapeUpdateData {
            /**
            *
            * Returns the fill formatting of this shape.
            *
            * [Api set: ExcelApi 1.9]
            */
            fill?: Excel.Interfaces.ShapeFillUpdateData;
            /**
            *
            * Returns the line formatting of this shape.
            *
            * [Api set: ExcelApi 1.9]
            */
            lineFormat?: Excel.Interfaces.ShapeLineFormatUpdateData;
            /**
             *
             * Returns or sets the alternative description text for a Shape object.
             *
             * [Api set: ExcelApi 1.9]
             */
            altTextDescription?: string;
            /**
             *
             * Returns or sets the alternative title text for a Shape object.
             *
             * [Api set: ExcelApi 1.9]
             */
            altTextTitle?: string;
            /**
             *
             * Represents the geometric shape type of this geometric shape. See Excel.GeometricShapeType for details. Returns null if the shape type is not "GeometricShape".
             *
             * [Api set: ExcelApi 1.9]
             */
            geometricShapeType?: Excel.GeometricShapeType | "LineInverse" | "Triangle" | "RightTriangle" | "Rectangle" | "Diamond" | "Parallelogram" | "Trapezoid" | "NonIsoscelesTrapezoid" | "Pentagon" | "Hexagon" | "Heptagon" | "Octagon" | "Decagon" | "Dodecagon" | "Star4" | "Star5" | "Star6" | "Star7" | "Star8" | "Star10" | "Star12" | "Star16" | "Star24" | "Star32" | "RoundRectangle" | "Round1Rectangle" | "Round2SameRectangle" | "Round2DiagonalRectangle" | "SnipRoundRectangle" | "Snip1Rectangle" | "Snip2SameRectangle" | "Snip2DiagonalRectangle" | "Plaque" | "Ellipse" | "Teardrop" | "HomePlate" | "Chevron" | "PieWedge" | "Pie" | "BlockArc" | "Donut" | "NoSmoking" | "RightArrow" | "LeftArrow" | "UpArrow" | "DownArrow" | "StripedRightArrow" | "NotchedRightArrow" | "BentUpArrow" | "LeftRightArrow" | "UpDownArrow" | "LeftUpArrow" | "LeftRightUpArrow" | "QuadArrow" | "LeftArrowCallout" | "RightArrowCallout" | "UpArrowCallout" | "DownArrowCallout" | "LeftRightArrowCallout" | "UpDownArrowCallout" | "QuadArrowCallout" | "BentArrow" | "UturnArrow" | "CircularArrow" | "LeftCircularArrow" | "LeftRightCircularArrow" | "CurvedRightArrow" | "CurvedLeftArrow" | "CurvedUpArrow" | "CurvedDownArrow" | "SwooshArrow" | "Cube" | "Can" | "LightningBolt" | "Heart" | "Sun" | "Moon" | "SmileyFace" | "IrregularSeal1" | "IrregularSeal2" | "FoldedCorner" | "Bevel" | "Frame" | "HalfFrame" | "Corner" | "DiagonalStripe" | "Chord" | "Arc" | "LeftBracket" | "RightBracket" | "LeftBrace" | "RightBrace" | "BracketPair" | "BracePair" | "Callout1" | "Callout2" | "Callout3" | "AccentCallout1" | "AccentCallout2" | "AccentCallout3" | "BorderCallout1" | "BorderCallout2" | "BorderCallout3" | "AccentBorderCallout1" | "AccentBorderCallout2" | "AccentBorderCallout3" | "WedgeRectCallout" | "WedgeRRectCallout" | "WedgeEllipseCallout" | "CloudCallout" | "Cloud" | "Ribbon" | "Ribbon2" | "EllipseRibbon" | "EllipseRibbon2" | "LeftRightRibbon" | "VerticalScroll" | "HorizontalScroll" | "Wave" | "DoubleWave" | "Plus" | "FlowChartProcess" | "FlowChartDecision" | "FlowChartInputOutput" | "FlowChartPredefinedProcess" | "FlowChartInternalStorage" | "FlowChartDocument" | "FlowChartMultidocument" | "FlowChartTerminator" | "FlowChartPreparation" | "FlowChartManualInput" | "FlowChartManualOperation" | "FlowChartConnector" | "FlowChartPunchedCard" | "FlowChartPunchedTape" | "FlowChartSummingJunction" | "FlowChartOr" | "FlowChartCollate" | "FlowChartSort" | "FlowChartExtract" | "FlowChartMerge" | "FlowChartOfflineStorage" | "FlowChartOnlineStorage" | "FlowChartMagneticTape" | "FlowChartMagneticDisk" | "FlowChartMagneticDrum" | "FlowChartDisplay" | "FlowChartDelay" | "FlowChartAlternateProcess" | "FlowChartOffpageConnector" | "ActionButtonBlank" | "ActionButtonHome" | "ActionButtonHelp" | "ActionButtonInformation" | "ActionButtonForwardNext" | "ActionButtonBackPrevious" | "ActionButtonEnd" | "ActionButtonBeginning" | "ActionButtonReturn" | "ActionButtonDocument" | "ActionButtonSound" | "ActionButtonMovie" | "Gear6" | "Gear9" | "Funnel" | "MathPlus" | "MathMinus" | "MathMultiply" | "MathDivide" | "MathEqual" | "MathNotEqual" | "CornerTabs" | "SquareTabs" | "PlaqueTabs" | "ChartX" | "ChartStar" | "ChartPlus";
            /**
             *
             * Represents the height, in points, of the shape.
            Throws an invalid argument exception when set with a negative value or zero as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            height?: number;
            /**
             *
             * The distance, in points, from the left side of the shape to the left side of the worksheet.
            Throws an invalid argument exception when set with a negative value as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            left?: number;
            /**
             *
             * Specifies whether or not the aspect ratio of this shape is locked.
             *
             * [Api set: ExcelApi 1.9]
             */
            lockAspectRatio?: boolean;
            /**
             *
             * Represents the name of the shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            name?: string;
            /**
             *
             * Represents the rotation, in degrees, of the shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            rotation?: number;
            /**
             *
             * The distance, in points, from the top edge of the shape to the top edge of the worksheet.
            Throws an invalid argument exception when set with a negative value as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            top?: number;
            /**
             *
             * Represents the visibility of this shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            visible?: boolean;
            /**
             *
             * Represents the width, in points, of the shape.
            Throws an invalid argument exception when set with a negative value or zero as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            width?: number;
        }
        /** An interface for updating data on the GroupShapeCollection object, for use in "groupShapeCollection.set({ ... })". */
        interface GroupShapeCollectionUpdateData {
            items?: Excel.Interfaces.ShapeData[];
        }
        /** An interface for updating data on the Line object, for use in "line.set({ ... })". */
        interface LineUpdateData {
            /**
             *
             * Represents the length of the arrowhead at the beginning of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            beginArrowheadLength?: Excel.ArrowheadLength | "Short" | "Medium" | "Long";
            /**
             *
             * Represents the style of the arrowhead at the beginning of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            beginArrowheadStyle?: Excel.ArrowheadStyle | "None" | "Triangle" | "Stealth" | "Diamond" | "Oval" | "Open";
            /**
             *
             * Represents the width of the arrowhead at the beginning of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            beginArrowheadWidth?: Excel.ArrowheadWidth | "Narrow" | "Medium" | "Wide";
            /**
             *
             * Represents the length of the arrowhead at the end of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            endArrowheadLength?: Excel.ArrowheadLength | "Short" | "Medium" | "Long";
            /**
             *
             * Represents the style of the arrowhead at the end of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            endArrowheadStyle?: Excel.ArrowheadStyle | "None" | "Triangle" | "Stealth" | "Diamond" | "Oval" | "Open";
            /**
             *
             * Represents the width of the arrowhead at the end of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            endArrowheadWidth?: Excel.ArrowheadWidth | "Narrow" | "Medium" | "Wide";
            /**
             *
             * Represents the connector type for the line.
             *
             * [Api set: ExcelApi 1.9]
             */
            connectorType?: Excel.ConnectorType | "Straight" | "Elbow" | "Curve";
        }
        /** An interface for updating data on the ShapeFill object, for use in "shapeFill.set({ ... })". */
        interface ShapeFillUpdateData {
            /**
             *
             * Represents the shape fill foreground color in HTML color format, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange")
             *
             * [Api set: ExcelApi 1.9]
             */
            foregroundColor?: string;
            /**
             *
             * Returns or sets the transparency percentage of the fill as a value from 0.0 (opaque) through 1.0 (clear). Returns null if the shape type does not support transparency or the shape fill has inconsistent transparency, such as with a gradient fill type.
             *
             * [Api set: ExcelApi 1.9]
             */
            transparency?: number;
        }
        /** An interface for updating data on the ShapeLineFormat object, for use in "shapeLineFormat.set({ ... })". */
        interface ShapeLineFormatUpdateData {
            /**
             *
             * Represents the line color in HTML color format, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.9]
             */
            color?: string;
            /**
             *
             * Represents the line style of the shape. Returns null when the line is not visible or there are inconsistent dash styles. See Excel.ShapeLineStyle for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            dashStyle?: Excel.ShapeLineDashStyle | "Dash" | "DashDot" | "DashDotDot" | "LongDash" | "LongDashDot" | "RoundDot" | "Solid" | "SquareDot" | "LongDashDotDot" | "SystemDash" | "SystemDot" | "SystemDashDot";
            /**
             *
             * Represents the line style of the shape. Returns null when the line is not visible or there are inconsistent styles. See Excel.ShapeLineStyle for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            style?: Excel.ShapeLineStyle | "Single" | "ThickBetweenThin" | "ThickThin" | "ThinThick" | "ThinThin";
            /**
             *
             * Represents the degree of transparency of the specified line as a value from 0.0 (opaque) through 1.0 (clear). Returns null when the shape has inconsistent transparencies.
             *
             * [Api set: ExcelApi 1.9]
             */
            transparency?: number;
            /**
             *
             * Represents whether or not the line formatting of a shape element is visible. Returns null when the shape has inconsistent visibilities.
             *
             * [Api set: ExcelApi 1.9]
             */
            visible?: boolean;
            /**
             *
             * Represents the weight of the line, in points. Returns null when the line is not visible or there are inconsistent line weights.
             *
             * [Api set: ExcelApi 1.9]
             */
            weight?: number;
        }
        /** An interface for updating data on the TextFrame object, for use in "textFrame.set({ ... })". */
        interface TextFrameUpdateData {
            /**
             *
             * Gets or sets the automatic sizing settings for the text frame. A text frame can be set to automatically fit the text to the text frame, to automatically fit the text frame to the text, or not perform any automatic sizing.
             *
             * [Api set: ExcelApi 1.9]
             */
            autoSizeSetting?: Excel.ShapeAutoSize | "AutoSizeNone" | "AutoSizeTextToFitShape" | "AutoSizeShapeToFitText" | "AutoSizeMixed";
            /**
             *
             * Represents the bottom margin, in points, of the text frame.
             *
             * [Api set: ExcelApi 1.9]
             */
            bottomMargin?: number;
            /**
             *
             * Represents the horizontal alignment of the text frame. See Excel.ShapeTextHorizontalAlignment for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            horizontalAlignment?: Excel.ShapeTextHorizontalAlignment | "Left" | "Center" | "Right" | "Justify" | "JustifyLow" | "Distributed" | "ThaiDistributed";
            /**
             *
             * Represents the horizontal overflow behavior of the text frame. See Excel.ShapeTextHorizontalOverflow for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            horizontalOverflow?: Excel.ShapeTextHorizontalOverflow | "Overflow" | "Clip";
            /**
             *
             * Represents the left margin, in points, of the text frame.
             *
             * [Api set: ExcelApi 1.9]
             */
            leftMargin?: number;
            /**
             *
             * Represents the text orientation of the text frame. See Excel.ShapeTextOrientation for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            orientation?: Excel.ShapeTextOrientation | "Horizontal" | "Vertical" | "Vertical270" | "WordArtVertical" | "EastAsianVertical" | "MongolianVertical" | "WordArtVerticalRTL";
            /**
             *
             * Represents the reading order of the text frame, either left-to-right or right-to-left. See Excel.ShapeTextReadingOrder for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            readingOrder?: Excel.ShapeTextReadingOrder | "LeftToRight" | "RightToLeft";
            /**
             *
             * Represents the right margin, in points, of the text frame.
             *
             * [Api set: ExcelApi 1.9]
             */
            rightMargin?: number;
            /**
             *
             * Represents the top margin, in points, of the text frame.
             *
             * [Api set: ExcelApi 1.9]
             */
            topMargin?: number;
            /**
             *
             * Represents the vertical alignment of the text frame. See Excel.ShapeTextVerticalAlignment for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            verticalAlignment?: Excel.ShapeTextVerticalAlignment | "Top" | "Middle" | "Bottom" | "Justified" | "Distributed";
            /**
             *
             * Represents the vertical overflow behavior of the text frame. See Excel.ShapeTextVerticalOverflow for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            verticalOverflow?: Excel.ShapeTextVerticalOverflow | "Overflow" | "Ellipsis" | "Clip";
        }
        /** An interface for updating data on the TextRange object, for use in "textRange.set({ ... })". */
        interface TextRangeUpdateData {
            /**
            *
            * Returns a ShapeFont object that represents the font attributes for the text range.
            *
            * [Api set: ExcelApi 1.9]
            */
            font?: Excel.Interfaces.ShapeFontUpdateData;
            /**
             *
             * Represents the plain text content of the text range.
             *
             * [Api set: ExcelApi 1.9]
             */
            text?: string;
        }
        /** An interface for updating data on the ShapeFont object, for use in "shapeFont.set({ ... })". */
        interface ShapeFontUpdateData {
            /**
             *
             * Represents the bold status of font. Returns null the TextRange includes both bold and non-bold text fragments.
             *
             * [Api set: ExcelApi 1.9]
             */
            bold?: boolean;
            /**
             *
             * The HTML color code representation of the text color (e.g. "#FF0000" represents red). Returns null if the TextRange includes text fragments with different colors.
             *
             * [Api set: ExcelApi 1.9]
             */
            color?: string;
            /**
             *
             * Represents the italic status of font. Returns null if the TextRange includes both italic and non-italic text fragments.
             *
             * [Api set: ExcelApi 1.9]
             */
            italic?: boolean;
            /**
             *
             * Represents font name (e.g. "Calibri"). If the text is Complex Script or East Asian language, this is the corresponding font name; otherwise it is the Latin font name.
             *
             * [Api set: ExcelApi 1.9]
             */
            name?: string;
            /**
             *
             * Represents font size in points (e.g. 11). Returns null if the TextRange includes text fragments with different font sizes.
             *
             * [Api set: ExcelApi 1.9]
             */
            size?: number;
            /**
             *
             * Type of underline applied to the font. Returns null if the TextRange includes text fragments with different underline styles. See Excel.ShapeFontUnderlineStyle for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            underline?: Excel.ShapeFontUnderlineStyle | "None" | "Single" | "Double" | "Heavy" | "Dotted" | "DottedHeavy" | "Dash" | "DashHeavy" | "DashLong" | "DashLongHeavy" | "DotDash" | "DotDashHeavy" | "DotDotDash" | "DotDotDashHeavy" | "Wavy" | "WavyHeavy" | "WavyDouble";
        }
        /** An interface describing the data returned by calling "runtime.toJSON()". */
        interface RuntimeData {
            /**
             *
             * Toggle JavaScript events in the current task pane or content add-in.
             *
             * [Api set: ExcelApi 1.8]
             */
            enableEvents?: boolean;
        }
        /** An interface describing the data returned by calling "application.toJSON()". */
        interface ApplicationData {
            /**
            *
            * Returns the Iterative Calculation settings.
            On Excel for Windows and Excel for Mac, the settings will apply to the Excel Application.
            On Excel Online and Excel for other platforms, the settings will apply to the active workbook.
            *
            * [Api set: ExcelApi 1.9]
            */
            iterativeCalculation?: Excel.Interfaces.IterativeCalculationData;
            /**
             *
             * Returns the Excel calculation engine version used for the last full recalculation. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            calculationEngineVersion?: number;
            /**
             *
             * Returns the calculation mode used in the workbook, as defined by the constants in Excel.CalculationMode. Possible values are: `Automatic`, where Excel controls recalculation; `AutomaticExceptTables`, where Excel controls recalculation but ignores changes in tables; `Manual`, where calculation is done when the user requests it.
             *
             * [Api set: ExcelApi 1.1 for get, 1.8 for set]
             */
            calculationMode?: Excel.CalculationMode | "Automatic" | "AutomaticExceptTables" | "Manual";
            /**
             *
             * Returns the calculation state of the application. See Excel.CalculationState for details. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            calculationState?: Excel.CalculationState | "Done" | "Calculating" | "Pending";
        }
        /** An interface describing the data returned by calling "iterativeCalculation.toJSON()". */
        interface IterativeCalculationData {
            /**
             *
             * True if Excel will use iteration to resolve circular references.
             *
             * [Api set: ExcelApi 1.9]
             */
            enabled?: boolean;
            /**
             *
             * Returns or sets the maximum amount of change between each iteration as Excel resolves circular references.
             *
             * [Api set: ExcelApi 1.9]
             */
            maxChange?: number;
            /**
             *
             * Returns or sets the maximum number of iterations that Excel can use to resolve a circular reference.
             *
             * [Api set: ExcelApi 1.9]
             */
            maxIteration?: number;
        }
        /** An interface describing the data returned by calling "workbook.toJSON()". */
        interface WorkbookData {
            /**
            *
            * Represents a collection of bindings that are part of the workbook. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            bindings?: Excel.Interfaces.BindingData[];
            /**
            *
            * Represents the collection of custom XML parts contained by this workbook. Read-only.
            *
            * [Api set: ExcelApi 1.5]
            */
            customXmlParts?: Excel.Interfaces.CustomXmlPartData[];
            /**
            *
            * Represents a collection of workbook scoped named items (named ranges and constants). Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            names?: Excel.Interfaces.NamedItemData[];
            /**
            *
            * Represents a collection of PivotTables associated with the workbook. Read-only.
            *
            * [Api set: ExcelApi 1.3]
            */
            pivotTables?: Excel.Interfaces.PivotTableData[];
            /**
            *
            * Gets the workbook properties. Read-only.
            *
            * [Api set: ExcelApi 1.7]
            */
            properties?: Excel.Interfaces.DocumentPropertiesData;
            /**
            *
            * Returns workbook protection object for a workbook. Read-only.
            *
            * [Api set: ExcelApi 1.7]
            */
            protection?: Excel.Interfaces.WorkbookProtectionData;
            /**
            *
            * Represents a collection of Settings associated with the workbook. Read-only.
            *
            * [Api set: ExcelApi 1.4]
            */
            settings?: Excel.Interfaces.SettingData[];
            /**
            *
            * Represents a collection of styles associated with the workbook. Read-only.
            *
            * [Api set: ExcelApi 1.7]
            */
            styles?: Excel.Interfaces.StyleData[];
            /**
            *
            * Represents a collection of tables associated with the workbook. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            tables?: Excel.Interfaces.TableData[];
            /**
            *
            * Represents a collection of worksheets associated with the workbook. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            worksheets?: Excel.Interfaces.WorksheetData[];
            /**
             *
             * Specifies whether or not the workbook is in autosave mode. Read-Only.
             *
             * [Api set: ExcelApi 1.9]
             */
            autoSave?: boolean;
            /**
             *
             * Returns a number about the version of Excel Calculation Engine. Read-Only.
             *
             * [Api set: ExcelApi 1.9]
             */
            calculationEngineVersion?: number;
            /**
             *
             * True if all charts in the workbook are tracking the actual data points to which they are attached.
            False if the charts track the index of the data points.
             *
             * [Api set: ExcelApi 1.9]
             */
            chartDataPointTrack?: boolean;
            /**
             *
             * Specifies whether or not changes have been made since the workbook was last saved.
            You can set this property to true if you want to close a modified workbook without either saving it or being prompted to save it.
             *
             * [Api set: ExcelApi 1.9]
             */
            isDirty?: boolean;
            /**
             *
             * Gets the workbook name. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            name?: string;
            /**
             *
             * Specifies whether or not the workbook has ever been saved locally or online. Read-Only.
             *
             * [Api set: ExcelApi 1.9]
             */
            previouslySaved?: boolean;
            /**
             *
             * True if the workbook is open in Read-only mode. Read-only.
             *
             * [Api set: ExcelApi 1.8]
             */
            readOnly?: boolean;
            /**
             *
             * True if calculations in this workbook will be done using only the precision of the numbers as they're displayed.
            Data will permanently lose accuracy when switching this property from false to true.
             *
             * [Api set: ExcelApi 1.9]
             */
            usePrecisionAsDisplayed?: boolean;
        }
        /** An interface describing the data returned by calling "workbookProtection.toJSON()". */
        interface WorkbookProtectionData {
            /**
             *
             * Indicates if the workbook is protected. Read-Only.
             *
             * [Api set: ExcelApi 1.7]
             */
            protected?: boolean;
        }
        /** An interface describing the data returned by calling "workbookCreated.toJSON()". */
        interface WorkbookCreatedData {
        }
        /** An interface describing the data returned by calling "worksheet.toJSON()". */
        interface WorksheetData {
            /**
            *
            * Represents the AutoFilter object of the worksheet. Read-Only.
            *
            * [Api set: ExcelApi 1.9]
            */
            autoFilter?: Excel.Interfaces.AutoFilterData;
            /**
            *
            * Returns collection of charts that are part of the worksheet. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            charts?: Excel.Interfaces.ChartData[];
            /**
            *
            * Gets the horizontal page break collection for the worksheet. This collection only contains manual page breaks.
            *
            * [Api set: ExcelApi 1.9]
            */
            horizontalPageBreaks?: Excel.Interfaces.PageBreakData[];
            /**
            *
            * Collection of names scoped to the current worksheet. Read-only.
            *
            * [Api set: ExcelApi 1.4]
            */
            names?: Excel.Interfaces.NamedItemData[];
            /**
            *
            * Gets the PageLayout object of the worksheet.
            *
            * [Api set: ExcelApi 1.9]
            */
            pageLayout?: Excel.Interfaces.PageLayoutData;
            /**
            *
            * Collection of PivotTables that are part of the worksheet. Read-only.
            *
            * [Api set: ExcelApi 1.3]
            */
            pivotTables?: Excel.Interfaces.PivotTableData[];
            /**
            *
            * Returns sheet protection object for a worksheet. Read-only.
            *
            * [Api set: ExcelApi 1.2]
            */
            protection?: Excel.Interfaces.WorksheetProtectionData;
            /**
            *
            * Returns the collection of all the Shape objects on the worksheet. Read-only.
            *
            * [Api set: ExcelApi 1.9]
            */
            shapes?: Excel.Interfaces.ShapeData[];
            /**
            *
            * Collection of tables that are part of the worksheet. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            tables?: Excel.Interfaces.TableData[];
            /**
            *
            * Gets the vertical page break collection for the worksheet. This collection only contains manual page breaks.
            *
            * [Api set: ExcelApi 1.9]
            */
            verticalPageBreaks?: Excel.Interfaces.PageBreakData[];
            /**
             *
             * Gets or sets the enableCalculation property of the worksheet.
            True if Excel recalculates the worksheet when necessary. False if Excel doesn't recalculate the sheet.
             *
             * [Api set: ExcelApi 1.9]
             */
            enableCalculation?: boolean;
            /**
             *
             * Returns a value that uniquely identifies the worksheet in a given workbook. The value of the identifier remains the same even when the worksheet is renamed or moved. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            id?: string;
            /**
             *
             * The display name of the worksheet.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: string;
            /**
             *
             * The zero-based position of the worksheet within the workbook.
             *
             * [Api set: ExcelApi 1.1]
             */
            position?: number;
            /**
             *
             * Gets or sets the worksheet's gridlines flag.
            This flag determines whether gridlines are visible to the user.
             *
             * [Api set: ExcelApi 1.8]
             */
            showGridlines?: boolean;
            /**
             *
             * Gets or sets the worksheet's headings flag.
            This flag determines whether headings are visible to the user.
             *
             * [Api set: ExcelApi 1.8]
             */
            showHeadings?: boolean;
            /**
             *
             * Returns the standard (default) height of all the rows in the worksheet, in points. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            standardHeight?: number;
            /**
             *
             * Returns or sets the standard (default) width of all the columns in the worksheet.
            One unit of column width is equal to the width of one character in the Normal style. For proportional fonts, the width of the character 0 (zero) is used.
             *
             * [Api set: ExcelApi 1.7]
             */
            standardWidth?: number;
            /**
             *
             * Gets or sets the worksheet tab color.
            When retrieving the tab color, if the worksheet is invisible, the value will be null. If the worksheet is visible but the tab color is set to auto, an empty string will be returned. Otherwise, the property will be set to a color, in the form "#123456"
            When setting the color, use an empty-string to set an "auto" color, or a real color otherwise.
             *
             * [Api set: ExcelApi 1.7]
             */
            tabColor?: string;
            /**
             *
             * The Visibility of the worksheet.
             *
             * [Api set: ExcelApi 1.1 for reading visibility; 1.2 for setting it.]
             */
            visibility?: Excel.SheetVisibility | "Visible" | "Hidden" | "VeryHidden";
        }
        /** An interface describing the data returned by calling "worksheetCollection.toJSON()". */
        interface WorksheetCollectionData {
            items?: Excel.Interfaces.WorksheetData[];
        }
        /** An interface describing the data returned by calling "worksheetProtection.toJSON()". */
        interface WorksheetProtectionData {
            /**
             *
             * Sheet protection options. Read-only.
             *
             * [Api set: ExcelApi 1.2]
             */
            options?: Excel.WorksheetProtectionOptions;
            /**
             *
             * Indicates if the worksheet is protected. Read-only.
             *
             * [Api set: ExcelApi 1.2]
             */
            protected?: boolean;
        }
        /** An interface describing the data returned by calling "range.toJSON()". */
        interface RangeData {
            /**
            *
            * Collection of ConditionalFormats that intersect the range. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            conditionalFormats?: Excel.Interfaces.ConditionalFormatData[];
            /**
            *
            * Returns a data validation object.
            *
            * [Api set: ExcelApi 1.8]
            */
            dataValidation?: Excel.Interfaces.DataValidationData;
            /**
            *
            * Returns a format object, encapsulating the range's font, fill, borders, alignment, and other properties. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.RangeFormatData;
            /**
             *
             * Represents the range reference in A1-style. Address value will contain the Sheet reference (e.g. "Sheet1!A1:B4"). Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            address?: string;
            /**
             *
             * Represents range reference for the specified range in the language of the user. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            addressLocal?: string;
            /**
             *
             * Number of cells in the range. This API will return -1 if the cell count exceeds 2^31-1 (2,147,483,647). Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            cellCount?: number;
            /**
             *
             * Represents the total number of columns in the range. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            columnCount?: number;
            /**
             *
             * Represents if all columns of the current range are hidden.
             *
             * [Api set: ExcelApi 1.2]
             */
            columnHidden?: boolean;
            /**
             *
             * Represents the column number of the first cell in the range. Zero-indexed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            columnIndex?: number;
            /**
             *
             * Represents the formula in A1-style notation.
            When setting formulas to a range, the value argument can be either a single value (a string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            formulas?: any[][];
            /**
             *
             * Represents the formula in A1-style notation, in the user's language and number-formatting locale.  For example, the English "=SUM(A1, 1.5)" formula would become "=SUMME(A1; 1,5)" in German.
            When setting formulas to a range, the value argument can be either a single value (a string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            formulasLocal?: any[][];
            /**
             *
             * Represents the formula in R1C1-style notation.
            When setting formulas to a range, the value argument can be either a single value (a string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.2]
             */
            formulasR1C1?: any[][];
            /**
             *
             * Represents if all cells of the current range are hidden. Read-only.
             *
             * [Api set: ExcelApi 1.2]
             */
            hidden?: boolean;
            /**
             *
             * Represents the hyperlink for the current range.
             *
             * [Api set: ExcelApi 1.7]
             */
            hyperlink?: Excel.RangeHyperlink;
            /**
             *
             * Represents if the current range is an entire column. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            isEntireColumn?: boolean;
            /**
             *
             * Represents if the current range is an entire row. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            isEntireRow?: boolean;
            /**
             *
             * Represents the data type state of each cell. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            linkedDataTypeState?: Excel.LinkedDataTypeState[][];
            /**
             *
             * Represents Excel's number format code for the given range.
            When setting number format to a range, the value argument can be either a single value (string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            numberFormat?: any[][];
            /**
             *
             * Represents Excel's number format code for the given range as a string in the language of the user.
            When setting number format local to a range, the value argument can be either a single value (string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.7]
             */
            numberFormatLocal?: any[][];
            /**
             *
             * Returns the total number of rows in the range. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            rowCount?: number;
            /**
             *
             * Represents if all rows of the current range are hidden.
             *
             * [Api set: ExcelApi 1.2]
             */
            rowHidden?: boolean;
            /**
             *
             * Returns the row number of the first cell in the range. Zero-indexed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            rowIndex?: number;
            /**
             *
             * Represents the style of the current range.
            If the styles of the cells are inconsistent, null will be returned.
            For custom styles, the style name will be returned. For built-in styles, a string representing a value in the BuiltInStyle enum will be returned.
             *
             * [Api set: ExcelApi 1.7]
             */
            style?: string;
            /**
             *
             * Text values of the specified range. The Text value will not depend on the cell width. The # sign substitution that happens in Excel UI will not affect the text value returned by the API. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            text?: string[][];
            /**
             *
             * Represents the type of data of each cell. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            valueTypes?: Excel.RangeValueType[][];
            /**
             *
             * Represents the raw values of the specified range. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
            When setting values to a range, the value argument can be either a single value (string, number or boolean) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            values?: any[][];
        }
        /** An interface describing the data returned by calling "rangeAreas.toJSON()". */
        interface RangeAreasData {
            /**
            *
            * Returns a collection of rectangular ranges that comprise this RangeAreas object.
            *
            * [Api set: ExcelApi 1.9]
            */
            areas?: Excel.Interfaces.RangeData[];
            /**
            *
            * Returns a collection of ConditionalFormats that intersect with any cells in this RangeAreas object. Read-only.
            *
            * [Api set: ExcelApi 1.9]
            */
            conditionalFormats?: Excel.Interfaces.ConditionalFormatData[];
            /**
            *
            * Returns a dataValidation object for all ranges in the RangeAreas.
            *
            * [Api set: ExcelApi 1.9]
            */
            dataValidation?: Excel.Interfaces.DataValidationData;
            /**
            *
            * Returns a rangeFormat object, encapsulating the the font, fill, borders, alignment, and other properties for all ranges in the RangeAreas object. Read-only.
            *
            * [Api set: ExcelApi 1.9]
            */
            format?: Excel.Interfaces.RangeFormatData;
            /**
             *
             * Returns the RageAreas reference in A1-style. Address value will contain the worksheet name for each rectangular block of cells (e.g. "Sheet1!A1:B4, Sheet1!D1:D4"). Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            address?: string;
            /**
             *
             * Returns the RageAreas reference in the user locale. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            addressLocal?: string;
            /**
             *
             * Returns the number of rectangular ranges that comprise this RangeAreas object.
             *
             * [Api set: ExcelApi 1.9]
             */
            areaCount?: number;
            /**
             *
             * Returns the number of cells in the RangeAreas object, summing up the cell counts of all of the individual rectangular ranges. Returns -1 if the cell count exceeds 2^31-1 (2,147,483,647). Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            cellCount?: number;
            /**
             *
             * Indicates whether all the ranges on this RangeAreas object represent entire columns (e.g., "A:C, Q:Z"). Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            isEntireColumn?: boolean;
            /**
             *
             * Indicates whether all the ranges on this RangeAreas object represent entire rows (e.g., "1:3, 5:7"). Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            isEntireRow?: boolean;
            /**
             *
             * Represents the style for all ranges in this RangeAreas object.
            If the styles of the cells are inconsistent, null will be returned.
            For custom styles, the style name will be returned. For built-in styles, a string representing a value in the BuiltInStyle enum will be returned.
             *
             * [Api set: ExcelApi 1.9]
             */
            style?: string;
        }
        /** An interface describing the data returned by calling "rangeView.toJSON()". */
        interface RangeViewData {
            /**
            *
            * Represents a collection of range views associated with the range. Read-only.
            *
            * [Api set: ExcelApi 1.3]
            */
            rows?: Excel.Interfaces.RangeViewData[];
            /**
             *
             * Represents the cell addresses of the RangeView. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            cellAddresses?: any[][];
            /**
             *
             * Returns the number of visible columns. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            columnCount?: number;
            /**
             *
             * Represents the formula in A1-style notation.
             *
             * [Api set: ExcelApi 1.3]
             */
            formulas?: any[][];
            /**
             *
             * Represents the formula in A1-style notation, in the user's language and number-formatting locale.  For example, the English "=SUM(A1, 1.5)" formula would become "=SUMME(A1; 1,5)" in German.
             *
             * [Api set: ExcelApi 1.3]
             */
            formulasLocal?: any[][];
            /**
             *
             * Represents the formula in R1C1-style notation.
             *
             * [Api set: ExcelApi 1.3]
             */
            formulasR1C1?: any[][];
            /**
             *
             * Returns a value that represents the index of the RangeView. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            index?: number;
            /**
             *
             * Represents Excel's number format code for the given cell.
             *
             * [Api set: ExcelApi 1.3]
             */
            numberFormat?: any[][];
            /**
             *
             * Returns the number of visible rows. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            rowCount?: number;
            /**
             *
             * Text values of the specified range. The Text value will not depend on the cell width. The # sign substitution that happens in Excel UI will not affect the text value returned by the API. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            text?: string[][];
            /**
             *
             * Represents the type of data of each cell. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            valueTypes?: Excel.RangeValueType[][];
            /**
             *
             * Represents the raw values of the specified range view. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
             *
             * [Api set: ExcelApi 1.3]
             */
            values?: any[][];
        }
        /** An interface describing the data returned by calling "rangeViewCollection.toJSON()". */
        interface RangeViewCollectionData {
            items?: Excel.Interfaces.RangeViewData[];
        }
        /** An interface describing the data returned by calling "settingCollection.toJSON()". */
        interface SettingCollectionData {
            items?: Excel.Interfaces.SettingData[];
        }
        /** An interface describing the data returned by calling "setting.toJSON()". */
        interface SettingData {
            /**
             *
             * Returns the key that represents the id of the Setting. Read-only.
             *
             * [Api set: ExcelApi 1.4]
             */
            key?: string;
            /**
             *
             * Represents the value stored for this setting.
             *
             * [Api set: ExcelApi 1.4]
             */
            value?: any;
        }
        /** An interface describing the data returned by calling "namedItemCollection.toJSON()". */
        interface NamedItemCollectionData {
            items?: Excel.Interfaces.NamedItemData[];
        }
        /** An interface describing the data returned by calling "namedItem.toJSON()". */
        interface NamedItemData {
            /**
            *
            * Returns an object containing values and types of the named item. Read-only.
            *
            * [Api set: ExcelApi 1.7]
            */
            arrayValues?: Excel.Interfaces.NamedItemArrayValuesData;
            /**
             *
             * Represents the comment associated with this name.
             *
             * [Api set: ExcelApi 1.4]
             */
            comment?: string;
            /**
             *
             * Gets or sets the formula of the named item.  Formula always starts with a '=' sign.
             *
             * [Api set: ExcelApi 1.7]
             */
            formula?: any;
            /**
             *
             * The name of the object. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: string;
            /**
             *
             * Indicates whether the name is scoped to the workbook or to a specific worksheet. Possible values are: Worksheet, Workbook. Read-only.
             *
             * [Api set: ExcelApi 1.4]
             */
            scope?: Excel.NamedItemScope | "Worksheet" | "Workbook";
            /**
             *
             * Indicates the type of the value returned by the name's formula. See Excel.NamedItemType for details. Read-only.
             *
             * [Api set: ExcelApi 1.1 for String,Integer,Double,Boolean,Range,Error; 1.7 for Array]
             */
            type?: Excel.NamedItemType | "String" | "Integer" | "Double" | "Boolean" | "Range" | "Error" | "Array";
            /**
             *
             * Represents the value computed by the name's formula. For a named range, will return the range address. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            value?: any;
            /**
             *
             * Specifies whether the object is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
        }
        /** An interface describing the data returned by calling "namedItemArrayValues.toJSON()". */
        interface NamedItemArrayValuesData {
            /**
             *
             * Represents the types for each item in the named item array
             *
             * [Api set: ExcelApi 1.7]
             */
            types?: Excel.RangeValueType[][];
            /**
             *
             * Represents the values of each item in the named item array.
             *
             * [Api set: ExcelApi 1.7]
             */
            values?: any[][];
        }
        /** An interface describing the data returned by calling "binding.toJSON()". */
        interface BindingData {
            /**
             *
             * Represents binding identifier. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            id?: string;
            /**
             *
             * Returns the type of the binding. See Excel.BindingType for details. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            type?: Excel.BindingType | "Range" | "Table" | "Text";
        }
        /** An interface describing the data returned by calling "bindingCollection.toJSON()". */
        interface BindingCollectionData {
            items?: Excel.Interfaces.BindingData[];
        }
        /** An interface describing the data returned by calling "tableCollection.toJSON()". */
        interface TableCollectionData {
            items?: Excel.Interfaces.TableData[];
        }
        /** An interface describing the data returned by calling "tableScopedCollection.toJSON()". */
        interface TableScopedCollectionData {
            items?: Excel.Interfaces.TableData[];
        }
        /** An interface describing the data returned by calling "table.toJSON()". */
        interface TableData {
            /**
            *
            * Represents the AutoFilter object of the table. Read-Only.
            *
            * [Api set: ExcelApi 1.9]
            */
            autoFilter?: Excel.Interfaces.AutoFilterData;
            /**
            *
            * Represents a collection of all the columns in the table. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            columns?: Excel.Interfaces.TableColumnData[];
            /**
            *
            * Represents a collection of all the rows in the table. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            rows?: Excel.Interfaces.TableRowData[];
            /**
            *
            * Represents the sorting for the table. Read-only.
            *
            * [Api set: ExcelApi 1.2]
            */
            sort?: Excel.Interfaces.TableSortData;
            /**
             *
             * Indicates whether the first column contains special formatting.
             *
             * [Api set: ExcelApi 1.3]
             */
            highlightFirstColumn?: boolean;
            /**
             *
             * Indicates whether the last column contains special formatting.
             *
             * [Api set: ExcelApi 1.3]
             */
            highlightLastColumn?: boolean;
            /**
             *
             * Returns a value that uniquely identifies the table in a given workbook. The value of the identifier remains the same even when the table is renamed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            id?: string;
            /**
             *
             * Returns a numeric id.
             *
             * [Api set: ExcelApi 1.8]
             */
            legacyId?: string;
            /**
             *
             * Name of the table.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: string;
            /**
             *
             * Indicates whether the columns show banded formatting in which odd columns are highlighted differently from even ones to make reading the table easier.
             *
             * [Api set: ExcelApi 1.3]
             */
            showBandedColumns?: boolean;
            /**
             *
             * Indicates whether the rows show banded formatting in which odd rows are highlighted differently from even ones to make reading the table easier.
             *
             * [Api set: ExcelApi 1.3]
             */
            showBandedRows?: boolean;
            /**
             *
             * Indicates whether the filter buttons are visible at the top of each column header. Setting this is only allowed if the table contains a header row.
             *
             * [Api set: ExcelApi 1.3]
             */
            showFilterButton?: boolean;
            /**
             *
             * Indicates whether the header row is visible or not. This value can be set to show or remove the header row.
             *
             * [Api set: ExcelApi 1.1]
             */
            showHeaders?: boolean;
            /**
             *
             * Indicates whether the total row is visible or not. This value can be set to show or remove the total row.
             *
             * [Api set: ExcelApi 1.1]
             */
            showTotals?: boolean;
            /**
             *
             * Constant value that represents the Table style. Possible values are: "TableStyleLight1" through "TableStyleLight21", "TableStyleMedium1" through "TableStyleMedium28", "TableStyleStyleDark1" through "TableStyleStyleDark11". A custom user-defined style present in the workbook can also be specified.
             *
             * [Api set: ExcelApi 1.1]
             */
            style?: string;
        }
        /** An interface describing the data returned by calling "tableColumnCollection.toJSON()". */
        interface TableColumnCollectionData {
            items?: Excel.Interfaces.TableColumnData[];
        }
        /** An interface describing the data returned by calling "tableColumn.toJSON()". */
        interface TableColumnData {
            /**
            *
            * Retrieve the filter applied to the column. Read-only.
            *
            * [Api set: ExcelApi 1.2]
            */
            filter?: Excel.Interfaces.FilterData;
            /**
             *
             * Returns a unique key that identifies the column within the table. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            id?: number;
            /**
             *
             * Returns the index number of the column within the columns collection of the table. Zero-indexed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            index?: number;
            /**
             *
             * Represents the name of the table column.
             *
             * [Api set: ExcelApi 1.1 for getting the name; 1.4 for setting it.]
             */
            name?: string;
            /**
             *
             * Represents the raw values of the specified range. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
             *
             * [Api set: ExcelApi 1.1]
             */
            values?: any[][];
        }
        /** An interface describing the data returned by calling "tableRowCollection.toJSON()". */
        interface TableRowCollectionData {
            items?: Excel.Interfaces.TableRowData[];
        }
        /** An interface describing the data returned by calling "tableRow.toJSON()". */
        interface TableRowData {
            /**
             *
             * Returns the index number of the row within the rows collection of the table. Zero-indexed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            index?: number;
            /**
             *
             * Represents the raw values of the specified range. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
             *
             * [Api set: ExcelApi 1.1]
             */
            values?: any[][];
        }
        /** An interface describing the data returned by calling "dataValidation.toJSON()". */
        interface DataValidationData {
            /**
             *
             * Error alert when user enters invalid data.
             *
             * [Api set: ExcelApi 1.8]
             */
            errorAlert?: Excel.DataValidationErrorAlert;
            /**
             *
             * Ignore blanks: no data validation will be performed on blank cells, it defaults to true.
             *
             * [Api set: ExcelApi 1.8]
             */
            ignoreBlanks?: boolean;
            /**
             *
             * Prompt when users select a cell.
             *
             * [Api set: ExcelApi 1.8]
             */
            prompt?: Excel.DataValidationPrompt;
            /**
             *
             * Data validation rule that contains different type of data validation criteria.
             *
             * [Api set: ExcelApi 1.8]
             */
            rule?: Excel.DataValidationRule;
            /**
             *
             * Type of the data validation, see Excel.DataValidationType for details.
             *
             * [Api set: ExcelApi 1.8]
             */
            type?: Excel.DataValidationType | "None" | "WholeNumber" | "Decimal" | "List" | "Date" | "Time" | "TextLength" | "Custom" | "Inconsistent" | "MixedCriteria";
            /**
             *
             * Represents if all cell values are valid according to the data validation rules.
            Returns true if all cell values are valid, or false if all cell values are invalid.
            Returns null if there are both valid and invalid cell values within the range.
             *
             * [Api set: ExcelApi 1.8]
             */
            valid?: boolean;
        }
        /** An interface describing the data returned by calling "removeDuplicatesResult.toJSON()". */
        interface RemoveDuplicatesResultData {
            /**
             *
             * Number of duplicated rows removed by the operation.
             *
             * [Api set: ExcelApi 1.9]
             */
            removed?: number;
            /**
             *
             * Number of remaining unique rows present in the resulting range.
             *
             * [Api set: ExcelApi 1.9]
             */
            uniqueRemaining?: number;
        }
        /** An interface describing the data returned by calling "rangeFormat.toJSON()". */
        interface RangeFormatData {
            /**
            *
            * Collection of border objects that apply to the overall range. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            borders?: Excel.Interfaces.RangeBorderData[];
            /**
            *
            * Returns the fill object defined on the overall range. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            fill?: Excel.Interfaces.RangeFillData;
            /**
            *
            * Returns the font object defined on the overall range. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.RangeFontData;
            /**
            *
            * Returns the format protection object for a range. Read-only.
            *
            * [Api set: ExcelApi 1.2]
            */
            protection?: Excel.Interfaces.FormatProtectionData;
            /**
             *
             * Indicates if text is automatically indented when text alignment is set to equal distribution.
             *
             * [Api set: ExcelApi 1.9]
             */
            autoIndent?: boolean;
            /**
             *
             * Gets or sets the width of all colums within the range. If the column widths are not uniform, null will be returned.
             *
             * [Api set: ExcelApi 1.2]
             */
            columnWidth?: number;
            /**
             *
             * Represents the horizontal alignment for the specified object. See Excel.HorizontalAlignment for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            horizontalAlignment?: Excel.HorizontalAlignment | "General" | "Left" | "Center" | "Right" | "Fill" | "Justify" | "CenterAcrossSelection" | "Distributed";
            /**
             *
             * An integer from 0 to 250 that indicates the indent level.
             *
             * [Api set: ExcelApi 1.9]
             */
            indentLevel?: number;
            /**
             *
             * The reading order for the range.
             *
             * [Api set: ExcelApi 1.9]
             */
            readingOrder?: Excel.ReadingOrder | "Context" | "LeftToRight" | "RightToLeft";
            /**
             *
             * Gets or sets the height of all rows in the range. If the row heights are not uniform, null will be returned.
             *
             * [Api set: ExcelApi 1.2]
             */
            rowHeight?: number;
            /**
             *
             * Indicates if text automatically shrinks to fit in the available column width.
             *
             * [Api set: ExcelApi 1.9]
             */
            shrinkToFit?: boolean;
            /**
             *
             * Gets or sets the text orientation of all the cells within the range.
            The text orientation should be an integer either from -90 to 90, or 180 for vertically-oriented text.
            If the orientation within a range are not uniform, then null will be returned.
             *
             * [Api set: ExcelApi 1.7]
             */
            textOrientation?: number;
            /**
             *
             * Determines if the row height of the Range object equals the standard height of the sheet.
            Returns True if the row height of the Range object equals the standard height of the sheet.
            Returns Null if the range contains more than one row and the rows aren't all the same height.
            Returns False otherwise.
             *
             * [Api set: ExcelApi 1.7]
             */
            useStandardHeight?: boolean;
            /**
             *
             * Indicates whether the column width of the Range object equals the standard width of the sheet.
            Returns True if the column width of the Range object equals the standard width of the sheet.
            Returns Null if the range contains more than one column and the columns aren't all the same height.
            Returns False otherwise.
             *
             * [Api set: ExcelApi 1.7]
             */
            useStandardWidth?: boolean;
            /**
             *
             * Represents the vertical alignment for the specified object. See Excel.VerticalAlignment for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            verticalAlignment?: Excel.VerticalAlignment | "Top" | "Center" | "Bottom" | "Justify" | "Distributed";
            /**
             *
             * Indicates if Excel wraps the text in the object. A null value indicates that the entire range doesn't have uniform wrap setting
             *
             * [Api set: ExcelApi 1.1]
             */
            wrapText?: boolean;
        }
        /** An interface describing the data returned by calling "formatProtection.toJSON()". */
        interface FormatProtectionData {
            /**
             *
             * Indicates if Excel hides the formula for the cells in the range. A null value indicates that the entire range doesn't have uniform formula hidden setting.
             *
             * [Api set: ExcelApi 1.2]
             */
            formulaHidden?: boolean;
            /**
             *
             * Indicates if Excel locks the cells in the object. A null value indicates that the entire range doesn't have uniform lock setting.
             *
             * [Api set: ExcelApi 1.2]
             */
            locked?: boolean;
        }
        /** An interface describing the data returned by calling "rangeFill.toJSON()". */
        interface RangeFillData {
            /**
             *
             * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange")
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: string;
            /**
             *
             * Gets or sets the pattern of a Range. See Excel.FillPattern for details. LinearGradient and RectangularGradient are not supported.
            A null value indicates that the entire range doesn't have uniform pattern setting.
             *
             * [Api set: ExcelApi 1.9]
             */
            pattern?: Excel.FillPattern | "None" | "Solid" | "Gray50" | "Gray75" | "Gray25" | "Horizontal" | "Vertical" | "Down" | "Up" | "Checker" | "SemiGray75" | "LightHorizontal" | "LightVertical" | "LightDown" | "LightUp" | "Grid" | "CrissCross" | "Gray16" | "Gray8" | "LinearGradient" | "RectangularGradient";
            /**
             *
             * Sets HTML color code representing the color of the Range pattern, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            Gets HTML color code representing the color of the Range pattern, of the form #RRGGBB (e.g. "FFA500").
             *
             * [Api set: ExcelApi 1.9]
             */
            patternColor?: string;
            /**
             *
             * Returns or sets a double that lightens or darkens a pattern color for Range Fill, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            If the pattern tintAndShades are not uniform, null will be returned.
             *
             * [Api set: ExcelApi 1.9]
             */
            patternTintAndShade?: number;
            /**
             *
             * Returns or sets a double that lightens or darkens a color for Range Fill, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            If the tintAndShades are not uniform, null will be returned.
             *
             * [Api set: ExcelApi 1.9]
             */
            tintAndShade?: number;
        }
        /** An interface describing the data returned by calling "rangeBorder.toJSON()". */
        interface RangeBorderData {
            /**
             *
             * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: string;
            /**
             *
             * Constant value that indicates the specific side of the border. See Excel.BorderIndex for details. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            sideIndex?: Excel.BorderIndex | "EdgeTop" | "EdgeBottom" | "EdgeLeft" | "EdgeRight" | "InsideVertical" | "InsideHorizontal" | "DiagonalDown" | "DiagonalUp";
            /**
             *
             * One of the constants of line style specifying the line style for the border. See Excel.BorderLineStyle for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            style?: Excel.BorderLineStyle | "None" | "Continuous" | "Dash" | "DashDot" | "DashDotDot" | "Dot" | "Double" | "SlantDashDot";
            /**
             *
             * Returns or sets a double that lightens or darkens a color for Range Border, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            A null value indicates that the border doesn't have uniform tintAndShade setting.
             *
             * [Api set: ExcelApi 1.9]
             */
            tintAndShade?: number;
            /**
             *
             * Specifies the weight of the border around a range. See Excel.BorderWeight for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            weight?: Excel.BorderWeight | "Hairline" | "Thin" | "Medium" | "Thick";
        }
        /** An interface describing the data returned by calling "rangeBorderCollection.toJSON()". */
        interface RangeBorderCollectionData {
            items?: Excel.Interfaces.RangeBorderData[];
        }
        /** An interface describing the data returned by calling "rangeFont.toJSON()". */
        interface RangeFontData {
            /**
             *
             * Represents the bold status of font.
             *
             * [Api set: ExcelApi 1.1]
             */
            bold?: boolean;
            /**
             *
             * HTML color code representation of the text color. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: string;
            /**
             *
             * Represents the italic status of the font.
             *
             * [Api set: ExcelApi 1.1]
             */
            italic?: boolean;
            /**
             *
             * Font name (e.g. "Calibri")
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: string;
            /**
             *
             * Font size.
             *
             * [Api set: ExcelApi 1.1]
             */
            size?: number;
            /**
             *
             * Represents the strikethrough status of font. A null value indicates that the entire range doesn't have uniform Strikethrough setting.
             *
             * [Api set: ExcelApi 1.9]
             */
            strikethrough?: boolean;
            /**
             *
             * Represents the Subscript status of font.
            Returns True if all the fonts of the range are Subscript.
            Returns False if all the fonts of the range are Superscript or normal (neither Superscript, nor Subscript).
            Returns Null otherwise.
             *
             * [Api set: ExcelApi 1.9]
             */
            subscript?: boolean;
            /**
             *
             * Represents the Superscript status of font.
            Returns True if all the fonts of the range are Superscript.
            Returns False if all the fonts of the range are Subscript or normal (neither Superscript, nor Subscript).
            Returns Null otherwise.
             *
             * [Api set: ExcelApi 1.9]
             */
            superscript?: boolean;
            /**
             *
             * Returns or sets a double that lightens or darkens a color for Range Font, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            A null value indicates that the entire range doesn't have uniform font tintAndShade setting.
             *
             * [Api set: ExcelApi 1.9]
             */
            tintAndShade?: number;
            /**
             *
             * Type of underline applied to the font. See Excel.RangeUnderlineStyle for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            underline?: Excel.RangeUnderlineStyle | "None" | "Single" | "Double" | "SingleAccountant" | "DoubleAccountant";
        }
        /** An interface describing the data returned by calling "chartCollection.toJSON()". */
        interface ChartCollectionData {
            items?: Excel.Interfaces.ChartData[];
        }
        /** An interface describing the data returned by calling "chart.toJSON()". */
        interface ChartData {
            /**
            *
            * Represents chart axes. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            axes?: Excel.Interfaces.ChartAxesData;
            /**
            *
            * Represents the datalabels on the chart. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            dataLabels?: Excel.Interfaces.ChartDataLabelsData;
            /**
            *
            * Encapsulates the format properties for the chart area. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartAreaFormatData;
            /**
            *
            * Represents the legend for the chart. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            legend?: Excel.Interfaces.ChartLegendData;
            /**
            *
            * Encapsulates the options for a pivot chart. Read-only.
            *
            * [Api set: ExcelApi 1.9]
            */
            pivotOptions?: Excel.Interfaces.ChartPivotOptionsData;
            /**
            *
            * Represents the plotArea for the chart.
            *
            * [Api set: ExcelApi 1.8]
            */
            plotArea?: Excel.Interfaces.ChartPlotAreaData;
            /**
            *
            * Represents either a single series or collection of series in the chart. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            series?: Excel.Interfaces.ChartSeriesData[];
            /**
            *
            * Represents the title of the specified chart, including the text, visibility, position, and formatting of the title. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            title?: Excel.Interfaces.ChartTitleData;
            /**
             *
             * Returns or sets a ChartCategoryLabelLevel enumeration constant referring to
            the level of where the category labels are being sourced from. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            categoryLabelLevel?: number;
            /**
             *
             * Represents the type of the chart. See Excel.ChartType for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            chartType?: Excel.ChartType | "Invalid" | "ColumnClustered" | "ColumnStacked" | "ColumnStacked100" | "3DColumnClustered" | "3DColumnStacked" | "3DColumnStacked100" | "BarClustered" | "BarStacked" | "BarStacked100" | "3DBarClustered" | "3DBarStacked" | "3DBarStacked100" | "LineStacked" | "LineStacked100" | "LineMarkers" | "LineMarkersStacked" | "LineMarkersStacked100" | "PieOfPie" | "PieExploded" | "3DPieExploded" | "BarOfPie" | "XYScatterSmooth" | "XYScatterSmoothNoMarkers" | "XYScatterLines" | "XYScatterLinesNoMarkers" | "AreaStacked" | "AreaStacked100" | "3DAreaStacked" | "3DAreaStacked100" | "DoughnutExploded" | "RadarMarkers" | "RadarFilled" | "Surface" | "SurfaceWireframe" | "SurfaceTopView" | "SurfaceTopViewWireframe" | "Bubble" | "Bubble3DEffect" | "StockHLC" | "StockOHLC" | "StockVHLC" | "StockVOHLC" | "CylinderColClustered" | "CylinderColStacked" | "CylinderColStacked100" | "CylinderBarClustered" | "CylinderBarStacked" | "CylinderBarStacked100" | "CylinderCol" | "ConeColClustered" | "ConeColStacked" | "ConeColStacked100" | "ConeBarClustered" | "ConeBarStacked" | "ConeBarStacked100" | "ConeCol" | "PyramidColClustered" | "PyramidColStacked" | "PyramidColStacked100" | "PyramidBarClustered" | "PyramidBarStacked" | "PyramidBarStacked100" | "PyramidCol" | "3DColumn" | "Line" | "3DLine" | "3DPie" | "Pie" | "XYScatter" | "3DArea" | "Area" | "Doughnut" | "Radar" | "Histogram" | "Boxwhisker" | "Pareto" | "RegionMap" | "Treemap" | "Waterfall" | "Sunburst" | "Funnel";
            /**
             *
             * Returns or sets the way that blank cells are plotted on a chart. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            displayBlanksAs?: Excel.ChartDisplayBlanksAs | "NotPlotted" | "Zero" | "Interplotted";
            /**
             *
             * Represents the height, in points, of the chart object.
             *
             * [Api set: ExcelApi 1.1]
             */
            height?: number;
            /**
             *
             * The unique id of chart. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            id?: string;
            /**
             *
             * The distance, in points, from the left side of the chart to the worksheet origin.
             *
             * [Api set: ExcelApi 1.1]
             */
            left?: number;
            /**
             *
             * Represents the name of a chart object.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: string;
            /**
             *
             * Returns or sets the way columns or rows are used as data series on the chart. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            plotBy?: Excel.ChartPlotBy | "Rows" | "Columns";
            /**
             *
             * True if only visible cells are plotted. False if both visible and hidden cells are plotted. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            plotVisibleOnly?: boolean;
            /**
             *
             * Returns or sets a ChartSeriesNameLevel enumeration constant referring to
            the level of where the series names are being sourced from. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            seriesNameLevel?: number;
            /**
             *
             * Represents whether to display all field buttons on a PivotChart.
             *
             * [Api set: ExcelApi 1.7]
             */
            showAllFieldButtons?: boolean;
            /**
             *
             * Represents whether to show the data labels when the value is greater than the maximum value on the value axis.
            If value axis became smaller than the size of data points, you can use this property to set whether to show the data labels.
            This property applies to 2-D charts only.
             *
             * [Api set: ExcelApi 1.8]
             */
            showDataLabelsOverMaximum?: boolean;
            /**
             *
             * Returns or sets the chart style for the chart. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            style?: number;
            /**
             *
             * Represents the distance, in points, from the top edge of the object to the top of row 1 (on a worksheet) or the top of the chart area (on a chart).
             *
             * [Api set: ExcelApi 1.1]
             */
            top?: number;
            /**
             *
             * Represents the width, in points, of the chart object.
             *
             * [Api set: ExcelApi 1.1]
             */
            width?: number;
        }
        /** An interface describing the data returned by calling "chartPivotOptions.toJSON()". */
        interface ChartPivotOptionsData {
            /**
             *
             * Specifies whether or not to display the axis field buttons on a PivotChart. The ShowAxisFieldButtons property corresponds to the "Show Axis Field Buttons" command on the "Field Buttons" drop-down list of the "Analyze" tab, which is available when a PivotChart is selected.
             *
             * [Api set: ExcelApi 1.9]
             */
            showAxisFieldButtons?: boolean;
            /**
             *
             * Specifies whether or not to display the legend field buttons on a PivotChart
             *
             * [Api set: ExcelApi 1.9]
             */
            showLegendFieldButtons?: boolean;
            /**
             *
             * Specifies whether or not to display the report filter field buttons on a PivotChart.
             *
             * [Api set: ExcelApi 1.9]
             */
            showReportFilterFieldButtons?: boolean;
            /**
             *
             * Specifies whether or not to display the show value field buttons on a PivotChart
             *
             * [Api set: ExcelApi 1.9]
             */
            showValueFieldButtons?: boolean;
        }
        /** An interface describing the data returned by calling "chartAreaFormat.toJSON()". */
        interface ChartAreaFormatData {
            /**
            *
            * Represents the border format of chart area, which includes color, linestyle, and weight. Read-only.
            *
            * [Api set: ExcelApi 1.7]
            */
            border?: Excel.Interfaces.ChartBorderData;
            /**
            *
            * Represents the font attributes (font name, font size, color, etc.) for the current object. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontData;
            /**
             *
             * Returns or sets color scheme of the chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            colorScheme?: Excel.ChartColorScheme | "ColorfulPalette1" | "ColorfulPalette2" | "ColorfulPalette3" | "ColorfulPalette4" | "MonochromaticPalette1" | "MonochromaticPalette2" | "MonochromaticPalette3" | "MonochromaticPalette4" | "MonochromaticPalette5" | "MonochromaticPalette6" | "MonochromaticPalette7" | "MonochromaticPalette8" | "MonochromaticPalette9" | "MonochromaticPalette10" | "MonochromaticPalette11" | "MonochromaticPalette12" | "MonochromaticPalette13";
            /**
             *
             * Specifies whether or not chart area of the chart has rounded corners. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            roundedCorners?: boolean;
        }
        /** An interface describing the data returned by calling "chartSeriesCollection.toJSON()". */
        interface ChartSeriesCollectionData {
            items?: Excel.Interfaces.ChartSeriesData[];
        }
        /** An interface describing the data returned by calling "chartSeries.toJSON()". */
        interface ChartSeriesData {
            /**
            *
            * Encapsulates the bin options for histogram charts and pareto charts. Read-only.
            *
            * [Api set: ExcelApi 1.9]
            */
            binOptions?: Excel.Interfaces.ChartBinOptionsData;
            /**
            *
            * Encapsulates the options for the box and whisker charts. Read-only.
            *
            * [Api set: ExcelApi 1.9]
            */
            boxwhiskerOptions?: Excel.Interfaces.ChartBoxwhiskerOptionsData;
            /**
            *
            * Represents a collection of all dataLabels in the series.
            *
            * [Api set: ExcelApi 1.8]
            */
            dataLabels?: Excel.Interfaces.ChartDataLabelsData;
            /**
            *
            * Represents the formatting of a chart series, which includes fill and line formatting. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartSeriesFormatData;
            /**
            *
            * Encapsulates the options for a region map chart. Read-only.
            *
            * [Api set: ExcelApi 1.9]
            */
            mapOptions?: Excel.Interfaces.ChartMapOptionsData;
            /**
            *
            * Represents a collection of all points in the series. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            points?: Excel.Interfaces.ChartPointData[];
            /**
            *
            * Represents a collection of trendlines in the series. Read-only.
            *
            * [Api set: ExcelApi 1.7]
            */
            trendlines?: Excel.Interfaces.ChartTrendlineData[];
            /**
            *
            * Represents the error bar object of a chart series.
            *
            * [Api set: ExcelApi 1.9]
            */
            xErrorBars?: Excel.Interfaces.ChartErrorBarsData;
            /**
            *
            * Represents the error bar object of a chart series.
            *
            * [Api set: ExcelApi 1.9]
            */
            yErrorBars?: Excel.Interfaces.ChartErrorBarsData;
            /**
             *
             * Returns or sets the group for the specified series. Read/Write
             *
             * [Api set: ExcelApi 1.8]
             */
            axisGroup?: Excel.ChartAxisGroup | "Primary" | "Secondary";
            /**
             *
             * This can be an integer value from 0 (zero) to 300, representing the percentage of the default size. This property only applies to bubble charts. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            bubbleScale?: number;
            /**
             *
             * Represents the chart type of a series. See Excel.ChartType for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            chartType?: Excel.ChartType | "Invalid" | "ColumnClustered" | "ColumnStacked" | "ColumnStacked100" | "3DColumnClustered" | "3DColumnStacked" | "3DColumnStacked100" | "BarClustered" | "BarStacked" | "BarStacked100" | "3DBarClustered" | "3DBarStacked" | "3DBarStacked100" | "LineStacked" | "LineStacked100" | "LineMarkers" | "LineMarkersStacked" | "LineMarkersStacked100" | "PieOfPie" | "PieExploded" | "3DPieExploded" | "BarOfPie" | "XYScatterSmooth" | "XYScatterSmoothNoMarkers" | "XYScatterLines" | "XYScatterLinesNoMarkers" | "AreaStacked" | "AreaStacked100" | "3DAreaStacked" | "3DAreaStacked100" | "DoughnutExploded" | "RadarMarkers" | "RadarFilled" | "Surface" | "SurfaceWireframe" | "SurfaceTopView" | "SurfaceTopViewWireframe" | "Bubble" | "Bubble3DEffect" | "StockHLC" | "StockOHLC" | "StockVHLC" | "StockVOHLC" | "CylinderColClustered" | "CylinderColStacked" | "CylinderColStacked100" | "CylinderBarClustered" | "CylinderBarStacked" | "CylinderBarStacked100" | "CylinderCol" | "ConeColClustered" | "ConeColStacked" | "ConeColStacked100" | "ConeBarClustered" | "ConeBarStacked" | "ConeBarStacked100" | "ConeCol" | "PyramidColClustered" | "PyramidColStacked" | "PyramidColStacked100" | "PyramidBarClustered" | "PyramidBarStacked" | "PyramidBarStacked100" | "PyramidCol" | "3DColumn" | "Line" | "3DLine" | "3DPie" | "Pie" | "XYScatter" | "3DArea" | "Area" | "Doughnut" | "Radar" | "Histogram" | "Boxwhisker" | "Pareto" | "RegionMap" | "Treemap" | "Waterfall" | "Sunburst" | "Funnel";
            /**
             *
             * Represents the doughnut hole size of a chart series.  Only valid on doughnut and doughnutExploded charts.
            Throws an invalid argument exception on invalid charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            doughnutHoleSize?: number;
            /**
             *
             * Returns or sets the explosion value for a pie-chart or doughnut-chart slice. Returns 0 (zero) if there's no explosion (the tip of the slice is in the center of the pie). Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            explosion?: number;
            /**
             *
             * Boolean value representing if the series is filtered or not. Not applicable for surface charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            filtered?: boolean;
            /**
             *
             * Returns or sets the angle of the first pie-chart or doughnut-chart slice, in degrees (clockwise from vertical). Applies only to pie, 3-D pie, and doughnut charts. Can be a value from 0 through 360. Read/Write
             *
             * [Api set: ExcelApi 1.8]
             */
            firstSliceAngle?: number;
            /**
             *
             * Represents the gap width of a chart series.  Only valid on bar and column charts, as well as
            specific classes of line and pie charts.  Throws an invalid argument exception on invalid charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            gapWidth?: number;
            /**
             *
             * Returns or sets the color for maximum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMaximumColor?: string;
            /**
             *
             * Returns or sets the type for maximum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMaximumType?: Excel.ChartGradientStyleType | "ExtremeValue" | "Number" | "Percent";
            /**
             *
             * Returns or sets the maximum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMaximumValue?: number;
            /**
             *
             * Returns or sets the color for midpoint value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMidpointColor?: string;
            /**
             *
             * Returns or sets the type for midpoint value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMidpointType?: Excel.ChartGradientStyleType | "ExtremeValue" | "Number" | "Percent";
            /**
             *
             * Returns or sets the midpoint value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMidpointValue?: number;
            /**
             *
             * Returns or sets the color for minimum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMinimumColor?: string;
            /**
             *
             * Returns or sets the type for minimum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMinimumType?: Excel.ChartGradientStyleType | "ExtremeValue" | "Number" | "Percent";
            /**
             *
             * Returns or sets the minimum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMinimumValue?: number;
            /**
             *
             * Returns or sets series gradient style of a region map chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientStyle?: Excel.ChartGradientStyle | "TwoPhaseColor" | "ThreePhaseColor";
            /**
             *
             * Boolean value representing if the series has data labels or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            hasDataLabels?: boolean;
            /**
             *
             * Returns or sets the fill color for negative data points in a series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            invertColor?: string;
            /**
             *
             * True if Microsoft Excel inverts the pattern in the item when it corresponds to a negative number. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            invertIfNegative?: boolean;
            /**
             *
             * Represents markers background color of a chart series.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerBackgroundColor?: string;
            /**
             *
             * Represents markers foreground color of a chart series.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerForegroundColor?: string;
            /**
             *
             * Represents marker size of a chart series.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerSize?: number;
            /**
             *
             * Represents marker style of a chart series. See Excel.ChartMarkerStyle for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerStyle?: Excel.ChartMarkerStyle | "Invalid" | "Automatic" | "None" | "Square" | "Diamond" | "Triangle" | "X" | "Star" | "Dot" | "Dash" | "Circle" | "Plus" | "Picture";
            /**
             *
             * Represents the name of a series in a chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: string;
            /**
             *
             * Specifies how bars and columns are positioned. Can be a value between –100 and 100. Applies only to 2-D bar and 2-D column charts. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            overlap?: number;
            /**
             *
             * Returns or sets the series parent label strategy area for a treemap chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            parentLabelStrategy?: Excel.ChartParentLabelStrategy | "None" | "Banner" | "Overlapping";
            /**
             *
             * Represents the plot order of a chart series within the chart group.
             *
             * [Api set: ExcelApi 1.7]
             */
            plotOrder?: number;
            /**
             *
             * Returns or sets the size of the secondary section of either a pie of pie chart or a bar of pie chart, as a percentage of the size of the primary pie. Can be a value from 5 to 200. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            secondPlotSize?: number;
            /**
             *
             * Specifies whether or not connector lines are shown in waterfall charts. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showConnectorLines?: boolean;
            /**
             *
             * Specifies whether or not leader lines are displayed for each data label in the series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showLeaderLines?: boolean;
            /**
             *
             * Boolean value representing if the series has a shadow or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showShadow?: boolean;
            /**
             *
             * Boolean value representing if the series is smooth or not. Only applicable to line and scatter charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            smooth?: boolean;
            /**
             *
             * Returns or sets the way the two sections of either a pie of pie chart or a bar of pie chart are split. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            splitType?: Excel.ChartSplitType | "SplitByPosition" | "SplitByValue" | "SplitByPercentValue" | "SplitByCustomSplit";
            /**
             *
             * Returns or sets the threshold value that separates two sections of either a pie-of-pie chart or a bar-of-pie chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            splitValue?: number;
            /**
             *
             * True if Microsoft Excel assigns a different color or pattern to each data marker. The chart must contain only one series. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            varyByCategories?: boolean;
        }
        /** An interface describing the data returned by calling "chartSeriesFormat.toJSON()". */
        interface ChartSeriesFormatData {
            /**
            *
            * Represents line formatting. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            line?: Excel.Interfaces.ChartLineFormatData;
        }
        /** An interface describing the data returned by calling "chartPointsCollection.toJSON()". */
        interface ChartPointsCollectionData {
            items?: Excel.Interfaces.ChartPointData[];
        }
        /** An interface describing the data returned by calling "chartPoint.toJSON()". */
        interface ChartPointData {
            /**
            *
            * Returns the data label of a chart point. Read-only.
            *
            * [Api set: ExcelApi 1.7]
            */
            dataLabel?: Excel.Interfaces.ChartDataLabelData;
            /**
            *
            * Encapsulates the format properties chart point. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartPointFormatData;
            /**
             *
             * Represents whether a data point has a data label. Not applicable for surface charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            hasDataLabel?: boolean;
            /**
             *
             * HTML color code representation of the marker background color of data point. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerBackgroundColor?: string;
            /**
             *
             * HTML color code representation of the marker foreground color of data point. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerForegroundColor?: string;
            /**
             *
             * Represents marker size of data point.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerSize?: number;
            /**
             *
             * Represents marker style of a chart data point. See Excel.ChartMarkerStyle for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerStyle?: Excel.ChartMarkerStyle | "Invalid" | "Automatic" | "None" | "Square" | "Diamond" | "Triangle" | "X" | "Star" | "Dot" | "Dash" | "Circle" | "Plus" | "Picture";
            /**
             *
             * Returns the value of a chart point. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            value?: any;
        }
        /** An interface describing the data returned by calling "chartPointFormat.toJSON()". */
        interface ChartPointFormatData {
            /**
            *
            * Represents the border format of a chart data point, which includes color, style, and weight information. Read-only.
            *
            * [Api set: ExcelApi 1.7]
            */
            border?: Excel.Interfaces.ChartBorderData;
        }
        /** An interface describing the data returned by calling "chartAxes.toJSON()". */
        interface ChartAxesData {
            /**
            *
            * Represents the category axis in a chart. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            categoryAxis?: Excel.Interfaces.ChartAxisData;
            /**
            *
            * Represents the series axis of a 3-dimensional chart. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            seriesAxis?: Excel.Interfaces.ChartAxisData;
            /**
            *
            * Represents the value axis in an axis. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            valueAxis?: Excel.Interfaces.ChartAxisData;
        }
        /** An interface describing the data returned by calling "chartAxis.toJSON()". */
        interface ChartAxisData {
            /**
            *
            * Represents the formatting of a chart object, which includes line and font formatting. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartAxisFormatData;
            /**
            *
            * Returns a Gridlines object that represents the major gridlines for the specified axis. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            majorGridlines?: Excel.Interfaces.ChartGridlinesData;
            /**
            *
            * Returns a Gridlines object that represents the minor gridlines for the specified axis. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            minorGridlines?: Excel.Interfaces.ChartGridlinesData;
            /**
            *
            * Represents the axis title. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            title?: Excel.Interfaces.ChartAxisTitleData;
            /**
             *
             * Represents the alignment for the specified axis tick label. See Excel.ChartTextHorizontalAlignment for detail.
             *
             * [Api set: ExcelApi 1.8]
             */
            alignment?: Excel.ChartTickLabelAlignment | "Center" | "Left" | "Right";
            /**
             *
             * Represents the group for the specified axis. See Excel.ChartAxisGroup for details. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            axisGroup?: Excel.ChartAxisGroup | "Primary" | "Secondary";
            /**
             *
             * Returns or sets the base unit for the specified category axis.
             *
             * [Api set: ExcelApi 1.7]
             */
            baseTimeUnit?: Excel.ChartAxisTimeUnit | "Days" | "Months" | "Years";
            /**
             *
             * Returns or sets the category axis type.
             *
             * [Api set: ExcelApi 1.7]
             */
            categoryType?: Excel.ChartAxisCategoryType | "Automatic" | "TextAxis" | "DateAxis";
            /**
             *
             * Represents the custom axis display unit value. Read-only. To set this property, please use the SetCustomDisplayUnit(double) method.
             *
             * [Api set: ExcelApi 1.7]
             */
            customDisplayUnit?: number;
            /**
             *
             * Represents the axis display unit. See Excel.ChartAxisDisplayUnit for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            displayUnit?: Excel.ChartAxisDisplayUnit | "None" | "Hundreds" | "Thousands" | "TenThousands" | "HundredThousands" | "Millions" | "TenMillions" | "HundredMillions" | "Billions" | "Trillions" | "Custom";
            /**
             *
             * Represents the height, in points, of the chart axis. Null if the axis is not visible. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            height?: number;
            /**
             *
             * Represents whether value axis crosses the category axis between categories.
             *
             * [Api set: ExcelApi 1.8]
             */
            isBetweenCategories?: boolean;
            /**
             *
             * Represents the distance, in points, from the left edge of the axis to the left of chart area. Null if the axis is not visible. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            left?: number;
            /**
             *
             * Represents whether or not the number format is linked to the cells. If true, the number format will change in the labels when it changes in the cells.
             *
             * [Api set: ExcelApi 1.9]
             */
            linkNumberFormat?: boolean;
            /**
             *
             * Represents the base of the logarithm when using logarithmic scales.
             *
             * [Api set: ExcelApi 1.7]
             */
            logBase?: number;
            /**
             *
             * Represents the type of major tick mark for the specified axis. See Excel.ChartAxisTickMark for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            majorTickMark?: Excel.ChartAxisTickMark | "None" | "Cross" | "Inside" | "Outside";
            /**
             *
             * Returns or sets the major unit scale value for the category axis when the CategoryType property is set to TimeScale.
             *
             * [Api set: ExcelApi 1.7]
             */
            majorTimeUnitScale?: Excel.ChartAxisTimeUnit | "Days" | "Months" | "Years";
            /**
             *
             * Represents the interval between two major tick marks. Can be set to a numeric value or an empty string.  The returned value is always a number.
             *
             * [Api set: ExcelApi 1.1]
             */
            majorUnit?: any;
            /**
             *
             * Represents the maximum value on the value axis.  Can be set to a numeric value or an empty string (for automatic axis values).  The returned value is always a number.
             *
             * [Api set: ExcelApi 1.1]
             */
            maximum?: any;
            /**
             *
             * Represents the minimum value on the value axis. Can be set to a numeric value or an empty string (for automatic axis values).  The returned value is always a number.
             *
             * [Api set: ExcelApi 1.1]
             */
            minimum?: any;
            /**
             *
             * Represents the type of minor tick mark for the specified axis. See Excel.ChartAxisTickMark for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            minorTickMark?: Excel.ChartAxisTickMark | "None" | "Cross" | "Inside" | "Outside";
            /**
             *
             * Returns or sets the minor unit scale value for the category axis when the CategoryType property is set to TimeScale.
             *
             * [Api set: ExcelApi 1.7]
             */
            minorTimeUnitScale?: Excel.ChartAxisTimeUnit | "Days" | "Months" | "Years";
            /**
             *
             * Represents the interval between two minor tick marks. Can be set to a numeric value or an empty string (for automatic axis values). The returned value is always a number.
             *
             * [Api set: ExcelApi 1.1]
             */
            minorUnit?: any;
            /**
             *
             * Represents whether an axis is multilevel or not.
             *
             * [Api set: ExcelApi 1.8]
             */
            multiLevel?: boolean;
            /**
             *
             * Represents the format code for the axis tick label.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: string;
            /**
             *
             * Represents the distance between the levels of labels, and the distance between the first level and the axis line. The value should be an integer from 0 to 1000.
             *
             * [Api set: ExcelApi 1.8]
             */
            offset?: number;
            /**
             *
             * Represents the specified axis position where the other axis crosses. See Excel.ChartAxisPosition for details.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: Excel.ChartAxisPosition | "Automatic" | "Maximum" | "Minimum" | "Custom";
            /**
             *
             * Represents the specified axis position where the other axis crosses at. You should use the SetPositionAt(double) method to set this property.
             *
             * [Api set: ExcelApi 1.8]
             */
            positionAt?: number;
            /**
             *
             * Represents whether Microsoft Excel plots data points from last to first.
             *
             * [Api set: ExcelApi 1.7]
             */
            reversePlotOrder?: boolean;
            /**
             *
             * Represents the value axis scale type. See Excel.ChartAxisScaleType for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            scaleType?: Excel.ChartAxisScaleType | "Linear" | "Logarithmic";
            /**
             *
             * Represents whether the axis display unit label is visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            showDisplayUnitLabel?: boolean;
            /**
             *
             * Represents the text orientation of the axis tick label. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: any;
            /**
             *
             * Represents the position of tick-mark labels on the specified axis. See Excel.ChartAxisTickLabelPosition for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            tickLabelPosition?: Excel.ChartAxisTickLabelPosition | "NextToAxis" | "High" | "Low" | "None";
            /**
             *
             * Represents the number of categories or series between tick-mark labels. Can be a value from 1 through 31999 or an empty string for automatic setting. The returned value is always a number.
             *
             * [Api set: ExcelApi 1.7]
             */
            tickLabelSpacing?: any;
            /**
             *
             * Represents the number of categories or series between tick marks.
             *
             * [Api set: ExcelApi 1.7]
             */
            tickMarkSpacing?: number;
            /**
             *
             * Represents the distance, in points, from the top edge of the axis to the top of chart area. Null if the axis is not visible. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            top?: number;
            /**
             *
             * Represents the axis type. See Excel.ChartAxisType for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            type?: Excel.ChartAxisType | "Invalid" | "Category" | "Value" | "Series";
            /**
             *
             * A boolean value represents the visibility of the axis.
             *
             * [Api set: ExcelApi 1.7]
             */
            visible?: boolean;
            /**
             *
             * Represents the width, in points, of the chart axis. Null if the axis is not visible. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            width?: number;
        }
        /** An interface describing the data returned by calling "chartAxisFormat.toJSON()". */
        interface ChartAxisFormatData {
            /**
            *
            * Represents the font attributes (font name, font size, color, etc.) for a chart axis element. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontData;
            /**
            *
            * Represents chart line formatting. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            line?: Excel.Interfaces.ChartLineFormatData;
        }
        /** An interface describing the data returned by calling "chartAxisTitle.toJSON()". */
        interface ChartAxisTitleData {
            /**
            *
            * Represents the formatting of chart axis title. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartAxisTitleFormatData;
            /**
             *
             * Represents the axis title.
             *
             * [Api set: ExcelApi 1.1]
             */
            text?: string;
            /**
             *
             * A boolean that specifies the visibility of an axis title.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
        }
        /** An interface describing the data returned by calling "chartAxisTitleFormat.toJSON()". */
        interface ChartAxisTitleFormatData {
            /**
            *
            * Represents the border format, which includes color, linestyle, and weight.
            *
            * [Api set: ExcelApi 1.8]
            */
            border?: Excel.Interfaces.ChartBorderData;
            /**
            *
            * Represents the font attributes, such as font name, font size, color, etc. of chart axis title object. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontData;
        }
        /** An interface describing the data returned by calling "chartDataLabels.toJSON()". */
        interface ChartDataLabelsData {
            /**
            *
            * Represents the format of chart data labels, which includes fill and font formatting. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartDataLabelFormatData;
            /**
             *
             * Represents whether data labels automatically generate appropriate text based on context.
             *
             * [Api set: ExcelApi 1.8]
             */
            autoText?: boolean;
            /**
             *
             * Represents the horizontal alignment for chart data label. See Excel.ChartTextHorizontalAlignment for details.
            This property is valid only when TextOrientation of data label is 0.
             *
             * [Api set: ExcelApi 1.8]
             */
            horizontalAlignment?: Excel.ChartTextHorizontalAlignment | "Center" | "Left" | "Right" | "Justify" | "Distributed";
            /**
             *
             * Represents whether or not the number format is linked to the cells. If true, the number format will change in the labels when it changes in the cells
             *
             * [Api set: ExcelApi 1.9]
             */
            linkNumberFormat?: boolean;
            /**
             *
             * Represents the format code for data labels.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: string;
            /**
             *
             * DataLabelPosition value that represents the position of the data label. See Excel.ChartDataLabelPosition for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            position?: Excel.ChartDataLabelPosition | "Invalid" | "None" | "Center" | "InsideEnd" | "InsideBase" | "OutsideEnd" | "Left" | "Right" | "Top" | "Bottom" | "BestFit" | "Callout";
            /**
             *
             * String representing the separator used for the data labels on a chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            separator?: string;
            /**
             *
             * Boolean value representing if the data label bubble size is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showBubbleSize?: boolean;
            /**
             *
             * Boolean value representing if the data label category name is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showCategoryName?: boolean;
            /**
             *
             * Boolean value representing if the data label legend key is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showLegendKey?: boolean;
            /**
             *
             * Boolean value representing if the data label percentage is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showPercentage?: boolean;
            /**
             *
             * Boolean value representing if the data label series name is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showSeriesName?: boolean;
            /**
             *
             * Boolean value representing if the data label value is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showValue?: boolean;
            /**
             *
             * Represents the text orientation of data labels. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: number;
            /**
             *
             * Represents the vertical alignment of chart data label. See Excel.ChartTextVerticalAlignment for details.
            This property is valid only when TextOrientation of data label is 90, -90 or 180.
             *
             * [Api set: ExcelApi 1.8]
             */
            verticalAlignment?: Excel.ChartTextVerticalAlignment | "Center" | "Bottom" | "Top" | "Justify" | "Distributed";
        }
        /** An interface describing the data returned by calling "chartDataLabel.toJSON()". */
        interface ChartDataLabelData {
            /**
            *
            * Represents the format of chart data label.
            *
            * [Api set: ExcelApi 1.8]
            */
            format?: Excel.Interfaces.ChartDataLabelFormatData;
            /**
             *
             * Boolean value representing if data label automatically generates appropriate text based on context.
             *
             * [Api set: ExcelApi 1.8]
             */
            autoText?: boolean;
            /**
             *
             * String value that represents the formula of chart data label using A1-style notation.
             *
             * [Api set: ExcelApi 1.8]
             */
            formula?: string;
            /**
             *
             * Returns the height, in points, of the chart data label. Read-only. Null if chart data label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            height?: number;
            /**
             *
             * Represents the horizontal alignment for chart data label. See Excel.ChartTextHorizontalAlignment for details.
            This property is valid only when TextOrientation of data label is 90, -90 or 180.
             *
             * [Api set: ExcelApi 1.8]
             */
            horizontalAlignment?: Excel.ChartTextHorizontalAlignment | "Center" | "Left" | "Right" | "Justify" | "Distributed";
            /**
             *
             * Represents the distance, in points, from the left edge of chart data label to the left edge of chart area. Null if chart data label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            left?: number;
            /**
             *
             * Boolean value representing if the number format is linked to the cells (so that the number format changes in the labels when it changes in the cells).
             *
             * [Api set: ExcelApi 1.9]
             */
            linkNumberFormat?: boolean;
            /**
             *
             * String value that represents the format code for data label.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: string;
            /**
             *
             * DataLabelPosition value that represents the position of the data label. See Excel.ChartDataLabelPosition for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            position?: Excel.ChartDataLabelPosition | "Invalid" | "None" | "Center" | "InsideEnd" | "InsideBase" | "OutsideEnd" | "Left" | "Right" | "Top" | "Bottom" | "BestFit" | "Callout";
            /**
             *
             * String representing the separator used for the data label on a chart.
             *
             * [Api set: ExcelApi 1.7]
             */
            separator?: string;
            /**
             *
             * Boolean value representing if the data label bubble size is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showBubbleSize?: boolean;
            /**
             *
             * Boolean value representing if the data label category name is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showCategoryName?: boolean;
            /**
             *
             * Boolean value representing if the data label legend key is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showLegendKey?: boolean;
            /**
             *
             * Boolean value representing if the data label percentage is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showPercentage?: boolean;
            /**
             *
             * Boolean value representing if the data label series name is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showSeriesName?: boolean;
            /**
             *
             * Boolean value representing if the data label value is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showValue?: boolean;
            /**
             *
             * String representing the text of the data label on a chart.
             *
             * [Api set: ExcelApi 1.8]
             */
            text?: string;
            /**
             *
             * Represents the text orientation of chart data label. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: number;
            /**
             *
             * Represents the distance, in points, from the top edge of chart data label to the top of chart area. Null if chart data label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            top?: number;
            /**
             *
             * Represents the vertical alignment of chart data label. See Excel.ChartTextVerticalAlignment for details.
            This property is valid only when TextOrientation of data label is 0.
             *
             * [Api set: ExcelApi 1.8]
             */
            verticalAlignment?: Excel.ChartTextVerticalAlignment | "Center" | "Bottom" | "Top" | "Justify" | "Distributed";
            /**
             *
             * Returns the width, in points, of the chart data label. Read-only. Null if chart data label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            width?: number;
        }
        /** An interface describing the data returned by calling "chartDataLabelFormat.toJSON()". */
        interface ChartDataLabelFormatData {
            /**
            *
            * Represents the border format, which includes color, linestyle, and weight. Read-only.
            *
            * [Api set: ExcelApi 1.8]
            */
            border?: Excel.Interfaces.ChartBorderData;
            /**
            *
            * Represents the font attributes (font name, font size, color, etc.) for a chart data label. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontData;
        }
        /** An interface describing the data returned by calling "chartErrorBars.toJSON()". */
        interface ChartErrorBarsData {
            /**
            *
            * Specifies the formatting type of the error bars.
            *
            * [Api set: ExcelApi 1.9]
            */
            format?: Excel.Interfaces.ChartErrorBarsFormatData;
            /**
             *
             * Specifies whether or not the error bars have an end style cap.
             *
             * [Api set: ExcelApi 1.9]
             */
            endStyleCap?: boolean;
            /**
             *
             * Specifies which parts of the error bars to include.
             *
             * [Api set: ExcelApi 1.9]
             */
            include?: Excel.ChartErrorBarsInclude | "Both" | "MinusValues" | "PlusValues";
            /**
             *
             * The type of range marked by the error bars.
             *
             * [Api set: ExcelApi 1.9]
             */
            type?: Excel.ChartErrorBarsType | "FixedValue" | "Percent" | "StDev" | "StError" | "Custom";
            /**
             *
             * Specifies whether or not the error bars are displayed.
             *
             * [Api set: ExcelApi 1.9]
             */
            visible?: boolean;
        }
        /** An interface describing the data returned by calling "chartErrorBarsFormat.toJSON()". */
        interface ChartErrorBarsFormatData {
            /**
            *
            * Represents the chart line formatting.
            *
            * [Api set: ExcelApi 1.9]
            */
            line?: Excel.Interfaces.ChartLineFormatData;
        }
        /** An interface describing the data returned by calling "chartGridlines.toJSON()". */
        interface ChartGridlinesData {
            /**
            *
            * Represents the formatting of chart gridlines. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartGridlinesFormatData;
            /**
             *
             * Boolean value representing if the axis gridlines are visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
        }
        /** An interface describing the data returned by calling "chartGridlinesFormat.toJSON()". */
        interface ChartGridlinesFormatData {
            /**
            *
            * Represents chart line formatting. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            line?: Excel.Interfaces.ChartLineFormatData;
        }
        /** An interface describing the data returned by calling "chartLegend.toJSON()". */
        interface ChartLegendData {
            /**
            *
            * Represents the formatting of a chart legend, which includes fill and font formatting. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartLegendFormatData;
            /**
            *
            * Represents a collection of legendEntries in the legend. Read-only.
            *
            * [Api set: ExcelApi 1.7]
            */
            legendEntries?: Excel.Interfaces.ChartLegendEntryData[];
            /**
             *
             * Represents the height, in points, of the legend on the chart. Null if legend is not visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            height?: number;
            /**
             *
             * Represents the left, in points, of a chart legend. Null if legend is not visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            left?: number;
            /**
             *
             * Boolean value for whether the chart legend should overlap with the main body of the chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            overlay?: boolean;
            /**
             *
             * Represents the position of the legend on the chart. See Excel.ChartLegendPosition for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            position?: Excel.ChartLegendPosition | "Invalid" | "Top" | "Bottom" | "Left" | "Right" | "Corner" | "Custom";
            /**
             *
             * Represents if the legend has a shadow on the chart.
             *
             * [Api set: ExcelApi 1.7]
             */
            showShadow?: boolean;
            /**
             *
             * Represents the top of a chart legend.
             *
             * [Api set: ExcelApi 1.7]
             */
            top?: number;
            /**
             *
             * A boolean value the represents the visibility of a ChartLegend object.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
            /**
             *
             * Represents the width, in points, of the legend on the chart. Null if legend is not visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            width?: number;
        }
        /** An interface describing the data returned by calling "chartLegendEntry.toJSON()". */
        interface ChartLegendEntryData {
            /**
             *
             * Represents the height of the legendEntry on the chart legend.
             *
             * [Api set: ExcelApi 1.8]
             */
            height?: number;
            /**
             *
             * Represents the index of the legendEntry in the chart legend.
             *
             * [Api set: ExcelApi 1.8]
             */
            index?: number;
            /**
             *
             * Represents the left of a chart legendEntry.
             *
             * [Api set: ExcelApi 1.8]
             */
            left?: number;
            /**
             *
             * Represents the top of a chart legendEntry.
             *
             * [Api set: ExcelApi 1.8]
             */
            top?: number;
            /**
             *
             * Represents the visible of a chart legend entry.
             *
             * [Api set: ExcelApi 1.7]
             */
            visible?: boolean;
            /**
             *
             * Represents the width of the legendEntry on the chart Legend.
             *
             * [Api set: ExcelApi 1.8]
             */
            width?: number;
        }
        /** An interface describing the data returned by calling "chartLegendEntryCollection.toJSON()". */
        interface ChartLegendEntryCollectionData {
            items?: Excel.Interfaces.ChartLegendEntryData[];
        }
        /** An interface describing the data returned by calling "chartLegendFormat.toJSON()". */
        interface ChartLegendFormatData {
            /**
            *
            * Represents the border format, which includes color, linestyle, and weight. Read-only.
            *
            * [Api set: ExcelApi 1.8]
            */
            border?: Excel.Interfaces.ChartBorderData;
            /**
            *
            * Represents the font attributes such as font name, font size, color, etc. of a chart legend. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontData;
        }
        /** An interface describing the data returned by calling "chartMapOptions.toJSON()". */
        interface ChartMapOptionsData {
            /**
             *
             * Returns or sets the series map labels strategy of a region map chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            labelStrategy?: Excel.ChartMapLabelStrategy | "None" | "BestFit" | "ShowAll";
            /**
             *
             * Returns or sets the series mapping level of a region map chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            level?: Excel.ChartMapAreaLevel | "Automatic" | "DataOnly" | "City" | "County" | "State" | "Country" | "Continent" | "World";
            /**
             *
             * Returns or sets the series projection type of a region map chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            projectionType?: Excel.ChartMapProjectionType | "Automatic" | "Mercator" | "Miller" | "Robinson" | "Albers";
        }
        /** An interface describing the data returned by calling "chartTitle.toJSON()". */
        interface ChartTitleData {
            /**
            *
            * Represents the formatting of a chart title, which includes fill and font formatting. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartTitleFormatData;
            /**
             *
             * Returns the height, in points, of the chart title. Null if chart title is not visible. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            height?: number;
            /**
             *
             * Represents the horizontal alignment for chart title.
             *
             * [Api set: ExcelApi 1.7]
             */
            horizontalAlignment?: Excel.ChartTextHorizontalAlignment | "Center" | "Left" | "Right" | "Justify" | "Distributed";
            /**
             *
             * Represents the distance, in points, from the left edge of chart title to the left edge of chart area. Null if chart title is not visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            left?: number;
            /**
             *
             * Boolean value representing if the chart title will overlay the chart or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            overlay?: boolean;
            /**
             *
             * Represents the position of chart title. See Excel.ChartTitlePosition for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            position?: Excel.ChartTitlePosition | "Automatic" | "Top" | "Bottom" | "Left" | "Right";
            /**
             *
             * Represents a boolean value that determines if the chart title has a shadow.
             *
             * [Api set: ExcelApi 1.7]
             */
            showShadow?: boolean;
            /**
             *
             * Represents the title text of a chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            text?: string;
            /**
             *
             * Represents the text orientation of chart title. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
             *
             * [Api set: ExcelApi 1.7]
             */
            textOrientation?: number;
            /**
             *
             * Represents the distance, in points, from the top edge of chart title to the top of chart area. Null if chart title is not visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            top?: number;
            /**
             *
             * Represents the vertical alignment of chart title. See Excel.ChartTextVerticalAlignment for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            verticalAlignment?: Excel.ChartTextVerticalAlignment | "Center" | "Bottom" | "Top" | "Justify" | "Distributed";
            /**
             *
             * A boolean value the represents the visibility of a chart title object.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
            /**
             *
             * Returns the width, in points, of the chart title. Null if chart title is not visible. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            width?: number;
        }
        /** An interface describing the data returned by calling "chartFormatString.toJSON()". */
        interface ChartFormatStringData {
            /**
            *
            * Represents the font attributes, such as font name, font size, color, etc. of chart characters object.
            *
            * [Api set: ExcelApi 1.7]
            */
            font?: Excel.Interfaces.ChartFontData;
        }
        /** An interface describing the data returned by calling "chartTitleFormat.toJSON()". */
        interface ChartTitleFormatData {
            /**
            *
            * Represents the border format of chart title, which includes color, linestyle, and weight. Read-only.
            *
            * [Api set: ExcelApi 1.7]
            */
            border?: Excel.Interfaces.ChartBorderData;
            /**
            *
            * Represents the font attributes (font name, font size, color, etc.) for an object. Read-only.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontData;
        }
        /** An interface describing the data returned by calling "chartBorder.toJSON()". */
        interface ChartBorderData {
            /**
             *
             * HTML color code representing the color of borders in the chart.
             *
             * [Api set: ExcelApi 1.7]
             */
            color?: string;
            /**
             *
             * Represents the line style of the border. See Excel.ChartLineStyle for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            lineStyle?: Excel.ChartLineStyle | "None" | "Continuous" | "Dash" | "DashDot" | "DashDotDot" | "Dot" | "Grey25" | "Grey50" | "Grey75" | "Automatic" | "RoundDot";
            /**
             *
             * Represents weight of the border, in points.
             *
             * [Api set: ExcelApi 1.7]
             */
            weight?: number;
        }
        /** An interface describing the data returned by calling "chartBinOptions.toJSON()". */
        interface ChartBinOptionsData {
            /**
             *
             * Specifies whether or not the bin overflow is enabled in a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            allowOverflow?: boolean;
            /**
             *
             * Specifies whether or not the bin underflow is enabled in a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            allowUnderflow?: boolean;
            /**
             *
             * Returns or sets the bin count of a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            count?: number;
            /**
             *
             * Returns or sets the bin overflow value of a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            overflowValue?: number;
            /**
             *
             * Returns or sets the bin's type for a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            type?: Excel.ChartBinType | "Category" | "Auto" | "BinWidth" | "BinCount";
            /**
             *
             * Returns or sets the bin underflow value of a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            underflowValue?: number;
            /**
             *
             * Returns or sets the bin width value of a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            width?: number;
        }
        /** An interface describing the data returned by calling "chartBoxwhiskerOptions.toJSON()". */
        interface ChartBoxwhiskerOptionsData {
            /**
             *
             * Returns or sets the quartile calculation type of a box and whisker chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            quartileCalculation?: Excel.ChartBoxQuartileCalculation | "Inclusive" | "Exclusive";
            /**
             *
             * Specifies whether or not the inner points are shown in a box and whisker chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showInnerPoints?: boolean;
            /**
             *
             * Specifies whether or not the mean line is shown in a box and whisker chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showMeanLine?: boolean;
            /**
             *
             * Specifies whether or not the mean marker is shown in a box and whisker chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showMeanMarker?: boolean;
            /**
             *
             * Specifies whether or not outlier points are shown in a box and whisker chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showOutlierPoints?: boolean;
        }
        /** An interface describing the data returned by calling "chartLineFormat.toJSON()". */
        interface ChartLineFormatData {
            /**
             *
             * HTML color code representing the color of lines in the chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: string;
            /**
             *
             * Represents the line style. See Excel.ChartLineStyle for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            lineStyle?: Excel.ChartLineStyle | "None" | "Continuous" | "Dash" | "DashDot" | "DashDotDot" | "Dot" | "Grey25" | "Grey50" | "Grey75" | "Automatic" | "RoundDot";
            /**
             *
             * Represents weight of the line, in points.
             *
             * [Api set: ExcelApi 1.7]
             */
            weight?: number;
        }
        /** An interface describing the data returned by calling "chartFont.toJSON()". */
        interface ChartFontData {
            /**
             *
             * Represents the bold status of font.
             *
             * [Api set: ExcelApi 1.1]
             */
            bold?: boolean;
            /**
             *
             * HTML color code representation of the text color. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: string;
            /**
             *
             * Represents the italic status of the font.
             *
             * [Api set: ExcelApi 1.1]
             */
            italic?: boolean;
            /**
             *
             * Font name (e.g. "Calibri")
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: string;
            /**
             *
             * Size of the font (e.g. 11)
             *
             * [Api set: ExcelApi 1.1]
             */
            size?: number;
            /**
             *
             * Type of underline applied to the font. See Excel.ChartUnderlineStyle for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            underline?: Excel.ChartUnderlineStyle | "None" | "Single";
        }
        /** An interface describing the data returned by calling "chartTrendline.toJSON()". */
        interface ChartTrendlineData {
            /**
            *
            * Represents the formatting of a chart trendline.
            *
            * [Api set: ExcelApi 1.7]
            */
            format?: Excel.Interfaces.ChartTrendlineFormatData;
            /**
            *
            * Represents the label of a chart trendline.
            *
            * [Api set: ExcelApi 1.8]
            */
            label?: Excel.Interfaces.ChartTrendlineLabelData;
            /**
             *
             * Represents the number of periods that the trendline extends backward.
             *
             * [Api set: ExcelApi 1.8]
             */
            backwardPeriod?: number;
            /**
             *
             * Represents the number of periods that the trendline extends forward.
             *
             * [Api set: ExcelApi 1.8]
             */
            forwardPeriod?: number;
            /**
             *
             * Represents the intercept value of the trendline. Can be set to a numeric value or an empty string (for automatic values). The returned value is always a number.
             *
             * [Api set: ExcelApi 1.7]
             */
            intercept?: any;
            /**
             *
             * Represents the period of a chart trendline. Only applicable for trendline with MovingAverage type.
             *
             * [Api set: ExcelApi 1.7]
             */
            movingAveragePeriod?: number;
            /**
             *
             * Represents the name of the trendline. Can be set to a string value, or can be set to null value represents automatic values. The returned value is always a string
             *
             * [Api set: ExcelApi 1.7]
             */
            name?: string;
            /**
             *
             * Represents the order of a chart trendline. Only applicable for trendline with Polynomial type.
             *
             * [Api set: ExcelApi 1.7]
             */
            polynomialOrder?: number;
            /**
             *
             * True if the equation for the trendline is displayed on the chart.
             *
             * [Api set: ExcelApi 1.8]
             */
            showEquation?: boolean;
            /**
             *
             * True if the R-squared for the trendline is displayed on the chart.
             *
             * [Api set: ExcelApi 1.8]
             */
            showRSquared?: boolean;
            /**
             *
             * Represents the type of a chart trendline.
             *
             * [Api set: ExcelApi 1.7]
             */
            type?: Excel.ChartTrendlineType | "Linear" | "Exponential" | "Logarithmic" | "MovingAverage" | "Polynomial" | "Power";
        }
        /** An interface describing the data returned by calling "chartTrendlineCollection.toJSON()". */
        interface ChartTrendlineCollectionData {
            items?: Excel.Interfaces.ChartTrendlineData[];
        }
        /** An interface describing the data returned by calling "chartTrendlineFormat.toJSON()". */
        interface ChartTrendlineFormatData {
            /**
            *
            * Represents chart line formatting. Read-only.
            *
            * [Api set: ExcelApi 1.7]
            */
            line?: Excel.Interfaces.ChartLineFormatData;
        }
        /** An interface describing the data returned by calling "chartTrendlineLabel.toJSON()". */
        interface ChartTrendlineLabelData {
            /**
            *
            * Represents the format of chart trendline label.
            *
            * [Api set: ExcelApi 1.8]
            */
            format?: Excel.Interfaces.ChartTrendlineLabelFormatData;
            /**
             *
             * Boolean value representing if trendline label automatically generates appropriate text based on context.
             *
             * [Api set: ExcelApi 1.8]
             */
            autoText?: boolean;
            /**
             *
             * String value that represents the formula of chart trendline label using A1-style notation.
             *
             * [Api set: ExcelApi 1.8]
             */
            formula?: string;
            /**
             *
             * Returns the height, in points, of the chart trendline label. Read-only. Null if chart trendline label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            height?: number;
            /**
             *
             * Represents the horizontal alignment for chart trendline label. See Excel.ChartTextHorizontalAlignment for details.
            This property is valid only when TextOrientation of trendline label is 90, -90 or 180.
             *
             * [Api set: ExcelApi 1.8]
             */
            horizontalAlignment?: Excel.ChartTextHorizontalAlignment | "Center" | "Left" | "Right" | "Justify" | "Distributed";
            /**
             *
             * Represents the distance, in points, from the left edge of chart trendline label to the left edge of chart area. Null if chart trendline label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            left?: number;
            /**
             *
             * Boolean value representing if the number format is linked to the cells (so that the number format changes in the labels when it changes in the cells).
             *
             * [Api set: ExcelApi 1.9]
             */
            linkNumberFormat?: boolean;
            /**
             *
             * String value that represents the format code for trendline label.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: string;
            /**
             *
             * String representing the text of the trendline label on a chart.
             *
             * [Api set: ExcelApi 1.8]
             */
            text?: string;
            /**
             *
             * Represents the text orientation of chart trendline label. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: number;
            /**
             *
             * Represents the distance, in points, from the top edge of chart trendline label to the top of chart area. Null if chart trendline label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            top?: number;
            /**
             *
             * Represents the vertical alignment of chart trendline label. See Excel.ChartTextVerticalAlignment for details.
            This property is valid only when TextOrientation of trendline label is 0.
             *
             * [Api set: ExcelApi 1.8]
             */
            verticalAlignment?: Excel.ChartTextVerticalAlignment | "Center" | "Bottom" | "Top" | "Justify" | "Distributed";
            /**
             *
             * Returns the width, in points, of the chart trendline label. Read-only. Null if chart trendline label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            width?: number;
        }
        /** An interface describing the data returned by calling "chartTrendlineLabelFormat.toJSON()". */
        interface ChartTrendlineLabelFormatData {
            /**
            *
            * Represents the border format, which includes color, linestyle, and weight.
            *
            * [Api set: ExcelApi 1.8]
            */
            border?: Excel.Interfaces.ChartBorderData;
            /**
            *
            * Represents the font attributes (font name, font size, color, etc.) for a chart trendline label.
            *
            * [Api set: ExcelApi 1.8]
            */
            font?: Excel.Interfaces.ChartFontData;
        }
        /** An interface describing the data returned by calling "chartPlotArea.toJSON()". */
        interface ChartPlotAreaData {
            /**
            *
            * Represents the formatting of a chart plotArea.
            *
            * [Api set: ExcelApi 1.8]
            */
            format?: Excel.Interfaces.ChartPlotAreaFormatData;
            /**
             *
             * Represents the height value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            height?: number;
            /**
             *
             * Represents the insideHeight value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            insideHeight?: number;
            /**
             *
             * Represents the insideLeft value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            insideLeft?: number;
            /**
             *
             * Represents the insideTop value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            insideTop?: number;
            /**
             *
             * Represents the insideWidth value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            insideWidth?: number;
            /**
             *
             * Represents the left value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            left?: number;
            /**
             *
             * Represents the position of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: Excel.ChartPlotAreaPosition | "Automatic" | "Custom";
            /**
             *
             * Represents the top value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            top?: number;
            /**
             *
             * Represents the width value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            width?: number;
        }
        /** An interface describing the data returned by calling "chartPlotAreaFormat.toJSON()". */
        interface ChartPlotAreaFormatData {
            /**
            *
            * Represents the border attributes of a chart plotArea.
            *
            * [Api set: ExcelApi 1.8]
            */
            border?: Excel.Interfaces.ChartBorderData;
        }
        /** An interface describing the data returned by calling "tableSort.toJSON()". */
        interface TableSortData {
            /**
             *
             * Represents the current conditions used to last sort the table. Read-only.
             *
             * [Api set: ExcelApi 1.2]
             */
            fields?: Excel.SortField[];
            /**
             *
             * Represents whether the casing impacted the last sort of the table. Read-only.
             *
             * [Api set: ExcelApi 1.2]
             */
            matchCase?: boolean;
            /**
             *
             * Represents Chinese character ordering method last used to sort the table. Read-only.
             *
             * [Api set: ExcelApi 1.2]
             */
            method?: Excel.SortMethod | "PinYin" | "StrokeCount";
        }
        /** An interface describing the data returned by calling "filter.toJSON()". */
        interface FilterData {
            /**
             *
             * The currently applied filter on the given column. Read-only.
             *
             * [Api set: ExcelApi 1.2]
             */
            criteria?: Excel.FilterCriteria;
        }
        /** An interface describing the data returned by calling "autoFilter.toJSON()". */
        interface AutoFilterData {
            /**
             *
             * An array that holds all the filter criteria in the autofiltered range. Read-Only.
             *
             * [Api set: ExcelApi 1.9]
             */
            criteria?: Excel.FilterCriteria[];
            /**
             *
             * Indicates if the AutoFilter is enabled or not. Read-Only.
             *
             * [Api set: ExcelApi 1.9]
             */
            enabled?: boolean;
            /**
             *
             * Indicates if the AutoFilter has filter criteria. Read-Only.
             *
             * [Api set: ExcelApi 1.9]
             */
            isDataFiltered?: boolean;
        }
        /** An interface describing the data returned by calling "customXmlPartScopedCollection.toJSON()". */
        interface CustomXmlPartScopedCollectionData {
            items?: Excel.Interfaces.CustomXmlPartData[];
        }
        /** An interface describing the data returned by calling "customXmlPartCollection.toJSON()". */
        interface CustomXmlPartCollectionData {
            items?: Excel.Interfaces.CustomXmlPartData[];
        }
        /** An interface describing the data returned by calling "customXmlPart.toJSON()". */
        interface CustomXmlPartData {
            /**
             *
             * The custom XML part's ID. Read-only.
             *
             * [Api set: ExcelApi 1.5]
             */
            id?: string;
            /**
             *
             * The custom XML part's namespace URI. Read-only.
             *
             * [Api set: ExcelApi 1.5]
             */
            namespaceUri?: string;
        }
        /** An interface describing the data returned by calling "pivotTableCollection.toJSON()". */
        interface PivotTableCollectionData {
            items?: Excel.Interfaces.PivotTableData[];
        }
        /** An interface describing the data returned by calling "pivotTable.toJSON()". */
        interface PivotTableData {
            /**
            *
            * The Column Pivot Hierarchies of the PivotTable.
            *
            * [Api set: ExcelApi 1.8]
            */
            columnHierarchies?: Excel.Interfaces.RowColumnPivotHierarchyData[];
            /**
            *
            * The Data Pivot Hierarchies of the PivotTable.
            *
            * [Api set: ExcelApi 1.8]
            */
            dataHierarchies?: Excel.Interfaces.DataPivotHierarchyData[];
            /**
            *
            * The Filter Pivot Hierarchies of the PivotTable.
            *
            * [Api set: ExcelApi 1.8]
            */
            filterHierarchies?: Excel.Interfaces.FilterPivotHierarchyData[];
            /**
            *
            * The Pivot Hierarchies of the PivotTable.
            *
            * [Api set: ExcelApi 1.8]
            */
            hierarchies?: Excel.Interfaces.PivotHierarchyData[];
            /**
            *
            * The Row Pivot Hierarchies of the PivotTable.
            *
            * [Api set: ExcelApi 1.8]
            */
            rowHierarchies?: Excel.Interfaces.RowColumnPivotHierarchyData[];
            /**
             *
             * Specifies whether the PivotTable allows values in the data body to be edited by the user.
             *
             * [Api set: ExcelApi 1.9]
             */
            enableDataValueEditing?: boolean;
            /**
             *
             * Id of the PivotTable. Read-only.
             *
             * [Api set: ExcelApi 1.5]
             */
            id?: string;
            /**
             *
             * Name of the PivotTable.
             *
             * [Api set: ExcelApi 1.3]
             */
            name?: string;
            /**
             *
             * Specifies whether the PivotTable uses custom lists when sorting.
             *
             * [Api set: ExcelApi 1.9]
             */
            useCustomSortLists?: boolean;
        }
        /** An interface describing the data returned by calling "pivotLayout.toJSON()". */
        interface PivotLayoutData {
            /**
             *
             * Specifies whether formatting will be automatically formatted when it’s refreshed or when fields are moved
             *
             * [Api set: ExcelApi 1.9]
             */
            autoFormat?: boolean;
            /**
             *
             * This property indicates the PivotLayoutType of all fields on the PivotTable. If fields have different states, this will be null.
             *
             * [Api set: ExcelApi 1.8]
             */
            layoutType?: Excel.PivotLayoutType | "Compact" | "Tabular" | "Outline";
            /**
             *
             * Specifies whether formatting is preserved when the report is refreshed or recalculated by operations such as pivoting, sorting, or changing page field items.
             *
             * [Api set: ExcelApi 1.9]
             */
            preserveFormatting?: boolean;
            /**
             *
             * Specifies whether the PivotTable report shows grand totals for columns.
             *
             * [Api set: ExcelApi 1.8]
             */
            showColumnGrandTotals?: boolean;
            /**
             *
             * Specifies whether the PivotTable report shows grand totals for rows.
             *
             * [Api set: ExcelApi 1.8]
             */
            showRowGrandTotals?: boolean;
            /**
             *
             * This property indicates the SubtotalLocationType of all fields on the PivotTable. If fields have different states, this will be null.
             *
             * [Api set: ExcelApi 1.8]
             */
            subtotalLocation?: Excel.SubtotalLocationType | "AtTop" | "AtBottom" | "Off";
        }
        /** An interface describing the data returned by calling "pivotHierarchyCollection.toJSON()". */
        interface PivotHierarchyCollectionData {
            items?: Excel.Interfaces.PivotHierarchyData[];
        }
        /** An interface describing the data returned by calling "pivotHierarchy.toJSON()". */
        interface PivotHierarchyData {
            /**
            *
            * Returns the PivotFields associated with the PivotHierarchy.
            *
            * [Api set: ExcelApi 1.8]
            */
            fields?: Excel.Interfaces.PivotFieldData[];
            /**
             *
             * Id of the PivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: string;
            /**
             *
             * Name of the PivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: string;
        }
        /** An interface describing the data returned by calling "rowColumnPivotHierarchyCollection.toJSON()". */
        interface RowColumnPivotHierarchyCollectionData {
            items?: Excel.Interfaces.RowColumnPivotHierarchyData[];
        }
        /** An interface describing the data returned by calling "rowColumnPivotHierarchy.toJSON()". */
        interface RowColumnPivotHierarchyData {
            /**
            *
            * Returns the PivotFields associated with the RowColumnPivotHierarchy.
            *
            * [Api set: ExcelApi 1.8]
            */
            fields?: Excel.Interfaces.PivotFieldData[];
            /**
             *
             * Id of the RowColumnPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: string;
            /**
             *
             * Name of the RowColumnPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: string;
            /**
             *
             * Position of the RowColumnPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: number;
        }
        /** An interface describing the data returned by calling "filterPivotHierarchyCollection.toJSON()". */
        interface FilterPivotHierarchyCollectionData {
            items?: Excel.Interfaces.FilterPivotHierarchyData[];
        }
        /** An interface describing the data returned by calling "filterPivotHierarchy.toJSON()". */
        interface FilterPivotHierarchyData {
            /**
            *
            * Returns the PivotFields associated with the FilterPivotHierarchy.
            *
            * [Api set: ExcelApi 1.8]
            */
            fields?: Excel.Interfaces.PivotFieldData[];
            /**
             *
             * Determines whether to allow multiple filter items.
             *
             * [Api set: ExcelApi 1.8]
             */
            enableMultipleFilterItems?: boolean;
            /**
             *
             * Id of the FilterPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: string;
            /**
             *
             * Name of the FilterPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: string;
            /**
             *
             * Position of the FilterPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: number;
        }
        /** An interface describing the data returned by calling "dataPivotHierarchyCollection.toJSON()". */
        interface DataPivotHierarchyCollectionData {
            items?: Excel.Interfaces.DataPivotHierarchyData[];
        }
        /** An interface describing the data returned by calling "dataPivotHierarchy.toJSON()". */
        interface DataPivotHierarchyData {
            /**
            *
            * Returns the PivotFields associated with the DataPivotHierarchy.
            *
            * [Api set: ExcelApi 1.8]
            */
            field?: Excel.Interfaces.PivotFieldData;
            /**
             *
             * Id of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: string;
            /**
             *
             * Name of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: string;
            /**
             *
             * Number format of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: string;
            /**
             *
             * Position of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: number;
            /**
             *
             * Determines whether the data should be shown as a specific summary calculation or not.
             *
             * [Api set: ExcelApi 1.8]
             */
            showAs?: Excel.ShowAsRule;
            /**
             *
             * Determines whether to show all items of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            summarizeBy?: Excel.AggregationFunction | "Unknown" | "Automatic" | "Sum" | "Count" | "Average" | "Max" | "Min" | "Product" | "CountNumbers" | "StandardDeviation" | "StandardDeviationP" | "Variance" | "VarianceP";
        }
        /** An interface describing the data returned by calling "pivotFieldCollection.toJSON()". */
        interface PivotFieldCollectionData {
            items?: Excel.Interfaces.PivotFieldData[];
        }
        /** An interface describing the data returned by calling "pivotField.toJSON()". */
        interface PivotFieldData {
            /**
            *
            * Returns the PivotFields associated with the PivotField.
            *
            * [Api set: ExcelApi 1.8]
            */
            items?: Excel.Interfaces.PivotItemData[];
            /**
             *
             * Id of the PivotField.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: string;
            /**
             *
             * Name of the PivotField.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: string;
            /**
             *
             * Determines whether to show all items of the PivotField.
             *
             * [Api set: ExcelApi 1.8]
             */
            showAllItems?: boolean;
            /**
             *
             * Subtotals of the PivotField.
             *
             * [Api set: ExcelApi 1.8]
             */
            subtotals?: Excel.Subtotals;
        }
        /** An interface describing the data returned by calling "pivotItemCollection.toJSON()". */
        interface PivotItemCollectionData {
            items?: Excel.Interfaces.PivotItemData[];
        }
        /** An interface describing the data returned by calling "pivotItem.toJSON()". */
        interface PivotItemData {
            /**
             *
             * Id of the PivotItem.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: string;
            /**
             *
             * Determines whether the item is expanded to show child items or if it's collapsed and child items are hidden.
             *
             * [Api set: ExcelApi 1.8]
             */
            isExpanded?: boolean;
            /**
             *
             * Name of the PivotItem.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: string;
            /**
             *
             * Determines whether the PivotItem is visible or not.
             *
             * [Api set: ExcelApi 1.8]
             */
            visible?: boolean;
        }
        /** An interface describing the data returned by calling "documentProperties.toJSON()". */
        interface DocumentPropertiesData {
            /**
            *
            * Gets the collection of custom properties of the workbook. Read only.
            *
            * [Api set: ExcelApi 1.7]
            */
            custom?: Excel.Interfaces.CustomPropertyData[];
            /**
             *
             * Gets or sets the author of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            author?: string;
            /**
             *
             * Gets or sets the category of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            category?: string;
            /**
             *
             * Gets or sets the comments of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            comments?: string;
            /**
             *
             * Gets or sets the company of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            company?: string;
            /**
             *
             * Gets the creation date of the workbook. Read only.
             *
             * [Api set: ExcelApi 1.7]
             */
            creationDate?: Date;
            /**
             *
             * Gets or sets the keywords of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            keywords?: string;
            /**
             *
             * Gets the last author of the workbook. Read only.
             *
             * [Api set: ExcelApi 1.7]
             */
            lastAuthor?: string;
            /**
             *
             * Gets or sets the manager of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            manager?: string;
            /**
             *
             * Gets the revision number of the workbook. Read only.
             *
             * [Api set: ExcelApi 1.7]
             */
            revisionNumber?: number;
            /**
             *
             * Gets or sets the subject of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            subject?: string;
            /**
             *
             * Gets or sets the title of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            title?: string;
        }
        /** An interface describing the data returned by calling "customProperty.toJSON()". */
        interface CustomPropertyData {
            /**
             *
             * Gets the key of the custom property. Read only.
             *
             * [Api set: ExcelApi 1.7]
             */
            key?: string;
            /**
             *
             * Gets the value type of the custom property. Read only.
             *
             * [Api set: ExcelApi 1.7]
             */
            type?: Excel.DocumentPropertyType | "Number" | "Boolean" | "Date" | "String" | "Float";
            /**
             *
             * Gets or sets the value of the custom property.
             *
             * [Api set: ExcelApi 1.7]
             */
            value?: any;
        }
        /** An interface describing the data returned by calling "customPropertyCollection.toJSON()". */
        interface CustomPropertyCollectionData {
            items?: Excel.Interfaces.CustomPropertyData[];
        }
        /** An interface describing the data returned by calling "conditionalFormatCollection.toJSON()". */
        interface ConditionalFormatCollectionData {
            items?: Excel.Interfaces.ConditionalFormatData[];
        }
        /** An interface describing the data returned by calling "conditionalFormat.toJSON()". */
        interface ConditionalFormatData {
            /**
            *
            * Returns the cell value conditional format properties if the current conditional format is a CellValue type.
            For example to format all cells between 5 and 10. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            cellValue?: Excel.Interfaces.CellValueConditionalFormatData;
            /**
            *
            * Returns the cell value conditional format properties if the current conditional format is a CellValue type.
            For example to format all cells between 5 and 10. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            cellValueOrNullObject?: Excel.Interfaces.CellValueConditionalFormatData;
            /**
            *
            * Returns the ColorScale conditional format properties if the current conditional format is an ColorScale type. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            colorScale?: Excel.Interfaces.ColorScaleConditionalFormatData;
            /**
            *
            * Returns the ColorScale conditional format properties if the current conditional format is an ColorScale type. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            colorScaleOrNullObject?: Excel.Interfaces.ColorScaleConditionalFormatData;
            /**
            *
            * Returns the custom conditional format properties if the current conditional format is a custom type. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            custom?: Excel.Interfaces.CustomConditionalFormatData;
            /**
            *
            * Returns the custom conditional format properties if the current conditional format is a custom type. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            customOrNullObject?: Excel.Interfaces.CustomConditionalFormatData;
            /**
            *
            * Returns the data bar properties if the current conditional format is a data bar. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            dataBar?: Excel.Interfaces.DataBarConditionalFormatData;
            /**
            *
            * Returns the data bar properties if the current conditional format is a data bar. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            dataBarOrNullObject?: Excel.Interfaces.DataBarConditionalFormatData;
            /**
            *
            * Returns the IconSet conditional format properties if the current conditional format is an IconSet type. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            iconSet?: Excel.Interfaces.IconSetConditionalFormatData;
            /**
            *
            * Returns the IconSet conditional format properties if the current conditional format is an IconSet type. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            iconSetOrNullObject?: Excel.Interfaces.IconSetConditionalFormatData;
            /**
            *
            * Returns the preset criteria conditional format. See Excel.PresetCriteriaConditionalFormat for more details.
            *
            * [Api set: ExcelApi 1.6]
            */
            preset?: Excel.Interfaces.PresetCriteriaConditionalFormatData;
            /**
            *
            * Returns the preset criteria conditional format. See Excel.PresetCriteriaConditionalFormat for more details.
            *
            * [Api set: ExcelApi 1.6]
            */
            presetOrNullObject?: Excel.Interfaces.PresetCriteriaConditionalFormatData;
            /**
            *
            * Returns the specific text conditional format properties if the current conditional format is a text type.
            For example to format cells matching the word "Text". Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            textComparison?: Excel.Interfaces.TextConditionalFormatData;
            /**
            *
            * Returns the specific text conditional format properties if the current conditional format is a text type.
            For example to format cells matching the word "Text". Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            textComparisonOrNullObject?: Excel.Interfaces.TextConditionalFormatData;
            /**
            *
            * Returns the Top/Bottom conditional format properties if the current conditional format is an TopBottom type.
            For example to format the top 10% or bottom 10 items. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            topBottom?: Excel.Interfaces.TopBottomConditionalFormatData;
            /**
            *
            * Returns the Top/Bottom conditional format properties if the current conditional format is an TopBottom type.
            For example to format the top 10% or bottom 10 items. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            topBottomOrNullObject?: Excel.Interfaces.TopBottomConditionalFormatData;
            /**
             *
             * The Priority of the Conditional Format within the current ConditionalFormatCollection. Read-only.
             *
             * [Api set: ExcelApi 1.6]
             */
            id?: string;
            /**
             *
             * The priority (or index) within the conditional format collection that this conditional format currently exists in. Changing this also
            changes other conditional formats' priorities, to allow for a contiguous priority order.
            Use a negative priority to begin from the back.
            Priorities greater than than bounds will get and set to the maximum (or minimum if negative) priority.
            Also note that if you change the priority, you have to re-fetch a new copy of the object at that new priority location if you want to make further changes to it. Read-only.
             *
             * [Api set: ExcelApi 1.6]
             */
            priority?: number;
            /**
             *
             * If the conditions of this conditional format are met, no lower-priority formats shall take effect on that cell.
            Null on databars, icon sets, and colorscales as there's no concept of StopIfTrue for these
             *
             * [Api set: ExcelApi 1.6]
             */
            stopIfTrue?: boolean;
            /**
             *
             * A type of conditional format. Only one can be set at a time. Read-only.
             *
             * [Api set: ExcelApi 1.6]
             */
            type?: Excel.ConditionalFormatType | "Custom" | "DataBar" | "ColorScale" | "IconSet" | "TopBottom" | "PresetCriteria" | "ContainsText" | "CellValue";
        }
        /** An interface describing the data returned by calling "dataBarConditionalFormat.toJSON()". */
        interface DataBarConditionalFormatData {
            /**
            *
            * Representation of all values to the left of the axis in an Excel data bar. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            negativeFormat?: Excel.Interfaces.ConditionalDataBarNegativeFormatData;
            /**
            *
            * Representation of all values to the right of the axis in an Excel data bar. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            positiveFormat?: Excel.Interfaces.ConditionalDataBarPositiveFormatData;
            /**
             *
             * HTML color code representing the color of the Axis line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            "" (empty string) if no axis is present or set.
             *
             * [Api set: ExcelApi 1.6]
             */
            axisColor?: string;
            /**
             *
             * Representation of how the axis is determined for an Excel data bar.
             *
             * [Api set: ExcelApi 1.6]
             */
            axisFormat?: Excel.ConditionalDataBarAxisFormat | "Automatic" | "None" | "CellMidPoint";
            /**
             *
             * Represents the direction that the data bar graphic should be based on.
             *
             * [Api set: ExcelApi 1.6]
             */
            barDirection?: Excel.ConditionalDataBarDirection | "Context" | "LeftToRight" | "RightToLeft";
            /**
             *
             * The rule for what consistutes the lower bound (and how to calculate it, if applicable) for a data bar.
             *
             * [Api set: ExcelApi 1.6]
             */
            lowerBoundRule?: Excel.ConditionalDataBarRule;
            /**
             *
             * If true, hides the values from the cells where the data bar is applied.
             *
             * [Api set: ExcelApi 1.6]
             */
            showDataBarOnly?: boolean;
            /**
             *
             * The rule for what constitutes the upper bound (and how to calculate it, if applicable) for a data bar.
             *
             * [Api set: ExcelApi 1.6]
             */
            upperBoundRule?: Excel.ConditionalDataBarRule;
        }
        /** An interface describing the data returned by calling "conditionalDataBarPositiveFormat.toJSON()". */
        interface ConditionalDataBarPositiveFormatData {
            /**
             *
             * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            "" (empty string) if no border is present or set.
             *
             * [Api set: ExcelApi 1.6]
             */
            borderColor?: string;
            /**
             *
             * HTML color code representing the fill color, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.6]
             */
            fillColor?: string;
            /**
             *
             * Boolean representation of whether or not the DataBar has a gradient.
             *
             * [Api set: ExcelApi 1.6]
             */
            gradientFill?: boolean;
        }
        /** An interface describing the data returned by calling "conditionalDataBarNegativeFormat.toJSON()". */
        interface ConditionalDataBarNegativeFormatData {
            /**
             *
             * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            "Empty String" if no border is present or set.
             *
             * [Api set: ExcelApi 1.6]
             */
            borderColor?: string;
            /**
             *
             * HTML color code representing the fill color, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.6]
             */
            fillColor?: string;
            /**
             *
             * Boolean representation of whether or not the negative DataBar has the same border color as the positive DataBar.
             *
             * [Api set: ExcelApi 1.6]
             */
            matchPositiveBorderColor?: boolean;
            /**
             *
             * Boolean representation of whether or not the negative DataBar has the same fill color as the positive DataBar.
             *
             * [Api set: ExcelApi 1.6]
             */
            matchPositiveFillColor?: boolean;
        }
        /** An interface describing the data returned by calling "customConditionalFormat.toJSON()". */
        interface CustomConditionalFormatData {
            /**
            *
            * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            format?: Excel.Interfaces.ConditionalRangeFormatData;
            /**
            *
            * Represents the Rule object on this conditional format. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            rule?: Excel.Interfaces.ConditionalFormatRuleData;
        }
        /** An interface describing the data returned by calling "conditionalFormatRule.toJSON()". */
        interface ConditionalFormatRuleData {
            /**
             *
             * The formula, if required, to evaluate the conditional format rule on.
             *
             * [Api set: ExcelApi 1.6]
             */
            formula?: string;
            /**
             *
             * The formula, if required, to evaluate the conditional format rule on in the user's language.
             *
             * [Api set: ExcelApi 1.6]
             */
            formulaLocal?: string;
            /**
             *
             * The formula, if required, to evaluate the conditional format rule on in R1C1-style notation.
             *
             * [Api set: ExcelApi 1.6]
             */
            formulaR1C1?: string;
        }
        /** An interface describing the data returned by calling "iconSetConditionalFormat.toJSON()". */
        interface IconSetConditionalFormatData {
            /**
             *
             * An array of Criteria and IconSets for the rules and potential custom icons for conditional icons. Note that for the first criterion only the custom icon can be modified, while type, formula, and operator will be ignored when set.
             *
             * [Api set: ExcelApi 1.6]
             */
            criteria?: Excel.ConditionalIconCriterion[];
            /**
             *
             * If true, reverses the icon orders for the IconSet. Note that this cannot be set if custom icons are used.
             *
             * [Api set: ExcelApi 1.6]
             */
            reverseIconOrder?: boolean;
            /**
             *
             * If true, hides the values and only shows icons.
             *
             * [Api set: ExcelApi 1.6]
             */
            showIconOnly?: boolean;
            /**
             *
             * If set, displays the IconSet option for the conditional format.
             *
             * [Api set: ExcelApi 1.6]
             */
            style?: Excel.IconSet | "Invalid" | "ThreeArrows" | "ThreeArrowsGray" | "ThreeFlags" | "ThreeTrafficLights1" | "ThreeTrafficLights2" | "ThreeSigns" | "ThreeSymbols" | "ThreeSymbols2" | "FourArrows" | "FourArrowsGray" | "FourRedToBlack" | "FourRating" | "FourTrafficLights" | "FiveArrows" | "FiveArrowsGray" | "FiveRating" | "FiveQuarters" | "ThreeStars" | "ThreeTriangles" | "FiveBoxes" | "LinkedEntityFinanceIcon" | "LinkedEntityMapIcon";
        }
        /** An interface describing the data returned by calling "colorScaleConditionalFormat.toJSON()". */
        interface ColorScaleConditionalFormatData {
            /**
             *
             * The criteria of the color scale. Midpoint is optional when using a two point color scale.
             *
             * [Api set: ExcelApi 1.6]
             */
            criteria?: Excel.ConditionalColorScaleCriteria;
            /**
             *
             * If true the color scale will have three points (minimum, midpoint, maximum), otherwise it will have two (minimum, maximum).
             *
             * [Api set: ExcelApi 1.6]
             */
            threeColorScale?: boolean;
        }
        /** An interface describing the data returned by calling "topBottomConditionalFormat.toJSON()". */
        interface TopBottomConditionalFormatData {
            /**
            *
            * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            format?: Excel.Interfaces.ConditionalRangeFormatData;
            /**
             *
             * The criteria of the Top/Bottom conditional format.
             *
             * [Api set: ExcelApi 1.6]
             */
            rule?: Excel.ConditionalTopBottomRule;
        }
        /** An interface describing the data returned by calling "presetCriteriaConditionalFormat.toJSON()". */
        interface PresetCriteriaConditionalFormatData {
            /**
            *
            * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties.
            *
            * [Api set: ExcelApi 1.6]
            */
            format?: Excel.Interfaces.ConditionalRangeFormatData;
            /**
             *
             * The rule of the conditional format.
             *
             * [Api set: ExcelApi 1.6]
             */
            rule?: Excel.ConditionalPresetCriteriaRule;
        }
        /** An interface describing the data returned by calling "textConditionalFormat.toJSON()". */
        interface TextConditionalFormatData {
            /**
            *
            * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            format?: Excel.Interfaces.ConditionalRangeFormatData;
            /**
             *
             * The rule of the conditional format.
             *
             * [Api set: ExcelApi 1.6]
             */
            rule?: Excel.ConditionalTextComparisonRule;
        }
        /** An interface describing the data returned by calling "cellValueConditionalFormat.toJSON()". */
        interface CellValueConditionalFormatData {
            /**
            *
            * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties.
            *
            * [Api set: ExcelApi 1.6]
            */
            format?: Excel.Interfaces.ConditionalRangeFormatData;
            /**
             *
             * Represents the Rule object on this conditional format.
             *
             * [Api set: ExcelApi 1.6]
             */
            rule?: Excel.ConditionalCellValueRule;
        }
        /** An interface describing the data returned by calling "conditionalRangeFormat.toJSON()". */
        interface ConditionalRangeFormatData {
            /**
            *
            * Collection of border objects that apply to the overall conditional format range. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            borders?: Excel.Interfaces.ConditionalRangeBorderData[];
            /**
            *
            * Returns the fill object defined on the overall conditional format range. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            fill?: Excel.Interfaces.ConditionalRangeFillData;
            /**
            *
            * Returns the font object defined on the overall conditional format range. Read-only.
            *
            * [Api set: ExcelApi 1.6]
            */
            font?: Excel.Interfaces.ConditionalRangeFontData;
            /**
             *
             * Represents Excel's number format code for the given range. Cleared if null is passed in.
             *
             * [Api set: ExcelApi 1.6]
             */
            numberFormat?: any;
        }
        /** An interface describing the data returned by calling "conditionalRangeFont.toJSON()". */
        interface ConditionalRangeFontData {
            /**
             *
             * Represents the bold status of font.
             *
             * [Api set: ExcelApi 1.6]
             */
            bold?: boolean;
            /**
             *
             * HTML color code representation of the text color. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.6]
             */
            color?: string;
            /**
             *
             * Represents the italic status of the font.
             *
             * [Api set: ExcelApi 1.6]
             */
            italic?: boolean;
            /**
             *
             * Represents the strikethrough status of the font.
             *
             * [Api set: ExcelApi 1.6]
             */
            strikethrough?: boolean;
            /**
             *
             * Type of underline applied to the font. See Excel.ConditionalRangeFontUnderlineStyle for details.
             *
             * [Api set: ExcelApi 1.6]
             */
            underline?: Excel.ConditionalRangeFontUnderlineStyle | "None" | "Single" | "Double";
        }
        /** An interface describing the data returned by calling "conditionalRangeFill.toJSON()". */
        interface ConditionalRangeFillData {
            /**
             *
             * HTML color code representing the color of the fill, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.6]
             */
            color?: string;
        }
        /** An interface describing the data returned by calling "conditionalRangeBorder.toJSON()". */
        interface ConditionalRangeBorderData {
            /**
             *
             * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.6]
             */
            color?: string;
            /**
             *
             * Constant value that indicates the specific side of the border. See Excel.ConditionalRangeBorderIndex for details. Read-only.
             *
             * [Api set: ExcelApi 1.6]
             */
            sideIndex?: Excel.ConditionalRangeBorderIndex | "EdgeTop" | "EdgeBottom" | "EdgeLeft" | "EdgeRight";
            /**
             *
             * One of the constants of line style specifying the line style for the border. See Excel.BorderLineStyle for details.
             *
             * [Api set: ExcelApi 1.6]
             */
            style?: Excel.ConditionalRangeBorderLineStyle | "None" | "Continuous" | "Dash" | "DashDot" | "DashDotDot" | "Dot";
        }
        /** An interface describing the data returned by calling "conditionalRangeBorderCollection.toJSON()". */
        interface ConditionalRangeBorderCollectionData {
            items?: Excel.Interfaces.ConditionalRangeBorderData[];
        }
        /** An interface describing the data returned by calling "style.toJSON()". */
        interface StyleData {
            /**
            *
            * A Border collection of four Border objects that represent the style of the four borders.
            *
            * [Api set: ExcelApi 1.7]
            */
            borders?: Excel.Interfaces.RangeBorderData[];
            /**
            *
            * The Fill of the style.
            *
            * [Api set: ExcelApi 1.7]
            */
            fill?: Excel.Interfaces.RangeFillData;
            /**
            *
            * A Font object that represents the font of the style.
            *
            * [Api set: ExcelApi 1.7]
            */
            font?: Excel.Interfaces.RangeFontData;
            /**
             *
             * Indicates if text is automatically indented when the text alignment in a cell is set to equal distribution.
             *
             * [Api set: ExcelApi 1.8]
             */
            autoIndent?: boolean;
            /**
             *
             * Indicates if the style is a built-in style.
             *
             * [Api set: ExcelApi 1.7]
             */
            builtIn?: boolean;
            /**
             *
             * Indicates if the formula will be hidden when the worksheet is protected.
             *
             * [Api set: ExcelApi 1.7]
             */
            formulaHidden?: boolean;
            /**
             *
             * Represents the horizontal alignment for the style. See Excel.HorizontalAlignment for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            horizontalAlignment?: Excel.HorizontalAlignment | "General" | "Left" | "Center" | "Right" | "Fill" | "Justify" | "CenterAcrossSelection" | "Distributed";
            /**
             *
             * Indicates if the style includes the AutoIndent, HorizontalAlignment, VerticalAlignment, WrapText, IndentLevel, and TextOrientation properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeAlignment?: boolean;
            /**
             *
             * Indicates if the style includes the Color, ColorIndex, LineStyle, and Weight border properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeBorder?: boolean;
            /**
             *
             * Indicates if the style includes the Background, Bold, Color, ColorIndex, FontStyle, Italic, Name, Size, Strikethrough, Subscript, Superscript, and Underline font properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeFont?: boolean;
            /**
             *
             * Indicates if the style includes the NumberFormat property.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeNumber?: boolean;
            /**
             *
             * Indicates if the style includes the Color, ColorIndex, InvertIfNegative, Pattern, PatternColor, and PatternColorIndex interior properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includePatterns?: boolean;
            /**
             *
             * Indicates if the style includes the FormulaHidden and Locked protection properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeProtection?: boolean;
            /**
             *
             * An integer from 0 to 250 that indicates the indent level for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            indentLevel?: number;
            /**
             *
             * Indicates if the object is locked when the worksheet is protected.
             *
             * [Api set: ExcelApi 1.7]
             */
            locked?: boolean;
            /**
             *
             * The name of the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            name?: string;
            /**
             *
             * The format code of the number format for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            numberFormat?: string;
            /**
             *
             * The localized format code of the number format for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            numberFormatLocal?: string;
            /**
             *
             * The reading order for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            readingOrder?: Excel.ReadingOrder | "Context" | "LeftToRight" | "RightToLeft";
            /**
             *
             * Indicates if text automatically shrinks to fit in the available column width.
             *
             * [Api set: ExcelApi 1.7]
             */
            shrinkToFit?: boolean;
            /**
             *
             * The text orientation for the style.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: number;
            /**
             *
             * Represents the vertical alignment for the style. See Excel.VerticalAlignment for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            verticalAlignment?: Excel.VerticalAlignment | "Top" | "Center" | "Bottom" | "Justify" | "Distributed";
            /**
             *
             * Indicates if Microsoft Excel wraps the text in the object.
             *
             * [Api set: ExcelApi 1.7]
             */
            wrapText?: boolean;
        }
        /** An interface describing the data returned by calling "styleCollection.toJSON()". */
        interface StyleCollectionData {
            items?: Excel.Interfaces.StyleData[];
        }
        /** An interface describing the data returned by calling "pageLayout.toJSON()". */
        interface PageLayoutData {
            /**
            *
            * Header and footer configuration for the worksheet.
            *
            * [Api set: ExcelApi 1.9]
            */
            headersFooters?: Excel.Interfaces.HeaderFooterGroupData;
            /**
             *
             * Gets or sets the worksheet's black and white print option.
             *
             * [Api set: ExcelApi 1.9]
             */
            blackAndWhite?: boolean;
            /**
             *
             * Gets or sets the worksheet's bottom page margin to use for printing in points.
             *
             * [Api set: ExcelApi 1.9]
             */
            bottomMargin?: number;
            /**
             *
             * Gets or sets the worksheet's center horizontally flag. This flag determines whether the worksheet will be centered horizontally when it's printed.
             *
             * [Api set: ExcelApi 1.9]
             */
            centerHorizontally?: boolean;
            /**
             *
             * Gets or sets the worksheet's center vertically flag. This flag determines whether the worksheet will be centered vertically when it's printed.
             *
             * [Api set: ExcelApi 1.9]
             */
            centerVertically?: boolean;
            /**
             *
             * Gets or sets the worksheet's draft mode option. If true the sheet will be printed without graphics.
             *
             * [Api set: ExcelApi 1.9]
             */
            draftMode?: boolean;
            /**
             *
             * Gets or sets the worksheet's first page number to print. Null value represents "auto" page numbering.
             *
             * [Api set: ExcelApi 1.9]
             */
            firstPageNumber?: number | "";
            /**
             *
             * Gets or sets the worksheet's footer margin, in points, for use when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            footerMargin?: number;
            /**
             *
             * Gets or sets the worksheet's header margin, in points, for use when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            headerMargin?: number;
            /**
             *
             * Gets or sets the worksheet's left margin, in points, for use when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            leftMargin?: number;
            /**
             *
             * Gets or sets the worksheet's orientation of the page.
             *
             * [Api set: ExcelApi 1.9]
             */
            orientation?: Excel.PageOrientation | "Portrait" | "Landscape";
            /**
             *
             * Gets or sets the worksheet's paper size of the page.
             *
             * [Api set: ExcelApi 1.9]
             */
            paperSize?: Excel.PaperType | "Letter" | "LetterSmall" | "Tabloid" | "Ledger" | "Legal" | "Statement" | "Executive" | "A3" | "A4" | "A4Small" | "A5" | "B4" | "B5" | "Folio" | "Quatro" | "Paper10x14" | "Paper11x17" | "Note" | "Envelope9" | "Envelope10" | "Envelope11" | "Envelope12" | "Envelope14" | "Csheet" | "Dsheet" | "Esheet" | "EnvelopeDL" | "EnvelopeC5" | "EnvelopeC3" | "EnvelopeC4" | "EnvelopeC6" | "EnvelopeC65" | "EnvelopeB4" | "EnvelopeB5" | "EnvelopeB6" | "EnvelopeItaly" | "EnvelopeMonarch" | "EnvelopePersonal" | "FanfoldUS" | "FanfoldStdGerman" | "FanfoldLegalGerman";
            /**
             *
             * Gets or sets whether the worksheet's comments should be displayed when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            printComments?: Excel.PrintComments | "NoComments" | "EndSheet" | "InPlace";
            /**
             *
             * Gets or sets the worksheet's print errors option.
             *
             * [Api set: ExcelApi 1.9]
             */
            printErrors?: Excel.PrintErrorType | "AsDisplayed" | "Blank" | "Dash" | "NotAvailable";
            /**
             *
             * Gets or sets the worksheet's print gridlines flag. This flag determines whether gridlines will be printed or not.
             *
             * [Api set: ExcelApi 1.9]
             */
            printGridlines?: boolean;
            /**
             *
             * Gets or sets the worksheet's print headings flag. This flag determines whether headings will be printed or not.
             *
             * [Api set: ExcelApi 1.9]
             */
            printHeadings?: boolean;
            /**
             *
             * Gets or sets the worksheet's page print order option. This specifies the order to use for processing the page number printed.
             *
             * [Api set: ExcelApi 1.9]
             */
            printOrder?: Excel.PrintOrder | "DownThenOver" | "OverThenDown";
            /**
             *
             * Gets or sets the worksheet's right margin, in points, for use when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            rightMargin?: number;
            /**
             *
             * Gets or sets the worksheet's top margin, in points, for use when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            topMargin?: number;
            /**
             *
             * Gets or sets the worksheet's print zoom options.
             *
             * [Api set: ExcelApi 1.9]
             */
            zoom?: Excel.PageLayoutZoomOptions;
        }
        /** An interface describing the data returned by calling "headerFooter.toJSON()". */
        interface HeaderFooterData {
            /**
             *
             * Gets or sets the center footer of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            centerFooter?: string;
            /**
             *
             * Gets or sets the center header of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            centerHeader?: string;
            /**
             *
             * Gets or sets the left footer of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            leftFooter?: string;
            /**
             *
             * Gets or sets the left header of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            leftHeader?: string;
            /**
             *
             * Gets or sets the right footer of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            rightFooter?: string;
            /**
             *
             * Gets or sets the right header of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            rightHeader?: string;
        }
        /** An interface describing the data returned by calling "headerFooterGroup.toJSON()". */
        interface HeaderFooterGroupData {
            /**
            *
            * The general header/footer, used for all pages unless even/odd or first page is specified.
            *
            * [Api set: ExcelApi 1.9]
            */
            defaultForAllPages?: Excel.Interfaces.HeaderFooterData;
            /**
            *
            * The header/footer to use for even pages, odd header/footer needs to be specified for odd pages.
            *
            * [Api set: ExcelApi 1.9]
            */
            evenPages?: Excel.Interfaces.HeaderFooterData;
            /**
            *
            * The first page header/footer, for all other pages general or even/odd is used.
            *
            * [Api set: ExcelApi 1.9]
            */
            firstPage?: Excel.Interfaces.HeaderFooterData;
            /**
            *
            * The header/footer to use for odd pages, even header/footer needs to be specified for even pages.
            *
            * [Api set: ExcelApi 1.9]
            */
            oddPages?: Excel.Interfaces.HeaderFooterData;
            /**
             *
             * Gets or sets the state of which headers/footers are set. See Excel.HeaderFooterState for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            state?: Excel.HeaderFooterState | "Default" | "FirstAndDefault" | "OddAndEven" | "FirstOddAndEven";
            /**
             *
             * Gets or sets a flag indicating if headers/footers are aligned with the page margins set in the page layout options for the worksheet.
             *
             * [Api set: ExcelApi 1.9]
             */
            useSheetMargins?: boolean;
            /**
             *
             * Gets or sets a flag indicating if headers/footers should be scaled by the page percentage scale set in the page layout options for the worksheet.
             *
             * [Api set: ExcelApi 1.9]
             */
            useSheetScale?: boolean;
        }
        /** An interface describing the data returned by calling "pageBreak.toJSON()". */
        interface PageBreakData {
            /**
             *
             * Represents the column index for the page break
             *
             * [Api set: ExcelApi 1.9]
             */
            columnIndex?: number;
            /**
             *
             * Represents the row index for the page break
             *
             * [Api set: ExcelApi 1.9]
             */
            rowIndex?: number;
        }
        /** An interface describing the data returned by calling "pageBreakCollection.toJSON()". */
        interface PageBreakCollectionData {
            items?: Excel.Interfaces.PageBreakData[];
        }
        /** An interface describing the data returned by calling "rangeCollection.toJSON()". */
        interface RangeCollectionData {
            items?: Excel.Interfaces.RangeData[];
        }
        /** An interface describing the data returned by calling "shapeCollection.toJSON()". */
        interface ShapeCollectionData {
            items?: Excel.Interfaces.ShapeData[];
        }
        /** An interface describing the data returned by calling "shape.toJSON()". */
        interface ShapeData {
            /**
            *
            * Returns the fill formatting of this shape. Read-only.
            *
            * [Api set: ExcelApi 1.9]
            */
            fill?: Excel.Interfaces.ShapeFillData;
            /**
            *
            * Returns the line formatting of this shape. Read-only.
            *
            * [Api set: ExcelApi 1.9]
            */
            lineFormat?: Excel.Interfaces.ShapeLineFormatData;
            /**
             *
             * Returns or sets the alternative description text for a Shape object.
             *
             * [Api set: ExcelApi 1.9]
             */
            altTextDescription?: string;
            /**
             *
             * Returns or sets the alternative title text for a Shape object.
             *
             * [Api set: ExcelApi 1.9]
             */
            altTextTitle?: string;
            /**
             *
             * Returns the number of connection sites on this shape. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            connectionSiteCount?: number;
            /**
             *
             * Represents the geometric shape type of this geometric shape. See Excel.GeometricShapeType for details. Returns null if the shape type is not "GeometricShape".
             *
             * [Api set: ExcelApi 1.9]
             */
            geometricShapeType?: Excel.GeometricShapeType | "LineInverse" | "Triangle" | "RightTriangle" | "Rectangle" | "Diamond" | "Parallelogram" | "Trapezoid" | "NonIsoscelesTrapezoid" | "Pentagon" | "Hexagon" | "Heptagon" | "Octagon" | "Decagon" | "Dodecagon" | "Star4" | "Star5" | "Star6" | "Star7" | "Star8" | "Star10" | "Star12" | "Star16" | "Star24" | "Star32" | "RoundRectangle" | "Round1Rectangle" | "Round2SameRectangle" | "Round2DiagonalRectangle" | "SnipRoundRectangle" | "Snip1Rectangle" | "Snip2SameRectangle" | "Snip2DiagonalRectangle" | "Plaque" | "Ellipse" | "Teardrop" | "HomePlate" | "Chevron" | "PieWedge" | "Pie" | "BlockArc" | "Donut" | "NoSmoking" | "RightArrow" | "LeftArrow" | "UpArrow" | "DownArrow" | "StripedRightArrow" | "NotchedRightArrow" | "BentUpArrow" | "LeftRightArrow" | "UpDownArrow" | "LeftUpArrow" | "LeftRightUpArrow" | "QuadArrow" | "LeftArrowCallout" | "RightArrowCallout" | "UpArrowCallout" | "DownArrowCallout" | "LeftRightArrowCallout" | "UpDownArrowCallout" | "QuadArrowCallout" | "BentArrow" | "UturnArrow" | "CircularArrow" | "LeftCircularArrow" | "LeftRightCircularArrow" | "CurvedRightArrow" | "CurvedLeftArrow" | "CurvedUpArrow" | "CurvedDownArrow" | "SwooshArrow" | "Cube" | "Can" | "LightningBolt" | "Heart" | "Sun" | "Moon" | "SmileyFace" | "IrregularSeal1" | "IrregularSeal2" | "FoldedCorner" | "Bevel" | "Frame" | "HalfFrame" | "Corner" | "DiagonalStripe" | "Chord" | "Arc" | "LeftBracket" | "RightBracket" | "LeftBrace" | "RightBrace" | "BracketPair" | "BracePair" | "Callout1" | "Callout2" | "Callout3" | "AccentCallout1" | "AccentCallout2" | "AccentCallout3" | "BorderCallout1" | "BorderCallout2" | "BorderCallout3" | "AccentBorderCallout1" | "AccentBorderCallout2" | "AccentBorderCallout3" | "WedgeRectCallout" | "WedgeRRectCallout" | "WedgeEllipseCallout" | "CloudCallout" | "Cloud" | "Ribbon" | "Ribbon2" | "EllipseRibbon" | "EllipseRibbon2" | "LeftRightRibbon" | "VerticalScroll" | "HorizontalScroll" | "Wave" | "DoubleWave" | "Plus" | "FlowChartProcess" | "FlowChartDecision" | "FlowChartInputOutput" | "FlowChartPredefinedProcess" | "FlowChartInternalStorage" | "FlowChartDocument" | "FlowChartMultidocument" | "FlowChartTerminator" | "FlowChartPreparation" | "FlowChartManualInput" | "FlowChartManualOperation" | "FlowChartConnector" | "FlowChartPunchedCard" | "FlowChartPunchedTape" | "FlowChartSummingJunction" | "FlowChartOr" | "FlowChartCollate" | "FlowChartSort" | "FlowChartExtract" | "FlowChartMerge" | "FlowChartOfflineStorage" | "FlowChartOnlineStorage" | "FlowChartMagneticTape" | "FlowChartMagneticDisk" | "FlowChartMagneticDrum" | "FlowChartDisplay" | "FlowChartDelay" | "FlowChartAlternateProcess" | "FlowChartOffpageConnector" | "ActionButtonBlank" | "ActionButtonHome" | "ActionButtonHelp" | "ActionButtonInformation" | "ActionButtonForwardNext" | "ActionButtonBackPrevious" | "ActionButtonEnd" | "ActionButtonBeginning" | "ActionButtonReturn" | "ActionButtonDocument" | "ActionButtonSound" | "ActionButtonMovie" | "Gear6" | "Gear9" | "Funnel" | "MathPlus" | "MathMinus" | "MathMultiply" | "MathDivide" | "MathEqual" | "MathNotEqual" | "CornerTabs" | "SquareTabs" | "PlaqueTabs" | "ChartX" | "ChartStar" | "ChartPlus";
            /**
             *
             * Represents the height, in points, of the shape.
            Throws an invalid argument exception when set with a negative value or zero as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            height?: number;
            /**
             *
             * Represents the shape identifier. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            id?: string;
            /**
             *
             * The distance, in points, from the left side of the shape to the left side of the worksheet.
            Throws an invalid argument exception when set with a negative value as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            left?: number;
            /**
             *
             * Represents the level of the specified shape. For example, a level of 0 means that the shape is not part of any groups, a level of 1 means the shape is part of a top-level group, and a level of 2 means the shape is part of a sub-group of the top level.
             *
             * [Api set: ExcelApi 1.9]
             */
            level?: number;
            /**
             *
             * Specifies whether or not the aspect ratio of this shape is locked.
             *
             * [Api set: ExcelApi 1.9]
             */
            lockAspectRatio?: boolean;
            /**
             *
             * Represents the name of the shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            name?: string;
            /**
             *
             * Represents the rotation, in degrees, of the shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            rotation?: number;
            /**
             *
             * The distance, in points, from the top edge of the shape to the top edge of the worksheet.
            Throws an invalid argument exception when set with a negative value as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            top?: number;
            /**
             *
             * Returns the type of this shape. See Excel.ShapeType for details. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            type?: Excel.ShapeType | "Unsupported" | "Image" | "GeometricShape" | "Group" | "Line";
            /**
             *
             * Represents the visibility of this shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            visible?: boolean;
            /**
             *
             * Represents the width, in points, of the shape.
            Throws an invalid argument exception when set with a negative value or zero as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            width?: number;
            /**
             *
             * Returns the position of the specified shape in the z-order, with 0 representing the bottom of the order stack. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            zOrderPosition?: number;
        }
        /** An interface describing the data returned by calling "geometricShape.toJSON()". */
        interface GeometricShapeData {
            /**
             *
             * Returns the shape identifier. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            id?: string;
        }
        /** An interface describing the data returned by calling "image.toJSON()". */
        interface ImageData {
            /**
             *
             * Represents the shape identifier for the image object. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            id?: string;
            /**
             *
             * Returns the format of the image. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            format?: Excel.PictureFormat | "UNKNOWN" | "BMP" | "JPEG" | "GIF" | "PNG" | "SVG";
        }
        /** An interface describing the data returned by calling "shapeGroup.toJSON()". */
        interface ShapeGroupData {
            /**
            *
            * Returns the collection of Shape objects. Read-only.
            *
            * [Api set: ExcelApi 1.9]
            */
            shapes?: Excel.Interfaces.ShapeData[];
            /**
             *
             * Represents the shape identifier. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            id?: string;
        }
        /** An interface describing the data returned by calling "groupShapeCollection.toJSON()". */
        interface GroupShapeCollectionData {
            items?: Excel.Interfaces.ShapeData[];
        }
        /** An interface describing the data returned by calling "line.toJSON()". */
        interface LineData {
            /**
             *
             * Represents the length of the arrowhead at the beginning of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            beginArrowheadLength?: Excel.ArrowheadLength | "Short" | "Medium" | "Long";
            /**
             *
             * Represents the style of the arrowhead at the beginning of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            beginArrowheadStyle?: Excel.ArrowheadStyle | "None" | "Triangle" | "Stealth" | "Diamond" | "Oval" | "Open";
            /**
             *
             * Represents the width of the arrowhead at the beginning of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            beginArrowheadWidth?: Excel.ArrowheadWidth | "Narrow" | "Medium" | "Wide";
            /**
             *
             * Represents the connection site to which the beginning of a connector is connected. Read-only. Returns null when the beginning of the line is not attached to any shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            beginConnectedSite?: number;
            /**
             *
             * Represents the length of the arrowhead at the end of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            endArrowheadLength?: Excel.ArrowheadLength | "Short" | "Medium" | "Long";
            /**
             *
             * Represents the style of the arrowhead at the end of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            endArrowheadStyle?: Excel.ArrowheadStyle | "None" | "Triangle" | "Stealth" | "Diamond" | "Oval" | "Open";
            /**
             *
             * Represents the width of the arrowhead at the end of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            endArrowheadWidth?: Excel.ArrowheadWidth | "Narrow" | "Medium" | "Wide";
            /**
             *
             * Represents the connection site to which the end of a connector is connected. Read-only. Returns null when the end of the line is not attached to any shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            endConnectedSite?: number;
            /**
             *
             * Represents the shape identifier. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            id?: string;
            /**
             *
             * Specifies whether or not the beginning of the specified line is connected to a shape. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            isBeginConnected?: boolean;
            /**
             *
             * Specifies whether or not the end of the specified line is connected to a shape. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            isEndConnected?: boolean;
            /**
             *
             * Represents the connector type for the line.
             *
             * [Api set: ExcelApi 1.9]
             */
            connectorType?: Excel.ConnectorType | "Straight" | "Elbow" | "Curve";
        }
        /** An interface describing the data returned by calling "shapeFill.toJSON()". */
        interface ShapeFillData {
            /**
             *
             * Represents the shape fill foreground color in HTML color format, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange")
             *
             * [Api set: ExcelApi 1.9]
             */
            foregroundColor?: string;
            /**
             *
             * Returns or sets the transparency percentage of the fill as a value from 0.0 (opaque) through 1.0 (clear). Returns null if the shape type does not support transparency or the shape fill has inconsistent transparency, such as with a gradient fill type.
             *
             * [Api set: ExcelApi 1.9]
             */
            transparency?: number;
            /**
             *
             * Returns the fill type of the shape. Read-only. See Excel.ShapeFillType for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            type?: Excel.ShapeFillType | "NoFill" | "Solid" | "Gradient" | "Pattern" | "PictureAndTexture" | "Mixed";
        }
        /** An interface describing the data returned by calling "shapeLineFormat.toJSON()". */
        interface ShapeLineFormatData {
            /**
             *
             * Represents the line color in HTML color format, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.9]
             */
            color?: string;
            /**
             *
             * Represents the line style of the shape. Returns null when the line is not visible or there are inconsistent dash styles. See Excel.ShapeLineStyle for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            dashStyle?: Excel.ShapeLineDashStyle | "Dash" | "DashDot" | "DashDotDot" | "LongDash" | "LongDashDot" | "RoundDot" | "Solid" | "SquareDot" | "LongDashDotDot" | "SystemDash" | "SystemDot" | "SystemDashDot";
            /**
             *
             * Represents the line style of the shape. Returns null when the line is not visible or there are inconsistent styles. See Excel.ShapeLineStyle for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            style?: Excel.ShapeLineStyle | "Single" | "ThickBetweenThin" | "ThickThin" | "ThinThick" | "ThinThin";
            /**
             *
             * Represents the degree of transparency of the specified line as a value from 0.0 (opaque) through 1.0 (clear). Returns null when the shape has inconsistent transparencies.
             *
             * [Api set: ExcelApi 1.9]
             */
            transparency?: number;
            /**
             *
             * Represents whether or not the line formatting of a shape element is visible. Returns null when the shape has inconsistent visibilities.
             *
             * [Api set: ExcelApi 1.9]
             */
            visible?: boolean;
            /**
             *
             * Represents the weight of the line, in points. Returns null when the line is not visible or there are inconsistent line weights.
             *
             * [Api set: ExcelApi 1.9]
             */
            weight?: number;
        }
        /** An interface describing the data returned by calling "textFrame.toJSON()". */
        interface TextFrameData {
            /**
             *
             * Gets or sets the automatic sizing settings for the text frame. A text frame can be set to automatically fit the text to the text frame, to automatically fit the text frame to the text, or not perform any automatic sizing.
             *
             * [Api set: ExcelApi 1.9]
             */
            autoSizeSetting?: Excel.ShapeAutoSize | "AutoSizeNone" | "AutoSizeTextToFitShape" | "AutoSizeShapeToFitText" | "AutoSizeMixed";
            /**
             *
             * Represents the bottom margin, in points, of the text frame.
             *
             * [Api set: ExcelApi 1.9]
             */
            bottomMargin?: number;
            /**
             *
             * Specifies whether the text frame contains text.
             *
             * [Api set: ExcelApi 1.9]
             */
            hasText?: boolean;
            /**
             *
             * Represents the horizontal alignment of the text frame. See Excel.ShapeTextHorizontalAlignment for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            horizontalAlignment?: Excel.ShapeTextHorizontalAlignment | "Left" | "Center" | "Right" | "Justify" | "JustifyLow" | "Distributed" | "ThaiDistributed";
            /**
             *
             * Represents the horizontal overflow behavior of the text frame. See Excel.ShapeTextHorizontalOverflow for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            horizontalOverflow?: Excel.ShapeTextHorizontalOverflow | "Overflow" | "Clip";
            /**
             *
             * Represents the left margin, in points, of the text frame.
             *
             * [Api set: ExcelApi 1.9]
             */
            leftMargin?: number;
            /**
             *
             * Represents the text orientation of the text frame. See Excel.ShapeTextOrientation for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            orientation?: Excel.ShapeTextOrientation | "Horizontal" | "Vertical" | "Vertical270" | "WordArtVertical" | "EastAsianVertical" | "MongolianVertical" | "WordArtVerticalRTL";
            /**
             *
             * Represents the reading order of the text frame, either left-to-right or right-to-left. See Excel.ShapeTextReadingOrder for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            readingOrder?: Excel.ShapeTextReadingOrder | "LeftToRight" | "RightToLeft";
            /**
             *
             * Represents the right margin, in points, of the text frame.
             *
             * [Api set: ExcelApi 1.9]
             */
            rightMargin?: number;
            /**
             *
             * Represents the top margin, in points, of the text frame.
             *
             * [Api set: ExcelApi 1.9]
             */
            topMargin?: number;
            /**
             *
             * Represents the vertical alignment of the text frame. See Excel.ShapeTextVerticalAlignment for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            verticalAlignment?: Excel.ShapeTextVerticalAlignment | "Top" | "Middle" | "Bottom" | "Justified" | "Distributed";
            /**
             *
             * Represents the vertical overflow behavior of the text frame. See Excel.ShapeTextVerticalOverflow for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            verticalOverflow?: Excel.ShapeTextVerticalOverflow | "Overflow" | "Ellipsis" | "Clip";
        }
        /** An interface describing the data returned by calling "textRange.toJSON()". */
        interface TextRangeData {
            /**
            *
            * Returns a ShapeFont object that represents the font attributes for the text range. Read-only.
            *
            * [Api set: ExcelApi 1.9]
            */
            font?: Excel.Interfaces.ShapeFontData;
            /**
             *
             * Represents the plain text content of the text range.
             *
             * [Api set: ExcelApi 1.9]
             */
            text?: string;
        }
        /** An interface describing the data returned by calling "shapeFont.toJSON()". */
        interface ShapeFontData {
            /**
             *
             * Represents the bold status of font. Returns null the TextRange includes both bold and non-bold text fragments.
             *
             * [Api set: ExcelApi 1.9]
             */
            bold?: boolean;
            /**
             *
             * The HTML color code representation of the text color (e.g. "#FF0000" represents red). Returns null if the TextRange includes text fragments with different colors.
             *
             * [Api set: ExcelApi 1.9]
             */
            color?: string;
            /**
             *
             * Represents the italic status of font. Returns null if the TextRange includes both italic and non-italic text fragments.
             *
             * [Api set: ExcelApi 1.9]
             */
            italic?: boolean;
            /**
             *
             * Represents font name (e.g. "Calibri"). If the text is Complex Script or East Asian language, this is the corresponding font name; otherwise it is the Latin font name.
             *
             * [Api set: ExcelApi 1.9]
             */
            name?: string;
            /**
             *
             * Represents font size in points (e.g. 11). Returns null if the TextRange includes text fragments with different font sizes.
             *
             * [Api set: ExcelApi 1.9]
             */
            size?: number;
            /**
             *
             * Type of underline applied to the font. Returns null if the TextRange includes text fragments with different underline styles. See Excel.ShapeFontUnderlineStyle for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            underline?: Excel.ShapeFontUnderlineStyle | "None" | "Single" | "Double" | "Heavy" | "Dotted" | "DottedHeavy" | "Dash" | "DashHeavy" | "DashLong" | "DashLongHeavy" | "DotDash" | "DotDashHeavy" | "DotDotDash" | "DotDotDashHeavy" | "Wavy" | "WavyHeavy" | "WavyDouble";
        }
        /** An interface describing the data returned by calling "functionResult.toJSON()". */
        interface FunctionResultData<T> {
            /**
             *
             * Error value (such as "#DIV/0") representing the error. If the error string is not set, then the function succeeded, and its result is written to the Value field. The error is always in the English locale.
             *
             * [Api set: ExcelApi 1.2]
             */
            error?: string;
            /**
             *
             * The value of function evaluation. The value field will be populated only if no error has occurred (i.e., the Error property is not set).
             *
             * [Api set: ExcelApi 1.2]
             */
            value?: T;
        }
        /**
         *
         * Represents the Excel Runtime class.
         *
         * [Api set: ExcelApi 1.5]
         */
        interface RuntimeLoadOptions {
            $all?: boolean;
            /**
             *
             * Toggle JavaScript events in the current task pane or content add-in.
             *
             * [Api set: ExcelApi 1.8]
             */
            enableEvents?: boolean;
        }
        /**
         *
         * Represents the Excel application that manages the workbook.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ApplicationLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns the Iterative Calculation settings.
            On Excel for Windows and Excel for Mac, the settings will apply to the Excel Application.
            On Excel Online and Excel for other platforms, the settings will apply to the active workbook.
            *
            * [Api set: ExcelApi 1.9]
            */
            iterativeCalculation?: Excel.Interfaces.IterativeCalculationLoadOptions;
            /**
             *
             * Returns the Excel calculation engine version used for the last full recalculation. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            calculationEngineVersion?: boolean;
            /**
             *
             * Returns the calculation mode used in the workbook, as defined by the constants in Excel.CalculationMode. Possible values are: `Automatic`, where Excel controls recalculation; `AutomaticExceptTables`, where Excel controls recalculation but ignores changes in tables; `Manual`, where calculation is done when the user requests it.
             *
             * [Api set: ExcelApi 1.1 for get, 1.8 for set]
             */
            calculationMode?: boolean;
            /**
             *
             * Returns the calculation state of the application. See Excel.CalculationState for details. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            calculationState?: boolean;
        }
        /**
         *
         * Represents the Iterative Calculation settings.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface IterativeCalculationLoadOptions {
            $all?: boolean;
            /**
             *
             * True if Excel will use iteration to resolve circular references.
             *
             * [Api set: ExcelApi 1.9]
             */
            enabled?: boolean;
            /**
             *
             * Returns or sets the maximum amount of change between each iteration as Excel resolves circular references.
             *
             * [Api set: ExcelApi 1.9]
             */
            maxChange?: boolean;
            /**
             *
             * Returns or sets the maximum number of iterations that Excel can use to resolve a circular reference.
             *
             * [Api set: ExcelApi 1.9]
             */
            maxIteration?: boolean;
        }
        /**
         *
         * Workbook is the top level object which contains related workbook objects such as worksheets, tables, ranges, etc.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface WorkbookLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the Excel application instance that contains this workbook.
            *
            * [Api set: ExcelApi 1.1]
            */
            application?: Excel.Interfaces.ApplicationLoadOptions;
            /**
            *
            * Represents a collection of bindings that are part of the workbook.
            *
            * [Api set: ExcelApi 1.1]
            */
            bindings?: Excel.Interfaces.BindingCollectionLoadOptions;
            /**
            *
            * Gets the workbook properties.
            *
            * [Api set: ExcelApi 1.7]
            */
            properties?: Excel.Interfaces.DocumentPropertiesLoadOptions;
            /**
            *
            * Returns workbook protection object for a workbook.
            *
            * [Api set: ExcelApi 1.7]
            */
            protection?: Excel.Interfaces.WorkbookProtectionLoadOptions;
            /**
            *
            * Represents a collection of tables associated with the workbook.
            *
            * [Api set: ExcelApi 1.1]
            */
            tables?: Excel.Interfaces.TableCollectionLoadOptions;
            /**
             *
             * Specifies whether or not the workbook is in autosave mode. Read-Only.
             *
             * [Api set: ExcelApi 1.9]
             */
            autoSave?: boolean;
            /**
             *
             * Returns a number about the version of Excel Calculation Engine. Read-Only.
             *
             * [Api set: ExcelApi 1.9]
             */
            calculationEngineVersion?: boolean;
            /**
             *
             * True if all charts in the workbook are tracking the actual data points to which they are attached.
            False if the charts track the index of the data points.
             *
             * [Api set: ExcelApi 1.9]
             */
            chartDataPointTrack?: boolean;
            /**
             *
             * Specifies whether or not changes have been made since the workbook was last saved.
            You can set this property to true if you want to close a modified workbook without either saving it or being prompted to save it.
             *
             * [Api set: ExcelApi 1.9]
             */
            isDirty?: boolean;
            /**
             *
             * Gets the workbook name. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            name?: boolean;
            /**
             *
             * Specifies whether or not the workbook has ever been saved locally or online. Read-Only.
             *
             * [Api set: ExcelApi 1.9]
             */
            previouslySaved?: boolean;
            /**
             *
             * True if the workbook is open in Read-only mode. Read-only.
             *
             * [Api set: ExcelApi 1.8]
             */
            readOnly?: boolean;
            /**
             *
             * True if calculations in this workbook will be done using only the precision of the numbers as they're displayed.
            Data will permanently lose accuracy when switching this property from false to true.
             *
             * [Api set: ExcelApi 1.9]
             */
            usePrecisionAsDisplayed?: boolean;
        }
        /**
         *
         * Represents the protection of a workbook object.
         *
         * [Api set: ExcelApi 1.7]
         */
        interface WorkbookProtectionLoadOptions {
            $all?: boolean;
            /**
             *
             * Indicates if the workbook is protected. Read-Only.
             *
             * [Api set: ExcelApi 1.7]
             */
            protected?: boolean;
        }
        /**
         *
         * An Excel worksheet is a grid of cells. It can contain data, tables, charts, etc.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface WorksheetLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the AutoFilter object of the worksheet.
            *
            * [Api set: ExcelApi 1.9]
            */
            autoFilter?: Excel.Interfaces.AutoFilterLoadOptions;
            /**
            *
            * Returns collection of charts that are part of the worksheet.
            *
            * [Api set: ExcelApi 1.1]
            */
            charts?: Excel.Interfaces.ChartCollectionLoadOptions;
            /**
            *
            * Gets the PageLayout object of the worksheet.
            *
            * [Api set: ExcelApi 1.9]
            */
            pageLayout?: Excel.Interfaces.PageLayoutLoadOptions;
            /**
            *
            * Returns sheet protection object for a worksheet.
            *
            * [Api set: ExcelApi 1.2]
            */
            protection?: Excel.Interfaces.WorksheetProtectionLoadOptions;
            /**
            *
            * Collection of tables that are part of the worksheet.
            *
            * [Api set: ExcelApi 1.1]
            */
            tables?: Excel.Interfaces.TableCollectionLoadOptions;
            /**
             *
             * Gets or sets the enableCalculation property of the worksheet.
            True if Excel recalculates the worksheet when necessary. False if Excel doesn't recalculate the sheet.
             *
             * [Api set: ExcelApi 1.9]
             */
            enableCalculation?: boolean;
            /**
             *
             * Returns a value that uniquely identifies the worksheet in a given workbook. The value of the identifier remains the same even when the worksheet is renamed or moved. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            id?: boolean;
            /**
             *
             * The display name of the worksheet.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: boolean;
            /**
             *
             * The zero-based position of the worksheet within the workbook.
             *
             * [Api set: ExcelApi 1.1]
             */
            position?: boolean;
            /**
             *
             * Gets or sets the worksheet's gridlines flag.
            This flag determines whether gridlines are visible to the user.
             *
             * [Api set: ExcelApi 1.8]
             */
            showGridlines?: boolean;
            /**
             *
             * Gets or sets the worksheet's headings flag.
            This flag determines whether headings are visible to the user.
             *
             * [Api set: ExcelApi 1.8]
             */
            showHeadings?: boolean;
            /**
             *
             * Returns the standard (default) height of all the rows in the worksheet, in points. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            standardHeight?: boolean;
            /**
             *
             * Returns or sets the standard (default) width of all the columns in the worksheet.
            One unit of column width is equal to the width of one character in the Normal style. For proportional fonts, the width of the character 0 (zero) is used.
             *
             * [Api set: ExcelApi 1.7]
             */
            standardWidth?: boolean;
            /**
             *
             * Gets or sets the worksheet tab color.
            When retrieving the tab color, if the worksheet is invisible, the value will be null. If the worksheet is visible but the tab color is set to auto, an empty string will be returned. Otherwise, the property will be set to a color, in the form "#123456"
            When setting the color, use an empty-string to set an "auto" color, or a real color otherwise.
             *
             * [Api set: ExcelApi 1.7]
             */
            tabColor?: boolean;
            /**
             *
             * The Visibility of the worksheet.
             *
             * [Api set: ExcelApi 1.1 for reading visibility; 1.2 for setting it.]
             */
            visibility?: boolean;
        }
        /**
         *
         * Represents a collection of worksheet objects that are part of the workbook.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface WorksheetCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Represents the AutoFilter object of the worksheet.
            *
            * [Api set: ExcelApi 1.9]
            */
            autoFilter?: Excel.Interfaces.AutoFilterLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns collection of charts that are part of the worksheet.
            *
            * [Api set: ExcelApi 1.1]
            */
            charts?: Excel.Interfaces.ChartCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the PageLayout object of the worksheet.
            *
            * [Api set: ExcelApi 1.9]
            */
            pageLayout?: Excel.Interfaces.PageLayoutLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns sheet protection object for a worksheet.
            *
            * [Api set: ExcelApi 1.2]
            */
            protection?: Excel.Interfaces.WorksheetProtectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Collection of tables that are part of the worksheet.
            *
            * [Api set: ExcelApi 1.1]
            */
            tables?: Excel.Interfaces.TableCollectionLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the enableCalculation property of the worksheet.
            True if Excel recalculates the worksheet when necessary. False if Excel doesn't recalculate the sheet.
             *
             * [Api set: ExcelApi 1.9]
             */
            enableCalculation?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns a value that uniquely identifies the worksheet in a given workbook. The value of the identifier remains the same even when the worksheet is renamed or moved. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The display name of the worksheet.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The zero-based position of the worksheet within the workbook.
             *
             * [Api set: ExcelApi 1.1]
             */
            position?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the worksheet's gridlines flag.
            This flag determines whether gridlines are visible to the user.
             *
             * [Api set: ExcelApi 1.8]
             */
            showGridlines?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the worksheet's headings flag.
            This flag determines whether headings are visible to the user.
             *
             * [Api set: ExcelApi 1.8]
             */
            showHeadings?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the standard (default) height of all the rows in the worksheet, in points. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            standardHeight?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the standard (default) width of all the columns in the worksheet.
            One unit of column width is equal to the width of one character in the Normal style. For proportional fonts, the width of the character 0 (zero) is used.
             *
             * [Api set: ExcelApi 1.7]
             */
            standardWidth?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the worksheet tab color.
            When retrieving the tab color, if the worksheet is invisible, the value will be null. If the worksheet is visible but the tab color is set to auto, an empty string will be returned. Otherwise, the property will be set to a color, in the form "#123456"
            When setting the color, use an empty-string to set an "auto" color, or a real color otherwise.
             *
             * [Api set: ExcelApi 1.7]
             */
            tabColor?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The Visibility of the worksheet.
             *
             * [Api set: ExcelApi 1.1 for reading visibility; 1.2 for setting it.]
             */
            visibility?: boolean;
        }
        /**
         *
         * Represents the protection of a sheet object.
         *
         * [Api set: ExcelApi 1.2]
         */
        interface WorksheetProtectionLoadOptions {
            $all?: boolean;
            /**
             *
             * Sheet protection options. Read-only.
             *
             * [Api set: ExcelApi 1.2]
             */
            options?: boolean;
            /**
             *
             * Indicates if the worksheet is protected. Read-only.
             *
             * [Api set: ExcelApi 1.2]
             */
            protected?: boolean;
        }
        /**
         *
         * Range represents a set of one or more contiguous cells such as a cell, a row, a column, block of cells, etc.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface RangeLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns a data validation object.
            *
            * [Api set: ExcelApi 1.8]
            */
            dataValidation?: Excel.Interfaces.DataValidationLoadOptions;
            /**
            *
            * Returns a format object, encapsulating the range's font, fill, borders, alignment, and other properties.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.RangeFormatLoadOptions;
            /**
            *
            * The worksheet containing the current range.
            *
            * [Api set: ExcelApi 1.1]
            */
            worksheet?: Excel.Interfaces.WorksheetLoadOptions;
            /**
             *
             * Represents the range reference in A1-style. Address value will contain the Sheet reference (e.g. "Sheet1!A1:B4"). Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            address?: boolean;
            /**
             *
             * Represents range reference for the specified range in the language of the user. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            addressLocal?: boolean;
            /**
             *
             * Number of cells in the range. This API will return -1 if the cell count exceeds 2^31-1 (2,147,483,647). Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            cellCount?: boolean;
            /**
             *
             * Represents the total number of columns in the range. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            columnCount?: boolean;
            /**
             *
             * Represents if all columns of the current range are hidden.
             *
             * [Api set: ExcelApi 1.2]
             */
            columnHidden?: boolean;
            /**
             *
             * Represents the column number of the first cell in the range. Zero-indexed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            columnIndex?: boolean;
            /**
             *
             * Represents the formula in A1-style notation.
            When setting formulas to a range, the value argument can be either a single value (a string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            formulas?: boolean;
            /**
             *
             * Represents the formula in A1-style notation, in the user's language and number-formatting locale.  For example, the English "=SUM(A1, 1.5)" formula would become "=SUMME(A1; 1,5)" in German.
            When setting formulas to a range, the value argument can be either a single value (a string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            formulasLocal?: boolean;
            /**
             *
             * Represents the formula in R1C1-style notation.
            When setting formulas to a range, the value argument can be either a single value (a string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.2]
             */
            formulasR1C1?: boolean;
            /**
             *
             * Represents if all cells of the current range are hidden. Read-only.
             *
             * [Api set: ExcelApi 1.2]
             */
            hidden?: boolean;
            /**
             *
             * Represents the hyperlink for the current range.
             *
             * [Api set: ExcelApi 1.7]
             */
            hyperlink?: boolean;
            /**
             *
             * Represents if the current range is an entire column. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            isEntireColumn?: boolean;
            /**
             *
             * Represents if the current range is an entire row. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            isEntireRow?: boolean;
            /**
             *
             * Represents the data type state of each cell. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            linkedDataTypeState?: boolean;
            /**
             *
             * Represents Excel's number format code for the given range.
            When setting number format to a range, the value argument can be either a single value (string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            numberFormat?: boolean;
            /**
             *
             * Represents Excel's number format code for the given range as a string in the language of the user.
            When setting number format local to a range, the value argument can be either a single value (string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.7]
             */
            numberFormatLocal?: boolean;
            /**
             *
             * Returns the total number of rows in the range. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            rowCount?: boolean;
            /**
             *
             * Represents if all rows of the current range are hidden.
             *
             * [Api set: ExcelApi 1.2]
             */
            rowHidden?: boolean;
            /**
             *
             * Returns the row number of the first cell in the range. Zero-indexed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            rowIndex?: boolean;
            /**
             *
             * Represents the style of the current range.
            If the styles of the cells are inconsistent, null will be returned.
            For custom styles, the style name will be returned. For built-in styles, a string representing a value in the BuiltInStyle enum will be returned.
             *
             * [Api set: ExcelApi 1.7]
             */
            style?: boolean;
            /**
             *
             * Text values of the specified range. The Text value will not depend on the cell width. The # sign substitution that happens in Excel UI will not affect the text value returned by the API. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            text?: boolean;
            /**
             *
             * Represents the type of data of each cell. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            valueTypes?: boolean;
            /**
             *
             * Represents the raw values of the specified range. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
            When setting values to a range, the value argument can be either a single value (string, number or boolean) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            values?: boolean;
        }
        /**
         *
         * RangeAreas represents a collection of one or more rectangular ranges in the same worksheet.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface RangeAreasLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns a dataValidation object for all ranges in the RangeAreas.
            *
            * [Api set: ExcelApi 1.9]
            */
            dataValidation?: Excel.Interfaces.DataValidationLoadOptions;
            /**
            *
            * Returns a rangeFormat object, encapsulating the the font, fill, borders, alignment, and other properties for all ranges in the RangeAreas object.
            *
            * [Api set: ExcelApi 1.9]
            */
            format?: Excel.Interfaces.RangeFormatLoadOptions;
            /**
            *
            * Returns the worksheet for the current RangeAreas.
            *
            * [Api set: ExcelApi 1.9]
            */
            worksheet?: Excel.Interfaces.WorksheetLoadOptions;
            /**
             *
             * Returns the RageAreas reference in A1-style. Address value will contain the worksheet name for each rectangular block of cells (e.g. "Sheet1!A1:B4, Sheet1!D1:D4"). Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            address?: boolean;
            /**
             *
             * Returns the RageAreas reference in the user locale. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            addressLocal?: boolean;
            /**
             *
             * Returns the number of rectangular ranges that comprise this RangeAreas object.
             *
             * [Api set: ExcelApi 1.9]
             */
            areaCount?: boolean;
            /**
             *
             * Returns the number of cells in the RangeAreas object, summing up the cell counts of all of the individual rectangular ranges. Returns -1 if the cell count exceeds 2^31-1 (2,147,483,647). Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            cellCount?: boolean;
            /**
             *
             * Indicates whether all the ranges on this RangeAreas object represent entire columns (e.g., "A:C, Q:Z"). Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            isEntireColumn?: boolean;
            /**
             *
             * Indicates whether all the ranges on this RangeAreas object represent entire rows (e.g., "1:3, 5:7"). Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            isEntireRow?: boolean;
            /**
             *
             * Represents the style for all ranges in this RangeAreas object.
            If the styles of the cells are inconsistent, null will be returned.
            For custom styles, the style name will be returned. For built-in styles, a string representing a value in the BuiltInStyle enum will be returned.
             *
             * [Api set: ExcelApi 1.9]
             */
            style?: boolean;
        }
        /**
         *
         * RangeView represents a set of visible cells of the parent range.
         *
         * [Api set: ExcelApi 1.3]
         */
        interface RangeViewLoadOptions {
            $all?: boolean;
            /**
             *
             * Represents the cell addresses of the RangeView. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            cellAddresses?: boolean;
            /**
             *
             * Returns the number of visible columns. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            columnCount?: boolean;
            /**
             *
             * Represents the formula in A1-style notation.
             *
             * [Api set: ExcelApi 1.3]
             */
            formulas?: boolean;
            /**
             *
             * Represents the formula in A1-style notation, in the user's language and number-formatting locale.  For example, the English "=SUM(A1, 1.5)" formula would become "=SUMME(A1; 1,5)" in German.
             *
             * [Api set: ExcelApi 1.3]
             */
            formulasLocal?: boolean;
            /**
             *
             * Represents the formula in R1C1-style notation.
             *
             * [Api set: ExcelApi 1.3]
             */
            formulasR1C1?: boolean;
            /**
             *
             * Returns a value that represents the index of the RangeView. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            index?: boolean;
            /**
             *
             * Represents Excel's number format code for the given cell.
             *
             * [Api set: ExcelApi 1.3]
             */
            numberFormat?: boolean;
            /**
             *
             * Returns the number of visible rows. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            rowCount?: boolean;
            /**
             *
             * Text values of the specified range. The Text value will not depend on the cell width. The # sign substitution that happens in Excel UI will not affect the text value returned by the API. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            text?: boolean;
            /**
             *
             * Represents the type of data of each cell. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            valueTypes?: boolean;
            /**
             *
             * Represents the raw values of the specified range view. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
             *
             * [Api set: ExcelApi 1.3]
             */
            values?: boolean;
        }
        /**
         *
         * Represents a collection of RangeView objects.
         *
         * [Api set: ExcelApi 1.3]
         */
        interface RangeViewCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the cell addresses of the RangeView. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            cellAddresses?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the number of visible columns. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            columnCount?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the formula in A1-style notation.
             *
             * [Api set: ExcelApi 1.3]
             */
            formulas?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the formula in A1-style notation, in the user's language and number-formatting locale.  For example, the English "=SUM(A1, 1.5)" formula would become "=SUMME(A1; 1,5)" in German.
             *
             * [Api set: ExcelApi 1.3]
             */
            formulasLocal?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the formula in R1C1-style notation.
             *
             * [Api set: ExcelApi 1.3]
             */
            formulasR1C1?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns a value that represents the index of the RangeView. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            index?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents Excel's number format code for the given cell.
             *
             * [Api set: ExcelApi 1.3]
             */
            numberFormat?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the number of visible rows. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            rowCount?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Text values of the specified range. The Text value will not depend on the cell width. The # sign substitution that happens in Excel UI will not affect the text value returned by the API. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            text?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the type of data of each cell. Read-only.
             *
             * [Api set: ExcelApi 1.3]
             */
            valueTypes?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the raw values of the specified range view. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
             *
             * [Api set: ExcelApi 1.3]
             */
            values?: boolean;
        }
        /**
         *
         * Represents a collection of key-value pair setting objects that are part of the workbook. The scope is limited to per file and add-in (task-pane or content) combination.
         *
         * [Api set: ExcelApi 1.4]
         */
        interface SettingCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the key that represents the id of the Setting. Read-only.
             *
             * [Api set: ExcelApi 1.4]
             */
            key?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the value stored for this setting.
             *
             * [Api set: ExcelApi 1.4]
             */
            value?: boolean;
        }
        /**
         *
         * Setting represents a key-value pair of a setting persisted to the document (per file per add-in). These custom key-value pair can be used to store state or lifecycle information needed by the content or task-pane add-in. Note that settings are persisted in the document and hence it is not a place to store any sensitive or protected information such as user information and password.
         *
         * [Api set: ExcelApi 1.4]
         */
        interface SettingLoadOptions {
            $all?: boolean;
            /**
             *
             * Returns the key that represents the id of the Setting. Read-only.
             *
             * [Api set: ExcelApi 1.4]
             */
            key?: boolean;
            /**
             *
             * Represents the value stored for this setting.
             *
             * [Api set: ExcelApi 1.4]
             */
            value?: boolean;
        }
        /**
         *
         * A collection of all the NamedItem objects that are part of the workbook or worksheet, depending on how it was reached.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface NamedItemCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Returns an object containing values and types of the named item.
            *
            * [Api set: ExcelApi 1.7]
            */
            arrayValues?: Excel.Interfaces.NamedItemArrayValuesLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the worksheet on which the named item is scoped to. Throws an error if the item is scoped to the workbook instead.
            *
            * [Api set: ExcelApi 1.4]
            */
            worksheet?: Excel.Interfaces.WorksheetLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the worksheet on which the named item is scoped to. Returns a null object if the item is scoped to the workbook instead.
            *
            * [Api set: ExcelApi 1.4]
            */
            worksheetOrNullObject?: Excel.Interfaces.WorksheetLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Represents the comment associated with this name.
             *
             * [Api set: ExcelApi 1.4]
             */
            comment?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the formula of the named item.  Formula always starts with a '=' sign.
             *
             * [Api set: ExcelApi 1.7]
             */
            formula?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The name of the object. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether the name is scoped to the workbook or to a specific worksheet. Possible values are: Worksheet, Workbook. Read-only.
             *
             * [Api set: ExcelApi 1.4]
             */
            scope?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates the type of the value returned by the name's formula. See Excel.NamedItemType for details. Read-only.
             *
             * [Api set: ExcelApi 1.1 for String,Integer,Double,Boolean,Range,Error; 1.7 for Array]
             */
            type?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the value computed by the name's formula. For a named range, will return the range address. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            value?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Specifies whether the object is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
        }
        /**
         *
         * Represents a defined name for a range of cells or value. Names can be primitive named objects (as seen in the type below), range object, or a reference to a range. This object can be used to obtain range object associated with names.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface NamedItemLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns an object containing values and types of the named item.
            *
            * [Api set: ExcelApi 1.7]
            */
            arrayValues?: Excel.Interfaces.NamedItemArrayValuesLoadOptions;
            /**
            *
            * Returns the worksheet on which the named item is scoped to. Throws an error if the item is scoped to the workbook instead.
            *
            * [Api set: ExcelApi 1.4]
            */
            worksheet?: Excel.Interfaces.WorksheetLoadOptions;
            /**
            *
            * Returns the worksheet on which the named item is scoped to. Returns a null object if the item is scoped to the workbook instead.
            *
            * [Api set: ExcelApi 1.4]
            */
            worksheetOrNullObject?: Excel.Interfaces.WorksheetLoadOptions;
            /**
             *
             * Represents the comment associated with this name.
             *
             * [Api set: ExcelApi 1.4]
             */
            comment?: boolean;
            /**
             *
             * Gets or sets the formula of the named item.  Formula always starts with a '=' sign.
             *
             * [Api set: ExcelApi 1.7]
             */
            formula?: boolean;
            /**
             *
             * The name of the object. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: boolean;
            /**
             *
             * Indicates whether the name is scoped to the workbook or to a specific worksheet. Possible values are: Worksheet, Workbook. Read-only.
             *
             * [Api set: ExcelApi 1.4]
             */
            scope?: boolean;
            /**
             *
             * Indicates the type of the value returned by the name's formula. See Excel.NamedItemType for details. Read-only.
             *
             * [Api set: ExcelApi 1.1 for String,Integer,Double,Boolean,Range,Error; 1.7 for Array]
             */
            type?: boolean;
            /**
             *
             * Represents the value computed by the name's formula. For a named range, will return the range address. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            value?: boolean;
            /**
             *
             * Specifies whether the object is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
        }
        /**
         *
         * Represents an object containing values and types of a named item.
         *
         * [Api set: ExcelApi 1.7]
         */
        interface NamedItemArrayValuesLoadOptions {
            $all?: boolean;
            /**
             *
             * Represents the types for each item in the named item array
             *
             * [Api set: ExcelApi 1.7]
             */
            types?: boolean;
            /**
             *
             * Represents the values of each item in the named item array.
             *
             * [Api set: ExcelApi 1.7]
             */
            values?: boolean;
        }
        /**
         *
         * Represents an Office.js binding that is defined in the workbook.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface BindingLoadOptions {
            $all?: boolean;
            /**
             *
             * Represents binding identifier. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            id?: boolean;
            /**
             *
             * Returns the type of the binding. See Excel.BindingType for details. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            type?: boolean;
        }
        /**
         *
         * Represents the collection of all the binding objects that are part of the workbook.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface BindingCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents binding identifier. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the type of the binding. See Excel.BindingType for details. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            type?: boolean;
        }
        /**
         *
         * Represents a collection of all the tables that are part of the workbook or worksheet, depending on how it was reached.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface TableCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Represents the AutoFilter object of the table.
            *
            * [Api set: ExcelApi 1.9]
            */
            autoFilter?: Excel.Interfaces.AutoFilterLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents a collection of all the columns in the table.
            *
            * [Api set: ExcelApi 1.1]
            */
            columns?: Excel.Interfaces.TableColumnCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents a collection of all the rows in the table.
            *
            * [Api set: ExcelApi 1.1]
            */
            rows?: Excel.Interfaces.TableRowCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents the sorting for the table.
            *
            * [Api set: ExcelApi 1.2]
            */
            sort?: Excel.Interfaces.TableSortLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: The worksheet containing the current table.
            *
            * [Api set: ExcelApi 1.2]
            */
            worksheet?: Excel.Interfaces.WorksheetLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether the first column contains special formatting.
             *
             * [Api set: ExcelApi 1.3]
             */
            highlightFirstColumn?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether the last column contains special formatting.
             *
             * [Api set: ExcelApi 1.3]
             */
            highlightLastColumn?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns a value that uniquely identifies the table in a given workbook. The value of the identifier remains the same even when the table is renamed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns a numeric id.
             *
             * [Api set: ExcelApi 1.8]
             */
            legacyId?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Name of the table.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether the columns show banded formatting in which odd columns are highlighted differently from even ones to make reading the table easier.
             *
             * [Api set: ExcelApi 1.3]
             */
            showBandedColumns?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether the rows show banded formatting in which odd rows are highlighted differently from even ones to make reading the table easier.
             *
             * [Api set: ExcelApi 1.3]
             */
            showBandedRows?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether the filter buttons are visible at the top of each column header. Setting this is only allowed if the table contains a header row.
             *
             * [Api set: ExcelApi 1.3]
             */
            showFilterButton?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether the header row is visible or not. This value can be set to show or remove the header row.
             *
             * [Api set: ExcelApi 1.1]
             */
            showHeaders?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether the total row is visible or not. This value can be set to show or remove the total row.
             *
             * [Api set: ExcelApi 1.1]
             */
            showTotals?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Constant value that represents the Table style. Possible values are: "TableStyleLight1" through "TableStyleLight21", "TableStyleMedium1" through "TableStyleMedium28", "TableStyleStyleDark1" through "TableStyleStyleDark11". A custom user-defined style present in the workbook can also be specified.
             *
             * [Api set: ExcelApi 1.1]
             */
            style?: boolean;
        }
        /**
         *
         * Represents a scoped collection of tables. For each table its top-left corner is considered its anchor location and the tables are sorted top to bottom and then left to right.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface TableScopedCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Represents the AutoFilter object of the table.
            *
            * [Api set: ExcelApi 1.9]
            */
            autoFilter?: Excel.Interfaces.AutoFilterLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents a collection of all the columns in the table.
            *
            * [Api set: ExcelApi 1.1]
            */
            columns?: Excel.Interfaces.TableColumnCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents a collection of all the rows in the table.
            *
            * [Api set: ExcelApi 1.1]
            */
            rows?: Excel.Interfaces.TableRowCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents the sorting for the table.
            *
            * [Api set: ExcelApi 1.2]
            */
            sort?: Excel.Interfaces.TableSortLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: The worksheet containing the current table.
            *
            * [Api set: ExcelApi 1.2]
            */
            worksheet?: Excel.Interfaces.WorksheetLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether the first column contains special formatting.
             *
             * [Api set: ExcelApi 1.3]
             */
            highlightFirstColumn?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether the last column contains special formatting.
             *
             * [Api set: ExcelApi 1.3]
             */
            highlightLastColumn?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns a value that uniquely identifies the table in a given workbook. The value of the identifier remains the same even when the table is renamed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns a numeric id.
             *
             * [Api set: ExcelApi 1.8]
             */
            legacyId?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Name of the table.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether the columns show banded formatting in which odd columns are highlighted differently from even ones to make reading the table easier.
             *
             * [Api set: ExcelApi 1.3]
             */
            showBandedColumns?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether the rows show banded formatting in which odd rows are highlighted differently from even ones to make reading the table easier.
             *
             * [Api set: ExcelApi 1.3]
             */
            showBandedRows?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether the filter buttons are visible at the top of each column header. Setting this is only allowed if the table contains a header row.
             *
             * [Api set: ExcelApi 1.3]
             */
            showFilterButton?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether the header row is visible or not. This value can be set to show or remove the header row.
             *
             * [Api set: ExcelApi 1.1]
             */
            showHeaders?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether the total row is visible or not. This value can be set to show or remove the total row.
             *
             * [Api set: ExcelApi 1.1]
             */
            showTotals?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Constant value that represents the Table style. Possible values are: "TableStyleLight1" through "TableStyleLight21", "TableStyleMedium1" through "TableStyleMedium28", "TableStyleStyleDark1" through "TableStyleStyleDark11". A custom user-defined style present in the workbook can also be specified.
             *
             * [Api set: ExcelApi 1.1]
             */
            style?: boolean;
        }
        /**
         *
         * Represents an Excel table.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface TableLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the AutoFilter object of the table.
            *
            * [Api set: ExcelApi 1.9]
            */
            autoFilter?: Excel.Interfaces.AutoFilterLoadOptions;
            /**
            *
            * Represents a collection of all the columns in the table.
            *
            * [Api set: ExcelApi 1.1]
            */
            columns?: Excel.Interfaces.TableColumnCollectionLoadOptions;
            /**
            *
            * Represents a collection of all the rows in the table.
            *
            * [Api set: ExcelApi 1.1]
            */
            rows?: Excel.Interfaces.TableRowCollectionLoadOptions;
            /**
            *
            * Represents the sorting for the table.
            *
            * [Api set: ExcelApi 1.2]
            */
            sort?: Excel.Interfaces.TableSortLoadOptions;
            /**
            *
            * The worksheet containing the current table.
            *
            * [Api set: ExcelApi 1.2]
            */
            worksheet?: Excel.Interfaces.WorksheetLoadOptions;
            /**
             *
             * Indicates whether the first column contains special formatting.
             *
             * [Api set: ExcelApi 1.3]
             */
            highlightFirstColumn?: boolean;
            /**
             *
             * Indicates whether the last column contains special formatting.
             *
             * [Api set: ExcelApi 1.3]
             */
            highlightLastColumn?: boolean;
            /**
             *
             * Returns a value that uniquely identifies the table in a given workbook. The value of the identifier remains the same even when the table is renamed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            id?: boolean;
            /**
             *
             * Returns a numeric id.
             *
             * [Api set: ExcelApi 1.8]
             */
            legacyId?: boolean;
            /**
             *
             * Name of the table.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: boolean;
            /**
             *
             * Indicates whether the columns show banded formatting in which odd columns are highlighted differently from even ones to make reading the table easier.
             *
             * [Api set: ExcelApi 1.3]
             */
            showBandedColumns?: boolean;
            /**
             *
             * Indicates whether the rows show banded formatting in which odd rows are highlighted differently from even ones to make reading the table easier.
             *
             * [Api set: ExcelApi 1.3]
             */
            showBandedRows?: boolean;
            /**
             *
             * Indicates whether the filter buttons are visible at the top of each column header. Setting this is only allowed if the table contains a header row.
             *
             * [Api set: ExcelApi 1.3]
             */
            showFilterButton?: boolean;
            /**
             *
             * Indicates whether the header row is visible or not. This value can be set to show or remove the header row.
             *
             * [Api set: ExcelApi 1.1]
             */
            showHeaders?: boolean;
            /**
             *
             * Indicates whether the total row is visible or not. This value can be set to show or remove the total row.
             *
             * [Api set: ExcelApi 1.1]
             */
            showTotals?: boolean;
            /**
             *
             * Constant value that represents the Table style. Possible values are: "TableStyleLight1" through "TableStyleLight21", "TableStyleMedium1" through "TableStyleMedium28", "TableStyleStyleDark1" through "TableStyleStyleDark11". A custom user-defined style present in the workbook can also be specified.
             *
             * [Api set: ExcelApi 1.1]
             */
            style?: boolean;
        }
        /**
         *
         * Represents a collection of all the columns that are part of the table.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface TableColumnCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Retrieve the filter applied to the column.
            *
            * [Api set: ExcelApi 1.2]
            */
            filter?: Excel.Interfaces.FilterLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Returns a unique key that identifies the column within the table. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the index number of the column within the columns collection of the table. Zero-indexed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            index?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the name of the table column.
             *
             * [Api set: ExcelApi 1.1 for getting the name; 1.4 for setting it.]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the raw values of the specified range. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
             *
             * [Api set: ExcelApi 1.1]
             */
            values?: boolean;
        }
        /**
         *
         * Represents a column in a table.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface TableColumnLoadOptions {
            $all?: boolean;
            /**
            *
            * Retrieve the filter applied to the column.
            *
            * [Api set: ExcelApi 1.2]
            */
            filter?: Excel.Interfaces.FilterLoadOptions;
            /**
             *
             * Returns a unique key that identifies the column within the table. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            id?: boolean;
            /**
             *
             * Returns the index number of the column within the columns collection of the table. Zero-indexed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            index?: boolean;
            /**
             *
             * Represents the name of the table column.
             *
             * [Api set: ExcelApi 1.1 for getting the name; 1.4 for setting it.]
             */
            name?: boolean;
            /**
             *
             * Represents the raw values of the specified range. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
             *
             * [Api set: ExcelApi 1.1]
             */
            values?: boolean;
        }
        /**
         *
         * Represents a collection of all the rows that are part of the table.
            
             Note that unlike Ranges or Columns, which will adjust if new rows/columns are added before them,
             a TableRow object represent the physical location of the table row, but not the data.
             That is, if the data is sorted or if new rows are added, a table row will continue
             to point at the index for which it was created.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface TableRowCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the index number of the row within the rows collection of the table. Zero-indexed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            index?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the raw values of the specified range. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
             *
             * [Api set: ExcelApi 1.1]
             */
            values?: boolean;
        }
        /**
         *
         * Represents a row in a table.
            
             Note that unlike Ranges or Columns, which will adjust if new rows/columns are added before them,
             a TableRow object represent the physical location of the table row, but not the data.
             That is, if the data is sorted or if new rows are added, a table row will continue
             to point at the index for which it was created.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface TableRowLoadOptions {
            $all?: boolean;
            /**
             *
             * Returns the index number of the row within the rows collection of the table. Zero-indexed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            index?: boolean;
            /**
             *
             * Represents the raw values of the specified range. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
             *
             * [Api set: ExcelApi 1.1]
             */
            values?: boolean;
        }
        /**
         *
         * Represents the data validation applied to the current range.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface DataValidationLoadOptions {
            $all?: boolean;
            /**
             *
             * Error alert when user enters invalid data.
             *
             * [Api set: ExcelApi 1.8]
             */
            errorAlert?: boolean;
            /**
             *
             * Ignore blanks: no data validation will be performed on blank cells, it defaults to true.
             *
             * [Api set: ExcelApi 1.8]
             */
            ignoreBlanks?: boolean;
            /**
             *
             * Prompt when users select a cell.
             *
             * [Api set: ExcelApi 1.8]
             */
            prompt?: boolean;
            /**
             *
             * Data validation rule that contains different type of data validation criteria.
             *
             * [Api set: ExcelApi 1.8]
             */
            rule?: boolean;
            /**
             *
             * Type of the data validation, see Excel.DataValidationType for details.
             *
             * [Api set: ExcelApi 1.8]
             */
            type?: boolean;
            /**
             *
             * Represents if all cell values are valid according to the data validation rules.
            Returns true if all cell values are valid, or false if all cell values are invalid.
            Returns null if there are both valid and invalid cell values within the range.
             *
             * [Api set: ExcelApi 1.8]
             */
            valid?: boolean;
        }
        /**
         *
         * Represents the results from the removeDuplicates method on range
         *
         * [Api set: ExcelApi 1.9]
         */
        interface RemoveDuplicatesResultLoadOptions {
            $all?: boolean;
            /**
             *
             * Number of duplicated rows removed by the operation.
             *
             * [Api set: ExcelApi 1.9]
             */
            removed?: boolean;
            /**
             *
             * Number of remaining unique rows present in the resulting range.
             *
             * [Api set: ExcelApi 1.9]
             */
            uniqueRemaining?: boolean;
        }
        /**
         *
         * A format object encapsulating the range's font, fill, borders, alignment, and other properties.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface RangeFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Collection of border objects that apply to the overall range.
            *
            * [Api set: ExcelApi 1.1]
            */
            borders?: Excel.Interfaces.RangeBorderCollectionLoadOptions;
            /**
            *
            * Returns the fill object defined on the overall range.
            *
            * [Api set: ExcelApi 1.1]
            */
            fill?: Excel.Interfaces.RangeFillLoadOptions;
            /**
            *
            * Returns the font object defined on the overall range.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.RangeFontLoadOptions;
            /**
            *
            * Returns the format protection object for a range.
            *
            * [Api set: ExcelApi 1.2]
            */
            protection?: Excel.Interfaces.FormatProtectionLoadOptions;
            /**
             *
             * Indicates if text is automatically indented when text alignment is set to equal distribution.
             *
             * [Api set: ExcelApi 1.9]
             */
            autoIndent?: boolean;
            /**
             *
             * Gets or sets the width of all colums within the range. If the column widths are not uniform, null will be returned.
             *
             * [Api set: ExcelApi 1.2]
             */
            columnWidth?: boolean;
            /**
             *
             * Represents the horizontal alignment for the specified object. See Excel.HorizontalAlignment for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            horizontalAlignment?: boolean;
            /**
             *
             * An integer from 0 to 250 that indicates the indent level.
             *
             * [Api set: ExcelApi 1.9]
             */
            indentLevel?: boolean;
            /**
             *
             * The reading order for the range.
             *
             * [Api set: ExcelApi 1.9]
             */
            readingOrder?: boolean;
            /**
             *
             * Gets or sets the height of all rows in the range. If the row heights are not uniform, null will be returned.
             *
             * [Api set: ExcelApi 1.2]
             */
            rowHeight?: boolean;
            /**
             *
             * Indicates if text automatically shrinks to fit in the available column width.
             *
             * [Api set: ExcelApi 1.9]
             */
            shrinkToFit?: boolean;
            /**
             *
             * Gets or sets the text orientation of all the cells within the range.
            The text orientation should be an integer either from -90 to 90, or 180 for vertically-oriented text.
            If the orientation within a range are not uniform, then null will be returned.
             *
             * [Api set: ExcelApi 1.7]
             */
            textOrientation?: boolean;
            /**
             *
             * Determines if the row height of the Range object equals the standard height of the sheet.
            Returns True if the row height of the Range object equals the standard height of the sheet.
            Returns Null if the range contains more than one row and the rows aren't all the same height.
            Returns False otherwise.
             *
             * [Api set: ExcelApi 1.7]
             */
            useStandardHeight?: boolean;
            /**
             *
             * Indicates whether the column width of the Range object equals the standard width of the sheet.
            Returns True if the column width of the Range object equals the standard width of the sheet.
            Returns Null if the range contains more than one column and the columns aren't all the same height.
            Returns False otherwise.
             *
             * [Api set: ExcelApi 1.7]
             */
            useStandardWidth?: boolean;
            /**
             *
             * Represents the vertical alignment for the specified object. See Excel.VerticalAlignment for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            verticalAlignment?: boolean;
            /**
             *
             * Indicates if Excel wraps the text in the object. A null value indicates that the entire range doesn't have uniform wrap setting
             *
             * [Api set: ExcelApi 1.1]
             */
            wrapText?: boolean;
        }
        /**
         *
         * Represents the format protection of a range object.
         *
         * [Api set: ExcelApi 1.2]
         */
        interface FormatProtectionLoadOptions {
            $all?: boolean;
            /**
             *
             * Indicates if Excel hides the formula for the cells in the range. A null value indicates that the entire range doesn't have uniform formula hidden setting.
             *
             * [Api set: ExcelApi 1.2]
             */
            formulaHidden?: boolean;
            /**
             *
             * Indicates if Excel locks the cells in the object. A null value indicates that the entire range doesn't have uniform lock setting.
             *
             * [Api set: ExcelApi 1.2]
             */
            locked?: boolean;
        }
        /**
         *
         * Represents the background of a range object.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface RangeFillLoadOptions {
            $all?: boolean;
            /**
             *
             * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange")
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: boolean;
            /**
             *
             * Gets or sets the pattern of a Range. See Excel.FillPattern for details. LinearGradient and RectangularGradient are not supported.
            A null value indicates that the entire range doesn't have uniform pattern setting.
             *
             * [Api set: ExcelApi 1.9]
             */
            pattern?: boolean;
            /**
             *
             * Sets HTML color code representing the color of the Range pattern, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            Gets HTML color code representing the color of the Range pattern, of the form #RRGGBB (e.g. "FFA500").
             *
             * [Api set: ExcelApi 1.9]
             */
            patternColor?: boolean;
            /**
             *
             * Returns or sets a double that lightens or darkens a pattern color for Range Fill, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            If the pattern tintAndShades are not uniform, null will be returned.
             *
             * [Api set: ExcelApi 1.9]
             */
            patternTintAndShade?: boolean;
            /**
             *
             * Returns or sets a double that lightens or darkens a color for Range Fill, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            If the tintAndShades are not uniform, null will be returned.
             *
             * [Api set: ExcelApi 1.9]
             */
            tintAndShade?: boolean;
        }
        /**
         *
         * Represents the border of an object.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface RangeBorderLoadOptions {
            $all?: boolean;
            /**
             *
             * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: boolean;
            /**
             *
             * Constant value that indicates the specific side of the border. See Excel.BorderIndex for details. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            sideIndex?: boolean;
            /**
             *
             * One of the constants of line style specifying the line style for the border. See Excel.BorderLineStyle for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            style?: boolean;
            /**
             *
             * Returns or sets a double that lightens or darkens a color for Range Border, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            A null value indicates that the border doesn't have uniform tintAndShade setting.
             *
             * [Api set: ExcelApi 1.9]
             */
            tintAndShade?: boolean;
            /**
             *
             * Specifies the weight of the border around a range. See Excel.BorderWeight for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            weight?: boolean;
        }
        /**
         *
         * Represents the border objects that make up the range border.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface RangeBorderCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Constant value that indicates the specific side of the border. See Excel.BorderIndex for details. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            sideIndex?: boolean;
            /**
             *
             * For EACH ITEM in the collection: One of the constants of line style specifying the line style for the border. See Excel.BorderLineStyle for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            style?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets a double that lightens or darkens a color for Range Border, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            A null value indicates that the border doesn't have uniform tintAndShade setting.
             *
             * [Api set: ExcelApi 1.9]
             */
            tintAndShade?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Specifies the weight of the border around a range. See Excel.BorderWeight for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            weight?: boolean;
        }
        /**
         *
         * This object represents the font attributes (font name, font size, color, etc.) for an object.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface RangeFontLoadOptions {
            $all?: boolean;
            /**
             *
             * Represents the bold status of font.
             *
             * [Api set: ExcelApi 1.1]
             */
            bold?: boolean;
            /**
             *
             * HTML color code representation of the text color. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: boolean;
            /**
             *
             * Represents the italic status of the font.
             *
             * [Api set: ExcelApi 1.1]
             */
            italic?: boolean;
            /**
             *
             * Font name (e.g. "Calibri")
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: boolean;
            /**
             *
             * Font size.
             *
             * [Api set: ExcelApi 1.1]
             */
            size?: boolean;
            /**
             *
             * Represents the strikethrough status of font. A null value indicates that the entire range doesn't have uniform Strikethrough setting.
             *
             * [Api set: ExcelApi 1.9]
             */
            strikethrough?: boolean;
            /**
             *
             * Represents the Subscript status of font.
            Returns True if all the fonts of the range are Subscript.
            Returns False if all the fonts of the range are Superscript or normal (neither Superscript, nor Subscript).
            Returns Null otherwise.
             *
             * [Api set: ExcelApi 1.9]
             */
            subscript?: boolean;
            /**
             *
             * Represents the Superscript status of font.
            Returns True if all the fonts of the range are Superscript.
            Returns False if all the fonts of the range are Subscript or normal (neither Superscript, nor Subscript).
            Returns Null otherwise.
             *
             * [Api set: ExcelApi 1.9]
             */
            superscript?: boolean;
            /**
             *
             * Returns or sets a double that lightens or darkens a color for Range Font, the value is between -1 (darkest) and 1 (brightest), with 0 for the original color.
            A null value indicates that the entire range doesn't have uniform font tintAndShade setting.
             *
             * [Api set: ExcelApi 1.9]
             */
            tintAndShade?: boolean;
            /**
             *
             * Type of underline applied to the font. See Excel.RangeUnderlineStyle for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            underline?: boolean;
        }
        /**
         *
         * A collection of all the chart objects on a worksheet.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Represents chart axes.
            *
            * [Api set: ExcelApi 1.1]
            */
            axes?: Excel.Interfaces.ChartAxesLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents the datalabels on the chart.
            *
            * [Api set: ExcelApi 1.1]
            */
            dataLabels?: Excel.Interfaces.ChartDataLabelsLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Encapsulates the format properties for the chart area.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartAreaFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents the legend for the chart.
            *
            * [Api set: ExcelApi 1.1]
            */
            legend?: Excel.Interfaces.ChartLegendLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Encapsulates the options for a pivot chart.
            *
            * [Api set: ExcelApi 1.9]
            */
            pivotOptions?: Excel.Interfaces.ChartPivotOptionsLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents the plotArea for the chart.
            *
            * [Api set: ExcelApi 1.8]
            */
            plotArea?: Excel.Interfaces.ChartPlotAreaLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents either a single series or collection of series in the chart.
            *
            * [Api set: ExcelApi 1.1]
            */
            series?: Excel.Interfaces.ChartSeriesCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents the title of the specified chart, including the text, visibility, position, and formatting of the title.
            *
            * [Api set: ExcelApi 1.1]
            */
            title?: Excel.Interfaces.ChartTitleLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: The worksheet containing the current chart.
            *
            * [Api set: ExcelApi 1.2]
            */
            worksheet?: Excel.Interfaces.WorksheetLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets a ChartCategoryLabelLevel enumeration constant referring to
            the level of where the category labels are being sourced from. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            categoryLabelLevel?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the type of the chart. See Excel.ChartType for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            chartType?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the way that blank cells are plotted on a chart. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            displayBlanksAs?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the height, in points, of the chart object.
             *
             * [Api set: ExcelApi 1.1]
             */
            height?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The unique id of chart. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The distance, in points, from the left side of the chart to the worksheet origin.
             *
             * [Api set: ExcelApi 1.1]
             */
            left?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the name of a chart object.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the way columns or rows are used as data series on the chart. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            plotBy?: boolean;
            /**
             *
             * For EACH ITEM in the collection: True if only visible cells are plotted. False if both visible and hidden cells are plotted. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            plotVisibleOnly?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets a ChartSeriesNameLevel enumeration constant referring to
            the level of where the series names are being sourced from. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            seriesNameLevel?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents whether to display all field buttons on a PivotChart.
             *
             * [Api set: ExcelApi 1.7]
             */
            showAllFieldButtons?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents whether to show the data labels when the value is greater than the maximum value on the value axis.
            If value axis became smaller than the size of data points, you can use this property to set whether to show the data labels.
            This property applies to 2-D charts only.
             *
             * [Api set: ExcelApi 1.8]
             */
            showDataLabelsOverMaximum?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the chart style for the chart. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            style?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the distance, in points, from the top edge of the object to the top of row 1 (on a worksheet) or the top of the chart area (on a chart).
             *
             * [Api set: ExcelApi 1.1]
             */
            top?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the width, in points, of the chart object.
             *
             * [Api set: ExcelApi 1.1]
             */
            width?: boolean;
        }
        /**
         *
         * Represents a chart object in a workbook.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents chart axes.
            *
            * [Api set: ExcelApi 1.1]
            */
            axes?: Excel.Interfaces.ChartAxesLoadOptions;
            /**
            *
            * Represents the datalabels on the chart.
            *
            * [Api set: ExcelApi 1.1]
            */
            dataLabels?: Excel.Interfaces.ChartDataLabelsLoadOptions;
            /**
            *
            * Encapsulates the format properties for the chart area.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartAreaFormatLoadOptions;
            /**
            *
            * Represents the legend for the chart.
            *
            * [Api set: ExcelApi 1.1]
            */
            legend?: Excel.Interfaces.ChartLegendLoadOptions;
            /**
            *
            * Encapsulates the options for a pivot chart.
            *
            * [Api set: ExcelApi 1.9]
            */
            pivotOptions?: Excel.Interfaces.ChartPivotOptionsLoadOptions;
            /**
            *
            * Represents the plotArea for the chart.
            *
            * [Api set: ExcelApi 1.8]
            */
            plotArea?: Excel.Interfaces.ChartPlotAreaLoadOptions;
            /**
            *
            * Represents either a single series or collection of series in the chart.
            *
            * [Api set: ExcelApi 1.1]
            */
            series?: Excel.Interfaces.ChartSeriesCollectionLoadOptions;
            /**
            *
            * Represents the title of the specified chart, including the text, visibility, position, and formatting of the title.
            *
            * [Api set: ExcelApi 1.1]
            */
            title?: Excel.Interfaces.ChartTitleLoadOptions;
            /**
            *
            * The worksheet containing the current chart.
            *
            * [Api set: ExcelApi 1.2]
            */
            worksheet?: Excel.Interfaces.WorksheetLoadOptions;
            /**
             *
             * Returns or sets a ChartCategoryLabelLevel enumeration constant referring to
            the level of where the category labels are being sourced from. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            categoryLabelLevel?: boolean;
            /**
             *
             * Represents the type of the chart. See Excel.ChartType for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            chartType?: boolean;
            /**
             *
             * Returns or sets the way that blank cells are plotted on a chart. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            displayBlanksAs?: boolean;
            /**
             *
             * Represents the height, in points, of the chart object.
             *
             * [Api set: ExcelApi 1.1]
             */
            height?: boolean;
            /**
             *
             * The unique id of chart. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            id?: boolean;
            /**
             *
             * The distance, in points, from the left side of the chart to the worksheet origin.
             *
             * [Api set: ExcelApi 1.1]
             */
            left?: boolean;
            /**
             *
             * Represents the name of a chart object.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: boolean;
            /**
             *
             * Returns or sets the way columns or rows are used as data series on the chart. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            plotBy?: boolean;
            /**
             *
             * True if only visible cells are plotted. False if both visible and hidden cells are plotted. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            plotVisibleOnly?: boolean;
            /**
             *
             * Returns or sets a ChartSeriesNameLevel enumeration constant referring to
            the level of where the series names are being sourced from. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            seriesNameLevel?: boolean;
            /**
             *
             * Represents whether to display all field buttons on a PivotChart.
             *
             * [Api set: ExcelApi 1.7]
             */
            showAllFieldButtons?: boolean;
            /**
             *
             * Represents whether to show the data labels when the value is greater than the maximum value on the value axis.
            If value axis became smaller than the size of data points, you can use this property to set whether to show the data labels.
            This property applies to 2-D charts only.
             *
             * [Api set: ExcelApi 1.8]
             */
            showDataLabelsOverMaximum?: boolean;
            /**
             *
             * Returns or sets the chart style for the chart. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            style?: boolean;
            /**
             *
             * Represents the distance, in points, from the top edge of the object to the top of row 1 (on a worksheet) or the top of the chart area (on a chart).
             *
             * [Api set: ExcelApi 1.1]
             */
            top?: boolean;
            /**
             *
             * Represents the width, in points, of the chart object.
             *
             * [Api set: ExcelApi 1.1]
             */
            width?: boolean;
        }
        /**
         *
         * Encapsulates the options for the pivot chart.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface ChartPivotOptionsLoadOptions {
            $all?: boolean;
            /**
             *
             * Specifies whether or not to display the axis field buttons on a PivotChart. The ShowAxisFieldButtons property corresponds to the "Show Axis Field Buttons" command on the "Field Buttons" drop-down list of the "Analyze" tab, which is available when a PivotChart is selected.
             *
             * [Api set: ExcelApi 1.9]
             */
            showAxisFieldButtons?: boolean;
            /**
             *
             * Specifies whether or not to display the legend field buttons on a PivotChart
             *
             * [Api set: ExcelApi 1.9]
             */
            showLegendFieldButtons?: boolean;
            /**
             *
             * Specifies whether or not to display the report filter field buttons on a PivotChart.
             *
             * [Api set: ExcelApi 1.9]
             */
            showReportFilterFieldButtons?: boolean;
            /**
             *
             * Specifies whether or not to display the show value field buttons on a PivotChart
             *
             * [Api set: ExcelApi 1.9]
             */
            showValueFieldButtons?: boolean;
        }
        /**
         *
         * Encapsulates the format properties for the overall chart area.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartAreaFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the border format of chart area, which includes color, linestyle, and weight.
            *
            * [Api set: ExcelApi 1.7]
            */
            border?: Excel.Interfaces.ChartBorderLoadOptions;
            /**
            *
            * Represents the font attributes (font name, font size, color, etc.) for the current object.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontLoadOptions;
            /**
             *
             * Returns or sets color scheme of the chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            colorScheme?: boolean;
            /**
             *
             * Specifies whether or not chart area of the chart has rounded corners. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            roundedCorners?: boolean;
        }
        /**
         *
         * Represents a collection of chart series.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartSeriesCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Encapsulates the bin options for histogram charts and pareto charts.
            *
            * [Api set: ExcelApi 1.9]
            */
            binOptions?: Excel.Interfaces.ChartBinOptionsLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Encapsulates the options for the box and whisker charts.
            *
            * [Api set: ExcelApi 1.9]
            */
            boxwhiskerOptions?: Excel.Interfaces.ChartBoxwhiskerOptionsLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents a collection of all dataLabels in the series.
            *
            * [Api set: ExcelApi 1.8]
            */
            dataLabels?: Excel.Interfaces.ChartDataLabelsLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents the formatting of a chart series, which includes fill and line formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartSeriesFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Encapsulates the options for a region map chart.
            *
            * [Api set: ExcelApi 1.9]
            */
            mapOptions?: Excel.Interfaces.ChartMapOptionsLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents a collection of all points in the series.
            *
            * [Api set: ExcelApi 1.1]
            */
            points?: Excel.Interfaces.ChartPointsCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents the error bar object of a chart series.
            *
            * [Api set: ExcelApi 1.9]
            */
            xErrorBars?: Excel.Interfaces.ChartErrorBarsLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents the error bar object of a chart series.
            *
            * [Api set: ExcelApi 1.9]
            */
            yErrorBars?: Excel.Interfaces.ChartErrorBarsLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the group for the specified series. Read/Write
             *
             * [Api set: ExcelApi 1.8]
             */
            axisGroup?: boolean;
            /**
             *
             * For EACH ITEM in the collection: This can be an integer value from 0 (zero) to 300, representing the percentage of the default size. This property only applies to bubble charts. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            bubbleScale?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the chart type of a series. See Excel.ChartType for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            chartType?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the doughnut hole size of a chart series.  Only valid on doughnut and doughnutExploded charts.
            Throws an invalid argument exception on invalid charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            doughnutHoleSize?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the explosion value for a pie-chart or doughnut-chart slice. Returns 0 (zero) if there's no explosion (the tip of the slice is in the center of the pie). Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            explosion?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Boolean value representing if the series is filtered or not. Not applicable for surface charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            filtered?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the angle of the first pie-chart or doughnut-chart slice, in degrees (clockwise from vertical). Applies only to pie, 3-D pie, and doughnut charts. Can be a value from 0 through 360. Read/Write
             *
             * [Api set: ExcelApi 1.8]
             */
            firstSliceAngle?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the gap width of a chart series.  Only valid on bar and column charts, as well as
            specific classes of line and pie charts.  Throws an invalid argument exception on invalid charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            gapWidth?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the color for maximum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMaximumColor?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the type for maximum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMaximumType?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the maximum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMaximumValue?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the color for midpoint value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMidpointColor?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the type for midpoint value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMidpointType?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the midpoint value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMidpointValue?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the color for minimum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMinimumColor?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the type for minimum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMinimumType?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the minimum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMinimumValue?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets series gradient style of a region map chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientStyle?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Boolean value representing if the series has data labels or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            hasDataLabels?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the fill color for negative data points in a series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            invertColor?: boolean;
            /**
             *
             * For EACH ITEM in the collection: True if Microsoft Excel inverts the pattern in the item when it corresponds to a negative number. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            invertIfNegative?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents markers background color of a chart series.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerBackgroundColor?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents markers foreground color of a chart series.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerForegroundColor?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents marker size of a chart series.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerSize?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents marker style of a chart series. See Excel.ChartMarkerStyle for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerStyle?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the name of a series in a chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Specifies how bars and columns are positioned. Can be a value between –100 and 100. Applies only to 2-D bar and 2-D column charts. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            overlap?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the series parent label strategy area for a treemap chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            parentLabelStrategy?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the plot order of a chart series within the chart group.
             *
             * [Api set: ExcelApi 1.7]
             */
            plotOrder?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the size of the secondary section of either a pie of pie chart or a bar of pie chart, as a percentage of the size of the primary pie. Can be a value from 5 to 200. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            secondPlotSize?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Specifies whether or not connector lines are shown in waterfall charts. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showConnectorLines?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Specifies whether or not leader lines are displayed for each data label in the series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showLeaderLines?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Boolean value representing if the series has a shadow or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showShadow?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Boolean value representing if the series is smooth or not. Only applicable to line and scatter charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            smooth?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the way the two sections of either a pie of pie chart or a bar of pie chart are split. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            splitType?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the threshold value that separates two sections of either a pie-of-pie chart or a bar-of-pie chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            splitValue?: boolean;
            /**
             *
             * For EACH ITEM in the collection: True if Microsoft Excel assigns a different color or pattern to each data marker. The chart must contain only one series. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            varyByCategories?: boolean;
        }
        /**
         *
         * Represents a series in a chart.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartSeriesLoadOptions {
            $all?: boolean;
            /**
            *
            * Encapsulates the bin options for histogram charts and pareto charts.
            *
            * [Api set: ExcelApi 1.9]
            */
            binOptions?: Excel.Interfaces.ChartBinOptionsLoadOptions;
            /**
            *
            * Encapsulates the options for the box and whisker charts.
            *
            * [Api set: ExcelApi 1.9]
            */
            boxwhiskerOptions?: Excel.Interfaces.ChartBoxwhiskerOptionsLoadOptions;
            /**
            *
            * Represents a collection of all dataLabels in the series.
            *
            * [Api set: ExcelApi 1.8]
            */
            dataLabels?: Excel.Interfaces.ChartDataLabelsLoadOptions;
            /**
            *
            * Represents the formatting of a chart series, which includes fill and line formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartSeriesFormatLoadOptions;
            /**
            *
            * Encapsulates the options for a region map chart.
            *
            * [Api set: ExcelApi 1.9]
            */
            mapOptions?: Excel.Interfaces.ChartMapOptionsLoadOptions;
            /**
            *
            * Represents a collection of all points in the series.
            *
            * [Api set: ExcelApi 1.1]
            */
            points?: Excel.Interfaces.ChartPointsCollectionLoadOptions;
            /**
            *
            * Represents the error bar object of a chart series.
            *
            * [Api set: ExcelApi 1.9]
            */
            xErrorBars?: Excel.Interfaces.ChartErrorBarsLoadOptions;
            /**
            *
            * Represents the error bar object of a chart series.
            *
            * [Api set: ExcelApi 1.9]
            */
            yErrorBars?: Excel.Interfaces.ChartErrorBarsLoadOptions;
            /**
             *
             * Returns or sets the group for the specified series. Read/Write
             *
             * [Api set: ExcelApi 1.8]
             */
            axisGroup?: boolean;
            /**
             *
             * This can be an integer value from 0 (zero) to 300, representing the percentage of the default size. This property only applies to bubble charts. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            bubbleScale?: boolean;
            /**
             *
             * Represents the chart type of a series. See Excel.ChartType for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            chartType?: boolean;
            /**
             *
             * Represents the doughnut hole size of a chart series.  Only valid on doughnut and doughnutExploded charts.
            Throws an invalid argument exception on invalid charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            doughnutHoleSize?: boolean;
            /**
             *
             * Returns or sets the explosion value for a pie-chart or doughnut-chart slice. Returns 0 (zero) if there's no explosion (the tip of the slice is in the center of the pie). Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            explosion?: boolean;
            /**
             *
             * Boolean value representing if the series is filtered or not. Not applicable for surface charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            filtered?: boolean;
            /**
             *
             * Returns or sets the angle of the first pie-chart or doughnut-chart slice, in degrees (clockwise from vertical). Applies only to pie, 3-D pie, and doughnut charts. Can be a value from 0 through 360. Read/Write
             *
             * [Api set: ExcelApi 1.8]
             */
            firstSliceAngle?: boolean;
            /**
             *
             * Represents the gap width of a chart series.  Only valid on bar and column charts, as well as
            specific classes of line and pie charts.  Throws an invalid argument exception on invalid charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            gapWidth?: boolean;
            /**
             *
             * Returns or sets the color for maximum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMaximumColor?: boolean;
            /**
             *
             * Returns or sets the type for maximum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMaximumType?: boolean;
            /**
             *
             * Returns or sets the maximum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMaximumValue?: boolean;
            /**
             *
             * Returns or sets the color for midpoint value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMidpointColor?: boolean;
            /**
             *
             * Returns or sets the type for midpoint value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMidpointType?: boolean;
            /**
             *
             * Returns or sets the midpoint value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMidpointValue?: boolean;
            /**
             *
             * Returns or sets the color for minimum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMinimumColor?: boolean;
            /**
             *
             * Returns or sets the type for minimum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMinimumType?: boolean;
            /**
             *
             * Returns or sets the minimum value of a region map chart series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientMinimumValue?: boolean;
            /**
             *
             * Returns or sets series gradient style of a region map chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            gradientStyle?: boolean;
            /**
             *
             * Boolean value representing if the series has data labels or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            hasDataLabels?: boolean;
            /**
             *
             * Returns or sets the fill color for negative data points in a series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            invertColor?: boolean;
            /**
             *
             * True if Microsoft Excel inverts the pattern in the item when it corresponds to a negative number. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            invertIfNegative?: boolean;
            /**
             *
             * Represents markers background color of a chart series.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerBackgroundColor?: boolean;
            /**
             *
             * Represents markers foreground color of a chart series.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerForegroundColor?: boolean;
            /**
             *
             * Represents marker size of a chart series.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerSize?: boolean;
            /**
             *
             * Represents marker style of a chart series. See Excel.ChartMarkerStyle for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerStyle?: boolean;
            /**
             *
             * Represents the name of a series in a chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: boolean;
            /**
             *
             * Specifies how bars and columns are positioned. Can be a value between –100 and 100. Applies only to 2-D bar and 2-D column charts. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            overlap?: boolean;
            /**
             *
             * Returns or sets the series parent label strategy area for a treemap chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            parentLabelStrategy?: boolean;
            /**
             *
             * Represents the plot order of a chart series within the chart group.
             *
             * [Api set: ExcelApi 1.7]
             */
            plotOrder?: boolean;
            /**
             *
             * Returns or sets the size of the secondary section of either a pie of pie chart or a bar of pie chart, as a percentage of the size of the primary pie. Can be a value from 5 to 200. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            secondPlotSize?: boolean;
            /**
             *
             * Specifies whether or not connector lines are shown in waterfall charts. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showConnectorLines?: boolean;
            /**
             *
             * Specifies whether or not leader lines are displayed for each data label in the series. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showLeaderLines?: boolean;
            /**
             *
             * Boolean value representing if the series has a shadow or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showShadow?: boolean;
            /**
             *
             * Boolean value representing if the series is smooth or not. Only applicable to line and scatter charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            smooth?: boolean;
            /**
             *
             * Returns or sets the way the two sections of either a pie of pie chart or a bar of pie chart are split. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            splitType?: boolean;
            /**
             *
             * Returns or sets the threshold value that separates two sections of either a pie-of-pie chart or a bar-of-pie chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            splitValue?: boolean;
            /**
             *
             * True if Microsoft Excel assigns a different color or pattern to each data marker. The chart must contain only one series. Read/Write.
             *
             * [Api set: ExcelApi 1.8]
             */
            varyByCategories?: boolean;
        }
        /**
         *
         * Encapsulates the format properties for the chart series
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartSeriesFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents line formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            line?: Excel.Interfaces.ChartLineFormatLoadOptions;
        }
        /**
         *
         * A collection of all the chart points within a series inside a chart.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartPointsCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Returns the data label of a chart point.
            *
            * [Api set: ExcelApi 1.7]
            */
            dataLabel?: Excel.Interfaces.ChartDataLabelLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Encapsulates the format properties chart point.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartPointFormatLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Represents whether a data point has a data label. Not applicable for surface charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            hasDataLabel?: boolean;
            /**
             *
             * For EACH ITEM in the collection: HTML color code representation of the marker background color of data point. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerBackgroundColor?: boolean;
            /**
             *
             * For EACH ITEM in the collection: HTML color code representation of the marker foreground color of data point. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerForegroundColor?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents marker size of data point.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerSize?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents marker style of a chart data point. See Excel.ChartMarkerStyle for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerStyle?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the value of a chart point. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            value?: boolean;
        }
        /**
         *
         * Represents a point of a series in a chart.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartPointLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns the data label of a chart point.
            *
            * [Api set: ExcelApi 1.7]
            */
            dataLabel?: Excel.Interfaces.ChartDataLabelLoadOptions;
            /**
            *
            * Encapsulates the format properties chart point.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartPointFormatLoadOptions;
            /**
             *
             * Represents whether a data point has a data label. Not applicable for surface charts.
             *
             * [Api set: ExcelApi 1.7]
             */
            hasDataLabel?: boolean;
            /**
             *
             * HTML color code representation of the marker background color of data point. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerBackgroundColor?: boolean;
            /**
             *
             * HTML color code representation of the marker foreground color of data point. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerForegroundColor?: boolean;
            /**
             *
             * Represents marker size of data point.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerSize?: boolean;
            /**
             *
             * Represents marker style of a chart data point. See Excel.ChartMarkerStyle for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            markerStyle?: boolean;
            /**
             *
             * Returns the value of a chart point. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            value?: boolean;
        }
        /**
         *
         * Represents formatting object for chart points.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartPointFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the border format of a chart data point, which includes color, style, and weight information.
            *
            * [Api set: ExcelApi 1.7]
            */
            border?: Excel.Interfaces.ChartBorderLoadOptions;
        }
        /**
         *
         * Represents the chart axes.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartAxesLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the category axis in a chart.
            *
            * [Api set: ExcelApi 1.1]
            */
            categoryAxis?: Excel.Interfaces.ChartAxisLoadOptions;
            /**
            *
            * Represents the series axis of a 3-dimensional chart.
            *
            * [Api set: ExcelApi 1.1]
            */
            seriesAxis?: Excel.Interfaces.ChartAxisLoadOptions;
            /**
            *
            * Represents the value axis in an axis.
            *
            * [Api set: ExcelApi 1.1]
            */
            valueAxis?: Excel.Interfaces.ChartAxisLoadOptions;
        }
        /**
         *
         * Represents a single axis in a chart.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartAxisLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the formatting of a chart object, which includes line and font formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartAxisFormatLoadOptions;
            /**
            *
            * Returns a Gridlines object that represents the major gridlines for the specified axis.
            *
            * [Api set: ExcelApi 1.1]
            */
            majorGridlines?: Excel.Interfaces.ChartGridlinesLoadOptions;
            /**
            *
            * Returns a Gridlines object that represents the minor gridlines for the specified axis.
            *
            * [Api set: ExcelApi 1.1]
            */
            minorGridlines?: Excel.Interfaces.ChartGridlinesLoadOptions;
            /**
            *
            * Represents the axis title.
            *
            * [Api set: ExcelApi 1.1]
            */
            title?: Excel.Interfaces.ChartAxisTitleLoadOptions;
            /**
             *
             * Represents the alignment for the specified axis tick label. See Excel.ChartTextHorizontalAlignment for detail.
             *
             * [Api set: ExcelApi 1.8]
             */
            alignment?: boolean;
            /**
             *
             * Represents the group for the specified axis. See Excel.ChartAxisGroup for details. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            axisGroup?: boolean;
            /**
             *
             * Returns or sets the base unit for the specified category axis.
             *
             * [Api set: ExcelApi 1.7]
             */
            baseTimeUnit?: boolean;
            /**
             *
             * Returns or sets the category axis type.
             *
             * [Api set: ExcelApi 1.7]
             */
            categoryType?: boolean;
            /**
             * [DEPRECATED; kept for back-compat with existing first-party solutions]. Please use `Position` instead.
             * Represents the specified axis where the other axis crosses. See Excel.ChartAxisPosition for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            crosses?: boolean;
            /**
             * [DEPRECATED; kept for back-compat with existing first-party solutions]. Please use `PositionAt` instead.
             * Represents the specified axis where the other axis crosses at. Read Only. Set to this property should use SetCrossesAt(double) method.
             *
             * [Api set: ExcelApi 1.7]
             */
            crossesAt?: boolean;
            /**
             *
             * Represents the custom axis display unit value. Read-only. To set this property, please use the SetCustomDisplayUnit(double) method.
             *
             * [Api set: ExcelApi 1.7]
             */
            customDisplayUnit?: boolean;
            /**
             *
             * Represents the axis display unit. See Excel.ChartAxisDisplayUnit for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            displayUnit?: boolean;
            /**
             *
             * Represents the height, in points, of the chart axis. Null if the axis is not visible. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            height?: boolean;
            /**
             *
             * Represents whether value axis crosses the category axis between categories.
             *
             * [Api set: ExcelApi 1.8]
             */
            isBetweenCategories?: boolean;
            /**
             *
             * Represents the distance, in points, from the left edge of the axis to the left of chart area. Null if the axis is not visible. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            left?: boolean;
            /**
             *
             * Represents whether or not the number format is linked to the cells. If true, the number format will change in the labels when it changes in the cells.
             *
             * [Api set: ExcelApi 1.9]
             */
            linkNumberFormat?: boolean;
            /**
             *
             * Represents the base of the logarithm when using logarithmic scales.
             *
             * [Api set: ExcelApi 1.7]
             */
            logBase?: boolean;
            /**
             *
             * Represents the type of major tick mark for the specified axis. See Excel.ChartAxisTickMark for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            majorTickMark?: boolean;
            /**
             *
             * Returns or sets the major unit scale value for the category axis when the CategoryType property is set to TimeScale.
             *
             * [Api set: ExcelApi 1.7]
             */
            majorTimeUnitScale?: boolean;
            /**
             *
             * Represents the interval between two major tick marks. Can be set to a numeric value or an empty string.  The returned value is always a number.
             *
             * [Api set: ExcelApi 1.1]
             */
            majorUnit?: boolean;
            /**
             *
             * Represents the maximum value on the value axis.  Can be set to a numeric value or an empty string (for automatic axis values).  The returned value is always a number.
             *
             * [Api set: ExcelApi 1.1]
             */
            maximum?: boolean;
            /**
             *
             * Represents the minimum value on the value axis. Can be set to a numeric value or an empty string (for automatic axis values).  The returned value is always a number.
             *
             * [Api set: ExcelApi 1.1]
             */
            minimum?: boolean;
            /**
             *
             * Represents the type of minor tick mark for the specified axis. See Excel.ChartAxisTickMark for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            minorTickMark?: boolean;
            /**
             *
             * Returns or sets the minor unit scale value for the category axis when the CategoryType property is set to TimeScale.
             *
             * [Api set: ExcelApi 1.7]
             */
            minorTimeUnitScale?: boolean;
            /**
             *
             * Represents the interval between two minor tick marks. Can be set to a numeric value or an empty string (for automatic axis values). The returned value is always a number.
             *
             * [Api set: ExcelApi 1.1]
             */
            minorUnit?: boolean;
            /**
             *
             * Represents whether an axis is multilevel or not.
             *
             * [Api set: ExcelApi 1.8]
             */
            multiLevel?: boolean;
            /**
             *
             * Represents the format code for the axis tick label.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: boolean;
            /**
             *
             * Represents the distance between the levels of labels, and the distance between the first level and the axis line. The value should be an integer from 0 to 1000.
             *
             * [Api set: ExcelApi 1.8]
             */
            offset?: boolean;
            /**
             *
             * Represents the specified axis position where the other axis crosses. See Excel.ChartAxisPosition for details.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: boolean;
            /**
             *
             * Represents the specified axis position where the other axis crosses at. You should use the SetPositionAt(double) method to set this property.
             *
             * [Api set: ExcelApi 1.8]
             */
            positionAt?: boolean;
            /**
             *
             * Represents whether Microsoft Excel plots data points from last to first.
             *
             * [Api set: ExcelApi 1.7]
             */
            reversePlotOrder?: boolean;
            /**
             *
             * Represents the value axis scale type. See Excel.ChartAxisScaleType for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            scaleType?: boolean;
            /**
             *
             * Represents whether the axis display unit label is visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            showDisplayUnitLabel?: boolean;
            /**
             *
             * Represents the text orientation of the axis tick label. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: boolean;
            /**
             *
             * Represents the position of tick-mark labels on the specified axis. See Excel.ChartAxisTickLabelPosition for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            tickLabelPosition?: boolean;
            /**
             *
             * Represents the number of categories or series between tick-mark labels. Can be a value from 1 through 31999 or an empty string for automatic setting. The returned value is always a number.
             *
             * [Api set: ExcelApi 1.7]
             */
            tickLabelSpacing?: boolean;
            /**
             *
             * Represents the number of categories or series between tick marks.
             *
             * [Api set: ExcelApi 1.7]
             */
            tickMarkSpacing?: boolean;
            /**
             *
             * Represents the distance, in points, from the top edge of the axis to the top of chart area. Null if the axis is not visible. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            top?: boolean;
            /**
             *
             * Represents the axis type. See Excel.ChartAxisType for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            type?: boolean;
            /**
             *
             * A boolean value represents the visibility of the axis.
             *
             * [Api set: ExcelApi 1.7]
             */
            visible?: boolean;
            /**
             *
             * Represents the width, in points, of the chart axis. Null if the axis is not visible. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            width?: boolean;
        }
        /**
         *
         * Encapsulates the format properties for the chart axis.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartAxisFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the font attributes (font name, font size, color, etc.) for a chart axis element.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontLoadOptions;
            /**
            *
            * Represents chart line formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            line?: Excel.Interfaces.ChartLineFormatLoadOptions;
        }
        /**
         *
         * Represents the title of a chart axis.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartAxisTitleLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the formatting of chart axis title.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartAxisTitleFormatLoadOptions;
            /**
             *
             * Represents the axis title.
             *
             * [Api set: ExcelApi 1.1]
             */
            text?: boolean;
            /**
             *
             * A boolean that specifies the visibility of an axis title.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
        }
        /**
         *
         * Represents the chart axis title formatting.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartAxisTitleFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the border format, which includes color, linestyle, and weight.
            *
            * [Api set: ExcelApi 1.8]
            */
            border?: Excel.Interfaces.ChartBorderLoadOptions;
            /**
            *
            * Represents the font attributes, such as font name, font size, color, etc. of chart axis title object.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontLoadOptions;
        }
        /**
         *
         * Represents a collection of all the data labels on a chart point.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartDataLabelsLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the format of chart data labels, which includes fill and font formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartDataLabelFormatLoadOptions;
            /**
             *
             * Represents whether data labels automatically generate appropriate text based on context.
             *
             * [Api set: ExcelApi 1.8]
             */
            autoText?: boolean;
            /**
             *
             * Represents the horizontal alignment for chart data label. See Excel.ChartTextHorizontalAlignment for details.
            This property is valid only when TextOrientation of data label is 0.
             *
             * [Api set: ExcelApi 1.8]
             */
            horizontalAlignment?: boolean;
            /**
             *
             * Represents whether or not the number format is linked to the cells. If true, the number format will change in the labels when it changes in the cells
             *
             * [Api set: ExcelApi 1.9]
             */
            linkNumberFormat?: boolean;
            /**
             *
             * Represents the format code for data labels.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: boolean;
            /**
             *
             * DataLabelPosition value that represents the position of the data label. See Excel.ChartDataLabelPosition for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            position?: boolean;
            /**
             *
             * String representing the separator used for the data labels on a chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            separator?: boolean;
            /**
             *
             * Boolean value representing if the data label bubble size is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showBubbleSize?: boolean;
            /**
             *
             * Boolean value representing if the data label category name is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showCategoryName?: boolean;
            /**
             *
             * Boolean value representing if the data label legend key is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showLegendKey?: boolean;
            /**
             *
             * Boolean value representing if the data label percentage is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showPercentage?: boolean;
            /**
             *
             * Boolean value representing if the data label series name is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showSeriesName?: boolean;
            /**
             *
             * Boolean value representing if the data label value is visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            showValue?: boolean;
            /**
             *
             * Represents the text orientation of data labels. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: boolean;
            /**
             *
             * Represents the vertical alignment of chart data label. See Excel.ChartTextVerticalAlignment for details.
            This property is valid only when TextOrientation of data label is 90, -90 or 180.
             *
             * [Api set: ExcelApi 1.8]
             */
            verticalAlignment?: boolean;
        }
        /**
         *
         * Represents the data label of a chart point.
         *
         * [Api set: ExcelApi 1.7]
         */
        interface ChartDataLabelLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the format of chart data label.
            *
            * [Api set: ExcelApi 1.8]
            */
            format?: Excel.Interfaces.ChartDataLabelFormatLoadOptions;
            /**
             *
             * Boolean value representing if data label automatically generates appropriate text based on context.
             *
             * [Api set: ExcelApi 1.8]
             */
            autoText?: boolean;
            /**
             *
             * String value that represents the formula of chart data label using A1-style notation.
             *
             * [Api set: ExcelApi 1.8]
             */
            formula?: boolean;
            /**
             *
             * Returns the height, in points, of the chart data label. Read-only. Null if chart data label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            height?: boolean;
            /**
             *
             * Represents the horizontal alignment for chart data label. See Excel.ChartTextHorizontalAlignment for details.
            This property is valid only when TextOrientation of data label is 90, -90 or 180.
             *
             * [Api set: ExcelApi 1.8]
             */
            horizontalAlignment?: boolean;
            /**
             *
             * Represents the distance, in points, from the left edge of chart data label to the left edge of chart area. Null if chart data label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            left?: boolean;
            /**
             *
             * Boolean value representing if the number format is linked to the cells (so that the number format changes in the labels when it changes in the cells).
             *
             * [Api set: ExcelApi 1.9]
             */
            linkNumberFormat?: boolean;
            /**
             *
             * String value that represents the format code for data label.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: boolean;
            /**
             *
             * DataLabelPosition value that represents the position of the data label. See Excel.ChartDataLabelPosition for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            position?: boolean;
            /**
             *
             * String representing the separator used for the data label on a chart.
             *
             * [Api set: ExcelApi 1.7]
             */
            separator?: boolean;
            /**
             *
             * Boolean value representing if the data label bubble size is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showBubbleSize?: boolean;
            /**
             *
             * Boolean value representing if the data label category name is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showCategoryName?: boolean;
            /**
             *
             * Boolean value representing if the data label legend key is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showLegendKey?: boolean;
            /**
             *
             * Boolean value representing if the data label percentage is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showPercentage?: boolean;
            /**
             *
             * Boolean value representing if the data label series name is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showSeriesName?: boolean;
            /**
             *
             * Boolean value representing if the data label value is visible or not.
             *
             * [Api set: ExcelApi 1.7]
             */
            showValue?: boolean;
            /**
             *
             * String representing the text of the data label on a chart.
             *
             * [Api set: ExcelApi 1.8]
             */
            text?: boolean;
            /**
             *
             * Represents the text orientation of chart data label. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: boolean;
            /**
             *
             * Represents the distance, in points, from the top edge of chart data label to the top of chart area. Null if chart data label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            top?: boolean;
            /**
             *
             * Represents the vertical alignment of chart data label. See Excel.ChartTextVerticalAlignment for details.
            This property is valid only when TextOrientation of data label is 0.
             *
             * [Api set: ExcelApi 1.8]
             */
            verticalAlignment?: boolean;
            /**
             *
             * Returns the width, in points, of the chart data label. Read-only. Null if chart data label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            width?: boolean;
        }
        /**
         *
         * Encapsulates the format properties for the chart data labels.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartDataLabelFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the border format, which includes color, linestyle, and weight.
            *
            * [Api set: ExcelApi 1.8]
            */
            border?: Excel.Interfaces.ChartBorderLoadOptions;
            /**
            *
            * Represents the font attributes (font name, font size, color, etc.) for a chart data label.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontLoadOptions;
        }
        /**
         *
         * This object represents the attributes for a chart's error bars.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface ChartErrorBarsLoadOptions {
            $all?: boolean;
            /**
            *
            * Specifies the formatting type of the error bars.
            *
            * [Api set: ExcelApi 1.9]
            */
            format?: Excel.Interfaces.ChartErrorBarsFormatLoadOptions;
            /**
             *
             * Specifies whether or not the error bars have an end style cap.
             *
             * [Api set: ExcelApi 1.9]
             */
            endStyleCap?: boolean;
            /**
             *
             * Specifies which parts of the error bars to include.
             *
             * [Api set: ExcelApi 1.9]
             */
            include?: boolean;
            /**
             *
             * The type of range marked by the error bars.
             *
             * [Api set: ExcelApi 1.9]
             */
            type?: boolean;
            /**
             *
             * Specifies whether or not the error bars are displayed.
             *
             * [Api set: ExcelApi 1.9]
             */
            visible?: boolean;
        }
        /**
         *
         * Encapsulates the format properties for chart error bars.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface ChartErrorBarsFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the chart line formatting.
            *
            * [Api set: ExcelApi 1.9]
            */
            line?: Excel.Interfaces.ChartLineFormatLoadOptions;
        }
        /**
         *
         * Represents major or minor gridlines on a chart axis.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartGridlinesLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the formatting of chart gridlines.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartGridlinesFormatLoadOptions;
            /**
             *
             * Boolean value representing if the axis gridlines are visible or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
        }
        /**
         *
         * Encapsulates the format properties for chart gridlines.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartGridlinesFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents chart line formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            line?: Excel.Interfaces.ChartLineFormatLoadOptions;
        }
        /**
         *
         * Represents the legend in a chart.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartLegendLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the formatting of a chart legend, which includes fill and font formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartLegendFormatLoadOptions;
            /**
             *
             * Represents the height, in points, of the legend on the chart. Null if legend is not visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            height?: boolean;
            /**
             *
             * Represents the left, in points, of a chart legend. Null if legend is not visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            left?: boolean;
            /**
             *
             * Boolean value for whether the chart legend should overlap with the main body of the chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            overlay?: boolean;
            /**
             *
             * Represents the position of the legend on the chart. See Excel.ChartLegendPosition for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            position?: boolean;
            /**
             *
             * Represents if the legend has a shadow on the chart.
             *
             * [Api set: ExcelApi 1.7]
             */
            showShadow?: boolean;
            /**
             *
             * Represents the top of a chart legend.
             *
             * [Api set: ExcelApi 1.7]
             */
            top?: boolean;
            /**
             *
             * A boolean value the represents the visibility of a ChartLegend object.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
            /**
             *
             * Represents the width, in points, of the legend on the chart. Null if legend is not visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            width?: boolean;
        }
        /**
         *
         * Represents the legendEntry in legendEntryCollection.
         *
         * [Api set: ExcelApi 1.7]
         */
        interface ChartLegendEntryLoadOptions {
            $all?: boolean;
            /**
             *
             * Represents the height of the legendEntry on the chart legend.
             *
             * [Api set: ExcelApi 1.8]
             */
            height?: boolean;
            /**
             *
             * Represents the index of the legendEntry in the chart legend.
             *
             * [Api set: ExcelApi 1.8]
             */
            index?: boolean;
            /**
             *
             * Represents the left of a chart legendEntry.
             *
             * [Api set: ExcelApi 1.8]
             */
            left?: boolean;
            /**
             *
             * Represents the top of a chart legendEntry.
             *
             * [Api set: ExcelApi 1.8]
             */
            top?: boolean;
            /**
             *
             * Represents the visible of a chart legend entry.
             *
             * [Api set: ExcelApi 1.7]
             */
            visible?: boolean;
            /**
             *
             * Represents the width of the legendEntry on the chart Legend.
             *
             * [Api set: ExcelApi 1.8]
             */
            width?: boolean;
        }
        /**
         *
         * Represents a collection of legendEntries.
         *
         * [Api set: ExcelApi 1.7]
         */
        interface ChartLegendEntryCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the height of the legendEntry on the chart legend.
             *
             * [Api set: ExcelApi 1.8]
             */
            height?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the index of the legendEntry in the chart legend.
             *
             * [Api set: ExcelApi 1.8]
             */
            index?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the left of a chart legendEntry.
             *
             * [Api set: ExcelApi 1.8]
             */
            left?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the top of a chart legendEntry.
             *
             * [Api set: ExcelApi 1.8]
             */
            top?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the visible of a chart legend entry.
             *
             * [Api set: ExcelApi 1.7]
             */
            visible?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the width of the legendEntry on the chart Legend.
             *
             * [Api set: ExcelApi 1.8]
             */
            width?: boolean;
        }
        /**
         *
         * Encapsulates the format properties of a chart legend.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartLegendFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the border format, which includes color, linestyle, and weight.
            *
            * [Api set: ExcelApi 1.8]
            */
            border?: Excel.Interfaces.ChartBorderLoadOptions;
            /**
            *
            * Represents the font attributes such as font name, font size, color, etc. of a chart legend.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontLoadOptions;
        }
        /**
         *
         * Encapsulates the properties for a region map chart.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface ChartMapOptionsLoadOptions {
            $all?: boolean;
            /**
             *
             * Returns or sets the series map labels strategy of a region map chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            labelStrategy?: boolean;
            /**
             *
             * Returns or sets the series mapping level of a region map chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            level?: boolean;
            /**
             *
             * Returns or sets the series projection type of a region map chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            projectionType?: boolean;
        }
        /**
         *
         * Represents a chart title object of a chart.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartTitleLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the formatting of a chart title, which includes fill and font formatting.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.ChartTitleFormatLoadOptions;
            /**
             *
             * Returns the height, in points, of the chart title. Null if chart title is not visible. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            height?: boolean;
            /**
             *
             * Represents the horizontal alignment for chart title.
             *
             * [Api set: ExcelApi 1.7]
             */
            horizontalAlignment?: boolean;
            /**
             *
             * Represents the distance, in points, from the left edge of chart title to the left edge of chart area. Null if chart title is not visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            left?: boolean;
            /**
             *
             * Boolean value representing if the chart title will overlay the chart or not.
             *
             * [Api set: ExcelApi 1.1]
             */
            overlay?: boolean;
            /**
             *
             * Represents the position of chart title. See Excel.ChartTitlePosition for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            position?: boolean;
            /**
             *
             * Represents a boolean value that determines if the chart title has a shadow.
             *
             * [Api set: ExcelApi 1.7]
             */
            showShadow?: boolean;
            /**
             *
             * Represents the title text of a chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            text?: boolean;
            /**
             *
             * Represents the text orientation of chart title. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
             *
             * [Api set: ExcelApi 1.7]
             */
            textOrientation?: boolean;
            /**
             *
             * Represents the distance, in points, from the top edge of chart title to the top of chart area. Null if chart title is not visible.
             *
             * [Api set: ExcelApi 1.7]
             */
            top?: boolean;
            /**
             *
             * Represents the vertical alignment of chart title. See Excel.ChartTextVerticalAlignment for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            verticalAlignment?: boolean;
            /**
             *
             * A boolean value the represents the visibility of a chart title object.
             *
             * [Api set: ExcelApi 1.1]
             */
            visible?: boolean;
            /**
             *
             * Returns the width, in points, of the chart title. Null if chart title is not visible. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            width?: boolean;
        }
        /**
         *
         * Represents the substring in chart related objects that contains text, like ChartTitle object, ChartAxisTitle object, etc.
         *
         * [Api set: ExcelApi 1.7]
         */
        interface ChartFormatStringLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the font attributes, such as font name, font size, color, etc. of chart characters object.
            *
            * [Api set: ExcelApi 1.7]
            */
            font?: Excel.Interfaces.ChartFontLoadOptions;
        }
        /**
         *
         * Provides access to the office art formatting for chart title.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartTitleFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the border format of chart title, which includes color, linestyle, and weight.
            *
            * [Api set: ExcelApi 1.7]
            */
            border?: Excel.Interfaces.ChartBorderLoadOptions;
            /**
            *
            * Represents the font attributes (font name, font size, color, etc.) for an object.
            *
            * [Api set: ExcelApi 1.1]
            */
            font?: Excel.Interfaces.ChartFontLoadOptions;
        }
        /**
         *
         * Represents the border formatting of a chart element.
         *
         * [Api set: ExcelApi 1.7]
         */
        interface ChartBorderLoadOptions {
            $all?: boolean;
            /**
             *
             * HTML color code representing the color of borders in the chart.
             *
             * [Api set: ExcelApi 1.7]
             */
            color?: boolean;
            /**
             *
             * Represents the line style of the border. See Excel.ChartLineStyle for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            lineStyle?: boolean;
            /**
             *
             * Represents weight of the border, in points.
             *
             * [Api set: ExcelApi 1.7]
             */
            weight?: boolean;
        }
        /**
         *
         * Encapsulates the bin options for histogram charts and pareto charts.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface ChartBinOptionsLoadOptions {
            $all?: boolean;
            /**
             *
             * Specifies whether or not the bin overflow is enabled in a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            allowOverflow?: boolean;
            /**
             *
             * Specifies whether or not the bin underflow is enabled in a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            allowUnderflow?: boolean;
            /**
             *
             * Returns or sets the bin count of a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            count?: boolean;
            /**
             *
             * Returns or sets the bin overflow value of a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            overflowValue?: boolean;
            /**
             *
             * Returns or sets the bin's type for a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            type?: boolean;
            /**
             *
             * Returns or sets the bin underflow value of a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            underflowValue?: boolean;
            /**
             *
             * Returns or sets the bin width value of a histogram chart or pareto chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            width?: boolean;
        }
        /**
         *
         * Represents the properties of a box and whisker chart.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface ChartBoxwhiskerOptionsLoadOptions {
            $all?: boolean;
            /**
             *
             * Returns or sets the quartile calculation type of a box and whisker chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            quartileCalculation?: boolean;
            /**
             *
             * Specifies whether or not the inner points are shown in a box and whisker chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showInnerPoints?: boolean;
            /**
             *
             * Specifies whether or not the mean line is shown in a box and whisker chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showMeanLine?: boolean;
            /**
             *
             * Specifies whether or not the mean marker is shown in a box and whisker chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showMeanMarker?: boolean;
            /**
             *
             * Specifies whether or not outlier points are shown in a box and whisker chart. Read/Write.
             *
             * [Api set: ExcelApi 1.9]
             */
            showOutlierPoints?: boolean;
        }
        /**
         *
         * Encapsulates the formatting options for line elements.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartLineFormatLoadOptions {
            $all?: boolean;
            /**
             *
             * HTML color code representing the color of lines in the chart.
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: boolean;
            /**
             *
             * Represents the line style. See Excel.ChartLineStyle for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            lineStyle?: boolean;
            /**
             *
             * Represents weight of the line, in points.
             *
             * [Api set: ExcelApi 1.7]
             */
            weight?: boolean;
        }
        /**
         *
         * This object represents the font attributes (font name, font size, color, etc.) for a chart object.
         *
         * [Api set: ExcelApi 1.1]
         */
        interface ChartFontLoadOptions {
            $all?: boolean;
            /**
             *
             * Represents the bold status of font.
             *
             * [Api set: ExcelApi 1.1]
             */
            bold?: boolean;
            /**
             *
             * HTML color code representation of the text color. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.1]
             */
            color?: boolean;
            /**
             *
             * Represents the italic status of the font.
             *
             * [Api set: ExcelApi 1.1]
             */
            italic?: boolean;
            /**
             *
             * Font name (e.g. "Calibri")
             *
             * [Api set: ExcelApi 1.1]
             */
            name?: boolean;
            /**
             *
             * Size of the font (e.g. 11)
             *
             * [Api set: ExcelApi 1.1]
             */
            size?: boolean;
            /**
             *
             * Type of underline applied to the font. See Excel.ChartUnderlineStyle for details.
             *
             * [Api set: ExcelApi 1.1]
             */
            underline?: boolean;
        }
        /**
         *
         * This object represents the attributes for a chart trendline object.
         *
         * [Api set: ExcelApi 1.7]
         */
        interface ChartTrendlineLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the formatting of a chart trendline.
            *
            * [Api set: ExcelApi 1.7]
            */
            format?: Excel.Interfaces.ChartTrendlineFormatLoadOptions;
            /**
            *
            * Represents the label of a chart trendline.
            *
            * [Api set: ExcelApi 1.8]
            */
            label?: Excel.Interfaces.ChartTrendlineLabelLoadOptions;
            /**
             *
             * Represents the number of periods that the trendline extends backward.
             *
             * [Api set: ExcelApi 1.8]
             */
            backwardPeriod?: boolean;
            /**
             *
             * Represents the number of periods that the trendline extends forward.
             *
             * [Api set: ExcelApi 1.8]
             */
            forwardPeriod?: boolean;
            /**
             *
             * Represents the intercept value of the trendline. Can be set to a numeric value or an empty string (for automatic values). The returned value is always a number.
             *
             * [Api set: ExcelApi 1.7]
             */
            intercept?: boolean;
            /**
             *
             * Represents the period of a chart trendline. Only applicable for trendline with MovingAverage type.
             *
             * [Api set: ExcelApi 1.7]
             */
            movingAveragePeriod?: boolean;
            /**
             *
             * Represents the name of the trendline. Can be set to a string value, or can be set to null value represents automatic values. The returned value is always a string
             *
             * [Api set: ExcelApi 1.7]
             */
            name?: boolean;
            /**
             *
             * Represents the order of a chart trendline. Only applicable for trendline with Polynomial type.
             *
             * [Api set: ExcelApi 1.7]
             */
            polynomialOrder?: boolean;
            /**
             *
             * True if the equation for the trendline is displayed on the chart.
             *
             * [Api set: ExcelApi 1.8]
             */
            showEquation?: boolean;
            /**
             *
             * True if the R-squared for the trendline is displayed on the chart.
             *
             * [Api set: ExcelApi 1.8]
             */
            showRSquared?: boolean;
            /**
             *
             * Represents the type of a chart trendline.
             *
             * [Api set: ExcelApi 1.7]
             */
            type?: boolean;
        }
        /**
         *
         * Represents a collection of Chart Trendlines.
         *
         * [Api set: ExcelApi 1.7]
         */
        interface ChartTrendlineCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Represents the formatting of a chart trendline.
            *
            * [Api set: ExcelApi 1.7]
            */
            format?: Excel.Interfaces.ChartTrendlineFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents the label of a chart trendline.
            *
            * [Api set: ExcelApi 1.8]
            */
            label?: Excel.Interfaces.ChartTrendlineLabelLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Represents the number of periods that the trendline extends backward.
             *
             * [Api set: ExcelApi 1.8]
             */
            backwardPeriod?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the number of periods that the trendline extends forward.
             *
             * [Api set: ExcelApi 1.8]
             */
            forwardPeriod?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the intercept value of the trendline. Can be set to a numeric value or an empty string (for automatic values). The returned value is always a number.
             *
             * [Api set: ExcelApi 1.7]
             */
            intercept?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the period of a chart trendline. Only applicable for trendline with MovingAverage type.
             *
             * [Api set: ExcelApi 1.7]
             */
            movingAveragePeriod?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the name of the trendline. Can be set to a string value, or can be set to null value represents automatic values. The returned value is always a string
             *
             * [Api set: ExcelApi 1.7]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the order of a chart trendline. Only applicable for trendline with Polynomial type.
             *
             * [Api set: ExcelApi 1.7]
             */
            polynomialOrder?: boolean;
            /**
             *
             * For EACH ITEM in the collection: True if the equation for the trendline is displayed on the chart.
             *
             * [Api set: ExcelApi 1.8]
             */
            showEquation?: boolean;
            /**
             *
             * For EACH ITEM in the collection: True if the R-squared for the trendline is displayed on the chart.
             *
             * [Api set: ExcelApi 1.8]
             */
            showRSquared?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the type of a chart trendline.
             *
             * [Api set: ExcelApi 1.7]
             */
            type?: boolean;
        }
        /**
         *
         * Represents the format properties for chart trendline.
         *
         * [Api set: ExcelApi 1.7]
         */
        interface ChartTrendlineFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents chart line formatting.
            *
            * [Api set: ExcelApi 1.7]
            */
            line?: Excel.Interfaces.ChartLineFormatLoadOptions;
        }
        /**
         *
         * This object represents the attributes for a chart trendline lable object.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface ChartTrendlineLabelLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the format of chart trendline label.
            *
            * [Api set: ExcelApi 1.8]
            */
            format?: Excel.Interfaces.ChartTrendlineLabelFormatLoadOptions;
            /**
             *
             * Boolean value representing if trendline label automatically generates appropriate text based on context.
             *
             * [Api set: ExcelApi 1.8]
             */
            autoText?: boolean;
            /**
             *
             * String value that represents the formula of chart trendline label using A1-style notation.
             *
             * [Api set: ExcelApi 1.8]
             */
            formula?: boolean;
            /**
             *
             * Returns the height, in points, of the chart trendline label. Read-only. Null if chart trendline label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            height?: boolean;
            /**
             *
             * Represents the horizontal alignment for chart trendline label. See Excel.ChartTextHorizontalAlignment for details.
            This property is valid only when TextOrientation of trendline label is 90, -90 or 180.
             *
             * [Api set: ExcelApi 1.8]
             */
            horizontalAlignment?: boolean;
            /**
             *
             * Represents the distance, in points, from the left edge of chart trendline label to the left edge of chart area. Null if chart trendline label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            left?: boolean;
            /**
             *
             * Boolean value representing if the number format is linked to the cells (so that the number format changes in the labels when it changes in the cells).
             *
             * [Api set: ExcelApi 1.9]
             */
            linkNumberFormat?: boolean;
            /**
             *
             * String value that represents the format code for trendline label.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: boolean;
            /**
             *
             * String representing the text of the trendline label on a chart.
             *
             * [Api set: ExcelApi 1.8]
             */
            text?: boolean;
            /**
             *
             * Represents the text orientation of chart trendline label. The value should be an integer either from -90 to 90, or 180 for vertically-oriented text.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: boolean;
            /**
             *
             * Represents the distance, in points, from the top edge of chart trendline label to the top of chart area. Null if chart trendline label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            top?: boolean;
            /**
             *
             * Represents the vertical alignment of chart trendline label. See Excel.ChartTextVerticalAlignment for details.
            This property is valid only when TextOrientation of trendline label is 0.
             *
             * [Api set: ExcelApi 1.8]
             */
            verticalAlignment?: boolean;
            /**
             *
             * Returns the width, in points, of the chart trendline label. Read-only. Null if chart trendline label is not visible.
             *
             * [Api set: ExcelApi 1.8]
             */
            width?: boolean;
        }
        /**
         *
         * Encapsulates the format properties for the chart trendline label.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface ChartTrendlineLabelFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the border format, which includes color, linestyle, and weight.
            *
            * [Api set: ExcelApi 1.8]
            */
            border?: Excel.Interfaces.ChartBorderLoadOptions;
            /**
            *
            * Represents the font attributes (font name, font size, color, etc.) for a chart trendline label.
            *
            * [Api set: ExcelApi 1.8]
            */
            font?: Excel.Interfaces.ChartFontLoadOptions;
        }
        /**
         *
         * This object represents the attributes for a chart plotArea object.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface ChartPlotAreaLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the formatting of a chart plotArea.
            *
            * [Api set: ExcelApi 1.8]
            */
            format?: Excel.Interfaces.ChartPlotAreaFormatLoadOptions;
            /**
             *
             * Represents the height value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            height?: boolean;
            /**
             *
             * Represents the insideHeight value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            insideHeight?: boolean;
            /**
             *
             * Represents the insideLeft value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            insideLeft?: boolean;
            /**
             *
             * Represents the insideTop value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            insideTop?: boolean;
            /**
             *
             * Represents the insideWidth value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            insideWidth?: boolean;
            /**
             *
             * Represents the left value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            left?: boolean;
            /**
             *
             * Represents the position of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: boolean;
            /**
             *
             * Represents the top value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            top?: boolean;
            /**
             *
             * Represents the width value of plotArea.
             *
             * [Api set: ExcelApi 1.8]
             */
            width?: boolean;
        }
        /**
         *
         * Represents the format properties for chart plotArea.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface ChartPlotAreaFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the border attributes of a chart plotArea.
            *
            * [Api set: ExcelApi 1.8]
            */
            border?: Excel.Interfaces.ChartBorderLoadOptions;
        }
        /**
         *
         * Manages sorting operations on Table objects.
         *
         * [Api set: ExcelApi 1.2]
         */
        interface TableSortLoadOptions {
            $all?: boolean;
            /**
             *
             * Represents the current conditions used to last sort the table. Read-only.
             *
             * [Api set: ExcelApi 1.2]
             */
            fields?: boolean;
            /**
             *
             * Represents whether the casing impacted the last sort of the table. Read-only.
             *
             * [Api set: ExcelApi 1.2]
             */
            matchCase?: boolean;
            /**
             *
             * Represents Chinese character ordering method last used to sort the table. Read-only.
             *
             * [Api set: ExcelApi 1.2]
             */
            method?: boolean;
        }
        /**
         *
         * Manages the filtering of a table's column.
         *
         * [Api set: ExcelApi 1.2]
         */
        interface FilterLoadOptions {
            $all?: boolean;
            /**
             *
             * The currently applied filter on the given column. Read-only.
             *
             * [Api set: ExcelApi 1.2]
             */
            criteria?: boolean;
        }
        /**
         *
         * Represents the AutoFilter object.
            AutoFilter turns the values in Excel column into specific filters based on the cell contents.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface AutoFilterLoadOptions {
            $all?: boolean;
            /**
             *
             * An array that holds all the filter criteria in the autofiltered range. Read-Only.
             *
             * [Api set: ExcelApi 1.9]
             */
            criteria?: boolean;
            /**
             *
             * Indicates if the AutoFilter is enabled or not. Read-Only.
             *
             * [Api set: ExcelApi 1.9]
             */
            enabled?: boolean;
            /**
             *
             * Indicates if the AutoFilter has filter criteria. Read-Only.
             *
             * [Api set: ExcelApi 1.9]
             */
            isDataFiltered?: boolean;
        }
        /**
         *
         * A scoped collection of custom XML parts.
            A scoped collection is the result of some operation, e.g. filtering by namespace.
            A scoped collection cannot be scoped any further.
         *
         * [Api set: ExcelApi 1.5]
         */
        interface CustomXmlPartScopedCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The custom XML part's ID. Read-only.
             *
             * [Api set: ExcelApi 1.5]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The custom XML part's namespace URI. Read-only.
             *
             * [Api set: ExcelApi 1.5]
             */
            namespaceUri?: boolean;
        }
        /**
         *
         * A collection of custom XML parts.
         *
         * [Api set: ExcelApi 1.5]
         */
        interface CustomXmlPartCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The custom XML part's ID. Read-only.
             *
             * [Api set: ExcelApi 1.5]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The custom XML part's namespace URI. Read-only.
             *
             * [Api set: ExcelApi 1.5]
             */
            namespaceUri?: boolean;
        }
        /**
         *
         * Represents a custom XML part object in a workbook.
         *
         * [Api set: ExcelApi 1.5]
         */
        interface CustomXmlPartLoadOptions {
            $all?: boolean;
            /**
             *
             * The custom XML part's ID. Read-only.
             *
             * [Api set: ExcelApi 1.5]
             */
            id?: boolean;
            /**
             *
             * The custom XML part's namespace URI. Read-only.
             *
             * [Api set: ExcelApi 1.5]
             */
            namespaceUri?: boolean;
        }
        /**
         *
         * Represents a collection of all the PivotTables that are part of the workbook or worksheet.
         *
         * [Api set: ExcelApi 1.3]
         */
        interface PivotTableCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: The PivotLayout describing the layout and visual structure of the PivotTable.
            *
            * [Api set: ExcelApi 1.8]
            */
            layout?: Excel.Interfaces.PivotLayoutLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: The worksheet containing the current PivotTable.
            *
            * [Api set: ExcelApi 1.3]
            */
            worksheet?: Excel.Interfaces.WorksheetLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Specifies whether the PivotTable allows values in the data body to be edited by the user.
             *
             * [Api set: ExcelApi 1.9]
             */
            enableDataValueEditing?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Id of the PivotTable. Read-only.
             *
             * [Api set: ExcelApi 1.5]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Name of the PivotTable.
             *
             * [Api set: ExcelApi 1.3]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Specifies whether the PivotTable uses custom lists when sorting.
             *
             * [Api set: ExcelApi 1.9]
             */
            useCustomSortLists?: boolean;
        }
        /**
         *
         * Represents an Excel PivotTable.
         *
         * [Api set: ExcelApi 1.3]
         */
        interface PivotTableLoadOptions {
            $all?: boolean;
            /**
            *
            * The PivotLayout describing the layout and visual structure of the PivotTable.
            *
            * [Api set: ExcelApi 1.8]
            */
            layout?: Excel.Interfaces.PivotLayoutLoadOptions;
            /**
            *
            * The worksheet containing the current PivotTable.
            *
            * [Api set: ExcelApi 1.3]
            */
            worksheet?: Excel.Interfaces.WorksheetLoadOptions;
            /**
             *
             * Specifies whether the PivotTable allows values in the data body to be edited by the user.
             *
             * [Api set: ExcelApi 1.9]
             */
            enableDataValueEditing?: boolean;
            /**
             *
             * Id of the PivotTable. Read-only.
             *
             * [Api set: ExcelApi 1.5]
             */
            id?: boolean;
            /**
             *
             * Name of the PivotTable.
             *
             * [Api set: ExcelApi 1.3]
             */
            name?: boolean;
            /**
             *
             * Specifies whether the PivotTable uses custom lists when sorting.
             *
             * [Api set: ExcelApi 1.9]
             */
            useCustomSortLists?: boolean;
        }
        /**
         *
         * Represents the visual layout of the PivotTable.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface PivotLayoutLoadOptions {
            $all?: boolean;
            /**
             *
             * Specifies whether formatting will be automatically formatted when it’s refreshed or when fields are moved
             *
             * [Api set: ExcelApi 1.9]
             */
            autoFormat?: boolean;
            /**
             *
             * This property indicates the PivotLayoutType of all fields on the PivotTable. If fields have different states, this will be null.
             *
             * [Api set: ExcelApi 1.8]
             */
            layoutType?: boolean;
            /**
             *
             * Specifies whether formatting is preserved when the report is refreshed or recalculated by operations such as pivoting, sorting, or changing page field items.
             *
             * [Api set: ExcelApi 1.9]
             */
            preserveFormatting?: boolean;
            /**
             *
             * Specifies whether the PivotTable report shows grand totals for columns.
             *
             * [Api set: ExcelApi 1.8]
             */
            showColumnGrandTotals?: boolean;
            /**
             *
             * Specifies whether the PivotTable report shows grand totals for rows.
             *
             * [Api set: ExcelApi 1.8]
             */
            showRowGrandTotals?: boolean;
            /**
             *
             * This property indicates the SubtotalLocationType of all fields on the PivotTable. If fields have different states, this will be null.
             *
             * [Api set: ExcelApi 1.8]
             */
            subtotalLocation?: boolean;
        }
        /**
         *
         * Represents a collection of all the PivotTables that are part of the workbook or worksheet.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface PivotHierarchyCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Id of the PivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Name of the PivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: boolean;
        }
        /**
         *
         * Represents the Excel PivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface PivotHierarchyLoadOptions {
            $all?: boolean;
            /**
             *
             * Id of the PivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: boolean;
            /**
             *
             * Name of the PivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: boolean;
        }
        /**
         *
         * Represents a collection of RowColumnPivotHierarchy items associated with the PivotTable.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface RowColumnPivotHierarchyCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Id of the RowColumnPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Name of the RowColumnPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Position of the RowColumnPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: boolean;
        }
        /**
         *
         * Represents the Excel RowColumnPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface RowColumnPivotHierarchyLoadOptions {
            $all?: boolean;
            /**
             *
             * Id of the RowColumnPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: boolean;
            /**
             *
             * Name of the RowColumnPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: boolean;
            /**
             *
             * Position of the RowColumnPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: boolean;
        }
        /**
         *
         * Represents a collection of FilterPivotHierarchy items associated with the PivotTable.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface FilterPivotHierarchyCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Determines whether to allow multiple filter items.
             *
             * [Api set: ExcelApi 1.8]
             */
            enableMultipleFilterItems?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Id of the FilterPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Name of the FilterPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Position of the FilterPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: boolean;
        }
        /**
         *
         * Represents the Excel FilterPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface FilterPivotHierarchyLoadOptions {
            $all?: boolean;
            /**
             *
             * Determines whether to allow multiple filter items.
             *
             * [Api set: ExcelApi 1.8]
             */
            enableMultipleFilterItems?: boolean;
            /**
             *
             * Id of the FilterPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: boolean;
            /**
             *
             * Name of the FilterPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: boolean;
            /**
             *
             * Position of the FilterPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: boolean;
        }
        /**
         *
         * Represents a collection of DataPivotHierarchy items associated with the PivotTable.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface DataPivotHierarchyCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Returns the PivotFields associated with the DataPivotHierarchy.
            *
            * [Api set: ExcelApi 1.8]
            */
            field?: Excel.Interfaces.PivotFieldLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Id of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Name of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Number format of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Position of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Determines whether the data should be shown as a specific summary calculation or not.
             *
             * [Api set: ExcelApi 1.8]
             */
            showAs?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Determines whether to show all items of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            summarizeBy?: boolean;
        }
        /**
         *
         * Represents the Excel DataPivotHierarchy.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface DataPivotHierarchyLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns the PivotFields associated with the DataPivotHierarchy.
            *
            * [Api set: ExcelApi 1.8]
            */
            field?: Excel.Interfaces.PivotFieldLoadOptions;
            /**
             *
             * Id of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: boolean;
            /**
             *
             * Name of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: boolean;
            /**
             *
             * Number format of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            numberFormat?: boolean;
            /**
             *
             * Position of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            position?: boolean;
            /**
             *
             * Determines whether the data should be shown as a specific summary calculation or not.
             *
             * [Api set: ExcelApi 1.8]
             */
            showAs?: boolean;
            /**
             *
             * Determines whether to show all items of the DataPivotHierarchy.
             *
             * [Api set: ExcelApi 1.8]
             */
            summarizeBy?: boolean;
        }
        /**
         *
         * Represents a collection of all the PivotTables that are part of the workbook or worksheet.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface PivotFieldCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Id of the PivotField.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Name of the PivotField.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Determines whether to show all items of the PivotField.
             *
             * [Api set: ExcelApi 1.8]
             */
            showAllItems?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Subtotals of the PivotField.
             *
             * [Api set: ExcelApi 1.8]
             */
            subtotals?: boolean;
        }
        /**
         *
         * Represents the Excel PivotField.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface PivotFieldLoadOptions {
            $all?: boolean;
            /**
             *
             * Id of the PivotField.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: boolean;
            /**
             *
             * Name of the PivotField.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: boolean;
            /**
             *
             * Determines whether to show all items of the PivotField.
             *
             * [Api set: ExcelApi 1.8]
             */
            showAllItems?: boolean;
            /**
             *
             * Subtotals of the PivotField.
             *
             * [Api set: ExcelApi 1.8]
             */
            subtotals?: boolean;
        }
        /**
         *
         * Represents a collection of all the Pivot Items related to their parent PivotField.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface PivotItemCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Id of the PivotItem.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Determines whether the item is expanded to show child items or if it's collapsed and child items are hidden.
             *
             * [Api set: ExcelApi 1.8]
             */
            isExpanded?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Name of the PivotItem.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Determines whether the PivotItem is visible or not.
             *
             * [Api set: ExcelApi 1.8]
             */
            visible?: boolean;
        }
        /**
         *
         * Represents the Excel PivotItem.
         *
         * [Api set: ExcelApi 1.8]
         */
        interface PivotItemLoadOptions {
            $all?: boolean;
            /**
             *
             * Id of the PivotItem.
             *
             * [Api set: ExcelApi 1.8]
             */
            id?: boolean;
            /**
             *
             * Determines whether the item is expanded to show child items or if it's collapsed and child items are hidden.
             *
             * [Api set: ExcelApi 1.8]
             */
            isExpanded?: boolean;
            /**
             *
             * Name of the PivotItem.
             *
             * [Api set: ExcelApi 1.8]
             */
            name?: boolean;
            /**
             *
             * Determines whether the PivotItem is visible or not.
             *
             * [Api set: ExcelApi 1.8]
             */
            visible?: boolean;
        }
        /**
         *
         * Represents workbook properties.
         *
         * [Api set: ExcelApi 1.7]
         */
        interface DocumentPropertiesLoadOptions {
            $all?: boolean;
            /**
             *
             * Gets or sets the author of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            author?: boolean;
            /**
             *
             * Gets or sets the category of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            category?: boolean;
            /**
             *
             * Gets or sets the comments of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            comments?: boolean;
            /**
             *
             * Gets or sets the company of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            company?: boolean;
            /**
             *
             * Gets the creation date of the workbook. Read only.
             *
             * [Api set: ExcelApi 1.7]
             */
            creationDate?: boolean;
            /**
             *
             * Gets or sets the keywords of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            keywords?: boolean;
            /**
             *
             * Gets the last author of the workbook. Read only.
             *
             * [Api set: ExcelApi 1.7]
             */
            lastAuthor?: boolean;
            /**
             *
             * Gets or sets the manager of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            manager?: boolean;
            /**
             *
             * Gets the revision number of the workbook. Read only.
             *
             * [Api set: ExcelApi 1.7]
             */
            revisionNumber?: boolean;
            /**
             *
             * Gets or sets the subject of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            subject?: boolean;
            /**
             *
             * Gets or sets the title of the workbook.
             *
             * [Api set: ExcelApi 1.7]
             */
            title?: boolean;
        }
        /**
         *
         * Represents a custom property.
         *
         * [Api set: ExcelApi 1.7]
         */
        interface CustomPropertyLoadOptions {
            $all?: boolean;
            /**
             *
             * Gets the key of the custom property. Read only.
             *
             * [Api set: ExcelApi 1.7]
             */
            key?: boolean;
            /**
             *
             * Gets the value type of the custom property. Read only.
             *
             * [Api set: ExcelApi 1.7]
             */
            type?: boolean;
            /**
             *
             * Gets or sets the value of the custom property.
             *
             * [Api set: ExcelApi 1.7]
             */
            value?: boolean;
        }
        /**
         *
         * Contains the collection of customProperty objects.
         *
         * [Api set: ExcelApi 1.7]
         */
        interface CustomPropertyCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the key of the custom property. Read only.
             *
             * [Api set: ExcelApi 1.7]
             */
            key?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the value type of the custom property. Read only.
             *
             * [Api set: ExcelApi 1.7]
             */
            type?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the value of the custom property.
             *
             * [Api set: ExcelApi 1.7]
             */
            value?: boolean;
        }
        /**
         *
         * Represents a collection of all the conditional formats that are overlap the range.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface ConditionalFormatCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Returns the cell value conditional format properties if the current conditional format is a CellValue type.
            For example to format all cells between 5 and 10.
            *
            * [Api set: ExcelApi 1.6]
            */
            cellValue?: Excel.Interfaces.CellValueConditionalFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the cell value conditional format properties if the current conditional format is a CellValue type.
            For example to format all cells between 5 and 10.
            *
            * [Api set: ExcelApi 1.6]
            */
            cellValueOrNullObject?: Excel.Interfaces.CellValueConditionalFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the ColorScale conditional format properties if the current conditional format is an ColorScale type.
            *
            * [Api set: ExcelApi 1.6]
            */
            colorScale?: Excel.Interfaces.ColorScaleConditionalFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the ColorScale conditional format properties if the current conditional format is an ColorScale type.
            *
            * [Api set: ExcelApi 1.6]
            */
            colorScaleOrNullObject?: Excel.Interfaces.ColorScaleConditionalFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the custom conditional format properties if the current conditional format is a custom type.
            *
            * [Api set: ExcelApi 1.6]
            */
            custom?: Excel.Interfaces.CustomConditionalFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the custom conditional format properties if the current conditional format is a custom type.
            *
            * [Api set: ExcelApi 1.6]
            */
            customOrNullObject?: Excel.Interfaces.CustomConditionalFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the data bar properties if the current conditional format is a data bar.
            *
            * [Api set: ExcelApi 1.6]
            */
            dataBar?: Excel.Interfaces.DataBarConditionalFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the data bar properties if the current conditional format is a data bar.
            *
            * [Api set: ExcelApi 1.6]
            */
            dataBarOrNullObject?: Excel.Interfaces.DataBarConditionalFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the IconSet conditional format properties if the current conditional format is an IconSet type.
            *
            * [Api set: ExcelApi 1.6]
            */
            iconSet?: Excel.Interfaces.IconSetConditionalFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the IconSet conditional format properties if the current conditional format is an IconSet type.
            *
            * [Api set: ExcelApi 1.6]
            */
            iconSetOrNullObject?: Excel.Interfaces.IconSetConditionalFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the preset criteria conditional format. See Excel.PresetCriteriaConditionalFormat for more details.
            *
            * [Api set: ExcelApi 1.6]
            */
            preset?: Excel.Interfaces.PresetCriteriaConditionalFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the preset criteria conditional format. See Excel.PresetCriteriaConditionalFormat for more details.
            *
            * [Api set: ExcelApi 1.6]
            */
            presetOrNullObject?: Excel.Interfaces.PresetCriteriaConditionalFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the specific text conditional format properties if the current conditional format is a text type.
            For example to format cells matching the word "Text".
            *
            * [Api set: ExcelApi 1.6]
            */
            textComparison?: Excel.Interfaces.TextConditionalFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the specific text conditional format properties if the current conditional format is a text type.
            For example to format cells matching the word "Text".
            *
            * [Api set: ExcelApi 1.6]
            */
            textComparisonOrNullObject?: Excel.Interfaces.TextConditionalFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the Top/Bottom conditional format properties if the current conditional format is an TopBottom type.
            For example to format the top 10% or bottom 10 items.
            *
            * [Api set: ExcelApi 1.6]
            */
            topBottom?: Excel.Interfaces.TopBottomConditionalFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the Top/Bottom conditional format properties if the current conditional format is an TopBottom type.
            For example to format the top 10% or bottom 10 items.
            *
            * [Api set: ExcelApi 1.6]
            */
            topBottomOrNullObject?: Excel.Interfaces.TopBottomConditionalFormatLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: The Priority of the Conditional Format within the current ConditionalFormatCollection. Read-only.
             *
             * [Api set: ExcelApi 1.6]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The priority (or index) within the conditional format collection that this conditional format currently exists in. Changing this also
            changes other conditional formats' priorities, to allow for a contiguous priority order.
            Use a negative priority to begin from the back.
            Priorities greater than than bounds will get and set to the maximum (or minimum if negative) priority.
            Also note that if you change the priority, you have to re-fetch a new copy of the object at that new priority location if you want to make further changes to it. Read-only.
             *
             * [Api set: ExcelApi 1.6]
             */
            priority?: boolean;
            /**
             *
             * For EACH ITEM in the collection: If the conditions of this conditional format are met, no lower-priority formats shall take effect on that cell.
            Null on databars, icon sets, and colorscales as there's no concept of StopIfTrue for these
             *
             * [Api set: ExcelApi 1.6]
             */
            stopIfTrue?: boolean;
            /**
             *
             * For EACH ITEM in the collection: A type of conditional format. Only one can be set at a time. Read-only.
             *
             * [Api set: ExcelApi 1.6]
             */
            type?: boolean;
        }
        /**
         *
         * An object encapsulating a conditional format's range, format, rule, and other properties.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface ConditionalFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns the cell value conditional format properties if the current conditional format is a CellValue type.
            For example to format all cells between 5 and 10.
            *
            * [Api set: ExcelApi 1.6]
            */
            cellValue?: Excel.Interfaces.CellValueConditionalFormatLoadOptions;
            /**
            *
            * Returns the cell value conditional format properties if the current conditional format is a CellValue type.
            For example to format all cells between 5 and 10.
            *
            * [Api set: ExcelApi 1.6]
            */
            cellValueOrNullObject?: Excel.Interfaces.CellValueConditionalFormatLoadOptions;
            /**
            *
            * Returns the ColorScale conditional format properties if the current conditional format is an ColorScale type.
            *
            * [Api set: ExcelApi 1.6]
            */
            colorScale?: Excel.Interfaces.ColorScaleConditionalFormatLoadOptions;
            /**
            *
            * Returns the ColorScale conditional format properties if the current conditional format is an ColorScale type.
            *
            * [Api set: ExcelApi 1.6]
            */
            colorScaleOrNullObject?: Excel.Interfaces.ColorScaleConditionalFormatLoadOptions;
            /**
            *
            * Returns the custom conditional format properties if the current conditional format is a custom type.
            *
            * [Api set: ExcelApi 1.6]
            */
            custom?: Excel.Interfaces.CustomConditionalFormatLoadOptions;
            /**
            *
            * Returns the custom conditional format properties if the current conditional format is a custom type.
            *
            * [Api set: ExcelApi 1.6]
            */
            customOrNullObject?: Excel.Interfaces.CustomConditionalFormatLoadOptions;
            /**
            *
            * Returns the data bar properties if the current conditional format is a data bar.
            *
            * [Api set: ExcelApi 1.6]
            */
            dataBar?: Excel.Interfaces.DataBarConditionalFormatLoadOptions;
            /**
            *
            * Returns the data bar properties if the current conditional format is a data bar.
            *
            * [Api set: ExcelApi 1.6]
            */
            dataBarOrNullObject?: Excel.Interfaces.DataBarConditionalFormatLoadOptions;
            /**
            *
            * Returns the IconSet conditional format properties if the current conditional format is an IconSet type.
            *
            * [Api set: ExcelApi 1.6]
            */
            iconSet?: Excel.Interfaces.IconSetConditionalFormatLoadOptions;
            /**
            *
            * Returns the IconSet conditional format properties if the current conditional format is an IconSet type.
            *
            * [Api set: ExcelApi 1.6]
            */
            iconSetOrNullObject?: Excel.Interfaces.IconSetConditionalFormatLoadOptions;
            /**
            *
            * Returns the preset criteria conditional format. See Excel.PresetCriteriaConditionalFormat for more details.
            *
            * [Api set: ExcelApi 1.6]
            */
            preset?: Excel.Interfaces.PresetCriteriaConditionalFormatLoadOptions;
            /**
            *
            * Returns the preset criteria conditional format. See Excel.PresetCriteriaConditionalFormat for more details.
            *
            * [Api set: ExcelApi 1.6]
            */
            presetOrNullObject?: Excel.Interfaces.PresetCriteriaConditionalFormatLoadOptions;
            /**
            *
            * Returns the specific text conditional format properties if the current conditional format is a text type.
            For example to format cells matching the word "Text".
            *
            * [Api set: ExcelApi 1.6]
            */
            textComparison?: Excel.Interfaces.TextConditionalFormatLoadOptions;
            /**
            *
            * Returns the specific text conditional format properties if the current conditional format is a text type.
            For example to format cells matching the word "Text".
            *
            * [Api set: ExcelApi 1.6]
            */
            textComparisonOrNullObject?: Excel.Interfaces.TextConditionalFormatLoadOptions;
            /**
            *
            * Returns the Top/Bottom conditional format properties if the current conditional format is an TopBottom type.
            For example to format the top 10% or bottom 10 items.
            *
            * [Api set: ExcelApi 1.6]
            */
            topBottom?: Excel.Interfaces.TopBottomConditionalFormatLoadOptions;
            /**
            *
            * Returns the Top/Bottom conditional format properties if the current conditional format is an TopBottom type.
            For example to format the top 10% or bottom 10 items.
            *
            * [Api set: ExcelApi 1.6]
            */
            topBottomOrNullObject?: Excel.Interfaces.TopBottomConditionalFormatLoadOptions;
            /**
             *
             * The Priority of the Conditional Format within the current ConditionalFormatCollection. Read-only.
             *
             * [Api set: ExcelApi 1.6]
             */
            id?: boolean;
            /**
             *
             * The priority (or index) within the conditional format collection that this conditional format currently exists in. Changing this also
            changes other conditional formats' priorities, to allow for a contiguous priority order.
            Use a negative priority to begin from the back.
            Priorities greater than than bounds will get and set to the maximum (or minimum if negative) priority.
            Also note that if you change the priority, you have to re-fetch a new copy of the object at that new priority location if you want to make further changes to it. Read-only.
             *
             * [Api set: ExcelApi 1.6]
             */
            priority?: boolean;
            /**
             *
             * If the conditions of this conditional format are met, no lower-priority formats shall take effect on that cell.
            Null on databars, icon sets, and colorscales as there's no concept of StopIfTrue for these
             *
             * [Api set: ExcelApi 1.6]
             */
            stopIfTrue?: boolean;
            /**
             *
             * A type of conditional format. Only one can be set at a time. Read-only.
             *
             * [Api set: ExcelApi 1.6]
             */
            type?: boolean;
        }
        /**
         *
         * Represents an Excel Conditional Data Bar Type.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface DataBarConditionalFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Representation of all values to the left of the axis in an Excel data bar.
            *
            * [Api set: ExcelApi 1.6]
            */
            negativeFormat?: Excel.Interfaces.ConditionalDataBarNegativeFormatLoadOptions;
            /**
            *
            * Representation of all values to the right of the axis in an Excel data bar.
            *
            * [Api set: ExcelApi 1.6]
            */
            positiveFormat?: Excel.Interfaces.ConditionalDataBarPositiveFormatLoadOptions;
            /**
             *
             * HTML color code representing the color of the Axis line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            "" (empty string) if no axis is present or set.
             *
             * [Api set: ExcelApi 1.6]
             */
            axisColor?: boolean;
            /**
             *
             * Representation of how the axis is determined for an Excel data bar.
             *
             * [Api set: ExcelApi 1.6]
             */
            axisFormat?: boolean;
            /**
             *
             * Represents the direction that the data bar graphic should be based on.
             *
             * [Api set: ExcelApi 1.6]
             */
            barDirection?: boolean;
            /**
             *
             * The rule for what consistutes the lower bound (and how to calculate it, if applicable) for a data bar.
             *
             * [Api set: ExcelApi 1.6]
             */
            lowerBoundRule?: boolean;
            /**
             *
             * If true, hides the values from the cells where the data bar is applied.
             *
             * [Api set: ExcelApi 1.6]
             */
            showDataBarOnly?: boolean;
            /**
             *
             * The rule for what constitutes the upper bound (and how to calculate it, if applicable) for a data bar.
             *
             * [Api set: ExcelApi 1.6]
             */
            upperBoundRule?: boolean;
        }
        /**
         *
         * Represents a conditional format DataBar Format for the positive side of the data bar.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface ConditionalDataBarPositiveFormatLoadOptions {
            $all?: boolean;
            /**
             *
             * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            "" (empty string) if no border is present or set.
             *
             * [Api set: ExcelApi 1.6]
             */
            borderColor?: boolean;
            /**
             *
             * HTML color code representing the fill color, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.6]
             */
            fillColor?: boolean;
            /**
             *
             * Boolean representation of whether or not the DataBar has a gradient.
             *
             * [Api set: ExcelApi 1.6]
             */
            gradientFill?: boolean;
        }
        /**
         *
         * Represents a conditional format DataBar Format for the negative side of the data bar.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface ConditionalDataBarNegativeFormatLoadOptions {
            $all?: boolean;
            /**
             *
             * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
            "Empty String" if no border is present or set.
             *
             * [Api set: ExcelApi 1.6]
             */
            borderColor?: boolean;
            /**
             *
             * HTML color code representing the fill color, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.6]
             */
            fillColor?: boolean;
            /**
             *
             * Boolean representation of whether or not the negative DataBar has the same border color as the positive DataBar.
             *
             * [Api set: ExcelApi 1.6]
             */
            matchPositiveBorderColor?: boolean;
            /**
             *
             * Boolean representation of whether or not the negative DataBar has the same fill color as the positive DataBar.
             *
             * [Api set: ExcelApi 1.6]
             */
            matchPositiveFillColor?: boolean;
        }
        /**
         *
         * Represents a custom conditional format type.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface CustomConditionalFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties.
            *
            * [Api set: ExcelApi 1.6]
            */
            format?: Excel.Interfaces.ConditionalRangeFormatLoadOptions;
            /**
            *
            * Represents the Rule object on this conditional format.
            *
            * [Api set: ExcelApi 1.6]
            */
            rule?: Excel.Interfaces.ConditionalFormatRuleLoadOptions;
        }
        /**
         *
         * Represents a rule, for all traditional rule/format pairings.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface ConditionalFormatRuleLoadOptions {
            $all?: boolean;
            /**
             *
             * The formula, if required, to evaluate the conditional format rule on.
             *
             * [Api set: ExcelApi 1.6]
             */
            formula?: boolean;
            /**
             *
             * The formula, if required, to evaluate the conditional format rule on in the user's language.
             *
             * [Api set: ExcelApi 1.6]
             */
            formulaLocal?: boolean;
            /**
             *
             * The formula, if required, to evaluate the conditional format rule on in R1C1-style notation.
             *
             * [Api set: ExcelApi 1.6]
             */
            formulaR1C1?: boolean;
        }
        /**
         *
         * Represents an IconSet criteria for conditional formatting.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface IconSetConditionalFormatLoadOptions {
            $all?: boolean;
            /**
             *
             * An array of Criteria and IconSets for the rules and potential custom icons for conditional icons. Note that for the first criterion only the custom icon can be modified, while type, formula, and operator will be ignored when set.
             *
             * [Api set: ExcelApi 1.6]
             */
            criteria?: boolean;
            /**
             *
             * If true, reverses the icon orders for the IconSet. Note that this cannot be set if custom icons are used.
             *
             * [Api set: ExcelApi 1.6]
             */
            reverseIconOrder?: boolean;
            /**
             *
             * If true, hides the values and only shows icons.
             *
             * [Api set: ExcelApi 1.6]
             */
            showIconOnly?: boolean;
            /**
             *
             * If set, displays the IconSet option for the conditional format.
             *
             * [Api set: ExcelApi 1.6]
             */
            style?: boolean;
        }
        /**
         *
         * Represents an IconSet criteria for conditional formatting.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface ColorScaleConditionalFormatLoadOptions {
            $all?: boolean;
            /**
             *
             * The criteria of the color scale. Midpoint is optional when using a two point color scale.
             *
             * [Api set: ExcelApi 1.6]
             */
            criteria?: boolean;
            /**
             *
             * If true the color scale will have three points (minimum, midpoint, maximum), otherwise it will have two (minimum, maximum).
             *
             * [Api set: ExcelApi 1.6]
             */
            threeColorScale?: boolean;
        }
        /**
         *
         * Represents a Top/Bottom conditional format.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface TopBottomConditionalFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties.
            *
            * [Api set: ExcelApi 1.6]
            */
            format?: Excel.Interfaces.ConditionalRangeFormatLoadOptions;
            /**
             *
             * The criteria of the Top/Bottom conditional format.
             *
             * [Api set: ExcelApi 1.6]
             */
            rule?: boolean;
        }
        /**
         *
         * Represents the the preset criteria conditional format such as above average, below average, unique values, contains blank, nonblank, error, and noerror.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface PresetCriteriaConditionalFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties.
            *
            * [Api set: ExcelApi 1.6]
            */
            format?: Excel.Interfaces.ConditionalRangeFormatLoadOptions;
            /**
             *
             * The rule of the conditional format.
             *
             * [Api set: ExcelApi 1.6]
             */
            rule?: boolean;
        }
        /**
         *
         * Represents a specific text conditional format.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface TextConditionalFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties.
            *
            * [Api set: ExcelApi 1.6]
            */
            format?: Excel.Interfaces.ConditionalRangeFormatLoadOptions;
            /**
             *
             * The rule of the conditional format.
             *
             * [Api set: ExcelApi 1.6]
             */
            rule?: boolean;
        }
        /**
         *
         * Represents a cell value conditional format.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface CellValueConditionalFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns a format object, encapsulating the conditional formats font, fill, borders, and other properties.
            *
            * [Api set: ExcelApi 1.6]
            */
            format?: Excel.Interfaces.ConditionalRangeFormatLoadOptions;
            /**
             *
             * Represents the Rule object on this conditional format.
             *
             * [Api set: ExcelApi 1.6]
             */
            rule?: boolean;
        }
        /**
         *
         * A format object encapsulating the conditional formats range's font, fill, borders, and other properties.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface ConditionalRangeFormatLoadOptions {
            $all?: boolean;
            /**
            *
            * Collection of border objects that apply to the overall conditional format range.
            *
            * [Api set: ExcelApi 1.6]
            */
            borders?: Excel.Interfaces.ConditionalRangeBorderCollectionLoadOptions;
            /**
            *
            * Returns the fill object defined on the overall conditional format range.
            *
            * [Api set: ExcelApi 1.6]
            */
            fill?: Excel.Interfaces.ConditionalRangeFillLoadOptions;
            /**
            *
            * Returns the font object defined on the overall conditional format range.
            *
            * [Api set: ExcelApi 1.6]
            */
            font?: Excel.Interfaces.ConditionalRangeFontLoadOptions;
            /**
             *
             * Represents Excel's number format code for the given range. Cleared if null is passed in.
             *
             * [Api set: ExcelApi 1.6]
             */
            numberFormat?: boolean;
        }
        /**
         *
         * This object represents the font attributes (font style, color, etc.) for an object.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface ConditionalRangeFontLoadOptions {
            $all?: boolean;
            /**
             *
             * Represents the bold status of font.
             *
             * [Api set: ExcelApi 1.6]
             */
            bold?: boolean;
            /**
             *
             * HTML color code representation of the text color. E.g. #FF0000 represents Red.
             *
             * [Api set: ExcelApi 1.6]
             */
            color?: boolean;
            /**
             *
             * Represents the italic status of the font.
             *
             * [Api set: ExcelApi 1.6]
             */
            italic?: boolean;
            /**
             *
             * Represents the strikethrough status of the font.
             *
             * [Api set: ExcelApi 1.6]
             */
            strikethrough?: boolean;
            /**
             *
             * Type of underline applied to the font. See Excel.ConditionalRangeFontUnderlineStyle for details.
             *
             * [Api set: ExcelApi 1.6]
             */
            underline?: boolean;
        }
        /**
         *
         * Represents the background of a conditional range object.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface ConditionalRangeFillLoadOptions {
            $all?: boolean;
            /**
             *
             * HTML color code representing the color of the fill, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.6]
             */
            color?: boolean;
        }
        /**
         *
         * Represents the border of an object.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface ConditionalRangeBorderLoadOptions {
            $all?: boolean;
            /**
             *
             * HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.6]
             */
            color?: boolean;
            /**
             *
             * Constant value that indicates the specific side of the border. See Excel.ConditionalRangeBorderIndex for details. Read-only.
             *
             * [Api set: ExcelApi 1.6]
             */
            sideIndex?: boolean;
            /**
             *
             * One of the constants of line style specifying the line style for the border. See Excel.BorderLineStyle for details.
             *
             * [Api set: ExcelApi 1.6]
             */
            style?: boolean;
        }
        /**
         *
         * Represents the border objects that make up range border.
         *
         * [Api set: ExcelApi 1.6]
         */
        interface ConditionalRangeBorderCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: HTML color code representing the color of the border line, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.6]
             */
            color?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Constant value that indicates the specific side of the border. See Excel.ConditionalRangeBorderIndex for details. Read-only.
             *
             * [Api set: ExcelApi 1.6]
             */
            sideIndex?: boolean;
            /**
             *
             * For EACH ITEM in the collection: One of the constants of line style specifying the line style for the border. See Excel.BorderLineStyle for details.
             *
             * [Api set: ExcelApi 1.6]
             */
            style?: boolean;
        }
        /**
         *
         * An object encapsulating a style's format and other properties.
         *
         * [Api set: ExcelApi 1.7]
         */
        interface StyleLoadOptions {
            $all?: boolean;
            /**
            *
            * A Border collection of four Border objects that represent the style of the four borders.
            *
            * [Api set: ExcelApi 1.7]
            */
            borders?: Excel.Interfaces.RangeBorderCollectionLoadOptions;
            /**
            *
            * The Fill of the style.
            *
            * [Api set: ExcelApi 1.7]
            */
            fill?: Excel.Interfaces.RangeFillLoadOptions;
            /**
            *
            * A Font object that represents the font of the style.
            *
            * [Api set: ExcelApi 1.7]
            */
            font?: Excel.Interfaces.RangeFontLoadOptions;
            /**
             *
             * Indicates if text is automatically indented when the text alignment in a cell is set to equal distribution.
             *
             * [Api set: ExcelApi 1.8]
             */
            autoIndent?: boolean;
            /**
             *
             * Indicates if the style is a built-in style.
             *
             * [Api set: ExcelApi 1.7]
             */
            builtIn?: boolean;
            /**
             *
             * Indicates if the formula will be hidden when the worksheet is protected.
             *
             * [Api set: ExcelApi 1.7]
             */
            formulaHidden?: boolean;
            /**
             *
             * Represents the horizontal alignment for the style. See Excel.HorizontalAlignment for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            horizontalAlignment?: boolean;
            /**
             *
             * Indicates if the style includes the AutoIndent, HorizontalAlignment, VerticalAlignment, WrapText, IndentLevel, and TextOrientation properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeAlignment?: boolean;
            /**
             *
             * Indicates if the style includes the Color, ColorIndex, LineStyle, and Weight border properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeBorder?: boolean;
            /**
             *
             * Indicates if the style includes the Background, Bold, Color, ColorIndex, FontStyle, Italic, Name, Size, Strikethrough, Subscript, Superscript, and Underline font properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeFont?: boolean;
            /**
             *
             * Indicates if the style includes the NumberFormat property.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeNumber?: boolean;
            /**
             *
             * Indicates if the style includes the Color, ColorIndex, InvertIfNegative, Pattern, PatternColor, and PatternColorIndex interior properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includePatterns?: boolean;
            /**
             *
             * Indicates if the style includes the FormulaHidden and Locked protection properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeProtection?: boolean;
            /**
             *
             * An integer from 0 to 250 that indicates the indent level for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            indentLevel?: boolean;
            /**
             *
             * Indicates if the object is locked when the worksheet is protected.
             *
             * [Api set: ExcelApi 1.7]
             */
            locked?: boolean;
            /**
             *
             * The name of the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            name?: boolean;
            /**
             *
             * The format code of the number format for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            numberFormat?: boolean;
            /**
             *
             * The localized format code of the number format for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            numberFormatLocal?: boolean;
            /**
             *
             * The reading order for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            readingOrder?: boolean;
            /**
             *
             * Indicates if text automatically shrinks to fit in the available column width.
             *
             * [Api set: ExcelApi 1.7]
             */
            shrinkToFit?: boolean;
            /**
             *
             * The text orientation for the style.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: boolean;
            /**
             *
             * Represents the vertical alignment for the style. See Excel.VerticalAlignment for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            verticalAlignment?: boolean;
            /**
             *
             * Indicates if Microsoft Excel wraps the text in the object.
             *
             * [Api set: ExcelApi 1.7]
             */
            wrapText?: boolean;
        }
        /**
         *
         * Represents a collection of all the styles.
         *
         * [Api set: ExcelApi 1.7]
         */
        interface StyleCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: A Border collection of four Border objects that represent the style of the four borders.
            *
            * [Api set: ExcelApi 1.7]
            */
            borders?: Excel.Interfaces.RangeBorderCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: The Fill of the style.
            *
            * [Api set: ExcelApi 1.7]
            */
            fill?: Excel.Interfaces.RangeFillLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: A Font object that represents the font of the style.
            *
            * [Api set: ExcelApi 1.7]
            */
            font?: Excel.Interfaces.RangeFontLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Indicates if text is automatically indented when the text alignment in a cell is set to equal distribution.
             *
             * [Api set: ExcelApi 1.8]
             */
            autoIndent?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates if the style is a built-in style.
             *
             * [Api set: ExcelApi 1.7]
             */
            builtIn?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates if the formula will be hidden when the worksheet is protected.
             *
             * [Api set: ExcelApi 1.7]
             */
            formulaHidden?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the horizontal alignment for the style. See Excel.HorizontalAlignment for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            horizontalAlignment?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates if the style includes the AutoIndent, HorizontalAlignment, VerticalAlignment, WrapText, IndentLevel, and TextOrientation properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeAlignment?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates if the style includes the Color, ColorIndex, LineStyle, and Weight border properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeBorder?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates if the style includes the Background, Bold, Color, ColorIndex, FontStyle, Italic, Name, Size, Strikethrough, Subscript, Superscript, and Underline font properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeFont?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates if the style includes the NumberFormat property.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeNumber?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates if the style includes the Color, ColorIndex, InvertIfNegative, Pattern, PatternColor, and PatternColorIndex interior properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includePatterns?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates if the style includes the FormulaHidden and Locked protection properties.
             *
             * [Api set: ExcelApi 1.7]
             */
            includeProtection?: boolean;
            /**
             *
             * For EACH ITEM in the collection: An integer from 0 to 250 that indicates the indent level for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            indentLevel?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates if the object is locked when the worksheet is protected.
             *
             * [Api set: ExcelApi 1.7]
             */
            locked?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The name of the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The format code of the number format for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            numberFormat?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The localized format code of the number format for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            numberFormatLocal?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The reading order for the style.
             *
             * [Api set: ExcelApi 1.7]
             */
            readingOrder?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates if text automatically shrinks to fit in the available column width.
             *
             * [Api set: ExcelApi 1.7]
             */
            shrinkToFit?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The text orientation for the style.
             *
             * [Api set: ExcelApi 1.8]
             */
            textOrientation?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the vertical alignment for the style. See Excel.VerticalAlignment for details.
             *
             * [Api set: ExcelApi 1.7]
             */
            verticalAlignment?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates if Microsoft Excel wraps the text in the object.
             *
             * [Api set: ExcelApi 1.7]
             */
            wrapText?: boolean;
        }
        /**
         * [Api set: ExcelApi 1.9]
         */
        interface PageLayoutLoadOptions {
            $all?: boolean;
            /**
            *
            * Header and footer configuration for the worksheet.
            *
            * [Api set: ExcelApi 1.9]
            */
            headersFooters?: Excel.Interfaces.HeaderFooterGroupLoadOptions;
            /**
             *
             * Gets or sets the worksheet's black and white print option.
             *
             * [Api set: ExcelApi 1.9]
             */
            blackAndWhite?: boolean;
            /**
             *
             * Gets or sets the worksheet's bottom page margin to use for printing in points.
             *
             * [Api set: ExcelApi 1.9]
             */
            bottomMargin?: boolean;
            /**
             *
             * Gets or sets the worksheet's center horizontally flag. This flag determines whether the worksheet will be centered horizontally when it's printed.
             *
             * [Api set: ExcelApi 1.9]
             */
            centerHorizontally?: boolean;
            /**
             *
             * Gets or sets the worksheet's center vertically flag. This flag determines whether the worksheet will be centered vertically when it's printed.
             *
             * [Api set: ExcelApi 1.9]
             */
            centerVertically?: boolean;
            /**
             *
             * Gets or sets the worksheet's draft mode option. If true the sheet will be printed without graphics.
             *
             * [Api set: ExcelApi 1.9]
             */
            draftMode?: boolean;
            /**
             *
             * Gets or sets the worksheet's first page number to print. Null value represents "auto" page numbering.
             *
             * [Api set: ExcelApi 1.9]
             */
            firstPageNumber?: boolean;
            /**
             *
             * Gets or sets the worksheet's footer margin, in points, for use when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            footerMargin?: boolean;
            /**
             *
             * Gets or sets the worksheet's header margin, in points, for use when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            headerMargin?: boolean;
            /**
             *
             * Gets or sets the worksheet's left margin, in points, for use when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            leftMargin?: boolean;
            /**
             *
             * Gets or sets the worksheet's orientation of the page.
             *
             * [Api set: ExcelApi 1.9]
             */
            orientation?: boolean;
            /**
             *
             * Gets or sets the worksheet's paper size of the page.
             *
             * [Api set: ExcelApi 1.9]
             */
            paperSize?: boolean;
            /**
             *
             * Gets or sets whether the worksheet's comments should be displayed when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            printComments?: boolean;
            /**
             *
             * Gets or sets the worksheet's print errors option.
             *
             * [Api set: ExcelApi 1.9]
             */
            printErrors?: boolean;
            /**
             *
             * Gets or sets the worksheet's print gridlines flag. This flag determines whether gridlines will be printed or not.
             *
             * [Api set: ExcelApi 1.9]
             */
            printGridlines?: boolean;
            /**
             *
             * Gets or sets the worksheet's print headings flag. This flag determines whether headings will be printed or not.
             *
             * [Api set: ExcelApi 1.9]
             */
            printHeadings?: boolean;
            /**
             *
             * Gets or sets the worksheet's page print order option. This specifies the order to use for processing the page number printed.
             *
             * [Api set: ExcelApi 1.9]
             */
            printOrder?: boolean;
            /**
             *
             * Gets or sets the worksheet's right margin, in points, for use when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            rightMargin?: boolean;
            /**
             *
             * Gets or sets the worksheet's top margin, in points, for use when printing.
             *
             * [Api set: ExcelApi 1.9]
             */
            topMargin?: boolean;
            /**
             *
             * Gets or sets the worksheet's print zoom options.
             *
             * [Api set: ExcelApi 1.9]
             */
            zoom?: boolean;
        }
        /**
         * [Api set: ExcelApi 1.9]
         */
        interface HeaderFooterLoadOptions {
            $all?: boolean;
            /**
             *
             * Gets or sets the center footer of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            centerFooter?: boolean;
            /**
             *
             * Gets or sets the center header of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            centerHeader?: boolean;
            /**
             *
             * Gets or sets the left footer of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            leftFooter?: boolean;
            /**
             *
             * Gets or sets the left header of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            leftHeader?: boolean;
            /**
             *
             * Gets or sets the right footer of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            rightFooter?: boolean;
            /**
             *
             * Gets or sets the right header of the worksheet.
            To apply font formatting or insert a variable value, use format codes specified here: https://msdn.microsoft.com/en-us/library/bb225426.aspx.
             *
             * [Api set: ExcelApi 1.9]
             */
            rightHeader?: boolean;
        }
        /**
         * [Api set: ExcelApi 1.9]
         */
        interface HeaderFooterGroupLoadOptions {
            $all?: boolean;
            /**
            *
            * The general header/footer, used for all pages unless even/odd or first page is specified.
            *
            * [Api set: ExcelApi 1.9]
            */
            defaultForAllPages?: Excel.Interfaces.HeaderFooterLoadOptions;
            /**
            *
            * The header/footer to use for even pages, odd header/footer needs to be specified for odd pages.
            *
            * [Api set: ExcelApi 1.9]
            */
            evenPages?: Excel.Interfaces.HeaderFooterLoadOptions;
            /**
            *
            * The first page header/footer, for all other pages general or even/odd is used.
            *
            * [Api set: ExcelApi 1.9]
            */
            firstPage?: Excel.Interfaces.HeaderFooterLoadOptions;
            /**
            *
            * The header/footer to use for odd pages, even header/footer needs to be specified for even pages.
            *
            * [Api set: ExcelApi 1.9]
            */
            oddPages?: Excel.Interfaces.HeaderFooterLoadOptions;
            /**
             *
             * Gets or sets the state of which headers/footers are set. See Excel.HeaderFooterState for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            state?: boolean;
            /**
             *
             * Gets or sets a flag indicating if headers/footers are aligned with the page margins set in the page layout options for the worksheet.
             *
             * [Api set: ExcelApi 1.9]
             */
            useSheetMargins?: boolean;
            /**
             *
             * Gets or sets a flag indicating if headers/footers should be scaled by the page percentage scale set in the page layout options for the worksheet.
             *
             * [Api set: ExcelApi 1.9]
             */
            useSheetScale?: boolean;
        }
        /**
         * [Api set: ExcelApi 1.9]
         */
        interface PageBreakLoadOptions {
            $all?: boolean;
            /**
             *
             * Represents the column index for the page break
             *
             * [Api set: ExcelApi 1.9]
             */
            columnIndex?: boolean;
            /**
             *
             * Represents the row index for the page break
             *
             * [Api set: ExcelApi 1.9]
             */
            rowIndex?: boolean;
        }
        /**
         * [Api set: ExcelApi 1.9]
         */
        interface PageBreakCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the column index for the page break
             *
             * [Api set: ExcelApi 1.9]
             */
            columnIndex?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the row index for the page break
             *
             * [Api set: ExcelApi 1.9]
             */
            rowIndex?: boolean;
        }
        /**
         * [Api set: ExcelApi 1.9]
         */
        interface RangeCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Returns a data validation object.
            *
            * [Api set: ExcelApi 1.8]
            */
            dataValidation?: Excel.Interfaces.DataValidationLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns a format object, encapsulating the range's font, fill, borders, alignment, and other properties.
            *
            * [Api set: ExcelApi 1.1]
            */
            format?: Excel.Interfaces.RangeFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: The worksheet containing the current range.
            *
            * [Api set: ExcelApi 1.1]
            */
            worksheet?: Excel.Interfaces.WorksheetLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Represents the range reference in A1-style. Address value will contain the Sheet reference (e.g. "Sheet1!A1:B4"). Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            address?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents range reference for the specified range in the language of the user. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            addressLocal?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Number of cells in the range. This API will return -1 if the cell count exceeds 2^31-1 (2,147,483,647). Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            cellCount?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the total number of columns in the range. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            columnCount?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents if all columns of the current range are hidden.
             *
             * [Api set: ExcelApi 1.2]
             */
            columnHidden?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the column number of the first cell in the range. Zero-indexed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            columnIndex?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the formula in A1-style notation.
            When setting formulas to a range, the value argument can be either a single value (a string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            formulas?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the formula in A1-style notation, in the user's language and number-formatting locale.  For example, the English "=SUM(A1, 1.5)" formula would become "=SUMME(A1; 1,5)" in German.
            When setting formulas to a range, the value argument can be either a single value (a string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            formulasLocal?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the formula in R1C1-style notation.
            When setting formulas to a range, the value argument can be either a single value (a string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.2]
             */
            formulasR1C1?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents if all cells of the current range are hidden. Read-only.
             *
             * [Api set: ExcelApi 1.2]
             */
            hidden?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the hyperlink for the current range.
             *
             * [Api set: ExcelApi 1.7]
             */
            hyperlink?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents if the current range is an entire column. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            isEntireColumn?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents if the current range is an entire row. Read-only.
             *
             * [Api set: ExcelApi 1.7]
             */
            isEntireRow?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the data type state of each cell. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            linkedDataTypeState?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents Excel's number format code for the given range.
            When setting number format to a range, the value argument can be either a single value (string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            numberFormat?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents Excel's number format code for the given range as a string in the language of the user.
            When setting number format local to a range, the value argument can be either a single value (string) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.7]
             */
            numberFormatLocal?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the total number of rows in the range. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            rowCount?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents if all rows of the current range are hidden.
             *
             * [Api set: ExcelApi 1.2]
             */
            rowHidden?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the row number of the first cell in the range. Zero-indexed. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            rowIndex?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the style of the current range.
            If the styles of the cells are inconsistent, null will be returned.
            For custom styles, the style name will be returned. For built-in styles, a string representing a value in the BuiltInStyle enum will be returned.
             *
             * [Api set: ExcelApi 1.7]
             */
            style?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Text values of the specified range. The Text value will not depend on the cell width. The # sign substitution that happens in Excel UI will not affect the text value returned by the API. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            text?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the type of data of each cell. Read-only.
             *
             * [Api set: ExcelApi 1.1]
             */
            valueTypes?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the raw values of the specified range. The data returned could be of type string, number, or a boolean. Cells that contain an error will return the error string.
            When setting values to a range, the value argument can be either a single value (string, number or boolean) or a two-dimensional array. If the argument is a single value, it will be applied to all cells in the range.
             *
             * [Api set: ExcelApi 1.1]
             */
            values?: boolean;
        }
        /**
         *
         * Represents a collection of all the shapes in the worksheet.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface ShapeCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Returns the fill formatting of this shape.
            *
            * [Api set: ExcelApi 1.9]
            */
            fill?: Excel.Interfaces.ShapeFillLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the geometric shape associated with the shape. An error will be thrown if the shape type is not "GeometricShape".
            *
            * [Api set: ExcelApi 1.9]
            */
            geometricShape?: Excel.Interfaces.GeometricShapeLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the shape group associated with the shape. An error will be thrown if the shape type is not "GroupShape".
            *
            * [Api set: ExcelApi 1.9]
            */
            group?: Excel.Interfaces.ShapeGroupLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the image associated with the shape. An error will be thrown if the shape type is not "Image".
            *
            * [Api set: ExcelApi 1.9]
            */
            image?: Excel.Interfaces.ImageLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the line associated with the shape. An error will be thrown if the shape type is not "Line".
            *
            * [Api set: ExcelApi 1.9]
            */
            line?: Excel.Interfaces.LineLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the line formatting of this shape.
            *
            * [Api set: ExcelApi 1.9]
            */
            lineFormat?: Excel.Interfaces.ShapeLineFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents the parent group of this shape.
            *
            * [Api set: ExcelApi 1.9]
            */
            parentGroup?: Excel.Interfaces.ShapeLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the text frame object of this shape. Read only.
            *
            * [Api set: ExcelApi 1.9]
            */
            textFrame?: Excel.Interfaces.TextFrameLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the alternative description text for a Shape object.
             *
             * [Api set: ExcelApi 1.9]
             */
            altTextDescription?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the alternative title text for a Shape object.
             *
             * [Api set: ExcelApi 1.9]
             */
            altTextTitle?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the number of connection sites on this shape. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            connectionSiteCount?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the geometric shape type of this geometric shape. See Excel.GeometricShapeType for details. Returns null if the shape type is not "GeometricShape".
             *
             * [Api set: ExcelApi 1.9]
             */
            geometricShapeType?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the height, in points, of the shape.
            Throws an invalid argument exception when set with a negative value or zero as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            height?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the shape identifier. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The distance, in points, from the left side of the shape to the left side of the worksheet.
            Throws an invalid argument exception when set with a negative value as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            left?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the level of the specified shape. For example, a level of 0 means that the shape is not part of any groups, a level of 1 means the shape is part of a top-level group, and a level of 2 means the shape is part of a sub-group of the top level.
             *
             * [Api set: ExcelApi 1.9]
             */
            level?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Specifies whether or not the aspect ratio of this shape is locked.
             *
             * [Api set: ExcelApi 1.9]
             */
            lockAspectRatio?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the name of the shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the rotation, in degrees, of the shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            rotation?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The distance, in points, from the top edge of the shape to the top edge of the worksheet.
            Throws an invalid argument exception when set with a negative value as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            top?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the type of this shape. See Excel.ShapeType for details. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            type?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the visibility of this shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            visible?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the width, in points, of the shape.
            Throws an invalid argument exception when set with a negative value or zero as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            width?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the position of the specified shape in the z-order, with 0 representing the bottom of the order stack. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            zOrderPosition?: boolean;
        }
        /**
         *
         * Represents a generic shape object in the worksheet. A shape could be a geometric shape, a line, a group of shapes, etc.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface ShapeLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns the fill formatting of this shape.
            *
            * [Api set: ExcelApi 1.9]
            */
            fill?: Excel.Interfaces.ShapeFillLoadOptions;
            /**
            *
            * Returns the geometric shape associated with the shape. An error will be thrown if the shape type is not "GeometricShape".
            *
            * [Api set: ExcelApi 1.9]
            */
            geometricShape?: Excel.Interfaces.GeometricShapeLoadOptions;
            /**
            *
            * Returns the shape group associated with the shape. An error will be thrown if the shape type is not "GroupShape".
            *
            * [Api set: ExcelApi 1.9]
            */
            group?: Excel.Interfaces.ShapeGroupLoadOptions;
            /**
            *
            * Returns the image associated with the shape. An error will be thrown if the shape type is not "Image".
            *
            * [Api set: ExcelApi 1.9]
            */
            image?: Excel.Interfaces.ImageLoadOptions;
            /**
            *
            * Returns the line associated with the shape. An error will be thrown if the shape type is not "Line".
            *
            * [Api set: ExcelApi 1.9]
            */
            line?: Excel.Interfaces.LineLoadOptions;
            /**
            *
            * Returns the line formatting of this shape.
            *
            * [Api set: ExcelApi 1.9]
            */
            lineFormat?: Excel.Interfaces.ShapeLineFormatLoadOptions;
            /**
            *
            * Represents the parent group of this shape.
            *
            * [Api set: ExcelApi 1.9]
            */
            parentGroup?: Excel.Interfaces.ShapeLoadOptions;
            /**
            *
            * Returns the text frame object of this shape. Read only.
            *
            * [Api set: ExcelApi 1.9]
            */
            textFrame?: Excel.Interfaces.TextFrameLoadOptions;
            /**
             *
             * Returns or sets the alternative description text for a Shape object.
             *
             * [Api set: ExcelApi 1.9]
             */
            altTextDescription?: boolean;
            /**
             *
             * Returns or sets the alternative title text for a Shape object.
             *
             * [Api set: ExcelApi 1.9]
             */
            altTextTitle?: boolean;
            /**
             *
             * Returns the number of connection sites on this shape. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            connectionSiteCount?: boolean;
            /**
             *
             * Represents the geometric shape type of this geometric shape. See Excel.GeometricShapeType for details. Returns null if the shape type is not "GeometricShape".
             *
             * [Api set: ExcelApi 1.9]
             */
            geometricShapeType?: boolean;
            /**
             *
             * Represents the height, in points, of the shape.
            Throws an invalid argument exception when set with a negative value or zero as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            height?: boolean;
            /**
             *
             * Represents the shape identifier. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            id?: boolean;
            /**
             *
             * The distance, in points, from the left side of the shape to the left side of the worksheet.
            Throws an invalid argument exception when set with a negative value as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            left?: boolean;
            /**
             *
             * Represents the level of the specified shape. For example, a level of 0 means that the shape is not part of any groups, a level of 1 means the shape is part of a top-level group, and a level of 2 means the shape is part of a sub-group of the top level.
             *
             * [Api set: ExcelApi 1.9]
             */
            level?: boolean;
            /**
             *
             * Specifies whether or not the aspect ratio of this shape is locked.
             *
             * [Api set: ExcelApi 1.9]
             */
            lockAspectRatio?: boolean;
            /**
             *
             * Represents the name of the shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            name?: boolean;
            /**
             *
             * Represents the rotation, in degrees, of the shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            rotation?: boolean;
            /**
             *
             * The distance, in points, from the top edge of the shape to the top edge of the worksheet.
            Throws an invalid argument exception when set with a negative value as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            top?: boolean;
            /**
             *
             * Returns the type of this shape. See Excel.ShapeType for details. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            type?: boolean;
            /**
             *
             * Represents the visibility of this shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            visible?: boolean;
            /**
             *
             * Represents the width, in points, of the shape.
            Throws an invalid argument exception when set with a negative value or zero as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            width?: boolean;
            /**
             *
             * Returns the position of the specified shape in the z-order, with 0 representing the bottom of the order stack. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            zOrderPosition?: boolean;
        }
        /**
         *
         * Represents a geometric shape inside a worksheet. A geometric shape can be a rectangle, block arrow, equation symbol, flowchart item, star, banner, callout, or any other basic shape in Excel.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface GeometricShapeLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns the Shape object for the geometric shape.
            *
            * [Api set: ExcelApi 1.9]
            */
            shape?: Excel.Interfaces.ShapeLoadOptions;
            /**
             *
             * Returns the shape identifier. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            id?: boolean;
        }
        /**
         *
         * Represents an image in the worksheet. To get the corresponding Shape object, use Image.shape.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface ImageLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns the Shape object associated with the image.
            *
            * [Api set: ExcelApi 1.9]
            */
            shape?: Excel.Interfaces.ShapeLoadOptions;
            /**
             *
             * Represents the shape identifier for the image object. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            id?: boolean;
            /**
             *
             * Returns the format of the image. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            format?: boolean;
        }
        /**
         *
         * Represents a shape group inside a worksheet. To get the corresponding Shape object, use `ShapeGroup.shape`.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface ShapeGroupLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns the Shape object associated with the group.
            *
            * [Api set: ExcelApi 1.9]
            */
            shape?: Excel.Interfaces.ShapeLoadOptions;
            /**
             *
             * Represents the shape identifier. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            id?: boolean;
        }
        /**
         *
         * Represents the shape collection inside a shape group.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface GroupShapeCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Returns the fill formatting of this shape.
            *
            * [Api set: ExcelApi 1.9]
            */
            fill?: Excel.Interfaces.ShapeFillLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the geometric shape associated with the shape. An error will be thrown if the shape type is not "GeometricShape".
            *
            * [Api set: ExcelApi 1.9]
            */
            geometricShape?: Excel.Interfaces.GeometricShapeLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the shape group associated with the shape. An error will be thrown if the shape type is not "GroupShape".
            *
            * [Api set: ExcelApi 1.9]
            */
            group?: Excel.Interfaces.ShapeGroupLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the image associated with the shape. An error will be thrown if the shape type is not "Image".
            *
            * [Api set: ExcelApi 1.9]
            */
            image?: Excel.Interfaces.ImageLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the line associated with the shape. An error will be thrown if the shape type is not "Line".
            *
            * [Api set: ExcelApi 1.9]
            */
            line?: Excel.Interfaces.LineLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the line formatting of this shape.
            *
            * [Api set: ExcelApi 1.9]
            */
            lineFormat?: Excel.Interfaces.ShapeLineFormatLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Represents the parent group of this shape.
            *
            * [Api set: ExcelApi 1.9]
            */
            parentGroup?: Excel.Interfaces.ShapeLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Returns the text frame object of this shape. Read only.
            *
            * [Api set: ExcelApi 1.9]
            */
            textFrame?: Excel.Interfaces.TextFrameLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the alternative description text for a Shape object.
             *
             * [Api set: ExcelApi 1.9]
             */
            altTextDescription?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns or sets the alternative title text for a Shape object.
             *
             * [Api set: ExcelApi 1.9]
             */
            altTextTitle?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the number of connection sites on this shape. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            connectionSiteCount?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the geometric shape type of this geometric shape. See Excel.GeometricShapeType for details. Returns null if the shape type is not "GeometricShape".
             *
             * [Api set: ExcelApi 1.9]
             */
            geometricShapeType?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the height, in points, of the shape.
            Throws an invalid argument exception when set with a negative value or zero as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            height?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the shape identifier. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The distance, in points, from the left side of the shape to the left side of the worksheet.
            Throws an invalid argument exception when set with a negative value as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            left?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the level of the specified shape. For example, a level of 0 means that the shape is not part of any groups, a level of 1 means the shape is part of a top-level group, and a level of 2 means the shape is part of a sub-group of the top level.
             *
             * [Api set: ExcelApi 1.9]
             */
            level?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Specifies whether or not the aspect ratio of this shape is locked.
             *
             * [Api set: ExcelApi 1.9]
             */
            lockAspectRatio?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the name of the shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the rotation, in degrees, of the shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            rotation?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The distance, in points, from the top edge of the shape to the top edge of the worksheet.
            Throws an invalid argument exception when set with a negative value as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            top?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the type of this shape. See Excel.ShapeType for details. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            type?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the visibility of this shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            visible?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Represents the width, in points, of the shape.
            Throws an invalid argument exception when set with a negative value or zero as input.
             *
             * [Api set: ExcelApi 1.9]
             */
            width?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the position of the specified shape in the z-order, with 0 representing the bottom of the order stack. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            zOrderPosition?: boolean;
        }
        /**
         *
         * Represents a line inside a worksheet. To get the corresponding Shape object, use `Line.shape`.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface LineLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the shape to which the beginning of the specified line is attached.
            *
            * [Api set: ExcelApi 1.9]
            */
            beginConnectedShape?: Excel.Interfaces.ShapeLoadOptions;
            /**
            *
            * Represents the shape to which the end of the specified line is attached.
            *
            * [Api set: ExcelApi 1.9]
            */
            endConnectedShape?: Excel.Interfaces.ShapeLoadOptions;
            /**
            *
            * Returns the Shape object associated with the line.
            *
            * [Api set: ExcelApi 1.9]
            */
            shape?: Excel.Interfaces.ShapeLoadOptions;
            /**
             *
             * Represents the length of the arrowhead at the beginning of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            beginArrowheadLength?: boolean;
            /**
             *
             * Represents the style of the arrowhead at the beginning of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            beginArrowheadStyle?: boolean;
            /**
             *
             * Represents the width of the arrowhead at the beginning of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            beginArrowheadWidth?: boolean;
            /**
             *
             * Represents the connection site to which the beginning of a connector is connected. Read-only. Returns null when the beginning of the line is not attached to any shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            beginConnectedSite?: boolean;
            /**
             *
             * Represents the length of the arrowhead at the end of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            endArrowheadLength?: boolean;
            /**
             *
             * Represents the style of the arrowhead at the end of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            endArrowheadStyle?: boolean;
            /**
             *
             * Represents the width of the arrowhead at the end of the specified line.
             *
             * [Api set: ExcelApi 1.9]
             */
            endArrowheadWidth?: boolean;
            /**
             *
             * Represents the connection site to which the end of a connector is connected. Read-only. Returns null when the end of the line is not attached to any shape.
             *
             * [Api set: ExcelApi 1.9]
             */
            endConnectedSite?: boolean;
            /**
             *
             * Represents the shape identifier. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            id?: boolean;
            /**
             *
             * Specifies whether or not the beginning of the specified line is connected to a shape. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            isBeginConnected?: boolean;
            /**
             *
             * Specifies whether or not the end of the specified line is connected to a shape. Read-only.
             *
             * [Api set: ExcelApi 1.9]
             */
            isEndConnected?: boolean;
            /**
             *
             * Represents the connector type for the line.
             *
             * [Api set: ExcelApi 1.9]
             */
            connectorType?: boolean;
        }
        /**
         *
         * Represents the fill formatting of a shape object.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface ShapeFillLoadOptions {
            $all?: boolean;
            /**
             *
             * Represents the shape fill foreground color in HTML color format, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange")
             *
             * [Api set: ExcelApi 1.9]
             */
            foregroundColor?: boolean;
            /**
             *
             * Returns or sets the transparency percentage of the fill as a value from 0.0 (opaque) through 1.0 (clear). Returns null if the shape type does not support transparency or the shape fill has inconsistent transparency, such as with a gradient fill type.
             *
             * [Api set: ExcelApi 1.9]
             */
            transparency?: boolean;
            /**
             *
             * Returns the fill type of the shape. Read-only. See Excel.ShapeFillType for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            type?: boolean;
        }
        /**
         *
         * Represents the line formatting for the shape object. For images and geometric shapes, line formatting represents the border of the shape.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface ShapeLineFormatLoadOptions {
            $all?: boolean;
            /**
             *
             * Represents the line color in HTML color format, of the form #RRGGBB (e.g. "FFA500") or as a named HTML color (e.g. "orange").
             *
             * [Api set: ExcelApi 1.9]
             */
            color?: boolean;
            /**
             *
             * Represents the line style of the shape. Returns null when the line is not visible or there are inconsistent dash styles. See Excel.ShapeLineStyle for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            dashStyle?: boolean;
            /**
             *
             * Represents the line style of the shape. Returns null when the line is not visible or there are inconsistent styles. See Excel.ShapeLineStyle for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            style?: boolean;
            /**
             *
             * Represents the degree of transparency of the specified line as a value from 0.0 (opaque) through 1.0 (clear). Returns null when the shape has inconsistent transparencies.
             *
             * [Api set: ExcelApi 1.9]
             */
            transparency?: boolean;
            /**
             *
             * Represents whether or not the line formatting of a shape element is visible. Returns null when the shape has inconsistent visibilities.
             *
             * [Api set: ExcelApi 1.9]
             */
            visible?: boolean;
            /**
             *
             * Represents the weight of the line, in points. Returns null when the line is not visible or there are inconsistent line weights.
             *
             * [Api set: ExcelApi 1.9]
             */
            weight?: boolean;
        }
        /**
         *
         * Represents the text frame of a shape object.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface TextFrameLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents the text that is attached to a shape in the text frame, and properties and methods for manipulating the text. See Excel.TextRange for details.
            *
            * [Api set: ExcelApi 1.9]
            */
            textRange?: Excel.Interfaces.TextRangeLoadOptions;
            /**
             *
             * Gets or sets the automatic sizing settings for the text frame. A text frame can be set to automatically fit the text to the text frame, to automatically fit the text frame to the text, or not perform any automatic sizing.
             *
             * [Api set: ExcelApi 1.9]
             */
            autoSizeSetting?: boolean;
            /**
             *
             * Represents the bottom margin, in points, of the text frame.
             *
             * [Api set: ExcelApi 1.9]
             */
            bottomMargin?: boolean;
            /**
             *
             * Specifies whether the text frame contains text.
             *
             * [Api set: ExcelApi 1.9]
             */
            hasText?: boolean;
            /**
             *
             * Represents the horizontal alignment of the text frame. See Excel.ShapeTextHorizontalAlignment for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            horizontalAlignment?: boolean;
            /**
             *
             * Represents the horizontal overflow behavior of the text frame. See Excel.ShapeTextHorizontalOverflow for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            horizontalOverflow?: boolean;
            /**
             *
             * Represents the left margin, in points, of the text frame.
             *
             * [Api set: ExcelApi 1.9]
             */
            leftMargin?: boolean;
            /**
             *
             * Represents the text orientation of the text frame. See Excel.ShapeTextOrientation for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            orientation?: boolean;
            /**
             *
             * Represents the reading order of the text frame, either left-to-right or right-to-left. See Excel.ShapeTextReadingOrder for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            readingOrder?: boolean;
            /**
             *
             * Represents the right margin, in points, of the text frame.
             *
             * [Api set: ExcelApi 1.9]
             */
            rightMargin?: boolean;
            /**
             *
             * Represents the top margin, in points, of the text frame.
             *
             * [Api set: ExcelApi 1.9]
             */
            topMargin?: boolean;
            /**
             *
             * Represents the vertical alignment of the text frame. See Excel.ShapeTextVerticalAlignment for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            verticalAlignment?: boolean;
            /**
             *
             * Represents the vertical overflow behavior of the text frame. See Excel.ShapeTextVerticalOverflow for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            verticalOverflow?: boolean;
        }
        /**
         *
         * Contains the text that is attached to a shape, in addition to properties and methods for manipulating the text.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface TextRangeLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns a ShapeFont object that represents the font attributes for the text range.
            *
            * [Api set: ExcelApi 1.9]
            */
            font?: Excel.Interfaces.ShapeFontLoadOptions;
            /**
             *
             * Represents the plain text content of the text range.
             *
             * [Api set: ExcelApi 1.9]
             */
            text?: boolean;
        }
        /**
         *
         * Represents the font attributes, such as font name, font size, and color, for a shape's TextRange object.
         *
         * [Api set: ExcelApi 1.9]
         */
        interface ShapeFontLoadOptions {
            $all?: boolean;
            /**
             *
             * Represents the bold status of font. Returns null the TextRange includes both bold and non-bold text fragments.
             *
             * [Api set: ExcelApi 1.9]
             */
            bold?: boolean;
            /**
             *
             * The HTML color code representation of the text color (e.g. "#FF0000" represents red). Returns null if the TextRange includes text fragments with different colors.
             *
             * [Api set: ExcelApi 1.9]
             */
            color?: boolean;
            /**
             *
             * Represents the italic status of font. Returns null if the TextRange includes both italic and non-italic text fragments.
             *
             * [Api set: ExcelApi 1.9]
             */
            italic?: boolean;
            /**
             *
             * Represents font name (e.g. "Calibri"). If the text is Complex Script or East Asian language, this is the corresponding font name; otherwise it is the Latin font name.
             *
             * [Api set: ExcelApi 1.9]
             */
            name?: boolean;
            /**
             *
             * Represents font size in points (e.g. 11). Returns null if the TextRange includes text fragments with different font sizes.
             *
             * [Api set: ExcelApi 1.9]
             */
            size?: boolean;
            /**
             *
             * Type of underline applied to the font. Returns null if the TextRange includes text fragments with different underline styles. See Excel.ShapeFontUnderlineStyle for details.
             *
             * [Api set: ExcelApi 1.9]
             */
            underline?: boolean;
        }
        /**
         *
         * An object containing the result of a function-evaluation operation
         *
         * [Api set: ExcelApi 1.2]
         */
        interface FunctionResultLoadOptions {
            $all?: boolean;
            /**
             *
             * Error value (such as "#DIV/0") representing the error. If the error string is not set, then the function succeeded, and its result is written to the Value field. The error is always in the English locale.
             *
             * [Api set: ExcelApi 1.2]
             */
            error?: boolean;
            /**
             *
             * The value of function evaluation. The value field will be populated only if no error has occurred (i.e., the Error property is not set).
             *
             * [Api set: ExcelApi 1.2]
             */
            value?: boolean;
        }
    }
}


////////////////////////////////////////////////////////////////
//////////////////////// End Excel APIs ////////////////////////
////////////////////////////////////////////////////////////////





////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////
/////////////////////// Begin Word APIs ////////////////////////
////////////////////////////////////////////////////////////////

declare namespace Word {
    /**
     *
     * Represents the application object.
     *
     * [Api set: WordApi 1.3]
     */
    class Application extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Creates a new document by using an optional base64 encoded .docx file.
         *
         * [Api set: WordApi 1.3]
         *
         * @param base64File Optional. The base64 encoded .docx file. The default value is null.
         */
        createDocument(base64File?: string): Word.DocumentCreated;
        /**
         * Create a new instance of Word.Application object
         */
        static newObject(context: OfficeExtension.ClientRequestContext): Word.Application;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.Application object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.ApplicationData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): {
            [key: string]: string;
        };
    }
    /**
     *
     * Represents the body of a document or a section.
     *
     * [Api set: WordApi 1.1]
     */
    class Body extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the collection of rich text content control objects in the body. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly contentControls: Word.ContentControlCollection;
        /**
         *
         * Gets the text format of the body. Use this to get and set font name, size, color and other properties. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly font: Word.Font;
        /**
         *
         * Gets the collection of InlinePicture objects in the body. The collection does not include floating images. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly inlinePictures: Word.InlinePictureCollection;
        /**
         *
         * Gets the collection of list objects in the body. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly lists: Word.ListCollection;
        /**
         *
         * Gets the collection of paragraph objects in the body. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly paragraphs: Word.ParagraphCollection;
        /**
         *
         * Gets the parent body of the body. For example, a table cell body's parent body could be a header. Throws if there isn't a parent body. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentBody: Word.Body;
        /**
         *
         * Gets the parent body of the body. For example, a table cell body's parent body could be a header. Returns a null object if there isn't a parent body. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentBodyOrNullObject: Word.Body;
        /**
         *
         * Gets the content control that contains the body. Throws if there isn't a parent content control. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly parentContentControl: Word.ContentControl;
        /**
         *
         * Gets the content control that contains the body. Returns a null object if there isn't a parent content control. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentContentControlOrNullObject: Word.ContentControl;
        /**
         *
         * Gets the parent section of the body. Throws if there isn't a parent section. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentSection: Word.Section;
        /**
         *
         * Gets the parent section of the body. Returns a null object if there isn't a parent section. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentSectionOrNullObject: Word.Section;
        /**
         *
         * Gets the collection of table objects in the body. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly tables: Word.TableCollection;
        /**
         *
         * Gets or sets the style name for the body. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
         *
         * [Api set: WordApi 1.1]
         */
        style: string;
        /**
         *
         * Gets or sets the built-in style name for the body. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
         *
         * [Api set: WordApi 1.3]
         */
        styleBuiltIn: Word.Style | "Other" | "Normal" | "Heading1" | "Heading2" | "Heading3" | "Heading4" | "Heading5" | "Heading6" | "Heading7" | "Heading8" | "Heading9" | "Toc1" | "Toc2" | "Toc3" | "Toc4" | "Toc5" | "Toc6" | "Toc7" | "Toc8" | "Toc9" | "FootnoteText" | "Header" | "Footer" | "Caption" | "FootnoteReference" | "EndnoteReference" | "EndnoteText" | "Title" | "Subtitle" | "Hyperlink" | "Strong" | "Emphasis" | "NoSpacing" | "ListParagraph" | "Quote" | "IntenseQuote" | "SubtleEmphasis" | "IntenseEmphasis" | "SubtleReference" | "IntenseReference" | "BookTitle" | "Bibliography" | "TocHeading" | "TableGrid" | "PlainTable1" | "PlainTable2" | "PlainTable3" | "PlainTable4" | "PlainTable5" | "TableGridLight" | "GridTable1Light" | "GridTable1Light_Accent1" | "GridTable1Light_Accent2" | "GridTable1Light_Accent3" | "GridTable1Light_Accent4" | "GridTable1Light_Accent5" | "GridTable1Light_Accent6" | "GridTable2" | "GridTable2_Accent1" | "GridTable2_Accent2" | "GridTable2_Accent3" | "GridTable2_Accent4" | "GridTable2_Accent5" | "GridTable2_Accent6" | "GridTable3" | "GridTable3_Accent1" | "GridTable3_Accent2" | "GridTable3_Accent3" | "GridTable3_Accent4" | "GridTable3_Accent5" | "GridTable3_Accent6" | "GridTable4" | "GridTable4_Accent1" | "GridTable4_Accent2" | "GridTable4_Accent3" | "GridTable4_Accent4" | "GridTable4_Accent5" | "GridTable4_Accent6" | "GridTable5Dark" | "GridTable5Dark_Accent1" | "GridTable5Dark_Accent2" | "GridTable5Dark_Accent3" | "GridTable5Dark_Accent4" | "GridTable5Dark_Accent5" | "GridTable5Dark_Accent6" | "GridTable6Colorful" | "GridTable6Colorful_Accent1" | "GridTable6Colorful_Accent2" | "GridTable6Colorful_Accent3" | "GridTable6Colorful_Accent4" | "GridTable6Colorful_Accent5" | "GridTable6Colorful_Accent6" | "GridTable7Colorful" | "GridTable7Colorful_Accent1" | "GridTable7Colorful_Accent2" | "GridTable7Colorful_Accent3" | "GridTable7Colorful_Accent4" | "GridTable7Colorful_Accent5" | "GridTable7Colorful_Accent6" | "ListTable1Light" | "ListTable1Light_Accent1" | "ListTable1Light_Accent2" | "ListTable1Light_Accent3" | "ListTable1Light_Accent4" | "ListTable1Light_Accent5" | "ListTable1Light_Accent6" | "ListTable2" | "ListTable2_Accent1" | "ListTable2_Accent2" | "ListTable2_Accent3" | "ListTable2_Accent4" | "ListTable2_Accent5" | "ListTable2_Accent6" | "ListTable3" | "ListTable3_Accent1" | "ListTable3_Accent2" | "ListTable3_Accent3" | "ListTable3_Accent4" | "ListTable3_Accent5" | "ListTable3_Accent6" | "ListTable4" | "ListTable4_Accent1" | "ListTable4_Accent2" | "ListTable4_Accent3" | "ListTable4_Accent4" | "ListTable4_Accent5" | "ListTable4_Accent6" | "ListTable5Dark" | "ListTable5Dark_Accent1" | "ListTable5Dark_Accent2" | "ListTable5Dark_Accent3" | "ListTable5Dark_Accent4" | "ListTable5Dark_Accent5" | "ListTable5Dark_Accent6" | "ListTable6Colorful" | "ListTable6Colorful_Accent1" | "ListTable6Colorful_Accent2" | "ListTable6Colorful_Accent3" | "ListTable6Colorful_Accent4" | "ListTable6Colorful_Accent5" | "ListTable6Colorful_Accent6" | "ListTable7Colorful" | "ListTable7Colorful_Accent1" | "ListTable7Colorful_Accent2" | "ListTable7Colorful_Accent3" | "ListTable7Colorful_Accent4" | "ListTable7Colorful_Accent5" | "ListTable7Colorful_Accent6";
        /**
         *
         * Gets the text of the body. Use the insertText method to insert text. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly text: string;
        /**
         *
         * Gets the type of the body. The type can be 'MainDoc', 'Section', 'Header', 'Footer', or 'TableCell'. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly type: Word.BodyType | "Unknown" | "MainDoc" | "Section" | "Header" | "Footer" | "TableCell";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.Body): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.BodyUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.Body): void;
        /**
         *
         * Clears the contents of the body object. The user can perform the undo operation on the cleared content.
         *
         * [Api set: WordApi 1.1]
         */
        clear(): void;
        /**
         *
         * Gets an HTML representation of the body object. When rendered in a web page or HTML viewer, the formatting will be a close, but not exact, match for of the formatting of the document. This method does not return the exact same HTML for the same document on different platforms (Windows, Mac, Word Online, etc.). If you need exact fidelity, or consistency across platforms, use `Body.getOoxml()` and convert the returned XML to HTML.
         *
         * [Api set: WordApi 1.1]
         */
        getHtml(): OfficeExtension.ClientResult<string>;
        /**
         *
         * Gets the OOXML (Office Open XML) representation of the body object.
         *
         * [Api set: WordApi 1.1]
         */
        getOoxml(): OfficeExtension.ClientResult<string>;
        /**
         *
         * Gets the whole body, or the starting or ending point of the body, as a range.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rangeLocation Optional. The range location can be 'Whole', 'Start', 'End', 'After', or 'Content'.
         */
        getRange(rangeLocation?: Word.RangeLocation): Word.Range;
        /**
         *
         * Gets the whole body, or the starting or ending point of the body, as a range.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rangeLocation Optional. The range location can be 'Whole', 'Start', 'End', 'After', or 'Content'.
         */
        getRange(rangeLocation?: "Whole" | "Start" | "End" | "Before" | "After" | "Content"): Word.Range;
        /**
         *
         * Inserts a break at the specified location in the main document. The insertLocation value can be 'Start' or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param breakType Required. The break type to add to the body.
         * @param insertLocation Required. The value can be 'Start' or 'End'.
         */
        insertBreak(breakType: Word.BreakType, insertLocation: Word.InsertLocation): void;
        /**
         *
         * Inserts a break at the specified location in the main document. The insertLocation value can be 'Start' or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param breakType Required. The break type to add to the body.
         * @param insertLocation Required. The value can be 'Start' or 'End'.
         */
        insertBreak(breakType: "Page" | "Next" | "SectionNext" | "SectionContinuous" | "SectionEven" | "SectionOdd" | "Line", insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): void;
        /**
         *
         * Wraps the body object with a Rich Text content control.
         *
         * [Api set: WordApi 1.1]
         */
        insertContentControl(): Word.ContentControl;
        /**
         *
         * Inserts a document into the body at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param base64File Required. The base64 encoded content of a .docx file.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertFileFromBase64(base64File: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts a document into the body at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param base64File Required. The base64 encoded content of a .docx file.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertFileFromBase64(base64File: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Inserts HTML at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param html Required. The HTML to be inserted in the document.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertHtml(html: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts HTML at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param html Required. The HTML to be inserted in the document.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertHtml(html: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Inserts a picture into the body at the specified location. The insertLocation value can be 'Start' or 'End'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param base64EncodedImage Required. The base64 encoded image to be inserted in the body.
         * @param insertLocation Required. The value can be 'Start' or 'End'.
         */
        insertInlinePictureFromBase64(base64EncodedImage: string, insertLocation: Word.InsertLocation): Word.InlinePicture;
        /**
         *
         * Inserts a picture into the body at the specified location. The insertLocation value can be 'Start' or 'End'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param base64EncodedImage Required. The base64 encoded image to be inserted in the body.
         * @param insertLocation Required. The value can be 'Start' or 'End'.
         */
        insertInlinePictureFromBase64(base64EncodedImage: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.InlinePicture;
        /**
         *
         * Inserts OOXML at the specified location.  The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param ooxml Required. The OOXML to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertOoxml(ooxml: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts OOXML at the specified location.  The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param ooxml Required. The OOXML to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertOoxml(ooxml: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Inserts a paragraph at the specified location. The insertLocation value can be 'Start' or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param paragraphText Required. The paragraph text to be inserted.
         * @param insertLocation Required. The value can be 'Start' or 'End'.
         */
        insertParagraph(paragraphText: string, insertLocation: Word.InsertLocation): Word.Paragraph;
        /**
         *
         * Inserts a paragraph at the specified location. The insertLocation value can be 'Start' or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param paragraphText Required. The paragraph text to be inserted.
         * @param insertLocation Required. The value can be 'Start' or 'End'.
         */
        insertParagraph(paragraphText: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Paragraph;
        /**
         *
         * Inserts a table with the specified number of rows and columns. The insertLocation value can be 'Start' or 'End'.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rowCount Required. The number of rows in the table.
         * @param columnCount Required. The number of columns in the table.
         * @param insertLocation Required. The value can be 'Start' or 'End'.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertTable(rowCount: number, columnCount: number, insertLocation: Word.InsertLocation, values?: string[][]): Word.Table;
        /**
         *
         * Inserts a table with the specified number of rows and columns. The insertLocation value can be 'Start' or 'End'.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rowCount Required. The number of rows in the table.
         * @param columnCount Required. The number of columns in the table.
         * @param insertLocation Required. The value can be 'Start' or 'End'.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertTable(rowCount: number, columnCount: number, insertLocation: "Before" | "After" | "Start" | "End" | "Replace", values?: string[][]): Word.Table;
        /**
         *
         * Inserts text into the body at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param text Required. Text to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertText(text: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts text into the body at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param text Required. Text to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertText(text: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Performs a search with the specified SearchOptions on the scope of the body object. The search results are a collection of range objects.
         *
         * [Api set: WordApi 1.1]
         *
         * @param searchText Required. The search text. Can be a maximum of 255 characters.
         * @param searchOptions Optional. Options for the search.
         */
        search(searchText: string, searchOptions?: Word.SearchOptions | {
            ignorePunct?: boolean;
            ignoreSpace?: boolean;
            matchCase?: boolean;
            matchPrefix?: boolean;
            matchSuffix?: boolean;
            matchWholeWord?: boolean;
            matchWildcards?: boolean;
        }): Word.RangeCollection;
        /**
         *
         * Selects the body and navigates the Word UI to it.
         *
         * [Api set: WordApi 1.1]
         *
         * @param selectionMode Optional. The selection mode can be 'Select', 'Start', or 'End'. 'Select' is the default.
         */
        select(selectionMode?: Word.SelectionMode): void;
        /**
         *
         * Selects the body and navigates the Word UI to it.
         *
         * [Api set: WordApi 1.1]
         *
         * @param selectionMode Optional. The selection mode can be 'Select', 'Start', or 'End'. 'Select' is the default.
         */
        select(selectionMode?: "Select" | "Start" | "End"): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.Body` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.Body` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.Body` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.BodyLoadOptions): Word.Body;
        load(option?: string | string[]): Word.Body;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.Body;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.Body;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.Body;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.Body object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.BodyData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.BodyData;
    }
    /**
     *
     * Represents a content control. Content controls are bounded and potentially labeled regions in a document that serve as containers for specific types of content. Individual content controls may contain contents such as images, tables, or paragraphs of formatted text. Currently, only rich text content controls are supported.
     *
     * [Api set: WordApi 1.1]
     */
    class ContentControl extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the collection of content control objects in the content control. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly contentControls: Word.ContentControlCollection;
        /**
         *
         * Gets the text format of the content control. Use this to get and set font name, size, color, and other properties. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly font: Word.Font;
        /**
         *
         * Gets the collection of inlinePicture objects in the content control. The collection does not include floating images. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly inlinePictures: Word.InlinePictureCollection;
        /**
         *
         * Gets the collection of list objects in the content control. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly lists: Word.ListCollection;
        /**
         *
         * Get the collection of paragraph objects in the content control. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly paragraphs: Word.ParagraphCollection;
        /**
         *
         * Gets the parent body of the content control. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentBody: Word.Body;
        /**
         *
         * Gets the content control that contains the content control. Throws if there isn't a parent content control. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly parentContentControl: Word.ContentControl;
        /**
         *
         * Gets the content control that contains the content control. Returns a null object if there isn't a parent content control. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentContentControlOrNullObject: Word.ContentControl;
        /**
         *
         * Gets the table that contains the content control. Throws if it is not contained in a table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTable: Word.Table;
        /**
         *
         * Gets the table cell that contains the content control. Throws if it is not contained in a table cell. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTableCell: Word.TableCell;
        /**
         *
         * Gets the table cell that contains the content control. Returns a null object if it is not contained in a table cell. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTableCellOrNullObject: Word.TableCell;
        /**
         *
         * Gets the table that contains the content control. Returns a null object if it is not contained in a table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTableOrNullObject: Word.Table;
        /**
         *
         * Gets the collection of table objects in the content control. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly tables: Word.TableCollection;
        /**
         *
         * Gets or sets the appearance of the content control. The value can be 'BoundingBox', 'Tags', or 'Hidden'.
         *
         * [Api set: WordApi 1.1]
         */
        appearance: Word.ContentControlAppearance | "BoundingBox" | "Tags" | "Hidden";
        /**
         *
         * Gets or sets a value that indicates whether the user can delete the content control. Mutually exclusive with removeWhenEdited.
         *
         * [Api set: WordApi 1.1]
         */
        cannotDelete: boolean;
        /**
         *
         * Gets or sets a value that indicates whether the user can edit the contents of the content control.
         *
         * [Api set: WordApi 1.1]
         */
        cannotEdit: boolean;
        /**
         *
         * Gets or sets the color of the content control. Color is specified in '#RRGGBB' format or by using the color name.
         *
         * [Api set: WordApi 1.1]
         */
        color: string;
        /**
         *
         * Gets an integer that represents the content control identifier. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly id: number;
        /**
         *
         * Gets or sets the placeholder text of the content control. Dimmed text will be displayed when the content control is empty.
         *
         * [Api set: WordApi 1.1]
         */
        placeholderText: string;
        /**
         *
         * Gets or sets a value that indicates whether the content control is removed after it is edited. Mutually exclusive with cannotDelete.
         *
         * [Api set: WordApi 1.1]
         */
        removeWhenEdited: boolean;
        /**
         *
         * Gets or sets the style name for the content control. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
         *
         * [Api set: WordApi 1.1]
         */
        style: string;
        /**
         *
         * Gets or sets the built-in style name for the content control. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
         *
         * [Api set: WordApi 1.3]
         */
        styleBuiltIn: Word.Style | "Other" | "Normal" | "Heading1" | "Heading2" | "Heading3" | "Heading4" | "Heading5" | "Heading6" | "Heading7" | "Heading8" | "Heading9" | "Toc1" | "Toc2" | "Toc3" | "Toc4" | "Toc5" | "Toc6" | "Toc7" | "Toc8" | "Toc9" | "FootnoteText" | "Header" | "Footer" | "Caption" | "FootnoteReference" | "EndnoteReference" | "EndnoteText" | "Title" | "Subtitle" | "Hyperlink" | "Strong" | "Emphasis" | "NoSpacing" | "ListParagraph" | "Quote" | "IntenseQuote" | "SubtleEmphasis" | "IntenseEmphasis" | "SubtleReference" | "IntenseReference" | "BookTitle" | "Bibliography" | "TocHeading" | "TableGrid" | "PlainTable1" | "PlainTable2" | "PlainTable3" | "PlainTable4" | "PlainTable5" | "TableGridLight" | "GridTable1Light" | "GridTable1Light_Accent1" | "GridTable1Light_Accent2" | "GridTable1Light_Accent3" | "GridTable1Light_Accent4" | "GridTable1Light_Accent5" | "GridTable1Light_Accent6" | "GridTable2" | "GridTable2_Accent1" | "GridTable2_Accent2" | "GridTable2_Accent3" | "GridTable2_Accent4" | "GridTable2_Accent5" | "GridTable2_Accent6" | "GridTable3" | "GridTable3_Accent1" | "GridTable3_Accent2" | "GridTable3_Accent3" | "GridTable3_Accent4" | "GridTable3_Accent5" | "GridTable3_Accent6" | "GridTable4" | "GridTable4_Accent1" | "GridTable4_Accent2" | "GridTable4_Accent3" | "GridTable4_Accent4" | "GridTable4_Accent5" | "GridTable4_Accent6" | "GridTable5Dark" | "GridTable5Dark_Accent1" | "GridTable5Dark_Accent2" | "GridTable5Dark_Accent3" | "GridTable5Dark_Accent4" | "GridTable5Dark_Accent5" | "GridTable5Dark_Accent6" | "GridTable6Colorful" | "GridTable6Colorful_Accent1" | "GridTable6Colorful_Accent2" | "GridTable6Colorful_Accent3" | "GridTable6Colorful_Accent4" | "GridTable6Colorful_Accent5" | "GridTable6Colorful_Accent6" | "GridTable7Colorful" | "GridTable7Colorful_Accent1" | "GridTable7Colorful_Accent2" | "GridTable7Colorful_Accent3" | "GridTable7Colorful_Accent4" | "GridTable7Colorful_Accent5" | "GridTable7Colorful_Accent6" | "ListTable1Light" | "ListTable1Light_Accent1" | "ListTable1Light_Accent2" | "ListTable1Light_Accent3" | "ListTable1Light_Accent4" | "ListTable1Light_Accent5" | "ListTable1Light_Accent6" | "ListTable2" | "ListTable2_Accent1" | "ListTable2_Accent2" | "ListTable2_Accent3" | "ListTable2_Accent4" | "ListTable2_Accent5" | "ListTable2_Accent6" | "ListTable3" | "ListTable3_Accent1" | "ListTable3_Accent2" | "ListTable3_Accent3" | "ListTable3_Accent4" | "ListTable3_Accent5" | "ListTable3_Accent6" | "ListTable4" | "ListTable4_Accent1" | "ListTable4_Accent2" | "ListTable4_Accent3" | "ListTable4_Accent4" | "ListTable4_Accent5" | "ListTable4_Accent6" | "ListTable5Dark" | "ListTable5Dark_Accent1" | "ListTable5Dark_Accent2" | "ListTable5Dark_Accent3" | "ListTable5Dark_Accent4" | "ListTable5Dark_Accent5" | "ListTable5Dark_Accent6" | "ListTable6Colorful" | "ListTable6Colorful_Accent1" | "ListTable6Colorful_Accent2" | "ListTable6Colorful_Accent3" | "ListTable6Colorful_Accent4" | "ListTable6Colorful_Accent5" | "ListTable6Colorful_Accent6" | "ListTable7Colorful" | "ListTable7Colorful_Accent1" | "ListTable7Colorful_Accent2" | "ListTable7Colorful_Accent3" | "ListTable7Colorful_Accent4" | "ListTable7Colorful_Accent5" | "ListTable7Colorful_Accent6";
        /**
         *
         * Gets the content control subtype. The subtype can be 'RichTextInline', 'RichTextParagraphs', 'RichTextTableCell', 'RichTextTableRow' and 'RichTextTable' for rich text content controls. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly subtype: Word.ContentControlType | "Unknown" | "RichTextInline" | "RichTextParagraphs" | "RichTextTableCell" | "RichTextTableRow" | "RichTextTable" | "PlainTextInline" | "PlainTextParagraph" | "Picture" | "BuildingBlockGallery" | "CheckBox" | "ComboBox" | "DropDownList" | "DatePicker" | "RepeatingSection" | "RichText" | "PlainText";
        /**
         *
         * Gets or sets a tag to identify a content control.
         *
         * [Api set: WordApi 1.1]
         */
        tag: string;
        /**
         *
         * Gets the text of the content control. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly text: string;
        /**
         *
         * Gets or sets the title for a content control.
         *
         * [Api set: WordApi 1.1]
         */
        title: string;
        /**
         *
         * Gets the content control type. Only rich text content controls are supported currently. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly type: Word.ContentControlType | "Unknown" | "RichTextInline" | "RichTextParagraphs" | "RichTextTableCell" | "RichTextTableRow" | "RichTextTable" | "PlainTextInline" | "PlainTextParagraph" | "Picture" | "BuildingBlockGallery" | "CheckBox" | "ComboBox" | "DropDownList" | "DatePicker" | "RepeatingSection" | "RichText" | "PlainText";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.ContentControl): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ContentControlUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.ContentControl): void;
        /**
         *
         * Clears the contents of the content control. The user can perform the undo operation on the cleared content.
         *
         * [Api set: WordApi 1.1]
         */
        clear(): void;
        /**
         *
         * Deletes the content control and its content. If keepContent is set to true, the content is not deleted.
         *
         * [Api set: WordApi 1.1]
         *
         * @param keepContent Required. Indicates whether the content should be deleted with the content control. If keepContent is set to true, the content is not deleted.
         */
        delete(keepContent: boolean): void;
        /**
         *
         * Gets an HTML representation of the content control object. When rendered in a web page or HTML viewer, the formatting will be a close, but not exact, match for of the formatting of the document. This method does not return the exact same HTML for the same document on different platforms (Windows, Mac, Word Online, etc.). If you need exact fidelity, or consistency across platforms, use `ContentControl.getOoxml()` and convert the returned XML to HTML.
         *
         * [Api set: WordApi 1.1]
         */
        getHtml(): OfficeExtension.ClientResult<string>;
        /**
         *
         * Gets the Office Open XML (OOXML) representation of the content control object.
         *
         * [Api set: WordApi 1.1]
         */
        getOoxml(): OfficeExtension.ClientResult<string>;
        /**
         *
         * Gets the whole content control, or the starting or ending point of the content control, as a range.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rangeLocation Optional. The range location can be 'Whole', 'Before', 'Start', 'End', 'After', or 'Content'.
         */
        getRange(rangeLocation?: Word.RangeLocation): Word.Range;
        /**
         *
         * Gets the whole content control, or the starting or ending point of the content control, as a range.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rangeLocation Optional. The range location can be 'Whole', 'Before', 'Start', 'End', 'After', or 'Content'.
         */
        getRange(rangeLocation?: "Whole" | "Start" | "End" | "Before" | "After" | "Content"): Word.Range;
        /**
         *
         * Gets the text ranges in the content control by using punctuation marks and/or other ending marks.
         *
         * [Api set: WordApi 1.3]
         *
         * @param endingMarks Required. The punctuation marks and/or other ending marks as an array of strings.
         * @param trimSpacing Optional. Indicates whether to trim spacing characters (spaces, tabs, column breaks, and paragraph end marks) from the start and end of the ranges returned in the range collection. Default is false which indicates that spacing characters at the start and end of the ranges are included in the range collection.
         */
        getTextRanges(endingMarks: string[], trimSpacing?: boolean): Word.RangeCollection;
        /**
         *
         * Inserts a break at the specified location in the main document. The insertLocation value can be 'Start', 'End', 'Before', or 'After'. This method cannot be used with 'RichTextTable', 'RichTextTableRow' and 'RichTextTableCell' content controls.
         *
         * [Api set: WordApi 1.1]
         *
         * @param breakType Required. Type of break.
         * @param insertLocation Required. The value can be 'Start', 'End', 'Before', or 'After'.
         */
        insertBreak(breakType: Word.BreakType, insertLocation: Word.InsertLocation): void;
        /**
         *
         * Inserts a break at the specified location in the main document. The insertLocation value can be 'Start', 'End', 'Before', or 'After'. This method cannot be used with 'RichTextTable', 'RichTextTableRow' and 'RichTextTableCell' content controls.
         *
         * [Api set: WordApi 1.1]
         *
         * @param breakType Required. Type of break.
         * @param insertLocation Required. The value can be 'Start', 'End', 'Before', or 'After'.
         */
        insertBreak(breakType: "Page" | "Next" | "SectionNext" | "SectionContinuous" | "SectionEven" | "SectionOdd" | "Line", insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): void;
        /**
         *
         * Inserts a document into the content control at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param base64File Required. The base64 encoded content of a .docx file.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'. 'Replace' cannot be used with 'RichTextTable' and 'RichTextTableRow' content controls.
         */
        insertFileFromBase64(base64File: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts a document into the content control at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param base64File Required. The base64 encoded content of a .docx file.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'. 'Replace' cannot be used with 'RichTextTable' and 'RichTextTableRow' content controls.
         */
        insertFileFromBase64(base64File: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Inserts HTML into the content control at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param html Required. The HTML to be inserted in to the content control.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'. 'Replace' cannot be used with 'RichTextTable' and 'RichTextTableRow' content controls.
         */
        insertHtml(html: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts HTML into the content control at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param html Required. The HTML to be inserted in to the content control.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'. 'Replace' cannot be used with 'RichTextTable' and 'RichTextTableRow' content controls.
         */
        insertHtml(html: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Inserts an inline picture into the content control at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param base64EncodedImage Required. The base64 encoded image to be inserted in the content control.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'. 'Replace' cannot be used with 'RichTextTable' and 'RichTextTableRow' content controls.
         */
        insertInlinePictureFromBase64(base64EncodedImage: string, insertLocation: Word.InsertLocation): Word.InlinePicture;
        /**
         *
         * Inserts an inline picture into the content control at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param base64EncodedImage Required. The base64 encoded image to be inserted in the content control.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'. 'Replace' cannot be used with 'RichTextTable' and 'RichTextTableRow' content controls.
         */
        insertInlinePictureFromBase64(base64EncodedImage: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.InlinePicture;
        /**
         *
         * Inserts OOXML into the content control at the specified location.  The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param ooxml Required. The OOXML to be inserted in to the content control.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'. 'Replace' cannot be used with 'RichTextTable' and 'RichTextTableRow' content controls.
         */
        insertOoxml(ooxml: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts OOXML into the content control at the specified location.  The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param ooxml Required. The OOXML to be inserted in to the content control.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'. 'Replace' cannot be used with 'RichTextTable' and 'RichTextTableRow' content controls.
         */
        insertOoxml(ooxml: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Inserts a paragraph at the specified location. The insertLocation value can be 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param paragraphText Required. The paragraph text to be inserted.
         * @param insertLocation Required. The value can be 'Start', 'End', 'Before', or 'After'. 'Before' and 'After' cannot be used with 'RichTextTable', 'RichTextTableRow' and 'RichTextTableCell' content controls.
         */
        insertParagraph(paragraphText: string, insertLocation: Word.InsertLocation): Word.Paragraph;
        /**
         *
         * Inserts a paragraph at the specified location. The insertLocation value can be 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param paragraphText Required. The paragraph text to be inserted.
         * @param insertLocation Required. The value can be 'Start', 'End', 'Before', or 'After'. 'Before' and 'After' cannot be used with 'RichTextTable', 'RichTextTableRow' and 'RichTextTableCell' content controls.
         */
        insertParagraph(paragraphText: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Paragraph;
        /**
         *
         * Inserts a table with the specified number of rows and columns into, or next to, a content control. The insertLocation value can be 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rowCount Required. The number of rows in the table.
         * @param columnCount Required. The number of columns in the table.
         * @param insertLocation Required. The value can be 'Start', 'End', 'Before', or 'After'. 'Before' and 'After' cannot be used with 'RichTextTable', 'RichTextTableRow' and 'RichTextTableCell' content controls.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertTable(rowCount: number, columnCount: number, insertLocation: Word.InsertLocation, values?: string[][]): Word.Table;
        /**
         *
         * Inserts a table with the specified number of rows and columns into, or next to, a content control. The insertLocation value can be 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rowCount Required. The number of rows in the table.
         * @param columnCount Required. The number of columns in the table.
         * @param insertLocation Required. The value can be 'Start', 'End', 'Before', or 'After'. 'Before' and 'After' cannot be used with 'RichTextTable', 'RichTextTableRow' and 'RichTextTableCell' content controls.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertTable(rowCount: number, columnCount: number, insertLocation: "Before" | "After" | "Start" | "End" | "Replace", values?: string[][]): Word.Table;
        /**
         *
         * Inserts text into the content control at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param text Required. The text to be inserted in to the content control.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'. 'Replace' cannot be used with 'RichTextTable' and 'RichTextTableRow' content controls.
         */
        insertText(text: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts text into the content control at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param text Required. The text to be inserted in to the content control.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'. 'Replace' cannot be used with 'RichTextTable' and 'RichTextTableRow' content controls.
         */
        insertText(text: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Performs a search with the specified SearchOptions on the scope of the content control object. The search results are a collection of range objects.
         *
         * [Api set: WordApi 1.1]
         *
         * @param searchText Required. The search text.
         * @param searchOptions Optional. Options for the search.
         */
        search(searchText: string, searchOptions?: Word.SearchOptions | {
            ignorePunct?: boolean;
            ignoreSpace?: boolean;
            matchCase?: boolean;
            matchPrefix?: boolean;
            matchSuffix?: boolean;
            matchWholeWord?: boolean;
            matchWildcards?: boolean;
        }): Word.RangeCollection;
        /**
         *
         * Selects the content control. This causes Word to scroll to the selection.
         *
         * [Api set: WordApi 1.1]
         *
         * @param selectionMode Optional. The selection mode can be 'Select', 'Start', or 'End'. 'Select' is the default.
         */
        select(selectionMode?: Word.SelectionMode): void;
        /**
         *
         * Selects the content control. This causes Word to scroll to the selection.
         *
         * [Api set: WordApi 1.1]
         *
         * @param selectionMode Optional. The selection mode can be 'Select', 'Start', or 'End'. 'Select' is the default.
         */
        select(selectionMode?: "Select" | "Start" | "End"): void;
        /**
         *
         * Splits the content control into child ranges by using delimiters.
         *
         * [Api set: WordApi 1.3]
         *
         * @param delimiters Required. The delimiters as an array of strings.
         * @param multiParagraphs Optional. Indicates whether a returned child range can cover multiple paragraphs. Default is false which indicates that the paragraph boundaries are also used as delimiters.
         * @param trimDelimiters Optional. Indicates whether to trim delimiters from the ranges in the range collection. Default is false which indicates that the delimiters are included in the ranges returned in the range collection.
         * @param trimSpacing Optional. Indicates whether to trim spacing characters (spaces, tabs, column breaks, and paragraph end marks) from the start and end of the ranges returned in the range collection. Default is false which indicates that spacing characters at the start and end of the ranges are included in the range collection.
         */
        split(delimiters: string[], multiParagraphs?: boolean, trimDelimiters?: boolean, trimSpacing?: boolean): Word.RangeCollection;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.ContentControl` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.ContentControl` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.ContentControl` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.ContentControlLoadOptions): Word.ContentControl;
        load(option?: string | string[]): Word.ContentControl;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.ContentControl;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.ContentControl;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.ContentControl;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.ContentControl object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.ContentControlData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.ContentControlData;
    }
    /**
     *
     * Contains a collection of {@link Word.ContentControl} objects. Content controls are bounded and potentially labeled regions in a document that serve as containers for specific types of content. Individual content controls may contain contents such as images, tables, or paragraphs of formatted text. Currently, only rich text content controls are supported.
     *
     * [Api set: WordApi 1.1]
     */
    class ContentControlCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Word.ContentControl[];
        /**
         *
         * Gets a content control by its identifier. Throws if there isn't a content control with the identifier in this collection.
         *
         * [Api set: WordApi 1.1]
         *
         * @param id Required. A content control identifier.
         */
        getById(id: number): Word.ContentControl;
        /**
         *
         * Gets a content control by its identifier. Returns a null object if there isn't a content control with the identifier in this collection.
         *
         * [Api set: WordApi 1.3]
         *
         * @param id Required. A content control identifier.
         */
        getByIdOrNullObject(id: number): Word.ContentControl;
        /**
         *
         * Gets the content controls that have the specified tag.
         *
         * [Api set: WordApi 1.1]
         *
         * @param tag Required. A tag set on a content control.
         */
        getByTag(tag: string): Word.ContentControlCollection;
        /**
         *
         * Gets the content controls that have the specified title.
         *
         * [Api set: WordApi 1.1]
         *
         * @param title Required. The title of a content control.
         */
        getByTitle(title: string): Word.ContentControlCollection;
        /**
         *
         * Gets the content controls that have the specified types and/or subtypes.
         *
         * [Api set: WordApi 1.3]
         *
         * @param types Required. An array of content control types and/or subtypes.
         */
        getByTypes(types: Word.ContentControlType[]): Word.ContentControlCollection;
        /**
         *
         * Gets the first content control in this collection. Throws if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirst(): Word.ContentControl;
        /**
         *
         * Gets the first content control in this collection. Returns a null object if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirstOrNullObject(): Word.ContentControl;
        /**
         *
         * Gets a content control by its index in the collection.
         *
         * [Api set: WordApi 1.1]
         *
         * @param index The index.
         */
        getItem(index: number): Word.ContentControl;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.ContentControlCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.ContentControlCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.ContentControlCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.ContentControlCollectionLoadOptions & Word.Interfaces.CollectionLoadOptions): Word.ContentControlCollection;
        load(option?: string | string[]): Word.ContentControlCollection;
        load(option?: OfficeExtension.LoadOption): Word.ContentControlCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.ContentControlCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.ContentControlCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Word.ContentControlCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.ContentControlCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Word.Interfaces.ContentControlCollectionData;
    }
    /**
     *
     * Represents a custom property.
     *
     * [Api set: WordApi 1.3]
     */
    class CustomProperty extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the key of the custom property. Read only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly key: string;
        /**
         *
         * Gets the value type of the custom property. Possible values are: String, Number, Date, Boolean. Read only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly type: Word.DocumentPropertyType | "String" | "Number" | "Date" | "Boolean";
        /**
         *
         * Gets or sets the value of the custom property. Note that even though Word Online and the docx file format allow these properties to be arbitrarily long, the desktop version of Word will truncate string values to 255 16-bit chars (possibly creating invalid unicode by breaking up a surrogate pair).
         *
         * [Api set: WordApi 1.3]
         */
        value: any;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.CustomProperty): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.CustomPropertyUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.CustomProperty): void;
        /**
         *
         * Deletes the custom property.
         *
         * [Api set: WordApi 1.3]
         */
        delete(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.CustomProperty` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.CustomProperty` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.CustomProperty` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.CustomPropertyLoadOptions): Word.CustomProperty;
        load(option?: string | string[]): Word.CustomProperty;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.CustomProperty;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.CustomProperty;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.CustomProperty;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.CustomProperty object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.CustomPropertyData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.CustomPropertyData;
    }
    /**
     *
     * Contains the collection of {@link Word.CustomProperty} objects.
     *
     * [Api set: WordApi 1.3]
     */
    class CustomPropertyCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Word.CustomProperty[];
        /**
         *
         * Creates a new or sets an existing custom property.
         *
         * [Api set: WordApi 1.3]
         *
         * @param key Required. The custom property's key, which is case-insensitive.
         * @param value Required. The custom property's value.
         */
        add(key: string, value: any): Word.CustomProperty;
        /**
         *
         * Deletes all custom properties in this collection.
         *
         * [Api set: WordApi 1.3]
         */
        deleteAll(): void;
        /**
         *
         * Gets the count of custom properties.
         *
         * [Api set: WordApi 1.3]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a custom property object by its key, which is case-insensitive. Throws if the custom property does not exist.
         *
         * [Api set: WordApi 1.3]
         *
         * @param key The key that identifies the custom property object.
         */
        getItem(key: string): Word.CustomProperty;
        /**
         *
         * Gets a custom property object by its key, which is case-insensitive. Returns a null object if the custom property does not exist.
         *
         * [Api set: WordApi 1.3]
         *
         * @param key Required. The key that identifies the custom property object.
         */
        getItemOrNullObject(key: string): Word.CustomProperty;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.CustomPropertyCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.CustomPropertyCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.CustomPropertyCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.CustomPropertyCollectionLoadOptions & Word.Interfaces.CollectionLoadOptions): Word.CustomPropertyCollection;
        load(option?: string | string[]): Word.CustomPropertyCollection;
        load(option?: OfficeExtension.LoadOption): Word.CustomPropertyCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.CustomPropertyCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.CustomPropertyCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Word.CustomPropertyCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.CustomPropertyCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Word.Interfaces.CustomPropertyCollectionData;
    }
    /**
     *
     * The Document object is the top level object. A Document object contains one or more sections, content controls, and the body that contains the contents of the document.
     *
     * [Api set: WordApi 1.1]
     */
    class Document extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the body object of the document. The body is the text that excludes headers, footers, footnotes, textboxes, etc.. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly body: Word.Body;
        /**
         *
         * Gets the collection of content control objects in the document. This includes content controls in the body of the document, headers, footers, textboxes, etc.. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly contentControls: Word.ContentControlCollection;
        /**
         *
         * Gets the properties of the document. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly properties: Word.DocumentProperties;
        /**
         *
         * Gets the collection of section objects in the document. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly sections: Word.SectionCollection;
        /**
         *
         * Indicates whether the changes in the document have been saved. A value of true indicates that the document hasn't changed since it was saved. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly saved: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.Document): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.DocumentUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.Document): void;
        /**
         *
         * Gets the current selection of the document. Multiple selections are not supported.
         *
         * [Api set: WordApi 1.1]
         */
        getSelection(): Word.Range;
        /**
         *
         * Saves the document. This will use the Word default file naming convention if the document has not been saved before.
         *
         * [Api set: WordApi 1.1]
         */
        save(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.Document` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.Document` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.Document` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.DocumentLoadOptions): Word.Document;
        load(option?: string | string[]): Word.Document;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.Document;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.Document;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.Document;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.Document object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.DocumentData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.DocumentData;
    }
    /**
     *
     * The DocumentCreated object is the top level object created by Application.CreateDocument. A DocumentCreated object is a special Document object.
     *
     * [Api set: WordApi 1.3]
     */
    class DocumentCreated extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the body object of the document. The body is the text that excludes headers, footers, footnotes, textboxes, etc.. Read-only.
         *
         * [Api set: WordApiHiddenDocument 1.3]
         */
        readonly body: Word.Body;
        /**
         *
         * Gets the collection of content control objects in the document. This includes content controls in the body of the document, headers, footers, textboxes, etc.. Read-only.
         *
         * [Api set: WordApiHiddenDocument 1.3]
         */
        readonly contentControls: Word.ContentControlCollection;
        /**
         *
         * Gets the properties of the document. Read-only.
         *
         * [Api set: WordApiHiddenDocument 1.3]
         */
        readonly properties: Word.DocumentProperties;
        /**
         *
         * Gets the collection of section objects in the document. Read-only.
         *
         * [Api set: WordApiHiddenDocument 1.3]
         */
        readonly sections: Word.SectionCollection;
        /**
         *
         * Indicates whether the changes in the document have been saved. A value of true indicates that the document hasn't changed since it was saved. Read-only.
         *
         * [Api set: WordApiHiddenDocument 1.3]
         */
        readonly saved: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.DocumentCreated): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.DocumentCreatedUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.DocumentCreated): void;
        /**
         *
         * Opens the document.
         *
         * [Api set: WordApi 1.3]
         */
        open(): void;
        /**
         *
         * Saves the document. This will use the Word default file naming convention if the document has not been saved before.
         *
         * [Api set: WordApiHiddenDocument 1.3]
         */
        save(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.DocumentCreated` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.DocumentCreated` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.DocumentCreated` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.DocumentCreatedLoadOptions): Word.DocumentCreated;
        load(option?: string | string[]): Word.DocumentCreated;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.DocumentCreated;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.DocumentCreated;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.DocumentCreated;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.DocumentCreated object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.DocumentCreatedData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.DocumentCreatedData;
    }
    /**
     *
     * Represents document properties.
     *
     * [Api set: WordApi 1.3]
     */
    class DocumentProperties extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the collection of custom properties of the document. Read only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly customProperties: Word.CustomPropertyCollection;
        /**
         *
         * Gets the application name of the document. Read only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly applicationName: string;
        /**
         *
         * Gets or sets the author of the document.
         *
         * [Api set: WordApi 1.3]
         */
        author: string;
        /**
         *
         * Gets or sets the category of the document.
         *
         * [Api set: WordApi 1.3]
         */
        category: string;
        /**
         *
         * Gets or sets the comments of the document.
         *
         * [Api set: WordApi 1.3]
         */
        comments: string;
        /**
         *
         * Gets or sets the company of the document.
         *
         * [Api set: WordApi 1.3]
         */
        company: string;
        /**
         *
         * Gets the creation date of the document. Read only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly creationDate: Date;
        /**
         *
         * Gets or sets the format of the document.
         *
         * [Api set: WordApi 1.3]
         */
        format: string;
        /**
         *
         * Gets or sets the keywords of the document.
         *
         * [Api set: WordApi 1.3]
         */
        keywords: string;
        /**
         *
         * Gets the last author of the document. Read only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly lastAuthor: string;
        /**
         *
         * Gets the last print date of the document. Read only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly lastPrintDate: Date;
        /**
         *
         * Gets the last save time of the document. Read only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly lastSaveTime: Date;
        /**
         *
         * Gets or sets the manager of the document.
         *
         * [Api set: WordApi 1.3]
         */
        manager: string;
        /**
         *
         * Gets the revision number of the document. Read only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly revisionNumber: string;
        /**
         *
         * Gets security settings of the document. Read only. Some are access restrictions on the file on disk. Others are Document Protection settings. Some possible values are 0 = File on disk is read/write; 1 = Protect Document: File is encrypted and requires a password to open; 2 = Protect Document: Always Open as Read-Only; 3 = Protect Document: Both #1 and #2; 4 = File on disk is read only; 5 = Both #1 and #4; 6 = Both #2 and #4; 7 = All of #1, #2, and #4; 8 = Protect Document: Restrict Edit to read-only; 9 = Both #1 and #8; 10 = Both #2 and #8; 11 = All of #1, #2, and #8; 12 = Both #4 and #8; 13 = All of #1, #4, and #8; 14 = All of #2, #4, and #8; 15 = All of #1, #2, #4, and #8.
         *
         * [Api set: WordApi 1.3]
         */
        readonly security: number;
        /**
         *
         * Gets or sets the subject of the document.
         *
         * [Api set: WordApi 1.3]
         */
        subject: string;
        /**
         *
         * Gets the template of the document. Read only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly template: string;
        /**
         *
         * Gets or sets the title of the document.
         *
         * [Api set: WordApi 1.3]
         */
        title: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.DocumentProperties): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.DocumentPropertiesUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.DocumentProperties): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.DocumentProperties` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.DocumentProperties` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.DocumentProperties` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.DocumentPropertiesLoadOptions): Word.DocumentProperties;
        load(option?: string | string[]): Word.DocumentProperties;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.DocumentProperties;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.DocumentProperties;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.DocumentProperties;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.DocumentProperties object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.DocumentPropertiesData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.DocumentPropertiesData;
    }
    /**
     *
     * Represents a font.
     *
     * [Api set: WordApi 1.1]
     */
    class Font extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets or sets a value that indicates whether the font is bold. True if the font is formatted as bold, otherwise, false.
         *
         * [Api set: WordApi 1.1]
         */
        bold: boolean;
        /**
         *
         * Gets or sets the color for the specified font. You can provide the value in the '#RRGGBB' format or the color name.
         *
         * [Api set: WordApi 1.1]
         */
        color: string;
        /**
         *
         * Gets or sets a value that indicates whether the font has a double strikethrough. True if the font is formatted as double strikethrough text, otherwise, false.
         *
         * [Api set: WordApi 1.1]
         */
        doubleStrikeThrough: boolean;
        /**
         *
         * Gets or sets the highlight color. To set it, use a value either in the '#RRGGBB' format or the color name. To remove highlight color, set it to null. The returned highlight color can be in the '#RRGGBB' format, an empty string for mixed highlight colors, or null for no highlight color.
         *
         * [Api set: WordApi 1.1]
         */
        highlightColor: string;
        /**
         *
         * Gets or sets a value that indicates whether the font is italicized. True if the font is italicized, otherwise, false.
         *
         * [Api set: WordApi 1.1]
         */
        italic: boolean;
        /**
         *
         * Gets or sets a value that represents the name of the font.
         *
         * [Api set: WordApi 1.1]
         */
        name: string;
        /**
         *
         * Gets or sets a value that represents the font size in points.
         *
         * [Api set: WordApi 1.1]
         */
        size: number;
        /**
         *
         * Gets or sets a value that indicates whether the font has a strikethrough. True if the font is formatted as strikethrough text, otherwise, false.
         *
         * [Api set: WordApi 1.1]
         */
        strikeThrough: boolean;
        /**
         *
         * Gets or sets a value that indicates whether the font is a subscript. True if the font is formatted as subscript, otherwise, false.
         *
         * [Api set: WordApi 1.1]
         */
        subscript: boolean;
        /**
         *
         * Gets or sets a value that indicates whether the font is a superscript. True if the font is formatted as superscript, otherwise, false.
         *
         * [Api set: WordApi 1.1]
         */
        superscript: boolean;
        /**
         *
         * Gets or sets a value that indicates the font's underline type. 'None' if the font is not underlined.
         *
         * [Api set: WordApi 1.1]
         */
        underline: Word.UnderlineType | "Mixed" | "None" | "Hidden" | "DotLine" | "Single" | "Word" | "Double" | "Thick" | "Dotted" | "DottedHeavy" | "DashLine" | "DashLineHeavy" | "DashLineLong" | "DashLineLongHeavy" | "DotDashLine" | "DotDashLineHeavy" | "TwoDotDashLine" | "TwoDotDashLineHeavy" | "Wave" | "WaveHeavy" | "WaveDouble";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.Font): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.FontUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.Font): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.Font` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.Font` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.Font` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.FontLoadOptions): Word.Font;
        load(option?: string | string[]): Word.Font;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.Font;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.Font;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.Font;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.Font object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.FontData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.FontData;
    }
    /**
     *
     * Represents an inline picture.
     *
     * [Api set: WordApi 1.1]
     */
    class InlinePicture extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the parent paragraph that contains the inline image. Read-only.
         *
         * [Api set: WordApi 1.2]
         */
        readonly paragraph: Word.Paragraph;
        /**
         *
         * Gets the content control that contains the inline image. Throws if there isn't a parent content control. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly parentContentControl: Word.ContentControl;
        /**
         *
         * Gets the content control that contains the inline image. Returns a null object if there isn't a parent content control. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentContentControlOrNullObject: Word.ContentControl;
        /**
         *
         * Gets the table that contains the inline image. Throws if it is not contained in a table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTable: Word.Table;
        /**
         *
         * Gets the table cell that contains the inline image. Throws if it is not contained in a table cell. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTableCell: Word.TableCell;
        /**
         *
         * Gets the table cell that contains the inline image. Returns a null object if it is not contained in a table cell. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTableCellOrNullObject: Word.TableCell;
        /**
         *
         * Gets the table that contains the inline image. Returns a null object if it is not contained in a table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTableOrNullObject: Word.Table;
        /**
         *
         * Gets or sets a string that represents the alternative text associated with the inline image.
         *
         * [Api set: WordApi 1.1]
         */
        altTextDescription: string;
        /**
         *
         * Gets or sets a string that contains the title for the inline image.
         *
         * [Api set: WordApi 1.1]
         */
        altTextTitle: string;
        /**
         *
         * Gets or sets a number that describes the height of the inline image.
         *
         * [Api set: WordApi 1.1]
         */
        height: number;
        /**
         *
         * Gets or sets a hyperlink on the image. Use a '#' to separate the address part from the optional location part.
         *
         * [Api set: WordApi 1.1]
         */
        hyperlink: string;
        /**
         *
         * Gets or sets a value that indicates whether the inline image retains its original proportions when you resize it.
         *
         * [Api set: WordApi 1.1]
         */
        lockAspectRatio: boolean;
        /**
         *
         * Gets or sets a number that describes the width of the inline image.
         *
         * [Api set: WordApi 1.1]
         */
        width: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.InlinePicture): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.InlinePictureUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.InlinePicture): void;
        /**
         *
         * Deletes the inline picture from the document.
         *
         * [Api set: WordApi 1.2]
         */
        delete(): void;
        /**
         *
         * Gets the base64 encoded string representation of the inline image.
         *
         * [Api set: WordApi 1.1]
         */
        getBase64ImageSrc(): OfficeExtension.ClientResult<string>;
        /**
         *
         * Gets the next inline image. Throws if this inline image is the last one.
         *
         * [Api set: WordApi 1.3]
         */
        getNext(): Word.InlinePicture;
        /**
         *
         * Gets the next inline image. Returns a null object if this inline image is the last one.
         *
         * [Api set: WordApi 1.3]
         */
        getNextOrNullObject(): Word.InlinePicture;
        /**
         *
         * Gets the picture, or the starting or ending point of the picture, as a range.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rangeLocation Optional. The range location can be 'Whole', 'Start', or 'End'.
         */
        getRange(rangeLocation?: Word.RangeLocation): Word.Range;
        /**
         *
         * Gets the picture, or the starting or ending point of the picture, as a range.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rangeLocation Optional. The range location can be 'Whole', 'Start', or 'End'.
         */
        getRange(rangeLocation?: "Whole" | "Start" | "End" | "Before" | "After" | "Content"): Word.Range;
        /**
         *
         * Inserts a break at the specified location in the main document. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param breakType Required. The break type to add.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertBreak(breakType: Word.BreakType, insertLocation: Word.InsertLocation): void;
        /**
         *
         * Inserts a break at the specified location in the main document. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param breakType Required. The break type to add.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertBreak(breakType: "Page" | "Next" | "SectionNext" | "SectionContinuous" | "SectionEven" | "SectionOdd" | "Line", insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): void;
        /**
         *
         * Wraps the inline picture with a rich text content control.
         *
         * [Api set: WordApi 1.1]
         */
        insertContentControl(): Word.ContentControl;
        /**
         *
         * Inserts a document at the specified location. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param base64File Required. The base64 encoded content of a .docx file.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertFileFromBase64(base64File: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts a document at the specified location. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param base64File Required. The base64 encoded content of a .docx file.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertFileFromBase64(base64File: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Inserts HTML at the specified location. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param html Required. The HTML to be inserted.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertHtml(html: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts HTML at the specified location. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param html Required. The HTML to be inserted.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertHtml(html: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Inserts an inline picture at the specified location. The insertLocation value can be 'Replace', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param base64EncodedImage Required. The base64 encoded image to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Before', or 'After'.
         */
        insertInlinePictureFromBase64(base64EncodedImage: string, insertLocation: Word.InsertLocation): Word.InlinePicture;
        /**
         *
         * Inserts an inline picture at the specified location. The insertLocation value can be 'Replace', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param base64EncodedImage Required. The base64 encoded image to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Before', or 'After'.
         */
        insertInlinePictureFromBase64(base64EncodedImage: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.InlinePicture;
        /**
         *
         * Inserts OOXML at the specified location.  The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param ooxml Required. The OOXML to be inserted.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertOoxml(ooxml: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts OOXML at the specified location.  The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param ooxml Required. The OOXML to be inserted.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertOoxml(ooxml: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Inserts a paragraph at the specified location. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param paragraphText Required. The paragraph text to be inserted.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertParagraph(paragraphText: string, insertLocation: Word.InsertLocation): Word.Paragraph;
        /**
         *
         * Inserts a paragraph at the specified location. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param paragraphText Required. The paragraph text to be inserted.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertParagraph(paragraphText: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Paragraph;
        /**
         *
         * Inserts text at the specified location. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param text Required. Text to be inserted.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertText(text: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts text at the specified location. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param text Required. Text to be inserted.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertText(text: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Selects the inline picture. This causes Word to scroll to the selection.
         *
         * [Api set: WordApi 1.2]
         *
         * @param selectionMode Optional. The selection mode can be 'Select', 'Start', or 'End'. 'Select' is the default.
         */
        select(selectionMode?: Word.SelectionMode): void;
        /**
         *
         * Selects the inline picture. This causes Word to scroll to the selection.
         *
         * [Api set: WordApi 1.2]
         *
         * @param selectionMode Optional. The selection mode can be 'Select', 'Start', or 'End'. 'Select' is the default.
         */
        select(selectionMode?: "Select" | "Start" | "End"): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.InlinePicture` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.InlinePicture` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.InlinePicture` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.InlinePictureLoadOptions): Word.InlinePicture;
        load(option?: string | string[]): Word.InlinePicture;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.InlinePicture;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.InlinePicture;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.InlinePicture;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.InlinePicture object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.InlinePictureData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.InlinePictureData;
    }
    /**
     *
     * Contains a collection of {@link Word.InlinePicture} objects.
     *
     * [Api set: WordApi 1.1]
     */
    class InlinePictureCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Word.InlinePicture[];
        /**
         *
         * Gets the first inline image in this collection. Throws if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirst(): Word.InlinePicture;
        /**
         *
         * Gets the first inline image in this collection. Returns a null object if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirstOrNullObject(): Word.InlinePicture;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.InlinePictureCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.InlinePictureCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.InlinePictureCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.InlinePictureCollectionLoadOptions & Word.Interfaces.CollectionLoadOptions): Word.InlinePictureCollection;
        load(option?: string | string[]): Word.InlinePictureCollection;
        load(option?: OfficeExtension.LoadOption): Word.InlinePictureCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.InlinePictureCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.InlinePictureCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Word.InlinePictureCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.InlinePictureCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Word.Interfaces.InlinePictureCollectionData;
    }
    /**
     *
     * Contains a collection of {@link Word.Paragraph} objects.
     *
     * [Api set: WordApi 1.3]
     */
    class List extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets paragraphs in the list. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly paragraphs: Word.ParagraphCollection;
        /**
         *
         * Gets the list's id.
         *
         * [Api set: WordApi 1.3]
         */
        readonly id: number;
        /**
         *
         * Checks whether each of the 9 levels exists in the list. A true value indicates the level exists, which means there is at least one list item at that level. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly levelExistences: boolean[];
        /**
         *
         * Gets all 9 level types in the list. Each type can be 'Bullet', 'Number', or 'Picture'. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly levelTypes: Word.ListLevelType[];
        /**
         *
         * Gets the paragraphs that occur at the specified level in the list.
         *
         * [Api set: WordApi 1.3]
         *
         * @param level Required. The level in the list.
         */
        getLevelParagraphs(level: number): Word.ParagraphCollection;
        /**
         *
         * Gets the bullet, number or picture at the specified level as a string.
         *
         * [Api set: WordApi 1.3]
         *
         * @param level Required. The level in the list.
         */
        getLevelString(level: number): OfficeExtension.ClientResult<string>;
        /**
         *
         * Inserts a paragraph at the specified location. The insertLocation value can be 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.3]
         *
         * @param paragraphText Required. The paragraph text to be inserted.
         * @param insertLocation Required. The value can be 'Start', 'End', 'Before', or 'After'.
         */
        insertParagraph(paragraphText: string, insertLocation: Word.InsertLocation): Word.Paragraph;
        /**
         *
         * Inserts a paragraph at the specified location. The insertLocation value can be 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.3]
         *
         * @param paragraphText Required. The paragraph text to be inserted.
         * @param insertLocation Required. The value can be 'Start', 'End', 'Before', or 'After'.
         */
        insertParagraph(paragraphText: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Paragraph;
        /**
         *
         * Sets the alignment of the bullet, number or picture at the specified level in the list.
         *
         * [Api set: WordApi 1.3]
         *
         * @param level Required. The level in the list.
         * @param alignment Required. The level alignment that can be 'Left', 'Centered', or 'Right'.
         */
        setLevelAlignment(level: number, alignment: Word.Alignment): void;
        /**
         *
         * Sets the alignment of the bullet, number or picture at the specified level in the list.
         *
         * [Api set: WordApi 1.3]
         *
         * @param level Required. The level in the list.
         * @param alignment Required. The level alignment that can be 'Left', 'Centered', or 'Right'.
         */
        setLevelAlignment(level: number, alignment: "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified"): void;
        /**
         *
         * Sets the bullet format at the specified level in the list. If the bullet is 'Custom', the charCode is required.
         *
         * [Api set: WordApi 1.3]
         *
         * @param level Required. The level in the list.
         * @param listBullet Required. The bullet.
         * @param charCode Optional. The bullet character's code value. Used only if the bullet is 'Custom'.
         * @param fontName Optional. The bullet's font name. Used only if the bullet is 'Custom'.
         */
        setLevelBullet(level: number, listBullet: Word.ListBullet, charCode?: number, fontName?: string): void;
        /**
         *
         * Sets the bullet format at the specified level in the list. If the bullet is 'Custom', the charCode is required.
         *
         * [Api set: WordApi 1.3]
         *
         * @param level Required. The level in the list.
         * @param listBullet Required. The bullet.
         * @param charCode Optional. The bullet character's code value. Used only if the bullet is 'Custom'.
         * @param fontName Optional. The bullet's font name. Used only if the bullet is 'Custom'.
         */
        setLevelBullet(level: number, listBullet: "Custom" | "Solid" | "Hollow" | "Square" | "Diamonds" | "Arrow" | "Checkmark", charCode?: number, fontName?: string): void;
        /**
         *
         * Sets the two indents of the specified level in the list.
         *
         * [Api set: WordApi 1.3]
         *
         * @param level Required. The level in the list.
         * @param textIndent Required. The text indent in points. It is the same as paragraph left indent.
         * @param bulletNumberPictureIndent Required. The relative indent, in points, of the bullet, number or picture. It is the same as paragraph first line indent.
         */
        setLevelIndents(level: number, textIndent: number, bulletNumberPictureIndent: number): void;
        /**
         *
         * Sets the numbering format at the specified level in the list.
         *
         * [Api set: WordApi 1.3]
         *
         * @param level Required. The level in the list.
         * @param listNumbering Required. The ordinal format.
         * @param formatString Optional. The numbering string format defined as an array of strings and/or integers. Each integer is a level of number type that is higher than or equal to this level. For example, an array of ["(", level - 1, ".", level, ")"] can define the format of "(2.c)", where 2 is the parent's item number and c is this level's item number.
         */
        setLevelNumbering(level: number, listNumbering: Word.ListNumbering, formatString?: Array<string | number>): void;
        /**
         *
         * Sets the numbering format at the specified level in the list.
         *
         * [Api set: WordApi 1.3]
         *
         * @param level Required. The level in the list.
         * @param listNumbering Required. The ordinal format.
         * @param formatString Optional. The numbering string format defined as an array of strings and/or integers. Each integer is a level of number type that is higher than or equal to this level. For example, an array of ["(", level - 1, ".", level, ")"] can define the format of "(2.c)", where 2 is the parent's item number and c is this level's item number.
         */
        setLevelNumbering(level: number, listNumbering: "None" | "Arabic" | "UpperRoman" | "LowerRoman" | "UpperLetter" | "LowerLetter", formatString?: Array<string | number>): void;
        /**
         *
         * Sets the starting number at the specified level in the list. Default value is 1.
         *
         * [Api set: WordApi 1.3]
         *
         * @param level Required. The level in the list.
         * @param startingNumber Required. The number to start with.
         */
        setLevelStartingNumber(level: number, startingNumber: number): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.List` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.List` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.List` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.ListLoadOptions): Word.List;
        load(option?: string | string[]): Word.List;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.List;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.List;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.List;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.List object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.ListData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.ListData;
    }
    /**
     *
     * Contains a collection of {@link Word.List} objects.
     *
     * [Api set: WordApi 1.3]
     */
    class ListCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Word.List[];
        /**
         *
         * Gets a list by its identifier. Throws if there isn't a list with the identifier in this collection.
         *
         * [Api set: WordApi 1.3]
         *
         * @param id Required. A list identifier.
         */
        getById(id: number): Word.List;
        /**
         *
         * Gets a list by its identifier. Returns a null object if there isn't a list with the identifier in this collection.
         *
         * [Api set: WordApi 1.3]
         *
         * @param id Required. A list identifier.
         */
        getByIdOrNullObject(id: number): Word.List;
        /**
         *
         * Gets the first list in this collection. Throws if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirst(): Word.List;
        /**
         *
         * Gets the first list in this collection. Returns a null object if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirstOrNullObject(): Word.List;
        /**
         *
         * Gets a list object by its index in the collection.
         *
         * [Api set: WordApi 1.3]
         *
         * @param index A number that identifies the index location of a list object.
         */
        getItem(index: number): Word.List;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.ListCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.ListCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.ListCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.ListCollectionLoadOptions & Word.Interfaces.CollectionLoadOptions): Word.ListCollection;
        load(option?: string | string[]): Word.ListCollection;
        load(option?: OfficeExtension.LoadOption): Word.ListCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.ListCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.ListCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Word.ListCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.ListCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Word.Interfaces.ListCollectionData;
    }
    /**
     *
     * Represents the paragraph list item format.
     *
     * [Api set: WordApi 1.3]
     */
    class ListItem extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets or sets the level of the item in the list.
         *
         * [Api set: WordApi 1.3]
         */
        level: number;
        /**
         *
         * Gets the list item bullet, number, or picture as a string. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly listString: string;
        /**
         *
         * Gets the list item order number in relation to its siblings. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly siblingIndex: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.ListItem): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ListItemUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.ListItem): void;
        /**
         *
         * Gets the list item parent, or the closest ancestor if the parent does not exist. Throws if the list item has no ancestor.
         *
         * [Api set: WordApi 1.3]
         *
         * @param parentOnly Optional. Specifies only the list item's parent will be returned. The default is false that specifies to get the lowest ancestor.
         */
        getAncestor(parentOnly?: boolean): Word.Paragraph;
        /**
         *
         * Gets the list item parent, or the closest ancestor if the parent does not exist. Returns a null object if the list item has no ancestor.
         *
         * [Api set: WordApi 1.3]
         *
         * @param parentOnly Optional. Specifies only the list item's parent will be returned. The default is false that specifies to get the lowest ancestor.
         */
        getAncestorOrNullObject(parentOnly?: boolean): Word.Paragraph;
        /**
         *
         * Gets all descendant list items of the list item.
         *
         * [Api set: WordApi 1.3]
         *
         * @param directChildrenOnly Optional. Specifies only the list item's direct children will be returned. The default is false that indicates to get all descendant items.
         */
        getDescendants(directChildrenOnly?: boolean): Word.ParagraphCollection;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.ListItem` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.ListItem` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.ListItem` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.ListItemLoadOptions): Word.ListItem;
        load(option?: string | string[]): Word.ListItem;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.ListItem;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.ListItem;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.ListItem;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.ListItem object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.ListItemData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.ListItemData;
    }
    /**
     *
     * Represents a single paragraph in a selection, range, content control, or document body.
     *
     * [Api set: WordApi 1.1]
     */
    class Paragraph extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the collection of content control objects in the paragraph. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly contentControls: Word.ContentControlCollection;
        /**
         *
         * Gets the text format of the paragraph. Use this to get and set font name, size, color, and other properties. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly font: Word.Font;
        /**
         *
         * Gets the collection of InlinePicture objects in the paragraph. The collection does not include floating images. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly inlinePictures: Word.InlinePictureCollection;
        /**
         *
         * Gets the List to which this paragraph belongs. Throws if the paragraph is not in a list. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly list: Word.List;
        /**
         *
         * Gets the ListItem for the paragraph. Throws if the paragraph is not part of a list. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly listItem: Word.ListItem;
        /**
         *
         * Gets the ListItem for the paragraph. Returns a null object if the paragraph is not part of a list. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly listItemOrNullObject: Word.ListItem;
        /**
         *
         * Gets the List to which this paragraph belongs. Returns a null object if the paragraph is not in a list. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly listOrNullObject: Word.List;
        /**
         *
         * Gets the parent body of the paragraph. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentBody: Word.Body;
        /**
         *
         * Gets the content control that contains the paragraph. Throws if there isn't a parent content control. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly parentContentControl: Word.ContentControl;
        /**
         *
         * Gets the content control that contains the paragraph. Returns a null object if there isn't a parent content control. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentContentControlOrNullObject: Word.ContentControl;
        /**
         *
         * Gets the table that contains the paragraph. Throws if it is not contained in a table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTable: Word.Table;
        /**
         *
         * Gets the table cell that contains the paragraph. Throws if it is not contained in a table cell. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTableCell: Word.TableCell;
        /**
         *
         * Gets the table cell that contains the paragraph. Returns a null object if it is not contained in a table cell. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTableCellOrNullObject: Word.TableCell;
        /**
         *
         * Gets the table that contains the paragraph. Returns a null object if it is not contained in a table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTableOrNullObject: Word.Table;
        /**
         *
         * Gets or sets the alignment for a paragraph. The value can be 'left', 'centered', 'right', or 'justified'.
         *
         * [Api set: WordApi 1.1]
         */
        alignment: Word.Alignment | "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified";
        /**
         *
         * Gets or sets the value, in points, for a first line or hanging indent. Use a positive value to set a first-line indent, and use a negative value to set a hanging indent.
         *
         * [Api set: WordApi 1.1]
         */
        firstLineIndent: number;
        /**
         *
         * Indicates the paragraph is the last one inside its parent body. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly isLastParagraph: boolean;
        /**
         *
         * Checks whether the paragraph is a list item. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly isListItem: boolean;
        /**
         *
         * Gets or sets the left indent value, in points, for the paragraph.
         *
         * [Api set: WordApi 1.1]
         */
        leftIndent: number;
        /**
         *
         * Gets or sets the line spacing, in points, for the specified paragraph. In the Word UI, this value is divided by 12.
         *
         * [Api set: WordApi 1.1]
         */
        lineSpacing: number;
        /**
         *
         * Gets or sets the amount of spacing, in grid lines, after the paragraph.
         *
         * [Api set: WordApi 1.1]
         */
        lineUnitAfter: number;
        /**
         *
         * Gets or sets the amount of spacing, in grid lines, before the paragraph.
         *
         * [Api set: WordApi 1.1]
         */
        lineUnitBefore: number;
        /**
         *
         * Gets or sets the outline level for the paragraph.
         *
         * [Api set: WordApi 1.1]
         */
        outlineLevel: number;
        /**
         *
         * Gets or sets the right indent value, in points, for the paragraph.
         *
         * [Api set: WordApi 1.1]
         */
        rightIndent: number;
        /**
         *
         * Gets or sets the spacing, in points, after the paragraph.
         *
         * [Api set: WordApi 1.1]
         */
        spaceAfter: number;
        /**
         *
         * Gets or sets the spacing, in points, before the paragraph.
         *
         * [Api set: WordApi 1.1]
         */
        spaceBefore: number;
        /**
         *
         * Gets or sets the style name for the paragraph. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
         *
         * [Api set: WordApi 1.1]
         */
        style: string;
        /**
         *
         * Gets or sets the built-in style name for the paragraph. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
         *
         * [Api set: WordApi 1.3]
         */
        styleBuiltIn: Word.Style | "Other" | "Normal" | "Heading1" | "Heading2" | "Heading3" | "Heading4" | "Heading5" | "Heading6" | "Heading7" | "Heading8" | "Heading9" | "Toc1" | "Toc2" | "Toc3" | "Toc4" | "Toc5" | "Toc6" | "Toc7" | "Toc8" | "Toc9" | "FootnoteText" | "Header" | "Footer" | "Caption" | "FootnoteReference" | "EndnoteReference" | "EndnoteText" | "Title" | "Subtitle" | "Hyperlink" | "Strong" | "Emphasis" | "NoSpacing" | "ListParagraph" | "Quote" | "IntenseQuote" | "SubtleEmphasis" | "IntenseEmphasis" | "SubtleReference" | "IntenseReference" | "BookTitle" | "Bibliography" | "TocHeading" | "TableGrid" | "PlainTable1" | "PlainTable2" | "PlainTable3" | "PlainTable4" | "PlainTable5" | "TableGridLight" | "GridTable1Light" | "GridTable1Light_Accent1" | "GridTable1Light_Accent2" | "GridTable1Light_Accent3" | "GridTable1Light_Accent4" | "GridTable1Light_Accent5" | "GridTable1Light_Accent6" | "GridTable2" | "GridTable2_Accent1" | "GridTable2_Accent2" | "GridTable2_Accent3" | "GridTable2_Accent4" | "GridTable2_Accent5" | "GridTable2_Accent6" | "GridTable3" | "GridTable3_Accent1" | "GridTable3_Accent2" | "GridTable3_Accent3" | "GridTable3_Accent4" | "GridTable3_Accent5" | "GridTable3_Accent6" | "GridTable4" | "GridTable4_Accent1" | "GridTable4_Accent2" | "GridTable4_Accent3" | "GridTable4_Accent4" | "GridTable4_Accent5" | "GridTable4_Accent6" | "GridTable5Dark" | "GridTable5Dark_Accent1" | "GridTable5Dark_Accent2" | "GridTable5Dark_Accent3" | "GridTable5Dark_Accent4" | "GridTable5Dark_Accent5" | "GridTable5Dark_Accent6" | "GridTable6Colorful" | "GridTable6Colorful_Accent1" | "GridTable6Colorful_Accent2" | "GridTable6Colorful_Accent3" | "GridTable6Colorful_Accent4" | "GridTable6Colorful_Accent5" | "GridTable6Colorful_Accent6" | "GridTable7Colorful" | "GridTable7Colorful_Accent1" | "GridTable7Colorful_Accent2" | "GridTable7Colorful_Accent3" | "GridTable7Colorful_Accent4" | "GridTable7Colorful_Accent5" | "GridTable7Colorful_Accent6" | "ListTable1Light" | "ListTable1Light_Accent1" | "ListTable1Light_Accent2" | "ListTable1Light_Accent3" | "ListTable1Light_Accent4" | "ListTable1Light_Accent5" | "ListTable1Light_Accent6" | "ListTable2" | "ListTable2_Accent1" | "ListTable2_Accent2" | "ListTable2_Accent3" | "ListTable2_Accent4" | "ListTable2_Accent5" | "ListTable2_Accent6" | "ListTable3" | "ListTable3_Accent1" | "ListTable3_Accent2" | "ListTable3_Accent3" | "ListTable3_Accent4" | "ListTable3_Accent5" | "ListTable3_Accent6" | "ListTable4" | "ListTable4_Accent1" | "ListTable4_Accent2" | "ListTable4_Accent3" | "ListTable4_Accent4" | "ListTable4_Accent5" | "ListTable4_Accent6" | "ListTable5Dark" | "ListTable5Dark_Accent1" | "ListTable5Dark_Accent2" | "ListTable5Dark_Accent3" | "ListTable5Dark_Accent4" | "ListTable5Dark_Accent5" | "ListTable5Dark_Accent6" | "ListTable6Colorful" | "ListTable6Colorful_Accent1" | "ListTable6Colorful_Accent2" | "ListTable6Colorful_Accent3" | "ListTable6Colorful_Accent4" | "ListTable6Colorful_Accent5" | "ListTable6Colorful_Accent6" | "ListTable7Colorful" | "ListTable7Colorful_Accent1" | "ListTable7Colorful_Accent2" | "ListTable7Colorful_Accent3" | "ListTable7Colorful_Accent4" | "ListTable7Colorful_Accent5" | "ListTable7Colorful_Accent6";
        /**
         *
         * Gets the level of the paragraph's table. It returns 0 if the paragraph is not in a table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly tableNestingLevel: number;
        /**
         *
         * Gets the text of the paragraph. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly text: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.Paragraph): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ParagraphUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.Paragraph): void;
        /**
         *
         * Lets the paragraph join an existing list at the specified level. Fails if the paragraph cannot join the list or if the paragraph is already a list item.
         *
         * [Api set: WordApi 1.3]
         *
         * @param listId Required. The ID of an existing list.
         * @param level Required. The level in the list.
         */
        attachToList(listId: number, level: number): Word.List;
        /**
         *
         * Clears the contents of the paragraph object. The user can perform the undo operation on the cleared content.
         *
         * [Api set: WordApi 1.1]
         */
        clear(): void;
        /**
         *
         * Deletes the paragraph and its content from the document.
         *
         * [Api set: WordApi 1.1]
         */
        delete(): void;
        /**
         *
         * Moves this paragraph out of its list, if the paragraph is a list item.
         *
         * [Api set: WordApi 1.3]
         */
        detachFromList(): void;
        /**
         *
         * Gets an HTML representation of the paragraph object. When rendered in a web page or HTML viewer, the formatting will be a close, but not exact, match for of the formatting of the document. This method does not return the exact same HTML for the same document on different platforms (Windows, Mac, Word Online, etc.). If you need exact fidelity, or consistency across platforms, use `Paragraph.getOoxml()` and convert the returned XML to HTML.
         *
         * [Api set: WordApi 1.1]
         */
        getHtml(): OfficeExtension.ClientResult<string>;
        /**
         *
         * Gets the next paragraph. Throws if the paragraph is the last one.
         *
         * [Api set: WordApi 1.3]
         */
        getNext(): Word.Paragraph;
        /**
         *
         * Gets the next paragraph. Returns a null object if the paragraph is the last one.
         *
         * [Api set: WordApi 1.3]
         */
        getNextOrNullObject(): Word.Paragraph;
        /**
         *
         * Gets the Office Open XML (OOXML) representation of the paragraph object.
         *
         * [Api set: WordApi 1.1]
         */
        getOoxml(): OfficeExtension.ClientResult<string>;
        /**
         *
         * Gets the previous paragraph. Throws if the paragraph is the first one.
         *
         * [Api set: WordApi 1.3]
         */
        getPrevious(): Word.Paragraph;
        /**
         *
         * Gets the previous paragraph. Returns a null object if the paragraph is the first one.
         *
         * [Api set: WordApi 1.3]
         */
        getPreviousOrNullObject(): Word.Paragraph;
        /**
         *
         * Gets the whole paragraph, or the starting or ending point of the paragraph, as a range.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rangeLocation Optional. The range location can be 'Whole', 'Start', 'End', 'After', or 'Content'.
         */
        getRange(rangeLocation?: Word.RangeLocation): Word.Range;
        /**
         *
         * Gets the whole paragraph, or the starting or ending point of the paragraph, as a range.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rangeLocation Optional. The range location can be 'Whole', 'Start', 'End', 'After', or 'Content'.
         */
        getRange(rangeLocation?: "Whole" | "Start" | "End" | "Before" | "After" | "Content"): Word.Range;
        /**
         *
         * Gets the text ranges in the paragraph by using punctuation marks and/or other ending marks.
         *
         * [Api set: WordApi 1.3]
         *
         * @param endingMarks Required. The punctuation marks and/or other ending marks as an array of strings.
         * @param trimSpacing Optional. Indicates whether to trim spacing characters (spaces, tabs, column breaks, and paragraph end marks) from the start and end of the ranges returned in the range collection. Default is false which indicates that spacing characters at the start and end of the ranges are included in the range collection.
         */
        getTextRanges(endingMarks: string[], trimSpacing?: boolean): Word.RangeCollection;
        /**
         *
         * Inserts a break at the specified location in the main document. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param breakType Required. The break type to add to the document.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertBreak(breakType: Word.BreakType, insertLocation: Word.InsertLocation): void;
        /**
         *
         * Inserts a break at the specified location in the main document. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param breakType Required. The break type to add to the document.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertBreak(breakType: "Page" | "Next" | "SectionNext" | "SectionContinuous" | "SectionEven" | "SectionOdd" | "Line", insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): void;
        /**
         *
         * Wraps the paragraph object with a rich text content control.
         *
         * [Api set: WordApi 1.1]
         */
        insertContentControl(): Word.ContentControl;
        /**
         *
         * Inserts a document into the paragraph at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param base64File Required. The base64 encoded content of a .docx file.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertFileFromBase64(base64File: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts a document into the paragraph at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param base64File Required. The base64 encoded content of a .docx file.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertFileFromBase64(base64File: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Inserts HTML into the paragraph at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param html Required. The HTML to be inserted in the paragraph.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertHtml(html: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts HTML into the paragraph at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param html Required. The HTML to be inserted in the paragraph.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertHtml(html: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Inserts a picture into the paragraph at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param base64EncodedImage Required. The base64 encoded image to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertInlinePictureFromBase64(base64EncodedImage: string, insertLocation: Word.InsertLocation): Word.InlinePicture;
        /**
         *
         * Inserts a picture into the paragraph at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param base64EncodedImage Required. The base64 encoded image to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertInlinePictureFromBase64(base64EncodedImage: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.InlinePicture;
        /**
         *
         * Inserts OOXML into the paragraph at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param ooxml Required. The OOXML to be inserted in the paragraph.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertOoxml(ooxml: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts OOXML into the paragraph at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param ooxml Required. The OOXML to be inserted in the paragraph.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertOoxml(ooxml: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Inserts a paragraph at the specified location. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param paragraphText Required. The paragraph text to be inserted.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertParagraph(paragraphText: string, insertLocation: Word.InsertLocation): Word.Paragraph;
        /**
         *
         * Inserts a paragraph at the specified location. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param paragraphText Required. The paragraph text to be inserted.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertParagraph(paragraphText: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Paragraph;
        /**
         *
         * Inserts a table with the specified number of rows and columns. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rowCount Required. The number of rows in the table.
         * @param columnCount Required. The number of columns in the table.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertTable(rowCount: number, columnCount: number, insertLocation: Word.InsertLocation, values?: string[][]): Word.Table;
        /**
         *
         * Inserts a table with the specified number of rows and columns. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rowCount Required. The number of rows in the table.
         * @param columnCount Required. The number of columns in the table.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertTable(rowCount: number, columnCount: number, insertLocation: "Before" | "After" | "Start" | "End" | "Replace", values?: string[][]): Word.Table;
        /**
         *
         * Inserts text into the paragraph at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param text Required. Text to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertText(text: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts text into the paragraph at the specified location. The insertLocation value can be 'Replace', 'Start', or 'End'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param text Required. Text to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', or 'End'.
         */
        insertText(text: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Performs a search with the specified SearchOptions on the scope of the paragraph object. The search results are a collection of range objects.
         *
         * [Api set: WordApi 1.1]
         *
         * @param searchText Required. The search text.
         * @param searchOptions Optional. Options for the search.
         */
        search(searchText: string, searchOptions?: Word.SearchOptions | {
            ignorePunct?: boolean;
            ignoreSpace?: boolean;
            matchCase?: boolean;
            matchPrefix?: boolean;
            matchSuffix?: boolean;
            matchWholeWord?: boolean;
            matchWildcards?: boolean;
        }): Word.RangeCollection;
        /**
         *
         * Selects and navigates the Word UI to the paragraph.
         *
         * [Api set: WordApi 1.1]
         *
         * @param selectionMode Optional. The selection mode can be 'Select', 'Start', or 'End'. 'Select' is the default.
         */
        select(selectionMode?: Word.SelectionMode): void;
        /**
         *
         * Selects and navigates the Word UI to the paragraph.
         *
         * [Api set: WordApi 1.1]
         *
         * @param selectionMode Optional. The selection mode can be 'Select', 'Start', or 'End'. 'Select' is the default.
         */
        select(selectionMode?: "Select" | "Start" | "End"): void;
        /**
         *
         * Splits the paragraph into child ranges by using delimiters.
         *
         * [Api set: WordApi 1.3]
         *
         * @param delimiters Required. The delimiters as an array of strings.
         * @param trimDelimiters Optional. Indicates whether to trim delimiters from the ranges in the range collection. Default is false which indicates that the delimiters are included in the ranges returned in the range collection.
         * @param trimSpacing Optional. Indicates whether to trim spacing characters (spaces, tabs, column breaks, and paragraph end marks) from the start and end of the ranges returned in the range collection. Default is false which indicates that spacing characters at the start and end of the ranges are included in the range collection.
         */
        split(delimiters: string[], trimDelimiters?: boolean, trimSpacing?: boolean): Word.RangeCollection;
        /**
         *
         * Starts a new list with this paragraph. Fails if the paragraph is already a list item.
         *
         * [Api set: WordApi 1.3]
         */
        startNewList(): Word.List;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.Paragraph` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.Paragraph` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.Paragraph` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.ParagraphLoadOptions): Word.Paragraph;
        load(option?: string | string[]): Word.Paragraph;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.Paragraph;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.Paragraph;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.Paragraph;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.Paragraph object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.ParagraphData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.ParagraphData;
    }
    /**
     *
     * Contains a collection of {@link Word.Paragraph} objects.
     *
     * [Api set: WordApi 1.1]
     */
    class ParagraphCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Word.Paragraph[];
        /**
         *
         * Gets the first paragraph in this collection. Throws if the collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirst(): Word.Paragraph;
        /**
         *
         * Gets the first paragraph in this collection. Returns a null object if the collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirstOrNullObject(): Word.Paragraph;
        /**
         *
         * Gets the last paragraph in this collection. Throws if the collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getLast(): Word.Paragraph;
        /**
         *
         * Gets the last paragraph in this collection. Returns a null object if the collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getLastOrNullObject(): Word.Paragraph;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.ParagraphCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.ParagraphCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.ParagraphCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.ParagraphCollectionLoadOptions & Word.Interfaces.CollectionLoadOptions): Word.ParagraphCollection;
        load(option?: string | string[]): Word.ParagraphCollection;
        load(option?: OfficeExtension.LoadOption): Word.ParagraphCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.ParagraphCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.ParagraphCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Word.ParagraphCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.ParagraphCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Word.Interfaces.ParagraphCollectionData;
    }
    /**
     *
     * Represents a contiguous area in a document.
     *
     * [Api set: WordApi 1.1]
     */
    class Range extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the collection of content control objects in the range. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly contentControls: Word.ContentControlCollection;
        /**
         *
         * Gets the text format of the range. Use this to get and set font name, size, color, and other properties. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly font: Word.Font;
        /**
         *
         * Gets the collection of inline picture objects in the range. Read-only.
         *
         * [Api set: WordApi 1.2]
         */
        readonly inlinePictures: Word.InlinePictureCollection;
        /**
         *
         * Gets the collection of list objects in the range. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly lists: Word.ListCollection;
        /**
         *
         * Gets the collection of paragraph objects in the range. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly paragraphs: Word.ParagraphCollection;
        /**
         *
         * Gets the parent body of the range. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentBody: Word.Body;
        /**
         *
         * Gets the content control that contains the range. Throws if there isn't a parent content control. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly parentContentControl: Word.ContentControl;
        /**
         *
         * Gets the content control that contains the range. Returns a null object if there isn't a parent content control. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentContentControlOrNullObject: Word.ContentControl;
        /**
         *
         * Gets the table that contains the range. Throws if it is not contained in a table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTable: Word.Table;
        /**
         *
         * Gets the table cell that contains the range. Throws if it is not contained in a table cell. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTableCell: Word.TableCell;
        /**
         *
         * Gets the table cell that contains the range. Returns a null object if it is not contained in a table cell. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTableCellOrNullObject: Word.TableCell;
        /**
         *
         * Gets the table that contains the range. Returns a null object if it is not contained in a table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTableOrNullObject: Word.Table;
        /**
         *
         * Gets the collection of table objects in the range. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly tables: Word.TableCollection;
        /**
         *
         * Gets the first hyperlink in the range, or sets a hyperlink on the range. All hyperlinks in the range are deleted when you set a new hyperlink on the range. Use a '#' to separate the address part from the optional location part.
         *
         * [Api set: WordApi 1.3]
         */
        hyperlink: string;
        /**
         *
         * Checks whether the range length is zero. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly isEmpty: boolean;
        /**
         *
         * Gets or sets the style name for the range. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
         *
         * [Api set: WordApi 1.1]
         */
        style: string;
        /**
         *
         * Gets or sets the built-in style name for the range. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
         *
         * [Api set: WordApi 1.3]
         */
        styleBuiltIn: Word.Style | "Other" | "Normal" | "Heading1" | "Heading2" | "Heading3" | "Heading4" | "Heading5" | "Heading6" | "Heading7" | "Heading8" | "Heading9" | "Toc1" | "Toc2" | "Toc3" | "Toc4" | "Toc5" | "Toc6" | "Toc7" | "Toc8" | "Toc9" | "FootnoteText" | "Header" | "Footer" | "Caption" | "FootnoteReference" | "EndnoteReference" | "EndnoteText" | "Title" | "Subtitle" | "Hyperlink" | "Strong" | "Emphasis" | "NoSpacing" | "ListParagraph" | "Quote" | "IntenseQuote" | "SubtleEmphasis" | "IntenseEmphasis" | "SubtleReference" | "IntenseReference" | "BookTitle" | "Bibliography" | "TocHeading" | "TableGrid" | "PlainTable1" | "PlainTable2" | "PlainTable3" | "PlainTable4" | "PlainTable5" | "TableGridLight" | "GridTable1Light" | "GridTable1Light_Accent1" | "GridTable1Light_Accent2" | "GridTable1Light_Accent3" | "GridTable1Light_Accent4" | "GridTable1Light_Accent5" | "GridTable1Light_Accent6" | "GridTable2" | "GridTable2_Accent1" | "GridTable2_Accent2" | "GridTable2_Accent3" | "GridTable2_Accent4" | "GridTable2_Accent5" | "GridTable2_Accent6" | "GridTable3" | "GridTable3_Accent1" | "GridTable3_Accent2" | "GridTable3_Accent3" | "GridTable3_Accent4" | "GridTable3_Accent5" | "GridTable3_Accent6" | "GridTable4" | "GridTable4_Accent1" | "GridTable4_Accent2" | "GridTable4_Accent3" | "GridTable4_Accent4" | "GridTable4_Accent5" | "GridTable4_Accent6" | "GridTable5Dark" | "GridTable5Dark_Accent1" | "GridTable5Dark_Accent2" | "GridTable5Dark_Accent3" | "GridTable5Dark_Accent4" | "GridTable5Dark_Accent5" | "GridTable5Dark_Accent6" | "GridTable6Colorful" | "GridTable6Colorful_Accent1" | "GridTable6Colorful_Accent2" | "GridTable6Colorful_Accent3" | "GridTable6Colorful_Accent4" | "GridTable6Colorful_Accent5" | "GridTable6Colorful_Accent6" | "GridTable7Colorful" | "GridTable7Colorful_Accent1" | "GridTable7Colorful_Accent2" | "GridTable7Colorful_Accent3" | "GridTable7Colorful_Accent4" | "GridTable7Colorful_Accent5" | "GridTable7Colorful_Accent6" | "ListTable1Light" | "ListTable1Light_Accent1" | "ListTable1Light_Accent2" | "ListTable1Light_Accent3" | "ListTable1Light_Accent4" | "ListTable1Light_Accent5" | "ListTable1Light_Accent6" | "ListTable2" | "ListTable2_Accent1" | "ListTable2_Accent2" | "ListTable2_Accent3" | "ListTable2_Accent4" | "ListTable2_Accent5" | "ListTable2_Accent6" | "ListTable3" | "ListTable3_Accent1" | "ListTable3_Accent2" | "ListTable3_Accent3" | "ListTable3_Accent4" | "ListTable3_Accent5" | "ListTable3_Accent6" | "ListTable4" | "ListTable4_Accent1" | "ListTable4_Accent2" | "ListTable4_Accent3" | "ListTable4_Accent4" | "ListTable4_Accent5" | "ListTable4_Accent6" | "ListTable5Dark" | "ListTable5Dark_Accent1" | "ListTable5Dark_Accent2" | "ListTable5Dark_Accent3" | "ListTable5Dark_Accent4" | "ListTable5Dark_Accent5" | "ListTable5Dark_Accent6" | "ListTable6Colorful" | "ListTable6Colorful_Accent1" | "ListTable6Colorful_Accent2" | "ListTable6Colorful_Accent3" | "ListTable6Colorful_Accent4" | "ListTable6Colorful_Accent5" | "ListTable6Colorful_Accent6" | "ListTable7Colorful" | "ListTable7Colorful_Accent1" | "ListTable7Colorful_Accent2" | "ListTable7Colorful_Accent3" | "ListTable7Colorful_Accent4" | "ListTable7Colorful_Accent5" | "ListTable7Colorful_Accent6";
        /**
         *
         * Gets the text of the range. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly text: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.Range): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.RangeUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.Range): void;
        /**
         *
         * Clears the contents of the range object. The user can perform the undo operation on the cleared content.
         *
         * [Api set: WordApi 1.1]
         */
        clear(): void;
        /**
         *
         * Compares this range's location with another range's location.
         *
         * [Api set: WordApi 1.3]
         *
         * @param range Required. The range to compare with this range.
         */
        compareLocationWith(range: Word.Range): OfficeExtension.ClientResult<Word.LocationRelation>;
        /**
         *
         * Deletes the range and its content from the document.
         *
         * [Api set: WordApi 1.1]
         */
        delete(): void;
        /**
         *
         * Returns a new range that extends from this range in either direction to cover another range. This range is not changed. Throws if the two ranges do not have a union.
         *
         * [Api set: WordApi 1.3]
         *
         * @param range Required. Another range.
         */
        expandTo(range: Word.Range): Word.Range;
        /**
         *
         * Returns a new range that extends from this range in either direction to cover another range. This range is not changed. Returns a null object if the two ranges do not have a union.
         *
         * [Api set: WordApi 1.3]
         *
         * @param range Required. Another range.
         */
        expandToOrNullObject(range: Word.Range): Word.Range;
        /**
         *
         * Gets an HTML representation of the range object. When rendered in a web page or HTML viewer, the formatting will be a close, but not exact, match for of the formatting of the document. This method does not return the exact same HTML for the same document on different platforms (Windows, Mac, Word Online, etc.). If you need exact fidelity, or consistency across platforms, use `Range.getOoxml()` and convert the returned XML to HTML.
         *
         * [Api set: WordApi 1.1]
         */
        getHtml(): OfficeExtension.ClientResult<string>;
        /**
         *
         * Gets hyperlink child ranges within the range.
         *
         * [Api set: WordApi 1.3]
         */
        getHyperlinkRanges(): Word.RangeCollection;
        /**
         *
         * Gets the next text range by using punctuation marks and/or other ending marks. Throws if this text range is the last one.
         *
         * [Api set: WordApi 1.3]
         *
         * @param endingMarks Required. The punctuation marks and/or other ending marks as an array of strings.
         * @param trimSpacing Optional. Indicates whether to trim spacing characters (spaces, tabs, column breaks, and paragraph end marks) from the start and end of the returned range. Default is false which indicates that spacing characters at the start and end of the range are included.
         */
        getNextTextRange(endingMarks: string[], trimSpacing?: boolean): Word.Range;
        /**
         *
         * Gets the next text range by using punctuation marks and/or other ending marks. Returns a null object if this text range is the last one.
         *
         * [Api set: WordApi 1.3]
         *
         * @param endingMarks Required. The punctuation marks and/or other ending marks as an array of strings.
         * @param trimSpacing Optional. Indicates whether to trim spacing characters (spaces, tabs, column breaks, and paragraph end marks) from the start and end of the returned range. Default is false which indicates that spacing characters at the start and end of the range are included.
         */
        getNextTextRangeOrNullObject(endingMarks: string[], trimSpacing?: boolean): Word.Range;
        /**
         *
         * Gets the OOXML representation of the range object.
         *
         * [Api set: WordApi 1.1]
         */
        getOoxml(): OfficeExtension.ClientResult<string>;
        /**
         *
         * Clones the range, or gets the starting or ending point of the range as a new range.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rangeLocation Optional. The range location can be 'Whole', 'Start', 'End', 'After', or 'Content'.
         */
        getRange(rangeLocation?: Word.RangeLocation): Word.Range;
        /**
         *
         * Clones the range, or gets the starting or ending point of the range as a new range.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rangeLocation Optional. The range location can be 'Whole', 'Start', 'End', 'After', or 'Content'.
         */
        getRange(rangeLocation?: "Whole" | "Start" | "End" | "Before" | "After" | "Content"): Word.Range;
        /**
         *
         * Gets the text child ranges in the range by using punctuation marks and/or other ending marks.
         *
         * [Api set: WordApi 1.3]
         *
         * @param endingMarks Required. The punctuation marks and/or other ending marks as an array of strings.
         * @param trimSpacing Optional. Indicates whether to trim spacing characters (spaces, tabs, column breaks, and paragraph end marks) from the start and end of the ranges returned in the range collection. Default is false which indicates that spacing characters at the start and end of the ranges are included in the range collection.
         */
        getTextRanges(endingMarks: string[], trimSpacing?: boolean): Word.RangeCollection;
        /**
         *
         * Inserts a break at the specified location in the main document. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param breakType Required. The break type to add.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertBreak(breakType: Word.BreakType, insertLocation: Word.InsertLocation): void;
        /**
         *
         * Inserts a break at the specified location in the main document. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param breakType Required. The break type to add.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertBreak(breakType: "Page" | "Next" | "SectionNext" | "SectionContinuous" | "SectionEven" | "SectionOdd" | "Line", insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): void;
        /**
         *
         * Wraps the range object with a rich text content control.
         *
         * [Api set: WordApi 1.1]
         */
        insertContentControl(): Word.ContentControl;
        /**
         *
         * Inserts a document at the specified location. The insertLocation value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param base64File Required. The base64 encoded content of a .docx file.
         * @param insertLocation Required. The value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         */
        insertFileFromBase64(base64File: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts a document at the specified location. The insertLocation value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param base64File Required. The base64 encoded content of a .docx file.
         * @param insertLocation Required. The value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         */
        insertFileFromBase64(base64File: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Inserts HTML at the specified location. The insertLocation value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param html Required. The HTML to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         */
        insertHtml(html: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts HTML at the specified location. The insertLocation value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param html Required. The HTML to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         */
        insertHtml(html: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Inserts a picture at the specified location. The insertLocation value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param base64EncodedImage Required. The base64 encoded image to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         */
        insertInlinePictureFromBase64(base64EncodedImage: string, insertLocation: Word.InsertLocation): Word.InlinePicture;
        /**
         *
         * Inserts a picture at the specified location. The insertLocation value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.2]
         *
         * @param base64EncodedImage Required. The base64 encoded image to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         */
        insertInlinePictureFromBase64(base64EncodedImage: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.InlinePicture;
        /**
         *
         * Inserts OOXML at the specified location.  The insertLocation value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param ooxml Required. The OOXML to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         */
        insertOoxml(ooxml: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts OOXML at the specified location.  The insertLocation value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param ooxml Required. The OOXML to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         */
        insertOoxml(ooxml: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Inserts a paragraph at the specified location. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param paragraphText Required. The paragraph text to be inserted.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertParagraph(paragraphText: string, insertLocation: Word.InsertLocation): Word.Paragraph;
        /**
         *
         * Inserts a paragraph at the specified location. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param paragraphText Required. The paragraph text to be inserted.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertParagraph(paragraphText: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Paragraph;
        /**
         *
         * Inserts a table with the specified number of rows and columns. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rowCount Required. The number of rows in the table.
         * @param columnCount Required. The number of columns in the table.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertTable(rowCount: number, columnCount: number, insertLocation: Word.InsertLocation, values?: string[][]): Word.Table;
        /**
         *
         * Inserts a table with the specified number of rows and columns. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rowCount Required. The number of rows in the table.
         * @param columnCount Required. The number of columns in the table.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertTable(rowCount: number, columnCount: number, insertLocation: "Before" | "After" | "Start" | "End" | "Replace", values?: string[][]): Word.Table;
        /**
         *
         * Inserts text at the specified location. The insertLocation value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param text Required. Text to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         */
        insertText(text: string, insertLocation: Word.InsertLocation): Word.Range;
        /**
         *
         * Inserts text at the specified location. The insertLocation value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         *
         * [Api set: WordApi 1.1]
         *
         * @param text Required. Text to be inserted.
         * @param insertLocation Required. The value can be 'Replace', 'Start', 'End', 'Before', or 'After'.
         */
        insertText(text: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Range;
        /**
         *
         * Returns a new range as the intersection of this range with another range. This range is not changed. Throws if the two ranges are not overlapped or adjacent.
         *
         * [Api set: WordApi 1.3]
         *
         * @param range Required. Another range.
         */
        intersectWith(range: Word.Range): Word.Range;
        /**
         *
         * Returns a new range as the intersection of this range with another range. This range is not changed. Returns a null object if the two ranges are not overlapped or adjacent.
         *
         * [Api set: WordApi 1.3]
         *
         * @param range Required. Another range.
         */
        intersectWithOrNullObject(range: Word.Range): Word.Range;
        /**
         *
         * Performs a search with the specified SearchOptions on the scope of the range object. The search results are a collection of range objects.
         *
         * [Api set: WordApi 1.1]
         *
         * @param searchText Required. The search text.
         * @param searchOptions Optional. Options for the search.
         */
        search(searchText: string, searchOptions?: Word.SearchOptions | {
            ignorePunct?: boolean;
            ignoreSpace?: boolean;
            matchCase?: boolean;
            matchPrefix?: boolean;
            matchSuffix?: boolean;
            matchWholeWord?: boolean;
            matchWildcards?: boolean;
        }): Word.RangeCollection;
        /**
         *
         * Selects and navigates the Word UI to the range.
         *
         * [Api set: WordApi 1.1]
         *
         * @param selectionMode Optional. The selection mode can be 'Select', 'Start', or 'End'. 'Select' is the default.
         */
        select(selectionMode?: Word.SelectionMode): void;
        /**
         *
         * Selects and navigates the Word UI to the range.
         *
         * [Api set: WordApi 1.1]
         *
         * @param selectionMode Optional. The selection mode can be 'Select', 'Start', or 'End'. 'Select' is the default.
         */
        select(selectionMode?: "Select" | "Start" | "End"): void;
        /**
         *
         * Splits the range into child ranges by using delimiters.
         *
         * [Api set: WordApi 1.3]
         *
         * @param delimiters Required. The delimiters as an array of strings.
         * @param multiParagraphs Optional. Indicates whether a returned child range can cover multiple paragraphs. Default is false which indicates that the paragraph boundaries are also used as delimiters.
         * @param trimDelimiters Optional. Indicates whether to trim delimiters from the ranges in the range collection. Default is false which indicates that the delimiters are included in the ranges returned in the range collection.
         * @param trimSpacing Optional. Indicates whether to trim spacing characters (spaces, tabs, column breaks, and paragraph end marks) from the start and end of the ranges returned in the range collection. Default is false which indicates that spacing characters at the start and end of the ranges are included in the range collection.
         */
        split(delimiters: string[], multiParagraphs?: boolean, trimDelimiters?: boolean, trimSpacing?: boolean): Word.RangeCollection;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.Range` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.Range` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.Range` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.RangeLoadOptions): Word.Range;
        load(option?: string | string[]): Word.Range;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.Range;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.Range;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.Range;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.Range object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.RangeData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.RangeData;
    }
    /**
     *
     * Contains a collection of {@link Word.Range} objects.
     *
     * [Api set: WordApi 1.1]
     */
    class RangeCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Word.Range[];
        /**
         *
         * Gets the first range in this collection. Throws if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirst(): Word.Range;
        /**
         *
         * Gets the first range in this collection. Returns a null object if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirstOrNullObject(): Word.Range;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.RangeCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.RangeCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.RangeCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.RangeCollectionLoadOptions & Word.Interfaces.CollectionLoadOptions): Word.RangeCollection;
        load(option?: string | string[]): Word.RangeCollection;
        load(option?: OfficeExtension.LoadOption): Word.RangeCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.RangeCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.RangeCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Word.RangeCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.RangeCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Word.Interfaces.RangeCollectionData;
    }
    /**
     *
     * Specifies the options to be included in a search operation.
     *
     * [Api set: WordApi 1.1]
     */
    class SearchOptions extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        matchWildCards: boolean;
        /**
         *
         * Gets or sets a value that indicates whether to ignore all punctuation characters between words. Corresponds to the Ignore punctuation check box in the Find and Replace dialog box.
         *
         * [Api set: WordApi 1.1]
         */
        ignorePunct: boolean;
        /**
         *
         * Gets or sets a value that indicates whether to ignore all whitespace between words. Corresponds to the Ignore whitespace characters check box in the Find and Replace dialog box.
         *
         * [Api set: WordApi 1.1]
         */
        ignoreSpace: boolean;
        /**
         *
         * Gets or sets a value that indicates whether to perform a case sensitive search. Corresponds to the Match case check box in the Find and Replace dialog box.
         *
         * [Api set: WordApi 1.1]
         */
        matchCase: boolean;
        /**
         *
         * Gets or sets a value that indicates whether to match words that begin with the search string. Corresponds to the Match prefix check box in the Find and Replace dialog box.
         *
         * [Api set: WordApi 1.1]
         */
        matchPrefix: boolean;
        /**
         *
         * Gets or sets a value that indicates whether to match words that end with the search string. Corresponds to the Match suffix check box in the Find and Replace dialog box.
         *
         * [Api set: WordApi 1.1]
         */
        matchSuffix: boolean;
        /**
         *
         * Gets or sets a value that indicates whether to find operation only entire words, not text that is part of a larger word. Corresponds to the Find whole words only check box in the Find and Replace dialog box.
         *
         * [Api set: WordApi 1.1]
         */
        matchWholeWord: boolean;
        /**
         *
         * Gets or sets a value that indicates whether the search will be performed using special search operators. Corresponds to the Use wildcards check box in the Find and Replace dialog box.
         *
         * [Api set: WordApi 1.1]
         */
        matchWildcards: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.SearchOptions): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.SearchOptionsUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.SearchOptions): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.SearchOptions` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.SearchOptions` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.SearchOptions` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.SearchOptionsLoadOptions): Word.SearchOptions;
        load(option?: string | string[]): Word.SearchOptions;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.SearchOptions;
        /**
         * Create a new instance of Word.SearchOptions object
         */
        static newObject(context: OfficeExtension.ClientRequestContext): Word.SearchOptions;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.SearchOptions object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.SearchOptionsData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.SearchOptionsData;
    }
    /**
     *
     * Represents a section in a Word document.
     *
     * [Api set: WordApi 1.1]
     */
    class Section extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the body object of the section. This does not include the header/footer and other section metadata. Read-only.
         *
         * [Api set: WordApi 1.1]
         */
        readonly body: Word.Body;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.Section): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.SectionUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.Section): void;
        /**
         *
         * Gets one of the section's footers.
         *
         * [Api set: WordApi 1.1]
         *
         * @param type Required. The type of footer to return. This value can be: 'Primary', 'FirstPage', or 'EvenPages'.
         */
        getFooter(type: Word.HeaderFooterType): Word.Body;
        /**
         *
         * Gets one of the section's footers.
         *
         * [Api set: WordApi 1.1]
         *
         * @param type Required. The type of footer to return. This value can be: 'Primary', 'FirstPage', or 'EvenPages'.
         */
        getFooter(type: "Primary" | "FirstPage" | "EvenPages"): Word.Body;
        /**
         *
         * Gets one of the section's headers.
         *
         * [Api set: WordApi 1.1]
         *
         * @param type Required. The type of header to return. This value can be: 'Primary', 'FirstPage', or 'EvenPages'.
         */
        getHeader(type: Word.HeaderFooterType): Word.Body;
        /**
         *
         * Gets one of the section's headers.
         *
         * [Api set: WordApi 1.1]
         *
         * @param type Required. The type of header to return. This value can be: 'Primary', 'FirstPage', or 'EvenPages'.
         */
        getHeader(type: "Primary" | "FirstPage" | "EvenPages"): Word.Body;
        /**
         *
         * Gets the next section. Throws if this section is the last one.
         *
         * [Api set: WordApi 1.3]
         */
        getNext(): Word.Section;
        /**
         *
         * Gets the next section. Returns a null object if this section is the last one.
         *
         * [Api set: WordApi 1.3]
         */
        getNextOrNullObject(): Word.Section;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.Section` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.Section` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.Section` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.SectionLoadOptions): Word.Section;
        load(option?: string | string[]): Word.Section;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.Section;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.Section;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.Section;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.Section object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.SectionData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.SectionData;
    }
    /**
     *
     * Contains the collection of the document's {@link Word.Section} objects.
     *
     * [Api set: WordApi 1.1]
     */
    class SectionCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Word.Section[];
        /**
         *
         * Gets the first section in this collection. Throws if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirst(): Word.Section;
        /**
         *
         * Gets the first section in this collection. Returns a null object if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirstOrNullObject(): Word.Section;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.SectionCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.SectionCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.SectionCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.SectionCollectionLoadOptions & Word.Interfaces.CollectionLoadOptions): Word.SectionCollection;
        load(option?: string | string[]): Word.SectionCollection;
        load(option?: OfficeExtension.LoadOption): Word.SectionCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.SectionCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.SectionCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Word.SectionCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.SectionCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Word.Interfaces.SectionCollectionData;
    }
    /**
     *
     * Represents a table in a Word document.
     *
     * [Api set: WordApi 1.3]
     */
    class Table extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the font. Use this to get and set font name, size, color, and other properties. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly font: Word.Font;
        /**
         *
         * Gets the parent body of the table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentBody: Word.Body;
        /**
         *
         * Gets the content control that contains the table. Throws if there isn't a parent content control. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentContentControl: Word.ContentControl;
        /**
         *
         * Gets the content control that contains the table. Returns a null object if there isn't a parent content control. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentContentControlOrNullObject: Word.ContentControl;
        /**
         *
         * Gets the table that contains this table. Throws if it is not contained in a table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTable: Word.Table;
        /**
         *
         * Gets the table cell that contains this table. Throws if it is not contained in a table cell. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTableCell: Word.TableCell;
        /**
         *
         * Gets the table cell that contains this table. Returns a null object if it is not contained in a table cell. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTableCellOrNullObject: Word.TableCell;
        /**
         *
         * Gets the table that contains this table. Returns a null object if it is not contained in a table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTableOrNullObject: Word.Table;
        /**
         *
         * Gets all of the table rows. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly rows: Word.TableRowCollection;
        /**
         *
         * Gets the child tables nested one level deeper. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly tables: Word.TableCollection;
        /**
         *
         * Gets or sets the alignment of the table against the page column. The value can be 'Left', 'Centered', or 'Right'.
         *
         * [Api set: WordApi 1.3]
         */
        alignment: Word.Alignment | "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified";
        /**
         *
         * Gets and sets the number of header rows.
         *
         * [Api set: WordApi 1.3]
         */
        headerRowCount: number;
        /**
         *
         * Gets and sets the horizontal alignment of every cell in the table. The value can be 'Left', 'Centered', 'Right', or 'Justified'.
         *
         * [Api set: WordApi 1.3]
         */
        horizontalAlignment: Word.Alignment | "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified";
        /**
         *
         * Indicates whether all of the table rows are uniform. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly isUniform: boolean;
        /**
         *
         * Gets the nesting level of the table. Top-level tables have level 1. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly nestingLevel: number;
        /**
         *
         * Gets the number of rows in the table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly rowCount: number;
        /**
         *
         * Gets and sets the shading color. Color is specified in "#RRGGBB" format or by using the color name.
         *
         * [Api set: WordApi 1.3]
         */
        shadingColor: string;
        /**
         *
         * Gets or sets the style name for the table. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
         *
         * [Api set: WordApi 1.3]
         */
        style: string;
        /**
         *
         * Gets and sets whether the table has banded columns.
         *
         * [Api set: WordApi 1.3]
         */
        styleBandedColumns: boolean;
        /**
         *
         * Gets and sets whether the table has banded rows.
         *
         * [Api set: WordApi 1.3]
         */
        styleBandedRows: boolean;
        /**
         *
         * Gets or sets the built-in style name for the table. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
         *
         * [Api set: WordApi 1.3]
         */
        styleBuiltIn: Word.Style | "Other" | "Normal" | "Heading1" | "Heading2" | "Heading3" | "Heading4" | "Heading5" | "Heading6" | "Heading7" | "Heading8" | "Heading9" | "Toc1" | "Toc2" | "Toc3" | "Toc4" | "Toc5" | "Toc6" | "Toc7" | "Toc8" | "Toc9" | "FootnoteText" | "Header" | "Footer" | "Caption" | "FootnoteReference" | "EndnoteReference" | "EndnoteText" | "Title" | "Subtitle" | "Hyperlink" | "Strong" | "Emphasis" | "NoSpacing" | "ListParagraph" | "Quote" | "IntenseQuote" | "SubtleEmphasis" | "IntenseEmphasis" | "SubtleReference" | "IntenseReference" | "BookTitle" | "Bibliography" | "TocHeading" | "TableGrid" | "PlainTable1" | "PlainTable2" | "PlainTable3" | "PlainTable4" | "PlainTable5" | "TableGridLight" | "GridTable1Light" | "GridTable1Light_Accent1" | "GridTable1Light_Accent2" | "GridTable1Light_Accent3" | "GridTable1Light_Accent4" | "GridTable1Light_Accent5" | "GridTable1Light_Accent6" | "GridTable2" | "GridTable2_Accent1" | "GridTable2_Accent2" | "GridTable2_Accent3" | "GridTable2_Accent4" | "GridTable2_Accent5" | "GridTable2_Accent6" | "GridTable3" | "GridTable3_Accent1" | "GridTable3_Accent2" | "GridTable3_Accent3" | "GridTable3_Accent4" | "GridTable3_Accent5" | "GridTable3_Accent6" | "GridTable4" | "GridTable4_Accent1" | "GridTable4_Accent2" | "GridTable4_Accent3" | "GridTable4_Accent4" | "GridTable4_Accent5" | "GridTable4_Accent6" | "GridTable5Dark" | "GridTable5Dark_Accent1" | "GridTable5Dark_Accent2" | "GridTable5Dark_Accent3" | "GridTable5Dark_Accent4" | "GridTable5Dark_Accent5" | "GridTable5Dark_Accent6" | "GridTable6Colorful" | "GridTable6Colorful_Accent1" | "GridTable6Colorful_Accent2" | "GridTable6Colorful_Accent3" | "GridTable6Colorful_Accent4" | "GridTable6Colorful_Accent5" | "GridTable6Colorful_Accent6" | "GridTable7Colorful" | "GridTable7Colorful_Accent1" | "GridTable7Colorful_Accent2" | "GridTable7Colorful_Accent3" | "GridTable7Colorful_Accent4" | "GridTable7Colorful_Accent5" | "GridTable7Colorful_Accent6" | "ListTable1Light" | "ListTable1Light_Accent1" | "ListTable1Light_Accent2" | "ListTable1Light_Accent3" | "ListTable1Light_Accent4" | "ListTable1Light_Accent5" | "ListTable1Light_Accent6" | "ListTable2" | "ListTable2_Accent1" | "ListTable2_Accent2" | "ListTable2_Accent3" | "ListTable2_Accent4" | "ListTable2_Accent5" | "ListTable2_Accent6" | "ListTable3" | "ListTable3_Accent1" | "ListTable3_Accent2" | "ListTable3_Accent3" | "ListTable3_Accent4" | "ListTable3_Accent5" | "ListTable3_Accent6" | "ListTable4" | "ListTable4_Accent1" | "ListTable4_Accent2" | "ListTable4_Accent3" | "ListTable4_Accent4" | "ListTable4_Accent5" | "ListTable4_Accent6" | "ListTable5Dark" | "ListTable5Dark_Accent1" | "ListTable5Dark_Accent2" | "ListTable5Dark_Accent3" | "ListTable5Dark_Accent4" | "ListTable5Dark_Accent5" | "ListTable5Dark_Accent6" | "ListTable6Colorful" | "ListTable6Colorful_Accent1" | "ListTable6Colorful_Accent2" | "ListTable6Colorful_Accent3" | "ListTable6Colorful_Accent4" | "ListTable6Colorful_Accent5" | "ListTable6Colorful_Accent6" | "ListTable7Colorful" | "ListTable7Colorful_Accent1" | "ListTable7Colorful_Accent2" | "ListTable7Colorful_Accent3" | "ListTable7Colorful_Accent4" | "ListTable7Colorful_Accent5" | "ListTable7Colorful_Accent6";
        /**
         *
         * Gets and sets whether the table has a first column with a special style.
         *
         * [Api set: WordApi 1.3]
         */
        styleFirstColumn: boolean;
        /**
         *
         * Gets and sets whether the table has a last column with a special style.
         *
         * [Api set: WordApi 1.3]
         */
        styleLastColumn: boolean;
        /**
         *
         * Gets and sets whether the table has a total (last) row with a special style.
         *
         * [Api set: WordApi 1.3]
         */
        styleTotalRow: boolean;
        /**
         *
         * Gets and sets the text values in the table, as a 2D Javascript array.
         *
         * [Api set: WordApi 1.3]
         */
        values: string[][];
        /**
         *
         * Gets and sets the vertical alignment of every cell in the table. The value can be 'Top', 'Center', or 'Bottom'.
         *
         * [Api set: WordApi 1.3]
         */
        verticalAlignment: Word.VerticalAlignment | "Mixed" | "Top" | "Center" | "Bottom";
        /**
         *
         * Gets and sets the width of the table in points.
         *
         * [Api set: WordApi 1.3]
         */
        width: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.Table): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.TableUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.Table): void;
        /**
         *
         * Adds columns to the start or end of the table, using the first or last existing column as a template. This is applicable to uniform tables. The string values, if specified, are set in the newly inserted rows.
         *
         * [Api set: WordApi 1.3]
         *
         * @param insertLocation Required. It can be 'Start' or 'End', corresponding to the appropriate side of the table.
         * @param columnCount Required. Number of columns to add.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        addColumns(insertLocation: Word.InsertLocation, columnCount: number, values?: string[][]): void;
        /**
         *
         * Adds columns to the start or end of the table, using the first or last existing column as a template. This is applicable to uniform tables. The string values, if specified, are set in the newly inserted rows.
         *
         * [Api set: WordApi 1.3]
         *
         * @param insertLocation Required. It can be 'Start' or 'End', corresponding to the appropriate side of the table.
         * @param columnCount Required. Number of columns to add.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        addColumns(insertLocation: "Before" | "After" | "Start" | "End" | "Replace", columnCount: number, values?: string[][]): void;
        /**
         *
         * Adds rows to the start or end of the table, using the first or last existing row as a template. The string values, if specified, are set in the newly inserted rows.
         *
         * [Api set: WordApi 1.3]
         *
         * @param insertLocation Required. It can be 'Start' or 'End'.
         * @param rowCount Required. Number of rows to add.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        addRows(insertLocation: Word.InsertLocation, rowCount: number, values?: string[][]): Word.TableRowCollection;
        /**
         *
         * Adds rows to the start or end of the table, using the first or last existing row as a template. The string values, if specified, are set in the newly inserted rows.
         *
         * [Api set: WordApi 1.3]
         *
         * @param insertLocation Required. It can be 'Start' or 'End'.
         * @param rowCount Required. Number of rows to add.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        addRows(insertLocation: "Before" | "After" | "Start" | "End" | "Replace", rowCount: number, values?: string[][]): Word.TableRowCollection;
        /**
         *
         * Autofits the table columns to the width of the window.
         *
         * [Api set: WordApi 1.3]
         */
        autoFitWindow(): void;
        /**
         *
         * Clears the contents of the table.
         *
         * [Api set: WordApi 1.3]
         */
        clear(): void;
        /**
         *
         * Deletes the entire table.
         *
         * [Api set: WordApi 1.3]
         */
        delete(): void;
        /**
         *
         * Deletes specific columns. This is applicable to uniform tables.
         *
         * [Api set: WordApi 1.3]
         *
         * @param columnIndex Required. The first column to delete.
         * @param columnCount Optional. The number of columns to delete. Default 1.
         */
        deleteColumns(columnIndex: number, columnCount?: number): void;
        /**
         *
         * Deletes specific rows.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rowIndex Required. The first row to delete.
         * @param rowCount Optional. The number of rows to delete. Default 1.
         */
        deleteRows(rowIndex: number, rowCount?: number): void;
        /**
         *
         * Distributes the column widths evenly. This is applicable to uniform tables.
         *
         * [Api set: WordApi 1.3]
         */
        distributeColumns(): void;
        /**
         *
         * Gets the border style for the specified border.
         *
         * [Api set: WordApi 1.3]
         *
         * @param borderLocation Required. The border location.
         */
        getBorder(borderLocation: Word.BorderLocation): Word.TableBorder;
        /**
         *
         * Gets the border style for the specified border.
         *
         * [Api set: WordApi 1.3]
         *
         * @param borderLocation Required. The border location.
         */
        getBorder(borderLocation: "Top" | "Left" | "Bottom" | "Right" | "InsideHorizontal" | "InsideVertical" | "Inside" | "Outside" | "All"): Word.TableBorder;
        /**
         *
         * Gets the table cell at a specified row and column. Throws if the specified table cell does not exist.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rowIndex Required. The index of the row.
         * @param cellIndex Required. The index of the cell in the row.
         */
        getCell(rowIndex: number, cellIndex: number): Word.TableCell;
        /**
         *
         * Gets the table cell at a specified row and column. Returns a null object if the specified table cell does not exist.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rowIndex Required. The index of the row.
         * @param cellIndex Required. The index of the cell in the row.
         */
        getCellOrNullObject(rowIndex: number, cellIndex: number): Word.TableCell;
        /**
         *
         * Gets cell padding in points.
         *
         * [Api set: WordApi 1.3]
         *
         * @param cellPaddingLocation Required. The cell padding location can be 'Top', 'Left', 'Bottom', or 'Right'.
         */
        getCellPadding(cellPaddingLocation: Word.CellPaddingLocation): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets cell padding in points.
         *
         * [Api set: WordApi 1.3]
         *
         * @param cellPaddingLocation Required. The cell padding location can be 'Top', 'Left', 'Bottom', or 'Right'.
         */
        getCellPadding(cellPaddingLocation: "Top" | "Left" | "Bottom" | "Right"): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets the next table. Throws if this table is the last one.
         *
         * [Api set: WordApi 1.3]
         */
        getNext(): Word.Table;
        /**
         *
         * Gets the next table. Returns a null object if this table is the last one.
         *
         * [Api set: WordApi 1.3]
         */
        getNextOrNullObject(): Word.Table;
        /**
         *
         * Gets the paragraph after the table. Throws if there isn't a paragraph after the table.
         *
         * [Api set: WordApi 1.3]
         */
        getParagraphAfter(): Word.Paragraph;
        /**
         *
         * Gets the paragraph after the table. Returns a null object if there isn't a paragraph after the table.
         *
         * [Api set: WordApi 1.3]
         */
        getParagraphAfterOrNullObject(): Word.Paragraph;
        /**
         *
         * Gets the paragraph before the table. Throws if there isn't a paragraph before the table.
         *
         * [Api set: WordApi 1.3]
         */
        getParagraphBefore(): Word.Paragraph;
        /**
         *
         * Gets the paragraph before the table. Returns a null object if there isn't a paragraph before the table.
         *
         * [Api set: WordApi 1.3]
         */
        getParagraphBeforeOrNullObject(): Word.Paragraph;
        /**
         *
         * Gets the range that contains this table, or the range at the start or end of the table.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rangeLocation Optional. The range location can be 'Whole', 'Start', 'End', or 'After'.
         */
        getRange(rangeLocation?: Word.RangeLocation): Word.Range;
        /**
         *
         * Gets the range that contains this table, or the range at the start or end of the table.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rangeLocation Optional. The range location can be 'Whole', 'Start', 'End', or 'After'.
         */
        getRange(rangeLocation?: "Whole" | "Start" | "End" | "Before" | "After" | "Content"): Word.Range;
        /**
         *
         * Inserts a content control on the table.
         *
         * [Api set: WordApi 1.3]
         */
        insertContentControl(): Word.ContentControl;
        /**
         *
         * Inserts a paragraph at the specified location. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.3]
         *
         * @param paragraphText Required. The paragraph text to be inserted.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertParagraph(paragraphText: string, insertLocation: Word.InsertLocation): Word.Paragraph;
        /**
         *
         * Inserts a paragraph at the specified location. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.3]
         *
         * @param paragraphText Required. The paragraph text to be inserted.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         */
        insertParagraph(paragraphText: string, insertLocation: "Before" | "After" | "Start" | "End" | "Replace"): Word.Paragraph;
        /**
         *
         * Inserts a table with the specified number of rows and columns. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rowCount Required. The number of rows in the table.
         * @param columnCount Required. The number of columns in the table.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertTable(rowCount: number, columnCount: number, insertLocation: Word.InsertLocation, values?: string[][]): Word.Table;
        /**
         *
         * Inserts a table with the specified number of rows and columns. The insertLocation value can be 'Before' or 'After'.
         *
         * [Api set: WordApi 1.3]
         *
         * @param rowCount Required. The number of rows in the table.
         * @param columnCount Required. The number of columns in the table.
         * @param insertLocation Required. The value can be 'Before' or 'After'.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertTable(rowCount: number, columnCount: number, insertLocation: "Before" | "After" | "Start" | "End" | "Replace", values?: string[][]): Word.Table;
        /**
         *
         * Performs a search with the specified SearchOptions on the scope of the table object. The search results are a collection of range objects.
         *
         * [Api set: WordApi 1.3]
         *
         * @param searchText Required. The search text.
         * @param searchOptions Optional. Options for the search.
         */
        search(searchText: string, searchOptions?: Word.SearchOptions | {
            ignorePunct?: boolean;
            ignoreSpace?: boolean;
            matchCase?: boolean;
            matchPrefix?: boolean;
            matchSuffix?: boolean;
            matchWholeWord?: boolean;
            matchWildcards?: boolean;
        }): Word.RangeCollection;
        /**
         *
         * Selects the table, or the position at the start or end of the table, and navigates the Word UI to it.
         *
         * [Api set: WordApi 1.3]
         *
         * @param selectionMode Optional. The selection mode can be 'Select', 'Start', or 'End'. 'Select' is the default.
         */
        select(selectionMode?: Word.SelectionMode): void;
        /**
         *
         * Selects the table, or the position at the start or end of the table, and navigates the Word UI to it.
         *
         * [Api set: WordApi 1.3]
         *
         * @param selectionMode Optional. The selection mode can be 'Select', 'Start', or 'End'. 'Select' is the default.
         */
        select(selectionMode?: "Select" | "Start" | "End"): void;
        /**
         *
         * Sets cell padding in points.
         *
         * [Api set: WordApi 1.3]
         *
         * @param cellPaddingLocation Required. The cell padding location can be 'Top', 'Left', 'Bottom', or 'Right'.
         * @param cellPadding Required. The cell padding.
         */
        setCellPadding(cellPaddingLocation: Word.CellPaddingLocation, cellPadding: number): void;
        /**
         *
         * Sets cell padding in points.
         *
         * [Api set: WordApi 1.3]
         *
         * @param cellPaddingLocation Required. The cell padding location can be 'Top', 'Left', 'Bottom', or 'Right'.
         * @param cellPadding Required. The cell padding.
         */
        setCellPadding(cellPaddingLocation: "Top" | "Left" | "Bottom" | "Right", cellPadding: number): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.Table` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.Table` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.Table` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.TableLoadOptions): Word.Table;
        load(option?: string | string[]): Word.Table;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.Table;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.Table;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.Table;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.Table object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.TableData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.TableData;
    }
    /**
     *
     * Contains the collection of the document's Table objects.
     *
     * [Api set: WordApi 1.3]
     */
    class TableCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Word.Table[];
        /**
         *
         * Gets the first table in this collection. Throws if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirst(): Word.Table;
        /**
         *
         * Gets the first table in this collection. Returns a null object if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirstOrNullObject(): Word.Table;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.TableCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.TableCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.TableCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.TableCollectionLoadOptions & Word.Interfaces.CollectionLoadOptions): Word.TableCollection;
        load(option?: string | string[]): Word.TableCollection;
        load(option?: OfficeExtension.LoadOption): Word.TableCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.TableCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.TableCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Word.TableCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.TableCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Word.Interfaces.TableCollectionData;
    }
    /**
     *
     * Represents a row in a Word document.
     *
     * [Api set: WordApi 1.3]
     */
    class TableRow extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets cells. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly cells: Word.TableCellCollection;
        /**
         *
         * Gets the font. Use this to get and set font name, size, color, and other properties. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly font: Word.Font;
        /**
         *
         * Gets parent table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTable: Word.Table;
        /**
         *
         * Gets the number of cells in the row. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly cellCount: number;
        /**
         *
         * Gets and sets the horizontal alignment of every cell in the row. The value can be 'Left', 'Centered', 'Right', or 'Justified'.
         *
         * [Api set: WordApi 1.3]
         */
        horizontalAlignment: Word.Alignment | "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified";
        /**
         *
         * Checks whether the row is a header row. Read-only. To set the number of header rows, use HeaderRowCount on the Table object.
         *
         * [Api set: WordApi 1.3]
         */
        readonly isHeader: boolean;
        /**
         *
         * Gets and sets the preferred height of the row in points.
         *
         * [Api set: WordApi 1.3]
         */
        preferredHeight: number;
        /**
         *
         * Gets the index of the row in its parent table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly rowIndex: number;
        /**
         *
         * Gets and sets the shading color. Color is specified in "#RRGGBB" format or by using the color name.
         *
         * [Api set: WordApi 1.3]
         */
        shadingColor: string;
        /**
         *
         * Gets and sets the text values in the row, as a 2D Javascript array.
         *
         * [Api set: WordApi 1.3]
         */
        values: string[][];
        /**
         *
         * Gets and sets the vertical alignment of the cells in the row. The value can be 'Top', 'Center', or 'Bottom'.
         *
         * [Api set: WordApi 1.3]
         */
        verticalAlignment: Word.VerticalAlignment | "Mixed" | "Top" | "Center" | "Bottom";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.TableRow): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.TableRowUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.TableRow): void;
        /**
         *
         * Clears the contents of the row.
         *
         * [Api set: WordApi 1.3]
         */
        clear(): void;
        /**
         *
         * Deletes the entire row.
         *
         * [Api set: WordApi 1.3]
         */
        delete(): void;
        /**
         *
         * Gets the border style of the cells in the row.
         *
         * [Api set: WordApi 1.3]
         *
         * @param borderLocation Required. The border location.
         */
        getBorder(borderLocation: Word.BorderLocation): Word.TableBorder;
        /**
         *
         * Gets the border style of the cells in the row.
         *
         * [Api set: WordApi 1.3]
         *
         * @param borderLocation Required. The border location.
         */
        getBorder(borderLocation: "Top" | "Left" | "Bottom" | "Right" | "InsideHorizontal" | "InsideVertical" | "Inside" | "Outside" | "All"): Word.TableBorder;
        /**
         *
         * Gets cell padding in points.
         *
         * [Api set: WordApi 1.3]
         *
         * @param cellPaddingLocation Required. The cell padding location can be 'Top', 'Left', 'Bottom', or 'Right'.
         */
        getCellPadding(cellPaddingLocation: Word.CellPaddingLocation): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets cell padding in points.
         *
         * [Api set: WordApi 1.3]
         *
         * @param cellPaddingLocation Required. The cell padding location can be 'Top', 'Left', 'Bottom', or 'Right'.
         */
        getCellPadding(cellPaddingLocation: "Top" | "Left" | "Bottom" | "Right"): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets the next row. Throws if this row is the last one.
         *
         * [Api set: WordApi 1.3]
         */
        getNext(): Word.TableRow;
        /**
         *
         * Gets the next row. Returns a null object if this row is the last one.
         *
         * [Api set: WordApi 1.3]
         */
        getNextOrNullObject(): Word.TableRow;
        /**
         *
         * Inserts rows using this row as a template. If values are specified, inserts the values into the new rows.
         *
         * [Api set: WordApi 1.3]
         *
         * @param insertLocation Required. Where the new rows should be inserted, relative to the current row. It can be 'Before' or 'After'.
         * @param rowCount Required. Number of rows to add
         * @param values Optional. Strings to insert in the new rows, specified as a 2D array. The number of cells in each row must not exceed the number of cells in the existing row.
         */
        insertRows(insertLocation: Word.InsertLocation, rowCount: number, values?: string[][]): Word.TableRowCollection;
        /**
         *
         * Inserts rows using this row as a template. If values are specified, inserts the values into the new rows.
         *
         * [Api set: WordApi 1.3]
         *
         * @param insertLocation Required. Where the new rows should be inserted, relative to the current row. It can be 'Before' or 'After'.
         * @param rowCount Required. Number of rows to add
         * @param values Optional. Strings to insert in the new rows, specified as a 2D array. The number of cells in each row must not exceed the number of cells in the existing row.
         */
        insertRows(insertLocation: "Before" | "After" | "Start" | "End" | "Replace", rowCount: number, values?: string[][]): Word.TableRowCollection;
        /**
         *
         * Performs a search with the specified SearchOptions on the scope of the row. The search results are a collection of range objects.
         *
         * [Api set: WordApi 1.3]
         *
         * @param searchText Required. The search text.
         * @param searchOptions Optional. Options for the search.
         */
        search(searchText: string, searchOptions?: Word.SearchOptions | {
            ignorePunct?: boolean;
            ignoreSpace?: boolean;
            matchCase?: boolean;
            matchPrefix?: boolean;
            matchSuffix?: boolean;
            matchWholeWord?: boolean;
            matchWildcards?: boolean;
        }): Word.RangeCollection;
        /**
         *
         * Selects the row and navigates the Word UI to it.
         *
         * [Api set: WordApi 1.3]
         *
         * @param selectionMode Optional. The selection mode can be 'Select', 'Start', or 'End'. 'Select' is the default.
         */
        select(selectionMode?: Word.SelectionMode): void;
        /**
         *
         * Selects the row and navigates the Word UI to it.
         *
         * [Api set: WordApi 1.3]
         *
         * @param selectionMode Optional. The selection mode can be 'Select', 'Start', or 'End'. 'Select' is the default.
         */
        select(selectionMode?: "Select" | "Start" | "End"): void;
        /**
         *
         * Sets cell padding in points.
         *
         * [Api set: WordApi 1.3]
         *
         * @param cellPaddingLocation Required. The cell padding location can be 'Top', 'Left', 'Bottom', or 'Right'.
         * @param cellPadding Required. The cell padding.
         */
        setCellPadding(cellPaddingLocation: Word.CellPaddingLocation, cellPadding: number): void;
        /**
         *
         * Sets cell padding in points.
         *
         * [Api set: WordApi 1.3]
         *
         * @param cellPaddingLocation Required. The cell padding location can be 'Top', 'Left', 'Bottom', or 'Right'.
         * @param cellPadding Required. The cell padding.
         */
        setCellPadding(cellPaddingLocation: "Top" | "Left" | "Bottom" | "Right", cellPadding: number): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.TableRow` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.TableRow` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.TableRow` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.TableRowLoadOptions): Word.TableRow;
        load(option?: string | string[]): Word.TableRow;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.TableRow;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.TableRow;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.TableRow;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.TableRow object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.TableRowData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.TableRowData;
    }
    /**
     *
     * Contains the collection of the document's TableRow objects.
     *
     * [Api set: WordApi 1.3]
     */
    class TableRowCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Word.TableRow[];
        /**
         *
         * Gets the first row in this collection. Throws if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirst(): Word.TableRow;
        /**
         *
         * Gets the first row in this collection. Returns a null object if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirstOrNullObject(): Word.TableRow;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.TableRowCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.TableRowCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.TableRowCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.TableRowCollectionLoadOptions & Word.Interfaces.CollectionLoadOptions): Word.TableRowCollection;
        load(option?: string | string[]): Word.TableRowCollection;
        load(option?: OfficeExtension.LoadOption): Word.TableRowCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.TableRowCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.TableRowCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Word.TableRowCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.TableRowCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Word.Interfaces.TableRowCollectionData;
    }
    /**
     *
     * Represents a table cell in a Word document.
     *
     * [Api set: WordApi 1.3]
     */
    class TableCell extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the body object of the cell. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly body: Word.Body;
        /**
         *
         * Gets the parent row of the cell. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentRow: Word.TableRow;
        /**
         *
         * Gets the parent table of the cell. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly parentTable: Word.Table;
        /**
         *
         * Gets the index of the cell in its row. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly cellIndex: number;
        /**
         *
         * Gets and sets the width of the cell's column in points. This is applicable to uniform tables.
         *
         * [Api set: WordApi 1.3]
         */
        columnWidth: number;
        /**
         *
         * Gets and sets the horizontal alignment of the cell. The value can be 'Left', 'Centered', 'Right', or 'Justified'.
         *
         * [Api set: WordApi 1.3]
         */
        horizontalAlignment: Word.Alignment | "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified";
        /**
         *
         * Gets the index of the cell's row in the table. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly rowIndex: number;
        /**
         *
         * Gets or sets the shading color of the cell. Color is specified in "#RRGGBB" format or by using the color name.
         *
         * [Api set: WordApi 1.3]
         */
        shadingColor: string;
        /**
         *
         * Gets and sets the text of the cell.
         *
         * [Api set: WordApi 1.3]
         */
        value: string;
        /**
         *
         * Gets and sets the vertical alignment of the cell. The value can be 'Top', 'Center', or 'Bottom'.
         *
         * [Api set: WordApi 1.3]
         */
        verticalAlignment: Word.VerticalAlignment | "Mixed" | "Top" | "Center" | "Bottom";
        /**
         *
         * Gets the width of the cell in points. Read-only.
         *
         * [Api set: WordApi 1.3]
         */
        readonly width: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.TableCell): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.TableCellUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.TableCell): void;
        /**
         *
         * Deletes the column containing this cell. This is applicable to uniform tables.
         *
         * [Api set: WordApi 1.3]
         */
        deleteColumn(): void;
        /**
         *
         * Deletes the row containing this cell.
         *
         * [Api set: WordApi 1.3]
         */
        deleteRow(): void;
        /**
         *
         * Gets the border style for the specified border.
         *
         * [Api set: WordApi 1.3]
         *
         * @param borderLocation Required. The border location.
         */
        getBorder(borderLocation: Word.BorderLocation): Word.TableBorder;
        /**
         *
         * Gets the border style for the specified border.
         *
         * [Api set: WordApi 1.3]
         *
         * @param borderLocation Required. The border location.
         */
        getBorder(borderLocation: "Top" | "Left" | "Bottom" | "Right" | "InsideHorizontal" | "InsideVertical" | "Inside" | "Outside" | "All"): Word.TableBorder;
        /**
         *
         * Gets cell padding in points.
         *
         * [Api set: WordApi 1.3]
         *
         * @param cellPaddingLocation Required. The cell padding location can be 'Top', 'Left', 'Bottom', or 'Right'.
         */
        getCellPadding(cellPaddingLocation: Word.CellPaddingLocation): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets cell padding in points.
         *
         * [Api set: WordApi 1.3]
         *
         * @param cellPaddingLocation Required. The cell padding location can be 'Top', 'Left', 'Bottom', or 'Right'.
         */
        getCellPadding(cellPaddingLocation: "Top" | "Left" | "Bottom" | "Right"): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets the next cell. Throws if this cell is the last one.
         *
         * [Api set: WordApi 1.3]
         */
        getNext(): Word.TableCell;
        /**
         *
         * Gets the next cell. Returns a null object if this cell is the last one.
         *
         * [Api set: WordApi 1.3]
         */
        getNextOrNullObject(): Word.TableCell;
        /**
         *
         * Adds columns to the left or right of the cell, using the cell's column as a template. This is applicable to uniform tables. The string values, if specified, are set in the newly inserted rows.
         *
         * [Api set: WordApi 1.3]
         *
         * @param insertLocation Required. It can be 'Before' or 'After'.
         * @param columnCount Required. Number of columns to add.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertColumns(insertLocation: Word.InsertLocation, columnCount: number, values?: string[][]): void;
        /**
         *
         * Adds columns to the left or right of the cell, using the cell's column as a template. This is applicable to uniform tables. The string values, if specified, are set in the newly inserted rows.
         *
         * [Api set: WordApi 1.3]
         *
         * @param insertLocation Required. It can be 'Before' or 'After'.
         * @param columnCount Required. Number of columns to add.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertColumns(insertLocation: "Before" | "After" | "Start" | "End" | "Replace", columnCount: number, values?: string[][]): void;
        /**
         *
         * Inserts rows above or below the cell, using the cell's row as a template. The string values, if specified, are set in the newly inserted rows.
         *
         * [Api set: WordApi 1.3]
         *
         * @param insertLocation Required. It can be 'Before' or 'After'.
         * @param rowCount Required. Number of rows to add.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertRows(insertLocation: Word.InsertLocation, rowCount: number, values?: string[][]): Word.TableRowCollection;
        /**
         *
         * Inserts rows above or below the cell, using the cell's row as a template. The string values, if specified, are set in the newly inserted rows.
         *
         * [Api set: WordApi 1.3]
         *
         * @param insertLocation Required. It can be 'Before' or 'After'.
         * @param rowCount Required. Number of rows to add.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertRows(insertLocation: "Before" | "After" | "Start" | "End" | "Replace", rowCount: number, values?: string[][]): Word.TableRowCollection;
        /**
         *
         * Sets cell padding in points.
         *
         * [Api set: WordApi 1.3]
         *
         * @param cellPaddingLocation Required. The cell padding location can be 'Top', 'Left', 'Bottom', or 'Right'.
         * @param cellPadding Required. The cell padding.
         */
        setCellPadding(cellPaddingLocation: Word.CellPaddingLocation, cellPadding: number): void;
        /**
         *
         * Sets cell padding in points.
         *
         * [Api set: WordApi 1.3]
         *
         * @param cellPaddingLocation Required. The cell padding location can be 'Top', 'Left', 'Bottom', or 'Right'.
         * @param cellPadding Required. The cell padding.
         */
        setCellPadding(cellPaddingLocation: "Top" | "Left" | "Bottom" | "Right", cellPadding: number): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.TableCell` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.TableCell` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.TableCell` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.TableCellLoadOptions): Word.TableCell;
        load(option?: string | string[]): Word.TableCell;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.TableCell;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.TableCell;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.TableCell;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.TableCell object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.TableCellData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.TableCellData;
    }
    /**
     *
     * Contains the collection of the document's TableCell objects.
     *
     * [Api set: WordApi 1.3]
     */
    class TableCellCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Word.TableCell[];
        /**
         *
         * Gets the first table cell in this collection. Throws if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirst(): Word.TableCell;
        /**
         *
         * Gets the first table cell in this collection. Returns a null object if this collection is empty.
         *
         * [Api set: WordApi 1.3]
         */
        getFirstOrNullObject(): Word.TableCell;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.TableCellCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.TableCellCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.TableCellCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.TableCellCollectionLoadOptions & Word.Interfaces.CollectionLoadOptions): Word.TableCellCollection;
        load(option?: string | string[]): Word.TableCellCollection;
        load(option?: OfficeExtension.LoadOption): Word.TableCellCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.TableCellCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.TableCellCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Word.TableCellCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.TableCellCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Word.Interfaces.TableCellCollectionData;
    }
    /**
     *
     * Specifies the border style.
     *
     * [Api set: WordApi 1.3]
     */
    class TableBorder extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets or sets the table border color.
         *
         * [Api set: WordApi 1.3]
         */
        color: string;
        /**
         *
         * Gets or sets the type of the table border.
         *
         * [Api set: WordApi 1.3]
         */
        type: Word.BorderType | "Mixed" | "None" | "Single" | "Double" | "Dotted" | "Dashed" | "DotDashed" | "Dot2Dashed" | "Triple" | "ThinThickSmall" | "ThickThinSmall" | "ThinThickThinSmall" | "ThinThickMed" | "ThickThinMed" | "ThinThickThinMed" | "ThinThickLarge" | "ThickThinLarge" | "ThinThickThinLarge" | "Wave" | "DoubleWave" | "DashedSmall" | "DashDotStroked" | "ThreeDEmboss" | "ThreeDEngrave";
        /**
         *
         * Gets or sets the width, in points, of the table border. Not applicable to table border types that have fixed widths.
         *
         * [Api set: WordApi 1.3]
         */
        width: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Word.TableBorder): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.TableBorderUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Word.TableBorder): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Word.TableBorder` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Word.TableBorder` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Word.TableBorder` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Word.Interfaces.TableBorderLoadOptions): Word.TableBorder;
        load(option?: string | string[]): Word.TableBorder;
        load(option?: {
            select?: string;
            expand?: string;
        }): Word.TableBorder;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): Word.TableBorder;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): Word.TableBorder;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Word.TableBorder object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Word.Interfaces.TableBorderData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Word.Interfaces.TableBorderData;
    }
    /**
     *
     * Provides information about the type of a raised event. For each object type, please keep the order of: deleted, selection changed, data changed, added.
     *
     * [Api set: WordApi]
     */
    enum EventType {
        /**
         *
         * ContentControlDeleted represent the event that the content control has been deleted.
         *
         */
        contentControlDeleted = "ContentControlDeleted",
        /**
         *
         * ContentControlSelectionChanged represents the event that the selection in the content control has been changed.
         *
         */
        contentControlSelectionChanged = "ContentControlSelectionChanged",
        /**
         *
         * ContentControlDataChanged represents the event that the data in the content control have been changed.
         *
         */
        contentControlDataChanged = "ContentControlDataChanged",
        /**
         *
         * ContentControlAdded represents the event a content control has been added to the document.
         *
         */
        contentControlAdded = "ContentControlAdded",
        /**
         *
         * AnnotationAdded represents the event an annotation has been added to the document.
         *
         */
        annotationAdded = "AnnotationAdded",
        /**
         *
         * AnnotationAdded represents the event an annotation has been updated in the document.
         *
         */
        annotationChanged = "AnnotationChanged",
        /**
         *
         * AnnotationAdded represents the event an annotation has been deleted from the document.
         *
         */
        annotationDeleted = "AnnotationDeleted",
    }
    /**
     *
     * Specifies supported content control types and subtypes.
     *
     * [Api set: WordApi]
     */
    enum ContentControlType {
        unknown = "Unknown",
        richTextInline = "RichTextInline",
        richTextParagraphs = "RichTextParagraphs",
        /**
         *
         * Contains a whole cell.
         *
         */
        richTextTableCell = "RichTextTableCell",
        /**
         *
         * Contains a whole row.
         *
         */
        richTextTableRow = "RichTextTableRow",
        /**
         *
         * Contains a whole table.
         *
         */
        richTextTable = "RichTextTable",
        plainTextInline = "PlainTextInline",
        plainTextParagraph = "PlainTextParagraph",
        picture = "Picture",
        buildingBlockGallery = "BuildingBlockGallery",
        checkBox = "CheckBox",
        comboBox = "ComboBox",
        dropDownList = "DropDownList",
        datePicker = "DatePicker",
        repeatingSection = "RepeatingSection",
        /**
         *
         * Identifies a rich text content control.
         *
         */
        richText = "RichText",
        plainText = "PlainText",
    }
    /**
     *
     * ContentControl appearance
     *
     * [Api set: WordApi]
     */
    enum ContentControlAppearance {
        /**
         *
         * Represents a content control shown as a shaded rectangle or bounding box (with optional title).
         *
         */
        boundingBox = "BoundingBox",
        /**
         *
         * Represents a content control shown as start and end markers.
         *
         */
        tags = "Tags",
        /**
         *
         * Represents a content control that is not shown.
         *
         */
        hidden = "Hidden",
    }
    /**
     *
     * Underline types
     *
     * [Api set: WordApi]
     */
    enum UnderlineType {
        mixed = "Mixed",
        /**
         *
         * No underline.
         *
         */
        none = "None",
        /**
         *
         * @deprecated Hidden is no longer supported.
         */
        hidden = "Hidden",
        /**
         *
         * @deprecated DotLine is no longer supported.
         */
        dotLine = "DotLine",
        /**
         *
         * A single underline. This is the default value.
         *
         */
        single = "Single",
        /**
         *
         * Only underline individual words.
         *
         */
        word = "Word",
        /**
         *
         * A double underline.
         *
         */
        double = "Double",
        /**
         *
         * A single thick underline.
         *
         */
        thick = "Thick",
        /**
         *
         * A dotted underline.
         *
         */
        dotted = "Dotted",
        dottedHeavy = "DottedHeavy",
        /**
         *
         * A single dash underline.
         *
         */
        dashLine = "DashLine",
        dashLineHeavy = "DashLineHeavy",
        dashLineLong = "DashLineLong",
        dashLineLongHeavy = "DashLineLongHeavy",
        /**
         *
         * An alternating dot-dash underline.
         *
         */
        dotDashLine = "DotDashLine",
        dotDashLineHeavy = "DotDashLineHeavy",
        /**
         *
         * An alternating dot-dot-dash underline.
         *
         */
        twoDotDashLine = "TwoDotDashLine",
        twoDotDashLineHeavy = "TwoDotDashLineHeavy",
        /**
         *
         * A single wavy underline.
         *
         */
        wave = "Wave",
        waveHeavy = "WaveHeavy",
        waveDouble = "WaveDouble",
    }
    /**
     *
     * Specifies the form of a break.
     *
     * [Api set: WordApi]
     */
    enum BreakType {
        /**
         *
         * Page break at the insertion point.
         *
         */
        page = "Page",
        /**
         *
         * @deprecated Use sectionNext instead.
         */
        next = "Next",
        /**
         *
         * Section break on next page.
         *
         */
        sectionNext = "SectionNext",
        /**
         *
         * New section without a corresponding page break.
         *
         */
        sectionContinuous = "SectionContinuous",
        /**
         *
         * Section break with the next section beginning on the next even-numbered page. If the section break falls on an even-numbered page, Word leaves the next odd-numbered page blank.
         *
         */
        sectionEven = "SectionEven",
        /**
         *
         * Section break with the next section beginning on the next odd-numbered page. If the section break falls on an odd-numbered page, Word leaves the next even-numbered page blank.
         *
         */
        sectionOdd = "SectionOdd",
        /**
         *
         * Line break.
         *
         */
        line = "Line",
    }
    /**
     *
     * The insertion location types
     *
     * [Api set: WordApi]
     */
    enum InsertLocation {
        /**
         *
         * Add content before the contents of the calling object.
         *
         */
        before = "Before",
        /**
         *
         * Add content after the contents of the calling object.
         *
         */
        after = "After",
        /**
         *
         * Prepend content to the contents of the calling object.
         *
         */
        start = "Start",
        /**
         *
         * Append content to the contents of the calling object.
         *
         */
        end = "End",
        /**
         *
         * Replace the contents of the current object.
         *
         */
        replace = "Replace",
    }
    /**
     * [Api set: WordApi]
     */
    enum Alignment {
        mixed = "Mixed",
        /**
         *
         * Unknown alignment.
         *
         */
        unknown = "Unknown",
        /**
         *
         * Alignment to the left.
         *
         */
        left = "Left",
        /**
         *
         * Alignment to the center.
         *
         */
        centered = "Centered",
        /**
         *
         * Alignment to the right.
         *
         */
        right = "Right",
        /**
         *
         * Fully justified alignment.
         *
         */
        justified = "Justified",
    }
    /**
     * [Api set: WordApi]
     */
    enum HeaderFooterType {
        /**
         *
         * Returns the header or footer on all pages of a section, with the first page or odd pages excluded if they are different.
         *
         */
        primary = "Primary",
        /**
         *
         * Returns the header or footer on the first page of a section.
         *
         */
        firstPage = "FirstPage",
        /**
         *
         * Returns all headers or footers on even-numbered pages of a section.
         *
         */
        evenPages = "EvenPages",
    }
    /**
     * [Api set: WordApi]
     */
    enum BodyType {
        unknown = "Unknown",
        mainDoc = "MainDoc",
        section = "Section",
        header = "Header",
        footer = "Footer",
        tableCell = "TableCell",
    }
    /**
     * [Api set: WordApi]
     */
    enum SelectionMode {
        select = "Select",
        start = "Start",
        end = "End",
    }
    /**
     * [Api set: WordApi]
     */
    enum ImageFormat {
        unsupported = "Unsupported",
        undefined = "Undefined",
        bmp = "Bmp",
        jpeg = "Jpeg",
        gif = "Gif",
        tiff = "Tiff",
        png = "Png",
        icon = "Icon",
        exif = "Exif",
        wmf = "Wmf",
        emf = "Emf",
        pict = "Pict",
        pdf = "Pdf",
        svg = "Svg",
    }
    /**
     * [Api set: WordApi]
     */
    enum RangeLocation {
        /**
         *
         * The object's whole range. If the object is a paragraph content control or table content control, the EOP or Table characters after the content control are also included.
         *
         */
        whole = "Whole",
        /**
         *
         * The starting point of the object. For content control, it is the point after the opening tag.
         *
         */
        start = "Start",
        /**
         *
         * The ending point of the object. For paragraph, it is the point before the EOP. For content control, it is the point before the closing tag.
         *
         */
        end = "End",
        /**
         *
         * For content control only. It is the point before the opening tag.
         *
         */
        before = "Before",
        /**
         *
         * The point after the object. If the object is a paragraph content control or table content control, it is the point after the EOP or Table characters.
         *
         */
        after = "After",
        /**
         *
         * The range between 'Start' and 'End'.
         *
         */
        content = "Content",
    }
    /**
     * [Api set: WordApi]
     */
    enum LocationRelation {
        /**
         *
         * Indicates that this instance and the range are in different sub-documents.
         *
         */
        unrelated = "Unrelated",
        /**
         *
         * Indicates that this instance and the range represent the same range.
         *
         */
        equal = "Equal",
        /**
         *
         * Indicates that this instance contains the range and that it shares the same start character. The range does not share the same end character as this instance.
         *
         */
        containsStart = "ContainsStart",
        /**
         *
         * Indicates that this instance contains the range and that it shares the same end character. The range does not share the same start character as this instance.
         *
         */
        containsEnd = "ContainsEnd",
        /**
         *
         * Indicates that this instance contains the range, with the exception of the start and end character of this instance.
         *
         */
        contains = "Contains",
        /**
         *
         * Indicates that this instance is inside the range and that it shares the same start character. The range does not share the same end character as this instance.
         *
         */
        insideStart = "InsideStart",
        /**
         *
         * Indicates that this instance is inside the range and that it shares the same end character. The range does not share the same start character as this instance.
         *
         */
        insideEnd = "InsideEnd",
        /**
         *
         * Indicates that this instance is inside the range. The range does not share the same start and end characters as this instance.
         *
         */
        inside = "Inside",
        /**
         *
         * Indicates that this instance occurs before, and is adjacent to, the range.
         *
         */
        adjacentBefore = "AdjacentBefore",
        /**
         *
         * Indicates that this instance starts before the range and overlaps the range’s first character.
         *
         */
        overlapsBefore = "OverlapsBefore",
        /**
         *
         * Indicates that this instance occurs before the range.
         *
         */
        before = "Before",
        /**
         *
         * Indicates that this instance occurs after, and is adjacent to, the range.
         *
         */
        adjacentAfter = "AdjacentAfter",
        /**
         *
         * Indicates that this instance starts inside the range and overlaps the range’s last character.
         *
         */
        overlapsAfter = "OverlapsAfter",
        /**
         *
         * Indicates that this instance occurs after the range.
         *
         */
        after = "After",
    }
    /**
     * [Api set: WordApi]
     */
    enum BorderLocation {
        top = "Top",
        left = "Left",
        bottom = "Bottom",
        right = "Right",
        insideHorizontal = "InsideHorizontal",
        insideVertical = "InsideVertical",
        inside = "Inside",
        outside = "Outside",
        all = "All",
    }
    /**
     * [Api set: WordApi]
     */
    enum CellPaddingLocation {
        top = "Top",
        left = "Left",
        bottom = "Bottom",
        right = "Right",
    }
    /**
     * [Api set: WordApi]
     */
    enum BorderType {
        mixed = "Mixed",
        none = "None",
        single = "Single",
        double = "Double",
        dotted = "Dotted",
        dashed = "Dashed",
        dotDashed = "DotDashed",
        dot2Dashed = "Dot2Dashed",
        triple = "Triple",
        thinThickSmall = "ThinThickSmall",
        thickThinSmall = "ThickThinSmall",
        thinThickThinSmall = "ThinThickThinSmall",
        thinThickMed = "ThinThickMed",
        thickThinMed = "ThickThinMed",
        thinThickThinMed = "ThinThickThinMed",
        thinThickLarge = "ThinThickLarge",
        thickThinLarge = "ThickThinLarge",
        thinThickThinLarge = "ThinThickThinLarge",
        wave = "Wave",
        doubleWave = "DoubleWave",
        dashedSmall = "DashedSmall",
        dashDotStroked = "DashDotStroked",
        threeDEmboss = "ThreeDEmboss",
        threeDEngrave = "ThreeDEngrave",
    }
    /**
     * [Api set: WordApi]
     */
    enum VerticalAlignment {
        mixed = "Mixed",
        top = "Top",
        center = "Center",
        bottom = "Bottom",
    }
    /**
     * [Api set: WordApi]
     */
    enum ListLevelType {
        bullet = "Bullet",
        number = "Number",
        picture = "Picture",
    }
    /**
     * [Api set: WordApi]
     */
    enum ListBullet {
        custom = "Custom",
        solid = "Solid",
        hollow = "Hollow",
        square = "Square",
        diamonds = "Diamonds",
        arrow = "Arrow",
        checkmark = "Checkmark",
    }
    /**
     * [Api set: WordApi]
     */
    enum ListNumbering {
        none = "None",
        arabic = "Arabic",
        upperRoman = "UpperRoman",
        lowerRoman = "LowerRoman",
        upperLetter = "UpperLetter",
        lowerLetter = "LowerLetter",
    }
    /**
     * [Api set: WordApi]
     */
    enum Style {
        /**
         *
         * Mixed styles or other style not in this list.
         *
         */
        other = "Other",
        /**
         *
         * Reset character and paragraph style to default.
         *
         */
        normal = "Normal",
        heading1 = "Heading1",
        heading2 = "Heading2",
        heading3 = "Heading3",
        heading4 = "Heading4",
        heading5 = "Heading5",
        heading6 = "Heading6",
        heading7 = "Heading7",
        heading8 = "Heading8",
        heading9 = "Heading9",
        /**
         *
         * Table-of-content level 1.
         *
         */
        toc1 = "Toc1",
        /**
         *
         * Table-of-content level 2.
         *
         */
        toc2 = "Toc2",
        /**
         *
         * Table-of-content level 3.
         *
         */
        toc3 = "Toc3",
        /**
         *
         * Table-of-content level 4.
         *
         */
        toc4 = "Toc4",
        /**
         *
         * Table-of-content level 5.
         *
         */
        toc5 = "Toc5",
        /**
         *
         * Table-of-content level 6.
         *
         */
        toc6 = "Toc6",
        /**
         *
         * Table-of-content level 7.
         *
         */
        toc7 = "Toc7",
        /**
         *
         * Table-of-content level 8.
         *
         */
        toc8 = "Toc8",
        /**
         *
         * Table-of-content level 9.
         *
         */
        toc9 = "Toc9",
        footnoteText = "FootnoteText",
        header = "Header",
        footer = "Footer",
        caption = "Caption",
        footnoteReference = "FootnoteReference",
        endnoteReference = "EndnoteReference",
        endnoteText = "EndnoteText",
        title = "Title",
        subtitle = "Subtitle",
        hyperlink = "Hyperlink",
        strong = "Strong",
        emphasis = "Emphasis",
        noSpacing = "NoSpacing",
        listParagraph = "ListParagraph",
        quote = "Quote",
        intenseQuote = "IntenseQuote",
        subtleEmphasis = "SubtleEmphasis",
        intenseEmphasis = "IntenseEmphasis",
        subtleReference = "SubtleReference",
        intenseReference = "IntenseReference",
        bookTitle = "BookTitle",
        bibliography = "Bibliography",
        /**
         *
         * Table-of-content heading.
         *
         */
        tocHeading = "TocHeading",
        tableGrid = "TableGrid",
        plainTable1 = "PlainTable1",
        plainTable2 = "PlainTable2",
        plainTable3 = "PlainTable3",
        plainTable4 = "PlainTable4",
        plainTable5 = "PlainTable5",
        tableGridLight = "TableGridLight",
        gridTable1Light = "GridTable1Light",
        gridTable1Light_Accent1 = "GridTable1Light_Accent1",
        gridTable1Light_Accent2 = "GridTable1Light_Accent2",
        gridTable1Light_Accent3 = "GridTable1Light_Accent3",
        gridTable1Light_Accent4 = "GridTable1Light_Accent4",
        gridTable1Light_Accent5 = "GridTable1Light_Accent5",
        gridTable1Light_Accent6 = "GridTable1Light_Accent6",
        gridTable2 = "GridTable2",
        gridTable2_Accent1 = "GridTable2_Accent1",
        gridTable2_Accent2 = "GridTable2_Accent2",
        gridTable2_Accent3 = "GridTable2_Accent3",
        gridTable2_Accent4 = "GridTable2_Accent4",
        gridTable2_Accent5 = "GridTable2_Accent5",
        gridTable2_Accent6 = "GridTable2_Accent6",
        gridTable3 = "GridTable3",
        gridTable3_Accent1 = "GridTable3_Accent1",
        gridTable3_Accent2 = "GridTable3_Accent2",
        gridTable3_Accent3 = "GridTable3_Accent3",
        gridTable3_Accent4 = "GridTable3_Accent4",
        gridTable3_Accent5 = "GridTable3_Accent5",
        gridTable3_Accent6 = "GridTable3_Accent6",
        gridTable4 = "GridTable4",
        gridTable4_Accent1 = "GridTable4_Accent1",
        gridTable4_Accent2 = "GridTable4_Accent2",
        gridTable4_Accent3 = "GridTable4_Accent3",
        gridTable4_Accent4 = "GridTable4_Accent4",
        gridTable4_Accent5 = "GridTable4_Accent5",
        gridTable4_Accent6 = "GridTable4_Accent6",
        gridTable5Dark = "GridTable5Dark",
        gridTable5Dark_Accent1 = "GridTable5Dark_Accent1",
        gridTable5Dark_Accent2 = "GridTable5Dark_Accent2",
        gridTable5Dark_Accent3 = "GridTable5Dark_Accent3",
        gridTable5Dark_Accent4 = "GridTable5Dark_Accent4",
        gridTable5Dark_Accent5 = "GridTable5Dark_Accent5",
        gridTable5Dark_Accent6 = "GridTable5Dark_Accent6",
        gridTable6Colorful = "GridTable6Colorful",
        gridTable6Colorful_Accent1 = "GridTable6Colorful_Accent1",
        gridTable6Colorful_Accent2 = "GridTable6Colorful_Accent2",
        gridTable6Colorful_Accent3 = "GridTable6Colorful_Accent3",
        gridTable6Colorful_Accent4 = "GridTable6Colorful_Accent4",
        gridTable6Colorful_Accent5 = "GridTable6Colorful_Accent5",
        gridTable6Colorful_Accent6 = "GridTable6Colorful_Accent6",
        gridTable7Colorful = "GridTable7Colorful",
        gridTable7Colorful_Accent1 = "GridTable7Colorful_Accent1",
        gridTable7Colorful_Accent2 = "GridTable7Colorful_Accent2",
        gridTable7Colorful_Accent3 = "GridTable7Colorful_Accent3",
        gridTable7Colorful_Accent4 = "GridTable7Colorful_Accent4",
        gridTable7Colorful_Accent5 = "GridTable7Colorful_Accent5",
        gridTable7Colorful_Accent6 = "GridTable7Colorful_Accent6",
        listTable1Light = "ListTable1Light",
        listTable1Light_Accent1 = "ListTable1Light_Accent1",
        listTable1Light_Accent2 = "ListTable1Light_Accent2",
        listTable1Light_Accent3 = "ListTable1Light_Accent3",
        listTable1Light_Accent4 = "ListTable1Light_Accent4",
        listTable1Light_Accent5 = "ListTable1Light_Accent5",
        listTable1Light_Accent6 = "ListTable1Light_Accent6",
        listTable2 = "ListTable2",
        listTable2_Accent1 = "ListTable2_Accent1",
        listTable2_Accent2 = "ListTable2_Accent2",
        listTable2_Accent3 = "ListTable2_Accent3",
        listTable2_Accent4 = "ListTable2_Accent4",
        listTable2_Accent5 = "ListTable2_Accent5",
        listTable2_Accent6 = "ListTable2_Accent6",
        listTable3 = "ListTable3",
        listTable3_Accent1 = "ListTable3_Accent1",
        listTable3_Accent2 = "ListTable3_Accent2",
        listTable3_Accent3 = "ListTable3_Accent3",
        listTable3_Accent4 = "ListTable3_Accent4",
        listTable3_Accent5 = "ListTable3_Accent5",
        listTable3_Accent6 = "ListTable3_Accent6",
        listTable4 = "ListTable4",
        listTable4_Accent1 = "ListTable4_Accent1",
        listTable4_Accent2 = "ListTable4_Accent2",
        listTable4_Accent3 = "ListTable4_Accent3",
        listTable4_Accent4 = "ListTable4_Accent4",
        listTable4_Accent5 = "ListTable4_Accent5",
        listTable4_Accent6 = "ListTable4_Accent6",
        listTable5Dark = "ListTable5Dark",
        listTable5Dark_Accent1 = "ListTable5Dark_Accent1",
        listTable5Dark_Accent2 = "ListTable5Dark_Accent2",
        listTable5Dark_Accent3 = "ListTable5Dark_Accent3",
        listTable5Dark_Accent4 = "ListTable5Dark_Accent4",
        listTable5Dark_Accent5 = "ListTable5Dark_Accent5",
        listTable5Dark_Accent6 = "ListTable5Dark_Accent6",
        listTable6Colorful = "ListTable6Colorful",
        listTable6Colorful_Accent1 = "ListTable6Colorful_Accent1",
        listTable6Colorful_Accent2 = "ListTable6Colorful_Accent2",
        listTable6Colorful_Accent3 = "ListTable6Colorful_Accent3",
        listTable6Colorful_Accent4 = "ListTable6Colorful_Accent4",
        listTable6Colorful_Accent5 = "ListTable6Colorful_Accent5",
        listTable6Colorful_Accent6 = "ListTable6Colorful_Accent6",
        listTable7Colorful = "ListTable7Colorful",
        listTable7Colorful_Accent1 = "ListTable7Colorful_Accent1",
        listTable7Colorful_Accent2 = "ListTable7Colorful_Accent2",
        listTable7Colorful_Accent3 = "ListTable7Colorful_Accent3",
        listTable7Colorful_Accent4 = "ListTable7Colorful_Accent4",
        listTable7Colorful_Accent5 = "ListTable7Colorful_Accent5",
        listTable7Colorful_Accent6 = "ListTable7Colorful_Accent6",
    }
    /**
     * [Api set: WordApi]
     */
    enum DocumentPropertyType {
        string = "String",
        number = "Number",
        date = "Date",
        boolean = "Boolean",
    }
    /**
     * [Api set: WordApi]
     */
    enum TapObjectType {
        chart = "Chart",
        smartArt = "SmartArt",
        table = "Table",
        image = "Image",
        slide = "Slide",
        ole = "OLE",
        text = "Text",
    }
    /**
     * [Api set: WordApi]
     */
    enum FileContentFormat {
        base64 = "Base64",
        html = "Html",
        ooxml = "Ooxml",
    }
    enum ErrorCodes {
        accessDenied = "AccessDenied",
        generalException = "GeneralException",
        invalidArgument = "InvalidArgument",
        itemNotFound = "ItemNotFound",
        notImplemented = "NotImplemented",
    }
    module Interfaces {
        /**
        * Provides ways to load properties of only a subset of members of a collection.
        */
        interface CollectionLoadOptions {
            /**
            * Specify the number of items in the queried collection to be included in the result.
            */
            $top?: number;
            /**
            * Specify the number of items in the collection that are to be skipped and not included in the result. If top is specified, the selection of result will start after skipping the specified number of items.
            */
            $skip?: number;
        }
        /** An interface for updating data on the Body object, for use in "body.set({ ... })". */
        interface BodyUpdateData {
            /**
            *
            * Gets the text format of the body. Use this to get and set font name, size, color and other properties.
            *
            * [Api set: WordApi 1.1]
            */
            font?: Word.Interfaces.FontUpdateData;
            /**
             *
             * Gets or sets the style name for the body. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.1]
             */
            style?: string;
            /**
             *
             * Gets or sets the built-in style name for the body. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: Word.Style | "Other" | "Normal" | "Heading1" | "Heading2" | "Heading3" | "Heading4" | "Heading5" | "Heading6" | "Heading7" | "Heading8" | "Heading9" | "Toc1" | "Toc2" | "Toc3" | "Toc4" | "Toc5" | "Toc6" | "Toc7" | "Toc8" | "Toc9" | "FootnoteText" | "Header" | "Footer" | "Caption" | "FootnoteReference" | "EndnoteReference" | "EndnoteText" | "Title" | "Subtitle" | "Hyperlink" | "Strong" | "Emphasis" | "NoSpacing" | "ListParagraph" | "Quote" | "IntenseQuote" | "SubtleEmphasis" | "IntenseEmphasis" | "SubtleReference" | "IntenseReference" | "BookTitle" | "Bibliography" | "TocHeading" | "TableGrid" | "PlainTable1" | "PlainTable2" | "PlainTable3" | "PlainTable4" | "PlainTable5" | "TableGridLight" | "GridTable1Light" | "GridTable1Light_Accent1" | "GridTable1Light_Accent2" | "GridTable1Light_Accent3" | "GridTable1Light_Accent4" | "GridTable1Light_Accent5" | "GridTable1Light_Accent6" | "GridTable2" | "GridTable2_Accent1" | "GridTable2_Accent2" | "GridTable2_Accent3" | "GridTable2_Accent4" | "GridTable2_Accent5" | "GridTable2_Accent6" | "GridTable3" | "GridTable3_Accent1" | "GridTable3_Accent2" | "GridTable3_Accent3" | "GridTable3_Accent4" | "GridTable3_Accent5" | "GridTable3_Accent6" | "GridTable4" | "GridTable4_Accent1" | "GridTable4_Accent2" | "GridTable4_Accent3" | "GridTable4_Accent4" | "GridTable4_Accent5" | "GridTable4_Accent6" | "GridTable5Dark" | "GridTable5Dark_Accent1" | "GridTable5Dark_Accent2" | "GridTable5Dark_Accent3" | "GridTable5Dark_Accent4" | "GridTable5Dark_Accent5" | "GridTable5Dark_Accent6" | "GridTable6Colorful" | "GridTable6Colorful_Accent1" | "GridTable6Colorful_Accent2" | "GridTable6Colorful_Accent3" | "GridTable6Colorful_Accent4" | "GridTable6Colorful_Accent5" | "GridTable6Colorful_Accent6" | "GridTable7Colorful" | "GridTable7Colorful_Accent1" | "GridTable7Colorful_Accent2" | "GridTable7Colorful_Accent3" | "GridTable7Colorful_Accent4" | "GridTable7Colorful_Accent5" | "GridTable7Colorful_Accent6" | "ListTable1Light" | "ListTable1Light_Accent1" | "ListTable1Light_Accent2" | "ListTable1Light_Accent3" | "ListTable1Light_Accent4" | "ListTable1Light_Accent5" | "ListTable1Light_Accent6" | "ListTable2" | "ListTable2_Accent1" | "ListTable2_Accent2" | "ListTable2_Accent3" | "ListTable2_Accent4" | "ListTable2_Accent5" | "ListTable2_Accent6" | "ListTable3" | "ListTable3_Accent1" | "ListTable3_Accent2" | "ListTable3_Accent3" | "ListTable3_Accent4" | "ListTable3_Accent5" | "ListTable3_Accent6" | "ListTable4" | "ListTable4_Accent1" | "ListTable4_Accent2" | "ListTable4_Accent3" | "ListTable4_Accent4" | "ListTable4_Accent5" | "ListTable4_Accent6" | "ListTable5Dark" | "ListTable5Dark_Accent1" | "ListTable5Dark_Accent2" | "ListTable5Dark_Accent3" | "ListTable5Dark_Accent4" | "ListTable5Dark_Accent5" | "ListTable5Dark_Accent6" | "ListTable6Colorful" | "ListTable6Colorful_Accent1" | "ListTable6Colorful_Accent2" | "ListTable6Colorful_Accent3" | "ListTable6Colorful_Accent4" | "ListTable6Colorful_Accent5" | "ListTable6Colorful_Accent6" | "ListTable7Colorful" | "ListTable7Colorful_Accent1" | "ListTable7Colorful_Accent2" | "ListTable7Colorful_Accent3" | "ListTable7Colorful_Accent4" | "ListTable7Colorful_Accent5" | "ListTable7Colorful_Accent6";
        }
        /** An interface for updating data on the ContentControl object, for use in "contentControl.set({ ... })". */
        interface ContentControlUpdateData {
            /**
            *
            * Gets the text format of the content control. Use this to get and set font name, size, color, and other properties.
            *
            * [Api set: WordApi 1.1]
            */
            font?: Word.Interfaces.FontUpdateData;
            /**
             *
             * Gets or sets the appearance of the content control. The value can be 'BoundingBox', 'Tags', or 'Hidden'.
             *
             * [Api set: WordApi 1.1]
             */
            appearance?: Word.ContentControlAppearance | "BoundingBox" | "Tags" | "Hidden";
            /**
             *
             * Gets or sets a value that indicates whether the user can delete the content control. Mutually exclusive with removeWhenEdited.
             *
             * [Api set: WordApi 1.1]
             */
            cannotDelete?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the user can edit the contents of the content control.
             *
             * [Api set: WordApi 1.1]
             */
            cannotEdit?: boolean;
            /**
             *
             * Gets or sets the color of the content control. Color is specified in '#RRGGBB' format or by using the color name.
             *
             * [Api set: WordApi 1.1]
             */
            color?: string;
            /**
             *
             * Gets or sets the placeholder text of the content control. Dimmed text will be displayed when the content control is empty.
             *
             * [Api set: WordApi 1.1]
             */
            placeholderText?: string;
            /**
             *
             * Gets or sets a value that indicates whether the content control is removed after it is edited. Mutually exclusive with cannotDelete.
             *
             * [Api set: WordApi 1.1]
             */
            removeWhenEdited?: boolean;
            /**
             *
             * Gets or sets the style name for the content control. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.1]
             */
            style?: string;
            /**
             *
             * Gets or sets the built-in style name for the content control. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: Word.Style | "Other" | "Normal" | "Heading1" | "Heading2" | "Heading3" | "Heading4" | "Heading5" | "Heading6" | "Heading7" | "Heading8" | "Heading9" | "Toc1" | "Toc2" | "Toc3" | "Toc4" | "Toc5" | "Toc6" | "Toc7" | "Toc8" | "Toc9" | "FootnoteText" | "Header" | "Footer" | "Caption" | "FootnoteReference" | "EndnoteReference" | "EndnoteText" | "Title" | "Subtitle" | "Hyperlink" | "Strong" | "Emphasis" | "NoSpacing" | "ListParagraph" | "Quote" | "IntenseQuote" | "SubtleEmphasis" | "IntenseEmphasis" | "SubtleReference" | "IntenseReference" | "BookTitle" | "Bibliography" | "TocHeading" | "TableGrid" | "PlainTable1" | "PlainTable2" | "PlainTable3" | "PlainTable4" | "PlainTable5" | "TableGridLight" | "GridTable1Light" | "GridTable1Light_Accent1" | "GridTable1Light_Accent2" | "GridTable1Light_Accent3" | "GridTable1Light_Accent4" | "GridTable1Light_Accent5" | "GridTable1Light_Accent6" | "GridTable2" | "GridTable2_Accent1" | "GridTable2_Accent2" | "GridTable2_Accent3" | "GridTable2_Accent4" | "GridTable2_Accent5" | "GridTable2_Accent6" | "GridTable3" | "GridTable3_Accent1" | "GridTable3_Accent2" | "GridTable3_Accent3" | "GridTable3_Accent4" | "GridTable3_Accent5" | "GridTable3_Accent6" | "GridTable4" | "GridTable4_Accent1" | "GridTable4_Accent2" | "GridTable4_Accent3" | "GridTable4_Accent4" | "GridTable4_Accent5" | "GridTable4_Accent6" | "GridTable5Dark" | "GridTable5Dark_Accent1" | "GridTable5Dark_Accent2" | "GridTable5Dark_Accent3" | "GridTable5Dark_Accent4" | "GridTable5Dark_Accent5" | "GridTable5Dark_Accent6" | "GridTable6Colorful" | "GridTable6Colorful_Accent1" | "GridTable6Colorful_Accent2" | "GridTable6Colorful_Accent3" | "GridTable6Colorful_Accent4" | "GridTable6Colorful_Accent5" | "GridTable6Colorful_Accent6" | "GridTable7Colorful" | "GridTable7Colorful_Accent1" | "GridTable7Colorful_Accent2" | "GridTable7Colorful_Accent3" | "GridTable7Colorful_Accent4" | "GridTable7Colorful_Accent5" | "GridTable7Colorful_Accent6" | "ListTable1Light" | "ListTable1Light_Accent1" | "ListTable1Light_Accent2" | "ListTable1Light_Accent3" | "ListTable1Light_Accent4" | "ListTable1Light_Accent5" | "ListTable1Light_Accent6" | "ListTable2" | "ListTable2_Accent1" | "ListTable2_Accent2" | "ListTable2_Accent3" | "ListTable2_Accent4" | "ListTable2_Accent5" | "ListTable2_Accent6" | "ListTable3" | "ListTable3_Accent1" | "ListTable3_Accent2" | "ListTable3_Accent3" | "ListTable3_Accent4" | "ListTable3_Accent5" | "ListTable3_Accent6" | "ListTable4" | "ListTable4_Accent1" | "ListTable4_Accent2" | "ListTable4_Accent3" | "ListTable4_Accent4" | "ListTable4_Accent5" | "ListTable4_Accent6" | "ListTable5Dark" | "ListTable5Dark_Accent1" | "ListTable5Dark_Accent2" | "ListTable5Dark_Accent3" | "ListTable5Dark_Accent4" | "ListTable5Dark_Accent5" | "ListTable5Dark_Accent6" | "ListTable6Colorful" | "ListTable6Colorful_Accent1" | "ListTable6Colorful_Accent2" | "ListTable6Colorful_Accent3" | "ListTable6Colorful_Accent4" | "ListTable6Colorful_Accent5" | "ListTable6Colorful_Accent6" | "ListTable7Colorful" | "ListTable7Colorful_Accent1" | "ListTable7Colorful_Accent2" | "ListTable7Colorful_Accent3" | "ListTable7Colorful_Accent4" | "ListTable7Colorful_Accent5" | "ListTable7Colorful_Accent6";
            /**
             *
             * Gets or sets a tag to identify a content control.
             *
             * [Api set: WordApi 1.1]
             */
            tag?: string;
            /**
             *
             * Gets or sets the title for a content control.
             *
             * [Api set: WordApi 1.1]
             */
            title?: string;
        }
        /** An interface for updating data on the ContentControlCollection object, for use in "contentControlCollection.set({ ... })". */
        interface ContentControlCollectionUpdateData {
            items?: Word.Interfaces.ContentControlData[];
        }
        /** An interface for updating data on the CustomProperty object, for use in "customProperty.set({ ... })". */
        interface CustomPropertyUpdateData {
            /**
             *
             * Gets or sets the value of the custom property. Note that even though Word Online and the docx file format allow these properties to be arbitrarily long, the desktop version of Word will truncate string values to 255 16-bit chars (possibly creating invalid unicode by breaking up a surrogate pair).
             *
             * [Api set: WordApi 1.3]
             */
            value?: any;
        }
        /** An interface for updating data on the CustomPropertyCollection object, for use in "customPropertyCollection.set({ ... })". */
        interface CustomPropertyCollectionUpdateData {
            items?: Word.Interfaces.CustomPropertyData[];
        }
        /** An interface for updating data on the Document object, for use in "document.set({ ... })". */
        interface DocumentUpdateData {
            /**
            *
            * Gets the body object of the document. The body is the text that excludes headers, footers, footnotes, textboxes, etc..
            *
            * [Api set: WordApi 1.1]
            */
            body?: Word.Interfaces.BodyUpdateData;
            /**
            *
            * Gets the properties of the document.
            *
            * [Api set: WordApi 1.3]
            */
            properties?: Word.Interfaces.DocumentPropertiesUpdateData;
        }
        /** An interface for updating data on the DocumentCreated object, for use in "documentCreated.set({ ... })". */
        interface DocumentCreatedUpdateData {
            /**
            *
            * Gets the body object of the document. The body is the text that excludes headers, footers, footnotes, textboxes, etc..
            *
            * [Api set: WordApiHiddenDocument 1.3]
            */
            body?: Word.Interfaces.BodyUpdateData;
            /**
            *
            * Gets the properties of the document.
            *
            * [Api set: WordApiHiddenDocument 1.3]
            */
            properties?: Word.Interfaces.DocumentPropertiesUpdateData;
        }
        /** An interface for updating data on the DocumentProperties object, for use in "documentProperties.set({ ... })". */
        interface DocumentPropertiesUpdateData {
            /**
             *
             * Gets or sets the author of the document.
             *
             * [Api set: WordApi 1.3]
             */
            author?: string;
            /**
             *
             * Gets or sets the category of the document.
             *
             * [Api set: WordApi 1.3]
             */
            category?: string;
            /**
             *
             * Gets or sets the comments of the document.
             *
             * [Api set: WordApi 1.3]
             */
            comments?: string;
            /**
             *
             * Gets or sets the company of the document.
             *
             * [Api set: WordApi 1.3]
             */
            company?: string;
            /**
             *
             * Gets or sets the format of the document.
             *
             * [Api set: WordApi 1.3]
             */
            format?: string;
            /**
             *
             * Gets or sets the keywords of the document.
             *
             * [Api set: WordApi 1.3]
             */
            keywords?: string;
            /**
             *
             * Gets or sets the manager of the document.
             *
             * [Api set: WordApi 1.3]
             */
            manager?: string;
            /**
             *
             * Gets or sets the subject of the document.
             *
             * [Api set: WordApi 1.3]
             */
            subject?: string;
            /**
             *
             * Gets or sets the title of the document.
             *
             * [Api set: WordApi 1.3]
             */
            title?: string;
        }
        /** An interface for updating data on the Font object, for use in "font.set({ ... })". */
        interface FontUpdateData {
            /**
             *
             * Gets or sets a value that indicates whether the font is bold. True if the font is formatted as bold, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            bold?: boolean;
            /**
             *
             * Gets or sets the color for the specified font. You can provide the value in the '#RRGGBB' format or the color name.
             *
             * [Api set: WordApi 1.1]
             */
            color?: string;
            /**
             *
             * Gets or sets a value that indicates whether the font has a double strikethrough. True if the font is formatted as double strikethrough text, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            doubleStrikeThrough?: boolean;
            /**
             *
             * Gets or sets the highlight color. To set it, use a value either in the '#RRGGBB' format or the color name. To remove highlight color, set it to null. The returned highlight color can be in the '#RRGGBB' format, an empty string for mixed highlight colors, or null for no highlight color.
             *
             * [Api set: WordApi 1.1]
             */
            highlightColor?: string;
            /**
             *
             * Gets or sets a value that indicates whether the font is italicized. True if the font is italicized, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            italic?: boolean;
            /**
             *
             * Gets or sets a value that represents the name of the font.
             *
             * [Api set: WordApi 1.1]
             */
            name?: string;
            /**
             *
             * Gets or sets a value that represents the font size in points.
             *
             * [Api set: WordApi 1.1]
             */
            size?: number;
            /**
             *
             * Gets or sets a value that indicates whether the font has a strikethrough. True if the font is formatted as strikethrough text, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            strikeThrough?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the font is a subscript. True if the font is formatted as subscript, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            subscript?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the font is a superscript. True if the font is formatted as superscript, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            superscript?: boolean;
            /**
             *
             * Gets or sets a value that indicates the font's underline type. 'None' if the font is not underlined.
             *
             * [Api set: WordApi 1.1]
             */
            underline?: Word.UnderlineType | "Mixed" | "None" | "Hidden" | "DotLine" | "Single" | "Word" | "Double" | "Thick" | "Dotted" | "DottedHeavy" | "DashLine" | "DashLineHeavy" | "DashLineLong" | "DashLineLongHeavy" | "DotDashLine" | "DotDashLineHeavy" | "TwoDotDashLine" | "TwoDotDashLineHeavy" | "Wave" | "WaveHeavy" | "WaveDouble";
        }
        /** An interface for updating data on the InlinePicture object, for use in "inlinePicture.set({ ... })". */
        interface InlinePictureUpdateData {
            /**
             *
             * Gets or sets a string that represents the alternative text associated with the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            altTextDescription?: string;
            /**
             *
             * Gets or sets a string that contains the title for the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            altTextTitle?: string;
            /**
             *
             * Gets or sets a number that describes the height of the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            height?: number;
            /**
             *
             * Gets or sets a hyperlink on the image. Use a '#' to separate the address part from the optional location part.
             *
             * [Api set: WordApi 1.1]
             */
            hyperlink?: string;
            /**
             *
             * Gets or sets a value that indicates whether the inline image retains its original proportions when you resize it.
             *
             * [Api set: WordApi 1.1]
             */
            lockAspectRatio?: boolean;
            /**
             *
             * Gets or sets a number that describes the width of the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            width?: number;
        }
        /** An interface for updating data on the InlinePictureCollection object, for use in "inlinePictureCollection.set({ ... })". */
        interface InlinePictureCollectionUpdateData {
            items?: Word.Interfaces.InlinePictureData[];
        }
        /** An interface for updating data on the ListCollection object, for use in "listCollection.set({ ... })". */
        interface ListCollectionUpdateData {
            items?: Word.Interfaces.ListData[];
        }
        /** An interface for updating data on the ListItem object, for use in "listItem.set({ ... })". */
        interface ListItemUpdateData {
            /**
             *
             * Gets or sets the level of the item in the list.
             *
             * [Api set: WordApi 1.3]
             */
            level?: number;
        }
        /** An interface for updating data on the Paragraph object, for use in "paragraph.set({ ... })". */
        interface ParagraphUpdateData {
            /**
            *
            * Gets the text format of the paragraph. Use this to get and set font name, size, color, and other properties.
            *
            * [Api set: WordApi 1.1]
            */
            font?: Word.Interfaces.FontUpdateData;
            /**
            *
            * Gets the ListItem for the paragraph. Throws if the paragraph is not part of a list.
            *
            * [Api set: WordApi 1.3]
            */
            listItem?: Word.Interfaces.ListItemUpdateData;
            /**
            *
            * Gets the ListItem for the paragraph. Returns a null object if the paragraph is not part of a list.
            *
            * [Api set: WordApi 1.3]
            */
            listItemOrNullObject?: Word.Interfaces.ListItemUpdateData;
            /**
             *
             * Gets or sets the alignment for a paragraph. The value can be 'left', 'centered', 'right', or 'justified'.
             *
             * [Api set: WordApi 1.1]
             */
            alignment?: Word.Alignment | "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified";
            /**
             *
             * Gets or sets the value, in points, for a first line or hanging indent. Use a positive value to set a first-line indent, and use a negative value to set a hanging indent.
             *
             * [Api set: WordApi 1.1]
             */
            firstLineIndent?: number;
            /**
             *
             * Gets or sets the left indent value, in points, for the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            leftIndent?: number;
            /**
             *
             * Gets or sets the line spacing, in points, for the specified paragraph. In the Word UI, this value is divided by 12.
             *
             * [Api set: WordApi 1.1]
             */
            lineSpacing?: number;
            /**
             *
             * Gets or sets the amount of spacing, in grid lines, after the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            lineUnitAfter?: number;
            /**
             *
             * Gets or sets the amount of spacing, in grid lines, before the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            lineUnitBefore?: number;
            /**
             *
             * Gets or sets the outline level for the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            outlineLevel?: number;
            /**
             *
             * Gets or sets the right indent value, in points, for the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            rightIndent?: number;
            /**
             *
             * Gets or sets the spacing, in points, after the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            spaceAfter?: number;
            /**
             *
             * Gets or sets the spacing, in points, before the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            spaceBefore?: number;
            /**
             *
             * Gets or sets the style name for the paragraph. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.1]
             */
            style?: string;
            /**
             *
             * Gets or sets the built-in style name for the paragraph. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: Word.Style | "Other" | "Normal" | "Heading1" | "Heading2" | "Heading3" | "Heading4" | "Heading5" | "Heading6" | "Heading7" | "Heading8" | "Heading9" | "Toc1" | "Toc2" | "Toc3" | "Toc4" | "Toc5" | "Toc6" | "Toc7" | "Toc8" | "Toc9" | "FootnoteText" | "Header" | "Footer" | "Caption" | "FootnoteReference" | "EndnoteReference" | "EndnoteText" | "Title" | "Subtitle" | "Hyperlink" | "Strong" | "Emphasis" | "NoSpacing" | "ListParagraph" | "Quote" | "IntenseQuote" | "SubtleEmphasis" | "IntenseEmphasis" | "SubtleReference" | "IntenseReference" | "BookTitle" | "Bibliography" | "TocHeading" | "TableGrid" | "PlainTable1" | "PlainTable2" | "PlainTable3" | "PlainTable4" | "PlainTable5" | "TableGridLight" | "GridTable1Light" | "GridTable1Light_Accent1" | "GridTable1Light_Accent2" | "GridTable1Light_Accent3" | "GridTable1Light_Accent4" | "GridTable1Light_Accent5" | "GridTable1Light_Accent6" | "GridTable2" | "GridTable2_Accent1" | "GridTable2_Accent2" | "GridTable2_Accent3" | "GridTable2_Accent4" | "GridTable2_Accent5" | "GridTable2_Accent6" | "GridTable3" | "GridTable3_Accent1" | "GridTable3_Accent2" | "GridTable3_Accent3" | "GridTable3_Accent4" | "GridTable3_Accent5" | "GridTable3_Accent6" | "GridTable4" | "GridTable4_Accent1" | "GridTable4_Accent2" | "GridTable4_Accent3" | "GridTable4_Accent4" | "GridTable4_Accent5" | "GridTable4_Accent6" | "GridTable5Dark" | "GridTable5Dark_Accent1" | "GridTable5Dark_Accent2" | "GridTable5Dark_Accent3" | "GridTable5Dark_Accent4" | "GridTable5Dark_Accent5" | "GridTable5Dark_Accent6" | "GridTable6Colorful" | "GridTable6Colorful_Accent1" | "GridTable6Colorful_Accent2" | "GridTable6Colorful_Accent3" | "GridTable6Colorful_Accent4" | "GridTable6Colorful_Accent5" | "GridTable6Colorful_Accent6" | "GridTable7Colorful" | "GridTable7Colorful_Accent1" | "GridTable7Colorful_Accent2" | "GridTable7Colorful_Accent3" | "GridTable7Colorful_Accent4" | "GridTable7Colorful_Accent5" | "GridTable7Colorful_Accent6" | "ListTable1Light" | "ListTable1Light_Accent1" | "ListTable1Light_Accent2" | "ListTable1Light_Accent3" | "ListTable1Light_Accent4" | "ListTable1Light_Accent5" | "ListTable1Light_Accent6" | "ListTable2" | "ListTable2_Accent1" | "ListTable2_Accent2" | "ListTable2_Accent3" | "ListTable2_Accent4" | "ListTable2_Accent5" | "ListTable2_Accent6" | "ListTable3" | "ListTable3_Accent1" | "ListTable3_Accent2" | "ListTable3_Accent3" | "ListTable3_Accent4" | "ListTable3_Accent5" | "ListTable3_Accent6" | "ListTable4" | "ListTable4_Accent1" | "ListTable4_Accent2" | "ListTable4_Accent3" | "ListTable4_Accent4" | "ListTable4_Accent5" | "ListTable4_Accent6" | "ListTable5Dark" | "ListTable5Dark_Accent1" | "ListTable5Dark_Accent2" | "ListTable5Dark_Accent3" | "ListTable5Dark_Accent4" | "ListTable5Dark_Accent5" | "ListTable5Dark_Accent6" | "ListTable6Colorful" | "ListTable6Colorful_Accent1" | "ListTable6Colorful_Accent2" | "ListTable6Colorful_Accent3" | "ListTable6Colorful_Accent4" | "ListTable6Colorful_Accent5" | "ListTable6Colorful_Accent6" | "ListTable7Colorful" | "ListTable7Colorful_Accent1" | "ListTable7Colorful_Accent2" | "ListTable7Colorful_Accent3" | "ListTable7Colorful_Accent4" | "ListTable7Colorful_Accent5" | "ListTable7Colorful_Accent6";
        }
        /** An interface for updating data on the ParagraphCollection object, for use in "paragraphCollection.set({ ... })". */
        interface ParagraphCollectionUpdateData {
            items?: Word.Interfaces.ParagraphData[];
        }
        /** An interface for updating data on the Range object, for use in "range.set({ ... })". */
        interface RangeUpdateData {
            /**
            *
            * Gets the text format of the range. Use this to get and set font name, size, color, and other properties.
            *
            * [Api set: WordApi 1.1]
            */
            font?: Word.Interfaces.FontUpdateData;
            /**
             *
             * Gets the first hyperlink in the range, or sets a hyperlink on the range. All hyperlinks in the range are deleted when you set a new hyperlink on the range. Use a '#' to separate the address part from the optional location part.
             *
             * [Api set: WordApi 1.3]
             */
            hyperlink?: string;
            /**
             *
             * Gets or sets the style name for the range. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.1]
             */
            style?: string;
            /**
             *
             * Gets or sets the built-in style name for the range. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: Word.Style | "Other" | "Normal" | "Heading1" | "Heading2" | "Heading3" | "Heading4" | "Heading5" | "Heading6" | "Heading7" | "Heading8" | "Heading9" | "Toc1" | "Toc2" | "Toc3" | "Toc4" | "Toc5" | "Toc6" | "Toc7" | "Toc8" | "Toc9" | "FootnoteText" | "Header" | "Footer" | "Caption" | "FootnoteReference" | "EndnoteReference" | "EndnoteText" | "Title" | "Subtitle" | "Hyperlink" | "Strong" | "Emphasis" | "NoSpacing" | "ListParagraph" | "Quote" | "IntenseQuote" | "SubtleEmphasis" | "IntenseEmphasis" | "SubtleReference" | "IntenseReference" | "BookTitle" | "Bibliography" | "TocHeading" | "TableGrid" | "PlainTable1" | "PlainTable2" | "PlainTable3" | "PlainTable4" | "PlainTable5" | "TableGridLight" | "GridTable1Light" | "GridTable1Light_Accent1" | "GridTable1Light_Accent2" | "GridTable1Light_Accent3" | "GridTable1Light_Accent4" | "GridTable1Light_Accent5" | "GridTable1Light_Accent6" | "GridTable2" | "GridTable2_Accent1" | "GridTable2_Accent2" | "GridTable2_Accent3" | "GridTable2_Accent4" | "GridTable2_Accent5" | "GridTable2_Accent6" | "GridTable3" | "GridTable3_Accent1" | "GridTable3_Accent2" | "GridTable3_Accent3" | "GridTable3_Accent4" | "GridTable3_Accent5" | "GridTable3_Accent6" | "GridTable4" | "GridTable4_Accent1" | "GridTable4_Accent2" | "GridTable4_Accent3" | "GridTable4_Accent4" | "GridTable4_Accent5" | "GridTable4_Accent6" | "GridTable5Dark" | "GridTable5Dark_Accent1" | "GridTable5Dark_Accent2" | "GridTable5Dark_Accent3" | "GridTable5Dark_Accent4" | "GridTable5Dark_Accent5" | "GridTable5Dark_Accent6" | "GridTable6Colorful" | "GridTable6Colorful_Accent1" | "GridTable6Colorful_Accent2" | "GridTable6Colorful_Accent3" | "GridTable6Colorful_Accent4" | "GridTable6Colorful_Accent5" | "GridTable6Colorful_Accent6" | "GridTable7Colorful" | "GridTable7Colorful_Accent1" | "GridTable7Colorful_Accent2" | "GridTable7Colorful_Accent3" | "GridTable7Colorful_Accent4" | "GridTable7Colorful_Accent5" | "GridTable7Colorful_Accent6" | "ListTable1Light" | "ListTable1Light_Accent1" | "ListTable1Light_Accent2" | "ListTable1Light_Accent3" | "ListTable1Light_Accent4" | "ListTable1Light_Accent5" | "ListTable1Light_Accent6" | "ListTable2" | "ListTable2_Accent1" | "ListTable2_Accent2" | "ListTable2_Accent3" | "ListTable2_Accent4" | "ListTable2_Accent5" | "ListTable2_Accent6" | "ListTable3" | "ListTable3_Accent1" | "ListTable3_Accent2" | "ListTable3_Accent3" | "ListTable3_Accent4" | "ListTable3_Accent5" | "ListTable3_Accent6" | "ListTable4" | "ListTable4_Accent1" | "ListTable4_Accent2" | "ListTable4_Accent3" | "ListTable4_Accent4" | "ListTable4_Accent5" | "ListTable4_Accent6" | "ListTable5Dark" | "ListTable5Dark_Accent1" | "ListTable5Dark_Accent2" | "ListTable5Dark_Accent3" | "ListTable5Dark_Accent4" | "ListTable5Dark_Accent5" | "ListTable5Dark_Accent6" | "ListTable6Colorful" | "ListTable6Colorful_Accent1" | "ListTable6Colorful_Accent2" | "ListTable6Colorful_Accent3" | "ListTable6Colorful_Accent4" | "ListTable6Colorful_Accent5" | "ListTable6Colorful_Accent6" | "ListTable7Colorful" | "ListTable7Colorful_Accent1" | "ListTable7Colorful_Accent2" | "ListTable7Colorful_Accent3" | "ListTable7Colorful_Accent4" | "ListTable7Colorful_Accent5" | "ListTable7Colorful_Accent6";
        }
        /** An interface for updating data on the RangeCollection object, for use in "rangeCollection.set({ ... })". */
        interface RangeCollectionUpdateData {
            items?: Word.Interfaces.RangeData[];
        }
        /** An interface for updating data on the SearchOptions object, for use in "searchOptions.set({ ... })". */
        interface SearchOptionsUpdateData {
            /**
             *
             * Gets or sets a value that indicates whether to ignore all punctuation characters between words. Corresponds to the Ignore punctuation check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            ignorePunct?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to ignore all whitespace between words. Corresponds to the Ignore whitespace characters check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            ignoreSpace?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to perform a case sensitive search. Corresponds to the Match case check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            matchCase?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to match words that begin with the search string. Corresponds to the Match prefix check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            matchPrefix?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to match words that end with the search string. Corresponds to the Match suffix check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            matchSuffix?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to find operation only entire words, not text that is part of a larger word. Corresponds to the Find whole words only check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            matchWholeWord?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the search will be performed using special search operators. Corresponds to the Use wildcards check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            matchWildcards?: boolean;
        }
        /** An interface for updating data on the Section object, for use in "section.set({ ... })". */
        interface SectionUpdateData {
            /**
            *
            * Gets the body object of the section. This does not include the header/footer and other section metadata.
            *
            * [Api set: WordApi 1.1]
            */
            body?: Word.Interfaces.BodyUpdateData;
        }
        /** An interface for updating data on the SectionCollection object, for use in "sectionCollection.set({ ... })". */
        interface SectionCollectionUpdateData {
            items?: Word.Interfaces.SectionData[];
        }
        /** An interface for updating data on the Table object, for use in "table.set({ ... })". */
        interface TableUpdateData {
            /**
            *
            * Gets the font. Use this to get and set font name, size, color, and other properties.
            *
            * [Api set: WordApi 1.3]
            */
            font?: Word.Interfaces.FontUpdateData;
            /**
             *
             * Gets or sets the alignment of the table against the page column. The value can be 'Left', 'Centered', or 'Right'.
             *
             * [Api set: WordApi 1.3]
             */
            alignment?: Word.Alignment | "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified";
            /**
             *
             * Gets and sets the number of header rows.
             *
             * [Api set: WordApi 1.3]
             */
            headerRowCount?: number;
            /**
             *
             * Gets and sets the horizontal alignment of every cell in the table. The value can be 'Left', 'Centered', 'Right', or 'Justified'.
             *
             * [Api set: WordApi 1.3]
             */
            horizontalAlignment?: Word.Alignment | "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified";
            /**
             *
             * Gets and sets the shading color. Color is specified in "#RRGGBB" format or by using the color name.
             *
             * [Api set: WordApi 1.3]
             */
            shadingColor?: string;
            /**
             *
             * Gets or sets the style name for the table. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.3]
             */
            style?: string;
            /**
             *
             * Gets and sets whether the table has banded columns.
             *
             * [Api set: WordApi 1.3]
             */
            styleBandedColumns?: boolean;
            /**
             *
             * Gets and sets whether the table has banded rows.
             *
             * [Api set: WordApi 1.3]
             */
            styleBandedRows?: boolean;
            /**
             *
             * Gets or sets the built-in style name for the table. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: Word.Style | "Other" | "Normal" | "Heading1" | "Heading2" | "Heading3" | "Heading4" | "Heading5" | "Heading6" | "Heading7" | "Heading8" | "Heading9" | "Toc1" | "Toc2" | "Toc3" | "Toc4" | "Toc5" | "Toc6" | "Toc7" | "Toc8" | "Toc9" | "FootnoteText" | "Header" | "Footer" | "Caption" | "FootnoteReference" | "EndnoteReference" | "EndnoteText" | "Title" | "Subtitle" | "Hyperlink" | "Strong" | "Emphasis" | "NoSpacing" | "ListParagraph" | "Quote" | "IntenseQuote" | "SubtleEmphasis" | "IntenseEmphasis" | "SubtleReference" | "IntenseReference" | "BookTitle" | "Bibliography" | "TocHeading" | "TableGrid" | "PlainTable1" | "PlainTable2" | "PlainTable3" | "PlainTable4" | "PlainTable5" | "TableGridLight" | "GridTable1Light" | "GridTable1Light_Accent1" | "GridTable1Light_Accent2" | "GridTable1Light_Accent3" | "GridTable1Light_Accent4" | "GridTable1Light_Accent5" | "GridTable1Light_Accent6" | "GridTable2" | "GridTable2_Accent1" | "GridTable2_Accent2" | "GridTable2_Accent3" | "GridTable2_Accent4" | "GridTable2_Accent5" | "GridTable2_Accent6" | "GridTable3" | "GridTable3_Accent1" | "GridTable3_Accent2" | "GridTable3_Accent3" | "GridTable3_Accent4" | "GridTable3_Accent5" | "GridTable3_Accent6" | "GridTable4" | "GridTable4_Accent1" | "GridTable4_Accent2" | "GridTable4_Accent3" | "GridTable4_Accent4" | "GridTable4_Accent5" | "GridTable4_Accent6" | "GridTable5Dark" | "GridTable5Dark_Accent1" | "GridTable5Dark_Accent2" | "GridTable5Dark_Accent3" | "GridTable5Dark_Accent4" | "GridTable5Dark_Accent5" | "GridTable5Dark_Accent6" | "GridTable6Colorful" | "GridTable6Colorful_Accent1" | "GridTable6Colorful_Accent2" | "GridTable6Colorful_Accent3" | "GridTable6Colorful_Accent4" | "GridTable6Colorful_Accent5" | "GridTable6Colorful_Accent6" | "GridTable7Colorful" | "GridTable7Colorful_Accent1" | "GridTable7Colorful_Accent2" | "GridTable7Colorful_Accent3" | "GridTable7Colorful_Accent4" | "GridTable7Colorful_Accent5" | "GridTable7Colorful_Accent6" | "ListTable1Light" | "ListTable1Light_Accent1" | "ListTable1Light_Accent2" | "ListTable1Light_Accent3" | "ListTable1Light_Accent4" | "ListTable1Light_Accent5" | "ListTable1Light_Accent6" | "ListTable2" | "ListTable2_Accent1" | "ListTable2_Accent2" | "ListTable2_Accent3" | "ListTable2_Accent4" | "ListTable2_Accent5" | "ListTable2_Accent6" | "ListTable3" | "ListTable3_Accent1" | "ListTable3_Accent2" | "ListTable3_Accent3" | "ListTable3_Accent4" | "ListTable3_Accent5" | "ListTable3_Accent6" | "ListTable4" | "ListTable4_Accent1" | "ListTable4_Accent2" | "ListTable4_Accent3" | "ListTable4_Accent4" | "ListTable4_Accent5" | "ListTable4_Accent6" | "ListTable5Dark" | "ListTable5Dark_Accent1" | "ListTable5Dark_Accent2" | "ListTable5Dark_Accent3" | "ListTable5Dark_Accent4" | "ListTable5Dark_Accent5" | "ListTable5Dark_Accent6" | "ListTable6Colorful" | "ListTable6Colorful_Accent1" | "ListTable6Colorful_Accent2" | "ListTable6Colorful_Accent3" | "ListTable6Colorful_Accent4" | "ListTable6Colorful_Accent5" | "ListTable6Colorful_Accent6" | "ListTable7Colorful" | "ListTable7Colorful_Accent1" | "ListTable7Colorful_Accent2" | "ListTable7Colorful_Accent3" | "ListTable7Colorful_Accent4" | "ListTable7Colorful_Accent5" | "ListTable7Colorful_Accent6";
            /**
             *
             * Gets and sets whether the table has a first column with a special style.
             *
             * [Api set: WordApi 1.3]
             */
            styleFirstColumn?: boolean;
            /**
             *
             * Gets and sets whether the table has a last column with a special style.
             *
             * [Api set: WordApi 1.3]
             */
            styleLastColumn?: boolean;
            /**
             *
             * Gets and sets whether the table has a total (last) row with a special style.
             *
             * [Api set: WordApi 1.3]
             */
            styleTotalRow?: boolean;
            /**
             *
             * Gets and sets the text values in the table, as a 2D Javascript array.
             *
             * [Api set: WordApi 1.3]
             */
            values?: string[][];
            /**
             *
             * Gets and sets the vertical alignment of every cell in the table. The value can be 'Top', 'Center', or 'Bottom'.
             *
             * [Api set: WordApi 1.3]
             */
            verticalAlignment?: Word.VerticalAlignment | "Mixed" | "Top" | "Center" | "Bottom";
            /**
             *
             * Gets and sets the width of the table in points.
             *
             * [Api set: WordApi 1.3]
             */
            width?: number;
        }
        /** An interface for updating data on the TableCollection object, for use in "tableCollection.set({ ... })". */
        interface TableCollectionUpdateData {
            items?: Word.Interfaces.TableData[];
        }
        /** An interface for updating data on the TableRow object, for use in "tableRow.set({ ... })". */
        interface TableRowUpdateData {
            /**
            *
            * Gets the font. Use this to get and set font name, size, color, and other properties.
            *
            * [Api set: WordApi 1.3]
            */
            font?: Word.Interfaces.FontUpdateData;
            /**
             *
             * Gets and sets the horizontal alignment of every cell in the row. The value can be 'Left', 'Centered', 'Right', or 'Justified'.
             *
             * [Api set: WordApi 1.3]
             */
            horizontalAlignment?: Word.Alignment | "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified";
            /**
             *
             * Gets and sets the preferred height of the row in points.
             *
             * [Api set: WordApi 1.3]
             */
            preferredHeight?: number;
            /**
             *
             * Gets and sets the shading color. Color is specified in "#RRGGBB" format or by using the color name.
             *
             * [Api set: WordApi 1.3]
             */
            shadingColor?: string;
            /**
             *
             * Gets and sets the text values in the row, as a 2D Javascript array.
             *
             * [Api set: WordApi 1.3]
             */
            values?: string[][];
            /**
             *
             * Gets and sets the vertical alignment of the cells in the row. The value can be 'Top', 'Center', or 'Bottom'.
             *
             * [Api set: WordApi 1.3]
             */
            verticalAlignment?: Word.VerticalAlignment | "Mixed" | "Top" | "Center" | "Bottom";
        }
        /** An interface for updating data on the TableRowCollection object, for use in "tableRowCollection.set({ ... })". */
        interface TableRowCollectionUpdateData {
            items?: Word.Interfaces.TableRowData[];
        }
        /** An interface for updating data on the TableCell object, for use in "tableCell.set({ ... })". */
        interface TableCellUpdateData {
            /**
            *
            * Gets the body object of the cell.
            *
            * [Api set: WordApi 1.3]
            */
            body?: Word.Interfaces.BodyUpdateData;
            /**
             *
             * Gets and sets the width of the cell's column in points. This is applicable to uniform tables.
             *
             * [Api set: WordApi 1.3]
             */
            columnWidth?: number;
            /**
             *
             * Gets and sets the horizontal alignment of the cell. The value can be 'Left', 'Centered', 'Right', or 'Justified'.
             *
             * [Api set: WordApi 1.3]
             */
            horizontalAlignment?: Word.Alignment | "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified";
            /**
             *
             * Gets or sets the shading color of the cell. Color is specified in "#RRGGBB" format or by using the color name.
             *
             * [Api set: WordApi 1.3]
             */
            shadingColor?: string;
            /**
             *
             * Gets and sets the text of the cell.
             *
             * [Api set: WordApi 1.3]
             */
            value?: string;
            /**
             *
             * Gets and sets the vertical alignment of the cell. The value can be 'Top', 'Center', or 'Bottom'.
             *
             * [Api set: WordApi 1.3]
             */
            verticalAlignment?: Word.VerticalAlignment | "Mixed" | "Top" | "Center" | "Bottom";
        }
        /** An interface for updating data on the TableCellCollection object, for use in "tableCellCollection.set({ ... })". */
        interface TableCellCollectionUpdateData {
            items?: Word.Interfaces.TableCellData[];
        }
        /** An interface for updating data on the TableBorder object, for use in "tableBorder.set({ ... })". */
        interface TableBorderUpdateData {
            /**
             *
             * Gets or sets the table border color.
             *
             * [Api set: WordApi 1.3]
             */
            color?: string;
            /**
             *
             * Gets or sets the type of the table border.
             *
             * [Api set: WordApi 1.3]
             */
            type?: Word.BorderType | "Mixed" | "None" | "Single" | "Double" | "Dotted" | "Dashed" | "DotDashed" | "Dot2Dashed" | "Triple" | "ThinThickSmall" | "ThickThinSmall" | "ThinThickThinSmall" | "ThinThickMed" | "ThickThinMed" | "ThinThickThinMed" | "ThinThickLarge" | "ThickThinLarge" | "ThinThickThinLarge" | "Wave" | "DoubleWave" | "DashedSmall" | "DashDotStroked" | "ThreeDEmboss" | "ThreeDEngrave";
            /**
             *
             * Gets or sets the width, in points, of the table border. Not applicable to table border types that have fixed widths.
             *
             * [Api set: WordApi 1.3]
             */
            width?: number;
        }
        /** An interface describing the data returned by calling "body.toJSON()". */
        interface BodyData {
            /**
            *
            * Gets the collection of rich text content control objects in the body. Read-only.
            *
            * [Api set: WordApi 1.1]
            */
            contentControls?: Word.Interfaces.ContentControlData[];
            /**
            *
            * Gets the text format of the body. Use this to get and set font name, size, color and other properties. Read-only.
            *
            * [Api set: WordApi 1.1]
            */
            font?: Word.Interfaces.FontData;
            /**
            *
            * Gets the collection of InlinePicture objects in the body. The collection does not include floating images. Read-only.
            *
            * [Api set: WordApi 1.1]
            */
            inlinePictures?: Word.Interfaces.InlinePictureData[];
            /**
            *
            * Gets the collection of list objects in the body. Read-only.
            *
            * [Api set: WordApi 1.3]
            */
            lists?: Word.Interfaces.ListData[];
            /**
            *
            * Gets the collection of paragraph objects in the body. Read-only.
            *
            * [Api set: WordApi 1.1]
            */
            paragraphs?: Word.Interfaces.ParagraphData[];
            /**
            *
            * Gets the collection of table objects in the body. Read-only.
            *
            * [Api set: WordApi 1.3]
            */
            tables?: Word.Interfaces.TableData[];
            /**
             *
             * Gets or sets the style name for the body. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.1]
             */
            style?: string;
            /**
             *
             * Gets or sets the built-in style name for the body. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: Word.Style | "Other" | "Normal" | "Heading1" | "Heading2" | "Heading3" | "Heading4" | "Heading5" | "Heading6" | "Heading7" | "Heading8" | "Heading9" | "Toc1" | "Toc2" | "Toc3" | "Toc4" | "Toc5" | "Toc6" | "Toc7" | "Toc8" | "Toc9" | "FootnoteText" | "Header" | "Footer" | "Caption" | "FootnoteReference" | "EndnoteReference" | "EndnoteText" | "Title" | "Subtitle" | "Hyperlink" | "Strong" | "Emphasis" | "NoSpacing" | "ListParagraph" | "Quote" | "IntenseQuote" | "SubtleEmphasis" | "IntenseEmphasis" | "SubtleReference" | "IntenseReference" | "BookTitle" | "Bibliography" | "TocHeading" | "TableGrid" | "PlainTable1" | "PlainTable2" | "PlainTable3" | "PlainTable4" | "PlainTable5" | "TableGridLight" | "GridTable1Light" | "GridTable1Light_Accent1" | "GridTable1Light_Accent2" | "GridTable1Light_Accent3" | "GridTable1Light_Accent4" | "GridTable1Light_Accent5" | "GridTable1Light_Accent6" | "GridTable2" | "GridTable2_Accent1" | "GridTable2_Accent2" | "GridTable2_Accent3" | "GridTable2_Accent4" | "GridTable2_Accent5" | "GridTable2_Accent6" | "GridTable3" | "GridTable3_Accent1" | "GridTable3_Accent2" | "GridTable3_Accent3" | "GridTable3_Accent4" | "GridTable3_Accent5" | "GridTable3_Accent6" | "GridTable4" | "GridTable4_Accent1" | "GridTable4_Accent2" | "GridTable4_Accent3" | "GridTable4_Accent4" | "GridTable4_Accent5" | "GridTable4_Accent6" | "GridTable5Dark" | "GridTable5Dark_Accent1" | "GridTable5Dark_Accent2" | "GridTable5Dark_Accent3" | "GridTable5Dark_Accent4" | "GridTable5Dark_Accent5" | "GridTable5Dark_Accent6" | "GridTable6Colorful" | "GridTable6Colorful_Accent1" | "GridTable6Colorful_Accent2" | "GridTable6Colorful_Accent3" | "GridTable6Colorful_Accent4" | "GridTable6Colorful_Accent5" | "GridTable6Colorful_Accent6" | "GridTable7Colorful" | "GridTable7Colorful_Accent1" | "GridTable7Colorful_Accent2" | "GridTable7Colorful_Accent3" | "GridTable7Colorful_Accent4" | "GridTable7Colorful_Accent5" | "GridTable7Colorful_Accent6" | "ListTable1Light" | "ListTable1Light_Accent1" | "ListTable1Light_Accent2" | "ListTable1Light_Accent3" | "ListTable1Light_Accent4" | "ListTable1Light_Accent5" | "ListTable1Light_Accent6" | "ListTable2" | "ListTable2_Accent1" | "ListTable2_Accent2" | "ListTable2_Accent3" | "ListTable2_Accent4" | "ListTable2_Accent5" | "ListTable2_Accent6" | "ListTable3" | "ListTable3_Accent1" | "ListTable3_Accent2" | "ListTable3_Accent3" | "ListTable3_Accent4" | "ListTable3_Accent5" | "ListTable3_Accent6" | "ListTable4" | "ListTable4_Accent1" | "ListTable4_Accent2" | "ListTable4_Accent3" | "ListTable4_Accent4" | "ListTable4_Accent5" | "ListTable4_Accent6" | "ListTable5Dark" | "ListTable5Dark_Accent1" | "ListTable5Dark_Accent2" | "ListTable5Dark_Accent3" | "ListTable5Dark_Accent4" | "ListTable5Dark_Accent5" | "ListTable5Dark_Accent6" | "ListTable6Colorful" | "ListTable6Colorful_Accent1" | "ListTable6Colorful_Accent2" | "ListTable6Colorful_Accent3" | "ListTable6Colorful_Accent4" | "ListTable6Colorful_Accent5" | "ListTable6Colorful_Accent6" | "ListTable7Colorful" | "ListTable7Colorful_Accent1" | "ListTable7Colorful_Accent2" | "ListTable7Colorful_Accent3" | "ListTable7Colorful_Accent4" | "ListTable7Colorful_Accent5" | "ListTable7Colorful_Accent6";
            /**
             *
             * Gets the text of the body. Use the insertText method to insert text. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            text?: string;
            /**
             *
             * Gets the type of the body. The type can be 'MainDoc', 'Section', 'Header', 'Footer', or 'TableCell'. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            type?: Word.BodyType | "Unknown" | "MainDoc" | "Section" | "Header" | "Footer" | "TableCell";
        }
        /** An interface describing the data returned by calling "contentControl.toJSON()". */
        interface ContentControlData {
            /**
            *
            * Gets the collection of content control objects in the content control. Read-only.
            *
            * [Api set: WordApi 1.1]
            */
            contentControls?: Word.Interfaces.ContentControlData[];
            /**
            *
            * Gets the text format of the content control. Use this to get and set font name, size, color, and other properties. Read-only.
            *
            * [Api set: WordApi 1.1]
            */
            font?: Word.Interfaces.FontData;
            /**
            *
            * Gets the collection of inlinePicture objects in the content control. The collection does not include floating images. Read-only.
            *
            * [Api set: WordApi 1.1]
            */
            inlinePictures?: Word.Interfaces.InlinePictureData[];
            /**
            *
            * Gets the collection of list objects in the content control. Read-only.
            *
            * [Api set: WordApi 1.3]
            */
            lists?: Word.Interfaces.ListData[];
            /**
            *
            * Get the collection of paragraph objects in the content control. Read-only.
            *
            * [Api set: WordApi 1.1]
            */
            paragraphs?: Word.Interfaces.ParagraphData[];
            /**
            *
            * Gets the collection of table objects in the content control. Read-only.
            *
            * [Api set: WordApi 1.3]
            */
            tables?: Word.Interfaces.TableData[];
            /**
             *
             * Gets or sets the appearance of the content control. The value can be 'BoundingBox', 'Tags', or 'Hidden'.
             *
             * [Api set: WordApi 1.1]
             */
            appearance?: Word.ContentControlAppearance | "BoundingBox" | "Tags" | "Hidden";
            /**
             *
             * Gets or sets a value that indicates whether the user can delete the content control. Mutually exclusive with removeWhenEdited.
             *
             * [Api set: WordApi 1.1]
             */
            cannotDelete?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the user can edit the contents of the content control.
             *
             * [Api set: WordApi 1.1]
             */
            cannotEdit?: boolean;
            /**
             *
             * Gets or sets the color of the content control. Color is specified in '#RRGGBB' format or by using the color name.
             *
             * [Api set: WordApi 1.1]
             */
            color?: string;
            /**
             *
             * Gets an integer that represents the content control identifier. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            id?: number;
            /**
             *
             * Gets or sets the placeholder text of the content control. Dimmed text will be displayed when the content control is empty.
             *
             * [Api set: WordApi 1.1]
             */
            placeholderText?: string;
            /**
             *
             * Gets or sets a value that indicates whether the content control is removed after it is edited. Mutually exclusive with cannotDelete.
             *
             * [Api set: WordApi 1.1]
             */
            removeWhenEdited?: boolean;
            /**
             *
             * Gets or sets the style name for the content control. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.1]
             */
            style?: string;
            /**
             *
             * Gets or sets the built-in style name for the content control. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: Word.Style | "Other" | "Normal" | "Heading1" | "Heading2" | "Heading3" | "Heading4" | "Heading5" | "Heading6" | "Heading7" | "Heading8" | "Heading9" | "Toc1" | "Toc2" | "Toc3" | "Toc4" | "Toc5" | "Toc6" | "Toc7" | "Toc8" | "Toc9" | "FootnoteText" | "Header" | "Footer" | "Caption" | "FootnoteReference" | "EndnoteReference" | "EndnoteText" | "Title" | "Subtitle" | "Hyperlink" | "Strong" | "Emphasis" | "NoSpacing" | "ListParagraph" | "Quote" | "IntenseQuote" | "SubtleEmphasis" | "IntenseEmphasis" | "SubtleReference" | "IntenseReference" | "BookTitle" | "Bibliography" | "TocHeading" | "TableGrid" | "PlainTable1" | "PlainTable2" | "PlainTable3" | "PlainTable4" | "PlainTable5" | "TableGridLight" | "GridTable1Light" | "GridTable1Light_Accent1" | "GridTable1Light_Accent2" | "GridTable1Light_Accent3" | "GridTable1Light_Accent4" | "GridTable1Light_Accent5" | "GridTable1Light_Accent6" | "GridTable2" | "GridTable2_Accent1" | "GridTable2_Accent2" | "GridTable2_Accent3" | "GridTable2_Accent4" | "GridTable2_Accent5" | "GridTable2_Accent6" | "GridTable3" | "GridTable3_Accent1" | "GridTable3_Accent2" | "GridTable3_Accent3" | "GridTable3_Accent4" | "GridTable3_Accent5" | "GridTable3_Accent6" | "GridTable4" | "GridTable4_Accent1" | "GridTable4_Accent2" | "GridTable4_Accent3" | "GridTable4_Accent4" | "GridTable4_Accent5" | "GridTable4_Accent6" | "GridTable5Dark" | "GridTable5Dark_Accent1" | "GridTable5Dark_Accent2" | "GridTable5Dark_Accent3" | "GridTable5Dark_Accent4" | "GridTable5Dark_Accent5" | "GridTable5Dark_Accent6" | "GridTable6Colorful" | "GridTable6Colorful_Accent1" | "GridTable6Colorful_Accent2" | "GridTable6Colorful_Accent3" | "GridTable6Colorful_Accent4" | "GridTable6Colorful_Accent5" | "GridTable6Colorful_Accent6" | "GridTable7Colorful" | "GridTable7Colorful_Accent1" | "GridTable7Colorful_Accent2" | "GridTable7Colorful_Accent3" | "GridTable7Colorful_Accent4" | "GridTable7Colorful_Accent5" | "GridTable7Colorful_Accent6" | "ListTable1Light" | "ListTable1Light_Accent1" | "ListTable1Light_Accent2" | "ListTable1Light_Accent3" | "ListTable1Light_Accent4" | "ListTable1Light_Accent5" | "ListTable1Light_Accent6" | "ListTable2" | "ListTable2_Accent1" | "ListTable2_Accent2" | "ListTable2_Accent3" | "ListTable2_Accent4" | "ListTable2_Accent5" | "ListTable2_Accent6" | "ListTable3" | "ListTable3_Accent1" | "ListTable3_Accent2" | "ListTable3_Accent3" | "ListTable3_Accent4" | "ListTable3_Accent5" | "ListTable3_Accent6" | "ListTable4" | "ListTable4_Accent1" | "ListTable4_Accent2" | "ListTable4_Accent3" | "ListTable4_Accent4" | "ListTable4_Accent5" | "ListTable4_Accent6" | "ListTable5Dark" | "ListTable5Dark_Accent1" | "ListTable5Dark_Accent2" | "ListTable5Dark_Accent3" | "ListTable5Dark_Accent4" | "ListTable5Dark_Accent5" | "ListTable5Dark_Accent6" | "ListTable6Colorful" | "ListTable6Colorful_Accent1" | "ListTable6Colorful_Accent2" | "ListTable6Colorful_Accent3" | "ListTable6Colorful_Accent4" | "ListTable6Colorful_Accent5" | "ListTable6Colorful_Accent6" | "ListTable7Colorful" | "ListTable7Colorful_Accent1" | "ListTable7Colorful_Accent2" | "ListTable7Colorful_Accent3" | "ListTable7Colorful_Accent4" | "ListTable7Colorful_Accent5" | "ListTable7Colorful_Accent6";
            /**
             *
             * Gets the content control subtype. The subtype can be 'RichTextInline', 'RichTextParagraphs', 'RichTextTableCell', 'RichTextTableRow' and 'RichTextTable' for rich text content controls. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            subtype?: Word.ContentControlType | "Unknown" | "RichTextInline" | "RichTextParagraphs" | "RichTextTableCell" | "RichTextTableRow" | "RichTextTable" | "PlainTextInline" | "PlainTextParagraph" | "Picture" | "BuildingBlockGallery" | "CheckBox" | "ComboBox" | "DropDownList" | "DatePicker" | "RepeatingSection" | "RichText" | "PlainText";
            /**
             *
             * Gets or sets a tag to identify a content control.
             *
             * [Api set: WordApi 1.1]
             */
            tag?: string;
            /**
             *
             * Gets the text of the content control. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            text?: string;
            /**
             *
             * Gets or sets the title for a content control.
             *
             * [Api set: WordApi 1.1]
             */
            title?: string;
            /**
             *
             * Gets the content control type. Only rich text content controls are supported currently. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            type?: Word.ContentControlType | "Unknown" | "RichTextInline" | "RichTextParagraphs" | "RichTextTableCell" | "RichTextTableRow" | "RichTextTable" | "PlainTextInline" | "PlainTextParagraph" | "Picture" | "BuildingBlockGallery" | "CheckBox" | "ComboBox" | "DropDownList" | "DatePicker" | "RepeatingSection" | "RichText" | "PlainText";
        }
        /** An interface describing the data returned by calling "contentControlCollection.toJSON()". */
        interface ContentControlCollectionData {
            items?: Word.Interfaces.ContentControlData[];
        }
        /** An interface describing the data returned by calling "customProperty.toJSON()". */
        interface CustomPropertyData {
            /**
             *
             * Gets the key of the custom property. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            key?: string;
            /**
             *
             * Gets the value type of the custom property. Possible values are: String, Number, Date, Boolean. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            type?: Word.DocumentPropertyType | "String" | "Number" | "Date" | "Boolean";
            /**
             *
             * Gets or sets the value of the custom property. Note that even though Word Online and the docx file format allow these properties to be arbitrarily long, the desktop version of Word will truncate string values to 255 16-bit chars (possibly creating invalid unicode by breaking up a surrogate pair).
             *
             * [Api set: WordApi 1.3]
             */
            value?: any;
        }
        /** An interface describing the data returned by calling "customPropertyCollection.toJSON()". */
        interface CustomPropertyCollectionData {
            items?: Word.Interfaces.CustomPropertyData[];
        }
        /** An interface describing the data returned by calling "document.toJSON()". */
        interface DocumentData {
            /**
            *
            * Gets the body object of the document. The body is the text that excludes headers, footers, footnotes, textboxes, etc.. Read-only.
            *
            * [Api set: WordApi 1.1]
            */
            body?: Word.Interfaces.BodyData;
            /**
            *
            * Gets the collection of content control objects in the document. This includes content controls in the body of the document, headers, footers, textboxes, etc.. Read-only.
            *
            * [Api set: WordApi 1.1]
            */
            contentControls?: Word.Interfaces.ContentControlData[];
            /**
            *
            * Gets the properties of the document. Read-only.
            *
            * [Api set: WordApi 1.3]
            */
            properties?: Word.Interfaces.DocumentPropertiesData;
            /**
            *
            * Gets the collection of section objects in the document. Read-only.
            *
            * [Api set: WordApi 1.1]
            */
            sections?: Word.Interfaces.SectionData[];
            /**
             *
             * Indicates whether the changes in the document have been saved. A value of true indicates that the document hasn't changed since it was saved. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            saved?: boolean;
        }
        /** An interface describing the data returned by calling "documentCreated.toJSON()". */
        interface DocumentCreatedData {
            /**
            *
            * Gets the body object of the document. The body is the text that excludes headers, footers, footnotes, textboxes, etc.. Read-only.
            *
            * [Api set: WordApiHiddenDocument 1.3]
            */
            body?: Word.Interfaces.BodyData;
            /**
            *
            * Gets the collection of content control objects in the document. This includes content controls in the body of the document, headers, footers, textboxes, etc.. Read-only.
            *
            * [Api set: WordApiHiddenDocument 1.3]
            */
            contentControls?: Word.Interfaces.ContentControlData[];
            /**
            *
            * Gets the properties of the document. Read-only.
            *
            * [Api set: WordApiHiddenDocument 1.3]
            */
            properties?: Word.Interfaces.DocumentPropertiesData;
            /**
            *
            * Gets the collection of section objects in the document. Read-only.
            *
            * [Api set: WordApiHiddenDocument 1.3]
            */
            sections?: Word.Interfaces.SectionData[];
            /**
             *
             * Indicates whether the changes in the document have been saved. A value of true indicates that the document hasn't changed since it was saved. Read-only.
             *
             * [Api set: WordApiHiddenDocument 1.3]
             */
            saved?: boolean;
        }
        /** An interface describing the data returned by calling "documentProperties.toJSON()". */
        interface DocumentPropertiesData {
            /**
            *
            * Gets the collection of custom properties of the document. Read only.
            *
            * [Api set: WordApi 1.3]
            */
            customProperties?: Word.Interfaces.CustomPropertyData[];
            /**
             *
             * Gets the application name of the document. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            applicationName?: string;
            /**
             *
             * Gets or sets the author of the document.
             *
             * [Api set: WordApi 1.3]
             */
            author?: string;
            /**
             *
             * Gets or sets the category of the document.
             *
             * [Api set: WordApi 1.3]
             */
            category?: string;
            /**
             *
             * Gets or sets the comments of the document.
             *
             * [Api set: WordApi 1.3]
             */
            comments?: string;
            /**
             *
             * Gets or sets the company of the document.
             *
             * [Api set: WordApi 1.3]
             */
            company?: string;
            /**
             *
             * Gets the creation date of the document. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            creationDate?: Date;
            /**
             *
             * Gets or sets the format of the document.
             *
             * [Api set: WordApi 1.3]
             */
            format?: string;
            /**
             *
             * Gets or sets the keywords of the document.
             *
             * [Api set: WordApi 1.3]
             */
            keywords?: string;
            /**
             *
             * Gets the last author of the document. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            lastAuthor?: string;
            /**
             *
             * Gets the last print date of the document. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            lastPrintDate?: Date;
            /**
             *
             * Gets the last save time of the document. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            lastSaveTime?: Date;
            /**
             *
             * Gets or sets the manager of the document.
             *
             * [Api set: WordApi 1.3]
             */
            manager?: string;
            /**
             *
             * Gets the revision number of the document. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            revisionNumber?: string;
            /**
             *
             * Gets security settings of the document. Read only. Some are access restrictions on the file on disk. Others are Document Protection settings. Some possible values are 0 = File on disk is read/write; 1 = Protect Document: File is encrypted and requires a password to open; 2 = Protect Document: Always Open as Read-Only; 3 = Protect Document: Both #1 and #2; 4 = File on disk is read only; 5 = Both #1 and #4; 6 = Both #2 and #4; 7 = All of #1, #2, and #4; 8 = Protect Document: Restrict Edit to read-only; 9 = Both #1 and #8; 10 = Both #2 and #8; 11 = All of #1, #2, and #8; 12 = Both #4 and #8; 13 = All of #1, #4, and #8; 14 = All of #2, #4, and #8; 15 = All of #1, #2, #4, and #8.
             *
             * [Api set: WordApi 1.3]
             */
            security?: number;
            /**
             *
             * Gets or sets the subject of the document.
             *
             * [Api set: WordApi 1.3]
             */
            subject?: string;
            /**
             *
             * Gets the template of the document. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            template?: string;
            /**
             *
             * Gets or sets the title of the document.
             *
             * [Api set: WordApi 1.3]
             */
            title?: string;
        }
        /** An interface describing the data returned by calling "font.toJSON()". */
        interface FontData {
            /**
             *
             * Gets or sets a value that indicates whether the font is bold. True if the font is formatted as bold, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            bold?: boolean;
            /**
             *
             * Gets or sets the color for the specified font. You can provide the value in the '#RRGGBB' format or the color name.
             *
             * [Api set: WordApi 1.1]
             */
            color?: string;
            /**
             *
             * Gets or sets a value that indicates whether the font has a double strikethrough. True if the font is formatted as double strikethrough text, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            doubleStrikeThrough?: boolean;
            /**
             *
             * Gets or sets the highlight color. To set it, use a value either in the '#RRGGBB' format or the color name. To remove highlight color, set it to null. The returned highlight color can be in the '#RRGGBB' format, an empty string for mixed highlight colors, or null for no highlight color.
             *
             * [Api set: WordApi 1.1]
             */
            highlightColor?: string;
            /**
             *
             * Gets or sets a value that indicates whether the font is italicized. True if the font is italicized, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            italic?: boolean;
            /**
             *
             * Gets or sets a value that represents the name of the font.
             *
             * [Api set: WordApi 1.1]
             */
            name?: string;
            /**
             *
             * Gets or sets a value that represents the font size in points.
             *
             * [Api set: WordApi 1.1]
             */
            size?: number;
            /**
             *
             * Gets or sets a value that indicates whether the font has a strikethrough. True if the font is formatted as strikethrough text, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            strikeThrough?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the font is a subscript. True if the font is formatted as subscript, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            subscript?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the font is a superscript. True if the font is formatted as superscript, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            superscript?: boolean;
            /**
             *
             * Gets or sets a value that indicates the font's underline type. 'None' if the font is not underlined.
             *
             * [Api set: WordApi 1.1]
             */
            underline?: Word.UnderlineType | "Mixed" | "None" | "Hidden" | "DotLine" | "Single" | "Word" | "Double" | "Thick" | "Dotted" | "DottedHeavy" | "DashLine" | "DashLineHeavy" | "DashLineLong" | "DashLineLongHeavy" | "DotDashLine" | "DotDashLineHeavy" | "TwoDotDashLine" | "TwoDotDashLineHeavy" | "Wave" | "WaveHeavy" | "WaveDouble";
        }
        /** An interface describing the data returned by calling "inlinePicture.toJSON()". */
        interface InlinePictureData {
            /**
             *
             * Gets or sets a string that represents the alternative text associated with the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            altTextDescription?: string;
            /**
             *
             * Gets or sets a string that contains the title for the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            altTextTitle?: string;
            /**
             *
             * Gets or sets a number that describes the height of the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            height?: number;
            /**
             *
             * Gets or sets a hyperlink on the image. Use a '#' to separate the address part from the optional location part.
             *
             * [Api set: WordApi 1.1]
             */
            hyperlink?: string;
            /**
             *
             * Gets or sets a value that indicates whether the inline image retains its original proportions when you resize it.
             *
             * [Api set: WordApi 1.1]
             */
            lockAspectRatio?: boolean;
            /**
             *
             * Gets or sets a number that describes the width of the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            width?: number;
        }
        /** An interface describing the data returned by calling "inlinePictureCollection.toJSON()". */
        interface InlinePictureCollectionData {
            items?: Word.Interfaces.InlinePictureData[];
        }
        /** An interface describing the data returned by calling "list.toJSON()". */
        interface ListData {
            /**
            *
            * Gets paragraphs in the list. Read-only.
            *
            * [Api set: WordApi 1.3]
            */
            paragraphs?: Word.Interfaces.ParagraphData[];
            /**
             *
             * Gets the list's id.
             *
             * [Api set: WordApi 1.3]
             */
            id?: number;
            /**
             *
             * Checks whether each of the 9 levels exists in the list. A true value indicates the level exists, which means there is at least one list item at that level. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            levelExistences?: boolean[];
            /**
             *
             * Gets all 9 level types in the list. Each type can be 'Bullet', 'Number', or 'Picture'. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            levelTypes?: Word.ListLevelType[];
        }
        /** An interface describing the data returned by calling "listCollection.toJSON()". */
        interface ListCollectionData {
            items?: Word.Interfaces.ListData[];
        }
        /** An interface describing the data returned by calling "listItem.toJSON()". */
        interface ListItemData {
            /**
             *
             * Gets or sets the level of the item in the list.
             *
             * [Api set: WordApi 1.3]
             */
            level?: number;
            /**
             *
             * Gets the list item bullet, number, or picture as a string. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            listString?: string;
            /**
             *
             * Gets the list item order number in relation to its siblings. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            siblingIndex?: number;
        }
        /** An interface describing the data returned by calling "paragraph.toJSON()". */
        interface ParagraphData {
            /**
            *
            * Gets the text format of the paragraph. Use this to get and set font name, size, color, and other properties. Read-only.
            *
            * [Api set: WordApi 1.1]
            */
            font?: Word.Interfaces.FontData;
            /**
            *
            * Gets the collection of InlinePicture objects in the paragraph. The collection does not include floating images. Read-only.
            *
            * [Api set: WordApi 1.1]
            */
            inlinePictures?: Word.Interfaces.InlinePictureData[];
            /**
            *
            * Gets the ListItem for the paragraph. Throws if the paragraph is not part of a list. Read-only.
            *
            * [Api set: WordApi 1.3]
            */
            listItem?: Word.Interfaces.ListItemData;
            /**
            *
            * Gets the ListItem for the paragraph. Returns a null object if the paragraph is not part of a list. Read-only.
            *
            * [Api set: WordApi 1.3]
            */
            listItemOrNullObject?: Word.Interfaces.ListItemData;
            /**
             *
             * Gets or sets the alignment for a paragraph. The value can be 'left', 'centered', 'right', or 'justified'.
             *
             * [Api set: WordApi 1.1]
             */
            alignment?: Word.Alignment | "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified";
            /**
             *
             * Gets or sets the value, in points, for a first line or hanging indent. Use a positive value to set a first-line indent, and use a negative value to set a hanging indent.
             *
             * [Api set: WordApi 1.1]
             */
            firstLineIndent?: number;
            /**
             *
             * Indicates the paragraph is the last one inside its parent body. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            isLastParagraph?: boolean;
            /**
             *
             * Checks whether the paragraph is a list item. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            isListItem?: boolean;
            /**
             *
             * Gets or sets the left indent value, in points, for the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            leftIndent?: number;
            /**
             *
             * Gets or sets the line spacing, in points, for the specified paragraph. In the Word UI, this value is divided by 12.
             *
             * [Api set: WordApi 1.1]
             */
            lineSpacing?: number;
            /**
             *
             * Gets or sets the amount of spacing, in grid lines, after the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            lineUnitAfter?: number;
            /**
             *
             * Gets or sets the amount of spacing, in grid lines, before the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            lineUnitBefore?: number;
            /**
             *
             * Gets or sets the outline level for the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            outlineLevel?: number;
            /**
             *
             * Gets or sets the right indent value, in points, for the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            rightIndent?: number;
            /**
             *
             * Gets or sets the spacing, in points, after the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            spaceAfter?: number;
            /**
             *
             * Gets or sets the spacing, in points, before the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            spaceBefore?: number;
            /**
             *
             * Gets or sets the style name for the paragraph. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.1]
             */
            style?: string;
            /**
             *
             * Gets or sets the built-in style name for the paragraph. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: Word.Style | "Other" | "Normal" | "Heading1" | "Heading2" | "Heading3" | "Heading4" | "Heading5" | "Heading6" | "Heading7" | "Heading8" | "Heading9" | "Toc1" | "Toc2" | "Toc3" | "Toc4" | "Toc5" | "Toc6" | "Toc7" | "Toc8" | "Toc9" | "FootnoteText" | "Header" | "Footer" | "Caption" | "FootnoteReference" | "EndnoteReference" | "EndnoteText" | "Title" | "Subtitle" | "Hyperlink" | "Strong" | "Emphasis" | "NoSpacing" | "ListParagraph" | "Quote" | "IntenseQuote" | "SubtleEmphasis" | "IntenseEmphasis" | "SubtleReference" | "IntenseReference" | "BookTitle" | "Bibliography" | "TocHeading" | "TableGrid" | "PlainTable1" | "PlainTable2" | "PlainTable3" | "PlainTable4" | "PlainTable5" | "TableGridLight" | "GridTable1Light" | "GridTable1Light_Accent1" | "GridTable1Light_Accent2" | "GridTable1Light_Accent3" | "GridTable1Light_Accent4" | "GridTable1Light_Accent5" | "GridTable1Light_Accent6" | "GridTable2" | "GridTable2_Accent1" | "GridTable2_Accent2" | "GridTable2_Accent3" | "GridTable2_Accent4" | "GridTable2_Accent5" | "GridTable2_Accent6" | "GridTable3" | "GridTable3_Accent1" | "GridTable3_Accent2" | "GridTable3_Accent3" | "GridTable3_Accent4" | "GridTable3_Accent5" | "GridTable3_Accent6" | "GridTable4" | "GridTable4_Accent1" | "GridTable4_Accent2" | "GridTable4_Accent3" | "GridTable4_Accent4" | "GridTable4_Accent5" | "GridTable4_Accent6" | "GridTable5Dark" | "GridTable5Dark_Accent1" | "GridTable5Dark_Accent2" | "GridTable5Dark_Accent3" | "GridTable5Dark_Accent4" | "GridTable5Dark_Accent5" | "GridTable5Dark_Accent6" | "GridTable6Colorful" | "GridTable6Colorful_Accent1" | "GridTable6Colorful_Accent2" | "GridTable6Colorful_Accent3" | "GridTable6Colorful_Accent4" | "GridTable6Colorful_Accent5" | "GridTable6Colorful_Accent6" | "GridTable7Colorful" | "GridTable7Colorful_Accent1" | "GridTable7Colorful_Accent2" | "GridTable7Colorful_Accent3" | "GridTable7Colorful_Accent4" | "GridTable7Colorful_Accent5" | "GridTable7Colorful_Accent6" | "ListTable1Light" | "ListTable1Light_Accent1" | "ListTable1Light_Accent2" | "ListTable1Light_Accent3" | "ListTable1Light_Accent4" | "ListTable1Light_Accent5" | "ListTable1Light_Accent6" | "ListTable2" | "ListTable2_Accent1" | "ListTable2_Accent2" | "ListTable2_Accent3" | "ListTable2_Accent4" | "ListTable2_Accent5" | "ListTable2_Accent6" | "ListTable3" | "ListTable3_Accent1" | "ListTable3_Accent2" | "ListTable3_Accent3" | "ListTable3_Accent4" | "ListTable3_Accent5" | "ListTable3_Accent6" | "ListTable4" | "ListTable4_Accent1" | "ListTable4_Accent2" | "ListTable4_Accent3" | "ListTable4_Accent4" | "ListTable4_Accent5" | "ListTable4_Accent6" | "ListTable5Dark" | "ListTable5Dark_Accent1" | "ListTable5Dark_Accent2" | "ListTable5Dark_Accent3" | "ListTable5Dark_Accent4" | "ListTable5Dark_Accent5" | "ListTable5Dark_Accent6" | "ListTable6Colorful" | "ListTable6Colorful_Accent1" | "ListTable6Colorful_Accent2" | "ListTable6Colorful_Accent3" | "ListTable6Colorful_Accent4" | "ListTable6Colorful_Accent5" | "ListTable6Colorful_Accent6" | "ListTable7Colorful" | "ListTable7Colorful_Accent1" | "ListTable7Colorful_Accent2" | "ListTable7Colorful_Accent3" | "ListTable7Colorful_Accent4" | "ListTable7Colorful_Accent5" | "ListTable7Colorful_Accent6";
            /**
             *
             * Gets the level of the paragraph's table. It returns 0 if the paragraph is not in a table. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            tableNestingLevel?: number;
            /**
             *
             * Gets the text of the paragraph. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            text?: string;
        }
        /** An interface describing the data returned by calling "paragraphCollection.toJSON()". */
        interface ParagraphCollectionData {
            items?: Word.Interfaces.ParagraphData[];
        }
        /** An interface describing the data returned by calling "range.toJSON()". */
        interface RangeData {
            /**
            *
            * Gets the text format of the range. Use this to get and set font name, size, color, and other properties. Read-only.
            *
            * [Api set: WordApi 1.1]
            */
            font?: Word.Interfaces.FontData;
            /**
            *
            * Gets the collection of inline picture objects in the range. Read-only.
            *
            * [Api set: WordApi 1.2]
            */
            inlinePictures?: Word.Interfaces.InlinePictureData[];
            /**
             *
             * Gets the first hyperlink in the range, or sets a hyperlink on the range. All hyperlinks in the range are deleted when you set a new hyperlink on the range. Use a '#' to separate the address part from the optional location part.
             *
             * [Api set: WordApi 1.3]
             */
            hyperlink?: string;
            /**
             *
             * Checks whether the range length is zero. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            isEmpty?: boolean;
            /**
             *
             * Gets or sets the style name for the range. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.1]
             */
            style?: string;
            /**
             *
             * Gets or sets the built-in style name for the range. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: Word.Style | "Other" | "Normal" | "Heading1" | "Heading2" | "Heading3" | "Heading4" | "Heading5" | "Heading6" | "Heading7" | "Heading8" | "Heading9" | "Toc1" | "Toc2" | "Toc3" | "Toc4" | "Toc5" | "Toc6" | "Toc7" | "Toc8" | "Toc9" | "FootnoteText" | "Header" | "Footer" | "Caption" | "FootnoteReference" | "EndnoteReference" | "EndnoteText" | "Title" | "Subtitle" | "Hyperlink" | "Strong" | "Emphasis" | "NoSpacing" | "ListParagraph" | "Quote" | "IntenseQuote" | "SubtleEmphasis" | "IntenseEmphasis" | "SubtleReference" | "IntenseReference" | "BookTitle" | "Bibliography" | "TocHeading" | "TableGrid" | "PlainTable1" | "PlainTable2" | "PlainTable3" | "PlainTable4" | "PlainTable5" | "TableGridLight" | "GridTable1Light" | "GridTable1Light_Accent1" | "GridTable1Light_Accent2" | "GridTable1Light_Accent3" | "GridTable1Light_Accent4" | "GridTable1Light_Accent5" | "GridTable1Light_Accent6" | "GridTable2" | "GridTable2_Accent1" | "GridTable2_Accent2" | "GridTable2_Accent3" | "GridTable2_Accent4" | "GridTable2_Accent5" | "GridTable2_Accent6" | "GridTable3" | "GridTable3_Accent1" | "GridTable3_Accent2" | "GridTable3_Accent3" | "GridTable3_Accent4" | "GridTable3_Accent5" | "GridTable3_Accent6" | "GridTable4" | "GridTable4_Accent1" | "GridTable4_Accent2" | "GridTable4_Accent3" | "GridTable4_Accent4" | "GridTable4_Accent5" | "GridTable4_Accent6" | "GridTable5Dark" | "GridTable5Dark_Accent1" | "GridTable5Dark_Accent2" | "GridTable5Dark_Accent3" | "GridTable5Dark_Accent4" | "GridTable5Dark_Accent5" | "GridTable5Dark_Accent6" | "GridTable6Colorful" | "GridTable6Colorful_Accent1" | "GridTable6Colorful_Accent2" | "GridTable6Colorful_Accent3" | "GridTable6Colorful_Accent4" | "GridTable6Colorful_Accent5" | "GridTable6Colorful_Accent6" | "GridTable7Colorful" | "GridTable7Colorful_Accent1" | "GridTable7Colorful_Accent2" | "GridTable7Colorful_Accent3" | "GridTable7Colorful_Accent4" | "GridTable7Colorful_Accent5" | "GridTable7Colorful_Accent6" | "ListTable1Light" | "ListTable1Light_Accent1" | "ListTable1Light_Accent2" | "ListTable1Light_Accent3" | "ListTable1Light_Accent4" | "ListTable1Light_Accent5" | "ListTable1Light_Accent6" | "ListTable2" | "ListTable2_Accent1" | "ListTable2_Accent2" | "ListTable2_Accent3" | "ListTable2_Accent4" | "ListTable2_Accent5" | "ListTable2_Accent6" | "ListTable3" | "ListTable3_Accent1" | "ListTable3_Accent2" | "ListTable3_Accent3" | "ListTable3_Accent4" | "ListTable3_Accent5" | "ListTable3_Accent6" | "ListTable4" | "ListTable4_Accent1" | "ListTable4_Accent2" | "ListTable4_Accent3" | "ListTable4_Accent4" | "ListTable4_Accent5" | "ListTable4_Accent6" | "ListTable5Dark" | "ListTable5Dark_Accent1" | "ListTable5Dark_Accent2" | "ListTable5Dark_Accent3" | "ListTable5Dark_Accent4" | "ListTable5Dark_Accent5" | "ListTable5Dark_Accent6" | "ListTable6Colorful" | "ListTable6Colorful_Accent1" | "ListTable6Colorful_Accent2" | "ListTable6Colorful_Accent3" | "ListTable6Colorful_Accent4" | "ListTable6Colorful_Accent5" | "ListTable6Colorful_Accent6" | "ListTable7Colorful" | "ListTable7Colorful_Accent1" | "ListTable7Colorful_Accent2" | "ListTable7Colorful_Accent3" | "ListTable7Colorful_Accent4" | "ListTable7Colorful_Accent5" | "ListTable7Colorful_Accent6";
            /**
             *
             * Gets the text of the range. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            text?: string;
        }
        /** An interface describing the data returned by calling "rangeCollection.toJSON()". */
        interface RangeCollectionData {
            items?: Word.Interfaces.RangeData[];
        }
        /** An interface describing the data returned by calling "searchOptions.toJSON()". */
        interface SearchOptionsData {
            /**
             *
             * Gets or sets a value that indicates whether to ignore all punctuation characters between words. Corresponds to the Ignore punctuation check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            ignorePunct?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to ignore all whitespace between words. Corresponds to the Ignore whitespace characters check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            ignoreSpace?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to perform a case sensitive search. Corresponds to the Match case check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            matchCase?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to match words that begin with the search string. Corresponds to the Match prefix check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            matchPrefix?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to match words that end with the search string. Corresponds to the Match suffix check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            matchSuffix?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to find operation only entire words, not text that is part of a larger word. Corresponds to the Find whole words only check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            matchWholeWord?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the search will be performed using special search operators. Corresponds to the Use wildcards check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            matchWildcards?: boolean;
        }
        /** An interface describing the data returned by calling "section.toJSON()". */
        interface SectionData {
            /**
            *
            * Gets the body object of the section. This does not include the header/footer and other section metadata. Read-only.
            *
            * [Api set: WordApi 1.1]
            */
            body?: Word.Interfaces.BodyData;
        }
        /** An interface describing the data returned by calling "sectionCollection.toJSON()". */
        interface SectionCollectionData {
            items?: Word.Interfaces.SectionData[];
        }
        /** An interface describing the data returned by calling "table.toJSON()". */
        interface TableData {
            /**
            *
            * Gets the font. Use this to get and set font name, size, color, and other properties. Read-only.
            *
            * [Api set: WordApi 1.3]
            */
            font?: Word.Interfaces.FontData;
            /**
            *
            * Gets all of the table rows. Read-only.
            *
            * [Api set: WordApi 1.3]
            */
            rows?: Word.Interfaces.TableRowData[];
            /**
            *
            * Gets the child tables nested one level deeper. Read-only.
            *
            * [Api set: WordApi 1.3]
            */
            tables?: Word.Interfaces.TableData[];
            /**
             *
             * Gets or sets the alignment of the table against the page column. The value can be 'Left', 'Centered', or 'Right'.
             *
             * [Api set: WordApi 1.3]
             */
            alignment?: Word.Alignment | "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified";
            /**
             *
             * Gets and sets the number of header rows.
             *
             * [Api set: WordApi 1.3]
             */
            headerRowCount?: number;
            /**
             *
             * Gets and sets the horizontal alignment of every cell in the table. The value can be 'Left', 'Centered', 'Right', or 'Justified'.
             *
             * [Api set: WordApi 1.3]
             */
            horizontalAlignment?: Word.Alignment | "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified";
            /**
             *
             * Indicates whether all of the table rows are uniform. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            isUniform?: boolean;
            /**
             *
             * Gets the nesting level of the table. Top-level tables have level 1. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            nestingLevel?: number;
            /**
             *
             * Gets the number of rows in the table. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            rowCount?: number;
            /**
             *
             * Gets and sets the shading color. Color is specified in "#RRGGBB" format or by using the color name.
             *
             * [Api set: WordApi 1.3]
             */
            shadingColor?: string;
            /**
             *
             * Gets or sets the style name for the table. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.3]
             */
            style?: string;
            /**
             *
             * Gets and sets whether the table has banded columns.
             *
             * [Api set: WordApi 1.3]
             */
            styleBandedColumns?: boolean;
            /**
             *
             * Gets and sets whether the table has banded rows.
             *
             * [Api set: WordApi 1.3]
             */
            styleBandedRows?: boolean;
            /**
             *
             * Gets or sets the built-in style name for the table. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: Word.Style | "Other" | "Normal" | "Heading1" | "Heading2" | "Heading3" | "Heading4" | "Heading5" | "Heading6" | "Heading7" | "Heading8" | "Heading9" | "Toc1" | "Toc2" | "Toc3" | "Toc4" | "Toc5" | "Toc6" | "Toc7" | "Toc8" | "Toc9" | "FootnoteText" | "Header" | "Footer" | "Caption" | "FootnoteReference" | "EndnoteReference" | "EndnoteText" | "Title" | "Subtitle" | "Hyperlink" | "Strong" | "Emphasis" | "NoSpacing" | "ListParagraph" | "Quote" | "IntenseQuote" | "SubtleEmphasis" | "IntenseEmphasis" | "SubtleReference" | "IntenseReference" | "BookTitle" | "Bibliography" | "TocHeading" | "TableGrid" | "PlainTable1" | "PlainTable2" | "PlainTable3" | "PlainTable4" | "PlainTable5" | "TableGridLight" | "GridTable1Light" | "GridTable1Light_Accent1" | "GridTable1Light_Accent2" | "GridTable1Light_Accent3" | "GridTable1Light_Accent4" | "GridTable1Light_Accent5" | "GridTable1Light_Accent6" | "GridTable2" | "GridTable2_Accent1" | "GridTable2_Accent2" | "GridTable2_Accent3" | "GridTable2_Accent4" | "GridTable2_Accent5" | "GridTable2_Accent6" | "GridTable3" | "GridTable3_Accent1" | "GridTable3_Accent2" | "GridTable3_Accent3" | "GridTable3_Accent4" | "GridTable3_Accent5" | "GridTable3_Accent6" | "GridTable4" | "GridTable4_Accent1" | "GridTable4_Accent2" | "GridTable4_Accent3" | "GridTable4_Accent4" | "GridTable4_Accent5" | "GridTable4_Accent6" | "GridTable5Dark" | "GridTable5Dark_Accent1" | "GridTable5Dark_Accent2" | "GridTable5Dark_Accent3" | "GridTable5Dark_Accent4" | "GridTable5Dark_Accent5" | "GridTable5Dark_Accent6" | "GridTable6Colorful" | "GridTable6Colorful_Accent1" | "GridTable6Colorful_Accent2" | "GridTable6Colorful_Accent3" | "GridTable6Colorful_Accent4" | "GridTable6Colorful_Accent5" | "GridTable6Colorful_Accent6" | "GridTable7Colorful" | "GridTable7Colorful_Accent1" | "GridTable7Colorful_Accent2" | "GridTable7Colorful_Accent3" | "GridTable7Colorful_Accent4" | "GridTable7Colorful_Accent5" | "GridTable7Colorful_Accent6" | "ListTable1Light" | "ListTable1Light_Accent1" | "ListTable1Light_Accent2" | "ListTable1Light_Accent3" | "ListTable1Light_Accent4" | "ListTable1Light_Accent5" | "ListTable1Light_Accent6" | "ListTable2" | "ListTable2_Accent1" | "ListTable2_Accent2" | "ListTable2_Accent3" | "ListTable2_Accent4" | "ListTable2_Accent5" | "ListTable2_Accent6" | "ListTable3" | "ListTable3_Accent1" | "ListTable3_Accent2" | "ListTable3_Accent3" | "ListTable3_Accent4" | "ListTable3_Accent5" | "ListTable3_Accent6" | "ListTable4" | "ListTable4_Accent1" | "ListTable4_Accent2" | "ListTable4_Accent3" | "ListTable4_Accent4" | "ListTable4_Accent5" | "ListTable4_Accent6" | "ListTable5Dark" | "ListTable5Dark_Accent1" | "ListTable5Dark_Accent2" | "ListTable5Dark_Accent3" | "ListTable5Dark_Accent4" | "ListTable5Dark_Accent5" | "ListTable5Dark_Accent6" | "ListTable6Colorful" | "ListTable6Colorful_Accent1" | "ListTable6Colorful_Accent2" | "ListTable6Colorful_Accent3" | "ListTable6Colorful_Accent4" | "ListTable6Colorful_Accent5" | "ListTable6Colorful_Accent6" | "ListTable7Colorful" | "ListTable7Colorful_Accent1" | "ListTable7Colorful_Accent2" | "ListTable7Colorful_Accent3" | "ListTable7Colorful_Accent4" | "ListTable7Colorful_Accent5" | "ListTable7Colorful_Accent6";
            /**
             *
             * Gets and sets whether the table has a first column with a special style.
             *
             * [Api set: WordApi 1.3]
             */
            styleFirstColumn?: boolean;
            /**
             *
             * Gets and sets whether the table has a last column with a special style.
             *
             * [Api set: WordApi 1.3]
             */
            styleLastColumn?: boolean;
            /**
             *
             * Gets and sets whether the table has a total (last) row with a special style.
             *
             * [Api set: WordApi 1.3]
             */
            styleTotalRow?: boolean;
            /**
             *
             * Gets and sets the text values in the table, as a 2D Javascript array.
             *
             * [Api set: WordApi 1.3]
             */
            values?: string[][];
            /**
             *
             * Gets and sets the vertical alignment of every cell in the table. The value can be 'Top', 'Center', or 'Bottom'.
             *
             * [Api set: WordApi 1.3]
             */
            verticalAlignment?: Word.VerticalAlignment | "Mixed" | "Top" | "Center" | "Bottom";
            /**
             *
             * Gets and sets the width of the table in points.
             *
             * [Api set: WordApi 1.3]
             */
            width?: number;
        }
        /** An interface describing the data returned by calling "tableCollection.toJSON()". */
        interface TableCollectionData {
            items?: Word.Interfaces.TableData[];
        }
        /** An interface describing the data returned by calling "tableRow.toJSON()". */
        interface TableRowData {
            /**
            *
            * Gets cells. Read-only.
            *
            * [Api set: WordApi 1.3]
            */
            cells?: Word.Interfaces.TableCellData[];
            /**
            *
            * Gets the font. Use this to get and set font name, size, color, and other properties. Read-only.
            *
            * [Api set: WordApi 1.3]
            */
            font?: Word.Interfaces.FontData;
            /**
             *
             * Gets the number of cells in the row. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            cellCount?: number;
            /**
             *
             * Gets and sets the horizontal alignment of every cell in the row. The value can be 'Left', 'Centered', 'Right', or 'Justified'.
             *
             * [Api set: WordApi 1.3]
             */
            horizontalAlignment?: Word.Alignment | "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified";
            /**
             *
             * Checks whether the row is a header row. Read-only. To set the number of header rows, use HeaderRowCount on the Table object.
             *
             * [Api set: WordApi 1.3]
             */
            isHeader?: boolean;
            /**
             *
             * Gets and sets the preferred height of the row in points.
             *
             * [Api set: WordApi 1.3]
             */
            preferredHeight?: number;
            /**
             *
             * Gets the index of the row in its parent table. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            rowIndex?: number;
            /**
             *
             * Gets and sets the shading color. Color is specified in "#RRGGBB" format or by using the color name.
             *
             * [Api set: WordApi 1.3]
             */
            shadingColor?: string;
            /**
             *
             * Gets and sets the text values in the row, as a 2D Javascript array.
             *
             * [Api set: WordApi 1.3]
             */
            values?: string[][];
            /**
             *
             * Gets and sets the vertical alignment of the cells in the row. The value can be 'Top', 'Center', or 'Bottom'.
             *
             * [Api set: WordApi 1.3]
             */
            verticalAlignment?: Word.VerticalAlignment | "Mixed" | "Top" | "Center" | "Bottom";
        }
        /** An interface describing the data returned by calling "tableRowCollection.toJSON()". */
        interface TableRowCollectionData {
            items?: Word.Interfaces.TableRowData[];
        }
        /** An interface describing the data returned by calling "tableCell.toJSON()". */
        interface TableCellData {
            /**
            *
            * Gets the body object of the cell. Read-only.
            *
            * [Api set: WordApi 1.3]
            */
            body?: Word.Interfaces.BodyData;
            /**
             *
             * Gets the index of the cell in its row. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            cellIndex?: number;
            /**
             *
             * Gets and sets the width of the cell's column in points. This is applicable to uniform tables.
             *
             * [Api set: WordApi 1.3]
             */
            columnWidth?: number;
            /**
             *
             * Gets and sets the horizontal alignment of the cell. The value can be 'Left', 'Centered', 'Right', or 'Justified'.
             *
             * [Api set: WordApi 1.3]
             */
            horizontalAlignment?: Word.Alignment | "Mixed" | "Unknown" | "Left" | "Centered" | "Right" | "Justified";
            /**
             *
             * Gets the index of the cell's row in the table. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            rowIndex?: number;
            /**
             *
             * Gets or sets the shading color of the cell. Color is specified in "#RRGGBB" format or by using the color name.
             *
             * [Api set: WordApi 1.3]
             */
            shadingColor?: string;
            /**
             *
             * Gets and sets the text of the cell.
             *
             * [Api set: WordApi 1.3]
             */
            value?: string;
            /**
             *
             * Gets and sets the vertical alignment of the cell. The value can be 'Top', 'Center', or 'Bottom'.
             *
             * [Api set: WordApi 1.3]
             */
            verticalAlignment?: Word.VerticalAlignment | "Mixed" | "Top" | "Center" | "Bottom";
            /**
             *
             * Gets the width of the cell in points. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            width?: number;
        }
        /** An interface describing the data returned by calling "tableCellCollection.toJSON()". */
        interface TableCellCollectionData {
            items?: Word.Interfaces.TableCellData[];
        }
        /** An interface describing the data returned by calling "tableBorder.toJSON()". */
        interface TableBorderData {
            /**
             *
             * Gets or sets the table border color.
             *
             * [Api set: WordApi 1.3]
             */
            color?: string;
            /**
             *
             * Gets or sets the type of the table border.
             *
             * [Api set: WordApi 1.3]
             */
            type?: Word.BorderType | "Mixed" | "None" | "Single" | "Double" | "Dotted" | "Dashed" | "DotDashed" | "Dot2Dashed" | "Triple" | "ThinThickSmall" | "ThickThinSmall" | "ThinThickThinSmall" | "ThinThickMed" | "ThickThinMed" | "ThinThickThinMed" | "ThinThickLarge" | "ThickThinLarge" | "ThinThickThinLarge" | "Wave" | "DoubleWave" | "DashedSmall" | "DashDotStroked" | "ThreeDEmboss" | "ThreeDEngrave";
            /**
             *
             * Gets or sets the width, in points, of the table border. Not applicable to table border types that have fixed widths.
             *
             * [Api set: WordApi 1.3]
             */
            width?: number;
        }
        /**
         *
         * Represents the body of a document or a section.
         *
         * [Api set: WordApi 1.1]
         */
        interface BodyLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the text format of the body. Use this to get and set font name, size, color and other properties.
            *
            * [Api set: WordApi 1.1]
            */
            font?: Word.Interfaces.FontLoadOptions;
            /**
            *
            * Gets the parent body of the body. For example, a table cell body's parent body could be a header. Throws if there isn't a parent body.
            *
            * [Api set: WordApi 1.3]
            */
            parentBody?: Word.Interfaces.BodyLoadOptions;
            /**
            *
            * Gets the parent body of the body. For example, a table cell body's parent body could be a header. Returns a null object if there isn't a parent body.
            *
            * [Api set: WordApi 1.3]
            */
            parentBodyOrNullObject?: Word.Interfaces.BodyLoadOptions;
            /**
            *
            * Gets the content control that contains the body. Throws if there isn't a parent content control.
            *
            * [Api set: WordApi 1.1]
            */
            parentContentControl?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * Gets the content control that contains the body. Returns a null object if there isn't a parent content control.
            *
            * [Api set: WordApi 1.3]
            */
            parentContentControlOrNullObject?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * Gets the parent section of the body. Throws if there isn't a parent section.
            *
            * [Api set: WordApi 1.3]
            */
            parentSection?: Word.Interfaces.SectionLoadOptions;
            /**
            *
            * Gets the parent section of the body. Returns a null object if there isn't a parent section.
            *
            * [Api set: WordApi 1.3]
            */
            parentSectionOrNullObject?: Word.Interfaces.SectionLoadOptions;
            /**
             *
             * Gets or sets the style name for the body. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.1]
             */
            style?: boolean;
            /**
             *
             * Gets or sets the built-in style name for the body. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: boolean;
            /**
             *
             * Gets the text of the body. Use the insertText method to insert text. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            text?: boolean;
            /**
             *
             * Gets the type of the body. The type can be 'MainDoc', 'Section', 'Header', 'Footer', or 'TableCell'. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            type?: boolean;
        }
        /**
         *
         * Represents a content control. Content controls are bounded and potentially labeled regions in a document that serve as containers for specific types of content. Individual content controls may contain contents such as images, tables, or paragraphs of formatted text. Currently, only rich text content controls are supported.
         *
         * [Api set: WordApi 1.1]
         */
        interface ContentControlLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the text format of the content control. Use this to get and set font name, size, color, and other properties.
            *
            * [Api set: WordApi 1.1]
            */
            font?: Word.Interfaces.FontLoadOptions;
            /**
            *
            * Gets the parent body of the content control.
            *
            * [Api set: WordApi 1.3]
            */
            parentBody?: Word.Interfaces.BodyLoadOptions;
            /**
            *
            * Gets the content control that contains the content control. Throws if there isn't a parent content control.
            *
            * [Api set: WordApi 1.1]
            */
            parentContentControl?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * Gets the content control that contains the content control. Returns a null object if there isn't a parent content control.
            *
            * [Api set: WordApi 1.3]
            */
            parentContentControlOrNullObject?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * Gets the table that contains the content control. Throws if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTable?: Word.Interfaces.TableLoadOptions;
            /**
            *
            * Gets the table cell that contains the content control. Throws if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCell?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * Gets the table cell that contains the content control. Returns a null object if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCellOrNullObject?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * Gets the table that contains the content control. Returns a null object if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableOrNullObject?: Word.Interfaces.TableLoadOptions;
            /**
             *
             * Gets or sets the appearance of the content control. The value can be 'BoundingBox', 'Tags', or 'Hidden'.
             *
             * [Api set: WordApi 1.1]
             */
            appearance?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the user can delete the content control. Mutually exclusive with removeWhenEdited.
             *
             * [Api set: WordApi 1.1]
             */
            cannotDelete?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the user can edit the contents of the content control.
             *
             * [Api set: WordApi 1.1]
             */
            cannotEdit?: boolean;
            /**
             *
             * Gets or sets the color of the content control. Color is specified in '#RRGGBB' format or by using the color name.
             *
             * [Api set: WordApi 1.1]
             */
            color?: boolean;
            /**
             *
             * Gets an integer that represents the content control identifier. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            id?: boolean;
            /**
             *
             * Gets or sets the placeholder text of the content control. Dimmed text will be displayed when the content control is empty.
             *
             * [Api set: WordApi 1.1]
             */
            placeholderText?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the content control is removed after it is edited. Mutually exclusive with cannotDelete.
             *
             * [Api set: WordApi 1.1]
             */
            removeWhenEdited?: boolean;
            /**
             *
             * Gets or sets the style name for the content control. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.1]
             */
            style?: boolean;
            /**
             *
             * Gets or sets the built-in style name for the content control. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: boolean;
            /**
             *
             * Gets the content control subtype. The subtype can be 'RichTextInline', 'RichTextParagraphs', 'RichTextTableCell', 'RichTextTableRow' and 'RichTextTable' for rich text content controls. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            subtype?: boolean;
            /**
             *
             * Gets or sets a tag to identify a content control.
             *
             * [Api set: WordApi 1.1]
             */
            tag?: boolean;
            /**
             *
             * Gets the text of the content control. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            text?: boolean;
            /**
             *
             * Gets or sets the title for a content control.
             *
             * [Api set: WordApi 1.1]
             */
            title?: boolean;
            /**
             *
             * Gets the content control type. Only rich text content controls are supported currently. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            type?: boolean;
        }
        /**
         *
         * Contains a collection of {@link Word.ContentControl} objects. Content controls are bounded and potentially labeled regions in a document that serve as containers for specific types of content. Individual content controls may contain contents such as images, tables, or paragraphs of formatted text. Currently, only rich text content controls are supported.
         *
         * [Api set: WordApi 1.1]
         */
        interface ContentControlCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Gets the text format of the content control. Use this to get and set font name, size, color, and other properties.
            *
            * [Api set: WordApi 1.1]
            */
            font?: Word.Interfaces.FontLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the parent body of the content control.
            *
            * [Api set: WordApi 1.3]
            */
            parentBody?: Word.Interfaces.BodyLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the content control that contains the content control. Throws if there isn't a parent content control.
            *
            * [Api set: WordApi 1.1]
            */
            parentContentControl?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the content control that contains the content control. Returns a null object if there isn't a parent content control.
            *
            * [Api set: WordApi 1.3]
            */
            parentContentControlOrNullObject?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table that contains the content control. Throws if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTable?: Word.Interfaces.TableLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table cell that contains the content control. Throws if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCell?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table cell that contains the content control. Returns a null object if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCellOrNullObject?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table that contains the content control. Returns a null object if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableOrNullObject?: Word.Interfaces.TableLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the appearance of the content control. The value can be 'BoundingBox', 'Tags', or 'Hidden'.
             *
             * [Api set: WordApi 1.1]
             */
            appearance?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets a value that indicates whether the user can delete the content control. Mutually exclusive with removeWhenEdited.
             *
             * [Api set: WordApi 1.1]
             */
            cannotDelete?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets a value that indicates whether the user can edit the contents of the content control.
             *
             * [Api set: WordApi 1.1]
             */
            cannotEdit?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the color of the content control. Color is specified in '#RRGGBB' format or by using the color name.
             *
             * [Api set: WordApi 1.1]
             */
            color?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets an integer that represents the content control identifier. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the placeholder text of the content control. Dimmed text will be displayed when the content control is empty.
             *
             * [Api set: WordApi 1.1]
             */
            placeholderText?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets a value that indicates whether the content control is removed after it is edited. Mutually exclusive with cannotDelete.
             *
             * [Api set: WordApi 1.1]
             */
            removeWhenEdited?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the style name for the content control. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.1]
             */
            style?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the built-in style name for the content control. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the content control subtype. The subtype can be 'RichTextInline', 'RichTextParagraphs', 'RichTextTableCell', 'RichTextTableRow' and 'RichTextTable' for rich text content controls. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            subtype?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets a tag to identify a content control.
             *
             * [Api set: WordApi 1.1]
             */
            tag?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the text of the content control. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            text?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the title for a content control.
             *
             * [Api set: WordApi 1.1]
             */
            title?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the content control type. Only rich text content controls are supported currently. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            type?: boolean;
        }
        /**
         *
         * Represents a custom property.
         *
         * [Api set: WordApi 1.3]
         */
        interface CustomPropertyLoadOptions {
            $all?: boolean;
            /**
             *
             * Gets the key of the custom property. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            key?: boolean;
            /**
             *
             * Gets the value type of the custom property. Possible values are: String, Number, Date, Boolean. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            type?: boolean;
            /**
             *
             * Gets or sets the value of the custom property. Note that even though Word Online and the docx file format allow these properties to be arbitrarily long, the desktop version of Word will truncate string values to 255 16-bit chars (possibly creating invalid unicode by breaking up a surrogate pair).
             *
             * [Api set: WordApi 1.3]
             */
            value?: boolean;
        }
        /**
         *
         * Contains the collection of {@link Word.CustomProperty} objects.
         *
         * [Api set: WordApi 1.3]
         */
        interface CustomPropertyCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the key of the custom property. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            key?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the value type of the custom property. Possible values are: String, Number, Date, Boolean. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            type?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the value of the custom property. Note that even though Word Online and the docx file format allow these properties to be arbitrarily long, the desktop version of Word will truncate string values to 255 16-bit chars (possibly creating invalid unicode by breaking up a surrogate pair).
             *
             * [Api set: WordApi 1.3]
             */
            value?: boolean;
        }
        /**
         *
         * The Document object is the top level object. A Document object contains one or more sections, content controls, and the body that contains the contents of the document.
         *
         * [Api set: WordApi 1.1]
         */
        interface DocumentLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the body object of the document. The body is the text that excludes headers, footers, footnotes, textboxes, etc..
            *
            * [Api set: WordApi 1.1]
            */
            body?: Word.Interfaces.BodyLoadOptions;
            /**
            *
            * Gets the properties of the document.
            *
            * [Api set: WordApi 1.3]
            */
            properties?: Word.Interfaces.DocumentPropertiesLoadOptions;
            /**
             *
             * Gets or sets a value that indicates that, when opening a new document, whether it is allowed to close this document even if this document is untitled. True to close, false otherwise.
             *
             * [Api set: WordApi]
             */
            allowCloseOnUntitled?: boolean;
            /**
             *
             * Indicates whether the changes in the document have been saved. A value of true indicates that the document hasn't changed since it was saved. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            saved?: boolean;
        }
        /**
         *
         * The DocumentCreated object is the top level object created by Application.CreateDocument. A DocumentCreated object is a special Document object.
         *
         * [Api set: WordApi 1.3]
         */
        interface DocumentCreatedLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the body object of the document. The body is the text that excludes headers, footers, footnotes, textboxes, etc..
            *
            * [Api set: WordApiHiddenDocument 1.3]
            */
            body?: Word.Interfaces.BodyLoadOptions;
            /**
            *
            * Gets the properties of the document.
            *
            * [Api set: WordApiHiddenDocument 1.3]
            */
            properties?: Word.Interfaces.DocumentPropertiesLoadOptions;
            /**
             *
             * Indicates whether the changes in the document have been saved. A value of true indicates that the document hasn't changed since it was saved. Read-only.
             *
             * [Api set: WordApiHiddenDocument 1.3]
             */
            saved?: boolean;
        }
        /**
         *
         * Represents document properties.
         *
         * [Api set: WordApi 1.3]
         */
        interface DocumentPropertiesLoadOptions {
            $all?: boolean;
            /**
             *
             * Gets the application name of the document. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            applicationName?: boolean;
            /**
             *
             * Gets or sets the author of the document.
             *
             * [Api set: WordApi 1.3]
             */
            author?: boolean;
            /**
             *
             * Gets or sets the category of the document.
             *
             * [Api set: WordApi 1.3]
             */
            category?: boolean;
            /**
             *
             * Gets or sets the comments of the document.
             *
             * [Api set: WordApi 1.3]
             */
            comments?: boolean;
            /**
             *
             * Gets or sets the company of the document.
             *
             * [Api set: WordApi 1.3]
             */
            company?: boolean;
            /**
             *
             * Gets the creation date of the document. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            creationDate?: boolean;
            /**
             *
             * Gets or sets the format of the document.
             *
             * [Api set: WordApi 1.3]
             */
            format?: boolean;
            /**
             *
             * Gets or sets the keywords of the document.
             *
             * [Api set: WordApi 1.3]
             */
            keywords?: boolean;
            /**
             *
             * Gets the last author of the document. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            lastAuthor?: boolean;
            /**
             *
             * Gets the last print date of the document. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            lastPrintDate?: boolean;
            /**
             *
             * Gets the last save time of the document. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            lastSaveTime?: boolean;
            /**
             *
             * Gets or sets the manager of the document.
             *
             * [Api set: WordApi 1.3]
             */
            manager?: boolean;
            /**
             *
             * Gets the revision number of the document. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            revisionNumber?: boolean;
            /**
             *
             * Gets security settings of the document. Read only. Some are access restrictions on the file on disk. Others are Document Protection settings. Some possible values are 0 = File on disk is read/write; 1 = Protect Document: File is encrypted and requires a password to open; 2 = Protect Document: Always Open as Read-Only; 3 = Protect Document: Both #1 and #2; 4 = File on disk is read only; 5 = Both #1 and #4; 6 = Both #2 and #4; 7 = All of #1, #2, and #4; 8 = Protect Document: Restrict Edit to read-only; 9 = Both #1 and #8; 10 = Both #2 and #8; 11 = All of #1, #2, and #8; 12 = Both #4 and #8; 13 = All of #1, #4, and #8; 14 = All of #2, #4, and #8; 15 = All of #1, #2, #4, and #8.
             *
             * [Api set: WordApi 1.3]
             */
            security?: boolean;
            /**
             *
             * Gets or sets the subject of the document.
             *
             * [Api set: WordApi 1.3]
             */
            subject?: boolean;
            /**
             *
             * Gets the template of the document. Read only.
             *
             * [Api set: WordApi 1.3]
             */
            template?: boolean;
            /**
             *
             * Gets or sets the title of the document.
             *
             * [Api set: WordApi 1.3]
             */
            title?: boolean;
        }
        /**
         *
         * Represents a font.
         *
         * [Api set: WordApi 1.1]
         */
        interface FontLoadOptions {
            $all?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the font is bold. True if the font is formatted as bold, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            bold?: boolean;
            /**
             *
             * Gets or sets the color for the specified font. You can provide the value in the '#RRGGBB' format or the color name.
             *
             * [Api set: WordApi 1.1]
             */
            color?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the font has a double strikethrough. True if the font is formatted as double strikethrough text, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            doubleStrikeThrough?: boolean;
            /**
             *
             * Gets or sets the highlight color. To set it, use a value either in the '#RRGGBB' format or the color name. To remove highlight color, set it to null. The returned highlight color can be in the '#RRGGBB' format, an empty string for mixed highlight colors, or null for no highlight color.
             *
             * [Api set: WordApi 1.1]
             */
            highlightColor?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the font is italicized. True if the font is italicized, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            italic?: boolean;
            /**
             *
             * Gets or sets a value that represents the name of the font.
             *
             * [Api set: WordApi 1.1]
             */
            name?: boolean;
            /**
             *
             * Gets or sets a value that represents the font size in points.
             *
             * [Api set: WordApi 1.1]
             */
            size?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the font has a strikethrough. True if the font is formatted as strikethrough text, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            strikeThrough?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the font is a subscript. True if the font is formatted as subscript, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            subscript?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the font is a superscript. True if the font is formatted as superscript, otherwise, false.
             *
             * [Api set: WordApi 1.1]
             */
            superscript?: boolean;
            /**
             *
             * Gets or sets a value that indicates the font's underline type. 'None' if the font is not underlined.
             *
             * [Api set: WordApi 1.1]
             */
            underline?: boolean;
        }
        /**
         *
         * Represents an inline picture.
         *
         * [Api set: WordApi 1.1]
         */
        interface InlinePictureLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the parent paragraph that contains the inline image.
            *
            * [Api set: WordApi 1.2]
            */
            paragraph?: Word.Interfaces.ParagraphLoadOptions;
            /**
            *
            * Gets the content control that contains the inline image. Throws if there isn't a parent content control.
            *
            * [Api set: WordApi 1.1]
            */
            parentContentControl?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * Gets the content control that contains the inline image. Returns a null object if there isn't a parent content control.
            *
            * [Api set: WordApi 1.3]
            */
            parentContentControlOrNullObject?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * Gets the table that contains the inline image. Throws if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTable?: Word.Interfaces.TableLoadOptions;
            /**
            *
            * Gets the table cell that contains the inline image. Throws if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCell?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * Gets the table cell that contains the inline image. Returns a null object if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCellOrNullObject?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * Gets the table that contains the inline image. Returns a null object if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableOrNullObject?: Word.Interfaces.TableLoadOptions;
            /**
             *
             * Gets or sets a string that represents the alternative text associated with the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            altTextDescription?: boolean;
            /**
             *
             * Gets or sets a string that contains the title for the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            altTextTitle?: boolean;
            /**
             *
             * Gets or sets a number that describes the height of the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            height?: boolean;
            /**
             *
             * Gets or sets a hyperlink on the image. Use a '#' to separate the address part from the optional location part.
             *
             * [Api set: WordApi 1.1]
             */
            hyperlink?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the inline image retains its original proportions when you resize it.
             *
             * [Api set: WordApi 1.1]
             */
            lockAspectRatio?: boolean;
            /**
             *
             * Gets or sets a number that describes the width of the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            width?: boolean;
        }
        /**
         *
         * Contains a collection of {@link Word.InlinePicture} objects.
         *
         * [Api set: WordApi 1.1]
         */
        interface InlinePictureCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Gets the parent paragraph that contains the inline image.
            *
            * [Api set: WordApi 1.2]
            */
            paragraph?: Word.Interfaces.ParagraphLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the content control that contains the inline image. Throws if there isn't a parent content control.
            *
            * [Api set: WordApi 1.1]
            */
            parentContentControl?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the content control that contains the inline image. Returns a null object if there isn't a parent content control.
            *
            * [Api set: WordApi 1.3]
            */
            parentContentControlOrNullObject?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table that contains the inline image. Throws if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTable?: Word.Interfaces.TableLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table cell that contains the inline image. Throws if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCell?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table cell that contains the inline image. Returns a null object if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCellOrNullObject?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table that contains the inline image. Returns a null object if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableOrNullObject?: Word.Interfaces.TableLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets a string that represents the alternative text associated with the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            altTextDescription?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets a string that contains the title for the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            altTextTitle?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets a number that describes the height of the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            height?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets a hyperlink on the image. Use a '#' to separate the address part from the optional location part.
             *
             * [Api set: WordApi 1.1]
             */
            hyperlink?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets a value that indicates whether the inline image retains its original proportions when you resize it.
             *
             * [Api set: WordApi 1.1]
             */
            lockAspectRatio?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets a number that describes the width of the inline image.
             *
             * [Api set: WordApi 1.1]
             */
            width?: boolean;
        }
        /**
         *
         * Contains a collection of {@link Word.Paragraph} objects.
         *
         * [Api set: WordApi 1.3]
         */
        interface ListLoadOptions {
            $all?: boolean;
            /**
             *
             * Gets the list's id.
             *
             * [Api set: WordApi 1.3]
             */
            id?: boolean;
            /**
             *
             * Checks whether each of the 9 levels exists in the list. A true value indicates the level exists, which means there is at least one list item at that level. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            levelExistences?: boolean;
            /**
             *
             * Gets all 9 level types in the list. Each type can be 'Bullet', 'Number', or 'Picture'. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            levelTypes?: boolean;
        }
        /**
         *
         * Contains a collection of {@link Word.List} objects.
         *
         * [Api set: WordApi 1.3]
         */
        interface ListCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the list's id.
             *
             * [Api set: WordApi 1.3]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Checks whether each of the 9 levels exists in the list. A true value indicates the level exists, which means there is at least one list item at that level. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            levelExistences?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets all 9 level types in the list. Each type can be 'Bullet', 'Number', or 'Picture'. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            levelTypes?: boolean;
        }
        /**
         *
         * Represents the paragraph list item format.
         *
         * [Api set: WordApi 1.3]
         */
        interface ListItemLoadOptions {
            $all?: boolean;
            /**
             *
             * Gets or sets the level of the item in the list.
             *
             * [Api set: WordApi 1.3]
             */
            level?: boolean;
            /**
             *
             * Gets the list item bullet, number, or picture as a string. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            listString?: boolean;
            /**
             *
             * Gets the list item order number in relation to its siblings. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            siblingIndex?: boolean;
        }
        /**
         *
         * Represents a single paragraph in a selection, range, content control, or document body.
         *
         * [Api set: WordApi 1.1]
         */
        interface ParagraphLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the text format of the paragraph. Use this to get and set font name, size, color, and other properties.
            *
            * [Api set: WordApi 1.1]
            */
            font?: Word.Interfaces.FontLoadOptions;
            /**
            *
            * Gets the List to which this paragraph belongs. Throws if the paragraph is not in a list.
            *
            * [Api set: WordApi 1.3]
            */
            list?: Word.Interfaces.ListLoadOptions;
            /**
            *
            * Gets the ListItem for the paragraph. Throws if the paragraph is not part of a list.
            *
            * [Api set: WordApi 1.3]
            */
            listItem?: Word.Interfaces.ListItemLoadOptions;
            /**
            *
            * Gets the ListItem for the paragraph. Returns a null object if the paragraph is not part of a list.
            *
            * [Api set: WordApi 1.3]
            */
            listItemOrNullObject?: Word.Interfaces.ListItemLoadOptions;
            /**
            *
            * Gets the List to which this paragraph belongs. Returns a null object if the paragraph is not in a list.
            *
            * [Api set: WordApi 1.3]
            */
            listOrNullObject?: Word.Interfaces.ListLoadOptions;
            /**
            *
            * Gets the parent body of the paragraph.
            *
            * [Api set: WordApi 1.3]
            */
            parentBody?: Word.Interfaces.BodyLoadOptions;
            /**
            *
            * Gets the content control that contains the paragraph. Throws if there isn't a parent content control.
            *
            * [Api set: WordApi 1.1]
            */
            parentContentControl?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * Gets the content control that contains the paragraph. Returns a null object if there isn't a parent content control.
            *
            * [Api set: WordApi 1.3]
            */
            parentContentControlOrNullObject?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * Gets the table that contains the paragraph. Throws if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTable?: Word.Interfaces.TableLoadOptions;
            /**
            *
            * Gets the table cell that contains the paragraph. Throws if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCell?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * Gets the table cell that contains the paragraph. Returns a null object if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCellOrNullObject?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * Gets the table that contains the paragraph. Returns a null object if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableOrNullObject?: Word.Interfaces.TableLoadOptions;
            /**
             *
             * Gets or sets the alignment for a paragraph. The value can be 'left', 'centered', 'right', or 'justified'.
             *
             * [Api set: WordApi 1.1]
             */
            alignment?: boolean;
            /**
             *
             * Gets or sets the value, in points, for a first line or hanging indent. Use a positive value to set a first-line indent, and use a negative value to set a hanging indent.
             *
             * [Api set: WordApi 1.1]
             */
            firstLineIndent?: boolean;
            /**
             *
             * Indicates the paragraph is the last one inside its parent body. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            isLastParagraph?: boolean;
            /**
             *
             * Checks whether the paragraph is a list item. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            isListItem?: boolean;
            /**
             *
             * Gets or sets the left indent value, in points, for the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            leftIndent?: boolean;
            /**
             *
             * Gets or sets the line spacing, in points, for the specified paragraph. In the Word UI, this value is divided by 12.
             *
             * [Api set: WordApi 1.1]
             */
            lineSpacing?: boolean;
            /**
             *
             * Gets or sets the amount of spacing, in grid lines, after the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            lineUnitAfter?: boolean;
            /**
             *
             * Gets or sets the amount of spacing, in grid lines, before the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            lineUnitBefore?: boolean;
            /**
             *
             * Gets or sets the outline level for the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            outlineLevel?: boolean;
            /**
             *
             * Gets or sets the right indent value, in points, for the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            rightIndent?: boolean;
            /**
             *
             * Gets or sets the spacing, in points, after the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            spaceAfter?: boolean;
            /**
             *
             * Gets or sets the spacing, in points, before the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            spaceBefore?: boolean;
            /**
             *
             * Gets or sets the style name for the paragraph. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.1]
             */
            style?: boolean;
            /**
             *
             * Gets or sets the built-in style name for the paragraph. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: boolean;
            /**
             *
             * Gets the level of the paragraph's table. It returns 0 if the paragraph is not in a table. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            tableNestingLevel?: boolean;
            /**
             *
             * Gets the text of the paragraph. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            text?: boolean;
        }
        /**
         *
         * Contains a collection of {@link Word.Paragraph} objects.
         *
         * [Api set: WordApi 1.1]
         */
        interface ParagraphCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Gets the text format of the paragraph. Use this to get and set font name, size, color, and other properties.
            *
            * [Api set: WordApi 1.1]
            */
            font?: Word.Interfaces.FontLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the List to which this paragraph belongs. Throws if the paragraph is not in a list.
            *
            * [Api set: WordApi 1.3]
            */
            list?: Word.Interfaces.ListLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the ListItem for the paragraph. Throws if the paragraph is not part of a list.
            *
            * [Api set: WordApi 1.3]
            */
            listItem?: Word.Interfaces.ListItemLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the ListItem for the paragraph. Returns a null object if the paragraph is not part of a list.
            *
            * [Api set: WordApi 1.3]
            */
            listItemOrNullObject?: Word.Interfaces.ListItemLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the List to which this paragraph belongs. Returns a null object if the paragraph is not in a list.
            *
            * [Api set: WordApi 1.3]
            */
            listOrNullObject?: Word.Interfaces.ListLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the parent body of the paragraph.
            *
            * [Api set: WordApi 1.3]
            */
            parentBody?: Word.Interfaces.BodyLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the content control that contains the paragraph. Throws if there isn't a parent content control.
            *
            * [Api set: WordApi 1.1]
            */
            parentContentControl?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the content control that contains the paragraph. Returns a null object if there isn't a parent content control.
            *
            * [Api set: WordApi 1.3]
            */
            parentContentControlOrNullObject?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table that contains the paragraph. Throws if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTable?: Word.Interfaces.TableLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table cell that contains the paragraph. Throws if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCell?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table cell that contains the paragraph. Returns a null object if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCellOrNullObject?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table that contains the paragraph. Returns a null object if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableOrNullObject?: Word.Interfaces.TableLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the alignment for a paragraph. The value can be 'left', 'centered', 'right', or 'justified'.
             *
             * [Api set: WordApi 1.1]
             */
            alignment?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the value, in points, for a first line or hanging indent. Use a positive value to set a first-line indent, and use a negative value to set a hanging indent.
             *
             * [Api set: WordApi 1.1]
             */
            firstLineIndent?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates the paragraph is the last one inside its parent body. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            isLastParagraph?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Checks whether the paragraph is a list item. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            isListItem?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the left indent value, in points, for the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            leftIndent?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the line spacing, in points, for the specified paragraph. In the Word UI, this value is divided by 12.
             *
             * [Api set: WordApi 1.1]
             */
            lineSpacing?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the amount of spacing, in grid lines, after the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            lineUnitAfter?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the amount of spacing, in grid lines, before the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            lineUnitBefore?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the outline level for the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            outlineLevel?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the right indent value, in points, for the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            rightIndent?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the spacing, in points, after the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            spaceAfter?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the spacing, in points, before the paragraph.
             *
             * [Api set: WordApi 1.1]
             */
            spaceBefore?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the style name for the paragraph. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.1]
             */
            style?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the built-in style name for the paragraph. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the level of the paragraph's table. It returns 0 if the paragraph is not in a table. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            tableNestingLevel?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the text of the paragraph. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            text?: boolean;
        }
        /**
         *
         * Represents a contiguous area in a document.
         *
         * [Api set: WordApi 1.1]
         */
        interface RangeLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the text format of the range. Use this to get and set font name, size, color, and other properties.
            *
            * [Api set: WordApi 1.1]
            */
            font?: Word.Interfaces.FontLoadOptions;
            /**
            *
            * Gets the parent body of the range.
            *
            * [Api set: WordApi 1.3]
            */
            parentBody?: Word.Interfaces.BodyLoadOptions;
            /**
            *
            * Gets the content control that contains the range. Throws if there isn't a parent content control.
            *
            * [Api set: WordApi 1.1]
            */
            parentContentControl?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * Gets the content control that contains the range. Returns a null object if there isn't a parent content control.
            *
            * [Api set: WordApi 1.3]
            */
            parentContentControlOrNullObject?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * Gets the table that contains the range. Throws if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTable?: Word.Interfaces.TableLoadOptions;
            /**
            *
            * Gets the table cell that contains the range. Throws if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCell?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * Gets the table cell that contains the range. Returns a null object if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCellOrNullObject?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * Gets the table that contains the range. Returns a null object if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableOrNullObject?: Word.Interfaces.TableLoadOptions;
            /**
             *
             * Gets the first hyperlink in the range, or sets a hyperlink on the range. All hyperlinks in the range are deleted when you set a new hyperlink on the range. Use a '#' to separate the address part from the optional location part.
             *
             * [Api set: WordApi 1.3]
             */
            hyperlink?: boolean;
            /**
             *
             * Checks whether the range length is zero. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            isEmpty?: boolean;
            /**
             *
             * Gets or sets the style name for the range. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.1]
             */
            style?: boolean;
            /**
             *
             * Gets or sets the built-in style name for the range. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: boolean;
            /**
             *
             * Gets the text of the range. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            text?: boolean;
        }
        /**
         *
         * Contains a collection of {@link Word.Range} objects.
         *
         * [Api set: WordApi 1.1]
         */
        interface RangeCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Gets the text format of the range. Use this to get and set font name, size, color, and other properties.
            *
            * [Api set: WordApi 1.1]
            */
            font?: Word.Interfaces.FontLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the parent body of the range.
            *
            * [Api set: WordApi 1.3]
            */
            parentBody?: Word.Interfaces.BodyLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the content control that contains the range. Throws if there isn't a parent content control.
            *
            * [Api set: WordApi 1.1]
            */
            parentContentControl?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the content control that contains the range. Returns a null object if there isn't a parent content control.
            *
            * [Api set: WordApi 1.3]
            */
            parentContentControlOrNullObject?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table that contains the range. Throws if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTable?: Word.Interfaces.TableLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table cell that contains the range. Throws if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCell?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table cell that contains the range. Returns a null object if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCellOrNullObject?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table that contains the range. Returns a null object if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableOrNullObject?: Word.Interfaces.TableLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets the first hyperlink in the range, or sets a hyperlink on the range. All hyperlinks in the range are deleted when you set a new hyperlink on the range. Use a '#' to separate the address part from the optional location part.
             *
             * [Api set: WordApi 1.3]
             */
            hyperlink?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Checks whether the range length is zero. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            isEmpty?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the style name for the range. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.1]
             */
            style?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the built-in style name for the range. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the text of the range. Read-only.
             *
             * [Api set: WordApi 1.1]
             */
            text?: boolean;
        }
        /**
         *
         * Specifies the options to be included in a search operation.
         *
         * [Api set: WordApi 1.1]
         */
        interface SearchOptionsLoadOptions {
            $all?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to ignore all punctuation characters between words. Corresponds to the Ignore punctuation check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            ignorePunct?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to ignore all whitespace between words. Corresponds to the Ignore whitespace characters check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            ignoreSpace?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to perform a case sensitive search. Corresponds to the Match case check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            matchCase?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to match words that begin with the search string. Corresponds to the Match prefix check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            matchPrefix?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to match words that end with the search string. Corresponds to the Match suffix check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            matchSuffix?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether to find operation only entire words, not text that is part of a larger word. Corresponds to the Find whole words only check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            matchWholeWord?: boolean;
            /**
             *
             * Gets or sets a value that indicates whether the search will be performed using special search operators. Corresponds to the Use wildcards check box in the Find and Replace dialog box.
             *
             * [Api set: WordApi 1.1]
             */
            matchWildcards?: boolean;
        }
        /**
         *
         * Represents a section in a Word document.
         *
         * [Api set: WordApi 1.1]
         */
        interface SectionLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the body object of the section. This does not include the header/footer and other section metadata.
            *
            * [Api set: WordApi 1.1]
            */
            body?: Word.Interfaces.BodyLoadOptions;
        }
        /**
         *
         * Contains the collection of the document's {@link Word.Section} objects.
         *
         * [Api set: WordApi 1.1]
         */
        interface SectionCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Gets the body object of the section. This does not include the header/footer and other section metadata.
            *
            * [Api set: WordApi 1.1]
            */
            body?: Word.Interfaces.BodyLoadOptions;
        }
        /**
         *
         * Represents a table in a Word document.
         *
         * [Api set: WordApi 1.3]
         */
        interface TableLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the font. Use this to get and set font name, size, color, and other properties.
            *
            * [Api set: WordApi 1.3]
            */
            font?: Word.Interfaces.FontLoadOptions;
            /**
            *
            * Gets the parent body of the table.
            *
            * [Api set: WordApi 1.3]
            */
            parentBody?: Word.Interfaces.BodyLoadOptions;
            /**
            *
            * Gets the content control that contains the table. Throws if there isn't a parent content control.
            *
            * [Api set: WordApi 1.3]
            */
            parentContentControl?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * Gets the content control that contains the table. Returns a null object if there isn't a parent content control.
            *
            * [Api set: WordApi 1.3]
            */
            parentContentControlOrNullObject?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * Gets the table that contains this table. Throws if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTable?: Word.Interfaces.TableLoadOptions;
            /**
            *
            * Gets the table cell that contains this table. Throws if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCell?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * Gets the table cell that contains this table. Returns a null object if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCellOrNullObject?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * Gets the table that contains this table. Returns a null object if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableOrNullObject?: Word.Interfaces.TableLoadOptions;
            /**
             *
             * Gets or sets the alignment of the table against the page column. The value can be 'Left', 'Centered', or 'Right'.
             *
             * [Api set: WordApi 1.3]
             */
            alignment?: boolean;
            /**
             *
             * Gets and sets the number of header rows.
             *
             * [Api set: WordApi 1.3]
             */
            headerRowCount?: boolean;
            /**
             *
             * Gets and sets the horizontal alignment of every cell in the table. The value can be 'Left', 'Centered', 'Right', or 'Justified'.
             *
             * [Api set: WordApi 1.3]
             */
            horizontalAlignment?: boolean;
            /**
             *
             * Indicates whether all of the table rows are uniform. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            isUniform?: boolean;
            /**
             *
             * Gets the nesting level of the table. Top-level tables have level 1. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            nestingLevel?: boolean;
            /**
             *
             * Gets the number of rows in the table. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            rowCount?: boolean;
            /**
             *
             * Gets and sets the shading color. Color is specified in "#RRGGBB" format or by using the color name.
             *
             * [Api set: WordApi 1.3]
             */
            shadingColor?: boolean;
            /**
             *
             * Gets or sets the style name for the table. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.3]
             */
            style?: boolean;
            /**
             *
             * Gets and sets whether the table has banded columns.
             *
             * [Api set: WordApi 1.3]
             */
            styleBandedColumns?: boolean;
            /**
             *
             * Gets and sets whether the table has banded rows.
             *
             * [Api set: WordApi 1.3]
             */
            styleBandedRows?: boolean;
            /**
             *
             * Gets or sets the built-in style name for the table. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: boolean;
            /**
             *
             * Gets and sets whether the table has a first column with a special style.
             *
             * [Api set: WordApi 1.3]
             */
            styleFirstColumn?: boolean;
            /**
             *
             * Gets and sets whether the table has a last column with a special style.
             *
             * [Api set: WordApi 1.3]
             */
            styleLastColumn?: boolean;
            /**
             *
             * Gets and sets whether the table has a total (last) row with a special style.
             *
             * [Api set: WordApi 1.3]
             */
            styleTotalRow?: boolean;
            /**
             *
             * Gets and sets the text values in the table, as a 2D Javascript array.
             *
             * [Api set: WordApi 1.3]
             */
            values?: boolean;
            /**
             *
             * Gets and sets the vertical alignment of every cell in the table. The value can be 'Top', 'Center', or 'Bottom'.
             *
             * [Api set: WordApi 1.3]
             */
            verticalAlignment?: boolean;
            /**
             *
             * Gets and sets the width of the table in points.
             *
             * [Api set: WordApi 1.3]
             */
            width?: boolean;
        }
        /**
         *
         * Contains the collection of the document's Table objects.
         *
         * [Api set: WordApi 1.3]
         */
        interface TableCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Gets the font. Use this to get and set font name, size, color, and other properties.
            *
            * [Api set: WordApi 1.3]
            */
            font?: Word.Interfaces.FontLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the parent body of the table.
            *
            * [Api set: WordApi 1.3]
            */
            parentBody?: Word.Interfaces.BodyLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the content control that contains the table. Throws if there isn't a parent content control.
            *
            * [Api set: WordApi 1.3]
            */
            parentContentControl?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the content control that contains the table. Returns a null object if there isn't a parent content control.
            *
            * [Api set: WordApi 1.3]
            */
            parentContentControlOrNullObject?: Word.Interfaces.ContentControlLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table that contains this table. Throws if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTable?: Word.Interfaces.TableLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table cell that contains this table. Throws if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCell?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table cell that contains this table. Returns a null object if it is not contained in a table cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableCellOrNullObject?: Word.Interfaces.TableCellLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the table that contains this table. Returns a null object if it is not contained in a table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTableOrNullObject?: Word.Interfaces.TableLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the alignment of the table against the page column. The value can be 'Left', 'Centered', or 'Right'.
             *
             * [Api set: WordApi 1.3]
             */
            alignment?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the number of header rows.
             *
             * [Api set: WordApi 1.3]
             */
            headerRowCount?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the horizontal alignment of every cell in the table. The value can be 'Left', 'Centered', 'Right', or 'Justified'.
             *
             * [Api set: WordApi 1.3]
             */
            horizontalAlignment?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Indicates whether all of the table rows are uniform. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            isUniform?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the nesting level of the table. Top-level tables have level 1. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            nestingLevel?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the number of rows in the table. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            rowCount?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the shading color. Color is specified in "#RRGGBB" format or by using the color name.
             *
             * [Api set: WordApi 1.3]
             */
            shadingColor?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the style name for the table. Use this property for custom styles and localized style names. To use the built-in styles that are portable between locales, see the "styleBuiltIn" property.
             *
             * [Api set: WordApi 1.3]
             */
            style?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets whether the table has banded columns.
             *
             * [Api set: WordApi 1.3]
             */
            styleBandedColumns?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets whether the table has banded rows.
             *
             * [Api set: WordApi 1.3]
             */
            styleBandedRows?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the built-in style name for the table. Use this property for built-in styles that are portable between locales. To use custom styles or localized style names, see the "style" property.
             *
             * [Api set: WordApi 1.3]
             */
            styleBuiltIn?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets whether the table has a first column with a special style.
             *
             * [Api set: WordApi 1.3]
             */
            styleFirstColumn?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets whether the table has a last column with a special style.
             *
             * [Api set: WordApi 1.3]
             */
            styleLastColumn?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets whether the table has a total (last) row with a special style.
             *
             * [Api set: WordApi 1.3]
             */
            styleTotalRow?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the text values in the table, as a 2D Javascript array.
             *
             * [Api set: WordApi 1.3]
             */
            values?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the vertical alignment of every cell in the table. The value can be 'Top', 'Center', or 'Bottom'.
             *
             * [Api set: WordApi 1.3]
             */
            verticalAlignment?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the width of the table in points.
             *
             * [Api set: WordApi 1.3]
             */
            width?: boolean;
        }
        /**
         *
         * Represents a row in a Word document.
         *
         * [Api set: WordApi 1.3]
         */
        interface TableRowLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the font. Use this to get and set font name, size, color, and other properties.
            *
            * [Api set: WordApi 1.3]
            */
            font?: Word.Interfaces.FontLoadOptions;
            /**
            *
            * Gets parent table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTable?: Word.Interfaces.TableLoadOptions;
            /**
             *
             * Gets the number of cells in the row. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            cellCount?: boolean;
            /**
             *
             * Gets and sets the horizontal alignment of every cell in the row. The value can be 'Left', 'Centered', 'Right', or 'Justified'.
             *
             * [Api set: WordApi 1.3]
             */
            horizontalAlignment?: boolean;
            /**
             *
             * Checks whether the row is a header row. Read-only. To set the number of header rows, use HeaderRowCount on the Table object.
             *
             * [Api set: WordApi 1.3]
             */
            isHeader?: boolean;
            /**
             *
             * Gets and sets the preferred height of the row in points.
             *
             * [Api set: WordApi 1.3]
             */
            preferredHeight?: boolean;
            /**
             *
             * Gets the index of the row in its parent table. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            rowIndex?: boolean;
            /**
             *
             * Gets and sets the shading color. Color is specified in "#RRGGBB" format or by using the color name.
             *
             * [Api set: WordApi 1.3]
             */
            shadingColor?: boolean;
            /**
             *
             * Gets and sets the text values in the row, as a 2D Javascript array.
             *
             * [Api set: WordApi 1.3]
             */
            values?: boolean;
            /**
             *
             * Gets and sets the vertical alignment of the cells in the row. The value can be 'Top', 'Center', or 'Bottom'.
             *
             * [Api set: WordApi 1.3]
             */
            verticalAlignment?: boolean;
        }
        /**
         *
         * Contains the collection of the document's TableRow objects.
         *
         * [Api set: WordApi 1.3]
         */
        interface TableRowCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Gets the font. Use this to get and set font name, size, color, and other properties.
            *
            * [Api set: WordApi 1.3]
            */
            font?: Word.Interfaces.FontLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets parent table.
            *
            * [Api set: WordApi 1.3]
            */
            parentTable?: Word.Interfaces.TableLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets the number of cells in the row. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            cellCount?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the horizontal alignment of every cell in the row. The value can be 'Left', 'Centered', 'Right', or 'Justified'.
             *
             * [Api set: WordApi 1.3]
             */
            horizontalAlignment?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Checks whether the row is a header row. Read-only. To set the number of header rows, use HeaderRowCount on the Table object.
             *
             * [Api set: WordApi 1.3]
             */
            isHeader?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the preferred height of the row in points.
             *
             * [Api set: WordApi 1.3]
             */
            preferredHeight?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the index of the row in its parent table. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            rowIndex?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the shading color. Color is specified in "#RRGGBB" format or by using the color name.
             *
             * [Api set: WordApi 1.3]
             */
            shadingColor?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the text values in the row, as a 2D Javascript array.
             *
             * [Api set: WordApi 1.3]
             */
            values?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the vertical alignment of the cells in the row. The value can be 'Top', 'Center', or 'Bottom'.
             *
             * [Api set: WordApi 1.3]
             */
            verticalAlignment?: boolean;
        }
        /**
         *
         * Represents a table cell in a Word document.
         *
         * [Api set: WordApi 1.3]
         */
        interface TableCellLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the body object of the cell.
            *
            * [Api set: WordApi 1.3]
            */
            body?: Word.Interfaces.BodyLoadOptions;
            /**
            *
            * Gets the parent row of the cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentRow?: Word.Interfaces.TableRowLoadOptions;
            /**
            *
            * Gets the parent table of the cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTable?: Word.Interfaces.TableLoadOptions;
            /**
             *
             * Gets the index of the cell in its row. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            cellIndex?: boolean;
            /**
             *
             * Gets and sets the width of the cell's column in points. This is applicable to uniform tables.
             *
             * [Api set: WordApi 1.3]
             */
            columnWidth?: boolean;
            /**
             *
             * Gets and sets the horizontal alignment of the cell. The value can be 'Left', 'Centered', 'Right', or 'Justified'.
             *
             * [Api set: WordApi 1.3]
             */
            horizontalAlignment?: boolean;
            /**
             *
             * Gets the index of the cell's row in the table. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            rowIndex?: boolean;
            /**
             *
             * Gets or sets the shading color of the cell. Color is specified in "#RRGGBB" format or by using the color name.
             *
             * [Api set: WordApi 1.3]
             */
            shadingColor?: boolean;
            /**
             *
             * Gets and sets the text of the cell.
             *
             * [Api set: WordApi 1.3]
             */
            value?: boolean;
            /**
             *
             * Gets and sets the vertical alignment of the cell. The value can be 'Top', 'Center', or 'Bottom'.
             *
             * [Api set: WordApi 1.3]
             */
            verticalAlignment?: boolean;
            /**
             *
             * Gets the width of the cell in points. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            width?: boolean;
        }
        /**
         *
         * Contains the collection of the document's TableCell objects.
         *
         * [Api set: WordApi 1.3]
         */
        interface TableCellCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Gets the body object of the cell.
            *
            * [Api set: WordApi 1.3]
            */
            body?: Word.Interfaces.BodyLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the parent row of the cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentRow?: Word.Interfaces.TableRowLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the parent table of the cell.
            *
            * [Api set: WordApi 1.3]
            */
            parentTable?: Word.Interfaces.TableLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets the index of the cell in its row. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            cellIndex?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the width of the cell's column in points. This is applicable to uniform tables.
             *
             * [Api set: WordApi 1.3]
             */
            columnWidth?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the horizontal alignment of the cell. The value can be 'Left', 'Centered', 'Right', or 'Justified'.
             *
             * [Api set: WordApi 1.3]
             */
            horizontalAlignment?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the index of the cell's row in the table. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            rowIndex?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the shading color of the cell. Color is specified in "#RRGGBB" format or by using the color name.
             *
             * [Api set: WordApi 1.3]
             */
            shadingColor?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the text of the cell.
             *
             * [Api set: WordApi 1.3]
             */
            value?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the vertical alignment of the cell. The value can be 'Top', 'Center', or 'Bottom'.
             *
             * [Api set: WordApi 1.3]
             */
            verticalAlignment?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the width of the cell in points. Read-only.
             *
             * [Api set: WordApi 1.3]
             */
            width?: boolean;
        }
        /**
         *
         * Specifies the border style.
         *
         * [Api set: WordApi 1.3]
         */
        interface TableBorderLoadOptions {
            $all?: boolean;
            /**
             *
             * Gets or sets the table border color.
             *
             * [Api set: WordApi 1.3]
             */
            color?: boolean;
            /**
             *
             * Gets or sets the type of the table border.
             *
             * [Api set: WordApi 1.3]
             */
            type?: boolean;
            /**
             *
             * Gets or sets the width, in points, of the table border. Not applicable to table border types that have fixed widths.
             *
             * [Api set: WordApi 1.3]
             */
            width?: boolean;
        }
    }
}
declare namespace Word {
    /**
     * The RequestContext object facilitates requests to the Word application. Since the Office add-in and the Word application run in two different processes, the request context is required to get access to the Word object model from the add-in.
     */
    class RequestContext extends OfficeCore.RequestContext {
        constructor(url?: string);
        readonly document: Document;
        readonly application: Application;
    }
    /**
     * Executes a batch script that performs actions on the Word object model, using the RequestContext of previously created API objects.
     * @param objects - An array of previously created API objects. The array will be validated to make sure that all of the objects share the same context. The batch will use this shared RequestContext, which means that any changes applied to these objects will be picked up by "context.sync()".
     * @param batch - A function that takes in a RequestContext and returns a promise (typically, just the result of "context.sync()"). The context parameter facilitates requests to the Word application. Since the Office add-in and the Word application run in two different processes, the RequestContext is required to get access to the Word object model from the add-in.
     */
    function run<T>(objects: OfficeExtension.ClientObject[], batch: (context: Word.RequestContext) => Promise<T>): Promise<T>;
    /**
     * Executes a batch script that performs actions on the Word object model, using the RequestContext of a previously created API object. When the promise is resolved, any tracked objects that were automatically allocated during execution will be released.
     * @param object - A previously created API object. The batch will use the same RequestContext as the passed-in object, which means that any changes applied to the object will be picked up by "context.sync()".
     * @param batch - A function that takes in a RequestContext and returns a promise (typically, just the result of "context.sync()"). The context parameter facilitates requests to the Word application. Since the Office add-in and the Word application run in two different processes, the RequestContext is required to get access to the Word object model from the add-in.
     */
    function run<T>(object: OfficeExtension.ClientObject, batch: (context: Word.RequestContext) => Promise<T>): Promise<T>;
    /**
     * Executes a batch script that performs actions on the Word object model, using a new RequestContext. When the promise is resolved, any tracked objects that were automatically allocated during execution will be released.
     * @param batch - A function that takes in a RequestContext and returns a promise (typically, just the result of "context.sync()"). The context parameter facilitates requests to the Word application. Since the Office add-in and the Word application run in two different processes, the RequestContext is required to get access to the Word object model from the add-in.
     *
     * @remarks
     *
     * In addition to this signature, the method also has the following signatures, which allow you to resume using the request context of previously created objects:
     *
     * run<T>(object: OfficeExtension.ClientObject, batch: (context: Word.RequestContext) => Promise<T>): Promise<T>;
     *
     * run<T>(objects: OfficeExtension.ClientObject[], batch: (context: Word.RequestContext) => Promise<T>): Promise<T>;
     */
    function run<T>(batch: (context: Word.RequestContext) => Promise<T>): Promise<T>;
}


////////////////////////////////////////////////////////////////
//////////////////////// End Word APIs /////////////////////////
////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////
////////////////////// Begin OneNote APIs //////////////////////
////////////////////////////////////////////////////////////////

declare namespace OneNote {
    /**
     *
     * Provides information about the binding that raised the DataChanged event
     *
     * [Api set: OneNoteApi 1.1]
     */
    interface NotebookChangedEventArgs {
        /**
         *
         * Notebook id before change
         *
         * [Api set: OneNoteApi 1.3]
         */
        oldId: string;
        /**
         *
         * Gets the type of the event. See EventType for details.
         *
         * [Api set: OneNoteApi 1.3]
         */
        type: "NotebookChanged";
    }
    /**
     *
     * Provides information about the binding that raised the StickyNotesContextMenuExecuted event
     *
     * [Api set: OneNoteApi 1.5]
     */
    interface StickyNotesContextMenuExecutedEventArgs {
        /**
         *
         * Gets the type of the event. See EventType for details.
         *
         * [Api set: OneNoteApi 1.5]
         */
        type: "StickyNotesContextMenuExecuted";
    }
    /**
     *
     * Provides information about the binding that raised the section selection changed event
     *
     * [Api set: OneNoteApi 1.5]
     */
    interface SectionSelectionChangedEventArgs {
        /**
         *
         * Active Section id before change
         *
         * [Api set: OneNoteApi 1.5]
         */
        oldId: string;
        /**
         *
         * Gets the type of the event. See EventType for details.
         *
         * [Api set: OneNoteApi 1.5]
         */
        type: "SectionSelectionChanged";
    }
    /**
     *
     * Provides information about the binding that raised the Page selection changed event
     *
     * [Api set: OneNoteApi 1.1]
     */
    interface PageSelectionChangedEventArgs {
        /**
         *
         * Active Page id before change
         *
         * [Api set: OneNoteApi 1.3]
         */
        oldId: string;
        /**
         *
         * Gets the type of the event. See EventType for details.
         *
         * [Api set: OneNoteApi 1.3]
         */
        type: "PageSelectionChanged";
    }
    /**
     *
     * Represents the top-level object that contains all globally addressable OneNote objects such as notebooks, the active notebook, and the active section.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class Application extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the collection of notebooks that are open in the OneNote application instance. In OneNote Online, only one notebook at a time is open in the application instance. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly notebooks: OneNote.NotebookCollection;
        /**
         *
         * Gets the active notebook if one exists. If no notebook is active, throws ItemNotFound.
            [Deprecated] use await OneNoteNavigation.getActiveNotebook instead. see OneNoteNavigation.ts
         *
         * [Api set: OneNoteApi 1.1]
         */
        getActiveNotebook(): OneNote.Notebook;
        /**
         *
         * Gets the active notebook if one exists. If no notebook is active, returns null.
            [Deprecated] use await OneNoteNavigation.getActiveNotebook instead. see OneNoteNavigation.ts
         *
         * [Api set: OneNoteApi 1.1]
         */
        getActiveNotebookOrNull(): OneNote.Notebook;
        /**
         *
         * Gets the active outline if one exists, If no outline is active, throws ItemNotFound.
            [Deprecated] use await OneNoteNavigation.getActiveOutline instead. see OneNoteNavigation.ts
         *
         * [Api set: OneNoteApi 1.1]
         */
        getActiveOutline(): OneNote.Outline;
        /**
         *
         * Gets the active outline if one exists, otherwise returns null.
            [Deprecated] use await OneNoteNavigation.getActiveOutline instead. see OneNoteNavigation.ts
         *
         * [Api set: OneNoteApi 1.1]
         */
        getActiveOutlineOrNull(): OneNote.Outline;
        /**
         *
         * Gets the active page if one exists. If no page is active, throws ItemNotFound.
            [Deprecated] use await OneNoteNavigation.getActivePage instead. see OneNoteNavigation.ts
         *
         * [Api set: OneNoteApi 1.1]
         */
        getActivePage(): OneNote.Page;
        /**
         *
         * Gets the active page if one exists. If no page is active, returns null.
            [Deprecated] use await OneNoteNavigation.getActivePage instead. see OneNoteNavigation.ts
         *
         * [Api set: OneNoteApi 1.1]
         */
        getActivePageOrNull(): OneNote.Page;
        /**
         *
         * Gets the active Paragraph if one exists, If no Paragraph is active, throws ItemNotFound.
            [Deprecated] use await OneNoteNavigation.getActiveParagraph instead. see OneNoteNavigation.ts
         *
         * [Api set: OneNoteApi 1.1]
         */
        getActiveParagraph(): OneNote.Paragraph;
        /**
         *
         * Gets the active Paragraph if one exists, otherwise returns null.
            [Deprecated] use await OneNoteNavigation.getActiveParagraph instead. see OneNoteNavigation.ts
         *
         * [Api set: OneNoteApi 1.1]
         */
        getActiveParagraphOrNull(): OneNote.Paragraph;
        /**
         *
         * Gets the active section if one exists. If no section is active, throws ItemNotFound.
            [Deprecated] use await OneNoteNavigation.getActiveSection instead. see OneNoteNavigation.ts
         *
         * [Api set: OneNoteApi 1.1]
         */
        getActiveSection(): OneNote.Section;
        /**
         *
         * Gets the active section if one exists. If no section is active, returns null.
            [Deprecated] use await OneNoteNavigation.getActiveSection instead. see OneNoteNavigation.ts
         *
         * [Api set: OneNoteApi 1.1]
         */
        getActiveSectionOrNull(): OneNote.Section;
        getWindowSize(): OfficeExtension.ClientResult<number[]>;
        insertHtmlAtCurrentPosition(html: string): void;
        isViewingDeletedNotes(): OfficeExtension.ClientResult<boolean>;
        /**
         *
         * Opens the specified page in the application instance.
            [Deprecated] use await OneNoteNavigation.navigateToPage instead. see OneNoteNavigation.ts
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param page The page to open.
         */
        navigateToPage(page: OneNote.Page): void;
        /**
         *
         * Gets the specified page, and opens it in the application instance.
            [Deprecated] use await OneNoteNavigation.navigateToPageWithClientUrl instead. see OneNoteNavigation.ts
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param url The client url of the page to open.
         */
        navigateToPageWithClientUrl(url: string): OneNote.Page;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.Application` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.Application` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.Application` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.ApplicationLoadOptions): OneNote.Application;
        load(option?: string | string[]): OneNote.Application;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.Application;
        /**
         *
         * Occurs when the notebook is changed.
         *
         * [Api set: OneNoteApi 1.3]
         *
         * @eventproperty
         */
        readonly onNotebookChanged: OfficeExtension.EventHandlers<OneNote.NotebookChangedEventArgs>;
        /**
         *
         * Occurs when the Page Selection change is made.
         *
         * [Api set: OneNoteApi 1.5]
         *
         * @eventproperty
         */
        readonly onPageSelectionChanged: OfficeExtension.EventHandlers<OneNote.PageSelectionChangedEventArgs>;
        /**
         *
         * Occurs when the Section Selection change is made.
         *
         * [Api set: OneNoteApi 1.5]
         *
         * @eventproperty
         */
        readonly onSectionSelectionChanged: OfficeExtension.EventHandlers<OneNote.SectionSelectionChangedEventArgs>;
        /**
         *
         * Occurs when the host sends a command to the add-ins.
         *
         * [Api set: OneNoteApi 1.5]
         *
         * @eventproperty
         */
        readonly onStickyNotesContextMenuExecutedEvent: OfficeExtension.EventHandlers<OneNote.StickyNotesContextMenuExecutedEventArgs>;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.Application object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.ApplicationData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.ApplicationData;
    }
    /**
     *
     * Represents ink analysis data for a given set of ink strokes.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class InkAnalysis extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the parent page object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly page: OneNote.Page;
        /**
         *
         * Gets the ID of the InkAnalysis object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: OneNote.InkAnalysis): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.InkAnalysisUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: OneNote.InkAnalysis): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.InkAnalysis` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.InkAnalysis` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.InkAnalysis` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.InkAnalysisLoadOptions): OneNote.InkAnalysis;
        load(option?: string | string[]): OneNote.InkAnalysis;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.InkAnalysis;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.InkAnalysis;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.InkAnalysis;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.InkAnalysis object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.InkAnalysisData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.InkAnalysisData;
    }
    /**
     *
     * Represents ink analysis data for an identified paragraph formed by ink strokes.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class InkAnalysisParagraph extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Reference to the parent InkAnalysisPage. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly inkAnalysis: OneNote.InkAnalysis;
        /**
         *
         * Gets the ink analysis lines in this ink analysis paragraph. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly lines: OneNote.InkAnalysisLineCollection;
        /**
         *
         * Gets the ID of the InkAnalysisParagraph object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: OneNote.InkAnalysisParagraph): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.InkAnalysisParagraphUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: OneNote.InkAnalysisParagraph): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.InkAnalysisParagraph` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.InkAnalysisParagraph` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.InkAnalysisParagraph` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.InkAnalysisParagraphLoadOptions): OneNote.InkAnalysisParagraph;
        load(option?: string | string[]): OneNote.InkAnalysisParagraph;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.InkAnalysisParagraph;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.InkAnalysisParagraph;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.InkAnalysisParagraph;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.InkAnalysisParagraph object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.InkAnalysisParagraphData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.InkAnalysisParagraphData;
    }
    /**
     *
     * Represents a collection of InkAnalysisParagraph objects.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class InkAnalysisParagraphCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: OneNote.InkAnalysisParagraph[];
        /**
         *
         * Returns the number of InkAnalysisParagraphs in the page. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Gets a InkAnalysisParagraph object by ID or by its index in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index The ID of the InkAnalysisParagraph object, or the index location of the InkAnalysisParagraph object in the collection.
         */
        getItem(index: number | string): OneNote.InkAnalysisParagraph;
        /**
         *
         * Gets a InkAnalysisParagraph on its position in the collection.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): OneNote.InkAnalysisParagraph;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.InkAnalysisParagraphCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.InkAnalysisParagraphCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.InkAnalysisParagraphCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.InkAnalysisParagraphCollectionLoadOptions & OneNote.Interfaces.CollectionLoadOptions): OneNote.InkAnalysisParagraphCollection;
        load(option?: string | string[]): OneNote.InkAnalysisParagraphCollection;
        load(option?: OfficeExtension.LoadOption): OneNote.InkAnalysisParagraphCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.InkAnalysisParagraphCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.InkAnalysisParagraphCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `OneNote.InkAnalysisParagraphCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.InkAnalysisParagraphCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): OneNote.Interfaces.InkAnalysisParagraphCollectionData;
    }
    /**
     *
     * Represents ink analysis data for an identified text line formed by ink strokes.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class InkAnalysisLine extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Reference to the parent InkAnalysisParagraph. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly paragraph: OneNote.InkAnalysisParagraph;
        /**
         *
         * Gets the ink analysis words in this ink analysis line. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly words: OneNote.InkAnalysisWordCollection;
        /**
         *
         * Gets the ID of the InkAnalysisLine object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: OneNote.InkAnalysisLine): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.InkAnalysisLineUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: OneNote.InkAnalysisLine): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.InkAnalysisLine` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.InkAnalysisLine` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.InkAnalysisLine` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.InkAnalysisLineLoadOptions): OneNote.InkAnalysisLine;
        load(option?: string | string[]): OneNote.InkAnalysisLine;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.InkAnalysisLine;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.InkAnalysisLine;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.InkAnalysisLine;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.InkAnalysisLine object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.InkAnalysisLineData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.InkAnalysisLineData;
    }
    /**
     *
     * Represents a collection of InkAnalysisLine objects.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class InkAnalysisLineCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: OneNote.InkAnalysisLine[];
        /**
         *
         * Returns the number of InkAnalysisLines in the page. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Gets a InkAnalysisLine object by ID or by its index in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index The ID of the InkAnalysisLine object, or the index location of the InkAnalysisLine object in the collection.
         */
        getItem(index: number | string): OneNote.InkAnalysisLine;
        /**
         *
         * Gets a InkAnalysisLine on its position in the collection.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): OneNote.InkAnalysisLine;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.InkAnalysisLineCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.InkAnalysisLineCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.InkAnalysisLineCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.InkAnalysisLineCollectionLoadOptions & OneNote.Interfaces.CollectionLoadOptions): OneNote.InkAnalysisLineCollection;
        load(option?: string | string[]): OneNote.InkAnalysisLineCollection;
        load(option?: OfficeExtension.LoadOption): OneNote.InkAnalysisLineCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.InkAnalysisLineCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.InkAnalysisLineCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `OneNote.InkAnalysisLineCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.InkAnalysisLineCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): OneNote.Interfaces.InkAnalysisLineCollectionData;
    }
    /**
     *
     * Represents ink analysis data for an identified word formed by ink strokes.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class InkAnalysisWord extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Reference to the parent InkAnalysisLine. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly line: OneNote.InkAnalysisLine;
        /**
         *
         * Gets the ID of the InkAnalysisWord object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         *
         * The id of the recognized language in this inkAnalysisWord. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly languageId: string;
        /**
         *
         * Weak references to the ink strokes that were recognized as part of this ink analysis word. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly strokePointers: OneNote.InkStrokePointer[];
        /**
         *
         * The words that were recognized in this ink word, in order of likelihood. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly wordAlternates: string[];
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: OneNote.InkAnalysisWord): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.InkAnalysisWordUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: OneNote.InkAnalysisWord): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.InkAnalysisWord` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.InkAnalysisWord` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.InkAnalysisWord` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.InkAnalysisWordLoadOptions): OneNote.InkAnalysisWord;
        load(option?: string | string[]): OneNote.InkAnalysisWord;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.InkAnalysisWord;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.InkAnalysisWord;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.InkAnalysisWord;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.InkAnalysisWord object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.InkAnalysisWordData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.InkAnalysisWordData;
    }
    /**
     *
     * Represents a collection of InkAnalysisWord objects.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class InkAnalysisWordCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: OneNote.InkAnalysisWord[];
        /**
         *
         * Returns the number of InkAnalysisWords in the page. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Gets a InkAnalysisWord object by ID or by its index in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index The ID of the InkAnalysisWord object, or the index location of the InkAnalysisWord object in the collection.
         */
        getItem(index: number | string): OneNote.InkAnalysisWord;
        /**
         *
         * Gets a InkAnalysisWord on its position in the collection.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): OneNote.InkAnalysisWord;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.InkAnalysisWordCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.InkAnalysisWordCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.InkAnalysisWordCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.InkAnalysisWordCollectionLoadOptions & OneNote.Interfaces.CollectionLoadOptions): OneNote.InkAnalysisWordCollection;
        load(option?: string | string[]): OneNote.InkAnalysisWordCollection;
        load(option?: OfficeExtension.LoadOption): OneNote.InkAnalysisWordCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.InkAnalysisWordCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.InkAnalysisWordCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `OneNote.InkAnalysisWordCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.InkAnalysisWordCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): OneNote.Interfaces.InkAnalysisWordCollectionData;
    }
    /**
     *
     * Represents a group of ink strokes.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class FloatingInk extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the strokes of the FloatingInk object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly inkStrokes: OneNote.InkStrokeCollection;
        /**
         *
         * Gets the PageContent parent of the FloatingInk object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly pageContent: OneNote.PageContent;
        /**
         *
         * Gets the ID of the FloatingInk object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.FloatingInk` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.FloatingInk` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.FloatingInk` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.FloatingInkLoadOptions): OneNote.FloatingInk;
        load(option?: string | string[]): OneNote.FloatingInk;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.FloatingInk;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.FloatingInk;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.FloatingInk;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.FloatingInk object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.FloatingInkData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.FloatingInkData;
    }
    /**
     *
     * Represents a single stroke of ink.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class InkStroke extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the ID of the InkStroke object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly floatingInk: OneNote.FloatingInk;
        /**
         *
         * Gets the ID of the InkStroke object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.InkStroke` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.InkStroke` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.InkStroke` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.InkStrokeLoadOptions): OneNote.InkStroke;
        load(option?: string | string[]): OneNote.InkStroke;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.InkStroke;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.InkStroke;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.InkStroke;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.InkStroke object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.InkStrokeData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.InkStrokeData;
    }
    /**
     *
     * Represents a collection of InkStroke objects.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class InkStrokeCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: OneNote.InkStroke[];
        /**
         *
         * Returns the number of InkStrokes in the page. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Gets a InkStroke object by ID or by its index in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index The ID of the InkStroke object, or the index location of the InkStroke object in the collection.
         */
        getItem(index: number | string): OneNote.InkStroke;
        /**
         *
         * Gets a InkStroke on its position in the collection.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): OneNote.InkStroke;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.InkStrokeCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.InkStrokeCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.InkStrokeCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.InkStrokeCollectionLoadOptions & OneNote.Interfaces.CollectionLoadOptions): OneNote.InkStrokeCollection;
        load(option?: string | string[]): OneNote.InkStrokeCollection;
        load(option?: OfficeExtension.LoadOption): OneNote.InkStrokeCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.InkStrokeCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.InkStrokeCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `OneNote.InkStrokeCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.InkStrokeCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): OneNote.Interfaces.InkStrokeCollectionData;
    }
    /**
     *
     * A container for the ink in a word in a paragraph.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class InkWord extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * The parent paragraph containing the ink word. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly paragraph: OneNote.Paragraph;
        /**
         *
         * Gets the ID of the InkWord object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         *
         * The id of the recognized language in this ink word. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly languageId: string;
        /**
         *
         * The words that were recognized in this ink word, in order of likelihood. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly wordAlternates: string[];
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.InkWord` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.InkWord` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.InkWord` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.InkWordLoadOptions): OneNote.InkWord;
        load(option?: string | string[]): OneNote.InkWord;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.InkWord;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.InkWord;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.InkWord;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.InkWord object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.InkWordData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.InkWordData;
    }
    /**
     *
     * Represents a collection of InkWord objects.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class InkWordCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: OneNote.InkWord[];
        /**
         *
         * Returns the number of InkWords in the page. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Gets a InkWord object by ID or by its index in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index The ID of the InkWord object, or the index location of the InkWord object in the collection.
         */
        getItem(index: number | string): OneNote.InkWord;
        /**
         *
         * Gets a InkWord on its position in the collection.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): OneNote.InkWord;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.InkWordCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.InkWordCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.InkWordCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.InkWordCollectionLoadOptions & OneNote.Interfaces.CollectionLoadOptions): OneNote.InkWordCollection;
        load(option?: string | string[]): OneNote.InkWordCollection;
        load(option?: OfficeExtension.LoadOption): OneNote.InkWordCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.InkWordCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.InkWordCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `OneNote.InkWordCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.InkWordCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): OneNote.Interfaces.InkWordCollectionData;
    }
    /**
     *
     * Represents a OneNote notebook. Notebooks contain section groups and sections.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class Notebook extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * The section groups in the notebook. Read only
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly sectionGroups: OneNote.SectionGroupCollection;
        /**
         *
         * The the sections of the notebook. Read only
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly sections: OneNote.SectionCollection;
        /**
         *
         * The url of the site that this notebook is located. Read only
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly baseUrl: string;
        /**
         *
         * The client url of the notebook. Read only
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly clientUrl: string;
        /**
         *
         * Gets the ID of the notebook. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         *
         * True if the Notebook is not created by the user (i.e. 'Misplaced Sections'). Read only
         *
         * [Api set: OneNoteApi 1.2]
         */
        readonly isVirtual: boolean;
        /**
         *
         * Gets the name of the notebook. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly name: string;
        /**
         *
         * Adds a new section to the end of the notebook.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param name The name of the new section.
         */
        addSection(name: string): OneNote.Section;
        /**
         *
         * Adds a new section group to the end of the notebook.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param name The name of the new section.
         */
        addSectionGroup(name: string): OneNote.SectionGroup;
        /**
         *
         * Gets the REST API ID.
         *
         * [Api set: OneNoteApi 1.1]
         */
        getRestApiId(): OfficeExtension.ClientResult<string>;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.Notebook` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.Notebook` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.Notebook` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.NotebookLoadOptions): OneNote.Notebook;
        load(option?: string | string[]): OneNote.Notebook;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.Notebook;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.Notebook;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.Notebook;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.Notebook object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.NotebookData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.NotebookData;
    }
    /**
     *
     * Represents a collection of notebooks.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class NotebookCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: OneNote.Notebook[];
        /**
         *
         * Returns the number of notebooks in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Gets the collection of notebooks with the specified name that are open in the application instance.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param name The name of the notebook.
         */
        getByName(name: string): OneNote.NotebookCollection;
        /**
         *
         * Gets a notebook by ID or by its index in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index The ID of the notebook, or the index location of the notebook in the collection.
         */
        getItem(index: number | string): OneNote.Notebook;
        /**
         *
         * Gets a notebook on its position in the collection.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): OneNote.Notebook;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.NotebookCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.NotebookCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.NotebookCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.NotebookCollectionLoadOptions & OneNote.Interfaces.CollectionLoadOptions): OneNote.NotebookCollection;
        load(option?: string | string[]): OneNote.NotebookCollection;
        load(option?: OfficeExtension.LoadOption): OneNote.NotebookCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.NotebookCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.NotebookCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `OneNote.NotebookCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.NotebookCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): OneNote.Interfaces.NotebookCollectionData;
    }
    /**
     *
     * Represents a OneNote section group. Section groups can contain sections and other section groups.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class SectionGroup extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the notebook that contains the section group. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly notebook: OneNote.Notebook;
        /**
         *
         * Gets the section group that contains the section group. Throws ItemNotFound if the section group is a direct child of the notebook. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly parentSectionGroup: OneNote.SectionGroup;
        /**
         *
         * Gets the section group that contains the section group. Returns null if the section group is a direct child of the notebook. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly parentSectionGroupOrNull: OneNote.SectionGroup;
        /**
         *
         * The collection of section groups in the section group. Read only
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly sectionGroups: OneNote.SectionGroupCollection;
        /**
         *
         * The collection of sections in the section group. Read only
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly sections: OneNote.SectionCollection;
        /**
         *
         * The client url of the section group. Read only
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly clientUrl: string;
        /**
         *
         * Gets the ID of the section group. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         *
         * Gets the name of the section group. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly name: string;
        /**
         *
         * Adds a new section to the end of the section group.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param title The name of the new section.
         */
        addSection(title: string): OneNote.Section;
        /**
         *
         * Adds a new section group to the end of this sectionGroup.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param name The name of the new section.
         */
        addSectionGroup(name: string): OneNote.SectionGroup;
        /**
         *
         * Gets the REST API ID.
         *
         * [Api set: OneNoteApi 1.1]
         */
        getRestApiId(): OfficeExtension.ClientResult<string>;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.SectionGroup` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.SectionGroup` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.SectionGroup` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.SectionGroupLoadOptions): OneNote.SectionGroup;
        load(option?: string | string[]): OneNote.SectionGroup;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.SectionGroup;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.SectionGroup;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.SectionGroup;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.SectionGroup object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.SectionGroupData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.SectionGroupData;
    }
    /**
     *
     * Represents a collection of section groups.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class SectionGroupCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: OneNote.SectionGroup[];
        /**
         *
         * Returns the number of section groups in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Gets the collection of section groups with the specified name.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param name The name of the section group.
         */
        getByName(name: string): OneNote.SectionGroupCollection;
        /**
         *
         * Gets a section group by ID or by its index in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index The ID of the section group, or the index location of the section group in the collection.
         */
        getItem(index: number | string): OneNote.SectionGroup;
        /**
         *
         * Gets a section group on its position in the collection.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): OneNote.SectionGroup;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.SectionGroupCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.SectionGroupCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.SectionGroupCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.SectionGroupCollectionLoadOptions & OneNote.Interfaces.CollectionLoadOptions): OneNote.SectionGroupCollection;
        load(option?: string | string[]): OneNote.SectionGroupCollection;
        load(option?: OfficeExtension.LoadOption): OneNote.SectionGroupCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.SectionGroupCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.SectionGroupCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `OneNote.SectionGroupCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.SectionGroupCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): OneNote.Interfaces.SectionGroupCollectionData;
    }
    /**
     *
     * Represents a OneNote section. Sections can contain pages.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class Section extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the notebook that contains the section. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly notebook: OneNote.Notebook;
        /**
         *
         * The collection of pages in the section. Read only
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly pages: OneNote.PageCollection;
        /**
         *
         * Gets the section group that contains the section. Throws ItemNotFound if the section is a direct child of the notebook. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly parentSectionGroup: OneNote.SectionGroup;
        /**
         *
         * Gets the section group that contains the section. Returns null if the section is a direct child of the notebook. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly parentSectionGroupOrNull: OneNote.SectionGroup;
        /**
         *
         * The client url of the section. Read only
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly clientUrl: string;
        /**
         *
         * Gets the ID of the section. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         *
         * True if this section is encrypted with a password. Read only
         *
         * [Api set: OneNoteApi 1.2]
         */
        readonly isEncrypted: boolean;
        /**
         *
         * True if this section is locked. Read only
         *
         * [Api set: OneNoteApi 1.2]
         */
        readonly isLocked: boolean;
        /**
         *
         * Gets the name of the section. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly name: string;
        /**
         *
         * The web url of the page. Read only
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly webUrl: string;
        /**
         *
         * Adds a new page to the end of the section.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param title The title of the new page.
         */
        addPage(title: string): OneNote.Page;
        /**
         *
         * Copies this section to specified notebook.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param destinationNotebook The notebook to copy this section to.
         */
        copyToNotebook(destinationNotebook: OneNote.Notebook): OneNote.Section;
        /**
         *
         * Copies this section to specified section group.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param destinationSectionGroup The section group to copy this section to.
         */
        copyToSectionGroup(destinationSectionGroup: OneNote.SectionGroup): OneNote.Section;
        /**
         *
         * Gets the REST API ID.
         *
         * [Api set: OneNoteApi 1.1]
         */
        getRestApiId(): OfficeExtension.ClientResult<string>;
        /**
         *
         * Inserts a new section before or after the current section.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param location The location of the new section relative to the current section.
         * @param title The name of the new section.
         */
        insertSectionAsSibling(location: OneNote.InsertLocation, title: string): OneNote.Section;
        /**
         *
         * Inserts a new section before or after the current section.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param location The location of the new section relative to the current section.
         * @param title The name of the new section.
         */
        insertSectionAsSibling(location: "Before" | "After", title: string): OneNote.Section;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.Section` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.Section` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.Section` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.SectionLoadOptions): OneNote.Section;
        load(option?: string | string[]): OneNote.Section;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.Section;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.Section;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.Section;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.Section object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.SectionData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.SectionData;
    }
    /**
     *
     * Represents a collection of sections.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class SectionCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: OneNote.Section[];
        /**
         *
         * Returns the number of sections in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Gets the collection of sections with the specified name.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param name The name of the section.
         */
        getByName(name: string): OneNote.SectionCollection;
        /**
         *
         * Gets a section by ID or by its index in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index The ID of the section, or the index location of the section in the collection.
         */
        getItem(index: number | string): OneNote.Section;
        /**
         *
         * Gets a section on its position in the collection.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): OneNote.Section;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.SectionCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.SectionCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.SectionCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.SectionCollectionLoadOptions & OneNote.Interfaces.CollectionLoadOptions): OneNote.SectionCollection;
        load(option?: string | string[]): OneNote.SectionCollection;
        load(option?: OfficeExtension.LoadOption): OneNote.SectionCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.SectionCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.SectionCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `OneNote.SectionCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.SectionCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): OneNote.Interfaces.SectionCollectionData;
    }
    /**
     *
     * Represents a OneNote page.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class Page extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * The collection of PageContent objects on the page. Read only
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly contents: OneNote.PageContentCollection;
        /**
         *
         * Text interpretation for the ink on the page. Returns null if there is no ink analysis information. Read only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly inkAnalysisOrNull: OneNote.InkAnalysis;
        /**
         *
         * Gets the section that contains the page. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly parentSection: OneNote.Section;
        /**
         *
         * Gets the ClassNotebookPageSource to the page.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly classNotebookPageSource: string;
        /**
         *
         * The client url of the page. Read only
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly clientUrl: string;
        /**
         *
         * Gets the ID of the page. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         *
         * Gets or sets the indentation level of the page.
         *
         * [Api set: OneNoteApi 1.1]
         */
        pageLevel: number;
        /**
         *
         * Gets or sets the title of the page.
         *
         * [Api set: OneNoteApi 1.1]
         */
        title: string;
        /**
         *
         * The web url of the page. Read only
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly webUrl: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: OneNote.Page): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.PageUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: OneNote.Page): void;
        /**
         *
         * Adds an Outline to the page at the specified position.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param left The left position of the top, left corner of the Outline.
         * @param top The top position of the top, left corner of the Outline.
         * @param html An HTML string that describes the visual presentation of the Outline. See {@link https://docs.microsoft.com/office/dev/add-ins/onenote/onenote-add-ins-page-content#supported-html | Supported HTML} for the OneNote add-ins JavaScript API.
         */
        addOutline(left: number, top: number, html: string): OneNote.Outline;
        /**
         *
         * Return a json string with node id and content in html format.
         *
         * [Api set: OneNoteApi 1.1]
         */
        analyzePage(): OfficeExtension.ClientResult<string>;
        /**
         *
         * Inserts a new page with translated content.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param translatedContent Translated content of the page
         */
        applyTranslation(translatedContent: string): void;
        /**
         *
         * Copies this page to specified section.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param destinationSection The section to copy this page to.
         */
        copyToSection(destinationSection: OneNote.Section): OneNote.Page;
        /**
         *
         * Copies this page to specified section and sets ClassNotebookPageSource.
         *
         * [Api set: OneNoteApi 1.1]
         */
        copyToSectionAndSetClassNotebookPageSource(destinationSection: OneNote.Section): OneNote.Page;
        /**
         *
         * Gets the REST API ID.
         *
         * [Api set: OneNoteApi 1.1]
         */
        getRestApiId(): OfficeExtension.ClientResult<string>;
        /**
         *
         * Does the page has content title.
         *
         * [Api set: OneNoteApi 1.1]
         */
        hasTitleContent(): OfficeExtension.ClientResult<boolean>;
        /**
         *
         * Inserts a new page before or after the current page.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param location The location of the new page relative to the current page.
         * @param title The title of the new page.
         */
        insertPageAsSibling(location: OneNote.InsertLocation, title: string): OneNote.Page;
        /**
         *
         * Inserts a new page before or after the current page.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param location The location of the new page relative to the current page.
         * @param title The title of the new page.
         */
        insertPageAsSibling(location: "Before" | "After", title: string): OneNote.Page;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.Page` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.Page` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.Page` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.PageLoadOptions): OneNote.Page;
        load(option?: string | string[]): OneNote.Page;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.Page;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.Page;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.Page;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.Page object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.PageData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.PageData;
    }
    /**
     *
     * Represents a collection of pages.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class PageCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: OneNote.Page[];
        /**
         *
         * Returns the number of pages in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Gets the collection of pages with the specified title.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param title The title of the page.
         */
        getByTitle(title: string): OneNote.PageCollection;
        /**
         *
         * Gets a page by ID or by its index in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index The ID of the page, or the index location of the page in the collection.
         */
        getItem(index: number | string): OneNote.Page;
        /**
         *
         * Gets a page on its position in the collection.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): OneNote.Page;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.PageCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.PageCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.PageCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.PageCollectionLoadOptions & OneNote.Interfaces.CollectionLoadOptions): OneNote.PageCollection;
        load(option?: string | string[]): OneNote.PageCollection;
        load(option?: OfficeExtension.LoadOption): OneNote.PageCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.PageCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.PageCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `OneNote.PageCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.PageCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): OneNote.Interfaces.PageCollectionData;
    }
    /**
     *
     * Represents a region on a page that contains top-level content types such as Outline or Image. A PageContent object can be assigned an XY position.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class PageContent extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the Image in the PageContent object. Throws an exception if PageContentType is not Image.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly image: OneNote.Image;
        /**
         *
         * Gets the ink in the PageContent object. Throws an exception if PageContentType is not Ink.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly ink: OneNote.FloatingInk;
        /**
         *
         * Gets the Outline in the PageContent object. Throws an exception if PageContentType is not Outline.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly outline: OneNote.Outline;
        /**
         *
         * Gets the page that contains the PageContent object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly parentPage: OneNote.Page;
        /**
         *
         * Gets the ID of the PageContent object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         *
         * Gets or sets the left (X-axis) position of the PageContent object.
         *
         * [Api set: OneNoteApi 1.1]
         */
        left: number;
        /**
         *
         * Gets or sets the top (Y-axis) position of the PageContent object.
         *
         * [Api set: OneNoteApi 1.1]
         */
        top: number;
        /**
         *
         * Gets the type of the PageContent object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly type: OneNote.PageContentType | "Outline" | "Image" | "Ink" | "Other";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: OneNote.PageContent): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.PageContentUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: OneNote.PageContent): void;
        /**
         *
         * Deletes the PageContent object.
         *
         * [Api set: OneNoteApi 1.1]
         */
        delete(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.PageContent` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.PageContent` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.PageContent` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.PageContentLoadOptions): OneNote.PageContent;
        load(option?: string | string[]): OneNote.PageContent;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.PageContent;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.PageContent;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.PageContent;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.PageContent object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.PageContentData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.PageContentData;
    }
    /**
     *
     * Represents the contents of a page, as a collection of PageContent objects.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class PageContentCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: OneNote.PageContent[];
        /**
         *
         * Returns the number of page contents in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Gets a PageContent object by ID or by its index in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index The ID of the PageContent object, or the index location of the PageContent object in the collection.
         */
        getItem(index: number | string): OneNote.PageContent;
        /**
         *
         * Gets a page content on its position in the collection.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): OneNote.PageContent;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.PageContentCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.PageContentCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.PageContentCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.PageContentCollectionLoadOptions & OneNote.Interfaces.CollectionLoadOptions): OneNote.PageContentCollection;
        load(option?: string | string[]): OneNote.PageContentCollection;
        load(option?: OfficeExtension.LoadOption): OneNote.PageContentCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.PageContentCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.PageContentCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `OneNote.PageContentCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.PageContentCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): OneNote.Interfaces.PageContentCollectionData;
    }
    /**
     *
     * Represents a container for Paragraph objects.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class Outline extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the PageContent object that contains the Outline. This object defines the position of the Outline on the page. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly pageContent: OneNote.PageContent;
        /**
         *
         * Gets the collection of Paragraph objects in the Outline. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly paragraphs: OneNote.ParagraphCollection;
        /**
         *
         * Gets the ID of the Outline object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         *
         * Adds the specified HTML to the bottom of the Outline.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param html The HTML string to append. See {@link https://docs.microsoft.com/office/dev/add-ins/onenote/onenote-add-ins-page-content#supported-html | Supported HTML} for the OneNote add-ins JavaScript API.
         */
        appendHtml(html: string): void;
        /**
         *
         * Adds the specified image to the bottom of the Outline.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param base64EncodedImage HTML string to append.
         * @param width Optional. Width in the unit of Points. The default value is null and image width will be respected.
         * @param height Optional. Height in the unit of Points. The default value is null and image height will be respected.
         */
        appendImage(base64EncodedImage: string, width: number, height: number): OneNote.Image;
        /**
         *
         * Adds the specified text to the bottom of the Outline.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param paragraphText HTML string to append.
         */
        appendRichText(paragraphText: string): OneNote.RichText;
        /**
         *
         * Adds a table with the specified number of rows and columns to the bottom of the outline.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param rowCount Required. The number of rows in the table.
         * @param columnCount Required. The number of columns in the table.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        appendTable(rowCount: number, columnCount: number, values?: string[][]): OneNote.Table;
        /**
         *
         * Check if the outline is title outline.
         *
         * [Api set: OneNoteApi 1.1]
         */
        isTitle(): OfficeExtension.ClientResult<boolean>;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.Outline` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.Outline` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.Outline` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.OutlineLoadOptions): OneNote.Outline;
        load(option?: string | string[]): OneNote.Outline;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.Outline;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.Outline;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.Outline;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.Outline object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.OutlineData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.OutlineData;
    }
    /**
     *
     * A container for the visible content on a page. A Paragraph can contain any one ParagraphType type of content.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class Paragraph extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the Image object in the Paragraph. Throws an exception if ParagraphType is not Image. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly image: OneNote.Image;
        /**
         *
         * Gets the Ink collection in the Paragraph. Throws an exception if ParagraphType is not Ink. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly inkWords: OneNote.InkWordCollection;
        /**
         *
         * Gets the Outline object that contains the Paragraph. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly outline: OneNote.Outline;
        /**
         *
         * The collection of paragraphs under this paragraph. Read only
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly paragraphs: OneNote.ParagraphCollection;
        /**
         *
         * Gets the parent paragraph object. Throws if a parent paragraph does not exist. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly parentParagraph: OneNote.Paragraph;
        /**
         *
         * Gets the parent paragraph object. Returns null if a parent paragraph does not exist. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly parentParagraphOrNull: OneNote.Paragraph;
        /**
         *
         * Gets the TableCell object that contains the Paragraph if one exists. If parent is not a TableCell, throws ItemNotFound. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly parentTableCell: OneNote.TableCell;
        /**
         *
         * Gets the TableCell object that contains the Paragraph if one exists. If parent is not a TableCell, returns null. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly parentTableCellOrNull: OneNote.TableCell;
        /**
         *
         * Gets the RichText object in the Paragraph. Throws an exception if ParagraphType is not RichText. Read-only
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly richText: OneNote.RichText;
        /**
         *
         * Gets the Table object in the Paragraph. Throws an exception if ParagraphType is not Table. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly table: OneNote.Table;
        /**
         *
         * Gets the ID of the Paragraph object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         *
         * Gets the type of the Paragraph object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly type: OneNote.ParagraphType | "RichText" | "Image" | "Table" | "Ink" | "Other";
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: OneNote.Paragraph): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ParagraphUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: OneNote.Paragraph): void;
        /**
         *
         * Add NoteTag to the paragraph.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param type The type of the NoteTag.
         * @param status The status of the NoteTag.
         */
        addNoteTag(type: OneNote.NoteTagType, status: OneNote.NoteTagStatus): OneNote.NoteTag;
        /**
         *
         * Add NoteTag to the paragraph.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param type The type of the NoteTag.
         * @param status The status of the NoteTag.
         */
        addNoteTag(type: "Unknown" | "ToDo" | "Important" | "Question" | "Contact" | "Address" | "PhoneNumber" | "Website" | "Idea" | "Critical" | "ToDoPriority1" | "ToDoPriority2", status: "Unknown" | "Normal" | "Completed" | "Disabled" | "OutlookTask" | "TaskNotSyncedYet" | "TaskRemoved"): OneNote.NoteTag;
        /**
         *
         * Deletes the paragraph
         *
         * [Api set: OneNoteApi 1.1]
         */
        delete(): void;
        /**
         *
         * Get list information of paragraph
         *
         * [Api set: OneNoteApi 1.1]
         */
        getParagraphInfo(): OfficeExtension.ClientResult<OneNote.ParagraphInfo>;
        /**
         *
         * Inserts the specified HTML content
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param insertLocation The location of new contents relative to the current Paragraph.
         * @param html An HTML string that describes the visual presentation of the content. See {@link https://docs.microsoft.com/office/dev/add-ins/onenote/onenote-add-ins-page-content#supported-html | Supported HTML} for the OneNote add-ins JavaScript API.
         */
        insertHtmlAsSibling(insertLocation: OneNote.InsertLocation, html: string): void;
        /**
         *
         * Inserts the specified HTML content
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param insertLocation The location of new contents relative to the current Paragraph.
         * @param html An HTML string that describes the visual presentation of the content. See {@link https://docs.microsoft.com/office/dev/add-ins/onenote/onenote-add-ins-page-content#supported-html | Supported HTML} for the OneNote add-ins JavaScript API.
         */
        insertHtmlAsSibling(insertLocation: "Before" | "After", html: string): void;
        /**
         *
         * Inserts the image at the specified insert location..
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param insertLocation The location of the table relative to the current Paragraph.
         * @param base64EncodedImage HTML string to append.
         * @param width Optional. Width in the unit of Points. The default value is null and image width will be respected.
         * @param height Optional. Height in the unit of Points. The default value is null and image height will be respected.
         */
        insertImageAsSibling(insertLocation: OneNote.InsertLocation, base64EncodedImage: string, width: number, height: number): OneNote.Image;
        /**
         *
         * Inserts the image at the specified insert location..
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param insertLocation The location of the table relative to the current Paragraph.
         * @param base64EncodedImage HTML string to append.
         * @param width Optional. Width in the unit of Points. The default value is null and image width will be respected.
         * @param height Optional. Height in the unit of Points. The default value is null and image height will be respected.
         */
        insertImageAsSibling(insertLocation: "Before" | "After", base64EncodedImage: string, width: number, height: number): OneNote.Image;
        /**
         *
         * Inserts the paragraph text at the specifiec insert location.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param insertLocation The location of the table relative to the current Paragraph.
         * @param paragraphText HTML string to append.
         */
        insertRichTextAsSibling(insertLocation: OneNote.InsertLocation, paragraphText: string): OneNote.RichText;
        /**
         *
         * Inserts the paragraph text at the specifiec insert location.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param insertLocation The location of the table relative to the current Paragraph.
         * @param paragraphText HTML string to append.
         */
        insertRichTextAsSibling(insertLocation: "Before" | "After", paragraphText: string): OneNote.RichText;
        /**
         *
         * Adds a table with the specified number of rows and columns before or after the current paragraph.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param insertLocation The location of the table relative to the current Paragraph.
         * @param rowCount The number of rows in the table.
         * @param columnCount The number of columns in the table.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertTableAsSibling(insertLocation: OneNote.InsertLocation, rowCount: number, columnCount: number, values?: string[][]): OneNote.Table;
        /**
         *
         * Adds a table with the specified number of rows and columns before or after the current paragraph.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param insertLocation The location of the table relative to the current Paragraph.
         * @param rowCount The number of rows in the table.
         * @param columnCount The number of columns in the table.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        insertTableAsSibling(insertLocation: "Before" | "After", rowCount: number, columnCount: number, values?: string[][]): OneNote.Table;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.Paragraph` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.Paragraph` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.Paragraph` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.ParagraphLoadOptions): OneNote.Paragraph;
        load(option?: string | string[]): OneNote.Paragraph;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.Paragraph;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.Paragraph;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.Paragraph;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.Paragraph object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.ParagraphData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.ParagraphData;
    }
    /**
     *
     * Represents a collection of Paragraph objects.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class ParagraphCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: OneNote.Paragraph[];
        /**
         *
         * Returns the number of paragraphs in the page. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Gets a Paragraph object by ID or by its index in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index The ID of the Paragraph object, or the index location of the Paragraph object in the collection.
         */
        getItem(index: number | string): OneNote.Paragraph;
        /**
         *
         * Gets a paragraph on its position in the collection.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): OneNote.Paragraph;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.ParagraphCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.ParagraphCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.ParagraphCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.ParagraphCollectionLoadOptions & OneNote.Interfaces.CollectionLoadOptions): OneNote.ParagraphCollection;
        load(option?: string | string[]): OneNote.ParagraphCollection;
        load(option?: OfficeExtension.LoadOption): OneNote.ParagraphCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.ParagraphCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.ParagraphCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `OneNote.ParagraphCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.ParagraphCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): OneNote.Interfaces.ParagraphCollectionData;
    }
    /**
     *
     * A container for the NoteTag in a paragraph.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class NoteTag extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the Id of the NoteTag object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         *
         * Gets the status of the NoteTag object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly status: OneNote.NoteTagStatus | "Unknown" | "Normal" | "Completed" | "Disabled" | "OutlookTask" | "TaskNotSyncedYet" | "TaskRemoved";
        /**
         *
         * Gets the type of the NoteTag object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly type: OneNote.NoteTagType | "Unknown" | "ToDo" | "Important" | "Question" | "Contact" | "Address" | "PhoneNumber" | "Website" | "Idea" | "Critical" | "ToDoPriority1" | "ToDoPriority2";
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.NoteTag` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.NoteTag` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.NoteTag` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.NoteTagLoadOptions): OneNote.NoteTag;
        load(option?: string | string[]): OneNote.NoteTag;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.NoteTag;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.NoteTag;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.NoteTag;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.NoteTag object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.NoteTagData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.NoteTagData;
    }
    /**
     *
     * Represents a RichText object in a Paragraph.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class RichText extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the Paragraph object that contains the RichText object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly paragraph: OneNote.Paragraph;
        /**
         *
         * Gets the ID of the RichText object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         *
         * The language id of the text. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly languageId: string;
        /**
         *
         * Gets the text content of the RichText object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly text: string;
        /**
         *
         * Get the HTML of the rich text
         *
         * [Api set: OneNoteApi 1.1]
         * @returns The html of the rich text
         */
        getHtml(): OfficeExtension.ClientResult<string>;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.RichText` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.RichText` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.RichText` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.RichTextLoadOptions): OneNote.RichText;
        load(option?: string | string[]): OneNote.RichText;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.RichText;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.RichText;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.RichText;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.RichText object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.RichTextData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.RichTextData;
    }
    /**
     *
     * Represents an Image. An Image can be a direct child of a PageContent object or a Paragraph object.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class Image extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the PageContent object that contains the Image. Throws if the Image is not a direct child of a PageContent. This object defines the position of the Image on the page. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly pageContent: OneNote.PageContent;
        /**
         *
         * Gets the Paragraph object that contains the Image. Throws if the Image is not a direct child of a Paragraph. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly paragraph: OneNote.Paragraph;
        /**
         *
         * Gets or sets the description of the Image.
         *
         * [Api set: OneNoteApi 1.1]
         */
        description: string;
        /**
         *
         * Gets or sets the height of the Image layout.
         *
         * [Api set: OneNoteApi 1.1]
         */
        height: number;
        /**
         *
         * Gets or sets the hyperlink of the Image.
         *
         * [Api set: OneNoteApi 1.1]
         */
        hyperlink: string;
        /**
         *
         * Gets the ID of the Image object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         *
         * Gets the data obtained by OCR (Optical Character Recognition) of this Image, such as OCR text and language.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly ocrData: OneNote.ImageOcrData;
        /**
         *
         * Gets or sets the width of the Image layout.
         *
         * [Api set: OneNoteApi 1.1]
         */
        width: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: OneNote.Image): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ImageUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: OneNote.Image): void;
        /**
         *
         * Gets the base64-encoded binary representation of the Image.
            Example: data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAADIA...
         *
         * [Api set: OneNoteApi 1.1]
         */
        getBase64Image(): OfficeExtension.ClientResult<string>;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.Image` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.Image` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.Image` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.ImageLoadOptions): OneNote.Image;
        load(option?: string | string[]): OneNote.Image;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.Image;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.Image;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.Image;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.Image object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.ImageData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.ImageData;
    }
    /**
     *
     * Represents a table in a OneNote page.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class Table extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the Paragraph object that contains the Table object. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly paragraph: OneNote.Paragraph;
        /**
         *
         * Gets all of the table rows. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly rows: OneNote.TableRowCollection;
        /**
         *
         * Gets or sets whether the borders are visible or not. True if they are visible, false if they are hidden.
         *
         * [Api set: OneNoteApi 1.1]
         */
        borderVisible: boolean;
        /**
         *
         * Gets the number of columns in the table.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly columnCount: number;
        /**
         *
         * Gets the ID of the table. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         *
         * Gets the number of rows in the table.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly rowCount: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: OneNote.Table): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.TableUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: OneNote.Table): void;
        /**
         *
         * Adds a column to the end of the table. Values, if specified, are set in the new column. Otherwise the column is empty.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param values Optional. Strings to insert in the new column, specified as an array. Must not have more values than rows in the table.
         */
        appendColumn(values?: string[]): void;
        /**
         *
         * Adds a row to the end of the table. Values, if specified, are set in the new row. Otherwise the row is empty.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param values Optional. Strings to insert in the new row, specified as an array. Must not have more values than columns in the table.
         */
        appendRow(values?: string[]): OneNote.TableRow;
        /**
         *
         * Clears the contents of the table.
         *
         * [Api set: OneNoteApi 1.1]
         */
        clear(): void;
        /**
         *
         * Gets the table cell at a specified row and column.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param rowIndex The index of the row.
         * @param cellIndex The index of the cell in the row.
         */
        getCell(rowIndex: number, cellIndex: number): OneNote.TableCell;
        /**
         *
         * Inserts a column at the given index in the table. Values, if specified, are set in the new column. Otherwise the column is empty.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index Index where the column will be inserted in the table.
         * @param values Optional. Strings to insert in the new column, specified as an array. Must not have more values than rows in the table.
         */
        insertColumn(index: number, values?: string[]): void;
        /**
         *
         * Inserts a row at the given index in the table. Values, if specified, are set in the new row. Otherwise the row is empty.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index Index where the row will be inserted in the table.
         * @param values Optional. Strings to insert in the new row, specified as an array. Must not have more values than columns in the table.
         */
        insertRow(index: number, values?: string[]): OneNote.TableRow;
        /**
         *
         * Sets the shading color of all cells in the table.
            The color code to set the cells to.
         *
         * [Api set: OneNoteApi 1.1]
         */
        setShadingColor(colorCode: string): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.Table` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.Table` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.Table` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.TableLoadOptions): OneNote.Table;
        load(option?: string | string[]): OneNote.Table;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.Table;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.Table;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.Table;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.Table object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.TableData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.TableData;
    }
    /**
     *
     * Represents a row in a table.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class TableRow extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the cells in the row. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly cells: OneNote.TableCellCollection;
        /**
         *
         * Gets the parent table. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly parentTable: OneNote.Table;
        /**
         *
         * Gets the number of cells in the row. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly cellCount: number;
        /**
         *
         * Gets the ID of the row. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         *
         * Gets the index of the row in its parent table. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly rowIndex: number;
        /**
         *
         * Clears the contents of the row.
         *
         * [Api set: OneNoteApi 1.1]
         */
        clear(): void;
        /**
         *
         * Inserts a row before or after the current row.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param insertLocation Where the new rows should be inserted relative to the current row.
         * @param values Strings to insert in the new row, specified as an array. Must not have more cells than in the current row. Optional.
         */
        insertRowAsSibling(insertLocation: OneNote.InsertLocation, values?: string[]): OneNote.TableRow;
        /**
         *
         * Inserts a row before or after the current row.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param insertLocation Where the new rows should be inserted relative to the current row.
         * @param values Strings to insert in the new row, specified as an array. Must not have more cells than in the current row. Optional.
         */
        insertRowAsSibling(insertLocation: "Before" | "After", values?: string[]): OneNote.TableRow;
        /**
         *
         * Sets the shading color of all cells in the row.
            The color code to set the cells to.
         *
         * [Api set: OneNoteApi 1.1]
         */
        setShadingColor(colorCode: string): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.TableRow` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.TableRow` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.TableRow` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.TableRowLoadOptions): OneNote.TableRow;
        load(option?: string | string[]): OneNote.TableRow;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.TableRow;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.TableRow;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.TableRow;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.TableRow object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.TableRowData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.TableRowData;
    }
    /**
     *
     * Contains a collection of TableRow objects.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class TableRowCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: OneNote.TableRow[];
        /**
         *
         * Returns the number of table rows in this collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Gets a table row object by ID or by its index in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index A number that identifies the index location of a table row object.
         */
        getItem(index: number | string): OneNote.TableRow;
        /**
         *
         * Gets a table row at its position in the collection.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): OneNote.TableRow;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.TableRowCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.TableRowCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.TableRowCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.TableRowCollectionLoadOptions & OneNote.Interfaces.CollectionLoadOptions): OneNote.TableRowCollection;
        load(option?: string | string[]): OneNote.TableRowCollection;
        load(option?: OfficeExtension.LoadOption): OneNote.TableRowCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.TableRowCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.TableRowCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `OneNote.TableRowCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.TableRowCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): OneNote.Interfaces.TableRowCollectionData;
    }
    /**
     *
     * Represents a cell in a OneNote table.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class TableCell extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the collection of Paragraph objects in the TableCell. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly paragraphs: OneNote.ParagraphCollection;
        /**
         *
         * Gets the parent row of the cell. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly parentRow: OneNote.TableRow;
        /**
         *
         * Gets the index of the cell in its row. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly cellIndex: number;
        /**
         *
         * Gets the ID of the cell. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly id: string;
        /**
         *
         * Gets the index of the cell's row in the table. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly rowIndex: number;
        /**
         *
         * Gets and sets the shading color of the cell
         *
         * [Api set: OneNoteApi 1.1]
         */
        shadingColor: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: OneNote.TableCell): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.TableCellUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: OneNote.TableCell): void;
        /**
         *
         * Adds the specified HTML to the bottom of the TableCell.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param html The HTML string to append. See {@link https://docs.microsoft.com/office/dev/add-ins/onenote/onenote-add-ins-page-content#supported-html | Supported HTML} for the OneNote add-ins JavaScript API.
         */
        appendHtml(html: string): void;
        /**
         *
         * Adds the specified image to table cell.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param base64EncodedImage HTML string to append.
         * @param width Optional. Width in the unit of Points. The default value is null and image width will be respected.
         * @param height Optional. Height in the unit of Points. The default value is null and image height will be respected.
         */
        appendImage(base64EncodedImage: string, width: number, height: number): OneNote.Image;
        /**
         *
         * Adds the specified text to table cell.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param paragraphText HTML string to append.
         */
        appendRichText(paragraphText: string): OneNote.RichText;
        /**
         *
         * Adds a table with the specified number of rows and columns to table cell.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param rowCount Required. The number of rows in the table.
         * @param columnCount Required. The number of columns in the table.
         * @param values Optional 2D array. Cells are filled if the corresponding strings are specified in the array.
         */
        appendTable(rowCount: number, columnCount: number, values?: string[][]): OneNote.Table;
        /**
         *
         * Clears the contents of the cell.
         *
         * [Api set: OneNoteApi 1.1]
         */
        clear(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.TableCell` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.TableCell` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.TableCell` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.TableCellLoadOptions): OneNote.TableCell;
        load(option?: string | string[]): OneNote.TableCell;
        load(option?: {
            select?: string;
            expand?: string;
        }): OneNote.TableCell;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.TableCell;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.TableCell;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original OneNote.TableCell object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.TableCellData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): OneNote.Interfaces.TableCellData;
    }
    /**
     *
     * Contains a collection of TableCell objects.
     *
     * [Api set: OneNoteApi 1.1]
     */
    class TableCellCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: OneNote.TableCell[];
        /**
         *
         * Returns the number of tablecells in this collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         */
        readonly count: number;
        /**
         *
         * Gets a table cell object by ID or by its index in the collection. Read-only.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index A number that identifies the index location of a table cell object.
         */
        getItem(index: number | string): OneNote.TableCell;
        /**
         *
         * Gets a tablecell at its position in the collection.
         *
         * [Api set: OneNoteApi 1.1]
         *
         * @param index Index value of the object to be retrieved. Zero-indexed.
         */
        getItemAt(index: number): OneNote.TableCell;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): OneNote.TableCellCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): OneNote.TableCellCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): OneNote.TableCellCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: OneNote.Interfaces.TableCellCollectionLoadOptions & OneNote.Interfaces.CollectionLoadOptions): OneNote.TableCellCollection;
        load(option?: string | string[]): OneNote.TableCellCollection;
        load(option?: OfficeExtension.LoadOption): OneNote.TableCellCollection;
        /**
         * Track the object for automatic adjustment based on surrounding changes in the document. This call is a shorthand for context.trackedObjects.add(thisObject). If you are using this object across ".sync" calls and outside the sequential execution of a ".run" batch, and get an "InvalidObjectPath" error when setting a property or invoking a method on the object, you needed to have added the object to the tracked object collection when the object was first created.
         */
        track(): OneNote.TableCellCollection;
        /**
         * Release the memory associated with this object, if it has previously been tracked. This call is shorthand for context.trackedObjects.remove(thisObject). Having many tracked objects slows down the host application, so please remember to free any objects you add, once you're done using them. You will need to call "context.sync()" before the memory release takes effect.
         */
        untrack(): OneNote.TableCellCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `OneNote.TableCellCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `OneNote.Interfaces.TableCellCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): OneNote.Interfaces.TableCellCollectionData;
    }
    /**
     *
     * Represents data obtained by OCR (optical character recognition) of an image.
     *
     * [Api set: OneNoteApi 1.1]
     */
    interface ImageOcrData {
        /**
         *
         * Represents the OCR language, with values such as EN-US
         *
         * [Api set: OneNoteApi 1.1]
         */
        ocrLanguageId: string;
        /**
         *
         * Represents the text obtained by OCR of the image
         *
         * [Api set: OneNoteApi 1.1]
         */
        ocrText: string;
    }
    /**
     *
     * Weak reference to an ink stroke object and its content parent.
     *
     * [Api set: OneNoteApi 1.1]
     */
    interface InkStrokePointer {
        /**
         *
         * Represents the id of the page content object corresponding to this stroke
         *
         * [Api set: OneNoteApi 1.1]
         */
        contentId: string;
        /**
         *
         * Represents the id of the ink stroke
         *
         * [Api set: OneNoteApi 1.1]
         */
        inkStrokeId: string;
    }
    /**
     *
     * List information for paragraph.
     *
     * [Api set: OneNoteApi 1.1]
     */
    interface ParagraphInfo {
        /**
         *
         * //
            Bullet list type of paragraph
         *
         * [Api set: OneNoteApi 1.1]
         */
        bulletType: string;
        /**
         *
         * //
            Index of paragraph in list
         *
         * [Api set: OneNoteApi 1.1]
         */
        index: number;
        /**
         *
         * //
            Type of list in paragraph
         *
         * [Api set: OneNoteApi 1.1]
         */
        listType: OneNote.ListType | "None" | "Number" | "Bullet";
        /**
         *
         * //
            number list type of paragraph
         *
         * [Api set: OneNoteApi 1.1]
         */
        numberType: OneNote.NumberType | "None" | "Arabic" | "UCRoman" | "LCRoman" | "UCLetter" | "LCLetter" | "Ordinal" | "Cardtext" | "Ordtext" | "Hex" | "ChiManSty" | "DbNum1" | "DbNum2" | "Aiueo" | "Iroha" | "DbChar" | "SbChar" | "DbNum3" | "DbNum4" | "Circlenum" | "DArabic" | "DAiueo" | "DIroha" | "ArabicLZ" | "Bullet" | "Ganada" | "Chosung" | "GB1" | "GB2" | "GB3" | "GB4" | "Zodiac1" | "Zodiac2" | "Zodiac3" | "TpeDbNum1" | "TpeDbNum2" | "TpeDbNum3" | "TpeDbNum4" | "ChnDbNum1" | "ChnDbNum2" | "ChnDbNum3" | "ChnDbNum4" | "KorDbNum1" | "KorDbNum2" | "KorDbNum3" | "KorDbNum4" | "Hebrew1" | "Arabic1" | "Hebrew2" | "Arabic2" | "Hindi1" | "Hindi2" | "Hindi3" | "Thai1" | "Thai2" | "NumInDash" | "LCRus" | "UCRus" | "LCGreek" | "UCGreek" | "Lim" | "Custom";
    }
    /**
     * [Api set: OneNoteApi 1.1]
     */
    enum InsertLocation {
        before = "Before",
        after = "After",
    }
    /**
     * [Api set: OneNoteApi 1.1]
     */
    enum PageContentType {
        outline = "Outline",
        image = "Image",
        ink = "Ink",
        other = "Other",
    }
    /**
     * [Api set: OneNoteApi 1.1]
     */
    enum ParagraphType {
        richText = "RichText",
        image = "Image",
        table = "Table",
        ink = "Ink",
        other = "Other",
    }
    /**
     * [Api set: OneNoteApi 1.1]
     */
    enum NoteTagType {
        unknown = "Unknown",
        toDo = "ToDo",
        important = "Important",
        question = "Question",
        contact = "Contact",
        address = "Address",
        phoneNumber = "PhoneNumber",
        website = "Website",
        idea = "Idea",
        critical = "Critical",
        toDoPriority1 = "ToDoPriority1",
        toDoPriority2 = "ToDoPriority2",
    }
    /**
     * [Api set: OneNoteApi 1.1]
     */
    enum NoteTagStatus {
        unknown = "Unknown",
        normal = "Normal",
        completed = "Completed",
        disabled = "Disabled",
        outlookTask = "OutlookTask",
        taskNotSyncedYet = "TaskNotSyncedYet",
        taskRemoved = "TaskRemoved",
    }
    /**
     * [Api set: OneNoteApi 1.1]
     */
    enum ListType {
        none = "None",
        number = "Number",
        bullet = "Bullet",
    }
    /**
     * [Api set: OneNoteApi 1.1]
     */
    enum NumberType {
        none = "None",
        arabic = "Arabic",
        ucroman = "UCRoman",
        lcroman = "LCRoman",
        ucletter = "UCLetter",
        lcletter = "LCLetter",
        ordinal = "Ordinal",
        cardtext = "Cardtext",
        ordtext = "Ordtext",
        hex = "Hex",
        chiManSty = "ChiManSty",
        dbNum1 = "DbNum1",
        dbNum2 = "DbNum2",
        aiueo = "Aiueo",
        iroha = "Iroha",
        dbChar = "DbChar",
        sbChar = "SbChar",
        dbNum3 = "DbNum3",
        dbNum4 = "DbNum4",
        circlenum = "Circlenum",
        darabic = "DArabic",
        daiueo = "DAiueo",
        diroha = "DIroha",
        arabicLZ = "ArabicLZ",
        bullet = "Bullet",
        ganada = "Ganada",
        chosung = "Chosung",
        gb1 = "GB1",
        gb2 = "GB2",
        gb3 = "GB3",
        gb4 = "GB4",
        zodiac1 = "Zodiac1",
        zodiac2 = "Zodiac2",
        zodiac3 = "Zodiac3",
        tpeDbNum1 = "TpeDbNum1",
        tpeDbNum2 = "TpeDbNum2",
        tpeDbNum3 = "TpeDbNum3",
        tpeDbNum4 = "TpeDbNum4",
        chnDbNum1 = "ChnDbNum1",
        chnDbNum2 = "ChnDbNum2",
        chnDbNum3 = "ChnDbNum3",
        chnDbNum4 = "ChnDbNum4",
        korDbNum1 = "KorDbNum1",
        korDbNum2 = "KorDbNum2",
        korDbNum3 = "KorDbNum3",
        korDbNum4 = "KorDbNum4",
        hebrew1 = "Hebrew1",
        arabic1 = "Arabic1",
        hebrew2 = "Hebrew2",
        arabic2 = "Arabic2",
        hindi1 = "Hindi1",
        hindi2 = "Hindi2",
        hindi3 = "Hindi3",
        thai1 = "Thai1",
        thai2 = "Thai2",
        numInDash = "NumInDash",
        lcrus = "LCRus",
        ucrus = "UCRus",
        lcgreek = "LCGreek",
        ucgreek = "UCGreek",
        lim = "Lim",
        custom = "Custom",
    }
    /**
     * [Api set: OneNoteApi 1.3]
     */
    enum EventType {
        notebookChanged = "NotebookChanged",
        stickyNotesContextMenuExecuted = "StickyNotesContextMenuExecuted",
        sectionSelectionChanged = "SectionSelectionChanged",
        pageSelectionChanged = "PageSelectionChanged",
    }
    enum ErrorCodes {
        generalException = "GeneralException",
        operationAborted = "OperationAborted",
    }
    module Interfaces {
        /**
        * Provides ways to load properties of only a subset of members of a collection.
        */
        interface CollectionLoadOptions {
            /**
            * Specify the number of items in the queried collection to be included in the result.
            */
            $top?: number;
            /**
            * Specify the number of items in the collection that are to be skipped and not included in the result. If top is specified, the selection of result will start after skipping the specified number of items.
            */
            $skip?: number;
        }
        /** An interface for updating data on the InkAnalysis object, for use in "inkAnalysis.set({ ... })". */
        interface InkAnalysisUpdateData {
            /**
            *
            * Gets the parent page object.
            *
            * [Api set: OneNoteApi 1.1]
            */
            page?: OneNote.Interfaces.PageUpdateData;
        }
        /** An interface for updating data on the InkAnalysisParagraph object, for use in "inkAnalysisParagraph.set({ ... })". */
        interface InkAnalysisParagraphUpdateData {
            /**
            *
            * Reference to the parent InkAnalysisPage.
            *
            * [Api set: OneNoteApi 1.1]
            */
            inkAnalysis?: OneNote.Interfaces.InkAnalysisUpdateData;
        }
        /** An interface for updating data on the InkAnalysisParagraphCollection object, for use in "inkAnalysisParagraphCollection.set({ ... })". */
        interface InkAnalysisParagraphCollectionUpdateData {
            items?: OneNote.Interfaces.InkAnalysisParagraphData[];
        }
        /** An interface for updating data on the InkAnalysisLine object, for use in "inkAnalysisLine.set({ ... })". */
        interface InkAnalysisLineUpdateData {
            /**
            *
            * Reference to the parent InkAnalysisParagraph.
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraph?: OneNote.Interfaces.InkAnalysisParagraphUpdateData;
        }
        /** An interface for updating data on the InkAnalysisLineCollection object, for use in "inkAnalysisLineCollection.set({ ... })". */
        interface InkAnalysisLineCollectionUpdateData {
            items?: OneNote.Interfaces.InkAnalysisLineData[];
        }
        /** An interface for updating data on the InkAnalysisWord object, for use in "inkAnalysisWord.set({ ... })". */
        interface InkAnalysisWordUpdateData {
            /**
            *
            * Reference to the parent InkAnalysisLine.
            *
            * [Api set: OneNoteApi 1.1]
            */
            line?: OneNote.Interfaces.InkAnalysisLineUpdateData;
        }
        /** An interface for updating data on the InkAnalysisWordCollection object, for use in "inkAnalysisWordCollection.set({ ... })". */
        interface InkAnalysisWordCollectionUpdateData {
            items?: OneNote.Interfaces.InkAnalysisWordData[];
        }
        /** An interface for updating data on the InkStrokeCollection object, for use in "inkStrokeCollection.set({ ... })". */
        interface InkStrokeCollectionUpdateData {
            items?: OneNote.Interfaces.InkStrokeData[];
        }
        /** An interface for updating data on the InkWordCollection object, for use in "inkWordCollection.set({ ... })". */
        interface InkWordCollectionUpdateData {
            items?: OneNote.Interfaces.InkWordData[];
        }
        /** An interface for updating data on the NotebookCollection object, for use in "notebookCollection.set({ ... })". */
        interface NotebookCollectionUpdateData {
            items?: OneNote.Interfaces.NotebookData[];
        }
        /** An interface for updating data on the SectionGroupCollection object, for use in "sectionGroupCollection.set({ ... })". */
        interface SectionGroupCollectionUpdateData {
            items?: OneNote.Interfaces.SectionGroupData[];
        }
        /** An interface for updating data on the SectionCollection object, for use in "sectionCollection.set({ ... })". */
        interface SectionCollectionUpdateData {
            items?: OneNote.Interfaces.SectionData[];
        }
        /** An interface for updating data on the Page object, for use in "page.set({ ... })". */
        interface PageUpdateData {
            /**
            *
            * Text interpretation for the ink on the page. Returns null if there is no ink analysis information. Read only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            inkAnalysisOrNull?: OneNote.Interfaces.InkAnalysisUpdateData;
            /**
             *
             * Gets or sets the indentation level of the page.
             *
             * [Api set: OneNoteApi 1.1]
             */
            pageLevel?: number;
            /**
             *
             * Gets or sets the title of the page.
             *
             * [Api set: OneNoteApi 1.1]
             */
            title?: string;
        }
        /** An interface for updating data on the PageCollection object, for use in "pageCollection.set({ ... })". */
        interface PageCollectionUpdateData {
            items?: OneNote.Interfaces.PageData[];
        }
        /** An interface for updating data on the PageContent object, for use in "pageContent.set({ ... })". */
        interface PageContentUpdateData {
            /**
            *
            * Gets the Image in the PageContent object. Throws an exception if PageContentType is not Image.
            *
            * [Api set: OneNoteApi 1.1]
            */
            image?: OneNote.Interfaces.ImageUpdateData;
            /**
             *
             * Gets or sets the left (X-axis) position of the PageContent object.
             *
             * [Api set: OneNoteApi 1.1]
             */
            left?: number;
            /**
             *
             * Gets or sets the top (Y-axis) position of the PageContent object.
             *
             * [Api set: OneNoteApi 1.1]
             */
            top?: number;
        }
        /** An interface for updating data on the PageContentCollection object, for use in "pageContentCollection.set({ ... })". */
        interface PageContentCollectionUpdateData {
            items?: OneNote.Interfaces.PageContentData[];
        }
        /** An interface for updating data on the Paragraph object, for use in "paragraph.set({ ... })". */
        interface ParagraphUpdateData {
            /**
            *
            * Gets the Image object in the Paragraph. Throws an exception if ParagraphType is not Image.
            *
            * [Api set: OneNoteApi 1.1]
            */
            image?: OneNote.Interfaces.ImageUpdateData;
            /**
            *
            * Gets the Table object in the Paragraph. Throws an exception if ParagraphType is not Table.
            *
            * [Api set: OneNoteApi 1.1]
            */
            table?: OneNote.Interfaces.TableUpdateData;
        }
        /** An interface for updating data on the ParagraphCollection object, for use in "paragraphCollection.set({ ... })". */
        interface ParagraphCollectionUpdateData {
            items?: OneNote.Interfaces.ParagraphData[];
        }
        /** An interface for updating data on the Image object, for use in "image.set({ ... })". */
        interface ImageUpdateData {
            /**
             *
             * Gets or sets the description of the Image.
             *
             * [Api set: OneNoteApi 1.1]
             */
            description?: string;
            /**
             *
             * Gets or sets the height of the Image layout.
             *
             * [Api set: OneNoteApi 1.1]
             */
            height?: number;
            /**
             *
             * Gets or sets the hyperlink of the Image.
             *
             * [Api set: OneNoteApi 1.1]
             */
            hyperlink?: string;
            /**
             *
             * Gets or sets the width of the Image layout.
             *
             * [Api set: OneNoteApi 1.1]
             */
            width?: number;
        }
        /** An interface for updating data on the Table object, for use in "table.set({ ... })". */
        interface TableUpdateData {
            /**
             *
             * Gets or sets whether the borders are visible or not. True if they are visible, false if they are hidden.
             *
             * [Api set: OneNoteApi 1.1]
             */
            borderVisible?: boolean;
        }
        /** An interface for updating data on the TableRowCollection object, for use in "tableRowCollection.set({ ... })". */
        interface TableRowCollectionUpdateData {
            items?: OneNote.Interfaces.TableRowData[];
        }
        /** An interface for updating data on the TableCell object, for use in "tableCell.set({ ... })". */
        interface TableCellUpdateData {
            /**
             *
             * Gets and sets the shading color of the cell
             *
             * [Api set: OneNoteApi 1.1]
             */
            shadingColor?: string;
        }
        /** An interface for updating data on the TableCellCollection object, for use in "tableCellCollection.set({ ... })". */
        interface TableCellCollectionUpdateData {
            items?: OneNote.Interfaces.TableCellData[];
        }
        /** An interface describing the data returned by calling "application.toJSON()". */
        interface ApplicationData {
            /**
            *
            * Gets the collection of notebooks that are open in the OneNote application instance. In OneNote Online, only one notebook at a time is open in the application instance. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            notebooks?: OneNote.Interfaces.NotebookData[];
        }
        /** An interface describing the data returned by calling "inkAnalysis.toJSON()". */
        interface InkAnalysisData {
            /**
            *
            * Gets the parent page object. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            page?: OneNote.Interfaces.PageData;
            /**
             *
             * Gets the ID of the InkAnalysis object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
        }
        /** An interface describing the data returned by calling "inkAnalysisParagraph.toJSON()". */
        interface InkAnalysisParagraphData {
            /**
            *
            * Reference to the parent InkAnalysisPage. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            inkAnalysis?: OneNote.Interfaces.InkAnalysisData;
            /**
            *
            * Gets the ink analysis lines in this ink analysis paragraph. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            lines?: OneNote.Interfaces.InkAnalysisLineData[];
            /**
             *
             * Gets the ID of the InkAnalysisParagraph object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
        }
        /** An interface describing the data returned by calling "inkAnalysisParagraphCollection.toJSON()". */
        interface InkAnalysisParagraphCollectionData {
            items?: OneNote.Interfaces.InkAnalysisParagraphData[];
        }
        /** An interface describing the data returned by calling "inkAnalysisLine.toJSON()". */
        interface InkAnalysisLineData {
            /**
            *
            * Reference to the parent InkAnalysisParagraph. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraph?: OneNote.Interfaces.InkAnalysisParagraphData;
            /**
            *
            * Gets the ink analysis words in this ink analysis line. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            words?: OneNote.Interfaces.InkAnalysisWordData[];
            /**
             *
             * Gets the ID of the InkAnalysisLine object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
        }
        /** An interface describing the data returned by calling "inkAnalysisLineCollection.toJSON()". */
        interface InkAnalysisLineCollectionData {
            items?: OneNote.Interfaces.InkAnalysisLineData[];
        }
        /** An interface describing the data returned by calling "inkAnalysisWord.toJSON()". */
        interface InkAnalysisWordData {
            /**
            *
            * Reference to the parent InkAnalysisLine. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            line?: OneNote.Interfaces.InkAnalysisLineData;
            /**
             *
             * Gets the ID of the InkAnalysisWord object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
            /**
             *
             * The id of the recognized language in this inkAnalysisWord. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            languageId?: string;
            /**
             *
             * Weak references to the ink strokes that were recognized as part of this ink analysis word. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            strokePointers?: OneNote.InkStrokePointer[];
            /**
             *
             * The words that were recognized in this ink word, in order of likelihood. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            wordAlternates?: string[];
        }
        /** An interface describing the data returned by calling "inkAnalysisWordCollection.toJSON()". */
        interface InkAnalysisWordCollectionData {
            items?: OneNote.Interfaces.InkAnalysisWordData[];
        }
        /** An interface describing the data returned by calling "floatingInk.toJSON()". */
        interface FloatingInkData {
            /**
            *
            * Gets the strokes of the FloatingInk object. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            inkStrokes?: OneNote.Interfaces.InkStrokeData[];
            /**
             *
             * Gets the ID of the FloatingInk object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
        }
        /** An interface describing the data returned by calling "inkStroke.toJSON()". */
        interface InkStrokeData {
            /**
            *
            * Gets the ID of the InkStroke object. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            floatingInk?: OneNote.Interfaces.FloatingInkData;
            /**
             *
             * Gets the ID of the InkStroke object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
        }
        /** An interface describing the data returned by calling "inkStrokeCollection.toJSON()". */
        interface InkStrokeCollectionData {
            items?: OneNote.Interfaces.InkStrokeData[];
        }
        /** An interface describing the data returned by calling "inkWord.toJSON()". */
        interface InkWordData {
            /**
             *
             * Gets the ID of the InkWord object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
            /**
             *
             * The id of the recognized language in this ink word. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            languageId?: string;
            /**
             *
             * The words that were recognized in this ink word, in order of likelihood. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            wordAlternates?: string[];
        }
        /** An interface describing the data returned by calling "inkWordCollection.toJSON()". */
        interface InkWordCollectionData {
            items?: OneNote.Interfaces.InkWordData[];
        }
        /** An interface describing the data returned by calling "notebook.toJSON()". */
        interface NotebookData {
            /**
            *
            * The section groups in the notebook. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            sectionGroups?: OneNote.Interfaces.SectionGroupData[];
            /**
            *
            * The the sections of the notebook. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            sections?: OneNote.Interfaces.SectionData[];
            /**
             *
             * The url of the site that this notebook is located. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            baseUrl?: string;
            /**
             *
             * The client url of the notebook. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            clientUrl?: string;
            /**
             *
             * Gets the ID of the notebook. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
            /**
             *
             * True if the Notebook is not created by the user (i.e. 'Misplaced Sections'). Read only
             *
             * [Api set: OneNoteApi 1.2]
             */
            isVirtual?: boolean;
            /**
             *
             * Gets the name of the notebook. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            name?: string;
        }
        /** An interface describing the data returned by calling "notebookCollection.toJSON()". */
        interface NotebookCollectionData {
            items?: OneNote.Interfaces.NotebookData[];
        }
        /** An interface describing the data returned by calling "sectionGroup.toJSON()". */
        interface SectionGroupData {
            /**
            *
            * The collection of section groups in the section group. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            sectionGroups?: OneNote.Interfaces.SectionGroupData[];
            /**
            *
            * The collection of sections in the section group. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            sections?: OneNote.Interfaces.SectionData[];
            /**
             *
             * The client url of the section group. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            clientUrl?: string;
            /**
             *
             * Gets the ID of the section group. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
            /**
             *
             * Gets the name of the section group. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            name?: string;
        }
        /** An interface describing the data returned by calling "sectionGroupCollection.toJSON()". */
        interface SectionGroupCollectionData {
            items?: OneNote.Interfaces.SectionGroupData[];
        }
        /** An interface describing the data returned by calling "section.toJSON()". */
        interface SectionData {
            /**
            *
            * The collection of pages in the section. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            pages?: OneNote.Interfaces.PageData[];
            /**
             *
             * The client url of the section. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            clientUrl?: string;
            /**
             *
             * Gets the ID of the section. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
            /**
             *
             * True if this section is encrypted with a password. Read only
             *
             * [Api set: OneNoteApi 1.2]
             */
            isEncrypted?: boolean;
            /**
             *
             * True if this section is locked. Read only
             *
             * [Api set: OneNoteApi 1.2]
             */
            isLocked?: boolean;
            /**
             *
             * Gets the name of the section. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            name?: string;
            /**
             *
             * The web url of the page. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            webUrl?: string;
        }
        /** An interface describing the data returned by calling "sectionCollection.toJSON()". */
        interface SectionCollectionData {
            items?: OneNote.Interfaces.SectionData[];
        }
        /** An interface describing the data returned by calling "page.toJSON()". */
        interface PageData {
            /**
            *
            * The collection of PageContent objects on the page. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            contents?: OneNote.Interfaces.PageContentData[];
            /**
            *
            * Text interpretation for the ink on the page. Returns null if there is no ink analysis information. Read only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            inkAnalysisOrNull?: OneNote.Interfaces.InkAnalysisData;
            /**
             *
             * Gets the ClassNotebookPageSource to the page.
             *
             * [Api set: OneNoteApi 1.1]
             */
            classNotebookPageSource?: string;
            /**
             *
             * The client url of the page. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            clientUrl?: string;
            /**
             *
             * Gets the ID of the page. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
            /**
             *
             * Gets or sets the indentation level of the page.
             *
             * [Api set: OneNoteApi 1.1]
             */
            pageLevel?: number;
            /**
             *
             * Gets or sets the title of the page.
             *
             * [Api set: OneNoteApi 1.1]
             */
            title?: string;
            /**
             *
             * The web url of the page. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            webUrl?: string;
        }
        /** An interface describing the data returned by calling "pageCollection.toJSON()". */
        interface PageCollectionData {
            items?: OneNote.Interfaces.PageData[];
        }
        /** An interface describing the data returned by calling "pageContent.toJSON()". */
        interface PageContentData {
            /**
            *
            * Gets the Image in the PageContent object. Throws an exception if PageContentType is not Image.
            *
            * [Api set: OneNoteApi 1.1]
            */
            image?: OneNote.Interfaces.ImageData;
            /**
            *
            * Gets the ink in the PageContent object. Throws an exception if PageContentType is not Ink.
            *
            * [Api set: OneNoteApi 1.1]
            */
            ink?: OneNote.Interfaces.FloatingInkData;
            /**
            *
            * Gets the Outline in the PageContent object. Throws an exception if PageContentType is not Outline.
            *
            * [Api set: OneNoteApi 1.1]
            */
            outline?: OneNote.Interfaces.OutlineData;
            /**
             *
             * Gets the ID of the PageContent object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
            /**
             *
             * Gets or sets the left (X-axis) position of the PageContent object.
             *
             * [Api set: OneNoteApi 1.1]
             */
            left?: number;
            /**
             *
             * Gets or sets the top (Y-axis) position of the PageContent object.
             *
             * [Api set: OneNoteApi 1.1]
             */
            top?: number;
            /**
             *
             * Gets the type of the PageContent object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            type?: OneNote.PageContentType | "Outline" | "Image" | "Ink" | "Other";
        }
        /** An interface describing the data returned by calling "pageContentCollection.toJSON()". */
        interface PageContentCollectionData {
            items?: OneNote.Interfaces.PageContentData[];
        }
        /** An interface describing the data returned by calling "outline.toJSON()". */
        interface OutlineData {
            /**
            *
            * Gets the collection of Paragraph objects in the Outline. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraphs?: OneNote.Interfaces.ParagraphData[];
            /**
             *
             * Gets the ID of the Outline object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
        }
        /** An interface describing the data returned by calling "paragraph.toJSON()". */
        interface ParagraphData {
            /**
            *
            * Gets the Image object in the Paragraph. Throws an exception if ParagraphType is not Image. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            image?: OneNote.Interfaces.ImageData;
            /**
            *
            * Gets the Ink collection in the Paragraph. Throws an exception if ParagraphType is not Ink. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            inkWords?: OneNote.Interfaces.InkWordData[];
            /**
            *
            * The collection of paragraphs under this paragraph. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraphs?: OneNote.Interfaces.ParagraphData[];
            /**
            *
            * Gets the RichText object in the Paragraph. Throws an exception if ParagraphType is not RichText. Read-only
            *
            * [Api set: OneNoteApi 1.1]
            */
            richText?: OneNote.Interfaces.RichTextData;
            /**
            *
            * Gets the Table object in the Paragraph. Throws an exception if ParagraphType is not Table. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            table?: OneNote.Interfaces.TableData;
            /**
             *
             * Gets the ID of the Paragraph object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
            /**
             *
             * Gets the type of the Paragraph object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            type?: OneNote.ParagraphType | "RichText" | "Image" | "Table" | "Ink" | "Other";
        }
        /** An interface describing the data returned by calling "paragraphCollection.toJSON()". */
        interface ParagraphCollectionData {
            items?: OneNote.Interfaces.ParagraphData[];
        }
        /** An interface describing the data returned by calling "noteTag.toJSON()". */
        interface NoteTagData {
            /**
             *
             * Gets the Id of the NoteTag object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
            /**
             *
             * Gets the status of the NoteTag object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            status?: OneNote.NoteTagStatus | "Unknown" | "Normal" | "Completed" | "Disabled" | "OutlookTask" | "TaskNotSyncedYet" | "TaskRemoved";
            /**
             *
             * Gets the type of the NoteTag object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            type?: OneNote.NoteTagType | "Unknown" | "ToDo" | "Important" | "Question" | "Contact" | "Address" | "PhoneNumber" | "Website" | "Idea" | "Critical" | "ToDoPriority1" | "ToDoPriority2";
        }
        /** An interface describing the data returned by calling "richText.toJSON()". */
        interface RichTextData {
            /**
             *
             * Gets the ID of the RichText object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
            /**
             *
             * The language id of the text. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            languageId?: string;
            /**
             *
             * Gets the text content of the RichText object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            text?: string;
        }
        /** An interface describing the data returned by calling "image.toJSON()". */
        interface ImageData {
            /**
             *
             * Gets or sets the description of the Image.
             *
             * [Api set: OneNoteApi 1.1]
             */
            description?: string;
            /**
             *
             * Gets or sets the height of the Image layout.
             *
             * [Api set: OneNoteApi 1.1]
             */
            height?: number;
            /**
             *
             * Gets or sets the hyperlink of the Image.
             *
             * [Api set: OneNoteApi 1.1]
             */
            hyperlink?: string;
            /**
             *
             * Gets the ID of the Image object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
            /**
             *
             * Gets the data obtained by OCR (Optical Character Recognition) of this Image, such as OCR text and language.
             *
             * [Api set: OneNoteApi 1.1]
             */
            ocrData?: OneNote.ImageOcrData;
            /**
             *
             * Gets or sets the width of the Image layout.
             *
             * [Api set: OneNoteApi 1.1]
             */
            width?: number;
        }
        /** An interface describing the data returned by calling "table.toJSON()". */
        interface TableData {
            /**
            *
            * Gets all of the table rows. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            rows?: OneNote.Interfaces.TableRowData[];
            /**
             *
             * Gets or sets whether the borders are visible or not. True if they are visible, false if they are hidden.
             *
             * [Api set: OneNoteApi 1.1]
             */
            borderVisible?: boolean;
            /**
             *
             * Gets the number of columns in the table.
             *
             * [Api set: OneNoteApi 1.1]
             */
            columnCount?: number;
            /**
             *
             * Gets the ID of the table. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
            /**
             *
             * Gets the number of rows in the table.
             *
             * [Api set: OneNoteApi 1.1]
             */
            rowCount?: number;
        }
        /** An interface describing the data returned by calling "tableRow.toJSON()". */
        interface TableRowData {
            /**
            *
            * Gets the cells in the row. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            cells?: OneNote.Interfaces.TableCellData[];
            /**
             *
             * Gets the number of cells in the row. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            cellCount?: number;
            /**
             *
             * Gets the ID of the row. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
            /**
             *
             * Gets the index of the row in its parent table. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            rowIndex?: number;
        }
        /** An interface describing the data returned by calling "tableRowCollection.toJSON()". */
        interface TableRowCollectionData {
            items?: OneNote.Interfaces.TableRowData[];
        }
        /** An interface describing the data returned by calling "tableCell.toJSON()". */
        interface TableCellData {
            /**
            *
            * Gets the collection of Paragraph objects in the TableCell. Read-only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraphs?: OneNote.Interfaces.ParagraphData[];
            /**
             *
             * Gets the index of the cell in its row. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            cellIndex?: number;
            /**
             *
             * Gets the ID of the cell. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: string;
            /**
             *
             * Gets the index of the cell's row in the table. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            rowIndex?: number;
            /**
             *
             * Gets and sets the shading color of the cell
             *
             * [Api set: OneNoteApi 1.1]
             */
            shadingColor?: string;
        }
        /** An interface describing the data returned by calling "tableCellCollection.toJSON()". */
        interface TableCellCollectionData {
            items?: OneNote.Interfaces.TableCellData[];
        }
        /**
         *
         * Represents the top-level object that contains all globally addressable OneNote objects such as notebooks, the active notebook, and the active section.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface ApplicationLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the collection of notebooks that are open in the OneNote application instance. In OneNote Online, only one notebook at a time is open in the application instance.
            *
            * [Api set: OneNoteApi 1.1]
            */
            notebooks?: OneNote.Interfaces.NotebookCollectionLoadOptions;
        }
        /**
         *
         * Represents ink analysis data for a given set of ink strokes.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface InkAnalysisLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the parent page object.
            *
            * [Api set: OneNoteApi 1.1]
            */
            page?: OneNote.Interfaces.PageLoadOptions;
            /**
             *
             * Gets the ID of the InkAnalysis object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
        }
        /**
         *
         * Represents ink analysis data for an identified paragraph formed by ink strokes.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface InkAnalysisParagraphLoadOptions {
            $all?: boolean;
            /**
            *
            * Reference to the parent InkAnalysisPage.
            *
            * [Api set: OneNoteApi 1.1]
            */
            inkAnalysis?: OneNote.Interfaces.InkAnalysisLoadOptions;
            /**
            *
            * Gets the ink analysis lines in this ink analysis paragraph.
            *
            * [Api set: OneNoteApi 1.1]
            */
            lines?: OneNote.Interfaces.InkAnalysisLineCollectionLoadOptions;
            /**
             *
             * Gets the ID of the InkAnalysisParagraph object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
        }
        /**
         *
         * Represents a collection of InkAnalysisParagraph objects.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface InkAnalysisParagraphCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Reference to the parent InkAnalysisPage.
            *
            * [Api set: OneNoteApi 1.1]
            */
            inkAnalysis?: OneNote.Interfaces.InkAnalysisLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the ink analysis lines in this ink analysis paragraph.
            *
            * [Api set: OneNoteApi 1.1]
            */
            lines?: OneNote.Interfaces.InkAnalysisLineCollectionLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets the ID of the InkAnalysisParagraph object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
        }
        /**
         *
         * Represents ink analysis data for an identified text line formed by ink strokes.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface InkAnalysisLineLoadOptions {
            $all?: boolean;
            /**
            *
            * Reference to the parent InkAnalysisParagraph.
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraph?: OneNote.Interfaces.InkAnalysisParagraphLoadOptions;
            /**
            *
            * Gets the ink analysis words in this ink analysis line.
            *
            * [Api set: OneNoteApi 1.1]
            */
            words?: OneNote.Interfaces.InkAnalysisWordCollectionLoadOptions;
            /**
             *
             * Gets the ID of the InkAnalysisLine object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
        }
        /**
         *
         * Represents a collection of InkAnalysisLine objects.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface InkAnalysisLineCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Reference to the parent InkAnalysisParagraph.
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraph?: OneNote.Interfaces.InkAnalysisParagraphLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the ink analysis words in this ink analysis line.
            *
            * [Api set: OneNoteApi 1.1]
            */
            words?: OneNote.Interfaces.InkAnalysisWordCollectionLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets the ID of the InkAnalysisLine object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
        }
        /**
         *
         * Represents ink analysis data for an identified word formed by ink strokes.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface InkAnalysisWordLoadOptions {
            $all?: boolean;
            /**
            *
            * Reference to the parent InkAnalysisLine.
            *
            * [Api set: OneNoteApi 1.1]
            */
            line?: OneNote.Interfaces.InkAnalysisLineLoadOptions;
            /**
             *
             * Gets the ID of the InkAnalysisWord object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * The id of the recognized language in this inkAnalysisWord. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            languageId?: boolean;
            /**
             *
             * Weak references to the ink strokes that were recognized as part of this ink analysis word. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            strokePointers?: boolean;
            /**
             *
             * The words that were recognized in this ink word, in order of likelihood. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            wordAlternates?: boolean;
        }
        /**
         *
         * Represents a collection of InkAnalysisWord objects.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface InkAnalysisWordCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Reference to the parent InkAnalysisLine.
            *
            * [Api set: OneNoteApi 1.1]
            */
            line?: OneNote.Interfaces.InkAnalysisLineLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets the ID of the InkAnalysisWord object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The id of the recognized language in this inkAnalysisWord. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            languageId?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Weak references to the ink strokes that were recognized as part of this ink analysis word. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            strokePointers?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The words that were recognized in this ink word, in order of likelihood. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            wordAlternates?: boolean;
        }
        /**
         *
         * Represents a group of ink strokes.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface FloatingInkLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the strokes of the FloatingInk object.
            *
            * [Api set: OneNoteApi 1.1]
            */
            inkStrokes?: OneNote.Interfaces.InkStrokeCollectionLoadOptions;
            /**
            *
            * Gets the PageContent parent of the FloatingInk object.
            *
            * [Api set: OneNoteApi 1.1]
            */
            pageContent?: OneNote.Interfaces.PageContentLoadOptions;
            /**
             *
             * Gets the ID of the FloatingInk object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
        }
        /**
         *
         * Represents a single stroke of ink.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface InkStrokeLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the ID of the InkStroke object.
            *
            * [Api set: OneNoteApi 1.1]
            */
            floatingInk?: OneNote.Interfaces.FloatingInkLoadOptions;
            /**
             *
             * Gets the ID of the InkStroke object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
        }
        /**
         *
         * Represents a collection of InkStroke objects.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface InkStrokeCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Gets the ID of the InkStroke object.
            *
            * [Api set: OneNoteApi 1.1]
            */
            floatingInk?: OneNote.Interfaces.FloatingInkLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets the ID of the InkStroke object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
        }
        /**
         *
         * A container for the ink in a word in a paragraph.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface InkWordLoadOptions {
            $all?: boolean;
            /**
            *
            * The parent paragraph containing the ink word.
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraph?: OneNote.Interfaces.ParagraphLoadOptions;
            /**
             *
             * Gets the ID of the InkWord object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * The id of the recognized language in this ink word. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            languageId?: boolean;
            /**
             *
             * The words that were recognized in this ink word, in order of likelihood. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            wordAlternates?: boolean;
        }
        /**
         *
         * Represents a collection of InkWord objects.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface InkWordCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: The parent paragraph containing the ink word.
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraph?: OneNote.Interfaces.ParagraphLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets the ID of the InkWord object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The id of the recognized language in this ink word. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            languageId?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The words that were recognized in this ink word, in order of likelihood. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            wordAlternates?: boolean;
        }
        /**
         *
         * Represents a OneNote notebook. Notebooks contain section groups and sections.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface NotebookLoadOptions {
            $all?: boolean;
            /**
            *
            * The section groups in the notebook. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            sectionGroups?: OneNote.Interfaces.SectionGroupCollectionLoadOptions;
            /**
            *
            * The the sections of the notebook. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            sections?: OneNote.Interfaces.SectionCollectionLoadOptions;
            /**
             *
             * The url of the site that this notebook is located. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            baseUrl?: boolean;
            /**
             *
             * The client url of the notebook. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            clientUrl?: boolean;
            /**
             *
             * Gets the ID of the notebook. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * True if the Notebook is not created by the user (i.e. 'Misplaced Sections'). Read only
             *
             * [Api set: OneNoteApi 1.2]
             */
            isVirtual?: boolean;
            /**
             *
             * Gets the name of the notebook. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            name?: boolean;
        }
        /**
         *
         * Represents a collection of notebooks.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface NotebookCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: The section groups in the notebook. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            sectionGroups?: OneNote.Interfaces.SectionGroupCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: The the sections of the notebook. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            sections?: OneNote.Interfaces.SectionCollectionLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: The url of the site that this notebook is located. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            baseUrl?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The client url of the notebook. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            clientUrl?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the ID of the notebook. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: True if the Notebook is not created by the user (i.e. 'Misplaced Sections'). Read only
             *
             * [Api set: OneNoteApi 1.2]
             */
            isVirtual?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the name of the notebook. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            name?: boolean;
        }
        /**
         *
         * Represents a OneNote section group. Section groups can contain sections and other section groups.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface SectionGroupLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the notebook that contains the section group.
            *
            * [Api set: OneNoteApi 1.1]
            */
            notebook?: OneNote.Interfaces.NotebookLoadOptions;
            /**
            *
            * Gets the section group that contains the section group. Throws ItemNotFound if the section group is a direct child of the notebook.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentSectionGroup?: OneNote.Interfaces.SectionGroupLoadOptions;
            /**
            *
            * Gets the section group that contains the section group. Returns null if the section group is a direct child of the notebook.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentSectionGroupOrNull?: OneNote.Interfaces.SectionGroupLoadOptions;
            /**
            *
            * The collection of section groups in the section group. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            sectionGroups?: OneNote.Interfaces.SectionGroupCollectionLoadOptions;
            /**
            *
            * The collection of sections in the section group. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            sections?: OneNote.Interfaces.SectionCollectionLoadOptions;
            /**
             *
             * The client url of the section group. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            clientUrl?: boolean;
            /**
             *
             * Gets the ID of the section group. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * Gets the name of the section group. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            name?: boolean;
        }
        /**
         *
         * Represents a collection of section groups.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface SectionGroupCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Gets the notebook that contains the section group.
            *
            * [Api set: OneNoteApi 1.1]
            */
            notebook?: OneNote.Interfaces.NotebookLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the section group that contains the section group. Throws ItemNotFound if the section group is a direct child of the notebook.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentSectionGroup?: OneNote.Interfaces.SectionGroupLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the section group that contains the section group. Returns null if the section group is a direct child of the notebook.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentSectionGroupOrNull?: OneNote.Interfaces.SectionGroupLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: The collection of section groups in the section group. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            sectionGroups?: OneNote.Interfaces.SectionGroupCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: The collection of sections in the section group. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            sections?: OneNote.Interfaces.SectionCollectionLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: The client url of the section group. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            clientUrl?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the ID of the section group. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the name of the section group. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            name?: boolean;
        }
        /**
         *
         * Represents a OneNote section. Sections can contain pages.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface SectionLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the notebook that contains the section.
            *
            * [Api set: OneNoteApi 1.1]
            */
            notebook?: OneNote.Interfaces.NotebookLoadOptions;
            /**
            *
            * The collection of pages in the section. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            pages?: OneNote.Interfaces.PageCollectionLoadOptions;
            /**
            *
            * Gets the section group that contains the section. Throws ItemNotFound if the section is a direct child of the notebook.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentSectionGroup?: OneNote.Interfaces.SectionGroupLoadOptions;
            /**
            *
            * Gets the section group that contains the section. Returns null if the section is a direct child of the notebook.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentSectionGroupOrNull?: OneNote.Interfaces.SectionGroupLoadOptions;
            /**
             *
             * The client url of the section. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            clientUrl?: boolean;
            /**
             *
             * Gets the ID of the section. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * True if this section is encrypted with a password. Read only
             *
             * [Api set: OneNoteApi 1.2]
             */
            isEncrypted?: boolean;
            /**
             *
             * True if this section is locked. Read only
             *
             * [Api set: OneNoteApi 1.2]
             */
            isLocked?: boolean;
            /**
             *
             * Gets the name of the section. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            name?: boolean;
            /**
             *
             * The web url of the page. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            webUrl?: boolean;
        }
        /**
         *
         * Represents a collection of sections.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface SectionCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Gets the notebook that contains the section.
            *
            * [Api set: OneNoteApi 1.1]
            */
            notebook?: OneNote.Interfaces.NotebookLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: The collection of pages in the section. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            pages?: OneNote.Interfaces.PageCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the section group that contains the section. Throws ItemNotFound if the section is a direct child of the notebook.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentSectionGroup?: OneNote.Interfaces.SectionGroupLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the section group that contains the section. Returns null if the section is a direct child of the notebook.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentSectionGroupOrNull?: OneNote.Interfaces.SectionGroupLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: The client url of the section. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            clientUrl?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the ID of the section. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: True if this section is encrypted with a password. Read only
             *
             * [Api set: OneNoteApi 1.2]
             */
            isEncrypted?: boolean;
            /**
             *
             * For EACH ITEM in the collection: True if this section is locked. Read only
             *
             * [Api set: OneNoteApi 1.2]
             */
            isLocked?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the name of the section. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The web url of the page. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            webUrl?: boolean;
        }
        /**
         *
         * Represents a OneNote page.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface PageLoadOptions {
            $all?: boolean;
            /**
            *
            * The collection of PageContent objects on the page. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            contents?: OneNote.Interfaces.PageContentCollectionLoadOptions;
            /**
            *
            * Text interpretation for the ink on the page. Returns null if there is no ink analysis information. Read only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            inkAnalysisOrNull?: OneNote.Interfaces.InkAnalysisLoadOptions;
            /**
            *
            * Gets the section that contains the page.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentSection?: OneNote.Interfaces.SectionLoadOptions;
            /**
             *
             * Gets the ClassNotebookPageSource to the page.
             *
             * [Api set: OneNoteApi 1.1]
             */
            classNotebookPageSource?: boolean;
            /**
             *
             * The client url of the page. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            clientUrl?: boolean;
            /**
             *
             * Gets the ID of the page. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * Gets or sets the indentation level of the page.
             *
             * [Api set: OneNoteApi 1.1]
             */
            pageLevel?: boolean;
            /**
             *
             * Gets or sets the title of the page.
             *
             * [Api set: OneNoteApi 1.1]
             */
            title?: boolean;
            /**
             *
             * The web url of the page. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            webUrl?: boolean;
        }
        /**
         *
         * Represents a collection of pages.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface PageCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: The collection of PageContent objects on the page. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            contents?: OneNote.Interfaces.PageContentCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Text interpretation for the ink on the page. Returns null if there is no ink analysis information. Read only.
            *
            * [Api set: OneNoteApi 1.1]
            */
            inkAnalysisOrNull?: OneNote.Interfaces.InkAnalysisLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the section that contains the page.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentSection?: OneNote.Interfaces.SectionLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets the ClassNotebookPageSource to the page.
             *
             * [Api set: OneNoteApi 1.1]
             */
            classNotebookPageSource?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The client url of the page. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            clientUrl?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the ID of the page. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the indentation level of the page.
             *
             * [Api set: OneNoteApi 1.1]
             */
            pageLevel?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the title of the page.
             *
             * [Api set: OneNoteApi 1.1]
             */
            title?: boolean;
            /**
             *
             * For EACH ITEM in the collection: The web url of the page. Read only
             *
             * [Api set: OneNoteApi 1.1]
             */
            webUrl?: boolean;
        }
        /**
         *
         * Represents a region on a page that contains top-level content types such as Outline or Image. A PageContent object can be assigned an XY position.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface PageContentLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the Image in the PageContent object. Throws an exception if PageContentType is not Image.
            *
            * [Api set: OneNoteApi 1.1]
            */
            image?: OneNote.Interfaces.ImageLoadOptions;
            /**
            *
            * Gets the ink in the PageContent object. Throws an exception if PageContentType is not Ink.
            *
            * [Api set: OneNoteApi 1.1]
            */
            ink?: OneNote.Interfaces.FloatingInkLoadOptions;
            /**
            *
            * Gets the Outline in the PageContent object. Throws an exception if PageContentType is not Outline.
            *
            * [Api set: OneNoteApi 1.1]
            */
            outline?: OneNote.Interfaces.OutlineLoadOptions;
            /**
            *
            * Gets the page that contains the PageContent object.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentPage?: OneNote.Interfaces.PageLoadOptions;
            /**
             *
             * Gets the ID of the PageContent object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * Gets or sets the left (X-axis) position of the PageContent object.
             *
             * [Api set: OneNoteApi 1.1]
             */
            left?: boolean;
            /**
             *
             * Gets or sets the top (Y-axis) position of the PageContent object.
             *
             * [Api set: OneNoteApi 1.1]
             */
            top?: boolean;
            /**
             *
             * Gets the type of the PageContent object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            type?: boolean;
        }
        /**
         *
         * Represents the contents of a page, as a collection of PageContent objects.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface PageContentCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Gets the Image in the PageContent object. Throws an exception if PageContentType is not Image.
            *
            * [Api set: OneNoteApi 1.1]
            */
            image?: OneNote.Interfaces.ImageLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the ink in the PageContent object. Throws an exception if PageContentType is not Ink.
            *
            * [Api set: OneNoteApi 1.1]
            */
            ink?: OneNote.Interfaces.FloatingInkLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the Outline in the PageContent object. Throws an exception if PageContentType is not Outline.
            *
            * [Api set: OneNoteApi 1.1]
            */
            outline?: OneNote.Interfaces.OutlineLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the page that contains the PageContent object.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentPage?: OneNote.Interfaces.PageLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets the ID of the PageContent object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the left (X-axis) position of the PageContent object.
             *
             * [Api set: OneNoteApi 1.1]
             */
            left?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets or sets the top (Y-axis) position of the PageContent object.
             *
             * [Api set: OneNoteApi 1.1]
             */
            top?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the type of the PageContent object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            type?: boolean;
        }
        /**
         *
         * Represents a container for Paragraph objects.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface OutlineLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the PageContent object that contains the Outline. This object defines the position of the Outline on the page.
            *
            * [Api set: OneNoteApi 1.1]
            */
            pageContent?: OneNote.Interfaces.PageContentLoadOptions;
            /**
            *
            * Gets the collection of Paragraph objects in the Outline.
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraphs?: OneNote.Interfaces.ParagraphCollectionLoadOptions;
            /**
             *
             * Gets the ID of the Outline object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
        }
        /**
         *
         * A container for the visible content on a page. A Paragraph can contain any one ParagraphType type of content.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface ParagraphLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the Image object in the Paragraph. Throws an exception if ParagraphType is not Image.
            *
            * [Api set: OneNoteApi 1.1]
            */
            image?: OneNote.Interfaces.ImageLoadOptions;
            /**
            *
            * Gets the Ink collection in the Paragraph. Throws an exception if ParagraphType is not Ink.
            *
            * [Api set: OneNoteApi 1.1]
            */
            inkWords?: OneNote.Interfaces.InkWordCollectionLoadOptions;
            /**
            *
            * Gets the Outline object that contains the Paragraph.
            *
            * [Api set: OneNoteApi 1.1]
            */
            outline?: OneNote.Interfaces.OutlineLoadOptions;
            /**
            *
            * The collection of paragraphs under this paragraph. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraphs?: OneNote.Interfaces.ParagraphCollectionLoadOptions;
            /**
            *
            * Gets the parent paragraph object. Throws if a parent paragraph does not exist.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentParagraph?: OneNote.Interfaces.ParagraphLoadOptions;
            /**
            *
            * Gets the parent paragraph object. Returns null if a parent paragraph does not exist.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentParagraphOrNull?: OneNote.Interfaces.ParagraphLoadOptions;
            /**
            *
            * Gets the TableCell object that contains the Paragraph if one exists. If parent is not a TableCell, throws ItemNotFound.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentTableCell?: OneNote.Interfaces.TableCellLoadOptions;
            /**
            *
            * Gets the TableCell object that contains the Paragraph if one exists. If parent is not a TableCell, returns null.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentTableCellOrNull?: OneNote.Interfaces.TableCellLoadOptions;
            /**
            *
            * Gets the RichText object in the Paragraph. Throws an exception if ParagraphType is not RichText.
            *
            * [Api set: OneNoteApi 1.1]
            */
            richText?: OneNote.Interfaces.RichTextLoadOptions;
            /**
            *
            * Gets the Table object in the Paragraph. Throws an exception if ParagraphType is not Table.
            *
            * [Api set: OneNoteApi 1.1]
            */
            table?: OneNote.Interfaces.TableLoadOptions;
            /**
             *
             * Gets the ID of the Paragraph object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * Gets the type of the Paragraph object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            type?: boolean;
        }
        /**
         *
         * Represents a collection of Paragraph objects.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface ParagraphCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Gets the Image object in the Paragraph. Throws an exception if ParagraphType is not Image.
            *
            * [Api set: OneNoteApi 1.1]
            */
            image?: OneNote.Interfaces.ImageLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the Ink collection in the Paragraph. Throws an exception if ParagraphType is not Ink.
            *
            * [Api set: OneNoteApi 1.1]
            */
            inkWords?: OneNote.Interfaces.InkWordCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the Outline object that contains the Paragraph.
            *
            * [Api set: OneNoteApi 1.1]
            */
            outline?: OneNote.Interfaces.OutlineLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: The collection of paragraphs under this paragraph. Read only
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraphs?: OneNote.Interfaces.ParagraphCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the parent paragraph object. Throws if a parent paragraph does not exist.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentParagraph?: OneNote.Interfaces.ParagraphLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the parent paragraph object. Returns null if a parent paragraph does not exist.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentParagraphOrNull?: OneNote.Interfaces.ParagraphLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the TableCell object that contains the Paragraph if one exists. If parent is not a TableCell, throws ItemNotFound.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentTableCell?: OneNote.Interfaces.TableCellLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the TableCell object that contains the Paragraph if one exists. If parent is not a TableCell, returns null.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentTableCellOrNull?: OneNote.Interfaces.TableCellLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the RichText object in the Paragraph. Throws an exception if ParagraphType is not RichText.
            *
            * [Api set: OneNoteApi 1.1]
            */
            richText?: OneNote.Interfaces.RichTextLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the Table object in the Paragraph. Throws an exception if ParagraphType is not Table.
            *
            * [Api set: OneNoteApi 1.1]
            */
            table?: OneNote.Interfaces.TableLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets the ID of the Paragraph object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the type of the Paragraph object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            type?: boolean;
        }
        /**
         *
         * A container for the NoteTag in a paragraph.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface NoteTagLoadOptions {
            $all?: boolean;
            /**
             *
             * Gets the Id of the NoteTag object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * Gets the status of the NoteTag object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            status?: boolean;
            /**
             *
             * Gets the type of the NoteTag object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            type?: boolean;
        }
        /**
         *
         * Represents a RichText object in a Paragraph.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface RichTextLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the Paragraph object that contains the RichText object.
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraph?: OneNote.Interfaces.ParagraphLoadOptions;
            /**
             *
             * Gets the ID of the RichText object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * The language id of the text. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            languageId?: boolean;
            /**
             *
             * Gets the text content of the RichText object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            text?: boolean;
        }
        /**
         *
         * Represents an Image. An Image can be a direct child of a PageContent object or a Paragraph object.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface ImageLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the PageContent object that contains the Image. Throws if the Image is not a direct child of a PageContent. This object defines the position of the Image on the page.
            *
            * [Api set: OneNoteApi 1.1]
            */
            pageContent?: OneNote.Interfaces.PageContentLoadOptions;
            /**
            *
            * Gets the Paragraph object that contains the Image. Throws if the Image is not a direct child of a Paragraph.
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraph?: OneNote.Interfaces.ParagraphLoadOptions;
            /**
             *
             * Gets or sets the description of the Image.
             *
             * [Api set: OneNoteApi 1.1]
             */
            description?: boolean;
            /**
             *
             * Gets or sets the height of the Image layout.
             *
             * [Api set: OneNoteApi 1.1]
             */
            height?: boolean;
            /**
             *
             * Gets or sets the hyperlink of the Image.
             *
             * [Api set: OneNoteApi 1.1]
             */
            hyperlink?: boolean;
            /**
             *
             * Gets the ID of the Image object. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * Gets the data obtained by OCR (Optical Character Recognition) of this Image, such as OCR text and language.
             *
             * [Api set: OneNoteApi 1.1]
             */
            ocrData?: boolean;
            /**
             *
             * Gets or sets the width of the Image layout.
             *
             * [Api set: OneNoteApi 1.1]
             */
            width?: boolean;
        }
        /**
         *
         * Represents a table in a OneNote page.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface TableLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the Paragraph object that contains the Table object.
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraph?: OneNote.Interfaces.ParagraphLoadOptions;
            /**
            *
            * Gets all of the table rows.
            *
            * [Api set: OneNoteApi 1.1]
            */
            rows?: OneNote.Interfaces.TableRowCollectionLoadOptions;
            /**
             *
             * Gets or sets whether the borders are visible or not. True if they are visible, false if they are hidden.
             *
             * [Api set: OneNoteApi 1.1]
             */
            borderVisible?: boolean;
            /**
             *
             * Gets the number of columns in the table.
             *
             * [Api set: OneNoteApi 1.1]
             */
            columnCount?: boolean;
            /**
             *
             * Gets the ID of the table. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * Gets the number of rows in the table.
             *
             * [Api set: OneNoteApi 1.1]
             */
            rowCount?: boolean;
        }
        /**
         *
         * Represents a row in a table.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface TableRowLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the cells in the row.
            *
            * [Api set: OneNoteApi 1.1]
            */
            cells?: OneNote.Interfaces.TableCellCollectionLoadOptions;
            /**
            *
            * Gets the parent table.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentTable?: OneNote.Interfaces.TableLoadOptions;
            /**
             *
             * Gets the number of cells in the row. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            cellCount?: boolean;
            /**
             *
             * Gets the ID of the row. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * Gets the index of the row in its parent table. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            rowIndex?: boolean;
        }
        /**
         *
         * Contains a collection of TableRow objects.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface TableRowCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Gets the cells in the row.
            *
            * [Api set: OneNoteApi 1.1]
            */
            cells?: OneNote.Interfaces.TableCellCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the parent table.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentTable?: OneNote.Interfaces.TableLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets the number of cells in the row. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            cellCount?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the ID of the row. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the index of the row in its parent table. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            rowIndex?: boolean;
        }
        /**
         *
         * Represents a cell in a OneNote table.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface TableCellLoadOptions {
            $all?: boolean;
            /**
            *
            * Gets the collection of Paragraph objects in the TableCell.
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraphs?: OneNote.Interfaces.ParagraphCollectionLoadOptions;
            /**
            *
            * Gets the parent row of the cell.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentRow?: OneNote.Interfaces.TableRowLoadOptions;
            /**
             *
             * Gets the index of the cell in its row. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            cellIndex?: boolean;
            /**
             *
             * Gets the ID of the cell. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * Gets the index of the cell's row in the table. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            rowIndex?: boolean;
            /**
             *
             * Gets and sets the shading color of the cell
             *
             * [Api set: OneNoteApi 1.1]
             */
            shadingColor?: boolean;
        }
        /**
         *
         * Contains a collection of TableCell objects.
         *
         * [Api set: OneNoteApi 1.1]
         */
        interface TableCellCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Gets the collection of Paragraph objects in the TableCell.
            *
            * [Api set: OneNoteApi 1.1]
            */
            paragraphs?: OneNote.Interfaces.ParagraphCollectionLoadOptions;
            /**
            *
            * For EACH ITEM in the collection: Gets the parent row of the cell.
            *
            * [Api set: OneNoteApi 1.1]
            */
            parentRow?: OneNote.Interfaces.TableRowLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Gets the index of the cell in its row. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            cellIndex?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the ID of the cell. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the index of the cell's row in the table. Read-only.
             *
             * [Api set: OneNoteApi 1.1]
             */
            rowIndex?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets and sets the shading color of the cell
             *
             * [Api set: OneNoteApi 1.1]
             */
            shadingColor?: boolean;
        }
    }
}
declare namespace OneNote {
    class RequestContext extends OfficeCore.RequestContext {
        constructor(url?: string);
        readonly application: Application;
    }
    /**
     * Executes a batch script that performs actions on the OneNote object model, using a new request context. When the promise is resolved, any tracked objects that were automatically allocated during execution will be released.
     * @param batch - A function that takes in an OneNote.RequestContext and returns a promise (typically, just the result of "context.sync()"). The context parameter facilitates requests to the OneNote application. Since the Office add-in and the OneNote application run in two different processes, the request context is required to get access to the OneNote object model from the add-in.
     */
    function run<T>(batch: (context: OneNote.RequestContext) => Promise<T>): Promise<T>;
    /**
     * Executes a batch script that performs actions on the OneNote object model, using the request context of a previously-created API object.
     * @param object - A previously-created API object. The batch will use the same request context as the passed-in object, which means that any changes applied to the object will be picked up by "context.sync()".
     * @param batch - A function that takes in an OneNote.RequestContext and returns a promise (typically, just the result of "context.sync()"). When the promise is resolved, any tracked objects that were automatically allocated during execution will be released.
     */
    function run<T>(object: OfficeExtension.ClientObject, batch: (context: OneNote.RequestContext) => Promise<T>): Promise<T>;
    /**
     * Executes a batch script that performs actions on the OneNote object model, using the request context of previously-created API objects.
     * @param object - An array of previously-created API objects. The array will be validated to make sure that all of the objects share the same context. The batch will use this shared request context, which means that any changes applied to these objects will be picked up by "context.sync()".
     * @param batch - A function that takes in an OneNote.RequestContext and returns a promise (typically, just the result of "context.sync()"). When the promise is resolved, any tracked objects that were automatically allocated during execution will be released.
     */
    function run<T>(objects: OfficeExtension.ClientObject[], batch: (context: OneNote.RequestContext) => Promise<T>): Promise<T>;
}


////////////////////////////////////////////////////////////////
/////////////////////// End OneNote APIs ///////////////////////
////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////
/////////////////////// Begin Visio APIs ///////////////////////
////////////////////////////////////////////////////////////////

declare namespace Visio {
    /**
     *
     * Provides information about the shape that raised the ShapeMouseEnter event.
     *
     * [Api set:  1.1]
     */
    interface ShapeMouseEnterEventArgs {
        /**
         *
         * Gets the name of the page which has the shape object that raised the ShapeMouseEnter event.
         *
         * [Api set:  1.1]
         */
        pageName: string;
        /**
         *
         * Gets the name of the shape object that raised the ShapeMouseEnter event.
         *
         * [Api set:  1.1]
         */
        shapeName: string;
    }
    /**
     *
     * Provides information about the shape that raised the ShapeMouseLeave event.
     *
     * [Api set:  1.1]
     */
    interface ShapeMouseLeaveEventArgs {
        /**
         *
         * Gets the name of the page which has the shape object that raised the ShapeMouseLeave event.
         *
         * [Api set:  1.1]
         */
        pageName: string;
        /**
         *
         * Gets the name of the shape object that raised the ShapeMouseLeave event.
         *
         * [Api set:  1.1]
         */
        shapeName: string;
    }
    /**
     *
     * Provides information about the page that raised the PageLoadComplete event.
     *
     * [Api set:  1.1]
     */
    interface PageLoadCompleteEventArgs {
        /**
         *
         * Gets the name of the page that raised the PageLoad event.
         *
         * [Api set:  1.1]
         */
        pageName: string;
        /**
         *
         * Gets the success or failure of the PageLoadComplete event.
         *
         * [Api set:  1.1]
         */
        success: boolean;
    }
    /**
     *
     * Provides information about the document that raised the DataRefreshComplete event.
     *
     * [Api set:  1.1]
     */
    interface DataRefreshCompleteEventArgs {
        /**
         *
         * Gets the document object that raised the DataRefreshComplete event.
         *
         * [Api set:  1.1]
         */
        document: Visio.Document;
        /**
         *
         * Gets the success or failure of the DataRefreshComplete event.
         *
         * [Api set:  1.1]
         */
        success: boolean;
    }
    /**
     *
     * Provides information about the shape collection that raised the SelectionChanged event.
     *
     * [Api set:  1.1]
     */
    interface SelectionChangedEventArgs {
        /**
         *
         * Gets the name of the page which has the ShapeCollection object that raised the SelectionChanged event.
         *
         * [Api set:  1.1]
         */
        pageName: string;
        /**
         *
         * Gets the array of shape names that raised the SelectionChanged event.
         *
         * [Api set:  1.1]
         */
        shapeNames: string[];
    }
    /**
     *
     * Provides information about the success or failure of the DocumentLoadComplete event.
     *
     * [Api set:  1.1]
     */
    interface DocumentLoadCompleteEventArgs {
        /**
         *
         * Gets the success or failure of the DocumentLoadComplete event.
         *
         * [Api set:  1.1]
         */
        success: boolean;
    }
    /**
     *
     * Provides information about the page that raised the PageRenderComplete event.
     *
     * [Api set:  1.1]
     */
    interface PageRenderCompleteEventArgs {
        /**
         *
         * Gets the name of the page that raised the PageLoad event.
         *
         * [Api set:  1.1]
         */
        pageName: string;
        /**
         *
         * Gets the success/failure of the PageRender event.
         *
         * [Api set:  1.1]
         */
        success: boolean;
    }
    /**
     *
     * Represents the Application.
     *
     * [Api set:  1.1]
     */
    class Application extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Show or hide the iFrame application borders.
         *
         * [Api set:  1.1]
         */
        showBorders: boolean;
        /**
         *
         * Show or hide the standard toolbars.
         *
         * [Api set:  1.1]
         */
        showToolbars: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Visio.Application): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ApplicationUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Visio.Application): void;
        /**
         *
         * Sets the visibility of a specific toolbar in the application.
         *
         * [Api set:  1.1]
         *
         * @param id The type of the Toolbar
         * @param show Whether the toolbar is visibile or not.
         */
        showToolbar(id: Visio.ToolBarType, show: boolean): void;
        /**
         *
         * Sets the visibility of a specific toolbar in the application.
         *
         * [Api set:  1.1]
         *
         * @param id The type of the Toolbar
         * @param show Whether the toolbar is visibile or not.
         */
        showToolbar(id: "CommandBar" | "PageNavigationBar" | "StatusBar", show: boolean): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.Application` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.Application` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.Application` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Visio.Interfaces.ApplicationLoadOptions): Visio.Application;
        load(option?: string | string[]): Visio.Application;
        load(option?: {
            select?: string;
            expand?: string;
        }): Visio.Application;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Visio.Application object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.ApplicationData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Visio.Interfaces.ApplicationData;
    }
    /**
     *
     * Represents the Document class.
     *
     * [Api set:  1.1]
     */
    class Document extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents a Visio application instance that contains this document. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly application: Visio.Application;
        /**
         *
         * Represents a collection of pages associated with the document. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly pages: Visio.PageCollection;
        /**
         *
         * Returns the DocumentView object. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly view: Visio.DocumentView;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Visio.Document): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.DocumentUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Visio.Document): void;
        /**
         *
         * Returns the Active Page of the document.
         *
         * [Api set:  1.1]
         */
        getActivePage(): Visio.Page;
        /**
         *
         * Set the Active Page of the document.
         *
         * [Api set:  1.1]
         *
         * @param PageName Name of the page
         */
        setActivePage(PageName: string): void;
        /**
         *
         * Triggers the refresh of the data in the Diagram, for all pages.
         *
         * [Api set:  1.1]
         */
        startDataRefresh(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.Document` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.Document` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.Document` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Visio.Interfaces.DocumentLoadOptions): Visio.Document;
        load(option?: string | string[]): Visio.Document;
        load(option?: {
            select?: string;
            expand?: string;
        }): Visio.Document;
        /**
         *
         * Occurs when the data is refreshed in the diagram.
         *
         * [Api set:  1.1]
         *
         * @eventproperty
         */
        readonly onDataRefreshComplete: OfficeExtension.EventHandlers<Visio.DataRefreshCompleteEventArgs>;
        /**
         *
         * Occurs when the Document is loaded, refreshed, or changed.
         *
         * [Api set:  1.1]
         *
         * @eventproperty
         */
        readonly onDocumentLoadComplete: OfficeExtension.EventHandlers<Visio.DocumentLoadCompleteEventArgs>;
        /**
         *
         * Occurs when the page is finished loading.
         *
         * [Api set:  1.1]
         *
         * @eventproperty
         */
        readonly onPageLoadComplete: OfficeExtension.EventHandlers<Visio.PageLoadCompleteEventArgs>;
        /**
         *
         * Occurs when the current selection of shapes changes.
         *
         * [Api set:  1.1]
         *
         * @eventproperty
         */
        readonly onSelectionChanged: OfficeExtension.EventHandlers<Visio.SelectionChangedEventArgs>;
        /**
         *
         * Occurs when the user moves the mouse pointer into the bounding box of a shape.
         *
         * [Api set:  1.1]
         *
         * @eventproperty
         */
        readonly onShapeMouseEnter: OfficeExtension.EventHandlers<Visio.ShapeMouseEnterEventArgs>;
        /**
         *
         * Occurs when the user moves the mouse out of the bounding box of a shape.
         *
         * [Api set:  1.1]
         *
         * @eventproperty
         */
        readonly onShapeMouseLeave: OfficeExtension.EventHandlers<Visio.ShapeMouseLeaveEventArgs>;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Visio.Document object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.DocumentData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Visio.Interfaces.DocumentData;
    }
    /**
     *
     * Represents the DocumentView class.
     *
     * [Api set:  1.1]
     */
    class DocumentView extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Disable Hyperlinks.
         *
         * [Api set:  1.1]
         */
        disableHyperlinks: boolean;
        /**
         *
         * Disable Pan.
         *
         * [Api set:  1.1]
         */
        disablePan: boolean;
        /**
         *
         * Disable PanZoomWindow.
         *
         * [Api set:  1.1]
         */
        disablePanZoomWindow: boolean;
        /**
         *
         * Disable Zoom.
         *
         * [Api set:  1.1]
         */
        disableZoom: boolean;
        /**
         *
         * Hide Diagram Boundary.
         *
         * [Api set:  1.1]
         */
        hideDiagramBoundary: boolean;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Visio.DocumentView): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.DocumentViewUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Visio.DocumentView): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.DocumentView` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.DocumentView` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.DocumentView` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Visio.Interfaces.DocumentViewLoadOptions): Visio.DocumentView;
        load(option?: string | string[]): Visio.DocumentView;
        load(option?: {
            select?: string;
            expand?: string;
        }): Visio.DocumentView;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Visio.DocumentView object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.DocumentViewData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Visio.Interfaces.DocumentViewData;
    }
    /**
     *
     * Represents the Page class.
     *
     * [Api set:  1.1]
     */
    class Page extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * All shapes in the Page, including subshapes. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly allShapes: Visio.ShapeCollection;
        /**
         *
         * Returns the Comments Collection.  Read-only.
         *
         * [Api set:  1.1]
         */
        readonly comments: Visio.CommentCollection;
        /**
         *
         * All top-level shapes in the Page.Read-only.
         *
         * [Api set:  1.1]
         */
        readonly shapes: Visio.ShapeCollection;
        /**
         *
         * Returns the view of the page. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly view: Visio.PageView;
        /**
         *
         * Returns the height of the page. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly height: number;
        /**
         *
         * Index of the Page. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly index: number;
        /**
         *
         * Whether the page is a background page or not. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly isBackground: boolean;
        /**
         *
         * Page name. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly name: string;
        /**
         *
         * Returns the width of the page. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly width: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Visio.Page): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.PageUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Visio.Page): void;
        /**
         *
         * Set the page as Active Page of the document.
         *
         * [Api set:  1.1]
         */
        activate(): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.Page` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.Page` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.Page` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Visio.Interfaces.PageLoadOptions): Visio.Page;
        load(option?: string | string[]): Visio.Page;
        load(option?: {
            select?: string;
            expand?: string;
        }): Visio.Page;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Visio.Page object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.PageData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Visio.Interfaces.PageData;
    }
    /**
     *
     * Represents the PageView class.
     *
     * [Api set:  1.1]
     */
    class PageView extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Get and set Page's Zoom level. The value can be between 10 and 400 and denotes the percentage of zoom.
         *
         * [Api set:  1.1]
         */
        zoom: number;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Visio.PageView): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.PageViewUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Visio.PageView): void;
        /**
         *
         * Pans the Visio drawing to place the specified shape in the center of the view.
         *
         * [Api set:  1.1]
         *
         * @param ShapeId ShapeId to be seen in the center.
         */
        centerViewportOnShape(ShapeId: number): void;
        /**
         *
         * Fit Page to current window.
         *
         * [Api set:  1.1]
         */
        fitToWindow(): void;
        /**
         *
         * Returns the position object that specifies the position of the page in the view.
         *
         * [Api set:  1.1]
         */
        getPosition(): OfficeExtension.ClientResult<Visio.Position>;
        /**
         *
         * Represents the Selection in the page.
         *
         * [Api set:  1.1]
         */
        getSelection(): Visio.Selection;
        /**
         *
         * To check if the shape is in view of the page or not.
         *
         * [Api set:  1.1]
         *
         * @param Shape Shape to be checked.
         */
        isShapeInViewport(Shape: Visio.Shape): OfficeExtension.ClientResult<boolean>;
        /**
         *
         * Sets the position of the page in the view.
         *
         * [Api set:  1.1]
         *
         * @param Position Position object that specifies the new position of the page in the view.
         */
        setPosition(Position: Visio.Position): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.PageView` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.PageView` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.PageView` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Visio.Interfaces.PageViewLoadOptions): Visio.PageView;
        load(option?: string | string[]): Visio.PageView;
        load(option?: {
            select?: string;
            expand?: string;
        }): Visio.PageView;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Visio.PageView object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.PageViewData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Visio.Interfaces.PageViewData;
    }
    /**
     *
     * Represents a collection of Page objects that are part of the document.
     *
     * [Api set:  1.1]
     */
    class PageCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Visio.Page[];
        /**
         *
         * Gets the number of pages in the collection.
         *
         * [Api set:  1.1]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a page using its key (name or Id).
         *
         * [Api set:  1.1]
         *
         * @param key Key is the name or Id of the page to be retrieved.
         */
        getItem(key: number | string): Visio.Page;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.PageCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.PageCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.PageCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Visio.Interfaces.PageCollectionLoadOptions & Visio.Interfaces.CollectionLoadOptions): Visio.PageCollection;
        load(option?: string | string[]): Visio.PageCollection;
        load(option?: OfficeExtension.LoadOption): Visio.PageCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Visio.PageCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.PageCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Visio.Interfaces.PageCollectionData;
    }
    /**
     *
     * Represents the Shape Collection.
     *
     * [Api set:  1.1]
     */
    class ShapeCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Visio.Shape[];
        /**
         *
         * Gets the number of Shapes in the collection.
         *
         * [Api set:  1.1]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a Shape using its key (name or Index).
         *
         * [Api set:  1.1]
         *
         * @param key Key is the Name or Index of the shape to be retrieved.
         */
        getItem(key: number | string): Visio.Shape;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.ShapeCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.ShapeCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.ShapeCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Visio.Interfaces.ShapeCollectionLoadOptions & Visio.Interfaces.CollectionLoadOptions): Visio.ShapeCollection;
        load(option?: string | string[]): Visio.ShapeCollection;
        load(option?: OfficeExtension.LoadOption): Visio.ShapeCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Visio.ShapeCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.ShapeCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Visio.Interfaces.ShapeCollectionData;
    }
    /**
     *
     * Represents the Shape class.
     *
     * [Api set:  1.1]
     */
    class Shape extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Returns the Comments Collection. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly comments: Visio.CommentCollection;
        /**
         *
         * Returns the Hyperlinks collection for a Shape object. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly hyperlinks: Visio.HyperlinkCollection;
        /**
         *
         * Returns the Shape's Data Section. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly shapeDataItems: Visio.ShapeDataItemCollection;
        /**
         *
         * Gets SubShape Collection. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly subShapes: Visio.ShapeCollection;
        /**
         *
         * Returns the view of the shape. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly view: Visio.ShapeView;
        /**
         *
         * Shape's identifier. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly id: number;
        /**
         *
         * Shape's name. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly name: string;
        /**
         *
         * Returns true, if shape is selected. User can set true to select the shape explicitly.
         *
         * [Api set:  1.1]
         */
        select: boolean;
        /**
         *
         * Shape's text. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly text: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Visio.Shape): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ShapeUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Visio.Shape): void;
        /**
         *
         * Returns the BoundingBox object that specifies bounding box of the shape.
         *
         * [Api set:  1.1]
         */
        getBounds(): OfficeExtension.ClientResult<Visio.BoundingBox>;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.Shape` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.Shape` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.Shape` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Visio.Interfaces.ShapeLoadOptions): Visio.Shape;
        load(option?: string | string[]): Visio.Shape;
        load(option?: {
            select?: string;
            expand?: string;
        }): Visio.Shape;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Visio.Shape object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.ShapeData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Visio.Interfaces.ShapeData;
    }
    /**
     *
     * Represents the ShapeView class.
     *
     * [Api set:  1.1]
     */
    class ShapeView extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Represents the highlight around the shape.
         *
         * [Api set:  1.1]
         */
        highlight: Visio.Highlight;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Visio.ShapeView): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.ShapeViewUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Visio.ShapeView): void;
        /**
         *
         * Adds an overlay on top of the shape.
         *
         * [Api set:  1.1]
         *
         * @param OverlayType An Overlay Type. Can be 'Text' or 'Image'.
         * @param Content Content of Overlay.
         * @param OverlayHorizontalAlignment Horizontal Alignment of Overlay. Can be 'Left', 'Center', or 'Right'.
         * @param OverlayVerticalAlignment Vertical Alignment of Overlay. Can be 'Top', 'Middle', 'Bottom'.
         * @param Width Overlay Width.
         * @param Height Overlay Height.
         */
        addOverlay(OverlayType: Visio.OverlayType, Content: string, OverlayHorizontalAlignment: Visio.OverlayHorizontalAlignment, OverlayVerticalAlignment: Visio.OverlayVerticalAlignment, Width: number, Height: number): OfficeExtension.ClientResult<number>;
        /**
         *
         * Adds an overlay on top of the shape.
         *
         * [Api set:  1.1]
         *
         * @param OverlayType An Overlay Type. Can be 'Text' or 'Image'.
         * @param Content Content of Overlay.
         * @param OverlayHorizontalAlignment Horizontal Alignment of Overlay. Can be 'Left', 'Center', or 'Right'.
         * @param OverlayVerticalAlignment Vertical Alignment of Overlay. Can be 'Top', 'Middle', 'Bottom'.
         * @param Width Overlay Width.
         * @param Height Overlay Height.
         */
        addOverlay(OverlayType: "Text" | "Image" | "Html", Content: string, OverlayHorizontalAlignment: "Left" | "Center" | "Right", OverlayVerticalAlignment: "Top" | "Middle" | "Bottom", Width: number, Height: number): OfficeExtension.ClientResult<number>;
        /**
         *
         * Removes particular overlay or all overlays on the Shape.
         *
         * [Api set:  1.1]
         *
         * @param OverlayId An Overlay Id. Removes the specific overlay id from the shape.
         */
        removeOverlay(OverlayId: number): void;
        /**
         *
         * Shows particular overlay on the Shape.
         *
         * [Api set:  1.1]
         *
         * @param overlayId overlay id in context
         * @param show to show or hide
         */
        showOverlay(overlayId: number, show: boolean): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.ShapeView` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.ShapeView` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.ShapeView` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Visio.Interfaces.ShapeViewLoadOptions): Visio.ShapeView;
        load(option?: string | string[]): Visio.ShapeView;
        load(option?: {
            select?: string;
            expand?: string;
        }): Visio.ShapeView;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Visio.ShapeView object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.ShapeViewData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Visio.Interfaces.ShapeViewData;
    }
    /**
     *
     * Represents the Position of the object in the view.
     *
     * [Api set:  1.1]
     */
    interface Position {
        /**
         *
         * An integer that specifies the x-coordinate of the object, which is the signed value of the distance in pixels from the viewport's center to the left boundary of the page.
         *
         * [Api set:  1.1]
         */
        x: number;
        /**
         *
         * An integer that specifies the y-coordinate of the object, which is the signed value of the distance in pixels from the viewport's center to the top boundary of the page.
         *
         * [Api set:  1.1]
         */
        y: number;
    }
    /**
     *
     * Represents the BoundingBox of the shape.
     *
     * [Api set:  1.1]
     */
    interface BoundingBox {
        /**
         *
         * The distance between the top and bottom edges of the bounding box of the shape, excluding any data graphics associated with the shape.
         *
         * [Api set:  1.1]
         */
        height: number;
        /**
         *
         * The distance between the left and right edges of the bounding box of the shape, excluding any data graphics associated with the shape.
         *
         * [Api set:  1.1]
         */
        width: number;
        /**
         *
         * An integer that specifies the x-coordinate of the bounding box.
         *
         * [Api set:  1.1]
         */
        x: number;
        /**
         *
         * An integer that specifies the y-coordinate of the bounding box.
         *
         * [Api set:  1.1]
         */
        y: number;
    }
    /**
     *
     * Represents the highlight data added to the shape.
     *
     * [Api set:  1.1]
     */
    interface Highlight {
        /**
         *
         * A string that specifies the color of the highlight. It must have the form "#RRGGBB", where each letter represents a hexadecimal digit between 0 and F, and where RR is the red value between 0 and 0xFF (255), GG the green value between 0 and 0xFF (255), and BB is the blue value between 0 and 0xFF (255).
         *
         * [Api set:  1.1]
         */
        color: string;
        /**
         *
         * A positive integer that specifies the width of the highlight's stroke in pixels.
         *
         * [Api set:  1.1]
         */
        width: number;
    }
    /**
     *
     * Represents the ShapeDataItemCollection for a given Shape.
     *
     * [Api set:  1.1]
     */
    class ShapeDataItemCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Visio.ShapeDataItem[];
        /**
         *
         * Gets the number of Shape Data Items.
         *
         * [Api set:  1.1]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets the ShapeDataItem using its name.
         *
         * [Api set:  1.1]
         *
         * @param key Key is the name of the ShapeDataItem to be retrieved.
         */
        getItem(key: string): Visio.ShapeDataItem;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.ShapeDataItemCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.ShapeDataItemCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.ShapeDataItemCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Visio.Interfaces.ShapeDataItemCollectionLoadOptions & Visio.Interfaces.CollectionLoadOptions): Visio.ShapeDataItemCollection;
        load(option?: string | string[]): Visio.ShapeDataItemCollection;
        load(option?: OfficeExtension.LoadOption): Visio.ShapeDataItemCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Visio.ShapeDataItemCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.ShapeDataItemCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Visio.Interfaces.ShapeDataItemCollectionData;
    }
    /**
     *
     * Represents the ShapeDataItem.
     *
     * [Api set:  1.1]
     */
    class ShapeDataItem extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * A string that specifies the format of the shape data item. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly format: string;
        /**
         *
         * A string that specifies the formatted value of the shape data item. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly formattedValue: string;
        /**
         *
         * A string that specifies the label of the shape data item. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly label: string;
        /**
         *
         * A string that specifies the value of the shape data item. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly value: string;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.ShapeDataItem` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.ShapeDataItem` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.ShapeDataItem` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Visio.Interfaces.ShapeDataItemLoadOptions): Visio.ShapeDataItem;
        load(option?: string | string[]): Visio.ShapeDataItem;
        load(option?: {
            select?: string;
            expand?: string;
        }): Visio.ShapeDataItem;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Visio.ShapeDataItem object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.ShapeDataItemData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Visio.Interfaces.ShapeDataItemData;
    }
    /**
     *
     * Represents the Hyperlink Collection.
     *
     * [Api set:  1.1]
     */
    class HyperlinkCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Visio.Hyperlink[];
        /**
         *
         * Gets the number of hyperlinks.
         *
         * [Api set:  1.1]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets a Hyperlink using its key (name or Id).
         *
         * [Api set:  1.1]
         *
         * @param Key Key is the name or index of the Hyperlink to be retrieved.
         */
        getItem(Key: number | string): Visio.Hyperlink;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.HyperlinkCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.HyperlinkCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.HyperlinkCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Visio.Interfaces.HyperlinkCollectionLoadOptions & Visio.Interfaces.CollectionLoadOptions): Visio.HyperlinkCollection;
        load(option?: string | string[]): Visio.HyperlinkCollection;
        load(option?: OfficeExtension.LoadOption): Visio.HyperlinkCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Visio.HyperlinkCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.HyperlinkCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Visio.Interfaces.HyperlinkCollectionData;
    }
    /**
     *
     * Represents the Hyperlink.
     *
     * [Api set:  1.1]
     */
    class Hyperlink extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the address of the Hyperlink object. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly address: string;
        /**
         *
         * Gets the description of a hyperlink. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly description: string;
        /**
         *
         * Gets the extra URL request information used to resolve the hyperlink's URL. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly extraInfo: string;
        /**
         *
         * Gets the sub-address of the Hyperlink object. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly subAddress: string;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.Hyperlink` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.Hyperlink` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.Hyperlink` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Visio.Interfaces.HyperlinkLoadOptions): Visio.Hyperlink;
        load(option?: string | string[]): Visio.Hyperlink;
        load(option?: {
            select?: string;
            expand?: string;
        }): Visio.Hyperlink;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Visio.Hyperlink object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.HyperlinkData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Visio.Interfaces.HyperlinkData;
    }
    /**
     *
     * Represents the CommentCollection for a given Shape.
     *
     * [Api set:  1.1]
     */
    class CommentCollection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /** Gets the loaded child items in this collection. */
        readonly items: Visio.Comment[];
        /**
         *
         * Gets the number of Comments.
         *
         * [Api set:  1.1]
         */
        getCount(): OfficeExtension.ClientResult<number>;
        /**
         *
         * Gets the Comment using its name.
         *
         * [Api set:  1.1]
         *
         * @param key Key is the name of the Comment to be retrieved.
         */
        getItem(key: string): Visio.Comment;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.CommentCollection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.CommentCollection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.CommentCollection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Visio.Interfaces.CommentCollectionLoadOptions & Visio.Interfaces.CollectionLoadOptions): Visio.CommentCollection;
        load(option?: string | string[]): Visio.CommentCollection;
        load(option?: OfficeExtension.LoadOption): Visio.CommentCollection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original `Visio.CommentCollection` object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.CommentCollectionData`) that contains an "items" array with shallow copies of any loaded properties from the collection's items.
        */
        toJSON(): Visio.Interfaces.CommentCollectionData;
    }
    /**
     *
     * Represents the Comment.
     *
     * [Api set:  1.1]
     */
    class Comment extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * A string that specifies the name of the author of the comment.
         *
         * [Api set:  1.1]
         */
        author: string;
        /**
         *
         * A string that specifies the date when the comment was created.
         *
         * [Api set:  1.1]
         */
        date: string;
        /**
         *
         * A string that contains the comment text.
         *
         * [Api set:  1.1]
         */
        text: string;
        /** Sets multiple properties of an object at the same time. You can pass either a plain object with the appropriate properties, or another API object of the same type.
         *
         * @remarks
         *
         * This method has the following additional signature:
         *
         * `set(properties: Visio.Comment): void`
         *
         * @param properties A JavaScript object with properties that are structured isomorphically to the properties of the object on which the method is called.
         * @param options Provides an option to suppress errors if the properties object tries to set any read-only properties.
         */
        set(properties: Interfaces.CommentUpdateData, options?: OfficeExtension.UpdateOptions): void;
        /** Sets multiple properties on the object at the same time, based on an existing loaded object. */
        set(properties: Visio.Comment): void;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.Comment` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.Comment` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.Comment` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: Visio.Interfaces.CommentLoadOptions): Visio.Comment;
        load(option?: string | string[]): Visio.Comment;
        load(option?: {
            select?: string;
            expand?: string;
        }): Visio.Comment;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Visio.Comment object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.CommentData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Visio.Interfaces.CommentData;
    }
    /**
     *
     * Represents the Selection in the page.
     *
     * [Api set:  1.1]
     */
    class Selection extends OfficeExtension.ClientObject {
        /** The request context associated with the object. This connects the add-in's process to the Office host application's process. */
        context: RequestContext; 
        /**
         *
         * Gets the Shapes of the Selection. Read-only.
         *
         * [Api set:  1.1]
         */
        readonly shapes: Visio.ShapeCollection;
        /**
         * Queues up a command to load the specified properties of the object. You must call "context.sync()" before reading the properties.
         *
         * @remarks
         *
         * In addition to this signature, this method has the following signatures:
         *
         * `load(option?: string | string[]): Visio.Selection` - Where option is a comma-delimited string or an array of strings that specify the properties to load.
         *
         * `load(option?: { select?: string; expand?: string; }): Visio.Selection` - Where option.select is a comma-delimited string that specifies the properties to load, and options.expand is a comma-delimited string that specifies the navigation properties to load.
         *
         * `load(option?: { select?: string; expand?: string; top?: number; skip?: number }): Visio.Selection` - Only available on collection types. It is similar to the preceding signature. Option.top specifies the maximum number of collection items that can be included in the result. Option.skip specifies the number of items that are to be skipped and not included in the result. If option.top is specified, the result set will start after skipping the specified number of items.
         *
         * @param options Provides options for which properties of the object to load.
         */
        load(option?: string | string[]): Visio.Selection;
        load(option?: {
            select?: string;
            expand?: string;
        }): Visio.Selection;
        /**
        * Overrides the JavaScript `toJSON()` method in order to provide more useful output when an API object is passed to `JSON.stringify()`. (`JSON.stringify`, in turn, calls the `toJSON` method of the object that is passed to it.)
        * Whereas the original Visio.Selection object is an API object, the `toJSON` method returns a plain JavaScript object (typed as `Visio.Interfaces.SelectionData`) that contains shallow copies of any loaded child properties from the original object.
        */
        toJSON(): Visio.Interfaces.SelectionData;
    }
    /**
     *
     * Represents the Horizontal Alignment of the Overlay relative to the shape.
     *
     * [Api set:  1.1]
     */
    enum OverlayHorizontalAlignment {
        /**
         *
         * left
         *
         */
        left = "Left",
        /**
         *
         * center
         *
         */
        center = "Center",
        /**
         *
         * right
         *
         */
        right = "Right",
    }
    /**
     *
     * Represents the Vertical Alignment of the Overlay relative to the shape.
     *
     * [Api set:  1.1]
     */
    enum OverlayVerticalAlignment {
        /**
         *
         * top
         *
         */
        top = "Top",
        /**
         *
         * middle
         *
         */
        middle = "Middle",
        /**
         *
         * bottom
         *
         */
        bottom = "Bottom",
    }
    /**
     *
     * Represents the type of the overlay.
     *
     * [Api set:  1.1]
     */
    enum OverlayType {
        /**
         *
         * text
         *
         */
        text = "Text",
        /**
         *
         * image
         *
         */
        image = "Image",
        /**
         *
         * html
         *
         */
        html = "Html",
    }
    /**
     *
     * Toolbar IDs of the app
     *
     * [Api set:  1.1]
     */
    enum ToolBarType {
        /**
         *
         * CommandBar
         *
         */
        commandBar = "CommandBar",
        /**
         *
         * PageNavigationBar
         *
         */
        pageNavigationBar = "PageNavigationBar",
        /**
         *
         * StatusBar
         *
         */
        statusBar = "StatusBar",
    }
    enum ErrorCodes {
        accessDenied = "AccessDenied",
        generalException = "GeneralException",
        invalidArgument = "InvalidArgument",
        itemNotFound = "ItemNotFound",
        notImplemented = "NotImplemented",
        unsupportedOperation = "UnsupportedOperation",
    }
    module Interfaces {
        /**
        * Provides ways to load properties of only a subset of members of a collection.
        */
        interface CollectionLoadOptions {
            /**
            * Specify the number of items in the queried collection to be included in the result.
            */
            $top?: number;
            /**
            * Specify the number of items in the collection that are to be skipped and not included in the result. If top is specified, the selection of result will start after skipping the specified number of items.
            */
            $skip?: number;
        }
        /** An interface for updating data on the Application object, for use in "application.set({ ... })". */
        interface ApplicationUpdateData {
            /**
             *
             * Show or hide the iFrame application borders.
             *
             * [Api set:  1.1]
             */
            showBorders?: boolean;
            /**
             *
             * Show or hide the standard toolbars.
             *
             * [Api set:  1.1]
             */
            showToolbars?: boolean;
        }
        /** An interface for updating data on the Document object, for use in "document.set({ ... })". */
        interface DocumentUpdateData {
            /**
            *
            * Represents a Visio application instance that contains this document.
            *
            * [Api set:  1.1]
            */
            application?: Visio.Interfaces.ApplicationUpdateData;
            /**
            *
            * Returns the DocumentView object.
            *
            * [Api set:  1.1]
            */
            view?: Visio.Interfaces.DocumentViewUpdateData;
        }
        /** An interface for updating data on the DocumentView object, for use in "documentView.set({ ... })". */
        interface DocumentViewUpdateData {
            /**
             *
             * Disable Hyperlinks.
             *
             * [Api set:  1.1]
             */
            disableHyperlinks?: boolean;
            /**
             *
             * Disable Pan.
             *
             * [Api set:  1.1]
             */
            disablePan?: boolean;
            /**
             *
             * Disable PanZoomWindow.
             *
             * [Api set:  1.1]
             */
            disablePanZoomWindow?: boolean;
            /**
             *
             * Disable Zoom.
             *
             * [Api set:  1.1]
             */
            disableZoom?: boolean;
            /**
             *
             * Hide Diagram Boundary.
             *
             * [Api set:  1.1]
             */
            hideDiagramBoundary?: boolean;
        }
        /** An interface for updating data on the Page object, for use in "page.set({ ... })". */
        interface PageUpdateData {
            /**
            *
            * Returns the view of the page.
            *
            * [Api set:  1.1]
            */
            view?: Visio.Interfaces.PageViewUpdateData;
        }
        /** An interface for updating data on the PageView object, for use in "pageView.set({ ... })". */
        interface PageViewUpdateData {
            /**
             *
             * Get and set Page's Zoom level. The value can be between 10 and 400 and denotes the percentage of zoom.
             *
             * [Api set:  1.1]
             */
            zoom?: number;
        }
        /** An interface for updating data on the PageCollection object, for use in "pageCollection.set({ ... })". */
        interface PageCollectionUpdateData {
            items?: Visio.Interfaces.PageData[];
        }
        /** An interface for updating data on the ShapeCollection object, for use in "shapeCollection.set({ ... })". */
        interface ShapeCollectionUpdateData {
            items?: Visio.Interfaces.ShapeData[];
        }
        /** An interface for updating data on the Shape object, for use in "shape.set({ ... })". */
        interface ShapeUpdateData {
            /**
            *
            * Returns the view of the shape.
            *
            * [Api set:  1.1]
            */
            view?: Visio.Interfaces.ShapeViewUpdateData;
            /**
             *
             * Returns true, if shape is selected. User can set true to select the shape explicitly.
             *
             * [Api set:  1.1]
             */
            select?: boolean;
        }
        /** An interface for updating data on the ShapeView object, for use in "shapeView.set({ ... })". */
        interface ShapeViewUpdateData {
            /**
             *
             * Represents the highlight around the shape.
             *
             * [Api set:  1.1]
             */
            highlight?: Visio.Highlight;
        }
        /** An interface for updating data on the ShapeDataItemCollection object, for use in "shapeDataItemCollection.set({ ... })". */
        interface ShapeDataItemCollectionUpdateData {
            items?: Visio.Interfaces.ShapeDataItemData[];
        }
        /** An interface for updating data on the HyperlinkCollection object, for use in "hyperlinkCollection.set({ ... })". */
        interface HyperlinkCollectionUpdateData {
            items?: Visio.Interfaces.HyperlinkData[];
        }
        /** An interface for updating data on the CommentCollection object, for use in "commentCollection.set({ ... })". */
        interface CommentCollectionUpdateData {
            items?: Visio.Interfaces.CommentData[];
        }
        /** An interface for updating data on the Comment object, for use in "comment.set({ ... })". */
        interface CommentUpdateData {
            /**
             *
             * A string that specifies the name of the author of the comment.
             *
             * [Api set:  1.1]
             */
            author?: string;
            /**
             *
             * A string that specifies the date when the comment was created.
             *
             * [Api set:  1.1]
             */
            date?: string;
            /**
             *
             * A string that contains the comment text.
             *
             * [Api set:  1.1]
             */
            text?: string;
        }
        /** An interface describing the data returned by calling "application.toJSON()". */
        interface ApplicationData {
            /**
             *
             * Show or hide the iFrame application borders.
             *
             * [Api set:  1.1]
             */
            showBorders?: boolean;
            /**
             *
             * Show or hide the standard toolbars.
             *
             * [Api set:  1.1]
             */
            showToolbars?: boolean;
        }
        /** An interface describing the data returned by calling "document.toJSON()". */
        interface DocumentData {
            /**
            *
            * Represents a Visio application instance that contains this document. Read-only.
            *
            * [Api set:  1.1]
            */
            application?: Visio.Interfaces.ApplicationData;
            /**
            *
            * Represents a collection of pages associated with the document. Read-only.
            *
            * [Api set:  1.1]
            */
            pages?: Visio.Interfaces.PageData[];
            /**
            *
            * Returns the DocumentView object. Read-only.
            *
            * [Api set:  1.1]
            */
            view?: Visio.Interfaces.DocumentViewData;
        }
        /** An interface describing the data returned by calling "documentView.toJSON()". */
        interface DocumentViewData {
            /**
             *
             * Disable Hyperlinks.
             *
             * [Api set:  1.1]
             */
            disableHyperlinks?: boolean;
            /**
             *
             * Disable Pan.
             *
             * [Api set:  1.1]
             */
            disablePan?: boolean;
            /**
             *
             * Disable PanZoomWindow.
             *
             * [Api set:  1.1]
             */
            disablePanZoomWindow?: boolean;
            /**
             *
             * Disable Zoom.
             *
             * [Api set:  1.1]
             */
            disableZoom?: boolean;
            /**
             *
             * Hide Diagram Boundary.
             *
             * [Api set:  1.1]
             */
            hideDiagramBoundary?: boolean;
        }
        /** An interface describing the data returned by calling "page.toJSON()". */
        interface PageData {
            /**
            *
            * All shapes in the Page, including subshapes. Read-only.
            *
            * [Api set:  1.1]
            */
            allShapes?: Visio.Interfaces.ShapeData[];
            /**
            *
            * Returns the Comments Collection.  Read-only.
            *
            * [Api set:  1.1]
            */
            comments?: Visio.Interfaces.CommentData[];
            /**
            *
            * All top-level shapes in the Page.Read-only.
            *
            * [Api set:  1.1]
            */
            shapes?: Visio.Interfaces.ShapeData[];
            /**
            *
            * Returns the view of the page. Read-only.
            *
            * [Api set:  1.1]
            */
            view?: Visio.Interfaces.PageViewData;
            /**
             *
             * Returns the height of the page. Read-only.
             *
             * [Api set:  1.1]
             */
            height?: number;
            /**
             *
             * Index of the Page. Read-only.
             *
             * [Api set:  1.1]
             */
            index?: number;
            /**
             *
             * Whether the page is a background page or not. Read-only.
             *
             * [Api set:  1.1]
             */
            isBackground?: boolean;
            /**
             *
             * Page name. Read-only.
             *
             * [Api set:  1.1]
             */
            name?: string;
            /**
             *
             * Returns the width of the page. Read-only.
             *
             * [Api set:  1.1]
             */
            width?: number;
        }
        /** An interface describing the data returned by calling "pageView.toJSON()". */
        interface PageViewData {
            /**
             *
             * Get and set Page's Zoom level. The value can be between 10 and 400 and denotes the percentage of zoom.
             *
             * [Api set:  1.1]
             */
            zoom?: number;
        }
        /** An interface describing the data returned by calling "pageCollection.toJSON()". */
        interface PageCollectionData {
            items?: Visio.Interfaces.PageData[];
        }
        /** An interface describing the data returned by calling "shapeCollection.toJSON()". */
        interface ShapeCollectionData {
            items?: Visio.Interfaces.ShapeData[];
        }
        /** An interface describing the data returned by calling "shape.toJSON()". */
        interface ShapeData {
            /**
            *
            * Returns the Comments Collection. Read-only.
            *
            * [Api set:  1.1]
            */
            comments?: Visio.Interfaces.CommentData[];
            /**
            *
            * Returns the Hyperlinks collection for a Shape object. Read-only.
            *
            * [Api set:  1.1]
            */
            hyperlinks?: Visio.Interfaces.HyperlinkData[];
            /**
            *
            * Returns the Shape's Data Section. Read-only.
            *
            * [Api set:  1.1]
            */
            shapeDataItems?: Visio.Interfaces.ShapeDataItemData[];
            /**
            *
            * Gets SubShape Collection. Read-only.
            *
            * [Api set:  1.1]
            */
            subShapes?: Visio.Interfaces.ShapeData[];
            /**
            *
            * Returns the view of the shape. Read-only.
            *
            * [Api set:  1.1]
            */
            view?: Visio.Interfaces.ShapeViewData;
            /**
             *
             * Shape's identifier. Read-only.
             *
             * [Api set:  1.1]
             */
            id?: number;
            /**
             *
             * Shape's name. Read-only.
             *
             * [Api set:  1.1]
             */
            name?: string;
            /**
             *
             * Returns true, if shape is selected. User can set true to select the shape explicitly.
             *
             * [Api set:  1.1]
             */
            select?: boolean;
            /**
             *
             * Shape's text. Read-only.
             *
             * [Api set:  1.1]
             */
            text?: string;
        }
        /** An interface describing the data returned by calling "shapeView.toJSON()". */
        interface ShapeViewData {
            /**
             *
             * Represents the highlight around the shape.
             *
             * [Api set:  1.1]
             */
            highlight?: Visio.Highlight;
        }
        /** An interface describing the data returned by calling "shapeDataItemCollection.toJSON()". */
        interface ShapeDataItemCollectionData {
            items?: Visio.Interfaces.ShapeDataItemData[];
        }
        /** An interface describing the data returned by calling "shapeDataItem.toJSON()". */
        interface ShapeDataItemData {
            /**
             *
             * A string that specifies the format of the shape data item. Read-only.
             *
             * [Api set:  1.1]
             */
            format?: string;
            /**
             *
             * A string that specifies the formatted value of the shape data item. Read-only.
             *
             * [Api set:  1.1]
             */
            formattedValue?: string;
            /**
             *
             * A string that specifies the label of the shape data item. Read-only.
             *
             * [Api set:  1.1]
             */
            label?: string;
            /**
             *
             * A string that specifies the value of the shape data item. Read-only.
             *
             * [Api set:  1.1]
             */
            value?: string;
        }
        /** An interface describing the data returned by calling "hyperlinkCollection.toJSON()". */
        interface HyperlinkCollectionData {
            items?: Visio.Interfaces.HyperlinkData[];
        }
        /** An interface describing the data returned by calling "hyperlink.toJSON()". */
        interface HyperlinkData {
            /**
             *
             * Gets the address of the Hyperlink object. Read-only.
             *
             * [Api set:  1.1]
             */
            address?: string;
            /**
             *
             * Gets the description of a hyperlink. Read-only.
             *
             * [Api set:  1.1]
             */
            description?: string;
            /**
             *
             * Gets the extra URL request information used to resolve the hyperlink's URL. Read-only.
             *
             * [Api set:  1.1]
             */
            extraInfo?: string;
            /**
             *
             * Gets the sub-address of the Hyperlink object. Read-only.
             *
             * [Api set:  1.1]
             */
            subAddress?: string;
        }
        /** An interface describing the data returned by calling "commentCollection.toJSON()". */
        interface CommentCollectionData {
            items?: Visio.Interfaces.CommentData[];
        }
        /** An interface describing the data returned by calling "comment.toJSON()". */
        interface CommentData {
            /**
             *
             * A string that specifies the name of the author of the comment.
             *
             * [Api set:  1.1]
             */
            author?: string;
            /**
             *
             * A string that specifies the date when the comment was created.
             *
             * [Api set:  1.1]
             */
            date?: string;
            /**
             *
             * A string that contains the comment text.
             *
             * [Api set:  1.1]
             */
            text?: string;
        }
        /** An interface describing the data returned by calling "selection.toJSON()". */
        interface SelectionData {
            /**
            *
            * Gets the Shapes of the Selection. Read-only.
            *
            * [Api set:  1.1]
            */
            shapes?: Visio.Interfaces.ShapeData[];
        }
        /**
         *
         * Represents the Application.
         *
         * [Api set:  1.1]
         */
        interface ApplicationLoadOptions {
            $all?: boolean;
            /**
             *
             * Show or hide the iFrame application borders.
             *
             * [Api set:  1.1]
             */
            showBorders?: boolean;
            /**
             *
             * Show or hide the standard toolbars.
             *
             * [Api set:  1.1]
             */
            showToolbars?: boolean;
        }
        /**
         *
         * Represents the Document class.
         *
         * [Api set:  1.1]
         */
        interface DocumentLoadOptions {
            $all?: boolean;
            /**
            *
            * Represents a Visio application instance that contains this document.
            *
            * [Api set:  1.1]
            */
            application?: Visio.Interfaces.ApplicationLoadOptions;
            /**
            *
            * Returns the DocumentView object.
            *
            * [Api set:  1.1]
            */
            view?: Visio.Interfaces.DocumentViewLoadOptions;
        }
        /**
         *
         * Represents the DocumentView class.
         *
         * [Api set:  1.1]
         */
        interface DocumentViewLoadOptions {
            $all?: boolean;
            /**
             *
             * Disable Hyperlinks.
             *
             * [Api set:  1.1]
             */
            disableHyperlinks?: boolean;
            /**
             *
             * Disable Pan.
             *
             * [Api set:  1.1]
             */
            disablePan?: boolean;
            /**
             *
             * Disable PanZoomWindow.
             *
             * [Api set:  1.1]
             */
            disablePanZoomWindow?: boolean;
            /**
             *
             * Disable Zoom.
             *
             * [Api set:  1.1]
             */
            disableZoom?: boolean;
            /**
             *
             * Hide Diagram Boundary.
             *
             * [Api set:  1.1]
             */
            hideDiagramBoundary?: boolean;
        }
        /**
         *
         * Represents the Page class.
         *
         * [Api set:  1.1]
         */
        interface PageLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns the view of the page.
            *
            * [Api set:  1.1]
            */
            view?: Visio.Interfaces.PageViewLoadOptions;
            /**
             *
             * Returns the height of the page. Read-only.
             *
             * [Api set:  1.1]
             */
            height?: boolean;
            /**
             *
             * Index of the Page. Read-only.
             *
             * [Api set:  1.1]
             */
            index?: boolean;
            /**
             *
             * Whether the page is a background page or not. Read-only.
             *
             * [Api set:  1.1]
             */
            isBackground?: boolean;
            /**
             *
             * Page name. Read-only.
             *
             * [Api set:  1.1]
             */
            name?: boolean;
            /**
             *
             * Returns the width of the page. Read-only.
             *
             * [Api set:  1.1]
             */
            width?: boolean;
        }
        /**
         *
         * Represents the PageView class.
         *
         * [Api set:  1.1]
         */
        interface PageViewLoadOptions {
            $all?: boolean;
            /**
             *
             * Get and set Page's Zoom level. The value can be between 10 and 400 and denotes the percentage of zoom.
             *
             * [Api set:  1.1]
             */
            zoom?: boolean;
        }
        /**
         *
         * Represents a collection of Page objects that are part of the document.
         *
         * [Api set:  1.1]
         */
        interface PageCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Returns the view of the page.
            *
            * [Api set:  1.1]
            */
            view?: Visio.Interfaces.PageViewLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Returns the height of the page. Read-only.
             *
             * [Api set:  1.1]
             */
            height?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Index of the Page. Read-only.
             *
             * [Api set:  1.1]
             */
            index?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Whether the page is a background page or not. Read-only.
             *
             * [Api set:  1.1]
             */
            isBackground?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Page name. Read-only.
             *
             * [Api set:  1.1]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns the width of the page. Read-only.
             *
             * [Api set:  1.1]
             */
            width?: boolean;
        }
        /**
         *
         * Represents the Shape Collection.
         *
         * [Api set:  1.1]
         */
        interface ShapeCollectionLoadOptions {
            $all?: boolean;
            /**
            *
            * For EACH ITEM in the collection: Returns the view of the shape.
            *
            * [Api set:  1.1]
            */
            view?: Visio.Interfaces.ShapeViewLoadOptions;
            /**
             *
             * For EACH ITEM in the collection: Shape's identifier. Read-only.
             *
             * [Api set:  1.1]
             */
            id?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Shape's name. Read-only.
             *
             * [Api set:  1.1]
             */
            name?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Returns true, if shape is selected. User can set true to select the shape explicitly.
             *
             * [Api set:  1.1]
             */
            select?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Shape's text. Read-only.
             *
             * [Api set:  1.1]
             */
            text?: boolean;
        }
        /**
         *
         * Represents the Shape class.
         *
         * [Api set:  1.1]
         */
        interface ShapeLoadOptions {
            $all?: boolean;
            /**
            *
            * Returns the view of the shape.
            *
            * [Api set:  1.1]
            */
            view?: Visio.Interfaces.ShapeViewLoadOptions;
            /**
             *
             * Shape's identifier. Read-only.
             *
             * [Api set:  1.1]
             */
            id?: boolean;
            /**
             *
             * Shape's name. Read-only.
             *
             * [Api set:  1.1]
             */
            name?: boolean;
            /**
             *
             * Returns true, if shape is selected. User can set true to select the shape explicitly.
             *
             * [Api set:  1.1]
             */
            select?: boolean;
            /**
             *
             * Shape's text. Read-only.
             *
             * [Api set:  1.1]
             */
            text?: boolean;
        }
        /**
         *
         * Represents the ShapeView class.
         *
         * [Api set:  1.1]
         */
        interface ShapeViewLoadOptions {
            $all?: boolean;
            /**
             *
             * Represents the highlight around the shape.
             *
             * [Api set:  1.1]
             */
            highlight?: boolean;
        }
        /**
         *
         * Represents the ShapeDataItemCollection for a given Shape.
         *
         * [Api set:  1.1]
         */
        interface ShapeDataItemCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: A string that specifies the format of the shape data item. Read-only.
             *
             * [Api set:  1.1]
             */
            format?: boolean;
            /**
             *
             * For EACH ITEM in the collection: A string that specifies the formatted value of the shape data item. Read-only.
             *
             * [Api set:  1.1]
             */
            formattedValue?: boolean;
            /**
             *
             * For EACH ITEM in the collection: A string that specifies the label of the shape data item. Read-only.
             *
             * [Api set:  1.1]
             */
            label?: boolean;
            /**
             *
             * For EACH ITEM in the collection: A string that specifies the value of the shape data item. Read-only.
             *
             * [Api set:  1.1]
             */
            value?: boolean;
        }
        /**
         *
         * Represents the ShapeDataItem.
         *
         * [Api set:  1.1]
         */
        interface ShapeDataItemLoadOptions {
            $all?: boolean;
            /**
             *
             * A string that specifies the format of the shape data item. Read-only.
             *
             * [Api set:  1.1]
             */
            format?: boolean;
            /**
             *
             * A string that specifies the formatted value of the shape data item. Read-only.
             *
             * [Api set:  1.1]
             */
            formattedValue?: boolean;
            /**
             *
             * A string that specifies the label of the shape data item. Read-only.
             *
             * [Api set:  1.1]
             */
            label?: boolean;
            /**
             *
             * A string that specifies the value of the shape data item. Read-only.
             *
             * [Api set:  1.1]
             */
            value?: boolean;
        }
        /**
         *
         * Represents the Hyperlink Collection.
         *
         * [Api set:  1.1]
         */
        interface HyperlinkCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the address of the Hyperlink object. Read-only.
             *
             * [Api set:  1.1]
             */
            address?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the description of a hyperlink. Read-only.
             *
             * [Api set:  1.1]
             */
            description?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the extra URL request information used to resolve the hyperlink's URL. Read-only.
             *
             * [Api set:  1.1]
             */
            extraInfo?: boolean;
            /**
             *
             * For EACH ITEM in the collection: Gets the sub-address of the Hyperlink object. Read-only.
             *
             * [Api set:  1.1]
             */
            subAddress?: boolean;
        }
        /**
         *
         * Represents the Hyperlink.
         *
         * [Api set:  1.1]
         */
        interface HyperlinkLoadOptions {
            $all?: boolean;
            /**
             *
             * Gets the address of the Hyperlink object. Read-only.
             *
             * [Api set:  1.1]
             */
            address?: boolean;
            /**
             *
             * Gets the description of a hyperlink. Read-only.
             *
             * [Api set:  1.1]
             */
            description?: boolean;
            /**
             *
             * Gets the extra URL request information used to resolve the hyperlink's URL. Read-only.
             *
             * [Api set:  1.1]
             */
            extraInfo?: boolean;
            /**
             *
             * Gets the sub-address of the Hyperlink object. Read-only.
             *
             * [Api set:  1.1]
             */
            subAddress?: boolean;
        }
        /**
         *
         * Represents the CommentCollection for a given Shape.
         *
         * [Api set:  1.1]
         */
        interface CommentCollectionLoadOptions {
            $all?: boolean;
            /**
             *
             * For EACH ITEM in the collection: A string that specifies the name of the author of the comment.
             *
             * [Api set:  1.1]
             */
            author?: boolean;
            /**
             *
             * For EACH ITEM in the collection: A string that specifies the date when the comment was created.
             *
             * [Api set:  1.1]
             */
            date?: boolean;
            /**
             *
             * For EACH ITEM in the collection: A string that contains the comment text.
             *
             * [Api set:  1.1]
             */
            text?: boolean;
        }
        /**
         *
         * Represents the Comment.
         *
         * [Api set:  1.1]
         */
        interface CommentLoadOptions {
            $all?: boolean;
            /**
             *
             * A string that specifies the name of the author of the comment.
             *
             * [Api set:  1.1]
             */
            author?: boolean;
            /**
             *
             * A string that specifies the date when the comment was created.
             *
             * [Api set:  1.1]
             */
            date?: boolean;
            /**
             *
             * A string that contains the comment text.
             *
             * [Api set:  1.1]
             */
            text?: boolean;
        }
    }
}
declare namespace Visio {
    /**
     * The RequestContext object facilitates requests to the Visio application. Since the Office add-in and the Visio application run in two different processes, the request context is required to get access to the Visio object model from the add-in.
     */
    class RequestContext extends OfficeCore.RequestContext {
        constructor(url?: string | OfficeExtension.EmbeddedSession);
        readonly document: Document;
    }
    /**
 * Executes a batch script that performs actions on the Visio object model, using a new request context. When the promise is resolved, any tracked objects that were automatically allocated during execution will be released.
 * @param batch - A function that takes in an Visio.RequestContext and returns a promise (typically, just the result of "context.sync()"). The context parameter facilitates requests to the Visio application. Since the Office add-in and the Visio application run in two different processes, the request context is required to get access to the Visio object model from the add-in.
 */
    function run<T>(batch: (context: Visio.RequestContext) => Promise<T>): Promise<T>;
    /**
     * Executes a batch script that performs actions on the Visio object model, using the request context of a previously-created API object.
     * @param object - A previously-created API object. The batch will use the same request context as the passed-in object, which means that any changes applied to the object will be picked up by "context.sync()".
     * @param batch - A function that takes in an Visio.RequestContext and returns a promise (typically, just the result of "context.sync()"). When the promise is resolved, any tracked objects that were automatically allocated during execution will be released.
     */
    function run<T>(object: OfficeExtension.ClientObject | OfficeExtension.EmbeddedSession, batch: (context: Visio.RequestContext) => Promise<T>): Promise<T>;
    /**
     * Executes a batch script that performs actions on the Visio object model, using the request context of previously-created API objects.
     * @param objects - An array of previously-created API objects. The array will be validated to make sure that all of the objects share the same context. The batch will use this shared request context, which means that any changes applied to these objects will be picked up by "context.sync()".
     * @param batch - A function that takes in a Visio.RequestContext and returns a promise (typically, just the result of "context.sync()"). When the promise is resolved, any tracked objects that were automatically allocated during execution will be released.
     */
    function run<T>(objects: OfficeExtension.ClientObject[], batch: (context: Visio.RequestContext) => Promise<T>): Promise<T>;
    /**
     * Executes a batch script that performs actions on the Visio object model, using the RequestContext of a previously-created object. When the promise is resolved, any tracked objects that were automatically allocated during execution will be released.
     * @param contextObject - A previously-created Visio.RequestContext. This context will get re-used by the batch function (instead of having a new context created). This means that the batch will be able to pick up changes made to existing API objects, if those objects were derived from this same context.
     * @param batch - A function that takes in a RequestContext and returns a promise (typically, just the result of "context.sync()"). The context parameter facilitates requests to the Visio application. Since the Office add-in and the Visio application run in two different processes, the RequestContext is required to get access to the Visio object model from the add-in.
     * @remarks
     * In addition to this signature, the method also has the following signatures:
     *
     * `run<T>(batch: (context: Visio.RequestContext) => Promise<T>): Promise<T>;`
     *
     * `run<T>(object: OfficeExtension.ClientObject | OfficeExtension.EmbeddedSession, batch: (context: Visio.RequestContext) => Promise<T>): Promise<T>;`
     *
     * `run<T>(objects: OfficeExtension.ClientObject[], batch: (context: Visio.RequestContext) => Promise<T>): Promise<T>;`
     */
    function run<T>(contextObject: OfficeExtension.ClientRequestContext, batch: (context: Visio.RequestContext) => Promise<T>): Promise<T>;
}


////////////////////////////////////////////////////////////////
//////////////////////// End Visio APIs ////////////////////////
////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////
///////////////////// Begin PowerPoint APIs ////////////////////
////////////////////////////////////////////////////////////////

declare namespace PowerPoint {
    /**
     * Creates and opens a new presentation. Optionally, the presentation can be pre-populated with a base64-encoded .pptx file.
     *
     * [Api set: PowerPointApi 1.1]
     *
     * @param base64File Optional. The base64-encoded .pptx file. The default value is null.
     */
    function createPresentation(base64File?: string): Promise<void>;
}  

////////////////////////////////////////////////////////////////
////////////////////// End PowerPoint APIs /////////////////////
////////////////////////////////////////////////////////////////